// Type definitions for ArcGIS Maps SDK for JavaScript 4.31
// Project: http://js.arcgis.com
// Definitions by: Esri <https://github.com/Esri>
//                 Andy Gup <https://github.com/andygup>
//                 Bjorn Svensson <https://github.com/bsvensson>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
// TypeScript Version: 3.7

type HashMap<T> = Record<string, T>;

interface IHandle {
  remove(): void;
}

type GroupKey<T> = Exclude<T, IHandle>;

interface __esriImplementsIteration<T> {
  [Symbol.iterator](): IterableIterator<T>;
}

declare namespace __esri {
  export class Accessor {
    constructor(obj?: any);

    destroyed: boolean;
    initialized: boolean;
    declaredClass: string;

    destroy(): void;

    get<T>(propertyName: string): T;
    get(propertyName: string): any;
    set<T>(propertyName: string, value: T): this;
    set(props: HashMap<any>): this;
    watch(path: string | string[], callback: WatchCallback, sync?: boolean): WatchHandle;

    addHandles<T>(handles: IHandle | IHandle[], groupKey?: GroupKey<T>): void;
    removeHandles<T>(groupKey?: GroupKey<T>): void;
    hasHandles<T>(groupKey?: GroupKey<T>): boolean;

    protected notifyChange(propertyName: string): void;
    protected _get(propertyName: string): any;
    protected _get<T>(propertyName: string): T;
    protected _set<T>(propertyName: string, value: T): this;
  }

  export interface AnonymousAccessor {
    get?<T>(propertyName: string): T;
    get?(propertyName: string): any;
    set?<T>(propertyName: string, value: T): this;
    set?(props: HashMap<any>): this;
    watch?(path: string | string[], callback: WatchCallback, sync?: boolean): WatchHandle;
  }

  export type ItemCallback<T> = (item: T, index: number) => void;
  export type ItemCompareCallback<T> = (firstItem: T, secondItem: T) => number;
  export type ItemMapCallback<T, R> = (item: T, index: number) => R;
  export type ItemReduceCallback<T, R> = (previousValue: R, currentValue: T, index: number) => R;
  export type ItemTestCallback<T> = (item: T, index: number) => boolean;

  export type CollectionAfterAddEventHandler<T> = (event: CollectionAfterItemEvent<T>) => void;
  export type CollectionAfterChangesEventHandler = (event: CollectionAfterEvent) => void;
  export type CollectionAfterRemoveEventHandler<T> = (event: CollectionAfterItemEvent<T>) => void;

  export type CollectionChangeEventHandler<T> = (event: CollectionChangeEvent<T>) => void;

  export type CollectionBeforeAddEventHandler<T> = (event: CollectionItemEvent<T>) => void;
  export type CollectionBeforeChangesEventHandler = (event: CollectionEvent) => void;
  export type CollectionBeforeRemoveEventHandler<T> = (event: CollectionItemEvent<T>) => void;

  export interface CollectionEvent {
    defaultPrevented: boolean;
    cancellable: boolean;

    preventDefault(): void;
  }

  export interface CollectionItemEvent<T = any> extends CollectionEvent {
    item: T;
  }

  export interface CollectionAfterEvent {}

  export interface CollectionAfterItemEvent<T = any> {
    item: T;
  }

  export interface CollectionChangeEvent<T = any> {
    added: T[];
    removed: T[];
    moved: T[];
  }

  interface Collection<T = any> extends Evented, __esriImplementsIteration<T> {
    on(type: "after-add", listener: CollectionAfterAddEventHandler<T>): IHandle;
    on(type: "after-changes", listener: CollectionAfterChangesEventHandler): IHandle;
    on(type: "after-remove", listener: CollectionAfterRemoveEventHandler<T>): IHandle;
    on(type: "before-add", listener: CollectionBeforeAddEventHandler<T>): IHandle;
    on(type: "before-changes", listener: CollectionBeforeChangesEventHandler): IHandle;
    on(type: "before-remove", listener: CollectionBeforeRemoveEventHandler<T>): IHandle;
    on(type: "change", listener: CollectionChangeEventHandler<T>): IHandle;
    on(type: string, listener: (event: any) => void): IHandle;
  }

  type Constructor<T> = new (...params: any[]) => T;

  interface Types<T extends Base, Base = T> {
    key: string | ((obj: any) => string);
    base: Constructor<Base> | Function;
    typeMap: HashMap<Constructor<T>>;
  }

  export class Collection<T = any> extends Accessor {
    readonly length: number;

    constructor(values?: any[] | Collection<any> | CollectionPropertiesBase<any>);

    at(index: number): T;
    add(item: T, index?: number): this;
    addMany(items: T[] | Collection<T>, index?: number): this;
    clone(): Collection<T>;
    concat(value: T[] | Collection<T>): Collection<T>;
    every(callback: ItemTestCallback<T>): boolean;
    filter(callback: ItemTestCallback<T>): Collection<T>;
    filter<S extends T>(callback: (item: T, index: number) => item is S): Collection<S>;
    find(callback: ItemTestCallback<T>): T;
    findIndex(callback: ItemTestCallback<T>): number;
    flatten(callback: (item: T, index: number) => T[] | Collection<T>): Collection<T>;
    flatten<U>(callback: (item: U, index: number) => U[] | Collection<U>): Collection<U>;
    forEach(callback: ItemCallback<T>): void;
    getItemAt(index: number): T;
    includes(searchElement: T): boolean;
    indexOf(searchElement: T, fromIndex?: number): number;
    join(separator?: string): string;
    lastIndexOf(searchElement: T, fromIndex?: number): number;
    map<R = T>(callback: ItemMapCallback<T, R>): Collection<R>;
    pop(): T;
    push(item: T): number;
    reduce<R = T>(callback: ItemReduceCallback<T, R>, initialValue?: R): R;
    reduceRight<R = T>(callback: ItemReduceCallback<T, R>, initialValue?: R): R;
    remove(item: T): T;
    removeAll(): T[];
    removeAt(index: number): T;
    removeMany(items: T[] | Collection<T>): T[];
    reorder(item: T, index: number): T;
    reverse(): this;
    shift(): T;
    slice(begin?: number, end?: number): Collection<T>;
    some(callback: ItemCallback<T>): boolean;
    sort(compareFunction?: ItemCompareCallback<T>): this;
    splice(start: number, deleteCount: number, ...items: T[]): T[];
    toArray(): T[];
    unshift(...items: T[]): number;

    static isCollection<T = any>(value: any | Collection<T>): value is Collection<T>;

    static ofType<T extends Base, Base = T>(
      type: Constructor<T> | Types<T, Base>,
    ): new (items?: (T[] | Collection<T>) | { items?: T[] | Collection<T> }) => Collection<T>;
  }

  type CollectionProperties<T = any> = T[] | Collection<T>;

  interface CollectionPropertiesBase<T = any> {
    items?: T[];
  }

  type DateProperties = number | string | Date;

  interface ReactiveWatchOptions {
    readonly initial?: boolean;
    readonly sync?: boolean;
    readonly once?: boolean;
    readonly equals?: (newValue: any, oldValue: any) => boolean;
  }

  interface ReactiveListenerOptions<Target> {
    sync?: boolean;
    once?: boolean;
    onListenerAdd?: (target: Target) => void;
    onListenerRemove?: (target: Target) => void;
  }

  interface reactiveUtils {
    watch<T>(getValue: () => T, callback: (newValue: T, oldValue: T) => void, options?: ReactiveWatchOptions): IHandle;
    when<T>(
      getValue: () => T | null | undefined,
      callback: (newValue: T, oldValue: T | null) => void,
      options?: ReactiveWatchOptions,
    ): IHandle;
    on<T extends Evented | EventTarget>(
      getTarget: () => T | null | undefined,
      eventName: string,
      callback: (value: any) => void,
      options?: ReactiveListenerOptions<T>,
    ): IHandle;
    once<T>(getValue: () => T, signal?: AbortSignal | { signal?: AbortSignal } | null): Promise<T>;
    whenOnce<T>(
      getValue: () => T | null | undefined,
      signal?: AbortSignal | { signal?: AbortSignal } | null,
    ): Promise<T>;
  }

  export const reactiveUtils: reactiveUtils;

  export type BaseDynamicLayerLayerviewCreateErrorEventHandler = (
    event: BaseDynamicLayerLayerviewCreateErrorEvent,
  ) => void;

  export type BaseDynamicLayerLayerviewCreateEventHandler = (event: BaseDynamicLayerLayerviewCreateEvent) => void;

  export type BaseDynamicLayerLayerviewDestroyEventHandler = (event: BaseDynamicLayerLayerviewDestroyEvent) => void;

  export type BaseDynamicLayerRefreshEventHandler = (event: BaseDynamicLayerRefreshEvent) => void;

  export type BasemapLayerListTriggerActionEventHandler = (event: BasemapLayerListTriggerActionEvent) => void;

  export type BaseTileLayerLayerviewCreateErrorEventHandler = (event: BaseTileLayerLayerviewCreateErrorEvent) => void;

  export type BaseTileLayerLayerviewCreateEventHandler = (event: BaseTileLayerLayerviewCreateEvent) => void;

  export type BaseTileLayerLayerviewDestroyEventHandler = (event: BaseTileLayerLayerviewDestroyEvent) => void;

  export type BaseTileLayerRefreshEventHandler = (event: BaseTileLayerRefreshEvent) => void;

  export type BingMapsLayerLayerviewCreateErrorEventHandler = (event: BingMapsLayerLayerviewCreateErrorEvent) => void;

  export type BingMapsLayerLayerviewCreateEventHandler = (event: BingMapsLayerLayerviewCreateEvent) => void;

  export type BingMapsLayerLayerviewDestroyEventHandler = (event: BingMapsLayerLayerviewDestroyEvent) => void;

  export type BingMapsLayerRefreshEventHandler = (event: BingMapsLayerRefreshEvent) => void;

  export type BookmarksBookmarkEditEventHandler = (event: BookmarksBookmarkEditEvent) => void;

  export type BookmarksBookmarkSelectEventHandler = (event: BookmarksBookmarkSelectEvent) => void;

  export type CatalogLayerLayerviewCreateErrorEventHandler = (event: CatalogLayerLayerviewCreateErrorEvent) => void;

  export type CatalogLayerLayerviewCreateEventHandler = (event: CatalogLayerLayerviewCreateEvent) => void;

  export type CatalogLayerLayerviewDestroyEventHandler = (event: CatalogLayerLayerviewDestroyEvent) => void;

  export type CatalogLayerListTriggerActionEventHandler = (event: CatalogLayerListTriggerActionEvent) => void;

  export type CatalogLayerListViewModelTriggerActionEventHandler = (
    event: CatalogLayerListViewModelTriggerActionEvent,
  ) => void;

  export type CatalogLayerRefreshEventHandler = (event: CatalogLayerRefreshEvent) => void;

  export type CredentialDestroyEventHandler = (event: CredentialDestroyEvent) => void;

  export type CredentialTokenChangeEventHandler = (event: CredentialTokenChangeEvent) => void;

  export type CSVLayerLayerviewCreateErrorEventHandler = (event: CSVLayerLayerviewCreateErrorEvent) => void;

  export type CSVLayerLayerviewCreateEventHandler = (event: CSVLayerLayerviewCreateEvent) => void;

  export type CSVLayerLayerviewDestroyEventHandler = (event: CSVLayerLayerviewDestroyEvent) => void;

  export type CSVLayerRefreshEventHandler = (event: CSVLayerRefreshEvent) => void;

  export type EditorSketchCreateEventHandler = (event: EditorSketchCreateEvent) => void;

  export type EditorSketchUpdateEventHandler = (event: EditorSketchUpdateEvent) => void;

  export type EditorViewModelSketchCreateEventHandler = (event: EditorViewModelSketchCreateEvent) => void;

  export type EditorViewModelSketchUpdateEventHandler = (event: EditorViewModelSketchUpdateEvent) => void;

  export type ElevationSamplerChangedEventHandler = (event: ElevationSamplerChangedEvent) => void;

  export type FeatureFormSubmitEventHandler = (event: FeatureFormSubmitEvent) => void;

  export type FeatureFormValueChangeEventHandler = (event: FeatureFormValueChangeEvent) => void;

  export type FeatureFormViewModelSubmitEventHandler = (event: FeatureFormViewModelSubmitEvent) => void;

  export type FeatureFormViewModelValueChangeEventHandler = (event: FeatureFormViewModelValueChangeEvent) => void;

  export type FeatureLayerEditsEventHandler = (event: FeatureLayerEditsEvent) => void;

  export type FeatureLayerLayerviewCreateErrorEventHandler = (event: FeatureLayerLayerviewCreateErrorEvent) => void;

  export type FeatureLayerLayerviewCreateEventHandler = (event: FeatureLayerLayerviewCreateEvent) => void;

  export type FeatureLayerLayerviewDestroyEventHandler = (event: FeatureLayerLayerviewDestroyEvent) => void;

  export type FeatureLayerRefreshEventHandler = (event: FeatureLayerRefreshEvent) => void;

  export type FeaturesTriggerActionEventHandler = (event: FeaturesTriggerActionEvent) => void;

  export type FeaturesViewModelTriggerActionEventHandler = (event: FeaturesViewModelTriggerActionEvent) => void;

  export type FeatureTableCellClickEventHandler = (event: FeatureTableCellClickEvent) => void;

  export type FeatureTableCellDblclickEventHandler = (event: FeatureTableCellDblclickEvent) => void;

  export type FeatureTableCellKeydownEventHandler = (event: FeatureTableCellKeydownEvent) => void;

  export type FeatureTableCellPointeroutEventHandler = (event: FeatureTableCellPointeroutEvent) => void;

  export type FeatureTableCellPointeroverEventHandler = (event: FeatureTableCellPointeroverEvent) => void;

  export type FeatureTableColumnReorderEventHandler = (event: FeatureTableColumnReorderEvent) => void;

  export type FeatureTableViewModelCellClickEventHandler = (event: FeatureTableViewModelCellClickEvent) => void;

  export type FeatureTableViewModelCellDblclickEventHandler = (event: FeatureTableViewModelCellDblclickEvent) => void;

  export type FeatureTableViewModelCellKeydownEventHandler = (event: FeatureTableViewModelCellKeydownEvent) => void;

  export type FeatureTableViewModelCellPointeroutEventHandler = (
    event: FeatureTableViewModelCellPointeroutEvent,
  ) => void;

  export type FeatureTableViewModelCellPointeroverEventHandler = (
    event: FeatureTableViewModelCellPointeroverEvent,
  ) => void;

  export type FeatureTableViewModelColumnReorderEventHandler = (event: FeatureTableViewModelColumnReorderEvent) => void;

  export type FeatureTemplatesSelectEventHandler = (event: FeatureTemplatesSelectEvent) => void;

  export type FeatureTemplatesViewModelSelectEventHandler = (event: FeatureTemplatesViewModelSelectEvent) => void;

  export type GeoJSONLayerEditsEventHandler = (event: GeoJSONLayerEditsEvent) => void;

  export type GeoJSONLayerLayerviewCreateErrorEventHandler = (event: GeoJSONLayerLayerviewCreateErrorEvent) => void;

  export type GeoJSONLayerLayerviewCreateEventHandler = (event: GeoJSONLayerLayerviewCreateEvent) => void;

  export type GeoJSONLayerLayerviewDestroyEventHandler = (event: GeoJSONLayerLayerviewDestroyEvent) => void;

  export type GeoJSONLayerRefreshEventHandler = (event: GeoJSONLayerRefreshEvent) => void;

  export type GeoRSSLayerLayerviewCreateErrorEventHandler = (event: GeoRSSLayerLayerviewCreateErrorEvent) => void;

  export type GeoRSSLayerLayerviewCreateEventHandler = (event: GeoRSSLayerLayerviewCreateEvent) => void;

  export type GeoRSSLayerLayerviewDestroyEventHandler = (event: GeoRSSLayerLayerviewDestroyEvent) => void;

  export type GeoRSSLayerRefreshEventHandler = (event: GeoRSSLayerRefreshEvent) => void;

  export type HeatmapSliderThumbChangeEventHandler = (event: HeatmapSliderThumbChangeEvent) => void;

  export type HeatmapSliderThumbDragEventHandler = (event: HeatmapSliderThumbDragEvent) => void;

  export type HistogramRangeSliderMaxChangeEventHandler = (event: HistogramRangeSliderMaxChangeEvent) => void;

  export type HistogramRangeSliderMinChangeEventHandler = (event: HistogramRangeSliderMinChangeEvent) => void;

  export type HistogramRangeSliderSegmentDragEventHandler = (event: HistogramRangeSliderSegmentDragEvent) => void;

  export type HistogramRangeSliderThumbChangeEventHandler = (event: HistogramRangeSliderThumbChangeEvent) => void;

  export type HistogramRangeSliderThumbDragEventHandler = (event: HistogramRangeSliderThumbDragEvent) => void;

  export type HomeGoEventHandler = (event: HomeGoEvent) => void;

  export type HomeViewModelGoEventHandler = (event: HomeViewModelGoEvent) => void;

  export type IdentityManagerCredentialCreateEventHandler = (event: IdentityManagerCredentialCreateEvent) => void;

  export type IdentityManagerDialogCreateEventHandler = (event: IdentityManagerDialogCreateEvent) => void;

  export type ImageryLayerLayerviewCreateErrorEventHandler = (event: ImageryLayerLayerviewCreateErrorEvent) => void;

  export type ImageryLayerLayerviewCreateEventHandler = (event: ImageryLayerLayerviewCreateEvent) => void;

  export type ImageryLayerLayerviewDestroyEventHandler = (event: ImageryLayerLayerviewDestroyEvent) => void;

  export type ImageryLayerRefreshEventHandler = (event: ImageryLayerRefreshEvent) => void;

  export type LayerLayerviewCreateErrorEventHandler = (event: LayerLayerviewCreateErrorEvent) => void;

  export type LayerLayerviewCreateEventHandler = (event: LayerLayerviewCreateEvent) => void;

  export type LayerLayerviewDestroyEventHandler = (event: LayerLayerviewDestroyEvent) => void;

  export type LayerListTriggerActionEventHandler = (event: LayerListTriggerActionEvent) => void;

  export type LayerListViewModelTriggerActionEventHandler = (event: LayerListViewModelTriggerActionEvent) => void;

  export type LocateLocateErrorEventHandler = (event: LocateLocateErrorEvent) => void;

  export type LocateLocateEventHandler = (event: LocateLocateEvent) => void;

  export type LocateViewModelLocateErrorEventHandler = (event: LocateViewModelLocateErrorEvent) => void;

  export type LocateViewModelLocateEventHandler = (event: LocateViewModelLocateEvent) => void;

  export type MapImageLayerLayerviewCreateErrorEventHandler = (event: MapImageLayerLayerviewCreateErrorEvent) => void;

  export type MapImageLayerLayerviewCreateEventHandler = (event: MapImageLayerLayerviewCreateEvent) => void;

  export type MapImageLayerLayerviewDestroyEventHandler = (event: MapImageLayerLayerviewDestroyEvent) => void;

  export type MapImageLayerRefreshEventHandler = (event: MapImageLayerRefreshEvent) => void;

  export interface AreaMeasurementAnalysis extends Accessor, Clonable {}

  export class AreaMeasurementAnalysis {
    /**
     * AreaMeasurementAnalysis computes the area of a polygonal region and displays the measurement in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html)
     */

    constructor(properties?: AreaMeasurementAnalysisProperties);

    /**
     * Polygon whose area is to be computed and displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html#geometry)
     */
    geometry: Polygon;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the computed area in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html#unit)
     */
    unit: SystemOrAreaUnit;
  }

  interface AreaMeasurementAnalysisProperties {
    /**
     * Polygon whose area is to be computed and displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html#geometry)
     */
    geometry?: PolygonProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the computed area in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
  }

  export interface DimensionAnalysis extends Accessor, Clonable {}

  export class DimensionAnalysis {
    /**
     * DimensionAnalysis enables the creation and display of measurement annotations for lengths and distances in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html)
     */

    constructor(properties?: DimensionAnalysisProperties);

    /**
     * A list of dimensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html#dimensions)
     */
    dimensions: Collection<LengthDimension>;
    /**
     * The style defines how the dimension objects of this analysis are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html#style)
     */
    style: DimensionSimpleStyle;
  }

  interface DimensionAnalysisProperties {
    /**
     * A list of dimensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html#dimensions)
     */
    dimensions?: CollectionProperties<LengthDimensionProperties>;
    /**
     * The style defines how the dimension objects of this analysis are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html#style)
     */
    style?: DimensionSimpleStyleProperties;
  }

  export interface DimensionSimpleStyle extends Accessor, Clonable {}

  export class DimensionSimpleStyle {
    /**
     * Style that specifies how dimensions and their labels are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html)
     */

    constructor(properties?: DimensionSimpleStyleProperties);

    /**
     * Color of dimension lines.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#color)
     */
    color: Color;
    /**
     * Size of text in dimension labels in points.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#fontSize)
     */
    fontSize: number;
    /**
     * Width of dimension lines in points.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#lineSize)
     */
    lineSize: number;
    /**
     * Background color of dimension labels.
     *
     * @default [255, 255, 255, 0.6]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#textBackgroundColor)
     */
    textBackgroundColor: Color;
    /**
     * Color of text in dimension labels.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#textColor)
     */
    textColor: Color;

    readonly type: "simple";
  }

  interface DimensionSimpleStyleProperties {
    /**
     * Color of dimension lines.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#color)
     */
    color?: Color | number[] | string;
    /**
     * Size of text in dimension labels in points.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#fontSize)
     */
    fontSize?: number | string;
    /**
     * Width of dimension lines in points.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#lineSize)
     */
    lineSize?: number | string;
    /**
     * Background color of dimension labels.
     *
     * @default [255, 255, 255, 0.6]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#textBackgroundColor)
     */
    textBackgroundColor?: Color | number[] | string;
    /**
     * Color of text in dimension labels.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionSimpleStyle.html#textColor)
     */
    textColor?: Color | number[] | string;
  }

  export interface DirectLineMeasurementAnalysis extends Accessor, Clonable {}

  export class DirectLineMeasurementAnalysis {
    /**
     * DirectLineMeasurementAnalysis computes the distance between two points and displays the measurement in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html)
     */

    constructor(properties?: DirectLineMeasurementAnalysisProperties);

    /**
     * Ending point for the measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#endPoint)
     */
    endPoint: Point;
    /**
     * Starting point for the measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#startPoint)
     */
    startPoint: Point;
    /**
     * Unit system (imperial, metric) or specific unit used for computing the distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#unit)
     */
    unit: SystemOrLengthUnit;
  }

  interface DirectLineMeasurementAnalysisProperties {
    /**
     * Ending point for the measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#endPoint)
     */
    endPoint?: PointProperties;
    /**
     * Starting point for the measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#startPoint)
     */
    startPoint?: PointProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for computing the distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
  }

  export interface LengthDimension extends Accessor, Clonable {}

  export class LengthDimension {
    /**
     * LengthDimension enables the measurement of linear distances between the specified [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint) points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html)
     */

    constructor(properties?: LengthDimensionProperties);

    /**
     * Ending point for the dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint)
     */
    endPoint: Point;
    /**
     * The type of length that should be measured between the [startPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint) and [endPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint).
     *
     * @default "direct"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#measureType)
     */
    measureType: "direct" | "horizontal" | "vertical";
    /**
     * Styling option that controls the shortest distance from the [startPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint) or [endPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint) to the dimension line in meters.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#offset)
     */
    offset: number;
    /**
     * The orientation determines the relative direction the dimension line is extended to.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#orientation)
     */
    orientation: number;
    /**
     * Starting point for the dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint)
     */
    startPoint: Point;
  }

  interface LengthDimensionProperties {
    /**
     * Ending point for the dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint)
     */
    endPoint?: PointProperties;
    /**
     * The type of length that should be measured between the [startPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint) and [endPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint).
     *
     * @default "direct"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#measureType)
     */
    measureType?: "direct" | "horizontal" | "vertical";
    /**
     * Styling option that controls the shortest distance from the [startPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint) or [endPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#endPoint) to the dimension line in meters.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#offset)
     */
    offset?: number;
    /**
     * The orientation determines the relative direction the dimension line is extended to.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#orientation)
     */
    orientation?: number;
    /**
     * Starting point for the dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html#startPoint)
     */
    startPoint?: PointProperties;
  }

  export interface LineOfSightAnalysis extends Accessor, Clonable {}

  export class LineOfSightAnalysis {
    /**
     * LineOfSightAnalysis computes the line of sight from a single observer position towards a set of targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html)
     */

    constructor(properties?: LineOfSightAnalysisProperties);

    /**
     * Observer location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#observer)
     */
    observer: LineOfSightAnalysisObserver;
    /**
     * Target locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#targets)
     */
    targets: Collection<LineOfSightAnalysisTarget>;
  }

  interface LineOfSightAnalysisProperties {
    /**
     * Observer location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#observer)
     */
    observer?: LineOfSightAnalysisObserverProperties;
    /**
     * Target locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#targets)
     */
    targets?: CollectionProperties<LineOfSightAnalysisTargetProperties>;
  }

  export interface FeatureReference {
    /**
     * Reference to a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#FeatureReference)
     */
    layer: FeatureReferenceLayer;
    /**
     * Feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#FeatureReference)
     */
    attributes: any;
  }

  export interface FeatureReferenceLayer {
    /**
     * The layer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#FeatureReference)
     */
    id: string | number;
    /**
     * The object id field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#FeatureReference)
     */
    objectIdField?: string;
  }

  export interface LineOfSightAnalysisObserver extends Accessor, Clonable {}

  export class LineOfSightAnalysisObserver {
    /**
     * The LineOfSightAnalysisObserver represents an observer of a [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html)
     */

    constructor(properties?: LineOfSightAnalysisObserverProperties);

    /**
     * Specifies how the observer is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#elevationInfo)
     */
    elevationInfo: LineOfSightAnalysisObserverElevationInfo;
    /**
     * References a feature which is excluded from the intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#feature)
     */
    feature: FeatureReference;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the position of the observer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#position)
     */
    position: Point;
  }

  interface LineOfSightAnalysisObserverProperties {
    /**
     * Specifies how the observer is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#elevationInfo)
     */
    elevationInfo?: LineOfSightAnalysisObserverElevationInfo;
    /**
     * References a feature which is excluded from the intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#feature)
     */
    feature?: FeatureReference;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the position of the observer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#position)
     */
    position?: PointProperties;
  }

  export interface LineOfSightAnalysisObserverElevationInfo {
    /**
     * Defines how the observer is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the observer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisObserver.html#elevationInfo)
     */
    offset?: number;
  }

  export interface LineOfSightAnalysisTarget extends Accessor, Clonable {}

  export class LineOfSightAnalysisTarget {
    /**
     * The LineOfSightAnalysisTarget represents a target of a [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html)
     */

    constructor(properties?: LineOfSightAnalysisTargetProperties);

    /**
     * Specifies how the target is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#elevationInfo)
     */
    elevationInfo: LineOfSightAnalysisTargetElevationInfo;
    /**
     * References a feature which is excluded from the intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#feature)
     */
    feature: FeatureReference;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the position of the target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#position)
     */
    position: Point;
  }

  interface LineOfSightAnalysisTargetProperties {
    /**
     * Specifies how the target is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#elevationInfo)
     */
    elevationInfo?: LineOfSightAnalysisTargetElevationInfo;
    /**
     * References a feature which is excluded from the intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#feature)
     */
    feature?: FeatureReference;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the position of the target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#position)
     */
    position?: PointProperties;
  }

  export interface LineOfSightAnalysisTargetElevationInfo {
    /**
     * Defines how the target is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysisTarget.html#elevationInfo)
     */
    offset?: number;
  }

  export interface SliceAnalysis extends Accessor, Clonable {}

  export class SliceAnalysis {
    /**
     * SliceAnalysis can be used to programmatically create a plane that slices through 3D features in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html)
     */

    constructor(properties?: SliceAnalysisProperties);

    /**
     * The shape used to slice elements in a 3D scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#shape)
     */
    shape: SlicePlane;
    /**
     * Whether the [shape](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#shape) supports a tilt angle or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#tiltEnabled)
     */
    tiltEnabled: boolean;

    readonly type: "slice";
  }

  interface SliceAnalysisProperties {
    /**
     * The shape used to slice elements in a 3D scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#shape)
     */
    shape?: SlicePlaneProperties;
    /**
     * Whether the [shape](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#shape) supports a tilt angle or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html#tiltEnabled)
     */
    tiltEnabled?: boolean;
  }

  export interface SlicePlane extends Accessor, JSONSupport, Clonable {}

  export class SlicePlane {
    /**
     * Provides the shape definition of a slice plane for a [Slice](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html) widget or [SliceAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html)
     */

    constructor(properties?: SlicePlaneProperties);

    /**
     * The heading angle (in degrees) of the slice plane.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#heading)
     */
    heading: number;
    /**
     * The height of the slice plane.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#height)
     */
    height: number;
    /**
     * A point specifying the position of the center of the plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#position)
     */
    position: Point;
    /**
     * The tilt angle (in degrees) of the slice plane.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#tilt)
     */
    tilt: number;
    /**
     * The string value representing the type of the slice shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#type)
     */
    readonly type: "plane";
    /**
     * The width of the slice plane.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#width)
     */
    width: number;

    static fromJSON(json: any): SlicePlane;
  }

  interface SlicePlaneProperties {
    /**
     * The heading angle (in degrees) of the slice plane.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#heading)
     */
    heading?: number;
    /**
     * The height of the slice plane.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#height)
     */
    height?: number;
    /**
     * A point specifying the position of the center of the plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#position)
     */
    position?: PointProperties;
    /**
     * The tilt angle (in degrees) of the slice plane.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#tilt)
     */
    tilt?: number;
    /**
     * The width of the slice plane.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SlicePlane.html#width)
     */
    width?: number;
  }

  export interface Viewshed extends Accessor, Clonable {}

  export class Viewshed {
    /**
     * Viewshed defines the geometry for a viewshed analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html)
     */

    constructor(properties?: ViewshedProperties);

    /**
     * The maximum distance from the observer in which to perform the viewshed analysis (in meters).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#farDistance)
     */
    farDistance: number;
    /**
     * References a feature from which the observer is internally offset, provided that its geometry faces are close enough to the observer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#feature)
     */
    feature: ViewshedFeatureReference;
    /**
     * The compass heading of the observer's view direction (in degrees).
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#heading)
     */
    heading: number;
    /**
     * The horizontal field of view (FOV) angle defines the width of the scope being analyzed (in degrees).
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#horizontalFieldOfView)
     */
    horizontalFieldOfView: number;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) specifying the position the viewshed is calculated from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#observer)
     */
    observer: Point;
    /**
     * The tilt of the observer's view direction (in degrees).
     *
     * @default 90
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#tilt)
     */
    tilt: number;
    /**
     * The vertical field of view (FOV) angle defines the height of the scope being analyzed (in degrees).
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#verticalFieldOfView)
     */
    verticalFieldOfView: number;
  }

  interface ViewshedProperties {
    /**
     * The maximum distance from the observer in which to perform the viewshed analysis (in meters).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#farDistance)
     */
    farDistance?: number;
    /**
     * References a feature from which the observer is internally offset, provided that its geometry faces are close enough to the observer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#feature)
     */
    feature?: ViewshedFeatureReference;
    /**
     * The compass heading of the observer's view direction (in degrees).
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#heading)
     */
    heading?: number;
    /**
     * The horizontal field of view (FOV) angle defines the width of the scope being analyzed (in degrees).
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#horizontalFieldOfView)
     */
    horizontalFieldOfView?: number;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) specifying the position the viewshed is calculated from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#observer)
     */
    observer?: PointProperties;
    /**
     * The tilt of the observer's view direction (in degrees).
     *
     * @default 90
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#tilt)
     */
    tilt?: number;
    /**
     * The vertical field of view (FOV) angle defines the height of the scope being analyzed (in degrees).
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#verticalFieldOfView)
     */
    verticalFieldOfView?: number;
  }

  export interface ViewshedFeatureReference {
    /**
     * Reference to a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#FeatureReference)
     */
    layer: ViewshedFeatureReferenceLayer;
    /**
     * Feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#FeatureReference)
     */
    attributes: any;
  }

  export interface ViewshedAnalysis extends Accessor, Clonable {}

  export class ViewshedAnalysis {
    /**
     * ViewshedAnalysis enables the creation and display of viewshed and view dome type of visibility analysis in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html)
     */

    constructor(properties?: ViewshedAnalysisProperties);

    readonly type: "viewshed";
    /**
     * A list of viewsheds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html#viewsheds)
     */
    viewsheds: Collection<Viewshed>;
  }

  interface ViewshedAnalysisProperties {
    /**
     * A list of viewsheds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html#viewsheds)
     */
    viewsheds?: CollectionProperties<ViewshedProperties>;
  }

  export interface ViewshedFeatureReferenceLayer {
    /**
     * The layer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#FeatureReference)
     */
    id: string | number;
    /**
     * The object id field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-Viewshed.html#FeatureReference)
     */
    objectIdField?: string;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Basemap utils for Components.
   */
  interface basemapUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Get the thumbnail url of an arcgis basemap.
     */
    getBasemapThumbnailUrl(basemap: Basemap): string;
  }

  export const basemapUtils: basemapUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Draw utils for Components.
   */
  interface drawUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * performs translation of given geometry dx, dy: how much to translate the geometry in x and y direction.
     */
    move(geometry: any): any;
  }

  export const drawUtils: drawUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Extract default units from map of view.
   */
  interface getDefaultUnits {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Extracts the default unit preferences defined the portal instance of a webmap/webscene used in a a view.
     */
    getDefaultUnitForView(view: MapView | SceneView): "imperial" | "metric";
  }

  export const getDefaultUnits: getDefaultUnits;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Background theme utils for Components.
   */
  interface gfxUtils {
    defaultThematicColor: Color;
  }

  export const gfxUtils: gfxUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Layer origin utils for Components.
   */
  interface layerOriginUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Clears property overrides in the given layer.
     */
    clearOverrides(layer: any, origin: any): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Disconnects the given layer from its source.
     */
    disconnectFromSource(layer: any, origin: any): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Returns origin information for properties of the given layer and overall status of the layer.
     */
    getLayerOriginInfo(layer: any, origin: any): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Indicates whether origin info can be returned for the given layer and its properties.
     */
    isSupportedLayer(layer: any): boolean;
  }

  export const layerOriginUtils: layerOriginUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Background theme utils for Components.
   */
  interface previewSymbol2D {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Determines the contrasting background theme ("light", "dark", or null) for a given symbol based on color luminance and a threshold.
     */
    getContrastingBackgroundTheme(symbol: any, threshold?: number): string;
  }

  export const previewSymbol2D: previewSymbol2D;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Style utils for Components.
   */
  interface styleUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Return the symbol URL for a given style item in the preferred format.
     */
    symbolUrlFromStyleItem(item: any, type: any, typePriorities: string[]): string;
  }

  export const styleUtils: styleUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * SVG utils for Components.
   */
  interface svgUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Computes and returns the bounding box that encompasses all given bounding boxes.
     */
    computeBBox(bboxes: any[]): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Calculates and returns the bounding box for a given shape.
     */
    getBoundingBox(shape: any): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Computes and returns a transformation matrix string for a given bounding box, considering scaling, rotation, and other transformations.
     */
    getTransformMatrix(
      bbox: any,
      width: number,
      height: number,
      overallStrokeWidth: number,
      scale: boolean,
      rotation: number,
      useRotationSize: boolean,
      offset?: number[],
      bloomSize?: number[],
    ): string;
  }

  export const svgUtils: svgUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Web Style Symbol utils for Components.
   */
  interface webStyleSymbolUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Fetch the symbol from a style by name:
     */
    fetchSymbolFromStyle(
      styleResponse: any,
      symbolName: string,
      context: any,
      type: any,
      styleItemRead: Function,
      options?: any,
    ): Promise<any>;
  }

  export const webStyleSymbolUtils: webStyleSymbolUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * VideoView utilities for the Excalibur application.
   */
  interface videoViewUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Creates a new VideoView instance.
     */
    createVideoView(properties: VideoViewParams): VideoView;
  }

  export const videoViewUtils: videoViewUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * VideoView
   */
  export interface VideoView {
    /**
     * The `id` or node representing the DOM element containing the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    container: HTMLDivElement;
    /**
     * Indicates if the browser focus is on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    focused: boolean;
    /**
     * The height of the view in pixels read from the view container element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    height: number;
    /**
     * The VideoLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    layer: VideoLayer;
    /**
     * The video view has satisfied all dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    ready: boolean;
    /**
     * Indicates whether the view is being resized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    resizing: boolean;
    /**
     * The height of the video view in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    videoHeight: number;
    /**
     * The width of the video view in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    videoWidth: number;
    /**
     * The width of the view in pixels read from the view container element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoView)
     */
    width: number;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * VideoViewParams
   */
  export interface VideoViewParams {
    /**
     * The `id` or node representing the DOM element containing the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoViewParams)
     */
    container: HTMLDivElement;
    /**
     * The VideoLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Excalibur-videoViewUtils.html#VideoViewParams)
     */
    layer: VideoLayer;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Sketch utilities for the experience builder application.
   */
  interface sketchUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Configures a Sketch widget to display custom tools.
     */
    setCustomTools(
      sketch: Sketch,
      toolOptions?: sketchUtilsSetCustomToolsToolOptions,
    ): Promise<ControlPointsGeoreference>;
  }

  export const sketchUtils: sketchUtils;

  export interface sketchUtilsSetCustomToolsToolOptions {
    /**
     * The text displayed in the tooltip for the custom tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    label?: string;
    /**
     * The string value indicating the [Calcite icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed for custom tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    icon?: string;
    /**
     * The unique identifier string for this custom tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    toolKey?: string;
    /**
     * The specific sketch create tool to use for the custom tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    toolName?: "point" | "polyline" | "polygon" | "rectangle" | "circle";
    /**
     * Specific options for the create tool being used by the custom tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    createOptions?: sketchUtilsSetCustomToolsToolOptionsCreateOptions;
  }

  export interface sketchUtilsSetCustomToolsToolOptionsCreateOptions {
    /**
     * Specifies how the graphic can be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    mode?: "hybrid" | "freehand" | "click";
    /**
     * Controls whether or not the width and height of the drawn geometry are kept equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-ExperienceBuilder-sketchUtils.html#setCustomTools)
     */
    preserveAspectRatio?: boolean;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Media layer utilities for the map viewer application.
   */
  interface mediaUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Creates a georeference for a media element and a view extent.
     */
    createDefaultControlPointsGeoreference(
      mediaElement: ImageElement | VideoElement,
      extent: Extent,
    ): Promise<ControlPointsGeoreference>;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Creates a georeference to be used in a MapView when authoring the source points of the element's georeference.
     */
    createLocalModeControlPointsGeoreference(mediaElement: ImageElement | VideoElement): ControlPointsGeoreference;
  }

  export const mediaUtils: mediaUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Table template utilities for the map viewer application.
   */
  interface templateUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Generates an AttributeTableTemplate based on the default layer fields.
     */
    createDefaultAttributeTableTemplateFromLayer(
      options: templateUtilsCreateDefaultAttributeTableTemplateFromLayerOptions,
    ): Promise<AttributeTableTemplate>;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Generates a TableTemplate based on the default layer fields.
     */
    createDefaultTableTemplateFromLayer(
      options: templateUtilsCreateDefaultTableTemplateFromLayerOptions,
    ): Promise<TableTemplate>;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Generates a TableTemplate based on a pre-existing AttributeTableTemplate.
     */
    createTableTemplateFromAttributeTableTemplate(
      options: templateUtilsCreateTableTemplateFromAttributeTableTemplateOptions,
    ): Promise<TableTemplate>;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Syncs a provided AttributeTableTemplate with the current state of a provided FeatureTable, using the 'columns' Collection on FeatureTable specifically.
     */
    syncAttributeTableTemplate(
      table: FeatureTable,
      template: AttributeTableTemplate,
      includeHiddenColumns?: AttributeTableTemplate,
    ): AttributeTableTemplate;
  }

  export const templateUtils: templateUtils;

  export interface templateUtilsCreateDefaultAttributeTableTemplateFromLayerOptions {
    /**
     * Layer from which to generate a default template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultAttributeTableTemplateFromLayer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * Indicates 'attachment' columns should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultAttributeTableTemplateFromLayer)
     */
    excludeAttachments?: boolean;
    /**
     * Indicates 'relationship' columns should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultAttributeTableTemplateFromLayer)
     */
    excludeRelationships?: boolean;
  }

  export interface templateUtilsCreateDefaultTableTemplateFromLayerOptions {
    /**
     * Layer from which to generate a default template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultTableTemplateFromLayer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * Indicates 'attachment' columns should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultTableTemplateFromLayer)
     */
    excludeAttachments?: boolean;
    /**
     * Indicates 'relationship' columns should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createDefaultTableTemplateFromLayer)
     */
    excludeRelationships?: boolean;
  }

  export interface templateUtilsCreateTableTemplateFromAttributeTableTemplateOptions {
    /**
     * Layer from which to generate a default template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createTableTemplateFromAttributeTableTemplate)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * Instance of an AttributeTableTemplate used to configure a new TableTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createTableTemplateFromAttributeTableTemplate)
     */
    template: AttributeTableTemplate;
    /**
     * Indicates columns should be generated for fields without associated elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-MapViewer-templateUtils.html#createTableTemplateFromAttributeTableTemplate)
     */
    includeHiddenFields?: boolean;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Color utilities for the Scene Viewer application.
   */
  interface SceneViewerColorUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Return true if a color is bright.
     */
    isBright(color: Color): boolean;
  }

  export const SceneViewerColorUtils: SceneViewerColorUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Development environment utilities used in the SceneViewer.
   */
  interface devEnvironmentUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Adjusts the "static.arcgis.com" part of an URL to point to the corresponding development hostname.
     */
    adjustStaticAGOUrl(url?: string, hostname?: string): string;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Check if we are running in a dev environment.
     */
    isDevEnvironment(hostname: string): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Checks whether for telemetry logging purposes, we are running in a development environment.
     */
    isTelemetryDevEnvironment(hostname: string): boolean;
  }

  export const devEnvironmentUtils: devEnvironmentUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Various layer utilities for the Scene Viewer application.
   */
  interface layerUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Get the origin of a property in the store.
     */
    originOfLayerProperty(
      layer: OperationalLayer,
      property: string,
    ): "defaults" | "service" | "portal-item" | "web-scene" | "web-map" | "link-chart" | "user";
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Reverts the value of a property to the value that was read from a specific origin.
     */
    revertLayerProperty(
      layer: OperationalLayer,
      property: string,
      origin: "service" | "portal-item" | "web-scene" | "web-map" | "link-chart",
    ): void;
  }

  export const layerUtils: layerUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Various utilities for the Scene Viewer application.
   */
  interface sceneViewerUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Checks for relative urls and returns whether or not this webscene allows the `saveAs()` operation.
     */
    canSaveAs(webscene: WebScene): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Helper function to close a catalog layer in the layer list.
     */
    closeCatalogLayer(layerList: LayerList): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Create a new TilingScheme based on the tileInfo
     */
    createTilingScheme(tileInfo: TileInfo): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Gets whether the editor still has any pending edits which should/could prevent the widget being discarded/destroyed (e.g.
     */
    editorHasPendingEdits(viewModel: EditorViewModel): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Navigates back to the previous page of the `Editor` widget.
     */
    editorNavigateBack(viewModel: EditorViewModel): Promise<void>;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Generates, if necessary, an equivalent TileInfo but with a different pixel size (256 or 512).
     */
    getCompatibleTileInfoForVTL(tileInfo: TileInfo, expectedTileSize: number): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Initializes all the schema validators so that validation is performed when saving web scenes or layers.
     */
    initializeSchemaValidators(): void;

    isHostedAgolServiceUrl(url: string): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Check if the error is a schema validation error.
     */
    isSchemaValidationError(saveError: void): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Checks whether the provided spatial reference is supported in the provided viewing mode.
     */
    isSpatialReferenceSupported(spatialReference: SpatialReference, viewingMode: "global" | "local"): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Check if a given TileInfo is supported.
     */
    isSupportedTileInfo(tileInfo: TileInfo): boolean;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Render svg vnode for elements provided in the descriptor
     */
    renderSVG(swatch: any[][], width: number, height: number, options?: any): any;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Warn about problem when trying to display message about the save state.
     */
    saveStateWarning(warningMessage: string, error: any): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Allows setting overrides for the `defaultsFromMap` used by a view.
     */
    setDefaultsFromMapSettings(view: SceneView, settings: DefaultsFromMapSettings): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Returns the start position of the cross hair while dragging to zoom.
     */
    zoomDragBeginPoint(view: SceneView): number[];
  }

  export const sceneViewerUtils: sceneViewerUtils;

  export interface DefaultsFromMapRequiredComponents {
    tileInfo: boolean;

    heightModelInfo: boolean;

    extent: boolean;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Settings to be used for the `defaultsFromMap` of a view.
   */
  export interface DefaultsFromMapSettings {
    /**
     * The default spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-SceneViewer-sceneViewerUtils.html#DefaultsFromMapSettings)
     */
    defaultSpatialReference?: SpatialReference;
    /**
     * Function which checks if a spatial reference is supported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-SceneViewer-sceneViewerUtils.html#DefaultsFromMapSettings)
     */
    isSpatialReferenceSupported?: IsSpatialReferenceSupportedCallback;
    /**
     * Layers to prioritize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-SceneViewer-sceneViewerUtils.html#DefaultsFromMapSettings)
     */
    priorityLayers?: Layer[];
    /**
     * The required components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-SceneViewer-sceneViewerUtils.html#DefaultsFromMapSettings)
     */
    required?: DefaultsFromMapRequiredComponents;
  }

  export type IsSpatialReferenceSupportedCallback = (spatialReference: SpatialReference, layer?: Layer) => void;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Symbol utilities for the Scene Viewer application.
   */
  interface SceneViewerSymbolUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Returns a promise resolving to an array containing the width, height and depth of the object symbol layer resource.
     */
    computeObjectLayerSize(symbolLayer: ObjectSymbol3DLayer): Promise<number[]>;
  }

  export const SceneViewerSymbolUtils: SceneViewerSymbolUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Mesh utilities for the Urban application.
   */
  interface UrbanMeshUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Converts a mesh to a new vertex space.
     */
    convertVertexSpaceEllipsoid(
      mesh: Mesh,
      targetVertexSpace: MeshGeoreferencedVertexSpace | MeshLocalVertexSpace,
      options?: meshUtilsConvertVertexSpaceEllipsoidOptions,
    ): Promise<Mesh>;
  }

  export const UrbanMeshUtils: UrbanMeshUtils;

  export interface meshUtilsConvertVertexSpaceEllipsoidOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-Urban-meshUtils.html#convertVertexSpaceEllipsoid)
     */
    signal?: AbortSignal;
  }

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Sketch utilities for the WebEditor application.
   */
  interface WebEditorSketchUtils {
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Hides all the tooltip visible elements.
     */
    hideAllVisibleElements(sketchViewModel: SketchViewModel): void;
    /**
     * Note: reserved for internal use only, this is not part of the stable public API.
     *
     * Shows a tooltip and focuses on a specific field.
     */
    showTooltipAndFocusField(
      sketchViewModel: SketchViewModel,
      fieldName:
        | "area"
        | "distance"
        | "direction"
        | "elevation"
        | "orientation"
        | "radius"
        | "rotation"
        | "scale"
        | "size"
        | "totalLength"
        | "coordinates",
      options?: ShowTooltipAndFocusFieldOptions,
    ): Promise<void>;
  }

  export const WebEditorSketchUtils: WebEditorSketchUtils;

  /**
   * Note: reserved for internal use only, this is not part of the stable public API.
   *
   * Options used to show the tooltip.
   */
  export interface ShowTooltipAndFocusFieldOptions {
    /**
     * AbortSignal which can be used to cancel the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-WebEditor-sketchUtils.html#ShowTooltipAndFocusFieldOptions)
     */
    signal?: AbortSignal;
    /**
     * The position where the tooltip should be shown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-WebEditor-sketchUtils.html#ShowTooltipAndFocusFieldOptions)
     */
    position?: number[];
    /**
     * Whether the tooltip should be hidden when it loses focus.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-WebEditor-sketchUtils.html#ShowTooltipAndFocusFieldOptions)
     */
    hideOnBlur?: boolean;

    /**
     * Callback function that is called when the tooltip is opened.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-WebEditor-sketchUtils.html#ShowTooltipAndFocusFieldOptions)
     */
    onOpen(): void;
    /**
     * Callback function that is called when the tooltip is closed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-applications-WebEditor-sketchUtils.html#ShowTooltipAndFocusFieldOptions)
     */
    onClose(): void;
  }

  /**
   * This module allows you to evaluate [Arcade expressions](https://developers.arcgis.com/javascript/latest/arcade/) outside traditional ArcGIS Arcade [profiles](https://developers.arcgis.com/javascript/latest/arcade/#where-can-i-use-arcade).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html)
   */
  interface arcade {
    /**
     * Compiles an Arcade expression and its profile into an executor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#createArcadeExecutor)
     */
    createArcadeExecutor(script: string, profile: Profile): Promise<ArcadeExecutor>;
    /**
     * Creates a [Profile](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#Profile) definition for an Arcade profile implemented in the ArcGIS Maps SDK for JavaScript.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#createArcadeProfile)
     */
    createArcadeProfile(
      profileName:
        | "aggregate-field"
        | "form-constraint"
        | "feature-z"
        | "field-calculation"
        | "form-calculation"
        | "labeling"
        | "popup"
        | "popup-element"
        | "feature-reduction-popup"
        | "feature-reduction-popup-element"
        | "visualization",
    ): Profile;
  }

  export const arcade: arcade;

  /**
   * An executor provides a synchronous (if applicable) and asynchronous function used to evaluate the compiled Arcade expression with the inputs of the declared profile variables.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
   */
  export interface ArcadeExecutor {
    /**
     * A function that can be invoked to evaluate the compiled expression for a set of profile variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
     */
    execute: ExecuteFunction;
    /**
     * Asynchronous function that evaluates the compiled expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
     */
    executeAsync: ExecuteFunctionAsync;
    /**
     * References the names of fields used in the compiled expression when working with the `$feature` profile variable or functions that expect field names, such as `Expects`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
     */
    fieldsUsed: string[];
    /**
     * Indicates whether the expression accesses or uses a feature's geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
     */
    geometryUsed: boolean;
    /**
     * Indicates whether the compiled expression accesses data using a FeatureSet function and therefore must be executed using the `executeAsync` function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArcadeExecutor)
     */
    isAsync: boolean;
  }

  /**
   * The type definition of a profile variable of type `array`.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayElementType)
   */
  export interface ArrayElementType {
    /**
     * The Arcade data type of the variable.
     *
     * [Read more...](global.html#type)
     */
    type:
      | "array"
      | "dictionary"
      | "feature"
      | "featureSet"
      | "featureSetCollection"
      | "geometry"
      | "number"
      | "text"
      | "date"
      | "dateOnly"
      | "time"
      | "boolean"
      | "knowledgeGraph";
    /**
     * Only applicable when `type` is `dictionary`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayElementType)
     */
    properties?: ProfileVariable[];
    /**
     * Only applicable when `type` is `array`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayElementType)
     */
    elementType?: ArrayElementType;
  }

  /**
   * The type definition for an Arcade array profile variable.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayVariable)
   */
  export interface ArrayVariable {
    /**
     * The name of the profile variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayVariable)
     */
    name: string;
    /**
     * The Arcade data type of the variable.
     *
     * [Read more...](global.html#type)
     */
    type: "array";
    /**
     * The type definition of the Array's items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ArrayVariable)
     */
    elementType?: ArrayElementType;
  }

  /**
   * The type definition for an Arcade dictionary profile variable.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#DictionaryVariable)
   */
  export interface DictionaryVariable {
    /**
     * The name of the profile variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#DictionaryVariable)
     */
    name: string;
    /**
     * The Arcade data type of the variable.
     *
     * [Read more...](global.html#type)
     */
    type: "dictionary";
    /**
     * The type definitions of the dictionary's properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#DictionaryVariable)
     */
    properties?: ProfileVariable[];
  }

  /**
   * The execution context of the Arcade expression.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ExecuteContext)
   */
  export interface ExecuteContext {
    /**
     * The expected spatial reference of input geometries in geometry functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ExecuteContext)
     */
    spatialReference?: SpatialReference;
    /**
     * _Since 4.28_ Defines the default time zone in which to create and display Arcade date types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ExecuteContext)
     */
    timeZone?: string;
  }

  export type ExecuteFunction = (profileVariableInstances: any, context?: ExecuteContext) => ResultType;

  export type ExecuteFunctionAsync = (profileVariableInstances: any, context?: ExecuteContext) => Promise<ResultType>;

  /**
   * Layer types that may be used to back FeatureSets in Arcade expressions.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#FeatureSetLayer)
   */
  export type FeatureSetLayer =
    | FeatureLayer
    | CSVLayer
    | GeoJSONLayer
    | WFSLayer
    | OrientedImageryLayer
    | SubtypeGroupLayer
    | CatalogLayer
    | SubtypeSublayer
    | CatalogFootprintLayer;

  /**
   * Defines the definitions of a profile's input variables.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#Profile)
   */
  export interface Profile {
    /**
     * The Arcade type definitions for profile variables used as input to the compiled expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#Profile)
     */
    variables: ProfileVariable[];
  }

  /**
   * The type definition of a profile variable.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ProfileVariable)
   */
  export type ProfileVariable = SimpleVariable | DictionaryVariable | ArrayVariable;

  /**
   * The JavaScript types that may be used as inputs to profile variables when evaluating compiled expressions with [ExecuteFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ExecuteFunction) or [ExecuteFunctionAsync](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ExecuteFunctionAsync).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ProfileVariableInstanceType)
   */
  export type ProfileVariableInstanceType =
    | Graphic
    | Geometry
    | FeatureSet
    | FeatureSetLayer
    | Map
    | Geometry
    | KnowledgeGraph
    | string
    | number
    | boolean
    | Date
    | any
    | any[];

  /**
   * The JavaScript type of the result returned from the Arcade expression.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#ResultType)
   */
  export type ResultType = number | string | Date | boolean | Graphic | Geometry | any | ResultType[];

  /**
   * The type definition for a simple Arcade profile variable.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#SimpleVariable)
   */
  export interface SimpleVariable {
    /**
     * The name of the profile variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-arcade.html#SimpleVariable)
     */
    name: string;
    /**
     * The Arcade data type of the variable.
     *
     * [Read more...](global.html#type)
     */
    type:
      | "feature"
      | "featureSet"
      | "featureSetCollection"
      | "geometry"
      | "number"
      | "text"
      | "date"
      | "dateOnly"
      | "time"
      | "boolean"
      | "knowledgeGraph";
  }

  export interface Basemap extends Accessor, Loadable, JSONSupport {}

  export class Basemap {
    /**
     * A basemap is a collection of layers that provide geographic context to a map or scene with data such as topographic features, road networks, buildings, and labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html)
     */

    constructor(properties?: BasemapProperties);

    /**
     * A collection of tile layers that make up the basemap's features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers)
     */
    baseLayers: Collection<Layer>;
    /**
     * An identifier used to refer to the basemap when referencing it elsewhere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#id)
     */
    id: string;
    /**
     * Indicates whether the basemap instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * A collection of reference layers which are displayed over the [base layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers) and all other layers in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#referenceLayers)
     */
    referenceLayers: Collection<Layer>;
    /**
     * The spatial reference of the Basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The style of the basemap from the [basemap styles service (v2)](https://developers.arcgis.com/rest/basemap-styles/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#style)
     */
    style: BasemapStyle;
    /**
     * The URL pointing to an image that represents the basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#thumbnailUrl)
     */
    thumbnailUrl: string;
    /**
     * The title of the basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#clone)
     */
    clone(): Basemap;
    /**
     * Destroys the basemap, and any associated resources, including its [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#layers) and [portalItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#portalItem).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#destroy)
     */
    destroy(): void;
    /**
     * Loads all the externally loadable resources associated with the basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#loadAll)
     */
    loadAll(): Promise<Basemap>;

    /**
     * Creates a new basemap instance from a [basemap id](esri-Map.html#basemap-id).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#fromId)
     */
    static fromId(id: string): Basemap;
    static fromJSON(json: any): Basemap;
  }

  interface BasemapProperties extends LoadableProperties {
    /**
     * A collection of tile layers that make up the basemap's features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers)
     */
    baseLayers?: CollectionProperties<LayerProperties>;
    /**
     * An identifier used to refer to the basemap when referencing it elsewhere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#id)
     */
    id?: string;
    /**
     * The portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * A collection of reference layers which are displayed over the [base layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers) and all other layers in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#referenceLayers)
     */
    referenceLayers?: CollectionProperties<LayerProperties>;
    /**
     * The spatial reference of the Basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The style of the basemap from the [basemap styles service (v2)](https://developers.arcgis.com/rest/basemap-styles/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#style)
     */
    style?: BasemapStyleProperties;
    /**
     * The URL pointing to an image that represents the basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#thumbnailUrl)
     */
    thumbnailUrl?: string;
    /**
     * The title of the basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#title)
     */
    title?: string;
  }

  export interface Camera extends Accessor, JSONSupport, Clonable {}

  export class Camera {
    /**
     * The camera defines the [position](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#position), [tilt](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#tilt), and [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading) of the point from which the [SceneView's](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) visible extent is observed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html)
     */

    constructor(properties?: CameraProperties);

    /**
     * The diagonal field of view (fov) angle for the camera.
     *
     * @default 55
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#fov)
     */
    fov: number;
    /**
     * The compass heading of the camera in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading)
     */
    heading: number;
    /**
     * The layout defines which sub-region of the camera is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#layout)
     */
    layout: CameraLayout;
    /**
     * The position of the camera defined by a map point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#position)
     */
    position: Point;
    /**
     * The tilt of the camera in degrees with respect to the surface as projected down from the camera position.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#tilt)
     */
    tilt: number;

    static fromJSON(json: any): Camera;
  }

  interface CameraProperties {
    /**
     * The diagonal field of view (fov) angle for the camera.
     *
     * @default 55
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#fov)
     */
    fov?: number;
    /**
     * The compass heading of the camera in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading)
     */
    heading?: number;
    /**
     * The layout defines which sub-region of the camera is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#layout)
     */
    layout?: CameraLayoutProperties;
    /**
     * The position of the camera defined by a map point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#position)
     */
    position?: PointProperties;
    /**
     * The tilt of the camera in degrees with respect to the surface as projected down from the camera position.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#tilt)
     */
    tilt?: number;
  }

  export interface CameraLayout extends Accessor, Clonable {}

  export class CameraLayout {
    /**
     * The camera layout defines the position of the current view in a distributed larger view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html)
     */

    constructor(properties?: CameraLayoutProperties);

    /**
     * The active column a display client renders to in a tiled display setup.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#column)
     */
    column: number;
    /**
     * The number of columns to decompose the camera in a tiled display setup.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#columns)
     */
    columns: number;
    /**
     * The active row a display client renders to in a tiled display setup.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#row)
     */
    row: number;
    /**
     * The number of rows to decompose the camera in a tiled display setup.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#rows)
     */
    rows: number;
  }

  interface CameraLayoutProperties {
    /**
     * The active column a display client renders to in a tiled display setup.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#column)
     */
    column?: number;
    /**
     * The number of columns to decompose the camera in a tiled display setup.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#columns)
     */
    columns?: number;
    /**
     * The active row a display client renders to in a tiled display setup.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#row)
     */
    row?: number;
    /**
     * The number of rows to decompose the camera in a tiled display setup.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-CameraLayout.html#rows)
     */
    rows?: number;
  }

  export class Color {
    /**
     * The alpha value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#a)
     */
    a: number;
    /**
     * The blue value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#b)
     */
    b: number;
    /**
     * The green value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#g)
     */
    g: number;
    /**
     * The red value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#r)
     */
    r: number;

    /**
     * Creates a deep clone of the Color instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#clone)
     */
    clone(): Color;
    /**
     * Takes an array of rgb(a) values, named string, hex string or an hsl(a) string, an object with `r`, `g`, `b`, and `a` properties, or a [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) object and sets this color instance to the input value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#setColor)
     */
    setColor(color: string | number[] | any): Color;
    /**
     * Returns a CSS color string in rgba form representing the Color instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toCss)
     */
    toCss(includeAlpha?: boolean): string;
    /**
     * Returns the color in hexadecimal format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toHex)
     */
    toHex(options?: ColorToHexOptions): string;
    /**
     * Returns a JSON object with all the values from a Color instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toJSON)
     */
    toJSON(): any;
    /**
     * Returns a 3-component array of rgb values that represent the Color instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toRgb)
     */
    toRgb(): number[];
    /**
     * Returns a 4-component array of rgba values that represent the Color instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toRgba)
     */
    toRgba(): number[];

    /**
     * Creates a Color instance by blending two colors using a weight factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#blendColors)
     */
    static blendColors(start: Color, end: Color, weight: number, out?: Color): Color;

    constructor(color: string | number[] | any);
    /**
     * Creates a Color instance using a 3 or 4 element array, mapping each element in sequence to the rgb(a) values of the color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#fromArray)
     */
    static fromArray(a: number[], t?: Color): Color;
    /**
     * Creates a Color from hexadecimal string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#fromHex)
     */
    static fromHex(hex: string, color?: Color): Color;
    /**
     * Creates a new Color instance, and initializes it with values from a JSON object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#fromJSON)
     */
    static fromJSON(json: any): Color;
    /**
     * Creates a Color instance from a string of the form "rgb()" or "rgba()".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#fromRgb)
     */
    static fromRgb(color: string, out?: Color): Color;
    /**
     * Creates a Color instance by parsing a generic string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#fromString)
     */
    static fromString(str: string, obj?: Color): Color;
  }

  export interface ColorToHexOptions {
    /**
     * When `true`, the hexadecimal number will be capitalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toHex)
     */
    capitalize?: boolean;
    /**
     * The intended size of the output hexadecimal number.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html#toHex)
     */
    digits?: number;
  }

  /**
   * Configure global properties of the library.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html)
   */
  interface config {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#apiKey)
     */
    apiKey: string;
    /**
     * The name of the application using the API.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#applicationName)
     */
    applicationName: string;
    /**
     * Overrides the URL for loading the API assets when using local builds of the `@arcgis/core` npm package.
     *
     * @default "https://js.arcgis.com/[4.x]/@arcgis/core/assets"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#assetsPath)
     */
    assetsPath: string;
    /**
     * The URL for font resources used by the [Font](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#family) class in non-MapImageLayer labels for both 2D MapViews and 3D SceneViews.
     *
     * @default "https://static.arcgis.com/fonts"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#fontsUrl)
     */
    fontsUrl: string;
    /**
     * The default [geometryService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html) used by widgets and other operations, such as on-the-fly projections.
     *
     * @default "https://utility.arcgisonline.com/arcgis/rest/services/Geometry/GeometryServer"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#geometryServiceUrl)
     */
    geometryServiceUrl: string;
    /**
     * The URL for the utility service used by [GeoRSSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html) to convert GeoRSS documents.
     *
     * @default "https://utility.arcgis.com/sharing/rss"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#geoRSSServiceUrl)
     */
    geoRSSServiceUrl: string;
    /**
     * The URL for the utility service used by [KMLLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html) to convert KML documents.
     *
     * @default "https://utility.arcgis.com/sharing/kml"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#kmlServiceUrl)
     */
    kmlServiceUrl: string;
    /**
     * An object with properties that control various aspects of log messages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#log)
     */
    log: configLog;
    /**
     * The default URL of new portal instances.
     *
     * @default "https://www.arcgis.com"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#portalUrl)
     */
    portalUrl: string;
    /**
     * An object with properties that control various aspects of communication between the library and web servers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    request: configRequest;
    /**
     * Controls whether the API respects the user's preference for reduced motion.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#respectPrefersReducedMotion)
     */
    respectPrefersReducedMotion: boolean;
    /**
     * The default routing service used by the [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) and the [Directions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html) widget.
     *
     * @default "https://route-api.arcgis.com/arcgis/rest/services/World/Route/NAServer/Route_World"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#routeServiceUrl)
     */
    routeServiceUrl: string;
    /**
     * Indicates whether layers should apply privileges of the authenticated user to their capabilities.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#userPrivilegesApplied)
     */
    userPrivilegesApplied: boolean;
    /**
     * An object with properties that control various aspects of [the workers framework](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    workers: configWorkers;
  }

  export const config: config;

  export type AfterInterceptorCallback = (response: RequestResponse) => void;

  export type BeforeInterceptorCallback = (params: any) => any;

  export interface configLog {
    /**
     * An array of custom interceptor functions that can be used to intercept log messages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#log)
     */
    interceptors: LogInterceptor[];
    /**
     * The level of messages to log.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#log)
     */
    level: "none" | "error" | "warn" | "info";
  }

  export interface configRequest {
    /**
     * List of domain suffixes known to support https.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    httpsDomains?: string[];
    /**
     * **Since:** 4.8  Allows developers to modify requests before or after they are sent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    interceptors?: RequestInterceptor[];
    /**
     * Maximum number of characters allowed in the URL for HTTP GET requests made by [request](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html).
     *
     * @default 2000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    maxUrlLength?: number;
    /**
     * **Since:** 4.24  The fetch and image network request priority.
     *
     * @default high
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    priority?: "auto" | "high" | "low";
    /**
     * A proxy rule defines a proxy for a set of resources with an identical URL prefix.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    proxyRules?: configRequestProxyRules[];
    /**
     * Resource proxy for your application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    proxyUrl?: string;
    /**
     * Number of milliseconds [request](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html) will wait for response from a server.
     *
     * @default 62000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    timeout?: number;
    /**
     * **Since:** 4.9  Indicates whether cross-origin requests made to the associated server should include credentials such as cookies and authorization headers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    trustedServers?: string[];
    /**
     * **Since:** 4.5  Indicates whether `esri/request` will request a credential from `IdentityManager`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    useIdentity?: boolean;
  }

  export interface configRequestProxyRules {
    /**
     * The URL of the proxy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    proxyUrl?: string;
    /**
     * URL prefix for resources that need to be accessed through a specific proxy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#request)
     */
    urlPrefix?: string;
  }

  export interface configWorkers {
    /**
     * The absolute url to the AMD or SystemJS loader used in the worker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    loaderUrl?: any;
    /**
     * This is used by the `@arcgis/core` npm package to control where to load a custom build of the `RemoteClient` from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    workerPath?: string;
    /**
     * The AMD configuration object that is set in each worker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    loaderConfig?: configWorkersLoaderConfig;
  }

  export interface configWorkersLoaderConfig {
    /**
     * The AMD loader loads all code relative to the baseUrl.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    baseUrl?: string;
    /**
     * Determines if the specified feature capabilities are supported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    has?: any;
    /**
     * Map of module id fragments to file paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    paths?: any;
    /**
     * Map paths in module identifiers to different paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    map?: any;
    /**
     * An array of objects which provide the package name and its location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#workers)
     */
    packages?: any[];
  }

  export type ErrorCallback = (error: Error) => void;

  export type LogInterceptor = (level: "error" | "warn" | "info", module: string, ...args: any[]) => boolean;

  /**
   * Specifies the object used for intercepting and modifying requests made via [esriRequest](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
   */
  export interface RequestInterceptor {
    /**
     * Makes changes to the response after the request is sent, but before it's returned to the caller.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    after?: AfterInterceptorCallback;
    /**
     * Make changes to the request URL or options before the request is sent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    before?: BeforeInterceptorCallback;
    /**
     * When an error occurs during the request processing, this function is called with an [Error](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html) object giving the details about what happened.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    error?: ErrorCallback;
    /**
     * Sets or adds headers into `requestOptions.headers`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    headers?: any;
    /**
     * Sets or adds query parameters into `requestOptions.query`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    query?: any;
    /**
     * Hardcodes the [response](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    responseData?: any;
    /**
     * Specifies the URL(s) to apply to the interceptors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#RequestInterceptor)
     */
    urls?: string | RegExp | (string | RegExp)[];
  }

  export type WatchCallback = (newValue: any, oldValue: any, propertyName: string, target: Accessor) => void;

  /**
   * Represents a watch or event handler which can be removed.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Accessor.html#WatchHandle)
   */
  export interface WatchHandle {
    /**
     * Removes the watch handle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Accessor.html#WatchHandle)
     */
    remove(): void;
  }

  /**
   * This module contains Accessor [TypeScript](https://www.typescriptlang.org/docs/handbook/decorators.html) decorators.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html)
   */
  interface decorators {
    /**
     * A property decorator that creates a two-way binding between the property it decorates and an inner property of one of its members.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#aliasOf)
     */
    aliasOf(propertyName: string): Function;
    /**
     * This method decorator is used to define the method that will cast a property from a class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#cast)
     */
    cast(propertyName: string): Function;
    /**
     * This property decorator is used to define the function or class for a property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#cast)
     */
    cast(functionOrClass: Function): Function;
    /**
     * This convenience decorator is used to define an [Accessor](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Accessor.html) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    property(propertyMetadata?: decoratorsPropertyPropertyMetadata): Function;
    /**
     * This decorator is used to define an [Accessor](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Accessor.html) subclass.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#subclass)
     */
    subclass(declaredClass?: string): Function;
  }

  export const decorators: decorators;

  export interface decoratorsPropertyPropertyMetadata {
    /**
     * Property names of dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    dependsOn?: string[];
    /**
     * The constructor used to [autocast](../autocasting/) the property.
     *
     * [Read more...](global.html#type)
     */
    type?: Function;
    /**
     * The function to use to [autocast](https://developers.arcgis.com/javascript/latest/autocasting/) the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    cast?: Function;
    /**
     * Indicates whether the property is read-only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    readOnly?: boolean;
    /**
     * Indicates whether the property can be set during construction but is otherwise read-only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    constructOnly?: boolean;
    /**
     * The property decorator that creates a two-way binding between the property it decorates and an inner property of one of its members.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    aliasOf?: string;
    /**
     * The default value for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-accessorSupport-decorators.html#property)
     */
    value?: any;
  }

  export class Clonable {
    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Clonable.html#clone)
     */
    clone(): this;
  }

  export interface CollectionAfterAddEvent {
    item: any;
  }

  export interface CollectionAfterChangesEvent {}

  export interface CollectionAfterRemoveEvent {
    item: any;
  }

  export interface CollectionBeforeAddEvent {
    cancellable: boolean;

    defaultPrevented: boolean;

    item: any;

    preventDefault: Function;
  }

  export interface CollectionBeforeChangesEvent {
    cancellable: boolean;

    defaultPrevented: boolean;

    item: any;

    preventDefault: Function;
  }

  export interface CollectionBeforeRemoveEvent {
    cancellable: boolean;

    defaultPrevented: boolean;

    item: any;

    preventDefault: Function;
  }

  export class Error {
    /**
     * The details object provides additional details specific to the error, giving more information about why the error was raised.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html#details)
     */
    details: any;
    /**
     * A message describing the details of the error.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html#message)
     */
    message: string;
    /**
     * A unique error name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html#name)
     */
    name: string;

    constructor(name: string, message?: string, details?: any);
  }

  export class Evented {
    /**
     * Emits an event on the instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Evented.html#emit)
     */
    emit(type: string, event?: any): boolean;
    /**
     * Indicates whether there is an event listener on the instance that matches the provided event name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Evented.html#hasEventListener)
     */
    hasEventListener(type: string): boolean;
    /**
     * Registers an event handler on the instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Evented.html#on)
     */
    on(type: string | string[], listener: EventHandler): IHandle;
  }

  export type EventHandler = (event: any) => void;

  export class Handles {
    /**
     * Adds a group of handles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#add)
     */
    add(handles: WatchHandle | WatchHandle[] | Collection<WatchHandle>, groupKey?: any): void;
    /**
     * Destroys the object, removing all the handles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#destroy)
     */
    destroy(): void;
    /**
     * Returns true if a group exists for the provided group key, false otherwise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#has)
     */
    has(groupKey: any): boolean;
    /**
     * Removes a group of handles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#remove)
     */
    remove(groupKey?: any): void;
    /**
     * Removes all handles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#removeAll)
     */
    removeAll(): void;

    constructor();
  }

  /**
   * A handle to a [highlight](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerView.html#highlight) call result.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#Handle)
   */
  export interface Handle {
    /**
     * Removes the handle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Handles.html#Handle)
     */
    remove(): void;
  }

  export class Identifiable {}

  export class JSONSupport {
    /**
     * Converts an instance of this class to its [ArcGIS portal JSON](https://developers.arcgis.com/documentation/common-data-types/geometry-objects.htm) representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-JSONSupport.html#toJSON)
     */
    toJSON(): any;

    /**
     * Creates a new instance of this class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-JSONSupport.html#fromJSON)
     */
    static fromJSON(json: any): any;
  }

  /**
   * Provides a utility method for deeply cloning objects with properties that are computed or have their own `clone()` method, such as [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-lang.html)
   */
  interface lang {
    /**
     * Use this method to deeply clone objects with properties that are computed or have their own `clone()` method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-lang.html#clone)
     */
    clone(elem: any): any;
  }

  export const lang: lang;

  export class Loadable {
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * A list of warnings which occurred while loading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#loadWarnings)
     */
    readonly loadWarnings: any[];

    /**
     * Cancels a [load()](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#load) operation if it is already in progress.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#cancelLoad)
     */
    cancelLoad(): void;
    /**
     * `isFulfilled()` may be used to verify if creating an instance of the class is fulfilled (either resolved or rejected).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#isFulfilled)
     */
    isFulfilled(): boolean;
    /**
     * `isRejected()` may be used to verify if creating an instance of the class is rejected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#isRejected)
     */
    isRejected(): boolean;
    /**
     * `isResolved()` may be used to verify if creating an instance of the class is resolved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#isResolved)
     */
    isResolved(): boolean;
    /**
     * Loads the resources referenced by this class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#load)
     */
    load(signal?: AbortSignal): Promise<any>;
    /**
     * `when()` may be leveraged once an instance of the class is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Loadable.html#when)
     */
    when(callback?: Function, errback?: Function): Promise<any>;
  }

  interface LoadableProperties {}

  export class corePromise {
    /**
     * `isFulfilled()` may be used to verify if creating an instance of the class is fulfilled (either resolved or rejected).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Promise.html#isFulfilled)
     */
    isFulfilled(): boolean;
    /**
     * `isRejected()` may be used to verify if creating an instance of the class is rejected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Promise.html#isRejected)
     */
    isRejected(): boolean;
    /**
     * `isResolved()` may be used to verify if creating an instance of the class is resolved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Promise.html#isResolved)
     */
    isResolved(): boolean;
    /**
     * `when()` may be leveraged once an instance of the class is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Promise.html#when)
     */
    when(callback?: Function, errback?: Function): Promise<any>;
  }

  /**
   * Various utilities and convenience functions for working with promises.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html)
   */
  interface promiseUtils {
    /**
     * Creates a special error object which is used to signal aborted requests in promise chains.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#createAbortError)
     */
    createAbortError(): Error;
    /**
     * A utility for ensuring an input function is not simultaneously invoked more than once at a time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#debounce)
     */
    debounce<T>(callback: T): T;
    /**
     * Convenience utility method to wait for a number of promises to either resolve or reject.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#eachAlways)
     */
    eachAlways(promises: Promise<any>[] | any): Promise<EachAlwaysResult[]> | any;
    /**
     * A convenience utility method for filtering an array of values using an asynchronous predicate function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#filter)
     */
    filter<T>(input: T[], predicate: FilterPredicateCallback): Promise<T[]>;
    /**
     * Check if the provided error object is the special kind of error with which promises are rejected when they are aborted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#isAbortError)
     */
    isAbortError(error: Error): boolean;
  }

  export const promiseUtils: promiseUtils;

  /**
   * The result object for a promise passed to [promiseUtils.eachAlways](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#eachAlways).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#EachAlwaysResult)
   */
  export interface EachAlwaysResult {
    /**
     * The promise that has been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#EachAlwaysResult)
     */
    promise: Promise<any>;
    /**
     * The value with which the promise resolved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#EachAlwaysResult)
     */
    value?: any;
    /**
     * The error with which the promise rejected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-promiseUtils.html#EachAlwaysResult)
     */
    error?: any;
  }

  export type Executor = (resolve: ResolveCallback, reject: RejectCallback) => void;

  export type FilterPredicateCallback = (value: any, index: number) => Promise<any>;

  export type RejectCallback = (error?: any) => void;

  export type ResolveCallback = (value?: any | Promise<any>) => void;

  /**
   * Quantity type used to represent measurements across the JS API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-quantity.html)
   */
  interface quantity {}

  export const quantity: quantity;

  export interface Angle {
    /**
     * Scalar value representing a angle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-quantity.html#Angle)
     */
    value: number;
    /**
     * Any unit which represents an angle.
     *
     * [Read more...](global.html#unit)
     */
    unit: AngleUnit;
    /**
     * Unit type.
     *
     * [Read more...](global.html#type)
     */
    type: "angle";
  }

  export interface Area {
    /**
     * Scalar value representing a area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-quantity.html#Area)
     */
    value: number;
    /**
     * Any unit which represents area.
     *
     * [Read more...](global.html#unit)
     */
    unit: AreaUnit;
    /**
     * Unit type.
     *
     * [Read more...](global.html#type)
     */
    type: "area";
  }

  export interface Length {
    /**
     * Scalar value representing a length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-quantity.html#Length)
     */
    value: number;
    /**
     * Any unit which represents length.
     *
     * [Read more...](global.html#unit)
     */
    unit: LengthUnit;
    /**
     * Unit type.
     *
     * [Read more...](global.html#type)
     */
    type: "length";
  }

  /**
   * Various utilities and convenience functions for executing code at various phases of browser frames.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html)
   */
  interface scheduling {
    /**
     * Registers a frame task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#addFrameTask)
     */
    addFrameTask(phases: PhaseCallbacks): FrameTaskHandle;
    /**
     * Schedules the execution of a `callback` function at the next web browser tick.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#schedule)
     */
    schedule(callback: Function): any;
  }

  export const scheduling: scheduling;

  /**
   * An object to remove or pause a frame task registered with [addFrameTask()](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#addFrameTask).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#FrameTaskHandle)
   */
  export interface FrameTaskHandle {
    /**
     * Pause the execution the frame task at every frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#FrameTaskHandle)
     */
    pause(): void;
    /**
     * Resumes the execution the frame task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#FrameTaskHandle)
     */
    resume(): void;
    /**
     * Removes the frame task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#FrameTaskHandle)
     */
    remove(): void;
  }

  export type PhaseCallback = (event?: PhaseEvent) => void;

  /**
   * A set of [callbacks](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseCallback) that will be called at specific phases of the animation frame.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseCallbacks)
   */
  export interface PhaseCallbacks {
    /**
     * A callback called before rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseCallbacks)
     */
    prepare?: PhaseCallback;
    /**
     * A callback to execute rendering logic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseCallbacks)
     */
    render?: PhaseCallback;
    /**
     * A callback to execute state update logic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseCallbacks)
     */
    update?: PhaseCallback;
  }

  /**
   * An object with timing information.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseEvent)
   */
  export interface PhaseEvent {
    /**
     * The absolute time at the start of the current animation frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseEvent)
     */
    time: number;
    /**
     * The elapsed time since the last animation frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseEvent)
     */
    deltaTime: number;
    /**
     * The amount of time spent within the current animation frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-scheduling.html#PhaseEvent)
     */
    elapsedFrameTime: number;
  }

  export type MutateFunction = (value: any) => void;

  /**
   * Signal which contains a single reactive value and methods to modify it and notify any other signals, Accessor properties or reactive contexts (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-signal.html#Signal)
   */
  export interface Signal {
    /**
     * The current value of the signal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-signal.html#Signal)
     */
    value?: boolean;
    /**
     * Calls a mutate function which can modify the value of the signal in place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-signal.html#Signal)
     */
    mutate?: MutateFunction;
  }

  /**
   * Creates a [WhereClause](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html) expression that adheres to standardized SQL expressions.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql.html)
   */
  interface sql {
    /**
     * Parses the given where clause string and returns an instance of [WhereClause](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html) when resolved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql.html#parseWhereClause)
     */
    parseWhereClause(clause: string, fieldsIndex: FieldsIndex, timeZone?: string): Promise<WhereClause>;
  }

  export const sql: sql;

  /**
   * The WhereClause is used to extract the features that meet a specified condition by parsing the provided results in to a value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html)
   */
  interface WhereClause {
    /**
     * An array of the field names used in the where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#fieldNames)
     */
    fieldNames: string[];
    /**
     * Returns `true` if the parsed where clause meets the requirements of standardized sql.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#isStandardized)
     */
    readonly isStandardized: boolean;
    /**
     * A parse tree is a data structure for representing a parsed sql statement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#parseTree)
     */
    parseTree: SQLNode;

    /**
     * Executes the where clause against a feature to generate a value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#calculateValue)
     */
    calculateValue(feature: any): any;
    /**
     * Tests the attributes of a feature against the `whereClause`, and returns `true` if the test passes, `false` otherwise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#testFeature)
     */
    testFeature(feature: any): boolean;
  }

  export const WhereClause: WhereClause;

  /**
   * A binary expression node.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BinaryNode)
   */
  export interface BinaryNode {
    type: "binary-expression";

    location: SQLSourceLocation;
    /**
     * Defines the way two expressions are combined to yield a single result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BinaryNode)
     */
    operator:
      | "AND"
      | "OR"
      | "IS"
      | "ISNOT"
      | "IN"
      | "NOT IN"
      | "BETWEEN"
      | "NOTBETWEEN"
      | "LIKE"
      | "NOT LIKE"
      | "<>"
      | "<"
      | ">"
      | ">="
      | "<="
      | "="
      | "*"
      | "-"
      | "+"
      | "/"
      | "||";
    /**
     * SQL node to the left of the binary node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BinaryNode)
     */
    left: SQLNode;
    /**
     * SQL node to the right of the binary node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BinaryNode)
     */
    right: SQLNode;
    /**
     * Escape string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BinaryNode)
     */
    escape?: string;
  }

  /**
   * A node that contains a boolean expression.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BoolNode)
   */
  export interface BoolNode {
    type: "boolean";

    location: SQLSourceLocation;
    /**
     * Values are `true` or `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#BoolNode)
     */
    value: boolean;
  }

  /**
   * A node that contains column name of a layer or table.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#ColumnNode)
   */
  export interface ColumnNode {
    type: "column-reference";

    location: SQLSourceLocation;
    /**
     * Column or field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#ColumnNode)
     */
    column: string;
  }

  /**
   * A node that contains `CURRENT_TIMESTAMP`, `CURRENT_TIME`, or `CURRENT_DATE` functions.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#CurrentTimeNode)
   */
  export interface CurrentTimeNode {
    type: "current-time";

    location: SQLSourceLocation;
    /**
     * The current time node mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#CurrentTimeNode)
     */
    mode: "timestamp" | "date" | "time";
  }

  /**
   * A node that contains `CURRENT_USER` function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#CurrentUserNode)
   */
  export interface CurrentUserNode {
    type: "current-user";

    location: SQLSourceLocation;
  }

  export interface DataTypeNode {
    type: "data-type";

    location: SQLSourceLocation;

    value: DataTypeNodeValue;
  }

  /**
   * A node that contains date.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#DateNode)
   */
  export interface DateNode {
    type: "date";

    location: SQLSourceLocation;
    /**
     * The date value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#DateNode)
     */
    value: string;
  }

  /**
   * A nodes that contains SQL function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#FunctionNode)
   */
  export interface FunctionNode {
    type: "function";

    location: SQLSourceLocation;
    /**
     * Name of the SQL function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#FunctionNode)
     */
    name: string;
    /**
     * Function arguments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#FunctionNode)
     */
    args: ListNode;
  }

  /**
   * Interval node indicates that the character literal is an interval.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalNode)
   */
  export interface IntervalNode {
    type: "interval";

    location: SQLSourceLocation;
    /**
     * Interval node value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalNode)
     */
    value: StringNode | ParamNode;
    /**
     * Interval node sign.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalNode)
     */
    op: "+" | "-" | "";
    /**
     * The interval qualifier can either be a single datetime field or be composed of two datetime fields, in the form: <leading field> TO <trailing field>.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalNode)
     */
    qualifier: IntervalQualifierNode | IntervalPeriodNode;
  }

  /**
   * Interval period node.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalPeriodNode)
   */
  export interface IntervalPeriodNode {
    type: "interval-period";

    location: SQLSourceLocation;
    /**
     * Interval node periods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalPeriodNode)
     */
    period: "day" | "month" | "hour" | "second" | "year" | "minute";
    /**
     * Precision for an interval data type includes interval leading precision, interval precision, and seconds precision.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalPeriodNode)
     */
    precision: number;
    /**
     * Seconds precision.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalPeriodNode)
     */
    secondary: number;
  }

  /**
   * Node that contains interval qualifier.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#IntervalQualifierNode)
   */
  export interface IntervalQualifierNode {
    type: "interval-qualifier";

    location: SQLSourceLocation;

    start: IntervalPeriodNode;

    end: IntervalPeriodNode;
  }

  /**
   * A node that contains a list.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#ListNode)
   */
  export interface ListNode {
    type: "expression-list";

    location: SQLSourceLocation;
    /**
     * SQL expressions for the list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#ListNode)
     */
    value: SQLNode[];
  }

  /**
   * A node that contains `NULL` value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#NullNode)
   */
  export interface NullNode {
    type: "null";

    location: SQLSourceLocation;
    /**
     * Null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#NullNode)
     */
    value: null;
  }

  /**
   * A node that contains number.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#NumberNode)
   */
  export interface NumberNode {
    type: "number";

    location: SQLSourceLocation;
    /**
     * Numeric value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#NumberNode)
     */
    value: number;
  }

  /**
   * ParamNode.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#ParamNode)
   */
  export interface ParamNode {
    type: "parameter";

    location: SQLSourceLocation;

    value: string;
  }

  /**
   * Node that contains case expression.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SearchedCaseNode)
   */
  export interface SearchedCaseNode {
    type: "case-expression";

    location: SQLSourceLocation;
    /**
     * Case expression format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SearchedCaseNode)
     */
    format: "searched";
    /**
     * When clauses.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SearchedCaseNode)
     */
    clauses: WhenNode[];
    /**
     * Else SQL node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SearchedCaseNode)
     */
    else: SQLNode;

    elseLocation?: SQLSourceLocation;
  }

  /**
   * Simple case expression node.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SimpleCaseNode)
   */
  export interface SimpleCaseNode {
    type: "case-expression";

    location: SQLSourceLocation;

    format: "simple";
    /**
     * When nodes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SimpleCaseNode)
     */
    clauses: WhenNode[];
    /**
     * SQL node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SimpleCaseNode)
     */
    operand: SQLNode;
    /**
     * else sql node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SimpleCaseNode)
     */
    else: SQLNode;

    elseLocation?: SQLSourceLocation;
  }

  /**
   * SQL parse tree is composed of SqlNodes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#SQLNode)
   */
  export type SQLNode =
    | BinaryNode
    | BoolNode
    | ColumnNode
    | CurrentTimeNode
    | CurrentUserNode
    | DataTypeNode
    | DateNode
    | FunctionNode
    | IntervalNode
    | IntervalPeriodNode
    | IntervalQualifierNode
    | ListNode
    | NullNode
    | NumberNode
    | ParamNode
    | SearchedCaseNode
    | SimpleCaseNode
    | StringNode
    | TimeNode
    | TimeStampNode
    | UnaryNode
    | WhenNode;

  export interface SQLSourceLocation {
    start: SQLSourcePosition;

    end: SQLSourcePosition;
  }

  export interface SQLSourcePosition {
    offset: number;

    line: number;

    column: number;
  }

  export interface SQLSyntaxError {
    name: "SyntaxError";

    location: SQLSourceLocation;

    message: string;
  }

  /**
   * A node that contains string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#StringNode)
   */
  export interface StringNode {
    type: "string";

    location: SQLSourceLocation;
    /**
     * The string value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#StringNode)
     */
    value: string;
  }

  /**
   * A node that contains time.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#TimeNode)
   */
  export interface TimeNode {
    type: "time";

    location: SQLSourceLocation;
    /**
     * The time value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#TimeNode)
     */
    value: string;
  }

  /**
   * A node that contains date.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#TimeStampNode)
   */
  export interface TimeStampNode {
    type: "timestamp";

    location: SQLSourceLocation;
    /**
     * String date value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#TimeStampNode)
     */
    value: string;
  }

  /**
   * A node that contains unary operator.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#UnaryNode)
   */
  export interface UnaryNode {
    type: "unary-expression";

    location: SQLSourceLocation;

    operator: "NOT";
    /**
     * The sql node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#UnaryNode)
     */
    expr: SQLNode;
  }

  /**
   * Node that contains when clause.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#WhenNode)
   */
  export interface WhenNode {
    type: "when-clause";

    location: SQLSourceLocation;
    /**
     * operand.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#WhenNode)
     */
    operand: SQLNode;
    /**
     * value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-sql-WhereClause.html#WhenNode)
     */
    value: SQLNode;
  }

  export interface DataTypeNodeValue {
    type: "integer" | "real" | "smallint" | "float" | "date" | "timestamp" | "varchar" | "time";

    size?: number;
  }

  /**
   * Types of units used across the JS API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html)
   */
  namespace units {
    /**
     * Units for lengths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#LengthUnit)
     */
    export type LengthUnit =
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";

    /**
     * Units for elevations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#ElevationUnit)
     */
    export type ElevationUnit = __esri.LengthUnit | "decimal-degrees" | "unknown";

    /**
     * Units for areas.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#AreaUnit)
     */
    export type AreaUnit =
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";

    /**
     * Units for volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#VolumeUnit)
     */
    export type VolumeUnit =
      | "liters"
      | "cubic-millimeters"
      | "cubic-centimeters"
      | "cubic-decimeters"
      | "cubic-meters"
      | "cubic-kilometers"
      | "cubic-inches"
      | "cubic-feet"
      | "cubic-yards"
      | "cubic-miles";

    /**
     * Units for angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#AngleUnit)
     */
    export type AngleUnit = "degrees" | "radians";

    /**
     * Units for angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#Unit)
     */
    export type Unit = __esri.LengthUnit | __esri.AreaUnit | __esri.VolumeUnit | __esri.AngleUnit;

    /**
     * Measurement systems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#MeasurementSystem)
     */
    export type MeasurementSystem = "imperial" | "metric";

    /**
     * Measurement systems or an area units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#SystemOrAreaUnit)
     */
    export type SystemOrAreaUnit = __esri.MeasurementSystem | __esri.AreaUnit;

    /**
     * Measurement system or an length unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#SystemOrLengthUnit)
     */
    export type SystemOrLengthUnit = __esri.MeasurementSystem | __esri.LengthUnit;
  }

  /**
   * Units for angles.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#AngleUnit)
   */
  export type AngleUnit = "degrees" | "radians";

  /**
   * Units for areas.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#AreaUnit)
   */
  export type AreaUnit =
    | "square-millimeters"
    | "square-centimeters"
    | "square-decimeters"
    | "square-meters"
    | "square-kilometers"
    | "square-inches"
    | "square-feet"
    | "square-yards"
    | "square-miles"
    | "square-us-feet"
    | "acres"
    | "ares"
    | "hectares";

  /**
   * Units for elevations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#ElevationUnit)
   */
  export type ElevationUnit = LengthUnit | "decimal-degrees" | "unknown";

  /**
   * Units for lengths.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#LengthUnit)
   */
  export type LengthUnit =
    | "millimeters"
    | "centimeters"
    | "decimeters"
    | "meters"
    | "kilometers"
    | "inches"
    | "feet"
    | "yards"
    | "miles"
    | "nautical-miles"
    | "us-feet";

  /**
   * Measurement systems.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#MeasurementSystem)
   */
  export type MeasurementSystem = "imperial" | "metric";

  /**
   * Measurement systems or an area units.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#SystemOrAreaUnit)
   */
  export type SystemOrAreaUnit = MeasurementSystem | AreaUnit;

  /**
   * Measurement system or an length unit.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#SystemOrLengthUnit)
   */
  export type SystemOrLengthUnit = MeasurementSystem | LengthUnit;

  /**
   * Units for angles.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#Unit)
   */
  export type Unit = LengthUnit | AreaUnit | VolumeUnit | AngleUnit;

  /**
   * Units for volumes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-units.html#VolumeUnit)
   */
  export type VolumeUnit =
    | "liters"
    | "cubic-millimeters"
    | "cubic-centimeters"
    | "cubic-decimeters"
    | "cubic-meters"
    | "cubic-kilometers"
    | "cubic-inches"
    | "cubic-feet"
    | "cubic-yards"
    | "cubic-miles";

  /**
   * Utility methods for working with URLs.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html)
   */
  interface urlUtils {
    /**
     * Adds the given proxy rule to the proxy rules list: `esriConfig.request.proxyRules`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html#addProxyRule)
     */
    addProxyRule(rule: urlUtilsAddProxyRuleRule): number;
    /**
     * Returns the proxy rule that matches the given URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html#getProxyRule)
     */
    getProxyRule(url: string): any;
    /**
     * Converts the URL arguments to an object representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html#urlToObject)
     */
    urlToObject(url: string): any;
  }

  export const urlUtils: urlUtils;

  export interface urlUtilsAddProxyRuleRule {
    /**
     * The URL of the proxy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html#addProxyRule)
     */
    proxyUrl: string;
    /**
     * The URL prefix of the resources that should be accessed through the given proxy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-urlUtils.html#addProxyRule)
     */
    urlPrefix: string;
  }

  /**
   * This module is a utility framework that simplifies the use of [Web Workers](https://developer.mozilla.org/en-US/docs/Web/API/Web_Workers_API/Using_web_workers) in the ArcGIS Maps SDK for JavaScript.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html)
   */
  interface workers {
    /**
     * Opens a connection to workers and loads a script with the workers framework.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html#open)
     */
    open(modulePath: string, options?: workersOpenOptions): Promise<Connection>;
  }

  export const workers: workers;

  export class Connection {
    /**
     * A convenient method that invokes a method on each worker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#broadcast)
     */
    broadcast(methodName: string, data?: any, options?: ConnectionBroadcastOptions): Promise<any>[];
    /**
     * Closes the existing connection instance to workers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#close)
     */
    close(): void;
    /**
     * Invokes a [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#basic-invocation) on the remote module loaded with the worker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#invoke)
     */
    invoke(methodName: string, data?: any, options?: ConnectionInvokeOptions): Promise<any>;

    constructor();
  }

  export interface ConnectionBroadcastOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the executions of the remote method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#broadcast)
     */
    signal?: AbortSignal;
  }

  export interface ConnectionInvokeOptions {
    /**
     * An array of Transferable objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#invoke)
     */
    transferList?: any[];
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the execution of the remote method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers-Connection.html#invoke)
     */
    signal?: AbortSignal;
  }

  export interface workersOpenOptions {
    /**
     * The objects defining the API accessible from the module.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html#open)
     */
    client?: any;
    /**
     * Indicates how to load the module.
     *
     * @default distributed
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html#open)
     */
    strategy?: "distributed" | "dedicated" | "local";
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable asynchronous job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-workers.html#open)
     */
    signal?: AbortSignal;
  }

  export class AttachmentElement extends Element {
    /**
     * An `AttachmentElement` defines how one or more attachments can participate in the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html)
     */

    constructor(properties?: AttachmentElementProperties);

    /**
     * Determines whether the user renaming an attachment is allowed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#allowUserRename)
     */
    allowUserRename: boolean;
    /**
     * A string to identify the attachment(s).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#attachmentKeyword)
     */
    attachmentKeyword: string;
    /**
     * Determines whether the file name should be displayed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#displayFilename)
     */
    displayFilename: boolean;
    /**
     * A reference to an Arcade expression that returns a boolean value.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#editableExpression)
     */
    editableExpression: string;
    /**
     * Determines the name of a new attachment.
     *
     * @default {attachmentKeyword}_{now}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#filenameExpression)
     */
    filenameExpression: string;
    /**
     * The input user interface to use for the attachment.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#input)
     */
    input: AttachmentInput | AudioInput | DocumentInput | ImageInput | SignatureInput | VideoInput;
    /**
     * Defines the maximum number of attachments allowed for this element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#maxAttachmentCount)
     */
    maxAttachmentCount: number;
    /**
     * Defines the minimum number of attachments required for this element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#minAttachmentCount)
     */
    minAttachmentCount: number;
    /**
     * The type of the [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#type)
     */
    readonly type: "attachment";
    /**
     * Determines whether the uploaded attachment's file name is preserved.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#useOriginalFilename)
     */
    useOriginalFilename: boolean;

    /**
     * Creates a deep clone of the AttachmentElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#clone)
     */
    clone(): AttachmentElement;

    static fromJSON(json: any): AttachmentElement;
  }

  interface AttachmentElementProperties extends ElementProperties {
    /**
     * Determines whether the user renaming an attachment is allowed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#allowUserRename)
     */
    allowUserRename?: boolean;
    /**
     * A string to identify the attachment(s).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#attachmentKeyword)
     */
    attachmentKeyword?: string;
    /**
     * Determines whether the file name should be displayed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#displayFilename)
     */
    displayFilename?: boolean;
    /**
     * A reference to an Arcade expression that returns a boolean value.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#editableExpression)
     */
    editableExpression?: string;
    /**
     * Determines the name of a new attachment.
     *
     * @default {attachmentKeyword}_{now}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#filenameExpression)
     */
    filenameExpression?: string;
    /**
     * The input user interface to use for the attachment.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#input)
     */
    input?:
      | (AttachmentInputProperties & { type: "attachment" })
      | (AudioInputProperties & { type: "audio" })
      | (DocumentInputProperties & { type: "document" })
      | (ImageInputProperties & { type: "image" })
      | (SignatureInputProperties & { type: "signature" })
      | (VideoInputProperties & { type: "video" });
    /**
     * Defines the maximum number of attachments allowed for this element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#maxAttachmentCount)
     */
    maxAttachmentCount?: number;
    /**
     * Defines the minimum number of attachments required for this element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#minAttachmentCount)
     */
    minAttachmentCount?: number;
    /**
     * Determines whether the uploaded attachment's file name is preserved.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-AttachmentElement.html#useOriginalFilename)
     */
    useOriginalFilename?: boolean;
  }

  export interface Element extends Accessor, JSONSupport {}

  export class Element {
    /**
     * Form elements define what should display within the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html)
     */

    constructor(properties?: ElementProperties);

    /**
     * The element's description providing the purpose behind it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#description)
     */
    description: string;
    /**
     * A string value containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#label)
     */
    label: string;
    /**
     * The type of form element to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#type)
     */
    readonly type: "attachment" | "field" | "group" | "relationship" | "text";
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#visibilityExpression)
     */
    visibilityExpression: string;

    static fromJSON(json: any): Element;
  }

  interface ElementProperties {
    /**
     * The element's description providing the purpose behind it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#description)
     */
    description?: string;
    /**
     * A string value containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#label)
     */
    label?: string;
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html#visibilityExpression)
     */
    visibilityExpression?: string;
  }

  export class FieldElement extends Element {
    /**
     * A `FieldElement` form element defines how a feature layer's [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) participates in the [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html)
     */

    constructor(properties?: FieldElementProperties);

    /**
     * The [coded value domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html) or [range domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html) of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#domain)
     */
    domain: CodedValueDomain | RangeDomain;
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the FormTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#editableExpression)
     */
    editableExpression: string;
    /**
     * The field name as defined by the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#fieldName)
     */
    fieldName: string;
    /**
     * Contains a hint used to help editors while editing fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#hint)
     */
    hint: string;
    /**
     * The module:esri/form/elements/inputs/Input to use for the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#input)
     */
    input:
      | BarcodeScannerInput
      | ComboBoxInput
      | DatePickerInput
      | DateTimeOffsetPickerInput
      | DateTimePickerInput
      | TextAreaInput
      | TextBoxInput
      | RadioButtonsInput
      | SwitchInput
      | TimePickerInput;
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#requiredExpression)
     */
    requiredExpression: string;
    /**
     * Indicates the type of form [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#type)
     */
    readonly type: "field";
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#valueExpression)
     */
    valueExpression: string;

    /**
     * Creates a deep clone of the FieldElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#clone)
     */
    clone(): FieldElement;

    static fromJSON(json: any): FieldElement;
  }

  interface FieldElementProperties extends ElementProperties {
    /**
     * The [coded value domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html) or [range domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html) of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#domain)
     */
    domain?: (CodedValueDomainProperties & { type: "coded-value" }) | (RangeDomainProperties & { type: "range" });
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the FormTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#editableExpression)
     */
    editableExpression?: string;
    /**
     * The field name as defined by the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#fieldName)
     */
    fieldName?: string;
    /**
     * Contains a hint used to help editors while editing fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#hint)
     */
    hint?: string;
    /**
     * The module:esri/form/elements/inputs/Input to use for the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#input)
     */
    input?:
      | (BarcodeScannerInputProperties & { type: "barcode-scanner" })
      | (ComboBoxInputProperties & { type: "combo-box" })
      | (DatePickerInputProperties & { type: "date-picker" })
      | (DateTimeOffsetPickerInputProperties & { type: "datetimeoffset-picker" })
      | (DateTimePickerInputProperties & { type: "datetime-picker" })
      | (TextAreaInputProperties & { type: "text-area" })
      | (TextBoxInputProperties & { type: "text-box" })
      | (RadioButtonsInputProperties & { type: "radio-buttons" })
      | (SwitchInputProperties & { type: "switch" })
      | (TimePickerInputProperties & { type: "time-picker" });
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#requiredExpression)
     */
    requiredExpression?: string;
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#valueExpression)
     */
    valueExpression?: string;
  }

  export class GroupElement extends Element {
    /**
     * A `GroupElement` form element defines a container that holds a set of [form elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#elements) that can be expanded, collapsed, or displayed together.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html)
     */

    constructor(properties?: GroupElementProperties);

    /**
     * An array of [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html), [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html), and [text](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html) elements to display as grouped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#elements)
     */
    elements: (FieldElement | RelationshipElement | TextElement)[];
    /**
     * Defines if the group should be expanded or collapsed when the form is initially displayed.
     *
     * @default "expanded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#initialState)
     */
    initialState: "collapsed" | "expanded";
    /**
     * The type of the [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#type)
     */
    readonly type: "group";

    /**
     * Creates a deep clone of the GroupElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#clone)
     */
    clone(): GroupElement;

    static fromJSON(json: any): GroupElement;
  }

  interface GroupElementProperties extends ElementProperties {
    /**
     * An array of [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html), [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html), and [text](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html) elements to display as grouped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#elements)
     */
    elements?: (
      | (FieldElementProperties & { type: "field" })
      | (RelationshipElementProperties & { type: "relationship" })
      | (TextElementProperties & { type: "text" })
    )[];
    /**
     * Defines if the group should be expanded or collapsed when the form is initially displayed.
     *
     * @default "expanded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-GroupElement.html#initialState)
     */
    initialState?: "collapsed" | "expanded";
  }

  export interface AttachmentInput extends Accessor, JSONSupport, Clonable {}

  export class AttachmentInput {
    /**
     * The `AttachmentInput` class defines a flexible user interface for an attachment input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html)
     */

    constructor(properties?: AttachmentInputProperties);

    /**
     * String value which indicates if existing attachments should be associated with the element and how they should be associated.
     *
     * @default "exact"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html#attachmentAssociationType)
     */
    attachmentAssociationType: "any" | "exact" | "exactOrNone";
    /**
     * Available inputs to add an attachment.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html#inputTypes)
     */
    inputTypes: (AudioInput | DocumentInput | ImageInput | SignatureInput | VideoInput)[];
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html#type)
     */
    readonly type: "attachment";

    static fromJSON(json: any): AttachmentInput;
  }

  interface AttachmentInputProperties {
    /**
     * String value which indicates if existing attachments should be associated with the element and how they should be associated.
     *
     * @default "exact"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html#attachmentAssociationType)
     */
    attachmentAssociationType?: "any" | "exact" | "exactOrNone";
    /**
     * Available inputs to add an attachment.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AttachmentInput.html#inputTypes)
     */
    inputTypes?: (
      | (AudioInputProperties & { type: "audio" })
      | (DocumentInputProperties & { type: "document" })
      | (ImageInputProperties & { type: "image" })
      | (SignatureInputProperties & { type: "signature" })
      | (VideoInputProperties & { type: "video" })
    )[];
  }

  export interface AudioInput extends Accessor, JSONSupport, Clonable {}

  export class AudioInput {
    /**
     * The `AudioInput` class defines a user interface for an input to which audio files can be attached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html)
     */

    constructor(properties?: AudioInputProperties);

    /**
     * The supported input methods used to attach an audio file.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html#inputMethod)
     */
    inputMethod: "any" | "capture" | "upload";
    /**
     * Defines the maximum length of an attachment for this element in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html#maxDuration)
     */
    maxDuration: number;
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html#type)
     */
    readonly type: "audio";

    static fromJSON(json: any): AudioInput;
  }

  interface AudioInputProperties {
    /**
     * The supported input methods used to attach an audio file.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html#inputMethod)
     */
    inputMethod?: "any" | "capture" | "upload";
    /**
     * Defines the maximum length of an attachment for this element in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-AudioInput.html#maxDuration)
     */
    maxDuration?: number;
  }

  export interface DocumentInput extends Accessor, JSONSupport, Clonable {}

  export class DocumentInput {
    /**
     * The `DocumentInput` class defines a user interface for an input to which document files can be attached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-DocumentInput.html)
     */

    constructor(properties?: DocumentInputProperties);

    /**
     * Defines the maximum length of an attachment for this element in megabytes.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-DocumentInput.html#maxFileSize)
     */
    maxFileSize: number;
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-DocumentInput.html#type)
     */
    readonly type: "document";

    static fromJSON(json: any): DocumentInput;
  }

  interface DocumentInputProperties {
    /**
     * Defines the maximum length of an attachment for this element in megabytes.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-DocumentInput.html#maxFileSize)
     */
    maxFileSize?: number;
  }

  export interface ImageInput extends Accessor, JSONSupport, Clonable {}

  export class ImageInput {
    /**
     * The `ImageInput` class defines a user interface for an input to which image files can be attached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html)
     */

    constructor(properties?: ImageInputProperties);

    /**
     * The supported input methods used to attach an image.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html#inputMethod)
     */
    inputMethod: "any" | "capture" | "upload";
    /**
     * Number of pixels on the longest edge depending on orientation.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html#maxImageSize)
     */
    maxImageSize: number;
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html#type)
     */
    readonly type: "image";

    static fromJSON(json: any): ImageInput;
  }

  interface ImageInputProperties {
    /**
     * The supported input methods used to attach an image.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html#inputMethod)
     */
    inputMethod?: "any" | "capture" | "upload";
    /**
     * Number of pixels on the longest edge depending on orientation.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-ImageInput.html#maxImageSize)
     */
    maxImageSize?: number;
  }

  export interface SignatureInput extends Accessor, JSONSupport, Clonable {}

  export class SignatureInput {
    /**
     * The `SignatureInput` class defines a user interface for an input to which a signature can be attached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-SignatureInput.html)
     */

    constructor(properties?: SignatureInputProperties);

    /**
     * The supported input methods used to attach a signature.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-SignatureInput.html#inputMethod)
     */
    inputMethod: "any" | "capture" | "upload";
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-SignatureInput.html#type)
     */
    readonly type: "signature";

    static fromJSON(json: any): SignatureInput;
  }

  interface SignatureInputProperties {
    /**
     * The supported input methods used to attach a signature.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-SignatureInput.html#inputMethod)
     */
    inputMethod?: "any" | "capture" | "upload";
  }

  interface inputs {}

  export const inputs: inputs;

  export interface VideoInput extends Accessor, JSONSupport, Clonable {}

  export class VideoInput {
    /**
     * The `VideoInput` class defines a user interface for an input to which a video file can be attached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html)
     */

    constructor(properties?: VideoInputProperties);

    /**
     * The supported input methods used to attach a video.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html#inputMethod)
     */
    inputMethod: "any" | "capture" | "upload";
    /**
     * Defines the maximum length of an attachment for this element in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html#maxDuration)
     */
    maxDuration: number;
    /**
     * The input type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html#type)
     */
    readonly type: "video";

    static fromJSON(json: any): VideoInput;
  }

  interface VideoInputProperties {
    /**
     * The supported input methods used to attach a video.
     *
     * @default any
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html#inputMethod)
     */
    inputMethod?: "any" | "capture" | "upload";
    /**
     * Defines the maximum length of an attachment for this element in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-attachments-VideoInput.html#maxDuration)
     */
    maxDuration?: number;
  }

  export interface BarcodeScannerInput extends Accessor, TextInput, JSONSupport {}

  export class BarcodeScannerInput {
    /**
     * The `BarcodeScannerInput` class defines the desired user interface for a barcode or QR code scanner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-BarcodeScannerInput.html)
     */

    constructor(properties?: BarcodeScannerInputProperties);

    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-BarcodeScannerInput.html#type)
     */
    readonly type: "barcode-scanner";

    static fromJSON(json: any): BarcodeScannerInput;
  }

  interface BarcodeScannerInputProperties extends TextInputProperties {}

  export interface ComboBoxInput extends Accessor, JSONSupport {}

  export class ComboBoxInput {
    /**
     * The `ComboBoxInput` class defines the desired user interface for a combo box  group input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html)
     */

    constructor(properties?: ComboBoxInputProperties);

    /**
     * The text used to represent a null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html#noValueOptionLabel)
     */
    noValueOptionLabel: string;
    /**
     * Determines whether a null value option is displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html#showNoValueOption)
     */
    showNoValueOption: boolean;
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html#type)
     */
    readonly type: "combo-box";

    static fromJSON(json: any): ComboBoxInput;
  }

  interface ComboBoxInputProperties {
    /**
     * The text used to represent a null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html#noValueOptionLabel)
     */
    noValueOptionLabel?: string;
    /**
     * Determines whether a null value option is displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-ComboBoxInput.html#showNoValueOption)
     */
    showNoValueOption?: boolean;
  }

  export interface DatePickerInput extends Accessor, JSONSupport {}

  export class DatePickerInput {
    /**
     * The `DatePickerInput` class defines the desired user interface for editing `date-only` field [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html)
     */

    constructor(properties?: DatePickerInputProperties);

    /**
     * The maximum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#max)
     */
    max: string;
    /**
     * The minimum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#min)
     */
    min: string;
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#type)
     */
    readonly type: "date-picker";

    /**
     * Creates a deep clone of the `DatePickerInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#clone)
     */
    clone(): DatePickerInput;

    static fromJSON(json: any): DatePickerInput;
  }

  interface DatePickerInputProperties {
    /**
     * The maximum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#max)
     */
    max?: string;
    /**
     * The minimum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DatePickerInput.html#min)
     */
    min?: string;
  }

  export interface DateTimeOffsetPickerInput extends Accessor, JSONSupport {}

  export class DateTimeOffsetPickerInput {
    /**
     * The `DateTimeOffsetPickerInput` class defines the desired user interface for editing `timestamp-offset` field [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html)
     */

    constructor(properties?: DateTimeOffsetPickerInputProperties);

    /**
     * Indicates if the input should provide an option to select the time offset.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#includeTimeOffset)
     */
    includeTimeOffset: boolean;
    /**
     * The maximum date/time offset to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#max)
     */
    max: string;
    /**
     * The minimum date/time offset to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#min)
     */
    min: string;
    /**
     * The level of detail used to represent time.
     *
     * @default "minutes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#timeResolution)
     */
    timeResolution: "minutes" | "seconds" | "milliseconds";
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#type)
     */
    readonly type: "datetimeoffset-picker";

    /**
     * Creates a deep clone of the `DateTimeOffsetPickerInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#clone)
     */
    clone(): DateTimeOffsetPickerInput;

    static fromJSON(json: any): DateTimeOffsetPickerInput;
  }

  interface DateTimeOffsetPickerInputProperties {
    /**
     * Indicates if the input should provide an option to select the time offset.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#includeTimeOffset)
     */
    includeTimeOffset?: boolean;
    /**
     * The maximum date/time offset to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#max)
     */
    max?: string;
    /**
     * The minimum date/time offset to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#min)
     */
    min?: string;
    /**
     * The level of detail used to represent time.
     *
     * @default "minutes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimeOffsetPickerInput.html#timeResolution)
     */
    timeResolution?: "minutes" | "seconds" | "milliseconds";
  }

  export interface DateTimePickerInput extends Accessor, JSONSupport {}

  export class DateTimePickerInput {
    /**
     * The `DateTimePickerInput` class defines the desired user interface for editing `date` field [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html)
     */

    constructor(properties?: DateTimePickerInputProperties);

    /**
     * Indicates if the input should provide an option to select the time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#includeTime)
     */
    includeTime: boolean;
    /**
     * The maximum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#max)
     */
    max: Date;
    /**
     * The minimum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#min)
     */
    min: Date;
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#type)
     */
    readonly type: "datetime-picker";

    /**
     * Creates a deep clone of the `DateTimePickerInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#clone)
     */
    clone(): DateTimePickerInput;

    static fromJSON(json: any): DateTimePickerInput;
  }

  interface DateTimePickerInputProperties {
    /**
     * Indicates if the input should provide an option to select the time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#includeTime)
     */
    includeTime?: boolean;
    /**
     * The maximum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#max)
     */
    max?: DateProperties;
    /**
     * The minimum date to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-DateTimePickerInput.html#min)
     */
    min?: DateProperties;
  }

  export interface RadioButtonsInput extends Accessor, JSONSupport {}

  export class RadioButtonsInput {
    /**
     * The `RadioButtonsInput` class defines the desired user interface for a radio button group input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html)
     */

    constructor(properties?: RadioButtonsInputProperties);

    /**
     * The text used to represent a null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html#noValueOptionLabel)
     */
    noValueOptionLabel: string;
    /**
     * Determines whether a null value option is displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html#showNoValueOption)
     */
    showNoValueOption: boolean;
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html#type)
     */
    readonly type: "radio-buttons";

    static fromJSON(json: any): RadioButtonsInput;
  }

  interface RadioButtonsInputProperties {
    /**
     * The text used to represent a null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html#noValueOptionLabel)
     */
    noValueOptionLabel?: string;
    /**
     * Determines whether a null value option is displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-RadioButtonsInput.html#showNoValueOption)
     */
    showNoValueOption?: boolean;
  }

  export interface SwitchInput extends Accessor, JSONSupport {}

  export class SwitchInput {
    /**
     * The `SwitchInput` class defines the desired user interface for a binary switch or toggle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html)
     */

    constructor(properties?: SwitchInputProperties);

    /**
     * Coded value used when the switch state is turned off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html#offValue)
     */
    offValue: string | number;
    /**
     * Coded value used when the switch state is turned on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html#onValue)
     */
    onValue: string | number;
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html#type)
     */
    readonly type: "switch";

    static fromJSON(json: any): SwitchInput;
  }

  interface SwitchInputProperties {
    /**
     * Coded value used when the switch state is turned off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html#offValue)
     */
    offValue?: string | number;
    /**
     * Coded value used when the switch state is turned on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-SwitchInput.html#onValue)
     */
    onValue?: string | number;
  }

  export interface TextAreaInput extends JSONSupport, TextInput {}

  export class TextAreaInput {
    /**
     * The `TextAreaInput` class defines the desired user interface as a multi-line text area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextAreaInput.html)
     */

    constructor(properties?: TextAreaInputProperties);

    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextAreaInput.html#type)
     */
    readonly type: "text-area";

    /**
     * Creates a deep clone of the `TextAreaInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextAreaInput.html#clone)
     */
    clone(): TextAreaInput;

    static fromJSON(json: any): TextAreaInput;
  }

  interface TextAreaInputProperties extends TextInputProperties {}

  export interface TextBoxInput extends Accessor, JSONSupport, TextInput {}

  export class TextBoxInput {
    /**
     * The `TextBoxInput` class defines the desired user interface as a single-line text box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextBoxInput.html)
     */

    constructor(properties?: TextBoxInputProperties);

    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextBoxInput.html#type)
     */
    readonly type: "text-box";

    /**
     * Creates a deep clone of the `TextBoxInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextBoxInput.html#clone)
     */
    clone(): TextBoxInput;

    static fromJSON(json: any): TextBoxInput;
  }

  interface TextBoxInputProperties extends TextInputProperties {}

  export class TextInput {
    /**
     * When set, defines the text input's maximum length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextInput.html#maxLength)
     */
    maxLength: number;
    /**
     * When set, defines the text input's minimum length.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextInput.html#minLength)
     */
    minLength: number;
  }

  interface TextInputProperties {
    /**
     * When set, defines the text input's maximum length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextInput.html#maxLength)
     */
    maxLength?: number;
    /**
     * When set, defines the text input's minimum length.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TextInput.html#minLength)
     */
    minLength?: number;
  }

  export interface TimePickerInput extends Accessor, JSONSupport {}

  export class TimePickerInput {
    /**
     * The `TimePickerInput` class defines the desired user interface for editing `time-only` field [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html)
     */

    constructor(properties?: TimePickerInputProperties);

    /**
     * The maximum time to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#max)
     */
    max: string;
    /**
     * The minimum time to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#min)
     */
    min: string;
    /**
     * The level of detail used to represent time.
     *
     * @default "minutes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#timeResolution)
     */
    timeResolution: "minutes" | "seconds";
    /**
     * The type of form element input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#type)
     */
    readonly type: "time-picker";

    /**
     * Creates a deep clone of the `TimePickerInput` class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#clone)
     */
    clone(): TimePickerInput;

    static fromJSON(json: any): TimePickerInput;
  }

  interface TimePickerInputProperties {
    /**
     * The maximum time to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#max)
     */
    max?: string;
    /**
     * The minimum time to allow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#min)
     */
    min?: string;
    /**
     * The level of detail used to represent time.
     *
     * @default "minutes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs-TimePickerInput.html#timeResolution)
     */
    timeResolution?: "minutes" | "seconds";
  }

  /**
   * The `BarcodeScannerInput` class defines the desired user interface is a barcode or QR code scanner.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#BarcodeScannerInput)
   */
  export type inputsBarcodeScannerInput = BarcodeScannerInput;

  /**
   * The `ComboBoxInput` class defines the desired user interface for a combo box group.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#ComboBoxInput)
   */
  export type inputsComboBoxInput = ComboBoxInput;

  /**
   * The `DatePickerInput` class defines the desired user interface for working with date-only types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#DatePickerInput)
   */
  export type inputsDatePickerInput = DatePickerInput;

  /**
   * The `DateTimeOffsetPickerInput` class defines the desired user interface for editing date and time fields in a form that also requires the option to include an offset from Coordinated Universal Time (UTC).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#DateTimeOffsetPickerInput)
   */
  export type inputsDateTimeOffsetPickerInput = DateTimeOffsetPickerInput;

  /**
   * The `DateTimePickerInput` class defines the desired user interface for editing date (including time) fields in a form.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#DateTimePickerInput)
   */
  export type inputsDateTimePickerInput = DateTimePickerInput;

  /**
   * Form element input types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#Input)
   */
  export type inputsInput =
    | BarcodeScannerInput
    | ComboBoxInput
    | DatePickerInput
    | DateTimeOffsetPickerInput
    | DateTimePickerInput
    | RadioButtonsInput
    | SwitchInput
    | TextAreaInput
    | TextBoxInput
    | TimePickerInput;

  /**
   * The `RadioButtonsInput` class defines the desired user interface for a radio button group.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#RadioButtonsInput)
   */
  export type inputsRadioButtonsInput = RadioButtonsInput;

  /**
   * The `SwitchInput` class defines the desired user interface for a binary switch or toggle.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#SwitchInput)
   */
  export type inputsSwitchInput = SwitchInput;

  /**
   * `TextAreaInput` defines the desired user interface is a multi-line text area.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#TextAreaInput)
   */
  export type inputsTextAreaInput = TextAreaInput;

  /**
   * `TextBoxInput` defines the desired user interface is a single-line text box.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#TextBoxInput)
   */
  export type inputsTextBoxInput = TextBoxInput;

  /**
   * The `TimePickerInput` class defines the desired user interface for working with time-only types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-inputs.html#TimePickerInput)
   */
  export type inputsTimePickerInput = TimePickerInput;

  export class RelationshipElement extends Element {
    /**
     * A `RelationshipElement` form element defines how a relationship between [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) and [tables](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#tables) participates in the [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html)
     */

    constructor(properties?: RelationshipElementProperties);

    /**
     * A numeric value indicating the maximum number of related features to display in the list of related records.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#displayCount)
     */
    displayCount: number;
    /**
     * A string value indicating how to display related records within the relationship content.
     *
     * @default "list"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#displayType)
     */
    displayType: "list";
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the FormTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#editableExpression)
     */
    editableExpression: string;
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#orderByFields)
     */
    orderByFields: RelatedRecordsInfoFieldOrder[];
    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#relationshipId)
     */
    relationshipId: number;
    /**
     * Indicates the type of form [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#type)
     */
    readonly type: "relationship";

    /**
     * Creates a deep clone of the RelationshipElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#clone)
     */
    clone(): RelationshipElement;

    static fromJSON(json: any): RelationshipElement;
  }

  interface RelationshipElementProperties extends ElementProperties {
    /**
     * A numeric value indicating the maximum number of related features to display in the list of related records.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#displayCount)
     */
    displayCount?: number;
    /**
     * A string value indicating how to display related records within the relationship content.
     *
     * @default "list"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#displayType)
     */
    displayType?: "list";
    /**
     * A reference to the [name](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name) of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression defined in the [expressionInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos) of the FormTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#editableExpression)
     */
    editableExpression?: string;
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#orderByFields)
     */
    orderByFields?: RelatedRecordsInfoFieldOrderProperties[];
    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-RelationshipElement.html#relationshipId)
     */
    relationshipId?: number;
  }

  /**
   * A convenience module for importing module:esri/form/elements/inputs/Input classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-support-inputs.html)
   */
  interface supportInputs {}

  export const supportInputs: supportInputs;

  export class TextElement extends Element {
    /**
     * A `TextElement` form element is used to define descriptive text as an element within a layer or [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) and can be used to aid those entering or updating information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html)
     */

    constructor(properties?: TextElementProperties);

    /**
     * The formatted string content to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#text)
     */
    text: string;
    /**
     * Defines the format of the `text` property.
     *
     * @default "plain-text"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#textFormat)
     */
    textFormat: "plain-text" | "markdown";
    /**
     * Indicates the type of form [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#type)
     */
    readonly type: "text";

    /**
     * Creates a deep clone of the TextElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#clone)
     */
    clone(): TextElement;

    static fromJSON(json: any): TextElement;
  }

  interface TextElementProperties extends ElementProperties {
    /**
     * The formatted string content to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#text)
     */
    text?: string;
    /**
     * Defines the format of the `text` property.
     *
     * @default "plain-text"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-TextElement.html#textFormat)
     */
    textFormat?: "plain-text" | "markdown";
  }

  export interface ExpressionInfo extends Accessor, JSONSupport {}

  export class ExpressionInfo {
    /**
     * The `ExpressionInfo` class defines the makeup of [visibility expressions](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#visibilityExpression), [required expressions](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#requiredExpression), [editable expressions](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#editableExpression), and [value expressions](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html)
     */

    constructor(properties?: ExpressionInfoProperties);

    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Form Constraint](https://developers.arcgis.com/javascript/latest/arcade/#forms) or [Form Calculation](https://developers.arcgis.com/javascript/latest/arcade/#form-calculation) profiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#expression)
     */
    expression: string;
    /**
     * The name of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name)
     */
    name: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#returnType)
     */
    returnType: "boolean" | "date" | "number" | "string";
    /**
     * The title used to describe the value returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the ExpressionInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#clone)
     */
    clone(): ExpressionInfo;

    static fromJSON(json: any): ExpressionInfo;
  }

  interface ExpressionInfoProperties {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Form Constraint](https://developers.arcgis.com/javascript/latest/arcade/#forms) or [Form Calculation](https://developers.arcgis.com/javascript/latest/arcade/#form-calculation) profiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#expression)
     */
    expression?: string;
    /**
     * The name of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#name)
     */
    name?: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#returnType)
     */
    returnType?: "boolean" | "date" | "number" | "string";
    /**
     * The title used to describe the value returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html#title)
     */
    title?: string;
  }

  export interface FormTemplate extends Accessor, JSONSupport {}

  export class FormTemplate {
    /**
     * A `FormTemplate` formats and defines the content of a [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) for a specific [Featurelayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or SubtypeSubLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html)
     */

    constructor(properties?: FormTemplateProperties);

    /**
     * The description of the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#description)
     */
    description: string;
    /**
     * An array of [form element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html) objects that represent an ordered list of form elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#elements)
     */
    elements: (AttachmentElement | FieldElement | GroupElement | RelationshipElement | TextElement)[];
    /**
     * An array of [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html) objects that reference [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions following the specification defined by the [Form Constraint Profile](https://developers.arcgis.com/javascript/latest/arcade/#constraint) or the [Form Calculation Profile](https://developers.arcgis.com/javascript/latest/arcade/#form-calculation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos)
     */
    expressionInfos: ExpressionInfo[];
    /**
     * Indicates whether to retain or clear a [form's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) [field element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#preserveFieldValuesWhenHidden)
     */
    preserveFieldValuesWhenHidden: boolean;
    /**
     * The string defining how to format the title displayed at the top of a form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the FormTemplate class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#clone)
     */
    clone(): FormTemplate;

    static fromJSON(json: any): FormTemplate;
  }

  interface FormTemplateProperties {
    /**
     * The description of the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#description)
     */
    description?: string;
    /**
     * An array of [form element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html) objects that represent an ordered list of form elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#elements)
     */
    elements?: (
      | (AttachmentElementProperties & { type: "attachment" })
      | (FieldElementProperties & { type: "field" })
      | (GroupElementProperties & { type: "group" })
      | (RelationshipElementProperties & { type: "relationship" })
      | (TextElementProperties & { type: "text" })
    )[];
    /**
     * An array of [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-ExpressionInfo.html) objects that reference [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions following the specification defined by the [Form Constraint Profile](https://developers.arcgis.com/javascript/latest/arcade/#constraint) or the [Form Calculation Profile](https://developers.arcgis.com/javascript/latest/arcade/#form-calculation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#expressionInfos)
     */
    expressionInfos?: ExpressionInfoProperties[];
    /**
     * Indicates whether to retain or clear a [form's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) [field element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-FieldElement.html) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#preserveFieldValuesWhenHidden)
     */
    preserveFieldValuesWhenHidden?: boolean;
    /**
     * The string defining how to format the title displayed at the top of a form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html#title)
     */
    title?: string;
  }

  /**
   * A convenience module for importing [Element](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-elements-Element.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-support-elements.html)
   */
  interface elements {}

  export const elements: elements;

  /**
   * A convenience module for importing [Geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html)
   */
  namespace geometry {
    /**
     * Spatial Reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#SpatialReference)
     */
    export type SpatialReference = __esri.SpatialReference;
    export const SpatialReference: typeof __esri.SpatialReference;

    /**
     * Geometry types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Geometry)
     */
    export type Geometry =
      | __esri.Extent
      | __esri.Multipoint
      | __esri.Point
      | __esri.Polygon
      | __esri.Polyline
      | __esri.Mesh;

    /**
     * Extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Extent)
     */
    export type Extent = __esri.Extent;
    export const Extent: typeof __esri.Extent;

    /**
     * Multipoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Multipoint)
     */
    export type Multipoint = __esri.Multipoint;
    export const Multipoint: typeof __esri.Multipoint;

    /**
     * Point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Point)
     */
    export type Point = __esri.Point;
    export const Point: typeof __esri.Point;

    /**
     * Polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Polygon)
     */
    export type Polygon = __esri.Polygon;
    export const Polygon: typeof __esri.Polygon;

    /**
     * Polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Polyline)
     */
    export type Polyline = __esri.Polyline;
    export const Polyline: typeof __esri.Polyline;

    /**
     * Mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Mesh)
     */
    export type Mesh = __esri.Mesh;
    export const Mesh: typeof __esri.Mesh;
  }

  export class Circle extends Polygon {
    /**
     * A circle is a [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) created by specifying a [center point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#center) and a [radius](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#radius).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html)
     */

    constructor(properties?: CircleProperties);

    /**
     * The center point of the circle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#center)
     */
    center: Point;
    /**
     * Applicable when the spatial reference of the center point is either set to Web Mercator (wkid: 3857) or geographic/geodesic (wkid: 4326).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#geodesic)
     */
    geodesic: boolean;
    /**
     * This value defines the number of points along the curve of the circle.
     *
     * @default 60
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#numberOfPoints)
     */
    numberOfPoints: number;
    /**
     * The radius of the circle.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#radius)
     */
    radius: number;
    /**
     * Unit of the radius.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#radiusUnit)
     */
    radiusUnit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";

    /**
     * Creates a deep clone of Circle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#clone)
     */
    clone(): Circle;

    static fromJSON(json: any): Circle;
  }

  interface CircleProperties extends PolygonProperties {
    /**
     * The center point of the circle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#center)
     */
    center?: PointProperties;
    /**
     * Applicable when the spatial reference of the center point is either set to Web Mercator (wkid: 3857) or geographic/geodesic (wkid: 4326).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#geodesic)
     */
    geodesic?: boolean;
    /**
     * This value defines the number of points along the curve of the circle.
     *
     * @default 60
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#numberOfPoints)
     */
    numberOfPoints?: number;
    /**
     * The radius of the circle.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#radius)
     */
    radius?: number;
    /**
     * Unit of the radius.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Circle.html#radiusUnit)
     */
    radiusUnit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
  }

  /**
   * Converts between [points](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) and formatted coordinates notation strings such as:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html)
   */
  interface coordinateFormatter {
    /**
     * Parses coordinates in latitude/longitude notation, and returns a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing that location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#fromLatitudeLongitude)
     */
    fromLatitudeLongitude(coordinates: string, spatialReference?: SpatialReference): Point;
    /**
     * Parses coordinates in Military Grid Reference System (MGRS) notation, and returns a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing that location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#fromMgrs)
     */
    fromMgrs(
      coordinates: string,
      spatialReference: SpatialReference,
      conversionMode:
        | "automatic"
        | "new-180-in-zone-01"
        | "new-180-in-zone-60"
        | "old-180-in-zone-01"
        | "old-180-in-zone-60",
    ): Point;
    /**
     * Parses coordinates in United States National Grid (USNG) notation, and returns a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing that location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#fromUsng)
     */
    fromUsng(coordinates: string, spatialReference?: SpatialReference): Point;
    /**
     * Parses coordinates in Universal Transverse Mercator (UTM) notation, and returns a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing that location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#fromUtm)
     */
    fromUtm(
      coordinates: string,
      spatialReference: SpatialReference,
      conversionMode: "latitude-band-indicators" | "north-south-indicators",
    ): Point;
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Indicates if this module is supported in the current browser.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#isSupported)
     */
    isSupported(): boolean;
    /**
     * Load this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#load)
     */
    load(): Promise<any>;
    /**
     * Returns formatted coordinates in latitude/longitude notation representing the given point's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#toLatitudeLongitude)
     */
    toLatitudeLongitude(point: Point, format: "dd" | "ddm" | "dms", decimalPlaces?: number): string;
    /**
     * Returns formatted coordinates in Military Grid Reference System (MGRS) notation representing the given point's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#toMgrs)
     */
    toMgrs(
      point: Point,
      conversionMode:
        | "automatic"
        | "new-180-in-zone-01"
        | "new-180-in-zone-60"
        | "old-180-in-zone-01"
        | "old-180-in-zone-60",
      precision?: number,
      addSpaces?: boolean,
    ): string;
    /**
     * Returns formatted coordinates in United States National Grid (USNG) notation representing the given point's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#toUsng)
     */
    toUsng(point: Point, precision?: number, addSpaces?: boolean): string;
    /**
     * Returns formatted coordinates in Universal Transverse Mercator (UTM) notation representing the given point's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-coordinateFormatter.html#toUtm)
     */
    toUtm(
      point: Point,
      conversionMode: "latitude-band-indicators" | "north-south-indicators",
      addSpaces?: boolean,
    ): string;
  }

  export const coordinateFormatter: coordinateFormatter;

  export class Extent extends Geometry {
    /**
     * The minimum and maximum X and Y coordinates of a bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html)
     */

    constructor(properties?: ExtentProperties);

    /**
     * The center point of the extent in map units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#center)
     */
    readonly center: Point;
    /**
     * The height of the extent in map units (the distance between [ymin](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymin) and [ymax](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymax)).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#height)
     */
    readonly height: number;
    /**
     * The maximum possible `m` value in an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#mmax)
     */
    mmax: number;
    /**
     * The minimum possible `m` value of an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#mmin)
     */
    mmin: number;

    readonly type: "extent";
    /**
     * The width of the extent in map units (the distance between [xmin](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmin) and [xmax](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmax)).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#width)
     */
    readonly width: number;
    /**
     * The maximum X-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmax)
     */
    xmax: number;
    /**
     * The minimum X-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmin)
     */
    xmin: number;
    /**
     * The maximum Y-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymax)
     */
    ymax: number;
    /**
     * The minimum Y-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymin)
     */
    ymin: number;
    /**
     * The maximum possible `z`, or elevation, value in an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#zmax)
     */
    zmax: number;
    /**
     * The minimum possible `z`, or elevation, value of an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#zmin)
     */
    zmin: number;

    /**
     * Centers the extent to the specified [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#centerAt)
     */
    centerAt(point: Point): Extent;
    /**
     * Creates a deep clone of Extent object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#clone)
     */
    clone(): Extent;
    /**
     * Checks if the input geometry is contained within the extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#contains)
     */
    contains(geometry: Point | Extent): boolean;
    /**
     * Indicates if the input extent is equal to the testing extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#equals)
     */
    equals(extent: Extent): boolean;
    /**
     * Expands the extent by the given factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#expand)
     */
    expand(factor: number): Extent;
    /**
     * Shrinks the original extent to the intersection with the input extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#intersection)
     */
    intersection(extent: Extent): Extent;
    /**
     * Tests to validate if the input geometry intersects the extent and returns a Boolean value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#intersects)
     */
    intersects(geometry: Geometry): boolean;
    /**
     * Returns an array with either one Extent that's been shifted to within +/- 180 or two Extents if the original extent intersects the International Dateline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#normalize)
     */
    normalize(): Extent[];
    /**
     * Modifies the extent geometry in-place with X and Y offsets in map units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#offset)
     */
    offset(dx: number, dy: number, dz: number): Extent;
    /**
     * Expands the original extent to include the extent of the input Extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#union)
     */
    union(extent: Extent): Extent;

    static fromJSON(json: any): Extent;
  }

  interface ExtentProperties extends GeometryProperties {
    /**
     * The maximum possible `m` value in an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#mmax)
     */
    mmax?: number;
    /**
     * The minimum possible `m` value of an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#mmin)
     */
    mmin?: number;
    /**
     * The maximum X-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmax)
     */
    xmax?: number;
    /**
     * The minimum X-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#xmin)
     */
    xmin?: number;
    /**
     * The maximum Y-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymax)
     */
    ymax?: number;
    /**
     * The minimum Y-coordinate of an extent envelope.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#ymin)
     */
    ymin?: number;
    /**
     * The maximum possible `z`, or elevation, value in an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#zmax)
     */
    zmax?: number;
    /**
     * The minimum possible `z`, or elevation, value of an extent envelope.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html#zmin)
     */
    zmin?: number;
  }

  export interface Geometry extends Accessor, JSONSupport {}

  export class Geometry {
    /**
     * The base class for geometry objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html)
     */

    constructor(properties?: GeometryProperties);

    /**
     * The cache is used to store values computed from geometries that need to be cleared or recomputed upon mutation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#cache)
     */
    readonly cache: any;
    /**
     * The extent of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#extent)
     */
    readonly extent: Extent;
    /**
     * Indicates if the geometry has M values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#hasM)
     */
    hasM: boolean;
    /**
     * Indicates if the geometry has z-values (elevation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#hasZ)
     */
    hasZ: boolean;
    /**
     * The spatial reference of the geometry.
     *
     * @default SpatialReference.WGS84 // wkid: 4326
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#type)
     */
    readonly type: "point" | "multipoint" | "polyline" | "polygon" | "extent" | "mesh";

    /**
     * Creates a deep clone of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#clone)
     */
    clone(): Geometry;

    static fromJSON(json: any): Geometry;
  }

  interface GeometryProperties {
    /**
     * Indicates if the geometry has M values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#hasM)
     */
    hasM?: boolean;
    /**
     * Indicates if the geometry has z-values (elevation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#hasZ)
     */
    hasZ?: boolean;
    /**
     * The spatial reference of the geometry.
     *
     * @default SpatialReference.WGS84 // wkid: 4326
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
  }

  /**
   * A client-side geometry engine for testing, measuring, and analyzing the spatial relationship between two or more 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html)
   */
  interface geometryEngine {
    /**
     * Creates planar (or Euclidean) buffer polygons at a specified distance around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#buffer)
     */
    buffer(
      geometry: Geometry | Geometry[],
      distance: number | number[],
      unit?: LinearUnits,
      unionResults?: boolean,
    ): Polygon | Polygon[];
    /**
     * Calculates the clipped geometry from a target geometry by an envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#clip)
     */
    clip(geometry: Geometry, envelope: Extent): Geometry;
    /**
     * Indicates if one geometry contains another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#contains)
     */
    contains(containerGeometry: Geometry, insideGeometry: Geometry): boolean;
    /**
     * Calculates the convex hull of one or more geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#convexHull)
     */
    convexHull(geometry: Geometry | Geometry[], merge?: boolean): Geometry | Geometry[];
    /**
     * Indicates if one geometry crosses another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#crosses)
     */
    crosses(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * Splits the input Polyline or Polygon where it crosses a cutting Polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#cut)
     */
    cut(geometry: Geometry, cutter: Polyline): Geometry[];
    /**
     * Densify geometries by plotting points between existing vertices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#densify)
     */
    densify(geometry: Geometry, maxSegmentLength: number, maxSegmentLengthUnit?: LinearUnits): Geometry;
    /**
     * Creates the difference of two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#difference)
     */
    difference(inputGeometry: Geometry | Geometry[], subtractor: Geometry): Geometry | Geometry[];
    /**
     * Indicates if one geometry is disjoint (doesn't intersect in any way) with another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#disjoint)
     */
    disjoint(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * Calculates the shortest planar distance between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#distance)
     */
    distance(geometry1: Geometry, geometry2: Geometry, distanceUnit?: LinearUnits): number;
    /**
     * Indicates if two geometries are equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#equals)
     */
    equals(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * Returns an object containing additional information about the input spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#extendedSpatialReferenceInfo)
     */
    extendedSpatialReferenceInfo(spatialReference: SpatialReference): SpatialReferenceInfo;
    /**
     * Flips a geometry on the horizontal axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#flipHorizontal)
     */
    flipHorizontal(geometry: Geometry, flipOrigin?: Point): Geometry;
    /**
     * Flips a geometry on the vertical axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#flipVertical)
     */
    flipVertical(geometry: Geometry, flipOrigin?: Point): Geometry;
    /**
     * Performs the generalize operation on the geometries in the cursor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#generalize)
     */
    generalize(
      geometry: Geometry,
      maxDeviation: number,
      removeDegenerateParts?: boolean,
      maxDeviationUnit?: LinearUnits,
    ): Geometry;
    /**
     * Calculates the area of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#geodesicArea)
     */
    geodesicArea(geometry: Polygon, unit?: AreaUnits): number;
    /**
     * Creates geodesic buffer polygons at a specified distance around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#geodesicBuffer)
     */
    geodesicBuffer(
      geometry: Geometry | Geometry[],
      distance: number | number[],
      unit?: LinearUnits,
      unionResults?: boolean,
    ): Polygon | Polygon[];
    /**
     * Returns a geodetically densified version of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#geodesicDensify)
     */
    geodesicDensify(
      geometry: Polyline | Polygon,
      maxSegmentLength: number,
      maxSegmentLengthUnit?: LinearUnits,
    ): Geometry;
    /**
     * Calculates the length of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#geodesicLength)
     */
    geodesicLength(geometry: Geometry, unit?: LinearUnits): number;
    /**
     * Creates new geometries from the intersections between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#intersect)
     */
    intersect(geometry1: Geometry | Geometry[], geometry2: Geometry): Geometry | Geometry[];
    /**
     * Returns an array of points at the intersecting locations of two input polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#intersectLinesToPoints)
     */
    intersectLinesToPoints(line1: Polyline, line2: Polyline): Point[];
    /**
     * Indicates if one geometry intersects another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#intersects)
     */
    intersects(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * Indicates if the given geometry is topologically simple.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#isSimple)
     */
    isSimple(geometry: Geometry): boolean;
    /**
     * Finds the coordinate of the geometry that is closest to the specified point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestCoordinate)
     */
    nearestCoordinate(geometry: Geometry, inputPoint: Point): NearestPointResult;
    /**
     * Finds the vertex on the geometry nearest to the specified point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestVertex)
     */
    nearestVertex(geometry: Geometry, inputPoint: Point): NearestPointResult;
    /**
     * Finds all vertices in the given distance from the specified point, sorted from the closest to the furthest and returns them as an array of Objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestVertices)
     */
    nearestVertices(
      geometry: Geometry,
      inputPoint: Point,
      searchRadius: number,
      maxVertexCountToReturn: number,
    ): NearestPointResult[];
    /**
     * The offset operation creates a geometry that is a constant planar distance from an input polyline or polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#offset)
     */
    offset(
      geometry: Geometry | Geometry[],
      offsetDistance: number,
      offsetUnit?: LinearUnits,
      joinType?: "round" | "bevel" | "miter" | "square",
      bevelRatio?: number,
      flattenError?: number,
    ): Geometry | Geometry[];
    /**
     * Indicates if one geometry overlaps another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#overlaps)
     */
    overlaps(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * Calculates the area of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#planarArea)
     */
    planarArea(geometry: Polygon, unit?: AreaUnits): number;
    /**
     * Calculates the length of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#planarLength)
     */
    planarLength(geometry: Geometry, unit?: LinearUnits): number;
    /**
     * Indicates if the given DE-9IM relation is true for the two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#relate)
     */
    relate(geometry1: Geometry, geometry2: Geometry, relation: string): boolean;
    /**
     * Rotates a geometry counterclockwise by the specified number of degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#rotate)
     */
    rotate(geometry: Geometry, angle: number, rotationOrigin?: Point): Geometry;
    /**
     * Performs the simplify operation on the geometry, which alters the given geometries to make their definitions topologically legal with respect to their geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#simplify)
     */
    simplify(geometry: Geometry): Geometry;
    /**
     * Creates the symmetric difference of two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#symmetricDifference)
     */
    symmetricDifference(leftGeometry: Geometry | Geometry[], rightGeometry: Geometry): Geometry | Geometry[];
    /**
     * Indicates if one geometry touches another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#touches)
     */
    touches(geometry1: Geometry, geometry2: Geometry): boolean;
    /**
     * All inputs must be of the same type of geometries and share one spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#union)
     */
    union(geometries: Geometry[]): Geometry;
    /**
     * Indicates if one geometry is within another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#within)
     */
    within(innerGeometry: Geometry, outerGeometry: Geometry): boolean;
  }

  export const geometryEngine: geometryEngine;

  /**
   * Units for area measurements.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#AreaUnits)
   */
  export type AreaUnits =
    | "acres"
    | "ares"
    | "hectares"
    | "square-feet"
    | "square-meters"
    | "square-yards"
    | "square-kilometers"
    | "square-miles"
    | number;

  /**
   * Units for linear measurements.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#LinearUnits)
   */
  export type LinearUnits = "meters" | "feet" | "kilometers" | "miles" | "nautical-miles" | "yards" | number;

  /**
   * Object returned from the [nearestCoordinate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestCoordinate), [nearestVertex()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestVertex), and [nearestVertices()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#nearestVertices) methods.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#NearestPointResult)
   */
  export interface NearestPointResult {
    /**
     * A vertex within the specified distance of the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#NearestPointResult)
     */
    coordinate: Point;
    /**
     * The distance from the `inputPoint` in the units of the view's spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#NearestPointResult)
     */
    distance: number;
    /**
     * The index of the vertex within the geometry's rings or paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#NearestPointResult)
     */
    vertexIndex: number;
    /**
     * Indicates if it is an empty geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#NearestPointResult)
     */
    isEmpty: boolean;
  }

  /**
   * The return object of the [extendedSpatialReferenceInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#extendedSpatialReferenceInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
   */
  export interface SpatialReferenceInfo {
    /**
     * The XY tolerance of the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
     */
    tolerance: number;
    /**
     * Base factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
     */
    unitBaseFactor: number;
    /**
     * Unit ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
     */
    unitID: number;
    /**
     * Square derivative.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
     */
    unitSquareDerivative: number;
    /**
     * Unit type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngine.html#SpatialReferenceInfo)
     */
    unitType: number;
  }

  /**
   * An asynchronous client-side geometry engine for testing, measuring, and analyzing the spatial relationship between two or more 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html)
   */
  interface geometryEngineAsync {
    /**
     * Creates planar (or Euclidean) buffer polygons at a specified distance around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#buffer)
     */
    buffer(
      geometry: Geometry | Geometry[],
      distance: number | number[],
      unit?: LinearUnits,
      unionResults?: boolean,
    ): Promise<Polygon | Polygon[]>;
    /**
     * Calculates the clipped geometry from a target geometry by an envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#clip)
     */
    clip(geometry: Geometry, envelope: Extent): Promise<Geometry>;
    /**
     * Indicates if one geometry contains another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#contains)
     */
    contains(containerGeometry: Geometry, insideGeometry: Geometry): Promise<boolean>;
    /**
     * Calculates the convex hull of one or more geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#convexHull)
     */
    convexHull(geometry: Geometry | Geometry[], merge?: boolean): Promise<Geometry | Geometry[]>;
    /**
     * Indicates if one geometry crosses another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#crosses)
     */
    crosses(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * Split the input Polyline or Polygon where it crosses a cutting Polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#cut)
     */
    cut(geometry: Geometry, cutter: Polyline): Promise<Geometry[]>;
    /**
     * Densify geometries by plotting points between existing vertices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#densify)
     */
    densify(geometry: Geometry, maxSegmentLength: number, maxSegmentLengthUnit?: LinearUnits): Promise<Geometry>;
    /**
     * Creates the difference of two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#difference)
     */
    difference(inputGeometry: Geometry | Geometry[], subtractor: Geometry): Promise<Geometry>;
    /**
     * Indicates if one geometry is disjoint (doesn't intersect in any way) with another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#disjoint)
     */
    disjoint(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * Calculates the shortest planar distance between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#distance)
     */
    distance(geometry1: Geometry, geometry2: Geometry, distanceUnit?: LinearUnits): Promise<number>;
    /**
     * Indicates if two geometries are equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#equals)
     */
    equals(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * Returns an object containing additional information about the input spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#extendedSpatialReferenceInfo)
     */
    extendedSpatialReferenceInfo(spatialReference: SpatialReference): Promise<geometryEngineAsyncSpatialReferenceInfo>;
    /**
     * Flips a geometry on the horizontal axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#flipHorizontal)
     */
    flipHorizontal(geometry: Geometry, flipOrigin?: Point): Promise<Geometry>;
    /**
     * Flips a geometry on the vertical axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#flipVertical)
     */
    flipVertical(geometry: Geometry, flipOrigin?: Point): Promise<Geometry>;
    /**
     * Performs the generalize operation on the geometries in the cursor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#generalize)
     */
    generalize(
      geometry: Geometry,
      maxDeviation: number,
      removeDegenerateParts?: boolean,
      maxDeviationUnit?: LinearUnits,
    ): Promise<Geometry>;
    /**
     * Calculates the area of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#geodesicArea)
     */
    geodesicArea(geometry: Polygon, unit?: AreaUnits): Promise<number>;
    /**
     * Creates geodesic buffer polygons at a specified distance around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#geodesicBuffer)
     */
    geodesicBuffer(
      geometry: Geometry | Geometry[],
      distance: number | number[],
      unit?: LinearUnits,
      unionResults?: boolean,
    ): Promise<Polygon | Polygon[]>;
    /**
     * Returns a geodetically densified version of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#geodesicDensify)
     */
    geodesicDensify(
      geometry: Polyline | Polygon,
      maxSegmentLength: number,
      maxSegmentLengthUnit?: LinearUnits,
    ): Promise<Geometry>;
    /**
     * Calculates the length of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#geodesicLength)
     */
    geodesicLength(geometry: Geometry, unit?: LinearUnits): Promise<number>;
    /**
     * Creates new geometries from the intersections between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#intersect)
     */
    intersect(geometry1: Geometry | Geometry[], geometry2: Geometry): Promise<Geometry>;
    /**
     * Resolves to an array of points at the intersecting locations of two input polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#intersectLinesToPoints)
     */
    intersectLinesToPoints(line1: Polyline, line2: Polyline): Promise<Point[]>;
    /**
     * Indicates if one geometry intersects another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#intersects)
     */
    intersects(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * Indicates if the given geometry is topologically simple.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#isSimple)
     */
    isSimple(geometry: Geometry): Promise<boolean>;
    /**
     * Finds the coordinate of the geometry that is closest to the specified point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#nearestCoordinate)
     */
    nearestCoordinate(geometry: Geometry, inputPoint: Point): Promise<NearestPointResult>;
    /**
     * Finds vertex on the geometry nearest to the specified point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#nearestVertex)
     */
    nearestVertex(geometry: Geometry, inputPoint: Point): Promise<NearestPointResult>;
    /**
     * Finds all vertices in the given distance from the specified point, sorted from the closest to the furthest and returns them as an array of Objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#nearestVertices)
     */
    nearestVertices(
      geometry: Geometry,
      inputPoint: Point,
      searchRadius: number,
      maxVertexCountToReturn: number,
    ): Promise<NearestPointResult>;
    /**
     * The offset operation creates a geometry that is a constant planar distance from an input polyline or polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#offset)
     */
    offset(
      geometry: Geometry | Geometry[],
      offsetDistance: number,
      offsetUnit?: LinearUnits,
      joinType?: "round" | "bevel" | "miter" | "square",
      bevelRatio?: number,
      flattenError?: number,
    ): Promise<Geometry | Geometry[]>;
    /**
     * Indicates if one geometry overlaps another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#overlaps)
     */
    overlaps(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * Calculates the area of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#planarArea)
     */
    planarArea(geometry: Polygon, unit?: AreaUnits): Promise<number>;
    /**
     * Calculates the length of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#planarLength)
     */
    planarLength(geometry: Geometry, unit?: LinearUnits): Promise<number>;
    /**
     * Indicates if the given DE-9IM relation holds for the two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#relate)
     */
    relate(geometry1: Geometry, geometry2: Geometry, relation: string): Promise<boolean>;
    /**
     * Rotates a geometry counterclockwise by the specified number of degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#rotate)
     */
    rotate(geometry: Geometry, angle: number, rotationOrigin?: Point): Promise<Geometry>;
    /**
     * Performs the simplify operation on the geometry, which alters the given geometries to make their definitions topologically legal with respect to their geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#simplify)
     */
    simplify(geometry: Geometry): Promise<Geometry>;
    /**
     * Creates the symmetric difference of two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#symmetricDifference)
     */
    symmetricDifference(leftGeometry: Geometry | Geometry[], rightGeometry: Geometry): Promise<Geometry | Geometry[]>;
    /**
     * Indicates if one geometry touches another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#touches)
     */
    touches(geometry1: Geometry, geometry2: Geometry): Promise<boolean>;
    /**
     * All inputs must be of the same type of geometries and share one spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#union)
     */
    union(geometries: Geometry[]): Promise<Geometry>;
    /**
     * Indicates if one geometry is within another geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#within)
     */
    within(innerGeometry: Geometry, outerGeometry: Geometry): Promise<boolean>;
  }

  export const geometryEngineAsync: geometryEngineAsync;

  /**
   * The return object of the [extendedSpatialReferenceInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#extendedSpatialReferenceInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
   */
  export interface geometryEngineAsyncSpatialReferenceInfo {
    /**
     * The XY tolerance of the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
     */
    tolerance: number;
    /**
     * Base factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
     */
    unitBaseFactor: number;
    /**
     * Unit ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
     */
    unitID: number;
    /**
     * Square derivative.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
     */
    unitSquareDerivative: number;
    /**
     * Unit type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-geometryEngineAsync.html#SpatialReferenceInfo)
     */
    unitType: number;
  }

  export interface HeightModelInfo extends Accessor, JSONSupport {}

  export class HeightModelInfo {
    /**
     * The height model info defines the characteristics of a vertical coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html)
     */

    constructor(properties?: HeightModelInfoProperties);

    /**
     * The surface type or height model of the vertical coordinate system (VCS).
     *
     * @default "gravity-related-height"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#heightModel)
     */
    readonly heightModel: "gravity-related-height" | "ellipsoidal";
    /**
     * The unit of the vertical coordinate system.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#heightUnit)
     */
    readonly heightUnit:
      | "meters"
      | "feet"
      | "us-feet"
      | "clarke-feet"
      | "clarke-yards"
      | "clarke-links"
      | "sears-yards"
      | "sears-feet"
      | "sears-chains"
      | "benoit-1895-b-chains"
      | "indian-yards"
      | "indian-1937-yards"
      | "gold-coast-feet"
      | "chains"
      | "sears-1922-truncated-chains"
      | "50-kilometers"
      | "150-kilometers";
    /**
     * The datum realization of the vertical coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#vertCRS)
     */
    readonly vertCRS: string;

    static fromJSON(json: any): HeightModelInfo;
  }

  interface HeightModelInfoProperties {
    /**
     * The surface type or height model of the vertical coordinate system (VCS).
     *
     * @default "gravity-related-height"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#heightModel)
     */
    heightModel?: "gravity-related-height" | "ellipsoidal";
    /**
     * The unit of the vertical coordinate system.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#heightUnit)
     */
    heightUnit?:
      | "meters"
      | "feet"
      | "us-feet"
      | "clarke-feet"
      | "clarke-yards"
      | "clarke-links"
      | "sears-yards"
      | "sears-feet"
      | "sears-chains"
      | "benoit-1895-b-chains"
      | "indian-yards"
      | "indian-1937-yards"
      | "gold-coast-feet"
      | "chains"
      | "sears-1922-truncated-chains"
      | "50-kilometers"
      | "150-kilometers";
    /**
     * The datum realization of the vertical coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-HeightModelInfo.html#vertCRS)
     */
    vertCRS?: string;
  }

  export interface Mesh extends Geometry, Loadable {}

  export class Mesh {
    /**
     * A mesh is a general, client-side 3D [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html) type composed of [vertices with attributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html)
     */

    constructor(properties?: MeshProperties);

    /**
     * An array of mesh components that can be used to apply materials to different regions of the same mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#components)
     */
    components: MeshComponent[];
    /**
     * The origin of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#origin)
     */
    readonly origin: Point;
    /**
     * Additional local transformation of the mesh vertices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#transform)
     */
    transform: MeshTransform;
    /**
     * The string value representing the type of geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#type)
     */
    readonly type: "mesh";
    /**
     * Object describing the attributes of each vertex of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    vertexAttributes: MeshVertexAttributes;
    /**
     * The vertex space of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexSpace)
     */
    vertexSpace: MeshGeoreferencedVertexSpace | MeshLocalVertexSpace;

    /**
     * Adds a component to the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#addComponent)
     */
    addComponent(component: MeshComponent | MeshComponentProperties): void;
    /**
     * Centers the mesh at the specified location without changing its scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#centerAt)
     */
    centerAt(location: Point, parameters?: MeshCenterAtParameters): Mesh;
    /**
     * Creates a deep clone of the Mesh object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#clone)
     */
    clone(): Mesh;
    /**
     * Offsets the mesh geometry by the specified distance in x, y, and z.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#offset)
     */
    offset(dx: number, dy: number, dz: number): Mesh;
    /**
     * Removes a component from the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#removeComponent)
     */
    removeComponent(component: MeshComponent): void;
    /**
     * Rotates the mesh geometry around its x, y and z axis (in that order).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#rotate)
     */
    rotate(angleX: number, angleY: number, angleZ: number, parameters?: MeshRotateParameters): Mesh;
    /**
     * Scales the mesh geometry by the specified factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#scale)
     */
    scale(factor: number, parameters?: MeshScaleParameters): Mesh;
    /**
     * Export a mesh to a binary glTF (glb) representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#toBinaryGLTF)
     */
    toBinaryGLTF(options?: MeshToBinaryGLTFOptions): Promise<ArrayBuffer>;
    /**
     * Notifies that any cached values that depend on vertex attributes need to be recalculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributesChanged)
     */
    vertexAttributesChanged(): void;

    /**
     * Creates a mesh representing a box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createBox)
     */
    static createBox(location: Point, parameters?: CreateBoxParameters): Mesh;
    /**
     * Creates a mesh representing a cylinder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createCylinder)
     */
    static createCylinder(location: Point, parameters?: CreateCylinderParameters): Mesh;
    /**
     * Creates a mesh geometry from a file or list of files and which is ready to be displayed in the view.
     *
     * @deprecated Use {@link module:module:esri/layers/SceneLayer~convertMesh convertMesh} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromFiles)
     */
    static createFromFiles(location: Point, files: File[], parameters?: MeshCreateFromFilesParameters): Promise<Mesh>;
    /**
     * Creates a new mesh geometry from a glTF model referenced by the `url` parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromGLTF)
     */
    static createFromGLTF(location: Point, url: string, parameters?: CreateFromGLTFParameters): Promise<Mesh>;
    /**
     * Creates a new mesh geometry from a polygon geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromPolygon)
     */
    static createFromPolygon(polygon: Polygon, parameters?: MeshCreateFromPolygonParameters): Mesh;
    /**
     * Creates a mesh representing a plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createPlane)
     */
    static createPlane(location: Point, parameters?: CreatePlaneParameters): Mesh;
    /**
     * Creates a mesh representing a sphere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createSphere)
     */
    static createSphere(location: Point, parameters?: CreateSphereParameters): Mesh;
    static fromJSON(json: any): Mesh;
  }

  interface MeshProperties extends GeometryProperties, LoadableProperties {
    /**
     * An array of mesh components that can be used to apply materials to different regions of the same mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#components)
     */
    components?: MeshComponentProperties[];
    /**
     * Additional local transformation of the mesh vertices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#transform)
     */
    transform?: MeshTransformProperties;
    /**
     * Object describing the attributes of each vertex of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    vertexAttributes?: MeshVertexAttributesProperties;
    /**
     * The vertex space of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexSpace)
     */
    vertexSpace?:
      | (MeshGeoreferencedVertexSpaceProperties & { type: "georeferenced" })
      | (MeshLocalVertexSpaceProperties & { type: "local" });
  }

  /**
   * Parameters used to configure box mesh creation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
   */
  export interface CreateBoxParameters {
    /**
     * A uniform size value or an object containing individual values width, height and depth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    size?: number | CreateBoxParametersSize;
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    geographic?: boolean;
    /**
     * The unit of the size (defaults to the unit of the location's spatial reference).
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
    /**
     * Specifies the vertex space of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    vertexSpace?: "local" | "georeferenced";
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    material?: MeshMaterial;
    /**
     * The face for generating image uv coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    imageFace?: "east" | "west" | "north" | "south" | "up" | "down";
  }

  /**
   * Parameters used to configure cylinder mesh creation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
   */
  export interface CreateCylinderParameters {
    /**
     * A uniform size value or an object containing individual values width, height and depth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    size?: number | any;
    /**
     * The width of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    width?: number;
    /**
     * The depth of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    depth?: number;
    /**
     * The height of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    height?: number;
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    geographic?: boolean;
    /**
     * The unit of the size (defaults to the unit of the location's spatial reference).
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
    /**
     * The additional number of subdivisions for generating the mesh representing a sphere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    densificationFactor?: number;
    /**
     * Specifies the vertex space of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    vertexSpace?: "local" | "georeferenced";
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateCylinderParameters)
     */
    material?: MeshMaterial;
  }

  /**
   * Parameters used to configure gltf mesh creation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateFromGLTFParameters)
   */
  export interface CreateFromGLTFParameters {
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateFromGLTFParameters)
     */
    geographic?: boolean;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the loading process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateFromGLTFParameters)
     */
    signal?: AbortSignal;
    /**
     * Specifies the vertex space of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateFromGLTFParameters)
     */
    vertexSpace?: "local" | "georeferenced";
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateFromGLTFParameters)
     */
    material?: MeshMaterial;
  }

  /**
   * Parameters used to configure plane mesh creation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
   */
  export interface CreatePlaneParameters {
    /**
     * A uniform size value or an object containing individual values width, height and depth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    size?: number | any;
    /**
     * The width of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    width?: number;
    /**
     * The depth of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    depth?: number;
    /**
     * The height of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    height?: number;
    /**
     * Direction the plane is facing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    facing?: "east" | "west" | "north" | "south" | "up" | "down";
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    geographic?: boolean;
    /**
     * The unit of the size (defaults to the unit of the location's spatial reference).
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
    /**
     * The additional number of subdivisions for generating the mesh representing a sphere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    densificationFactor?: number;
    /**
     * Specifies the vertex space of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    vertexSpace?: "local" | "georeferenced";
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreatePlaneParameters)
     */
    material?: MeshMaterial;
  }

  /**
   * Parameters used to configure sphere mesh creation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
   */
  export interface CreateSphereParameters {
    /**
     * A uniform size value or an object containing individual values width, height and depth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    size?: number | any;
    /**
     * The width of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    width?: number;
    /**
     * The depth of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    depth?: number;
    /**
     * The height of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    height?: number;
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    geographic?: boolean;
    /**
     * The unit of the size (defaults to the unit of the location's spatial reference).
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
    /**
     * The additional number of subdivisions for generating the mesh representing a sphere.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    densificationFactor?: number;
    /**
     * Specifies the vertex space of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    vertexSpace?: "local" | "georeferenced";
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateSphereParameters)
     */
    material?: MeshMaterial;
  }

  export interface MeshCenterAtParameters {
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#centerAt)
     */
    geographic?: boolean;
    /**
     * The origin at which to center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#centerAt)
     */
    origin?: Point;
  }

  export interface MeshCreateFromFilesParameters {
    /**
     * The layer to which the files are to be uploaded and which will perform any necessary conversions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromFiles)
     */
    layer?: SceneLayer;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the loading process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromFiles)
     */
    signal?: AbortSignal;
  }

  export interface MeshCreateFromPolygonParameters {
    /**
     * The material to be used for the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#createFromPolygon)
     */
    material?: MeshMaterial;
  }

  export interface MeshRotateParameters {
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#rotate)
     */
    geographic?: boolean;
    /**
     * The origin around which to rotate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#rotate)
     */
    origin?: Point;
  }

  export interface MeshScaleParameters {
    /**
     * Whether to georeference relative to the globe or the projected coordinate system (PCS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#scale)
     */
    geographic?: boolean;
    /**
     * The origin point for scaling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#scale)
     */
    origin?: Point;
  }

  export interface MeshToBinaryGLTFOptions {
    /**
     * The origin of the exported model, defaults to [`Mesh.origin`](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#origin).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#toBinaryGLTF)
     */
    origin?: Point;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#toBinaryGLTF)
     */
    signal?: AbortSignal;
  }

  export interface MeshVertexAttributesProperties {
    /**
     * A flat array of vertex positions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    position?: Float64Array | number[] | Float32Array;
    /**
     * A flat array of vertex uv coordinates (2 elements per vertex).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    uv?: Float32Array | number[] | Float64Array;
    /**
     * A flat array of the vertex normals (3 elements per vertex ranging from -1 to 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    normal?: Float32Array | number[] | Float64Array;
    /**
     * **Since: 4.9**  A flat array of the vertex colors (4 elements per vertex ranging from 0 to 255).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    color?: Uint8Array | number[] | Uint8ClampedArray;
    /**
     * **Since: 4.11**  A flat array of the vertex tangents (4 elements per vertex ranging from -1 to 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    tangent?: Float32Array | number[] | Float64Array;
  }

  export interface MeshVertexAttributes extends AnonymousAccessor {
    /**
     * A flat array of vertex positions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    position: Float64Array;
    /**
     * A flat array of vertex uv coordinates (2 elements per vertex).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    uv?: Float32Array;
    /**
     * A flat array of the vertex normals (3 elements per vertex ranging from -1 to 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    normal?: Float32Array;
    /**
     * **Since: 4.9**  A flat array of the vertex colors (4 elements per vertex ranging from 0 to 255).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    color?: Uint8Array;
    /**
     * **Since: 4.11**  A flat array of the vertex tangents (4 elements per vertex ranging from -1 to 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes)
     */
    tangent?: Float32Array;
  }

  export interface CreateBoxParametersSize {
    /**
     * The width of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    width?: number;
    /**
     * The depth of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    depth?: number;
    /**
     * The height of the created mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#CreateBoxParameters)
     */
    height?: number;
  }

  export class Multipoint extends Geometry {
    /**
     * An ordered collection of points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html)
     */

    constructor(properties?: MultipointProperties);

    /**
     * An array of points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#points)
     */
    points: number[][];
    /**
     * The string value representing the type of geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#type)
     */
    readonly type: "multipoint";

    /**
     * Adds a point to the Multipoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#addPoint)
     */
    addPoint(point: Point | number[]): Multipoint;
    /**
     * Creates a deep clone of Multipoint object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#clone)
     */
    clone(): Multipoint;
    /**
     * Returns the point at the specified index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#getPoint)
     */
    getPoint(index: number): Point;
    /**
     * Removes a point from the Multipoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#removePoint)
     */
    removePoint(index: number): Point;
    /**
     * Updates the point at the specified index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#setPoint)
     */
    setPoint(index: number, point: Point | number[]): Multipoint;

    static fromJSON(json: any): Multipoint;
  }

  interface MultipointProperties extends GeometryProperties {
    /**
     * An array of points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html#points)
     */
    points?: number[][];
  }

  /**
   * Applies an affine transformation to 2D geometries via the [Transformation](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html) class.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-affineTransformOperator.html)
   */
  interface affineTransformOperator {
    /**
     * Executes an affine transformation on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-affineTransformOperator.html#execute)
     */
    execute(geometry: Geometry, transformation: Transformation): Geometry;
    /**
     * Executes an affine transformation on multiple geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-affineTransformOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], transformation: Transformation): Geometry[];
  }

  export const affineTransformOperator: affineTransformOperator;

  /**
   * Calculates the alpha shape of 2D points (concave hull).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html)
   */
  interface alphaShapeOperator {
    /**
     * Calculates the alpha shape on the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#execute)
     */
    execute(geometry: Geometry, alpha: number): ExecuteResult;
    /**
     * Calculates the alpha shape on a set of geometries with the option to aggregate the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], alpha: number, options?: alphaShapeOperatorExecuteManyOptions): Polygon[];
  }

  export const alphaShapeOperator: alphaShapeOperator;

  export interface alphaShapeOperatorExecuteManyOptions {
    /**
     * Indicates if the alpha shape polygons should be merged.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#executeMany)
     */
    merge?: boolean;
  }

  /**
   * Object returned by the [executeMany()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#executeMany) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#ExecuteResult)
   */
  export interface ExecuteResult {
    /**
     * The alpha shape polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#ExecuteResult)
     */
    alphaShape: Polygon;
    /**
     * The calculated alpha value used to generate the alpha shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-alphaShapeOperator.html#ExecuteResult)
     */
    calculatedAlpha: number;
  }

  /**
   * Returns the planar area of a 2D geometry in its spatial reference units.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-areaOperator.html)
   */
  interface areaOperator {
    /**
     * Calculates the planar area of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-areaOperator.html#execute)
     */
    execute(geometry: Geometry, options?: areaOperatorExecuteOptions): number;
  }

  export const areaOperator: areaOperator;

  export interface areaOperatorExecuteOptions {
    /**
     * The area unit of the return value.
     *
     * [Read more...](global.html#unit)
     */
    unit?: AreaUnit;
  }

  /**
   * Fills the closed gaps between 2D polygons using polygon boundaries and polylines as the boundary for creating new polygons.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-autoCompleteOperator.html)
   */
  interface autoCompleteOperator {
    /**
     * Fills the gaps between polygons using the polylines as additional boundaries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-autoCompleteOperator.html#execute)
     */
    execute(polygons: Polygon[], polylines: Polyline[]): Polygon[];
  }

  export const autoCompleteOperator: autoCompleteOperator;

  /**
   * Calculates the topological boundary of a 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-boundaryOperator.html)
   */
  interface boundaryOperator {
    /**
     * Calculates the boundary on the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-boundaryOperator.html#execute)
     */
    execute(geometry: Geometry): Geometry;
    /**
     * Calculates the boundaries on a set of geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-boundaryOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Geometry[];
  }

  export const boundaryOperator: boundaryOperator;

  /**
   * Creates planar buffers around 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html)
   */
  interface bufferOperator {
    /**
     * Creates a buffer around the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html#execute)
     */
    execute(geometry: Geometry, distance: number, options?: bufferOperatorExecuteOptions): Geometry;
    /**
     * Creates a buffer around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], distances: number[], options?: bufferOperatorExecuteManyOptions): Geometry[];
  }

  export const bufferOperator: bufferOperator;

  export interface bufferOperatorExecuteManyOptions {
    /**
     * The max deviation of the result buffer from the true buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html#executeMany)
     */
    maxDeviation?: number;
    /**
     * The maximum number of vertices in the polygon produced from a buffered point.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html#executeMany)
     */
    maxVerticesInFullCircle?: number;
    /**
     * Indicates if the buffer geometries should be unioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-bufferOperator.html#executeMany)
     */
    union?: boolean;
    /**
     * The length unit of the buffer distances and max deviation.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface bufferOperatorExecuteOptions {
    /**
     * The length unit of the buffer distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Calculates the centroid for a 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-centroidOperator.html)
   */
  interface centroidOperator {
    /**
     * Performs the centroid operation on a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-centroidOperator.html#execute)
     */
    execute(geometry: Geometry): Point;
  }

  export const centroidOperator: centroidOperator;

  /**
   * Clips geometries with a 2D extent.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-clipOperator.html)
   */
  interface clipOperator {
    /**
     * Perform the clip operation on the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-clipOperator.html#execute)
     */
    execute(geometry: Geometry, extent: Extent): Geometry;
    /**
     * Perform the clip operation on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-clipOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], extent: Extent): Geometry[];
  }

  export const clipOperator: clipOperator;

  /**
   * Performs a relational operation to determine if one 2D geometry contains another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-containsOperator.html)
   */
  interface containsOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-containsOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform the contains operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-containsOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const containsOperator: containsOperator;

  /**
   * Calculates the convex hull of 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-convexHullOperator.html)
   */
  interface convexHullOperator {
    /**
     * Calculates the convex hull geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-convexHullOperator.html#execute)
     */
    execute(geometry: Geometry): Geometry;
    /**
     * Calculates the convex hull.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-convexHullOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], options?: convexHullOperatorExecuteManyOptions): Geometry[];
    /**
     * Checks if a geometry is convex.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-convexHullOperator.html#isConvex)
     */
    isConvex(geometry: Geometry): boolean;
  }

  export const convexHullOperator: convexHullOperator;

  export interface convexHullOperatorExecuteManyOptions {
    /**
     * Indicates if the convex hull geometries should be merged.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-convexHullOperator.html#executeMany)
     */
    merge?: boolean;
  }

  /**
   * Performs a relational operation to determine if one 2D geometry crosses another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-crossesOperator.html)
   */
  interface crossesOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-crossesOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform the crosses operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-crossesOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const crossesOperator: crossesOperator;

  /**
   * Cut 2D geometries with a polyline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-cutOperator.html)
   */
  interface cutOperator {
    /**
     * Performs the cut operation on a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-cutOperator.html#execute)
     */
    execute(geometry: Geometry, polyline: Polyline): Geometry[];
  }

  export const cutOperator: cutOperator;

  /**
   * Densifies 2D geometries by length, deviation and/or angle in a 2D plane.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html)
   */
  interface densifyOperator {
    /**
     * Performs the densify operation on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#execute)
     */
    execute(geometry: Geometry, maxSegmentLength: number, options?: densifyOperatorExecuteOptions): Geometry;
    /**
     * Performs the Densify operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#executeMany)
     */
    executeMany(
      geometries: Geometry[],
      maxSegmentLength: number,
      options?: densifyOperatorExecuteManyOptions,
    ): Geometry[];
  }

  export const densifyOperator: densifyOperator;

  export interface densifyOperatorExecuteManyOptions {
    /**
     * The maximum angle allowed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#executeMany)
     */
    maxAngleInDegrees?: number;
    /**
     * The maximum deviation allowed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#executeMany)
     */
    maxDeviation?: number;
    /**
     * The length unit of `maxSegmentLength` and `maxDeviation`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface densifyOperatorExecuteOptions {
    /**
     * The maximum angle allowed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#execute)
     */
    maxAngleInDegrees?: number;
    /**
     * The maximum deviation allowed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-densifyOperator.html#execute)
     */
    maxDeviation?: number;
    /**
     * The length unit of `maxSegmentLength` and `maxDeviation`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Performs a topological difference operation on 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-differenceOperator.html)
   */
  interface differenceOperator {
    /**
     * Performs the topological difference operation on the two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-differenceOperator.html#execute)
     */
    execute(geometry: Geometry, subtractor: Geometry): Geometry;
    /**
     * Performs the Topological difference operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-differenceOperator.html#executeMany)
     */
    executeMany(geometries: Geometry, subtractor: Geometry): Geometry[];
  }

  export const differenceOperator: differenceOperator;

  /**
   * Performs a relational operation to determine if one 2D geometry is disjoint (doesn't intersect in any way) with another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-disjointOperator.html)
   */
  interface disjointOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-disjointOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform a disjoint operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-disjointOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const disjointOperator: disjointOperator;

  /**
   * Calculates distance between 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-distanceOperator.html)
   */
  interface distanceOperator {
    /**
     * Returns the planar distance between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-distanceOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry, options?: distanceOperatorExecuteOptions): number;
  }

  export const distanceOperator: distanceOperator;

  export interface distanceOperatorExecuteOptions {
    /**
     * The length unit of the distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Performs a relational operation to determine if two 2D geometries are topologically equal.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-equalsOperator.html)
   */
  interface equalsOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-equalsOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform the equals operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-equalsOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const equalsOperator: equalsOperator;

  /**
   * Extend 2D polylines with a polyline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html)
   */
  interface extendOperator {
    /**
     * Performs the extend operation on a polyline using a polyline as the extender.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    execute(polyline1: Polyline, polyline2: Polyline, options?: extendOperatorExecuteOptions): Polyline;
  }

  export const extendOperator: extendOperator;

  export interface extendOperatorExecuteOptions {
    /**
     * If an extension is performed at an end, relocate the end point to the new position instead of leaving the old point and adding a new point at the new position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    relocateEnds?: boolean;
    /**
     * If an extension is performed at an end, do not extrapolate the end-segment's attributes for the new point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    keepEndAttributes?: boolean;
    /**
     * If an extension is performed at an end, do not extrapolate the end-segment's attributes for the new point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    noEndAttributes?: boolean;
    /**
     * Do not extend the 'from' end of any part.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    noExtendAtFrom?: boolean;
    /**
     * Do not extend the 'to' end of any part.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-extendOperator.html#execute)
     */
    noExtendAtTo?: boolean;
  }

  /**
   * Generalizes 2D geometries using Douglas-Peucker algorithm.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-generalizeOperator.html)
   */
  interface generalizeOperator {
    /**
     * Performs the generalize operation on the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-generalizeOperator.html#execute)
     */
    execute(geometry: Geometry, maxDeviation: number, options?: generalizeOperatorExecuteOptions): Geometry;
    /**
     * Performs the generalize operation on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-generalizeOperator.html#executeMany)
     */
    executeMany(
      geometries: Geometry[],
      maxDeviation: number,
      options?: generalizeOperatorExecuteManyOptions,
    ): Geometry[];
  }

  export const generalizeOperator: generalizeOperator;

  export interface generalizeOperatorExecuteManyOptions {
    /**
     * Preserves degenerate parts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-generalizeOperator.html#executeMany)
     */
    removeDegenerateParts?: boolean;
    /**
     * The length unit of `maxDeviation`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface generalizeOperatorExecuteOptions {
    /**
     * Preserves degenerate parts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-generalizeOperator.html#execute)
     */
    removeDegenerateParts?: boolean;
    /**
     * The length unit of `maxDeviation`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Geodesically buffer 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html)
   */
  interface geodesicBufferOperator {
    /**
     * Creates a geodesic buffer around the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#execute)
     */
    execute(geometry: Geometry, distance: number, options?: geodesicBufferOperatorExecuteOptions): Geometry;
    /**
     * Creates a geodesic buffer around the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#executeMany)
     */
    executeMany(
      geometries: Geometry[],
      distances: number[],
      options?: geodesicBufferOperatorExecuteManyOptions,
    ): Geometry[];
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodesicBufferOperator: geodesicBufferOperator;

  export interface geodesicBufferOperatorExecuteManyOptions {
    /**
     * The type of geodetic curve used to determine the buffer.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#executeMany)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The deviation offset to use for convergence.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#executeMany)
     */
    maxDeviation?: number;
    /**
     * If true, the resulting buffer geometries will be unioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#executeMany)
     */
    union?: boolean;
    /**
     * The length unit of the buffer distances and max deviation.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface geodesicBufferOperatorExecuteOptions {
    /**
     * The type of geodetic curve used to determine the buffer.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#execute)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The deviation offset to use for convergence.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicBufferOperator.html#execute)
     */
    maxDeviation?: number;
    /**
     * The length unit of the buffer distance and max deviation.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Finds closest vertices of a 2D geometry using geodesic distance.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html)
   */
  interface geodesicProximityOperator {
    /**
     * Returns the nearest coordinate on the geometry to the given input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestCoordinate)
     */
    getNearestCoordinate(
      geometry: Geometry,
      point: Point,
      options?: geodesicProximityOperatorGetNearestCoordinateOptions,
    ): ProximityResult;
    /**
     * Returns the nearest vertex on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestVertex)
     */
    getNearestVertex(
      geometry: Geometry,
      point: Point,
      options?: geodesicProximityOperatorGetNearestVertexOptions,
    ): ProximityResult;
    /**
     * Returns vertices of the geometry that are closer to the given point than the given radius.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestVertices)
     */
    getNearestVertices(
      geometry: Geometry,
      point: Point,
      searchRadius: number,
      maxVertexCountToReturn: number,
      options?: geodesicProximityOperatorGetNearestVerticesOptions,
    ): ProximityResult[];
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodesicProximityOperator: geodesicProximityOperator;

  export interface geodesicProximityOperatorGetNearestCoordinateOptions {
    /**
     * When the parameter is set to true, this function will calculate the left/right side of a polyline or polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestCoordinate)
     */
    calculateLeftRightSide?: boolean;
    /**
     * The value controls the error of calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestCoordinate)
     */
    maxDeviation?: number;
    /**
     * When `geometry` is a polygon, the function will test if `point` is inside of the polygon.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestCoordinate)
     */
    testPolygonInterior?: boolean;
    /**
     * The length unit of the max deviation and result distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface geodesicProximityOperatorGetNearestVertexOptions {
    /**
     * The length unit of the result distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface geodesicProximityOperatorGetNearestVerticesOptions {
    /**
     * The length unit of the search radius and result distances.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Object returned from the [getNearestCoordinate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestCoordinate), [getNearestVertex()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#nearestVegetNearestVertex), and [getNearestVertices()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#getNearestVertices) methods.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
   */
  export interface ProximityResult {
    /**
     * The nearest coordinate on the geometry to the given input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
     */
    coordinate: Point;
    /**
     * The distance to the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
     */
    distance: number;
    /**
     * Indicates if the result is empty.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
     */
    isEmpty: boolean;
    /**
     * Indicates if the nearest coordinate is on the right side or left side of the input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
     */
    isRightSide?: boolean;
    /**
     * The index of the vertex on the geometry that is closest to the input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodesicProximityOperator.html#ProximityResult)
     */
    vertexIndex: number;
  }

  /**
   * Returns the geodetic area of a 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticAreaOperator.html)
   */
  interface geodeticAreaOperator {
    /**
     * Calculates the geodetic area of the input Geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticAreaOperator.html#execute)
     */
    execute(geometry: Geometry, options?: geodeticAreaOperatorExecuteOptions): number;
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticAreaOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticAreaOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodeticAreaOperator: geodeticAreaOperator;

  export interface geodeticAreaOperatorExecuteOptions {
    /**
     * The type of geodetic curve used to determine the area.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticAreaOperator.html#execute)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The area unit of the return value.
     *
     * [Read more...](global.html#unit)
     */
    unit?: AreaUnit;
  }

  /**
   * Densifies line segments by length in a 2D plane, making them run along specified geodetic curves.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html)
   */
  interface geodeticDensifyOperator {
    /**
     * Densifies the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#execute)
     */
    execute(geometry: Geometry, maxSegmentLength: number, options?: geodeticDensifyOperatorExecuteOptions): Geometry;
    /**
     * Densifies the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#executeMany)
     */
    executeMany(
      geometries: Geometry[],
      maxSegmentLength: number,
      options?: geodeticDensifyOperatorExecuteManyOptions,
    ): Geometry[];
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodeticDensifyOperator: geodeticDensifyOperator;

  export interface geodeticDensifyOperatorExecuteManyOptions {
    /**
     * The type of geodetic curve used to densify the geometries.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#executeMany)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The length unit of `maxSegmentLength`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface geodeticDensifyOperatorExecuteOptions {
    /**
     * The type of geodetic curve used to densify the geometry.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDensifyOperator.html#execute)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The length unit of `maxSegmentLength`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Calculates the shortest geodetic distance between two 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDistanceOperator.html)
   */
  interface geodeticDistanceOperator {
    /**
     * Calculates the shortest geodetic distance between two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDistanceOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry, options?: geodeticDistanceOperatorExecuteOptions): number;
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDistanceOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDistanceOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodeticDistanceOperator: geodeticDistanceOperator;

  export interface geodeticDistanceOperatorExecuteOptions {
    /**
     * The type of geodetic curve used to determine the distance.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticDistanceOperator.html#execute)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection";
    /**
     * The length unit of the distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Returns the geodetic length of a 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticLengthOperator.html)
   */
  interface geodeticLengthOperator {
    /**
     * Calculates the geodetic length of the input Geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticLengthOperator.html#execute)
     */
    execute(geometry: Geometry, options?: geodeticLengthOperatorExecuteOptions): number;
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticLengthOperator.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticLengthOperator.html#load)
     */
    load(): Promise<any>;
  }

  export const geodeticLengthOperator: geodeticLengthOperator;

  export interface geodeticLengthOperatorExecuteOptions {
    /**
     * The type of geodetic curve used to determine the length.
     *
     * @default "geodesic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-geodeticLengthOperator.html#execute)
     */
    curveType?: "geodesic" | "loxodrome" | "great-elliptic" | "normal-selection" | "shape-preserving";
    /**
     * The length unit of the return value.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Creates planar buffers around 2D geometries using graphical joins and caps.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html)
   */
  interface graphicBufferOperator {
    /**
     * Creates a buffer around the input geometries, using graphical joins and caps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html#executeMany)
     */
    executeMany(
      geometries: Geometry[],
      distances: number[],
      joins: "round" | "miter" | "bevel",
      caps: "round" | "butt" | "square",
      options?: graphicBufferOperatorExecuteManyOptions,
    ): Geometry[];
  }

  export const graphicBufferOperator: graphicBufferOperator;

  export interface graphicBufferOperatorExecuteManyOptions {
    /**
     * The max deviation of the result buffer from the true buffer in the geometries spatial reference units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html#executeMany)
     */
    maxDeviation?: number;
    /**
     * The maximum number of vertices in the polygon produced from a buffered point.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html#executeMany)
     */
    maxVerticesInFullCircle?: number;
    /**
     * Defines when miter joins are replaced with bevel joins.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html#executeMany)
     */
    miterLimit?: number;
    /**
     * Indicates if the buffer geometries should be unioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-graphicBufferOperator.html#executeMany)
     */
    union?: boolean;
    /**
     * The length unit of the buffer distances and max deviation.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Performs an Integration operation on a set of 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-integrateOperator.html)
   */
  interface integrateOperator {
    /**
     * Performs the topological integration of the geometry set in the XY plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-integrateOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Geometry[];
  }

  export const integrateOperator: integrateOperator;

  /**
   * Create new geometries using the topological intersection of 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectionOperator.html)
   */
  interface intersectionOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectionOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Performs the topological intersection operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectionOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): Geometry;
    /**
     * Performs the topological intersection operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectionOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], intersector: Geometry): Geometry[];
  }

  export const intersectionOperator: intersectionOperator;

  /**
   * Performs a relational operation to determine if one 2D geometry intersects another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectsOperator.html)
   */
  interface intersectsOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectsOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform a intersects operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-intersectsOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const intersectsOperator: intersectsOperator;

  /**
   * Performs a 2D relational operation that checks if two geometries are near each other.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-isNearOperator.html)
   */
  interface isNearOperator {
    /**
     * Returns true if the geometries are not further than the given 2D distance from each other.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-isNearOperator.html#execute)
     */
    execute(
      geometry1: Geometry,
      geometry2: Geometry,
      distance: Geometry,
      options?: isNearOperatorExecuteOptions,
    ): boolean;
  }

  export const isNearOperator: isNearOperator;

  export interface isNearOperatorExecuteOptions {
    /**
     * The length unit of the distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Calculates a label point for the given 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-labelPointOperator.html)
   */
  interface labelPointOperator {
    /**
     * Performs the label point operation on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-labelPointOperator.html#execute)
     */
    execute(geometry: Geometry): Point;
    /**
     * Performs the label point operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-labelPointOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Point[];
  }

  export const labelPointOperator: labelPointOperator;

  /**
   * Returns the planar length of a 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-lengthOperator.html)
   */
  interface lengthOperator {
    /**
     * Calculates the planar length of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-lengthOperator.html#execute)
     */
    execute(geometry: Geometry, options?: lengthOperatorExecuteOptions): number;
  }

  export const lengthOperator: lengthOperator;

  export interface lengthOperatorExecuteOptions {
    /**
     * The length unit of the return value.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Performs the topological operation of breaking the input set of 2D polygons and polylines into segments and rebuilding a new set of polygons from the non-intersecting areas.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-linesToPolygonsOperator.html)
   */
  interface linesToPolygonsOperator {
    /**
     * Performs the topological lines to polygons operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-linesToPolygonsOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Polygon[];
  }

  export const linesToPolygonsOperator: linesToPolygonsOperator;

  /**
   * Performs an OGC locate between operation on M values for the given 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-locateBetweenOperator.html)
   */
  interface locateBetweenOperator {
    /**
     * Performs the locate between or locate along operation on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-locateBetweenOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], startM: number, endM: number): Geometry[];
  }

  export const locateBetweenOperator: locateBetweenOperator;

  /**
   * Create a minimum bounding circle for the input geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-minimumBoundingCircleOperator.html)
   */
  interface minimumBoundingCircleOperator {
    /**
     * Performs the minimum bounding circle operation on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-minimumBoundingCircleOperator.html#execute)
     */
    execute(geometry: Geometry): Polygon;
    /**
     * Performs the minimum bounding circle operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-minimumBoundingCircleOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], options?: minimumBoundingCircleOperatorExecuteManyOptions): Polygon[];
  }

  export const minimumBoundingCircleOperator: minimumBoundingCircleOperator;

  export interface minimumBoundingCircleOperatorExecuteManyOptions {
    /**
     * Indicates if the minimum bounding circle geometries should be merged.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-minimumBoundingCircleOperator.html#executeMany)
     */
    merge?: boolean;
  }

  /**
   * Convert multipart 2D geometries to single part geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-multiPartToSinglePartOperator.html)
   */
  interface multiPartToSinglePartOperator {
    /**
     * Performs the multipart to single part operation on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-multiPartToSinglePartOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], options?: multiPartToSinglePartOperatorExecuteManyOptions): Geometry[];
  }

  export const multiPartToSinglePartOperator: multiPartToSinglePartOperator;

  export interface multiPartToSinglePartOperatorExecuteManyOptions {
    /**
     * When this parameter is set to false, the input polygons will not be simplified before converting them to single part geometries.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-multiPartToSinglePartOperator.html#executeMany)
     */
    simplifyPolygons?: boolean;
  }

  /**
   * Offset 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html)
   */
  interface offsetOperator {
    /**
     * Creates an offset version of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#execute)
     */
    execute(geometry: Geometry, distance: number, options?: offsetOperatorExecuteOptions): Geometry;
    /**
     * Creates offset versions of the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], distance: number, options?: offsetOperatorExecuteManyOptions): Geometry[];
  }

  export const offsetOperator: offsetOperator;

  export interface offsetOperatorExecuteManyOptions {
    /**
     * The maximum distance of the resulting segments compared to the true circular arc (used only when joins is round).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#executeMany)
     */
    flattenError?: number;
    /**
     * Defines the join type of the offset geometry.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#executeMany)
     */
    joins?: "round" | "miter" | "bevel" | "square";
    /**
     * Defines when miter joins are replaced with bevel joins.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#executeMany)
     */
    miterLimit?: number;
    /**
     * The length unit of the offset distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface offsetOperatorExecuteOptions {
    /**
     * The maximum distance of the resulting segments compared to the true circular arc (used only when `joins` is set to `"round"`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#execute)
     */
    flattenError?: number;
    /**
     * Defines the join type of the offset geometry.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#execute)
     */
    joins?: "round" | "miter" | "bevel" | "square";
    /**
     * Defines when miter joins are replaced with bevel joins.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-offsetOperator.html#execute)
     */
    miterLimit?: number;
    /**
     * The length unit of the offset distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Performs a relational operation to determine if two 2D geometries of the same dimension overlap.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-overlapsOperator.html)
   */
  interface overlapsOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-overlapsOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform the overlaps operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-overlapsOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const overlapsOperator: overlapsOperator;

  /**
   * Performs an overlay operation on a set of 2D polygons in the XY plane.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html)
   */
  interface polygonOverlayOperator {
    /**
     * Performs the topological overlay of the geometry set in the XY plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[], options?: polygonOverlayOperatorExecuteManyOptions): Geometry[];
  }

  export const polygonOverlayOperator: polygonOverlayOperator;

  export interface polygonOverlayOperatorExecuteManyOptions {
    /**
     * Create polygons to fill gaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    gaps?: boolean;
    /**
     * Create polygons to fill holes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    holes?: boolean;
    /**
     * Output non-overlapping areas of polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    noOverlaps?: boolean;
    /**
     * Output overlapping areas of polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    overlaps?: boolean;
    /**
     * Output multipart polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonOverlayOperator.html#executeMany)
     */
    multiPart?: boolean;
  }

  /**
   * Performs a topological operation for slicing a 2D polygon into smaller polygons.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html)
   */
  interface polygonSlicerOperator {
    /**
     * Finds positions of the sliced strips for a given polygon given the number of equal area parts to slice the polygon into.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#findSlicesByArea)
     */
    findSlicesByArea(
      polygon: Polygon,
      partCount: number,
      remainingArea: number,
      options?: polygonSlicerOperatorFindSlicesByAreaOptions,
    ): number[];
    /**
     * Slices the given polygon into equal area parts, not necessarily strips.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#recursiveSliceEqualArea)
     */
    recursiveSliceEqualArea(
      polygon: Polygon,
      partCount: number,
      options?: polygonSlicerOperatorRecursiveSliceEqualAreaOptions,
    ): Polygon[];
    /**
     * Slices the given polygon into strips.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#sliceIntoStrips)
     */
    sliceIntoStrips(
      polygon: Polygon,
      ySlices: number[],
      options?: polygonSlicerOperatorSliceIntoStripsOptions,
    ): Polygon[];
  }

  export const polygonSlicerOperator: polygonSlicerOperator;

  export interface polygonSlicerOperatorFindSlicesByAreaOptions {
    /**
     * The affine transformation to apply to the polygon before slicing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#findSlicesByArea)
     */
    transform?: Transformation;
    /**
     * The area unit of the remaining area.
     *
     * [Read more...](global.html#unit)
     */
    unit?: AreaUnit;
  }

  export interface polygonSlicerOperatorRecursiveSliceEqualAreaOptions {
    /**
     * The affine transformation to apply to the polygon before slicing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#recursiveSliceEqualArea)
     */
    transform?: Transformation;
  }

  export interface polygonSlicerOperatorSliceIntoStripsOptions {
    /**
     * The affine transformation to apply to the polygon before slicing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-polygonSlicerOperator.html#sliceIntoStrips)
     */
    transform?: Transformation;
  }

  /**
   * Find the closest vertices of the 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html)
   */
  interface proximityOperator {
    /**
     * Returns the nearest coordinate on the geometry to the given input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestCoordinate)
     */
    getNearestCoordinate(
      geometry: Geometry,
      point: Point,
      options?: proximityOperatorGetNearestCoordinateOptions,
    ): proximityOperatorProximityResult;
    /**
     * Returns the nearest vertex on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestVertex)
     */
    getNearestVertex(
      geometry: Geometry,
      point: Point,
      options?: proximityOperatorGetNearestVertexOptions,
    ): proximityOperatorProximityResult;
    /**
     * Returns vertices of the geometry that are closer to the given point than the given radius.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestVertices)
     */
    getNearestVertices(
      geometry: Geometry,
      point: Point,
      searchRadius: number,
      maxVertexCountToReturn: number,
      options?: proximityOperatorGetNearestVerticesOptions,
    ): proximityOperatorProximityResult[];
  }

  export const proximityOperator: proximityOperator;

  export interface proximityOperatorGetNearestCoordinateOptions {
    /**
     * When the parameter is set to true, this function will calculate the left/right side of a polyline or polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestCoordinate)
     */
    calculateLeftRightSide?: boolean;
    /**
     * When `geometry` is a polygon, the function will test if `point` is inside of the polygon.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestCoordinate)
     */
    testPolygonInterior?: boolean;
    /**
     * The length unit of the result distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface proximityOperatorGetNearestVertexOptions {
    /**
     * The length unit of the result distance.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface proximityOperatorGetNearestVerticesOptions {
    /**
     * The length unit of the search radius and result distances.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Object returned from the [getNearestCoordinate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestCoordinate), [getNearestVertex()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#nearestVegetNearestVertex), and [getNearestVertices()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#getNearestVertices) methods.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
   */
  export interface proximityOperatorProximityResult {
    /**
     * The nearest coordinate on the geometry to the given input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
     */
    coordinate: Point;
    /**
     * The planar distance between the input point and the nearest coordinate on the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
     */
    distance: number;
    /**
     * Indicates if the result is empty.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
     */
    isEmpty: boolean;
    /**
     * Indicates if the nearest coordinate is on the right side or left side of the input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
     */
    isRightSide?: boolean;
    /**
     * The index of the vertex on the geometry that is closest to the input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-proximityOperator.html#ProximityResult)
     */
    vertexIndex: number;
  }

  /**
   * Performs a relational operation between two 2D geometries using the DE-9IM matrix encoded as a string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-relateOperator.html)
   */
  interface relateOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-relateOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;

    execute(geometry1: Geometry, geometry2: Geometry, relation: string): boolean;

    isValidDE9IM(relation: void): void;
  }

  export const relateOperator: relateOperator;

  /**
   * Reshape 2D polygons or polylines with a single path polyline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-reshapeOperator.html)
   */
  interface reshapeOperator {
    /**
     * Performs the reshape operation on a polygon or polyline using a single path polyline as the reshaper.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-reshapeOperator.html#execute)
     */
    execute(geometry: Polygon | Polyline, reshaper: Polyline): Polygon | Polyline;
  }

  export const reshapeOperator: reshapeOperator;

  /**
   * Simplifies 2D geometries by removing unnecessary vertices while preserving the geometry shape.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-simplifyOperator.html)
   */
  interface simplifyOperator {
    /**
     * Performs the simplify operation on a single geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-simplifyOperator.html#execute)
     */
    execute(geometry: Geometry): Geometry;
    /**
     * Performs the simplify operation on the geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-simplifyOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Geometry[];
    /**
     * Indicates if the given geometry is topologically simple.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-simplifyOperator.html#isSimple)
     */
    isSimple(geometry: Geometry): boolean;
  }

  export const simplifyOperator: simplifyOperator;

  export class Transformation {
    /**
     * Calculates distance errors for a transformation on a given set of the input points to the output points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#calculateErrors)
     */
    calculateErrors(input: Point[], output: Point[]): ErrorResult;
    /**
     * Flips all the `x` coordinates of geometries on the vertical y-axis (right or left) that is centered between `x0` and `x1`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#flipX)
     */
    flipX(x0: number, x1: number): void;
    /**
     * Flips all the `y` coordinates of geometries on the horizontal x-axis (up or down) that is centered between `y0` and `y1`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#flipY)
     */
    flipY(y0: number, y1: number): void;
    /**
     * Initializes a new transformation from the input and output control points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#initializeFromControlPoints)
     */
    initializeFromControlPoints(type: "conformal" | "general", input: Point[], output: Point[], inverseOut?: any): void;
    /**
     * Determines if the transformation is in its default state, which is an identity matrix.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#isIdentity)
     */
    isIdentity(): boolean;
    /**
     * Rotates the geometries by the specified angle in degrees around the point specified by the x and y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#rotate)
     */
    rotate(angleInDegrees: number, rotationX: number, rotationY: number): void;
    /**
     * Resizes the geometries by the specified scale factors defined by the x and y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#scale)
     */
    scale(x: number, y: number): void;
    /**
     * Use this method to reset the transformation to its default state, which is an identity matrix.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#setIdentity)
     */
    setIdentity(): void;
    /**
     * Use this method to swap the x and y coordinate values in the transformation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#setSwapCoordinates)
     */
    setSwapCoordinates(): void;
    /**
     * Shifts a geometry's points by the specified proportion in parallel to the x and y directions, except those points that are along the shear axis, which is a line defined by the x and y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#shear)
     */
    shear(proportionX: number, proportionY: number): void;
    /**
     * Shifts all the coordinates of the geometries by the distance and direction between the specified `x` and `y` coordinates and their current location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#shift)
     */
    shift(x: number, y: number): void;

    constructor();
  }

  /**
   * Object returned from the [calculateErrors()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#calculateErrors) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#ErrorResult)
   */
  export interface ErrorResult {
    /**
     * The root mean square (`rms`) of the 'average' distance error between the specified `output` points and the transformed `input` points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#ErrorResult)
     */
    rms: number;
    /**
     * Contains the errors of the individual distance differences between specific pairs of points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-support-Transformation.html#ErrorResult)
     */
    errorsOut: number[];
  }

  /**
   * Returns the symmetric difference between 2D geometries, also known as exclusive OR, or XOR.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-symmetricDifferenceOperator.html)
   */
  interface symmetricDifferenceOperator {
    /**
     * Performs the symmetric difference (XOR) operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-symmetricDifferenceOperator.html#execute)
     */
    execute(leftGeometry: Geometry, rightGeometry: Geometry): Geometry;
    /**
     * Performs the symmetric difference (XOR) operation on every geometry in `inputGeometries` with `rightGeometry`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-symmetricDifferenceOperator.html#executeMany)
     */
    executeMany(inputGeometries: Geometry[], rightGeometry: Geometry): Geometry[];
  }

  export const symmetricDifferenceOperator: symmetricDifferenceOperator;

  /**
   * Perform a relational operation to determine if one 2D geometry touches another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-touchesOperator.html)
   */
  interface touchesOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-touchesOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform a touches operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-touchesOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): boolean;
  }

  export const touchesOperator: touchesOperator;

  /**
   * Perform a topological union (dissolve) operation on 2D geometries.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-unionOperator.html)
   */
  interface unionOperator {
    /**
     * Perform a topological union operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-unionOperator.html#execute)
     */
    execute(geometry1: Geometry, geometry2: Geometry): Geometry;
    /**
     * Perform a topological union operation on a geometry set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-unionOperator.html#executeMany)
     */
    executeMany(geometries: Geometry[]): Geometry;
  }

  export const unionOperator: unionOperator;

  /**
   * Perform a relational operation to determine if one 2D geometry is within another 2D geometry.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-withinOperator.html)
   */
  interface withinOperator {
    /**
     * Accelerate a geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-withinOperator.html#accelerateGeometry)
     */
    accelerateGeometry(geometry: Geometry): boolean;
    /**
     * Perform a within operation on two geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-operators-withinOperator.html#execute)
     */
    execute(inner: Geometry, outer: Geometry): boolean;
  }

  export const withinOperator: withinOperator;

  export class Point extends Geometry {
    /**
     * A location defined by X, Y, and Z coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html)
     */

    constructor(properties?: PointProperties);

    /**
     * The latitude of the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#latitude)
     */
    latitude: number;
    /**
     * The longitude of the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#longitude)
     */
    longitude: number;
    /**
     * The m-coordinate of the point in map units.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#m)
     */
    m: number;
    /**
     * The string value representing the type of geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#type)
     */
    readonly type: "point";
    /**
     * The x-coordinate (easting) of the point in map units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#x)
     */
    x: number;
    /**
     * The y-coordinate (northing) of the point in map units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#y)
     */
    y: number;
    /**
     * The z-coordinate (or elevation) of the point in map units.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#z)
     */
    z: number;

    /**
     * Creates a deep clone of Point object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#clone)
     */
    clone(): Point;
    /**
     * Copies all values from another Point instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#copy)
     */
    copy(other: Point): this;
    /**
     * Computes the Euclidean distance between this Point and a given Point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#distance)
     */
    distance(other: Point): number;
    /**
     * Determines if the input point is equal to the point calling the function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#equals)
     */
    equals(point: Point): boolean;
    /**
     * Modifies the point geometry in-place by shifting the X-coordinate to within +/- 180 span in map units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#normalize)
     */
    normalize(): Point;

    static fromJSON(json: any): Point;
  }

  interface PointProperties extends GeometryProperties {
    /**
     * The latitude of the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#latitude)
     */
    latitude?: number;
    /**
     * The longitude of the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#longitude)
     */
    longitude?: number;
    /**
     * The m-coordinate of the point in map units.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#m)
     */
    m?: number;
    /**
     * The x-coordinate (easting) of the point in map units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#x)
     */
    x?: number;
    /**
     * The y-coordinate (northing) of the point in map units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#y)
     */
    y?: number;
    /**
     * The z-coordinate (or elevation) of the point in map units.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html#z)
     */
    z?: number;
  }

  export class Polygon extends Geometry {
    /**
     * A polygon contains an array of [rings](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#rings) and a [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#spatialReference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html)
     */

    constructor(properties?: PolygonProperties);

    /**
     * The centroid of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#centroid)
     */
    centroid: Point;
    /**
     * Checks to see if polygon rings cross each other and indicates if the polygon is self-intersecting, which means the ring of the polygon crosses itself.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#isSelfIntersecting)
     */
    isSelfIntersecting: boolean;
    /**
     * An array of rings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#rings)
     */
    rings: number[][][];
    /**
     * The string value representing the type of geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#type)
     */
    readonly type: "polygon";

    /**
     * Adds a ring to the Polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#addRing)
     */
    addRing(points: Point[] | number[][]): Polygon;
    /**
     * Creates a deep clone of Polygon object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#clone)
     */
    clone(): Polygon;
    /**
     * Checks on the client if the input point is inside the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#contains)
     */
    contains(point: Point): boolean;
    /**
     * Returns a point specified by a ring and point in the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#getPoint)
     */
    getPoint(ringIndex: number, pointIndex: number): Point;
    /**
     * Inserts a new point into the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#insertPoint)
     */
    insertPoint(ringIndex: number, pointIndex: number, point: Point | number[]): Polygon;
    /**
     * Checks if a Polygon ring is clockwise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#isClockwise)
     */
    isClockwise(ring: Point[] | number[][]): boolean;
    /**
     * Removes a point from the polygon at the given `pointIndex` within the ring identified by `ringIndex`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#removePoint)
     */
    removePoint(ringIndex: number, pointIndex: number): Point[];
    /**
     * Removes a ring from the Polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#removeRing)
     */
    removeRing(index: number): Point[];
    /**
     * Updates a point in the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#setPoint)
     */
    setPoint(ringIndex: number, pointIndex: number, point: Point | number[]): Polygon;

    /**
     * Converts the given Extent to a Polygon instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#fromExtent)
     */
    static fromExtent(extent: Extent): Polygon;
    static fromJSON(json: any): Polygon;
  }

  interface PolygonProperties extends GeometryProperties {
    /**
     * The centroid of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#centroid)
     */
    centroid?: PointProperties;
    /**
     * Checks to see if polygon rings cross each other and indicates if the polygon is self-intersecting, which means the ring of the polygon crosses itself.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#isSelfIntersecting)
     */
    isSelfIntersecting?: boolean;
    /**
     * An array of rings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html#rings)
     */
    rings?: number[][][];
  }

  export class Polyline extends Geometry {
    /**
     * A polyline contains an array of [paths](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#paths) and [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#spatialReference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html)
     */

    constructor(properties?: PolylineProperties);

    /**
     * An array of paths, or line segments, that make up the polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#paths)
     */
    paths: number[][][];

    readonly type: "polyline";

    /**
     * Adds a path, or line segment, to the polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#addPath)
     */
    addPath(points: Point[] | number[][]): Polyline;
    /**
     * Creates a deep clone of Polyline object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#clone)
     */
    clone(): Polyline;
    /**
     * Returns a point specified by a path and point in the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#getPoint)
     */
    getPoint(pathIndex: number, pointIndex: number): Point;
    /**
     * Inserts a new point into a polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#insertPoint)
     */
    insertPoint(pathIndex: number, pointIndex: number, point: Point | number[]): Polyline;
    /**
     * Removes a path from the Polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#removePath)
     */
    removePath(index: number): Point[];
    /**
     * Removes a point from the polyline at the given `pointIndex` within the path identified by the given `pathIndex`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#removePoint)
     */
    removePoint(pathIndex: number, pointIndex: number): Point;
    /**
     * Updates a point in a polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#setPoint)
     */
    setPoint(pathIndex: number, pointIndex: number, point: Point | number[]): Polyline;

    static fromJSON(json: any): Polyline;
  }

  interface PolylineProperties extends GeometryProperties {
    /**
     * An array of paths, or line segments, that make up the polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html#paths)
     */
    paths?: number[][][];
  }

  /**
   * A client-side projection engine for converting geometries from one [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) to another.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html)
   */
  interface projection {
    /**
     * Returns the default geographic transformation used to convert the geometry from the input spatial reference to the output spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html#getTransformation)
     */
    getTransformation(
      inSpatialReference: SpatialReference | SpatialReferenceProperties,
      outSpatialReference: SpatialReference | SpatialReferenceProperties,
      extent?: Extent,
    ): GeographicTransformation;
    /**
     * Returns a list of all geographic transformations suitable to convert geometries from the input spatial reference to the specified output spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html#getTransformations)
     */
    getTransformations(
      inSpatialReference: SpatialReference | SpatialReferenceProperties,
      outSpatialReference: SpatialReference | SpatialReferenceProperties,
      extent?: Extent,
    ): GeographicTransformation[];
    /**
     * Indicates if all dependencies of this module have been loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html#isLoaded)
     */
    isLoaded(): boolean;
    /**
     * Loads this module's dependencies.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html#load)
     */
    load(): Promise<any>;
    /**
     * Projects a geometry or an array of geometries to the specified output spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-projection.html#project)
     */
    project(
      geometry: Geometry | Geometry[],
      outSpatialReference: SpatialReference | SpatialReferenceProperties,
      geographicTransformation?: GeographicTransformation,
    ): Geometry | Geometry[];
  }

  export const projection: projection;

  export interface SpatialReference extends Accessor, JSONSupport {}

  export class SpatialReference {
    /**
     * Defines the spatial reference of a view, layer, or method parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html)
     */

    constructor(properties?: SpatialReferenceProperties);

    /**
     * An [image coordinate system](https://developers.arcgis.com/rest/services-reference/raster-ics.htm) defines the spatial reference used to display the image in its original coordinates without distortion, map transformations or ortho-rectification.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#imageCoordinateSystem)
     */
    imageCoordinateSystem: any;
    /**
     * Indicates if the spatial reference refers to a geographic coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#isGeographic)
     */
    readonly isGeographic: boolean;
    /**
     * Indicates if the [wkid](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkid) of the spatial reference object is one of the following values: `102113`, `102100`, `3857`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#isWebMercator)
     */
    readonly isWebMercator: boolean;
    /**
     * Indicates if the [wkid](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkid) of the spatial reference object is `4326`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#isWGS84)
     */
    readonly isWGS84: boolean;
    /**
     * Indicates if the spatial reference of the map supports wrapping around the International Date Line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#isWrappable)
     */
    readonly isWrappable: boolean;
    /**
     * The factor to convert one unit value in the spatial reference's [unit](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#unit) to meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#metersPerUnit)
     */
    readonly metersPerUnit: number;
    /**
     * The unit of the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#unit)
     */
    readonly unit:
      | "degrees"
      | "meters"
      | "feet"
      | "us-feet"
      | "clarke-feet"
      | "clarke-yards"
      | "clarke-links"
      | "sears-yards"
      | "sears-feet"
      | "sears-chains"
      | "benoit-1895-b-chains"
      | "indian-yards"
      | "indian-1937-yards"
      | "gold-coast-feet"
      | "chains"
      | "sears-1922-truncated-chains"
      | "50-kilometers"
      | "150-kilometers";
    /**
     * The well-known ID of a spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkid)
     */
    wkid: number;
    /**
     * The well-known text that defines a spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkt)
     */
    wkt: string;
    /**
     * The well-known text of the coordinate system as defined by OGC standard for well-known text strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkt2)
     */
    wkt2: string;

    /**
     * Returns a deep clone of the spatial reference object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#clone)
     */
    clone(): SpatialReference;
    /**
     * Checks if the specified spatial reference object has the same [wkid](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkid) or [wkt](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkt) as this spatial reference object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#equals)
     */
    equals(spatialReference: SpatialReference): boolean;

    /**
     * A convenience spatial reference instance for Web Mercator.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#WebMercator)
     */
    static WebMercator: SpatialReference;
    /**
     * A convenience spatial reference instance for WGS84.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#WGS84)
     */
    static WGS84: SpatialReference;

    static fromJSON(json: any): SpatialReference;
  }

  interface SpatialReferenceProperties {
    /**
     * An [image coordinate system](https://developers.arcgis.com/rest/services-reference/raster-ics.htm) defines the spatial reference used to display the image in its original coordinates without distortion, map transformations or ortho-rectification.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#imageCoordinateSystem)
     */
    imageCoordinateSystem?: any;
    /**
     * The well-known ID of a spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkid)
     */
    wkid?: number;
    /**
     * The well-known text that defines a spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkt)
     */
    wkt?: string;
    /**
     * The well-known text of the coordinate system as defined by OGC standard for well-known text strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html#wkt2)
     */
    wkt2?: string;
  }

  /**
   * This class performs geodetic computations for Earth and 70+ non-Earth spheroids.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html)
   */
  interface geodesicUtils {
    /**
     * Geodetically computes the area for one or more polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#geodesicAreas)
     */
    geodesicAreas(
      polygons: Polygon[],
      unit?:
        | "square-millimeters"
        | "square-centimeters"
        | "square-decimeters"
        | "square-meters"
        | "square-kilometers"
        | "square-inches"
        | "square-feet"
        | "square-yards"
        | "square-miles"
        | "square-us-feet"
        | "acres"
        | "ares"
        | "hectares",
    ): number[];
    /**
     * Computes and returns a densified polyline or polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#geodesicDensify)
     */
    geodesicDensify(geometry: Polyline | Polygon, maxSegmentLength: number): Polyline | Polygon;
    /**
     * Geodetically computes the direction and distance between two known locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#geodesicDistance)
     */
    geodesicDistance(from: Point, to: Point, unit?: LengthUnit): GeodesicDistanceResult;
    /**
     * Geodetically computes polygon perimeter or polyline length for one or more geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#geodesicLengths)
     */
    geodesicLengths(geometries: Polyline[] | Polygon[], unit?: LengthUnit): number[];
    /**
     * Geodetically computes the location at a defined distance and direction from a known location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#pointFromDistance)
     */
    pointFromDistance(point: Point, distance: number, azimuth: number): Point;
  }

  export const geodesicUtils: geodesicUtils;

  /**
   * Computed distance and direction between two known locations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#GeodesicDistanceResult)
   */
  export interface GeodesicDistanceResult {
    /**
     * The distance between the two locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#GeodesicDistanceResult)
     */
    distance?: number;
    /**
     * The azimuth (or "bearing") in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#GeodesicDistanceResult)
     */
    azimuth?: number;
    /**
     * The azimuth in degrees in the reverse direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-geodesicUtils.html#GeodesicDistanceResult)
     */
    reverseAzimuth?: number;
  }

  export class GeographicTransformation {
    /**
     * Geographic transformation steps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformation.html#steps)
     */
    steps: GeographicTransformationStep[];

    /**
     * Returns the inverse of the geographic transformation calling this method or `null` if the transformation is not invertible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformation.html#getInverse)
     */
    getInverse(): GeographicTransformation;

    constructor(properties?: any);
  }

  export class GeographicTransformationStep {
    /**
     * Indicates with the geographic transformation is inverted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformationStep.html#isInverse)
     */
    isInverse: boolean;
    /**
     * The well-known id (wkid) hat represents a known geographic transformation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformationStep.html#wkid)
     */
    wkid: number;
    /**
     * The well-known text (wkt) that represents a known geographic transformation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformationStep.html#wkt)
     */
    wkt: string;

    /**
     * Gets the inverse of the geographic transformation step used to call this method or `null` if the transformation step is not invertible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-GeographicTransformationStep.html#getInverse)
     */
    getInverse(): GeographicTransformationStep;

    constructor(properties?: any);
  }

  /**
   * Provides utility methods for working with ArcGIS JSON geometry objects.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-jsonUtils.html)
   */
  interface jsonUtils {
    /**
     * Creates a new instance of an appropriate [Geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html) class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-jsonUtils.html#fromJSON)
     */
    fromJSON(json: any): Geometry;
    /**
     * Returns the type for a given geometry in the JSON format used by ArcGIS.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-jsonUtils.html#getJsonType)
     */
    getJsonType(geometry: Geometry): string;
  }

  export const jsonUtils: jsonUtils;

  export class MeshComponent extends Accessor {
    /**
     * The MeshComponent class is used to apply one or more materials to a single [Mesh](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html)
     */

    constructor(properties?: MeshComponentProperties);

    /**
     * A flat array of indices that refer to vertices in the [vertexAttributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes) of the mesh to which the component belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#faces)
     */
    faces: Uint32Array;
    /**
     * The material determines how the component is visualized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#material)
     */
    material: MeshMaterial | MeshMaterialMetallicRoughness;
    /**
     * Specifies a name of the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#name)
     */
    name: string;
    /**
     * Specifies the type of normals used for lighting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#shading)
     */
    shading: "source" | "flat" | "smooth";

    /**
     * Creates a deep clone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#clone)
     */
    clone(): MeshComponent;
  }

  interface MeshComponentProperties {
    /**
     * A flat array of indices that refer to vertices in the [vertexAttributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html#vertexAttributes) of the mesh to which the component belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#faces)
     */
    faces?: Uint32Array | number[] | Uint16Array;
    /**
     * The material determines how the component is visualized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#material)
     */
    material?: MeshMaterialProperties | MeshMaterialMetallicRoughnessProperties;
    /**
     * Specifies a name of the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#name)
     */
    name?: string;
    /**
     * Specifies the type of normals used for lighting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html#shading)
     */
    shading?: "source" | "flat" | "smooth";
  }

  export interface MeshGeoreferencedVertexSpace extends Accessor, JSONSupport, Clonable {}

  export class MeshGeoreferencedVertexSpace {
    /**
     * A mesh vertex space indicating that mesh vertices are either absolute georeferenced map coordinates or relative offsets in map space to a fully georeferenced origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshGeoreferencedVertexSpace.html)
     */

    constructor(properties?: MeshGeoreferencedVertexSpaceProperties);

    /**
     * Origin of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshGeoreferencedVertexSpace.html#origin)
     */
    origin: number[];
    /**
     * Type of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshGeoreferencedVertexSpace.html#type)
     */
    readonly type: "georeferenced";

    static fromJSON(json: any): MeshGeoreferencedVertexSpace;
  }

  interface MeshGeoreferencedVertexSpaceProperties {
    /**
     * Origin of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshGeoreferencedVertexSpace.html#origin)
     */
    origin?: number[];
  }

  export interface MeshLocalVertexSpace extends Accessor, JSONSupport, Clonable {}

  export class MeshLocalVertexSpace {
    /**
     * A mesh vertex space that indicates mesh vertices to be in a plain cartesian space as often encountered in 3D modeling and CAD applications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshLocalVertexSpace.html)
     */

    constructor(properties?: MeshLocalVertexSpaceProperties);

    /**
     * Origin of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshLocalVertexSpace.html#origin)
     */
    origin: number[];
    /**
     * Type of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshLocalVertexSpace.html#type)
     */
    readonly type: "local";

    static fromJSON(json: any): MeshLocalVertexSpace;
  }

  interface MeshLocalVertexSpaceProperties {
    /**
     * Origin of the vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshLocalVertexSpace.html#origin)
     */
    origin?: number[];
  }

  export class MeshMaterial extends Accessor {
    /**
     * The material determines how a [MeshComponent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html) is visualized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html)
     */

    constructor(properties?: MeshMaterialProperties);

    /**
     * Specifies how transparency on the object is handled.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#alphaCutoff)
     */
    alphaCutoff: number;
    /**
     * Specifies how transparency on the object is handled.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#alphaMode)
     */
    alphaMode: "auto" | "blend" | "opaque" | "mask";
    /**
     * Specifies a single, uniform color for the mesh component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#color)
     */
    color: Color;
    /**
     * Specifies a texture from which to get color information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#colorTexture)
     */
    colorTexture: MeshTexture;
    /**
     * A transformation of UV mesh coordinates used to sample the color texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#colorTextureTransform)
     */
    colorTextureTransform: MeshTextureTransform;
    /**
     * Specifies whether both sides of each triangle are displayed, or only the front sides.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#doubleSided)
     */
    doubleSided: boolean;
    /**
     * Specifies a texture from which to get normal information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#normalTexture)
     */
    normalTexture: MeshTexture;
    /**
     * A transformation of UV mesh coordinates used to sample the normal texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#normalTextureTransform)
     */
    normalTextureTransform: MeshTextureTransform;
  }

  interface MeshMaterialProperties {
    /**
     * Specifies how transparency on the object is handled.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#alphaCutoff)
     */
    alphaCutoff?: number;
    /**
     * Specifies how transparency on the object is handled.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#alphaMode)
     */
    alphaMode?: "auto" | "blend" | "opaque" | "mask";
    /**
     * Specifies a single, uniform color for the mesh component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#color)
     */
    color?: Color | number[] | string;
    /**
     * Specifies a texture from which to get color information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#colorTexture)
     */
    colorTexture?: MeshTextureProperties | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | ImageData | string;
    /**
     * A transformation of UV mesh coordinates used to sample the color texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#colorTextureTransform)
     */
    colorTextureTransform?: MeshTextureTransformProperties;
    /**
     * Specifies whether both sides of each triangle are displayed, or only the front sides.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#doubleSided)
     */
    doubleSided?: boolean;
    /**
     * Specifies a texture from which to get normal information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#normalTexture)
     */
    normalTexture?:
      | MeshTextureProperties
      | HTMLImageElement
      | HTMLVideoElement
      | HTMLCanvasElement
      | ImageData
      | string;
    /**
     * A transformation of UV mesh coordinates used to sample the normal texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html#normalTextureTransform)
     */
    normalTextureTransform?: MeshTextureTransformProperties;
  }

  export class MeshMaterialMetallicRoughness extends MeshMaterial {
    /**
     * A material determines how a [MeshComponent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html) is visualized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html)
     */

    constructor(properties?: MeshMaterialMetallicRoughnessProperties);

    /**
     * Specifies a single, uniform emissive color for the [MeshComponent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveColor)
     */
    emissiveColor: Color;
    /**
     * Specifies a texture from which to get emissive color information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveTexture)
     */
    emissiveTexture: MeshTexture;
    /**
     * A transformation of UV mesh coordinates used to sample the emissive texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveTextureTransform)
     */
    emissiveTextureTransform: MeshTextureTransform;
    /**
     * Specifies how much the material behaves like a metal.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#metallic)
     */
    metallic: number;
    /**
     * Specifies a texture from which to get the combined metallic/roughness information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#metallicRoughnessTexture)
     */
    metallicRoughnessTexture: MeshTexture;
    /**
     * Allows to specify a texture to get the occlusion information from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#occlusionTexture)
     */
    occlusionTexture: MeshTexture;
    /**
     * A transformation of UV mesh coordinates used to sample the occlusion texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#occlusionTextureTransform)
     */
    occlusionTextureTransform: MeshTextureTransform;
    /**
     * Indicates how rough the surface of the material is.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#roughness)
     */
    roughness: number;
  }

  interface MeshMaterialMetallicRoughnessProperties extends MeshMaterialProperties {
    /**
     * Specifies a single, uniform emissive color for the [MeshComponent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshComponent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveColor)
     */
    emissiveColor?: Color | number[] | string;
    /**
     * Specifies a texture from which to get emissive color information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveTexture)
     */
    emissiveTexture?:
      | MeshTextureProperties
      | HTMLImageElement
      | HTMLVideoElement
      | HTMLCanvasElement
      | ImageData
      | string;
    /**
     * A transformation of UV mesh coordinates used to sample the emissive texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#emissiveTextureTransform)
     */
    emissiveTextureTransform?: MeshTextureTransformProperties;
    /**
     * Specifies how much the material behaves like a metal.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#metallic)
     */
    metallic?: number;
    /**
     * Specifies a texture from which to get the combined metallic/roughness information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#metallicRoughnessTexture)
     */
    metallicRoughnessTexture?:
      | MeshTextureProperties
      | HTMLImageElement
      | HTMLCanvasElement
      | HTMLVideoElement
      | ImageData
      | string;
    /**
     * Allows to specify a texture to get the occlusion information from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#occlusionTexture)
     */
    occlusionTexture?:
      | MeshTextureProperties
      | HTMLImageElement
      | HTMLVideoElement
      | HTMLCanvasElement
      | ImageData
      | string;
    /**
     * A transformation of UV mesh coordinates used to sample the occlusion texture.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#occlusionTextureTransform)
     */
    occlusionTextureTransform?: MeshTextureTransformProperties;
    /**
     * Indicates how rough the surface of the material is.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html#roughness)
     */
    roughness?: number;
  }

  export class MeshTexture extends Accessor {
    /**
     * MeshTexture represents image data to be used for [MeshMaterial](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterial.html) or [MeshMaterialMetallicRoughness](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshMaterialMetallicRoughness.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html)
     */

    constructor(properties?: MeshTextureProperties);

    /**
     * A direct reference to the image or video data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#data)
     */
    data: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | ImageData | CompressedTextureData;
    /**
     * Indicates whether the image data should be interpreted as being semi-transparent.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#transparent)
     */
    transparent: boolean;
    /**
     * The url to the image resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#url)
     */
    url: string;
    /**
     * Specifies how uv coordinates outside the [0, 1] range are handled.
     *
     * @default "repeat"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#wrap)
     */
    wrap: "clamp" | "repeat" | "mirror" | SeparableWrapModes;

    /**
     * Creates a deep clone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#clone)
     */
    clone(): MeshTexture;
  }

  interface MeshTextureProperties {
    /**
     * A direct reference to the image or video data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#data)
     */
    data?:
      | HTMLImageElement
      | HTMLCanvasElement
      | HTMLVideoElement
      | ImageData
      | (CompressedTextureData & { type: "encoded-mesh-texture" });
    /**
     * Indicates whether the image data should be interpreted as being semi-transparent.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#transparent)
     */
    transparent?: boolean;
    /**
     * The url to the image resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#url)
     */
    url?: string;
    /**
     * Specifies how uv coordinates outside the [0, 1] range are handled.
     *
     * @default "repeat"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#wrap)
     */
    wrap?: "clamp" | "repeat" | "mirror" | SeparableWrapModes;
  }

  /**
   * The definition of compressed texture data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#CompressedTextureData)
   */
  export interface CompressedTextureData {
    /**
     * The type.
     *
     * [Read more...](global.html#type)
     */
    type: "encoded-mesh-texture";
    /**
     * The texture encoding.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#CompressedTextureData)
     */
    encoding: "image/ktx2";
    /**
     * The compressed texture data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#CompressedTextureData)
     */
    data: Uint8Array;
  }

  /**
   * A separable wrap configuration for horizontal and vertical wrapping modes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#SeparableWrapModes)
   */
  export interface SeparableWrapModes {
    /**
     * Horizontal wrapping mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#SeparableWrapModes)
     */
    horizontal: "clamp" | "repeat" | "mirror";
    /**
     * Vertical wrapping mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html#SeparableWrapModes)
     */
    vertical: "clamp" | "repeat" | "mirror";
  }

  export interface MeshTextureTransform extends Accessor, Clonable {}

  export class MeshTextureTransform {
    /**
     * MeshTextureTransform represents a transformation of UV mesh coordinates used to sample a [MeshTexture](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTexture.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html)
     */

    constructor(properties?: MeshTextureTransformProperties);

    /**
     * The offset of the UV coordinate origin as a factor of the texture dimensions.
     *
     * @default [0, 0]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#offset)
     */
    offset: number[];
    /**
     * The rotation of the UV coordinates in degrees, counter-clockwise around the origin.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#rotation)
     */
    rotation: number;
    /**
     * The scale factor applied to the components of the UV coordinates.
     *
     * @default [1, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#scale)
     */
    scale: number[];
  }

  interface MeshTextureTransformProperties {
    /**
     * The offset of the UV coordinate origin as a factor of the texture dimensions.
     *
     * @default [0, 0]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#offset)
     */
    offset?: number[];
    /**
     * The rotation of the UV coordinates in degrees, counter-clockwise around the origin.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#rotation)
     */
    rotation?: number;
    /**
     * The scale factor applied to the components of the UV coordinates.
     *
     * @default [1, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTextureTransform.html#scale)
     */
    scale?: number[];
  }

  export interface MeshTransform extends Accessor, JSONSupport {}

  export class MeshTransform {
    /**
     * A mesh transform.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html)
     */

    constructor(properties?: MeshTransformProperties);

    /**
     * Rotation angle in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#rotationAngle)
     */
    rotationAngle: number;
    /**
     * Axis of rotation.
     *
     * @default [0, 0, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#rotationAxis)
     */
    rotationAxis: number[];
    /**
     * Scale.
     *
     * @default [1, 1, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#scale)
     */
    scale: number[];
    /**
     * Translation.
     *
     * @default [0, 0, 0]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#translation)
     */
    translation: number[];

    /**
     * Creates a deep clone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#clone)
     */
    clone(): MeshTransform;

    static fromJSON(json: any): MeshTransform;
  }

  interface MeshTransformProperties {
    /**
     * Rotation angle in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#rotationAngle)
     */
    rotationAngle?: number;
    /**
     * Axis of rotation.
     *
     * @default [0, 0, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#rotationAxis)
     */
    rotationAxis?: number[];
    /**
     * Scale.
     *
     * @default [1, 1, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#scale)
     */
    scale?: number[];
    /**
     * Translation.
     *
     * @default [0, 0, 0]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-MeshTransform.html#translation)
     */
    translation?: number[];
  }

  /**
   * Various utilities and convenience functions for working with [Mesh](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html) objects.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html)
   */
  interface meshUtils {
    /**
     * Converts a mesh to a new vertex space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#convertVertexSpace)
     */
    convertVertexSpace(
      mesh: Mesh,
      targetVertexSpace: MeshGeoreferencedVertexSpace | MeshLocalVertexSpace,
      options?: meshUtilsConvertVertexSpaceOptions,
    ): Promise<Mesh>;
    /**
     * Creates an elevation sampler from a mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#createElevationSampler)
     */
    createElevationSampler(mesh: Mesh, options?: meshUtilsCreateElevationSamplerOptions): Promise<ElevationSampler>;
    /**
     * Creates a mesh geometry by sampling elevation data from an elevation service on a regular grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#createFromElevation)
     */
    createFromElevation(
      source: ElevationLayer | Ground | ElevationSampler,
      extent: Extent,
      options?: meshUtilsCreateFromElevationOptions,
    ): Promise<Mesh>;
    /**
     * Georeferences vertices specified in a Cartesian coordinate system.
     *
     * @deprecated since version 4.30. Use [`convertVertexSpace`](#convertVertexSpace) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#georeference)
     */
    georeference(
      vertexAttributes: VertexAttributes,
      location: Point,
      options?: meshUtilsGeoreferenceOptions,
    ): VertexAttributes;
    /**
     * Merges multiple meshes into a single mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#merge)
     */
    merge(geometries: Mesh[]): Mesh;
    /**
     * Projects georeferenced vertices to a Cartesian coordinate system.
     *
     * @deprecated since version 4.30. Use [`convertVertexSpace`](#convertVertexSpace) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#ungeoreference)
     */
    ungeoreference(
      vertexAttributes: VertexAttributes,
      location: Point,
      options?: meshUtilsUngeoreferenceOptions,
    ): VertexAttributes;
  }

  export const meshUtils: meshUtils;

  export interface meshUtilsConvertVertexSpaceOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#convertVertexSpace)
     */
    signal?: AbortSignal;
  }

  export interface meshUtilsCreateElevationSamplerOptions {
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#createElevationSampler)
     */
    noDataValue?: number;
  }

  export interface meshUtilsCreateFromElevationOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#createFromElevation)
     */
    demResolution?: number | string;
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#createFromElevation)
     */
    signal?: AbortSignal;
  }

  export interface meshUtilsGeoreferenceOptions {
    /**
     * Indicates whether to georeference relative to the globe or the projected coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#georeference)
     */
    geographic?: boolean;
    /**
     * Indicates the unit of the source data.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  export interface meshUtilsUngeoreferenceOptions {
    /**
     * Indicates whether the coordinates are georeferenced relative to the globe or the projected coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#ungeoreference)
     */
    geographic?: boolean;
    /**
     * Indicates the unit of the resulting data.
     *
     * [Read more...](global.html#unit)
     */
    unit?: LengthUnit;
  }

  /**
   * Represents the position and normal vertex attribute buffers of a mesh.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#VertexAttributes)
   */
  export interface VertexAttributes {
    /**
     * The position buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#VertexAttributes)
     */
    position: Float64Array;
    /**
     * The normal buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#VertexAttributes)
     */
    normal?: Float32Array;
    /**
     * The tangent buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-meshUtils.html#VertexAttributes)
     */
    tangent?: Float32Array;
  }

  /**
   * Provides a utility method that normalizes geometries that intersect the central meridian or fall outside the world extent so they stay within the coordinate system of the view.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-normalizeUtils.html)
   */
  interface normalizeUtils {
    /**
     * Returns an [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) over the dateline that is smaller than the normalized width if it visually contains the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-normalizeUtils.html#getDenormalizedExtent)
     */
    getDenormalizedExtent(geometry: Geometry): Extent;
    /**
     * Normalizes geometries that intersect the central meridian or fall outside the world extent so they stay within the coordinate system of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-normalizeUtils.html#normalizeCentralMeridian)
     */
    normalizeCentralMeridian(geometries: Geometry[], url?: string, requestOptions?: any): Promise<Geometry[]>;
  }

  export const normalizeUtils: normalizeUtils;

  /**
   * Converts Web Mercator coordinates to geographic coordinates and vice versa.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html)
   */
  interface webMercatorUtils {
    /**
     * Returns `true` if the `source` spatial reference can be projected to the `target` spatial reference with the [project()](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#project) function, or if the `source` and `target` are the same [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#canProject)
     */
    canProject(source: SpatialReference | any, target: SpatialReference | any): boolean;
    /**
     * Converts a geometry from geographic units (wkid: 4326) to Web Mercator units (wkid: 3857).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#geographicToWebMercator)
     */
    geographicToWebMercator(geometry: Geometry): Geometry;
    /**
     * Translates the given latitude and longitude (decimal degree) values to Web Mercator XY values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#lngLatToXY)
     */
    lngLatToXY(long: number, lat: number): number[];
    /**
     * Projects the geometry clientside (if possible).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#project)
     */
    project(geometry: Geometry, spatialReference: SpatialReference | any): Geometry;
    /**
     * Converts a geometry from Web Mercator units (wkid: 3857) to geographic units (wkid: 4326).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#webMercatorToGeographic)
     */
    webMercatorToGeographic(geometry: Geometry, isLinear?: boolean): Geometry;
    /**
     * Translates the given Web Mercator coordinates to Longitude and Latitude values (decimal degrees).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-support-webMercatorUtils.html#xyToLngLat)
     */
    xyToLngLat(x: number, y: number): number[];
  }

  export const webMercatorUtils: webMercatorUtils;

  /**
   * Extent.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Extent)
   */
  export type geometryExtent = Extent;

  /**
   * Geometry types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Geometry)
   */
  export type geometryGeometry = Extent | Multipoint | Point | Polygon | Polyline | Mesh;

  /**
   * Mesh.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Mesh)
   */
  export type geometryMesh = Mesh;

  /**
   * Multipoint.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Multipoint)
   */
  export type geometryMultipoint = Multipoint;

  /**
   * Point.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Point)
   */
  export type geometryPoint = Point;

  /**
   * Polygon.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Polygon)
   */
  export type geometryPolygon = Polygon;

  /**
   * Polyline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#Polyline)
   */
  export type geometryPolyline = Polyline;

  /**
   * Spatial Reference.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry.html#SpatialReference)
   */
  export type geometrySpatialReference = SpatialReference;

  export interface Graphic extends Accessor, JSONSupport, Clonable {}

  export class Graphic {
    /**
     * A Graphic is a vector representation of real world geographic phenomena.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html)
     */

    constructor(properties?: GraphicProperties);

    /**
     * The aggregateGeometries contains spatial aggregation geometries when [statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType) query is executed with `envelope-aggregate`, `centroid-aggregate` and/or `convex-hull-aggregate` statistics type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#aggregateGeometries)
     */
    aggregateGeometries: any;
    /**
     * Name-value pairs of fields and field values associated with the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#attributes)
     */
    attributes: any;
    /**
     * The geometry that defines the graphic's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#geometry)
     */
    geometry: Geometry;
    /**
     * Indicates whether the graphic refers to an aggregate, or [cluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#isAggregate)
     */
    readonly isAggregate: boolean;
    /**
     * If applicable, references the layer in which the graphic is stored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#layer)
     */
    layer: Layer;
    /**
     * Returns information about an origin of a graphic if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#origin)
     */
    origin: GraphicOrigin;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The [Symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) for the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#symbol)
     */
    symbol: Symbol;
    /**
     * Indicates the visibility of the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#visible)
     */
    visible: boolean;

    /**
     * Returns the value of the specified attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#getAttribute)
     */
    getAttribute(name: string): any;
    /**
     * Returns the popup template applicable for the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#getEffectivePopupTemplate)
     */
    getEffectivePopupTemplate(defaultPopupTemplateEnabled?: boolean): PopupTemplate;
    /**
     * Returns the Object ID of the feature associated with the graphic, if it exists.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#getObjectId)
     */
    getObjectId(): number;
    /**
     * Sets a new value to the specified attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#setAttribute)
     */
    setAttribute(name: string, newValue: any): void;

    static fromJSON(json: any): Graphic;
  }

  interface GraphicProperties {
    /**
     * The aggregateGeometries contains spatial aggregation geometries when [statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType) query is executed with `envelope-aggregate`, `centroid-aggregate` and/or `convex-hull-aggregate` statistics type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#aggregateGeometries)
     */
    aggregateGeometries?: any;
    /**
     * Name-value pairs of fields and field values associated with the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#attributes)
     */
    attributes?: any;
    /**
     * The geometry that defines the graphic's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * If applicable, references the layer in which the graphic is stored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#layer)
     */
    layer?: LayerProperties;
    /**
     * Returns information about an origin of a graphic if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#origin)
     */
    origin?: VectorTileOrigin;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The [Symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) for the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#symbol)
     */
    symbol?: SymbolProperties;
    /**
     * Indicates the visibility of the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#visible)
     */
    visible?: boolean;
  }

  /**
   * Information about an [origin](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#origin) of a graphic.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#GraphicOrigin)
   */
  export type GraphicOrigin = VectorTileOrigin;

  /**
   * Origin information about a graphic returned as a result of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method for [VectorTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#VectorTileOrigin)
   */
  export interface VectorTileOrigin {
    type: "vector-tile";
    /**
     * The [unique identifier](https://maplibre.org/maplibre-style-spec/layers/#id) of the style layer in the [vector tile style](https://maplibre.org/maplibre-style-spec).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#VectorTileOrigin)
     */
    layerId: string;
    /**
     * The layer index of the style layer in the [vector tile style](https://maplibre.org/maplibre-style-spec).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#VectorTileOrigin)
     */
    layerIndex: number;
    /**
     * VectorTileLayer where the graphic originates from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#VectorTileOrigin)
     */
    layer: VectorTileLayer;
  }

  export interface Ground extends Accessor, Loadable, JSONSupport {}

  export class Ground {
    /**
     * The Ground class contains properties that specify how the ground surface is displayed in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html)
     */

    constructor(properties?: GroundProperties);

    /**
     * A collection of [ElevationLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html) that define the elevation or terrain that makes up the ground surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers)
     */
    layers: Collection<ElevationLayer | BaseElevationLayer>;
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * Specifies the user navigation constraints relative to the ground surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#navigationConstraint)
     */
    navigationConstraint: GroundNavigationConstraint;
    /**
     * Opacity of the ground, including surface default color and the basemap (without reference layers).
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#opacity)
     */
    opacity: number;
    /**
     * The color of the ground surface, displayed underneath the basemap.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#surfaceColor)
     */
    surfaceColor: Color;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#clone)
     */
    clone(): Ground;
    /**
     * Creates an elevation sampler for the given extent by querying the ground layers for elevation data and caching it so values may be sampled quickly afterwards.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#createElevationSampler)
     */
    createElevationSampler(extent: Extent, options?: GroundCreateElevationSamplerOptions): Promise<ElevationSampler>;
    /**
     * Destroys the ground and its [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#destroy)
     */
    destroy(): void;
    /**
     * Loads all the externally loadable resources associated with the ground.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#loadAll)
     */
    loadAll(): Promise<Ground>;
    /**
     * Query the ground layer services for elevation values for the given geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation)
     */
    queryElevation(
      geometry: Point | Multipoint | Polyline,
      options?: GroundQueryElevationOptions,
    ): Promise<ElevationQueryResult>;

    static fromJSON(json: any): Ground;
  }

  interface GroundProperties extends LoadableProperties {
    /**
     * A collection of [ElevationLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html) that define the elevation or terrain that makes up the ground surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers)
     */
    layers?: CollectionProperties<
      (ElevationLayerProperties & { type: "elevation" }) | (BaseElevationLayerProperties & { type: "base-elevation" })
    >;
    /**
     * Specifies the user navigation constraints relative to the ground surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#navigationConstraint)
     */
    navigationConstraint?: GroundNavigationConstraintProperties;
    /**
     * Opacity of the ground, including surface default color and the basemap (without reference layers).
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#opacity)
     */
    opacity?: number;
    /**
     * The color of the ground surface, displayed underneath the basemap.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#surfaceColor)
     */
    surfaceColor?: Color | number[] | string;
  }

  /**
   * Object returned when [queryElevation()](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation) promise resolves:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
   */
  export interface ElevationQueryResult {
    /**
     * The geometry with sampled z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
     */
    geometry: Point | Multipoint | Polyline;
    /**
     * Contains additional information about how the geometry was sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
     */
    sampleInfo?: ElevationQueryResultSampleInfo[];
    /**
     * The value used when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
     */
    noDataValue: number;
  }

  export interface GroundCreateElevationSamplerOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#createElevationSampler)
     */
    demResolution?: number | string;
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#createElevationSampler)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#createElevationSampler)
     */
    signal?: AbortSignal;
  }

  export interface GroundNavigationConstraintProperties {
    /**
     * The type of the constraint.
     *
     * [Read more...](global.html#type)
     */
    type?: "stay-above" | "none";
  }

  export interface GroundNavigationConstraint extends AnonymousAccessor {
    /**
     * The type of the constraint.
     *
     * [Read more...](global.html#type)
     */
    type: "stay-above" | "none";
  }

  export interface GroundQueryElevationOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation)
     */
    demResolution?: number | string;
    /**
     * Indicates whether to return additional sample information for each sampled coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation)
     */
    returnSampleInfo?: boolean;
    /**
     * The value that appears in the resulting geometry when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#queryElevation)
     */
    signal?: AbortSignal;
  }

  export interface ElevationQueryResultSampleInfo {
    /**
     * The resolution at which the z-value was sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
     */
    demResolution: number;
    /**
     * The elevation source from which the data for the corresponding coordinate was sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#ElevationQueryResult)
     */
    source: ElevationLayer;
  }

  export interface Credential extends Accessor, Evented {}

  export class Credential {
    constructor(properties?: CredentialProperties);

    /**
     * Token expiration time specified as number of milliseconds since 1 January 1970 00:00:00 UTC.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#expires)
     */
    expires: number;
    /**
     * Indicates that this credential was created to access the [ArcGIS Server Administrator REST API](https://developers.arcgis.com/rest/enterprise-administration/server/overview/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#isAdmin)
     */
    isAdmin: boolean;
    /**
     * The Identity Manager's [setOAuthRedirectionHandler](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setOAuthRedirectionHandler) returns an object that contains a `state` property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#oAuthState)
     */
    oAuthState: any;
    /**
     * The server url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#server)
     */
    server: string;
    /**
     * Indicates whether the resources accessed using this credential should be fetched over HTTPS protocol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#ssl)
     */
    ssl: boolean;
    /**
     * Token generated by the token service using the specified userId and password.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#token)
     */
    token: string;
    /**
     * User associated with the Credential object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#userId)
     */
    userId: string;

    /**
     * Destroys the credential.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#destroy)
     */
    destroy(): void;
    /**
     * Generates a new token and updates the Credential's [token](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#token) property with the newly acquired token.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#refreshToken)
     */
    refreshToken(): void;

    on(name: "token-change", eventHandler: CredentialTokenChangeEventHandler): IHandle;

    on(name: "destroy", eventHandler: CredentialDestroyEventHandler): IHandle;
  }

  interface CredentialProperties {
    /**
     * Token expiration time specified as number of milliseconds since 1 January 1970 00:00:00 UTC.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#expires)
     */
    expires?: number;
    /**
     * Indicates that this credential was created to access the [ArcGIS Server Administrator REST API](https://developers.arcgis.com/rest/enterprise-administration/server/overview/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#isAdmin)
     */
    isAdmin?: boolean;
    /**
     * The Identity Manager's [setOAuthRedirectionHandler](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setOAuthRedirectionHandler) returns an object that contains a `state` property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#oAuthState)
     */
    oAuthState?: any;
    /**
     * The server url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#server)
     */
    server?: string;
    /**
     * Indicates whether the resources accessed using this credential should be fetched over HTTPS protocol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#ssl)
     */
    ssl?: boolean;
    /**
     * Token generated by the token service using the specified userId and password.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#token)
     */
    token?: string;
    /**
     * User associated with the Credential object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html#userId)
     */
    userId?: string;
  }

  export interface CredentialDestroyEvent {}

  export interface CredentialTokenChangeEvent {}

  export class IdentityManager extends Evented {
    /**
     * Dialog box widget used to challenge the user for their credentials when the application attempts to access a secure resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#dialog)
     */
    dialog: Widget;
    /**
     * The suggested lifetime of the token in minutes.
     *
     * @default 60
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#tokenValidity)
     */
    tokenValidity: number;

    /**
     * Returns a [credential](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html) if the user has already signed in to access the given resource and is allowed to do so when using the given application id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#checkAppAccess)
     */
    checkAppAccess(resUrl: string, appId: string): Promise<any>;
    /**
     * Returns the [Credential](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html) if the user has already signed in to access the given resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#checkSignInStatus)
     */
    checkSignInStatus(resUrl: string): Promise<Credential>;
    /**
     * Destroys all credentials.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#destroyCredentials)
     */
    destroyCredentials(): void;
    /**
     * Disables the use of `window.postMessage` to serve authentication requests that were enabled by [enablePostMessageAuth](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#enablePostMessageAuth).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#disablePostMessageAuth)
     */
    disablePostMessageAuth(): void;
    /**
     * Enables the IdentityManager to serve authentication requests for the given resource from apps running in child iframes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#enablePostMessageAuth)
     */
    enablePostMessageAuth(resUrl?: string): void;
    /**
     * Returns the [Credential](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html) for the resource identified by the specified url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#findCredential)
     */
    findCredential(url: string, userId?: string): Credential;
    /**
     * Returns the [OAuthInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html) configuration for the passed in Portal server URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#findOAuthInfo)
     */
    findOAuthInfo(url: string): OAuthInfo;
    /**
     * Returns information about the server that is hosting the specified URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#findServerInfo)
     */
    findServerInfo(url: string): ServerInfo;
    /**
     * Returns an object containing a token and its expiration time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#generateToken)
     */
    generateToken(serverInfo: ServerInfo, userInfo: any, options?: IdentityManagerGenerateTokenOptions): Promise<any>;
    /**
     * Returns a [Credential](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-Credential.html) object that can be used to access the secured resource identified by the input URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#getCredential)
     */
    getCredential(url: string, options?: IdentityManagerGetCredentialOptions): Promise<Credential>;
    /**
     * Call this method during application initialization with the JSON previously obtained from the [toJSON()](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#toJSON) method used to re-hydrate the state of IdentityManager.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#initialize)
     */
    initialize(json: any): void;
    /**
     * Indicates if the IdentityManager is busy accepting user input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#isBusy)
     */
    isBusy(): boolean;
    /**
     * Registers OAuth 2.0 configurations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerOAuthInfos)
     */
    registerOAuthInfos(oAuthInfos: OAuthInfo[]): void;
    /**
     * Register secure servers and the token endpoints.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerServers)
     */
    registerServers(serverInfos: ServerInfo[]): void;
    /**
     * Registers the given OAuth 2.0 access token or ArcGIS Server token with the IdentityManager.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    registerToken(properties: IdentityManagerRegisterTokenProperties): void;
    /**
     * Once a user successfully logs in, they are redirected back to the application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setOAuthRedirectionHandler)
     */
    setOAuthRedirectionHandler(handlerFunction: HandlerCallback): void;
    /**
     * Use this method in the popup callback page to pass the token and other values back to the IdentityManager.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setOAuthResponseHash)
     */
    setOAuthResponseHash(hash: string): void;
    /**
     * When accessing secured resources, the IdentityManager may prompt for username and password and send them to the server using a secure connection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setProtocolErrorHandler)
     */
    setProtocolErrorHandler(handlerFunction: IdentityManagerSetProtocolErrorHandlerHandlerFunction): void;
    /**
     * Return properties of this object in JSON format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#toJSON)
     */
    toJSON(): any;

    on(name: "dialog-create", eventHandler: IdentityManagerDialogCreateEventHandler): IHandle;

    on(name: "credential-create", eventHandler: IdentityManagerCredentialCreateEventHandler): IHandle;
  }

  export interface IdentityManagerCredentialCreateEvent {
    credential: Credential;
  }

  export interface IdentityManagerDialogCreateEvent {}

  export type HandlerCallback = (
    authorizeParams: any,
    authorizeUrl: string,
    oAuthInfo: OAuthInfo,
    resourceUrl: string,
    serverInfo: ServerInfo,
  ) => void;

  export interface IdentityManagerGenerateTokenOptions {
    /**
     * The server URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#generateToken)
     */
    serverUrl: string;
    /**
     * The server token.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#generateToken)
     */
    token: string;
    /**
     * Indicates if the server requires SSL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#generateToken)
     */
    ssl: boolean;
  }

  export interface IdentityManagerGetCredentialOptions {
    /**
     * Error object returned by the server from a previous attempt to fetch the given URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#getCredential)
     */
    error?: Error;
    /**
     * If set to *false*, the user will not be shown a dialog before the OAuth popup window is opened.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#getCredential)
     */
    oAuthPopupConfirmation?: boolean;
    /**
     * Token used for a previous unsuccessful attempt to fetch the given URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#getCredential)
     */
    token?: string;
  }

  export interface IdentityManagerRegisterTokenProperties {
    /**
     * Token expiration time specified as number of milliseconds since 1 January 1970 00:00:00 UTC.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    expires?: number;
    /**
     * For ArcGIS Online or Portal, this is https://www.arcgis.com/sharing/rest or similar to https://www.example.com/portal/sharing/rest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    server: string;
    /**
     * Set this to `true` if the user has an ArcGIS Online organizational account and the organization is configured to allow access to resources only through SSL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    ssl?: boolean;
    /**
     * The access token.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    token: string;
    /**
     * The id of the user who owns the access token.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#registerToken)
     */
    userId?: string;
  }

  export interface IdentityManagerSetProtocolErrorHandlerHandlerFunction {
    /**
     * The secure resource URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setProtocolErrorHandler)
     */
    resourceUrl: string;
    /**
     * ServerInfo object describing the server where the secure resource is hosted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-IdentityManager.html#setProtocolErrorHandler)
     */
    serverInfo: ServerInfo;
  }

  export interface OAuthInfo extends Accessor, JSONSupport {}

  export class OAuthInfo {
    /**
     * This class contains information about an OAuth 2.0 configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html)
     */

    constructor(properties?: OAuthInfoProperties);

    /**
     * The registered application id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#appId)
     */
    appId: string;
    /**
     * Applications with the same value will share the stored token on the same host.
     *
     * @default / (forward slash)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#authNamespace)
     */
    authNamespace: string;
    /**
     * The number of minutes that the token is valid.
     *
     * @default 20160 (two weeks)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#expiration)
     */
    expiration: number;
    /**
     * Set this property to specify the type of authentication to use.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#flowType)
     */
    flowType: "auto" | "authorization-code" | "implicit";
    /**
     * Set this property to `true` to force the user to sign in with the id in [userId](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#userId).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#forceUserId)
     */
    forceUserId: boolean;
    /**
     * The locale for the OAuth sign-in page.
     *
     * @default // Based on your browser/OS and the organization locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#locale)
     */
    locale: string;
    /**
     * The minimum time in minutes before a saved token is due to expire that should still be considered valid for use.
     *
     * @default 30
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#minTimeUntilExpiration)
     */
    minTimeUntilExpiration: number;
    /**
     * Set to `true` to show the OAuth sign-in page in a popup window.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popup)
     */
    popup: boolean;
    /**
     * Applicable if working with the popup user-login workflow.
     *
     * @default "oauth-callback.html"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popupCallbackUrl)
     */
    popupCallbackUrl: string;
    /**
     * The window features passed to [window.open()](https://developer.mozilla.org/en-US/docs/Web/API/Window/open).
     *
     * @default "height=490,width=800,resizable,scrollbars,status"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popupWindowFeatures)
     */
    popupWindowFeatures: string;
    /**
     * The ArcGIS Enterprise portal URL.
     *
     * @default "https://www.arcgis.com"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#portalUrl)
     */
    portalUrl: string;
    /**
     * Set this property to `true` when [popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popup) is `false` in order to have the window's location hash value restored after signing in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#preserveUrlHash)
     */
    preserveUrlHash: boolean;
    /**
     * The user id used when `forceUserId` is `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#userId)
     */
    userId: string;

    /**
     * Creates a copy of the OAuthInfo object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#clone)
     */
    clone(): OAuthInfo;

    static fromJSON(json: any): OAuthInfo;
  }

  interface OAuthInfoProperties {
    /**
     * The registered application id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#appId)
     */
    appId?: string;
    /**
     * Applications with the same value will share the stored token on the same host.
     *
     * @default / (forward slash)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#authNamespace)
     */
    authNamespace?: string;
    /**
     * The number of minutes that the token is valid.
     *
     * @default 20160 (two weeks)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#expiration)
     */
    expiration?: number;
    /**
     * Set this property to specify the type of authentication to use.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#flowType)
     */
    flowType?: "auto" | "authorization-code" | "implicit";
    /**
     * Set this property to `true` to force the user to sign in with the id in [userId](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#userId).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#forceUserId)
     */
    forceUserId?: boolean;
    /**
     * The locale for the OAuth sign-in page.
     *
     * @default // Based on your browser/OS and the organization locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#locale)
     */
    locale?: string;
    /**
     * The minimum time in minutes before a saved token is due to expire that should still be considered valid for use.
     *
     * @default 30
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#minTimeUntilExpiration)
     */
    minTimeUntilExpiration?: number;
    /**
     * Set to `true` to show the OAuth sign-in page in a popup window.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popup)
     */
    popup?: boolean;
    /**
     * Applicable if working with the popup user-login workflow.
     *
     * @default "oauth-callback.html"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popupCallbackUrl)
     */
    popupCallbackUrl?: string;
    /**
     * The window features passed to [window.open()](https://developer.mozilla.org/en-US/docs/Web/API/Window/open).
     *
     * @default "height=490,width=800,resizable,scrollbars,status"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popupWindowFeatures)
     */
    popupWindowFeatures?: string;
    /**
     * The ArcGIS Enterprise portal URL.
     *
     * @default "https://www.arcgis.com"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#portalUrl)
     */
    portalUrl?: string;
    /**
     * Set this property to `true` when [popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#popup) is `false` in order to have the window's location hash value restored after signing in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#preserveUrlHash)
     */
    preserveUrlHash?: boolean;
    /**
     * The user id used when `forceUserId` is `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-OAuthInfo.html#userId)
     */
    userId?: string;
  }

  export interface ServerInfo extends Accessor, JSONSupport {}

  export class ServerInfo {
    /**
     * This class contains information about an ArcGIS Server and its token endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html)
     */

    constructor(properties?: ServerInfoProperties);

    /**
     * The token service URL used to generate tokens for ArcGIS Server Admin resources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#adminTokenServiceUrl)
     */
    adminTokenServiceUrl: string;
    /**
     * Version of the ArcGIS Server or Portal deployed on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#currentVersion)
     */
    currentVersion: number;
    /**
     * Indicates whether the [server](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server) is a [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#hasPortal)
     */
    hasPortal: boolean;
    /**
     * Indicates whether the [server](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server) is an [ArcGIS Server](https://enterprise.arcgis.com/en/server/latest/get-started/windows/what-is-arcgis-for-server-.htm) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#hasServer)
     */
    hasServer: boolean;
    /**
     * The server URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server)
     */
    server: string;
    /**
     * Validity of short-lived token in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#shortLivedTokenValidity)
     */
    shortLivedTokenValidity: number;
    /**
     * The token service URL used to generate tokens for the secured resources on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#tokenServiceUrl)
     */
    tokenServiceUrl: string;
    /**
     * Indicates whether the server is configured to work with web tier authentication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#webTierAuth)
     */
    webTierAuth: boolean;

    static fromJSON(json: any): ServerInfo;
  }

  interface ServerInfoProperties {
    /**
     * The token service URL used to generate tokens for ArcGIS Server Admin resources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#adminTokenServiceUrl)
     */
    adminTokenServiceUrl?: string;
    /**
     * Version of the ArcGIS Server or Portal deployed on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#currentVersion)
     */
    currentVersion?: number;
    /**
     * Indicates whether the [server](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server) is a [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#hasPortal)
     */
    hasPortal?: boolean;
    /**
     * Indicates whether the [server](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server) is an [ArcGIS Server](https://enterprise.arcgis.com/en/server/latest/get-started/windows/what-is-arcgis-for-server-.htm) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#hasServer)
     */
    hasServer?: boolean;
    /**
     * The server URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#server)
     */
    server?: string;
    /**
     * Validity of short-lived token in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#shortLivedTokenValidity)
     */
    shortLivedTokenValidity?: number;
    /**
     * The token service URL used to generate tokens for the secured resources on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#tokenServiceUrl)
     */
    tokenServiceUrl?: string;
    /**
     * Indicates whether the server is configured to work with web tier authentication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-identity-ServerInfo.html#webTierAuth)
     */
    webTierAuth?: boolean;
  }

  /**
   * This module provides the ability to set the app locale along with date and number formatting methods and supporting utilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html)
   */
  interface intl {
    /**
     * Converts a [web map date format string](https://developers.arcgis.com/web-map-specification/objects/format/) to an [Intl.DateTimeFormat](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/DateTimeFormat#Parameters) options object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#convertDateFormatToIntlOptions)
     */
    convertDateFormatToIntlOptions(
      format?:
        | "short-date"
        | "short-date-short-time"
        | "short-date-short-time-24"
        | "short-date-long-time"
        | "short-date-long-time-24"
        | "long-month-day-year"
        | "long-month-day-year-short-time"
        | "long-month-day-year-short-time-24"
        | "long-month-day-year-long-time"
        | "long-month-day-year-long-time-24"
        | "day-short-month-year"
        | "day-short-month-year-short-time"
        | "day-short-month-year-short-time-24"
        | "day-short-month-year-long-time"
        | "day-short-month-year-long-time-24"
        | "long-date"
        | "long-date-short-time"
        | "long-date-short-time-24"
        | "long-date-long-time"
        | "long-date-long-time-24"
        | "long-month-year"
        | "short-month-year"
        | "year"
        | "short-time"
        | "long-time",
    ): Intl.DateTimeFormatOptions;
    /**
     * Converts a [NumberFormat](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#NumberFormat) to an [Intl.NumberFormatOptions](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/NumberFormat#Parameters) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#convertNumberFormatToIntlOptions)
     */
    convertNumberFormatToIntlOptions(format?: NumberFormat): Intl.NumberFormatOptions;
    /**
     * Creates a message bundle loader specialized in loading translation files as JSON files.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#createJSONLoader)
     */
    createJSONLoader(params: intlCreateJSONLoaderParams): MessageBundleLoader;
    /**
     * Loads a localized message bundle used with the current API locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#fetchMessageBundle)
     */
    fetchMessageBundle(bundleId: string): Promise<any>;
    /**
     * Formats a `Date` or `Number` value to a string in the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#formatDate)
     */
    formatDate(value: Date | number, formatOptions?: Intl.DateTimeFormatOptions): string;
    /**
     * Formats a `date-only` field value to a string in the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#formatDateOnly)
     */
    formatDateOnly(value: string, options?: Intl.DateTimeFormatOptions): string;
    /**
     * Formats a `Number` value to a string in the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#formatNumber)
     */
    formatNumber(value: number, formatOptions?: Intl.NumberFormatOptions): string;
    /**
     * Formats a `time-only` field value to a string in the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#formatTimeOnly)
     */
    formatTimeOnly(value: string, options?: Intl.DateTimeFormatOptions): string;
    /**
     * Formats a `timestamp-offset` field value to a string in the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#formatTimestamp)
     */
    formatTimestamp(value: string, options?: Intl.DateTimeFormatOptions): string;
    /**
     * Returns the current locale used by the API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#getLocale)
     */
    getLocale(): string;
    /**
     * Returns one of the known message bundle locale for an input locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#normalizeMessageBundleLocale)
     */
    normalizeMessageBundleLocale(locale: string): string;
    /**
     * Registers a [callback](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#LocaleChangeCallback) that gets notified when the locale changes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#onLocaleChange)
     */
    onLocaleChange(callback: LocaleChangeCallback): Handle;
    /**
     * Provides right-to-left preference for input locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#prefersRTL)
     */
    prefersRTL(locale?: string): boolean;
    /**
     * Registers a message loader to load specified message bundles needed for translating strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#registerMessageBundleLoader)
     */
    registerMessageBundleLoader(loader: MessageBundleLoader): any;
    /**
     * Sets the locale used by the API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#setLocale)
     */
    setLocale(locale: string): void;
    /**
     * Use this to substitute keys in a `template` string with values from the argument `data`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#substitute)
     */
    substitute(template: string, data: any, options?: SubstituteOptions): string;
  }

  export const intl: intl;

  export type FetchMessageBundle = (bundleId: string, locale: string) => Promise<any>;

  export interface intlCreateJSONLoaderParams {
    /**
     * Used to check if the loader should be used to load a candidate's message bundle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#createJSONLoader)
     */
    pattern: string | RegExp;
    /**
     * Used to calculate the relative path of the file to load.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#createJSONLoader)
     */
    base: string;
    /**
     * The location of the translation files.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#createJSONLoader)
     */
    location: string | URL | Function;
  }

  export type LocaleChangeCallback = (newLocale: string) => void;

  /**
   * A message bundle loader is an object used to load translation strings in the user's locale.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#MessageBundleLoader)
   */
  export interface MessageBundleLoader {
    /**
     * Used to check if the loader should be used to load a candidate message bundle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#MessageBundleLoader)
     */
    pattern: string | RegExp;
    /**
     * Called to load the message bundle if the `pattern` matches the bundle identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#MessageBundleLoader)
     */
    fetchMessageBundle: FetchMessageBundle;
  }

  /**
   * The Web map definition for formatting numbers.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#NumberFormat)
   */
  export interface NumberFormat {
    /**
     * Indicates if the number should be formatted with a thousands separator.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#NumberFormat)
     */
    digitSeparator?: boolean;
    /**
     * Specifies the number of decimal places that should appear in the formatted number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#NumberFormat)
     */
    places?: number;
  }

  /**
   * The formatting options for date values.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteDateTimeFormatOptions)
   */
  export interface SubstituteDateTimeFormatOptions {
    /**
     * The type of this format.
     *
     * [Read more...](global.html#type)
     */
    type: "date";
    /**
     * The date format options for the [Intl.DateTimeFormat](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/DateTimeFormat) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteDateTimeFormatOptions)
     */
    intlOptions: Intl.DateTimeFormatOptions;
  }

  /**
   * The formatting options for numeric values.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteNumberFormatOptions)
   */
  export interface SubstituteNumberFormatOptions {
    /**
     * The type of this format.
     *
     * [Read more...](global.html#type)
     */
    type: "number";
    /**
     * The Intl number format options for the [Intl.NumberFormat](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/NumberFormat) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteNumberFormatOptions)
     */
    intlOptions: Intl.NumberFormatOptions;
  }

  /**
   * An object to specify substitution options.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteOptions)
   */
  export interface SubstituteOptions {
    /**
     * A hashmap of string keys to formatting options.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-intl.html#SubstituteOptions)
     */
    format: HashMap<SubstituteDateTimeFormatOptions | SubstituteNumberFormatOptions>;
  }

  /**
   * Utility for retrieving the current version of the API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-kernel.html)
   */
  interface kernel {
    /**
     * Current full version of the ArcGIS Maps SDK for JavaScript.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-kernel.html#fullVersion)
     */
    readonly fullVersion: string;
    /**
     * Current `major.minor` version of the ArcGIS Maps SDK for JavaScript.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-kernel.html#version)
     */
    readonly version: string;
  }

  export const kernel: kernel;

  export interface BaseDynamicLayer extends Layer, ScaleRangeLayer, RefreshableLayer, BlendLayer {}

  export class BaseDynamicLayer {
    /**
     * This class may be extended to create dynamic map layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseDynamicLayer.html)
     */

    constructor(properties?: BaseDynamicLayerProperties);

    readonly type: "base-dynamic";

    /**
     * Adds a promise to the layer's loadable chain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseDynamicLayer.html#addResolvingPromise)
     */
    addResolvingPromise(promiseToLoad: Promise<any>): Promise<any>;
    /**
     * This method fetches the image for the specified extent and size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseDynamicLayer.html#fetchImage)
     */
    fetchImage(
      extent: Extent,
      width: number,
      height: number,
      options?: BaseDynamicLayerFetchImageOptions,
    ): Promise<HTMLImageElement | HTMLCanvasElement>;
    /**
     * This method returns a URL to an image for a given extent, width, and height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseDynamicLayer.html#getImageUrl)
     */
    getImageUrl(extent: Extent, width: number, height: number): Promise<string> | string;

    on(name: "refresh", eventHandler: BaseDynamicLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: BaseDynamicLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: BaseDynamicLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: BaseDynamicLayerLayerviewDestroyEventHandler): IHandle;
  }

  interface BaseDynamicLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      BlendLayerProperties {}

  export interface BaseDynamicLayerFetchImageOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseDynamicLayer.html#fetchImage)
     */
    signal?: AbortSignal;
  }

  export interface BaseDynamicLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface BaseDynamicLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BaseDynamicLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BaseDynamicLayerRefreshEvent {
    dataChanged: boolean;
  }

  export class BaseElevationLayer extends Layer {
    /**
     * BaseElevationLayer is intended to be extended for creating custom elevation layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html)
     */

    constructor(properties?: BaseElevationLayerProperties);

    /**
     * The spatial reference of the layer.
     *
     * @default {@link module:esri/geometry/SpatialReference#WebMercator SpatialReference.WebMercator}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#tileInfo)
     */
    tileInfo: TileInfo;

    readonly type: "base-elevation";

    /**
     * Adds a promise to the layer's loadable chain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#addResolvingPromise)
     */
    addResolvingPromise(promiseToLoad: Promise<any>): Promise<any>;
    /**
     * Creates an elevation sampler for the given [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) by querying the service layer for elevation data and caching it so values may be sampled quickly afterwards.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#createElevationSampler)
     */
    createElevationSampler(
      extent: Extent,
      options?: BaseElevationLayerCreateElevationSamplerOptions,
    ): Promise<ElevationSampler>;
    /**
     * Fetches a tile at the given level, row, and column present in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#fetchTile)
     */
    fetchTile(
      level: number,
      row: number,
      column: number,
      options?: BaseElevationLayerFetchTileOptions,
    ): Promise<ElevationTileData>;
    /**
     * Returns the bounds of the tile as an array of four numbers that can be readily converted to an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#getTileBounds)
     */
    getTileBounds(level: number, row: number, column: number, out?: number[]): number[];
    /**
     * Queries the service layer for elevation values for the given geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#queryElevation)
     */
    queryElevation(
      geometry: Point | Multipoint | Polyline,
      options?: BaseElevationLayerQueryElevationOptions,
    ): Promise<ElevationLayerElevationQueryResult>;
  }

  interface BaseElevationLayerProperties extends LayerProperties {
    /**
     * The spatial reference of the layer.
     *
     * @default {@link module:esri/geometry/SpatialReference#WebMercator SpatialReference.WebMercator}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
  }

  export interface BaseElevationLayerCreateElevationSamplerOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#createElevationSampler)
     */
    demResolution?: number | string;
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#createElevationSampler)
     */
    noDataValue?: number;
  }

  export interface BaseElevationLayerFetchTileOptions {
    /**
     * The value representing pixels in the tile that don't contain an elevation value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#fetchTile)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface BaseElevationLayerQueryElevationOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#queryElevation)
     */
    demResolution?: number | string;
    /**
     * Indicates whether to return additional sample information for each coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#queryElevation)
     */
    returnSampleInfo?: boolean;
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#queryElevation)
     */
    noDataValue?: number;
  }

  /**
   * Describes elevation contained in the pixels that comprise an elevation tile.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
   */
  export interface ElevationTileData {
    /**
     * The elevation values of each pixel in the tile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
     */
    values: number[];
    /**
     * The width of the tile in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
     */
    width: number;
    /**
     * The height of the tile in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
     */
    height: number;
    /**
     * The maximum allowed error of the Z-value for each value in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
     */
    maxZError: number;
    /**
     * Indicating the pixel values where no elevation data is present.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseElevationLayer.html#ElevationTileData)
     */
    noDataValue: number;
  }

  export interface BaseTileLayer extends Layer, ScaleRangeLayer, RefreshableLayer, BlendLayer {}

  export class BaseTileLayer {
    /**
     * This class may be extended to create a custom [TileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html)
     */

    constructor(properties?: BaseTileLayerProperties);

    /**
     * The spatial reference of the layer.
     *
     * @default {@link module:esri/geometry/SpatialReference#WebMercator SpatialReference.WebMercator}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#tileInfo)
     */
    tileInfo: TileInfo;
    /**
     * For [BaseTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html) the type is `base-tile`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#type)
     */
    readonly type: "base-tile" | "bing-maps";

    /**
     * Adds a Promise to the layer's loadable chain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#addResolvingPromise)
     */
    addResolvingPromise(promiseToLoad: Promise<any>): Promise<any>;
    /**
     * This method fetches a tile for the given level, row and column present in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#fetchTile)
     */
    fetchTile(
      level: number,
      row: number,
      col: number,
      options?: BaseTileLayerFetchTileOptions,
    ): Promise<HTMLImageElement | HTMLCanvasElement>;
    /**
     * Returns the bounds of the tile as an array of four numbers that be readily converted to an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#getTileBounds)
     */
    getTileBounds(level: number, row: number, column: number, out?: number[]): number[];
    /**
     * This method returns a URL to an image for a given level, row and column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#getTileUrl)
     */
    getTileUrl(level: number, row: number, col: number): string;

    on(name: "refresh", eventHandler: BaseTileLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: BaseTileLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: BaseTileLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: BaseTileLayerLayerviewDestroyEventHandler): IHandle;
  }

  interface BaseTileLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      BlendLayerProperties {
    /**
     * The spatial reference of the layer.
     *
     * @default {@link module:esri/geometry/SpatialReference#WebMercator SpatialReference.WebMercator}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
  }

  export interface BaseTileLayerFetchTileOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BaseTileLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface BaseTileLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface BaseTileLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BaseTileLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BaseTileLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface BingMapsLayer extends BaseTileLayer, OperationalLayer, BlendLayer {}

  export class BingMapsLayer {
    /**
     * This layer supports Microsoft's Bing tiled map content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html)
     */

    constructor(properties?: BingMapsLayerProperties);

    /**
     * Expose Bing logo url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#bingLogo)
     */
    readonly bingLogo: string;
    /**
     * Copyright information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#copyright)
     */
    readonly copyright: string;
    /**
     * Provides culture specific map labels.
     *
     * @default "en-US"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#culture)
     */
    culture: string;
    /**
     * Indicates if the layer has attribution data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#hasAttributionData)
     */
    readonly hasAttributionData: boolean;
    /**
     * Bing Maps Key.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#key)
     */
    key: string;
    /**
     * This will alter Geopolitical disputed borders and labels to align with the specified user region.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#region)
     */
    region: string;
    /**
     * For more information on Bing map styles please visit: https://learn.microsoft.com/en-us/bingmaps/rest-services/imagery/get-imagery-metadata
     *
     * @default "road"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#style)
     */
    style: "road" | "aerial" | "hybrid";

    readonly type: "bing-maps";

    on(name: "refresh", eventHandler: BingMapsLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: BingMapsLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: BingMapsLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: BingMapsLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): BingMapsLayer;
  }

  interface BingMapsLayerProperties extends BaseTileLayerProperties, OperationalLayerProperties, BlendLayerProperties {
    /**
     * Provides culture specific map labels.
     *
     * @default "en-US"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#culture)
     */
    culture?: string;
    /**
     * Bing Maps Key.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#key)
     */
    key?: string;
    /**
     * This will alter Geopolitical disputed borders and labels to align with the specified user region.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#region)
     */
    region?: string;
    /**
     * For more information on Bing map styles please visit: https://learn.microsoft.com/en-us/bingmaps/rest-services/imagery/get-imagery-metadata
     *
     * @default "road"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BingMapsLayer.html#style)
     */
    style?: "road" | "aerial" | "hybrid";
  }

  export interface BingMapsLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface BingMapsLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BingMapsLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface BingMapsLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface BuildingSceneLayer
    extends Layer,
      SceneService,
      PortalLayer,
      ScaleRangeLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer {}

  export class BuildingSceneLayer {
    /**
     * The BuildingSceneLayer is designed for visualizing buildings with detailed interiors in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html)
     */

    constructor(properties?: BuildingSceneLayerProperties);

    /**
     * The id of the currently active filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#activeFilterId)
     */
    activeFilterId: string;
    /**
     * A flat [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of all the [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#sublayers) in the BuildingSublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#allSublayers)
     */
    readonly allSublayers: Collection<BuildingGroupSublayer | BuildingComponentSublayer>;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#elevationInfo)
     */
    elevationInfo: BuildingSceneLayerElevationInfo;
    /**
     * Collection of filters that can be used to show or hide specific features in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#filters)
     */
    filters: Collection<BuildingFilter>;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * An array of field names from the service to include with each feature in all sublayers.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Hierarchical structure of sublayers in a BuildingSceneLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#sublayers)
     */
    readonly sublayers: Collection<BuildingGroupSublayer | BuildingComponentSublayer>;
    /**
     * Summary statistics for all component layers in the building scene layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#summaryStatistics)
     */
    readonly summaryStatistics: BuildingSummaryStatistics;

    readonly type: "building-scene";

    /**
     * Loads the layer and all of its sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#loadAll)
     */
    loadAll(): Promise<BuildingSceneLayer>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem, options?: BuildingSceneLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): BuildingSceneLayer;
  }

  interface BuildingSceneLayerProperties
    extends LayerProperties,
      SceneServiceProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties {
    /**
     * The id of the currently active filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#activeFilterId)
     */
    activeFilterId?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#elevationInfo)
     */
    elevationInfo?: BuildingSceneLayerElevationInfo;
    /**
     * Collection of filters that can be used to show or hide specific features in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#filters)
     */
    filters?: CollectionProperties<BuildingFilterProperties>;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * An array of field names from the service to include with each feature in all sublayers.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#outFields)
     */
    outFields?: string[];
  }

  export interface BuildingSceneLayerElevationInfo {
    /**
     * Defines how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#elevationInfo)
     */
    mode: "absolute-height";
    /**
     * An elevation offset, which is added to the vertical position of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * The unit for `offset` values.
     *
     * @default meters
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface BuildingSceneLayerSaveAsOptions {
    /**
     * the folder where to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#saveAs)
     */
    folder?: PortalFolder;
  }

  export interface BuildingComponentSublayer extends BuildingSublayer, Loadable {}

  export class BuildingComponentSublayer {
    /**
     * BuildingComponentSublayers contain 3D Object features representing building components like doors, pipes or AC units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html)
     */

    constructor(properties?: BuildingComponentSublayerProperties);

    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#listMode)
     */
    listMode: "show" | "hide";
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#renderer)
     */
    renderer: Renderer;

    readonly type: "building-component";

    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates a query object that can be used to fetch features that satisfy the component sublayer's current [definition expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#definitionExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: BuildingComponentSublayerGetFieldDomainOptions): Domain;
    /**
     * Gets field usage information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldUsageInfo)
     */
    getFieldUsageInfo(fieldName: string): any;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the 2D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the 2D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query. At the moment the 3D Extent can be returned by using [BuildingComponentSublayerView.queryExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryExtent), but this will return the 3D extent only for features currently in the view. The query succeeds only if the BuildingSceneLayer has an associated feature layer. If an associated feature layer is not available, then an error with the name `buildingscenelayer:query-not-available` is thrown. Read more about queries in the Querying section of the BuildingSceneLayer class description. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: BuildingComponentSublayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the number of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the number of features that satisfy the query. The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `buildingscenelayer:query-not-available` is thrown. Read more about queries in the Querying section of the BuildingSceneLayer class description. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html). The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `buildingscenelayer:query-not-available` is thrown. Read more about queries in the Querying section of the BuildingSceneLayer class description. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns an array of ObjectIDs of the features that satisfy the input query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns an array of ObjectIDs of the features that satisfy the input query. The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `buildingscenelayer:query-not-available` is thrown. Read more about queries in the Querying section of the SceneLayer class description. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerQueryObjectIdsOptions,
    ): Promise<number[]>;

    static fromJSON(json: any): BuildingComponentSublayer;
  }

  interface BuildingComponentSublayerProperties extends BuildingSublayerProperties, LoadableProperties {
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#listMode)
     */
    listMode?: "show" | "hide";
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#renderer)
     */
    renderer?: RendererProperties;
  }

  export interface BuildingComponentSublayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface BuildingComponentSublayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export class BuildingGroupSublayer extends BuildingSublayer {
    /**
     * Hierarchical group of [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingGroupSublayer.html)
     */

    constructor(properties?: BuildingGroupSublayerProperties);

    /**
     * Indicates how the sublayer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingGroupSublayer.html#listMode)
     */
    listMode: "show" | "hide" | "hide-children";
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingGroupSublayer.html#sublayers)
     */
    readonly sublayers: Collection<BuildingGroupSublayer | BuildingComponentSublayer>;

    readonly type: "building-group";

    /**
     * Loads all contained sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingGroupSublayer.html#loadAll)
     */
    loadAll(): Promise<BuildingSceneLayer>;

    static fromJSON(json: any): BuildingGroupSublayer;
  }

  interface BuildingGroupSublayerProperties extends BuildingSublayerProperties {
    /**
     * Indicates how the sublayer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingGroupSublayer.html#listMode)
     */
    listMode?: "show" | "hide" | "hide-children";
  }

  export interface BuildingSublayer extends Accessor, Identifiable {}

  export class BuildingSublayer {
    /**
     * BuildingSublayer is the base class for sublayers of a BuildingSceneLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html)
     */

    constructor(properties?: BuildingSublayerProperties);

    /**
     * The sublayer's layer id as defined by the Scene Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#id)
     */
    readonly id: number;
    /**
     * Indicates if this sublayer is empty.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#isEmpty)
     */
    readonly isEmpty: boolean;
    /**
     * The modelName is a standard name for each sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#modelName)
     */
    readonly modelName: string;
    /**
     * Opacity of the sublayer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#opacity)
     */
    opacity: number;
    /**
     * The title of the sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#title)
     */
    title: string;
    /**
     * Indicates if the sublayer is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#visible)
     */
    visible: boolean;

    static fromJSON(json: any): BuildingSublayer;
  }

  interface BuildingSublayerProperties {
    /**
     * Opacity of the sublayer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#opacity)
     */
    opacity?: number;
    /**
     * The title of the sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#title)
     */
    title?: string;
    /**
     * Indicates if the sublayer is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingSublayer.html#visible)
     */
    visible?: boolean;
  }

  export interface CatalogDynamicGroupLayer extends Layer, BlendLayer, ScaleRangeLayer {}

  export class CatalogDynamicGroupLayer {
    /**
     * CatalogDynamicGroupLayer is a sublayer of the [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) that displays catalog items (layers) in the current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html)
     */

    constructor(properties?: CatalogDynamicGroupLayerProperties);

    /**
     * A collection of [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) referenced in the CatalogLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#layers)
     */
    readonly layers: Collection<Layer>;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The maximum number of sublayers that can be visible at once in a CatalogDynamicGroupLayer.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#maximumVisibleSublayers)
     */
    maximumVisibleSublayers: number;

    readonly type: "catalog-dynamic-group";
  }

  interface CatalogDynamicGroupLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      ScaleRangeLayerProperties {
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The maximum number of sublayers that can be visible at once in a CatalogDynamicGroupLayer.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#maximumVisibleSublayers)
     */
    maximumVisibleSublayers?: number;
  }

  export interface CatalogFootprintLayer extends Layer, BlendLayer, ScaleRangeLayer, FeatureEffectLayer, Clonable {}

  export class CatalogFootprintLayer {
    /**
     * CatalogFootprintLayer is a layer that represents the footprints of items referenced in a [catalog layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html)
     */

    constructor(properties?: CatalogFootprintLayerProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#apiKey)
     */
    readonly apiKey: string;
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#capabilities)
     */
    readonly capabilities: Capabilities;
    /**
     * Array of Chart Items of type WebMapWebChart available on the footprint layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#charts)
     */
    charts: any[];
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#customParameters)
     */
    readonly customParameters: any;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#dateFieldsTimeZone)
     */
    readonly dateFieldsTimeZone: string;
    /**
     * This property is set by the service publisher and indicates that dates should be considered without the local timezone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#datesInUnknownTimezone)
     */
    readonly datesInUnknownTimezone: any;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#definitionExpression)
     */
    readonly definitionExpression: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#displayField)
     */
    readonly displayField: string;
    /**
     * Describes effective capabilities of the layer taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#effectiveCapabilities)
     */
    readonly effectiveCapabilities: Capabilities;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    elevationInfo: CatalogFootprintLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#fields)
     */
    readonly fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * Provides information on the system maintained area and length fields along with their respective units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#geometryFieldsInfo)
     */
    readonly geometryFieldsInfo: GeometryFieldsInfo;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#geometryType)
     */
    readonly geometryType: "polygon";
    /**
     * Indicates whether the client-side features in the layer have `M` (measurement) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#hasM)
     */
    readonly hasM: boolean;
    /**
     * Indicates whether the client-side features in the layer have `Z` (elevation) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#hasZ)
     */
    readonly hasZ: boolean;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#objectIdField)
     */
    readonly objectIdField: string;
    /**
     * The rendering order of features in the view based on the CatalogLayer's [orderBy](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#orderBy) property.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#orderBy)
     */
    readonly orderBy: Accessor[];
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#outFields)
     */
    readonly outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The IANA time zone the author of the service intended data from date fields to be viewed in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#preferredTimeZone)
     */
    readonly preferredTimeZone: string;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * When `true`, indicates that m-values will be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#returnM)
     */
    readonly returnM: boolean;
    /**
     * When `true`, indicates that z-values will always be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#returnZ)
     */
    readonly returnZ: boolean;
    /**
     * The spatial reference the source data is stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The layer's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#timeExtent)
     */
    readonly timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#timeInfo)
     */
    readonly timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#timeOffset)
     */
    readonly timeOffset: TimeInterval;

    readonly type: "catalog-footprint";
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#useViewTime)
     */
    readonly useViewTime: boolean;

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#definitionExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: CatalogFootprintLayerGetFieldDomainOptions): Domain;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: CatalogFootprintLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: CatalogFootprintLayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: CatalogFootprintLayerQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: CatalogFootprintLayerQueryObjectIdsOptions,
    ): Promise<number[]>;
  }

  interface CatalogFootprintLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      ScaleRangeLayerProperties,
      FeatureEffectLayerProperties {
    /**
     * Array of Chart Items of type WebMapWebChart available on the footprint layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#charts)
     */
    charts?: any[];
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    elevationInfo?: CatalogFootprintLayerElevationInfo;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#renderer)
     */
    renderer?: RendererProperties;
  }

  export interface CatalogFootprintLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    featureExpressionInfo?: CatalogFootprintLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface CatalogFootprintLayerElevationInfoFeatureExpressionInfo {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface CatalogFootprintLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface CatalogFootprintLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface CatalogFootprintLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface CatalogFootprintLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface CatalogFootprintLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  /**
   * Provides utility functions for the [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-catalogUtils.html)
   */
  interface catalogUtils {
    /**
     * Utility method to get the parent [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) for a given layer in [CatalogDynamicGroupLayer.layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html#layers) collection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-catalogUtils.html#getCatalogLayerForLayer)
     */
    getCatalogLayerForLayer(layer: Layer): CatalogLayer;
    /**
     * Utility method to check if the layer is temporarily available in the map and can be removed at any time because it is part of a [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-catalogUtils.html#isLayerFromCatalog)
     */
    isLayerFromCatalog(layer: Layer): boolean;
  }

  export const catalogUtils: catalogUtils;

  export interface CatalogLayer
    extends Layer,
      APIKeyMixin,
      BlendLayer,
      FeatureLayerBase,
      CustomParametersMixin,
      OperationalLayer,
      OrderedLayer,
      PortalLayer,
      RefreshableLayer,
      ScaleRangeLayer,
      TemporalLayer,
      Clonable {}

  export class CatalogLayer {
    /**
     * CatalogLayer points to different portal items and services, helping you to better organize and manage your data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html)
     */

    constructor(properties?: CatalogLayerProperties);

    /**
     * The draw order field holds the value to sort catalog items (layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#drawOrderField)
     */
    readonly drawOrderField: "cd_draworder";
    /**
     * The dynamicGroupLayer includes the catalog items (layers) that are currently visible in your view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer)
     */
    readonly dynamicGroupLayer: CatalogDynamicGroupLayer;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#fields)
     */
    fields: Field[];
    /**
     * The footprint layer is a layer that displays footprints of items referenced in a CatalogLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#footprintLayer)
     */
    readonly footprintLayer: CatalogFootprintLayer;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#geometryType)
     */
    readonly geometryType: "polygon";
    /**
     * The item name field stores the name of the catalog item referenced in the CatalogLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#itemNameField)
     */
    readonly itemNameField: "cd_itemname";
    /**
     * The item source field stores the original source path of the catalog item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#itemSourceField)
     */
    readonly itemSourceField: "cd_itemsource";
    /**
     * The item type field stores the type of the catalog item, such as `Feature Service` or `Map Service`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#itemTypeField)
     */
    readonly itemTypeField: "cd_itemtype";
    /**
     * A collection of [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html) and [CatalogDynamicGroupLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogDynamicGroupLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#layers)
     */
    readonly layers: Collection<CatalogDynamicGroupLayer | CatalogFootprintLayer>;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The max scale field holds the maximum scale at which the catalog item is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#maxScaleField)
     */
    readonly maxScaleField: "cd_maxscale";
    /**
     * The min scale field holds the minimum scale at which the catalog item is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#minScaleField)
     */
    readonly minScaleField: "cd_minscale";
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#outFields)
     */
    outFields: string[];

    readonly type: "catalog";
    /**
     * The absolute URL of the REST endpoint of a catalog service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#url)
     */
    url: string;

    /**
     * Returns a footprint [graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) that represents a layer in the [dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#createFootprintFromLayer)
     */
    createFootprintFromLayer(layer: Layer): Graphic;
    /**
     * Creates a new instance of a layer for the given layer in the [dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer) based on its footprint feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#createLayerFromFootprint)
     */
    createLayerFromFootprint(footprint: Graphic): Promise<Layer>;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#definitionExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: CatalogLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: CatalogLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: CatalogLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: CatalogLayerQueryObjectIdsOptions): Promise<number[]>;

    on(name: "refresh", eventHandler: CatalogLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: CatalogLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: CatalogLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: CatalogLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): CatalogLayer;
  }

  interface CatalogLayerProperties
    extends LayerProperties,
      APIKeyMixinProperties,
      BlendLayerProperties,
      FeatureLayerBaseProperties,
      CustomParametersMixinProperties,
      OperationalLayerProperties,
      OrderedLayerProperties,
      PortalLayerProperties,
      RefreshableLayerProperties,
      ScaleRangeLayerProperties,
      TemporalLayerProperties {
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * The absolute URL of the REST endpoint of a catalog service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#url)
     */
    url?: string;
  }

  export interface CatalogLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface CatalogLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface CatalogLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface CatalogLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface CatalogLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface CatalogLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface CatalogLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface CatalogLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface CSVLayer
    extends Layer,
      OrderedLayer,
      ScaleRangeLayer,
      TemporalLayer,
      BlendLayer,
      FeatureEffectLayer,
      FeatureReductionLayer,
      PortalLayer {}

  export class CSVLayer {
    /**
     * The CSVLayer is a point layer based on a CSV file (.csv, .txt).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html)
     */

    constructor(properties?: CSVLayerProperties);

    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    readonly capabilities: CSVLayerCapabilities;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#copyright)
     */
    copyright: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#dateFieldsTimeZone)
     */
    readonly dateFieldsTimeZone: string;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The column delimiter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#delimiter)
     */
    delimiter: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    elevationInfo: CSVLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The geometry type of features in the CSVLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#geometryType)
     */
    geometryType: "point";
    /**
     * Returns `true` if the layer is loaded from a non-spatial table in a service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#isTable)
     */
    readonly isTable: boolean;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * The latitude field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#latitudeField)
     */
    latitudeField: string;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The longitude field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#longitudeField)
     */
    longitudeField: string;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * An array of field names to include in the CSVLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;

    readonly type: "csv";
    /**
     * The URL of the CSV file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#url)
     */
    url: string;

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameters that can be used to fetch features that satisfy the layer's current filters, and definitions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: CSVLayerGetFieldDomainOptions): Domain;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the CSV data and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: CSVLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the CSV data and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: CSVLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the CSV data and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: CSVLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the CSV data and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: CSVLayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#refresh)
     */
    refresh(): void;

    on(name: "refresh", eventHandler: CSVLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: CSVLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: CSVLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: CSVLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): CSVLayer;
  }

  interface CSVLayerProperties
    extends LayerProperties,
      OrderedLayerProperties,
      ScaleRangeLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      PortalLayerProperties {
    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#copyright)
     */
    copyright?: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The column delimiter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#delimiter)
     */
    delimiter?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    elevationInfo?: CSVLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the CSVLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#geometryType)
     */
    geometryType?: "point";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * The latitude field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#latitudeField)
     */
    latitudeField?: string;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The longitude field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#longitudeField)
     */
    longitudeField?: string;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * An array of field names to include in the CSVLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The URL of the CSV file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#url)
     */
    url?: string;
  }

  export interface CSVLayerCapabilities {
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    data: CSVLayerCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    editing: CSVLayerCapabilitiesEditing;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    operations: CSVLayerCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    query: CSVLayerCapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    queryRelated: CSVLayerCapabilitiesQueryRelated;
  }

  export interface CSVLayerCapabilitiesData {
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsAttachment: boolean;
    /**
     * Indicates if the features in the layer support m-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsM: boolean;
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsZ: boolean;
  }

  export interface CSVLayerCapabilitiesEditing {
    /**
     * Indicates if anonymous users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDeleteByAnonymous: boolean;
    /**
     * Indicates if logged in users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDeleteByOthers: boolean;
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalid` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailure` parameter can be set to `true` or `false` when running the synchronizeReplica operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsRollbackOnFailure: boolean;
    /**
     * Indicates if anonymous users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsUpdateByAnonymous: boolean;
    /**
     * Indicates if logged in users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsUpdateByOthers: boolean;
    /**
     * Indicates if `m-values` must be provided when updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsUpdateWithoutM: boolean;
    /**
     * Indicates if the layer supports uploading attachments by [UploadId](https://developers.arcgis.com/rest/services-reference/item.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsUploadWithItemId: boolean;
  }

  export interface CSVLayerCapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#applyEdits) to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#applyEdits) from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be [updated](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if features in the layer can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if values of one or more field values in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsCalculate: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsQueryAttachments: boolean;
    /**
     * Indicates if the layer supports a SQL-92 expression or where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsValidateSql: boolean;
    /**
     * Indicates if resized attachments are supported in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsResizeAttachments: boolean;
  }

  export interface CSVLayerCapabilitiesQuery {
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the query operation supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the query operation supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsHistoricMoment: boolean;
  }

  export interface CSVLayerCapabilitiesQueryRelated {
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#capabilities)
     */
    supportsPagination: boolean;
  }

  export interface CSVLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    featureExpressionInfo?: CSVLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface CSVLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface CSVLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface CSVLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface CSVLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface CSVLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface CSVLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface CSVLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface CSVLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface CSVLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface CSVLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface DimensionLayer extends Layer, OperationalLayer {}

  export class DimensionLayer {
    /**
     * The dimension layer displays measurement annotations of lengths and distances in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html)
     */

    constructor(properties?: DimensionLayerProperties);

    /**
     * The full extent of the layer which contains all dimensions referenced by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#fullExtent)
     */
    readonly fullExtent: Extent;
    /**
     * The [DimensionAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html) associated with the layer that stores the dimension shapes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#source)
     */
    source: DimensionAnalysis;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The style defines how the dimension objects of this layer are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#style)
     */
    style: DimensionSimpleStyle;

    readonly type: "dimension";

    static fromJSON(json: any): DimensionLayer;
  }

  interface DimensionLayerProperties extends LayerProperties, OperationalLayerProperties {
    /**
     * The [DimensionAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html) associated with the layer that stores the dimension shapes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#source)
     */
    source?: DimensionAnalysisProperties;
    /**
     * The style defines how the dimension objects of this layer are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#style)
     */
    style?: DimensionSimpleStyleProperties;
  }

  export interface ElevationLayer extends Layer, ArcGISCachedService, PortalLayer, OperationalLayer {}

  export class ElevationLayer {
    /**
     * ElevationLayer is a tile layer used for rendering elevations in [SceneViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html)
     */

    constructor(properties?: ElevationLayerProperties);

    /**
     * The [image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#sourceJSON)
     */
    sourceJSON: any;

    readonly type: "elevation";
    /**
     * URL pointing to the Elevation layer resource on an ArcGIS Image Server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#url)
     */
    url: string;

    /**
     * Creates an elevation sampler for the given [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) by querying the service layer for elevation data and caching it so values may be sampled quickly afterwards.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#createElevationSampler)
     */
    createElevationSampler(
      extent: Extent,
      options?: ElevationLayerCreateElevationSamplerOptions,
    ): Promise<ElevationSampler>;
    /**
     * Requests a tile from the service and decodes the data into a linear array of elevation values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#fetchTile)
     */
    fetchTile(
      level: number,
      row: number,
      column: number,
      options?: ElevationLayerFetchTileOptions,
    ): Promise<ElevationTileData>;
    /**
     * This method returns a URL to a tile for a given level, row and column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#getTileUrl)
     */
    getTileUrl(level: number, row: number, col: number): string;
    /**
     * Queries the service layer for elevation values for the given geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation)
     */
    queryElevation(
      geometry: Point | Multipoint | Polyline,
      options?: ElevationLayerQueryElevationOptions,
    ): Promise<ElevationLayerElevationQueryResult>;

    static fromJSON(json: any): ElevationLayer;
  }

  interface ElevationLayerProperties
    extends LayerProperties,
      ArcGISCachedServiceProperties,
      PortalLayerProperties,
      OperationalLayerProperties {
    /**
     * The [image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * URL pointing to the Elevation layer resource on an ArcGIS Image Server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#url)
     */
    url?: string;
  }

  export interface ElevationLayerCreateElevationSamplerOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#createElevationSampler)
     */
    demResolution?: number | string;
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#createElevationSampler)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#createElevationSampler)
     */
    signal?: AbortSignal;
  }

  export interface ElevationLayerFetchTileOptions {
    /**
     * The value representing pixels in the tile that don't contain an elevation value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#fetchTile)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface ElevationLayerQueryElevationOptions {
    /**
     * Controls the horizontal resolution (cell size) in meters from which elevation data is sampled (defaults to `auto`).
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation)
     */
    demResolution?: number | string;
    /**
     * Indicates whether to return additional sample information for each coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation)
     */
    returnSampleInfo?: boolean;
    /**
     * The value to use when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation)
     */
    noDataValue?: number;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation)
     */
    signal?: AbortSignal;
  }

  /**
   * Object returned when [queryElevation()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#queryElevation) promise resolves:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#ElevationQueryResult)
   */
  export interface ElevationLayerElevationQueryResult {
    /**
     * The geometry with sampled z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#ElevationQueryResult)
     */
    geometry: Point | Multipoint | Polyline;
    /**
     * Contains additional information about how the geometry was sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#ElevationQueryResult)
     */
    sampleInfo?: ElevationLayerElevationQueryResultSampleInfo[];
    /**
     * The value used when there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#ElevationQueryResult)
     */
    noDataValue: number;
  }

  export interface ElevationLayerElevationQueryResultSampleInfo {
    /**
     * The resolution at which the z-value was sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html#ElevationQueryResult)
     */
    demResolution: number;
  }

  export interface FeatureLayer
    extends Layer,
      OrderedLayer,
      PortalLayer,
      ScaleRangeLayer,
      TemporalLayer,
      BlendLayer,
      CustomParametersMixin,
      APIKeyMixin,
      FeatureEffectLayer,
      FeatureReductionLayer,
      PublishableLayer,
      OperationalLayer,
      Clonable,
      FeatureLayerBase {}

  export class FeatureLayer {
    /**
     * A FeatureLayer is a single layer that can be created from a [Map Service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-map-service.htm) or [Feature Service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-feature-service-.htm); ArcGIS Online or ArcGIS Enterprise portal items; or from an array of client-side features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html)
     */

    constructor(properties?: FeatureLayerProperties);

    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Array of Chart Items of type WebMapWebChart available on the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#charts)
     */
    charts: any[];
    /**
     * An object that allows you to create a dynamic layer with data either from map service sublayers or data from a registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#dynamicDataSource)
     */
    dynamicDataSource: DynamicMapLayer | DynamicDataLayer;
    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields)
     */
    fields: Field[];
    /**
     * The [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used in an associated layer's [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#formTemplate)
     */
    formTemplate: FormTemplate;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * A collection of [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) objects used to create a FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#source)
     */
    source: Collection<Graphic>;
    /**
     * An array of feature templates defined in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#templates)
     */
    templates: FeatureTemplate[];

    readonly type: "feature";
    /**
     * The name of the field holding the type ID for the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#typeIdField)
     */
    typeIdField: string;
    /**
     * An array of [types](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html) defined in the feature service exposed by ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#types)
     */
    types: FeatureType[];

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#definitionExpression), [gdbVersion](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#gdbVersion), and [historicMoment](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#historicMoment).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns a [FeatureType](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html) describing the feature's type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#getFeatureType)
     */
    getFeatureType(feature: Graphic): FeatureType;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: FeatureLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: FeatureLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: FeatureLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: FeatureLayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns the count of features or records that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeatureCount)
     */
    queryTopFeatureCount(
      topFeaturesQuery: TopFeaturesQuery | TopFeaturesQueryProperties,
      options?: FeatureLayerQueryTopFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeatures)
     */
    queryTopFeatures(
      topFeaturesQuery: TopFeaturesQuery | TopFeaturesQueryProperties,
      options?: FeatureLayerQueryTopFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeaturesExtent)
     */
    queryTopFeaturesExtent(
      topFeaturesQuery: TopFeaturesQuery | TopFeaturesQueryProperties,
      options?: FeatureLayerQueryTopFeaturesExtentOptions,
    ): Promise<any>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns an array of Object IDs of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopObjectIds)
     */
    queryTopObjectIds(
      topFeaturesQuery: TopFeaturesQuery | TopFeaturesQueryProperties,
      options?: FeatureLayerQueryTopObjectIdsOptions,
    ): Promise<number[]>;
    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#refresh)
     */
    refresh(): void;

    on(name: "refresh", eventHandler: FeatureLayerRefreshEventHandler): IHandle;

    on(name: "edits", eventHandler: FeatureLayerEditsEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: FeatureLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: FeatureLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: FeatureLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): FeatureLayer;
  }

  interface FeatureLayerProperties
    extends LayerProperties,
      OrderedLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      PublishableLayerProperties,
      OperationalLayerProperties,
      FeatureLayerBaseProperties {
    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Array of Chart Items of type WebMapWebChart available on the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#charts)
     */
    charts?: any[];
    /**
     * An object that allows you to create a dynamic layer with data either from map service sublayers or data from a registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#dynamicDataSource)
     */
    dynamicDataSource?: (DynamicMapLayer & { type: "map-layer" }) | (DynamicDataLayer & { type: "data-layer" });
    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used in an associated layer's [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#formTemplate)
     */
    formTemplate?: FormTemplateProperties;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * A collection of [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) objects used to create a FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#source)
     */
    source?: CollectionProperties<GraphicProperties>;
    /**
     * An array of feature templates defined in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#templates)
     */
    templates?: FeatureTemplateProperties[];
    /**
     * The name of the field holding the type ID for the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#typeIdField)
     */
    typeIdField?: string;
    /**
     * An array of [types](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html) defined in the feature service exposed by ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#types)
     */
    types?: FeatureTypeProperties[];
  }

  /**
   * AttachmentEdit represents an attachment that can be added, updated or deleted via [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
   */
  export interface AttachmentEdit {
    /**
     * The feature, `objectId` or `globalId` of feature associated with the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    feature: Graphic | number | string;
    /**
     * The attachment to be added, updated or deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    attachment: AttachmentEditAttachment;
  }

  /**
   * Describes the layer's supported capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
   */
  export interface Capabilities {
    /**
     * Describes what analytics capabilities are enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    analytics: CapabilitiesAnalytics;
    /**
     * Describes what attachment capabilities are enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    attachment: CapabilitiesAttachment;
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    data: CapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer via [applyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    editing: CapabilitiesEditing;
    /**
     * Describes the metadata contained on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    metadata: CapabilitiesMetadata;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    operations: CapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    query: CapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    queryRelated: CapabilitiesQueryRelated;
    /**
     * Describes [top features query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    queryTopFeatures: CapabilitiesQueryTopFeatures;
  }

  /**
   * Results returned from the [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits) method if the `returnServiceEditsOption` parameter is set to `original-and-current-features`.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
   */
  export interface EditedFeatureResult {
    /**
     * The layerId of the feature layer where features were edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    layerId: number;
    /**
     * Object containing all edited features belonging to the specified layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    editedFeatures: EditedFeatureResultEditedFeatures;
  }

  /**
   * The fields that record who adds or edits data in the feature service and when the edit is made.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditFieldsInfo)
   */
  export interface EditFieldsInfo {
    /**
     * The name of the field that stores the name of the user who created the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditFieldsInfo)
     */
    creatorField: string;
    /**
     * The name of the field that stores the date and time the feature was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditFieldsInfo)
     */
    creationDateField: string;
    /**
     * The name of the field that stores the name of the user who last edited the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditFieldsInfo)
     */
    editorField: string;
    /**
     * The name of the field that stores the date and time the feature was last edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditFieldsInfo)
     */
    editDateField: string;
  }

  /**
   * Specifies information about editing.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditingInfo)
   */
  export interface EditingInfo {
    /**
     * Indicates the last time the layer was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditingInfo)
     */
    lastEditDate: Date;
  }

  export interface FeatureLayerEditsEvent {
    addedAttachments: FeatureEditResult[];

    addedFeatures: FeatureEditResult[];

    deletedAttachments: FeatureEditResult[];

    deletedFeatures: FeatureEditResult[];

    editedFeatures: EditedFeatureResult;

    exceededTransferLimit: boolean;

    updatedAttachments: FeatureEditResult[];

    updatedFeatures: FeatureEditResult[];
  }

  /**
   * Results returned from the [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
   */
  export interface EditsResult {
    /**
     * Result of adding features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    addFeatureResults: FeatureEditResult[];
    /**
     * Result of updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    updateFeatureResults: FeatureEditResult[];
    /**
     * Result of deleting features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    deleteFeatureResults: FeatureEditResult[];
    /**
     * Result of adding attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    addAttachmentResults: FeatureEditResult[];
    /**
     * Result of updating attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    updateAttachmentResults: FeatureEditResult[];
    /**
     * Result of deleting attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    deleteAttachmentResults: FeatureEditResult[];
    /**
     * Edited features as result of editing a feature that participates in composite relationships in a database.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    editedFeatureResults?: EditedFeatureResult[];
    /**
     * The time edits were applied to the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditsResult)
     */
    editMoment?: number;
  }

  /**
   * FeatureEditResult represents the result of adding, updating or deleting a feature or an attachment.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
   */
  export interface FeatureEditResult {
    /**
     * The objectId of the feature or the attachmentId of the attachment that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
     */
    objectId: number;
    /**
     * The globalId of the feature or the attachment that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
     */
    globalId: string;
    /**
     * If the edit failed, the edit result includes an error.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
     */
    error: FeatureEditResultError;
  }

  export interface FeatureLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryTopFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryTopFeaturesExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeaturesExtent)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryTopFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeatures)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerQueryTopObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface FeatureLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface FeatureLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface FeatureLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface AttachmentEditAttachment {
    /**
     * The globalId of the attachment to be added or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    globalId: string;
    /**
     * The name of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    name?: string;
    /**
     * The content type of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    contentType?: string;
    /**
     * The id of pre-loaded attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    uploadId?: string;
    /**
     * The attachment data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#AttachmentEdit)
     */
    data?: Blob | File | string;
  }

  export interface CapabilitiesAnalytics {
    /**
     * Indicates if the feature service supports cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
  }

  export interface CapabilitiesAttachment {
    /**
     * Indicates if the [attachment operations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) support a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their content types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsContentType: boolean;
    /**
     * Indicates if the attachment [queries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) support `exifInfo`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsExifInfo: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their keywords.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsKeywords: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsName: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsSize: boolean;
    /**
     * Indicates if resized attachments are supported in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsResize: boolean;
  }

  export interface CapabilitiesData {
    /**
     * Indicates if the feature service is versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    isVersioned: boolean;
    /**
     * Indicates if the feature service is branch versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    isBranchVersioned: boolean;
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsAttachment: boolean;
    /**
     * Indicates if the features in the layer support m-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsM: boolean;
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsZ: boolean;
  }

  export interface CapabilitiesEditing {
    /**
     * Indicates if anonymous users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDeleteByAnonymous: boolean;
    /**
     * Indicates if logged in users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDeleteByOthers: boolean;
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalId` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailureEnabled` parameter can be set to `true` or `false` when editing features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsRollbackOnFailure: boolean;
    /**
     * Indicates if anonymous users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsUpdateByAnonymous: boolean;
    /**
     * Indicates if logged in users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsUpdateByOthers: boolean;
    /**
     * Indicates if the layer supports uploading attachments by [UploadId](https://developers.arcgis.com/rest/services-reference/item.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsUploadWithItemId: boolean;
    /**
     * Indicates if `m-values` must be provided when updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsUpdateWithoutM: boolean;
  }

  export interface CapabilitiesMetadata {
    /**
     * Indicates whether to provide a user-defined field description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsAdvancedFieldProperties: boolean;
  }

  export interface CapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits) to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if values of one or more field values in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCalculate: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits) from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQueryAttachments: boolean;
    /**
     * Deprecated since 4.24.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsResizeAttachments: boolean;
    /**
     * Indicates if the layer supports [REST API queryTopFeatures](https://developers.arcgis.com/rest/services-reference/enterprise/query-top-features-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQueryTopFeatures: boolean;
    /**
     * Indicates if features in the layer can be [updated](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if the layer supports a SQL-92 expression or where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsValidateSql: boolean;
  }

  export interface CapabilitiesQuery {
    /**
     * The maximum number of records that will be returned for a given query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    maxRecordCount: number;
    /**
     * Indicates if the query operation supports a [cache hint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#cacheHint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsHistoricMoment: boolean;
    /**
     * Indicates if the layer supports [Query.maxRecordCountFactor](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#maxRecordCountFactor) on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsMaxRecordCountFactor: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the layer supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the layer supports spatial extent, center or convex hull to be returned for each distinct group when [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsSpatialAggregationStatistics: boolean;
    /**
     * List of supported aggregated geometries returned for each distinct group when [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportedSpatialAggregationStatistics: CapabilitiesQuerySupportedSpatialAggregationStatistics;
  }

  export interface CapabilitiesQueryRelated {
    /**
     * Indicates if the [relationship query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) supports a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsPagination: boolean;
  }

  export interface CapabilitiesQuerySupportedSpatialAggregationStatistics {
    /**
     * Indicates if the layer can return centroid for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    centroid: boolean;
    /**
     * Indicates if the layer can return extent for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    envelope: boolean;
    /**
     * Indicates if the layer can return convex hull for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    convexHull: boolean;
  }

  export interface CapabilitiesQueryTopFeatures {
    /**
     * Indicates if the [top query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) supports a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
  }

  export interface EditedFeatureResultEditedFeatures {
    /**
     * Newly added features as a result of editing a feature that participates in a composite relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    adds?: Graphic[];
    /**
     * Object containing original and updated features as a result of editing a feature that participates in a composite relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    updates?: EditedFeatureResultEditedFeaturesUpdates[];
    /**
     * Deleted features as a result of editing a feature that participates in a composite relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    deletes?: Graphic[];
    /**
     * Edited features are returned in the spatial reference of the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    spatialReference?: SpatialReference;
  }

  export interface EditedFeatureResultEditedFeaturesUpdates {
    /**
     * Original feature before the update took place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    original?: Graphic[];
    /**
     * Updated feature as a result of editing a feature that participates in a composite relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult)
     */
    current?: Graphic[];
  }

  export interface FeatureEditResultError {
    /**
     * Error name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
     */
    name: string;
    /**
     * Message describing the error.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#FeatureEditResult)
     */
    message: string;
  }

  export interface GeoJSONLayer
    extends Layer,
      BlendLayer,
      OrderedLayer,
      ScaleRangeLayer,
      FeatureEffectLayer,
      FeatureReductionLayer,
      OperationalLayer {}

  export class GeoJSONLayer {
    /**
     * The GeoJSONLayer class is used to create a layer based on [GeoJSON](http://geojson.org/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html)
     */

    constructor(properties?: GeoJSONLayerProperties);

    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    readonly capabilities: GeoJSONLayerCapabilities;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#copyright)
     */
    copyright: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#dateFieldsTimeZone)
     */
    readonly dateFieldsTimeZone: string;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#displayField)
     */
    displayField: string;
    /**
     * Indicates if the layer is editable.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    elevationInfo: GeoJSONLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#geometryType)
     */
    geometryType: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * Indicates whether the client-side features in the layer have `Z` (elevation) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#hasZ)
     */
    readonly hasZ: boolean;
    /**
     * Returns `true` if the layer is loaded from a non-spatial table in a service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#isTable)
     */
    readonly isTable: boolean;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The name of a [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * An array of field names from the geoJSON file to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The portal item referencing the geojson file from which the GeoJSONLayer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * An array of feature templates defined in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#templates)
     */
    templates: FeatureTemplate[];
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeInfo)
     */
    timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeOffset)
     */
    timeOffset: TimeInterval;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * @default "GeoJSON"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#title)
     */
    title: string;

    readonly type: "geojson";
    /**
     * The URL of the GeoJSON file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#url)
     */
    url: string;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#useViewTime)
     */
    useViewTime: boolean;

    /**
     * Applies edits to features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits)
     */
    applyEdits(edits: GeoJSONLayerApplyEditsEdits): Promise<any>;
    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#definitionExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the Domain associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: GeoJSONLayerGetFieldDomainOptions): Domain;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: GeoJSONLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: GeoJSONLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: GeoJSONLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: GeoJSONLayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#refresh)
     */
    refresh(): void;

    on(name: "refresh", eventHandler: GeoJSONLayerRefreshEventHandler): IHandle;

    on(name: "edits", eventHandler: GeoJSONLayerEditsEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: GeoJSONLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: GeoJSONLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: GeoJSONLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): GeoJSONLayer;
  }

  interface GeoJSONLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      OrderedLayerProperties,
      ScaleRangeLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      OperationalLayerProperties {
    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#copyright)
     */
    copyright?: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Indicates if the layer is editable.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    elevationInfo?: GeoJSONLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#geometryType)
     */
    geometryType?: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The name of a [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * An array of field names from the geoJSON file to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The portal item referencing the geojson file from which the GeoJSONLayer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * An array of feature templates defined in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#templates)
     */
    templates?: FeatureTemplateProperties[];
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeInfo)
     */
    timeInfo?: TimeInfoProperties;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#timeOffset)
     */
    timeOffset?: TimeIntervalProperties;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * @default "GeoJSON"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#title)
     */
    title?: string;
    /**
     * The URL of the GeoJSON file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#url)
     */
    url?: string;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#useViewTime)
     */
    useViewTime?: boolean;
  }

  export interface GeoJSONLayerEditsEvent {
    addedFeatures: GeoJSONLayerEditsEventAddedFeatures[];

    deletedFeatures: GeoJSONLayerEditsEventDeletedFeatures[];

    updatedFeatures: GeoJSONLayerEditsEventUpdatedFeatures[];
  }

  export interface GeoJSONLayerApplyEditsEdits {
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits)
     */
    addFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits)
     */
    updateFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features, or objects to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits)
     */
    deleteFeatures?: Graphic[] | any[] | Collection<Graphic>;
  }

  export interface GeoJSONLayerCapabilities {
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    data: GeoJSONLayerCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    editing: GeoJSONLayerCapabilitiesEditing;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    operations: GeoJSONLayerCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    query: GeoJSONLayerCapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    queryRelated: GeoJSONLayerCapabilitiesQueryRelated;
  }

  export interface GeoJSONLayerCapabilitiesData {
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsAttachment: boolean;
    /**
     * Indicates if the features in the layer support m-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsM: boolean;
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsZ: boolean;
  }

  export interface GeoJSONLayerCapabilitiesEditing {
    /**
     * Indicates if anonymous users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDeleteByAnonymous: boolean;
    /**
     * Indicates if logged in users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDeleteByOthers: boolean;
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalid` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailure` parameter can be set to `true` or `false` when running the synchronizeReplica operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsRollbackOnFailure: boolean;
    /**
     * Indicates if anonymous users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsUpdateByAnonymous: boolean;
    /**
     * Indicates if logged in users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsUpdateByOthers: boolean;
    /**
     * Indicates if `m-values` must be provided when updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsUpdateWithoutM: boolean;
    /**
     * Indicates if the layer supports uploading attachments by [UploadId](https://developers.arcgis.com/rest/services-reference/item.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsUploadWithItemId: boolean;
  }

  export interface GeoJSONLayerCapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits) to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits) from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be [updated](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if features in the layer can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if values of one or more field values in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsCalculate: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsQueryAttachments: boolean;
    /**
     * Indicates if the layer supports a SQL-92 expression or where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsValidateSql: boolean;
    /**
     * Indicates if resized attachments are supported in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsResizeAttachments: boolean;
  }

  export interface GeoJSONLayerCapabilitiesQuery {
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the query operation supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the query operation supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsHistoricMoment: boolean;
  }

  export interface GeoJSONLayerCapabilitiesQueryRelated {
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#capabilities)
     */
    supportsPagination: boolean;
  }

  export interface GeoJSONLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    featureExpressionInfo?: GeoJSONLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface GeoJSONLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface GeoJSONLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface GeoJSONLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface GeoJSONLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface GeoJSONLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface GeoJSONLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface GeoJSONLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface GeoJSONLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface GeoJSONLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface GeoJSONLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface GeoJSONLayerEditsEventAddedFeatures {
    objectId: number;
  }

  export interface GeoJSONLayerEditsEventDeletedFeatures {
    objectId: number;
  }

  export interface GeoJSONLayerEditsEventUpdatedFeatures {
    objectId: number;
  }

  export interface GeoRSSLayer extends Layer, ScaleRangeLayer, BlendLayer, OperationalLayer {}

  export class GeoRSSLayer {
    /**
     * The GeoRSSLayer class is used to create a layer based on [GeoRSS](https://www.ogc.org/standards/georss).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html)
     */

    constructor(properties?: GeoRSSLayerProperties);

    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Symbol used to represent line features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#lineSymbol)
     */
    lineSymbol: SimpleLineSymbol;
    /**
     * Symbol used to represent point features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#pointSymbol)
     */
    pointSymbol: PictureMarkerSymbol | SimpleMarkerSymbol;
    /**
     * Symbol used to represent polygon features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#polygonSymbol)
     */
    polygonSymbol: SimpleFillSymbol;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#title)
     */
    title: string;

    readonly type: "geo-rss";
    /**
     * The URL pointing to a GeoRSS file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#url)
     */
    url: string;

    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#refresh)
     */
    refresh(): void;

    on(name: "refresh", eventHandler: GeoRSSLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: GeoRSSLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: GeoRSSLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: GeoRSSLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): GeoRSSLayer;
  }

  interface GeoRSSLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties {
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Symbol used to represent line features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#lineSymbol)
     */
    lineSymbol?: SimpleLineSymbolProperties;
    /**
     * Symbol used to represent point features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#pointSymbol)
     */
    pointSymbol?:
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" });
    /**
     * Symbol used to represent polygon features from the GeoRSS feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#polygonSymbol)
     */
    polygonSymbol?: SimpleFillSymbolProperties;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#title)
     */
    title?: string;
    /**
     * The URL pointing to a GeoRSS file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html#url)
     */
    url?: string;
  }

  export interface GeoRSSLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface GeoRSSLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface GeoRSSLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface GeoRSSLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface GraphicsLayer extends Layer, ScaleRangeLayer, BlendLayer {}

  export class GraphicsLayer {
    /**
     * A GraphicsLayer contains one or more client-side [Graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html)
     */

    constructor(properties?: GraphicsLayerProperties);

    /**
     * Specifies how graphics are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    elevationInfo: GraphicsLayerElevationInfo;
    /**
     * A collection of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#graphics)
     */
    graphics: Collection<Graphic>;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;

    readonly type: "graphics";

    /**
     * Adds a graphic to the layer's graphic collection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#add)
     */
    add(graphic: Graphic): void;
    /**
     * Adds an array of graphics to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#addMany)
     */
    addMany(graphics: Graphic[]): void;
    /**
     * Removes a graphic from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#remove)
     */
    remove(graphic: Graphic): void;
    /**
     * Clears all the graphics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#removeAll)
     */
    removeAll(): void;
    /**
     * Removes an array of graphics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#removeMany)
     */
    removeMany(graphics: Graphic[]): void;
  }

  interface GraphicsLayerProperties extends LayerProperties, ScaleRangeLayerProperties, BlendLayerProperties {
    /**
     * Specifies how graphics are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    elevationInfo?: GraphicsLayerElevationInfo;
    /**
     * A collection of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#graphics)
     */
    graphics?: CollectionProperties<GraphicProperties>;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
  }

  export interface GraphicsLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    featureExpressionInfo?: GraphicsLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface GraphicsLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface GroupLayer extends Layer, LayersMixin, TablesMixin, PortalLayer, BlendLayer, OperationalLayer {}

  export class GroupLayer {
    /**
     * GroupLayer provides the ability to organize several sublayers into one common layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html)
     */

    constructor(properties?: GroupLayerProperties);

    /**
     * A flattened collection of all [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) in the group layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#allLayers)
     */
    readonly allLayers: Collection<Layer>;
    /**
     * A flattened collection of tables anywhere in the group layer's hierarchy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#allTables)
     */
    readonly allTables: Collection<Layer>;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#minScale)
     */
    minScale: number;

    readonly type: "group";
    /**
     * Indicates how to manage the visibility of the children layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#visibilityMode)
     */
    visibilityMode: "independent" | "inherited" | "exclusive";

    /**
     * Loads all the externally loadable resources associated with the group layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#loadAll)
     */
    loadAll(): Promise<GroupLayer>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#save)
     */
    save(options?: GroupLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: GroupLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): GroupLayer;
  }

  interface GroupLayerProperties
    extends LayerProperties,
      LayersMixinProperties,
      TablesMixinProperties,
      PortalLayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties {
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#minScale)
     */
    minScale?: number;
    /**
     * Indicates how to manage the visibility of the children layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#visibilityMode)
     */
    visibilityMode?: "independent" | "inherited" | "exclusive";
  }

  export interface GroupLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#saveAs)
     */
    validationOptions?: GroupLayerSaveAsOptionsValidationOptions;
  }

  export interface GroupLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface GroupLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#save)
     */
    validationOptions?: GroupLayerSaveOptionsValidationOptions;
  }

  export interface GroupLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export interface ImageryLayer
    extends Layer,
      ArcGISImageService,
      OperationalLayer,
      PortalLayer,
      RasterPresetRendererMixin,
      RefreshableLayer,
      ScaleRangeLayer,
      TemporalLayer,
      BlendLayer,
      Clonable {}

  export class ImageryLayer {
    /**
     * Represents a dynamic [image service resource](https://developers.arcgis.com/rest/services-reference/image-service.htm) as a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html)
     */

    constructor(properties?: ImageryLayerProperties);

    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;

    readonly type: "imagery";

    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Executes the [pixelFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#pixelFilter) function and redraws the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#redraw)
     */
    redraw(): void;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#save)
     */
    save(options?: ImageryLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: ImageryLayerSaveAsOptions): Promise<PortalItem>;

    on(name: "refresh", eventHandler: ImageryLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: ImageryLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: ImageryLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: ImageryLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): ImageryLayer;
  }

  interface ImageryLayerProperties
    extends LayerProperties,
      ArcGISImageServiceProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      RasterPresetRendererMixinProperties,
      RefreshableLayerProperties,
      ScaleRangeLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties {
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
  }

  export interface ImageryLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#saveAs)
     */
    validationOptions?: ImageryLayerSaveAsOptionsValidationOptions;
  }

  export interface ImageryLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface ImageryLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#save)
     */
    validationOptions?: ImageryLayerSaveOptionsValidationOptions;
  }

  export interface ImageryLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export interface ImageryLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface ImageryLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface ImageryLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  /**
   * An object containing measurement values returned from a mensuration operation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurementValue)
   */
  export interface MeasurementValue {
    /**
     * Measured value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurementValue)
     */
    value: number;
    /**
     * Uncertainty of the measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurementValue)
     */
    uncertainty: number;
    /**
     * Unit used in the mensuration operation.
     *
     * [Read more...](global.html#unit)
     */
    unit: string;
    /**
     * Measured value represented in significant figures in string format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurementValue)
     */
    displayValue: string;
  }

  /**
   * An object that provides the user access to [pixels](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels) and their values in the layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#PixelData)
   */
  export interface PixelData {
    /**
     * The extent of the `pixelBlock`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#PixelData)
     */
    extent: Extent;
    /**
     * An object representing the pixels in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#PixelData)
     */
    pixelBlock: PixelBlock;
  }

  export type PixelFilterFunction = (pixelData: PixelData) => void;

  /**
   * Raster statistics information returned that meets the specified [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html) from the `computeStatisticsHistograms()` method on [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeStatisticsHistograms) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#computeStatisticsHistograms).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
   */
  export interface RasterBandStatistics {
    /**
     * Minimum value of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    min: number;
    /**
     * Maximum value of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    max: number;
    /**
     * Average of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    avg?: number;
    /**
     * Standard deviation of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    stddev?: number;
    /**
     * Count of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    count?: number;
    /**
     * Mode value of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    mode?: number;
    /**
     * Median value of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    median?: number;
    /**
     * Sum of the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics)
     */
    sum?: number;
  }

  /**
   * The raster function published with the image service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
   */
  export interface RasterFunctionInfo {
    /**
     * Name of the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
     */
    name: string;
    /**
     * Description of the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
     */
    description: string;
    /**
     * Additional info describing what the raster function does.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
     */
    help: string;
    /**
     * The function type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
     */
    functionType: number;
    /**
     * Inline base64 encoded thumbnail of the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterFunctionInfo)
     */
    thumbnail: string;
  }

  /**
   * Raster histogram information returned that meets the specified [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html) from the [computeHistograms()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeHistograms) or [computeStatisticsHistograms()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeStatisticsHistograms) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram)
   */
  export interface RasterHistogram {
    /**
     * Number of bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram)
     */
    size: number;
    /**
     * The minimum pixel value of the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram)
     */
    min: number;
    /**
     * The maximum pixel value of the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram)
     */
    max: number;
    /**
     * Count of pixels that fall into each bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram)
     */
    counts: number[] | Uint32Array;
  }

  export interface ImageryLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface ImageryTileLayer
    extends Layer,
      ImageryTileMixin,
      OperationalLayer,
      PortalLayer,
      RasterPresetRendererMixin,
      ScaleRangeLayer,
      BlendLayer,
      Clonable {}

  export class ImageryTileLayer {
    /**
     * ImageryTileLayer presents raster data from a tiled image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html)
     */

    constructor(properties?: ImageryTileLayerProperties);

    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * A complete list of fields that consists of service pixel value and vector fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterFields)
     */
    readonly rasterFields: Field[];
    /**
     * The client-side raster functions are operations that apply processing directly to the source image pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterFunction)
     */
    rasterFunction: RasterFunction;
    /**
     * The client-side renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#renderer)
     */
    renderer:
      | ClassBreaksRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | RasterShadedReliefRenderer
      | RasterColormapRenderer
      | VectorFieldRenderer
      | FlowRenderer;
    /**
     * The data source for client-side ImageryTileLayer can be a [coverage JSON](https://www.ogc.org/standard/coveragejson/) object or [PixelData](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#PixelData).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#source)
     */
    source: any | PixelData;
    /**
     * The [tiled image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#tileInfo)
     */
    tileInfo: TileInfo;

    readonly type: "imagery-tile";
    /**
     * The URL of the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#url)
     */
    url: string;
    /**
     * The version of ArcGIS Server in which the image service is published.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#version)
     */
    readonly version: number;

    /**
     * Computes [statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics) and [histograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram) for the provided [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#computeStatisticsHistograms)
     */
    computeStatisticsHistograms(
      parameters: ImageHistogramParameters | ImageHistogramParametersProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * This method fetches a tile for the given level, row and column present in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#fetchTile)
     */
    fetchTile(level: number, row: number, col: number, options?: ImageryTileLayerFetchTileOptions): Promise<any>;
    /**
     * Generates a new raster info for the specified client side raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#generateRasterInfo)
     */
    generateRasterInfo(
      rasterFunction: RasterFunction | RasterFunctionProperties,
      options?: ImageryTileLayerGenerateRasterInfoOptions,
    ): Promise<RasterInfo>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#save)
     */
    save(options?: ImageryTileLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: ImageryTileLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): ImageryTileLayer;
  }

  interface ImageryTileLayerProperties
    extends LayerProperties,
      ImageryTileMixinProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      RasterPresetRendererMixinProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties {
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The client-side raster functions are operations that apply processing directly to the source image pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterFunction)
     */
    rasterFunction?: RasterFunctionProperties;
    /**
     * The client-side renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#renderer)
     */
    renderer?:
      | (ClassBreaksRendererProperties & { type: "class-breaks" })
      | (UniqueValueRendererProperties & { type: "unique-value" })
      | (RasterStretchRendererProperties & { type: "raster-stretch" })
      | (RasterShadedReliefRendererProperties & { type: "raster-shaded-relief" })
      | (RasterColormapRendererProperties & { type: "raster-colormap" })
      | (VectorFieldRendererProperties & { type: "vector-field" })
      | (FlowRendererProperties & { type: "flow" });
    /**
     * The data source for client-side ImageryTileLayer can be a [coverage JSON](https://www.ogc.org/standard/coveragejson/) object or [PixelData](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#PixelData).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#source)
     */
    source?: any | PixelData;
    /**
     * The [tiled image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
    /**
     * The URL of the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#url)
     */
    url?: string;
  }

  export interface ImageryTileLayerFetchTileOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface ImageryTileLayerGenerateRasterInfoOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#generateRasterInfo)
     */
    signal?: AbortSignal;
  }

  export interface ImageryTileLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#saveAs)
     */
    validationOptions?: ImageryTileLayerSaveAsOptionsValidationOptions;
  }

  export interface ImageryTileLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface ImageryTileLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#save)
     */
    validationOptions?: ImageryTileLayerSaveOptionsValidationOptions;
  }

  export interface ImageryTileLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  /**
   * Additional options to set for `identify()` method on [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#identify) and [WCSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#identify).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyOptions)
   */
  export interface RasterIdentifyOptions {
    /**
     * A dimensional definition used for the `identify` operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyOptions)
     */
    multidimensionalDefinition?: DimensionalDefinition[];
    /**
     * Name of the transposed variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyOptions)
     */
    transposedVariableName?: string;
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyOptions)
     */
    signal?: AbortSignal;
  }

  /**
   * The result of `identify` operation on [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#identify) and [WCSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#identify).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult)
   */
  export interface RasterIdentifyResult {
    /**
     * The identified location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult)
     */
    location: Point;
    /**
     * Image service pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult)
     */
    value: number[];
    /**
     * The processed pixel values showing results of the [rasterFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterFunction) applied to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult)
     */
    processedValue: number[];
    /**
     * Transposed multidimensional data for the requested location and variable returned from a transposed multidimensional [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#identify).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult)
     */
    dataSeries?: RasterSliceValue[];
  }

  /**
   * Data series returned in [RasterIdentifyResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterIdentifyResult) when [identify()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#identify) is called on a transposed multidimensional ImageryTileLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterSliceValue)
   */
  export interface RasterSliceValue {
    /**
     * Image service pixel values for a given slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterSliceValue)
     */
    value: number[];
    /**
     * A dimensional definition associated with a given slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterSliceValue)
     */
    multidimensionalDefinition: DimensionalDefinition[];
    /**
     * The magnitude and direction values for services containing vector data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#RasterSliceValue)
     */
    magdirValue?: number[];
  }

  export interface IntegratedMesh3DTilesLayer
    extends Layer,
      PortalLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer,
      ScaleRangeLayer {}

  export class IntegratedMesh3DTilesLayer {
    /**
     * The IntegratedMesh3DTilesLayer is designed for visualizing accurate representations of built and natural environments in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html)
     */

    constructor(properties?: IntegratedMesh3DTilesLayerProperties);

    /**
     * Specifies how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#elevationInfo)
     */
    elevationInfo: IntegratedMesh3DTilesLayerElevationInfo;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;

    readonly type: "integrated-mesh-3dtiles";
    /**
     * The URL of the root json file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#url)
     */
    url: string;

    static fromJSON(json: any): IntegratedMesh3DTilesLayer;
  }

  interface IntegratedMesh3DTilesLayerProperties
    extends LayerProperties,
      PortalLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties,
      ScaleRangeLayerProperties {
    /**
     * Specifies how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#elevationInfo)
     */
    elevationInfo?: IntegratedMesh3DTilesLayerElevationInfo;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The URL of the root json file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#url)
     */
    url?: string;
  }

  export interface IntegratedMesh3DTilesLayerElevationInfo {
    /**
     * Defines how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#elevationInfo)
     */
    mode: "absolute-height";
    /**
     * An elevation offset, which is added to the vertical position of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMesh3DTilesLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * The unit for `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface IntegratedMeshLayer
    extends Layer,
      SceneService,
      PortalLayer,
      ScaleRangeLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer {}

  export class IntegratedMeshLayer {
    /**
     * The IntegratedMeshLayer is designed for visualizing accurate representations of built and natural environments in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html)
     */

    constructor(properties?: IntegratedMeshLayerProperties);

    /**
     * Specifies how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#elevationInfo)
     */
    elevationInfo: IntegratedMeshLayerElevationInfo;
    /**
     * A collection of [SceneModification](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html) with polygons and types to apply client-side modifications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#modifications)
     */
    modifications: SceneModifications;

    readonly type: "integrated-mesh";

    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem, options?: IntegratedMeshLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): IntegratedMeshLayer;
  }

  interface IntegratedMeshLayerProperties
    extends LayerProperties,
      SceneServiceProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties {
    /**
     * Specifies how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#elevationInfo)
     */
    elevationInfo?: IntegratedMeshLayerElevationInfo;
    /**
     * A collection of [SceneModification](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html) with polygons and types to apply client-side modifications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#modifications)
     */
    modifications?: SceneModificationsProperties;
  }

  export interface IntegratedMeshLayerElevationInfo {
    /**
     * Defines how the mesh is placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#elevationInfo)
     */
    mode: "absolute-height";
    /**
     * An elevation offset, which is added to the vertical position of the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * The unit for `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface IntegratedMeshLayerSaveAsOptions {
    /**
     * the folder where to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-IntegratedMeshLayer.html#saveAs)
     */
    folder?: PortalFolder;
  }

  export interface KMLLayer extends Layer, OperationalLayer, PortalLayer, ScaleRangeLayer, BlendLayer {}

  export class KMLLayer {
    /**
     * The KMLLayer class is used to create a layer based on a KML file (.kml, .kmz).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html)
     */

    constructor(properties?: KMLLayerProperties);

    /**
     * A collection of [KMLSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#sublayers)
     */
    sublayers: Collection<KMLSublayer>;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#title)
     */
    title: string;

    readonly type: "kml";
    /**
     * The publicly accessible URL for a .kml or .kmz file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#url)
     */
    url: string;

    static fromJSON(json: any): KMLLayer;
  }

  interface KMLLayerProperties
    extends LayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties {
    /**
     * A collection of [KMLSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#sublayers)
     */
    sublayers?: CollectionProperties<KMLSublayerProperties>;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#title)
     */
    title?: string;
    /**
     * The publicly accessible URL for a .kml or .kmz file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html#url)
     */
    url?: string;
  }

  export interface KnowledgeGraphSublayer
    extends Layer,
      BlendLayer,
      FeatureEffectLayer,
      FeatureReductionLayer,
      OrderedLayer,
      TemporalLayer {}

  export class KnowledgeGraphSublayer {
    /**
     * A KnowledgeGraphSublayer is a sublayer of a [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html) or [LinkChartLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html) that contains all instances of a named type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html)
     */

    constructor(properties?: KnowledgeGraphSublayerProperties);

    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#capabilities)
     */
    readonly capabilities: KnowledgeGraphSublayerCapabilities;
    /**
     * The definition of the default popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#defaultPopupTemplate)
     */
    readonly defaultPopupTemplate: PopupTemplate;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The name of the sublayer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    elevationInfo: KnowledgeGraphSublayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The name of the geometry field for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#geometryFieldName)
     */
    geometryFieldName: string;
    /**
     * The geometry type of features in the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon";
    /**
     * Specifies whether the sublayer represents an [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#graphType)
     */
    graphType: "entity" | "relationship";
    /**
     * Indicates whether the features in the sublayer have `M` (measurement) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#hasM)
     */
    hasM: boolean;
    /**
     * Indicates whether the features in the layer have `Z` (elevation) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#hasZ)
     */
    hasZ: boolean;
    /**
     * The label definition for this sublayer, specified as an array of [label classes](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this sublayer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#minScale)
     */
    minScale: number;
    /**
     * The name of an `oid` containing a unique value or identifier for each feature in the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * Outlines the [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html) definition for the named type represented by the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#objectType)
     */
    objectType: GraphObjectType;
    /**
     * The [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html) of which this is a sublayer for a specific entity or relationship type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#parentCompositeLayer)
     */
    parentCompositeLayer: KnowledgeGraphLayer;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#renderer)
     */
    renderer: Renderer;

    readonly type: "knowledge-graph-sublayer";

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#definitionExpression),.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: KnowledgeGraphSublayerGetFieldDomainOptions): Domain;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the knowledge graph service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: KnowledgeGraphSublayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: KnowledgeGraphSublayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: KnowledgeGraphSublayerQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the knowledge graph service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: KnowledgeGraphSublayerQueryObjectIdsOptions,
    ): Promise<number[]>;
  }

  interface KnowledgeGraphSublayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      OrderedLayerProperties,
      TemporalLayerProperties {
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The name of the sublayer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    elevationInfo?: KnowledgeGraphSublayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The name of the geometry field for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#geometryFieldName)
     */
    geometryFieldName?: string;
    /**
     * The geometry type of features in the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#geometryType)
     */
    geometryType?: "point" | "multipoint" | "polyline" | "polygon";
    /**
     * Specifies whether the sublayer represents an [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#graphType)
     */
    graphType?: "entity" | "relationship";
    /**
     * Indicates whether the features in the sublayer have `M` (measurement) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#hasM)
     */
    hasM?: boolean;
    /**
     * Indicates whether the features in the layer have `Z` (elevation) values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#hasZ)
     */
    hasZ?: boolean;
    /**
     * The label definition for this sublayer, specified as an array of [label classes](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this sublayer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#minScale)
     */
    minScale?: number;
    /**
     * The name of an `oid` containing a unique value or identifier for each feature in the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * Outlines the [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html) definition for the named type represented by the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#objectType)
     */
    objectType?: GraphObjectTypeProperties;
    /**
     * The [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html) of which this is a sublayer for a specific entity or relationship type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#parentCompositeLayer)
     */
    parentCompositeLayer?: KnowledgeGraphLayerProperties;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#renderer)
     */
    renderer?: RendererProperties;
  }

  /**
   * Describes the layer's supported capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
   */
  export interface KnowledgeGraphSublayerCapabilities {
    /**
     * Describes what analytics capabilities are enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    analytics: KnowledgeGraphSublayerCapabilitiesAnalytics;
    /**
     * Describes what attachment capabilities are enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    attachment: KnowledgeGraphSublayerCapabilitiesAttachment;
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    data: KnowledgeGraphSublayerCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    editing: KnowledgeGraphSublayerCapabilitiesEditing;
    /**
     * Describes the metadata contained on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    metadata: KnowledgeGraphSublayerCapabilitiesMetadata;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    operations: KnowledgeGraphSublayerCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    query: KnowledgeGraphSublayerCapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    queryRelated: KnowledgeGraphSublayerCapabilitiesQueryRelated;
    /**
     * Describes [top features query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    queryTopFeatures: KnowledgeGraphSublayerCapabilitiesQueryTopFeatures;
  }

  export interface KnowledgeGraphSublayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    featureExpressionInfo?: KnowledgeGraphSublayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface KnowledgeGraphSublayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface KnowledgeGraphSublayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface KnowledgeGraphSublayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface KnowledgeGraphSublayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface KnowledgeGraphSublayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface KnowledgeGraphSublayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface KnowledgeGraphSublayerCapabilitiesAnalytics {
    /**
     * Indicates if the feature service supports cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesAttachment {
    /**
     * Indicates if the [attachment operations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) support a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their content types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsContentType: boolean;
    /**
     * Indicates if the attachment [queries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) support `exifInfo`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsExifInfo: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their keywords.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsKeywords: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsName: boolean;
    /**
     * Indicates if the attachments can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html) by their sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsSize: boolean;
    /**
     * Indicates if resized attachments are supported in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsResize: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesData {
    /**
     * Indicates if the feature service is versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    isVersioned: boolean;
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsAttachment: boolean;
    /**
     * Indicates if the features in the layer support m-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsM: boolean;
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsZ: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesEditing {
    /**
     * Indicates if anonymous users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDeleteByAnonymous: boolean;
    /**
     * Indicates if logged in users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDeleteByOthers: boolean;
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalId` values provided by the client are used in editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailureEnabled` parameter can be set to `true` or `false` when editing features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsRollbackOnFailure: boolean;
    /**
     * Indicates if anonymous users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsUpdateByAnonymous: boolean;
    /**
     * Indicates if logged in users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsUpdateByOthers: boolean;
    /**
     * Indicates if the layer supports uploading attachments by [UploadId](https://developers.arcgis.com/rest/services-reference/item.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsUploadWithItemId: boolean;
    /**
     * Indicates if `m-values` must be provided when updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsUpdateWithoutM: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesMetadata {
    /**
     * Indicates whether to provide a user-defined field description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsAdvancedFieldProperties: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesOperations {
    /**
     * Indicates if new features can be added to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if values of one or more field values in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCalculate: boolean;
    /**
     * Indicates if features can be deleted from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQueryAttachments: boolean;
    /**
     * Deprecated since 4.24.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsResizeAttachments: boolean;
    /**
     * Indicates if the layer supports [REST API queryTopFeatures](https://developers.arcgis.com/rest/services-reference/enterprise/query-top-features-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQueryTopFeatures: boolean;
    /**
     * Indicates if features in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if the layer supports a SQL-92 expression or where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsValidateSql: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesQuery {
    /**
     * The maximum number of records that will be returned for a given query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    maxRecordCount?: number;
    /**
     * Indicates if the query operation supports a [cache hint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#cacheHint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsHistoricMoment: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the layer supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the layer supports spatial extent, center or convex hull to be returned for each distinct group when [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsSpatialAggregationStatistics: boolean;
    /**
     * List of supported aggregated geometries returned for each distinct group when [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportedSpatialAggregationStatistics: KnowledgeGraphSublayerCapabilitiesQuerySupportedSpatialAggregationStatistics;
  }

  export interface KnowledgeGraphSublayerCapabilitiesQueryRelated {
    /**
     * Indicates if the [relationship query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) supports a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsPagination: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesQuerySupportedSpatialAggregationStatistics {
    /**
     * Indicates if the layer can return centroid for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    centroid: boolean;
    /**
     * Indicates if the layer can return extent for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    envelope: boolean;
    /**
     * Indicates if the layer can return convex hull for each distinct group for [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    convexHull: boolean;
  }

  export interface KnowledgeGraphSublayerCapabilitiesQueryTopFeatures {
    /**
     * Indicates if the [top query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) supports a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
  }

  export class KnowledgeGraphLayer extends Layer {
    /**
     * A KnowledgeGraphLayer is a composite layer that can be created from a [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html)
     */

    constructor(properties?: KnowledgeGraphLayerProperties);

    /**
     * Defines a set of [named types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) and/or [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#inclusionModeDefinition)
     */
    inclusionModeDefinition: InclusionModeDefinition;
    /**
     * The [data model](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html), [service definition](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html) and [url](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#url) of the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) that contains the data for the KnowledgeGraphLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#knowledgeGraph)
     */
    readonly knowledgeGraph: KnowledgeGraph;
    /**
     * A collection of operational [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#layers)
     */
    readonly layers: Collection<KnowledgeGraphSublayer>;
    /**
     * The [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) included in the KnowledgeGraphLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#memberEntityTypes)
     */
    readonly memberEntityTypes: EntityType[];
    /**
     * The [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) included in the KnowledgeGraphLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#memberRelationshipTypes)
     */
    readonly memberRelationshipTypes: RelationshipType[];
    /**
     * Contains the sublayer ids that have been cached.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#sublayerIdsCache)
     */
    readonly sublayerIdsCache: globalThis.Map<string, Set<string>>;
    /**
     * All non-spatial [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#tables)
     */
    readonly tables: Collection<KnowledgeGraphSublayer>;
    /**
     * The layer type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#type)
     */
    readonly type: "knowledge-graph";
    /**
     * The url of the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#url)
     */
    url: string;

    /**
     * Adds new entities or relationships to the knowledge graph layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#addRecords)
     */
    addRecords(records: IdTypePair[]): Promise<void>;
    /**
     * Assumes that data for all of the members defined in the inclusionModeDefinition is already loaded into local storage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#loadLayerAssumingLocalCache)
     */
    loadLayerAssumingLocalCache(): void;
    /**
     * Removes entities or relationships from the knowledge graph layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#removeRecords)
     */
    removeRecords(records: IdTypePair[]): Promise<IdTypePair[]>;
  }

  interface KnowledgeGraphLayerProperties extends LayerProperties {
    /**
     * Defines a set of [named types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) and/or [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#inclusionModeDefinition)
     */
    inclusionModeDefinition?: InclusionModeDefinition;
    /**
     * The url of the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#url)
     */
    url?: string;
  }

  /**
   * The id and named type of a record to be added or removed from the knowledge graph.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#IdTypePair)
   */
  export interface IdTypePair {
    /**
     * The global ID of the record to be added to or removed from the the knowledge graph layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#IdTypePair)
     */
    id: string;
    /**
     * the named type of the record to be added or removed from the knowledge graph layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#IdTypePair)
     */
    typeName: string;
  }

  /**
   * Defines the sublayer structure and the named types that will be in the KnowledgeGraphLayer
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#InclusionModeDefinition)
   */
  export interface InclusionModeDefinition {
    /**
     * Specifies whether to create sublayers for each [named type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) regardless of whether or not they are included in the `namedTypeDefinition`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#InclusionModeDefinition)
     */
    generateAllSublayers: boolean;
    /**
     * A map of named types and the specific instances of those types to include in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#InclusionModeDefinition)
     */
    namedTypeDefinitions: globalThis.Map<string, LayerInclusionDefinition>;
  }

  /**
   * Defines the contents of each specified named type in the [`inclusionModeDefinition`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#inclusionModeDefinition) and whether all instance of the named type will be used or a specified subset.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#LayerInclusionDefinition)
   */
  export interface LayerInclusionDefinition {
    /**
     * If `true` all of the instances of the named type will be included regardless of if there is a defined list of members.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#LayerInclusionDefinition)
     */
    useAllData: boolean;
    /**
     * A map of the specific members of the named type to be included.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#LayerInclusionDefinition)
     */
    members?: globalThis.Map<string, LayerInclusionMemberDefinition>;
  }

  /**
   * Defines the list of `members` for a named type in the [`inclusionModeDefinition`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#inclusionModeDefinition).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#LayerInclusionMemberDefinition)
   */
  export interface LayerInclusionMemberDefinition {
    /**
     * The global ID of the specific instance of the named type to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html#LayerInclusionMemberDefinition)
     */
    id: string;
  }

  export interface Layer extends Accessor, Loadable, Evented, Identifiable {}

  export class Layer {
    /**
     * The layer is the most fundamental component of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)
     */

    constructor(properties?: LayerProperties);

    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The unique ID assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#id)
     */
    id: string;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode)
     */
    listMode: "show" | "hide" | "hide-children";
    /**
     * Indicates whether the layer's resources have loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The opacity of the layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#opacity)
     */
    opacity: number;
    /**
     * The parent to which the layer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#parent)
     */
    parent: Map | Basemap | Ground | GroupLayer | CatalogDynamicGroupLayer | CatalogLayer;
    /**
     * When `true`, the layer can be persisted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled)
     */
    persistenceEnabled: boolean;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#title)
     */
    title: string;
    /**
     * The layer type provides a convenient way to check the type of the layer without the need to import specific layer modules.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#type)
     */
    readonly type:
      | "base-dynamic"
      | "base-elevation"
      | "base-tile"
      | "bing-maps"
      | "building-scene"
      | "catalog"
      | "catalog-dynamic-group"
      | "catalog-footprint"
      | "csv"
      | "dimension"
      | "elevation"
      | "feature"
      | "geojson"
      | "geo-rss"
      | "graphics"
      | "group"
      | "imagery"
      | "imagery-tile"
      | "integrated-mesh"
      | "integrated-mesh-3dtiles"
      | "kml"
      | "line-of-sight"
      | "map-image"
      | "map-notes"
      | "media"
      | "ogc-feature"
      | "open-street-map"
      | "oriented-imagery"
      | "point-cloud"
      | "route"
      | "scene"
      | "georeferenced-image"
      | "stream"
      | "tile"
      | "unknown"
      | "unsupported"
      | "vector-tile"
      | "viewshed"
      | "wcs"
      | "web-tile"
      | "wfs"
      | "wms"
      | "wmts"
      | "video"
      | "voxel"
      | "subtype-group"
      | "knowledge-graph"
      | "knowledge-graph-sublayer";
    /**
     * Specifies a fixed [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) during which a layer should be visible.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#visibilityTimeExtent)
     */
    visibilityTimeExtent: TimeExtent;
    /**
     * Indicates if the layer is visible in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#visible)
     */
    visible: boolean;

    /**
     * Called by the views, such as [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) and [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html), when the layer is added to the [Map.layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) collection and a layer view must be created for it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#createLayerView)
     */
    createLayerView(view: any, options?: LayerCreateLayerViewOptions): Promise<LayerView>;
    /**
     * Destroys the layer and any associated resources (including its [portalItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#portalItem), if it is a property on the layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#destroy)
     */
    destroy(): void;
    /**
     * Fetches custom attribution data for the layer when it becomes available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fetchAttributionData)
     */
    fetchAttributionData(): Promise<any>;

    on(name: "layerview-create", eventHandler: LayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: LayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: LayerLayerviewDestroyEventHandler): IHandle;

    /**
     * Creates a new layer instance from an ArcGIS Server URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fromArcGISServerUrl)
     */
    static fromArcGISServerUrl(params: LayerFromArcGISServerUrlParams): Promise<Layer>;
    /**
     * Creates a new layer instance of the appropriate layer class from an ArcGIS Online or ArcGIS Enterprise [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fromPortalItem)
     */
    static fromPortalItem(params: LayerFromPortalItemParams): Promise<Layer>;
  }

  interface LayerProperties extends LoadableProperties {
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The unique ID assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#id)
     */
    id?: string;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode)
     */
    listMode?: "show" | "hide" | "hide-children";
    /**
     * The opacity of the layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#opacity)
     */
    opacity?: number;
    /**
     * The parent to which the layer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#parent)
     */
    parent?:
      | MapProperties
      | BasemapProperties
      | GroundProperties
      | (GroupLayerProperties & { type: "group" })
      | (CatalogDynamicGroupLayerProperties & { type: "catalog-dynamic-group" })
      | (CatalogLayerProperties & { type: "catalog" });
    /**
     * When `true`, the layer can be persisted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled)
     */
    persistenceEnabled?: boolean;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#title)
     */
    title?: string;
    /**
     * Specifies a fixed [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) during which a layer should be visible.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#visibilityTimeExtent)
     */
    visibilityTimeExtent?: TimeExtentProperties;
    /**
     * Indicates if the layer is visible in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#visible)
     */
    visible?: boolean;
  }

  export interface LayerCreateLayerViewOptions {
    /**
     * A signal to abort the creation of the layerview.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#createLayerView)
     */
    signal?: AbortSignal;
  }

  export interface LayerFromArcGISServerUrlParams {
    /**
     * The ArcGIS Server URL used to create the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fromArcGISServerUrl)
     */
    url: string;
    /**
     * Set any of the layer's properties here for constructing the layer instance (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fromArcGISServerUrl)
     */
    properties?: any;
  }

  export interface LayerFromPortalItemParams {
    /**
     * The object representing an ArcGIS Online or ArcGIS Enterprise portal item from which to load the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#fromPortalItem)
     */
    portalItem: PortalItem;
  }

  export interface LayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface LayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface LayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface LineOfSightLayer extends Layer, OperationalLayer {}

  export class LineOfSightLayer {
    /**
     * LineOfSightLayer computes the line of sight from a single observer position towards a set of targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html)
     */

    constructor(properties?: LineOfSightLayerProperties);

    /**
     * The [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html) associated with the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#analysis)
     */
    analysis: LineOfSightAnalysis;
    /**
     * The full extent of the layer computed from the observer and targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#fullExtent)
     */
    readonly fullExtent: Extent;
    /**
     * The observer defines the point from which the line of sight analysis is performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#observer)
     */
    observer: LineOfSightAnalysisObserver;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * Target locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#targets)
     */
    targets: Collection<LineOfSightAnalysisTarget>;

    readonly type: "line-of-sight";

    static fromJSON(json: any): LineOfSightLayer;
  }

  interface LineOfSightLayerProperties extends LayerProperties, OperationalLayerProperties {
    /**
     * The [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html) associated with the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#analysis)
     */
    analysis?: LineOfSightAnalysisProperties;
    /**
     * The observer defines the point from which the line of sight analysis is performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#observer)
     */
    observer?: LineOfSightAnalysisObserverProperties;
    /**
     * Target locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LineOfSightLayer.html#targets)
     */
    targets?: CollectionProperties<LineOfSightAnalysisTargetProperties>;
  }

  export class LinkChartLayer extends Layer {
    /**
     * A LinkChartLayer is a composite layer that can be created from a [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html)
     */

    constructor(properties?: LinkChartLayerProperties);

    /**
     * Defines a set of [named types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) and/or [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#inclusionModeDefinition)
     */
    inclusionModeDefinition: InclusionModeDefinition;
    /**
     * A collection of operational [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#layers)
     */
    readonly layers: Collection<KnowledgeGraphSublayer>;
    /**
     * All non-spatial [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#tables)
     */
    readonly tables: Collection<KnowledgeGraphSublayer>;
    /**
     * The url of the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#url)
     */
    url: string;
  }

  interface LinkChartLayerProperties extends LayerProperties {
    /**
     * Defines a set of [named types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) and/or [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#inclusionModeDefinition)
     */
    inclusionModeDefinition?: InclusionModeDefinition;
    /**
     * The url of the [knowledge graph service](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#url)
     */
    url?: string;
  }

  /**
   * Defines the sublayer structure and the named types that will be in the LinkChartLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#InclusionModeDefinition)
   */
  export interface LinkChartLayerInclusionModeDefinition {
    /**
     * Specifies whether to create sublayers for each [named type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) regardless of whether or not they are included in the `namedTypeDefinition`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#InclusionModeDefinition)
     */
    generateAllSublayers: boolean;
    /**
     * A map of named types and the specific instances of those types to include in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#InclusionModeDefinition)
     */
    namedTypeDefinitions: globalThis.Map<string, LinkChartLayerLayerInclusionDefinition>;
  }

  /**
   * Defines the contents of each specified named type in the [`inclusionModeDefinition`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#inclusionModeDefinition) and whether all instance of the named type will be used for a specified subset.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionDefinition)
   */
  export interface LinkChartLayerLayerInclusionDefinition {
    /**
     * If `true`, all of the instances of the named type will be included regardless of if there is a defined list of members.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionDefinition)
     */
    useAllData: boolean;
    /**
     * A map of the specific members of the named type to be included.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionDefinition)
     */
    members?: globalThis.Map<string, LinkChartLayerLayerInclusionMemberDefinition>;
  }

  /**
   * Defines the list of `members` for a named type in the [`inclusionModeDefinition`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#inclusionModeDefinition).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionMemberDefinition)
   */
  export interface LinkChartLayerLayerInclusionMemberDefinition {
    /**
     * The unique identifier of the specific instance of the named type to be included in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionMemberDefinition)
     */
    id: string;
    /**
     * The exact point location of the entity on the LinkChart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-LinkChartLayer.html#LayerInclusionMemberDefinition)
     */
    linkChartLocation?: Geometry;
  }

  export interface MapImageLayer
    extends Layer,
      SublayersOwner,
      ArcGISMapService,
      ScaleRangeLayer,
      RefreshableLayer,
      TemporalLayer,
      BlendLayer,
      CustomParametersMixin,
      OperationalLayer {}

  export class MapImageLayer {
    /**
     * MapImageLayer allows you to display and analyze data from [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sublayers) defined in a [map service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-map-service.htm), exporting images instead of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html)
     */

    constructor(properties?: MapImageLayerProperties);

    /**
     * A flat [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of all the [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sublayers) in the MapImageLayer including the sublayers of its sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#allSublayers)
     */
    readonly allSublayers: Collection<Sublayer>;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#dateFieldsTimeZone)
     */
    readonly dateFieldsTimeZone: string;
    /**
     * This property is set by the service publisher and indicates that dates should be considered without the local timezone.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#datesInUnknownTimezone)
     */
    readonly datesInUnknownTimezone: boolean;
    /**
     * The output dots per inch (DPI) of the MapImageLayer.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#dpi)
     */
    dpi: number;
    /**
     * The version of the geodatabase of the map service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The output image type.
     *
     * @default "png24"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageFormat)
     */
    imageFormat: "png" | "png8" | "png24" | "png32" | "jpg" | "pdf" | "bmp" | "gif" | "pngjpg";
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageMaxHeight)
     */
    imageMaxHeight: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageMaxWidth)
     */
    imageMaxWidth: number;
    /**
     * Indicates whether the background of the image exported by the service is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageTransparency)
     */
    imageTransparency: boolean;
    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * The IANA time zone the author of the service intended data from date fields to be viewed in.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#preferredTimeZone)
     */
    readonly preferredTimeZone: string;
    /**
     * The [map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) objects that allow you to alter the properties of one or more sublayers of the MapImageLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sublayers)
     */
    sublayers: Collection<Sublayer>;

    readonly type: "map-image";
    /**
     * The URL to the REST endpoint of the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#url)
     */
    url: string;

    /**
     * Gets the parameters of the exported image to use when calling the [export REST operation](https://developers.arcgis.com/rest/services-reference/export-map.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#createExportImageParameters)
     */
    createExportImageParameters(
      extent: Extent,
      width: number,
      height: number,
      options?: MapImageLayerCreateExportImageParametersOptions,
    ): any;
    /**
     * This method fetches the image for the specified extent and size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#fetchImage)
     */
    fetchImage(
      extent: Extent,
      width: number,
      height: number,
      options?: MapImageLayerFetchImageOptions,
    ): Promise<HTMLImageElement>;
    /**
     * Loads all of the sublayers and subtables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#loadAll)
     */
    loadAll(): Promise<Sublayer>;

    on(name: "refresh", eventHandler: MapImageLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: MapImageLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: MapImageLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: MapImageLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): MapImageLayer;
  }

  interface MapImageLayerProperties
    extends LayerProperties,
      SublayersOwnerProperties,
      ArcGISMapServiceProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties,
      CustomParametersMixinProperties,
      OperationalLayerProperties {
    /**
     * The output dots per inch (DPI) of the MapImageLayer.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#dpi)
     */
    dpi?: number;
    /**
     * The version of the geodatabase of the map service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The output image type.
     *
     * @default "png24"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageFormat)
     */
    imageFormat?: "png" | "png8" | "png24" | "png32" | "jpg" | "pdf" | "bmp" | "gif" | "pngjpg";
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageMaxHeight)
     */
    imageMaxHeight?: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageMaxWidth)
     */
    imageMaxWidth?: number;
    /**
     * Indicates whether the background of the image exported by the service is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#imageTransparency)
     */
    imageTransparency?: boolean;
    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * The [map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) objects that allow you to alter the properties of one or more sublayers of the MapImageLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#sublayers)
     */
    sublayers?: CollectionProperties<SublayerProperties>;
    /**
     * The URL to the REST endpoint of the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#url)
     */
    url?: string;
  }

  export interface MapImageLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface MapImageLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface MapImageLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface MapImageLayerCreateExportImageParametersOptions {
    /**
     * The rotation in degrees of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#createExportImageParameters)
     */
    rotation?: number;
    /**
     * The pixel ratio to apply to the dpi of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#createExportImageParameters)
     */
    pixelRatio?: number;
    /**
     * The time instant or time extent of content to render.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#createExportImageParameters)
     */
    timeExtent?: any;
  }

  export interface MapImageLayerFetchImageOptions {
    /**
     * The rotation in degrees of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#fetchImage)
     */
    rotation?: number;
    /**
     * The ratio of the resolution in physical pixels of the image to the resolution it will be displayed at.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#fetchImage)
     */
    pixelRatio?: number;
    /**
     * The time instant or time extent of content to render.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#fetchImage)
     */
    timeExtent?: any;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html#fetchImage)
     */
    signal?: AbortSignal;
  }

  export interface MapImageLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface MapNotesLayer extends Layer, OperationalLayer, PortalLayer, ScaleRangeLayer, BlendLayer {}

  export class MapNotesLayer {
    /**
     * The MapNotesLayer lets you display and modify map notes (features sketched on a web map) from the [Map Viewer](https://www.arcgis.com/apps/mapviewer/index.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html)
     */

    constructor(properties?: MapNotesLayerProperties);

    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#capabilities)
     */
    readonly capabilities: MapNotesLayerCapabilities;
    /**
     * A layer containing a collection of graphics with multipoint geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#multipointLayer)
     */
    readonly multipointLayer: GraphicsLayer;
    /**
     * A layer containing a collection of graphics with point geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#pointLayer)
     */
    readonly pointLayer: GraphicsLayer;
    /**
     * A layer containing a collection of graphics with polygon geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#polygonLayer)
     */
    readonly polygonLayer: GraphicsLayer;
    /**
     * A layer containing a collection of graphics with polyline geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#polylineLayer)
     */
    readonly polylineLayer: GraphicsLayer;
    /**
     * A layer containing a collection of text graphics with point geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#textLayer)
     */
    readonly textLayer: GraphicsLayer;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets.
     *
     * @default "Map Notes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#title)
     */
    title: string;

    readonly type: "map-notes";

    static fromJSON(json: any): MapNotesLayer;
  }

  interface MapNotesLayerProperties
    extends LayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties {
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets.
     *
     * @default "Map Notes"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#title)
     */
    title?: string;
  }

  export interface MapNotesLayerCapabilities {
    /**
     * Describes operations that can be performed on map notes in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#capabilities)
     */
    operations: MapNotesLayerCapabilitiesOperations;
  }

  export interface MapNotesLayerCapabilitiesOperations {
    /**
     * Indicates if map notes in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapNotesLayer.html#capabilities)
     */
    supportsMapNotesEditing: boolean;
  }

  export interface MediaLayer extends Layer, ScaleRangeLayer, PortalLayer, BlendLayer, OperationalLayer {}

  export class MediaLayer {
    /**
     * The MediaLayer class is used to add [image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) and [video](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) elements to the map at a specified geographic [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#georeference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html)
     */

    constructor(properties?: MediaLayerProperties);

    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#copyright)
     */
    copyright: string;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#fullExtent)
     */
    readonly fullExtent: Extent;
    /**
     * The source for the MediaLayer that will be displayed on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source)
     */
    source: ImageElement | VideoElement | LocalMediaElementSource;
    /**
     * The spatial reference of the layer and defines the spatial reference of the layer's [fullExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#fullExtent).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;

    readonly type: "media";

    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#save)
     */
    save(options?: MediaLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: MediaLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): MediaLayer;
  }

  interface MediaLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      PortalLayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties {
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#copyright)
     */
    copyright?: string;
    /**
     * The source for the MediaLayer that will be displayed on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source)
     */
    source?:
      | (ImageElementProperties & { type: "image" })
      | (VideoElementProperties & { type: "video" })
      | LocalMediaElementSourceProperties
      | CollectionProperties<
          (ImageElementProperties & { type: "image" }) | (VideoElementProperties & { type: "video" })
        >
      | ((ImageElementProperties & { type: "image" }) | (VideoElementProperties & { type: "video" }))[];
    /**
     * The spatial reference of the layer and defines the spatial reference of the layer's [fullExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#fullExtent).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
  }

  export interface MediaLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#saveAs)
     */
    validationOptions?: MediaLayerSaveAsOptionsValidationOptions;
  }

  export interface MediaLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface MediaLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#save)
     */
    validationOptions?: MediaLayerSaveOptionsValidationOptions;
  }

  export interface MediaLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export class APIKeyMixin {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-APIKeyMixin.html#apiKey)
     */
    apiKey: string;
  }

  interface APIKeyMixinProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-APIKeyMixin.html#apiKey)
     */
    apiKey?: string;
  }

  export class ArcGISCachedService {
    constructor(properties?: ArcGISCachedServiceProperties);

    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISCachedService.html#copyright)
     */
    copyright: string;
    /**
     * The spatial reference of the layer as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISCachedService.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * Contains information about the tiling scheme for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISCachedService.html#tileInfo)
     */
    tileInfo: TileInfo;

    static fromJSON(json: any): ArcGISCachedService;
  }

  interface ArcGISCachedServiceProperties {
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISCachedService.html#copyright)
     */
    copyright?: string;
    /**
     * Contains information about the tiling scheme for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISCachedService.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
  }

  export class ArcGISImageService {
    constructor(properties?: ArcGISImageServiceProperties);

    /**
     * Defines a band combination using 0-based band indexes.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#bandIds)
     */
    bandIds: number[];
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    capabilities: ArcGISImageServiceCapabilities;
    /**
     * The compression quality value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#compressionQuality)
     */
    compressionQuality: number;
    /**
     * Controls the tolerance of the lerc compression algorithm.
     *
     * @default 0.01
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#compressionTolerance)
     */
    compressionTolerance: number;
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#copyright)
     */
    copyright: string;
    /**
     * Default mosaic rule of the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#defaultMosaicRule)
     */
    readonly defaultMosaicRule: MosaicRule;
    /**
     * The SQL where clause used to filter rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fields) by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The format of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#format)
     */
    format: "png" | "png8" | "png24" | "png32" | "jpg" | "bmp" | "gif" | "jpgpng" | "lerc" | "tiff" | "bip" | "bsq";
    /**
     * Indicates if the layer has [multidimensionalInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#multidimensionalInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#hasMultidimensions)
     */
    hasMultidimensions: boolean;
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 4100
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageMaxHeight)
     */
    imageMaxHeight: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 15000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageMaxWidth)
     */
    imageMaxWidth: number;
    /**
     * Defines how to interpolate pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#interpolation)
     */
    interpolation: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * Defines how overlapping images should be mosaicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#mosaicRule)
     */
    mosaicRule: MosaicRule;
    /**
     * The multidimensional information associated with the layer if the layer's [hasMultidimensions](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#hasMultidimensions) property is `true`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#multidimensionalInfo)
     */
    readonly multidimensionalInfo: RasterMultidimensionalInfo;
    /**
     * Represents a multidimensional subset of raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#multidimensionalSubset)
     */
    multidimensionalSubset: MultidimensionalSubset;
    /**
     * The pixel value representing no available information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noData)
     */
    noData: number | number[];
    /**
     * Interpretation of the [noData](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noData) setting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noDataInterpretation)
     */
    noDataInterpretation: "any" | "all";
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fields) containing a unique value or identifier for each raster in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#objectIdField)
     */
    objectIdField: string;
    /**
     * A function that processes [pixelData](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#pixelData).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#pixelFilter)
     */
    pixelFilter: PixelFilterFunction;
    /**
     * Raster source pixel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#pixelType)
     */
    pixelType:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * A complete list of fields that consists of raster attribute table fields, item pixel value, service pixel value, service pixel value with various server defined function templates, and raster attribute table fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#rasterFields)
     */
    readonly rasterFields: Field[];
    /**
     * Specifies the rule for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#rasterFunction)
     */
    rasterFunction: RasterFunction;
    /**
     * Returns raster function information for the image services, including the name, description, help, function type, and a thumbnail of pre-configured raster function templates.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#rasterFunctionInfos)
     */
    readonly rasterFunctionInfos: RasterFunctionInfo[];
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#renderer)
     */
    renderer:
      | ClassBreaksRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | RasterShadedReliefRenderer
      | RasterColormapRenderer
      | VectorFieldRenderer
      | FlowRenderer;
    /**
     * Source raster information of the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#serviceRasterInfo)
     */
    readonly serviceRasterInfo: RasterInfo;
    /**
     * The [image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * Image service data source type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#sourceType)
     */
    readonly sourceType: "mosaic-dataset" | "raster-dataset";
    /**
     * The spatial reference of the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The URL to the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#url)
     */
    url: string;
    /**
     * The version of ArcGIS Server in which the image service is published.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#version)
     */
    readonly version: number;

    /**
     * Computes the rotation angle of a ImageryLayer at a given location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#computeAngles)
     */
    computeAngles(
      parameters: ImageAngleParameters | ImageAngleParametersProperties,
      requestOptions?: any,
    ): Promise<ImageAngleResult>;
    /**
     * Computes histograms based on the provided [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#computeHistograms)
     */
    computeHistograms(
      parameters: ImageHistogramParameters | ImageHistogramParametersProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Computes the corresponding pixel location in columns and rows for an image based on input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#computePixelSpaceLocations)
     */
    computePixelSpaceLocations(
      parameters: ImagePixelLocationParameters,
      requestOptions?: any,
    ): Promise<ImagePixelLocationResult>;
    /**
     * Computes [statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics) and [histograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram) for the provided [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#computeStatisticsHistograms)
     */
    computeStatisticsHistograms(
      parameters: ImageHistogramParameters | ImageHistogramParametersProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Returns an image using the [export REST operation](https://developers.arcgis.com/rest/services-reference/export-image.htm) that displays data from an [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fetchImage)
     */
    fetchImage(
      extent: Extent,
      width: number,
      height: number,
      options?: ArcGISImageServiceFetchImageOptions,
    ): Promise<any>;
    /**
     * Finds images based on the provided [FindImagesParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#findImages)
     */
    findImages(
      parameters: FindImagesParameters | FindImagesParametersProperties,
      requestOptions?: any,
    ): Promise<FindImagesResult>;
    /**
     * Generates raster info for the specified raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#generateRasterInfo)
     */
    generateRasterInfo(
      rasterFunction: RasterFunction | RasterFunctionProperties,
      options?: ArcGISImageServiceGenerateRasterInfoOptions,
    ): Promise<RasterInfo>;
    /**
     * Gets the [image coordinate system](https://developers.arcgis.com/rest/services-reference/raster-ics.htm) information of a catalog item in an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getCatalogItemICSInfo)
     */
    getCatalogItemICSInfo(rasterId: number, options?: ArcGISImageServiceGetCatalogItemICSInfoOptions): Promise<any>;
    /**
     * Get the [raster info](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html) of a [catalog item](https://developers.arcgis.com/rest/services-reference/raster-catalog-item.htm) in an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getCatalogItemRasterInfo)
     */
    getCatalogItemRasterInfo(
      rasterId: number,
      options?: ArcGISImageServiceGetCatalogItemRasterInfoOptions,
    ): Promise<RasterInfo>;
    /**
     * Retrieves an image's url using the provided [ImageUrlParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getImageUrl)
     */
    getImageUrl(
      parameters: ImageUrlParameters | ImageUrlParametersProperties,
      requestOptions?: any,
    ): Promise<ImageUrlResult>;
    /**
     * Returns sample point locations, pixel values and corresponding resolutions of the source data for a given geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getSamples)
     */
    getSamples(
      parameters: ImageSampleParameters | ImageSampleParametersProperties,
      requestOptions?: any,
    ): Promise<ImageSampleResult>;
    /**
     * Sends a request to the ArcGIS REST image service to identify content based on the specified [ImageIdentifyParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#identify)
     */
    identify(
      parameters: ImageIdentifyParameters | ImageIdentifyParametersProperties,
      requestOptions?: any,
    ): Promise<ImageIdentifyResult>;
    /**
     * Converts a geometry from an image space to a map space using the provided [ImageToMapParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageToMap)
     */
    imageToMap(
      parameters: ImageToMapParameters | ImageToMapParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Creates a map space geometry from multiray image space geometries using the provided [ImageToMapMultirayParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageToMapMultiray)
     */
    imageToMapMultiray(
      parameters: ImageToMapMultirayParameters | ImageToMapMultirayParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Converts a given geometry from a map space to an image space using the provided [MapToImageParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#mapToImage)
     */
    mapToImage(
      parameters: MapToImageParameters | MapToImageParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Calculates the area and perimeter of a given geometry on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measureAreaAndPerimeter)
     */
    measureAreaAndPerimeter(
      parameters: ImageAreaParameters | ImageAreaParametersProperties,
      requestOptions?: any,
    ): Promise<ImageAreaResult>;
    /**
     * Measures the area and the perimeter of a polygon in an image space on a selected raster when the following conditions are met:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measureAreaFromImage)
     */
    measureAreaFromImage(
      parameters: MeasureFromImageParameters | MeasureFromImageParametersProperties,
      requestOptions?: any,
    ): Promise<MeasureAreaFromImageResult>;
    /**
     * Calculates the distance and angle between two points on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measureDistanceAndAngle)
     */
    measureDistanceAndAngle(
      parameters: ImageDistanceParameters | ImageDistanceParametersProperties,
      requestOptions?: any,
    ): Promise<ImageDistanceResult>;
    /**
     * Calculates the height of an object between two points on an image service if the sensor info is available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measureHeight)
     */
    measureHeight(
      parameters: ImageHeightParameters | ImageHeightParametersProperties,
      requestOptions?: any,
    ): Promise<ImageHeightResult>;
    /**
     * Measures the length of a polyline in an image space on a selected raster when the following conditions are met:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measureLengthFromImage)
     */
    measureLengthFromImage(
      parameters: MeasureFromImageParameters | MeasureFromImageParametersProperties,
      requestOptions?: any,
    ): Promise<MeasureLengthFromImageResult>;
    /**
     * Returns the location for a given point or centroid of a given area on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#measurePointOrCentroid)
     */
    measurePointOrCentroid(
      parameters: ImagePointParameters | ImagePointParametersProperties,
      requestOptions?: any,
    ): Promise<ImagePointResult>;
    /**
     * Returns the boundary of an image for the provided [ImageBoundaryParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#queryBoundary)
     */
    queryBoundary(
      parameters: ImageBoundaryParameters | ImageBoundaryParametersProperties,
      requestOptions?: any,
    ): Promise<ImageBoundaryResult>;
    /**
     * Returns GPS information for the provided [ImageGPSInfoParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#queryGPSInfo)
     */
    queryGPSInfo(
      parameters: ImageGPSInfoParameters | ImageGPSInfoParametersProperties,
      requestOptions?: any,
    ): Promise<ImageGPSInfoResult>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the image service and returns an array of Object IDs for the rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, requestOptions?: any): Promise<number[]>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the image service and returns the number of rasters that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#queryRasterCount)
     */
    queryRasterCount(query?: Query | QueryProperties, requestOptions?: any): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against an image service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#queryRasters)
     */
    queryRasters(query: Query | QueryProperties, requestOptions?: any): Promise<FeatureSet>;

    static fromJSON(json: any): ArcGISImageService;
  }

  interface ArcGISImageServiceProperties {
    /**
     * Defines a band combination using 0-based band indexes.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#bandIds)
     */
    bandIds?: number[];
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    capabilities?: ArcGISImageServiceCapabilities;
    /**
     * The compression quality value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#compressionQuality)
     */
    compressionQuality?: number;
    /**
     * Controls the tolerance of the lerc compression algorithm.
     *
     * @default 0.01
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#compressionTolerance)
     */
    compressionTolerance?: number;
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#copyright)
     */
    copyright?: string;
    /**
     * The SQL where clause used to filter rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The format of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#format)
     */
    format?: "png" | "png8" | "png24" | "png32" | "jpg" | "bmp" | "gif" | "jpgpng" | "lerc" | "tiff" | "bip" | "bsq";
    /**
     * Indicates if the layer has [multidimensionalInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#multidimensionalInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#hasMultidimensions)
     */
    hasMultidimensions?: boolean;
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 4100
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageMaxHeight)
     */
    imageMaxHeight?: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 15000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#imageMaxWidth)
     */
    imageMaxWidth?: number;
    /**
     * Defines how to interpolate pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#interpolation)
     */
    interpolation?: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * Defines how overlapping images should be mosaicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#mosaicRule)
     */
    mosaicRule?: MosaicRuleProperties;
    /**
     * Represents a multidimensional subset of raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#multidimensionalSubset)
     */
    multidimensionalSubset?: MultidimensionalSubsetProperties;
    /**
     * The pixel value representing no available information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noData)
     */
    noData?: number | number[];
    /**
     * Interpretation of the [noData](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noData) setting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#noDataInterpretation)
     */
    noDataInterpretation?: "any" | "all";
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fields) containing a unique value or identifier for each raster in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * A function that processes [pixelData](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#pixelData).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#pixelFilter)
     */
    pixelFilter?: PixelFilterFunction;
    /**
     * Raster source pixel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#pixelType)
     */
    pixelType?:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * Specifies the rule for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#rasterFunction)
     */
    rasterFunction?: RasterFunctionProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#renderer)
     */
    renderer?:
      | (ClassBreaksRendererProperties & { type: "class-breaks" })
      | (UniqueValueRendererProperties & { type: "unique-value" })
      | (RasterStretchRendererProperties & { type: "raster-stretch" })
      | (RasterShadedReliefRendererProperties & { type: "raster-shaded-relief" })
      | (RasterColormapRendererProperties & { type: "raster-colormap" })
      | (VectorFieldRendererProperties & { type: "vector-field" })
      | (FlowRendererProperties & { type: "flow" });
    /**
     * The [image service's metadata JSON](https://developers.arcgis.com/rest/services-reference/image-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * The URL to the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#url)
     */
    url?: string;
  }

  export interface ArcGISImageServiceCapabilities {
    /**
     * Describes [mensuration operations](https://developers.arcgis.com/rest/services-reference/enterprise/measure.htm) supported by the ImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    mensuration: ArcGISImageServiceCapabilitiesMensuration;
    /**
     * Describes operations supported by the ImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    operations: ArcGISImageServiceCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations supported by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    query: ArcGISImageServiceCapabilitiesQuery;
  }

  export interface ArcGISImageServiceCapabilitiesMensuration {
    /**
     * Indicates if the layer supports 3D mensuration operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supports3D: boolean;
    /**
     * Indicates if the layer supports area and perimeter mensuration operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsAreaAndPerimeter: boolean;
    /**
     * Indicates if the layer supports distance and angle mensuration operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsDistanceAndAngle: boolean;
    /**
     * Indicates if the layer supports a mensuration operation for calculating a height of a ground feature by measuring from the base of the object to the top of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsHeightFromBaseAndTop: boolean;
    /**
     * Indicates if the layer supports a mensuration operation for calculating a height of a feature by measuring from the base of the object to the top of the object's shadow on the ground.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsHeightFromBaseAndTopShadow: boolean;
    /**
     * Indicates if the layer supports mensuration operation for calculating a height from the top of the object to the top of the objects's shadow on the ground.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsHeightFromTopAndTopShadow: boolean;
    /**
     * Indicates if the layer supports point or centroid mensuration operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsPointOrCentroid: boolean;
  }

  export interface ArcGISImageServiceCapabilitiesOperations {
    /**
     * Indicates if the layer supports a [compute histograms](https://developers.arcgis.com/rest/services-reference/compute-histograms.htm) operation from an area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsComputeHistograms: boolean;
    /**
     * Indicates if the layer supports an [export image](https://developers.arcgis.com/rest/services-reference/export-image.htm) operation based on a bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsExportImage: boolean;
    /**
     * Indicates if the layer supports an [identify](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html#identify) operation on pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsIdentify: boolean;
    /**
     * Indicates if the layer supports mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsMeasure: boolean;
    /**
     * Indicates if the layer supports image to map operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsImageToMap: boolean;
    /**
     * Indicates if the layer supports image to map multiray operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsImageToMapMultiray: boolean;
    /**
     * Indicates if the layer supports find images operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsFindImages: boolean;
    /**
     * Indicates if the layer supports get image url operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsGetImageUrl: boolean;
    /**
     * Indicates if the layer supports map to image operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsMapToImage: boolean;
    /**
     * Indicates if the layer allows the source image to be [downloaded](https://developers.arcgis.com/rest/services-reference/download-rasters.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsDownload: boolean;
    /**
     * Indicates if the layer supports an operation to [query images](https://developers.arcgis.com/rest/services-reference/query-image-service-.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports a [sampling operation](https://developers.arcgis.com/rest/services-reference/get-samples.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsGetSamples: boolean;
    /**
     * Indicates if the layer supports [projection of geometries](https://developers.arcgis.com/rest/services-reference/project.htm), including from/to image spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsProject: boolean;
    /**
     * Indicates if the layer supports an operation [computing statistics and histograms](https://developers.arcgis.com/rest/services-reference/compute-statistics-and-histograms.htm) from an area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsComputeStatisticsHistograms: boolean;
    /**
     * Indicates if the layer supports an operation to [query the service's boundary](https://developers.arcgis.com/rest/services-reference/query-boundary.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsQueryBoundary: boolean;
    /**
     * Indicates if the layer supports an operation to [query the service's gps info](https://developers.arcgis.com/rest/services-reference/enterprise/query-gps-.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsQueryGPSInfo: boolean;
    /**
     * Indicates if the elevation layer supports an operation to [compute volumes](https://developers.arcgis.com/rest/services-reference/calculate-volume.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsCalculateVolume: boolean;
    /**
     * Indicates if the layer supports [computing pixel location](https://developers.arcgis.com/rest/services-reference/compute-pixel-location.htm) in pixel space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsComputePixelLocation: boolean;
  }

  export interface ArcGISImageServiceCapabilitiesQuery {
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the layer supports paged queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports use of standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * The maximum number of records that will be returned for a given query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#capabilities)
     */
    maxRecordCount?: number;
  }

  export interface ArcGISImageServiceFetchImageOptions {
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#fetchImage)
     */
    signal?: AbortSignal;
  }

  export interface ArcGISImageServiceGenerateRasterInfoOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#generateRasterInfo)
     */
    signal?: AbortSignal;
  }

  export interface ArcGISImageServiceGetCatalogItemICSInfoOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getCatalogItemICSInfo)
     */
    signal?: AbortSignal;
  }

  export interface ArcGISImageServiceGetCatalogItemRasterInfoOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISImageService.html#getCatalogItemRasterInfo)
     */
    signal?: AbortSignal;
  }

  export class ArcGISMapService {
    constructor(properties?: ArcGISMapServiceProperties);

    /**
     * Indicates the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    readonly capabilities: ArcGISMapServiceCapabilities;
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#copyright)
     */
    copyright: string;
    /**
     * The full extent of the layer as defined by the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The spatial reference of the layer as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The version of ArcGIS Server in which the map service is published.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#version)
     */
    readonly version: number;

    static fromJSON(json: any): ArcGISMapService;
  }

  interface ArcGISMapServiceProperties {
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#copyright)
     */
    copyright?: string;
    /**
     * The full extent of the layer as defined by the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#legendEnabled)
     */
    legendEnabled?: boolean;
  }

  export interface ArcGISMapServiceCapabilities {
    /**
     * Indicates options supported by the exportMap operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    exportMap: ArcGISMapServiceCapabilitiesExportMap;
    /**
     * Indicates options supported by the exportTiles operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    exportTiles: ArcGISMapServiceCapabilitiesExportTiles;
    /**
     * Indicates operations that can be performed on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    operations: ArcGISMapServiceCapabilitiesOperations;
  }

  export interface ArcGISMapServiceCapabilitiesExportMap {
    /**
     * Indicates if sublayers support Arcade expressions for labeling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsArcadeExpressionForLabeling: boolean;
    /**
     * Indicates if sublayers rendering can be modified or added using dynamic layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsDynamicLayers: boolean;
    /**
     * Indicates if sublayers can be added, or removed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsSublayersChanges: boolean;
    /**
     * Indicates if sublayers [definition expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#definitionExpression) can be set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsSublayerDefinitionExpression: boolean;
    /**
     * Indicates if sublayers [visibility](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#visible) can be changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsSublayerVisibility: boolean;
    /**
     * _Since 4.23_ Indicates if [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) can be used in a sublayer's [renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#renderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsCIMSymbols: boolean;
  }

  export interface ArcGISMapServiceCapabilitiesExportTiles {
    /**
     * Specifies the maximum number of tiles that can be exported to a cache dataset or a tile package.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    maxExportTilesCount: number;
  }

  export interface ArcGISMapServiceCapabilitiesOperations {
    /**
     * Indicates if the service can generate images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsExportMap: boolean;
    /**
     * Indicates if the tiles from the service can be exported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsExportTiles: boolean;
    /**
     * Indicates if the service supports the [identify operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html#identify).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsIdentify: boolean;
    /**
     * Indicates if features in the sublayers can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the service exposes a tile map that describes the presence of tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ArcGISMapService.html#capabilities)
     */
    supportsTileMap: boolean;
  }

  export class BlendLayer {
    /**
     * Blend modes are used to blend layers together to create an interesting effect in a layer, or even to produce what seems like a new layer.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-BlendLayer.html#blendMode)
     */
    blendMode:
      | "average"
      | "color-burn"
      | "color-dodge"
      | "color"
      | "darken"
      | "destination-atop"
      | "destination-in"
      | "destination-out"
      | "destination-over"
      | "difference"
      | "exclusion"
      | "hard-light"
      | "hue"
      | "invert"
      | "lighten"
      | "lighter"
      | "luminosity"
      | "minus"
      | "multiply"
      | "normal"
      | "overlay"
      | "plus"
      | "reflect"
      | "saturation"
      | "screen"
      | "soft-light"
      | "source-atop"
      | "source-in"
      | "source-out"
      | "vivid-light"
      | "xor";
    /**
     * Effect provides various filter functions that can be performed on the layer to achieve different visual effects similar to how image filters work.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-BlendLayer.html#effect)
     */
    effect: Effect;
  }

  interface BlendLayerProperties {
    /**
     * Blend modes are used to blend layers together to create an interesting effect in a layer, or even to produce what seems like a new layer.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-BlendLayer.html#blendMode)
     */
    blendMode?:
      | "average"
      | "color-burn"
      | "color-dodge"
      | "color"
      | "darken"
      | "destination-atop"
      | "destination-in"
      | "destination-out"
      | "destination-over"
      | "difference"
      | "exclusion"
      | "hard-light"
      | "hue"
      | "invert"
      | "lighten"
      | "lighter"
      | "luminosity"
      | "minus"
      | "multiply"
      | "normal"
      | "overlay"
      | "plus"
      | "reflect"
      | "saturation"
      | "screen"
      | "soft-light"
      | "source-atop"
      | "source-in"
      | "source-out"
      | "vivid-light"
      | "xor";
    /**
     * Effect provides various filter functions that can be performed on the layer to achieve different visual effects similar to how image filters work.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-BlendLayer.html#effect)
     */
    effect?: any[] | string;
  }

  export class CustomParametersMixin {
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-CustomParametersMixin.html#customParameters)
     */
    customParameters: any;
  }

  interface CustomParametersMixinProperties {
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-CustomParametersMixin.html#customParameters)
     */
    customParameters?: any;
  }

  export class FeatureEffectLayer {
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureEffectLayer.html#featureEffect)
     */
    featureEffect: FeatureEffect;
  }

  interface FeatureEffectLayerProperties {
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureEffectLayer.html#featureEffect)
     */
    featureEffect?: FeatureEffectProperties;
  }

  export class FeatureLayerBase {
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#capabilities)
     */
    readonly capabilities: Capabilities;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#copyright)
     */
    copyright: string;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#dateFieldsTimeZone)
     */
    dateFieldsTimeZone: string;
    /**
     * This property is set by the service publisher and indicates that dates should be considered without the local timezone.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#datesInUnknownTimezone)
     */
    readonly datesInUnknownTimezone: boolean;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#displayField)
     */
    displayField: string;
    /**
     * The editor tracking fields, which record who adds or edits the data through the feature service and when edits are made.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#editFieldsInfo)
     */
    readonly editFieldsInfo: EditFieldsInfo;
    /**
     * Specifies information about editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#editingInfo)
     */
    readonly editingInfo: EditingInfo;
    /**
     * Describes effective capabilities of the layer taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#effectiveCapabilities)
     */
    readonly effectiveCapabilities: Capabilities;
    /**
     * Indicates whether the layer is editable taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#effectiveEditingEnabled)
     */
    readonly effectiveEditingEnabled: boolean;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    elevationInfo: FeatureLayerBaseElevationInfo;
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * When a feature layer is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#floorInfo)
     */
    floorInfo: LayerFloorInfo;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The version of the geodatabase of the feature service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * Provides information on the system maintained area and length fields along with their respective units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#geometryFieldsInfo)
     */
    readonly geometryFieldsInfo: GeometryFieldsInfo;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "multipatch" | "mesh";
    /**
     * Indicates whether the client-side features in the layer have `M` (measurement) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#hasM)
     */
    hasM: boolean;
    /**
     * Indicates whether the client-side features in the layer have `Z` (elevation) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#hasZ)
     */
    hasZ: boolean;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#historicMoment)
     */
    historicMoment: Date;
    /**
     * Returns `true` if the layer is loaded from a non-spatial table in a service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#isTable)
     */
    readonly isTable: boolean;
    /**
     * The layer ID, or layer index, of a Feature Service layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#layerId)
     */
    layerId: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#objectIdField)
     */
    objectIdField: string;
    /**
     * The IANA time zone the author of the service intended data from date fields to be viewed in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#preferredTimeZone)
     */
    readonly preferredTimeZone: string;
    /**
     * Array of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html) set up for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#relationships)
     */
    readonly relationships: Relationship[];
    /**
     * When `true`, indicates that M values will be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#returnM)
     */
    returnM: boolean;
    /**
     * When `true`, indicates that z-values will always be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#returnZ)
     */
    returnZ: boolean;
    /**
     * The service definition expression limits the features available for display and query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#serviceDefinitionExpression)
     */
    readonly serviceDefinitionExpression: string;
    /**
     * Indicates the portal item of the hosted feature service that contains this layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#serviceItemId)
     */
    readonly serviceItemId: string;
    /**
     * The [feature service's metadata JSON](https://developers.arcgis.com/rest/services-reference/layer-feature-service-.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The name of the field which holds the id of the [subtypes](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#subtypes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#subtypeField)
     */
    readonly subtypeField: string;
    /**
     * An array of [subtypes](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html) defined in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#subtypes)
     */
    readonly subtypes: Subtype[];
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#title)
     */
    title: string;
    /**
     * The absolute URL of the REST endpoint of the layer, non-spatial table or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#url)
     */
    url: string;
    /**
     * The version of ArcGIS Server in which the layer is published.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#version)
     */
    readonly version: number;

    /**
     * Adds an attachment to a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#addAttachment)
     */
    addAttachment(feature: Graphic, attachment: HTMLFormElement | FormData): Promise<FeatureEditResult>;
    /**
     * Applies edits to features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    applyEdits(
      edits: FeatureLayerBaseApplyEditsEdits,
      options?: FeatureLayerBaseApplyEditsOptions,
    ): Promise<EditsResult>;
    /**
     * Deletes attachments from a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#deleteAttachments)
     */
    deleteAttachments(feature: Graphic, attachmentIds: number[]): Promise<FeatureEditResult[]>;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: FeatureLayerBaseGetFieldDomainOptions): Domain;
    /**
     * Query information about attachments associated with features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryAttachments)
     */
    queryAttachments(
      attachmentQuery: AttachmentQuery | AttachmentQueryProperties,
      options?: FeatureLayerBaseQueryAttachmentsOptions,
    ): Promise<any>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service and returns [FeatureSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) grouped by source layer or table objectIds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryRelatedFeatures)
     */
    queryRelatedFeatures(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: FeatureLayerBaseQueryRelatedFeaturesOptions,
    ): Promise<any>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service and when resolved, it returns an `object` containing key value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryRelatedFeaturesCount)
     */
    queryRelatedFeaturesCount(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: FeatureLayerBaseQueryRelatedFeaturesCountOptions,
    ): Promise<any>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#save)
     */
    save(options?: FeatureLayerBaseSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: FeatureLayerBaseSaveAsOptions): Promise<PortalItem>;
    /**
     * Updates an existing attachment for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#updateAttachment)
     */
    updateAttachment(
      feature: Graphic,
      attachmentId: number,
      attachment: HTMLFormElement | FormData,
    ): Promise<FeatureEditResult>;
  }

  interface FeatureLayerBaseProperties {
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#copyright)
     */
    copyright?: string;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#dateFieldsTimeZone)
     */
    dateFieldsTimeZone?: string;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    elevationInfo?: FeatureLayerBaseElevationInfo;
    /**
     * When a feature layer is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#floorInfo)
     */
    floorInfo?: LayerFloorInfoProperties;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The version of the geodatabase of the feature service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#geometryType)
     */
    geometryType?: "point" | "multipoint" | "polyline" | "polygon" | "multipatch" | "mesh";
    /**
     * Indicates whether the client-side features in the layer have `M` (measurement) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#hasM)
     */
    hasM?: boolean;
    /**
     * Indicates whether the client-side features in the layer have `Z` (elevation) values.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#hasZ)
     */
    hasZ?: boolean;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#historicMoment)
     */
    historicMoment?: DateProperties;
    /**
     * The layer ID, or layer index, of a Feature Service layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#layerId)
     */
    layerId?: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * When `true`, indicates that M values will be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#returnM)
     */
    returnM?: boolean;
    /**
     * When `true`, indicates that z-values will always be returned.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * The [feature service's metadata JSON](https://developers.arcgis.com/rest/services-reference/layer-feature-service-.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#title)
     */
    title?: string;
    /**
     * The absolute URL of the REST endpoint of the layer, non-spatial table or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#url)
     */
    url?: string;
  }

  export interface FeatureLayerBaseApplyEditsEdits {
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    addFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    updateFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features, or an array of objects with `objectId` or `globalId` of each feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    deleteFeatures?: Graphic[] | Collection<Graphic> | any[];
    /**
     * An array of attachments to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    addAttachments?: AttachmentEdit[];
    /**
     * An array of attachments to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    updateAttachments?: AttachmentEdit[];
    /**
     * An array of [globalId](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#AttachmentEdit)s for attachments to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    deleteAttachments?: string[];
  }

  export interface FeatureLayerBaseApplyEditsOptions {
    /**
     * The geodatabase version to apply the edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    gdbVersion?: string;
    /**
     * Indicates whether the edit results should return the time edits were applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    returnEditMoment?: boolean;
    /**
     * If set to `original-and-current-features`, the [EditedFeatureResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult) parameter will be included in the `applyEdits` response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    returnServiceEditsOption?: "none" | "original-and-current-features";
    /**
     * Indicates whether the edits should be applied only if all submitted edits succeed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    rollbackOnFailureEnabled?: boolean;
    /**
     * Indicates whether the edits can be applied using globalIds of features or attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#applyEdits)
     */
    globalIdUsed?: boolean;
  }

  export interface FeatureLayerBaseElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    featureExpressionInfo?: FeatureLayerBaseElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface FeatureLayerBaseElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#elevationInfo)
     */
    expression?: string;
  }

  export interface FeatureLayerBaseGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface FeatureLayerBaseQueryAttachmentsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryAttachments)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerBaseQueryRelatedFeaturesCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryRelatedFeaturesCount)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerBaseQueryRelatedFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#queryRelatedFeatures)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerBaseSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#saveAs)
     */
    validationOptions?: FeatureLayerBaseSaveAsOptionsValidationOptions;
  }

  export interface FeatureLayerBaseSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface FeatureLayerBaseSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#save)
     */
    validationOptions?: FeatureLayerBaseSaveOptionsValidationOptions;
  }

  export interface FeatureLayerBaseSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureLayerBase.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export class FeatureReductionLayer {
    /**
     * Configures the method for reducing the number of features in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureReductionLayer.html#featureReduction)
     */
    featureReduction: FeatureReductionBinning | FeatureReductionCluster | FeatureReductionSelection;
  }

  interface FeatureReductionLayerProperties {
    /**
     * Configures the method for reducing the number of features in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-FeatureReductionLayer.html#featureReduction)
     */
    featureReduction?:
      | (FeatureReductionBinningProperties & { type: "binning" })
      | (FeatureReductionClusterProperties & { type: "cluster" })
      | (FeatureReductionSelectionProperties & { type: "selection" });
  }

  export class ImageryTileMixin {
    /**
     * Defines a band combination using 0-based band indexes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#bandIds)
     */
    bandIds: number[];
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#copyright)
     */
    copyright: string;
    /**
     * Defines how to interpolate pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#interpolation)
     */
    interpolation: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The multidimensional definitions associated with the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#multidimensionalDefinition)
     */
    multidimensionalDefinition: DimensionalDefinition[];
    /**
     * Represents a multidimensional subset of raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#multidimensionalSubset)
     */
    multidimensionalSubset: MultidimensionalSubset;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Raster information retrieved from data source.
     *
     * @deprecated since 4.29. Use {@link module:esri/layers/ImageryTileLayer#serviceRasterInfo serviceRasterInfo} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#rasterInfo)
     */
    readonly rasterInfo: RasterInfo;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#renderer)
     */
    renderer:
      | ClassBreaksRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | RasterShadedReliefRenderer
      | RasterColormapRenderer
      | VectorFieldRenderer
      | FlowRenderer;
    /**
     * Raster information retrieved from tiled imagery data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#serviceRasterInfo)
     */
    readonly serviceRasterInfo: RasterInfo;
    /**
     * The layer's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeInfo)
     */
    timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeOffset)
     */
    timeOffset: TimeInterval;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#useViewTime)
     */
    useViewTime: boolean;

    /**
     * Fetches pixels for a given extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#fetchPixels)
     */
    fetchPixels(
      extent: Extent,
      width: number,
      height: number,
      options?: ImageryTileMixinFetchPixelsOptions,
    ): Promise<any>;
    /**
     * Identify pixel values at a given location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#identify)
     */
    identify(point: Point | PointProperties, options?: RasterIdentifyOptions): Promise<RasterIdentifyResult>;
  }

  interface ImageryTileMixinProperties {
    /**
     * Defines a band combination using 0-based band indexes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#bandIds)
     */
    bandIds?: number[];
    /**
     * The copyright text as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#copyright)
     */
    copyright?: string;
    /**
     * Defines how to interpolate pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#interpolation)
     */
    interpolation?: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The multidimensional definitions associated with the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#multidimensionalDefinition)
     */
    multidimensionalDefinition?: DimensionalDefinitionProperties[];
    /**
     * Represents a multidimensional subset of raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#multidimensionalSubset)
     */
    multidimensionalSubset?: MultidimensionalSubsetProperties;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#renderer)
     */
    renderer?:
      | (ClassBreaksRendererProperties & { type: "class-breaks" })
      | (UniqueValueRendererProperties & { type: "unique-value" })
      | (RasterStretchRendererProperties & { type: "raster-stretch" })
      | (RasterShadedReliefRendererProperties & { type: "raster-shaded-relief" })
      | (RasterColormapRendererProperties & { type: "raster-colormap" })
      | (VectorFieldRendererProperties & { type: "vector-field" })
      | (FlowRendererProperties & { type: "flow" });
    /**
     * The layer's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeInfo)
     */
    timeInfo?: TimeInfoProperties;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#timeOffset)
     */
    timeOffset?: TimeIntervalProperties;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#useViewTime)
     */
    useViewTime?: boolean;
  }

  export interface ImageryTileMixinFetchPixelsOptions {
    /**
     * Added at version 4.23.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#fetchPixels)
     */
    interpolation?: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ImageryTileMixin.html#fetchPixels)
     */
    signal?: AbortSignal;
  }

  export class OperationalLayer {
    constructor(properties?: OperationalLayerProperties);

    /**
     * Enable persistence of the layer in a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OperationalLayer.html#persistenceEnabled)
     */
    persistenceEnabled: boolean;

    static fromJSON(json: any): OperationalLayer;
  }

  interface OperationalLayerProperties {
    /**
     * Enable persistence of the layer in a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OperationalLayer.html#persistenceEnabled)
     */
    persistenceEnabled?: boolean;
  }

  export class OrderedLayer {
    /**
     * Determines the order in which features are drawn in the view.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    orderBy: OrderedLayerOrderBy[];
  }

  interface OrderedLayerProperties {
    /**
     * Determines the order in which features are drawn in the view.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    orderBy?: OrderedLayerOrderByProperties[];
  }

  export interface OrderedLayerOrderByProperties {
    /**
     * The number or date field whose values will be used to sort features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    field?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#feature-sorting).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    valueExpression?: string;
    /**
     * The sort order.
     *
     * @default ascending
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    order?: "ascending" | "descending";
  }

  export interface OrderedLayerOrderBy extends AnonymousAccessor {
    /**
     * The number or date field whose values will be used to sort features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    field?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#feature-sorting).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    valueExpression?: string;
    /**
     * The sort order.
     *
     * @default ascending
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-OrderedLayer.html#orderBy)
     */
    order?: "ascending" | "descending";
  }

  export class PortalLayer {
    constructor(properties?: PortalLayerProperties);

    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-PortalLayer.html#portalItem)
     */
    portalItem: PortalItem;

    static fromJSON(json: any): PortalLayer;
  }

  interface PortalLayerProperties {
    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-PortalLayer.html#portalItem)
     */
    portalItem?: PortalItemProperties;
  }

  export class PublishableLayer {
    /**
     * Checks layer's publishing status while the layer is being published to the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-PublishableLayer.html#publishingInfo)
     */
    readonly publishingInfo: PublishingInfo;
  }

  interface PublishableLayerProperties {}

  export class RasterPresetRendererMixin {
    /**
     * The active preset renderer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RasterPresetRendererMixin.html#activePresetRendererName)
     */
    activePresetRendererName: string;
    /**
     * A list of preset renderers that defines a preferred renderer for a given multidimensional variable, a given raster function template, or an additional generic predefined renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RasterPresetRendererMixin.html#presetRenderers)
     */
    presetRenderers: RasterPresetRenderer[];
  }

  interface RasterPresetRendererMixinProperties {
    /**
     * The active preset renderer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RasterPresetRendererMixin.html#activePresetRendererName)
     */
    activePresetRendererName?: string;
    /**
     * A list of preset renderers that defines a preferred renderer for a given multidimensional variable, a given raster function template, or an additional generic predefined renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RasterPresetRendererMixin.html#presetRenderers)
     */
    presetRenderers?: RasterPresetRendererProperties[];
  }

  export class RefreshableLayer {
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RefreshableLayer.html#refreshInterval)
     */
    refreshInterval: number;

    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RefreshableLayer.html#refresh)
     */
    refresh(): void;
  }

  interface RefreshableLayerProperties {
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-RefreshableLayer.html#refreshInterval)
     */
    refreshInterval?: number;
  }

  export class ScaleRangeLayer {
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ScaleRangeLayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ScaleRangeLayer.html#minScale)
     */
    minScale: number;
  }

  interface ScaleRangeLayerProperties {
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ScaleRangeLayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-ScaleRangeLayer.html#minScale)
     */
    minScale?: number;
  }

  export class SceneService {
    constructor(properties?: SceneServiceProperties);

    /**
     * The copyright text as defined by the scene service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#copyright)
     */
    copyright: string;
    /**
     * The layer ID, or layer index, of a Scene Service layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#layerId)
     */
    layerId: number;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The URL of the REST endpoint of the layer or scene service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#url)
     */
    url: string;
    /**
     * The version of the scene service specification used for this service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#version)
     */
    readonly version: SceneServiceVersion;

    static fromJSON(json: any): SceneService;
  }

  interface SceneServiceProperties {
    /**
     * The copyright text as defined by the scene service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#copyright)
     */
    copyright?: string;
    /**
     * The layer ID, or layer index, of a Scene Service layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#layerId)
     */
    layerId?: number;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The URL of the REST endpoint of the layer or scene service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#url)
     */
    url?: string;
  }

  export interface SceneServiceVersion {
    /**
     * The major version of the scene layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#version)
     */
    major: number;
    /**
     * The minor version of the scene layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#version)
     */
    minor: number;
    /**
     * The complete version string of the scene layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SceneService.html#version)
     */
    versionString: string;
  }

  export class SublayersOwner {
    /**
     * A flat [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of all the tables in the layer including the tables of its sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SublayersOwner.html#subtables)
     */
    subtables: Collection<Sublayer>;

    /**
     * Returns a deep clone of a map service's [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SublayersOwner.html#createServiceSublayers)
     */
    createServiceSublayers(): Collection<Sublayer>;
    /**
     * Returns the sublayer with the given layerId.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SublayersOwner.html#findSublayerById)
     */
    findSublayerById(id: number): Sublayer;
  }

  interface SublayersOwnerProperties {
    /**
     * A flat [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of all the tables in the layer including the tables of its sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-SublayersOwner.html#subtables)
     */
    subtables?: CollectionProperties<SublayerProperties>;
  }

  export class TemporalLayer {
    /**
     * The layer's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeInfo)
     */
    timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeOffset)
     */
    timeOffset: TimeInterval;
    /**
     * Determines if the time enabled layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#useViewTime)
     */
    useViewTime: boolean;
  }

  interface TemporalLayerProperties {
    /**
     * The layer's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeInfo)
     */
    timeInfo?: TimeInfoProperties;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#timeOffset)
     */
    timeOffset?: TimeIntervalProperties;
    /**
     * Determines if the time enabled layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalLayer.html#useViewTime)
     */
    useViewTime?: boolean;
  }

  export class TemporalSceneLayer {
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeInfo)
     */
    timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeOffset)
     */
    timeOffset: TimeInterval;
  }

  interface TemporalSceneLayerProperties {
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeInfo)
     */
    timeInfo?: TimeInfoProperties;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-mixins-TemporalSceneLayer.html#timeOffset)
     */
    timeOffset?: TimeIntervalProperties;
  }

  /**
   * Provides utility functions for the [WCSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html)
   */
  interface wcsUtils {
    /**
     * Fetches the capabilities metadata offered by the WCS service, including supported versions and coverages information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#getCapabilities)
     */
    getCapabilities(url: string, options?: wcsUtilsGetCapabilitiesOptions): Promise<WCSCapabilities>;
  }

  export const wcsUtils: wcsUtils;

  /**
   * Coverage information associated with a WCS service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#CoverageBrief)
   */
  export interface CoverageBrief {
    /**
     * The id of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#CoverageBrief)
     */
    id?: string;
    /**
     * The extent of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#CoverageBrief)
     */
    lonLatEnvelope?: Extent;
    /**
     * The subtype of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#CoverageBrief)
     */
    coverageSubType?: string;
  }

  /**
   * A list of URLs for the WCS service resources.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#OnlineResources)
   */
  export interface OnlineResources {
    /**
     * The URL to the getCapabilities endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#OnlineResources)
     */
    getCapabilities: string;
    /**
     * The URL to the describeCoverage endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#OnlineResources)
     */
    describeCoverage: string;
    /**
     * The URL to the getCoverage endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#OnlineResources)
     */
    getCoverage: string;
  }

  /**
   * WCS service information about the available coverages, versions, extensions and more.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
   */
  export interface WCSCapabilities {
    /**
     * Name of the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    name: string;
    /**
     * Online resources for the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    onlineResources: OnlineResources;
    /**
     * Information about the available coverages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    coverages: CoverageBrief[];
    /**
     * Information about the available grid coverages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    gridCoverages: CoverageBrief[];
    /**
     * The versions supported by the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    supportedVersions: string[];
    /**
     * The WCS service version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    version: "1.0.0" | "1.1.0" | "1.1.1" | "1.1.2" | "2.0.1";
    /**
     * Formats supported by the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    supportedFormats?: string[];
    /**
     * Application profiles supported by the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    profiles?: string[];
    /**
     * Interpolations supported by the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#WCSCapabilities)
     */
    supportedInterpolations?: string[];
  }

  export interface wcsUtilsGetCapabilitiesOptions {
    /**
     * The version of the WCS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#getCapabilities)
     */
    version?: "1.0.0" | "1.1.0" | "1.1.1" | "1.1.2" | "2.0.1";
    /**
     * A list of key-value pairs of parameters to append to the url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#getCapabilities)
     */
    customParameters?: any;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the executions of the remote method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wcsUtils.html#getCapabilities)
     */
    signal?: AbortSignal;
  }

  /**
   * Provides utility functions for the [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html)
   */
  interface wfsUtils {
    /**
     * Fetches the GetCapabilities document of a WFS service which contains information about the list of layers in the service and the available operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getCapabilities)
     */
    getCapabilities(url: string, options?: wfsUtilsGetCapabilitiesOptions): Promise<WFSCapabilities>;
    /**
     * Returns the [WFSLayerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo) from the capabilities of the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getWFSLayerInfo)
     */
    getWFSLayerInfo(
      capabilities: WFSCapabilities,
      name?: string,
      namespaceUri?: string,
      options?: wfsUtilsGetWFSLayerInfoOptions,
    ): Promise<WFSLayerInfo>;
  }

  export const wfsUtils: wfsUtils;

  /**
   * WFS service information about the available layers and operations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSCapabilities)
   */
  export interface WFSCapabilities {
    /**
     * Information about the operations needed to consume the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSCapabilities)
     */
    operations: WFSOperations;
    /**
     * An array of featureTypes (or layers) available in the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSCapabilities)
     */
    featureTypes: WFSFeatureType[];
  }

  /**
   * Provides information about an individual feature type, or layer, found in the WFS service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
   */
  export interface WFSFeatureType {
    /**
     * The type name of the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    typeName: string;
    /**
     * The name of the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    name: string;
    /**
     * The title of the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    title: string;
    /**
     * The description of the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    description: string;
    /**
     * The extent of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    extent: Extent;
    /**
     * The prefix of the namespace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    namespacePrefix: string;
    /**
     * The namespaceUri of the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    namespaceUri: string;
    /**
     * the default spatialReference wkid used when not explicitely specified in the GetFeature call.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    defaultSpatialReference: number;
    /**
     * An array of spatialReference wkids that are supported on the featureType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSFeatureType)
     */
    supportedSpatialReferences: number[];
  }

  /**
   * The layer info from the WFS service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
   */
  export interface WFSLayerInfo {
    /**
     * The URL to the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    url: string;
    /**
     * The name of the layer in the WFS service to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    name: string;
    /**
     * The namespace URI for the layer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    namespaceUri: string;
    /**
     * The fields on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    fields: Field[];
    /**
     * The geometry type of the features in the layer.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon";
    /**
     * The field containing the object ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    objectIdField: string;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    spatialReference: SpatialReference;
    /**
     * The extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    extent: Extent;
    /**
     * Determines whether the X and Y coordinates should be swapped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    swapXY: boolean;

    wfsCapabilities: WFSCapabilities;
    /**
     * The custom parameters applied to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSLayerInfo)
     */
    customParameters: any;
  }

  /**
   * Information about some operations of the WFS service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
   */
  export interface WFSOperations {
    GetCapabilities: WFSOperationsGetCapabilities;

    DescribeFeatureType: WFSOperationsDescribeFeatureType;

    GetFeature: WFSOperationsGetFeature;
  }

  export interface wfsUtilsGetCapabilitiesOptions {
    /**
     * A list of key-value pairs of parameters to append to the url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getCapabilities)
     */
    customParameters?: any;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the executions of the remote method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getCapabilities)
     */
    signal?: AbortSignal;
  }

  export interface wfsUtilsGetWFSLayerInfoOptions {
    /**
     * A list of key-value pairs of parameters to append to the url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getWFSLayerInfo)
     */
    customParameters?: any;
    /**
     * The desired spatial reference for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getWFSLayerInfo)
     */
    spatialReference?: any;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the executions of the remote method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getWFSLayerInfo)
     */
    signal?: AbortSignal;
  }

  export interface WFSOperationsDescribeFeatureType {
    /**
     * The URL to the DescribeFeatureType endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
     */
    url: string;
  }

  export interface WFSOperationsGetCapabilities {
    /**
     * The URL to the GetCapabilities endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
     */
    url: string;
  }

  export interface WFSOperationsGetFeature {
    /**
     * The URL to the GetFeature endpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
     */
    url: string;
    /**
     * Value of the output format parameter to send with the GetFeature call.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
     */
    outputFormat: string;
    /**
     * Indicates if the GetFeature request supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#WFSOperations)
     */
    supportsPagination: boolean;
  }

  export interface OGCFeatureLayer
    extends Layer,
      APIKeyMixin,
      BlendLayer,
      CustomParametersMixin,
      OperationalLayer,
      OrderedLayer,
      PortalLayer,
      RefreshableLayer,
      ScaleRangeLayer,
      FeatureEffectLayer,
      FeatureReductionLayer {}

  export class OGCFeatureLayer {
    /**
     * The OGCFeatureLayer class is used to create a layer based on individual collections from a [OGC API Features](https://www.opengeospatial.org/standards/ogcapi-features) service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html)
     */

    constructor(properties?: OGCFeatureLayerProperties);

    /**
     * The unique identifier of the collection on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#collectionId)
     */
    collectionId: string;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#copyright)
     */
    copyright: string;
    /**
     * Description of the features in the collection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#description)
     */
    readonly description: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    elevationInfo: OGCFeatureLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#geometryType)
     */
    geometryType: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Explicitly set the maximum number of features that can be returned in a single request.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#maxRecordCount)
     */
    maxRecordCount: number;
    /**
     * The OGCFeatureLayer requires that each feature be uniquely identified with an object id.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * The spatial reference the source data is stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#title)
     */
    title: string;

    readonly type: "ogc-feature";
    /**
     * The URL to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#url)
     */
    url: string;

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#getField)
     */
    getField(fieldName: string): Field;

    on(name: "refresh", eventHandler: OGCFeatureLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: OGCFeatureLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: OGCFeatureLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: OGCFeatureLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): OGCFeatureLayer;
  }

  interface OGCFeatureLayerProperties
    extends LayerProperties,
      APIKeyMixinProperties,
      BlendLayerProperties,
      CustomParametersMixinProperties,
      OperationalLayerProperties,
      OrderedLayerProperties,
      PortalLayerProperties,
      RefreshableLayerProperties,
      ScaleRangeLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties {
    /**
     * The unique identifier of the collection on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#collectionId)
     */
    collectionId?: string;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#copyright)
     */
    copyright?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    elevationInfo?: OGCFeatureLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#geometryType)
     */
    geometryType?: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Explicitly set the maximum number of features that can be returned in a single request.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#maxRecordCount)
     */
    maxRecordCount?: number;
    /**
     * The OGCFeatureLayer requires that each feature be uniquely identified with an object id.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#title)
     */
    title?: string;
    /**
     * The URL to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#url)
     */
    url?: string;
  }

  export interface OGCFeatureLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface OGCFeatureLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface OGCFeatureLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface OGCFeatureLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    featureExpressionInfo?: OGCFeatureLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface OGCFeatureLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface OGCFeatureLayerRefreshEvent {
    dataChanged: boolean;
  }

  export class OpenStreetMapLayer extends WebTileLayer {
    /**
     * Allows you to use [basemaps](http://wiki.openstreetmap.org/wiki/List_of_OSM-based_services) from [OpenStreetMap](http://www.openstreetmap.org/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OpenStreetMapLayer.html)
     */

    constructor(properties?: OpenStreetMapLayerProperties);

    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OpenStreetMapLayer.html#fullExtent)
     */
    readonly fullExtent: Extent;

    readonly type: "open-street-map";

    on(name: "refresh", eventHandler: OpenStreetMapLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: OpenStreetMapLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: OpenStreetMapLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: OpenStreetMapLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): OpenStreetMapLayer;
  }

  interface OpenStreetMapLayerProperties extends WebTileLayerProperties {}

  export interface OpenStreetMapLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface OpenStreetMapLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface OpenStreetMapLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface OpenStreetMapLayerRefreshEvent {
    dataChanged: boolean;
  }

  /**
   * Provides the utility function to convert image coordinates to geographic coordinates.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html)
   */
  interface imageToWorld {
    /**
     * Transforms image coordinates in pixels to a point or an array of points specifying the corresponding geographic world coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#imageToWorld)
     */
    imageToWorld(
      pixelOrPixels: imageToWorldPixelLocation | imageToWorldPixelLocation[],
      properties: ImageToWorldProperties,
      updateElevationProps: UpdateElevationProps,
    ): Promise<Point | Point[]>;
  }

  export const imageToWorld: imageToWorld;

  /**
   * Represents the properties required for transforming image coordinates to world coordinates.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
   */
  export interface ImageToWorldProperties {
    /**
     * The height of the ground at the camera’s location (specified by cameraLocation), determined by subtracting the camera’s height from the absolute z-coordinate of the location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    averageElevation: number;
    /**
     * The camera orientation of the second rotation around the x-axis of the camera.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    cameraPitch: number;
    /**
     * The camera orientation of the final rotation around the z-axis of the camera in the positive clockwise direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    cameraRoll: number;
    /**
     * The farthest usable distance of the imagery from the camera position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    farDistance: number;
    /**
     * A fixed length array of 6 numbers corresponding to the camera affine values A0, A1, A2, B0, B1, B2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    affineTransformations: number[];
    /**
     * A Point representing the camera location in world coordinates where the image was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    cameraLocation: any;
    /**
     * The focal length of the camera lens.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    focalLength: number;
    /**
     * A fixed length array of 2 numbers specifying the x-coordinate and  y-coordinate of the principal point of the autocollimation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    principalOffsetPoint?: number[];
    /**
     * The radial distortion is specified as an array of three numbers, indicating [K1,K2,K3] coefficient values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    radialDistortionCoefficients?: number[];
    /**
     * The row-wise sorted rotation matrix that defines the transformation from image space to map space, specified as an array of nine floating-point values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    rotationMatrix: number[];
    /**
     * The tangential distortion is specified as an array of three numbers, indicating [P1,P2] coefficient values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    tangentialDistortionCoefficients?: number[];
    /**
     * The camera’s field-of-view in a horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    horizontalFieldOfView: number;
    /**
     * The camera’s field-of-view in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    verticalFieldOfView: number;
    /**
     * Height of the image used for world to image transformation, in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    imageHeight: number;
    /**
     * image used for world to image transformation, in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#ImageToWorldProperties)
     */
    imageWidth: number;
  }

  /**
   * Represents the pixel location in image space, where (0, 0) corresponds to the center of the upper left pixel.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#PixelLocation)
   */
  export interface imageToWorldPixelLocation {
    /**
     * The row the pixel location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#PixelLocation)
     */
    x: number;
    /**
     * The column the pixel location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#PixelLocation)
     */
    y: number;
  }

  /**
   * Represents the properties required for updating the elevation of the output point.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#UpdateElevationProps)
   */
  export interface UpdateElevationProps {
    /**
     * A cache of elevation values created from an elevation service used for synchronously querying elevation information for geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#UpdateElevationProps)
     */
    elevationSampler: any;
    /**
     * The constant ground elevation value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-imageToWorld.html#UpdateElevationProps)
     */
    constantElevation: number;
  }

  /**
   * Provides the utility function to convert geographic coordinates to image coordinates.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html)
   */
  interface worldToImage {
    /**
     * Transforms a point or an array of points from geographic world coordinates to image space coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#worldToImage)
     */
    worldToImage(
      pointOrPoints: Point | Point[] | Collection<Point>,
      properties: WorldToImageProperties,
    ): PixelLocation | PixelLocation[];
  }

  export const worldToImage: worldToImage;

  /**
   * Represents the pixel location in image space, where (0, 0) corresponds to the center of the upper left pixel.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#PixelLocation)
   */
  export interface PixelLocation {
    /**
     * The row the pixel location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#PixelLocation)
     */
    x: number;
    /**
     * The column the pixel location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#PixelLocation)
     */
    y: number;
  }

  /**
   * Represents the properties required for transforming world coordinates to image coordinates.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
   */
  export interface WorldToImageProperties {
    /**
     * A fixed length array of 6 numbers corresponding to the camera affine values A0, A1, A2, B0, B1, B2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    affineTransformations: number[];
    /**
     * A Point representing the camera location in world coordinates where the image was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    cameraLocation: any;
    /**
     * The focal length of the camera lens.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    focalLength: number;
    /**
     * A fixed length array of 2 numbers specifying the x-coordinate and  y-coordinate of the principal point of the autocollimation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    principalOffsetPoint?: number[];
    /**
     * The radial distortion is specified as an array of three numbers, indicating [K1,K2,K3] coefficient values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    radialDistortionCoefficients?: number[];
    /**
     * The row-wise sorted rotation matrix that defines the transformation from image space to map space, specified as an array of nine floating-point values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    rotationMatrix: number[];
    /**
     * The tangential distortion is specified as  an array of three numbers, indicating [P1,P2] coefficient values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    tangentialDistortionCoefficients?: number[];
    /**
     * The camera’s field-of-view in a horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    horizontalFieldOfView: number;
    /**
     * The camera’s field-of-view in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    verticalFieldOfView: number;
    /**
     * Height of the image used for world to image transformation, in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    imageHeight: number;
    /**
     * image used for world to image transformation, in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-orientedImagery-transformations-worldToImage.html#WorldToImageProperties)
     */
    imageWidth: number;
  }

  export interface OrientedImageryLayer
    extends Layer,
      BlendLayer,
      OrderedLayer,
      ScaleRangeLayer,
      FeatureEffectLayer,
      FeatureReductionLayer,
      OperationalLayer,
      Clonable {}

  export class OrientedImageryLayer {
    /**
     * An Oriented Imagery Layer is a single spatial layer that can be created from a [Feature Service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-feature-service-.htm); or ArcGIS Online or ArcGIS Enterprise portal items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html)
     */

    constructor(properties?: OrientedImageryLayerProperties);

    /**
     * Camera orientation defining the first rotation around z-axis of the camera.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraHeading)
     */
    cameraHeading: number;
    /**
     * The height of camera above the ground when the imagery was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraHeight)
     */
    cameraHeight: number;
    /**
     * Camera orientation defining the second rotation around x-axis of the camera in the positive counterclockwise direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraPitch)
     */
    cameraPitch: number;
    /**
     * Camera orientation defining the final rotation around z-axis of the camera in the positive clockwise direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraRoll)
     */
    cameraRoll: number;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#copyright)
     */
    copyright: string;
    /**
     * Modifies the extent of the image's ground footprint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#coveragePercent)
     */
    coveragePercent: number;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * Prefix used to build the DEM url path in conjunction with the [elevationSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource) attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#demPathPrefix)
     */
    demPathPrefix: string;
    /**
     * Suffix used to build the DEM url path in conjunction with the [elevationSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource) attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#demPathSuffix)
     */
    demPathSuffix: string;
    /**
     * Prefix used to build the depth image url path in conjunction with the depth image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#depthImagePathPrefix)
     */
    depthImagePathPrefix: string;
    /**
     * Suffix used to build the depth image url path in conjunction with the depth image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#depthImagePathSuffix)
     */
    depthImagePathSuffix: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    elevationInfo: OrientedImageryLayerElevationInfo;
    /**
     * The source of elevation, that will be used to compute ground to image transformations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource)
     */
    elevationSource: ConstantElevation | ElevationSource;
    /**
     * The farthest usable distance of the imagery from the camera position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#farDistance)
     */
    farDistance: number;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#fields)
     */
    fields: Field[];
    /**
     * The geometry type of features in the OrientedImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#geometryType)
     */
    geometryType: "point";
    /**
     * The camera's scope in horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#horizontalFieldOfView)
     */
    horizontalFieldOfView: number;
    /**
     * Defines the unit that will be used for all horizontal measurements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#horizontalMeasurementUnit)
     */
    horizontalMeasurementUnit: string;
    /**
     * Prefix used to build the image url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imagePathPrefix)
     */
    imagePathPrefix: string;
    /**
     * Suffix used to build the image url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imagePathSuffix)
     */
    imagePathSuffix: string;
    /**
     * The orientation of the camera in degrees relative to the scene when the image was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imageRotation)
     */
    imageRotation: number;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * A unique id that identifies an operational layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#layerId)
     */
    readonly layerId: number;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Maximum search distance to be used while querying the feature service specified in the Oriented Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#maximumDistance)
     */
    maximumDistance: number;
    /**
     * The nearest usable distance of the imagery from the camera position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#nearDistance)
     */
    nearDistance: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * Semicolon-delimited string used to store standard deviation values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#orientationAccuracy)
     */
    orientationAccuracy: number[];
    /**
     * String that defines the imagery type used in the particular Oriented Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#orientedImageryType)
     */
    orientedImageryType: "horizontal" | "nadir" | "oblique" | "360" | "inspection";
    /**
     * An array of field names to include in the OrientedImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The portal item referencing the Oriented Imagery layer item from which the OrientedImageryLayer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * Defines the unit of time used in the viewer's time selector tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#timeIntervalUnit)
     */
    timeIntervalUnit: "days" | "hours" | "minutes" | "months" | "weeks" | "years";
    /**
     * The layer type provides a convenient way to check the type of the layer without the need to import specific layer modules.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#type)
     */
    readonly type: "oriented-imagery";
    /**
     * The URL of the Oriented Imagery layer service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#url)
     */
    url: string;
    /**
     * The camera's scope in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#verticalFieldOfView)
     */
    verticalFieldOfView: number;
    /**
     * Defines the primary unit to be used for all vertical measurements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#verticalMeasurementUnit)
     */
    verticalMeasurementUnit: "feet" | "meter";
    /**
     * Prefix used to build the video url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#videoPathPrefix)
     */
    videoPathPrefix: string;
    /**
     * Suffix used to build the video url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#videoPathSuffix)
     */
    videoPathSuffix: string;

    /**
     * Creates query parameters that can be used to fetch features that satisfy the layer's current filters, and definitions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: OrientedImageryLayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the Oriented Imagery features and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: OrientedImageryLayerQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: OrientedImageryLayerQueryObjectIdsOptions,
    ): Promise<number[]>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#save)
     */
    save(options?: OrientedImageryLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#saveAs)
     */
    saveAs(
      portalItem: PortalItem | PortalItemProperties,
      options?: OrientedImageryLayerSaveAsOptions,
    ): Promise<PortalItem>;

    static fromJSON(json: any): OrientedImageryLayer;
  }

  interface OrientedImageryLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      OrderedLayerProperties,
      ScaleRangeLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      OperationalLayerProperties {
    /**
     * Camera orientation defining the first rotation around z-axis of the camera.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraHeading)
     */
    cameraHeading?: number;
    /**
     * The height of camera above the ground when the imagery was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraHeight)
     */
    cameraHeight?: number;
    /**
     * Camera orientation defining the second rotation around x-axis of the camera in the positive counterclockwise direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraPitch)
     */
    cameraPitch?: number;
    /**
     * Camera orientation defining the final rotation around z-axis of the camera in the positive clockwise direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#cameraRoll)
     */
    cameraRoll?: number;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#copyright)
     */
    copyright?: string;
    /**
     * Modifies the extent of the image's ground footprint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#coveragePercent)
     */
    coveragePercent?: number;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * Prefix used to build the DEM url path in conjunction with the [elevationSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource) attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#demPathPrefix)
     */
    demPathPrefix?: string;
    /**
     * Suffix used to build the DEM url path in conjunction with the [elevationSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource) attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#demPathSuffix)
     */
    demPathSuffix?: string;
    /**
     * Prefix used to build the depth image url path in conjunction with the depth image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#depthImagePathPrefix)
     */
    depthImagePathPrefix?: string;
    /**
     * Suffix used to build the depth image url path in conjunction with the depth image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#depthImagePathSuffix)
     */
    depthImagePathSuffix?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    elevationInfo?: OrientedImageryLayerElevationInfo;
    /**
     * The source of elevation, that will be used to compute ground to image transformations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationSource)
     */
    elevationSource?: ConstantElevation | ElevationSource;
    /**
     * The farthest usable distance of the imagery from the camera position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#farDistance)
     */
    farDistance?: number;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the OrientedImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#geometryType)
     */
    geometryType?: "point";
    /**
     * The camera's scope in horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#horizontalFieldOfView)
     */
    horizontalFieldOfView?: number;
    /**
     * Defines the unit that will be used for all horizontal measurements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#horizontalMeasurementUnit)
     */
    horizontalMeasurementUnit?: string;
    /**
     * Prefix used to build the image url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imagePathPrefix)
     */
    imagePathPrefix?: string;
    /**
     * Suffix used to build the image url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imagePathSuffix)
     */
    imagePathSuffix?: string;
    /**
     * The orientation of the camera in degrees relative to the scene when the image was captured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#imageRotation)
     */
    imageRotation?: number;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Maximum search distance to be used while querying the feature service specified in the Oriented Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#maximumDistance)
     */
    maximumDistance?: number;
    /**
     * The nearest usable distance of the imagery from the camera position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#nearDistance)
     */
    nearDistance?: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * Semicolon-delimited string used to store standard deviation values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#orientationAccuracy)
     */
    orientationAccuracy?: number[];
    /**
     * String that defines the imagery type used in the particular Oriented Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#orientedImageryType)
     */
    orientedImageryType?: "horizontal" | "nadir" | "oblique" | "360" | "inspection";
    /**
     * An array of field names to include in the OrientedImageryLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The portal item referencing the Oriented Imagery layer item from which the OrientedImageryLayer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Defines the unit of time used in the viewer's time selector tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#timeIntervalUnit)
     */
    timeIntervalUnit?: "days" | "hours" | "minutes" | "months" | "weeks" | "years";
    /**
     * The URL of the Oriented Imagery layer service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#url)
     */
    url?: string;
    /**
     * The camera's scope in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#verticalFieldOfView)
     */
    verticalFieldOfView?: number;
    /**
     * Defines the primary unit to be used for all vertical measurements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#verticalMeasurementUnit)
     */
    verticalMeasurementUnit?: "feet" | "meter";
    /**
     * Prefix used to build the video url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#videoPathPrefix)
     */
    videoPathPrefix?: string;
    /**
     * Suffix used to build the video url path in conjunction with the image attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#videoPathSuffix)
     */
    videoPathSuffix?: string;
  }

  /**
   * The unit of constant elevation value measured in meters.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ConstantElevation)
   */
  export interface ConstantElevation {
    /**
     * The constant associated with the elevation source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ConstantElevation)
     */
    constantElevation: number;
  }

  /**
   * The elevation source.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ElevationSource)
   */
  export interface ElevationSource {
    /**
     * The url of the elevation source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ElevationSource)
     */
    url: string;
    /**
     * The raster function used in the elevation source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ElevationSource)
     */
    rasterFunction?: string;
    /**
     * The level of details (LODs) of the elevation source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#ElevationSource)
     */
    lod: number;
  }

  export interface OrientedImageryLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    featureExpressionInfo?: OrientedImageryLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface OrientedImageryLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface OrientedImageryLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface OrientedImageryLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface OrientedImageryLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface OrientedImageryLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#saveAs)
     */
    validationOptions?: OrientedImageryLayerSaveAsOptionsValidationOptions;
  }

  export interface OrientedImageryLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface OrientedImageryLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#save)
     */
    validationOptions?: OrientedImageryLayerSaveOptionsValidationOptions;
  }

  export interface OrientedImageryLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export class PointCloudBitfieldFilter extends PointCloudFilter {
    /**
     * Points can be classified using flags that indicate special characteristics associated with the point like the direction of the flight or the method used to create the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html)
     */

    constructor(properties?: PointCloudBitfieldFilterProperties);

    /**
     * An array of bit positions where the bit value must be `0`, otherwise the point will not be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html#requiredClearBits)
     */
    requiredClearBits: number[];
    /**
     * An array of bit positions where the bit value must be `1`, otherwise the point will not be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html#requiredSetBits)
     */
    requiredSetBits: number[];
    /**
     * For PointCloudBitfieldFilter the type is always `bitfield`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html#type)
     */
    readonly type: "bitfield";

    static fromJSON(json: any): PointCloudBitfieldFilter;
  }

  interface PointCloudBitfieldFilterProperties extends PointCloudFilterProperties {
    /**
     * An array of bit positions where the bit value must be `0`, otherwise the point will not be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html#requiredClearBits)
     */
    requiredClearBits?: number[];
    /**
     * An array of bit positions where the bit value must be `1`, otherwise the point will not be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudBitfieldFilter.html#requiredSetBits)
     */
    requiredSetBits?: number[];
  }

  export interface PointCloudFilter extends Accessor, JSONSupport {}

  export class PointCloudFilter {
    /**
     * Point cloud filters are used to hide points that do not satisfy the filter criteria.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html)
     */

    constructor(properties?: PointCloudFilterProperties);

    /**
     * The field used for applying the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html#field)
     */
    field: string;
    /**
     * The type of filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html#type)
     */
    readonly type: "value" | "bitfield" | "return";

    static fromJSON(json: any): PointCloudFilter;
  }

  interface PointCloudFilterProperties {
    /**
     * The field used for applying the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html#field)
     */
    field?: string;
  }

  export class PointCloudReturnFilter extends PointCloudFilter {
    /**
     * Laser pulses emitted from a lidar system can have several returns depending on the surfaces that they encounter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudReturnFilter.html)
     */

    constructor(properties?: PointCloudReturnFilterProperties);

    /**
     * An array of return types used to filter points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudReturnFilter.html#includedReturns)
     */
    includedReturns: ("firstOfMany" | "last" | "lastOfMany" | "single")[];

    readonly type: "return";

    static fromJSON(json: any): PointCloudReturnFilter;
  }

  interface PointCloudReturnFilterProperties extends PointCloudFilterProperties {
    /**
     * An array of return types used to filter points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudReturnFilter.html#includedReturns)
     */
    includedReturns?: ("firstOfMany" | "last" | "lastOfMany" | "single")[];
  }

  export class PointCloudValueFilter extends PointCloudFilter {
    /**
     * Every lidar point can have a classification that defines the type of surface that reflected the laser pulse.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudValueFilter.html)
     */

    constructor(properties?: PointCloudValueFilterProperties);

    /**
     * Whether points should be included or excluded from the filter.
     *
     * @default "exclude"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudValueFilter.html#mode)
     */
    mode: "include" | "exclude";

    readonly type: "value";
    /**
     * An array of numeric values representing the classification codes that the filter should apply.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudValueFilter.html#values)
     */
    values: number[];

    static fromJSON(json: any): PointCloudValueFilter;
  }

  interface PointCloudValueFilterProperties extends PointCloudFilterProperties {
    /**
     * Whether points should be included or excluded from the filter.
     *
     * @default "exclude"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudValueFilter.html#mode)
     */
    mode?: "include" | "exclude";
    /**
     * An array of numeric values representing the classification codes that the filter should apply.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudValueFilter.html#values)
     */
    values?: number[];
  }

  export interface PointCloudLayer
    extends Layer,
      SceneService,
      PortalLayer,
      ScaleRangeLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer {}

  export class PointCloudLayer {
    /**
     * The PointCloudLayer is designed for visualizing large collections of points in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html)
     */

    constructor(properties?: PointCloudLayerProperties);

    /**
     * Specifies how points are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#elevationInfo)
     */
    elevationInfo: PointCloudLayerElevationInfo;
    /**
     * An array of fields accessible in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * An array of [pointCloudFilters](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html) used to filter points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#filters)
     */
    filters: (PointCloudValueFilter | PointCloudReturnFilter | PointCloudBitfieldFilter)[];
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when points in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#renderer)
     */
    renderer: PointCloudRenderer;

    readonly type: "point-cloud";

    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string): Domain;
    /**
     * Queries cached statistics from the service for a given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#queryCachedStatistics)
     */
    queryCachedStatistics(fieldName: string, options?: PointCloudLayerQueryCachedStatisticsOptions): any;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem, options?: PointCloudLayerSaveAsOptions): Promise<PortalItem>;

    static fromJSON(json: any): PointCloudLayer;
  }

  interface PointCloudLayerProperties
    extends LayerProperties,
      SceneServiceProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties {
    /**
     * Specifies how points are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#elevationInfo)
     */
    elevationInfo?: PointCloudLayerElevationInfo;
    /**
     * An array of fields accessible in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * An array of [pointCloudFilters](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-pointCloudFilters-PointCloudFilter.html) used to filter points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#filters)
     */
    filters?: (
      | (PointCloudValueFilterProperties & { type: "value" })
      | (PointCloudReturnFilterProperties & { type: "return" })
      | (PointCloudBitfieldFilterProperties & { type: "bitfield" })
    )[];
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when points in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#renderer)
     */
    renderer?: PointCloudRendererProperties;
  }

  export interface PointCloudLayerElevationInfo {
    /**
     * Defines how the points are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#elevationInfo)
     */
    mode: "absolute-height";
    /**
     * An elevation offset, which is added to the vertical position of each point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * The unit for `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface PointCloudLayerQueryCachedStatisticsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#queryCachedStatistics)
     */
    signal?: AbortSignal;
  }

  export interface PointCloudLayerSaveAsOptions {
    /**
     * the folder where to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html#saveAs)
     */
    folder?: PortalFolder;
  }

  export interface RouteLayer extends Layer, BlendLayer, OperationalLayer, PortalLayer, ScaleRangeLayer {}

  export class RouteLayer {
    /**
     * RouteLayer is a layer for visualizing and solving routes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html)
     */

    constructor(properties?: RouteLayerProperties);

    /**
     * The default symbol used for new stops and route results generated by [solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#solve).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#defaultSymbols)
     */
    defaultSymbols: RouteSymbols;
    /**
     * A collection of [DirectionLine](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html) that represent the linear path between [directionPoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionPoints) of the solved route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionLines)
     */
    readonly directionLines: Collection<DirectionLine>;
    /**
     * A collection of [DirectionPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html) that represent the turn-by-turn directions along a solved route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionPoints)
     */
    readonly directionPoints: Collection<DirectionPoint>;
    /**
     * A collection of [PointBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#pointBarriers)
     */
    pointBarriers: Collection<PointBarrier>;
    /**
     * A collection of [PolygonBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polygonBarriers)
     */
    polygonBarriers: Collection<PolygonBarrier>;
    /**
     * A collection of [PolylineBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polylineBarriers)
     */
    polylineBarriers: Collection<PolylineBarrier>;
    /**
     * This property contains the solved route's geometry and other route information like overall time and distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#routeInfo)
     */
    readonly routeInfo: RouteInfo;
    /**
     * A collection of [Stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html)s define the start, end, waypoint, or break of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops)
     */
    stops: Collection<Stop>;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets and in the portal item created by [saveAs()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#saveas).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#title)
     */
    title: string;

    readonly type: "route";
    /**
     * The URL of the REST endpoint of the Route service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#url)
     */
    url: string;

    /**
     * Update the currently assigned [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) using information in this RouteLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the route layer to a new [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#saveAs)
     */
    saveAs(portalItem: Promise<PortalItem>, options?: RouteLayerSaveAsOptions): Promise<PortalItem>;
    /**
     * This method will solve the route using the route service referenced by [routeServiceUrl](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#routeServiceUrl) and the network objects [pointBarriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#pointBarriers), [polylineBarriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polylineBarriers), [polygonBarriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polygonBarriers), and [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#solve)
     */
    solve(routeParameters: RouteParameters, requestOptions?: any): Promise<RouteLayerSolveResult>;
    /**
     * Updates the layer with the results from a solved route layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    update(routeLayerSolveResult: RouteLayerUpdateRouteLayerSolveResult): void;

    static fromJSON(json: any): RouteLayer;
  }

  interface RouteLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties {
    /**
     * The default symbol used for new stops and route results generated by [solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#solve).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#defaultSymbols)
     */
    defaultSymbols?: RouteSymbolsProperties;
    /**
     * A collection of [PointBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#pointBarriers)
     */
    pointBarriers?: CollectionProperties<PointBarrierProperties>;
    /**
     * A collection of [PolygonBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polygonBarriers)
     */
    polygonBarriers?: CollectionProperties<PolygonBarrierProperties>;
    /**
     * A collection of [PolylineBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html) are used to prevent navigation through the street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polylineBarriers)
     */
    polylineBarriers?: CollectionProperties<PolylineBarrierProperties>;
    /**
     * A collection of [Stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html)s define the start, end, waypoint, or break of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops)
     */
    stops?: CollectionProperties<StopProperties>;
    /**
     * The title of the layer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets and in the portal item created by [saveAs()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#saveas).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#title)
     */
    title?: string;
    /**
     * The URL of the REST endpoint of the Route service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#url)
     */
    url?: string;
  }

  export interface RouteLayerSaveAsOptions {
    /**
     * The folder in which to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#saveAs)
     */
    folder?: PortalFolder;
  }

  /**
   * The results of computing a route and directions.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
   */
  export interface RouteLayerSolveResult {
    /**
     * Represents polyline segments associated with individual direction items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    directionLines: Collection<DirectionLine>;
    /**
     * Represents direction items as points with various display information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    directionPoints: Collection<DirectionPoint>;
    /**
     * Point barrier(s) to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    pointBarriers?: Collection<PointBarrier>;
    /**
     * Polygon barrier(s) to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    polygonBarriers?: Collection<PolygonBarrier>;
    /**
     * Polyline barrier(s) to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    polylineBarriers?: Collection<PolylineBarrier>;
    /**
     * Contains information about a solved route including the routes geometry and overall distance and time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    routeInfo: RouteInfo;
    /**
     * Respresents the start, end, or midpoint of a route created using the [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#RouteLayerSolveResult)
     */
    stops: Collection<Stop>;
  }

  export interface RouteLayerUpdateRouteLayerSolveResult {
    /**
     * Collection of direction polylines associated with line segments between turns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    directionLines: Collection<DirectionLine>;
    /**
     * Collection of direction items as points with various display information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    directionPoints: Collection<DirectionPoint>;
    /**
     * Collection of point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    readPointBarriers: Collection<PointBarrier>;
    /**
     * Collection of polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    polygonBarriers: Collection<PolygonBarrier>;
    /**
     * Collection of polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    polylineBarriers: Collection<PolylineBarrier>;
    /**
     * Information about a solved route including the routes geometry and overall distance and time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    routeInfo: RouteInfo;
    /**
     * Collection of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#update)
     */
    stops: Collection<Stop>;
  }

  export interface SceneLayer
    extends Layer,
      SceneService,
      PortalLayer,
      ScaleRangeLayer,
      TemporalSceneLayer,
      CustomParametersMixin,
      APIKeyMixin,
      Clonable {}

  export class SceneLayer {
    /**
     * The SceneLayer is a layer type designed for on-demand streaming and displaying large amounts of data in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html)
     */

    constructor(properties?: SceneLayerProperties);

    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#capabilities)
     */
    readonly capabilities: SceneLayerCapabilities;
    /**
     * The SQL where clause used to filter features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * Describes effective capabilities of the layer taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#effectiveCapabilities)
     */
    readonly effectiveCapabilities: SceneLayerCapabilities;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#elevationInfo)
     */
    elevationInfo: SceneLayerElevationInfo;
    /**
     * List of ObjectIDs not being displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#excludeObjectIds)
     */
    excludeObjectIds: Collection<number>;
    /**
     * Configures the method for decluttering overlapping features in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#featureReduction)
     */
    featureReduction: FeatureReductionSelection;
    /**
     * An array of fields accessible in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#fields)
     */
    readonly fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * A collection of polygons [geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#geometries) which will mask out different parts of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#filter)
     */
    filter: SceneFilter;
    /**
     * When a scene layer is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#floorInfo)
     */
    floorInfo: LayerFloorInfo;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#geometryType)
     */
    geometryType: "point" | "mesh";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The name of the field containing each graphic's Object ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Array of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html) set up for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#relationships)
     */
    readonly relationships: Relationship[];
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;

    readonly type: "scene";
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#useViewTime)
     */
    useViewTime: boolean;

    /**
     * Applies edits to the features in the associated [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    applyEdits(edits: SceneLayerApplyEditsEdits, options?: SceneLayerApplyEditsOptions): Promise<SceneLayerEditsResult>;
    /**
     * Converts a file or list of files to mesh geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#convertMesh)
     */
    convertMesh(files: File[], options?: ConvertMeshOptions): Promise<Mesh>;
    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates a query object that can be used to fetch features that satisfy the layer's current definition expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: SceneLayerGetFieldDomainOptions): Domain;
    /**
     * Gets field usage information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldUsageInfo)
     */
    getFieldUsageInfo(fieldName: string): any;
    /**
     * Query information about attachments associated with features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryAttachments)
     */
    queryAttachments(
      attachmentQuery: AttachmentQuery | AttachmentQueryProperties,
      options?: SceneLayerQueryAttachmentsOptions,
    ): Promise<any>;
    /**
     * Queries cached statistics from the service for a given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryCachedStatistics)
     */
    queryCachedStatistics(fieldName: string, options?: SceneLayerQueryCachedStatisticsOptions): any;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the 2D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the 2D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query. At the moment the 3D Extent can be returned by using [SceneLayerView.queryExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryExtent), but this will return the 3D extent only for features currently in the view. The query succeeds only if the SceneLayer has an associated feature layer. If an associated feature layer is not available, then an error with the name `scenelayer:query-not-available` is thrown. Read more about queries in the [Querying](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#querying) section of the class description above. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: SceneLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the number of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns the number of features that satisfy the query. The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `scenelayer:query-not-available` is thrown. Read more about queries in the [Querying](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#querying) section of the class description above. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: SceneLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html). The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `scenelayer:query-not-available` is thrown. Read more about queries in the [Querying](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#querying) section of the class description above. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: SceneLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns an array of ObjectIDs of the features that satisfy the input query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the associated feature service and returns an array of ObjectIDs of the features that satisfy the input query. The query succeeds only if the layer's `supportsLayerQuery` capability is enabled. Use the [getFieldUsageInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldUsageInfo) method to check if the layer supports queries. If querying is not enabled, then an error with the name `scenelayer:query-not-available` is thrown. Read more about queries in the [Querying](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#querying) section of the class description above. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: SceneLayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service associated with the scene layer and returns [FeatureSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) grouped by source layer or table objectIds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryRelatedFeatures)
     */
    queryRelatedFeatures(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SceneLayerQueryRelatedFeaturesOptions,
    ): Promise<any>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service associated with the scene layer and when resolved, it returns an `object` containing key value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryRelatedFeaturesCount)
     */
    queryRelatedFeaturesCount(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SceneLayerQueryRelatedFeaturesCountOptions,
    ): Promise<any>;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem, options?: SceneLayerSaveAsOptions): Promise<PortalItem>;

    on(name: "edits", eventHandler: SceneLayerEditsEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: SceneLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: SceneLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: SceneLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): SceneLayer;
  }

  interface SceneLayerProperties
    extends LayerProperties,
      SceneServiceProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      TemporalSceneLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties {
    /**
     * The SQL where clause used to filter features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#elevationInfo)
     */
    elevationInfo?: SceneLayerElevationInfo;
    /**
     * List of ObjectIDs not being displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#excludeObjectIds)
     */
    excludeObjectIds?: CollectionProperties<number>;
    /**
     * Configures the method for decluttering overlapping features in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#featureReduction)
     */
    featureReduction?: FeatureReductionSelectionProperties;
    /**
     * A collection of polygons [geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#geometries) which will mask out different parts of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#filter)
     */
    filter?: SceneFilterProperties;
    /**
     * When a scene layer is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#floorInfo)
     */
    floorInfo?: LayerFloorInfoProperties;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#geometryType)
     */
    geometryType?: "point" | "mesh";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The name of the field containing each graphic's Object ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#useViewTime)
     */
    useViewTime?: boolean;
  }

  /**
   * Describes the layer's supported capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
   */
  export interface SceneLayerCapabilities {
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    query: SceneLayerCapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    queryRelated: SceneLayerCapabilitiesQueryRelated;
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    data: SceneLayerCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer via [applyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    editing: SceneLayerCapabilitiesEditing;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    operations: SceneLayerCapabilitiesOperations;
  }

  /**
   * Options used to configure mesh conversion.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#ConvertMeshOptions)
   */
  export interface ConvertMeshOptions {
    /**
     * The location of the origin of the model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#ConvertMeshOptions)
     */
    location?: Point;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the loading process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#ConvertMeshOptions)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerEditsEvent {
    addedFeatures: FeatureEditResult[];

    deletedFeatures: FeatureEditResult[];

    exceededTransferLimit: boolean;

    updatedFeatures: FeatureEditResult[];
  }

  /**
   * Results returned from the [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#EditsResult)
   */
  export interface SceneLayerEditsResult {
    /**
     * Result of adding features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#EditsResult)
     */
    addFeatureResults: FeatureEditResult[];
    /**
     * Result of updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#EditsResult)
     */
    updateFeatureResults: FeatureEditResult[];
    /**
     * Result of deleting features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#EditsResult)
     */
    deleteFeatureResults: FeatureEditResult[];
  }

  /**
   * Alternative representation of features to be deleted with applyEdits.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#FeatureIdentifier)
   */
  export interface FeatureIdentifier {
    /**
     * The objectId of the feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#FeatureIdentifier)
     */
    objectId: number | any;
    /**
     * The globalId of the feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#FeatureIdentifier)
     */
    globalId: string | any;
  }

  export interface SceneLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface SceneLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface SceneLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface SceneLayerApplyEditsEdits {
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    addFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    updateFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features, or an array of objects with [globalId](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#globalIdField) of each feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    deleteFeatures?: Graphic[] | Collection<Graphic> | FeatureIdentifier[];
  }

  export interface SceneLayerApplyEditsOptions {
    /**
     * Indicates whether the edits should be applied only if all submitted edits succeed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    rollbackOnFailureEnabled?: boolean;
    /**
     * Indicates whether the edits can be applied using globalIds of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits)
     */
    globalIdUsed?: boolean;
  }

  export interface SceneLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * The unit for `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface SceneLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface SceneLayerQueryAttachmentsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryAttachments)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryCachedStatisticsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryCachedStatistics)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryRelatedFeaturesCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryRelatedFeaturesCount)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerQueryRelatedFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryRelatedFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerSaveAsOptions {
    /**
     * the folder where to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#saveAs)
     */
    folder?: PortalFolder;
  }

  export interface SceneLayerCapabilitiesData {
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsZ: boolean;
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsAttachment: boolean;
  }

  export interface SceneLayerCapabilitiesEditing {
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalId` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailureEnabled` parameter can be set to `true` or `false` when editing features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsRollbackOnFailure: boolean;
  }

  export interface SceneLayerCapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits) to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits) from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be [updated](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if features in the layer can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsQueryAttachments: boolean;
  }

  export interface SceneLayerCapabilitiesQuery {
    /**
     * The maximum number of records that will be returned for a given query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    maxRecordCount: number;
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the query operation supports a [cache hint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#cacheHint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the query operation supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the query operation supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsHistoricMoment: boolean;
  }

  export interface SceneLayerCapabilitiesQueryRelated {
    /**
     * Indicates if the [relationship query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) supports a cache hint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsCacheHint: boolean;
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#Capabilities)
     */
    supportsPagination: boolean;
  }

  export interface StreamLayer
    extends Layer,
      ScaleRangeLayer,
      TemporalLayer,
      BlendLayer,
      FeatureEffectLayer,
      FeatureReductionLayer,
      OperationalLayer,
      Clonable {}

  export class StreamLayer {
    /**
     * StreamLayer connects to a [stream service](https://enterprise.arcgis.com/en/geoevent) or a [custom WebSocket service](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#streamlayer-from-websocket), displaying the observation streams associated with a set of [tracked objects](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#track-aware) in real-time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html)
     */

    constructor(properties?: StreamLayerProperties);

    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#copyright)
     */
    copyright: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The SQL where clause used to filter features based on their attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    elevationInfo: StreamLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * An [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object used to filter features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#geometryDefinition)
     */
    geometryDefinition: Extent;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#geometryType)
     */
    geometryType: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The maximum number of attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#maxReconnectionAttempts)
     */
    maxReconnectionAttempts: number;
    /**
     * The maximum time to wait in seconds between attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#maxReconnectionInterval)
     */
    maxReconnectionInterval: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Options for purging stale features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    purgeOptions: StreamLayerPurgeOptions;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * The [stream service's metadata JSON](https://developers.arcgis.com/rest/services-reference/stream-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * For [StreamLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html) the type is `stream`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#type)
     */
    readonly type: "stream";
    /**
     * The minimum rate (ms) at which to poll for updates over the websocket connection.
     *
     * @default 300
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#updateInterval)
     */
    updateInterval: number;
    /**
     * The URL of the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#url)
     */
    url: string;
    /**
     * The URL of a websocket connection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#webSocketUrl)
     */
    webSocketUrl: string;

    /**
     * Establishes a connection to a web socket that satisfy the specified connection parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#connect)
     */
    connect(parameters?: ConnectionParameters): Promise<StreamConnection>;
    /**
     * Creates a [ConnectionParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters) object that can be used to establish a connection to a web socket that satisfies the layer's configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#createConnectionParameters)
     */
    createConnectionParameters(): ConnectionParameters;
    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: StreamLayerGetFieldDomainOptions): Domain;
    /**
     * Saves the layer to its existing portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#save)
     */
    save(options?: StreamLayerSaveOptions): Promise<PortalItem>;
    /**
     * Saves the layer to a new portal item in the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html) authenticated within the user's current session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: StreamLayerSaveAsOptions): Promise<PortalItem>;
    /**
     * Sends client-side only messages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#sendMessageToClient)
     */
    sendMessageToClient(message: any): void;
    /**
     * Sends a message over the websocket to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#sendMessageToSocket)
     */
    sendMessageToSocket(message: any): void;

    static fromJSON(json: any): StreamLayer;
  }

  interface StreamLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties,
      OperationalLayerProperties {
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#copyright)
     */
    copyright?: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The SQL where clause used to filter features based on their attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    elevationInfo?: StreamLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * An [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object used to filter features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#geometryDefinition)
     */
    geometryDefinition?: ExtentProperties;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#geometryType)
     */
    geometryType?: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The maximum number of attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#maxReconnectionAttempts)
     */
    maxReconnectionAttempts?: number;
    /**
     * The maximum time to wait in seconds between attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#maxReconnectionInterval)
     */
    maxReconnectionInterval?: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Options for purging stale features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    purgeOptions?: StreamLayerPurgeOptions;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * The [stream service's metadata JSON](https://developers.arcgis.com/rest/services-reference/stream-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The minimum rate (ms) at which to poll for updates over the websocket connection.
     *
     * @default 300
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#updateInterval)
     */
    updateInterval?: number;
    /**
     * The URL of the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#url)
     */
    url?: string;
    /**
     * The URL of a websocket connection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#webSocketUrl)
     */
    webSocketUrl?: string;
  }

  /**
   * The connection parameters that can be used to establish a connection to a web socket when the [connect()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#connect) method is called.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
   */
  export interface ConnectionParameters {
    /**
     * The spatial reference of the geometries emitted by the connection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    spatialReference?: string;
    /**
     * A list of custom parameters appended to the URL of the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    customParameters?: any;
    /**
     * The SQL where clause used to filter features based on their attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    definitionExpression?: string;
    /**
     * An [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object used to filter features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    geometryDefinition?: Extent;
    /**
     * The maximum number of attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    maxReconnectionAttempts?: number;
    /**
     * The maximum time to wait in seconds between attempts to reconnect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#ConnectionParameters)
     */
    maxReconnectionInterval?: number;
  }

  export interface StreamLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    featureExpressionInfo?: StreamLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface StreamLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface StreamLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface StreamLayerPurgeOptions {
    /**
     * Specifies the maximum age of a feature in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    age?: number;
    /**
     * Specifies the maximum age of a feature in minutes since it arrived in the application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    ageReceived?: number;
    /**
     * Controls the overall maximum number of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    displayCount?: number;
    /**
     * Controls the maximum number of observations to show within a [track](https://enterprise.arcgis.com/en/geoevent/latest/get-started/essential-geoevent-server-vocabulary.htm#ESRI_SECTION1_F45BBCE9ADFA4E57AF38DD225921EFCD).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#purgeOptions)
     */
    maxObservations?: number;
  }

  export interface StreamLayerSaveAsOptions {
    /**
     * The portal folder where the layer's portal item will be saved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#saveAs)
     */
    validationOptions?: StreamLayerSaveAsOptionsValidationOptions;
  }

  export interface StreamLayerSaveAsOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface StreamLayerSaveOptions {
    /**
     * Options for validating the save operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#save)
     */
    validationOptions?: StreamLayerSaveOptionsValidationOptions;
  }

  export interface StreamLayerSaveOptionsValidationOptions {
    /**
     * Indicates whether to ignore saving unsupported layers or layers with unsupported content, such as unsupported symbology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export interface SubtypeGroupLayer
    extends Layer,
      PortalLayer,
      ScaleRangeLayer,
      RefreshableLayer,
      TemporalLayer,
      BlendLayer,
      CustomParametersMixin,
      FeatureLayerBase,
      OperationalLayer,
      Clonable {}

  export class SubtypeGroupLayer {
    /**
     * The SubtypeGroupLayer is a single layer that automatically creates a [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) for each subtype in its corresponding [feature service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-feature-service-.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html)
     */

    constructor(properties?: SubtypeGroupLayerProperties);

    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#fields)
     */
    fields: Field[];
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#outFields)
     */
    outFields: string[];
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) objects that allow you to alter the properties of one or more sublayers of the SubtypeGroupLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#sublayers)
     */
    sublayers: Collection<SubtypeSublayer>;

    readonly type: "subtype-group";

    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations such as [definitionExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#definitionExpression), [gdbVersion](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#gdbVersion), and [historicMoment](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#historicMoment).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the sublayer to which the given feature belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#findSublayerForFeature)
     */
    findSublayerForFeature(feature: Graphic): SubtypeSublayer;
    /**
     * Returns the sublayer with the [subtypeCode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#subtypeCode) that matches the number passed as an argument.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#findSublayerForSubtypeCode)
     */
    findSublayerForSubtypeCode(subtypeCode: number): SubtypeSublayer;
    /**
     * Loads all of the sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#loadAll)
     */
    loadAll(): Promise<Sublayer>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: SubtypeGroupLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: SubtypeGroupLayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: SubtypeGroupLayerQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: SubtypeGroupLayerQueryObjectIdsOptions,
    ): Promise<number[]>;

    on(name: "refresh", eventHandler: SubtypeGroupLayerRefreshEventHandler): IHandle;

    on(name: "edits", eventHandler: SubtypeGroupLayerEditsEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: SubtypeGroupLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: SubtypeGroupLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: SubtypeGroupLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): SubtypeGroupLayer;
  }

  interface SubtypeGroupLayerProperties
    extends LayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      TemporalLayerProperties,
      BlendLayerProperties,
      CustomParametersMixinProperties,
      FeatureLayerBaseProperties,
      OperationalLayerProperties {
    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * An array of field names from the service to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) objects that allow you to alter the properties of one or more sublayers of the SubtypeGroupLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#sublayers)
     */
    sublayers?: CollectionProperties<SubtypeSublayerProperties>;
  }

  export interface SubtypeGroupLayerEditsEvent {
    addedAttachments: FeatureEditResult[];

    addedFeatures: FeatureEditResult[];

    deletedAttachments: FeatureEditResult[];

    deletedFeatures: FeatureEditResult[];

    updatedAttachments: FeatureEditResult[];

    updatedFeatures: FeatureEditResult[];
  }

  export interface SubtypeGroupLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface SubtypeGroupLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface SubtypeGroupLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface SubtypeGroupLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface SubtypeGroupLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeGroupLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeGroupLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeGroupLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface AggregateField extends Accessor, JSONSupport {}

  export class AggregateField {
    /**
     * Defines the aggregate fields used in a layer visualized with [FeatureReductionBinning](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html) or [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html)
     */

    constructor(properties?: AggregateFieldProperties);

    /**
     * The display name that describes the aggregate field in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html), [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), and other UI elements.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#alias)
     */
    alias: string;
    /**
     * Indicates whether the field was created internally by the JS API's rendering engine for default [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualizations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#isAutoGenerated)
     */
    isAutoGenerated: boolean;
    /**
     * The name of the aggregate field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#name)
     */
    name: string;
    /**
     * An object containing an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression, which evaluates for each child feature represented by the aggregate graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticExpression)
     */
    onStatisticExpression: supportExpressionInfo;
    /**
     * The name of the [layer field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) to summarize with the given [statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticField)
     */
    onStatisticField: string;
    /**
     * Defines the type of statistic used to aggregate data returned from [onStatisticField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticField) or [onStatisticExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#statisticType)
     */
    statisticType: "avg" | "count" | "max" | "min" | "mode" | "sum" | "avg_angle";

    /**
     * Creates a deep clone of the AggregateField object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#clone)
     */
    clone(): AggregateField;

    static fromJSON(json: any): AggregateField;
  }

  interface AggregateFieldProperties {
    /**
     * The display name that describes the aggregate field in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html), [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), and other UI elements.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#alias)
     */
    alias?: string;
    /**
     * Indicates whether the field was created internally by the JS API's rendering engine for default [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualizations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#isAutoGenerated)
     */
    isAutoGenerated?: boolean;
    /**
     * The name of the aggregate field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#name)
     */
    name?: string;
    /**
     * An object containing an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression, which evaluates for each child feature represented by the aggregate graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticExpression)
     */
    onStatisticExpression?: supportExpressionInfoProperties;
    /**
     * The name of the [layer field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) to summarize with the given [statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticField)
     */
    onStatisticField?: string;
    /**
     * Defines the type of statistic used to aggregate data returned from [onStatisticField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticField) or [onStatisticExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#onStatisticExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#statisticType)
     */
    statisticType?: "avg" | "count" | "max" | "min" | "mode" | "sum" | "avg_angle";
  }

  /**
   * Contains convenience methods for getting Arcade expressions defined on a layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html)
   */
  interface arcadeUtils {
    /**
     * Returns all Arcade expressions defined on a given layer and provides metadata describing the context for which each expression was authored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#getExpressionsFromLayer)
     */
    getExpressionsFromLayer(
      layer:
        | CSVLayer
        | FeatureLayer
        | GeoJSONLayer
        | Sublayer
        | OGCFeatureLayer
        | SceneLayer
        | WFSLayer
        | OrientedImageryLayer
        | CatalogFootprintLayer
        | KnowledgeGraphSublayer,
    ): arcadeUtilsExpressionInfo[];
  }

  export const arcadeUtils: arcadeUtils;

  /**
   * Represents the result of the [getExpressionsFromLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#getExpressionsFromLayer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
   */
  export interface arcadeUtilsExpressionInfo {
    /**
     * An Arcade expression defined on a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    expression: string;
    /**
     * The name of the expression used when referencing it in popup and form templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    name?: string;
    /**
     * The expression's title, describing it in the legend and other UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    title?: string;
    /**
     * Information about the profile, or environment, where the expression was defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    profileInfo: ExpressionInfoProfileInfo;
  }

  /**
   * Describes the context for which an Arcade expression was defined using the Feature Z profile.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#FeatureZProfileContext)
   */
  export type FeatureZProfileContext = "order-by" | "elevation-info";

  /**
   * Describes the context for which an Arcade expression was defined using the popup or popup element profile.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#PopupProfileContext)
   */
  export type PopupProfileContext = "popup-template" | "popup-expression-content";

  /**
   * Describes the context for which an Arcade expression was defined.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ProfileContext)
   */
  export type ProfileContext =
    | VisualizationProfileContext
    | PopupProfileContext
    | FeatureZProfileContext
    | "label-class"
    | "form-template"
    | "aggregate-field";

  /**
   * Describes the context for which an Arcade expression was defined using the visualization profile.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#VisualizationProfileContext)
   */
  export type VisualizationProfileContext =
    | "unique-value-renderer"
    | "class-breaks-renderer"
    | "dot-density-renderer"
    | "heatmap-renderer"
    | "pie-chart-renderer"
    | "color-variable"
    | "size-variable"
    | "opacity-variable"
    | "rotation-variable";

  export interface ExpressionInfoProfileInfo {
    /**
     * Indicates which [profile](https://developers.arcgis.com/arcade/profiles/) is used to execute the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    name:
      | "aggregate-field"
      | "form-constraint"
      | "feature-z"
      | "field-calculation"
      | "form-calculation"
      | "labeling"
      | "popup"
      | "popup-element"
      | "feature-reduction-popup"
      | "feature-reduction-popup-element"
      | "visualization";
    /**
     * Describes the context (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    context: ProfileContext;
    /**
     * Indicates the data source used to hydrate the expression's profile variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    source: "layer" | "feature-reduction";
    /**
     * The return type of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-arcadeUtils.html#ExpressionInfo)
     */
    returnType?: "boolean" | "date" | "number" | "string" | "dictionary";
  }

  export interface BuildingFilter extends Accessor, JSONSupport {}

  export class BuildingFilter {
    /**
     * The `BuildingFilter` class defines a set of conditions that can be used to show or hide specific features of a [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html)
     */

    constructor(properties?: BuildingFilterProperties);

    /**
     * Description of the filter for display in UIs, for example when the filter is edited in ArcGIS Pro.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#description)
     */
    description: string;
    /**
     * Filter blocks define which features should be visible in the affected [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) and how the filtered features are drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#filterBlocks)
     */
    filterBlocks: Collection<BuildingFilterBlock>;
    /**
     * Unique filter id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#id)
     */
    readonly id: string;
    /**
     * Name of the filter for display in UIs, for example when the filter is edited in ArcGIS Pro.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#name)
     */
    name: string;

    static fromJSON(json: any): BuildingFilter;
  }

  interface BuildingFilterProperties {
    /**
     * Description of the filter for display in UIs, for example when the filter is edited in ArcGIS Pro.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#description)
     */
    description?: string;
    /**
     * Filter blocks define which features should be visible in the affected [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) and how the filtered features are drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#filterBlocks)
     */
    filterBlocks?: CollectionProperties<BuildingFilterBlock>;
    /**
     * Name of the filter for display in UIs, for example when the filter is edited in ArcGIS Pro.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#name)
     */
    name?: string;
  }

  /**
   * Object contained in the [filterBlocks](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#filterBlocks) collection:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#BuildingFilterBlock)
   */
  export interface BuildingFilterBlock {
    /**
     * The SQL where clause used to match features to this filter block.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#BuildingFilterBlock)
     */
    filterExpression: string;
    /**
     * This object contains information about the filter mode of the filter block.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#BuildingFilterBlock)
     */
    filterMode?: BuildingFilterBlockFilterMode;
    /**
     * Title of the filter for display in UIs, for example when the filter block is edited in ArcGIS Pro.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#BuildingFilterBlock)
     */
    title?: string;
  }

  export interface BuildingFilterBlockFilterMode {
    /**
     * Defines how features are drawn while the filter is active.
     *
     * @default solid
     *
     * [Read more...](global.html#type)
     */
    type?: "solid" | "wire-frame" | "x-ray";
    /**
     * Defines the type of edges used to display a [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) filtered in `wire-frame` mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingFilter.html#BuildingFilterBlock)
     */
    edges?: SolidEdges3D | SketchEdges3D;
  }

  export interface BuildingSummaryStatistics extends Accessor, Loadable, JSONSupport {}

  export class BuildingSummaryStatistics {
    /**
     * Contains statistics about the fields in all sublayers of a [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html)
     */

    constructor(properties?: BuildingSummaryStatisticsProperties);

    /**
     * An array of statistics on all fields in all sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#fields)
     */
    fields: BuildingFieldStatistics[];

    static fromJSON(json: any): BuildingSummaryStatistics;
  }

  interface BuildingSummaryStatisticsProperties extends LoadableProperties {
    /**
     * An array of statistics on all fields in all sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#fields)
     */
    fields?: BuildingFieldStatistics[];
  }

  /**
   * Summary statistics for a field in a [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
   */
  export interface BuildingFieldStatistics {
    /**
     * Name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    fieldName: string;
    /**
     * Model Name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    modelName: string;
    /**
     * Label of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    label: string;
    /**
     * Minimum value of the field for numeric values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    min: number;
    /**
     * Maximum value of the field for numeric values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    max: number;
    /**
     * List of the most frequent values of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    mostFrequentValues: string[] | number[];
    /**
     * List of sublayer ids where this field is available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-BuildingSummaryStatistics.html#BuildingFieldStatistics)
     */
    subLayerIds: number[];
  }

  export class CodedValueDomain extends Domain {
    /**
     * Information about the coded values belonging to the domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html)
     */

    constructor(properties?: CodedValueDomainProperties);

    /**
     * An array of the coded values in the domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#codedValues)
     */
    codedValues: CodedValue[];

    readonly type: "coded-value";

    /**
     * Returns the name of the coded-value associated with the specified code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#getName)
     */
    getName(code: string | number): string;

    static fromJSON(json: any): CodedValueDomain;
  }

  interface CodedValueDomainProperties extends DomainProperties {
    /**
     * An array of the coded values in the domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#codedValues)
     */
    codedValues?: CodedValue[];
  }

  /**
   * The coded value in a domain.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#CodedValue)
   */
  export interface CodedValue {
    /**
     * The name of the coded value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#CodedValue)
     */
    name: string;
    /**
     * The value of the code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html#CodedValue)
     */
    code: string | number;
  }

  export class ControlPointsGeoreference extends Accessor {
    /**
     * ControlPointsGeoreference is used to set the [geographic location](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#georeference) of the [ImageElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) or [VideoElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) referenced in the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) using the [controlPoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints), [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#width), and [height](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#height) parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html)
     */

    constructor(properties?: ControlPointsGeoreferenceProperties);

    /**
     * An array of two, three, or four [controlPoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints) positions the media element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints)
     */
    controlPoints: ControlPoint[];
    /**
     * Defines the size of the element displayed, typically the element's height in pixels.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#height)
     */
    height: number;

    type: "control-points";
    /**
     * Defines the size of the element displayed, typically the element's width in pixels.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#width)
     */
    width: number;

    /**
     * Converts the given [sourcePoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#SourcePoint) to a [mapPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#toMap)
     */
    toMap(sourcePoint: SourcePoint): Point;
    /**
     * Converts the given [mapPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) to a [sourcePoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#SourcePoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#toSource)
     */
    toSource(point: Point): SourcePoint;
  }

  interface ControlPointsGeoreferenceProperties {
    /**
     * An array of two, three, or four [controlPoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints) positions the media element.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints)
     */
    controlPoints?: ControlPoint[];
    /**
     * Defines the size of the element displayed, typically the element's height in pixels.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#height)
     */
    height?: number;

    type?: "control-points";
    /**
     * Defines the size of the element displayed, typically the element's width in pixels.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#width)
     */
    width?: number;
  }

  /**
   * A control point object in the [controlPoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#controlPoints) array.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#ControlPoint)
   */
  export interface ControlPoint {
    /**
     * The coordinate of the control point in the image coordinate space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#ControlPoint)
     */
    sourcePoint?: SourcePoint;
    /**
     * The coordinate of the the control point in the map coordinate space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#ControlPoint)
     */
    mapPoint?: Point;
  }

  /**
   * An object representing a point on the element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#SourcePoint)
   */
  export interface SourcePoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#SourcePoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ControlPointsGeoreference.html#SourcePoint)
     */
    y: number;
  }

  export class CornersGeoreference extends Accessor {
    /**
     * CornersGeoreference is used to set the [geographic location](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#georeference) of the [image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) or [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) referenced in the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) using corner points of a bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html)
     */

    constructor(properties?: CornersGeoreferenceProperties);

    /**
     * The bottom left [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#bottomLeft)
     */
    bottomLeft: Point;
    /**
     * The bottom right [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#bottomRight)
     */
    bottomRight: Point;
    /**
     * The top left [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#topLeft)
     */
    topLeft: Point;
    /**
     * The top right [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#topRight)
     */
    topRight: Point;
    /**
     * The georeference type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#type)
     */
    readonly type: "corners";
  }

  interface CornersGeoreferenceProperties {
    /**
     * The bottom left [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#bottomLeft)
     */
    bottomLeft?: PointProperties;
    /**
     * The bottom right [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#bottomRight)
     */
    bottomRight?: PointProperties;
    /**
     * The top left [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#topLeft)
     */
    topLeft?: PointProperties;
    /**
     * The top right [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) of the bounding box representing the geographic location of the image or video element being added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CornersGeoreference.html#topRight)
     */
    topRight?: PointProperties;
  }

  export interface DimensionalDefinition extends Accessor, JSONSupport {}

  export class DimensionalDefinition {
    /**
     * A dimensional definition defines a filter based on one variable and one dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html)
     */

    constructor(properties?: DimensionalDefinitionProperties);

    /**
     * The dimension associated with the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#dimensionName)
     */
    dimensionName: string;
    /**
     * Indicates whether the values indicate slices (rather than ranges).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#isSlice)
     */
    isSlice: boolean;
    /**
     * An array of single values or tuples [min, max] each defining a range of valid values along the specified dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#values)
     */
    values: (number | number[])[];
    /**
     * The required variable name by which to filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#variableName)
     */
    variableName: string;

    /**
     * Creates a clone of the DimensionalDefinition object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#clone)
     */
    clone(): DimensionalDefinition;

    static fromJSON(json: any): DimensionalDefinition;
  }

  interface DimensionalDefinitionProperties {
    /**
     * The dimension associated with the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#dimensionName)
     */
    dimensionName?: string;
    /**
     * Indicates whether the values indicate slices (rather than ranges).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#isSlice)
     */
    isSlice?: boolean;
    /**
     * An array of single values or tuples [min, max] each defining a range of valid values along the specified dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#values)
     */
    values?: (number | number[])[];
    /**
     * The required variable name by which to filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-DimensionalDefinition.html#variableName)
     */
    variableName?: string;
  }

  export interface Domain extends Accessor, JSONSupport {}

  export class Domain {
    /**
     * Domains define constraints on a [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html)
     */

    constructor(properties?: DomainProperties);

    /**
     * The domain name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html#name)
     */
    name: string;
    /**
     * The domain type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html#type)
     */
    type: "range" | "coded-value" | "inherited";

    static fromJSON(json: any): Domain;
  }

  interface DomainProperties {
    /**
     * The domain name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html#name)
     */
    name?: string;
    /**
     * The domain type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html#type)
     */
    type?: "range" | "coded-value" | "inherited";
  }

  export class ElevationSampler {
    /**
     * The minimum and maximum resolution of the data in the sampler.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#demResolution)
     */
    readonly demResolution: ElevationSamplerDemResolution;
    /**
     * The extent within which the sampler can be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#extent)
     */
    readonly extent: Extent;
    /**
     * The value that is used to represent areas where there is no data available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#noDataValue)
     */
    readonly noDataValue: number;
    /**
     * The spatial reference of the sampler.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;

    /**
     * Get elevation for a coordinate specified in the spatial reference of the sampler.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#elevationAt)
     */
    elevationAt(x: number, y: number): number;
    /**
     * Query elevation for a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) or [Multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#queryElevation)
     */
    queryElevation(geometry: Point | Multipoint | Polyline): Point | Multipoint | Polyline;

    on(name: "changed", eventHandler: ElevationSamplerChangedEventHandler): IHandle;
  }

  export interface ElevationSamplerChangedEvent {}

  export interface ElevationSamplerDemResolution {
    /**
     * The minimum resolution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#demResolution)
     */
    min: number;
    /**
     * The maximum resolution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ElevationSampler.html#demResolution)
     */
    max: number;
  }

  export interface supportExpressionInfo extends Accessor, JSONSupport, Clonable {}

  export class supportExpressionInfo {
    /**
     * The `ExpressionInfo` class references an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html)
     */

    constructor(properties?: supportExpressionInfoProperties);

    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#expression)
     */
    expression: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#returnType)
     */
    returnType: "string" | "number";
    /**
     * The title used to describe the value returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#title)
     */
    title: string;

    static fromJSON(json: any): supportExpressionInfo;
  }

  interface supportExpressionInfoProperties {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#expression)
     */
    expression?: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#returnType)
     */
    returnType?: "string" | "number";
    /**
     * The title used to describe the value returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExpressionInfo.html#title)
     */
    title?: string;
  }

  export class ExtentAndRotationGeoreference extends Accessor {
    /**
     * ExtentAndRotationGeoreference is used to set the [geographic location](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#georeference) of the [image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) or [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) referenced in the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) using the [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#extent) and [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#rotation) parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html)
     */

    constructor(properties?: ExtentAndRotationGeoreferenceProperties);

    /**
     * An extent of the image or video element representing its geographic location when added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#extent)
     */
    extent: Extent;
    /**
     * A rotation of the image or video element when added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#rotation)
     */
    rotation: number;
    /**
     * The georeference type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#type)
     */
    readonly type: "extent-and-rotation";
  }

  interface ExtentAndRotationGeoreferenceProperties {
    /**
     * An extent of the image or video element representing its geographic location when added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * A rotation of the image or video element when added to the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ExtentAndRotationGeoreference.html#rotation)
     */
    rotation?: number;
  }

  export interface FacilityLayerInfo extends Accessor, JSONSupport {}

  export class FacilityLayerInfo {
    /**
     * The FacilityLayerInfo describes the footprints of managed buildings and other structures.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html)
     */

    constructor(properties?: FacilityLayerInfoProperties);

    /**
     * The field name from the layer that defines the facility unique ID for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#facilityIdField)
     */
    facilityIdField: string;
    /**
     * Identifies an operational layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#layerId)
     */
    layerId: string;
    /**
     * The field name from the layer that defines the facility name for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#nameField)
     */
    nameField: string;
    /**
     * The field name from the layer that records the unique ID of a feature's associated site and can be used to identify a feature's associated site feature in floor-aware maps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#siteIdField)
     */
    siteIdField: string;
    /**
     * This value references the numeric ID of the sublayer if the Facility layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#sublayerId)
     */
    sublayerId: number;

    static fromJSON(json: any): FacilityLayerInfo;
  }

  interface FacilityLayerInfoProperties {
    /**
     * The field name from the layer that defines the facility unique ID for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#facilityIdField)
     */
    facilityIdField?: string;
    /**
     * Identifies an operational layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#layerId)
     */
    layerId?: string;
    /**
     * The field name from the layer that defines the facility name for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#nameField)
     */
    nameField?: string;
    /**
     * The field name from the layer that records the unique ID of a feature's associated site and can be used to identify a feature's associated site feature in floor-aware maps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#siteIdField)
     */
    siteIdField?: string;
    /**
     * This value references the numeric ID of the sublayer if the Facility layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FacilityLayerInfo.html#sublayerId)
     */
    sublayerId?: number;
  }

  export interface FeatureEffect extends Accessor, JSONSupport {}

  export class FeatureEffect {
    /**
     * FeatureEffect allows you to emphasize or deemphasize features that satisfy a [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter) in 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html)
     */

    constructor(properties?: FeatureEffectProperties);

    /**
     * The [effect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#Effect) applied to features that do not meet the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter) requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedEffect)
     */
    excludedEffect: Effect;
    /**
     * Indicates if labels are visible for features that are [excluded](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedEffect) from the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedLabelsVisible)
     */
    excludedLabelsVisible: boolean;
    /**
     * The [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) that drives the effect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter)
     */
    filter: FeatureFilter;
    /**
     * The [effect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#Effect) applied to features that meet the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter) requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#includedEffect)
     */
    includedEffect: Effect;

    /**
     * Creates a deep clone of FeatureEffect object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#clone)
     */
    clone(): FeatureEffect;

    static fromJSON(json: any): FeatureEffect;
  }

  interface FeatureEffectProperties {
    /**
     * The [effect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#Effect) applied to features that do not meet the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter) requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedEffect)
     */
    excludedEffect?: any[] | string;
    /**
     * Indicates if labels are visible for features that are [excluded](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedEffect) from the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#excludedLabelsVisible)
     */
    excludedLabelsVisible?: boolean;
    /**
     * The [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) that drives the effect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter)
     */
    filter?: FeatureFilterProperties;
    /**
     * The [effect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#Effect) applied to features that meet the [filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#filter) requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#includedEffect)
     */
    includedEffect?: any[] | string;
  }

  /**
   * Effect provides various filter functions that can be performed on a layer or a layerView to achieve different visual effects similar to how image filters (photo apps) work.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureEffect.html#Effect)
   */
  export type Effect = any[] | string;

  export interface FeatureFilter extends Accessor, JSONSupport {}

  export class FeatureFilter {
    /**
     * This class defines parameters for setting a client-side filter on a [featureEffect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#featureEffect) or [layer view](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerView.html#filter).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html)
     */

    constructor(properties?: FeatureFilterProperties);

    /**
     * Specifies a search distance from a given [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry) in a spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#distance)
     */
    distance: number;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry)
     */
    geometry: Geometry;
    /**
     * An array of objectIds of the features to be filtered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#objectIds)
     */
    objectIds: number[];
    /**
     * For spatial filters, this parameter defines the spatial relationship to filter features in the layer view against the filter [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry).
     *
     * @default "intersects"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#spatialRelationship)
     */
    spatialRelationship: string;
    /**
     * A range of time with start and end date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The unit for calculating the buffer distance when [distance](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#distance) is specified in a spatial filter.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#units)
     */
    units: "feet" | "miles" | "nautical-miles" | "us-nautical-miles" | "meters" | "kilometers";
    /**
     * A where clause for the feature filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where)
     */
    where: string;

    /**
     * Creates a deep clone of the FeatureFilter object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#clone)
     */
    clone(): FeatureFilter;
    /**
     * Creates [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) parameters that can be used to fetch features that satisfy the layer's current filters and definitions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#createQuery)
     */
    createQuery(): Query;

    static fromJSON(json: any): FeatureFilter;
  }

  interface FeatureFilterProperties {
    /**
     * Specifies a search distance from a given [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry) in a spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#distance)
     */
    distance?: number;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * An array of objectIds of the features to be filtered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#objectIds)
     */
    objectIds?: number[];
    /**
     * For spatial filters, this parameter defines the spatial relationship to filter features in the layer view against the filter [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry).
     *
     * @default "intersects"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#spatialRelationship)
     */
    spatialRelationship?: string;
    /**
     * A range of time with start and end date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The unit for calculating the buffer distance when [distance](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#distance) is specified in a spatial filter.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#units)
     */
    units?: "feet" | "miles" | "nautical-miles" | "us-nautical-miles" | "meters" | "kilometers";
    /**
     * A where clause for the feature filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where)
     */
    where?: string;
  }

  export interface FeatureReductionBinning extends Accessor, JSONSupport {}

  export class FeatureReductionBinning {
    /**
     * Aggregates and summarizes dense features in a layer to bins in geographic space based on predefined [geohashes](https://en.wikipedia.org/wiki/Geohash).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html)
     */

    constructor(properties?: FeatureReductionBinningProperties);

    /**
     * An array of aggregate fields that summarize layer [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) from features contained within each bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fields)
     */
    fields: AggregateField[];
    /**
     * The fixed geohash level used to create bins.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fixedBinLevel)
     */
    fixedBinLevel: number;
    /**
     * Defines labels for bins as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for the bins.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Defines the maximum [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which binning is enabled.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#maxScale)
     */
    maxScale: number;
    /**
     * Indicates whether to display a popup when a user clicks or touches a bin.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) to apply to bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer used to style the bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#renderer)
     */
    renderer: Renderer;
    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#type)
     */
    type: "binning";

    /**
     * Creates a deep clone of the FeatureReductionBinning object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#clone)
     */
    clone(): FeatureReductionBinning;

    static fromJSON(json: any): FeatureReductionBinning;
  }

  interface FeatureReductionBinningProperties {
    /**
     * An array of aggregate fields that summarize layer [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) from features contained within each bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fields)
     */
    fields?: AggregateFieldProperties[];
    /**
     * The fixed geohash level used to create bins.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fixedBinLevel)
     */
    fixedBinLevel?: number;
    /**
     * Defines labels for bins as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for the bins.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Defines the maximum [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which binning is enabled.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#maxScale)
     */
    maxScale?: number;
    /**
     * Indicates whether to display a popup when a user clicks or touches a bin.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) to apply to bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer used to style the bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#type)
     */
    type?: "binning";
  }

  export interface FeatureReductionCluster extends Accessor, JSONSupport {}

  export class FeatureReductionCluster {
    /**
     * Aggregates and summarizes dense features in a layer to clusters defined in screen space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html)
     */

    constructor(properties?: FeatureReductionClusterProperties);

    /**
     * Defines the symbol size of the largest cluster in points (or pixels if specified).
     *
     * @default 37.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterMaxSize)
     */
    clusterMaxSize: number;
    /**
     * Defines the symbol size of the smallest cluster in points (or pixels if specified).
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterMinSize)
     */
    clusterMinSize: number;
    /**
     * Defines the radius in points (or pixels if specified) of the area in which multiple features will be grouped and visualized as a single cluster.
     *
     * @default 60
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterRadius)
     */
    clusterRadius: number;
    /**
     * An array of aggregate fields that summarize layer [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) from features contained within each cluster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#fields)
     */
    fields: AggregateField[];
    /**
     * Defines labels for clusters as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for the clusters.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Defines the maximum [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which clustering is enabled.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#maxScale)
     */
    maxScale: number;
    /**
     * Indicates whether to display the cluster popup.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) to apply to clustered graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer used to override the default style of the clusters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#renderer)
     */
    renderer: Renderer;
    /**
     * A symbol used to override the default cluster style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#symbol)
     */
    symbol: PictureMarkerSymbol | SimpleMarkerSymbol | TextSymbol | CIMSymbol | WebStyleSymbol;
    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#type)
     */
    type: "cluster";

    /**
     * Creates a deep clone of the FeatureReductionCluster object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clone)
     */
    clone(): FeatureReductionCluster;

    static fromJSON(json: any): FeatureReductionCluster;
  }

  interface FeatureReductionClusterProperties {
    /**
     * Defines the symbol size of the largest cluster in points (or pixels if specified).
     *
     * @default 37.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterMaxSize)
     */
    clusterMaxSize?: number | string;
    /**
     * Defines the symbol size of the smallest cluster in points (or pixels if specified).
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterMinSize)
     */
    clusterMinSize?: number | string;
    /**
     * Defines the radius in points (or pixels if specified) of the area in which multiple features will be grouped and visualized as a single cluster.
     *
     * @default 60
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterRadius)
     */
    clusterRadius?: number | string;
    /**
     * An array of aggregate fields that summarize layer [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#fields) from features contained within each cluster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#fields)
     */
    fields?: AggregateFieldProperties[];
    /**
     * Defines labels for clusters as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for the clusters.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Defines the maximum [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which clustering is enabled.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#maxScale)
     */
    maxScale?: number;
    /**
     * Indicates whether to display the cluster popup.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) to apply to clustered graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer used to override the default style of the clusters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * A symbol used to override the default cluster style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#symbol)
     */
    symbol?:
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (TextSymbolProperties & { type: "text" })
      | (CIMSymbolProperties & { type: "cim" })
      | (WebStyleSymbolProperties & { type: "web-style" });
    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#type)
     */
    type?: "cluster";
  }

  export interface FeatureReductionSelection extends Accessor, JSONSupport {}

  export class FeatureReductionSelection {
    /**
     * Declutters points in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) by thinning overlapping features so no features intersect on screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionSelection.html)
     */

    constructor(properties?: FeatureReductionSelectionProperties);

    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionSelection.html#type)
     */
    type: "selection";

    static fromJSON(json: any): FeatureReductionSelection;
  }

  interface FeatureReductionSelectionProperties {
    /**
     * The feature reduction type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionSelection.html#type)
     */
    type?: "selection";
  }

  export interface FeatureTemplate extends Accessor, JSONSupport, Clonable {}

  export class FeatureTemplate {
    /**
     * Feature templates define all the information required to create a new feature in a [feature layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html)
     */

    constructor(properties?: FeatureTemplateProperties);

    /**
     * Description of the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#description)
     */
    description: string;
    /**
     * Name of the default drawing tool defined for the template to create a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#drawingTool)
     */
    drawingTool:
      | "auto-complete-polygon"
      | "circle"
      | "ellipse"
      | "freehand"
      | "line"
      | "none"
      | "point"
      | "polygon"
      | "rectangle"
      | "arrow"
      | "triangle"
      | "left-arrow"
      | "right-arrow"
      | "up-arrow"
      | "down-arrow";
    /**
     * Name of the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#name)
     */
    name: string;
    /**
     * An instance of the prototypical feature described by the [feature template](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#prototype)
     */
    prototype: any;
    /**
     * An object used to create a thumbnail image that represents a feature type in the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    thumbnail: FeatureTemplateThumbnail;

    static fromJSON(json: any): FeatureTemplate;
  }

  interface FeatureTemplateProperties {
    /**
     * Description of the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#description)
     */
    description?: string;
    /**
     * Name of the default drawing tool defined for the template to create a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#drawingTool)
     */
    drawingTool?:
      | "auto-complete-polygon"
      | "circle"
      | "ellipse"
      | "freehand"
      | "line"
      | "none"
      | "point"
      | "polygon"
      | "rectangle"
      | "arrow"
      | "triangle"
      | "left-arrow"
      | "right-arrow"
      | "up-arrow"
      | "down-arrow";
    /**
     * Name of the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#name)
     */
    name?: string;
    /**
     * An instance of the prototypical feature described by the [feature template](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#prototype)
     */
    prototype?: any;
    /**
     * An object used to create a thumbnail image that represents a feature type in the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    thumbnail?: FeatureTemplateThumbnail;
  }

  export interface FeatureTemplateThumbnail {
    /**
     * The MIME type of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    contentType: "image" | "png" | "jpg" | "jpeg";
    /**
     * The [base64EncodedImageData](https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding) presenting the thumbnail image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    imageData: string;
    /**
     * The height of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    height: number;
    /**
     * The width of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html#thumbnail)
     */
    width: number;
  }

  export interface FeatureType extends Accessor, JSONSupport, Clonable {}

  export class FeatureType {
    /**
     * FeatureType is a subset of features defined in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) that share the same attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html)
     */

    constructor(properties?: FeatureTypeProperties);

    /**
     * Domains associated with the feature type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#domains)
     */
    domains: HashMap<CodedValueDomain | RangeDomain | InheritedDomain>;
    /**
     * The feature type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#id)
     */
    id: number | string;
    /**
     * The feature type name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#name)
     */
    name: string;
    /**
     * Array of [feature templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html) associated with the feature type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#templates)
     */
    templates: FeatureTemplate[];

    static fromJSON(json: any): FeatureType;
  }

  interface FeatureTypeProperties {
    /**
     * Domains associated with the feature type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#domains)
     */
    domains?: HashMap<
      | (CodedValueDomainProperties & { type: "coded-value" })
      | (RangeDomainProperties & { type: "range" })
      | (InheritedDomainProperties & { type: "inherited" })
    >;
    /**
     * The feature type identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#id)
     */
    id?: number | string;
    /**
     * The feature type name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#name)
     */
    name?: string;
    /**
     * Array of [feature templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html) associated with the feature type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html#templates)
     */
    templates?: FeatureTemplateProperties[];
  }

  export interface Field extends Accessor, JSONSupport {}

  export class Field {
    /**
     * Information about each field in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html)
     */

    constructor(properties?: FieldProperties);

    /**
     * The display name for the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#alias)
     */
    alias: string;
    /**
     * The default value set for the field.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#defaultValue)
     */
    defaultValue: number | string;
    /**
     * Contains information describing the purpose of each field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#description)
     */
    description: string;
    /**
     * The domain associated with the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#domain)
     */
    domain: CodedValueDomain | RangeDomain | InheritedDomain;
    /**
     * Indicates whether the field is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#editable)
     */
    editable: boolean;
    /**
     * The field length.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#length)
     */
    length: number;
    /**
     * The name of the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#name)
     */
    name: string;
    /**
     * Indicates if the field can accept `null` values.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#nullable)
     */
    nullable: boolean;
    /**
     * The data type of the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type)
     */
    type:
      | "small-integer"
      | "integer"
      | "big-integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "date-only"
      | "time-only"
      | "timestamp-offset"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";
    /**
     * The types of values that can be assigned to a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#valueType)
     */
    valueType:
      | "binary"
      | "coordinate"
      | "count-or-amount"
      | "date-and-time"
      | "description"
      | "location-or-place-name"
      | "measurement"
      | "name-or-title"
      | "none"
      | "ordered-or-ranked"
      | "percentage-or-ratio"
      | "type-or-category"
      | "unique-identifier";

    static fromJSON(json: any): Field;
  }

  interface FieldProperties {
    /**
     * The display name for the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#alias)
     */
    alias?: string;
    /**
     * The default value set for the field.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#defaultValue)
     */
    defaultValue?: number | string;
    /**
     * Contains information describing the purpose of each field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#description)
     */
    description?: string;
    /**
     * The domain associated with the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#domain)
     */
    domain?:
      | (CodedValueDomainProperties & { type: "coded-value" })
      | (RangeDomainProperties & { type: "range" })
      | (InheritedDomainProperties & { type: "inherited" });
    /**
     * Indicates whether the field is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#editable)
     */
    editable?: boolean;
    /**
     * The field length.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#length)
     */
    length?: number;
    /**
     * The name of the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#name)
     */
    name?: string;
    /**
     * Indicates if the field can accept `null` values.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#nullable)
     */
    nullable?: boolean;
    /**
     * The data type of the field.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type)
     */
    type?:
      | "small-integer"
      | "integer"
      | "big-integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "date-only"
      | "time-only"
      | "timestamp-offset"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";
    /**
     * The types of values that can be assigned to a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#valueType)
     */
    valueType?:
      | "binary"
      | "coordinate"
      | "count-or-amount"
      | "date-and-time"
      | "description"
      | "location-or-place-name"
      | "measurement"
      | "name-or-title"
      | "none"
      | "ordered-or-ranked"
      | "percentage-or-ratio"
      | "type-or-category"
      | "unique-identifier";
  }

  export class FieldsIndex {
    /**
     * An array of date fields or field json objects.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FieldsIndex.html#dateFields)
     */
    dateFields: Field[] | any[];

    /**
     * Returns a field with the specified field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FieldsIndex.html#get)
     */
    get(fieldName: string): Field;
    /**
     * Returns a [time zone](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones) for a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FieldsIndex.html#getTimeZone)
     */
    getTimeZone(fieldOrFieldName: string | Field): string;
    /**
     * Checks if a field with the specified field name exists in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FieldsIndex.html#has)
     */
    has(fieldName: string): boolean;
    /**
     * Checks if a field with the specified field name is a date field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FieldsIndex.html#isDateField)
     */
    isDateField(fieldName: string): boolean;

    constructor(properties?: any);
  }

  /**
   * Convenience methods for getting field names used for feature layer [labeling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelingInfo), [elevation](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#elevationInfo), [editor tracking](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#editFieldsInfo) and time span.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html)
   */
  interface fieldUtils {
    /**
     * Gets the appropriate display field name to label a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getDisplayFieldName)
     */
    getDisplayFieldName(
      layer: FeatureLayer | CSVLayer | GeoJSONLayer | OGCFeatureLayer | SceneLayer | StreamLayer,
    ): string;
    /**
     * Returns an array of field names used in the Arcade expression for calculating the z-values of features in the given feature layer's [FeatureLayer.elevationInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#elevationInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getElevationFields)
     */
    getElevationFields(layer: FeatureLayer): Promise<string[]>;
    /**
     * Returns an array of field names referenced in one or more [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions to be set on the given layer in either the renderer, labels, or popup template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getExpressionFields)
     */
    getExpressionFields(
      layer: FeatureLayer | CSVLayer | GeoJSONLayer | SceneLayer | StreamLayer,
      expressions: string[],
    ): Promise<string[]>;
    /**
     * Returns an array of editor tracking field names for a given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getFeatureEditFields)
     */
    getFeatureEditFields(layer: FeatureLayer | SubtypeGroupLayer): string[];
    /**
     * Returns an array of geometry field names for a given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getFeatureGeometryFields)
     */
    getFeatureGeometryFields(layer: FeatureLayer | SubtypeGroupLayer): string[];
    /**
     * Returns an array of field names used in the Arcade expression for labeling features in the given feature layer's [FeatureLayer.labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#labelingInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getLabelingFields)
     */
    getLabelingFields(layer: FeatureLayer): Promise<string[]>;
    /**
     * Returns an array of field names used in the layer's [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getRendererFields)
     */
    getRendererFields(renderer: Renderer, fieldsIndex: FieldsIndex): Promise<string[]>;
    /**
     * Returns an array of field names related to time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-fieldUtils.html#getTimeFields)
     */
    getTimeFields(layer: FeatureLayer): Promise<string[]>;
  }

  export const fieldUtils: fieldUtils;

  export interface GeometryFieldsInfo extends Accessor, JSONSupport {}

  export class GeometryFieldsInfo {
    /**
     * The `GeometryFieldsInfo` class returns information about the system maintained geometry information associated with a specific feature in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-GeometryFieldsInfo.html)
     */

    constructor(properties?: GeometryFieldsInfoProperties);

    /**
     * The name of the field that stores the feature's area, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-GeometryFieldsInfo.html#shapeAreaField)
     */
    readonly shapeAreaField: string;
    /**
     * The name of the field that stores the feature's length, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-GeometryFieldsInfo.html#shapeLengthField)
     */
    readonly shapeLengthField: string;
    /**
     * The units of measure for the area and length field values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-GeometryFieldsInfo.html#units)
     */
    readonly units: string;

    static fromJSON(json: any): GeometryFieldsInfo;
  }

  interface GeometryFieldsInfoProperties {}

  export interface ImageElement extends Accessor, Loadable, MediaElementBase {}

  export class ImageElement {
    /**
     * Represents an image element referenced in the MediaLayer's [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html)
     */

    constructor(properties?: ImageElementProperties);

    /**
     * The animation options for the image element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#animationOptions)
     */
    animationOptions: AnimationOptions;
    /**
     * The image content referenced in the image element instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#content)
     */
    readonly content: HTMLImageElement | HTMLCanvasElement | ImageData;
    /**
     * The image element to be added to the [media layer's source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#image)
     */
    image: string | HTMLImageElement | HTMLCanvasElement | ImageData;
    /**
     * The element type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#type)
     */
    readonly type: "image";
  }

  interface ImageElementProperties extends LoadableProperties, MediaElementBaseProperties {
    /**
     * The animation options for the image element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#animationOptions)
     */
    animationOptions?: AnimationOptions;
    /**
     * The image element to be added to the [media layer's source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#image)
     */
    image?: string | HTMLImageElement | HTMLCanvasElement | ImageData;
  }

  /**
   * Represents animation options, a collection of properties that apply when the [image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#image) is an animated GIF or APNG.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#AnimationOptions)
   */
  export interface AnimationOptions {
    /**
     * Indicates whether the animated image should play its animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#AnimationOptions)
     */
    playing?: boolean;
    /**
     * Represents the number of seconds it takes to play through the layer's animation once.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#AnimationOptions)
     */
    duration?: number;
    /**
     * Determines how to repeat the animation of a layer when the animation cycle ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#AnimationOptions)
     */
    repeatType?: "none" | "loop" | "oscillate";
    /**
     * Represents the number of seconds to delay before repeating an animation cycle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html#AnimationOptions)
     */
    repeatDelay?: number;
  }

  export class InheritedDomain extends Domain {
    /**
     * This is a subclass of [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-InheritedDomain.html)
     */

    constructor(properties?: InheritedDomainProperties);

    /**
     * The domain type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-InheritedDomain.html#type)
     */
    readonly type: "inherited";

    static fromJSON(json: any): InheritedDomain;
  }

  interface InheritedDomainProperties extends DomainProperties {}

  export interface KMLSublayer extends Accessor, JSONSupport {}

  export class KMLSublayer {
    /**
     * Represents a sublayer in a [KMLLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html)
     */

    constructor(properties?: KMLSublayerProperties);

    /**
     * Description for the KML sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#description)
     */
    description: string;
    /**
     * The id for the KML sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#id)
     */
    id: number;
    /**
     * The [KMLLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#layer)
     */
    layer: KMLLayer;
    /**
     * Network link info for the current layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#networkLink)
     */
    networkLink: any;
    /**
     * The parent layer to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#parent)
     */
    parent: KMLSublayer | KMLLayer;
    /**
     * The raw KML data for this sublayer, in JSON format, as returned by the [KML utility service](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#kmlServiceUrl).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * A collection of [KMLSublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#sublayers)
     */
    sublayers: Collection<KMLSublayer>;
    /**
     * The title of the KML sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#title)
     */
    title: string;
    /**
     * Indicates if the sublayer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#visible)
     */
    visible: boolean;

    static fromJSON(json: any): KMLSublayer;
  }

  interface KMLSublayerProperties {
    /**
     * Description for the KML sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#description)
     */
    description?: string;
    /**
     * The id for the KML sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#id)
     */
    id?: number;
    /**
     * The [KMLLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#layer)
     */
    layer?: KMLLayerProperties;
    /**
     * Network link info for the current layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#networkLink)
     */
    networkLink?: any;
    /**
     * The parent layer to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#parent)
     */
    parent?: KMLSublayerProperties | (KMLLayerProperties & { type: "kml" });
    /**
     * The raw KML data for this sublayer, in JSON format, as returned by the [KML utility service](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#kmlServiceUrl).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * A collection of [KMLSublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#sublayers)
     */
    sublayers?: CollectionProperties<KMLSublayerProperties>;
    /**
     * The title of the KML sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#title)
     */
    title?: string;
    /**
     * Indicates if the sublayer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-KMLSublayer.html#visible)
     */
    visible?: boolean;
  }

  export interface LabelClass extends Accessor, JSONSupport {}

  export class LabelClass {
    /**
     * Defines label expressions, symbols, scale ranges, label priorities, and label placement options for labels on a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html)
     */

    constructor(properties?: LabelClassProperties);

    /**
     * Specifies whether or not a polyline label can overrun the feature being labeled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#allowOverrun)
     */
    allowOverrun: boolean;
    /**
     * Defines how labels should be placed relative to one another.
     *
     * @default "static"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#deconflictionStrategy)
     */
    deconflictionStrategy: "none" | "static";
    /**
     * Defines the labels for a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpression)
     */
    labelExpression: string;
    /**
     * Defines the labels for a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo)
     */
    labelExpressionInfo: LabelClassLabelExpressionInfo;
    /**
     * The position of the label.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelPlacement)
     */
    labelPlacement:
      | "above-center"
      | "above-left"
      | "above-right"
      | "below-center"
      | "below-left"
      | "below-right"
      | "center-center"
      | "center-left"
      | "center-right"
      | "above-after"
      | "above-along"
      | "above-before"
      | "above-start"
      | "above-end"
      | "below-after"
      | "below-along"
      | "below-before"
      | "below-start"
      | "below-end"
      | "center-after"
      | "center-along"
      | "center-before"
      | "center-start"
      | "center-end"
      | "always-horizontal";
    /**
     * Specifies the orientation of the label position of a single line polyline label.
     *
     * @default "curved"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelPosition)
     */
    labelPosition: "curved" | "parallel";
    /**
     * The maximum scale (most zoomed in) at which labels are visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which labels are visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#minScale)
     */
    minScale: number;
    /**
     * Indicates whether or not to repeat the label along the polyline feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#repeatLabel)
     */
    repeatLabel: boolean;
    /**
     * The size in points of the distance between labels on a polyline.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#repeatLabelDistance)
     */
    repeatLabelDistance: number;
    /**
     * Defines the symbol used for rendering the label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#symbol)
     */
    symbol: TextSymbol | LabelSymbol3D;
    /**
     * Indicates whether to use domain names if the fields in the [labelExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpression) or [labelExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo) have domains.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#useCodedValues)
     */
    useCodedValues: boolean;
    /**
     * A SQL where clause used to determine the features to which the label class should be applied.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#where)
     */
    where: string;

    /**
     * Creates a deep clone of the LabelClass.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#clone)
     */
    clone(): LabelClass;

    static fromJSON(json: any): LabelClass;
  }

  interface LabelClassProperties {
    /**
     * Specifies whether or not a polyline label can overrun the feature being labeled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#allowOverrun)
     */
    allowOverrun?: boolean;
    /**
     * Defines how labels should be placed relative to one another.
     *
     * @default "static"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#deconflictionStrategy)
     */
    deconflictionStrategy?: "none" | "static";
    /**
     * Defines the labels for a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpression)
     */
    labelExpression?: string;
    /**
     * Defines the labels for a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo)
     */
    labelExpressionInfo?: LabelClassLabelExpressionInfo;
    /**
     * The position of the label.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelPlacement)
     */
    labelPlacement?:
      | "above-center"
      | "above-left"
      | "above-right"
      | "below-center"
      | "below-left"
      | "below-right"
      | "center-center"
      | "center-left"
      | "center-right"
      | "above-after"
      | "above-along"
      | "above-before"
      | "above-start"
      | "above-end"
      | "below-after"
      | "below-along"
      | "below-before"
      | "below-start"
      | "below-end"
      | "center-after"
      | "center-along"
      | "center-before"
      | "center-start"
      | "center-end"
      | "always-horizontal";
    /**
     * Specifies the orientation of the label position of a single line polyline label.
     *
     * @default "curved"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelPosition)
     */
    labelPosition?: "curved" | "parallel";
    /**
     * The maximum scale (most zoomed in) at which labels are visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which labels are visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#minScale)
     */
    minScale?: number;
    /**
     * Indicates whether or not to repeat the label along the polyline feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#repeatLabel)
     */
    repeatLabel?: boolean;
    /**
     * The size in points of the distance between labels on a polyline.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#repeatLabelDistance)
     */
    repeatLabelDistance?: number | string;
    /**
     * Defines the symbol used for rendering the label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#symbol)
     */
    symbol?: (TextSymbolProperties & { type: "text" }) | (LabelSymbol3DProperties & { type: "label-3d" });
    /**
     * Indicates whether to use domain names if the fields in the [labelExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpression) or [labelExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo) have domains.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#useCodedValues)
     */
    useCodedValues?: boolean;
    /**
     * A SQL where clause used to determine the features to which the label class should be applied.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#where)
     */
    where?: string;
  }

  export interface LabelClassLabelExpressionInfo {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Labeling Profile](https://developers.arcgis.com/javascript/latest/arcade/#labeling).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo)
     */
    expression: string;
    /**
     * The title of the label expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html#labelExpressionInfo)
     */
    title?: string;
  }

  export interface LayerFloorInfo extends Accessor, JSONSupport {}

  export class LayerFloorInfo {
    /**
     * LayerFloorInfo contains properties that allow a layer to be floor-aware.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html)
     */

    constructor(properties?: LayerFloorInfoProperties);

    /**
     * The field name derived from a floor-aware layer and used to filter features by floor level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html#floorField)
     */
    floorField: string;

    /**
     * Creates a deep clone of LayerFloorInfo object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html#clone)
     */
    clone(): LayerFloorInfo;

    static fromJSON(json: any): LayerFloorInfo;
  }

  interface LayerFloorInfoProperties {
    /**
     * The field name derived from a floor-aware layer and used to filter features by floor level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html#floorField)
     */
    floorField?: string;
  }

  export interface LevelLayerInfo extends Accessor, JSONSupport {}

  export class LevelLayerInfo {
    /**
     * The LevelLayerInfo class describes the footprint of each occupiable floor contained in a managed facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html)
     */

    constructor(properties?: LevelLayerInfoProperties);

    /**
     * The field name from the layer that defines the unique ID of a feature's associated facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#facilityIdField)
     */
    facilityIdField: string;
    /**
     * The id for an operational layer in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#layerId)
     */
    layerId: string;
    /**
     * The field name from the layer that defines a unique ID for the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#levelIdField)
     */
    levelIdField: string;
    /**
     * The field name from the layer that defines the level floor number in a particular facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#levelNumberField)
     */
    levelNumberField: string;
    /**
     * The field name from the layer that defines the level name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#longNameField)
     */
    longNameField: string;
    /**
     * The field name from the layer that defines the level short name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#shortNameField)
     */
    shortNameField: string;
    /**
     * This value references the numeric ID of the sublayer if the Level layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#sublayerId)
     */
    sublayerId: number;
    /**
     * The field name from the layer that defines the order of display and reference to floors in the [Indoor Positioning System](https://doc.arcgis.com/en/indoors/android/configure-indoor-positioning.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#verticalOrderField)
     */
    verticalOrderField: string;

    static fromJSON(json: any): LevelLayerInfo;
  }

  interface LevelLayerInfoProperties {
    /**
     * The field name from the layer that defines the unique ID of a feature's associated facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#facilityIdField)
     */
    facilityIdField?: string;
    /**
     * The id for an operational layer in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#layerId)
     */
    layerId?: string;
    /**
     * The field name from the layer that defines a unique ID for the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#levelIdField)
     */
    levelIdField?: string;
    /**
     * The field name from the layer that defines the level floor number in a particular facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#levelNumberField)
     */
    levelNumberField?: string;
    /**
     * The field name from the layer that defines the level name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#longNameField)
     */
    longNameField?: string;
    /**
     * The field name from the layer that defines the level short name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#shortNameField)
     */
    shortNameField?: string;
    /**
     * This value references the numeric ID of the sublayer if the Level layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#sublayerId)
     */
    sublayerId?: number;
    /**
     * The field name from the layer that defines the order of display and reference to floors in the [Indoor Positioning System](https://doc.arcgis.com/en/indoors/android/configure-indoor-positioning.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LevelLayerInfo.html#verticalOrderField)
     */
    verticalOrderField?: string;
  }

  export class LocalMediaElementSource extends Accessor {
    /**
     * The LocalMediaElementSource is the default [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source) for the MediaLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LocalMediaElementSource.html)
     */

    constructor(properties?: LocalMediaElementSourceProperties);

    /**
     * [Image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) and [video](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) elements referenced in the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LocalMediaElementSource.html#elements)
     */
    elements: Collection<MediaElement>;
  }

  interface LocalMediaElementSourceProperties {
    /**
     * [Image](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-ImageElement.html) and [video](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html) elements referenced in the [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LocalMediaElementSource.html#elements)
     */
    elements?:
      | CollectionProperties<
          (ImageElementProperties & { type: "image" }) | (VideoElementProperties & { type: "video" })
        >
      | ((ImageElementProperties & { type: "image" }) | (VideoElementProperties & { type: "video" }))[];
  }

  /**
   * Image and video elements referenced in the MediaLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LocalMediaElementSource.html#MediaElement)
   */
  export type MediaElement = ImageElement | VideoElement;

  export interface LOD extends Accessor, JSONSupport {}

  export class LOD {
    /**
     * A [TileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html) has a number of LODs (Levels of Detail).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html)
     */

    constructor(properties?: LODProperties);

    /**
     * ID for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#level)
     */
    level: number;
    /**
     * String to be used when constructing a URL to access a tile from this LOD.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#levelValue)
     */
    levelValue: string;
    /**
     * Resolution in map units of each pixel in a tile for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#resolution)
     */
    resolution: number;
    /**
     * Scale for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#scale)
     */
    scale: number;

    static fromJSON(json: any): LOD;
  }

  interface LODProperties {
    /**
     * ID for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#level)
     */
    level?: number;
    /**
     * String to be used when constructing a URL to access a tile from this LOD.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#levelValue)
     */
    levelValue?: string;
    /**
     * Resolution in map units of each pixel in a tile for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#resolution)
     */
    resolution?: number;
    /**
     * Scale for each level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html#scale)
     */
    scale?: number;
  }

  export interface MapImage extends Accessor, JSONSupport {}

  export class MapImage {
    /**
     * Represents the data object for the dynamically generated map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html)
     */

    constructor(properties?: MapImageProperties);

    /**
     * The extent of the exported map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#extent)
     */
    extent: Extent;
    /**
     * The requested image height in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#height)
     */
    height: number;
    /**
     * URL to the returned image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#href)
     */
    href: string;
    /**
     * The opacity of the image.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#opacity)
     */
    opacity: number;
    /**
     * Scale of the requested dynamic map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#scale)
     */
    scale: number;
    /**
     * Indicates if the requested image is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#visible)
     */
    visible: boolean;
    /**
     * The requested image width in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#width)
     */
    width: number;

    static fromJSON(json: any): MapImage;
  }

  interface MapImageProperties {
    /**
     * The extent of the exported map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * The requested image height in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#height)
     */
    height?: number;
    /**
     * URL to the returned image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#href)
     */
    href?: string;
    /**
     * The opacity of the image.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#opacity)
     */
    opacity?: number;
    /**
     * Scale of the requested dynamic map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#scale)
     */
    scale?: number;
    /**
     * Indicates if the requested image is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#visible)
     */
    visible?: boolean;
    /**
     * The requested image width in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MapImage.html#width)
     */
    width?: number;
  }

  export class MediaElementBase {
    /**
     * The geographic location of the image or video element to be placed on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MediaElementBase.html#georeference)
     */
    georeference: ExtentAndRotationGeoreference | CornersGeoreference | ControlPointsGeoreference;
    /**
     * The opacity of the element.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MediaElementBase.html#opacity)
     */
    opacity: number;
  }

  interface MediaElementBaseProperties {
    /**
     * The geographic location of the image or video element to be placed on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MediaElementBase.html#georeference)
     */
    georeference?:
      | (ExtentAndRotationGeoreferenceProperties & { type: "extent-and-rotation" })
      | (CornersGeoreferenceProperties & { type: "corners" })
      | (ControlPointsGeoreferenceProperties & { type: "control-points" });
    /**
     * The opacity of the element.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MediaElementBase.html#opacity)
     */
    opacity?: number;
  }

  export interface MosaicRule extends Accessor, JSONSupport {}

  export class MosaicRule {
    /**
     * Specifies the mosaic rule when defining how individual images should be mosaicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html)
     */

    constructor(properties?: MosaicRuleProperties);

    /**
     * Indicates whether the sort should be ascending.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#ascending)
     */
    ascending: boolean;
    /**
     * The raster function applied on items before mosaicking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#itemRasterFunction)
     */
    itemRasterFunction: RasterFunction;
    /**
     * An array of raster Ids.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#lockRasterIds)
     */
    lockRasterIds: number[];
    /**
     * The mosaic method determines how the selected rasters are ordered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method)
     */
    method: "none" | "center" | "nadir" | "viewpoint" | "attribute" | "lock-raster" | "northwest" | "seamline";
    /**
     * The multidimensional definitions associated with the [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#mosaicRule).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#multidimensionalDefinition)
     */
    multidimensionalDefinition: DimensionalDefinition[];
    /**
     * Defines a selection using a set of ObjectIDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#objectIds)
     */
    objectIds: number[];
    /**
     * Defines the mosaic operation used to resolve overlapping pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#operation)
     */
    operation: "first" | "last" | "min" | "max" | "mean" | "blend" | "sum";
    /**
     * The name of the attribute field that is used with a constant sortValue to define the mosaicking order when the mosaic [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method) is set to `attribute`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#sortField)
     */
    sortField: string;
    /**
     * A constant value defining a reference or base value for the sort field when the mosaic [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method) is set to `attribute`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#sortValue)
     */
    sortValue: string | number;
    /**
     * Defines the viewpoint location on which the ordering is defined based on the distance from the viewpoint and the nadir of rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#viewpoint)
     */
    viewpoint: Point;
    /**
     * The where clause determines which rasters will participate in the mosaic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#where)
     */
    where: string;

    /**
     * Creates a clone of the MosaicRule object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#clone)
     */
    clone(): MosaicRule;

    static fromJSON(json: any): MosaicRule;
  }

  interface MosaicRuleProperties {
    /**
     * Indicates whether the sort should be ascending.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#ascending)
     */
    ascending?: boolean;
    /**
     * The raster function applied on items before mosaicking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#itemRasterFunction)
     */
    itemRasterFunction?: RasterFunctionProperties;
    /**
     * An array of raster Ids.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#lockRasterIds)
     */
    lockRasterIds?: number[];
    /**
     * The mosaic method determines how the selected rasters are ordered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method)
     */
    method?: "none" | "center" | "nadir" | "viewpoint" | "attribute" | "lock-raster" | "northwest" | "seamline";
    /**
     * The multidimensional definitions associated with the [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#mosaicRule).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#multidimensionalDefinition)
     */
    multidimensionalDefinition?: DimensionalDefinitionProperties[];
    /**
     * Defines a selection using a set of ObjectIDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#objectIds)
     */
    objectIds?: number[];
    /**
     * Defines the mosaic operation used to resolve overlapping pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#operation)
     */
    operation?: "first" | "last" | "min" | "max" | "mean" | "blend" | "sum";
    /**
     * The name of the attribute field that is used with a constant sortValue to define the mosaicking order when the mosaic [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method) is set to `attribute`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#sortField)
     */
    sortField?: string;
    /**
     * A constant value defining a reference or base value for the sort field when the mosaic [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#method) is set to `attribute`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#sortValue)
     */
    sortValue?: string | number;
    /**
     * Defines the viewpoint location on which the ordering is defined based on the distance from the viewpoint and the nadir of rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#viewpoint)
     */
    viewpoint?: PointProperties;
    /**
     * The where clause determines which rasters will participate in the mosaic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html#where)
     */
    where?: string;
  }

  export interface MultidimensionalSubset extends Accessor, JSONSupport {}

  export class MultidimensionalSubset {
    /**
     * A subset of multidimensional raster data created by slicing the data along defined variables and dimensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html)
     */

    constructor(properties?: MultidimensionalSubsetProperties);

    /**
     * The spatial area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#areaOfInterest)
     */
    areaOfInterest: Extent | Polygon;
    /**
     * The aggregated dimension names and their extents or ranges computed from the [subsetDefinitions](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#subsetDefinitions).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#dimensions)
     */
    readonly dimensions: SubsetDimension[];
    /**
     * The variable and dimension subset definitions to set the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#subsetDefinitions)
     */
    subsetDefinitions: DimensionalDefinition[];
    /**
     * The aggregated variables list computed from the [subsetDefinitions](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#subsetDefinitions).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#variables)
     */
    readonly variables: string[];

    /**
     * Creates a deep clone of the MultidimensionalSubset object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#clone)
     */
    clone(): MultidimensionalSubset;

    static fromJSON(json: any): MultidimensionalSubset;
  }

  interface MultidimensionalSubsetProperties {
    /**
     * The spatial area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#areaOfInterest)
     */
    areaOfInterest?: (ExtentProperties & { type: "extent" }) | (PolygonProperties & { type: "polygon" });
    /**
     * The variable and dimension subset definitions to set the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#subsetDefinitions)
     */
    subsetDefinitions?: DimensionalDefinitionProperties[];
  }

  /**
   * Dimension name and its extent or range computed from the [subsetDefinitions](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#subsetDefinitions) and it is added to [dimensions](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#dimensions).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#SubsetDimension)
   */
  export interface SubsetDimension {
    /**
     * The dimension name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#SubsetDimension)
     */
    name: string;
    /**
     * The computed extent or ranges for the given dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MultidimensionalSubset.html#SubsetDimension)
     */
    extent: number[];
  }

  export class PixelBlock extends Accessor {
    /**
     * An object representing the pixel arrays in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html)
     */

    constructor(properties?: PixelBlockProperties);

    /**
     * The height (or number of rows) of the PixelBlock in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#height)
     */
    height: number;
    /**
     * An array of nodata mask.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#mask)
     */
    mask: Uint8Array;
    /**
     * Indicates whether mask should be used as alpha values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#maskIsAlpha)
     */
    maskIsAlpha: boolean;
    /**
     * A two dimensional array representing the pixels from the Image Service displayed on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels)
     */
    pixels: (
      | number[]
      | Int8Array
      | Uint8Array
      | Uint8ClampedArray
      | Int16Array
      | Uint16Array
      | Int32Array
      | Uint32Array
      | Float32Array
      | Float64Array
    )[];
    /**
     * The pixel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixelType)
     */
    pixelType:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * An array of objects containing numeric statistical properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#statistics)
     */
    statistics: PixelBlockStatistics[];
    /**
     * Number of valid pixels
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#validPixelCount)
     */
    validPixelCount: number;
    /**
     * The width (or number of columns) of the PixelBlock in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#width)
     */
    width: number;

    /**
     * Adds another plane to the PixelBlock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#addData)
     */
    addData(planeData: PixelBlockAddDataPlaneData): void;
    /**
     * Returns pixels and masks using a single array in bip format (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#getAsRGBA)
     */
    getAsRGBA(): Uint8ClampedArray;
    /**
     * Similar to [getAsRGBA](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#getAsRGBA), but returns floating point data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#getAsRGBAFloat)
     */
    getAsRGBAFloat(): Float32Array;
    /**
     * Returns the plane band count of the PixelBlock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#getPlaneCount)
     */
    getPlaneCount(): number;
  }

  interface PixelBlockProperties {
    /**
     * The height (or number of rows) of the PixelBlock in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#height)
     */
    height?: number;
    /**
     * An array of nodata mask.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#mask)
     */
    mask?: Uint8Array;
    /**
     * Indicates whether mask should be used as alpha values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#maskIsAlpha)
     */
    maskIsAlpha?: boolean;
    /**
     * A two dimensional array representing the pixels from the Image Service displayed on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels)
     */
    pixels?: (
      | number[]
      | Int8Array
      | Uint8Array
      | Uint8ClampedArray
      | Int16Array
      | Uint16Array
      | Int32Array
      | Uint32Array
      | Float32Array
      | Float64Array
    )[];
    /**
     * The pixel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixelType)
     */
    pixelType?:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * An array of objects containing numeric statistical properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#statistics)
     */
    statistics?: PixelBlockStatistics[];
    /**
     * Number of valid pixels
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#validPixelCount)
     */
    validPixelCount?: number;
    /**
     * The width (or number of columns) of the PixelBlock in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#width)
     */
    width?: number;
  }

  export interface PixelBlockAddDataPlaneData {
    /**
     * A two dimensional array representing the pixels to add.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#addData)
     */
    pixels: number[][];
    /**
     * An array of objects containing numeric statistical properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#addData)
     */
    statistics: any[];
  }

  export interface PixelBlockStatistics {
    /**
     * The maximum pixel value in the [pixels](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels) array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#statistics)
     */
    maxValue?: number;
    /**
     * The minimum pixel value in the [pixels](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels) array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#statistics)
     */
    minValue?: number;
    /**
     * Value representing areas of no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#statistics)
     */
    noDataValue?: number;
  }

  export class PublishingInfo extends Accessor {
    /**
     * This stops when the status has been determined or when the layer is destroyed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PublishingInfo.html)
     */

    constructor(properties?: PublishingInfoProperties);

    /**
     * Layer's publishing status while the layer is being published to the portal.
     *
     * @default "unknown"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PublishingInfo.html#status)
     */
    readonly status: "unknown" | "unavailable" | "publishing" | "published";
    /**
     * Indicates if the layer's status is still being updated.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PublishingInfo.html#updating)
     */
    readonly updating: boolean;
  }

  interface PublishingInfoProperties {}

  export class RangeDomain extends Domain {
    /**
     * Range domains specify a valid [minimum](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#minValue) and [maximum](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#maxValue) valid value that can be stored in numeric and date [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html)
     */

    constructor(properties?: RangeDomainProperties);

    /**
     * The maximum valid value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#maxValue)
     */
    maxValue: number | string;
    /**
     * The minimum valid value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#minValue)
     */
    minValue: number | string;
    /**
     * The domain type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#type)
     */
    readonly type: "range";

    static fromJSON(json: any): RangeDomain;
  }

  interface RangeDomainProperties extends DomainProperties {
    /**
     * The maximum valid value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#maxValue)
     */
    maxValue?: number | string;
    /**
     * The minimum valid value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html#minValue)
     */
    minValue?: number | string;
  }

  export interface RasterBandInfo extends Accessor, JSONSupport {}

  export class RasterBandInfo {
    /**
     * referenced by [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#serviceRasterInfo) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html)
     */

    constructor(properties?: RasterBandInfoProperties);

    /**
     * The maximum wavelength of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#maxWavelength)
     */
    maxWavelength: number;
    /**
     * The minimum wavelength of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#minWavelength)
     */
    minWavelength: number;
    /**
     * The name of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#name)
     */
    name: string;
    /**
     * The radiance bias of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#radianceBias)
     */
    radianceBias: number;
    /**
     * The radiance gain of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#radianceGain)
     */
    radianceGain: number;
    /**
     * The reflectance bias of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#reflectanceBias)
     */
    reflectanceBias: number;
    /**
     * The reflectance gain of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#reflectanceGain)
     */
    reflectanceGain: number;
    /**
     * The solar irradiance of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#solarIrradiance)
     */
    solarIrradiance: number;

    /**
     * Creates a deep clone of the raster band info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#clone)
     */
    clone(): RasterBandInfo;

    static fromJSON(json: any): RasterBandInfo;
  }

  interface RasterBandInfoProperties {
    /**
     * The maximum wavelength of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#maxWavelength)
     */
    maxWavelength?: number;
    /**
     * The minimum wavelength of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#minWavelength)
     */
    minWavelength?: number;
    /**
     * The name of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#name)
     */
    name?: string;
    /**
     * The radiance bias of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#radianceBias)
     */
    radianceBias?: number;
    /**
     * The radiance gain of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#radianceGain)
     */
    radianceGain?: number;
    /**
     * The reflectance bias of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#reflectanceBias)
     */
    reflectanceBias?: number;
    /**
     * The reflectance gain of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#reflectanceGain)
     */
    reflectanceGain?: number;
    /**
     * The solar irradiance of the band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterBandInfo.html#solarIrradiance)
     */
    solarIrradiance?: number;
  }

  export interface RasterFunction extends Accessor, JSONSupport {}

  export class RasterFunction {
    /**
     * Raster functions specify processing to be done to the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html)
     */

    constructor(properties?: RasterFunctionProperties);

    /**
     * The arguments for the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#functionArguments)
     */
    functionArguments: any;
    /**
     * The raster function name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#functionName)
     */
    functionName: string;
    /**
     * Defines the pixel type of the output image.
     *
     * @default "unknown"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#outputPixelType)
     */
    outputPixelType:
      | "c128"
      | "c64"
      | "f32"
      | "f64"
      | "s16"
      | "s32"
      | "s8"
      | "u1"
      | "u16"
      | "u2"
      | "u32"
      | "u4"
      | "u8"
      | "unknown";
    /**
     * Property where Raster Function template is passed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#rasterFunctionDefinition)
     */
    rasterFunctionDefinition: any;
    /**
     * The variable name for the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#variableName)
     */
    variableName: string;

    /**
     * Creates a deep clone of the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#clone)
     */
    clone(): RasterFunction;

    static fromJSON(json: any): RasterFunction;
  }

  interface RasterFunctionProperties {
    /**
     * The arguments for the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#functionArguments)
     */
    functionArguments?: any;
    /**
     * The raster function name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#functionName)
     */
    functionName?: string;
    /**
     * Defines the pixel type of the output image.
     *
     * @default "unknown"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#outputPixelType)
     */
    outputPixelType?:
      | "c128"
      | "c64"
      | "f32"
      | "f64"
      | "s16"
      | "s32"
      | "s8"
      | "u1"
      | "u16"
      | "u2"
      | "u32"
      | "u4"
      | "u8"
      | "unknown";
    /**
     * Property where Raster Function template is passed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#rasterFunctionDefinition)
     */
    rasterFunctionDefinition?: any;
    /**
     * The variable name for the raster function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html#variableName)
     */
    variableName?: string;
  }

  /**
   * Various constant values used by different raster functions when setting [ImageryTileLayer.rasterFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterFunction) or [ImageryLayer.rasterFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#rasterFunction) properties.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html)
   */
  interface rasterFunctionConstants {
    /**
     * Method name constants used by the [band index raster functions](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_4D75298B90D94E6B87B0C9F28D5AE269).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    readonly bandIndexType: rasterFunctionConstantsBandIndexType;
    /**
     * The local cell statistics operations type constants.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    readonly cellStatisticalOperation: rasterFunctionConstantsCellStatisticalOperation;
    /**
     * Predefined raster color map name constants used for the [Colormap raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#GUID-5DDFC961-A857-445F-A983-13AF67D45833).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    readonly colormapName: rasterFunctionConstantsColormapName;
    /**
     * Predefined raster color ramp name constants used for the [Colormap raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#GUID-5DDFC961-A857-445F-A983-13AF67D45833).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    readonly colorRampName: rasterFunctionConstantsColorRampName;
    /**
     * Kernel type constants used for the [Convolution raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_B69A71FDFD7C46A6860A7302A9AD3B17).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    readonly convolutionKernel: rasterFunctionConstantsConvolutionKernel;
    /**
     * Curvature type constants used for the [curvature raster function](https://developers.arcgis.com/rest/services-reference/enterprise/raster-function-objects/#curvature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#curvatureType)
     */
    readonly curvatureType: rasterFunctionConstantsCurvatureType;
    /**
     * Hillshade type constants used for the [hillshade raster function](https://developers.arcgis.com/rest/services-reference/enterprise/raster-function-objects/#hillshade).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#hillshadeType)
     */
    readonly hillshadeType: rasterFunctionConstantsHillshadeType;
    /**
     * The local arithmetic operations types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    localArithmeticOperation: rasterFunctionConstantsLocalArithmeticOperation;
    /**
     * The local conditional operations type constants.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localConditionalOperation)
     */
    readonly localConditionalOperation: rasterFunctionConstantsLocalConditionalOperation;
    /**
     * The local logical operations type constants.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    readonly localLogicalOperation: rasterFunctionConstantsLocalLogicalOperation;
    /**
     * The local trigonometric operations type constants.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    readonly localTrigonometricOperation: rasterFunctionConstantsLocalTrigonometricOperation;
    /**
     * The missing band action constants available for the [Extract band raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_2FC6FEAA2801446B9578A7C90B1DD6AB).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#missingBandAction)
     */
    readonly missingBandAction: rasterFunctionConstantsMissingBandAction;
    /**
     * The NoData interpretation constants used for the [Mask raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_5515B13D320B408B9F70B9B8344028FF).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#noDataInterpretation)
     */
    readonly noDataInterpretation: rasterFunctionConstantsNoDataInterpretation;
    /**
     * Slope type constants used for the [slope raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_641F092319AE46D1A9441F8D1FDF186C).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#slopeType)
     */
    readonly slopeType: rasterFunctionConstantsSlopeType;
    /**
     * Stretch type constants used for the [stretch raster function](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm#ESRI_SECTION1_7545363F0B8A4B7B931A54B3C4189D9D).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    readonly stretchType: rasterFunctionConstantsStretchType;
  }

  export const rasterFunctionConstants: rasterFunctionConstants;

  export interface rasterFunctionConstantsBandIndexType {
    /**
     * User defined method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    userDefined: number;
    /**
     * The Normalized Difference Vegetation Index (NDVI) method is a standardized index allowing you to generate an image displaying greenness (relative biomass).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDVI: number;
    /**
     * The Red-Edge NDVI (NDVIre) method is a vegetation index for estimating vegetation health using the red-edge band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDVIRe: number;
    /**
     * The Burn Area Index (BAI) uses the reflectance values in the red and NIR portion of the spectrum to identify the areas of the terrain affected by fire.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    BAI: number;
    /**
     * The Normalized Burn Ratio Index (NBRI) uses the NIR and SWIR bands to emphasize burned areas, while mitigating illumination and atmospheric effects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NBR: number;
    /**
     * The Normalized Difference Built-up Index (NDBI) uses the NIR and SWIR bands to emphasize manufactured built-up areas.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDBI: number;
    /**
     * The Normalized Difference Moisture Index (NDMI) is sensitive to the moisture levels in vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDMI: number;
    /**
     * The Normalized Difference Snow Index (NDSI) is designed to use MODIS (band 4 and band 6) and Landsat TM (band 2 and band 5) for identification of snow cover while ignoring cloud cover.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDSI: number;
    /**
     * The Global Environmental Monitoring Index (GEMI) method is a nonlinear vegetation index for global environmental monitoring from satellite imagery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    GEMI: number;
    /**
     * The Green Vegetation Index (GVI) method was originally designed from Landsat MSS imagery and has been modified for Landsat TM imagery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    GVITM: number;
    /**
     * The Perpendicular Vegetation Index (PVI) method is similar to a difference vegetation index; however, it is sensitive to atmospheric variations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    PVI: string;
    /**
     * The Sultan's process takes a six-band 8-bit image and uses the Sultan's Formula method to produce a three-band 8-bit image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    Sultan: number;
    /**
     * The Visible Atmospherically Resistant Index (VARI) method is a vegetation index for estimating vegetation fraction quantitatively with only the visible range of the spectrum.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    VARI: number;
    /**
     * The Green Normalized Difference Vegetation Index (GNDVI) method is a vegetation index for estimating photo synthetic activity and is a commonly used vegetation index to determine water and nitrogen uptake into the plant canopy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    GNDVI: number;
    /**
     * The Soil-Adjusted Vegetation Index (SAVI) method is a vegetation index that attempts to minimize soil brightness influences using a soil-brightness correction factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    SAVI: number;
    /**
     * The Transformed Soil Adjusted Vegetation Index (TSAVI) method is a vegetation index that minimizes soil brightness influences by assuming the soil line has an arbitrary slope and intercept.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    TSAVI: number;
    /**
     * The Modified Soil Adjusted Vegetation Index (MSAVI) method minimizes the effect of bare soil on the SAVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    MSAVI: number;
    /**
     * The Simple Ratio (SR) method is a common vegetation index for estimating the amount of vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    SR: number;
    /**
     * The Red-Edge Simple Ratio (SRre) method is a vegetation index for estimating the amount of healthy and stressed vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    SRRe: number;
    /**
     * The Modified Triangular Vegetation Index (MTVI2) method is a vegetation index for detecting leaf chlorophyll content at the canopy scale while being relatively insensitive to leaf area index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    MTVI2: number;
    /**
     * The Red-Edge Triangulated Vegetation Index (RTVICore) method is a vegetation index for estimating leaf area index and biomass.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    RTVICore: number;
    /**
     * The Chlorophyll Index - Red-Edge (CIre) method is a vegetation index for estimating the chlorophyll content in leaves using the ratio of reflectivity in the NIR and red-edge bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    CIRe: number;
    /**
     * Chlorophyll index - Green (CIG) method is a vegetation index for estimating the chlorophyll content in leaves using the ratio of reflectivity in the NIR and green bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    CIG: number;
    /**
     * The Enhanced Vegetation Index (EVI) method is an optimized vegetation index that accounts for atmospheric influences and vegetation background signal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    EVI: number;
    /**
     * The Iron Oxide (ironOxide) ratio method is a geological index for identifying rock features that have experienced oxidation of iron-bearing sulfides using the red and blue bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    ironOxide: number;
    /**
     * The Ferrous Minerals (ferrousMinerals) ratio method is a geological index for identifying rock features containing some quantity of iron-bearing minerals using the SWIR and NIR bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    ferrousMinerals: number;
    /**
     * The Clay Minerals (clayMinerals) ratio method is a geological index for identifying mineral features containing clay and alunite using two shortwave infrared (SWIR) bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    clayMinerals: number;
    /**
     * The Normalized Difference Water Index (NDWI) method is an index for delineating and monitoring content changes in surface water.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    NDWI: number;
    /**
     * The Weighted Normalized Difference Water Index (WNDWI) method is a water index developed to reduce errors typically encountered in other water indices, including water turbidity, small water bodies, or shadow in remote sensing scenes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    WNDWI: number;
    /**
     * The Modified Normalized Difference Water Index (MNDWI) uses green and SWIR bands for the enhancement of open water features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#bandIndexType)
     */
    MNDWI: number;
  }

  export interface rasterFunctionConstantsCellStatisticalOperation {
    /**
     * Determines the majority (value that occurs most often) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    majority: number;
    /**
     * Determines the maximum (largest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    max: number;
    /**
     * Determines the mean (average value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    mean: number;
    /**
     * Calculates the median of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    med: number;
    /**
     * Determines the minimum (smallest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    min: number;
    /**
     * Determines the minority (value that occurs least often) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    minority: number;
    /**
     * Calculates the range (difference between largest and smallest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    range: number;
    /**
     * Calculates the standard deviation of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    stddev: number;
    /**
     * Calculates the sum (total of all values) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    sum: number;
    /**
     * Calculates the variety (number of unique values) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    variety: number;
    /**
     * Determines the majority (value that occurs most often) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    majorityIgnoreNoData: number;
    /**
     * Determines the maximum (largest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    maxIgnoreNoData: number;
    /**
     * Determines the mean (average value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    meanIgnoreNoData: number;
    /**
     * Determines the median of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    medIgnoreNoData: number;
    /**
     * Determines the minimum (smallest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    minIgnoreNoData: number;
    /**
     * Determines the minority (value that occurs least often) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    minorityIgnoreNoData: number;
    /**
     * Calculates the range (difference between largest and smallest value) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    rangeIgnoreNoData: number;
    /**
     * Calculates the standard deviation of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    stddevIgnoreNoData: number;
    /**
     * Calculates the sum (total of all values) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    sumIgnoreNoData: number;
    /**
     * Calculates the variety (number of unique values) of the inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#cellStatisticalOperation)
     */
    varietyIgnoreNoData: number;
  }

  export interface rasterFunctionConstantsColormapName {
    /**
     * A random colormap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    random: string;
    /**
     * colormap to visualize vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    NDVI: string;
    /**
     * A colormap to visualize vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    NDVI2: string;
    /**
     * A colormap to visualize vegetation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    NDVI3: string;
    /**
     * A color map that gradually changes from cyan to purple to black.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    elevation: string;
    /**
     * A color map that gradually changes from black to white.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    gray: string;
    /**
     * A colormap to visualize a hillshade product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colormapName)
     */
    hillshade: string;
  }

  export interface rasterFunctionConstantsColorRampName {
    /**
     * Aspect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    aspect: string;
    /**
     * Black to White.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    blackToWhite: string;
    /**
     * Blue Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    blueBright: string;
    /**
     * Blue Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    blueLightToDark: string;
    /**
     * Blue-Green Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    blueGreenBright: string;
    /**
     * Blue-Green Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    blueGreenLightToDark: string;
    /**
     * Brown Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    brownLightToDark: string;
    /**
     * Brown to Blue Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    brownToBlueGreenDivergingBright: string;
    /**
     * Brown to Blue Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    brownToBlueGreenDivergingDark: string;
    /**
     * Coefficient Bias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    coefficientBias: string;
    /**
     * Cold to Hot Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    coldToHotDiverging: string;
    /**
     * Condition Number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    conditionNumber: string;
    /**
     * Cyan to Purple.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    cyanToPurple: string;
    /**
     * Cyan-Light to Blue-Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    cyanLightToBlueDark: string;
    /**
     * Distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    distance: string;
    /**
     * Elevation #1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    elevation1: string;
    /**
     * Elevation #2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    elevation2: string;
    /**
     * Errors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    errors: string;
    /**
     * Gray Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    grayLightToDark: string;
    /**
     * Green Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    greenBright: string;
    /**
     * Green Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    greenLightToDark: string;
    /**
     * Green to Blue.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    greenToBlue: string;
    /**
     * Orange Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    orangeBright: string;
    /**
     * Orange Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    orangeLightToDark: string;
    /**
     * Partial Spectrum.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    partialSpectrum: string;
    /**
     * Partial Spectrum 1 Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    partialSpectrum1Diverging: string;
    /**
     * Partial Spectrum 2 Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    partialSpectrum2Diverging: string;
    /**
     * Pink to YellowGreen Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    pinkToYellowGreenDivergingBright: string;
    /**
     * Pink to YellowGreen Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    pinkToYellowGreenDivergingDark: string;
    /**
     * Precipitation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    precipitation: string;
    /**
     * Prediction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    prediction: string;
    /**
     * Purple Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleBright: string;
    /**
     * Purple to Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleToGreenDivergingBright: string;
    /**
     * Purple to Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleToGreenDivergingDark: string;
    /**
     * Purple-Blue Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleBlueBright: string;
    /**
     * Purple-Blue Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleBlueLightToDark: string;
    /**
     * Purple-Red Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleRedBright: string;
    /**
     * Purple-Red Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    purpleRedLightToDark: string;
    /**
     * Red Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redBright: string;
    /**
     * Red Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redLightToDark: string;
    /**
     * Red to Blue Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redToBlueDivergingBright: string;
    /**
     * Red to Blue Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redToBlueDivergingDark: string;
    /**
     * Red to Green.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redToGreen: string;
    /**
     * Red to Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redToGreenDivergingBright: string;
    /**
     * Red to Green Diverging.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    redToGreenDivergingDark: string;
    /**
     * Slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    slope: string;
    /**
     * Spectrum-Full Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    spectrumFullBright: string;
    /**
     * Spectrum-Full Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    spectrumFullDark: string;
    /**
     * Spectrum-Full Light.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    spectrumFullLight: string;
    /**
     * Surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    surface: string;
    /**
     * Temperature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    temperature: string;
    /**
     * White to Black.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    whiteToBlack: string;
    /**
     * Yellow to Dark Red.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    yellowToDarkRed: string;
    /**
     * Yellow to Green to Dark Blue.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    yellowToGreenToDarkBlue: string;
    /**
     * Yellow to Red.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    yellowToRed: string;
    /**
     * Yellow-Green Bright.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    yellowGreenBright: string;
    /**
     * Yellow-Green Light to Dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#colorRampName)
     */
    yellowGreenLightToDark: string;
  }

  export interface rasterFunctionConstantsConvolutionKernel {
    /**
     * User defined kernel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    userDefined: number;
    /**
     * Horizontal line detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    lineDetectionHorizontal: number;
    /**
     * Vertical line detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    lineDetectionVertical: number;
    /**
     * Left diagonal line detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    lineDetectionLeftDiagonal: number;
    /**
     * Right diagonal line detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    lineDetectionRightDiagonal: number;
    /**
     * North gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientNorth: number;
    /**
     * West gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientWest: number;
    /**
     * East gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientEast: number;
    /**
     * South gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientSouth: number;
    /**
     * North east gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientNorthEast: number;
    /**
     * North west gradient detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    gradientNorthWest: number;
    /**
     * Smooths the data by reducing local variation and removing noise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    smoothArithmeticMean: number;
    /**
     * Smooths (low-pass) the data by reducing local variation and removing noise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    smoothing3x3: number;
    /**
     * Smooths (low-pass) the data by reducing local variation and removing noise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    smoothing5x5: number;
    /**
     * Sharpens the date by calculating the focal sum statistic for each cell of the input using a weighted kernel neighborhood.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sharpen: number;
    /**
     * Sharpens the date by calculating the focal sum statistic for each cell of the input using a weighted kernel neighborhood.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sharpen2: number;
    /**
     * Sharpens the date by calculating the focal sum statistic for each cell of the input using a weighted kernel neighborhood.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sharpening3x3: number;
    /**
     * Sharpens the date by calculating the focal sum statistic for each cell of the input using a weighted kernel neighborhood.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sharpening5x5: number;
    /**
     * Laplacian filters are often used for edge detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    laplacian3x3: number;
    /**
     * Laplacian filters are often used for edge detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    laplacian5x5: number;
    /**
     * The horizontal Sobel filter is used for edge detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sobelHorizontal: number;
    /**
     * The vertical Sobel filter is used for edge detection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    sobelVertical: number;
    /**
     * The point spread function portrays the distribution of light from a point source through a lense.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    pointSpread: number;
    /**
     * No kernel type is specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#convolutionKernel)
     */
    none: number;
  }

  export interface rasterFunctionConstantsCurvatureType {
    /**
     * Combines the profile and planform curvatures.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#curvatureType)
     */
    standard: number;
    /**
     * Is perpendicular to the direction of the maximum slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#curvatureType)
     */
    planform: number;
    /**
     * Is parallel to the slope and indicates the direction of maximum slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#curvatureType)
     */
    profile: number;
  }

  export interface rasterFunctionConstantsHillshadeType {
    /**
     * Calculates hillshade from a single illumination direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#hillshadeType)
     */
    traditional: number;
    /**
     * Combines light from multiple sources to represent an enhanced visualization of the terrain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#hillshadeType)
     */
    multidirectional: number;
  }

  export interface rasterFunctionConstantsLocalArithmeticOperation {
    /**
     * Adds (sums) the values of two rasters on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    plus: number;
    /**
     * Subtracts the value of the second input raster from the value of the first input raster on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    minus: number;
    /**
     * Multiplies the values of two rasters on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    times: number;
    /**
     * Calculates the square root of the cell values in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    sqrt: number;
    /**
     * Raises the cell values in a raster to the power of the values found in another raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    power: number;
    /**
     * Calculates the absolute value of the cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    abs: number;
    /**
     * Divides the values of two rasters on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    divide: number;
    /**
     * Calculates the base e exponential of the cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    exp: number;
    /**
     * Calculates the base 10 exponential of the cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    exp10: number;
    /**
     * Calculates the base 2 exponential of the cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    exp2: number;
    /**
     * Converts each cell value of a raster to an integer by truncation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    int: number;
    /**
     * Converts each cell value of a raster into a floating-point representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    float: number;
    /**
     * Calculates the natural logarithm (base e) of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    ln: number;
    /**
     * Calculates the base 10 logarithm of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    log10: number;
    /**
     * Calculates the base 2 logarithm of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    log2: number;
    /**
     * Finds the remainder (modulo) of the first raster when divided by the second raster on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    mod: number;
    /**
     * Changes the sign (multiplies by -1) of the cell values of the input raster on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    negate: number;
    /**
     * Returns the next lower integer value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    roundDown: number;
    /**
     * Returns the next higher integer value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    roundUp: number;
    /**
     * Calculates the square of the cell values in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localArithmeticOperation)
     */
    square: number;
  }

  export interface rasterFunctionConstantsLocalConditionalOperation {
    /**
     * Set Null sets identified cell locations to NoData based on a specified criteria.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localConditionalOperation)
     */
    setNull: number;
    /**
     * Performs a conditional If, Then, Else operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localConditionalOperation)
     */
    conditional: number;
  }

  export interface rasterFunctionConstantsLocalLogicalOperation {
    /**
     * Performs a Bitwise And operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseAnd: number;
    /**
     * Performs a Bitwise Left Shift operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseLeftShift: number;
    /**
     * Performs a Bitwise Not (complement) operation on the binary value of an input raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseNot: number;
    /**
     * Performs a Bitwise Or operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseOr: number;
    /**
     * Performs a Bitwise Right Shift operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseRightShift: number;
    /**
     * Performs a Bitwise eXclusive Or operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    bitwiseXOr: number;
    /**
     * Performs a Boolean And operation on the cell values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    booleanAnd: number;
    /**
     * Performs a Boolean Not (complement) operation on the cell values of the input raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    booleanNot: number;
    /**
     * Performs a Boolean Or operation on the cell values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    booleanOr: number;
    /**
     * Performs a Boolean eXclusive Or operation on the cell values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    booleanXOr: number;
    /**
     * Performs a Relational equal-to operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    equalTo: number;
    /**
     * Performs a Relational greater-than operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    greaterThan: number;
    /**
     * Performs a Relational greater-than-or-equal-to operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    greaterThanEqual: number;
    /**
     * Performs a Relational less-than operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    lessThan: number;
    /**
     * Performs a Relational less-than-or-equal-to operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    lessThanEqual: number;
    /**
     * Determines which values from the input raster are NoData on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    isNull: number;
    /**
     * Performs a Relational not-equal-to operation on two inputs on a cell-by-cell basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localLogicalOperation)
     */
    notEqual: number;
  }

  export interface rasterFunctionConstantsLocalTrigonometricOperation {
    /**
     * Calculates the inverse cosine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    acos: number;
    /**
     * Calculates the inverse sine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    asin: number;
    /**
     * Calculates the inverse tangent of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    atan: number;
    /**
     * Calculates the inverse hyperbolic tangent of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    atanh: number;
    /**
     * Calculates the cosine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    cos: number;
    /**
     * Calculates the hyperbolic cosine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    cosh: number;
    /**
     * Calculates the sine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    sin: number;
    /**
     * Calculates the hyperbolic sine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    sinh: number;
    /**
     * Calculates the tangent of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    tan: number;
    /**
     * Calculates the hyperbolic tangent of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    tanh: number;
    /**
     * Calculates the inverse hyperbolic cosine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    acosh: number;
    /**
     * Calculates the inverse hyperbolic sine of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    asinh: number;
    /**
     * Calculates the inverse tangent (based on x,y) of cells in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#localTrigonometricOperation)
     */
    atan2: number;
  }

  export interface rasterFunctionConstantsMissingBandAction {
    /**
     * Finds the best available band to use in place of the missing band based on wavelength.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#missingBandAction)
     */
    bestMatch: number;
    /**
     * If the input dataset is missing any band specified in the Band parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#missingBandAction)
     */
    fail: number;
  }

  export interface rasterFunctionConstantsNoDataInterpretation {
    /**
     * If the NoData value you specify occurs for a cell in a specified band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#noDataInterpretation)
     */
    matchAny: number;
    /**
     * The NoData values you specify for each band must occur in the same pixel for the output image to contain the NoData pixel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#noDataInterpretation)
     */
    matchAll: number;
  }

  export interface rasterFunctionConstantsSlopeType {
    /**
     * The inclination of slope is calculated in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#slopeType)
     */
    degree: number;
    /**
     * The inclination of slope is calculated as percentage values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#slopeType)
     */
    percentRise: number;
    /**
     * The inclination of slope is calculated the same as DEGREE.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#slopeType)
     */
    adjusted: number;
  }

  export interface rasterFunctionConstantsStretchType {
    /**
     * If the stretch type is None.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    none: number;
    /**
     * The standard deviation stretch type applies a linear stretch between the values defined by the standard deviation (n) value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    standardDeviation: number;
    /**
     * The histogram equalization stretch type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    histogramEqualization: number;
    /**
     * The minMax stretch type applies a linear stretch based on the output minimum and output maximum pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    minMax: number;
    /**
     * The percent clip stretch type applies a linear stretch between the defined percent clip minimum and percent clip maximum pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    percentClip: number;
    /**
     * The Sigmoid contrast stretch is designed to highlight moderate pixel values in your imagery while maintaining sufficient contrast at the extremes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionConstants.html#stretchType)
     */
    sigmoid: number;
  }

  /**
   * Various utility functions that create [RasterFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html) for imagery processing.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html)
   */
  interface rasterFunctionUtils {
    /**
     * A token representing the image service raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#defaultRaster)
     */
    readonly defaultRaster: "$$";

    /**
     * Creates a raster function that calculates the absolute value of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#abs)
     */
    abs(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse cosine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#acos)
     */
    acos(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse hyperbolic cosine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#acosh)
     */
    acosh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse sine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#asin)
     */
    asin(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse hyperbolic sine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#asinh)
     */
    asinh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Aspect function to identify the downslope direction of the maximum rate of change in value from each cell to its neighbors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#aspect)
     */
    aspect(parameters: rasterFunctionUtilsAspectParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse tangent of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#atan)
     */
    atan(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse tangent (based on x,y) of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#atan2)
     */
    atan2(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the inverse hyperbolic tangent of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#atanh)
     */
    atanh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate BAI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticBAI)
     */
    bandArithmeticBAI(parameters: rasterFunctionUtilsBandArithmeticBAIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate CIg.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIg)
     */
    bandArithmeticCIg(parameters: rasterFunctionUtilsBandArithmeticCIgParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate CIre.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIre)
     */
    bandArithmeticCIre(parameters: rasterFunctionUtilsBandArithmeticCIreParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate ClayMinerals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticClayMinerals)
     */
    bandArithmeticClayMinerals(parameters: rasterFunctionUtilsBandArithmeticClayMineralsParameters): RasterFunction;
    /**
     * Creates a custom Band Arithmetic function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCustom)
     */
    bandArithmeticCustom(parameters: rasterFunctionUtilsBandArithmeticCustomParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate EVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticEVI)
     */
    bandArithmeticEVI(parameters: rasterFunctionUtilsBandArithmeticEVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate FerrousMinerals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticFerrousMinerals)
     */
    bandArithmeticFerrousMinerals(
      parameters: rasterFunctionUtilsBandArithmeticFerrousMineralsParameters,
    ): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate GEMI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGEMI)
     */
    bandArithmeticGEMI(parameters: rasterFunctionUtilsBandArithmeticGEMIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate GNDVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGNDVI)
     */
    bandArithmeticGNDVI(parameters: rasterFunctionUtilsBandArithmeticGNDVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate GVITM.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGVITM)
     */
    bandArithmeticGVITM(parameters: rasterFunctionUtilsBandArithmeticGVITMParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate IronOxide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticIronOxide)
     */
    bandArithmeticIronOxide(parameters: rasterFunctionUtilsBandArithmeticIronOxideParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate MNDWI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMNDWI)
     */
    bandArithmeticMNDWI(parameters: rasterFunctionUtilsBandArithmeticMNDWIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate MSAVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMSAVI)
     */
    bandArithmeticMSAVI(parameters: rasterFunctionUtilsBandArithmeticMSAVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate MTVI2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMTVI2)
     */
    bandArithmeticMTVI2(parameters: rasterFunctionUtilsBandArithmeticMTVI2Parameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NBR.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNBR)
     */
    bandArithmeticNBR(parameters: rasterFunctionUtilsBandArithmeticNBRParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NDBI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDBI)
     */
    bandArithmeticNDBI(parameters: rasterFunctionUtilsBandArithmeticNDBIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NDMI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDMI)
     */
    bandArithmeticNDMI(parameters: rasterFunctionUtilsBandArithmeticNDMIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NDSI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDSI)
     */
    bandArithmeticNDSI(parameters: rasterFunctionUtilsBandArithmeticNDSIParameters): RasterFunction;
    /**
     * Creates a NDVI function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVI)
     */
    bandArithmeticNDVI(parameters: rasterFunctionUtilsBandArithmeticNDVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NDVIre.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVIre)
     */
    bandArithmeticNDVIre(parameters: rasterFunctionUtilsBandArithmeticNDVIreParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate NDWI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDWI)
     */
    bandArithmeticNDWI(parameters: rasterFunctionUtilsBandArithmeticNDWIParameters): RasterFunction;
    /**
     * CCreates a Band Arithmetic function to calculate PVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    bandArithmeticPVI(parameters: rasterFunctionUtilsBandArithmeticPVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate RTVICore.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticRTVICore)
     */
    bandArithmeticRTVICore(parameters: rasterFunctionUtilsBandArithmeticRTVICoreParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate SAVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSAVI)
     */
    bandArithmeticSAVI(parameters: rasterFunctionUtilsBandArithmeticSAVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate SR.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSR)
     */
    bandArithmeticSR(parameters: rasterFunctionUtilsBandArithmeticSRParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate SRre.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSRre)
     */
    bandArithmeticSRre(parameters: rasterFunctionUtilsBandArithmeticSRreParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate Sultan index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSultan)
     */
    bandArithmeticSultan(parameters: rasterFunctionUtilsBandArithmeticSultanParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate TSAVI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    bandArithmeticTSAVI(parameters: rasterFunctionUtilsBandArithmeticTSAVIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate VARI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticVARI)
     */
    bandArithmeticVARI(parameters: rasterFunctionUtilsBandArithmeticVARIParameters): RasterFunction;
    /**
     * Creates a Band Arithmetic function to calculate WNDWI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    bandArithmeticWNDWI(parameters: rasterFunctionUtilsBandArithmeticWNDWIParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise And operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseAnd)
     */
    bitwiseAnd(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise Left Shift operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseLeftShift)
     */
    bitwiseLeftShift(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise Not (complement) operation on the binary value of an input raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseNot)
     */
    bitwiseNot(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise Or operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseOr)
     */
    bitwiseOr(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise Right Shift operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseRightShift)
     */
    bitwiseRightShift(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Bitwise Xor operation on the binary values of two input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bitwiseXor)
     */
    bitwiseXor(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Boolean And operation on the pixel values of two input rasters See [Boolean And function](https://pro.arcgis.com/en/pro-app/latest/help/analysis/raster-functions/boolean-and-function.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#booleanAnd)
     */
    booleanAnd(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Boolean Not (complement) operation on the pixel values of the input raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#booleanNot)
     */
    booleanNot(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Boolean Or operation on the pixel values of two input rasters See [Boolean Or function](https://pro.arcgis.com/en/pro-app/latest/help/analysis/raster-functions/boolean-or-function.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#booleanOr)
     */
    booleanOr(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a Boolean Xor operation on the pixel values of two input rasters See [Boolean Xor function](https://pro.arcgis.com/en/pro-app/latest/help/analysis/raster-functions/boolean-xor-function.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#booleanXor)
     */
    booleanXor(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that calculates a statistic from multiple rasters, on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cellStatistics)
     */
    cellStatistics(parameters: rasterFunctionUtilsCellStatisticsParameters): RasterFunction;
    /**
     * Extracts a portion of an image based on an [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) or a [polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#clip)
     */
    clip(parameters: rasterFunctionUtilsClipParameters): RasterFunction;
    /**
     * Creates a Colormap function to define a colormap for a raster by specifying a corresponding color for each pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#colormap)
     */
    colormap(
      parameters: RasterColormapByMapParameters | RasterColormapByNameParameters | RasterColormapByRampParameters,
    ): RasterFunction;
    /**
     * Works with a single band image service that has an internal color map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#colormapToRGB)
     */
    colormapToRGB(parameters: rasterFunctionUtilsColormapToRGBParameters): RasterFunction;
    /**
     * Creates a Composite Bands function to combine multiple inputs into one multiband raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#compositeBands)
     */
    compositeBands(parameters: rasterFunctionUtilsCompositeBandsParameters): RasterFunction;
    /**
     * Creates a raster function that sets the truthy pixels (value is not 0) to the pixel value from a true raster, and falsy pixels (value is 0) to the pixel value of the false raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#conditional)
     */
    conditional(parameters: ConditionalParameters): RasterFunction;
    /**
     * Creates a Contrast And Brightness function that enhances the appearance of raster data by modifying the brightness and contrast within the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#contrastBrightness)
     */
    contrastBrightness(parameters: rasterFunctionUtilsContrastBrightnessParameters): RasterFunction;
    /**
     * Creates a Convolution function that performs filtering using the given kernel to enhance the image, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    convolution(parameters: rasterFunctionUtilsConvolutionParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the cosine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cos)
     */
    cos(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the hyperbolic cosine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cosh)
     */
    cosh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Curvature function that calculates the shape or curvature of the slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#curvature)
     */
    curvature(parameters: rasterFunctionUtilsCurvatureParameters): RasterFunction;
    /**
     * Creates a raster function that divides the values of two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#divide)
     */
    divide(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs an equal-to operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#equalTo)
     */
    equalTo(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the base e exponential of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#exp)
     */
    exp(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the base 10 exponential of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#exp10)
     */
    exp10(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the base 2 exponential of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#exp2)
     */
    exp2(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates an Extract Band function to extract one or more bands from a multiband raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#extractBand)
     */
    extractBand(
      parameters:
        | ExtractBandByIdParameters
        | ExtractBandByNameParameters
        | ExtractBandByWavelengthParameters
        | rasterFunctionUtilsExtractBandParameters,
    ): RasterFunction;
    /**
     * Creates a raster function that converts each pixel value of a raster into a floating-point representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#float)
     */
    float(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Converts a multiband image into a single-band grayscale image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#grayscale)
     */
    grayscale(parameters: rasterFunctionUtilsGrayscaleParameters): RasterFunction;
    /**
     * Creates a raster function that performs a relational greater-than operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#greaterThan)
     */
    greaterThan(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a relational greater-than-or-equal-to operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#greaterThanEqual)
     */
    greaterThanEqual(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a hillshade function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    hillshade(parameters: rasterFunctionUtilsHillshadeParameters): RasterFunction;
    /**
     * Creates a raster function that converts each pixel value of a raster to an integer by truncation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#int)
     */
    int(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that determines which values from the input raster are NoData on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#isNull)
     */
    isNull(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that performs a relational less-than operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#lessThan)
     */
    lessThan(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that performs a relational less-than-or-equal-to operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#lessThanEqual)
     */
    lessThanEqual(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the natural logarithm (base e) of each pixel in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#log)
     */
    log(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the base 10 logarithm of each pixel in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#log10)
     */
    log10(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the base 2 logarithm of each pixel in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#log2)
     */
    log2(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Mask function to specify one or more NoData values, or a range of valid pixel values, to be removed from an output raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mask)
     */
    mask(parameters: rasterFunctionUtilsMaskParameters): RasterFunction;
    /**
     * Creates a raster function that subtracts the value of the second input raster from the value of the first input raster on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#minus)
     */
    minus(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that finds the remainder (modulo) of the first raster when divided by the second raster on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mod)
     */
    mod(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that finds the changes the sign (multiplies by -1) of the pixel values of the input raster on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#negate)
     */
    negate(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that performs a relational not-equal-to operation on two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#notEqual)
     */
    notEqual(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that adds (sums) the values of two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#plus)
     */
    plus(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a raster function that raises the pixel values in a raster to the power of the values found in another raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#power)
     */
    power(parameters: Math2RastersParameters): RasterFunction;
    /**
     * Creates a Remap function to change or reclassify the pixel values of the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#remap)
     */
    remap(parameters: rasterFunctionUtilsRemapParameters): RasterFunction;
    /**
     * Creates a raster function that returns the next lower integer, as a floating-point value, for each pixel in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#roundDown)
     */
    roundDown(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that returns the next higher integer, as a floating-point value, for each pixel in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#roundUp)
     */
    roundUp(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that sets the truthy pixels (value is not 0) to NoData, and falsy pixels (value is 0) to the pixel value of the false raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#setNull)
     */
    setNull(parameters: SetNullParameters): RasterFunction;
    /**
     * The ShadedRelief function creates a color 3D model of the terrain by merging the images from the elevation-coded and hillshade methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#shadedRelief)
     */
    shadedRelief(
      parameters:
        | ShadedReliefWithColorRampParameters
        | ShadedReliefWithColormapParameters
        | ShadedReliefWithColorRampNameParameters,
    ): RasterFunction;
    /**
     * Creates a raster function that calculates the sine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#sin)
     */
    sin(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the hyperbolic sine of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#sinh)
     */
    sinh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Slope function that calculates the rate of change of elevation for each digital elevation model (DEM) cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    slope(parameters: rasterFunctionUtilsSlopeParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the square root of the pixel values in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#sqrt)
     */
    sqrt(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the square of the pixel values in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#square)
     */
    square(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a Statistics function that calculates focal statistics for each pixel of an image based on a defined focal neighborhood.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    statistics(parameters: rasterFunctionUtilsStatisticsParameters): RasterFunction;
    /**
     * Creates a Statistics And Histogram function to define the statistics and histogram of a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statisticsHistogram)
     */
    statisticsHistogram(parameters: rasterFunctionUtilsStatisticsHistogramParameters): RasterFunction;
    /**
     * Creates a Stretch function using min-max stretch type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    stretchMinMax(parameters: rasterFunctionUtilsStretchMinMaxParameters): RasterFunction;
    /**
     * Creates a Stretch function using percent-clip stretch type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    stretchPercentClip(parameters: rasterFunctionUtilsStretchPercentClipParameters): RasterFunction;
    /**
     * Creates a Stretch function using standard-deviation stretch type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    stretchStandardDeviation(parameters: rasterFunctionUtilsStretchStandardDeviationParameters): RasterFunction;
    /**
     * Creates an Attribute Table function to specify an attribute table for the input categorical raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#table)
     */
    table(parameters: rasterFunctionUtilsTableParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the tangent of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#tan)
     */
    tan(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that calculates the hyperbolic tangent of the pixels in a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#tanh)
     */
    tanh(parameters: Math1RasterParameters): RasterFunction;
    /**
     * Creates a raster function that multiplies the values of two rasters on a pixel-by-pixel basis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#times)
     */
    times(parameters: Math2RastersParameters): RasterFunction;
  }

  export const rasterFunctionUtils: rasterFunctionUtils;

  /**
   * Conditional parameters
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ConditionalParameters)
   */
  export interface ConditionalParameters {
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster: RasterArgument;
    /**
     * The true raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ConditionalParameters)
     */
    trueRaster: RasterArgument;
    /**
     * The false raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ConditionalParameters)
     */
    falseRaster: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ConditionalParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Extract bands parameters specified using band ids.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByIdParameters)
   */
  export interface ExtractBandByIdParameters {
    /**
     * The 0-based band ids.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByIdParameters)
     */
    bandIds: number[];
    /**
     * Indicate whether to use best possible matching band when the requested band is missing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByIdParameters)
     */
    missingBandAction?: "fail" | "best-match";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByIdParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Extract bands parameters specified using band wavelengths.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByNameParameters)
   */
  export interface ExtractBandByNameParameters {
    /**
     * The band wavelengths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByNameParameters)
     */
    bandWavelengths: number[];
    /**
     * Indicate whether to use best possible matching band when the requested band is missing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByNameParameters)
     */
    missingBandAction?: "fail" | "best-match";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByNameParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Extract bands parameters specified using band names.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByWavelengthParameters)
   */
  export interface ExtractBandByWavelengthParameters {
    /**
     * The band names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByWavelengthParameters)
     */
    bandNames: string[];
    /**
     * Indicate whether to use best possible matching band when the requested band is missing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByWavelengthParameters)
     */
    missingBandAction?: "fail" | "best-match";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ExtractBandByWavelengthParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Input parameters for performing math operations on one input raster.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#Math1RasterParameters)
   */
  export interface Math1RasterParameters {
    /**
     * The first raster.
     *
     * [Read more...](global.html#raster)
     */
    raster: RasterArgument;
    /**
     * The output pixel type, default is `unknown`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#Math1RasterParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Input parameters for perform math operations between two rasters.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#Math2RastersParameters)
   */
  export interface Math2RastersParameters {
    /**
     * The first raster input.
     *
     * [Read more...](global.html#raster)
     */
    raster: RasterArgument;
    /**
     * The second raster input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#Math2RastersParameters)
     */
    raster2: RasterArgument;
    /**
     * The output pixel type, default is `unknown`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#Math2RastersParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Range map pixel values.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#PixelValueRangeMap)
   */
  export interface PixelValueRangeMap {
    /**
     * The pixel value range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#PixelValueRangeMap)
     */
    range: number[];
    /**
     * The mapped pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#PixelValueRangeMap)
     */
    output: number;
  }

  /**
   * Raster argument used to specify raster(s) used in raster processing.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterArgument)
   */
  export type RasterArgument = RasterFunction | string | number;

  /**
   * Raster colormap input parameters specified using a colormap.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByMapParameters)
   */
  export interface RasterColormapByMapParameters {
    /**
     * The colormap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByMapParameters)
     */
    colormap: RasterValueToColor[] | number[][];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByMapParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Raster colormap input parameters specified using colormap name.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByNameParameters)
   */
  export interface RasterColormapByNameParameters {
    /**
     * The name of a predefined colorramp or colormap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByNameParameters)
     */
    colorRampName:
      | "aspect"
      | "black-to-white"
      | "blue-bright"
      | "blue-light-to-dark"
      | "blue-green-bright"
      | "blue-green-light-to-dark"
      | "brown-light-to-dark"
      | "brown-to-blue-green-diverging-right"
      | "brown-to-blue-green-diverging-dark"
      | "coefficient-bias"
      | "cold-to-hot-diverging"
      | "condition-number"
      | "cyan-to-purple"
      | "cyan-light-to-blue-dark"
      | "distance"
      | "elevation1"
      | "elevation2"
      | "errors"
      | "gray-light-to-dark"
      | "green-bright"
      | "green-light-to-dark"
      | "green-to-blue"
      | "orange-bright"
      | "orange-light-to-dark"
      | "partial-spectrum"
      | "partial-spectrum-1-diverging"
      | "partial-spectrum-2-diverging"
      | "pink-to-yellow-green-diverging-bright"
      | "pink-to-yellow-green-diverging-dark"
      | "precipitation"
      | "prediction"
      | "purple-bright"
      | "purple-to-green-diverging-bright"
      | "purple-to-green-diverging-dark"
      | "purple-blue-bright"
      | "purple-blue-light-to-dark"
      | "purple-red-bright"
      | "purple-red-light-to-dark"
      | "red-bright"
      | "red-light-to-dark"
      | "red-to-blue-diverging-bright"
      | "red-to-blue-diverging-dark"
      | "red-to-green"
      | "red-to-green-diverging-bright"
      | "red-to-green-diverging-dark"
      | "slope"
      | "spectrum-full-bright"
      | "spectrum-full-dark"
      | "spectrum-full-light"
      | "surface"
      | "temperature"
      | "white-to-black"
      | "yellow-to-dark-red"
      | "yellow-to-green-to-dark-blue"
      | "yellow-to-red"
      | "yellow-green-bright"
      | "yellow-green-light-to-dark"
      | "random"
      | "elevation"
      | "gray"
      | "hillshade"
      | "ndvi"
      | "ndvi2"
      | "ndvi3";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByNameParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * Raster colormap input parameters specified using a colorramp.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByRampParameters)
   */
  export interface RasterColormapByRampParameters {
    /**
     * A colorramp
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByRampParameters)
     */
    colorRamp: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterColormapByRampParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsAspectParameters {
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#aspect)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticBAIParameters {
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticBAI)
     */
    redBandId: number;
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticBAI)
     */
    nirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticBAI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticCIgParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIg)
     */
    nirBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIg)
     */
    greenBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIg)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticCIreParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIre)
     */
    nirBandId: number;
    /**
     * The 0-based red edge band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIre)
     */
    reBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCIre)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticClayMineralsParameters {
    /**
     * The 0-based shortwave infrared band id centering around 1.5—1.75µm.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticClayMinerals)
     */
    swir1BandId: number;
    /**
     * The 0-based shortwave infrared band id centering around 2.08—2.35µm.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticClayMinerals)
     */
    swir2BandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticClayMinerals)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticCustomParameters {
    /**
     * The custom band index, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCustom)
     */
    bandIndexes: string;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticCustom)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticEVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticEVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticEVI)
     */
    redBandId: number;
    /**
     * The 0-based blue band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticEVI)
     */
    blueBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticEVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticFerrousMineralsParameters {
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticFerrousMinerals)
     */
    swir1BandId: number;
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticFerrousMinerals)
     */
    nirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticFerrousMinerals)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticGEMIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGEMI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGEMI)
     */
    redBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGEMI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticGNDVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGNDVI)
     */
    nirBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGNDVI)
     */
    greenBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGNDVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticGVITMParameters {
    /**
     * The bandIds array representing Landsat band 1, 2, 3, 4, 5, 7.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGVITM)
     */
    bandIds: number[];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticGVITM)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticIronOxideParameters {
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticIronOxide)
     */
    redBandId: number;
    /**
     * The 0-based blue band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticIronOxide)
     */
    blueBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticIronOxide)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticMNDWIParameters {
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMNDWI)
     */
    greenBandId: number;
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMNDWI)
     */
    swirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMNDWI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticMSAVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMSAVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMSAVI)
     */
    redBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMSAVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticMTVI2Parameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMTVI2)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMTVI2)
     */
    redBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMTVI2)
     */
    greenBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticMTVI2)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNBRParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNBR)
     */
    nirBandId: number;
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNBR)
     */
    swirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNBR)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDBIParameters {
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDBI)
     */
    swirBandId: number;
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDBI)
     */
    nirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDBI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDMIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDMI)
     */
    nirBandId: number;
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDMI)
     */
    swirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDMI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDSIParameters {
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDSI)
     */
    greenBandId: number;
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDSI)
     */
    swirBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDSI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVI)
     */
    redBandId: number;
    /**
     * Output values are linearly scaled to 0 to 200 when scientificOutput is false.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVI)
     */
    scientificOutput?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDVIreParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVIre)
     */
    nirBandId: number;
    /**
     * The 0-based red edge band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVIre)
     */
    reBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDVIre)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticNDWIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDWI)
     */
    nirBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDWI)
     */
    greenBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticNDWI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticPVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    redBandId: number;
    /**
     * The soil line slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    slope?: boolean;
    /**
     * The soil line gradient.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    gradient?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticPVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticRTVICoreParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticRTVICore)
     */
    nirBandId: number;
    /**
     * The 0-based red edge band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticRTVICore)
     */
    reBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticRTVICore)
     */
    greenBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticRTVICore)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticSAVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSAVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSAVI)
     */
    redBandId: number;
    /**
     * The amount of green vegetation cover from 0 to 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSAVI)
     */
    factor?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSAVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticSRParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSR)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSR)
     */
    redBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSR)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticSRreParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSRre)
     */
    nirBandId: number;
    /**
     * The 0-based red edge band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSRre)
     */
    reBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSRre)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticSultanParameters {
    /**
     * The bandIds array representing band 1, 3, 4, 5, 6.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSultan)
     */
    bandIds: number[];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticSultan)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticTSAVIParameters {
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    nirBandId: number;
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    redBandId: number;
    /**
     * The soil line slope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    slope?: boolean;
    /**
     * The soil line intercept.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    intercept?: boolean;
    /**
     * The adjustment factor to minimize soil noise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    factor?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticTSAVI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticVARIParameters {
    /**
     * The 0-based red band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticVARI)
     */
    redBandId: number;
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticVARI)
     */
    greenBandId: number;
    /**
     * The 0-based blue band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticVARI)
     */
    blueBandId: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticVARI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsBandArithmeticWNDWIParameters {
    /**
     * The 0-based green band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    greenBandId: number;
    /**
     * The 0-based near infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    nirBandId: number;
    /**
     * The 0-based shortwave infrared band id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    swirBandId: number;
    /**
     * A weighted coefficient ranging from 0 to 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    alpha?: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#bandArithmeticWNDWI)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsCellStatisticsParameters {
    /**
     * The input rasters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cellStatistics)
     */
    rasters: RasterArgument[];
    /**
     * The true raster, if the first input is a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cellStatistics)
     */
    statType: "min" | "max" | "majority" | "mean" | "minority" | "range" | "stddev" | "sum" | "variety" | "median";
    /**
     * When true, process the single input raster as multiple bands.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cellStatistics)
     */
    processAsMultiband?: boolean;
    /**
     * The output pixel type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#cellStatistics)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsClipParameters {
    /**
     * An extent or a polygon geometry to define the clip shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#clip)
     */
    geometry: Polygon | Extent;
    /**
     * When `false`, the imagery within the clip geometry will be kept in the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#clip)
     */
    keepOutside?: boolean;
    /**
     * The output pixel type.
     *
     * @default "unknown"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#clip)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsColormapToRGBParameters {
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument[];
  }

  export interface rasterFunctionUtilsCompositeBandsParameters {
    /**
     * The input raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#compositeBands)
     */
    rasters?: RasterArgument[];
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#compositeBands)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsContrastBrightnessParameters {
    /**
     * Contrast offset from -100 to 100.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#contrastBrightness)
     */
    contrastOffset: number;
    /**
     * Brightness offset from -100 to 100.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#contrastBrightness)
     */
    brightnessOffset: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#contrastBrightness)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsConvolutionParameters {
    /**
     * Choose a predefined kernel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    convolutionType:
      | "user-defined"
      | "line-detection-horizontal"
      | "line-detection-vertical"
      | "line-detection-left-diagonal"
      | "line-detection-right-diagonal"
      | "gradient-north"
      | "gradient-west"
      | "gradient-east"
      | "gradient-south"
      | "gradient-north-east"
      | "gradient-north-west"
      | "smooth-arithmetic-mean"
      | "smoothing3x3"
      | "smoothing5x5"
      | "sharpening3x3"
      | "sharpening5x5"
      | "laplacian3x3"
      | "laplacian5x5"
      | "sobel-horizontal"
      | "sobel-vertical"
      | "sharpen"
      | "sharpen2"
      | "point-spread"
      | "none";
    /**
     * The number of rows, only needed for user-defined kernel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    rows?: number;
    /**
     * The number of columns, only needed for user-defined kernel
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    cols?: number;
    /**
     * The kernel size of rows x columns arranged in row-major order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    kernel?: number[];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#convolution)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsCurvatureParameters {
    /**
     * The curvature type
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#curvature)
     */
    curvatureType: "standard" | "planform" | "profile";
    /**
     * A ratio of z unit / xy unit, with optional exaggeration factored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#curvature)
     */
    zFactor: number;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#curvature)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsExtractBandParameters {
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#extractBand)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsGrayscaleParameters {
    /**
     * The weights applied to each input band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#grayscale)
     */
    weights: number[];
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#grayscale)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsHillshadeParameters {
    /**
     * The hillshade type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    hillshadeType: "traditional" | "multi-directional";
    /**
     * Altitude is the sun's angle of elevation above the horizon and ranges from 0 to 90 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    altitude?: number;
    /**
     * Azimuth is the sun's relative position along the horizon (in degrees).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    azimuth?: number;
    /**
     * The zFactor is a scaling factor used to convert the elevation values for two purposes: 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    zFactor: number;
    /**
     * The shaded result is scaled dynamically by adjusting the zFactor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    scalingType?: "none" | "adjusted";
    /**
     * It accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    pixelSizePower?: number;
    /**
     * It accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    pixelSizeZFactor?: number;
    /**
     * Use to avoid any resampling artifacts that may occur along the edges of a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    removeEdgeEffect?: boolean;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#hillshade)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsMaskParameters {
    /**
     * The included values for each band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mask)
     */
    includedRanges: number[][];
    /**
     * The nodata values for each band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mask)
     */
    noDataValues: number[][];
    /**
     * Result is no data if all bands are nodata when match-all is specified, any if any band is nodata when match-any is specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mask)
     */
    noDataInterpretation?: "match-any" | "match-all";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#mask)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsRemapParameters {
    /**
     * Range maps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#remap)
     */
    rangeMaps: PixelValueRangeMap[];
    /**
     * Unmatched pixel values are passed through when true, or set to NoData when false.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#remap)
     */
    allowUnmatched?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#remap)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsSlopeParameters {
    /**
     * The slope type
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    slopeType: "degree" | "percent-rise" | "adjusted";
    /**
     * A ratio of z unit / xy unit, with optional exaggeration factored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    zFactor: number;
    /**
     * Pixel size factor accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    pixelSizePower?: number;
    /**
     * Pixel Size Power accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    pixelSizeZFactor?: number;
    /**
     * Remove edge effect when true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    removeEdgeEffect?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#slope)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsStatisticsHistogramParameters {
    /**
     * The statistics parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statisticsHistogram)
     */
    statistics?: RasterBandStatistics[];
    /**
     * The histogram parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statisticsHistogram)
     */
    histograms?: RasterHistogram[];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statisticsHistogram)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsStatisticsParameters {
    /**
     * The statistics type
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    statisticsType: "min" | "max" | "majority" | "mean" | "minority" | "stddev" | "median";
    /**
     * Number of rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    rows: number;
    /**
     * Number of columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    cols: number;
    /**
     * When true only fills noData pixels
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    fillNoDataOnly?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#statistics)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsStretchMinMaxParameters {
    /**
     * Default is the input data source's statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    statistics?: RasterBandStatistics[];
    /**
     * The minimum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    outputMin?: number;
    /**
     * The maximum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    outputMax?: number;
    /**
     * When true, gamma stretch is applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    useGamma?: boolean;
    /**
     * The gamma values used for each band, needed when useGamma is true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    gamma?: number[];
    /**
     * When true, perform dynamic range adjustment (not supported on client side).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchMinMax)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsStretchPercentClipParameters {
    /**
     * Percentage of the lowest pixel values to be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    minPercent: number;
    /**
     * Percentage of the highest pixel values to be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    maxPercent: number;
    /**
     * The minimum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    outputMin?: number;
    /**
     * The maximum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    outputMax?: number;
    /**
     * When true, gamma stretch is applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    useGamma?: boolean;
    /**
     * The gamma values used for each band, needed when useGamma is true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    gamma?: number[];
    /**
     * When true, perform dynamic range adjustment (not supported on client side)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchPercentClip)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsStretchStandardDeviationParameters {
    /**
     * Values with in the given number of standard deviations are stretched to [outputMin, outputMax], those fall outside are clamped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    numberOfStandardDeviations: number;
    /**
     * The minimum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    outputMin?: number;
    /**
     * The maximum output value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    outputMax?: number;
    /**
     * When true, gamma stretch is applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    useGamma?: boolean;
    /**
     * The gamma values used for each band, needed when useGamma is true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    gamma?: number[];
    /**
     * The avg and stddev in each band's statistics is used along with numberOfStandardDeviations to calculate the cut off pixels values beling stretched.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    statistics?: RasterBandStatistics[];
    /**
     * When true, perform dynamic range adjustment (not supported on client side).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#stretchStandardDeviation)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface rasterFunctionUtilsTableParameters {
    /**
     * The attribute table represented using a feature set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#table)
     */
    attributeTable: FeatureSet;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#table)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * A pixel value to color mapping.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterValueToColor)
   */
  export interface RasterValueToColor {
    /**
     * Pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterValueToColor)
     */
    value: number;
    /**
     * Color represented using [r,g,b] or [r,g,b,a], or the color's name or hex string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterValueToColor)
     */
    color: number[] | string;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#RasterValueToColor)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * SetNull parameters
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#SetNullParameters)
   */
  export interface SetNullParameters {
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster: RasterArgument;
    /**
     * The false raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#SetNullParameters)
     */
    falseRaster: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#SetNullParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * The input parameters for creating a [shadedRelief](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#shadedRelief) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
   */
  export interface ShadedReliefWithColormapParameters {
    /**
     * The hillshade type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    hillshadeType: "traditional" | "multi-directional";
    /**
     * Altitude is the sun's angle of elevation above the horizon and ranges from 0 to 90 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    altitude?: number;
    /**
     * Azimuth is the sun's relative position along the horizon (in degrees).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    azimuth?: number;
    /**
     * The zFactor is a scaling factor used to convert the elevation values for two purposes: 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    zFactor: number;
    /**
     * The shaded result is scaled dynamically by adjusting the zFactor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    scalingType?: "none" | "adjusted";
    /**
     * It accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    pixelSizePower?: number;
    /**
     * It accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    pixelSizeZFactor?: number;
    /**
     * Use to avoid any resampling artifacts that may occur along the edges of a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    removeEdgeEffect?: boolean;
    /**
     * The color map used to generate the shaded relief.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    colormap: number[][];
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColormapParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * The input parameters for creating a [shadedRelief](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#shadedRelief) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
   */
  export interface ShadedReliefWithColorRampNameParameters {
    /**
     * The hillshade type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    hillshadeType: "traditional" | "multi-directional";
    /**
     * Altitude is the sun's angle of elevation above the horizon and ranges from 0 to 90 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    altitude?: number;
    /**
     * Azimuth is the sun's relative position along the horizon (in degrees).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    azimuth?: number;
    /**
     * The zFactor is a scaling factor used to convert the elevation values for two purposes: 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    zFactor: number;
    /**
     * The shaded result is scaled dynamically by adjusting the zFactor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    scalingType?: "none" | "adjusted";
    /**
     * It accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    pixelSizePower?: number;
    /**
     * It accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    pixelSizeZFactor?: number;
    /**
     * Use to avoid any resampling artifacts that may occur along the edges of a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    removeEdgeEffect?: boolean;
    /**
     * The color ramp name used to generate the shaded relief.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    colorRampName?:
      | "aspect"
      | "black-to-white"
      | "blue-bright"
      | "blue-light-to-dark"
      | "blue-green-bright"
      | "blue-green-light-to-dark"
      | "brown-light-to-dark"
      | "brown-to-blue-green-diverging-right"
      | "brown-to-blue-green-diverging-dark"
      | "coefficient-bias"
      | "cold-to-hot-diverging"
      | "condition-number"
      | "cyan-to-purple"
      | "cyan-light-to-blue-dark"
      | "distance"
      | "elevation1"
      | "elevation2"
      | "errors"
      | "gray-light-to-dark"
      | "green-bright"
      | "green-light-to-dark"
      | "green-to-blue"
      | "orange-bright"
      | "orange-light-to-dark"
      | "partial-spectrum"
      | "partial-spectrum-1-diverging"
      | "partial-spectrum-2-diverging"
      | "pink-to-yellow-green-diverging-bright"
      | "pink-to-yellow-green-diverging-dark"
      | "precipitation"
      | "prediction"
      | "purple-bright"
      | "purple-to-green-diverging-bright"
      | "purple-to-green-diverging-dark"
      | "purple-blue-bright"
      | "purple-blue-light-to-dark"
      | "purple-red-bright"
      | "purple-red-light-to-dark"
      | "red-bright"
      | "red-light-to-dark"
      | "red-to-blue-diverging-bright"
      | "red-to-blue-diverging-dark"
      | "red-to-green"
      | "red-to-green-diverging-bright"
      | "red-to-green-diverging-dark"
      | "slope"
      | "spectrum-full-bright"
      | "spectrum-full-dark"
      | "spectrum-full-light"
      | "surface"
      | "temperature"
      | "white-to-black"
      | "yellow-to-dark-red"
      | "yellow-to-green-to-dark-blue"
      | "yellow-to-red"
      | "yellow-green-bright"
      | "yellow-green-light-to-dark"
      | "random"
      | "elevation"
      | "gray"
      | "hillshade"
      | "ndvi"
      | "ndvi2"
      | "ndvi3";
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampNameParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  /**
   * The input parameters for creating a [shadedRelief](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#shadedRelief) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
   */
  export interface ShadedReliefWithColorRampParameters {
    /**
     * The hillshade type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    hillshadeType: "traditional" | "multi-directional";
    /**
     * Altitude is the sun's angle of elevation above the horizon and ranges from 0 to 90 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    altitude?: number;
    /**
     * Azimuth is the sun's relative position along the horizon (in degrees).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    azimuth?: number;
    /**
     * The zFactor is a scaling factor used to convert the elevation values for two purposes: 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    zFactor: number;
    /**
     * The shaded result is scaled dynamically by adjusting the zFactor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    scalingType?: "none" | "adjusted";
    /**
     * It accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    pixelSizePower?: number;
    /**
     * It accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    pixelSizeZFactor?: number;
    /**
     * Use to avoid any resampling artifacts that may occur along the edges of a raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    removeEdgeEffect?: boolean;
    /**
     * The color ramp used to generate the shaded relief.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    colorRamp: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * The input raster.
     *
     * [Read more...](global.html#raster)
     */
    raster?: RasterArgument;
    /**
     * The output pixel type, default is unknown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-rasterFunctionUtils.html#ShadedReliefWithColorRampParameters)
     */
    outputPixelType?: "unknown" | "s8" | "s16" | "s32" | "u8" | "u16" | "u32" | "f32" | "f64";
  }

  export interface RasterInfo extends Accessor, JSONSupport {}

  export class RasterInfo {
    /**
     * Describes general raster data information exposed by the ArcGIS REST API for [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html) and [WCSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html)
     */

    constructor(properties?: RasterInfoProperties);

    /**
     * The raster attribute table associated with an imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#attributeTable)
     */
    attributeTable: FeatureSet;
    /**
     * Raster band count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#bandCount)
     */
    bandCount: number;
    /**
     * This property provides additional information for each band in the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#bandInfos)
     */
    bandInfos: RasterBandInfo[];
    /**
     * Raster colormap that can be used to display the imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#colormap)
     */
    colormap: number[][];
    /**
     * Raster data type controls how the data is rendered by default.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#dataType)
     */
    dataType:
      | "generic"
      | "elevation"
      | "thematic"
      | "processed"
      | "scientific"
      | "vector-uv"
      | "vector-u"
      | "vector-v"
      | "vector-magdir"
      | "vector-magnitude"
      | "vector-direction"
      | "standard-time";
    /**
     * The minimum and maximum X and Y coordinates of a bounding box containing all the raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#extent)
     */
    extent: Extent;
    /**
     * Indicates whether the source multidimensional data has been transposed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#hasMultidimensionalTranspose)
     */
    hasMultidimensionalTranspose: boolean;
    /**
     * Raster height (row count) in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#height)
     */
    height: number;
    /**
     * Raster histograms return basic name-value pairs for number of bins, min and max bounding values, counts of pixels in each bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#histograms)
     */
    histograms: any[];
    /**
     * Raster key properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#keyProperties)
     */
    keyProperties: any;
    /**
     * Returns the multidimensional information associated with the raster service referenced in an imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#multidimensionalInfo)
     */
    multidimensionalInfo: RasterMultidimensionalInfo;
    /**
     * The pixel value representing no available information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#noDataValue)
     */
    noDataValue: number | (number | any)[];
    /**
     * Raster pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelSize)
     */
    pixelSize: RasterInfoPixelSize;
    /**
     * Pixel type for the raster data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelType)
     */
    pixelType:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * The sensor information associated with an image service referenced by a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#sensorInfo)
     */
    sensorInfo: RasterSensorInfo;
    /**
     * The spatial reference of the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * Raster band statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    statistics: RasterInfoStatistics[];
    /**
     * Raster width (column count) in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#width)
     */
    width: number;

    static fromJSON(json: any): RasterInfo;
  }

  interface RasterInfoProperties {
    /**
     * The raster attribute table associated with an imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#attributeTable)
     */
    attributeTable?: FeatureSetProperties;
    /**
     * Raster band count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#bandCount)
     */
    bandCount?: number;
    /**
     * This property provides additional information for each band in the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#bandInfos)
     */
    bandInfos?: RasterBandInfoProperties[];
    /**
     * Raster colormap that can be used to display the imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#colormap)
     */
    colormap?: number[][];
    /**
     * Raster data type controls how the data is rendered by default.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#dataType)
     */
    dataType?:
      | "generic"
      | "elevation"
      | "thematic"
      | "processed"
      | "scientific"
      | "vector-uv"
      | "vector-u"
      | "vector-v"
      | "vector-magdir"
      | "vector-magnitude"
      | "vector-direction"
      | "standard-time";
    /**
     * The minimum and maximum X and Y coordinates of a bounding box containing all the raster data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Indicates whether the source multidimensional data has been transposed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#hasMultidimensionalTranspose)
     */
    hasMultidimensionalTranspose?: boolean;
    /**
     * Raster height (row count) in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#height)
     */
    height?: number;
    /**
     * Raster histograms return basic name-value pairs for number of bins, min and max bounding values, counts of pixels in each bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#histograms)
     */
    histograms?: any[];
    /**
     * Raster key properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#keyProperties)
     */
    keyProperties?: any;
    /**
     * Returns the multidimensional information associated with the raster service referenced in an imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#multidimensionalInfo)
     */
    multidimensionalInfo?: RasterMultidimensionalInfo;
    /**
     * The pixel value representing no available information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#noDataValue)
     */
    noDataValue?: number | (number | any)[];
    /**
     * Raster pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelSize)
     */
    pixelSize?: RasterInfoPixelSize;
    /**
     * Pixel type for the raster data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelType)
     */
    pixelType?:
      | "unknown"
      | "s8"
      | "s16"
      | "s32"
      | "u1"
      | "u2"
      | "u4"
      | "u8"
      | "u16"
      | "u32"
      | "f32"
      | "f64"
      | "c64"
      | "c128";
    /**
     * The sensor information associated with an image service referenced by a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#sensorInfo)
     */
    sensorInfo?: RasterSensorInfoProperties;
    /**
     * The spatial reference of the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Raster band statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    statistics?: RasterInfoStatistics[];
    /**
     * Raster width (column count) in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#width)
     */
    width?: number;
  }

  export interface RasterInfoPixelSize {
    /**
     * Pixel size along the x axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelSize)
     */
    x: number;
    /**
     * Pixel size along the y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#pixelSize)
     */
    y: number;
  }

  export interface RasterInfoStatistics {
    /**
     * The minimum pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    min: number;
    /**
     * The maximum pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    max: number;
    /**
     * The standard deviation of calculated pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    stddev?: number;
    /**
     * The average of calculated pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#statistics)
     */
    avg?: number;
  }

  /**
   * RasterMultidimensionalInfo contains dimensions for each variable in the service  describing information about the images collected at multiple times, depths, or heights.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
   */
  export interface RasterMultidimensionalInfo {
    /**
     * The multi dimensional variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    variables: RasterMultidimensionalInfoVariables[];
  }

  export interface RasterMultidimensionalInfoVariables {
    /**
     * Variable name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    name: string;
    /**
     * Variable description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    description?: string;
    /**
     * Unit of the variable measured in.
     *
     * [Read more...](global.html#unit)
     */
    unit?: string;
    /**
     * A dimension may be used to represent real physical dimensions such as time or depth/height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    dimensions: RasterMultidimensionalInfoVariablesDimensions[];
    /**
     * Variable statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    statistics?: RasterBandStatistics[];
    /**
     * Variable histograms.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    histograms?: RasterHistogram[];
  }

  export interface RasterMultidimensionalInfoVariablesDimensions {
    /**
     * Dimension name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    name: string;
    /**
     * Dimension description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    description?: string;
    /**
     * Dimension unit.
     *
     * [Read more...](global.html#unit)
     */
    unit?: string;
    /**
     * An array of single values or tuples [min, max] each defining a range of valid values along the specified dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    values?: number[] | number[][];
    /**
     * Indicates if each dimension value is represented using a range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    hasRanges?: boolean;
    /**
     * Indicates if the dimension is recorded at regular intervals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    hasRegularIntervals?: boolean;
    /**
     * Indicates if the dimension is recurring, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    recurring?: boolean;
    /**
     * Dimension interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    interval?: number;
    /**
     * Dimension interval unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    intervalUnit?: string;
    /**
     * The extent of dimension values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterInfo.html#RasterMultidimensionalInfo)
     */
    extent?: number[];
  }

  export interface RasterSensorInfo extends Accessor, JSONSupport {}

  export class RasterSensorInfo {
    /**
     * The `RasterSensorInfo` class provides additional information on the raster sensor associated with an image service referenced by [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#serviceRasterInfo) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#rasterInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html)
     */

    constructor(properties?: RasterSensorInfoProperties);

    /**
     * The acquisition date.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#acquisitionDate)
     */
    acquisitionDate: Date;
    /**
     * The cloud coverage (0-1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#cloudCover)
     */
    cloudCover: number;
    /**
     * The satellite product name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#productName)
     */
    productName: string;
    /**
     * The sensor azimuth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorAzimuth)
     */
    sensorAzimuth: number;
    /**
     * The sensor elevation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorElevation)
     */
    sensorElevation: number;
    /**
     * The sensor name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorName)
     */
    sensorName: string;
    /**
     * The sun azimuth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sunAzimuth)
     */
    sunAzimuth: number;
    /**
     * The sun elevation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sunElevation)
     */
    sunElevation: number;

    /**
     * Creates a deep clone of the raster sensor info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#clone)
     */
    clone(): RasterSensorInfo;

    static fromJSON(json: any): RasterSensorInfo;
  }

  interface RasterSensorInfoProperties {
    /**
     * The acquisition date.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#acquisitionDate)
     */
    acquisitionDate?: DateProperties;
    /**
     * The cloud coverage (0-1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#cloudCover)
     */
    cloudCover?: number;
    /**
     * The satellite product name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#productName)
     */
    productName?: string;
    /**
     * The sensor azimuth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorAzimuth)
     */
    sensorAzimuth?: number;
    /**
     * The sensor elevation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorElevation)
     */
    sensorElevation?: number;
    /**
     * The sensor name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sensorName)
     */
    sensorName?: string;
    /**
     * The sun azimuth.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sunAzimuth)
     */
    sunAzimuth?: number;
    /**
     * The sun elevation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterSensorInfo.html#sunElevation)
     */
    sunElevation?: number;
  }

  export interface Relationship extends Accessor, JSONSupport, Clonable {}

  export class Relationship {
    /**
     * Describes a [layer's](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) relationship with another layer or table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html)
     */

    constructor(properties?: RelationshipProperties);

    /**
     * The cardinality which specifies the number of objects in the origin [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) related to the number of objects in the destination [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#cardinality)
     */
    cardinality: "one-to-one" | "one-to-many" | "many-to-many";
    /**
     * Indicates whether the relationship is composite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#composite)
     */
    composite: boolean;
    /**
     * The unique ID for the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#id)
     */
    id: number;
    /**
     * The field used to establish the relate within the [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyField)
     */
    keyField: string;
    /**
     * The key field in an attributed relationship class table that matches the [keyField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyFieldInRelationshipTable)
     */
    keyFieldInRelationshipTable: string;
    /**
     * The name of the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#name)
     */
    name: string;
    /**
     * The unique ID of the related [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#relatedTableId)
     */
    relatedTableId: number;
    /**
     * The relationship table id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#relationshipTableId)
     */
    relationshipTableId: number;
    /**
     * Indicates whether the table participating in the relationship is the `origin` or `destination` table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#role)
     */
    role: "origin" | "destination";

    static fromJSON(json: any): Relationship;
  }

  interface RelationshipProperties {
    /**
     * The cardinality which specifies the number of objects in the origin [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) related to the number of objects in the destination [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#cardinality)
     */
    cardinality?: "one-to-one" | "one-to-many" | "many-to-many";
    /**
     * Indicates whether the relationship is composite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#composite)
     */
    composite?: boolean;
    /**
     * The unique ID for the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#id)
     */
    id?: number;
    /**
     * The field used to establish the relate within the [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyField)
     */
    keyField?: string;
    /**
     * The key field in an attributed relationship class table that matches the [keyField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#keyFieldInRelationshipTable)
     */
    keyFieldInRelationshipTable?: string;
    /**
     * The name of the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#name)
     */
    name?: string;
    /**
     * The unique ID of the related [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#relatedTableId)
     */
    relatedTableId?: number;
    /**
     * The relationship table id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#relationshipTableId)
     */
    relationshipTableId?: number;
    /**
     * Indicates whether the table participating in the relationship is the `origin` or `destination` table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html#role)
     */
    role?: "origin" | "destination";
  }

  export interface RouteStopSymbols extends Accessor, JSONSupport {}

  export class RouteStopSymbols {
    /**
     * RouteStopSymbols contains a set of symbols that will be used by [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) to symbolize new stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html)
     */

    constructor(properties?: RouteStopSymbolsProperties);

    /**
     * The default symbol for _break_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#break)
     */
    break: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for the first [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [0, 255, 0], size: 20, outline: { color: [255, 255, 255], width: 4 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#first)
     */
    first: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for a _last_ [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 0, 0], size: 20, outline: { color: [255, 255, 255], width: 4 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#last)
     */
    last: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for _middle_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [51, 51, 51],, size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#middle)
     */
    middle: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for an _unlocated_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 0, 0], size: 12, outline: { color: [255, 255, 255], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#unlocated)
     */
    unlocated: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for _waypoint_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#waypoint)
     */
    waypoint: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;

    static fromJSON(json: any): RouteStopSymbols;
  }

  interface RouteStopSymbolsProperties {
    /**
     * The default symbol for _break_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#break)
     */
    break?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for the first [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [0, 255, 0], size: 20, outline: { color: [255, 255, 255], width: 4 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#first)
     */
    first?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for a _last_ [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 0, 0], size: 20, outline: { color: [255, 255, 255], width: 4 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#last)
     */
    last?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for _middle_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [51, 51, 51],, size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#middle)
     */
    middle?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for an _unlocated_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 0, 0], size: 12, outline: { color: [255, 255, 255], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#unlocated)
     */
    unlocated?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for _waypoint_ [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 12, outline: { color: [0, 122, 194], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteStopSymbols.html#waypoint)
     */
    waypoint?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
  }

  export interface RouteSymbols extends Accessor, JSONSupport {}

  export class RouteSymbols {
    /**
     * RouteSymbols contains a set of symbols that will be used by [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) to symbolize new stops, barriers and directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html)
     */

    constructor(properties?: RouteSymbolsProperties);

    /**
     * The default symbol for [direction lines](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionLines).
     *
     * @default { type: "simple-line", color: [0, 122, 194], width: 6 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#directionLines)
     */
    directionLines: SimpleLineSymbol | LineSymbol3D | CIMSymbol;
    /**
     * The default symbol for [direction points](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionPoints).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 6, outline: { color: [0, 122, 194], width: 2 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#directionPoints)
     */
    directionPoints: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#pointBarriers).
     *
     * @default { type: "simple-marker", style: "x", size: 10, outline: { color: [255, 0, 0], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#pointBarriers)
     */
    pointBarriers: SimpleMarkerSymbol | PictureMarkerSymbol | PointSymbol3D | CIMSymbol;
    /**
     * The default symbol for [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polygonBarriers).
     *
     * @default { type: "simple-fill", color: [255, 170, 0, 0.6], outline: { width: 7.5, color: [255, 0, 0, 0.6] } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#polygonBarriers)
     */
    polygonBarriers: PictureFillSymbol | SimpleFillSymbol | PolygonSymbol3D | CIMSymbol;
    /**
     * The default symbol for [polyline Barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polylineBarriers).
     *
     * @default { type: "simple-line", color: [255, 85, 0, 0.7], width: 7.5 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#polylineBarriers)
     */
    polylineBarriers: SimpleLineSymbol | LineSymbol3D | CIMSymbol;
    /**
     * The default symbol for the overall [route](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#routeInfo).
     *
     * @default { type: "simple-line", color: [20, 89, 127], width: 8 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#routeInfo)
     */
    routeInfo: SimpleLineSymbol | LineSymbol3D | CIMSymbol;
    /**
     * The default symbol for [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#stops)
     */
    stops: RouteStopSymbols;

    static fromJSON(json: any): RouteSymbols;
  }

  interface RouteSymbolsProperties {
    /**
     * The default symbol for [direction lines](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionLines).
     *
     * @default { type: "simple-line", color: [0, 122, 194], width: 6 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#directionLines)
     */
    directionLines?:
      | (SimpleLineSymbolProperties & { type: "simple-line" })
      | (LineSymbol3DProperties & { type: "line-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for [direction points](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#directionPoints).
     *
     * @default { type: "simple-marker", color: [255, 255, 255], size: 6, outline: { color: [0, 122, 194], width: 2 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#directionPoints)
     */
    directionPoints?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#pointBarriers).
     *
     * @default { type: "simple-marker", style: "x", size: 10, outline: { color: [255, 0, 0], width: 3 } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#pointBarriers)
     */
    pointBarriers?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polygonBarriers).
     *
     * @default { type: "simple-fill", color: [255, 170, 0, 0.6], outline: { width: 7.5, color: [255, 0, 0, 0.6] } }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#polygonBarriers)
     */
    polygonBarriers?:
      | (PictureFillSymbolProperties & { type: "picture-fill" })
      | (SimpleFillSymbolProperties & { type: "simple-fill" })
      | (PolygonSymbol3DProperties & { type: "polygon-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for [polyline Barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#polylineBarriers).
     *
     * @default { type: "simple-line", color: [255, 85, 0, 0.7], width: 7.5 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#polylineBarriers)
     */
    polylineBarriers?:
      | (SimpleLineSymbolProperties & { type: "simple-line" })
      | (LineSymbol3DProperties & { type: "line-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for the overall [route](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#routeInfo).
     *
     * @default { type: "simple-line", color: [20, 89, 127], width: 8 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#routeInfo)
     */
    routeInfo?:
      | (SimpleLineSymbolProperties & { type: "simple-line" })
      | (LineSymbol3DProperties & { type: "line-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The default symbol for [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RouteSymbols.html#stops)
     */
    stops?: RouteStopSymbolsProperties;
  }

  export class SceneFilter extends Accessor {
    /**
     * A SceneFilter defines parameters for setting a client-side spatial filter on a [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#filter).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html)
     */

    constructor(properties?: SceneFilterProperties);

    /**
     * The geometries to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#geometries)
     */
    geometries: Collection<any>;
    /**
     * The type of masking to perform.
     *
     * @default "disjoint"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#spatialRelationship)
     */
    spatialRelationship: "disjoint" | "contains";

    /**
     * Creates a deep clone of the SceneFilter object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#clone)
     */
    clone(): SceneFilter;
  }

  interface SceneFilterProperties {
    /**
     * The geometries to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#geometries)
     */
    geometries?: CollectionProperties<any>;
    /**
     * The type of masking to perform.
     *
     * @default "disjoint"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneFilter.html#spatialRelationship)
     */
    spatialRelationship?: "disjoint" | "contains";
  }

  export interface SceneModification extends Accessor, JSONSupport {}

  export class SceneModification {
    /**
     * The SceneModification is used to perform a client-side geometric modifications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html)
     */

    constructor(properties?: SceneModificationProperties);

    /**
     * The geometry representing the location of the modification.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html#geometry)
     */
    geometry: Polygon;
    /**
     * The type of modification to perform.
     *
     * @default "clip"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html#type)
     */
    type: "clip" | "mask" | "replace";

    /**
     * Creates a clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html#clone)
     */
    clone(): SceneModification;

    static fromJSON(json: any): SceneModification;
  }

  interface SceneModificationProperties {
    /**
     * The geometry representing the location of the modification.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html#geometry)
     */
    geometry?: PolygonProperties;
    /**
     * The type of modification to perform.
     *
     * @default "clip"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html#type)
     */
    type?: "clip" | "mask" | "replace";
  }

  export interface SceneModifications extends Collection<SceneModification>, JSONSupport {}

  export class SceneModifications {
    /**
     * A collection of [SceneModification](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModification.html) with polygons and types to apply client-side modifications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModifications.html)
     */

    constructor(
      properties?:
        | SceneModificationsProperties
        | SceneModificationProperties[]
        | Collection<SceneModificationProperties>,
    );

    /**
     * Creates a clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SceneModifications.html#clone)
     */
    clone(): SceneModifications;

    static fromJSON(json: any): SceneModifications;
  }

  interface SceneModificationsProperties extends CollectionPropertiesBase<SceneModificationProperties> {}

  export interface SiteLayerInfo extends Accessor, JSONSupport {}

  export class SiteLayerInfo {
    /**
     * The SiteLayerInfo class describes the boundaries of managed sites and is used for visualizing groups of facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html)
     */

    constructor(properties?: SiteLayerInfoProperties);

    /**
     * The id for an operational layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#layerId)
     */
    layerId: string;
    /**
     * The field name from the layer that defines the site name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#nameField)
     */
    nameField: string;
    /**
     * The field name from the layer that defines a site unique ID for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#siteIdField)
     */
    siteIdField: string;
    /**
     * This value references the numeric ID of the sublayer if the Site layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#sublayerId)
     */
    sublayerId: number;

    static fromJSON(json: any): SiteLayerInfo;
  }

  interface SiteLayerInfoProperties {
    /**
     * The id for an operational layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#layerId)
     */
    layerId?: string;
    /**
     * The field name from the layer that defines the site name of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#nameField)
     */
    nameField?: string;
    /**
     * The field name from the layer that defines a site unique ID for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#siteIdField)
     */
    siteIdField?: string;
    /**
     * This value references the numeric ID of the sublayer if the Site layer is a map service sublayer instead of a feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SiteLayerInfo.html#sublayerId)
     */
    sublayerId?: number;
  }

  /**
   * A web socket connection to a stream service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-StreamConnection.html)
   */
  interface StreamConnection {
    /**
     * The error that explains an unsuccessful attempt to connect to the stream service or an unexpected disconnection from the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-StreamConnection.html#connectionError)
     */
    readonly connectionError: Error;
    /**
     * The status of the Web Socket connection with the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-StreamConnection.html#connectionStatus)
     */
    readonly connectionStatus: "connected" | "disconnected";

    /**
     * Destroys the existing connection instance to the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-StreamConnection.html#destroy)
     */
    destroy(): void;
  }

  export const StreamConnection: StreamConnection;

  export interface StreamConnectionDataReceivedEvent {
    attributes: any;

    geometry: any;
  }

  export interface Sublayer extends Accessor, Loadable, Identifiable {}

  export class Sublayer {
    /**
     * Represents a sublayer in a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) or a [TileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html)
     */

    constructor(properties?: SublayerProperties);

    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    readonly capabilities: SublayerCapabilities;
    /**
     * A SQL where clause used to filter features in the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * An array of fields in the Sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#fields)
     */
    readonly fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#field) by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * If a map image layer contains a sublayer which is meant to be floor-aware, then that sublayer must have a floorInfo property, containing a [LayerFloorInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html) object which has a string property to represent the floorField.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#floorInfo)
     */
    floorInfo: LayerFloorInfo;
    /**
     * The full extent of the Sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#fullExtent)
     */
    readonly fullExtent: Extent;
    /**
     * The sublayer's layer ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#id)
     */
    id: number;
    /**
     * Returns `true` if the sublayer is a non-spatial table in a map service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#isTable)
     */
    readonly isTable: boolean;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates if labels for the sublayer will be visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * The [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) or [TileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#layer)
     */
    layer: MapImageLayer | TileLayer;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#listMode)
     */
    listMode: "show" | "hide" | "hide-children";
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#minScale)
     */
    minScale: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#fields) containing a unique value or identifier for each feature in the Sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#objectIdField)
     */
    readonly objectIdField: string;
    /**
     * The level of opacity to set on the sublayer on a scale from 0.0 - 1.0 where `0` is fully transparent and `1.0` is fully opaque.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#opacity)
     */
    opacity: number;
    /**
     * The parent layer to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#parent)
     */
    parent: Sublayer | MapImageLayer | TileLayer;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * An array of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html) associated with the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#relationships)
     */
    readonly relationships: Relationship[];
    /**
     * The renderer to apply to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * An object that allows you to create a dynamic layer with data either from the map service sublayers or data from a registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#source)
     */
    source: DynamicMapLayer | DynamicDataLayer;
    /**
     * The [map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * The [spatial reference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) of the sublayer as defined by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * If a sublayer contains sublayers, this property is a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) objects belonging to the given sublayer with sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#sublayers)
     */
    sublayers: Collection<Sublayer>;
    /**
     * The title of the sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#title)
     */
    title: string;

    readonly type: "sublayer";
    /**
     * The name of the field holding the type ID or subtypes for the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#typeIdField)
     */
    readonly typeIdField: string;
    /**
     * An array of [types](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html) defined in the feature service exposed by ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#types)
     */
    readonly types: FeatureType[];
    /**
     * The URL to the REST endpoint of the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#url)
     */
    url: string;
    /**
     * Indicates if the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#visible)
     */
    visible: boolean;

    /**
     * Creates a deep clone of the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#clone)
     */
    clone(): Sublayer;
    /**
     * A convenient method for creating a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) instance based on the Sublayer's configuration, including [dynamic sources](https://enterprise.arcgis.com/en/server/latest/publish-services/linux/about-dynamic-layers.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#createFeatureLayer)
     */
    createFeatureLayer(): Promise<FeatureLayer>;
    /**
     * Creates a popup template for the sublayer, populated with all the fields of the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) object with default values representing the layer's state, including filters (definition expression) on the layer's features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns a [FeatureType](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureType.html) describing the feature's type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#getFeatureType)
     */
    getFeatureType(feature: Graphic): FeatureType;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: SublayerGetFieldDomainOptions): Domain;
    /**
     * Query information about attachments associated with features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryAttachments)
     */
    queryAttachments(
      attachmentQuery: AttachmentQuery | AttachmentQueryProperties,
      options?: SublayerQueryAttachmentsOptions,
    ): void;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the sublayer and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: SublayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html) against features in the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: SublayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the sublayer and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: SublayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the sublayer and returns [FeatureSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) grouped by source layer or table objectIds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryRelatedFeatures)
     */
    queryRelatedFeatures(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SublayerQueryRelatedFeaturesOptions,
    ): Promise<any>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the sublayer and when resolved, it returns an `object` containing key value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryRelatedFeaturesCount)
     */
    queryRelatedFeaturesCount(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SublayerQueryRelatedFeaturesCountOptions,
    ): Promise<any>;
  }

  interface SublayerProperties extends LoadableProperties {
    /**
     * A SQL where clause used to filter features in the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * If a map image layer contains a sublayer which is meant to be floor-aware, then that sublayer must have a floorInfo property, containing a [LayerFloorInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LayerFloorInfo.html) object which has a string property to represent the floorField.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#floorInfo)
     */
    floorInfo?: LayerFloorInfoProperties;
    /**
     * The sublayer's layer ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#id)
     */
    id?: number;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates if labels for the sublayer will be visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * The [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) or [TileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#layer)
     */
    layer?: (MapImageLayerProperties & { type: "map-image" }) | (TileLayerProperties & { type: "tile" });
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Indicates how the layer should display in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * @default "show"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#listMode)
     */
    listMode?: "show" | "hide" | "hide-children";
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#minScale)
     */
    minScale?: number;
    /**
     * The level of opacity to set on the sublayer on a scale from 0.0 - 1.0 where `0` is fully transparent and `1.0` is fully opaque.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#opacity)
     */
    opacity?: number;
    /**
     * The parent layer to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#parent)
     */
    parent?:
      | (SublayerProperties & { type: "sublayer" })
      | (MapImageLayerProperties & { type: "map-image" })
      | (TileLayerProperties & { type: "tile" });
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer to apply to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * An object that allows you to create a dynamic layer with data either from the map service sublayers or data from a registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#source)
     */
    source?: (DynamicMapLayer & { type: "map-layer" }) | (DynamicDataLayer & { type: "data-layer" });
    /**
     * The [map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * If a sublayer contains sublayers, this property is a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) objects belonging to the given sublayer with sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#sublayers)
     */
    sublayers?: CollectionProperties<SublayerProperties>;
    /**
     * The title of the sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#title)
     */
    title?: string;
    /**
     * The URL to the REST endpoint of the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#url)
     */
    url?: string;
    /**
     * Indicates if the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#visible)
     */
    visible?: boolean;
  }

  /**
   * A dynamic data layer is a layer created on-the-fly with data stored in a [registered workspace](https://enterprise.arcgis.com/en/server/latest/manage-data/windows/overview-register-data-with-arcgis-server.htm).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicDataLayer)
   */
  export interface DynamicDataLayer {
    /**
     * This value is always `data-layer` and is inferred when the `dataSource` property is set.
     *
     * [Read more...](global.html#type)
     */
    type: "data-layer";
    /**
     * A table, feature class, or raster that resides in a registered workspace (either a folder or geodatabase).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicDataLayer)
     */
    dataSource: TableDataSource | QueryTableDataSource | RasterDataSource | JoinTableDataSource;
    /**
     * Controls field visibility in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicDataLayer)
     */
    fields: DynamicDataLayerFields[];
  }

  /**
   * A dynamic map layer refers to a layer published in a map service that has dynamic layers enabled.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicMapLayer)
   */
  export interface DynamicMapLayer {
    /**
     * This value is always `map-layer` and is inferred when the `mapLayerId` property is set.
     *
     * [Read more...](global.html#type)
     */
    type: "map-layer";
    /**
     * The [id](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#id) of the service sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicMapLayer)
     */
    mapLayerId: number;
    /**
     * An optional property for specifying the GDB version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicMapLayer)
     */
    gdbVersion: string;
  }

  /**
   * The result of an on-the-fly join operation at runtime.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
   */
  export interface JoinTableDataSource {
    /**
     * This value is always `join-table` and is inferred when other join table properties of this object are set.
     *
     * [Read more...](global.html#type)
     */
    type: "join-table";
    /**
     * The field name used for joining or matching records in the left table to records in the right table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
     */
    leftTableKey: string;
    /**
     * The field name used for joining or matching records in the right table to records in the left table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
     */
    rightTableKey: string;
    /**
     * The left table for joining to the right table source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
     */
    leftTableSource: DynamicMapLayer | DynamicDataLayer;
    /**
     * The right table for joining to the left table source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
     */
    rightTableSource: DynamicMapLayer | DynamicDataLayer;
    /**
     * The type of join that will be performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#JoinTableDataSource)
     */
    joinType: "left-outer-join" | "left-inner-join";
  }

  /**
   * A query table is a feature class or table defined by a SQL query on the fly.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#QueryTableDataSource)
   */
  export interface QueryTableDataSource {
    /**
     * This value is always `query-table` and is inferred when the `query` property of this object is set.
     *
     * [Read more...](global.html#type)
     */
    type: "query-table";
    /**
     * The workspace where the data resides (defined in ArcGIS Server Manager).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#QueryTableDataSource)
     */
    workspaceId: string;
    /**
     * The SQL query used to filter records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#QueryTableDataSource)
     */
    query: string;
    /**
     * The field name(s) containing the unique IDs for each record in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#QueryTableDataSource)
     */
    oidFields: string;
    /**
     * The spatial reference of the geometry of each feature in the table source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#QueryTableDataSource)
     */
    spatialReference: SpatialReference;
    /**
     * The geometry type of each record in the table.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "multipatch";
  }

  /**
   * A file-based raster that resides in a registered raster workspace.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#RasterDataSource)
   */
  export interface RasterDataSource {
    /**
     * This value is always `raster`.
     *
     * [Read more...](global.html#type)
     */
    type: "raster";
    /**
     * The workspace where the raster resides as defined in the ArcGIS Server Manager.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#RasterDataSource)
     */
    workspaceId: string;
    /**
     * The name of the raster in the registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#RasterDataSource)
     */
    dataSourceName: string;
  }

  export interface SublayerCapabilities {
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    data: SublayerCapabilitiesData;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    operations: SublayerCapabilitiesOperations;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    queryRelated: SublayerCapabilitiesQueryRelated;
  }

  export interface SublayerCapabilitiesData {
    /**
     * Indicates if attachments are enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsAttachment: boolean;
  }

  export interface SublayerCapabilitiesOperations {
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsQueryAttachments: boolean;
  }

  export interface SublayerCapabilitiesQueryRelated {
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#capabilities)
     */
    supportsPagination: boolean;
  }

  export interface SublayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface SublayerQueryAttachmentsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryAttachments)
     */
    signal?: AbortSignal;
  }

  export interface SublayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface SublayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SublayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface SublayerQueryRelatedFeaturesCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryRelatedFeaturesCount)
     */
    signal?: AbortSignal;
  }

  export interface SublayerQueryRelatedFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#queryRelatedFeatures)
     */
    signal?: AbortSignal;
  }

  /**
   * A table or feature class that resides in a registered workspace (either a folder or geodatabase).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#TableDataSource)
   */
  export interface TableDataSource {
    /**
     * This value is always `table`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The workspace where the table resides as defined in the ArcGIS Server Manager.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#TableDataSource)
     */
    workspaceId: string;
    /**
     * The name of the table in the registered workspace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#TableDataSource)
     */
    dataSourceName: string;
    /**
     * References the geodatabase version if multiple versions exist in the geodatabase.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#TableDataSource)
     */
    gdbVersion: string;
  }

  export interface DynamicDataLayerFields {
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicDataLayer)
     */
    name: string;
    /**
     * The alias of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html#DynamicDataLayer)
     */
    alias: string;
  }

  export interface Subtype extends Accessor, JSONSupport {}

  export class Subtype {
    /**
     * Subtypes categorize a subset of features in a layer that share the same attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html)
     */

    constructor(properties?: SubtypeProperties);

    /**
     * The subtype unique identifier number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#code)
     */
    code: number;
    /**
     * Name-value pairs of fields and their default field values associated with the subtype.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#defaultValues)
     */
    defaultValues: any;
    /**
     * [Domains](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the subtype.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#domains)
     */
    domains: HashMap<CodedValueDomain | RangeDomain | InheritedDomain>;
    /**
     * The subtype name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#name)
     */
    name: string;

    static fromJSON(json: any): Subtype;
  }

  interface SubtypeProperties {
    /**
     * The subtype unique identifier number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#code)
     */
    code?: number;
    /**
     * Name-value pairs of fields and their default field values associated with the subtype.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#defaultValues)
     */
    defaultValues?: any;
    /**
     * [Domains](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the subtype.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#domains)
     */
    domains?: HashMap<
      | (CodedValueDomainProperties & { type: "coded-value" })
      | (RangeDomainProperties & { type: "range" })
      | (InheritedDomainProperties & { type: "inherited" })
    >;
    /**
     * The subtype name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Subtype.html#name)
     */
    name?: string;
  }

  export interface SubtypeSublayer extends Accessor, Loadable, Identifiable {}

  export class SubtypeSublayer {
    /**
     * Represents a sublayer in a [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html)
     */

    constructor(properties?: SubtypeSublayerProperties);

    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#capabilities)
     */
    readonly capabilities: Capabilities;
    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * Describes effective capabilities of the layer taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#effectiveCapabilities)
     */
    readonly effectiveCapabilities: Capabilities;
    /**
     * Indicates whether the layer is editable taking in to consideration privileges of the currently signed-in user and whether the parent subtype group layer is editable or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#effectiveEditingEnabled)
     */
    readonly effectiveEditingEnabled: boolean;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#fields)
     */
    readonly fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used in an associated layer's [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#formTemplate)
     */
    formTemplate: FormTemplate;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#geometryType)
     */
    readonly geometryType: "point" | "multipoint" | "polyline" | "polygon" | "multipatch" | "mesh";
    /**
     * The unique ID assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#id)
     */
    id: string;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#minScale)
     */
    minScale: number;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#objectIdField)
     */
    readonly objectIdField: string;
    /**
     * The opacity of the layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#opacity)
     */
    opacity: number;
    /**
     * The [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#parent)
     */
    parent: SubtypeGroupLayer;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Array of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html) set up for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#relationships)
     */
    readonly relationships: Relationship[];
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The unique identifier representing the [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) created from the [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#subtypeCode)
     */
    subtypeCode: number;
    /**
     * The name of the field which holds the id of the [subtypes](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#subtypes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#subtypeField)
     */
    readonly subtypeField: string;
    /**
     * An array of feature templates defined in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#templates)
     */
    templates: FeatureTemplate[];
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#title)
     */
    title: string;

    readonly type: "subtype-sublayer";
    /**
     * The absolute URL of the REST endpoint of the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#url)
     */
    url: string;
    /**
     * Indicates if the layer is visible in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#visible)
     */
    visible: boolean;

    /**
     * Adds an attachment to a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#addAttachment)
     */
    addAttachment(feature: Graphic, attachment: HTMLFormElement | FormData): Promise<FeatureEditResult>;
    /**
     * Applies edits to features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    applyEdits(edits: SubtypeSublayerApplyEditsEdits, options?: SubtypeSublayerApplyEditsOptions): Promise<EditsResult>;
    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameters that can be used to fetch features that satisfy the layer's current filters, and definitions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Deletes attachments from a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#deleteAttachments)
     */
    deleteAttachments(feature: Graphic, attachmentIds: number[]): Promise<FeatureEditResult>;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string): Domain;
    /**
     * Query information about attachments associated with features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryAttachments)
     */
    queryAttachments(
      attachmentQuery: AttachmentQuery | AttachmentQueryProperties,
      options?: SubtypeSublayerQueryAttachmentsOptions,
    ): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: SubtypeSublayerQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: SubtypeSublayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the service and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: SubtypeSublayerQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service and returns [FeatureSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) grouped by source layer or table objectIds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryRelatedFeatures)
     */
    queryRelatedFeatures(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SubtypeSublayerQueryRelatedFeaturesOptions,
    ): Promise<any>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the feature service and when resolved, it returns an `object` containing key value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryRelatedFeaturesCount)
     */
    queryRelatedFeaturesCount(
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      options?: SubtypeSublayerQueryRelatedFeaturesCountOptions,
    ): Promise<any>;
    /**
     * Updates an existing attachment for a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#updateAttachment)
     */
    updateAttachment(
      feature: Graphic,
      attachmentId: number,
      attachment: HTMLFormElement | FormData,
    ): Promise<FeatureEditResult>;
  }

  interface SubtypeSublayerProperties extends LoadableProperties {
    /**
     * This property is used to configure the associated layer's [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Determines if the layer is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * The [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used in an associated layer's [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#formTemplate)
     */
    formTemplate?: FormTemplateProperties;
    /**
     * The unique ID assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#id)
     */
    id?: string;
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#minScale)
     */
    minScale?: number;
    /**
     * The opacity of the layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#opacity)
     */
    opacity?: number;
    /**
     * The [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#parent)
     */
    parent?: SubtypeGroupLayerProperties;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The unique identifier representing the [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) created from the [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#subtypeCode)
     */
    subtypeCode?: number;
    /**
     * An array of feature templates defined in the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#templates)
     */
    templates?: FeatureTemplateProperties[];
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#title)
     */
    title?: string;
    /**
     * The absolute URL of the REST endpoint of the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#url)
     */
    url?: string;
    /**
     * Indicates if the layer is visible in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#visible)
     */
    visible?: boolean;
  }

  export interface SubtypeSublayerApplyEditsEdits {
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    addFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    updateFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features, or an array of objects with `objectId` or `globalId` of each feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    deleteFeatures?: Graphic[] | Collection<Graphic> | any[];
    /**
     * An array of attachments to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    addAttachments?: AttachmentEdit[];
    /**
     * An array of attachments to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    updateAttachments?: AttachmentEdit[];
    /**
     * An array of [globalId](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#AttachmentEdit)s for attachments to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    deleteAttachments?: string[];
  }

  export interface SubtypeSublayerApplyEditsOptions {
    /**
     * The geodatabase version to apply the edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    gdbVersion?: string;
    /**
     * Indicates whether the edit results should return the time edits were applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    returnEditMoment?: boolean;
    /**
     * If set to `original-and-current-features`, the [EditedFeatureResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#EditedFeatureResult) parameter will be included in the `applyEdits` response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    returnServiceEditsOption?: "none" | "original-and-current-features";
    /**
     * Indicates whether the edits should be applied only if all submitted edits succeed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    rollbackOnFailureEnabled?: boolean;
    /**
     * Indicates whether the edits can be applied using globalIds of features or attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#applyEdits)
     */
    globalIdUsed?: boolean;
  }

  export interface SubtypeSublayerQueryAttachmentsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryAttachments)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeSublayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeSublayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeSublayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeSublayerQueryRelatedFeaturesCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryRelatedFeaturesCount)
     */
    signal?: AbortSignal;
  }

  export interface SubtypeSublayerQueryRelatedFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html#queryRelatedFeatures)
     */
    signal?: AbortSignal;
  }

  export interface TelemetryData extends Accessor, JSONSupport, Clonable {}

  export class TelemetryData {
    /**
     * The TelemetryData class is used to represent the telemetry data for a [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html)
     */

    constructor(properties?: TelemetryDataProperties);

    /**
     * The center of the frame.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#frameCenter)
     */
    frameCenter: Point;
    /**
     * The outline of the frame.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#frameOutline)
     */
    frameOutline: Polygon;
    /**
     * The line of sight.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#lineOfSight)
     */
    lineOfSight: Polyline;
    /**
     * The location of the sensor.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#sensorLocation)
     */
    sensorLocation: Point;
    /**
     * The trail of the sensor.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#sensorTrail)
     */
    sensorTrail: Polyline;

    static fromJSON(json: any): TelemetryData;
  }

  interface TelemetryDataProperties {
    /**
     * The center of the frame.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#frameCenter)
     */
    frameCenter?: PointProperties;
    /**
     * The outline of the frame.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#frameOutline)
     */
    frameOutline?: PolygonProperties;
    /**
     * The line of sight.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#lineOfSight)
     */
    lineOfSight?: PolylineProperties;
    /**
     * The location of the sensor.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#sensorLocation)
     */
    sensorLocation?: PointProperties;
    /**
     * The trail of the sensor.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryData.html#sensorTrail)
     */
    sensorTrail?: PolylineProperties;
  }

  export interface TelemetryDisplay extends Accessor, JSONSupport, Clonable {}

  export class TelemetryDisplay {
    /**
     * The TelemetryDisplay class is used to choose what telemetry data to display on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html)
     */

    constructor(properties?: TelemetryDisplayProperties);

    /**
     * Determines if the frame image is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frame)
     */
    frame: boolean;
    /**
     * Determines if the frame center is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frameCenter)
     */
    frameCenter: boolean;
    /**
     * Determines if the frame outline is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frameOutline)
     */
    frameOutline: boolean;
    /**
     * Determines if the line of sight is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#lineOfSight)
     */
    lineOfSight: boolean;
    /**
     * Determines if the sensor location is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#sensorLocation)
     */
    sensorLocation: boolean;
    /**
     * Determines if the sensor trail is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#sensorTrail)
     */
    sensorTrail: boolean;

    static fromJSON(json: any): TelemetryDisplay;
  }

  interface TelemetryDisplayProperties {
    /**
     * Determines if the frame image is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frame)
     */
    frame?: boolean;
    /**
     * Determines if the frame center is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frameCenter)
     */
    frameCenter?: boolean;
    /**
     * Determines if the frame outline is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#frameOutline)
     */
    frameOutline?: boolean;
    /**
     * Determines if the line of sight is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#lineOfSight)
     */
    lineOfSight?: boolean;
    /**
     * Determines if the sensor location is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#sensorLocation)
     */
    sensorLocation?: boolean;
    /**
     * Determines if the sensor trail is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TelemetryDisplay.html#sensorTrail)
     */
    sensorTrail?: boolean;
  }

  export interface TileInfo extends Accessor, JSONSupport {}

  export class TileInfo {
    /**
     * Contains information about the tiling scheme for [TileLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html), [ElevationLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html), [ImageryTileLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html), [VectorTileLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html), and [WebTileLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html)
     */

    constructor(properties?: TileInfoProperties);

    /**
     * The dots per inch (DPI) of the tiling scheme.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#dpi)
     */
    dpi: number;
    /**
     * Image format of the cached tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#format)
     */
    format:
      | "png"
      | "png8"
      | "png24"
      | "png32"
      | "jpg"
      | "dib"
      | "tiff"
      | "emf"
      | "ps"
      | "pdf"
      | "gif"
      | "svg"
      | "svgz"
      | "mixed"
      | "lerc"
      | "lerc2d"
      | "pbf"
      | "raw";
    /**
     * Indicates if the tiling scheme supports wrap around.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#isWrappable)
     */
    isWrappable: boolean;
    /**
     * An array of levels of detail that define the tiling scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#lods)
     */
    lods: LOD[];
    /**
     * The tiling scheme origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#origin)
     */
    origin: Point;
    /**
     * Size of tiles in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#size)
     */
    size: number[];
    /**
     * The spatial reference of the tiling schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#spatialReference)
     */
    spatialReference: SpatialReference;

    /**
     * Utility method used to convert a scale value to its corresponding zoom value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#scaleToZoom)
     */
    scaleToZoom(scale: number): number;
    /**
     * Utility method used to convert a zoom value to its corresponding scale value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#zoomToScale)
     */
    zoomToScale(zoom: number): number;

    /**
     * A convenience method used to create a new TileInfo instance with preset properties like [LODs](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#lods).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#create)
     */
    static create(options?: TileInfoCreateOptions): TileInfo;
    static fromJSON(json: any): TileInfo;
  }

  interface TileInfoProperties {
    /**
     * The dots per inch (DPI) of the tiling scheme.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#dpi)
     */
    dpi?: number;
    /**
     * Image format of the cached tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#format)
     */
    format?:
      | "png"
      | "png8"
      | "png24"
      | "png32"
      | "jpg"
      | "dib"
      | "tiff"
      | "emf"
      | "ps"
      | "pdf"
      | "gif"
      | "svg"
      | "svgz"
      | "mixed"
      | "lerc"
      | "lerc2d"
      | "pbf"
      | "raw";
    /**
     * Indicates if the tiling scheme supports wrap around.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#isWrappable)
     */
    isWrappable?: boolean;
    /**
     * An array of levels of detail that define the tiling scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#lods)
     */
    lods?: LODProperties[];
    /**
     * The tiling scheme origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#origin)
     */
    origin?: PointProperties;
    /**
     * Size of tiles in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#size)
     */
    size?: number[];
    /**
     * The spatial reference of the tiling schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
  }

  export interface TileInfoCreateOptions {
    /**
     * The size of each tile in pixels.
     *
     * @default 256
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#create)
     */
    size?: number;
    /**
     * Total number of LODs to create.
     *
     * @default 24
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#create)
     */
    numLODs?: number;
    /**
     * The spatial reference for the new TileInfo instance.
     *
     * @default WebMercator
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#create)
     */
    spatialReference?: SpatialReference;
    /**
     * An array of scale values to use for the TileInfo.
     *
     * @default The scales provided by ArcGIS Online basemaps
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileInfo.html#create)
     */
    scales?: number[];
  }

  export interface TileMatrixSet extends Accessor, JSONSupport {}

  export class TileMatrixSet {
    /**
     * Contains information about the tiling scheme for [WMTSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html)
     */

    constructor(properties?: TileMatrixSetProperties);

    /**
     * The full extent of the TileMatrixSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The unique ID assigned to the TileMatrixSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#id)
     */
    id: string;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#tileInfo)
     */
    tileInfo: TileInfo;

    /**
     * Creates a deep clone of this TileMatrixSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#clone)
     */
    clone(): TileMatrixSet;

    static fromJSON(json: any): TileMatrixSet;
  }

  interface TileMatrixSetProperties {
    /**
     * The full extent of the TileMatrixSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The unique ID assigned to the TileMatrixSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#id)
     */
    id?: string;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
  }

  export interface TimeInfo extends Accessor, JSONSupport, Clonable {}

  export class TimeInfo {
    /**
     * Time info represents the temporal data of a time-aware layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html)
     */

    constructor(properties?: TimeInfoProperties);

    /**
     * The name of the field containing the end time information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField)
     */
    endField: string;
    /**
     * The time extent defines the start time and end time for all data in the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent)
     */
    fullTimeExtent: TimeExtent;
    /**
     * The time interval defines the granularity of the temporal data and allows you to visualize the data at specified intervals using the [time slider widget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#interval)
     */
    interval: TimeInterval;
    /**
     * The name of the field containing the start time information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField)
     */
    startField: string;
    /**
     * Indicates the time instants that the layer has data for.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#stops)
     */
    stops: Date[];
    /**
     * The IANA time zone that dates are stored in.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#timeZone)
     */
    timeZone: string;
    /**
     * The name of the field used to join or group discrete locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#trackIdField)
     */
    trackIdField: string;

    static fromJSON(json: any): TimeInfo;
  }

  interface TimeInfoProperties {
    /**
     * The name of the field containing the end time information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField)
     */
    endField?: string;
    /**
     * The time extent defines the start time and end time for all data in the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent)
     */
    fullTimeExtent?: TimeExtentProperties;
    /**
     * The time interval defines the granularity of the temporal data and allows you to visualize the data at specified intervals using the [time slider widget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#interval)
     */
    interval?: TimeIntervalProperties;
    /**
     * The name of the field containing the start time information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField)
     */
    startField?: string;
    /**
     * Indicates the time instants that the layer has data for.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#stops)
     */
    stops?: DateProperties[];
    /**
     * The IANA time zone that dates are stored in.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#timeZone)
     */
    timeZone?: string;
    /**
     * The name of the field used to join or group discrete locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#trackIdField)
     */
    trackIdField?: string;
  }

  export interface VideoElement extends Accessor, Loadable, MediaElementBase {}

  export class VideoElement {
    /**
     * Represents a video element referenced in the MediaLayer's [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html)
     */

    constructor(properties?: VideoElementProperties);

    /**
     * The video content referenced in the video element instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html#content)
     */
    readonly content: HTMLVideoElement;
    /**
     * The element type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html#type)
     */
    readonly type: "video";
    /**
     * The video element to be added to the [media layer's source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html#video)
     */
    video: string | HTMLVideoElement;
  }

  interface VideoElementProperties extends LoadableProperties, MediaElementBaseProperties {
    /**
     * The video element to be added to the [media layer's source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoElement.html#video)
     */
    video?: string | HTMLVideoElement;
  }

  export interface VideoTimeExtent extends Accessor, JSONSupport, Clonable {}

  export class VideoTimeExtent {
    /**
     * The VideoTimeExtent class is used to represent the temporal extent of a video.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html)
     */

    constructor(properties?: VideoTimeExtentProperties);

    /**
     * The duration of the video in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#duration)
     */
    duration: number;
    /**
     * The end time of the video as a Date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#end)
     */
    end: Date;
    /**
     * The start time of the video as a Date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#start)
     */
    start: Date;
    /**
     * The timezone of the video.
     *
     * @default "UTC"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#timezone)
     */
    timezone: string;

    static fromJSON(json: any): VideoTimeExtent;
  }

  interface VideoTimeExtentProperties {
    /**
     * The duration of the video in seconds.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#duration)
     */
    duration?: number;
    /**
     * The end time of the video as a Date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#end)
     */
    end?: DateProperties;
    /**
     * The start time of the video as a Date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#start)
     */
    start?: DateProperties;
    /**
     * The timezone of the video.
     *
     * @default "UTC"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-VideoTimeExtent.html#timezone)
     */
    timezone?: string;
  }

  export interface WMSSublayer extends Accessor, Identifiable {}

  export class WMSSublayer {
    /**
     * Represents a sublayer in a [WMSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)
     */

    constructor(properties?: WMSSublayerProperties);

    /**
     * Description for the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#description)
     */
    description: string;
    /**
     * An array of time, elevation and other dimensions for the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#dimensions)
     */
    readonly dimensions: (TimeDimension | ElevationDimension | GenericDimension)[];
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The id for the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#id)
     */
    id: number;
    /**
     * The [WMSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#layer)
     */
    layer: WMSLayer;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * A string url pointing to a legend image for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#legendUrl)
     */
    legendUrl: string;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#maxScale)
     */
    maxScale: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#minScale)
     */
    minScale: number;
    /**
     * Name of the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#name)
     */
    name: string;
    /**
     * Returns a reference to the parent WMS sublayer or layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#parent)
     */
    parent: WMSSublayer | WMSLayer;
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * Indicates if the layer can be queried, i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#queryable)
     */
    queryable: boolean;
    /**
     * List of spatialReferences (WKID) derived from the CRS elements of the first layer in the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#spatialReferences)
     */
    spatialReferences: number[];
    /**
     * A collection of [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#sublayers)
     */
    sublayers: Collection<WMSSublayer>;
    /**
     * The title of the WMS sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#title)
     */
    title: string;
    /**
     * Indicates if the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#visible)
     */
    visible: boolean;

    /**
     * Creates a deep clone of the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#clone)
     */
    clone(): WMSSublayer;
  }

  interface WMSSublayerProperties {
    /**
     * Description for the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#description)
     */
    description?: string;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The id for the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#id)
     */
    id?: number;
    /**
     * The [WMSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#layer)
     */
    layer?: WMSLayerProperties;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * A string url pointing to a legend image for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#legendUrl)
     */
    legendUrl?: string;
    /**
     * The maximum scale (most zoomed in) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#maxScale)
     */
    maxScale?: number;
    /**
     * The minimum scale (most zoomed out) at which the layer is visible in the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#minScale)
     */
    minScale?: number;
    /**
     * Name of the WMS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#name)
     */
    name?: string;
    /**
     * Returns a reference to the parent WMS sublayer or layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#parent)
     */
    parent?: WMSSublayerProperties | (WMSLayerProperties & { type: "wms" });
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * Indicates if the layer can be queried, i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#queryable)
     */
    queryable?: boolean;
    /**
     * List of spatialReferences (WKID) derived from the CRS elements of the first layer in the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#spatialReferences)
     */
    spatialReferences?: number[];
    /**
     * A collection of [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#sublayers)
     */
    sublayers?: CollectionProperties<WMSSublayerProperties>;
    /**
     * The title of the WMS sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#title)
     */
    title?: string;
    /**
     * Indicates if the layer is visible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#visible)
     */
    visible?: boolean;
  }

  /**
   * Elevation dimension information.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#ElevationDimension)
   */
  export interface ElevationDimension {
    /**
     * Name of dimensional axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#ElevationDimension)
     */
    name: "elevation";
  }

  /**
   * Generic dimension information.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#GenericDimension)
   */
  export interface GenericDimension {
    /**
     * Name of dimensional axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#GenericDimension)
     */
    name: string;
  }

  /**
   * Time dimension information.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
   */
  export interface TimeDimension {
    /**
     * Name of dimensional axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    name: "time";
    /**
     * Units of dimensional axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    units: "ISO8601";
    /**
     * Available value(s) for the time dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    extent: Date[] | TimeDimensionInterval[];
    /**
     * Default value(s) for the time dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    default: Date[] | TimeDimensionInterval[];
    /**
     * Indicates whether multiple values of the dimension may be requested.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    multipleValues: boolean;
    /**
     * Indicates whether the nearest value of the dimension will be returned in response to a request for a nearby value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimension)
     */
    nearestValue: boolean;
  }

  /**
   * The time dimension interval.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
   */
  export interface TimeDimensionInterval {
    /**
     * Start of interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    min: Date;
    /**
     * End of interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    max: Date;
    /**
     * Interval duration defined by the following properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    resolution: TimeDimensionIntervalResolution;
  }

  export interface TimeDimensionIntervalResolution {
    /**
     * Number of years.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    years: number;
    /**
     * Number of months.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    months: number;
    /**
     * Number of days.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    days: number;
    /**
     * Number of hours.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    hours: number;
    /**
     * Number of minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    minutes: number;
    /**
     * Number of seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html#TimeDimensionInterval)
     */
    seconds: number;
  }

  export interface WMTSStyle extends Accessor, JSONSupport {}

  export class WMTSStyle {
    /**
     * Contains information about the WMTS Style for [WMTSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html)
     */

    constructor(properties?: WMTSStyleProperties);

    /**
     * Description for the WMTS style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#description)
     */
    description: string;
    /**
     * The unique ID assigned to the style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#id)
     */
    id: string;
    /**
     * The URL to the legend which gets used in [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#legendUrl)
     */
    legendUrl: string;
    /**
     * The title of the WMTS style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the WMTSStyle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#clone)
     */
    clone(): WMTSStyle;

    static fromJSON(json: any): WMTSStyle;
  }

  interface WMTSStyleProperties {
    /**
     * Description for the WMTS style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#description)
     */
    description?: string;
    /**
     * The unique ID assigned to the style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#id)
     */
    id?: string;
    /**
     * The URL to the legend which gets used in [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#legendUrl)
     */
    legendUrl?: string;
    /**
     * The title of the WMTS style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html#title)
     */
    title?: string;
  }

  export interface WMTSSublayer extends Accessor, JSONSupport {}

  export class WMTSSublayer {
    /**
     * Represents a sublayer in a [WMTSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html)
     */

    constructor(properties?: WMTSSublayerProperties);

    /**
     * Description for the WMTS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#description)
     */
    description: string;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The unique ID assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#id)
     */
    id: string;
    /**
     * The map image format (MIME type) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#imageFormat)
     */
    imageFormat: string;
    /**
     * Supported image formats as retrieved from the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#imageFormats)
     */
    imageFormats: string[];
    /**
     * The [WMTSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#layer)
     */
    layer: WMTSLayer;
    /**
     * The parent [WMTSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#parent)
     */
    parent: WMTSLayer;
    /**
     * The [WMTSStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#styleId)
     */
    styleId: string;
    /**
     * A collection of supported [WMTSStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html)s as retrieved from the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#styles)
     */
    styles: Collection<WMTSStyle>;
    /**
     * The [TileMatrixSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSet)
     */
    tileMatrixSet: TileMatrixSet;
    /**
     * The id of the [TileMatrixSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSetId)
     */
    tileMatrixSetId: string;
    /**
     * A collection of supported [TileMatrixSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSets)
     */
    tileMatrixSets: Collection<TileMatrixSet>;
    /**
     * The title of the WMTS sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the WMTS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#clone)
     */
    clone(): WMTSSublayer;

    static fromJSON(json: any): WMTSSublayer;
  }

  interface WMTSSublayerProperties {
    /**
     * Description for the WMTS sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#description)
     */
    description?: string;
    /**
     * The full extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The unique ID assigned to the sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#id)
     */
    id?: string;
    /**
     * The map image format (MIME type) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#imageFormat)
     */
    imageFormat?: string;
    /**
     * Supported image formats as retrieved from the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#imageFormats)
     */
    imageFormats?: string[];
    /**
     * The [WMTSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#layer)
     */
    layer?: WMTSLayerProperties;
    /**
     * The parent [WMTSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html) to which the sublayer belongs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#parent)
     */
    parent?: WMTSLayerProperties;
    /**
     * The [WMTSStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#styleId)
     */
    styleId?: string;
    /**
     * A collection of supported [WMTSStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSStyle.html)s as retrieved from the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#styles)
     */
    styles?: CollectionProperties<WMTSStyleProperties>;
    /**
     * The [TileMatrixSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSet)
     */
    tileMatrixSet?: TileMatrixSetProperties;
    /**
     * The id of the [TileMatrixSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSetId)
     */
    tileMatrixSetId?: string;
    /**
     * A collection of supported [TileMatrixSets](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TileMatrixSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#tileMatrixSets)
     */
    tileMatrixSets?: CollectionProperties<TileMatrixSetProperties>;
    /**
     * The title of the WMTS sublayer used to identify it in places such as the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) and [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html#title)
     */
    title?: string;
  }

  export interface TileLayer
    extends Layer,
      SublayersOwner,
      ArcGISMapService,
      ArcGISCachedService,
      RefreshableLayer,
      ScaleRangeLayer,
      PortalLayer,
      BlendLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer {}

  export class TileLayer {
    /**
     * The TileLayer allows you work with a cached [map service](https://enterprise.arcgis.com/en/server/latest/publish-services/windows/what-is-a-map-service.htm) exposed by the ArcGIS Server REST API and add it to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) as a tile layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html)
     */

    constructor(properties?: TileLayerProperties);

    /**
     * A flat [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of all the [sublayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#sublayers) in the TileLayer including the sublayers of its sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#allSublayers)
     */
    readonly allSublayers: Collection<Sublayer>;
    /**
     * The URL that points to the location of the layer's attribution data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#attributionDataUrl)
     */
    readonly attributionDataUrl: string;
    /**
     * Indicates if the layer has attribution data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#hasAttributionData)
     */
    readonly hasAttributionData: boolean;
    /**
     * Resampling is enabled by default in 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) and 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#resampling)
     */
    resampling: boolean;
    /**
     * The [tiled map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#sublayers)
     */
    readonly sublayers: Collection<Sublayer>;
    /**
     * An array of tile servers used for changing map tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#tileServers)
     */
    tileServers: string[];

    readonly type: "tile";
    /**
     * The URL of the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#url)
     */
    url: string;

    /**
     * This method fetches a tile for the given level, row and column present in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#fetchTile)
     */
    fetchTile(level: number, row: number, col: number, options?: TileLayerFetchTileOptions): Promise<HTMLImageElement>;
    /**
     * This method returns a URL to a tile for a given level, row and column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#getTileUrl)
     */
    getTileUrl(level: number, row: number, col: number): string;
    /**
     * Loads all of the sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#loadAll)
     */
    loadAll(): Promise<Sublayer>;

    on(name: "refresh", eventHandler: TileLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: TileLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: TileLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: TileLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): TileLayer;
  }

  interface TileLayerProperties
    extends LayerProperties,
      SublayersOwnerProperties,
      ArcGISMapServiceProperties,
      ArcGISCachedServiceProperties,
      RefreshableLayerProperties,
      ScaleRangeLayerProperties,
      PortalLayerProperties,
      BlendLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties {
    /**
     * Resampling is enabled by default in 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) and 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#resampling)
     */
    resampling?: boolean;
    /**
     * The [tiled map service's metadata JSON](https://developers.arcgis.com/rest/services-reference/map-service.htm) exposed by the ArcGIS REST API.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * An array of tile servers used for changing map tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#tileServers)
     */
    tileServers?: string[];
    /**
     * The URL of the REST endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#url)
     */
    url?: string;
  }

  export interface TileLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface TileLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface TileLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface TileLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface TileLayerFetchTileOptions {
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export class UnknownLayer extends Layer {
    /**
     * Represents a layer whose type could not be determined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-UnknownLayer.html)
     */

    constructor(properties?: UnknownLayerProperties);

    readonly type: "unknown";
  }

  interface UnknownLayerProperties extends LayerProperties {}

  export class UnsupportedLayer extends Layer {
    /**
     * Represents an unsupported layer instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-UnsupportedLayer.html)
     */

    constructor(properties?: UnsupportedLayerProperties);

    readonly type: "unsupported";
  }

  interface UnsupportedLayerProperties extends LayerProperties {}

  export interface VectorTileLayer extends Layer, OperationalLayer, ScaleRangeLayer, BlendLayer {}

  export class VectorTileLayer {
    /**
     * VectorTileLayer accesses cached tiles of data and renders it in vector format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html)
     */

    constructor(properties?: VectorTileLayerProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#apiKey)
     */
    apiKey: string;
    /**
     * The URL that points to the location of the layer's attribution data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#attributionDataUrl)
     */
    readonly attributionDataUrl: string;
    /**
     * Indicates the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    readonly capabilities: VectorTileLayerCapabilities;
    /**
     * The current style information of the VectorTileLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    readonly currentStyleInfo: VectorTileLayerCurrentStyleInfo;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The initial extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#initialExtent)
     */
    initialExtent: Extent;
    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * A style JSON object of vector tiles that will be used to render the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#style)
     */
    style: any;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#tileInfo)
     */
    tileInfo: TileInfo;

    readonly type: "vector-tile";
    /**
     * The URL to the vector tile service, or the URL to the style resource of vector tiles that will be used to render the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#url)
     */
    url: string;

    /**
     * Deletes the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/) from the VectorTileLayer's [currentStyleInfo.style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#deleteStyleLayer)
     */
    deleteStyleLayer(layerId: string): void;
    /**
     * Returns an instance of [layout](https://maplibre.org/maplibre-style-spec/layers/#layout-property) properties for the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getLayoutProperties)
     */
    getLayoutProperties(layerId: string): any;
    /**
     * Returns an instance of [paint](https://maplibre.org/maplibre-style-spec/layers/#paint) properties for the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getPaintProperties)
     */
    getPaintProperties(layerId: string): any;
    /**
     * Returns an instance of a [style layer](https://maplibre.org/maplibre-style-spec/layers) from the VectorTileLayer's [currentStyleInfo.style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getStyleLayer)
     */
    getStyleLayer(layerId: string): any;
    /**
     * Returns the layer [id](https://maplibre.org/maplibre-style-spec/layers/#id) of the [style layer](https://maplibre.org/maplibre-style-spec/layers) based on its index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getStyleLayerId)
     */
    getStyleLayerId(index: number): string;
    /**
     * Returns the index of the [style layer](https://maplibre.org/maplibre-style-spec/layers) from the VectorTileLayer's [currentStyleInfo.style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getStyleLayerIndex)
     */
    getStyleLayerIndex(layerId: string): number;
    /**
     * Gets the visibility of the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/) from the VectorTileLayer's [currentStyleInfo.style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#getStyleLayerVisibility)
     */
    getStyleLayerVisibility(layerId: string): string;
    /**
     * Loads a style to render a layer from the specified URL to a style resource or style JSON object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#loadStyle)
     */
    loadStyle(style: string | any, options?: VectorTileLayerLoadStyleOptions): Promise<any>;
    /**
     * Fetches all the data for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#refresh)
     */
    refresh(): void;
    /**
     * Updates the [layout](https://maplibre.org/maplibre-style-spec/layers/#layout-property) properties to the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setLayoutProperties)
     */
    setLayoutProperties(layerId: string, layout: any): void;
    /**
     * Updates the [paint](https://maplibre.org/maplibre-style-spec/layers/#paint) properties to the specified [style layer](https://maplibre.org/maplibre-style-spec/layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setPaintProperties)
     */
    setPaintProperties(layerId: string, painter: any): void;
    /**
     * Changes the sprite source associated with the VectorTileLayer's [current style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setSpriteSource)
     */
    setSpriteSource(spriteSourceInfo: SpriteSourceImageInfo | SpriteSourceUrlInfo): Promise<SpriteSource>;
    /**
     * Changes the layer properties of the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setStyleLayer)
     */
    setStyleLayer(layer: any, index?: number): void;
    /**
     * Toggles the visibility of the specified [style layer](https://maplibre.org/maplibre-style-spec/layers/) in the VectorTileLayer's [currentStyleInfo.style](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setStyleLayerVisibility)
     */
    setStyleLayerVisibility(layerId: string, visibility: "none" | "visible"): void;

    on(name: "refresh", eventHandler: VectorTileLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: VectorTileLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: VectorTileLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: VectorTileLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): VectorTileLayer;
  }

  interface VectorTileLayerProperties
    extends LayerProperties,
      OperationalLayerProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#apiKey)
     */
    apiKey?: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The initial extent of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#initialExtent)
     */
    initialExtent?: ExtentProperties;
    /**
     * The portal item from which the layer is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * Refresh interval of the layer in minutes.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * A style JSON object of vector tiles that will be used to render the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#style)
     */
    style?: any;
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
    /**
     * The URL to the vector tile service, or the URL to the style resource of vector tiles that will be used to render the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#url)
     */
    url?: string;
  }

  export type GetSpriteInfo = (name: string) => SpriteInfo;

  /**
   * Defines an image object that can be used when setting [SpriteSourceImageInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#ImageObject)
   */
  export interface ImageObject {
    /**
     * The width of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#ImageObject)
     */
    width: number;
    /**
     * The height of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#ImageObject)
     */
    height: number;
    /**
     * The image data for the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#ImageObject)
     */
    data: ArrayBuffer;
  }

  export interface VectorTileLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface VectorTileLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface VectorTileLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface VectorTileLayerRefreshEvent {
    dataChanged: boolean;
  }

  /**
   * A description of each image contained in the sprite.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
   */
  export interface SpriteInfo {
    /**
     * The x position of the upper-left corner of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    x: number;
    /**
     * The y position of the upper-left corner of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    y: number;
    /**
     * The height of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    height: number;
    /**
     * The width of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    width: number;
    /**
     * The pixel ratio of the sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    pixelRatio?: number;
    /**
     * Whether the sprite is SDF or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteInfo)
     */
    sdf?: number;
  }

  /**
   * The sprite source returned from the [setSpriteSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setSpriteSource) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
   */
  export interface SpriteSource {
    /**
     * The base URL of the sprites source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    baseUrl: string;
    /**
     * Indicates whether the sprite source is loaded successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    loadStatus: any | any | any | any;
    /**
     * The device pixel ratio of the sprite source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    devicePixelRatio: number;
    /**
     * The height in pixels of the sprite source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    height: number;
    /**
     * The width in pixels of the sprite source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    width: number;
    /**
     * The sprite image data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    image: Uint8Array;
    /**
     * Returns the sprite info for the given sprite.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSource)
     */
    getSpriteInfo: GetSpriteInfo;
  }

  /**
   * Defines sprite source from an image and index json when the [setSpriteSource()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setSpriteSource) method is called.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
   */
  export interface SpriteSourceImageInfo {
    /**
     * For SpriteSourceImageInfo the type is always "image".
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The sprite source url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    spriteUrl?: string;
    /**
     * The device pixel ratio of the sprite source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    pixelRatio?: number;
    /**
     * Sprite source for non high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    spriteSource1x?: SpriteSourceImageInfoSpriteSource1x;
    /**
     * Sprite source for high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    spriteSource2x?: SpriteSourceImageInfoSpriteSource2x;
  }

  /**
   * The URL of the sprite source to be fetched when the [setSpriteSource()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#setSpriteSource) method is called.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceUrlInfo)
   */
  export interface SpriteSourceUrlInfo {
    /**
     * For SpriteSourceUrlInfo the type is always "url".
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The sprite source url.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceUrlInfo)
     */
    spriteUrl: string;
    /**
     * The device pixel ratio used to fetch the appropriate sprite source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceUrlInfo)
     */
    pixelRatio?: number;
    /**
     * The sprite image format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceUrlInfo)
     */
    spriteFormat?: "png" | "webp";
  }

  export interface VectorTileLayerCapabilities {
    /**
     * Indicates options supported by the exportTiles operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    exportTiles: VectorTileLayerCapabilitiesExportTiles;
    /**
     * Indicates operations that can be performed on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    operations: VectorTileLayerCapabilitiesOperations;
  }

  export interface VectorTileLayerCapabilitiesExportTiles {
    /**
     * Specifies the maximum number of tiles that can be exported to a cache dataset or a tile package.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    maxExportTilesCount: number;
  }

  export interface VectorTileLayerCapabilitiesOperations {
    /**
     * Indicates if the tiles from the service can be exported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    supportsExportTiles: boolean;
    /**
     * Indicates if the service exposes a tile map that describes the presence of tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#capabilities)
     */
    supportsTileMap: boolean;
  }

  export interface VectorTileLayerCurrentStyleInfo {
    /**
     * Absolute URL for a vector tile service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    serviceUrl: string;
    /**
     * Absolute URL for vector tile service style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    styleUrl: string;
    /**
     * Absolute URL for sprites included in a style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    spriteUrl: string;
    /**
     * Absolute template URL for font sets included in a style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    glyphsUrl: string;
    /**
     * Style JSON object for vector tiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    style: any;
    /**
     * Vector tile service information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#currentStyleInfo)
     */
    layerDefinition: any;
  }

  export interface VectorTileLayerLoadStyleOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#loadStyle)
     */
    signal?: AbortSignal;
  }

  export interface SpriteSourceImageInfoSpriteSource1x {
    /**
     * Image containing sprite data for non high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    image: HTMLImageElement | HTMLCanvasElement | any | ImageData | ImageObject;
    /**
     * The JSON representation of an index file containing a description of each image in the sprite for non high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    json: SpriteInfo;
  }

  export interface SpriteSourceImageInfoSpriteSource2x {
    /**
     * Image containing sprite data for high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    image: HTMLImageElement | HTMLCanvasElement | any | ImageData | ImageObject;
    /**
     * The JSON representation of an index file containing a description of each image in the sprite for high-DPI devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html#SpriteSourceImageInfo)
     */
    json: SpriteInfo;
  }

  export interface VideoLayer extends Layer, CustomParametersMixin, ScaleRangeLayer, BlendLayer {}

  export class VideoLayer {
    /**
     * The VideoLayer provides video content from on-demand and livestream feeds from an [ArcGIS Video Server](https://enterprise.arcgis.com/en/video/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html)
     */

    constructor(properties?: VideoLayerProperties);

    /**
     * Represents the length of the currently buffered video in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#buffered)
     */
    readonly buffered: number;
    /**
     * The capabilities of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    readonly capabilities: VideoLayerCapabilities;
    /**
     * The copyright information for the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#copyright)
     */
    copyright: string;
    /**
     * The date the video layer was created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#created)
     */
    readonly created: Date;
    /**
     * The current time of the video layer in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#currentTime)
     */
    currentTime: number;
    /**
     * The description of the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#description)
     */
    description: string;
    /**
     * The duration of the video layer in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#duration)
     */
    readonly duration: number;
    /**
     * Indicates if the video layer has ended and the [current time](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#currentTime) is equal to the [duration](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#duration) of the video.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#ended)
     */
    readonly ended: boolean;
    /**
     * The total number of frames in the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#frameCount)
     */
    readonly frameCount: number;
    /**
     * The initial extent of the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#initialExtent)
     */
    initialExtent: Extent;
    /**
     * Indicates if the video layer should loop.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#loop)
     */
    loop: boolean;
    /**
     * The metadata for the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#metadata)
     */
    readonly metadata: globalThis.Map<number, VideoMetadataEntry>;
    /**
     * Indicates if the video layer is muted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#muted)
     */
    muted: boolean;
    /**
     * The playback rate of the video layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#playbackRate)
     */
    playbackRate: number;
    /**
     * Indicates if the video layer is playing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#playing)
     */
    readonly playing: boolean;
    /**
     * The URL to the poster image for the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#posterUrl)
     */
    readonly posterUrl: string;
    /**
     * The available video qualities for the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#qualities)
     */
    readonly qualities: ("sd" | "hd" | "fhd" | "qhd" | "uhd")[];
    /**
     * The ArcGIS Enterprise Portal item id of the video service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#serviceItemId)
     */
    readonly serviceItemId: string;
    /**
     * The source quality of the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#sourceQuality)
     */
    readonly sourceQuality: "sd" | "hd" | "fhd" | "qhd" | "uhd";
    /**
     * The source type of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#sourceType)
     */
    readonly sourceType: "ondemand" | "livestream";
    /**
     * The spatial reference of the video layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The current state of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#state)
     */
    readonly state: "can-play" | "not-ready" | "paused" | "playing" | "ready" | "waiting" | "data-loaded";
    /**
     * The telemetry data for the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#telemetry)
     */
    readonly telemetry: TelemetryData;
    /**
     * The telemetry display for the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#telemetryDisplay)
     */
    telemetryDisplay: TelemetryDisplay;

    readonly type: "video";
    /**
     * The URL to the REST endpoint of the video service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#url)
     */
    url: string;
    /**
     * The height of the video in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#videoHeight)
     */
    readonly videoHeight: number;
    /**
     * Defines layer information for video layers published within the same video service
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#videoLayersInfo)
     */
    readonly videoLayersInfo: VideoServiceLayerInfo[];
    /**
     * The time extent of the video.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#videoTimeExtent)
     */
    readonly videoTimeExtent: VideoTimeExtent;
    /**
     * The width of the video in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#videoWidth)
     */
    readonly videoWidth: number;
    /**
     * Indicates if the video layer is waiting for data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#waiting)
     */
    readonly waiting: boolean;

    /**
     * Pauses the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#pause)
     */
    pause(): void;
    /**
     * Plays the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#play)
     */
    play(): void;
    /**
     * Resets the video layer to its initial state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#reset)
     */
    reset(): void;
    /**
     * Sets the current time, in seconds, of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#setCurrentTime)
     */
    setCurrentTime(timestamp: number): void;
  }

  interface VideoLayerProperties
    extends LayerProperties,
      CustomParametersMixinProperties,
      ScaleRangeLayerProperties,
      BlendLayerProperties {
    /**
     * The copyright information for the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#copyright)
     */
    copyright?: string;
    /**
     * The current time of the video layer in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#currentTime)
     */
    currentTime?: number;
    /**
     * The description of the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#description)
     */
    description?: string;
    /**
     * The initial extent of the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#initialExtent)
     */
    initialExtent?: ExtentProperties;
    /**
     * Indicates if the video layer should loop.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#loop)
     */
    loop?: boolean;
    /**
     * Indicates if the video layer is muted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#muted)
     */
    muted?: boolean;
    /**
     * The playback rate of the video layer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#playbackRate)
     */
    playbackRate?: number;
    /**
     * The spatial reference of the video layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The telemetry display for the video layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#telemetryDisplay)
     */
    telemetryDisplay?: TelemetryDisplayProperties;
    /**
     * The URL to the REST endpoint of the video service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#url)
     */
    url?: string;
  }

  export interface VideoLayerCapabilities {
    /**
     * The operations capabilities of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    operations: VideoLayerCapabilitiesOperations;
  }

  export interface VideoLayerCapabilitiesOperations {
    /**
     * Indicates if the video layer supports appending data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsAppend: boolean;
    /**
     * Indicates if the video layer supports coverage queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsCoverageQuery: boolean;
    /**
     * Indicates if the video layer supports exporting clips.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsExportClip: boolean;
    /**
     * Indicates if the video layer supports exporting framesets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsExportFrameset: boolean;
    /**
     * Indicates if the video layer supports mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsMensuration: boolean;
    /**
     * Indicates if the video layer supports updating data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#capabilities)
     */
    supportsUpdate: boolean;
  }

  /**
   * The **VideoMetadataEntry** is an object that represents a metadata entry for the video layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoMetadataEntry)
   */
  export interface VideoMetadataEntry {
    /**
     * The name of the metadata entry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoMetadataEntry)
     */
    name: string;
    /**
     * The tag id of the metadata entry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoMetadataEntry)
     */
    tagId: number;
    /**
     * The value of the metadata entry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoMetadataEntry)
     */
    value: any;
  }

  /**
   * The **VideoPoint** is an object that represents a point on the current video frame.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoPoint)
   */
  export interface VideoPoint {
    /**
     * The x coordinate on the current video frame in pixels from the top left corner
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoPoint)
     */
    x: number;
    /**
     * The y coordinate on the current video frame in pixels from the top left corner
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoPoint)
     */
    y: number;
  }

  /**
   * The **VideoServiceLayerInfo** defines layer information for video layers published within the same video service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoServiceLayerInfo)
   */
  export interface VideoServiceLayerInfo {
    /**
     * The layer id of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoServiceLayerInfo)
     */
    layerId: number;
    /**
     * The URL to the poster image for the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoServiceLayerInfo)
     */
    posterUrl: string;
    /**
     * The source type of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoServiceLayerInfo)
     */
    sourceType: "ondemand" | "livestream";
    /**
     * The title of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html#VideoServiceLayerInfo)
     */
    title: string;
    /**
     * The layer type.
     *
     * [Read more...](global.html#type)
     */
    type: "Video Layer";
  }

  export interface ViewshedLayer extends Layer, OperationalLayer {}

  export class ViewshedLayer {
    /**
     * The viewshed layer enables the creation and display of viewshed and view dome type of visibility analysis in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html)
     */

    constructor(properties?: ViewshedLayerProperties);

    /**
     * The full extent of the layer which contains all viewsheds referenced by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html#fullExtent)
     */
    readonly fullExtent: Extent;
    /**
     * The [ViewshedAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html) associated with the layer that stores the viewsheds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html#source)
     */
    source: ViewshedAnalysis;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;

    readonly type: "viewshed";

    static fromJSON(json: any): ViewshedLayer;
  }

  interface ViewshedLayerProperties extends LayerProperties, OperationalLayerProperties {
    /**
     * The [ViewshedAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html) associated with the layer that stores the viewsheds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html#source)
     */
    source?: ViewshedAnalysisProperties;
  }

  export interface VoxelDynamicSection extends JSONSupport, Clonable {}

  export class VoxelDynamicSection {
    /**
     * The VoxelDynamicSection allows you to define the properties of an individual dynamic section.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html)
     */

    constructor(properties?: VoxelDynamicSectionProperties);

    /**
     * Whether or not the dynamic section is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#enabled)
     */
    enabled: boolean;
    /**
     * The label for the dynamic section.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#label)
     */
    label: string;
    /**
     * The orientation angle (in the degrees) of the dynamic section plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#orientation)
     */
    orientation: number;
    /**
     * A point on the dynamic section plane specified as [x ,y, z] in voxel space for XYZ and XYZT volumes and as [x, y, t] for XYT volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#point)
     */
    point: number[];
    /**
     * The tilt angle (in degrees) of the dynamic section plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#tilt)
     */
    tilt: number;

    static fromJSON(json: any): VoxelDynamicSection;
  }

  interface VoxelDynamicSectionProperties {
    /**
     * Whether or not the dynamic section is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#enabled)
     */
    enabled?: boolean;
    /**
     * The label for the dynamic section.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#label)
     */
    label?: string;
    /**
     * The orientation angle (in the degrees) of the dynamic section plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#orientation)
     */
    orientation?: number;
    /**
     * A point on the dynamic section plane specified as [x ,y, z] in voxel space for XYZ and XYZT volumes and as [x, y, t] for XYT volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#point)
     */
    point?: number[];
    /**
     * The tilt angle (in degrees) of the dynamic section plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html#tilt)
     */
    tilt?: number;
  }

  export interface VoxelSlice extends JSONSupport, Clonable {}

  export class VoxelSlice {
    /**
     * The VoxelSlice allows you to define the properties of an individual slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html)
     */

    constructor(properties?: VoxelSliceProperties);

    /**
     * Whether or not the slice is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#enabled)
     */
    enabled: boolean;
    /**
     * The label for the slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#label)
     */
    label: string;
    /**
     * The orientation angle (in the degrees) of the slice plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#orientation)
     */
    orientation: number;
    /**
     * A point on the slice plane specified as [x ,y, z] in voxel space for XYZ and XYZT volumes and as [x, y, t] for XYT volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#point)
     */
    point: number[];
    /**
     * The tilt angle (in degrees) of the slice plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#tilt)
     */
    tilt: number;

    static fromJSON(json: any): VoxelSlice;
  }

  interface VoxelSliceProperties {
    /**
     * Whether or not the slice is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#enabled)
     */
    enabled?: boolean;
    /**
     * The label for the slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#label)
     */
    label?: string;
    /**
     * The orientation angle (in the degrees) of the slice plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#orientation)
     */
    orientation?: number;
    /**
     * A point on the slice plane specified as [x ,y, z] in voxel space for XYZ and XYZT volumes and as [x, y, t] for XYT volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#point)
     */
    point?: number[];
    /**
     * The tilt angle (in degrees) of the slice plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html#tilt)
     */
    tilt?: number;
  }

  export interface VoxelTransferFunctionStyle extends JSONSupport, Clonable {}

  export class VoxelTransferFunctionStyle {
    /**
     * The VoxelTransferFunctionStyle allows you to define how an individual continuous variable is rendered as a volume or as sections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html)
     */

    constructor(properties?: VoxelTransferFunctionStyleProperties);

    /**
     * A collection of colors and normalized positions which describe how to colorize the data values that are within the [stretchRange](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#colorStops)
     */
    colorStops: Collection<VoxelColorStop>;
    /**
     * A collection of transparency values and normalized positions which describe how to apply transparency to the data values that are within the [stretchRange](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#opacityStops)
     */
    opacityStops: Collection<VoxelOpacityStop>;
    /**
     * Defines the data range which will be rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#rangeFilter)
     */
    rangeFilter: VoxelRangeFilter;
    /**
     * The data range to apply the color and alpha stops to, specified as [minimum, maximum] in the units of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange)
     */
    stretchRange: number[];

    static fromJSON(json: any): VoxelTransferFunctionStyle;
  }

  interface VoxelTransferFunctionStyleProperties {
    /**
     * A collection of colors and normalized positions which describe how to colorize the data values that are within the [stretchRange](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#colorStops)
     */
    colorStops?: CollectionProperties<VoxelColorStop>;
    /**
     * A collection of transparency values and normalized positions which describe how to apply transparency to the data values that are within the [stretchRange](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#opacityStops)
     */
    opacityStops?: CollectionProperties<VoxelOpacityStop>;
    /**
     * Defines the data range which will be rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#rangeFilter)
     */
    rangeFilter?: VoxelRangeFilter;
    /**
     * The data range to apply the color and alpha stops to, specified as [minimum, maximum] in the units of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#stretchRange)
     */
    stretchRange?: number[];
  }

  /**
   * VoxelColorStop defines a color and a normalized position.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelColorStop)
   */
  export interface VoxelColorStop {
    /**
     * The color to render data at this position with.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelColorStop)
     */
    color: Color;
    /**
     * The normalized (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelColorStop)
     */
    position: number;
  }

  /**
   * VoxelOpacityStop defines an opacity value and a normalized position.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelOpacityStop)
   */
  export interface VoxelOpacityStop {
    /**
     * The opacity value (0 = transparent, 1 = opaque) to apply to data at this position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelOpacityStop)
     */
    opacity: number;
    /**
     * The normalized (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelOpacityStop)
     */
    position: number;
  }

  /**
   * VoxelRangeFilter defines the data range which will be rendered.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelRangeFilter)
   */
  export interface VoxelRangeFilter {
    /**
     * Whether or not the filter is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelRangeFilter)
     */
    enabled?: boolean;
    /**
     * The data range to render, specified as [minimum, maximum].
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html#VoxelRangeFilter)
     */
    range: number[];
  }

  /**
   * Describes a single variable in the [VoxelLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html)
   */
  interface VoxelVariable {
    /**
     * The original data source description of the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#description)
     */
    readonly description: string;
    /**
     * The unique identifier for the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#id)
     */
    readonly id: number;
    /**
     * The original data source name of the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#name)
     */
    readonly name: string;
    /**
     * A description of various properties of the variable as it is represented on the GPU.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#renderingFormat)
     */
    readonly renderingFormat: VoxelVariableRenderingFormat;
    /**
     * The variable units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#unit)
     */
    readonly unit: string;
    /**
     * The volumeId of the variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#volumeId)
     */
    readonly volumeId: number;
  }

  export const VoxelVariable: VoxelVariable;

  export interface VoxelVariableRenderingFormat {
    /**
     * Whether or not the data is discrete or continuous.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#renderingFormat)
     */
    continuity: "discrete" | "continuous";
    /**
     * Whether or not the data has a value representing NoData.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#renderingFormat)
     */
    hasNoData?: boolean;
    /**
     * The NoData value if there is one.
     *
     * @default 0.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html#renderingFormat)
     */
    noData?: number;
    /**
     * Indicates how the data are encoded.
     *
     * [Read more...](global.html#type)
     */
    type: "Int8" | "UInt8" | "Int16" | "UInt16" | "Int32" | "UInt32" | "Float32" | "Float64";
  }

  export interface VoxelVariableStyle extends JSONSupport, Clonable {}

  export class VoxelVariableStyle {
    /**
     * The VoxelVariableStyle allows you to define how an individual variable is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html)
     */

    constructor(properties?: VoxelVariableStyleProperties);

    /**
     * The collection of continuous variable isosurfaces.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#isosurfaces)
     */
    isosurfaces: Collection<VoxelIsosurface>;
    /**
     * The variable label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#label)
     */
    label: string;
    /**
     * The transferFunction describes how to render continuous variable volumes and sections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#transferFunction)
     */
    transferFunction: VoxelTransferFunctionStyle;
    /**
     * The collection of unique values describes how to render discrete variable volumes and sections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#uniqueValues)
     */
    uniqueValues: Collection<VoxelUniqueValue>;
    /**
     * Id of the [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html) that the style applies to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#variableId)
     */
    variableId: number;

    static fromJSON(json: any): VoxelVariableStyle;
  }

  interface VoxelVariableStyleProperties {
    /**
     * The collection of continuous variable isosurfaces.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#isosurfaces)
     */
    isosurfaces?: CollectionProperties<VoxelIsosurface>;
    /**
     * The variable label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#label)
     */
    label?: string;
    /**
     * The transferFunction describes how to render continuous variable volumes and sections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#transferFunction)
     */
    transferFunction?: VoxelTransferFunctionStyleProperties;
    /**
     * The collection of unique values describes how to render discrete variable volumes and sections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#uniqueValues)
     */
    uniqueValues?: CollectionProperties<VoxelUniqueValue>;
    /**
     * Id of the [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html) that the style applies to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#variableId)
     */
    variableId?: number;
  }

  /**
   * VoxelIsosurface describes an isosurface value and how to render it.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
   */
  export interface VoxelIsosurface {
    /**
     * The color to render this isosurface with.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
     */
    color: Color;
    /**
     * The data value for this isosurface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
     */
    value: number;
    /**
     * Whether or not to render this isosurface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
     */
    enabled?: boolean;
    /**
     * The label string for this isosurface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
     */
    label?: string;
    /**
     * Whether or not the isosurface color is automatically updated when the variable's transfer function's color stops change.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelIsosurface)
     */
    colorLocked?: boolean;
  }

  /**
   * VoxelUniqueValue describes a particular voxel discrete value how to render it.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelUniqueValue)
   */
  export interface VoxelUniqueValue {
    /**
     * The color to render this value with.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelUniqueValue)
     */
    color: Color;
    /**
     * The data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelUniqueValue)
     */
    value: number;
    /**
     * Whether or not to render data with this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelUniqueValue)
     */
    enabled?: boolean;
    /**
     * The label string for this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html#VoxelUniqueValue)
     */
    label?: string;
  }

  /**
   * The VoxelVolume exposes properties that describe the volume and methods to convert to and from voxel space.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html)
   */
  interface VoxelVolume {
    /**
     * The unique identifier for the volume.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html#id)
     */
    readonly id: number;
    /**
     * A 3-component array containing the volume size in voxels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html#sizeInVoxels)
     */
    sizeInVoxels: number[];
    /**
     * Returns the type of the current variable's volume.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html#volumeType)
     */
    volumeType: "xyz" | "xyzt" | "xyt";

    /**
     * Convert a position in voxel space to the spatialReference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html#computeLayerSpaceLocation)
     */
    computeLayerSpaceLocation(posInVoxelSpace: number[]): Point;
    /**
     * Convert a position from the layer's spatialReference to voxel space for XYZ or XYZT volumes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html#computeVoxelSpaceLocation)
     */
    computeVoxelSpaceLocation(pos: Point): number[];
  }

  export const VoxelVolume: VoxelVolume;

  export interface VoxelVolumeStyle extends JSONSupport, Clonable {}

  export class VoxelVolumeStyle {
    /**
     * The VolumeStyle allows you to define rendering properties that apply to the volume itself, such as vertical exaggeration and offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html)
     */

    constructor(properties?: VoxelVolumeStyleProperties);

    /**
     * The collection of [VoxelDynamicSection](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#dynamicSections)
     */
    dynamicSections: Collection<VoxelDynamicSection>;
    /**
     * The collection of [VoxelSlice](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#slices)
     */
    slices: Collection<VoxelSlice>;
    /**
     * The vertical exaggeration factor.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#verticalExaggeration)
     */
    verticalExaggeration: number;
    /**
     * The vertical offset amount.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#verticalOffset)
     */
    verticalOffset: number;
    /**
     * Id of the volume.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#volumeId)
     */
    readonly volumeId: number;

    static fromJSON(json: any): VoxelVolumeStyle;
  }

  interface VoxelVolumeStyleProperties {
    /**
     * The collection of [VoxelDynamicSection](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelDynamicSection.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#dynamicSections)
     */
    dynamicSections?: CollectionProperties<VoxelDynamicSectionProperties>;
    /**
     * The collection of [VoxelSlice](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelSlice.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#slices)
     */
    slices?: CollectionProperties<VoxelSliceProperties>;
    /**
     * The vertical exaggeration factor.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#verticalExaggeration)
     */
    verticalExaggeration?: number;
    /**
     * The vertical offset amount.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html#verticalOffset)
     */
    verticalOffset?: number;
  }

  export interface VoxelLayer
    extends Layer,
      SceneService,
      PortalLayer,
      ScaleRangeLayer,
      CustomParametersMixin,
      APIKeyMixin,
      OperationalLayer {}

  export class VoxelLayer {
    /**
     * A voxel layer represents multidimensional volumetric data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html)
     */

    constructor(properties?: VoxelLayerProperties);

    /**
     * The variable that is being displayed.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#currentVariableId)
     */
    currentVariableId: number;
    /**
     * Controls whether or not to globally disable all dynamic sections in the current [VoxelVolumeStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableDynamicSections)
     */
    enableDynamicSections: boolean;
    /**
     * Controls whether or not to globally disable all isosurfaces in the current [VoxelVariableStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableIsosurfaces)
     */
    enableIsosurfaces: boolean;
    /**
     * Controls whether or not to globally disable all slices in the current [VoxelVolumeStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableSlices)
     */
    enableSlices: boolean;
    /**
     * A complete list of fields that consists of fixed voxel fields and the voxel variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#fields)
     */
    readonly fields: Field[];
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * Indicates whether to display popups when voxels in the layer are clicked.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Current rendering mode for the [VoxelLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html).
     *
     * @default "volume"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#renderMode)
     */
    renderMode: "volume" | "surfaces";
    /**
     * TimeInfo provides information such as [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#stops) the layer has data for and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#timeInfo)
     */
    readonly timeInfo: TimeInfo;

    readonly type: "voxel";
    /**
     * The collection of variables that the VoxelLayer has data for.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#variables)
     */
    readonly variables: Collection<VoxelVariable>;
    /**
     * The collection of variable styles, containing exactly one [VoxelVariableStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html) for each [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#variableStyles)
     */
    variableStyles: Collection<VoxelVariableStyle>;
    /**
     * The collection of volumes that the VoxelLayer has variables for.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#volumes)
     */
    readonly volumes: Collection<VoxelVolume>;
    /**
     * The collection of volume styles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#volumeStyles)
     */
    volumeStyles: Collection<VoxelVolumeStyle>;

    /**
     * Creates a default popup template for the layer, populated with the default voxel fields and a field for each variable in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Returns a [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) for a given value for the continuous [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html) identified by variableId using the variable's [VoxelTransferFunctionStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelTransferFunctionStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getColorForContinuousDataValue)
     */
    getColorForContinuousDataValue(variableId: number, dataValue: number, applyOpacityStops: boolean): Color;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns a [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html) based on an id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getVariable)
     */
    getVariable(variableId: number): VoxelVariable;
    /**
     * Returns a [VoxelVariableStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html) based on an id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getVariableStyle)
     */
    getVariableStyle(variableId: number): VoxelVariableStyle;
    /**
     * Returns a [VoxelVolume](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolume.html) based on the id of a [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getVolume)
     */
    getVolume(variableId: number): VoxelVolume;
    /**
     * Returns a [VoxelVolumeStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html) based on the id of a [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#getVolumeStyle)
     */
    getVolumeStyle(variableId: number): VoxelVolumeStyle;

    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#timeExtent)
     */
    static timeExtent: TimeExtent;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#timeOffset)
     */
    static timeOffset: TimeInterval;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#useViewTime)
     */
    static useViewTime: boolean;

    static fromJSON(json: any): VoxelLayer;
  }

  interface VoxelLayerProperties
    extends LayerProperties,
      SceneServiceProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      CustomParametersMixinProperties,
      APIKeyMixinProperties,
      OperationalLayerProperties {
    /**
     * The variable that is being displayed.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#currentVariableId)
     */
    currentVariableId?: number;
    /**
     * Controls whether or not to globally disable all dynamic sections in the current [VoxelVolumeStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableDynamicSections)
     */
    enableDynamicSections?: boolean;
    /**
     * Controls whether or not to globally disable all isosurfaces in the current [VoxelVariableStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableIsosurfaces)
     */
    enableIsosurfaces?: boolean;
    /**
     * Controls whether or not to globally disable all slices in the current [VoxelVolumeStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVolumeStyle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#enableSlices)
     */
    enableSlices?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * Indicates whether to display popups when voxels in the layer are clicked.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Current rendering mode for the [VoxelLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html).
     *
     * @default "volume"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#renderMode)
     */
    renderMode?: "volume" | "surfaces";
    /**
     * The collection of variable styles, containing exactly one [VoxelVariableStyle](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariableStyle.html) for each [VoxelVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-voxel-VoxelVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#variableStyles)
     */
    variableStyles?: CollectionProperties<VoxelVariableStyleProperties>;
    /**
     * The collection of volume styles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VoxelLayer.html#volumeStyles)
     */
    volumeStyles?: CollectionProperties<VoxelVolumeStyleProperties>;
  }

  export interface WCSLayer
    extends Layer,
      BlendLayer,
      ImageryTileMixin,
      OperationalLayer,
      PortalLayer,
      ScaleRangeLayer {}

  export class WCSLayer {
    /**
     * WCS presents raster data from a [OGC Web Coverage Service](https://www.ogc.org/standards/wcs).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html)
     */

    constructor(properties?: WCSLayerProperties);

    /**
     * The coverage identifier for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#coverageId)
     */
    coverageId: string;
    /**
     * Coverage information retrieved from the WCS Server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#coverageInfo)
     */
    coverageInfo: CoverageInfo;
    /**
     * Use this property to append custom parameters to all WCS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * An array of raster fields in the layer that consists of service pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#rasterFields)
     */
    rasterFields: Field[];
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#renderer)
     */
    renderer:
      | ClassBreaksRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | RasterShadedReliefRenderer
      | RasterColormapRenderer
      | VectorFieldRenderer
      | FlowRenderer;

    readonly type: "wcs";
    /**
     * The URL of the WCS service endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#url)
     */
    url: string;
    /**
     * The version of Web Coverage Service currently being used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#version)
     */
    version: string;

    /**
     * Creates a default popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;

    static fromJSON(json: any): WCSLayer;
  }

  interface WCSLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      ImageryTileMixinProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties {
    /**
     * The coverage identifier for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#coverageId)
     */
    coverageId?: string;
    /**
     * Coverage information retrieved from the WCS Server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#coverageInfo)
     */
    coverageInfo?: CoverageInfo;
    /**
     * Use this property to append custom parameters to all WCS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * An array of raster fields in the layer that consists of service pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#rasterFields)
     */
    rasterFields?: FieldProperties[];
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#renderer)
     */
    renderer?:
      | (ClassBreaksRendererProperties & { type: "class-breaks" })
      | (UniqueValueRendererProperties & { type: "unique-value" })
      | (RasterStretchRendererProperties & { type: "raster-stretch" })
      | (RasterShadedReliefRendererProperties & { type: "raster-shaded-relief" })
      | (RasterColormapRendererProperties & { type: "raster-colormap" })
      | (VectorFieldRendererProperties & { type: "vector-field" })
      | (FlowRendererProperties & { type: "flow" });
    /**
     * The URL of the WCS service endpoint of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#url)
     */
    url?: string;
    /**
     * The version of Web Coverage Service currently being used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#version)
     */
    version?: string;
  }

  /**
   * Coverage description for WCS service version 1.0.0.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
   */
  export interface CoverageDescriptionV100 {
    /**
     * Coverage name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    name: string;
    /**
     * Coverage label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    label: string;
    /**
     * Coverage description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    description: string;
    /**
     * Formats supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    supportedFormats: string[];
    /**
     * Interpolations supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    supportedInterpolations: string[];
    /**
     * Coordinate reference systems supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    supportedCRSs: CoverageDescriptionV100SupportedCRSs;
    /**
     * Spatial extent of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    lonLatEnvelope: Extent;
    /**
     * Defines the properties (categories, measures, or values) assigned to each location in the domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    rangeSet: CoverageDescriptionV100RangeSet[];
    /**
     * Describes the spatial and temporal domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    domainSet: CoverageDescriptionV100DomainSet;
    /**
     * Coverage version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    version: "1.0";
  }

  /**
   * Coverage description for WCS service version 1.1.0.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
   */
  export interface CoverageDescriptionV110 {
    /**
     * Coverage title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    title: string;
    /**
     * Coverage abstract.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    abstract: string;
    /**
     * Coverage identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    identifier: string;
    /**
     * Formats supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    supportedFormats: string[];
    /**
     * Coordinate reference systems supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    supportedCRSs: string[];
    /**
     * Describes the spatial and temporal domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    domain: CoverageDescriptionV110Domain;
    /**
     * Defines the properties (categories, measures, or values) assigned to each location in the domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    range: CoverageDescriptionV110Range[];
    /**
     * Resolution x, y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    resolution: CoverageDescriptionV110Resolution;
    /**
     * Coverage metadata.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    metadata?: string;
    /**
     * Coverage version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    version: "1.1";
  }

  /**
   * Coverage description for WCS service version 2.0.1
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
   */
  export interface CoverageDescriptionV201 {
    /**
     * Coverage id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    coverageId: string;
    /**
     * Spatial and temporal extent of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    boundedBy: CoverageDescriptionV201BoundedBy;
    /**
     * Domains associated with a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    domainSet: CoverageDescriptionV201DomainSet;
    /**
     * Describes the semantics of the range set values supported by a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    rangeType: CoverageDescriptionV201RangeType[][];
    /**
     * Service parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    serviceParameters: any;
    /**
     * Service x,y resolution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    resolution: CoverageDescriptionV201Resolution;
    /**
     * Coverage function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    coverageFunction?: any;
    /**
     * Coverage extension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    extension?: string;
    /**
     * The WCS 2.0 Earth Observation coverage metadata.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    eoMetadata?: CoverageDescriptionV201EoMetadata;
    /**
     * Service version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    version: "2.0";
  }

  /**
   * Coverage information associated with a WCS service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
   */
  export interface CoverageInfo {
    /**
     * Service coverage id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    id: string;
    /**
     * Service coverage title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    title: string;
    /**
     * Service coverage description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    description: string;
    /**
     * Spatial extent of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    lonLatEnvelope?: Extent;
    /**
     * Coverage band names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    bandNames: string[];
    /**
     * Coverage raster info.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    rasterInfo: RasterInfo;
    /**
     * Formats supported by a service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    supportedFormats: string[];
    /**
     * Interpolation supported by a service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    supportedInterpolations?: ("nearest" | "bilinear" | "cubic" | "majority")[];
    /**
     * Coverage description for different versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    coverageDescription: CoverageDescriptionV100 | CoverageDescriptionV110 | CoverageDescriptionV201;
    /**
     * The service version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    version: "1.0.0" | "1.1.0" | "1.1.1" | "1.1.2" | "2.0.1";
    /**
     * Indicates if the EPSG axis is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageInfo)
     */
    useEPSGAxis: boolean;
  }

  /**
   * Temporal domain or extent of a coverage.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
   */
  export interface TemporalDomain {
    /**
     * Start date for the temporal domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
     */
    begin: Date;
    /**
     * End date for the temporal domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
     */
    end: Date;
    /**
     * Date values available in the temporal domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
     */
    values?: Date[];
    /**
     * The numerical value of the temporal domain or extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
     */
    resolution?: number;
    /**
     * Temporal units such as day, month, year etc.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#TemporalDomain)
     */
    units?: any;
  }

  export interface CoverageDescriptionV100DomainSet {
    /**
     * Spatial domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    spatialDomain: CoverageDescriptionV100DomainSetSpatialDomain;
    /**
     * Temporal domain or extent of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    temporalDomain?: TemporalDomain;
  }

  export interface CoverageDescriptionV100DomainSetSpatialDomain {
    /**
     * Spatial domain extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    envelope: Extent;
    /**
     * Spatial domain columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    columns: number;
    /**
     * Spatial domain rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    rows: number;
    /**
     * Spatial domain x, y offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    offset: CoverageDescriptionV100DomainSetSpatialDomainOffset;
    /**
     * Spatial domain x, y origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    origin: CoverageDescriptionV100DomainSetSpatialDomainOrigin;
  }

  export interface CoverageDescriptionV100DomainSetSpatialDomainOffset {
    /**
     * Spatial domain x offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    x: number;
    /**
     * Spatial domain y offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    y: any;
  }

  export interface CoverageDescriptionV100DomainSetSpatialDomainOrigin {
    /**
     * Spatial domain x origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    x: number;
    /**
     * Spatial domain y origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    y: any;
  }

  export interface CoverageDescriptionV100RangeSet {
    /**
     * Range set name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    name: string;
    /**
     * Range set label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    label: string;
    /**
     * The null values is used when valid values are not available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    nullValues?: number[];
    /**
     * Provides additional information on compound valued range set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    axis: CoverageDescriptionV100RangeSetAxis[];
  }

  export interface CoverageDescriptionV100RangeSetAxis {
    /**
     * Range axis name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    name: string;
    /**
     * Range axis label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    label: string;
    /**
     * Range axis values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    values: string[];
  }

  export interface CoverageDescriptionV100SupportedCRSs {
    /**
     * Coordinate reference systems in which the coverage can both accept GetCoverage requests and deliver coverage responses.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    requestResponseCRSs: string[];
    /**
     * The native coordinate reference system of a coverage – that is, the coordinate reference systems in which coverages can be obtained without any distortion or degradation of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV100)
     */
    nativeCRSs: string[];
  }

  export interface CoverageDescriptionV110Domain {
    /**
     * Spatial extent or domain of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    spatialDomain: CoverageDescriptionV110DomainSpatialDomain;
    /**
     * Temporal domain or extent of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    temporalDomain: TemporalDomain;
  }

  export interface CoverageDescriptionV110DomainSpatialDomain {
    /**
     * Spatial domain extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    envelope: Extent;
    /**
     * Spatial domain columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    columns: number;
    /**
     * Spatial domain rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    rows: number;
    /**
     * Spatial domain x, y offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    offset: CoverageDescriptionV110DomainSpatialDomainOffset;
    /**
     * Spatial domain x, y origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    origin: CoverageDescriptionV110DomainSpatialDomainOrigin;
    /**
     * Specifies spatial resolution of the coordinate reference system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    gridBaseCRS: string;
    /**
     * Indicates if the EPSG axis is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    useEPSGAxis: boolean;
  }

  export interface CoverageDescriptionV110DomainSpatialDomainOffset {
    /**
     * Spatial domain x offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    x: number;
    /**
     * Spatial domain y offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    y: any;
  }

  export interface CoverageDescriptionV110DomainSpatialDomainOrigin {
    /**
     * Spatial domain x origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    x: number;
    /**
     * Spatial domain y origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    y: any;
  }

  export interface CoverageDescriptionV110Range {
    /**
     * Range identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    identifier: string;
    /**
     * Range description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    description?: string;
    /**
     * Range definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    definition: string;
    /**
     * Range abstract.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    abstract?: string;
    /**
     * Range title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    title?: string;
    /**
     * Interpolations supported by the range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    supportedInterpolations: string[];
    /**
     * Null values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    nullValues: number[];
    /**
     * Provides additional information on compound valued range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    axis: CoverageDescriptionV110RangeAxis[];
  }

  export interface CoverageDescriptionV110RangeAxis {
    /**
     * The range axis identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    identifier: string;
    /**
     * The axis unit of measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    uom: string;
    /**
     * The axis data type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    dataType: string;
    /**
     * The axis values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    values: string[];
    /**
     * The axis bandNoDataValues.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    bandNoDataValues?: number[];
  }

  export interface CoverageDescriptionV110Resolution {
    /**
     * Resolution x.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    x: any;
    /**
     * Resolution y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV110)
     */
    y: any;
  }

  export interface CoverageDescriptionV201BoundedBy {
    /**
     * The spatial extent of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    envelope: Extent;
    /**
     * The coverage's axis name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    axisLabels: string[];
    /**
     * List of unit of measure (uom) labels for all the axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    uomLabels: string[];
    /**
     * The spatial extent dimensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    envelopeAllDims: CoverageDescriptionV201BoundedByEnvelopeAllDims;
    /**
     * Start date for the temporal extent of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    beginPosition?: Date;
    /**
     * End date for the temporal extent of the coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    endPosition?: Date;
    /**
     * Is east first.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    isEastFirst: boolean;
  }

  export interface CoverageDescriptionV201BoundedByEnvelopeAllDims {
    /**
     * Spatial extent min x and y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    mins: number[];
    /**
     * Spatial extent min x and y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    maxs: number[];
  }

  export interface CoverageDescriptionV201DomainSet {
    /**
     * Domain columns of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    columns: number;
    /**
     * Domain rows of a coverage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    rows: number;
    /**
     * Domain origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    origin: number[];
    /**
     * Domain offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    offset: number[];
    /**
     * Domain x, y resolution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    resolution: CoverageDescriptionV201DomainSetResolution;
    /**
     * Domain grid samples.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    gridSamples: number[];
    /**
     * Axis labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    axisLabels: string[];
    /**
     * Has same axis labels as bounded by.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    hasSameAxisLabelsAsBoundedBy: boolean;
  }

  export interface CoverageDescriptionV201DomainSetResolution {
    /**
     * Resolution x.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    x: number;
    /**
     * Resolution.y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    y: number;
  }

  export interface CoverageDescriptionV201EoMetadata {
    /**
     * Earth observation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    observation: CoverageDescriptionV201EoMetadataObservation;
  }

  export interface CoverageDescriptionV201EoMetadataObservation {
    /**
     * Earth observation phenomenon time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    phenomenonTime?: TemporalDomain;
    /**
     * Earth observation resultTime.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    resultTime?: TemporalDomain;
    /**
     * Earth observation footprint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    footprint?: Polygon;
    /**
     * Earth observation identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    identifier: string;
    /**
     * Earth observation acquisition type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    acquisitionType?: string;
    /**
     * Earth observation status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    status?: string;
  }

  export interface CoverageDescriptionV201RangeType {
    /**
     * Range type name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    name: string;
    /**
     * Range type description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    description?: string;
    /**
     * Unit of measurement for the range type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    uom: string;
    /**
     * Number that represents null value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    nilValue?: number;
    /**
     * Allowed values for the range type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    allowedValues?: number[];
  }

  export interface CoverageDescriptionV201Resolution {
    /**
     * Resolution x.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    x: number;
    /**
     * Resolution y.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html#CoverageDescriptionV201)
     */
    y: number;
  }

  export interface WebTileLayer
    extends Layer,
      ScaleRangeLayer,
      RefreshableLayer,
      PortalLayer,
      BlendLayer,
      OperationalLayer {}

  export class WebTileLayer {
    /**
     * WebTileLayer provides a simple way to add non-ArcGIS Server map tiles as a layer to a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html)
     */

    constructor(properties?: WebTileLayerProperties);

    /**
     * The attribution information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#copyright)
     */
    copyright: string;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WebMercator // wkid: 3857
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * A string of subDomain names where tiles are served to speed up tile retrieval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#subDomains)
     */
    subDomains: string[];
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#tileInfo)
     */
    tileInfo: TileInfo;
    /**
     * The list of tile server urls for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#tileServers)
     */
    readonly tileServers: string[];
    /**
     * For [WebTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html) the type is `web-tile`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#type)
     */
    readonly type: "web-tile" | "open-street-map";
    /**
     * The url template is a string that specifies the URL of the hosted tile images to load.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#urlTemplate)
     */
    urlTemplate: string;

    /**
     * This method fetches a tile for the given level, row and column present in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#fetchTile)
     */
    fetchTile(
      level: number,
      row: number,
      col: number,
      options?: WebTileLayerFetchTileOptions,
    ): Promise<HTMLImageElement>;
    /**
     * This method returns a URL to a tile for a given level, row and column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#getTileUrl)
     */
    getTileUrl(level: number, row: number, col: number): string;

    on(name: "refresh", eventHandler: WebTileLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: WebTileLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: WebTileLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: WebTileLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): WebTileLayer;
  }

  interface WebTileLayerProperties
    extends LayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      PortalLayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties {
    /**
     * The attribution information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#copyright)
     */
    copyright?: string;
    /**
     * A string of subDomain names where tiles are served to speed up tile retrieval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#subDomains)
     */
    subDomains?: string[];
    /**
     * The tiling scheme information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#tileInfo)
     */
    tileInfo?: TileInfoProperties;
    /**
     * The url template is a string that specifies the URL of the hosted tile images to load.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#urlTemplate)
     */
    urlTemplate?: string;
  }

  export interface WebTileLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface WebTileLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WebTileLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WebTileLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface WebTileLayerFetchTileOptions {
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WebTileLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface WFSLayer
    extends Layer,
      OrderedLayer,
      ScaleRangeLayer,
      RefreshableLayer,
      BlendLayer,
      OperationalLayer,
      PortalLayer,
      FeatureEffectLayer,
      FeatureReductionLayer {}

  export class WFSLayer {
    /**
     * The WFSLayer is used to create a layer based on an [OGC Web Feature Service](https://www.ogc.org/standards/wfs) (WFS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html)
     */

    constructor(properties?: WFSLayerProperties);

    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    readonly capabilities: WFSLayerCapabilities;
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#copyright)
     */
    copyright: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The time zone that dates are stored in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#dateFieldsTimeZone)
     */
    readonly dateFieldsTimeZone: string;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#definitionExpression)
     */
    definitionExpression: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#displayField)
     */
    displayField: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    elevationInfo: WFSLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fields)
     */
    fields: Field[];
    /**
     * A convenient property that can be used to make case-insensitive lookups for a field by name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fieldsIndex)
     */
    readonly fieldsIndex: FieldsIndex;
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#geometryType)
     */
    geometryType: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#labelingInfo)
     */
    labelingInfo: LabelClass[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#labelsVisible)
     */
    labelsVisible: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The maximum number of queries allowed to fetch the whole dataset from the service.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#maxPageCount)
     */
    maxPageCount: number;
    /**
     * The maximum number of features that can be returned in a single request.
     *
     * @default 3000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#maxRecordCount)
     */
    maxRecordCount: number;
    /**
     * The name of the layer in the WFS service to display, excluding the namespace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#name)
     */
    name: string;
    /**
     * The namespace URI for the layer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#namespaceUri)
     */
    namespaceUri: string;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#objectIdField)
     */
    objectIdField: string;
    /**
     * An array of field names from the WFS layer to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#renderer)
     */
    renderer: Renderer;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#title)
     */
    title: string;

    readonly type: "wfs";
    /**
     * The url to the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#url)
     */
    url: string;
    /**
     * WFS service information about the available layers and operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#wfsCapabilities)
     */
    wfsCapabilities: WFSCapabilities;

    /**
     * Creates a popup template for the layer, populated with all the fields of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#createPopupTemplate)
     */
    createPopupTemplate(options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates query parameter object that can be used to fetch features that satisfy the layer's configurations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Returns the [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) instance for a field name (case-insensitive).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#getField)
     */
    getField(fieldName: string): Field;
    /**
     * Returns the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) associated with the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#getFieldDomain)
     */
    getFieldDomain(fieldName: string, options?: WFSLayerGetFieldDomainOptions): Domain;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: WFSLayerQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryFeatureCount)
     */
    queryFeatureCount(query?: Query | QueryProperties, options?: WFSLayerQueryFeatureCountOptions): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html), which can be accessed using the `.then()` method once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: WFSLayerQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer and returns an array of Object IDs for features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: WFSLayerQueryObjectIdsOptions): Promise<number[]>;

    on(name: "refresh", eventHandler: WFSLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: WFSLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: WFSLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: WFSLayerLayerviewDestroyEventHandler): IHandle;

    /**
     * Creates a WFSLayer from an object created by [wfsUtils.getWFSLayerInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ogc-wfsUtils.html#getWFSLayerInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fromWFSLayerInfo)
     */
    static fromWFSLayerInfo(layerInfo: WFSLayerInfo): WFSLayer;
    static fromJSON(json: any): WFSLayer;
  }

  interface WFSLayerProperties
    extends LayerProperties,
      OrderedLayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      FeatureEffectLayerProperties,
      FeatureReductionLayerProperties {
    /**
     * Copyright information for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#copyright)
     */
    copyright?: string;
    /**
     * A list of custom parameters appended to the URL of all resources fetched by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The SQL where clause used to filter features on the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#definitionExpression)
     */
    definitionExpression?: string;
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#displayField)
     */
    displayField?: string;
    /**
     * Specifies how features are placed on the vertical axis (z).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    elevationInfo?: WFSLayerElevationInfo;
    /**
     * An array of fields in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#geometryType)
     */
    geometryType?: "point" | "polygon" | "polyline" | "multipoint";
    /**
     * The label definition for this layer, specified as an array of [LabelClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LabelClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#labelingInfo)
     */
    labelingInfo?: LabelClassProperties[];
    /**
     * Indicates whether to display labels for this layer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#labelsVisible)
     */
    labelsVisible?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The maximum number of queries allowed to fetch the whole dataset from the service.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#maxPageCount)
     */
    maxPageCount?: number;
    /**
     * The maximum number of features that can be returned in a single request.
     *
     * @default 3000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#maxRecordCount)
     */
    maxRecordCount?: number;
    /**
     * The name of the layer in the WFS service to display, excluding the namespace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#name)
     */
    name?: string;
    /**
     * The namespace URI for the layer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#namespaceUri)
     */
    namespaceUri?: string;
    /**
     * The name of an `oid` [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#fields) containing a unique value or identifier for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#objectIdField)
     */
    objectIdField?: string;
    /**
     * An array of field names from the WFS layer to include with each feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether to display popups when features in the layer are clicked.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The renderer assigned to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#renderer)
     */
    renderer?: RendererProperties;
    /**
     * Apply perspective scaling to screen-size point symbols in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#screenSizePerspectiveEnabled)
     */
    screenSizePerspectiveEnabled?: boolean;
    /**
     * The spatial reference of the layer.
     *
     * @default SpatialReference.WGS84
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The title of the layer used to identify it in places such as the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) and [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#title)
     */
    title?: string;
    /**
     * The url to the WFS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#url)
     */
    url?: string;
    /**
     * WFS service information about the available layers and operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#wfsCapabilities)
     */
    wfsCapabilities?: WFSCapabilities;
  }

  export interface WFSLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface WFSLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WFSLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WFSLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface WFSLayerCapabilities {
    /**
     * Describes characteristics of the data in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    data: WFSLayerCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    editing: WFSLayerCapabilitiesEditing;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    operations: WFSLayerCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    query: WFSLayerCapabilitiesQuery;
    /**
     * Indicates if the layer's query operation supports querying features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    queryRelated: WFSLayerCapabilitiesQueryRelated;
  }

  export interface WFSLayerCapabilitiesData {
    /**
     * Indicates if the attachment is enabled on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsAttachment: boolean;
    /**
     * Indicates if the features in the layer support m-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsM: boolean;
    /**
     * Indicates if the features in the layer support z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsZ: boolean;
  }

  export interface WFSLayerCapabilitiesEditing {
    /**
     * Indicates if anonymous users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDeleteByAnonymous: boolean;
    /**
     * Indicates if logged in users can delete features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDeleteByOthers: boolean;
    /**
     * Indicates if the geometry of the features in the layer can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsGeometryUpdate: boolean;
    /**
     * Indicates if the `globalid` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates if the `rollbackOnFailure` parameter can be set to `true` or `false` when running the synchronizeReplica operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsRollbackOnFailure: boolean;
    /**
     * Indicates if anonymous users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsUpdateByAnonymous: boolean;
    /**
     * Indicates if logged in users can update features created by others.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsUpdateByOthers: boolean;
    /**
     * Indicates if `m-values` must be provided when updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsUpdateWithoutM: boolean;
    /**
     * Indicates if the layer supports uploading attachments by [UploadId](https://developers.arcgis.com/rest/services-reference/item.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsUploadWithItemId: boolean;
  }

  export interface WFSLayerCapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#applyEdits) to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#applyEdits) from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the layer can be [updated](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsUpdate: boolean;
    /**
     * Indicates if features in the layer can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if values of one or more field values in the layer can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsCalculate: boolean;
    /**
     * Indicates if features in the layer can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates if the layer supports [REST API queryAttachments](https://developers.arcgis.com/rest/services-reference/query-attachments-feature-service-layer-.htm) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsQueryAttachments: boolean;
    /**
     * Indicates if the layer supports a SQL-92 expression or where clause.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsValidateSql: boolean;
    /**
     * Indicates if resized attachments are supported in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsResizeAttachments: boolean;
  }

  export interface WFSLayerCapabilitiesQuery {
    /**
     * Indicates if the geometry centroid associated with each polygon feature can be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsCentroid: boolean;
    /**
     * Indicates if the layer's query operation supports a buffer distance for input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDistance: boolean;
    /**
     * Indicates if the layer supports queries for distinct values based on fields specified in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDistinct: boolean;
    /**
     * Indicates if the query operation supports `disjoint` [spatial relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#spatialRelationship).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsDisjointSpatialRelationship: boolean;
    /**
     * Indicates if the layer's query response includes the extent of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsExtent: boolean;
    /**
     * Indicates if the layer's query response contains geometry attributes, including shape area and length attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsGeometryProperties: boolean;
    /**
     * Indicates if the layer supports the [having](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having) clause on the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsHavingClause: boolean;
    /**
     * Indicates if features returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsPagination: boolean;
    /**
     * Indicates if the layer supports [percentile statisticType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsPercentileStatistics: boolean;
    /**
     * Indicates if the query response includes the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsQueryGeometry: boolean;
    /**
     * Indicates if the query operation supports the projection of geometries onto a virtual grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsQuantization: boolean;
    /**
     * Indicates if the query operation supports quantization designed to be used in edit mode (highest resolution at the given spatial reference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsQuantizationEditMode: boolean;
    /**
     * Indicates if the number of features returned by the query operation can be controlled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsResultType: boolean;
    /**
     * Indicates if the query operation supports SQL expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsSqlExpression: boolean;
    /**
     * Indicates if the query operation supports using standardized queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsStandardizedQueriesOnly: boolean;
    /**
     * Indicates if the layer supports field-based statistical functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsStatistics: boolean;
    /**
     * Indicates if the layer supports historic moment query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsHistoricMoment: boolean;
  }

  export interface WFSLayerCapabilitiesQueryRelated {
    /**
     * Indicates if the layer's query response includes the number of features or records related to features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsCount: boolean;
    /**
     * Indicates if the related features or records returned in the query response can be ordered by one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsOrderBy: boolean;
    /**
     * Indicates if the query response supports pagination for related features or records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#capabilities)
     */
    supportsPagination: boolean;
  }

  export interface WFSLayerElevationInfo {
    /**
     * Defines how the feature is placed with respect to the terrain surface or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    mode: "on-the-ground" | "relative-to-ground" | "absolute-height" | "relative-to-scene";
    /**
     * An elevation offset, which is added to the vertical position of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    offset?: number;
    /**
     * Defines how to override a feature's Z-value based on its attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    featureExpressionInfo?: WFSLayerElevationInfoFeatureExpressionInfo;
    /**
     * The unit for `featureExpressionInfo` and `offset` values.
     *
     * [Read more...](global.html#unit)
     */
    unit?: ElevationUnit;
  }

  export interface WFSLayerElevationInfoFeatureExpressionInfo {
    /**
     * Title of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Feature Z Profile](https://developers.arcgis.com/javascript/latest/arcade/#z-values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#elevationInfo)
     */
    expression?: string;
  }

  export interface WFSLayerGetFieldDomainOptions {
    /**
     * The feature to which the [Domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Domain.html) is assigned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#getFieldDomain)
     */
    feature: Graphic;
  }

  export interface WFSLayerQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface WFSLayerQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface WFSLayerQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface WFSLayerQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface WMSLayer
    extends Layer,
      BlendLayer,
      OperationalLayer,
      PortalLayer,
      ScaleRangeLayer,
      RefreshableLayer {}

  export class WMSLayer {
    /**
     * The WMSLayer is used to create layers based on OGC Web Map Services (WMS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html)
     */

    constructor(properties?: WMSLayerProperties);

    /**
     * A flattened collection of all [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s based on the [`sublayers`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#sublayers) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#allSublayers)
     */
    allSublayers: Collection<WMSSublayer>;
    /**
     * Copyright information for the WMS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#copyright)
     */
    copyright: string;
    /**
     * Use this to append different custom parameters to the WMS map requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#customLayerParameters)
     */
    customLayerParameters: any;
    /**
     * Use this to append custom parameters to all WMS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * Description for the WMS layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#description)
     */
    description: string;
    /**
     * An array of time, elevation and other dimensions for the root layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#dimensions)
     */
    readonly dimensions: (TimeDimension | ElevationDimension | GenericDimension)[];
    /**
     * The MIME type that will be requested by popups.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#featureInfoFormat)
     */
    featureInfoFormat: "text/html" | "text/plain";
    /**
     * This property lists all available MIME-types that can be used with the WMS service's _GetFeatureInfo_ request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#featureInfoFormats)
     */
    readonly featureInfoFormats: string[];
    /**
     * The URL for the WMS GetFeatureInfo call.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#featureInfoUrl)
     */
    featureInfoUrl: string;
    /**
     * Function to override the default popup behavior of `WMSLayer`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchFeatureInfoFunction)
     */
    fetchFeatureInfoFunction: FetchFeatureInfoFunction;
    /**
     * All bounding boxes defined for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fullExtents)
     */
    fullExtents: Extent[];
    /**
     * The map image format (MIME type) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageFormat)
     */
    imageFormat: string;
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageMaxHeight)
     */
    imageMaxHeight: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageMaxWidth)
     */
    imageMaxWidth: number;
    /**
     * Indicates whether the background of the image exported by the service is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageTransparency)
     */
    imageTransparency: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * List of spatialReference well known ids derived from the CRS elements of the first layer in the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#spatialReferences)
     */
    spatialReferences: number[];
    /**
     * A subset of the layer's [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s that will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#sublayers)
     */
    sublayers: Collection<WMSSublayer>;
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeInfo)
     */
    timeInfo: TimeInfo;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeOffset)
     */
    timeOffset: TimeInterval;

    readonly type: "wms";
    /**
     * The URL of the WMS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#url)
     */
    url: string;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#useViewTime)
     */
    useViewTime: boolean;
    /**
     * Version of the [WMS specification](http://www.opengeospatial.org/standards/wms) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#version)
     */
    version: string;

    /**
     * Fetching the WMS image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchImage)
     */
    fetchImage(extent: Extent, width: number, height: number, options?: WMSLayerFetchImageOptions): Promise<any>;
    /**
     * Returns a [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html) based on the given sublayer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#findSublayerById)
     */
    findSublayerById(id: number): WMSSublayer;
    /**
     * Returns a [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html) based on the given sublayer name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#findSublayerByName)
     */
    findSublayerByName(name: string): WMSSublayer;

    on(name: "refresh", eventHandler: WMSLayerRefreshEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: WMSLayerLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: WMSLayerLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: WMSLayerLayerviewDestroyEventHandler): IHandle;

    static fromJSON(json: any): WMSLayer;
  }

  interface WMSLayerProperties
    extends LayerProperties,
      BlendLayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties {
    /**
     * A flattened collection of all [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s based on the [`sublayers`](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#sublayers) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#allSublayers)
     */
    allSublayers?: CollectionProperties<WMSSublayerProperties>;
    /**
     * Copyright information for the WMS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#copyright)
     */
    copyright?: string;
    /**
     * Use this to append different custom parameters to the WMS map requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#customLayerParameters)
     */
    customLayerParameters?: any;
    /**
     * Use this to append custom parameters to all WMS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * Description for the WMS layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#description)
     */
    description?: string;
    /**
     * The MIME type that will be requested by popups.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#featureInfoFormat)
     */
    featureInfoFormat?: "text/html" | "text/plain";
    /**
     * The URL for the WMS GetFeatureInfo call.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#featureInfoUrl)
     */
    featureInfoUrl?: string;
    /**
     * Function to override the default popup behavior of `WMSLayer`.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchFeatureInfoFunction)
     */
    fetchFeatureInfoFunction?: FetchFeatureInfoFunction;
    /**
     * All bounding boxes defined for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fullExtents)
     */
    fullExtents?: ExtentProperties[];
    /**
     * The map image format (MIME type) to request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageFormat)
     */
    imageFormat?: string;
    /**
     * Indicates the maximum height of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageMaxHeight)
     */
    imageMaxHeight?: number;
    /**
     * Indicates the maximum width of the image exported by the service.
     *
     * @default 2048
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageMaxWidth)
     */
    imageMaxWidth?: number;
    /**
     * Indicates whether the background of the image exported by the service is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#imageTransparency)
     */
    imageTransparency?: boolean;
    /**
     * Indicates whether the layer will be included in the legend.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * The spatial reference of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * List of spatialReference well known ids derived from the CRS elements of the first layer in the GetCapabilities request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#spatialReferences)
     */
    spatialReferences?: number[];
    /**
     * A subset of the layer's [WMSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMSSublayer.html)s that will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#sublayers)
     */
    sublayers?: CollectionProperties<WMSSublayerProperties>;
    /**
     * The layer's time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * TimeInfo provides information such as date fields that store [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#startField) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#endField) time for each feature and the [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#fullTimeExtent) for the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeInfo)
     */
    timeInfo?: TimeInfoProperties;
    /**
     * A temporary offset of the time data based on a certain [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#timeOffset)
     */
    timeOffset?: TimeIntervalProperties;
    /**
     * The URL of the WMS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#url)
     */
    url?: string;
    /**
     * Determines if the layer will update its temporal data based on the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#useViewTime)
     */
    useViewTime?: boolean;
    /**
     * Version of the [WMS specification](http://www.opengeospatial.org/standards/wms) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#version)
     */
    version?: string;
  }

  export type FetchFeatureInfoFunction = (query: any) => Promise<Graphic[]>;

  export interface WMSLayerLayerviewCreateErrorEvent {
    error: Error;

    view: View;
  }

  export interface WMSLayerLayerviewCreateEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WMSLayerLayerviewDestroyEvent {
    layerView: LayerView;

    view: View;
  }

  export interface WMSLayerRefreshEvent {
    dataChanged: boolean;
  }

  export interface WMSLayerFetchImageOptions {
    /**
     * The ratio of the resolution in physical pixels of the image to the resolution it will be displayed at.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchImage)
     */
    pixelRatio?: number;
    /**
     * The rotation in degrees of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchImage)
     */
    rotation?: number;
    /**
     * The [TimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) of the exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchImage)
     */
    timeExent?: any;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html#fetchImage)
     */
    signal?: AbortSignal;
  }

  export interface WMTSLayer
    extends Layer,
      OperationalLayer,
      PortalLayer,
      ScaleRangeLayer,
      RefreshableLayer,
      BlendLayer {}

  export class WMTSLayer {
    /**
     * The WMTSLayer is used to create layers based on OGC Web Map Tile Services (WMTS).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html)
     */

    constructor(properties?: WMTSLayerProperties);

    /**
     * Currently active sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#activeLayer)
     */
    activeLayer: WMTSSublayer;
    /**
     * Copyright information for the WMTS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#copyright)
     */
    copyright: string;
    /**
     * Use this to append different custom parameters to the WMTS tile requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#customLayerParameters)
     */
    customLayerParameters: any;
    /**
     * Use this to append custom parameters to all WMTS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#customParameters)
     */
    customParameters: any;
    /**
     * The service mode for the WMTS layer.
     *
     * @default "RESTful"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#serviceMode)
     */
    serviceMode: "RESTful" | "KVP";
    /**
     * A collection of [WMTSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#sublayers)
     */
    sublayers: Collection<WMTSSublayer>;

    readonly type: "wmts";
    /**
     * The URL of the WMTS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#url)
     */
    url: string;
    /**
     * Version of the [WMTS specification](http://www.opengeospatial.org/standards/wmts) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#version)
     */
    version: string;

    /**
     * Requests a tile from the WMTS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#fetchTile)
     */
    fetchTile(level: number, row: number, col: number, options?: WMTSLayerFetchTileOptions): Promise<HTMLImageElement>;
    /**
     * Returns a [WMTSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html) based on the given sublayer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#findSublayerById)
     */
    findSublayerById(id: string): WMTSSublayer;

    static fromJSON(json: any): WMTSLayer;
  }

  interface WMTSLayerProperties
    extends LayerProperties,
      OperationalLayerProperties,
      PortalLayerProperties,
      ScaleRangeLayerProperties,
      RefreshableLayerProperties,
      BlendLayerProperties {
    /**
     * Currently active sublayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#activeLayer)
     */
    activeLayer?: WMTSSublayerProperties;
    /**
     * Copyright information for the WMTS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#copyright)
     */
    copyright?: string;
    /**
     * Use this to append different custom parameters to the WMTS tile requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#customLayerParameters)
     */
    customLayerParameters?: any;
    /**
     * Use this to append custom parameters to all WMTS requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#customParameters)
     */
    customParameters?: any;
    /**
     * The service mode for the WMTS layer.
     *
     * @default "RESTful"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#serviceMode)
     */
    serviceMode?: "RESTful" | "KVP";
    /**
     * A collection of [WMTSSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-WMTSSublayer.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#sublayers)
     */
    sublayers?: CollectionProperties<WMTSSublayerProperties>;
    /**
     * The URL of the WMTS service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#url)
     */
    url?: string;
    /**
     * Version of the [WMTS specification](http://www.opengeospatial.org/standards/wmts) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#version)
     */
    version?: string;
  }

  export interface WMTSLayerFetchTileOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMTSLayer.html#fetchTile)
     */
    signal?: AbortSignal;
  }

  export interface Map extends Accessor, LayersMixin, TablesMixin {}

  export class Map {
    /**
     * The Map class contains properties and methods for storing, managing, and overlaying [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) common to both 2D and 3D viewing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html)
     */

    constructor(properties?: MapProperties);

    /**
     * A flattened collection of all [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#allLayers)
     */
    readonly allLayers: Collection<Layer>;
    /**
     * A flattened collection of tables anywhere in the map's hierarchy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#allTables)
     */
    readonly allTables: Collection<Layer>;
    /**
     * Specifies a basemap for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap)
     */
    basemap: Basemap;
    /**
     * A collection of editable layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#editableLayers)
     */
    readonly editableLayers: Collection<
      FeatureLayer | SceneLayer | SubtypeGroupLayer | GeoJSONLayer | OrientedImageryLayer
    >;
    /**
     * Specifies the surface properties for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#ground)
     */
    ground: Ground;

    /**
     * Destroys the map, and any associated resources, including its [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers), [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap), [tables](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#tables),  and [ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#ground).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#destroy)
     */
    destroy(): void;
  }

  interface MapProperties extends LayersMixinProperties, TablesMixinProperties {
    /**
     * Specifies a basemap for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap)
     */
    basemap?: BasemapProperties | string;
    /**
     * Specifies the surface properties for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#ground)
     */
    ground?: GroundProperties | string;
  }

  export interface Network extends Accessor, JSONSupport, Loadable {}

  export class Network {
    /**
     * Class defining high level properties that describes utility networks and trace networks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html)
     */

    constructor(properties?: NetworkProperties);

    /**
     * The full network definition, accessible only when the network is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#dataElement)
     */
    dataElement: any;
    /**
     * The physical dataset name of the network as defined in the backend database.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#datasetName)
     */
    readonly datasetName: string;
    /**
     * Returns the root feature service url which this network is part of.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#featureServiceUrl)
     */
    readonly featureServiceUrl: string;
    /**
     * The full extent of the network, defined from the service territory used to create the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#fullExtent)
     */
    fullExtent: Extent;
    /**
     * The version of the geodatabase of the feature service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#historicMoment)
     */
    historicMoment: Date;
    /**
     * Random unique id (UUID) to identify a network as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#id)
     */
    id: string;
    /**
     * The layer id of the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#layerId)
     */
    readonly layerId: number;
    /**
     * The full url to the network layer id as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#layerUrl)
     */
    layerUrl: string;
    /**
     * Indicates whether the network instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * Returns the url of network server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#networkServiceUrl)
     */
    readonly networkServiceUrl: string;
    /**
     * Contains the url and IDs of the utility network rules, subnetworks, and dirty areas tables or layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#networkSystemLayers)
     */
    readonly networkSystemLayers: NetworkSystemLayers;
    /**
     * The portal user owner of the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#owner)
     */
    readonly owner: string;
    /**
     * Converts url to a url object
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#parsedUrl)
     */
    readonly parsedUrl: NetworkParsedUrl;
    /**
     * The schema version of the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#schemaGeneration)
     */
    readonly schemaGeneration: number;

    sourceJSON: any;
    /**
     * The spatial reference of the network, defined at the creation of the network, usually from the service territory class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The name of the network as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#title)
     */
    title: string;
    /**
     * The type of the dataset.
     *
     * @default "utility"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#type)
     */
    readonly type: "utility" | "trace";

    /**
     * The network consists of sources (classes) and each source has a unique Id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#getLayerIdBySourceId)
     */
    getLayerIdBySourceId(id: number): number;
    /**
     * Groups network elements by their layerId.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#getObjectIdsFromElements)
     */
    getObjectIdsFromElements(elements: NetworkElement[]): LayerInfo[];
    /**
     * Triggers the loading of the network instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#load)
     */
    load(): Promise<any>;
    /**
     * Named trace configurations allow you to add and store complex traces in a network that can be shared across an organization through web maps and consumed by web and field applications.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#queryNamedTraceConfigurations)
     */
    queryNamedTraceConfigurations(
      query?: NetworkQueryNamedTraceConfigurationsQuery,
      options?: RequestOptions,
    ): Promise<NamedTraceConfiguration[]>;
    /**
     * Whenever the network is edited or modified, the network and its features become out of date in the [network topology](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/about-network-topology.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob)
     */
    submitTopologyValidationJob(
      parameters: ValidateTopologyProps,
      options?: RequestOptions,
    ): Promise<TopologyValidationJobInfo>;
    /**
     * Whenever the network is edited or modified, the network and its features become out of date in the [network topology]( https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/about-network-topology.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#validateTopology)
     */
    validateTopology(
      parameters: ValidateTopologyProps,
      options?: RequestOptions,
    ): Promise<ValidateNetworkTopologyResult>;

    /**
     * Creates a new network instance from an ArcGIS Online or ArcGIS Enterprise [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#fromPortalItem)
     */
    static fromPortalItem(params: NetworkFromPortalItemParams): Promise<UtilityNetwork>;
    static fromJSON(json: any): Network;
  }

  interface NetworkProperties extends LoadableProperties {
    /**
     * The full network definition, accessible only when the network is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#dataElement)
     */
    dataElement?: any;
    /**
     * The full extent of the network, defined from the service territory used to create the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#fullExtent)
     */
    fullExtent?: ExtentProperties;
    /**
     * The version of the geodatabase of the feature service data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#historicMoment)
     */
    historicMoment?: DateProperties;
    /**
     * Random unique id (UUID) to identify a network as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#id)
     */
    id?: string;
    /**
     * The full url to the network layer id as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#layerUrl)
     */
    layerUrl?: string;

    sourceJSON?: any;
    /**
     * The spatial reference of the network, defined at the creation of the network, usually from the service territory class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The name of the network as defined in the webmap spec.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#title)
     */
    title?: string;
  }

  /**
   * Contains layerUrl, objectIds, and outFields.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#LayerInfo)
   */
  export interface LayerInfo {
    /**
     * The layerUrl of the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#LayerInfo)
     */
    layerUrl: string;
    /**
     * The objectIds of the elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#LayerInfo)
     */
    objectIds: number[];
    /**
     * The outFields of a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#LayerInfo)
     */
    outFields?: string;
  }

  export interface NetworkFromPortalItemParams {
    /**
     * The object representing an ArcGIS Online or ArcGIS Enterprise portal item from which to load the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#fromPortalItem)
     */
    portalItem: PortalItem;
  }

  export interface NetworkParsedUrl {
    /**
     * the url path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#parsedUrl)
     */
    path: string;
    /**
     * the url query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#parsedUrl)
     */
    query?: HashMap<string>;
    /**
     * the url hash.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#parsedUrl)
     */
    hash?: string;
  }

  export interface NetworkQueryNamedTraceConfigurationsQuery {
    /**
     * An array of the named trace configuration creators to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#queryNamedTraceConfigurations)
     */
    creators?: string[];
    /**
     * An array of named trace configuration globalIds (UUID) to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#queryNamedTraceConfigurations)
     */
    globalIds?: string[];
    /**
     * An array of named trace configuration names to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#queryNamedTraceConfigurations)
     */
    names?: string[];
    /**
     * An array of named trace configuration user tags to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#queryNamedTraceConfigurations)
     */
    tags?: string[];
  }

  /**
   * ValidateTopologyProps represents the parameters for validating a network topology.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
   */
  export interface ValidateTopologyProps {
    /**
     * Specifies the geodatabase version name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
     */
    gdbVersion?: string;
    /**
     * Specifies the GUID used to lock the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
     */
    sessionID?: string;
    /**
     * Specifies the envelope of the area to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
     */
    validateArea: Extent;
    /**
     * Specifies the set of features and objects to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
     */
    validationSet?: ValidationSetItemJSON[];
    /**
     * Specifies the validation to perform.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#ValidateTopologyProps)
     */
    validationType?: "normal" | "rebuild" | "force-rebuild";
  }

  export class NamedTraceConfiguration {
    /**
     * The date/time when the trace configuration has been added to the utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#creationDate)
     */
    creationDate: Date;
    /**
     * The portal user who created the trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#creator)
     */
    creator: string;
    /**
     * Short description of what kind of trace this trace configuration performs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#description)
     */
    description: string;
    /**
     * The `globalId` (UUID) uniquely identifies a trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#globalId)
     */
    globalId: string;
    /**
     * The minimum number of starting points required to perform a trace with this particular trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#minStartingPoints)
     */
    minStartingPoints: "none" | "one" | "many";
    /**
     * The result types of the trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#resultTypes)
     */
    resultTypes: any[];
    /**
     * Labels that help identify and search for a particular trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#tags)
     */
    tags: string[];
    /**
     * The title or the name of the trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#title)
     */
    title: string;
    /**
     * The full definition of the trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#traceConfiguration)
     */
    traceConfiguration: TraceConfiguration | UNTraceConfiguration;
    /**
     * The trace type defined in this trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NamedTraceConfiguration.html#traceType)
     */
    traceType:
      | "connected"
      | "upstream"
      | "downstream"
      | "shortest-path"
      | "subnetwork"
      | "subnetwork-controller"
      | "loops"
      | "isolation";

    constructor(properties?: any);
  }

  export class NetworkSystemLayers extends Accessor {
    /**
     * The NetworkSystemLayers contains the url and IDs of the utility network rules, subnetworks, and dirty areas tables or layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html)
     */

    constructor(properties?: NetworkSystemLayersProperties);

    readonly associationsTableId: number;

    readonly associationsTableUrl: string;
    /**
     * The layer ID of the service containing the utility network's [dirty areas](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/dirty-areas-in-a-utility-network.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#dirtyAreasLayerId)
     */
    readonly dirtyAreasLayerId: number;
    /**
     * The service url containing the utility network's [dirty areas](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/dirty-areas-in-a-utility-network.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#dirtyAreasLayerUrl)
     */
    readonly dirtyAreasLayerUrl: string;
    /**
     * The layer ID of the service containing the network rules table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#rulesTableId)
     */
    readonly rulesTableId: number;
    /**
     * The service url containing the network rules table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#rulesTableUrl)
     */
    readonly rulesTableUrl: string;
    /**
     * The layer ID of the service containing the utility network's [Subnetworks table](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/subnetworks-table.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#subnetworksTableId)
     */
    readonly subnetworksTableId: number;
    /**
     * The service url containing the utility network's [Subnetworks table](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/subnetworks-table.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-NetworkSystemLayers.html#subnetworksTableUrl)
     */
    readonly subnetworksTableUrl: string;
  }

  interface NetworkSystemLayersProperties {}

  export interface Terminal extends Accessor, JSONSupport {}

  export class Terminal {
    /**
     * A device feature can be assigned a terminal configuration, which could have one or more terminals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html)
     */

    constructor(properties?: TerminalProperties);

    /**
     * A unique numeric identifier for the terminal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html#id)
     */
    id: number;
    /**
     * The name of the terminal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html#name)
     */
    name: string;

    static fromJSON(json: any): Terminal;
  }

  interface TerminalProperties {
    /**
     * A unique numeric identifier for the terminal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html#id)
     */
    id?: number;
    /**
     * The name of the terminal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html#name)
     */
    name?: string;
  }

  export interface TerminalConfiguration extends Accessor, JSONSupport {}

  export class TerminalConfiguration {
    /**
     * A device feature can be assigned a terminal configuration which could have one or more terminals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html)
     */

    constructor(properties?: TerminalConfigurationProperties);

    /**
     * The default terminal configuration path defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#defaultConfiguration)
     */
    defaultConfiguration: string;
    /**
     * A unique numeric identifier of the terminal configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#id)
     */
    id: number;
    /**
     * The name of the terminal configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#name)
     */
    name: string;
    /**
     * An array of [terminals](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html) defined in this configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#terminals)
     */
    terminals: Terminal[];
    /**
     * Terminal configurations can be defined as directional or bidirectional.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#traversabilityModel)
     */
    traversabilityModel: "directional" | "bidirectional";

    static fromJSON(json: any): TerminalConfiguration;
  }

  interface TerminalConfigurationProperties {
    /**
     * The default terminal configuration path defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#defaultConfiguration)
     */
    defaultConfiguration?: string;
    /**
     * A unique numeric identifier of the terminal configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#id)
     */
    id?: number;
    /**
     * The name of the terminal configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#name)
     */
    name?: string;
    /**
     * An array of [terminals](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html) defined in this configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#terminals)
     */
    terminals?: TerminalProperties[];
    /**
     * Terminal configurations can be defined as directional or bidirectional.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TerminalConfiguration.html#traversabilityModel)
     */
    traversabilityModel?: "directional" | "bidirectional";
  }

  export interface TopologyValidationJobInfo extends Accessor, JSONSupport {}

  export class TopologyValidationJobInfo {
    /**
     * Represents information pertaining to the execution of an asynchronous [submitTopologyValidationJob()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob) request on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html)
     */

    constructor(properties?: TopologyValidationJobInfoProperties);

    /**
     * The last date and time the service was updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#lastUpdatedTime)
     */
    lastUpdatedTime: Date;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#status)
     */
    status: "job-waiting" | "job-executing" | "job-succeeded";
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the validate network topology request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#statusUrl)
     */
    statusUrl: string;
    /**
     * The date and time in which [submitTopologyValidationJob()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob) is first called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#submissionTime)
     */
    submissionTime: Date;

    /**
     * Sends a request for the current state of this job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#checkJobStatus)
     */
    checkJobStatus(requestOptions?: any): Promise<TopologyValidationJobInfo>;
    /**
     * Stop monitoring this job for status updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#destroy)
     */
    destroy(): void;
    /**
     * Resolves when an asynchronous job has completed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#waitForJobCompletion)
     */
    waitForJobCompletion(
      options?: TopologyValidationJobInfoWaitForJobCompletionOptions,
    ): Promise<TopologyValidationJobInfo>;

    static fromJSON(json: any): TopologyValidationJobInfo;
  }

  interface TopologyValidationJobInfoProperties {
    /**
     * The last date and time the service was updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#lastUpdatedTime)
     */
    lastUpdatedTime?: DateProperties;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#status)
     */
    status?: "job-waiting" | "job-executing" | "job-succeeded";
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the validate network topology request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#statusUrl)
     */
    statusUrl?: string;
    /**
     * The date and time in which [submitTopologyValidationJob()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob) is first called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#submissionTime)
     */
    submissionTime?: DateProperties;
  }

  export interface TopologyValidationJobInfoWaitForJobCompletionOptions {
    /**
     * The time in millisecond between remote job status requests.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#waitForJobCompletion)
     */
    interval?: number;
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable asynchronous job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#waitForJobCompletion)
     */
    signal?: any;
    /**
     * Callback function that is called at the specified interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TopologyValidationJobInfo.html#waitForJobCompletion)
     */
    statusCallback?: Function;
  }

  export class TraceConfiguration {
    /**
     * An array of objects representing network attribute or category conditions that serve as barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#conditionBarriers)
     */
    conditionBarriers: any[];
    /**
     * An array of objects representing function barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#functionBarriers)
     */
    functionBarriers: any[];
    /**
     * An array of objects representing function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#functions)
     */
    functions: any[];
    /**
     * Do not stop the trace if the starting point is a barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#ignoreBarriersAtStartingPoints)
     */
    ignoreBarriersAtStartingPoints: boolean;
    /**
     * Specifies whether the traversability barrier features will be included in the trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#includeBarriers)
     */
    includeBarriers: boolean;
    /**
     * Specifies the type of features returned based on a network attribute or check for a category string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#outputConditions)
     */
    outputConditions: any[];
    /**
     * Specifies the network attribute name used for determining the shortest path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#shortestPathNetworkAttributeName)
     */
    shortestPathNetworkAttributeName: string;
    /**
     * Determines whether traversability is applied to both junctions and edges, junctions only, or edges only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#traversabilityScope)
     */
    traversabilityScope: "junctions" | "edges" | "junctionsAndEdges";
    /**
     * Specifies whether an error will be returned if dirty areas are encountered in any of the traversed features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceConfiguration.html#validateConsistency)
     */
    validateConsistency: boolean;

    constructor(properties?: any);
  }

  export interface TraceJobInfo extends Accessor, JSONSupport {}

  export class TraceJobInfo {
    /**
     * Represents information pertaining to the execution of an asynchronous request on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html)
     */

    constructor(properties?: TraceJobInfoProperties);

    /**
     * The last date and time the service was updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#lastUpdatedTime)
     */
    lastUpdatedTime: Date;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#status)
     */
    status: "job-waiting" | "job-executing" | "job-succeeded";
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the validate network topology request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#statusUrl)
     */
    statusUrl: string;
    /**
     * The date and time in which [submitTopologyValidationJob()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob) is first called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#submissionTime)
     */
    submissionTime: Date;

    /**
     * Sends a request for the current state of this job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#checkJobStatus)
     */
    checkJobStatus(requestOptions?: any): Promise<TraceJobInfo>;
    /**
     * Stop monitoring this job for status updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#destroy)
     */
    destroy(): void;
    /**
     * Resolves when an asynchronous job has completed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#waitForJobCompletion)
     */
    waitForJobCompletion(options?: TraceJobInfoWaitForJobCompletionOptions): Promise<TraceJobInfo>;

    static fromJSON(json: any): TraceJobInfo;
  }

  interface TraceJobInfoProperties {
    /**
     * The last date and time the service was updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#lastUpdatedTime)
     */
    lastUpdatedTime?: DateProperties;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#status)
     */
    status?: "job-waiting" | "job-executing" | "job-succeeded";
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the validate network topology request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#statusUrl)
     */
    statusUrl?: string;
    /**
     * The date and time in which [submitTopologyValidationJob()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#submitTopologyValidationJob) is first called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#submissionTime)
     */
    submissionTime?: DateProperties;
  }

  export interface TraceJobInfoWaitForJobCompletionOptions {
    /**
     * The time in millisecond between remote job status requests.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#waitForJobCompletion)
     */
    interval?: any;
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable asynchronous job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#waitForJobCompletion)
     */
    signal?: any;
    /**
     * Callback function that is called at the specified interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-TraceJobInfo.html#waitForJobCompletion)
     */
    statusCallback?: any;
  }

  export class UNTraceConfiguration {
    /**
     * Specifies whether to allow IndeterminateFlow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#allowIndeterminateFlow)
     */
    allowIndeterminateFlow: boolean;
    /**
     * Allows users to input arcade expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#arcadeExpressionBarrier)
     */
    arcadeExpressionBarrier: string;
    /**
     * Specifies the diagram Template Name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#diagramTemplateName)
     */
    diagramTemplateName: string;
    /**
     * Specifies the name of the domain network where the trace will be run.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#domainNetworkName)
     */
    domainNetworkName: string;
    /**
     * An array of objects representing specific categories or network attributes where the trace will stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#filterBarriers)
     */
    filterBarriers: any[];
    /**
     * Ensures the trace results include any bit that is set in the starting points for the network attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#filterBitsetNetworkAttributeName)
     */
    filterBitsetNetworkAttributeName: string;
    /**
     * An array of objects representing filter function barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#filterFunctionBarriers)
     */
    filterFunctionBarriers: any[];
    /**
     * Specifies where the filter will be applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#filterScope)
     */
    filterScope: "junctions" | "edges" | "junctionsAndEdges";
    /**
     * Specifies if the container features will be included in the trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#includeContainers)
     */
    includeContainers: boolean;
    /**
     * Specifies if the content in containers will be included in the results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#includeContent)
     */
    includeContent: boolean;
    /**
     * Specifies whether to include isolated features for an isolation trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#includeIsolated)
     */
    includeIsolated: boolean;
    /**
     * Specifies if structure features and objects will be included in the trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#includeStructures)
     */
    includeStructures: boolean;
    /**
     * Specifies whether to limit the containers returned to include only those encountered up to, and including, the first spatial container for each network element in the trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#includeUpToFirstSpatialContainer)
     */
    includeUpToFirstSpatialContainer: boolean;
    /**
     * Specifies the parameters needed for calculating nearest neighbors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#nearestNeighbor)
     */
    nearestNeighbor: any;
    /**
     * An array of objects representing the output filter categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#outputFilterCategories)
     */
    outputFilterCategories: any[];
    /**
     * An array of objects used to control what is returned in the results of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#outputFilters)
     */
    outputFilters: any[];
    /**
     * A propagator defines the propagation of a network attribute along a traversal and provides a filter to stop traversal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#propagators)
     */
    propagators: any[];
    /**
     * Specifies the name of the [subnetwork](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/subnetworks.htm) where the trace will be run.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#subnetworkName)
     */
    subnetworkName: string;
    /**
     * Specifies the name of the tier where an upstream or downstream trace ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#targetTierName)
     */
    targetTierName: string;
    /**
     * Specifies the name of the tier where the trace will be run.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#tierName)
     */
    tierName: string;
    /**
     * Specifies if its necessary to validate whether traversed junction or edge objects have the necessary containment, attachment, or connectivity association in their association hierarchy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-UNTraceConfiguration.html#validateLocatability)
     */
    validateLocatability: boolean;

    constructor(properties?: any);
  }

  export class UtilityNetwork extends Network {
    /**
     * This class contains metadata about the utility network dataset retrieved from a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html)
     */

    constructor(properties?: UtilityNetworkProperties);

    /**
     * Returns all the [domain networks](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/domain-network.htm) in the utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#domainNetworkNames)
     */
    readonly domainNetworkNames: string[];
    /**
     * The layer id of the service territory class used to define the extent of the utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#serviceTerritoryFeatureLayerId)
     */
    readonly serviceTerritoryFeatureLayerId: number;
    /**
     * This property returns the list of trace configurations shared on the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#sharedNamedTraceConfigurations)
     */
    sharedNamedTraceConfigurations: NamedTraceConfiguration[];
    /**
     * Returns all the terminal configurations on the utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#terminalConfigurations)
     */
    readonly terminalConfigurations: TerminalConfiguration[];
    /**
     * The type of the dataset.
     *
     * @default "utility"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#type)
     */
    readonly type: "utility";

    /**
     * Returns `true` if the given [Association](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) is valid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#canAddAssociation)
     */
    canAddAssociation(association: Association): Promise<boolean>;
    /**
     * Returns [ServiceEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits) which are used to add an [Association](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) via the `applyEdits` method on the [FeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html) class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#generateAddAssociations)
     */
    generateAddAssociations(association: Association[]): ServiceEdits;
    /**
     * Returns [ServiceEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits) which are used to delete an [Association](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) via the `applyEdits` method on the [FeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html) class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#generateDeleteAssociations)
     */
    generateDeleteAssociations(association: Association[]): ServiceEdits;
    /**
     * All devices features have terminal configurations (default single terminal).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#getTerminalConfiguration)
     */
    getTerminalConfiguration(feature: Graphic): TerminalConfiguration;
    /**
     * Takes the name of a domain network and returns an array with the names of its tiers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#getTierNames)
     */
    getTierNames(domainNetworkName: string): string[];
    /**
     * Returns all associations filtered by the [QueryAssociationsParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html) in a utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#queryAssociations)
     */
    queryAssociations(props: QueryAssociationsParameters): Promise<Association>;
    /**
     * Takes the traceProps and preforms an async trace on a [UtilityNetwork](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    submitTraceJob(props?: UtilityNetworkSubmitTraceJobProps): Promise<TraceJobInfo>;
    /**
     * Given an extent, returns all associations within this extent and their synthesized geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#synthesizeAssociationGeometries)
     */
    synthesizeAssociationGeometries(
      params: SynthesizeAssociationGeometriesParameters,
    ): Promise<AssociationGeometriesResult>;
    /**
     * Takes the traceProps and preforms a trace on a [UtilityNetwork](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    trace(props?: UtilityNetworkTraceProps): Promise<TraceResult>;

    static fromJSON(json: any): UtilityNetwork;
  }

  interface UtilityNetworkProperties extends NetworkProperties {
    /**
     * This property returns the list of trace configurations shared on the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#sharedNamedTraceConfigurations)
     */
    sharedNamedTraceConfigurations?: NamedTraceConfiguration[];
  }

  /**
   * Results returned from the [generateAddAssociations](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#generateAddAssociations) method and [generateDeleteAssociations](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#generateDeleteAssociations).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
   */
  export interface ServiceEdits {
    /**
     * The layerId of the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    id: number;
    /**
     * Identifier for globalId and objectId.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    identifierFields: ServiceEditsIdentifierFields[];
    /**
     * Object of generated add features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    addFeatures: any[];
    /**
     * Object of generated delete features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    deleteFeatures: any[];
  }

  export interface UtilityNetworkSubmitTraceJobProps {
    /**
     * The globalId (UUID) of the named trace configuration persisted in the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    namedTraceConfigurationGlobalId?: string;
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    gdbVersion?: string;
    /**
     * The list of starting points and barriers that will define where the trace starts and stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    traceLocations?: TraceLocation[];
    /**
     * The spatial reference that should be used to project the [aggregated geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html) returned by the trace (if applicable).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    outSpatialReference?: SpatialReference;
    /**
     * Defines the properties of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    traceConfiguration?: UNTraceConfiguration;
    /**
     * Parameter specifying the types of results to return after running a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    resultTypes?: ResultTypeJSON[];
    /**
     * The [traceType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceType) defined in this trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#submitTraceJob)
     */
    traceType?: string;
  }

  export interface UtilityNetworkTraceProps {
    /**
     * The globalId (UUID) of the named trace configuration persisted in the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    namedTraceConfigurationGlobalId?: string;
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    gdbVersion?: string;
    /**
     * The list of starting points and barriers that will define where the trace starts and stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    traceLocations?: TraceLocation[];
    /**
     * The spatial reference that should be used to project the [aggregated geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html) returned by the trace (if applicable).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    outSpatialReference?: SpatialReference;
    /**
     * Defines the properties of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    traceConfiguration?: UNTraceConfiguration;
    /**
     * Parameter specifying the types of results to return after running a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    resultTypes?: ResultTypeJSON[];
    /**
     * The [traceType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceType) defined in this trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#trace)
     */
    traceType?: string;
  }

  export interface ServiceEditsIdentifierFields {
    /**
     * The globalId identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    globalIdField: string;
    /**
     * The objectId identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#ServiceEdits)
     */
    objectIdField: string;
  }

  /**
   * A convenience module for importing [PointCloudRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html)
   */
  namespace pointCloudRenderers {
    /**
     * Renderer types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudRenderer)
     */
    export type PointCloudRenderer =
      | __esri.PointCloudClassBreaksRenderer
      | __esri.PointCloudRGBRenderer
      | __esri.PointCloudStretchRenderer
      | __esri.PointCloudUniqueValueRenderer;

    /**
     * PointCloudClassBreaksRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudClassBreaksRenderer)
     */
    export type PointCloudClassBreaksRenderer = __esri.PointCloudClassBreaksRenderer;
    export const PointCloudClassBreaksRenderer: typeof __esri.PointCloudClassBreaksRenderer;

    /**
     * PointCloudRGBRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudRGBRenderer)
     */
    export type PointCloudRGBRenderer = __esri.PointCloudRGBRenderer;
    export const PointCloudRGBRenderer: typeof __esri.PointCloudRGBRenderer;

    /**
     * PointCloudStretchRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudStretchRenderer)
     */
    export type PointCloudStretchRenderer = __esri.PointCloudStretchRenderer;
    export const PointCloudStretchRenderer: typeof __esri.PointCloudStretchRenderer;

    /**
     * PointCloudUniqueValueRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudUniqueValueRenderer)
     */
    export type PointCloudUniqueValueRenderer = __esri.PointCloudUniqueValueRenderer;
    export const PointCloudUniqueValueRenderer: typeof __esri.PointCloudUniqueValueRenderer;
  }

  /**
   * PointCloudClassBreaksRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudClassBreaksRenderer)
   */
  export type pointCloudRenderersPointCloudClassBreaksRenderer = PointCloudClassBreaksRenderer;

  /**
   * Renderer types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudRenderer)
   */
  export type pointCloudRenderersPointCloudRenderer =
    | PointCloudClassBreaksRenderer
    | PointCloudRGBRenderer
    | PointCloudStretchRenderer
    | PointCloudUniqueValueRenderer;

  /**
   * PointCloudRGBRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudRGBRenderer)
   */
  export type pointCloudRenderersPointCloudRGBRenderer = PointCloudRGBRenderer;

  /**
   * PointCloudStretchRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudStretchRenderer)
   */
  export type pointCloudRenderersPointCloudStretchRenderer = PointCloudStretchRenderer;

  /**
   * PointCloudUniqueValueRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-pointCloudRenderers.html#PointCloudUniqueValueRenderer)
   */
  export type pointCloudRenderersPointCloudUniqueValueRenderer = PointCloudUniqueValueRenderer;

  /**
   * A convenience module for importing [Content](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-Content.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html)
   */
  namespace content {
    /**
     * Popup content element types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#Content)
     */
    export type Content =
      | __esri.TextContent
      | __esri.MediaContent
      | __esri.FieldsContent
      | __esri.AttachmentsContent
      | __esri.CustomContent
      | __esri.ExpressionContent
      | __esri.RelationshipContent
      | __esri.UtilityNetworkAssociationsContent;

    /**
     * TextContent defines descriptive text as an element within the [PopupTemplate's](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#TextContent)
     */
    export type TextContent = __esri.TextContent;
    export const TextContent: typeof __esri.TextContent;

    /**
     * MediaContent contains an individual or array of chart and/or image media elements to display within a popup's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#MediaContent)
     */
    export type MediaContent = __esri.MediaContent;
    export const MediaContent: typeof __esri.MediaContent;

    /**
     * FieldsContent represents the [FieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#FieldsContent)
     */
    export type FieldsContent = __esri.FieldsContent;
    export const FieldsContent: typeof __esri.FieldsContent;

    /**
     * AttachmentsContent represents an attachment element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#AttachmentsContent)
     */
    export type AttachmentsContent = __esri.AttachmentsContent;
    export const AttachmentsContent: typeof __esri.AttachmentsContent;

    /**
     * CustomContent represents a custom content element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#CustomContent)
     */
    export type CustomContent = __esri.CustomContent;
    export const CustomContent: typeof __esri.CustomContent;

    /**
     * ExpressionContent represents an expression element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#ExpressionContent)
     */
    export type ExpressionContent = __esri.ExpressionContent;
    export const ExpressionContent: typeof __esri.ExpressionContent;

    /**
     * RelationshipContent represents an relationship element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#RelationshipContent)
     */
    export type RelationshipContent = __esri.RelationshipContent;
    export const RelationshipContent: typeof __esri.RelationshipContent;

    /**
     * UtilityNetworkAssociationsContent represents an utility network associations element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#UtilityNetworkAssociationsContent)
     */
    export type UtilityNetworkAssociationsContent = __esri.UtilityNetworkAssociationsContent;
    export const UtilityNetworkAssociationsContent: typeof __esri.UtilityNetworkAssociationsContent;
  }

  export interface AttachmentsContent extends Content, JSONSupport {}

  export class AttachmentsContent {
    /**
     * An `AttachmentsContent` popup element represents an attachment element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html)
     */

    constructor(properties?: AttachmentsContentProperties);

    /**
     * Describes the attachment's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#description)
     */
    description: string;
    /**
     * A string value indicating how to display an attachment.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#displayType)
     */
    displayType: "auto" | "preview" | "list";
    /**
     * A heading indicating what the attachment's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#title)
     */
    title: string;
    /**
     * The type of popup element displayed.
     *
     * @default "attachments"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#type)
     */
    readonly type: "attachments";

    static fromJSON(json: any): AttachmentsContent;
  }

  interface AttachmentsContentProperties extends ContentProperties {
    /**
     * Describes the attachment's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#description)
     */
    description?: string;
    /**
     * A string value indicating how to display an attachment.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#displayType)
     */
    displayType?: "auto" | "preview" | "list";
    /**
     * A heading indicating what the attachment's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html#title)
     */
    title?: string;
  }

  export interface BarChartMediaInfo extends Accessor, JSONSupport, MediaInfo, ChartMediaInfo {}

  export class BarChartMediaInfo {
    /**
     * A `BarChartMediaInfo` is a type of chart media element that represents a bar chart displayed within a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-BarChartMediaInfo.html)
     */

    constructor(properties?: BarChartMediaInfoProperties);

    /**
     * Indicates the type of chart.
     *
     * @default "bar-chart"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-BarChartMediaInfo.html#type)
     */
    readonly type: string;

    /**
     * Creates a deep clone of the BarChartMediaInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-BarChartMediaInfo.html#clone)
     */
    clone(): BarChartMediaInfo;

    static fromJSON(json: any): BarChartMediaInfo;
  }

  interface BarChartMediaInfoProperties extends MediaInfoProperties, ChartMediaInfoProperties {}

  export interface ColumnChartMediaInfo extends JSONSupport, MediaInfo, ChartMediaInfo {}

  export class ColumnChartMediaInfo {
    /**
     * A `ColumnChartMediaInfo` is a type of chart media element that represents a column chart displayed within a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ColumnChartMediaInfo.html)
     */

    constructor(properties?: ColumnChartMediaInfoProperties);

    /**
     * Indicates the type of chart.
     *
     * @default "column-chart"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ColumnChartMediaInfo.html#type)
     */
    readonly type: string;

    /**
     * Creates a deep clone of the ColumnChartMediaInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ColumnChartMediaInfo.html#clone)
     */
    clone(): ColumnChartMediaInfo;

    static fromJSON(json: any): ColumnChartMediaInfo;
  }

  interface ColumnChartMediaInfoProperties extends MediaInfoProperties, ChartMediaInfoProperties {}

  export interface Content extends Accessor, JSONSupport {}

  export class Content {
    /**
     * Content elements define what should display within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-Content.html)
     */

    constructor(properties?: ContentProperties);

    /**
     * The type of popup element displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-Content.html#type)
     */
    readonly type:
      | "text"
      | "fields"
      | "media"
      | "attachments"
      | "custom"
      | "expression"
      | "relationship"
      | "utility-network-associations";

    static fromJSON(json: any): Content;
  }

  interface ContentProperties {}

  export interface CustomContent extends Content, JSONSupport {}

  export class CustomContent {
    /**
     * A `CustomContent` popup element is used to provide a way to customize the [popup's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html)
     */

    constructor(properties?: CustomContentProperties);

    /**
     * The function that is called to create the custom content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#creator)
     */
    creator: PopupTemplateContentCreator | PopupTemplateContent | Promise<PopupTemplateContent>;
    /**
     * The called function to cleanup custom content when it is no longer necessary.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#destroyer)
     */
    destroyer: PopupTemplateContentDestroyer;
    /**
     * An array of field names used in the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#outFields)
     */
    outFields: string[];
    /**
     * The type of popup element displayed.
     *
     * @default "custom"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#type)
     */
    readonly type: "custom";

    /**
     * Creates a deep clone of the CustomContent class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#clone)
     */
    clone(): CustomContent;

    static fromJSON(json: any): CustomContent;
  }

  interface CustomContentProperties extends ContentProperties {
    /**
     * The function that is called to create the custom content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#creator)
     */
    creator?:
      | PopupTemplateContentCreator
      | string
      | HTMLElement
      | WidgetProperties
      | Promise<string | HTMLElement | WidgetProperties>;
    /**
     * The called function to cleanup custom content when it is no longer necessary.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#destroyer)
     */
    destroyer?: PopupTemplateContentDestroyer;
    /**
     * An array of field names used in the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#outFields)
     */
    outFields?: string[];
  }

  /**
   * Custom element content.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#PopupTemplateContent)
   */
  export type PopupTemplateContent = string | HTMLElement | Widget;

  export type PopupTemplateContentCreator = (
    event?: PopupTemplateCreatorEvent,
  ) => PopupTemplateContent | Promise<PopupTemplateContent>;

  export type PopupTemplateContentDestroyer = (event?: PopupTemplateCreatorEvent) => void;

  /**
   * The event that contains the [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#PopupTemplateCreatorEvent)
   */
  export interface PopupTemplateCreatorEvent {
    /**
     * The [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html#PopupTemplateCreatorEvent)
     */
    graphic: Graphic;
  }

  export interface ExpressionContent extends Content, JSONSupport {}

  export class ExpressionContent {
    /**
     * An ExpressionContent element allows you to define a popup content element with an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html)
     */

    constructor(properties?: ExpressionContentProperties);

    /**
     * Contains the Arcade expression used to create a popup content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html#expressionInfo)
     */
    expressionInfo: ElementExpressionInfo;
    /**
     * The type of popup element displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html#type)
     */
    readonly type: "expression";

    /**
     * Creates a deep clone of the ExpressionContent instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html#clone)
     */
    clone(): ExpressionContent;

    static fromJSON(json: any): ExpressionContent;
  }

  interface ExpressionContentProperties extends ContentProperties {
    /**
     * Contains the Arcade expression used to create a popup content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html#expressionInfo)
     */
    expressionInfo?: ElementExpressionInfoProperties;
  }

  export interface FieldsContent extends Content, JSONSupport {}

  export class FieldsContent {
    /**
     * A `FieldsContent` popup element represents the [FieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html)
     */

    constructor(properties?: FieldsContentProperties);

    /**
     * Describes the field's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#description)
     */
    description: string;
    /**
     * Array of [fieldInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#fieldInfos)
     */
    fieldInfos: FieldInfo[];
    /**
     * Heading indicating what the field's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#title)
     */
    title: string;
    /**
     * The type of popup element displayed.
     *
     * @default "fields"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#type)
     */
    readonly type: "fields";

    /**
     * Creates a deep clone of the FieldsContent class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#clone)
     */
    clone(): FieldsContent;

    static fromJSON(json: any): FieldsContent;
  }

  interface FieldsContentProperties extends ContentProperties {
    /**
     * Describes the field's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#description)
     */
    description?: string;
    /**
     * Array of [fieldInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#fieldInfos)
     */
    fieldInfos?: FieldInfoProperties[];
    /**
     * Heading indicating what the field's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html#title)
     */
    title?: string;
  }

  export interface ImageMediaInfo extends Accessor, JSONSupport, MediaInfo {}

  export class ImageMediaInfo {
    /**
     * An `ImageMediaInfo` is a type of media element that represents images to display within a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html)
     */

    constructor(properties?: ImageMediaInfoProperties);

    /**
     * Refresh interval of the layer in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#refreshInterval)
     */
    refreshInterval: number;
    /**
     * The type of popup element.
     *
     * @default "image"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#type)
     */
    readonly type: string;
    /**
     * Defines the value format of the image media element and how the images should be retrieved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#value)
     */
    value: ImageMediaInfoValue;

    /**
     * Creates a deep clone of the ImageMediaInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#clone)
     */
    clone(): ImageMediaInfo;

    static fromJSON(json: any): ImageMediaInfo;
  }

  interface ImageMediaInfoProperties extends MediaInfoProperties {
    /**
     * Refresh interval of the layer in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#refreshInterval)
     */
    refreshInterval?: number;
    /**
     * Defines the value format of the image media element and how the images should be retrieved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ImageMediaInfo.html#value)
     */
    value?: ImageMediaInfoValueProperties;
  }

  export interface LineChartMediaInfo extends Accessor, JSONSupport, MediaInfo, ChartMediaInfo {}

  export class LineChartMediaInfo {
    /**
     * A `LineChartMediaInfo` is a type of chart media element that represents a line chart displayed within a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-LineChartMediaInfo.html)
     */

    constructor(properties?: LineChartMediaInfoProperties);

    /**
     * Indicates the type of chart.
     *
     * @default "line-chart"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-LineChartMediaInfo.html#type)
     */
    readonly type: string;

    /**
     * Creates a deep clone of the LineChartMediaInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-LineChartMediaInfo.html#clone)
     */
    clone(): LineChartMediaInfo;

    static fromJSON(json: any): LineChartMediaInfo;
  }

  interface LineChartMediaInfoProperties extends MediaInfoProperties, ChartMediaInfoProperties {}

  export interface MediaContent extends Content, JSONSupport {}

  export class MediaContent {
    /**
     * A `MediaContent` popup element contains an individual or array of chart and/or image media elements to display within a popup's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html)
     */

    constructor(properties?: MediaContentProperties);

    /**
     * Index of the current active media within the popup's media content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#activeMediaInfoIndex)
     */
    activeMediaInfoIndex: number;
    /**
     * Describes the media's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#description)
     */
    description: string;
    /**
     * Contains the media elements representing images or charts to display within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#mediaInfos)
     */
    mediaInfos:
      | BarChartMediaInfo
      | ColumnChartMediaInfo
      | ImageMediaInfo
      | LineChartMediaInfo
      | PieChartMediaInfo
      | any[];
    /**
     * Heading indicating what the media's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#title)
     */
    title: string;
    /**
     * The type of popup element displayed.
     *
     * @default "media"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#type)
     */
    readonly type: "media";

    /**
     * Creates a deep clone of the MediaContent class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#clone)
     */
    clone(): MediaContent;

    static fromJSON(json: any): MediaContent;
  }

  interface MediaContentProperties extends ContentProperties {
    /**
     * Index of the current active media within the popup's media content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#activeMediaInfoIndex)
     */
    activeMediaInfoIndex?: number;
    /**
     * Describes the media's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#description)
     */
    description?: string;
    /**
     * Contains the media elements representing images or charts to display within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#mediaInfos)
     */
    mediaInfos?:
      | BarChartMediaInfoProperties
      | ColumnChartMediaInfoProperties
      | ImageMediaInfoProperties
      | LineChartMediaInfoProperties
      | PieChartMediaInfoProperties
      | any[];
    /**
     * Heading indicating what the media's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html#title)
     */
    title?: string;
  }

  export class ChartMediaInfo {
    /**
     * Defines the chart value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-ChartMediaInfo.html#value)
     */
    value: ChartMediaInfoValue;
  }

  interface ChartMediaInfoProperties {
    /**
     * Defines the chart value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-ChartMediaInfo.html#value)
     */
    value?: ChartMediaInfoValueProperties;
  }

  export class MediaInfo {
    /**
     * Provides an alternate text for an image if the image cannot be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#altText)
     */
    altText: string;
    /**
     * Defines a caption for the media.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#caption)
     */
    caption: string;
    /**
     * The title of the media element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#title)
     */
    title: string;
  }

  interface MediaInfoProperties {
    /**
     * Provides an alternate text for an image if the image cannot be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#altText)
     */
    altText?: string;
    /**
     * Defines a caption for the media.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#caption)
     */
    caption?: string;
    /**
     * The title of the media element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-mixins-MediaInfo.html#title)
     */
    title?: string;
  }

  export interface PieChartMediaInfo extends Accessor, JSONSupport, MediaInfo, ChartMediaInfo {}

  export class PieChartMediaInfo {
    /**
     * A `PieChartMediaInfo` is a type of chart media element that represents a pie chart displayed within a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-PieChartMediaInfo.html)
     */

    constructor(properties?: PieChartMediaInfoProperties);

    /**
     * Indicates the type of chart.
     *
     * @default "pie-chart"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-PieChartMediaInfo.html#type)
     */
    readonly type: string;

    /**
     * Creates a deep clone of the PieChartMediaInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-PieChartMediaInfo.html#clone)
     */
    clone(): PieChartMediaInfo;

    static fromJSON(json: any): PieChartMediaInfo;
  }

  interface PieChartMediaInfoProperties extends MediaInfoProperties, ChartMediaInfoProperties {}

  export interface RelationshipContent extends Content, JSONSupport, Clonable {}

  export class RelationshipContent {
    /**
     * A `RelationshipContent` popup element represents a relationship element associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html)
     */

    constructor(properties?: RelationshipContentProperties);

    /**
     * Describes the relationship's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#description)
     */
    description: string;
    /**
     * A numeric value indicating the maximum number of related features to display in the list of related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#displayCount)
     */
    displayCount: number;
    /**
     * A string value indicating how to display related records within the relationship content.
     *
     * @default "list"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#displayType)
     */
    displayType: "list";
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) indicating the display order for the related records, and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#orderByFields)
     */
    orderByFields: RelatedRecordsInfoFieldOrder[];
    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#relationshipId)
     */
    relationshipId: number;
    /**
     * A heading indicating what the relationship's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#title)
     */
    title: string;
    /**
     * The type of popup element displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#type)
     */
    readonly type: "relationship";

    static fromJSON(json: any): RelationshipContent;
  }

  interface RelationshipContentProperties extends ContentProperties {
    /**
     * Describes the relationship's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#description)
     */
    description?: string;
    /**
     * A numeric value indicating the maximum number of related features to display in the list of related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#displayCount)
     */
    displayCount?: number;
    /**
     * A string value indicating how to display related records within the relationship content.
     *
     * @default "list"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#displayType)
     */
    displayType?: "list";
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) indicating the display order for the related records, and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#orderByFields)
     */
    orderByFields?: RelatedRecordsInfoFieldOrderProperties[];
    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#relationshipId)
     */
    relationshipId?: number;
    /**
     * A heading indicating what the relationship's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-RelationshipContent.html#title)
     */
    title?: string;
  }

  export interface ChartMediaInfoValue extends Accessor, JSONSupport {}

  export class ChartMediaInfoValue {
    /**
     * The `ChartMediaInfoValue` class contains information for popups regarding how charts should be constructed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html)
     */

    constructor(properties?: ChartMediaInfoValueProperties);

    /**
     * An optional array of [colors](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) where each color corresponds respectively to a field in the [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#fields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#colors)
     */
    colors: Color[];
    /**
     * An array of strings, with each string containing the name of a field to display in the chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#fields)
     */
    fields: string[];
    /**
     * A string containing the name of a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#normalizeField)
     */
    normalizeField: string;
    /**
     * An array of [ChartMediaInfoValueSeries](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html) objects which provide information of x/y data that is plotted in a chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#series)
     */
    series: ChartMediaInfoValueSeries[];
    /**
     * String value indicating the tooltip for a chart specified from another field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#tooltipField)
     */
    tooltipField: string;

    static fromJSON(json: any): ChartMediaInfoValue;
  }

  interface ChartMediaInfoValueProperties {
    /**
     * An optional array of [colors](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) where each color corresponds respectively to a field in the [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#fields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#colors)
     */
    colors?: Color[];
    /**
     * An array of strings, with each string containing the name of a field to display in the chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#fields)
     */
    fields?: string[];
    /**
     * A string containing the name of a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#normalizeField)
     */
    normalizeField?: string;
    /**
     * An array of [ChartMediaInfoValueSeries](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html) objects which provide information of x/y data that is plotted in a chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#series)
     */
    series?: ChartMediaInfoValueSeriesProperties[];
    /**
     * String value indicating the tooltip for a chart specified from another field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValue.html#tooltipField)
     */
    tooltipField?: string;
  }

  export class ChartMediaInfoValueSeries extends Accessor {
    /**
     * The `ChartMediaInfoValueSeries` class is a read-only support class that represents information specific to how data should be plotted in a chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html)
     */

    constructor(properties?: ChartMediaInfoValueSeriesProperties);

    /**
     * The [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) representing the field for a series.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html#color)
     */
    readonly color: Color;
    /**
     * String value indicating the field's name for a series.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html#fieldName)
     */
    readonly fieldName: string;
    /**
     * String value indicating the tooltip for a series.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html#tooltip)
     */
    readonly tooltip: string;
    /**
     * Numerical value for the chart series.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html#value)
     */
    readonly value: number;

    /**
     * Creates a deep clone of the ChartMediaInfoValueSeries class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ChartMediaInfoValueSeries.html#clone)
     */
    clone(): ChartMediaInfoValueSeries;
  }

  interface ChartMediaInfoValueSeriesProperties {}

  export interface ImageMediaInfoValue extends Accessor, JSONSupport {}

  export class ImageMediaInfoValue {
    /**
     * The `ImageMediaInfoValue` class contains information for popups regarding how images should be retrieved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html)
     */

    constructor(properties?: ImageMediaInfoValueProperties);

    /**
     * A string containing a URL to be launched in a browser when a user clicks the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html#linkURL)
     */
    linkURL: string;
    /**
     * A string containing the URL to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html#sourceURL)
     */
    sourceURL: string;

    /**
     * Creates a deep clone of the ImageMediaInfoValue class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html#clone)
     */
    clone(): ImageMediaInfoValue;

    static fromJSON(json: any): ImageMediaInfoValue;
  }

  interface ImageMediaInfoValueProperties {
    /**
     * A string containing a URL to be launched in a browser when a user clicks the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html#linkURL)
     */
    linkURL?: string;
    /**
     * A string containing the URL to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-support-ImageMediaInfoValue.html#sourceURL)
     */
    sourceURL?: string;
  }

  export interface TextContent extends Content, JSONSupport {}

  export class TextContent {
    /**
     * A `TextContent` popup element is used to define descriptive text as an element within a [PopupTemplate's](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html)
     */

    constructor(properties?: TextContentProperties);

    /**
     * The formatted string content to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html#text)
     */
    text: string;
    /**
     * The type of popup element displayed.
     *
     * @default "text"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html#type)
     */
    readonly type: "text";

    /**
     * Creates a deep clone of the TextContent class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html#clone)
     */
    clone(): TextContent;

    static fromJSON(json: any): TextContent;
  }

  interface TextContentProperties extends ContentProperties {
    /**
     * The formatted string content to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html#text)
     */
    text?: string;
  }

  export interface UtilityNetworkAssociationsContent extends Content, JSONSupport, Clonable {}

  export class UtilityNetworkAssociationsContent {
    /**
     * A `UtilityNetworkAssociationsContent` popup element represents an association element of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html)
     */

    constructor(properties?: UtilityNetworkAssociationsContentProperties);

    /**
     * The popup association types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#associationTypes)
     */
    associationTypes: UtilityNetworkAssociationType[];
    /**
     * Describes the relationship's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#description)
     */
    description: string;
    /**
     * A heading indicating what the relationship's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#title)
     */
    title: string;
    /**
     * The type of popup element displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#type)
     */
    readonly type: "utility-network-associations";

    static fromJSON(json: any): UtilityNetworkAssociationsContent;
  }

  interface UtilityNetworkAssociationsContentProperties extends ContentProperties {
    /**
     * The popup association types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#associationTypes)
     */
    associationTypes?: UtilityNetworkAssociationType[];
    /**
     * Describes the relationship's content in detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#description)
     */
    description?: string;
    /**
     * A heading indicating what the relationship's content represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-UtilityNetworkAssociationsContent.html#title)
     */
    title?: string;
  }

  /**
   * AttachmentsContent represents an attachment element associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#AttachmentsContent)
   */
  export type contentAttachmentsContent = AttachmentsContent;

  /**
   * Popup content element types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#Content)
   */
  export type contentContent =
    | TextContent
    | MediaContent
    | FieldsContent
    | AttachmentsContent
    | CustomContent
    | ExpressionContent
    | RelationshipContent
    | UtilityNetworkAssociationsContent;

  /**
   * CustomContent represents a custom content element associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#CustomContent)
   */
  export type contentCustomContent = CustomContent;

  /**
   * ExpressionContent represents an expression element associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#ExpressionContent)
   */
  export type contentExpressionContent = ExpressionContent;

  /**
   * FieldsContent represents the [FieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#FieldsContent)
   */
  export type contentFieldsContent = FieldsContent;

  /**
   * MediaContent contains an individual or array of chart and/or image media elements to display within a popup's content.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#MediaContent)
   */
  export type contentMediaContent = MediaContent;

  /**
   * RelationshipContent represents an relationship element associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#RelationshipContent)
   */
  export type contentRelationshipContent = RelationshipContent;

  /**
   * TextContent defines descriptive text as an element within the [PopupTemplate's](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) content.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#TextContent)
   */
  export type contentTextContent = TextContent;

  /**
   * UtilityNetworkAssociationsContent represents an utility network associations element associated with a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content.html#UtilityNetworkAssociationsContent)
   */
  export type contentUtilityNetworkAssociationsContent = UtilityNetworkAssociationsContent;

  export interface ElementExpressionInfo extends Accessor, JSONSupport {}

  export class ElementExpressionInfo {
    /**
     * Defines an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression used to create an [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) element in a [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html)
     */

    constructor(properties?: ElementExpressionInfoProperties);

    /**
     * The [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression evaluating to a dictionary.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#expression)
     */
    expression: string;
    /**
     * The return type of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#returnType)
     */
    returnType: "dictionary";
    /**
     * The title used to describe the popup element returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the ElementExpressionInfo instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#clone)
     */
    clone(): ElementExpressionInfo;

    static fromJSON(json: any): ElementExpressionInfo;
  }

  interface ElementExpressionInfoProperties {
    /**
     * The [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression evaluating to a dictionary.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#expression)
     */
    expression?: string;
    /**
     * The return type of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#returnType)
     */
    returnType?: "dictionary";
    /**
     * The title used to describe the popup element returned by the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ElementExpressionInfo.html#title)
     */
    title?: string;
  }

  export interface popupExpressionInfo extends Accessor, JSONSupport {}

  export class popupExpressionInfo {
    /**
     * The `ExpressionInfo` class references [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions following the specification defined by the [Arcade Popup Profile](https://developers.arcgis.com/javascript/latest/arcade/#popup).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html)
     */

    constructor(properties?: popupExpressionInfoProperties);

    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Popup Profile](https://developers.arcgis.com/javascript/latest/arcade/#popup).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#expression)
     */
    expression: string;
    /**
     * The name of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#name)
     */
    name: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#returnType)
     */
    returnType: "string" | "number";
    /**
     * The title used to describe the value returned by the expression in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of the ExpressionInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#clone)
     */
    clone(): popupExpressionInfo;

    static fromJSON(json: any): popupExpressionInfo;
  }

  interface popupExpressionInfoProperties {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Popup Profile](https://developers.arcgis.com/javascript/latest/arcade/#popup).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#expression)
     */
    expression?: string;
    /**
     * The name of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#name)
     */
    name?: string;
    /**
     * Indicates the return type of the Arcade expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#returnType)
     */
    returnType?: "string" | "number";
    /**
     * The title used to describe the value returned by the expression in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html#title)
     */
    title?: string;
  }

  export interface FieldInfo extends Accessor, JSONSupport {}

  export class FieldInfo {
    /**
     * The `FieldInfo` class defines how a [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) participates, or in some cases, does not participate, in a [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html)
     */

    constructor(properties?: FieldInfoProperties);

    /**
     * The field name as defined by the service or the name of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#fieldName)
     */
    fieldName: string;
    /**
     * Class which provides formatting options for numerical or date fields and how they should display within a popup.
     *
     *   * If displaying a `timestamp-offset` or `date` [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) field in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), [Features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html), or [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) widget and the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) has a [timezone](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#timeZone) set other than `unknown`, the abbreviated time zone suffix will be omitted from the attribute value. For example, the attribute value will display as `9/28/2014, 7:58 PM`. To display an abbreviated time zone suffix, see the `Read more` section. >>>esri-read-more[Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#format)
     */
    format: FieldInfoFormat;
    /**
     * A Boolean determining whether users can edit this field.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#isEditable)
     */
    isEditable: boolean;
    /**
     * A string containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#label)
     */
    label: string;
    /**
     * Used in a `one:many` or `many:many` relationship to compute the statistics on the field to show in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#statisticType)
     */
    statisticType: "count" | "sum" | "min" | "max" | "avg" | "stddev" | "var";
    /**
     * A string determining what type of input box editors see when editing the field.
     *
     * @default "text-box"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#stringFieldOption)
     */
    stringFieldOption: "rich-text" | "text-area" | "text-box";
    /**
     * A string providing an editing hint for editors of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#tooltip)
     */
    tooltip: string;
    /**
     * Indicates whether the field is visible in the popup window.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#visible)
     */
    visible: boolean;

    /**
     * Creates a deep clone of the FieldInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#clone)
     */
    clone(): FieldInfo;

    static fromJSON(json: any): FieldInfo;
  }

  interface FieldInfoProperties {
    /**
     * The field name as defined by the service or the name of an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#fieldName)
     */
    fieldName?: string;
    /**
     * Class which provides formatting options for numerical or date fields and how they should display within a popup.
     *
     *   * If displaying a `timestamp-offset` or `date` [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) field in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), [Features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html), or [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) widget and the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) has a [timezone](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#timeZone) set other than `unknown`, the abbreviated time zone suffix will be omitted from the attribute value. For example, the attribute value will display as `9/28/2014, 7:58 PM`. To display an abbreviated time zone suffix, see the `Read more` section. >>>esri-read-more[Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#format)
     */
    format?: FieldInfoFormatProperties;
    /**
     * A Boolean determining whether users can edit this field.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#isEditable)
     */
    isEditable?: boolean;
    /**
     * A string containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#label)
     */
    label?: string;
    /**
     * Used in a `one:many` or `many:many` relationship to compute the statistics on the field to show in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#statisticType)
     */
    statisticType?: "count" | "sum" | "min" | "max" | "avg" | "stddev" | "var";
    /**
     * A string determining what type of input box editors see when editing the field.
     *
     * @default "text-box"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#stringFieldOption)
     */
    stringFieldOption?: "rich-text" | "text-area" | "text-box";
    /**
     * A string providing an editing hint for editors of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#tooltip)
     */
    tooltip?: string;
    /**
     * Indicates whether the field is visible in the popup window.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html#visible)
     */
    visible?: boolean;
  }

  export interface LayerOptions extends Accessor, JSONSupport {}

  export class LayerOptions {
    /**
     * The `LayerOptions` class defines additional options that can be defined for a layer's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html)
     */

    constructor(properties?: LayerOptionsProperties);

    /**
     * Applicable to [Imagery Layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html#returnTopmostRaster)
     */
    returnTopmostRaster: boolean;
    /**
     * Applicable to [Imagery Layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html#showNoDataRecords)
     */
    showNoDataRecords: boolean;

    /**
     * Creates a deep clone of the LayerOptions class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html#clone)
     */
    clone(): LayerOptions;

    static fromJSON(json: any): LayerOptions;
  }

  interface LayerOptionsProperties {
    /**
     * Applicable to [Imagery Layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html#returnTopmostRaster)
     */
    returnTopmostRaster?: boolean;
    /**
     * Applicable to [Imagery Layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-LayerOptions.html#showNoDataRecords)
     */
    showNoDataRecords?: boolean;
  }

  export interface RelatedRecordsInfo extends Accessor, JSONSupport {}

  export class RelatedRecordsInfo {
    /**
     * The `RelatedRecordsInfo` class provides additional sorting options when working with related records in a layer's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html)
     */

    constructor(properties?: RelatedRecordsInfoProperties);

    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html#orderByFields)
     */
    orderByFields: RelatedRecordsInfoFieldOrder[];
    /**
     * Indicates whether to display related records in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html#showRelatedRecords)
     */
    showRelatedRecords: boolean;

    /**
     * Creates a deep clone of the RelatedRecordsInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html#clone)
     */
    clone(): RelatedRecordsInfo;

    static fromJSON(json: any): RelatedRecordsInfo;
  }

  interface RelatedRecordsInfoProperties {
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html#orderByFields)
     */
    orderByFields?: RelatedRecordsInfoFieldOrderProperties[];
    /**
     * Indicates whether to display related records in the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-RelatedRecordsInfo.html#showRelatedRecords)
     */
    showRelatedRecords?: boolean;
  }

  export interface FieldInfoFormat extends Accessor, JSONSupport, Clonable {}

  export class FieldInfoFormat {
    /**
     * The `FieldInfoFormat` class is used with numerical or date fields to provide more detail about how the value should be displayed in a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html)
     */

    constructor(properties?: FieldInfoFormatProperties);

    /**
     * Used only with `Date` fields.
     *
     *   * If displaying a `timestamp-offset` or `date` [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) field in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), [Features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html), or [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) widget and the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) has a [timezone](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#timeZone) set other than `unknown`, the abbreviated time zone suffix will be omitted from the attribute value. For example, the attribute value will display as `9/28/2014, 7:58 PM`. To display an abbreviated time zone suffix, see the `Read more` section. >>>esri-read-more[Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#dateFormat)
     */
    dateFormat:
      | "short-date"
      | "short-date-short-time"
      | "short-date-short-time-24"
      | "short-date-long-time"
      | "short-date-long-time-24"
      | "short-date-le"
      | "short-date-le-short-time"
      | "short-date-le-short-time-24"
      | "short-date-le-long-time"
      | "short-date-le-long-time-24"
      | "long-month-day-year"
      | "long-month-day-year-short-time"
      | "long-month-day-year-short-time-24"
      | "long-month-day-year-long-time"
      | "long-month-day-year-long-time-24"
      | "day-short-month-year"
      | "day-short-month-year-short-time"
      | "day-short-month-year-short-time-24"
      | "day-short-month-year-long-time"
      | "day-short-month-year-long-time-24"
      | "long-date"
      | "long-date-short-time"
      | "long-date-short-time-24"
      | "long-date-long-time"
      | "long-date-long-time-24"
      | "long-month-year"
      | "short-month-year"
      | "year";
    /**
     * Used only with `Number` fields.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#digitSeparator)
     */
    digitSeparator: boolean;
    /**
     * Used only with `Number` fields to specify the number of supported decimal places that should appear in popups.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#places)
     */
    places: number;

    static fromJSON(json: any): FieldInfoFormat;
  }

  interface FieldInfoFormatProperties {
    /**
     * Used only with `Date` fields.
     *
     *   * If displaying a `timestamp-offset` or `date` [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html#type) field in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), [Features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html), or [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) widget and the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) has a [timezone](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#timeZone) set other than `unknown`, the abbreviated time zone suffix will be omitted from the attribute value. For example, the attribute value will display as `9/28/2014, 7:58 PM`. To display an abbreviated time zone suffix, see the `Read more` section. >>>esri-read-more[Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#dateFormat)
     */
    dateFormat?:
      | "short-date"
      | "short-date-short-time"
      | "short-date-short-time-24"
      | "short-date-long-time"
      | "short-date-long-time-24"
      | "short-date-le"
      | "short-date-le-short-time"
      | "short-date-le-short-time-24"
      | "short-date-le-long-time"
      | "short-date-le-long-time-24"
      | "long-month-day-year"
      | "long-month-day-year-short-time"
      | "long-month-day-year-short-time-24"
      | "long-month-day-year-long-time"
      | "long-month-day-year-long-time-24"
      | "day-short-month-year"
      | "day-short-month-year-short-time"
      | "day-short-month-year-short-time-24"
      | "day-short-month-year-long-time"
      | "day-short-month-year-long-time-24"
      | "long-date"
      | "long-date-short-time"
      | "long-date-short-time-24"
      | "long-date-long-time"
      | "long-date-long-time-24"
      | "long-month-year"
      | "short-month-year"
      | "year";
    /**
     * Used only with `Number` fields.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#digitSeparator)
     */
    digitSeparator?: boolean;
    /**
     * Used only with `Number` fields to specify the number of supported decimal places that should appear in popups.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-FieldInfoFormat.html#places)
     */
    places?: number;
  }

  export interface RelatedRecordsInfoFieldOrder extends Accessor, JSONSupport {}

  export class RelatedRecordsInfoFieldOrder {
    /**
     * The `RelatedRecordsInfoFieldOrder` class indicates the field display order for the related records in a layer's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html)
     */

    constructor(properties?: RelatedRecordsInfoFieldOrderProperties);

    /**
     * The attribute value of the field selected that will drive the sorting of related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html#field)
     */
    field: string;
    /**
     * Set the ascending or descending sort order of the returned related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html#order)
     */
    order: "asc" | "desc";

    /**
     * Creates a deep clone of the RelatedRecordsInfoFieldOrder class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html#clone)
     */
    clone(): RelatedRecordsInfoFieldOrder;

    static fromJSON(json: any): RelatedRecordsInfoFieldOrder;
  }

  interface RelatedRecordsInfoFieldOrderProperties {
    /**
     * The attribute value of the field selected that will drive the sorting of related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html#field)
     */
    field?: string;
    /**
     * Set the ascending or descending sort order of the returned related records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html#order)
     */
    order?: "asc" | "desc";
  }

  /**
   * The association type defined for the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html)
   */
  interface UtilityNetworkAssociationType {
    /**
     * This property is used to filter the associations by asset group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#associatedAssetGroup)
     */
    associatedAssetGroup: number;
    /**
     * This property is used to filter the associations by asset type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#associatedAssetType)
     */
    associatedAssetType: number;
    /**
     * This property is used to filter the associations by network sourceId.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#associatedNetworkSourceId)
     */
    associatedNetworkSourceId: number;
    /**
     * The description for the [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#type) of association in the list of association [types](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#type) on the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#description)
     */
    description: string;
    /**
     * Triggers viewModel to query the proper features when a specific type of association is selected from the list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#open)
     */
    open: boolean;
    /**
     * The title for the [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#type) of association in the list of association [types](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#type) on the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#title)
     */
    title: string;
    /**
     * The type of association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-UtilityNetworkAssociationType.html#type)
     */
    type: "attachment" | "connectivity" | "container" | "content" | "structure";
  }

  export const UtilityNetworkAssociationType: UtilityNetworkAssociationType;

  export interface PopupTemplate extends Accessor, Clonable, JSONSupport {}

  export class PopupTemplate {
    /**
     * A PopupTemplate formats and defines the content of a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) for a specific [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) or [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html)
     */

    constructor(properties?: PopupTemplateProperties);

    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#actions)
     */
    actions: Collection<ActionButton | ActionToggle>;
    /**
     * The template for defining and formatting a popup's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#content)
     */
    content: Content[] | string | Function | Promise<any>;
    /**
     * An array of objects or [ExpressionInfo[]](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) that reference [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions following the specification defined by the [Arcade Popup Profile](https://developers.arcgis.com/javascript/latest/arcade/#popup).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#expressionInfos)
     */
    expressionInfos: popupExpressionInfo[];
    /**
     * An array of [FieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) that defines how fields in the dataset or values from [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions participate in a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#fieldInfos)
     */
    fieldInfos: FieldInfo[];
    /**
     * Indicates whether or not editor tracking should display.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#lastEditInfoEnabled)
     */
    lastEditInfoEnabled: boolean;
    /**
     * Additional options that can be defined for the popup layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#layerOptions)
     */
    layerOptions: LayerOptions;
    /**
     * An array of field names used in the PopupTemplate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#outFields)
     */
    outFields: string[];
    /**
     * Indicates whether actions should replace existing [popup actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#overwriteActions)
     */
    overwriteActions: boolean;
    /**
     * Indicates whether to include the feature's geometry for use by the template.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * The template for defining how to format the title used in a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#title)
     */
    title: string | Function | Promise<any>;

    static fromJSON(json: any): PopupTemplate;
  }

  interface PopupTemplateProperties {
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#actions)
     */
    actions?: CollectionProperties<
      (ActionButtonProperties & { type: "button" }) | (ActionToggleProperties & { type: "toggle" })
    >;
    /**
     * The template for defining and formatting a popup's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#content)
     */
    content?: ContentProperties[] | string | Function | Promise<any>;
    /**
     * An array of objects or [ExpressionInfo[]](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) that reference [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions following the specification defined by the [Arcade Popup Profile](https://developers.arcgis.com/javascript/latest/arcade/#popup).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#expressionInfos)
     */
    expressionInfos?: popupExpressionInfoProperties[];
    /**
     * An array of [FieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) that defines how fields in the dataset or values from [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions participate in a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#fieldInfos)
     */
    fieldInfos?: FieldInfoProperties[];
    /**
     * Indicates whether or not editor tracking should display.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#lastEditInfoEnabled)
     */
    lastEditInfoEnabled?: boolean;
    /**
     * Additional options that can be defined for the popup layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#layerOptions)
     */
    layerOptions?: LayerOptionsProperties;
    /**
     * An array of field names used in the PopupTemplate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#outFields)
     */
    outFields?: string[];
    /**
     * Indicates whether actions should replace existing [popup actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#overwriteActions)
     */
    overwriteActions?: boolean;
    /**
     * Indicates whether to include the feature's geometry for use by the template.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * The template for defining how to format the title used in a popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#title)
     */
    title?: string | Function | Promise<any>;
  }

  export interface Portal extends Accessor, Loadable {}

  export class Portal {
    /**
     * The Portal class is part of the [ArcGIS Enterprise portal](https://enterprise.arcgis.com/en/portal/) that provides a way to build applications that work with content from [ArcGIS Online](https://www.arcgis.com/home/) or an [ArcGIS Enterprise portal](https://enterprise.arcgis.com/en/portal/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html)
     */

    constructor(properties?: PortalProperties);

    /**
     * The access level of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#access)
     */
    access: "public" | "private";
    /**
     * When `true`, access to the organization's Portal resources must occur over SSL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#allSSL)
     */
    allSSL: boolean;
    /**
     * The authentication mode for handling authentication when the user attempts to access a secure resource.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#authMode)
     */
    authMode: "anonymous" | "auto" | "immediate" | "no-prompt";
    /**
     * Array of trusted servers to send credentials to when making Cross-Origin Resource Sharing (CORS) requests to access services secured with web-tier authentication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#authorizedCrossOriginDomains)
     */
    authorizedCrossOriginDomains: string[];
    /**
     * The query that defines the basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when [useVectorBasemaps](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps) is not true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery)
     */
    basemapGalleryGroupQuery: string;
    /**
     * The query that defines the 3D basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery3D)
     */
    basemapGalleryGroupQuery3D: string;
    /**
     * The Bing key to use for web maps using Bing Maps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#bingKey)
     */
    bingKey: string;
    /**
     * Indicates whether an organization can list applications in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListApps)
     */
    canListApps: boolean;
    /**
     * Indicates whether an organization can list data services in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListData)
     */
    canListData: boolean;
    /**
     * Indicates whether an organization can list pre-provisioned items in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListPreProvisionedItems)
     */
    canListPreProvisionedItems: boolean;
    /**
     * Indicates whether an organization can provision direct purchases in the marketplace without customer request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canProvisionDirectPurchase)
     */
    canProvisionDirectPurchase: boolean;
    /**
     * When `true`, the organization's public items, groups and users are included in search queries.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSearchPublic)
     */
    canSearchPublic: boolean;
    /**
     * The Bing key can be shared to the public and is returned as part of a portal's description call (`/sharing/rest/portals/<orgid>`).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canShareBingPublic)
     */
    canShareBingPublic: boolean;
    /**
     * When `true`, members of the organization can share resources outside the organization.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSharePublic)
     */
    canSharePublic: boolean;
    /**
     * Indicates whether to allow an organization with an enterprise IDP configured to be able to turn on or off the ArcGIS sign in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSignInArcGIS)
     */
    canSignInArcGIS: boolean;
    /**
     * Indicates whether to allow an organization with an enterprise IDP configured to be able to turn on or off the enterprise sign in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSignInIDP)
     */
    canSignInIDP: boolean;
    /**
     * The query that identifies the group containing the color sets used for rendering in the map viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#colorSetsGroupQuery)
     */
    colorSetsGroupQuery: string;
    /**
     * Indicates whether to allow the organization to disable commenting.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#commentsEnabled)
     */
    commentsEnabled: boolean;
    /**
     * Date the organization was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#created)
     */
    created: Date;
    /**
     * The default locale (language and country) information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#culture)
     */
    culture: string;
    /**
     * The custom base URL for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#customBaseUrl)
     */
    customBaseUrl: string;
    /**
     * The query that defines the default 3d basemap to use in scene views for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#default3DBasemapQuery)
     */
    default3DBasemapQuery: string;
    /**
     * The default basemap to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultBasemap)
     */
    defaultBasemap: Basemap;
    /**
     * The default developer basemap to use for the portal when an [apiKey](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#apiKey) is defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultDevBasemap)
     */
    defaultDevBasemap: Basemap;
    /**
     * The default extent to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultExtent)
     */
    defaultExtent: Extent;
    /**
     * The default vector basemap to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultVectorBasemap)
     */
    defaultVectorBasemap: Basemap;
    /**
     * A description of the organization/portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#description)
     */
    description: string;
    /**
     * The query that defines the basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when an [apiKey](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#apiKey) is defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#devBasemapGalleryGroupQuery)
     */
    devBasemapGalleryGroupQuery: string;
    /**
     * Boolean value indicating whether to opt-in to the [Esri User Experience Improvement (EUEI) program](https://doc.arcgis.com/en/arcgis-online/reference/faq.htm#ESRI_QUESTIONANSWER_AED97F28DCD84F7682623C2FA9E5CE49).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#eueiEnabled)
     */
    eueiEnabled: boolean;
    /**
     * The featured groups for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredGroups)
     */
    featuredGroups: PortalFeaturedGroups[];
    /**
     * The query that defines the featured group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredItemsGroupQuery)
     */
    featuredItemsGroupQuery: string;
    /**
     * The query that identifies the group containing features items for the gallery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#galleryTemplatesGroupQuery)
     */
    galleryTemplatesGroupQuery: string;
    /**
     * Indicates whether the organization has content categories.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#hasCategorySchema)
     */
    hasCategorySchema: boolean;
    /**
     * Indicates whether the organization has classification schema.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#hasClassificationSchema)
     */
    hasClassificationSchema: boolean;
    /**
     * This class contains properties to obtain information for various web services available on the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#helperServices)
     */
    helperServices: any;
    /**
     * The group that contains featured content to be displayed on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#homePageFeaturedContent)
     */
    homePageFeaturedContent: string;
    /**
     * The number of featured items that can be displayed on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#homePageFeaturedContentCount)
     */
    homePageFeaturedContentCount: number;
    /**
     * The port used by the portal for HTTP communication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#httpPort)
     */
    httpPort: number;
    /**
     * The port used by the portal for HTTPS communication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#httpsPort)
     */
    httpsPort: number;
    /**
     * The id of the organization that owns this portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#id)
     */
    id: string;
    /**
     * The country code of the calling IP (ArcGIS Online only).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#ipCntryCode)
     */
    ipCntryCode: string;
    /**
     * Indicates whether the portal is an organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#isOrganization)
     */
    readonly isOrganization: boolean;
    /**
     * Indicates if the portal is on-premises.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#isPortal)
     */
    isPortal: boolean;
    /**
     * Indicates if the portal is in read-only mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#isReadOnly)
     */
    isReadOnly: boolean;
    /**
     * The query that identifies the group containing editing templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#layerTemplatesGroupQuery)
     */
    layerTemplatesGroupQuery: string;
    /**
     * Indicates whether the portal's resources have loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The maximum validity in minutes of tokens issued for users of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#maxTokenExpirationMinutes)
     */
    maxTokenExpirationMinutes: number;
    /**
     * Date the organization was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#modified)
     */
    modified: Date;
    /**
     * Name of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#name)
     */
    name: string;
    /**
     * URL of the portal host.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalHostname)
     */
    portalHostname: string;
    /**
     * The portal mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalMode)
     */
    portalMode: "multitenant" | "singletenant";
    /**
     * Properties specific to the organization, for example the "contact us" link.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalProperties)
     */
    portalProperties: any;
    /**
     * Indicates whether the recycle bin is enabled for the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#recycleBinEnabled)
     */
    recycleBinEnabled: boolean;
    /**
     * The region for the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#region)
     */
    region: string;
    /**
     * The REST URL for the portal, for example "https://www.arcgis.com/sharing/rest" for ArcGIS Online and "https://www.example.com/arcgis/sharing/rest" for your in-house portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#restUrl)
     */
    readonly restUrl: string;
    /**
     * Custom HTML for the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#rotatorPanels)
     */
    rotatorPanels: any[];
    /**
     * Indicates whether the description of your organization displays on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#showHomePageDescription)
     */
    showHomePageDescription: boolean;
    /**
     * The JSON used to create the property values when the `Portal` is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * Indicates whether hosted services are supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#supportsHostedServices)
     */
    supportsHostedServices: boolean;
    /**
     * The query that defines the symbols sets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#symbolSetsGroupQuery)
     */
    symbolSetsGroupQuery: string;
    /**
     * The query that defines the collection of templates that will appear in the template gallery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#templatesGroupQuery)
     */
    templatesGroupQuery: string;
    /**
     * The URL to the thumbnail of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#thumbnailUrl)
     */
    readonly thumbnailUrl: string;
    /**
     * Sets the units of measure for the organization's users.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#units)
     */
    units: "english" | "metric";
    /**
     * The URL to the portal instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#url)
     */
    url: string;
    /**
     * The prefix selected by the organization's administrator to be used with the customBaseURL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#urlKey)
     */
    urlKey: string;
    /**
     * When `false`, 3D basemaps are hidden from the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html), regardless of the type of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#use3dBasemaps)
     */
    use3dBasemaps: boolean;
    /**
     * When `false`, The default 3d basemap is not used in the SceneViewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useDefault3dBasemap)
     */
    useDefault3dBasemap: boolean;
    /**
     * Information representing a registered user of the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#user)
     */
    user: PortalUser;
    /**
     * When `true`, only simple where clauses that are compliant with SQL92 can be used when querying layers and tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useStandardizedQuery)
     */
    useStandardizedQuery: boolean;
    /**
     * When `true`, the organization has opted in to use the vector tile basemaps, and (a) [vectorBasemapGalleryGroupQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#vectorBasemapGalleryGroupQuery) should be used instead of [basemapGalleryGroupQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery), while (b) [defaultVectorBasemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultVectorBasemap) should be used instead of [defaultBasemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultBasemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps)
     */
    useVectorBasemaps: boolean;
    /**
     * The query that defines the vector tiles basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when [useVectorBasemaps](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps) is true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#vectorBasemapGalleryGroupQuery)
     */
    vectorBasemapGalleryGroupQuery: string;

    /**
     * A helper function that returns an array of [ElevationsLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html) derived from the Portal's [Limited Error Raster Compression (LERC) elevation helper service](https://enterprise.arcgis.com/en/portal/latest/administer/windows/about-utility-services.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#createElevationLayers)
     */
    createElevationLayers(): Promise<ElevationLayer[]>;
    /**
     * Fetches the [basemaps](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) that are displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchBasemaps)
     */
    fetchBasemaps(basemapGalleryGroupQuery?: string, options?: PortalFetchBasemapsOptions): Promise<Basemap[]>;
    /**
     * If present, fetches the organization's category schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchCategorySchema)
     */
    fetchCategorySchema(options?: PortalFetchCategorySchemaOptions): Promise<any[]>;
    /**
     * Fetches the default 3d [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) to use in [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) for this portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchDefault3DBasemap)
     */
    fetchDefault3DBasemap(options?: PortalFetchDefault3DBasemapOptions): Promise<Basemap>;
    /**
     * Fetches the featured groups in the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchFeaturedGroups)
     */
    fetchFeaturedGroups(options?: PortalFetchFeaturedGroupsOptions): Promise<PortalGroup[]>;
    /**
     * Fetches and returns the associated regions with the portal instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchRegions)
     */
    fetchRegions(options?: PortalFetchRegionsOptions): Promise<any[]>;
    /**
     * Fetches and returns the portal settings as seen by the current user(s), whether anonymous or signed in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchSettings)
     */
    fetchSettings(options?: PortalFetchSettingsOptions): Promise<any>;
    /**
     * Executes a query against the Portal to return an array of [PortalGroup](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html) objects that match the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryGroups)
     */
    queryGroups(
      queryParams: PortalQueryParams | PortalQueryParamsProperties,
      options?: PortalQueryGroupsOptions,
    ): Promise<PortalQueryResult>;
    /**
     * Executes a query against the Portal to return an array of [PortalItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) objects that match the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryItems)
     */
    queryItems(
      queryParams: PortalQueryParams | PortalQueryParamsProperties,
      options?: PortalQueryItemsOptions,
    ): Promise<PortalQueryResult>;
    /**
     * Executes a query against the Portal to return an array of [PortalUser](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html) objects that match the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryUsers)
     */
    queryUsers(
      queryParams: PortalQueryParams | PortalQueryParamsProperties,
      options?: PortalQueryUsersOptions,
    ): Promise<PortalQueryResult>;

    /**
     * A new `Portal` instance is created the first time this method is called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#getDefault)
     */
    static getDefault(): Portal;
  }

  interface PortalProperties extends LoadableProperties {
    /**
     * The access level of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#access)
     */
    access?: "public" | "private";
    /**
     * When `true`, access to the organization's Portal resources must occur over SSL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#allSSL)
     */
    allSSL?: boolean;
    /**
     * The authentication mode for handling authentication when the user attempts to access a secure resource.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#authMode)
     */
    authMode?: "anonymous" | "auto" | "immediate" | "no-prompt";
    /**
     * Array of trusted servers to send credentials to when making Cross-Origin Resource Sharing (CORS) requests to access services secured with web-tier authentication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#authorizedCrossOriginDomains)
     */
    authorizedCrossOriginDomains?: string[];
    /**
     * The query that defines the basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when [useVectorBasemaps](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps) is not true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery)
     */
    basemapGalleryGroupQuery?: string;
    /**
     * The query that defines the 3D basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery3D)
     */
    basemapGalleryGroupQuery3D?: string;
    /**
     * The Bing key to use for web maps using Bing Maps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#bingKey)
     */
    bingKey?: string;
    /**
     * Indicates whether an organization can list applications in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListApps)
     */
    canListApps?: boolean;
    /**
     * Indicates whether an organization can list data services in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListData)
     */
    canListData?: boolean;
    /**
     * Indicates whether an organization can list pre-provisioned items in the marketplace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canListPreProvisionedItems)
     */
    canListPreProvisionedItems?: boolean;
    /**
     * Indicates whether an organization can provision direct purchases in the marketplace without customer request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canProvisionDirectPurchase)
     */
    canProvisionDirectPurchase?: boolean;
    /**
     * When `true`, the organization's public items, groups and users are included in search queries.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSearchPublic)
     */
    canSearchPublic?: boolean;
    /**
     * The Bing key can be shared to the public and is returned as part of a portal's description call (`/sharing/rest/portals/<orgid>`).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canShareBingPublic)
     */
    canShareBingPublic?: boolean;
    /**
     * When `true`, members of the organization can share resources outside the organization.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSharePublic)
     */
    canSharePublic?: boolean;
    /**
     * Indicates whether to allow an organization with an enterprise IDP configured to be able to turn on or off the ArcGIS sign in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSignInArcGIS)
     */
    canSignInArcGIS?: boolean;
    /**
     * Indicates whether to allow an organization with an enterprise IDP configured to be able to turn on or off the enterprise sign in.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#canSignInIDP)
     */
    canSignInIDP?: boolean;
    /**
     * The query that identifies the group containing the color sets used for rendering in the map viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#colorSetsGroupQuery)
     */
    colorSetsGroupQuery?: string;
    /**
     * Indicates whether to allow the organization to disable commenting.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#commentsEnabled)
     */
    commentsEnabled?: boolean;
    /**
     * Date the organization was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#created)
     */
    created?: DateProperties;
    /**
     * The default locale (language and country) information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#culture)
     */
    culture?: string;
    /**
     * The custom base URL for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#customBaseUrl)
     */
    customBaseUrl?: string;
    /**
     * The query that defines the default 3d basemap to use in scene views for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#default3DBasemapQuery)
     */
    default3DBasemapQuery?: string;
    /**
     * The default basemap to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultBasemap)
     */
    defaultBasemap?: BasemapProperties;
    /**
     * The default developer basemap to use for the portal when an [apiKey](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#apiKey) is defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultDevBasemap)
     */
    defaultDevBasemap?: BasemapProperties;
    /**
     * The default extent to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultExtent)
     */
    defaultExtent?: ExtentProperties;
    /**
     * The default vector basemap to use for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultVectorBasemap)
     */
    defaultVectorBasemap?: BasemapProperties;
    /**
     * A description of the organization/portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#description)
     */
    description?: string;
    /**
     * The query that defines the basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when an [apiKey](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#apiKey) is defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#devBasemapGalleryGroupQuery)
     */
    devBasemapGalleryGroupQuery?: string;
    /**
     * Boolean value indicating whether to opt-in to the [Esri User Experience Improvement (EUEI) program](https://doc.arcgis.com/en/arcgis-online/reference/faq.htm#ESRI_QUESTIONANSWER_AED97F28DCD84F7682623C2FA9E5CE49).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#eueiEnabled)
     */
    eueiEnabled?: boolean;
    /**
     * The featured groups for the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredGroups)
     */
    featuredGroups?: PortalFeaturedGroups[];
    /**
     * The query that defines the featured group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredItemsGroupQuery)
     */
    featuredItemsGroupQuery?: string;
    /**
     * The query that identifies the group containing features items for the gallery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#galleryTemplatesGroupQuery)
     */
    galleryTemplatesGroupQuery?: string;
    /**
     * Indicates whether the organization has content categories.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#hasCategorySchema)
     */
    hasCategorySchema?: boolean;
    /**
     * Indicates whether the organization has classification schema.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#hasClassificationSchema)
     */
    hasClassificationSchema?: boolean;
    /**
     * This class contains properties to obtain information for various web services available on the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#helperServices)
     */
    helperServices?: any;
    /**
     * The group that contains featured content to be displayed on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#homePageFeaturedContent)
     */
    homePageFeaturedContent?: string;
    /**
     * The number of featured items that can be displayed on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#homePageFeaturedContentCount)
     */
    homePageFeaturedContentCount?: number;
    /**
     * The port used by the portal for HTTP communication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#httpPort)
     */
    httpPort?: number;
    /**
     * The port used by the portal for HTTPS communication.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#httpsPort)
     */
    httpsPort?: number;
    /**
     * The id of the organization that owns this portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#id)
     */
    id?: string;
    /**
     * The country code of the calling IP (ArcGIS Online only).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#ipCntryCode)
     */
    ipCntryCode?: string;
    /**
     * Indicates if the portal is on-premises.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#isPortal)
     */
    isPortal?: boolean;
    /**
     * Indicates if the portal is in read-only mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#isReadOnly)
     */
    isReadOnly?: boolean;
    /**
     * The query that identifies the group containing editing templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#layerTemplatesGroupQuery)
     */
    layerTemplatesGroupQuery?: string;
    /**
     * The maximum validity in minutes of tokens issued for users of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#maxTokenExpirationMinutes)
     */
    maxTokenExpirationMinutes?: number;
    /**
     * Date the organization was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#modified)
     */
    modified?: DateProperties;
    /**
     * Name of the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#name)
     */
    name?: string;
    /**
     * URL of the portal host.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalHostname)
     */
    portalHostname?: string;
    /**
     * The portal mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalMode)
     */
    portalMode?: "multitenant" | "singletenant";
    /**
     * Properties specific to the organization, for example the "contact us" link.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#portalProperties)
     */
    portalProperties?: any;
    /**
     * Indicates whether the recycle bin is enabled for the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#recycleBinEnabled)
     */
    recycleBinEnabled?: boolean;
    /**
     * The region for the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#region)
     */
    region?: string;
    /**
     * Custom HTML for the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#rotatorPanels)
     */
    rotatorPanels?: any[];
    /**
     * Indicates whether the description of your organization displays on the home page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#showHomePageDescription)
     */
    showHomePageDescription?: boolean;
    /**
     * The JSON used to create the property values when the `Portal` is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * Indicates whether hosted services are supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#supportsHostedServices)
     */
    supportsHostedServices?: boolean;
    /**
     * The query that defines the symbols sets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#symbolSetsGroupQuery)
     */
    symbolSetsGroupQuery?: string;
    /**
     * The query that defines the collection of templates that will appear in the template gallery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#templatesGroupQuery)
     */
    templatesGroupQuery?: string;
    /**
     * Sets the units of measure for the organization's users.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#units)
     */
    units?: "english" | "metric";
    /**
     * The URL to the portal instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#url)
     */
    url?: string;
    /**
     * The prefix selected by the organization's administrator to be used with the customBaseURL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#urlKey)
     */
    urlKey?: string;
    /**
     * When `false`, 3D basemaps are hidden from the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html), regardless of the type of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#use3dBasemaps)
     */
    use3dBasemaps?: boolean;
    /**
     * When `false`, The default 3d basemap is not used in the SceneViewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useDefault3dBasemap)
     */
    useDefault3dBasemap?: boolean;
    /**
     * Information representing a registered user of the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#user)
     */
    user?: PortalUserProperties;
    /**
     * When `true`, only simple where clauses that are compliant with SQL92 can be used when querying layers and tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useStandardizedQuery)
     */
    useStandardizedQuery?: boolean;
    /**
     * When `true`, the organization has opted in to use the vector tile basemaps, and (a) [vectorBasemapGalleryGroupQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#vectorBasemapGalleryGroupQuery) should be used instead of [basemapGalleryGroupQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery), while (b) [defaultVectorBasemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultVectorBasemap) should be used instead of [defaultBasemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#defaultBasemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps)
     */
    useVectorBasemaps?: boolean;
    /**
     * The query that defines the vector tiles basemaps that should be displayed in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) when [useVectorBasemaps](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#useVectorBasemaps) is true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#vectorBasemapGalleryGroupQuery)
     */
    vectorBasemapGalleryGroupQuery?: string;
  }

  export interface PortalFeaturedGroups {
    /**
     * Name of the group owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredGroups)
     */
    owner: string;
    /**
     * Group title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#featuredGroups)
     */
    title: string;
  }

  export interface PortalFetchBasemapsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchBasemaps)
     */
    signal?: AbortSignal;
    /**
     * When `true` the basemaps based on [basemapGalleryGroupQuery3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#basemapGalleryGroupQuery3D) are also fetched, if no `basemapGalleryGroupQuery` is passed as an argument.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchBasemaps)
     */
    include3d?: boolean;
  }

  export interface PortalFetchCategorySchemaOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchCategorySchema)
     */
    signal?: AbortSignal;
  }

  export interface PortalFetchDefault3DBasemapOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchDefault3DBasemap)
     */
    signal?: AbortSignal;
  }

  export interface PortalFetchFeaturedGroupsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchFeaturedGroups)
     */
    signal?: AbortSignal;
  }

  export interface PortalFetchRegionsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchRegions)
     */
    signal?: AbortSignal;
  }

  export interface PortalFetchSettingsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#fetchSettings)
     */
    signal?: AbortSignal;
  }

  export interface PortalQueryGroupsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryGroups)
     */
    signal?: AbortSignal;
  }

  export interface PortalQueryItemsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryItems)
     */
    signal?: AbortSignal;
  }

  export interface PortalQueryUsersOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html#queryUsers)
     */
    signal?: AbortSignal;
  }

  export class PortalFolder extends Accessor {
    /**
     * Provides information about folders used to organize content in a portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html)
     */

    constructor(properties?: PortalFolderProperties);

    /**
     * The date the folder was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#created)
     */
    created: Date;
    /**
     * The unique id of the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#id)
     */
    id: string;
    /**
     * The portal associated with the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#portal)
     */
    portal: Portal;
    /**
     * The title of the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#title)
     */
    title: string;
    /**
     * The URL to the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#url)
     */
    readonly url: string;
  }

  interface PortalFolderProperties {
    /**
     * The date the folder was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#created)
     */
    created?: DateProperties;
    /**
     * The unique id of the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#id)
     */
    id?: string;
    /**
     * The portal associated with the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#portal)
     */
    portal?: PortalProperties;
    /**
     * The title of the folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalFolder.html#title)
     */
    title?: string;
  }

  export class PortalGroup extends Accessor {
    /**
     * The group resource represents a group within the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html)
     */

    constructor(properties?: PortalGroupProperties);

    /**
     * The access privileges on the group which determines who can see and access the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#access)
     */
    access: "private" | "org" | "public";
    /**
     * The date the group was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#created)
     */
    created: Date;
    /**
     * A detailed description of the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#description)
     */
    description: string;
    /**
     * The unique id for the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#id)
     */
    id: string;
    /**
     * If set to `true`, then users will not be able to apply to join the group.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#isInvitationOnly)
     */
    isInvitationOnly: boolean;
    /**
     * The date the group was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#modified)
     */
    modified: Date;
    /**
     * The username of the group's owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#owner)
     */
    owner: string;
    /**
     * The portal associated with the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#portal)
     */
    portal: Portal;
    /**
     * A short summary that describes the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#snippet)
     */
    snippet: string;
    /**
     * The JSON used to create the property values when the `PortalGroup` is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * User defined tags that describe the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#tags)
     */
    tags: string[];
    /**
     * The URL to the thumbnail used for the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#thumbnailUrl)
     */
    readonly thumbnailUrl: string;
    /**
     * The title of the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#title)
     */
    title: string;
    /**
     * The URL to the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#url)
     */
    readonly url: string;

    /**
     * If present, fetches the group's category schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#fetchCategorySchema)
     */
    fetchCategorySchema(options?: PortalGroupFetchCategorySchemaOptions): Promise<any[]>;
    /**
     * Fetches the current members of the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#fetchMembers)
     */
    fetchMembers(options?: PortalGroupFetchMembersOptions): Promise<any>;
    /**
     * Get the URL to the thumbnail image for the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#getThumbnailUrl)
     */
    getThumbnailUrl(width?: number): string;
    /**
     * Executes a query against the group to return an array of [PortalItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) objects that match the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#queryItems)
     */
    queryItems(
      queryParams?: PortalQueryParams | PortalQueryParamsProperties,
      options?: PortalGroupQueryItemsOptions,
    ): Promise<PortalQueryResult>;
  }

  interface PortalGroupProperties {
    /**
     * The access privileges on the group which determines who can see and access the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#access)
     */
    access?: "private" | "org" | "public";
    /**
     * The date the group was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#created)
     */
    created?: DateProperties;
    /**
     * A detailed description of the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#description)
     */
    description?: string;
    /**
     * The unique id for the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#id)
     */
    id?: string;
    /**
     * If set to `true`, then users will not be able to apply to join the group.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#isInvitationOnly)
     */
    isInvitationOnly?: boolean;
    /**
     * The date the group was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#modified)
     */
    modified?: DateProperties;
    /**
     * The username of the group's owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#owner)
     */
    owner?: string;
    /**
     * The portal associated with the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#portal)
     */
    portal?: PortalProperties;
    /**
     * A short summary that describes the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#snippet)
     */
    snippet?: string;
    /**
     * The JSON used to create the property values when the `PortalGroup` is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * User defined tags that describe the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#tags)
     */
    tags?: string[];
    /**
     * The title of the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#title)
     */
    title?: string;
  }

  export interface PortalGroupFetchCategorySchemaOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#fetchCategorySchema)
     */
    signal?: AbortSignal;
  }

  export interface PortalGroupFetchMembersOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#fetchMembers)
     */
    signal?: AbortSignal;
  }

  export interface PortalGroupQueryItemsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalGroup.html#queryItems)
     */
    signal?: AbortSignal;
  }

  export interface PortalItem extends Accessor, Loadable, JSONSupport {}

  export class PortalItem {
    /**
     * An item (a unit of content) in the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html)
     */

    constructor(properties?: PortalItemProperties);

    /**
     * Indicates the level of access to this item: `private`, `shared`, `org`, or `public`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#access)
     */
    access: "private" | "shared" | "org" | "public";
    /**
     * Information on the source of the item and its copyright status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#accessInformation)
     */
    accessInformation: string;
    /**
     * An authorization string used to access the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#apiKey)
     */
    apiKey: string;
    /**
     * Contains an array of objects containing proxy information for premium platform services.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#applicationProxies)
     */
    readonly applicationProxies: PortalItemApplicationProxies[];
    /**
     * Average rating.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#avgRating)
     */
    avgRating: number;
    /**
     * An array of organization categories that are set on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#categories)
     */
    categories: string[];
    /**
     * The classification information for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#classification)
     */
    classification: HashMap<any>;
    /**
     * The date the item was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#created)
     */
    created: Date;
    /**
     * The item's locale information (language and country).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#culture)
     */
    culture: string;
    /**
     * The detailed description of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#description)
     */
    description: string;
    /**
     * The geographic extent, or bounding rectangle, of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#extent)
     */
    extent: Extent;
    /**
     * An array of group categories set on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#groupCategories)
     */
    groupCategories: string[];
    /**
     * The unique id for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#id)
     */
    id: string;
    /**
     * Indicates whether a layer can be created from this item using [Layer.fromPortalItem()](esri-layers-Layer.html#fromPortalItem).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#isLayer)
     */
    readonly isLayer: boolean;
    /**
     * Indicates whether this item and the user whose credential was used to fetch this item belong to the same ArcGIS Enterprise Portal or ArcGIS Online Organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#isOrgItem)
     */
    readonly isOrgItem: boolean;
    /**
     * Indicates whether an item can be updated and deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#itemControl)
     */
    readonly itemControl: "admin" | "update" | "null";
    /**
     * The URL to the [Item page](https://doc.arcgis.com/en/arcgis-online/manage-data/item-details.htm) on the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#itemPageUrl)
     */
    readonly itemPageUrl: string;
    /**
     * The URL to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#itemUrl)
     */
    readonly itemUrl: string;
    /**
     * Information on license or restrictions related to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#licenseInfo)
     */
    licenseInfo: string;
    /**
     * Indicates whether the item's resources have loaded from the portal.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The date the item was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#modified)
     */
    modified: Date;
    /**
     * The name of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#name)
     */
    name: string;
    /**
     * Number of comments on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numComments)
     */
    numComments: number;
    /**
     * Number of ratings on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numRatings)
     */
    numRatings: number;
    /**
     * Number of views on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numViews)
     */
    numViews: number;
    /**
     * The username of the user who owns this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#owner)
     */
    owner: string;
    /**
     * The ID of the folder in which the owner has stored the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#ownerFolder)
     */
    ownerFolder: string;
    /**
     * The portal that contains the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#portal)
     */
    portal: Portal;
    /**
     * An array of string URLs.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#screenshots)
     */
    screenshots: string[];
    /**
     * The size of the item (in bytes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#size)
     */
    size: number;
    /**
     * A summary description of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#snippet)
     */
    snippet: string;
    /**
     * The JSON used to create the property values when the `PortalItem` is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * User defined tags that describe the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#tags)
     */
    tags: string[];
    /**
     * The URL to the thumbnail used for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#thumbnailUrl)
     */
    readonly thumbnailUrl: string;
    /**
     * The title for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#title)
     */
    title: string;
    /**
     * The GIS content type of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#type)
     */
    type: string;
    /**
     * Type keywords that describe the type of content of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#typeKeywords)
     */
    typeKeywords: string[];
    /**
     * The service URL of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#url)
     */
    url: string;

    /**
     * Adds a rating to an accessible item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#addRating)
     */
    addRating(rating: number | PortalRating): Promise<PortalRating>;
    /**
     * Adds a new [resource](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html) to the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#addResource)
     */
    addResource(resource: PortalItemResource, content: Blob, options?: PortalItemAddResourceOptions): Promise<any>;
    /**
     * Creates a clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#clone)
     */
    clone(): PortalItem;
    /**
     * Deletes a rating for the specified item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#deleteRating)
     */
    deleteRating(): Promise<any>;
    /**
     * Destroys the portal item, and any associated resources, including its associated [portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#portal).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#destroy)
     */
    destroy(): void;
    /**
     * Requests a PortalItem in the format specified in `responseType`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchData)
     */
    fetchData(
      responseType?: "json" | "xml" | "text" | "blob" | "array-buffer" | "document",
      options?: PortalItemFetchDataOptions,
    ): Promise<any>;
    /**
     * Returns the rating (if any) given to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRating)
     */
    fetchRating(options?: PortalItemFetchRatingOptions): Promise<PortalRating>;
    /**
     * Gets all the related items of a certain relationship type for the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRelatedItems)
     */
    fetchRelatedItems(
      params: PortalItemFetchRelatedItemsParams,
      options?: PortalItemFetchRelatedItemsOptions,
    ): Promise<PortalItem[]>;
    /**
     * Retrieves references to all the [portal item resources](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchResources)
     */
    fetchResources(
      params?: FetchResourcesParams,
      options?: PortalItemFetchResourcesOptions,
    ): Promise<FetchResourcesResult>;
    /**
     * Get the URL to the thumbnail image for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#getThumbnailUrl)
     */
    getThumbnailUrl(width?: number): string;
    /**
     * Reloads a loaded item's properties from the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#reload)
     */
    reload(): Promise<PortalItem>;
    /**
     * Removes all the [resources](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html) from the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#removeAllResources)
     */
    removeAllResources(options?: PortalItemRemoveAllResourcesOptions): Promise<any>;
    /**
     * Removes a [resource](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html) from the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#removeResource)
     */
    removeResource(resource: PortalItemResource, options?: PortalItemRemoveResourceOptions): Promise<any>;
    /**
     * Updates the item's properties to the portal, and optionally its data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#update)
     */
    update(params?: PortalItemUpdateParams): Promise<PortalItem>;
    /**
     * Updates the item's thumbnail on the portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#updateThumbnail)
     */
    updateThumbnail(params: PortalItemUpdateThumbnailParams): Promise<PortalItem>;

    static fromJSON(json: any): PortalItem;
  }

  interface PortalItemProperties extends LoadableProperties {
    /**
     * Indicates the level of access to this item: `private`, `shared`, `org`, or `public`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#access)
     */
    access?: "private" | "shared" | "org" | "public";
    /**
     * Information on the source of the item and its copyright status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#accessInformation)
     */
    accessInformation?: string;
    /**
     * An authorization string used to access the portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#apiKey)
     */
    apiKey?: string;
    /**
     * Average rating.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#avgRating)
     */
    avgRating?: number;
    /**
     * An array of organization categories that are set on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#categories)
     */
    categories?: string[];
    /**
     * The classification information for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#classification)
     */
    classification?: HashMap<any>;
    /**
     * The date the item was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#created)
     */
    created?: DateProperties;
    /**
     * The item's locale information (language and country).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#culture)
     */
    culture?: string;
    /**
     * The detailed description of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#description)
     */
    description?: string;
    /**
     * The geographic extent, or bounding rectangle, of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * An array of group categories set on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#groupCategories)
     */
    groupCategories?: string[];
    /**
     * The unique id for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#id)
     */
    id?: string;
    /**
     * Information on license or restrictions related to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#licenseInfo)
     */
    licenseInfo?: string;
    /**
     * The date the item was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#modified)
     */
    modified?: DateProperties;
    /**
     * The name of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#name)
     */
    name?: string;
    /**
     * Number of comments on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numComments)
     */
    numComments?: number;
    /**
     * Number of ratings on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numRatings)
     */
    numRatings?: number;
    /**
     * Number of views on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#numViews)
     */
    numViews?: number;
    /**
     * The username of the user who owns this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#owner)
     */
    owner?: string;
    /**
     * The ID of the folder in which the owner has stored the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#ownerFolder)
     */
    ownerFolder?: string;
    /**
     * The portal that contains the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#portal)
     */
    portal?: PortalProperties;
    /**
     * An array of string URLs.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#screenshots)
     */
    screenshots?: string[];
    /**
     * The size of the item (in bytes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#size)
     */
    size?: number;
    /**
     * A summary description of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#snippet)
     */
    snippet?: string;
    /**
     * The JSON used to create the property values when the `PortalItem` is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * User defined tags that describe the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#tags)
     */
    tags?: string[];
    /**
     * The title for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#title)
     */
    title?: string;
    /**
     * The GIS content type of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#type)
     */
    type?: string;
    /**
     * Type keywords that describe the type of content of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#typeKeywords)
     */
    typeKeywords?: string[];
    /**
     * The service URL of this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#url)
     */
    url?: string;
  }

  /**
   * A resource reference returned in the result of [fetchResources()](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchResources).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResource)
   */
  export interface FetchResource {
    /**
     * Relative path of the resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResource)
     */
    resource: PortalItemResource;
    /**
     * When the resource was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResource)
     */
    created: Date;
    /**
     * The size of the resource in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResource)
     */
    size: number;
  }

  /**
   * Parameters used when fetching portal item resources using [fetchResources()](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchResources).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesParams)
   */
  export interface FetchResourcesParams {
    /**
     * The maximum number of results to be included in the result set response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesParams)
     */
    num?: number;
    /**
     * The index of the first entry in the result set response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesParams)
     */
    start?: number;
    /**
     * The order in which to sort the results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesParams)
     */
    sortOrder?: "asc" | "desc";
    /**
     * A field to sort the results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesParams)
     */
    sortField?: "created" | "size" | "resource";
  }

  /**
   * Object returned when [fetchResources()](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchResources) promise resolves.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesResult)
   */
  export interface FetchResourcesResult {
    /**
     * The fetched resources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesResult)
     */
    resources: FetchResource[];
    /**
     * The next entry index if the current result set doesn't contain all results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesResult)
     */
    nextStart: number;
    /**
     * The total number of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#FetchResourcesResult)
     */
    total: number;
  }

  export interface PortalItemAddResourceOptions {
    /**
     * Indicates the level of access to the resource.
     *
     * @default "inherit"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#addResource)
     */
    access?: "inherit" | "private";
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#addResource)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemApplicationProxies {
    /**
     * The URL of the premium map service layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#applicationProxies)
     */
    sourceUrl: string;
    /**
     * The proxy URL for the source URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#applicationProxies)
     */
    proxyUrl: string;
    /**
     * The proxy ID registered in ArcGIS Online or ArcGIS Enterprise Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#applicationProxies)
     */
    proxyId: string;
  }

  export interface PortalItemFetchDataOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchData)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemFetchRatingOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRating)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemFetchRelatedItemsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRelatedItems)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemFetchRelatedItemsParams {
    /**
     * The type of relationship between the two items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRelatedItems)
     */
    relationshipType: string;
    /**
     * The direction of the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchRelatedItems)
     */
    direction: "forward" | "reverse";
  }

  export interface PortalItemFetchResourcesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#fetchResources)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemRemoveAllResourcesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#removeAllResources)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemRemoveResourceOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#removeResource)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemUpdateParams {
    /**
     * **Optional**.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#update)
     */
    data?: string | any;
  }

  export interface PortalItemUpdateThumbnailParams {
    /**
     * A URL, Data URI, Blob, or File.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#updateThumbnail)
     */
    thumbnail: Blob | string;
    /**
     * The file name used for the thumbnail in [thumbnailUrl](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#thumbnailUrl).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html#updateThumbnail)
     */
    filename?: string;
  }

  export class PortalItemResource extends Accessor {
    /**
     * A reference to a portal item resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html)
     */

    constructor(properties?: PortalItemResourceProperties);

    /**
     * Path of the resource relative to `{ITEM}/resources/`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#path)
     */
    path: string;
    /**
     * The [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) that owns the resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * The absolute url to the item resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#url)
     */
    readonly url: string;

    /**
     * Requests the PortalItemResource data in the format specified for the `responseType`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#fetch)
     */
    fetch(
      responseType?: "json" | "xml" | "text" | "blob" | "array-buffer" | "document",
      options?: PortalItemResourceFetchOptions,
    ): Promise<any>;
    /**
     * Updates an existing resource with new content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#update)
     */
    update(content: Blob, options?: PortalItemResourceUpdateOptions): Promise<any>;
  }

  interface PortalItemResourceProperties {
    /**
     * Path of the resource relative to `{ITEM}/resources/`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#path)
     */
    path?: string;
    /**
     * The [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) that owns the resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#portalItem)
     */
    portalItem?: PortalItemProperties;
  }

  export interface PortalItemResourceFetchOptions {
    /**
     * If `true`, the browser will send a request to the server instead of using the browser's local cache.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#fetch)
     */
    cacheBust?: boolean;
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#fetch)
     */
    signal?: AbortSignal;
  }

  export interface PortalItemResourceUpdateOptions {
    /**
     * Indicates the level of access to the resource.
     *
     * @default "inherit"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#update)
     */
    access?: "inherit" | "private";
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItemResource.html#update)
     */
    signal?: AbortSignal;
  }

  export class PortalQueryParams extends Accessor {
    /**
     * The parameters used to perform a query for Items, Groups, and Users within a [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html)
     */

    constructor(properties?: PortalQueryParamsProperties);

    /**
     * An array of categories stored within the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#categories)
     */
    categories: (string | string[])[];
    /**
     * Only relevant when querying for [PortalItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#extent)
     */
    extent: Extent;
    /**
     * Structured filter to use instead of the [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#query) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#filter)
     */
    filter: string;
    /**
     * The maximum number of results to be included in the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#results) set response.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#num)
     */
    num: number;
    /**
     * The query string used for the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#query)
     */
    query: string;
    /**
     * A comma-delimited list of fields to sort.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#sortField)
     */
    sortField:
      | "title"
      | "uploaded"
      | "modified"
      | "username"
      | "created"
      | "type"
      | "owner"
      | "avg-rating"
      | "num-ratings"
      | "num-comments"
      | "num-views";
    /**
     * The order in which to sort the results.
     *
     * @default "asc"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#sortOrder)
     */
    sortOrder: "asc" | "desc";
    /**
     * The index of the first entry in the result set response.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#start)
     */
    start: number;

    /**
     * Creates a deep clone of the instance of PortalQueryParams that calls this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#clone)
     */
    clone(): PortalQueryParams;
  }

  interface PortalQueryParamsProperties {
    /**
     * An array of categories stored within the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#categories)
     */
    categories?: (string | string[])[];
    /**
     * Only relevant when querying for [PortalItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Structured filter to use instead of the [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#query) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#filter)
     */
    filter?: string;
    /**
     * The maximum number of results to be included in the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#results) set response.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#num)
     */
    num?: number;
    /**
     * The query string used for the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#query)
     */
    query?: string;
    /**
     * A comma-delimited list of fields to sort.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#sortField)
     */
    sortField?:
      | "title"
      | "uploaded"
      | "modified"
      | "username"
      | "created"
      | "type"
      | "owner"
      | "avg-rating"
      | "num-ratings"
      | "num-comments"
      | "num-views";
    /**
     * The order in which to sort the results.
     *
     * @default "asc"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#sortOrder)
     */
    sortOrder?: "asc" | "desc";
    /**
     * The index of the first entry in the result set response.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryParams.html#start)
     */
    start?: number;
  }

  export class PortalQueryResult extends Accessor {
    /**
     * Represents the result object returned from a portal query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html)
     */

    constructor(properties?: PortalQueryResultProperties);

    /**
     * The query parameters for the next set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#nextQueryParams)
     */
    nextQueryParams: PortalQueryParams;
    /**
     * The query parameters for the first set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#queryParams)
     */
    queryParams: PortalQueryParams;
    /**
     * An array of result item objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#results)
     */
    results: any[];
    /**
     * The total number of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#total)
     */
    total: number;
  }

  interface PortalQueryResultProperties {
    /**
     * The query parameters for the next set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#nextQueryParams)
     */
    nextQueryParams?: PortalQueryParamsProperties;
    /**
     * The query parameters for the first set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#queryParams)
     */
    queryParams?: PortalQueryParamsProperties;
    /**
     * An array of result item objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#results)
     */
    results?: any[];
    /**
     * The total number of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalQueryResult.html#total)
     */
    total?: number;
  }

  export class PortalRating extends Accessor {
    /**
     * PortalRating provides details about the rating associated with a [Portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalRating.html)
     */

    constructor(properties?: PortalRatingProperties);

    /**
     * Date the rating was added to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalRating.html#created)
     */
    created: Date;
    /**
     * A rating between 1.0 and 5.0 for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalRating.html#rating)
     */
    rating: number;
  }

  interface PortalRatingProperties {
    /**
     * Date the rating was added to the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalRating.html#created)
     */
    created?: DateProperties;
    /**
     * A rating between 1.0 and 5.0 for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalRating.html#rating)
     */
    rating?: number;
  }

  export class PortalUser extends Accessor {
    /**
     * Represents a registered user of the [Portal](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-Portal.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html)
     */

    constructor(properties?: PortalUserProperties);

    /**
     * Indicates the level of access of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#access)
     */
    access: "private" | "org" | "public";
    /**
     * The date the user was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#created)
     */
    created: Date;
    /**
     * The culture information for the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#culture)
     */
    culture: string;
    /**
     * A description of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#description)
     */
    description: string;
    /**
     * The user's e-mail address.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#email)
     */
    email: string;
    /**
     * The user's full name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fullName)
     */
    fullName: string;
    /**
     * The date the user was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#modified)
     */
    modified: Date;
    /**
     * The ID of the organization the user belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#orgId)
     */
    orgId: string;
    /**
     * The portal associated with the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#portal)
     */
    portal: Portal;
    /**
     * The user's preferred view for content, either web or GIS.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#preferredView)
     */
    preferredView: string;
    /**
     * The user's privileges based on their user type or role in their organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#privileges)
     */
    privileges: string[];
    /**
     * The user preferred region, used to set the featured maps on the home page, content in the gallery, and the default extent of new maps in the Viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#region)
     */
    region: string;
    /**
     * Defines the user's role in the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#role)
     */
    role: "org_admin" | "org_publisher" | "org_user";
    /**
     * The ID of the user's role.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#roleId)
     */
    roleId: string;
    /**
     * The JSON used to create the property values when the `PortalUser` is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#sourceJSON)
     */
    sourceJSON: any;
    /**
     * The URL to the thumbnail image for the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#thumbnailUrl)
     */
    readonly thumbnailUrl: string;
    /**
     * The user's personal units of measure setting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#units)
     */
    units: "english" | "metric";
    /**
     * The URL for the user's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#userContentUrl)
     */
    readonly userContentUrl: string;
    /**
     * The username of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#username)
     */
    username: string;

    /**
     * Adds an item to the user's portal content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#addItem)
     */
    addItem(params: PortalUserAddItemParams): Promise<PortalItem>;
    /**
     * Deletes an item from the user's portal content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#deleteItem)
     */
    deleteItem(item: PortalItem, permanentDelete?: boolean): Promise<any>;
    /**
     * Deletes items from the user's portal content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#deleteItems)
     */
    deleteItems(items: PortalItem[], permanentDelete?: boolean): Promise<DeleteItemsResult[]>;
    /**
     * Fetches all of the user's folders used to organize portal content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchFolders)
     */
    fetchFolders(): Promise<PortalFolder[]>;
    /**
     * Fetches all the groups that the portal user has permission to access.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchGroups)
     */
    fetchGroups(): Promise<PortalGroup[]>;
    /**
     * Retrieves all the items in either the user's root folder or the specified folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    fetchItems(params?: PortalUserFetchItemsParams): Promise<FetchItemsResult>;
    /**
     * Fetches the tag objects that have been created by the portal user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchTags)
     */
    fetchTags(): Promise<any[]>;
    /**
     * Get the URL to the thumbnail image for the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#getThumbnailUrl)
     */
    getThumbnailUrl(width?: number): string;
    /**
     * Executes a query against the user's favorite group to return an array of [PortalItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) objects that match the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#queryFavorites)
     */
    queryFavorites(queryParams?: PortalQueryParams): Promise<PortalQueryResult>;
    /**
     * Restores an item from the user's recycle bin to their content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#restoreItem)
     */
    restoreItem(item: PortalItem, folder?: PortalFolder | string): Promise<any>;
  }

  interface PortalUserProperties {
    /**
     * Indicates the level of access of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#access)
     */
    access?: "private" | "org" | "public";
    /**
     * The date the user was created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#created)
     */
    created?: DateProperties;
    /**
     * The culture information for the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#culture)
     */
    culture?: string;
    /**
     * A description of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#description)
     */
    description?: string;
    /**
     * The user's e-mail address.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#email)
     */
    email?: string;
    /**
     * The user's full name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fullName)
     */
    fullName?: string;
    /**
     * The date the user was last modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#modified)
     */
    modified?: DateProperties;
    /**
     * The ID of the organization the user belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#orgId)
     */
    orgId?: string;
    /**
     * The portal associated with the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#portal)
     */
    portal?: PortalProperties;
    /**
     * The user's preferred view for content, either web or GIS.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#preferredView)
     */
    preferredView?: string;
    /**
     * The user's privileges based on their user type or role in their organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#privileges)
     */
    privileges?: string[];
    /**
     * The user preferred region, used to set the featured maps on the home page, content in the gallery, and the default extent of new maps in the Viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#region)
     */
    region?: string;
    /**
     * Defines the user's role in the organization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#role)
     */
    role?: "org_admin" | "org_publisher" | "org_user";
    /**
     * The ID of the user's role.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#roleId)
     */
    roleId?: string;
    /**
     * The JSON used to create the property values when the `PortalUser` is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#sourceJSON)
     */
    sourceJSON?: any;
    /**
     * The user's personal units of measure setting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#units)
     */
    units?: "english" | "metric";
    /**
     * The username of the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#username)
     */
    username?: string;
  }

  /**
   * The result of the [`deleteItems()`](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#deleteItems) method containing the [item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html), if deletion is successful, and error, if any.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#DeleteItemsResult)
   */
  export interface DeleteItemsResult {
    /**
     * An item from the `items` parameter of [`deleteItems()`](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#deleteItems).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#DeleteItemsResult)
     */
    item: PortalItem;
    /**
     * Indicates whether the item was successfully deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#DeleteItemsResult)
     */
    success: boolean;
    /**
     * The error if the item was not deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#DeleteItemsResult)
     */
    error?: Error;
  }

  /**
   * The result of the [`fetchItems()`](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems) method containing an array of the fetched [portal items](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html), the next entry index, and the total number of results.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#FetchItemsResult)
   */
  export interface FetchItemsResult {
    /**
     * An array containing user's portal items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#FetchItemsResult)
     */
    items: PortalItem[];
    /**
     * The next entry index if the current result set doesn't contain all results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#FetchItemsResult)
     */
    nextStart: number;
    /**
     * The total number of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#FetchItemsResult)
     */
    total: number;
  }

  export interface PortalUserAddItemParams {
    /**
     * The item to add to the user's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#addItem)
     */
    item: PortalItem;
    /**
     * The component used to stream the data represented by the item to the client.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#addItem)
     */
    data?: string | any;
    /**
     * The portal folder in which to store the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#addItem)
     */
    folder?: string | PortalFolder;
  }

  export interface PortalUserFetchItemsParams {
    /**
     * The folder to retrieve items from.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    folder?: PortalFolder;
    /**
     * Option to retrieve items from the recycle bin instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    inRecycleBin?: boolean;
    /**
     * Option to include items from subfolders along with items in the root folder.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    includeSubfolderItems?: boolean;
    /**
     * The maximum number of results to be included in the result set response.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    num?: number;
    /**
     * A comma-delimited list of fields to sort by.
     *
     * @default created
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    sortField?: string;
    /**
     * The order in which to sort the results.
     *
     * @default asc
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    sortOrder?: string;
    /**
     * The index of the first entry in the result set response.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalUser.html#fetchItems)
     */
    start?: number;
  }

  /**
   * A convenience module for importing renderer classes that can be used to render [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html)
   */
  namespace rasterRenderers {
    /**
     * FlowRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#FlowRenderer)
     */
    export type FlowRenderer = __esri.FlowRenderer;
    export const FlowRenderer: typeof __esri.FlowRenderer;

    /**
     * ClassBreaksRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#ClassBreaksRenderer)
     */
    export type ClassBreaksRenderer = __esri.ClassBreaksRenderer;
    export const ClassBreaksRenderer: typeof __esri.ClassBreaksRenderer;

    /**
     * UniqueValueRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#UniqueValueRenderer)
     */
    export type UniqueValueRenderer = __esri.UniqueValueRenderer;
    export const UniqueValueRenderer: typeof __esri.UniqueValueRenderer;

    /**
     * RasterColormapRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterColormapRenderer)
     */
    export type RasterColormapRenderer = __esri.RasterColormapRenderer;
    export const RasterColormapRenderer: typeof __esri.RasterColormapRenderer;

    /**
     * RasterStretchRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterStretchRenderer)
     */
    export type RasterStretchRenderer = __esri.RasterStretchRenderer;
    export const RasterStretchRenderer: typeof __esri.RasterStretchRenderer;

    /**
     * VectorFieldRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#VectorFieldRenderer)
     */
    export type VectorFieldRenderer = __esri.VectorFieldRenderer;
    export const VectorFieldRenderer: typeof __esri.VectorFieldRenderer;

    /**
     * RasterShadedReliefRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterShadedReliefRenderer)
     */
    export type RasterShadedReliefRenderer = __esri.RasterShadedReliefRenderer;
    export const RasterShadedReliefRenderer: typeof __esri.RasterShadedReliefRenderer;
  }

  /**
   * ClassBreaksRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#ClassBreaksRenderer)
   */
  export type rasterRenderersClassBreaksRenderer = ClassBreaksRenderer;

  /**
   * FlowRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#FlowRenderer)
   */
  export type rasterRenderersFlowRenderer = FlowRenderer;

  /**
   * RasterColormapRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterColormapRenderer)
   */
  export type rasterRenderersRasterColormapRenderer = RasterColormapRenderer;

  /**
   * RasterShadedReliefRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterShadedReliefRenderer)
   */
  export type rasterRenderersRasterShadedReliefRenderer = RasterShadedReliefRenderer;

  /**
   * RasterStretchRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#RasterStretchRenderer)
   */
  export type rasterRenderersRasterStretchRenderer = RasterStretchRenderer;

  /**
   * UniqueValueRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#UniqueValueRenderer)
   */
  export type rasterRenderersUniqueValueRenderer = UniqueValueRenderer;

  /**
   * VectorFieldRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rasterRenderers.html#VectorFieldRenderer)
   */
  export type rasterRenderersVectorFieldRenderer = VectorFieldRenderer;

  /**
   * A convenience module for importing [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html)
   */
  namespace renderers {
    /**
     * DictionaryRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#DictionaryRenderer)
     */
    export type DictionaryRenderer = __esri.DictionaryRenderer;
    export const DictionaryRenderer: typeof __esri.DictionaryRenderer;

    /**
     * SimpleRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#SimpleRenderer)
     */
    export type SimpleRenderer = __esri.SimpleRenderer;
    export const SimpleRenderer: typeof __esri.SimpleRenderer;

    /**
     * ClassBreaksRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#ClassBreaksRenderer)
     */
    export type ClassBreaksRenderer = __esri.ClassBreaksRenderer;
    export const ClassBreaksRenderer: typeof __esri.ClassBreaksRenderer;

    /**
     * UniqueValueRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#UniqueValueRenderer)
     */
    export type UniqueValueRenderer = __esri.UniqueValueRenderer;
    export const UniqueValueRenderer: typeof __esri.UniqueValueRenderer;

    /**
     * DotDensityRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#DotDensityRenderer)
     */
    export type DotDensityRenderer = __esri.DotDensityRenderer;
    export const DotDensityRenderer: typeof __esri.DotDensityRenderer;

    /**
     * PieChartRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#PieChartRenderer)
     */
    export type PieChartRenderer = __esri.PieChartRenderer;
    export const PieChartRenderer: typeof __esri.PieChartRenderer;

    /**
     * RendererWithVisualVariables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#RendererWithVisualVariables)
     */
    export type RendererWithVisualVariables =
      | __esri.SimpleRenderer
      | __esri.ClassBreaksRenderer
      | __esri.UniqueValueRenderer
      | __esri.DotDensityRenderer
      | __esri.DictionaryRenderer
      | __esri.PieChartRenderer;

    /**
     * HeatmapRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#HeatmapRenderer)
     */
    export type HeatmapRenderer = __esri.HeatmapRenderer;
    export const HeatmapRenderer: typeof __esri.HeatmapRenderer;

    /**
     * Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#Renderer)
     */
    export type Renderer = __esri.HeatmapRenderer | __esri.RendererWithVisualVariables;
  }

  export interface ClassBreaksRenderer extends Renderer, VisualVariablesMixin {}

  export class ClassBreaksRenderer {
    /**
     * ClassBreaksRenderer defines the symbol of each feature in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) based on the value of a numeric attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html)
     */

    constructor(properties?: ClassBreaksRendererProperties);

    /**
     * When symbolizing polygon features with graduated symbols, set a [FillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html) on this property to visualize the boundaries of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol: FillSymbol | PolygonSymbol3D | CIMSymbol;
    /**
     * Each element in the array is an object that provides information about a class break associated with the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos)
     */
    classBreakInfos: ClassBreakInfo[];
    /**
     * Label used in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) to describe features assigned the [default symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultLabel)
     */
    defaultLabel: string;
    /**
     * The default symbol assigned to features with a value not matched to a given break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultSymbol)
     */
    defaultSymbol: Symbol;
    /**
     * The name of a numeric attribute field whose data determines the symbol of each feature based on the class breaks defined in [classBreakInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#field)
     */
    field: string;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#legendOptions)
     */
    legendOptions: ClassBreaksRendererLegendOptions;
    /**
     * When [normalizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType) is `field`, this property contains the attribute field name used for normalization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationField)
     */
    normalizationField: string;
    /**
     * When [normalizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType) is `percent-of-total`, this property contains the total of all data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationTotal)
     */
    normalizationTotal: number;
    /**
     * Indicates how the data is normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType)
     */
    normalizationType: "field" | "log" | "percent-of-total";
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#type)
     */
    readonly type: "class-breaks";
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpression)
     */
    valueExpression: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle: string;

    /**
     * Adds a class break to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#addClassBreakInfo)
     */
    addClassBreakInfo(min: number | any, max?: number, symbol?: Symbol): void;
    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#clone)
     */
    clone(): ClassBreaksRenderer;
    /**
     * Returns the [classBreakInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos) information (as defined by the renderer) associated with the given graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#getClassBreakInfo)
     */
    getClassBreakInfo(graphic: Graphic): Promise<ClassBreakInfo>;
    /**
     * Removes a break from the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#removeClassBreakInfo)
     */
    removeClassBreakInfo(min: number, max: number): void;

    static fromJSON(json: any): ClassBreaksRenderer;
  }

  interface ClassBreaksRendererProperties extends RendererProperties, VisualVariablesMixinProperties {
    /**
     * When symbolizing polygon features with graduated symbols, set a [FillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html) on this property to visualize the boundaries of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol?:
      | FillSymbolProperties
      | (PolygonSymbol3DProperties & { type: "polygon-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * Each element in the array is an object that provides information about a class break associated with the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos)
     */
    classBreakInfos?: ClassBreakInfoProperties[];
    /**
     * Label used in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) to describe features assigned the [default symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultLabel)
     */
    defaultLabel?: string;
    /**
     * The default symbol assigned to features with a value not matched to a given break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#defaultSymbol)
     */
    defaultSymbol?: SymbolProperties;
    /**
     * The name of a numeric attribute field whose data determines the symbol of each feature based on the class breaks defined in [classBreakInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#field)
     */
    field?: string;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#legendOptions)
     */
    legendOptions?: ClassBreaksRendererLegendOptions;
    /**
     * When [normalizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType) is `field`, this property contains the attribute field name used for normalization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationField)
     */
    normalizationField?: string;
    /**
     * When [normalizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType) is `percent-of-total`, this property contains the total of all data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationTotal)
     */
    normalizationTotal?: number;
    /**
     * Indicates how the data is normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#normalizationType)
     */
    normalizationType?: "field" | "log" | "percent-of-total";
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpression)
     */
    valueExpression?: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle?: string;
  }

  export interface ClassBreaksRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#legendOptions)
     */
    title: string;
  }

  export interface DictionaryRenderer extends Renderer, VisualVariablesMixin {}

  export class DictionaryRenderer {
    /**
     * Dictionary Renderer is used to symbolize layers using a dictionary of [CIMSymbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) configured with multiple attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html)
     */

    constructor(properties?: DictionaryRendererProperties);

    /**
     * This property allows you to set display options that can be configured on the dictionary symbol style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#config)
     */
    config: any;
    /**
     * Defines a field mapping that maps input fields from the feature to the dictionary symbol style's expected fields for symbols and text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#fieldMap)
     */
    fieldMap: HashMap<string>;
    /**
     * A scaling expression can be set to increase or decrease the size of the dictionary symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpression)
     */
    scaleExpression: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [scaleExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpressionTitle)
     */
    scaleExpressionTitle: string;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#type)
     */
    type: "dictionary";
    /**
     * The URL to the dictionary style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#url)
     */
    url: string;

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#clone)
     */
    clone(): DictionaryRenderer;
    /**
     * This method will return the symbol for a given graphic using the dictionary renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#getSymbolAsync)
     */
    getSymbolAsync(graphic: Graphic): Promise<CIMSymbol>;

    static fromJSON(json: any): DictionaryRenderer;
  }

  interface DictionaryRendererProperties extends RendererProperties, VisualVariablesMixinProperties {
    /**
     * This property allows you to set display options that can be configured on the dictionary symbol style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#config)
     */
    config?: any;
    /**
     * Defines a field mapping that maps input fields from the feature to the dictionary symbol style's expected fields for symbols and text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#fieldMap)
     */
    fieldMap?: HashMap<string>;
    /**
     * A scaling expression can be set to increase or decrease the size of the dictionary symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpression)
     */
    scaleExpression?: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [scaleExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#scaleExpressionTitle)
     */
    scaleExpressionTitle?: string;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#type)
     */
    type?: "dictionary";
    /**
     * The URL to the dictionary style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DictionaryRenderer.html#url)
     */
    url?: string;
  }

  export class DotDensityRenderer extends Renderer {
    /**
     * DotDensityRenderer allows you to create dot density visualizations for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html)
     */

    constructor(properties?: DotDensityRendererProperties);

    /**
     * Defines the variable(s) used to visualize density.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#attributes)
     */
    attributes: AttributeColorInfo[];
    /**
     * The color used to shade the polygon fill behind the dots.
     *
     * @default [0, 0, 0, 0.25] - black, semitransparent
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#backgroundColor)
     */
    backgroundColor: Color;
    /**
     * Only applicable when two or more [attributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#attributes) are specified.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotBlendingEnabled)
     */
    dotBlendingEnabled: boolean;
    /**
     * Defines the size of the dots in points.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotSize)
     */
    dotSize: number;
    /**
     * Defines the initial dot value used for visualizing density.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotValue)
     */
    dotValue: number;
    /**
     * An object providing options for configuring the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#legendOptions)
     */
    legendOptions: DotDensityRendererLegendOptions;
    /**
     * The outline of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#outline)
     */
    outline: SimpleLineSymbol;
    /**
     * When defined, the renderer will recalculate the dot value linearly based on the change in the view's scale using the [calculateDotValue()](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#calculateDotValue) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#referenceScale)
     */
    referenceScale: number;
    /**
     * When set to a consistent value, dot placements will be preserved for the same scale given all parameters are the same in the renderer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#seed)
     */
    seed: number;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#type)
     */
    readonly type: "dot-density";
    /**
     * An array of [Size Visual Variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#visualVariables)
     */
    visualVariables: VisualVariable[];

    /**
     * Calculates an updated dot value for the given scale for the cases where a [referenceScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#referenceScale) is provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#calculateDotValue)
     */
    calculateDotValue(scale: number): number;
    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#clone)
     */
    clone(): DotDensityRenderer;

    static fromJSON(json: any): DotDensityRenderer;
  }

  interface DotDensityRendererProperties extends RendererProperties {
    /**
     * Defines the variable(s) used to visualize density.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#attributes)
     */
    attributes?: AttributeColorInfoProperties[];
    /**
     * The color used to shade the polygon fill behind the dots.
     *
     * @default [0, 0, 0, 0.25] - black, semitransparent
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#backgroundColor)
     */
    backgroundColor?: Color | number[] | string;
    /**
     * Only applicable when two or more [attributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#attributes) are specified.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotBlendingEnabled)
     */
    dotBlendingEnabled?: boolean;
    /**
     * Defines the size of the dots in points.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotSize)
     */
    dotSize?: number;
    /**
     * Defines the initial dot value used for visualizing density.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#dotValue)
     */
    dotValue?: number;
    /**
     * An object providing options for configuring the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#legendOptions)
     */
    legendOptions?: DotDensityRendererLegendOptions;
    /**
     * The outline of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#outline)
     */
    outline?: SimpleLineSymbolProperties;
    /**
     * When defined, the renderer will recalculate the dot value linearly based on the change in the view's scale using the [calculateDotValue()](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#calculateDotValue) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#referenceScale)
     */
    referenceScale?: number;
    /**
     * When set to a consistent value, dot placements will be preserved for the same scale given all parameters are the same in the renderer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#seed)
     */
    seed?: number;
    /**
     * An array of [Size Visual Variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#visualVariables)
     */
    visualVariables?: VisualVariableProperties[];
  }

  export interface DotDensityRendererLegendOptions {
    /**
     * Indicates the unit of the data being visualized.
     *
     * [Read more...](global.html#unit)
     */
    unit?: string;
  }

  export interface FlowRenderer extends Accessor, JSONSupport {}

  export class FlowRenderer {
    /**
     * The FlowRenderer allows you to visualize your raster data with animated streamlines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html)
     */

    constructor(properties?: FlowRendererProperties);

    /**
     * Contains metadata about renderers generated from the [flowRendererCreator.createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer) method, including information for setting UI elements such as sliders and themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#authoringInfo)
     */
    authoringInfo: AuthoringInfo;
    /**
     * The color of the animated streamlines.
     *
     * @default [255, 255, 255, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#color)
     */
    color: Color;
    /**
     * The density of the streamlines.
     *
     * @default 0.8
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#density)
     */
    density: number;
    /**
     * Defines the flow direction of the data.
     *
     * @default "flow-from"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#flowRepresentation)
     */
    flowRepresentation: "flow-from" | "flow-to";
    /**
     * The speed of the animated streamlines, relative to the simulation time.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#flowSpeed)
     */
    flowSpeed: number;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#legendOptions)
     */
    legendOptions: FlowRendererLegendOptions;
    /**
     * The maximum path length streamlines will travel in points.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#maxPathLength)
     */
    maxPathLength: number;
    /**
     * The front cap of the streamline.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailCap)
     */
    trailCap: "butt" | "round";
    /**
     * The approximate visible length of the streamline in points.
     *
     * @default 100
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailLength)
     */
    trailLength: number;
    /**
     * The width of the streamline trail in points.
     *
     * @default 1.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailWidth)
     */
    trailWidth: number;
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#type)
     */
    readonly type: "flow";
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#visualVariables)
     */
    visualVariables: VisualVariable[];

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#clone)
     */
    clone(): FlowRenderer;

    static fromJSON(json: any): FlowRenderer;
  }

  interface FlowRendererProperties {
    /**
     * Contains metadata about renderers generated from the [flowRendererCreator.createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer) method, including information for setting UI elements such as sliders and themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#authoringInfo)
     */
    authoringInfo?: AuthoringInfoProperties;
    /**
     * The color of the animated streamlines.
     *
     * @default [255, 255, 255, 1]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#color)
     */
    color?: Color;
    /**
     * The density of the streamlines.
     *
     * @default 0.8
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#density)
     */
    density?: number;
    /**
     * Defines the flow direction of the data.
     *
     * @default "flow-from"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#flowRepresentation)
     */
    flowRepresentation?: "flow-from" | "flow-to";
    /**
     * The speed of the animated streamlines, relative to the simulation time.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#flowSpeed)
     */
    flowSpeed?: number;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#legendOptions)
     */
    legendOptions?: FlowRendererLegendOptions;
    /**
     * The maximum path length streamlines will travel in points.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#maxPathLength)
     */
    maxPathLength?: number | string;
    /**
     * The front cap of the streamline.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailCap)
     */
    trailCap?: "butt" | "round";
    /**
     * The approximate visible length of the streamline in points.
     *
     * @default 100
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailLength)
     */
    trailLength?: number;
    /**
     * The width of the streamline trail in points.
     *
     * @default 1.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailWidth)
     */
    trailWidth?: number | string;
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#visualVariables)
     */
    visualVariables?: VisualVariableProperties[];
  }

  export interface FlowRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#legendOptions)
     */
    title: string;
  }

  export class HeatmapRenderer extends Renderer {
    /**
     * The HeatmapRenderer uses [kernel density](https://pro.arcgis.com/en/pro-app/2.8/tool-reference/spatial-analyst/how-kernel-density-works.htm) to render point features in [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [CSVLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [GeoJSONLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html) and [OGCFeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html) as a raster surface.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html)
     */

    constructor(properties?: HeatmapRendererProperties);

    /**
     * An array of objects describing the renderer's color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#colorStops)
     */
    colorStops: HeatmapColorStop[];
    /**
     * The name of the attribute field used to weight the density of each heatmap point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#field)
     */
    field: string;
    /**
     * An object providing options for describing the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#legendOptions)
     */
    legendOptions: HeatmapRendererLegendOptions;
    /**
     * The max density value to be assigned a color in the heatmap surface.
     *
     * @default 0.04
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#maxDensity)
     */
    maxDensity: number;
    /**
     * The minimum density value to be assigned a color in the heatmap surface.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#minDensity)
     */
    minDensity: number;
    /**
     * The search radius (in points) used to create a smooth kernel surface fitted around each point.
     *
     * @default 18
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#radius)
     */
    radius: number;
    /**
     * When set, the heatmap's visualization at the given scale will remain static and not change as the user zooms in and out of the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#referenceScale)
     */
    referenceScale: number;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#type)
     */
    readonly type: "heatmap";
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpression)
     */
    valueExpression: string;
    /**
     * The title identifying and describing the [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle: string;

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#clone)
     */
    clone(): HeatmapRenderer;

    static fromJSON(json: any): HeatmapRenderer;
  }

  interface HeatmapRendererProperties extends RendererProperties {
    /**
     * An array of objects describing the renderer's color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#colorStops)
     */
    colorStops?: HeatmapColorStopProperties[];
    /**
     * The name of the attribute field used to weight the density of each heatmap point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#field)
     */
    field?: string;
    /**
     * An object providing options for describing the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#legendOptions)
     */
    legendOptions?: HeatmapRendererLegendOptions;
    /**
     * The max density value to be assigned a color in the heatmap surface.
     *
     * @default 0.04
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#maxDensity)
     */
    maxDensity?: number;
    /**
     * The minimum density value to be assigned a color in the heatmap surface.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#minDensity)
     */
    minDensity?: number;
    /**
     * The search radius (in points) used to create a smooth kernel surface fitted around each point.
     *
     * @default 18
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#radius)
     */
    radius?: number | string;
    /**
     * When set, the heatmap's visualization at the given scale will remain static and not change as the user zooms in and out of the view.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#referenceScale)
     */
    referenceScale?: number;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpression)
     */
    valueExpression?: string;
    /**
     * The title identifying and describing the [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle?: string;
  }

  export interface HeatmapRendererLegendOptions {
    /**
     * The label used to describe low density areas in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#legendOptions)
     */
    minLabel: string;
    /**
     * The label used to describe max density areas in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#legendOptions)
     */
    maxLabel: string;
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#legendOptions)
     */
    title: string;
  }

  export class VisualVariablesMixin {
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-mixins-VisualVariablesMixin.html#visualVariables)
     */
    visualVariables: VisualVariable[];
  }

  interface VisualVariablesMixinProperties {
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-mixins-VisualVariablesMixin.html#visualVariables)
     */
    visualVariables?: VisualVariableProperties[];
  }

  export class PieChartRenderer extends Renderer {
    /**
     * PieChartRenderer allows you to create a pie chart for each feature in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html)
     */

    constructor(properties?: PieChartRendererProperties);

    /**
     * Defines the variable(s) to include in the pie charts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#attributes)
     */
    attributes: AttributeColorInfo[];
    /**
     * The symbol used to render polygons behind the pie symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol: SimpleFillSymbol | CIMSymbol;
    /**
     * The color used to visualize features whose [attributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#attributes) all have null or empty values.
     *
     * @default new Color([0, 0, 0, 0])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultColor)
     */
    defaultColor: Color;
    /**
     * Describes the [defaultColor](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultColor) in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultLabel)
     */
    defaultLabel: string;
    /**
     * Use this property to create a donut chart.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#holePercentage)
     */
    holePercentage: number;
    /**
     * An object providing options for describing the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#legendOptions)
     */
    legendOptions: PieChartRendererLegendOptions;
    /**
     * Defines the rules for how to aggregate small categories to a generic "others" category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#othersCategory)
     */
    othersCategory: PieChartRendererOthersCategory;
    /**
     * Defines the outline of the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#outline)
     */
    outline: SimpleLineSymbol;
    /**
     * Defines the size of each pie chart in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#size)
     */
    size: number;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#type)
     */
    readonly type: "pie-chart";
    /**
     * An array of [Size Visual Variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#visualVariables)
     */
    visualVariables: (SizeVariable | OpacityVariable)[];

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#clone)
     */
    clone(): PieChartRenderer;

    static fromJSON(json: any): PieChartRenderer;
  }

  interface PieChartRendererProperties extends RendererProperties {
    /**
     * Defines the variable(s) to include in the pie charts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#attributes)
     */
    attributes?: AttributeColorInfoProperties[];
    /**
     * The symbol used to render polygons behind the pie symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol?:
      | (SimpleFillSymbolProperties & { type: "simple-fill" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The color used to visualize features whose [attributes](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#attributes) all have null or empty values.
     *
     * @default new Color([0, 0, 0, 0])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultColor)
     */
    defaultColor?: Color;
    /**
     * Describes the [defaultColor](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultColor) in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#defaultLabel)
     */
    defaultLabel?: string;
    /**
     * Use this property to create a donut chart.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#holePercentage)
     */
    holePercentage?: number;
    /**
     * An object providing options for describing the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#legendOptions)
     */
    legendOptions?: PieChartRendererLegendOptions;
    /**
     * Defines the rules for how to aggregate small categories to a generic "others" category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#othersCategory)
     */
    othersCategory?: PieChartRendererOthersCategory;
    /**
     * Defines the outline of the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#outline)
     */
    outline?: SimpleLineSymbolProperties;
    /**
     * Defines the size of each pie chart in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#size)
     */
    size?: number | string;
    /**
     * An array of [Size Visual Variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#visualVariables)
     */
    visualVariables?: (
      | (SizeVariableProperties & { type: "size" })
      | (OpacityVariableProperties & { type: "opacity" })
    )[];
  }

  export interface PieChartRendererLegendOptions {
    /**
     * Describes the data used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#legendOptions)
     */
    title: string;
  }

  export interface PieChartRendererOthersCategory {
    /**
     * Defines the color used to represent all categories whose pie slices are smaller than the percentage defined by `threshold`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#othersCategory)
     */
    color?: Color;
    /**
     * Describes the `color` grouping categories smaller than the `threshold` in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#othersCategory)
     */
    label?: string;
    /**
     * Represents the minimum size of individual slices as a percentage of the entire pie.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#othersCategory)
     */
    threshold?: number;
  }

  export class PointCloudClassBreaksRenderer extends PointCloudRenderer {
    /**
     * This class defines the color of each point in a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) based on the value of a numeric attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html)
     */

    constructor(properties?: PointCloudClassBreaksRendererProperties);

    /**
     * Each element in the array is an object that provides information about a class break associated with the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    colorClassBreakInfos: PointCloudClassBreaksRendererColorClassBreakInfos[];
    /**
     * The name of the field that is used to drive the color visualization for the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#field)
     */
    field: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#fieldTransformType)
     */
    fieldTransformType: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#legendOptions)
     */
    legendOptions: PointCloudClassBreaksRendererLegendOptions;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#type)
     */
    readonly type: "point-cloud-class-breaks";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#clone)
     */
    clone(): PointCloudClassBreaksRenderer;

    static fromJSON(json: any): PointCloudClassBreaksRenderer;
  }

  interface PointCloudClassBreaksRendererProperties extends PointCloudRendererProperties {
    /**
     * Each element in the array is an object that provides information about a class break associated with the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    colorClassBreakInfos?: PointCloudClassBreaksRendererColorClassBreakInfos[];
    /**
     * The name of the field that is used to drive the color visualization for the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#field)
     */
    field?: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#fieldTransformType)
     */
    fieldTransformType?: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#legendOptions)
     */
    legendOptions?: PointCloudClassBreaksRendererLegendOptions;
  }

  export interface PointCloudClassBreaksRendererColorClassBreakInfos {
    /**
     * The minimum value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    minValue: number;
    /**
     * The maximum value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    maxValue: number;
    /**
     * The color used to colorize the points where values are between the `minValue` and `maxValue` of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    color: Color;
    /**
     * The label used to describe point in the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#colorClassBreakInfos)
     */
    label?: string;
  }

  export interface PointCloudClassBreaksRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudClassBreaksRenderer.html#legendOptions)
     */
    title: string;
  }

  export interface PointCloudRenderer extends Accessor, JSONSupport {}

  export class PointCloudRenderer {
    /**
     * A PointCloudRenderer allows you to specify how points in a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) are rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html)
     */

    constructor(properties?: PointCloudRendererProperties);

    /**
     * Reduces the brightness of the point's color, based on the value of another field, usually `intensity`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#colorModulation)
     */
    colorModulation: PointCloudRendererColorModulation;
    /**
     * Specifies how the size of the points in the point cloud is computed for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointSizeAlgorithm)
     */
    pointSizeAlgorithm: PointCloudRendererPointSizeAlgorithm;
    /**
     * The number of points to draw per display inch.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointsPerInch)
     */
    pointsPerInch: number;
    /**
     * The point cloud renderer type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#type)
     */
    readonly type: "point-cloud-class-breaks" | "point-cloud-rgb" | "point-cloud-stretch" | "point-cloud-unique-value";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#clone)
     */
    clone(): PointCloudRenderer;

    static fromJSON(json: any): PointCloudRenderer;
  }

  interface PointCloudRendererProperties {
    /**
     * Reduces the brightness of the point's color, based on the value of another field, usually `intensity`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#colorModulation)
     */
    colorModulation?: PointCloudRendererColorModulation;
    /**
     * Specifies how the size of the points in the point cloud is computed for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointSizeAlgorithm)
     */
    pointSizeAlgorithm?: PointCloudRendererPointSizeAlgorithm;
    /**
     * The number of points to draw per display inch.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointsPerInch)
     */
    pointsPerInch?: number;
  }

  export interface PointCloudRendererColorModulation {
    /**
     * Field to use for the color modulation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#colorModulation)
     */
    field: string;
    /**
     * Field value at which the point color becomes darkest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#colorModulation)
     */
    minValue?: number;
    /**
     * Field value at which the point color will be left unchanged.
     *
     * @default 255
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#colorModulation)
     */
    maxValue?: number;
  }

  export interface PointCloudRendererPointSizeAlgorithm {
    /**
     * `fixed-size` or `splat`.
     *
     * [Read more...](global.html#type)
     */
    type: "fixed-size" | "splat";
    /**
     * Specifies whether `size` is in real world units or screen-space units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointSizeAlgorithm)
     */
    useRealWorldSymbolSizes?: boolean;
    /**
     * Point size in meters (`useRealWorldSymbolSizes = true`) or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointSizeAlgorithm)
     */
    size?: number;
    /**
     * Specifies the scale factor that is applied to the size estimated by the density.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRenderer.html#pointSizeAlgorithm)
     */
    scaleFactor?: number;
  }

  export class PointCloudRGBRenderer extends PointCloudRenderer {
    /**
     * PointCloudRGBRenderer defines the color of each point in a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) based on the value of a color attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html)
     */

    constructor(properties?: PointCloudRGBRendererProperties);

    /**
     * The name of the field containing RGB values used to drive the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html#field)
     */
    field: string;
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html#type)
     */
    readonly type: "point-cloud-rgb";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html#clone)
     */
    clone(): PointCloudRGBRenderer;

    static fromJSON(json: any): PointCloudRGBRenderer;
  }

  interface PointCloudRGBRendererProperties extends PointCloudRendererProperties {
    /**
     * The name of the field containing RGB values used to drive the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html#field)
     */
    field?: string;
  }

  export class PointCloudStretchRenderer extends PointCloudRenderer {
    /**
     * PointCloudStretchRenderer defines the color of each point in a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) based on the value of a numeric attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html)
     */

    constructor(properties?: PointCloudStretchRendererProperties);

    /**
     * The name of the number field whose values are used to drive the continuous color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#field)
     */
    field: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#fieldTransformType)
     */
    fieldTransformType: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#legendOptions)
     */
    legendOptions: PointCloudStretchRendererLegendOptions;
    /**
     * An array of color value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#stops)
     */
    stops: ColorStop[];
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#type)
     */
    readonly type: "point-cloud-stretch";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#clone)
     */
    clone(): PointCloudStretchRenderer;

    static fromJSON(json: any): PointCloudStretchRenderer;
  }

  interface PointCloudStretchRendererProperties extends PointCloudRendererProperties {
    /**
     * The name of the number field whose values are used to drive the continuous color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#field)
     */
    field?: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#fieldTransformType)
     */
    fieldTransformType?: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#legendOptions)
     */
    legendOptions?: PointCloudStretchRendererLegendOptions;
    /**
     * An array of color value pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#stops)
     */
    stops?: ColorStopProperties[];
  }

  export interface PointCloudStretchRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html#legendOptions)
     */
    title: string;
  }

  export class PointCloudUniqueValueRenderer extends PointCloudRenderer {
    /**
     * PointCloudUniqueValueRenderer allows you to colorize points in a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) based on an attribute value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html)
     */

    constructor(properties?: PointCloudUniqueValueRendererProperties);

    /**
     * Each element in the array is an object that matches a unique value with a specific color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#colorUniqueValueInfos)
     */
    colorUniqueValueInfos: PointCloudUniqueValueRendererColorUniqueValueInfos[];
    /**
     * The name of the field whose values are used to drive the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#field)
     */
    field: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#fieldTransformType)
     */
    fieldTransformType: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#legendOptions)
     */
    legendOptions: PointCloudUniqueValueRendererLegendOptions;
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#type)
     */
    readonly type: "point-cloud-unique-value";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#clone)
     */
    clone(): PointCloudUniqueValueRenderer;

    static fromJSON(json: any): PointCloudUniqueValueRenderer;
  }

  interface PointCloudUniqueValueRendererProperties extends PointCloudRendererProperties {
    /**
     * Each element in the array is an object that matches a unique value with a specific color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#colorUniqueValueInfos)
     */
    colorUniqueValueInfos?: PointCloudUniqueValueRendererColorUniqueValueInfos[];
    /**
     * The name of the field whose values are used to drive the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#field)
     */
    field?: string;
    /**
     * A transform that is applied to the field value before evaluating the renderer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#fieldTransformType)
     */
    fieldTransformType?: "none" | "low-four-bit" | "high-four-bit" | "absolute-value" | "modulo-ten";
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#legendOptions)
     */
    legendOptions?: PointCloudUniqueValueRendererLegendOptions;
  }

  export interface PointCloudUniqueValueRendererColorUniqueValueInfos {
    /**
     * A unique combination of values that will be represented with the given `color`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#colorUniqueValueInfos)
     */
    values: string[];
    /**
     * The color used to represent points whose value matches `values`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#colorUniqueValueInfos)
     */
    color: Color;
    /**
     * Label used to describe points matched to the unique value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#colorUniqueValueInfos)
     */
    label?: string;
  }

  export interface PointCloudUniqueValueRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html#legendOptions)
     */
    title: string;
  }

  export interface RasterColormapRenderer extends Accessor, JSONSupport {}

  export class RasterColormapRenderer {
    /**
     * The RasterColormapRenderer defines the symbology to display raster data based on specific colors, aiding in visual analysis of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html)
     */

    constructor(properties?: RasterColormapRendererProperties);

    /**
     * A colormap info array containing mappings for pixel and RGB color values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html#colormapInfos)
     */
    colormapInfos: ColormapInfo[];
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html#type)
     */
    readonly type: "raster-colormap";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html#clone)
     */
    clone(): RasterColormapRenderer;

    /**
     * Creates a new instance [RasterColormapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html) from an array of color maps where pixel values with its corresponding RGB color values specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html#createFromColormap)
     */
    static createFromColormap(colormap: number[][]): RasterColormapRenderer;
    static fromJSON(json: any): RasterColormapRenderer;
  }

  interface RasterColormapRendererProperties {
    /**
     * A colormap info array containing mappings for pixel and RGB color values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html#colormapInfos)
     */
    colormapInfos?: ColormapInfoProperties[];
  }

  export interface RasterShadedReliefRenderer extends Accessor, JSONSupport {}

  export class RasterShadedReliefRenderer {
    /**
     * RasterShadedReliefRenderer produces a grayscale or colored 3D representation of the surface on an [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html), with the sun's relative position taken into account for shading the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html)
     */

    constructor(properties?: RasterShadedReliefRendererProperties);

    /**
     * The sun's angle of elevation above the horizon, ranging from 0 to 90 degrees.
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#altitude)
     */
    altitude: number;
    /**
     * The sun's relative position along the horizon, ranging from 0 to 360 degrees.
     *
     * @default 315
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#azimuth)
     */
    azimuth: number;
    /**
     * The color ramp to display the shaded relief.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#colorRamp)
     */
    colorRamp: ColorRamp;
    /**
     * The type of hillshading being applied on the elevation surface.
     *
     * @default "traditional"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#hillshadeType)
     */
    hillshadeType: "traditional" | "multi-directional";
    /**
     * Pixel size factor accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * @default 0.024
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#pixelSizeFactor)
     */
    pixelSizeFactor: number;
    /**
     * Pixel Size Power accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * @default 0.664
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#pixelSizePower)
     */
    pixelSizePower: number;
    /**
     * Applies a constant or adjusted z-factor based on resolution changes.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#scalingType)
     */
    scalingType: "none" | "adjusted";
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#type)
     */
    readonly type: "raster-shaded-relief";
    /**
     * A ratio of z unit / xy unit, with optional exaggeration factored in.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#zFactor)
     */
    zFactor: number;

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#clone)
     */
    clone(): RasterShadedReliefRenderer;

    static fromJSON(json: any): RasterShadedReliefRenderer;
  }

  interface RasterShadedReliefRendererProperties {
    /**
     * The sun's angle of elevation above the horizon, ranging from 0 to 90 degrees.
     *
     * @default 45
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#altitude)
     */
    altitude?: number;
    /**
     * The sun's relative position along the horizon, ranging from 0 to 360 degrees.
     *
     * @default 315
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#azimuth)
     */
    azimuth?: number;
    /**
     * The color ramp to display the shaded relief.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#colorRamp)
     */
    colorRamp?: ColorRampProperties;
    /**
     * The type of hillshading being applied on the elevation surface.
     *
     * @default "traditional"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#hillshadeType)
     */
    hillshadeType?: "traditional" | "multi-directional";
    /**
     * Pixel size factor accounts for changes in scale as the viewer zooms in and out on the map display.
     *
     * @default 0.024
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#pixelSizeFactor)
     */
    pixelSizeFactor?: number;
    /**
     * Pixel Size Power accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display.
     *
     * @default 0.664
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#pixelSizePower)
     */
    pixelSizePower?: number;
    /**
     * Applies a constant or adjusted z-factor based on resolution changes.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#scalingType)
     */
    scalingType?: "none" | "adjusted";
    /**
     * A ratio of z unit / xy unit, with optional exaggeration factored in.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html#zFactor)
     */
    zFactor?: number;
  }

  export interface RasterStretchRenderer extends Accessor, JSONSupport {}

  export class RasterStretchRenderer {
    /**
     * RasterStretchRenderer defines the symbology with a gradual ramp of colors for each pixel in a [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html), and [WCSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WCSLayer.html) based on the pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html)
     */

    constructor(properties?: RasterStretchRendererProperties);

    /**
     * The stretched values are mapped to this specified color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#colorRamp)
     */
    colorRamp: ColorRamp;
    /**
     * The computeGamma automatically calculates best gamma value to render exported image based on empirical model.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#computeGamma)
     */
    computeGamma: boolean;
    /**
     * The input band statistics can be specified through the customStatistics property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#customStatistics)
     */
    customStatistics: RasterBandStatistics[];
    /**
     * When Dynamic Range Adjustment is `true`, the statistics based on the current display extent are calculated as you zoom and pan around the image.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#dynamicRangeAdjustment)
     */
    dynamicRangeAdjustment: boolean;
    /**
     * The gamma values to be used if [useGamma](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#useGamma) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#gamma)
     */
    gamma: number[];
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `percent-clip`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#maxPercent)
     */
    maxPercent: number;
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `percent-clip`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#minPercent)
     */
    minPercent: number;
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `standard-deviation`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#numberOfStandardDeviations)
     */
    numberOfStandardDeviations: number;
    /**
     * The outputMax denotes the output maximum, which is the highest pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#outputMax)
     */
    outputMax: number;
    /**
     * The outputMin denotes the output minimum, which is the lowest pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#outputMin)
     */
    outputMin: number;
    /**
     * The sigmoid strength level determines how much of the sigmoidal function will be used in the stretch.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#sigmoidStrengthLevel)
     */
    sigmoidStrengthLevel: number;
    /**
     * The input statistics can be specified through the statistics property.
     *
     * @deprecated since 4.31. Use {@link module:esri/renderers/RasterStretchRenderer#customStatistics customStatistics} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#statistics)
     */
    statistics: number[][] | RasterBandStatistics[];
    /**
     * The stretch type defines a histogram stretch that will be applied to the rasters to enhance their appearance.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType)
     */
    stretchType: "none" | "standard-deviation" | "histogram-equalization" | "min-max" | "percent-clip" | "sigmoid";
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#type)
     */
    readonly type: "raster-stretch";
    /**
     * Denotes whether the [gamma](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#gamma) value should be used.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#useGamma)
     */
    useGamma: boolean;

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#clone)
     */
    clone(): RasterStretchRenderer;

    static fromJSON(json: any): RasterStretchRenderer;
  }

  interface RasterStretchRendererProperties {
    /**
     * The stretched values are mapped to this specified color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#colorRamp)
     */
    colorRamp?: ColorRampProperties;
    /**
     * The computeGamma automatically calculates best gamma value to render exported image based on empirical model.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#computeGamma)
     */
    computeGamma?: boolean;
    /**
     * The input band statistics can be specified through the customStatistics property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#customStatistics)
     */
    customStatistics?: RasterBandStatistics[];
    /**
     * When Dynamic Range Adjustment is `true`, the statistics based on the current display extent are calculated as you zoom and pan around the image.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#dynamicRangeAdjustment)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * The gamma values to be used if [useGamma](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#useGamma) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#gamma)
     */
    gamma?: number[];
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `percent-clip`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#maxPercent)
     */
    maxPercent?: number;
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `percent-clip`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#minPercent)
     */
    minPercent?: number;
    /**
     * Applicable when [stretchType](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType) is `standard-deviation`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#numberOfStandardDeviations)
     */
    numberOfStandardDeviations?: number;
    /**
     * The outputMax denotes the output maximum, which is the highest pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#outputMax)
     */
    outputMax?: number;
    /**
     * The outputMin denotes the output minimum, which is the lowest pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#outputMin)
     */
    outputMin?: number;
    /**
     * The sigmoid strength level determines how much of the sigmoidal function will be used in the stretch.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#sigmoidStrengthLevel)
     */
    sigmoidStrengthLevel?: number;
    /**
     * The input statistics can be specified through the statistics property.
     *
     * @deprecated since 4.31. Use {@link module:esri/renderers/RasterStretchRenderer#customStatistics customStatistics} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#statistics)
     */
    statistics?: number[][] | RasterBandStatistics[];
    /**
     * The stretch type defines a histogram stretch that will be applied to the rasters to enhance their appearance.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#stretchType)
     */
    stretchType?: "none" | "standard-deviation" | "histogram-equalization" | "min-max" | "percent-clip" | "sigmoid";
    /**
     * Denotes whether the [gamma](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#gamma) value should be used.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html#useGamma)
     */
    useGamma?: boolean;
  }

  export interface Renderer extends Accessor, JSONSupport {}

  export class Renderer {
    /**
     * Renderers define how to visually represent each feature in one of the following layer types:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html)
     */

    constructor(properties?: RendererProperties);

    /**
     * Authoring metadata only included in renderers generated from one of the Smart Mapping creator methods, such as [sizeRendererCreator.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) or [colorRendererCreator.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html#authoringInfo)
     */
    authoringInfo: AuthoringInfo;
    /**
     * The renderer type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html#type)
     */
    readonly type: "class-breaks" | "dictionary" | "dot-density" | "heatmap" | "pie-chart" | "simple" | "unique-value";

    static fromJSON(json: any): Renderer;
  }

  interface RendererProperties {
    /**
     * Authoring metadata only included in renderers generated from one of the Smart Mapping creator methods, such as [sizeRendererCreator.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) or [colorRendererCreator.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html#authoringInfo)
     */
    authoringInfo?: AuthoringInfoProperties;
  }

  export interface SimpleRenderer extends Renderer, VisualVariablesMixin {}

  export class SimpleRenderer {
    /**
     * SimpleRenderer renders all features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) with one [Symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html)
     */

    constructor(properties?: SimpleRendererProperties);

    /**
     * The label for the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#label)
     */
    label: string;
    /**
     * The symbol used by the renderer to visualize all features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#symbol)
     */
    symbol: Symbol;
    /**
     * The type of renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#type)
     */
    readonly type: "simple";

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#clone)
     */
    clone(): SimpleRenderer;

    static fromJSON(json: any): SimpleRenderer;
  }

  interface SimpleRendererProperties extends RendererProperties, VisualVariablesMixinProperties {
    /**
     * The label for the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#label)
     */
    label?: string;
    /**
     * The symbol used by the renderer to visualize all features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-SimpleRenderer.html#symbol)
     */
    symbol?: SymbolProperties;
  }

  export interface AttributeColorInfo extends Accessor, JSONSupport {}

  export class AttributeColorInfo {
    /**
     * Defines colors for dots in a [DotDensityRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html#attributes), or colors for pie chart slices in a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html#attributes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html)
     */

    constructor(properties?: AttributeColorInfoProperties);

    /**
     * The color used to render dots representing the given [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#field) in a [DotDensityRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html) or the color used to represent a pie chart slice in a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#color)
     */
    color: Color;
    /**
     * The name of the numeric attribute field represented by the given [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#field)
     */
    field: string;
    /**
     * The label used to describe the field or attribute in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#label)
     */
    label: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpression)
     */
    valueExpression: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpressionTitle)
     */
    valueExpressionTitle: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#clone)
     */
    clone(): AttributeColorInfo;

    static fromJSON(json: any): AttributeColorInfo;
  }

  interface AttributeColorInfoProperties {
    /**
     * The color used to render dots representing the given [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#field) in a [DotDensityRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html) or the color used to represent a pie chart slice in a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#color)
     */
    color?: Color | number[] | string;
    /**
     * The name of the numeric attribute field represented by the given [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#field)
     */
    field?: string;
    /**
     * The label used to describe the field or attribute in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#label)
     */
    label?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpression)
     */
    valueExpression?: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AttributeColorInfo.html#valueExpressionTitle)
     */
    valueExpressionTitle?: string;
  }

  export interface AuthoringInfo extends Accessor, JSONSupport {}

  export class AuthoringInfo {
    /**
     * Authoring information related to generating renderers and visual variables with the Smart Mapping methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html)
     */

    constructor(properties?: AuthoringInfoProperties);

    /**
     * Indicates which classification method was used if a classed color or classed size renderer was generated using one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#classificationMethod)
     */
    classificationMethod:
      | "equal-interval"
      | "defined-interval"
      | "manual"
      | "natural-breaks"
      | "quantile"
      | "standard-deviation";
    /**
     * Indicates the color ramp was used to create the symbols for Unique Value or Class Breaks renderer for Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#colorRamp)
     */
    colorRamp: ColorRamp;
    /**
     * Only applicable to [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) created with the [heatmap renderer creator](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#fadeRatio)
     */
    fadeRatio: number;
    /**
     * A numeric field used for generating a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html) along with [field2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    field1: AuthoringInfoField1;
    /**
     * A numeric field used for generating a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html) along with [field1](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    field2: AuthoringInfoField2;
    /**
     * An array of string values representing field names used for creating a [predominance renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#fields)
     */
    fields: string[];
    /**
     * Only applicable to flow renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#flowTheme)
     */
    flowTheme: "flow-line" | "wave-front";
    /**
     * The focus of a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#focus)
     */
    focus: string;
    /**
     * Indicates whether the renderer was created internally by the JS API's rendering engine for default [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualizations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#isAutoGenerated)
     */
    isAutoGenerated: boolean;
    /**
     * **Only applicable to renderer used in web scenes.** Indicates the unit used in real-world sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#lengthUnit)
     */
    lengthUnit:
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers";
    /**
     * Indicates the value of the upper handle if a slider was used to generate the dot value for dot density renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#maxSliderValue)
     */
    maxSliderValue: number;
    /**
     * Indicates the value of the lower handle if a slider was used to generate the dot value for dot density renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#minSliderValue)
     */
    minSliderValue: number;
    /**
     * The number of classes used to classify each field of a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#numClasses)
     */
    numClasses: number;
    /**
     * Indicates the standard deviation interval for each stop in a classed color or classed size renderer that was generated using the `standard-deviation` classification method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#standardDeviationInterval)
     */
    standardDeviationInterval: number;
    /**
     * Only for renderers of type `univariate-color-size` with an `above-and-below` [univariateTheme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#statistics)
     */
    statistics: AuthoringInfoStatistics;
    /**
     * Indicates the renderer type generated from one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#type)
     */
    type:
      | "class-breaks-size"
      | "class-breaks-color"
      | "classed-color"
      | "classed-size"
      | "dot-density"
      | "flow"
      | "predominance"
      | "relationship"
      | "univariate-color-size";
    /**
     * Only applicable to [univariateColorSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html) renderers with an `above-and-below` [univariateTheme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateSymbolStyle)
     */
    univariateSymbolStyle:
      | "caret"
      | "circle-caret"
      | "arrow"
      | "circle-arrow"
      | "plus-minus"
      | "circle-plus-minus"
      | "square"
      | "circle"
      | "triangle"
      | "happy-sad"
      | "thumb"
      | "custom";
    /**
     * Only applicable to [univariateColorSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html) renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme)
     */
    univariateTheme: "high-to-low" | "above" | "below" | "above-and-below";
    /**
     * Contains authoring properties of visual variables generated from one of the Smart Mapping methods or sliders.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#visualVariables)
     */
    visualVariables: AuthoringInfoVisualVariable[];

    /**
     * Creates a deep clone of the instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#clone)
     */
    clone(): AuthoringInfo;

    static fromJSON(json: any): AuthoringInfo;
  }

  interface AuthoringInfoProperties {
    /**
     * Indicates which classification method was used if a classed color or classed size renderer was generated using one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#classificationMethod)
     */
    classificationMethod?:
      | "equal-interval"
      | "defined-interval"
      | "manual"
      | "natural-breaks"
      | "quantile"
      | "standard-deviation";
    /**
     * Indicates the color ramp was used to create the symbols for Unique Value or Class Breaks renderer for Imagery Layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#colorRamp)
     */
    colorRamp?: ColorRampProperties;
    /**
     * Only applicable to [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) created with the [heatmap renderer creator](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#fadeRatio)
     */
    fadeRatio?: number;
    /**
     * A numeric field used for generating a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html) along with [field2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    field1?: AuthoringInfoField1;
    /**
     * A numeric field used for generating a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html) along with [field1](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    field2?: AuthoringInfoField2;
    /**
     * An array of string values representing field names used for creating a [predominance renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#fields)
     */
    fields?: string[];
    /**
     * Only applicable to flow renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#flowTheme)
     */
    flowTheme?: "flow-line" | "wave-front";
    /**
     * The focus of a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#focus)
     */
    focus?: string;
    /**
     * Indicates whether the renderer was created internally by the JS API's rendering engine for default [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualizations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#isAutoGenerated)
     */
    isAutoGenerated?: boolean;
    /**
     * **Only applicable to renderer used in web scenes.** Indicates the unit used in real-world sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#lengthUnit)
     */
    lengthUnit?:
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers";
    /**
     * Indicates the value of the upper handle if a slider was used to generate the dot value for dot density renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#maxSliderValue)
     */
    maxSliderValue?: number;
    /**
     * Indicates the value of the lower handle if a slider was used to generate the dot value for dot density renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#minSliderValue)
     */
    minSliderValue?: number;
    /**
     * The number of classes used to classify each field of a [relationship renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#numClasses)
     */
    numClasses?: number;
    /**
     * Indicates the standard deviation interval for each stop in a classed color or classed size renderer that was generated using the `standard-deviation` classification method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#standardDeviationInterval)
     */
    standardDeviationInterval?: number;
    /**
     * Only for renderers of type `univariate-color-size` with an `above-and-below` [univariateTheme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#statistics)
     */
    statistics?: AuthoringInfoStatistics;
    /**
     * Indicates the renderer type generated from one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#type)
     */
    type?:
      | "class-breaks-size"
      | "class-breaks-color"
      | "classed-color"
      | "classed-size"
      | "dot-density"
      | "flow"
      | "predominance"
      | "relationship"
      | "univariate-color-size";
    /**
     * Only applicable to [univariateColorSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html) renderers with an `above-and-below` [univariateTheme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateSymbolStyle)
     */
    univariateSymbolStyle?:
      | "caret"
      | "circle-caret"
      | "arrow"
      | "circle-arrow"
      | "plus-minus"
      | "circle-plus-minus"
      | "square"
      | "circle"
      | "triangle"
      | "happy-sad"
      | "thumb"
      | "custom";
    /**
     * Only applicable to [univariateColorSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html) renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#univariateTheme)
     */
    univariateTheme?: "high-to-low" | "above" | "below" | "above-and-below";
    /**
     * Contains authoring properties of visual variables generated from one of the Smart Mapping methods or sliders.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#visualVariables)
     */
    visualVariables?: AuthoringInfoVisualVariableProperties[];
  }

  export interface AuthoringInfoField1 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    normalizationField?: string;
    /**
     * Describes the class breaks generated for this field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    classBreakInfos?: AuthoringInfoField1ClassBreakInfos[];
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    label?: string;
  }

  export interface AuthoringInfoField1ClassBreakInfos {
    /**
     * The maximum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    maxValue?: number;
    /**
     * The minimum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field1)
     */
    minValue?: number;
  }

  export interface AuthoringInfoField2 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    normalizationField?: string;
    /**
     * Describes the class breaks generated for this field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    classBreakInfos?: AuthoringInfoField2ClassBreakInfos[];
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    label?: string;
  }

  export interface AuthoringInfoField2ClassBreakInfos {
    /**
     * The maximum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    maxValue?: number;
    /**
     * The minimum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#field2)
     */
    minValue?: number;
  }

  export interface AuthoringInfoStatistics {
    /**
     * The maximum data value of the attribute represented by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#statistics)
     */
    max: number;
    /**
     * The minimum data value of the attribute represented by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfo.html#statistics)
     */
    min: number;
  }

  export interface AuthoringInfoVisualVariable extends Accessor, JSONSupport {}

  export class AuthoringInfoVisualVariable {
    /**
     * Contains authoring properties of visual variables generated from one of the Smart Mapping methods or sliders.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html)
     */

    constructor(properties?: AuthoringInfoVisualVariableProperties);

    /**
     * If an age or timeline renderer was generated, indicates the end time of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#endTime)
     */
    endTime: string | number;
    /**
     * Indicates the field name used for generating the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#field)
     */
    field: string;
    /**
     * Indicates the value of the upper handle if a slider was used to generate the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#maxSliderValue)
     */
    maxSliderValue: number;
    /**
     * Indicates the value of the lower handle if a slider was used to generate the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#minSliderValue)
     */
    minSliderValue: number;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#normalizationField)
     */
    normalizationField: string;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#referenceSizeScale)
     */
    referenceSizeScale: number;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#referenceSizeSymbolStyle)
     */
    referenceSizeSymbolStyle: "circle" | "diamond" | "hexagon-flat" | "hexagon-pointy" | "square";
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#sizeStops)
     */
    sizeStops: SizeStop[];
    /**
     * If an age or timeline renderer was generated, indicates the start time of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#startTime)
     */
    startTime: string | number;
    /**
     * If the UI offers the option to display values based on a ratio or percentage, this indicates which selection was made.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#style)
     */
    style: "percent" | "percent-of-total" | "ratio";
    /**
     * Indicates the theme selected by the user when generating a renderer or visual variable with one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme)
     */
    theme: "above" | "below" | "above-and-below" | "centered-on" | "extremes" | "high-to-low" | "reference-size";
    /**
     * The type of visual variable generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#type)
     */
    type: "color" | "size" | "opacity";
    /**
     * If an age or timeline renderer was generated, indicates the time units used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#units)
     */
    units: "seconds" | "minutes" | "hours" | "days" | "months" | "years";

    /**
     * Creates a deep clone of the instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#clone)
     */
    clone(): AuthoringInfoVisualVariable;

    static fromJSON(json: any): AuthoringInfoVisualVariable;
  }

  interface AuthoringInfoVisualVariableProperties {
    /**
     * If an age or timeline renderer was generated, indicates the end time of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#endTime)
     */
    endTime?: string | number;
    /**
     * Indicates the field name used for generating the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#field)
     */
    field?: string;
    /**
     * Indicates the value of the upper handle if a slider was used to generate the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#maxSliderValue)
     */
    maxSliderValue?: number;
    /**
     * Indicates the value of the lower handle if a slider was used to generate the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#minSliderValue)
     */
    minSliderValue?: number;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#normalizationField)
     */
    normalizationField?: string;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#referenceSizeScale)
     */
    referenceSizeScale?: number;
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#referenceSizeSymbolStyle)
     */
    referenceSizeSymbolStyle?: "circle" | "diamond" | "hexagon-flat" | "hexagon-pointy" | "square";
    /**
     * Only applicable when [theme](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme) is `reference-size`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#sizeStops)
     */
    sizeStops?: SizeStopProperties[];
    /**
     * If an age or timeline renderer was generated, indicates the start time of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#startTime)
     */
    startTime?: string | number;
    /**
     * If the UI offers the option to display values based on a ratio or percentage, this indicates which selection was made.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#style)
     */
    style?: "percent" | "percent-of-total" | "ratio";
    /**
     * Indicates the theme selected by the user when generating a renderer or visual variable with one of the Smart Mapping functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#theme)
     */
    theme?: "above" | "below" | "above-and-below" | "centered-on" | "extremes" | "high-to-low" | "reference-size";
    /**
     * The type of visual variable generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#type)
     */
    type?: "color" | "size" | "opacity";
    /**
     * If an age or timeline renderer was generated, indicates the time units used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-AuthoringInfoVisualVariable.html#units)
     */
    units?: "seconds" | "minutes" | "hours" | "days" | "months" | "years";
  }

  export interface ClassBreakInfo extends Accessor, JSONSupport {}

  export class ClassBreakInfo {
    /**
     * Defines a class break for a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html)
     */

    constructor(properties?: ClassBreakInfoProperties);

    /**
     * Describes the data represented by the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#label)
     */
    label: string;
    /**
     * Sets the maximum value for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#maxValue)
     */
    maxValue: number;
    /**
     * Sets the minimum value for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#minValue)
     */
    minValue: number;
    /**
     * Defines the symbol used to render features with data values that are within the bounds defined for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#symbol)
     */
    symbol: Symbol;

    /**
     * Creates a deep clone of the class break info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#clone)
     */
    clone(): ClassBreakInfo;

    static fromJSON(json: any): ClassBreakInfo;
  }

  interface ClassBreakInfoProperties {
    /**
     * Describes the data represented by the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#label)
     */
    label?: string;
    /**
     * Sets the maximum value for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#maxValue)
     */
    maxValue?: number;
    /**
     * Sets the minimum value for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#minValue)
     */
    minValue?: number;
    /**
     * Defines the symbol used to render features with data values that are within the bounds defined for the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ClassBreakInfo.html#symbol)
     */
    symbol?: SymbolProperties;
  }

  export interface ColormapInfo extends Accessor, JSONSupport {}

  export class ColormapInfo {
    /**
     * The ColormapInfo describes pixel value, RGB colors and labels to color the raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html)
     */

    constructor(properties?: ColormapInfoProperties);

    /**
     * The color of a given pixel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#color)
     */
    color: Color;
    /**
     * The label for a given pixel value and color mapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#label)
     */
    label: string;
    /**
     * The raster pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#value)
     */
    value: number;

    static fromJSON(json: any): ColormapInfo;
  }

  interface ColormapInfoProperties {
    /**
     * The color of a given pixel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#color)
     */
    color?: Color;
    /**
     * The label for a given pixel value and color mapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#label)
     */
    label?: string;
    /**
     * The raster pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-ColormapInfo.html#value)
     */
    value?: number;
  }

  export class HeatmapColorStop extends Accessor {
    /**
     * This class is used to define an array of objects describing the [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html)'s color ramp and associated density value ratios.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html)
     */

    constructor(properties?: HeatmapColorStopProperties);

    /**
     * The color to shade a given pixel based on its calculated density [ratio](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#color)
     */
    color: Color;
    /**
     * The ratio of a pixel's density value to the [maxDensity](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#maxDensity) of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio)
     */
    ratio: number;

    /**
     * Creates a deep clone of the color stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#clone)
     */
    clone(): HeatmapColorStop;
  }

  interface HeatmapColorStopProperties {
    /**
     * The color to shade a given pixel based on its calculated density [ratio](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#color)
     */
    color?: Color | number[] | string;
    /**
     * The ratio of a pixel's density value to the [maxDensity](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#maxDensity) of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio)
     */
    ratio?: number;
  }

  /**
   * Provides a utility method used to deserialize a JSON renderer object returned by the REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-jsonUtils.html)
   */
  interface supportJsonUtils {
    /**
     * Creates a new instance of an appropriate [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-jsonUtils.html#fromJSON)
     */
    fromJSON(json: any): Renderer;
  }

  export const supportJsonUtils: supportJsonUtils;

  export interface RasterPresetRenderer extends Accessor, Clonable, JSONSupport {}

  export class RasterPresetRenderer {
    /**
     * Defines a predefined raster renderer associated with an [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#presetRenderers) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#presetRenderers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html)
     */

    constructor(properties?: RasterPresetRendererProperties);

    /**
     * The band ids associated with the raster renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#bandIds)
     */
    bandIds: number[];
    /**
     * Specifies how to match the [renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#renderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method)
     */
    method: "raster-function-template" | "variable" | "none";
    /**
     * The name of the preset renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#name)
     */
    name: string;
    /**
     * The raster renderer associated with the selected [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#value) and [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#renderer)
     */
    renderer:
      | ClassBreaksRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | RasterShadedReliefRenderer
      | RasterColormapRenderer
      | VectorFieldRenderer
      | FlowRenderer;
    /**
     * The corresponding value of selected [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#value)
     */
    value: string;

    static fromJSON(json: any): RasterPresetRenderer;
  }

  interface RasterPresetRendererProperties {
    /**
     * The band ids associated with the raster renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#bandIds)
     */
    bandIds?: number[];
    /**
     * Specifies how to match the [renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#renderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method)
     */
    method?: "raster-function-template" | "variable" | "none";
    /**
     * The name of the preset renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#name)
     */
    name?: string;
    /**
     * The raster renderer associated with the selected [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#value) and [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#renderer)
     */
    renderer?:
      | (ClassBreaksRendererProperties & { type: "class-breaks" })
      | (UniqueValueRendererProperties & { type: "unique-value" })
      | (RasterStretchRendererProperties & { type: "raster-stretch" })
      | (RasterShadedReliefRendererProperties & { type: "raster-shaded-relief" })
      | (RasterColormapRendererProperties & { type: "raster-colormap" })
      | (VectorFieldRendererProperties & { type: "vector-field" })
      | (FlowRendererProperties & { type: "flow" });
    /**
     * The corresponding value of selected [method](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#method).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-RasterPresetRenderer.html#value)
     */
    value?: string;
  }

  export interface UniqueValue extends Accessor, JSONSupport {}

  export class UniqueValue {
    /**
     * Defines combinations of values to expect from up to three fields of categorical data in a [UniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html)
     */

    constructor(properties?: UniqueValueProperties);

    /**
     * Defines a value (possibly in combination with [value2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2) and [value3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)) returned from the field referenced in [UniqueValueRenderer.field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field) or returned from an Arcade expression defined in [UniqueValueRenderer.valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression) to be categorized in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value)
     */
    value: string | number;
    /**
     * Defines a value returned from the field referenced in [UniqueValueRenderer.field2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field2) to be categorized in combination with [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value) (and possibly [value3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)) in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2)
     */
    value2: string | number;
    /**
     * Defines a value returned from the field referenced in [UniqueValueRenderer.field3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field3) to be categorized in combination with [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value) and [value2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2) in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)
     */
    value3: string | number;

    static fromJSON(json: any): UniqueValue;
  }

  interface UniqueValueProperties {
    /**
     * Defines a value (possibly in combination with [value2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2) and [value3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)) returned from the field referenced in [UniqueValueRenderer.field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field) or returned from an Arcade expression defined in [UniqueValueRenderer.valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression) to be categorized in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value)
     */
    value?: string | number;
    /**
     * Defines a value returned from the field referenced in [UniqueValueRenderer.field2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field2) to be categorized in combination with [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value) (and possibly [value3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)) in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2)
     */
    value2?: string | number;
    /**
     * Defines a value returned from the field referenced in [UniqueValueRenderer.field3](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field3) to be categorized in combination with [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value) and [value2](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value2) in a [UniqueValueClass](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValue.html#value3)
     */
    value3?: string | number;
  }

  export interface UniqueValueClass extends Accessor, Clonable, JSONSupport {}

  export class UniqueValueClass {
    /**
     * Defines a category within a [UniqueValueGroup](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html)
     */

    constructor(properties?: UniqueValueClassProperties);

    /**
     * Describes the [values](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values) represented by the [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#symbol) in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#label)
     */
    label: string;
    /**
     * Defines the symbol used to represent features containing the given [values](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#symbol)
     */
    symbol: Symbol;
    /**
     * An array of unique values that should be rendered with the same symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values)
     */
    values: UniqueValue[];

    static fromJSON(json: any): UniqueValueClass;
  }

  interface UniqueValueClassProperties {
    /**
     * Describes the [values](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values) represented by the [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#symbol) in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#label)
     */
    label?: string;
    /**
     * Defines the symbol used to represent features containing the given [values](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#symbol)
     */
    symbol?: SymbolProperties;
    /**
     * An array of unique values that should be rendered with the same symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html#values)
     */
    values?: UniqueValueProperties[] | string | number;
  }

  export interface UniqueValueGroup extends Accessor, Clonable, JSONSupport {}

  export class UniqueValueGroup {
    /**
     * UniqueValueGroup represents a group of [unique value classes](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueClass.html) (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html)
     */

    constructor(properties?: UniqueValueGroupProperties);

    /**
     * Specifies the classes (or unique categories) to group under a [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#heading).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#classes)
     */
    classes: UniqueValueClass[];
    /**
     * The heading to be displayed for the group of unique classes in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#heading)
     */
    heading: string;

    static fromJSON(json: any): UniqueValueGroup;
  }

  interface UniqueValueGroupProperties {
    /**
     * Specifies the classes (or unique categories) to group under a [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#heading).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#classes)
     */
    classes?: UniqueValueClassProperties[];
    /**
     * The heading to be displayed for the group of unique classes in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueGroup.html#heading)
     */
    heading?: string;
  }

  export interface UniqueValueInfo extends Accessor, JSONSupport {}

  export class UniqueValueInfo {
    /**
     * Defines the categories of a [UniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html)
     */

    constructor(properties?: UniqueValueInfoProperties);

    /**
     * Describes the [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value) represented by the [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#label)
     */
    label: string;
    /**
     * Defines the symbol used to render features with the provided [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol)
     */
    symbol: Symbol;
    /**
     * Features with this value will be rendered with the given [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value)
     */
    value: string | number;

    /**
     * Creates a deep clone of the unique value info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#clone)
     */
    clone(): UniqueValueInfo;

    static fromJSON(json: any): UniqueValueInfo;
  }

  interface UniqueValueInfoProperties {
    /**
     * Describes the [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value) represented by the [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#label)
     */
    label?: string;
    /**
     * Defines the symbol used to render features with the provided [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol)
     */
    symbol?: SymbolProperties;
    /**
     * Features with this value will be rendered with the given [symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#symbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-UniqueValueInfo.html#value)
     */
    value?: string | number;
  }

  /**
   * Various utilities for accessing renderer colors that match to fields, Arcade expressions, or values.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html)
   */
  interface utils {
    /**
     * Resolves with a [FieldToValueColorMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#FieldToValueColorMap) that contains the fields and a map of values to its matched color in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#getColorsForRendererValues)
     */
    getColorsForRendererValues(
      renderer: ClassBreaksRenderer | SimpleRenderer | UniqueValueRenderer,
    ): Promise<FieldToValueColorMap>;
    /**
     * Resolves with a [Map](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map) of fields matched to the color that the field is represented by in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#getColorsFromRenderer)
     */
    getColorsFromRenderer(
      renderer:
        | ClassBreaksRenderer
        | DotDensityRenderer
        | HeatmapRenderer
        | PieChartRenderer
        | SimpleRenderer
        | UniqueValueRenderer,
    ): Promise<globalThis.Map<string, Color[]>>;
  }

  export const utils: utils;

  /**
   * A mapping of the field or Arcade expression to a [ValueToColorMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#ValueToColorMap) object.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#FieldToValueColorMap)
   */
  export type FieldToValueColorMap = globalThis.Map<string, ValueToColorMap>;

  /**
   * A Map object that holds a renderer value and its corresponding color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-utils.html#ValueToColorMap)
   */
  export type ValueToColorMap = globalThis.Map<string | number, Color>;

  export interface UniqueValueRenderer extends Renderer, VisualVariablesMixin {}

  export class UniqueValueRenderer {
    /**
     * UniqueValueRenderer allows you to symbolize features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) based on one or more categorical attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html)
     */

    constructor(properties?: UniqueValueRendererProperties);

    /**
     * This property is only relevant when symbolizing polygon features with marker symbols (or [IconSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html)) in the [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) of this renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol: FillSymbol | PolygonSymbol3D | CIMSymbol;
    /**
     * The label used in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) to describe features assigned the [default symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultLabel)
     */
    defaultLabel: string;
    /**
     * The symbol used to draw all features with values not referenced by [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) or [uniqueValueGroups](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultSymbol)
     */
    defaultSymbol: Symbol;
    /**
     * The name of the attribute field containing types or categorical values referenced in [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) or [uniqueValueGroups](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field)
     */
    field: string;
    /**
     * Specifies the name of an additional attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field2)
     */
    field2: string;
    /**
     * Specifies the name of a third attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field3)
     */
    field3: string;
    /**
     * A string used as a separator between the values in the legend if multiple attribute fields are used to categorize values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#fieldDelimiter)
     */
    fieldDelimiter: string;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#legendOptions)
     */
    legendOptions: UniqueValueRendererLegendOptions;
    /**
     * Indicates whether the order of the classes or [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) in the renderer definition should be used for the feature drawing order of the layer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#orderByClassesEnabled)
     */
    orderByClassesEnabled: boolean;
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#type)
     */
    readonly type: "unique-value";
    /**
     * An array of objects defining groups of unique values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups)
     */
    uniqueValueGroups: UniqueValueGroup[];
    /**
     * Defines categories and their corresponding symbols based on a set of values expected from the provided [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos)
     */
    uniqueValueInfos: UniqueValueInfo[];
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression)
     */
    valueExpression: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle: string;

    /**
     * Adds a unique value and symbol to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#addUniqueValueInfo)
     */
    addUniqueValueInfo(valueOrInfo: string | number | any, symbol?: Symbol | SymbolProperties): void;
    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#clone)
     */
    clone(): UniqueValueRenderer;
    /**
     * Returns rendering and legend information (as defined by the renderer) associated with the given graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#getUniqueValueInfo)
     */
    getUniqueValueInfo(graphic: Graphic): Promise<UniqueValueInfo>;
    /**
     * Removes a unique value from the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#removeUniqueValueInfo)
     */
    removeUniqueValueInfo(value: string | number): void;

    static fromJSON(json: any): UniqueValueRenderer;
  }

  interface UniqueValueRendererProperties extends RendererProperties, VisualVariablesMixinProperties {
    /**
     * This property is only relevant when symbolizing polygon features with marker symbols (or [IconSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html)) in the [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) of this renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#backgroundFillSymbol)
     */
    backgroundFillSymbol?:
      | FillSymbolProperties
      | (PolygonSymbol3DProperties & { type: "polygon-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The label used in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) to describe features assigned the [default symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultLabel)
     */
    defaultLabel?: string;
    /**
     * The symbol used to draw all features with values not referenced by [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) or [uniqueValueGroups](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#defaultSymbol)
     */
    defaultSymbol?: SymbolProperties;
    /**
     * The name of the attribute field containing types or categorical values referenced in [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) or [uniqueValueGroups](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field)
     */
    field?: string;
    /**
     * Specifies the name of an additional attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field2)
     */
    field2?: string;
    /**
     * Specifies the name of a third attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field3)
     */
    field3?: string;
    /**
     * A string used as a separator between the values in the legend if multiple attribute fields are used to categorize values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#fieldDelimiter)
     */
    fieldDelimiter?: string;
    /**
     * An object providing options for displaying the renderer in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#legendOptions)
     */
    legendOptions?: UniqueValueRendererLegendOptions;
    /**
     * Indicates whether the order of the classes or [uniqueValueInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos) in the renderer definition should be used for the feature drawing order of the layer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#orderByClassesEnabled)
     */
    orderByClassesEnabled?: boolean;
    /**
     * An array of objects defining groups of unique values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueGroups)
     */
    uniqueValueGroups?: UniqueValueGroupProperties[];
    /**
     * Defines categories and their corresponding symbols based on a set of values expected from the provided [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#uniqueValueInfos)
     */
    uniqueValueInfos?: UniqueValueInfoProperties[];
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression)
     */
    valueExpression?: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#valueExpressionTitle)
     */
    valueExpressionTitle?: string;
  }

  export interface UniqueValueRendererLegendOptions {
    /**
     * Describes the variable driving the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html#legendOptions)
     */
    title: string;
  }

  export interface VectorFieldRenderer extends Accessor, JSONSupport {}

  export class VectorFieldRenderer {
    /**
     * The VectorFieldRenderer allows you to display your raster data with vector symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html)
     */

    constructor(properties?: VectorFieldRendererProperties);

    /**
     * Attribute field presenting the magnitude.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#attributeField)
     */
    attributeField: "Magnitude";
    /**
     * Defines the flow direction of the data.
     *
     * @default "flow-from"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#flowRepresentation)
     */
    flowRepresentation: "flow-from" | "flow-to";
    /**
     * Predefined symbol styles used to represent the vector flow.
     *
     * @default "single-arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#style)
     */
    style:
      | "beaufort-ft"
      | "beaufort-km"
      | "beaufort-kn"
      | "beaufort-m"
      | "beaufort-mi"
      | "classified-arrow"
      | "ocean-current-kn"
      | "ocean-current-m"
      | "simple-scalar"
      | "single-arrow"
      | "wind-barb";
    /**
     * Determines the density of the symbols.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#symbolTileSize)
     */
    symbolTileSize: number;
    /**
     * The type of Renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#type)
     */
    readonly type: "vector-field";
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#visualVariables)
     */
    visualVariables: VisualVariable[];

    /**
     * Creates a deep clone of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#clone)
     */
    clone(): VectorFieldRenderer;

    static fromJSON(json: any): VectorFieldRenderer;
  }

  interface VectorFieldRendererProperties {
    /**
     * Attribute field presenting the magnitude.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#attributeField)
     */
    attributeField?: "Magnitude";
    /**
     * Defines the flow direction of the data.
     *
     * @default "flow-from"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#flowRepresentation)
     */
    flowRepresentation?: "flow-from" | "flow-to";
    /**
     * Predefined symbol styles used to represent the vector flow.
     *
     * @default "single-arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#style)
     */
    style?:
      | "beaufort-ft"
      | "beaufort-km"
      | "beaufort-kn"
      | "beaufort-m"
      | "beaufort-mi"
      | "classified-arrow"
      | "ocean-current-kn"
      | "ocean-current-m"
      | "simple-scalar"
      | "single-arrow"
      | "wind-barb";
    /**
     * Determines the density of the symbols.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#symbolTileSize)
     */
    symbolTileSize?: number;
    /**
     * An array of [VisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html#visualVariables)
     */
    visualVariables?: VisualVariableProperties[];
  }

  export interface ColorVariable extends VisualVariable, JSONSupport {}

  export class ColorVariable {
    /**
     * The color visual variable is used to visualize features along a continuous color ramp based on the values of a numeric attribute [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#field) or an [expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html)
     */

    constructor(properties?: ColorVariableProperties);

    /**
     * Name of the numeric attribute field by which to normalize the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#normalizationField)
     */
    normalizationField: string;
    /**
     * An array of sequential objects, or stops, that defines a continuous color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops)
     */
    stops: ColorStop[];
    /**
     * The visual variable type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#type)
     */
    readonly type: "color";

    /**
     * Creates a deep clone of the ColorVariable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#clone)
     */
    clone(): ColorVariable;

    static fromJSON(json: any): ColorVariable;
  }

  interface ColorVariableProperties extends VisualVariableProperties {
    /**
     * Name of the numeric attribute field by which to normalize the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#normalizationField)
     */
    normalizationField?: string;
    /**
     * An array of sequential objects, or stops, that defines a continuous color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops)
     */
    stops?: ColorStopProperties[];
  }

  export interface OpacityVariable extends VisualVariable, JSONSupport {}

  export class OpacityVariable {
    /**
     * The opacity visual variable defines the opacity of each feature's symbol based on a numeric [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#field) value or number returned from an [expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html)
     */

    constructor(properties?: OpacityVariableProperties);

    /**
     * Name of the numeric attribute field by which to normalize the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#normalizationField)
     */
    normalizationField: string;
    /**
     * An array of objects that defines the opacity to apply to features in a layer in a sequence of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#stops)
     */
    stops: OpacityStop[];
    /**
     * The visual variable type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#type)
     */
    readonly type: "opacity";

    /**
     * Creates a deep clone of the OpacityVariable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#clone)
     */
    clone(): OpacityVariable;

    static fromJSON(json: any): OpacityVariable;
  }

  interface OpacityVariableProperties extends VisualVariableProperties {
    /**
     * Name of the numeric attribute field by which to normalize the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#normalizationField)
     */
    normalizationField?: string;
    /**
     * An array of objects that defines the opacity to apply to features in a layer in a sequence of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#stops)
     */
    stops?: OpacityStopProperties[];
  }

  export interface RotationVariable extends VisualVariable, JSONSupport {}

  export class RotationVariable {
    /**
     * The rotation visual variable, determined by mapping the values to data in a field or by other arithmetic means with an Arcade expression, controls how features rendered with [marker symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html) or [text symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) , as well as with [marker symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html), [ObjectSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html) or [IconSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html), are rotated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html)
     */

    constructor(properties?: RotationVariableProperties);

    /**
     * Only applicable when working in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) with an [ObjectSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html).
     *
     * @default "heading"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#axis)
     */
    axis: "heading" | "tilt" | "roll";
    /**
     * Defines the origin and direction of rotation depending on how the angle of rotation was measured.
     *
     * @default "geographic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#rotationType)
     */
    rotationType: "geographic" | "arithmetic";
    /**
     * The visual variable type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#type)
     */
    readonly type: "rotation";

    /**
     * Creates a deep clone of the RotationVariable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#clone)
     */
    clone(): RotationVariable;

    static fromJSON(json: any): RotationVariable;
  }

  interface RotationVariableProperties extends VisualVariableProperties {
    /**
     * Only applicable when working in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) with an [ObjectSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html).
     *
     * @default "heading"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#axis)
     */
    axis?: "heading" | "tilt" | "roll";
    /**
     * Defines the origin and direction of rotation depending on how the angle of rotation was measured.
     *
     * @default "geographic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-RotationVariable.html#rotationType)
     */
    rotationType?: "geographic" | "arithmetic";
  }

  export interface SizeVariable extends VisualVariable, JSONSupport {}

  export class SizeVariable {
    /**
     * The size visual variable defines the size of individual features in a layer based on a numeric (often thematic) value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html)
     */

    constructor(properties?: SizeVariableProperties);

    /**
     * Only applicable when working in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#axis)
     */
    axis: "width" | "depth" | "height" | "width-and-depth" | "all";
    /**
     * The maximum data value used in the size ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxDataValue)
     */
    maxDataValue: number;
    /**
     * The size used to render a feature containing the maximum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxSize)
     */
    maxSize: number | ScaleDependentStops;
    /**
     * The minimum data value used in the size ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minDataValue)
     */
    minDataValue: number;
    /**
     * The size used to render a feature containing the minimum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minSize)
     */
    minSize: number | ScaleDependentStops;
    /**
     * The name of the numeric attribute field used to normalize the data in the given [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField)
     */
    normalizationField: string;
    /**
     * An array of objects that defines the mapping of data values returned from [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression) to icon sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#stops)
     */
    stops: SizeStop[];
    /**
     * This value must be `outline` when scaling polygon outline widths based on the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#target)
     */
    target: string;
    /**
     * The visual variable type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#type)
     */
    readonly type: "size";
    /**
     * When setting a size visual variable on a renderer using an [ObjectSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html), this property indicates whether to apply the value defined by the [height](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#height), [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#width), or [depth](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#depth) properties to the corresponding [axis](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#axis) of this visual variable instead of proportionally scaling this axis' value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#useSymbolValue)
     */
    useSymbolValue: boolean;
    /**
     * Specifies how to apply the data value when mapping real-world sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueRepresentation)
     */
    valueRepresentation: "radius" | "diameter" | "area" | "width" | "distance";
    /**
     * Indicates the unit of measurement used to interpret the value returned by [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueUnit)
     */
    valueUnit:
      | "unknown"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers";

    /**
     * Creates a deep clone of the SizeVisualVariable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#clone)
     */
    clone(): SizeVariable;
    /**
     * Modifies the SizeVariable in place by flipping the sizes in the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#flipSizes)
     */
    flipSizes(): void;

    static fromJSON(json: any): SizeVariable;
  }

  interface SizeVariableProperties extends VisualVariableProperties {
    /**
     * Only applicable when working in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#axis)
     */
    axis?: "width" | "depth" | "height" | "width-and-depth" | "all";
    /**
     * The maximum data value used in the size ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxDataValue)
     */
    maxDataValue?: number;
    /**
     * The size used to render a feature containing the maximum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxSize)
     */
    maxSize?: number | (ScaleDependentStops & { type: "size" }) | string;
    /**
     * The minimum data value used in the size ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minDataValue)
     */
    minDataValue?: number;
    /**
     * The size used to render a feature containing the minimum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minSize)
     */
    minSize?: number | (ScaleDependentStops & { type: "size" }) | string;
    /**
     * The name of the numeric attribute field used to normalize the data in the given [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField)
     */
    normalizationField?: string;
    /**
     * An array of objects that defines the mapping of data values returned from [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression) to icon sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#stops)
     */
    stops?: SizeStopProperties[];
    /**
     * This value must be `outline` when scaling polygon outline widths based on the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#target)
     */
    target?: string;
    /**
     * When setting a size visual variable on a renderer using an [ObjectSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html), this property indicates whether to apply the value defined by the [height](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#height), [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#width), or [depth](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#depth) properties to the corresponding [axis](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#axis) of this visual variable instead of proportionally scaling this axis' value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#useSymbolValue)
     */
    useSymbolValue?: boolean;
    /**
     * Specifies how to apply the data value when mapping real-world sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueRepresentation)
     */
    valueRepresentation?: "radius" | "diameter" | "area" | "width" | "distance";
    /**
     * Indicates the unit of measurement used to interpret the value returned by [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field) or [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueUnit)
     */
    valueUnit?:
      | "unknown"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers";
  }

  /**
   * Defines a size visual variable with minimum and maximum bounds.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
   */
  export interface BoundedMinMax {
    /**
     * Value must be `size`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * See [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    field?: string;
    /**
     * See [normalizationField](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    normalizationField?: string;
    /**
     * See [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    valueExpression?: string;
    /**
     * See [valueExpressionTitle](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpressionTitle).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    valueExpressionTitle?: string;
    /**
     * See [maxDataValue](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxDataValue).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    maxDataValue: number;
    /**
     * The size used to render a feature containing the maximum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    maxSize: string | number;
    /**
     * See [minDataValue](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minDataValue).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    minDataValue: number;
    /**
     * The size used to render a feature containing the minimum data value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax)
     */
    minSize: string | number;
  }

  /**
   * Defines a size visual variable where data values are interpreted as real-world sizes based on a given unit.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
   */
  export interface RealWorldSize {
    /**
     * See [axis](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    axis?: string;
    /**
     * Value must be `size`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * See [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    field?: string;
    /**
     * See [normalizationField](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    normalizationField?: string;
    /**
     * See [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    valueExpression?: string;
    /**
     * See [valueUnit](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueUnit).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    valueUnit?: string;
    /**
     * See [valueRepresentation](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueRepresentation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#RealWorldSize)
     */
    valueRepresentation?: string;
  }

  /**
   * Defines icon sizes in a size visual variable based on minimum and maximum bounds similar to the [BoundedMinMax](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#BoundedMinMax) case.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
   */
  export interface ScaleDependentIcons {
    /**
     * Value must be `size`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * See [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    field?: string;
    /**
     * See [normalizationField](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    normalizationField?: string;
    /**
     * See [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    valueExpression?: string;
    /**
     * See [maxDataValue](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#maxDataValue).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    maxDataValue: number;
    /**
     * An object defining the size of features whose data value (defined in `field` or `valueExpression`) is greater than or equal to the `maxDataValue` for the given view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    maxSize: ScaleDependentStops;
    /**
     * See [minDataValue](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#minDataValue).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    minDataValue: number;
    /**
     * An object defining the size of features whose data value (defined in `field` or `valueExpression`) is less than or equal to the `minDataValue` for the given view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons)
     */
    minSize: ScaleDependentStops;
  }

  /**
   * Defines feature sizes and outline widths in a [size visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#SizeVisualVariable) based on the [MapView.scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentStops)
   */
  export interface ScaleDependentStops {
    /**
     * Value must be `size`.
     *
     * [Read more...](global.html#type)
     */
    type: "size";
    /**
     * This value must be `$view.scale`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentStops)
     */
    valueExpression: string;
    /**
     * This value must be `outline` when scaling polygon outline widths based on the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentStops)
     */
    target?: "outline";
    /**
     * An array of objects that define the size of the icon (or alternatively the width of the polygon outline) at the given scale `value`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentStops)
     */
    stops: SizeStop[];
  }

  /**
   * Defines two or more stops at which feature sizes are mapped to data values in a size visual variable.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ThematicStops)
   */
  export interface ThematicStops {
    /**
     * Value must be `size`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * See [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#field).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ThematicStops)
     */
    field?: string;
    /**
     * See [normalizationField](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#normalizationField).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ThematicStops)
     */
    normalizationField?: string;
    /**
     * See [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#valueExpression).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ThematicStops)
     */
    valueExpression?: string;
    /**
     * An array of objects defining the thematic size ramp in a sequence of data or expression stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ThematicStops)
     */
    stops: SizeStop[];
  }

  export interface ColorSizeStop extends Accessor, JSONSupport {}

  export class ColorSizeStop {
    /**
     * Defines how to data values should be represented in the [ColorSizeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html) with an associated color and size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html)
     */

    constructor(properties?: ColorSizeStopProperties);

    /**
     * The [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#color)
     */
    color: Color;
    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#label)
     */
    label: string;
    /**
     * The size value in points used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#size)
     */
    size: number;
    /**
     * Specifies the data value to map to the given [size](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#size) and [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value)
     */
    value: number;

    /**
     * Creates a deep clone of the ColorSizeStop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#clone)
     */
    clone(): ColorSizeStop;

    static fromJSON(json: any): ColorSizeStop;
  }

  interface ColorSizeStopProperties {
    /**
     * The [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#color)
     */
    color?: Color | number[] | string;
    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#label)
     */
    label?: string;
    /**
     * The size value in points used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#size)
     */
    size?: number | string;
    /**
     * Specifies the data value to map to the given [size](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#size) and [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorSizeStop.html#value)
     */
    value?: number;
  }

  export interface ColorStop extends Accessor, JSONSupport {}

  export class ColorStop {
    /**
     * Defines a color stop used for creating a continuous color visualization in a [color visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html)
     */

    constructor(properties?: ColorStopProperties);

    /**
     * The [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#color)
     */
    color: Color;
    /**
     * A string value used to label the stop along the color ramp in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#label)
     */
    label: string;
    /**
     * Specifies the data value to map to the given [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#value)
     */
    value: number;

    /**
     * Creates a deep clone of the ColorStop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#clone)
     */
    clone(): ColorStop;

    static fromJSON(json: any): ColorStop;
  }

  interface ColorStopProperties {
    /**
     * The [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#color)
     */
    color?: Color | number[] | string;
    /**
     * A string value used to label the stop along the color ramp in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#label)
     */
    label?: string;
    /**
     * Specifies the data value to map to the given [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#color).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-ColorStop.html#value)
     */
    value?: number;
  }

  export interface OpacityStop extends Accessor, JSONSupport {}

  export class OpacityStop {
    /**
     * Defines an opacity stop used for creating a continuous opacity visualization in a [opacity visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html)
     */

    constructor(properties?: OpacityStopProperties);

    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#label)
     */
    label: string;
    /**
     * The opacity value (between `0.0` and `1.0`) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#opacity)
     */
    opacity: number;
    /**
     * Specifies the data value to map to the given [opacity value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#opacity).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#value)
     */
    value: number;

    /**
     * Creates a deep clone of the OpacityStop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#clone)
     */
    clone(): OpacityStop;

    static fromJSON(json: any): OpacityStop;
  }

  interface OpacityStopProperties {
    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#label)
     */
    label?: string;
    /**
     * The opacity value (between `0.0` and `1.0`) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#opacity)
     */
    opacity?: number | number[] | string;
    /**
     * Specifies the data value to map to the given [opacity value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#opacity).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-OpacityStop.html#value)
     */
    value?: number;
  }

  export interface SizeStop extends Accessor, JSONSupport {}

  export class SizeStop {
    /**
     * Defines a size stop used for creating a continuous size visualization in a [size visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html)
     */

    constructor(properties?: SizeStopProperties);

    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#label)
     */
    label: string;
    /**
     * The size value in points (between `0` and `90`) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#size)
     */
    size: number;
    /**
     * Specifies the data value to map to the given [size](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#size).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#value)
     */
    value: number;

    /**
     * Creates a deep clone of the SizeStop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#clone)
     */
    clone(): SizeStop;

    static fromJSON(json: any): SizeStop;
  }

  interface SizeStopProperties {
    /**
     * A string value used to label the stop in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#label)
     */
    label?: string;
    /**
     * The size value in points (between `0` and `90`) used to render features with the given [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#value).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#size)
     */
    size?: number | string;
    /**
     * Specifies the data value to map to the given [size](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#size).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-support-SizeStop.html#value)
     */
    value?: number;
  }

  export interface VisualVariable extends Accessor, JSONSupport {}

  export class VisualVariable {
    /**
     * The visual variable base class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html)
     */

    constructor(properties?: VisualVariableProperties);

    /**
     * The name of the numeric attribute field that contains the data values used to determine the color/opacity/size/rotation of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#field)
     */
    field: string;
    /**
     * An object providing options for displaying the visual variable in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#legendOptions)
     */
    legendOptions: VisualVariableLegendOptions;
    /**
     * The visual variable type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#type)
     */
    readonly type: "color" | "opacity" | "rotation" | "size";
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpression)
     */
    valueExpression: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpressionTitle)
     */
    valueExpressionTitle: string;

    static fromJSON(json: any): VisualVariable;
  }

  interface VisualVariableProperties {
    /**
     * The name of the numeric attribute field that contains the data values used to determine the color/opacity/size/rotation of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#field)
     */
    field?: string;
    /**
     * An object providing options for displaying the visual variable in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#legendOptions)
     */
    legendOptions?: VisualVariableLegendOptions;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpression)
     */
    valueExpression?: string;
    /**
     * The title identifying and describing the associated [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression as defined in the [valueExpression](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpression) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#valueExpressionTitle)
     */
    valueExpressionTitle?: string;
  }

  export interface VisualVariableLegendOptions {
    /**
     * Indicates whether to show the visual variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#legendOptions)
     */
    showLegend: boolean;
    /**
     * The title describing the visualization of the visual variable in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-VisualVariable.html#legendOptions)
     */
    title: string;
  }

  /**
   * ClassBreaksRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#ClassBreaksRenderer)
   */
  export type renderersClassBreaksRenderer = ClassBreaksRenderer;

  /**
   * DictionaryRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#DictionaryRenderer)
   */
  export type renderersDictionaryRenderer = DictionaryRenderer;

  /**
   * DotDensityRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#DotDensityRenderer)
   */
  export type renderersDotDensityRenderer = DotDensityRenderer;

  /**
   * HeatmapRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#HeatmapRenderer)
   */
  export type renderersHeatmapRenderer = HeatmapRenderer;

  /**
   * PieChartRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#PieChartRenderer)
   */
  export type renderersPieChartRenderer = PieChartRenderer;

  /**
   * Renderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#Renderer)
   */
  export type renderersRenderer = HeatmapRenderer | RendererWithVisualVariables;

  /**
   * RendererWithVisualVariables.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#RendererWithVisualVariables)
   */
  export type RendererWithVisualVariables =
    | SimpleRenderer
    | ClassBreaksRenderer
    | UniqueValueRenderer
    | DotDensityRenderer
    | DictionaryRenderer
    | PieChartRenderer;

  /**
   * SimpleRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#SimpleRenderer)
   */
  export type renderersSimpleRenderer = SimpleRenderer;

  /**
   * UniqueValueRenderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers.html#UniqueValueRenderer)
   */
  export type renderersUniqueValueRenderer = UniqueValueRenderer;

  /**
   * Retrieves data from a remote server or uploads a file.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html)
   */
  interface request {
    /**
     * Retrieves data from a remote server or uploads a file from a user's computer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#esriRequest)
     */
    esriRequest(url: string | URL, options?: RequestOptions): Promise<RequestResponse>;
  }

  const __requestMapped: request;
  export const request: typeof __requestMapped.esriRequest;

  /**
   * The specification of the [details object](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html#details) returned in an [Error](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html) object.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
   */
  export interface EsriErrorDetails {
    /**
     * A function to retrieve headers sent from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    getHeader: GetHeader;
    /**
     * A function for retrieving all headers sent from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    getAllHeaders: GetAllHeaders;
    /**
     * The status code of the http response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    httpStatus: number;
    /**
     * The error message code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    messageCode: string;
    /**
     * Additional error message(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    messages: string[];
    /**
     * The raw error object if the server returned a JSON error, or the response text otherwise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    raw: any | string;
    /**
     * The options used in the http request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    requestOptions: RequestOptions;
    /**
     * Indicates if the request required https.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    ssl: boolean;
    /**
     * The error message subcode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    subCode: number;
    /**
     * The URL of the request that returned an error message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#EsriErrorDetails)
     */
    url: string;
  }

  export type GetAllHeaders = () => string[][];

  export type GetHeader = (headerName: string) => string;

  /**
   * An object with the following properties that describe the request.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
   */
  export interface RequestOptions {
    /**
     * Indicates if and how requests to ArcGIS Services are authenticated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    authMode?: "auto" | "anonymous" | "immediate" | "no-prompt";
    /**
     * If uploading a file, specify the form data or element used to submit the file here.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    body?: FormData | HTMLFormElement | string;
    /**
     * If `true`, the browser will send a request to the server instead of using the browser's local cache.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    cacheBust?: boolean;
    /**
     * Headers to use for the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    headers?: any;
    /**
     * Indicates if the request should be made using the HTTP DELETE, HEAD, POST, or PUT method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    method?: "auto" | "delete" | "head" | "post" | "put";
    /**
     * Query parameters for the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    query?: any | URLSearchParams;
    /**
     * Response format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    responseType?: "json" | "text" | "array-buffer" | "blob" | "image" | "native" | "document" | "xml";
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    signal?: AbortSignal;
    /**
     * Indicates the amount of time in milliseconds to wait for a response from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    timeout?: number;
    /**
     * Indicates the request should use the proxy.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    useProxy?: boolean;
    /**
     * Indicates if cross-site `Access-Control` requests should use credentials.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestOptions)
     */
    withCredentials?: boolean;
  }

  /**
   * Returns a promise that resolves to an object with the following specification.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
   */
  export interface RequestResponse {
    /**
     * The requested data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    data?: any;
    /**
     * Method for getting a header sent from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    getHeader?: GetHeader;
    /**
     * Method for getting all headers sent from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    getAllHeaders?: GetAllHeaders;
    /**
     * _Since 4.26_ The status code of the http response.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    httpStatus?: number;
    /**
     * The options specified by the user in the data request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    requestOptions?: RequestOptions;
    /**
     * Indicates if the request required https.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    ssl?: boolean;
    /**
     * The URL used to request the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#RequestResponse)
     */
    url?: string;
  }

  /**
   * Helps you find closest facilities around any location (incident) on a network.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html)
   */
  interface closestFacility {
    /**
     * Solves the closest facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html#solve)
     */
    solve(url: string, params: ClosestFacilityParameters, requestOptions?: any): Promise<ClosestFacilitySolveResult>;
  }

  export const closestFacility: closestFacility;

  export class FeatureService {
    /**
     * Describes the layer's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#capabilities)
     */
    capabilities: FeatureServiceCapabilities;
    /**
     * Describes effective capabilities of the service taking in to consideration privileges of the currently signed-in user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#effectiveCapabilities)
     */
    readonly effectiveCapabilities: FeatureServiceCapabilities;
    /**
     * Contains info of all layers in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#layerInfos)
     */
    layerInfos: FeatureServiceLayerInfo[];
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * Contains info of all tables in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#tableInfos)
     */
    tableInfos: TableInfo[];
    /**
     * The absolute URL of the REST endpoint for the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#url)
     */
    url: string;
    /**
     * Describes the service's userTypeExtensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#userTypeExtensions)
     */
    userTypeExtensions: string[];
    /**
     * The url that points to the utility network layer, if it exists.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#utilityNetworkUrl)
     */
    readonly utilityNetworkUrl: string;
    /**
     * The url to the version management service, if the data is versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#versionManagementServiceUrl)
     */
    readonly versionManagementServiceUrl: string;

    /**
     * Applies edits to features in the feature service layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    applyEdits(
      edits: FeatureServiceApplyEditsEdits[],
      options?: FeatureServiceApplyEditsOptions,
    ): Promise<ServiceEditsResult[]>;
    /**
     * Triggers the loading of the feature service instance and fetches all layers and tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#fetchAllLayersAndTables)
     */
    fetchAllLayersAndTables(): Promise<ServiceContents>;
    /**
     * Triggers the loading of the feature service instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#load)
     */
    load(): Promise<any>;

    constructor(properties?: any);
  }

  /**
   * Describes the layer's supported capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
   */
  export interface FeatureServiceCapabilities {
    /**
     * Describes characteristics of the data in the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    data: FeatureServiceCapabilitiesData;
    /**
     * Describes editing capabilities that can be performed on the features in the layer via [applyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    editing: FeatureServiceCapabilitiesEditing;
    /**
     * Describes operations that can be performed on features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    operations: FeatureServiceCapabilitiesOperations;
    /**
     * Describes [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) operations that can be performed on the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    query: FeatureServiceCapabilitiesQuery;
    /**
     * Describes the synchronization capabilities of a service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    sync: CapabilitiesSync;
  }

  export interface FeatureServiceApplyEditsEdits {
    /**
     * Specifies the layer id of the layer that needs to be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    id: number;

    identifierFields: FeatureServiceApplyEditsEditsIdentifierFields;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    addFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    updateFeatures?: Graphic[] | Collection<Graphic>;
    /**
     * An array or a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of features, or an array of objects with `objectId` or `globalId` of each feature to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    deleteFeatures?: Graphic[] | Collection<Graphic> | any[];
    /**
     * An array of attachments to be added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    addAttachments?: AttachmentEdit[];
    /**
     * An array of attachments to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    updateAttachments?: AttachmentEdit[];
    /**
     * An array of [globalId](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#AttachmentEdit)s for attachments to be deleted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    deleteAttachments?: string[];
  }

  export interface FeatureServiceApplyEditsEditsIdentifierFields {
    /**
     * Specifies the globalIdField of the layer being editied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    globalIdField?: string;
    /**
     * Specifies the objectIdField of the layer being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    objectIdField: string;
  }

  export interface FeatureServiceApplyEditsOptions {
    /**
     * The geodatabase version to apply the edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    gdbVersion?: string;
    /**
     * Indicates whether the edits can be applied using globalIds of features or attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    globalIdUsed?: boolean;
    /**
     * Added at 10.5 and works with ArcGIS Server services only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    honorSequenceOfEdits?: boolean;
    /**
     * This option was added at 10.6.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits)
     */
    usePreviousEditMoment?: boolean;
  }

  /**
   * Contains information for a layer in the Feature Service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#LayerInfo)
   */
  export interface FeatureServiceLayerInfo {
    /**
     * The id of a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#LayerInfo)
     */
    id: number;
    /**
     * The name of a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#LayerInfo)
     */
    name: string;
    /**
     * The type of the layer.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The service url of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#LayerInfo)
     */
    url: string;
    /**
     * The geometry type of a layer.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType?: "point" | "polyline" | "polygon" | "envelope" | "multipoint";
  }

  /**
   * Contains information returned from the service containing layerInfos, tableInfos, and server capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
   */
  export interface ServiceContents {
    /**
     * Contains the Layer Definition of a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    layers: ServiceContentsLayers[];
    /**
     * Contains the Table Definition of a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    tables: ServiceContentsTables[];
  }

  /**
   * Results returned from the [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
   */
  export interface ServiceEditsResult {
    /**
     * The layerId of the feature layer where features were edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    id: number;
    /**
     * Results returned from an add operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    addFeatureResults: any[];
    /**
     * Results returned from an update operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    updateFeatureResults: any[];
    /**
     * Results returned from a delete operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    deleteFeatureResults: any[];
    /**
     * Results returned from an add attachments operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    addAttachmentResults: any[];
    /**
     * Results returned from an update attachments operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    updateAttachmentResults: any[];
    /**
     * Results returned from a delete attachments operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    deleteAttachmentResults: any[];
    /**
     * Object containing all edited features belonging to the specified layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    editedFeatures?: any[];
    /**
     * Returns the editMoment of an edit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceEditsResult)
     */
    editMoment?: number;
  }

  /**
   * Contains information for a table in the Feature Service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#TableInfo)
   */
  export interface TableInfo {
    /**
     * The id of a table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#TableInfo)
     */
    id: number;
    /**
     * The name of a table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#TableInfo)
     */
    name: string;
    /**
     * The type of the table.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The service url of the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#TableInfo)
     */
    url: string;
  }

  export interface FeatureServiceCapabilitiesData {
    /**
     * Indicates if the feature service is versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    isDataVersioned: boolean;
    /**
     * Indicates if the feature service is branch versioned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    isDataBranchVersioned: boolean;
  }

  export interface FeatureServiceCapabilitiesEditing {
    /**
     * Indicates if the `globalId` values provided by the client are used in [applyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsGlobalId: boolean;
    /**
     * Indicates that the service supports returning edits in the source spatial reference of the layer they belong to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsReturnServiceEditsInSourceSpatialReference: boolean;
    /**
     * Indicates if the service supports async apply edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsAsyncApplyEdits: boolean;
    /**
     * Indicates whether the service supports splitting features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsSplit: boolean;
  }

  export interface FeatureServiceCapabilitiesOperations {
    /**
     * Indicates if new features can be [added](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits) to the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsAdd: boolean;
    /**
     * Indicates whether the service supports change tracking for features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsChangeTracking: boolean;
    /**
     * Indicates if features can be [deleted](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits) from the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsDelete: boolean;
    /**
     * Indicates if features in the service can be [edited](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#applyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsEditing: boolean;
    /**
     * Indicates if features in the service can be [queried](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#queryFeatures).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsQuery: boolean;
    /**
     * Indicates whether the service supports querying data elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsQueryDataElements: boolean;
    /**
     * Indicates whether the service supports querying domains.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsQueryDomains: boolean;
    /**
     * Indicates whether the service supports querying contingent values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsQueryContingentValues: boolean;
    /**
     * Indicates whether the service supports synchronization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsSync: boolean;
    /**
     * Indicates whether the service supports updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsUpdate: boolean;
  }

  export interface FeatureServiceCapabilitiesQuery {
    /**
     * The maximum number of records that will be returned for a given query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    maxRecordCount: number;
    /**
     * This property is used to limit the number of records returned based on a factor of the `query.maxRecordCount`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    maxRecordCountFactor: number;
  }

  export interface CapabilitiesSync {
    /**
     * Indicates whether the service supports asynchronous synchronization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportsAsync: boolean;
    /**
     * Indicates the supported sync data options for the service
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    supportedSyncDataOptions: CapabilitiesSyncSupportedSyncDataOptions;
  }

  export interface CapabilitiesSyncSupportedSyncDataOptions {
    /**
     * Indicates whether annotations should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    annotations: boolean;
    /**
     * Indicates whether dimensions should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    dimensions: boolean;
    /**
     * Indicates whether contingent values should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    contingentValues: boolean;
    /**
     * Indicates whether attribute rules should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    attributeRules: boolean;
    /**
     * Indicates whether the utility network system should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    utilityNetworkSystem: boolean;
    /**
     * Indicates whether the full annotation model should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    annotationFullModel: boolean;
    /**
     * Indicates whether 3D objects should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    include3DObjects: boolean;
    /**
     * Indicates whether missing utility network layers should be included in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    utilityNetworkMissingLayers: boolean;
    /**
     * Indicates whether true curves should be preserved in the sync data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#Capabilities)
     */
    preserveTrueCurves: boolean;
  }

  export interface ServiceContentsLayers {
    /**
     * The capabilities of a layer in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    capabilities: Capabilities;
    /**
     * Contains information for a layer in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    layerInfos: FeatureServiceLayerInfo;
  }

  export interface ServiceContentsTables {
    /**
     * The capabilities of a table in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    capabilities: Capabilities;
    /**
     * Contains information for a table in the Feature Service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html#ServiceContents)
     */
    tableInfos: TableInfo;
  }

  /**
   * Provides utility methods for creating [FeatureServices](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html)
   */
  interface featureServiceUtils {
    /**
     * Used to create an instance of [FeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html) from an array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html#createFeatureServices)
     */
    createFeatureServices(layers: FeatureLayer[]): Promise<globalThis.Map<string, FeatureServiceResourcesBundle>>;
  }

  export const featureServiceUtils: featureServiceUtils;

  /**
   * Results returned from the [createFeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html#createFeatureService) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html#FeatureServiceResourcesBundle)
   */
  export interface FeatureServiceResourcesBundle {
    /**
     * Returns a featureService object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html#FeatureServiceResourcesBundle)
     */
    featureService: FeatureService;
    /**
     * Returns an array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-utils.html#FeatureServiceResourcesBundle)
     */
    layers: FeatureLayer[];
  }

  /**
   * Search a map service exposed by the ArcGIS Server REST API based on a string value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-find.html)
   */
  interface find {
    /**
     * Sends a request to the ArcGIS REST map service resource to perform a search based on the input [params](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-find.html#find)
     */
    find(url: string, params: FindParameters | FindParametersProperties, requestOptions?: any): Promise<any>;
  }

  export const find: find;

  /**
   * Represents geometry service resources exposed by the ArcGIS REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html)
   */
  interface geometryService {
    /**
     * Computes the area and length for the input [polygons](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#areasAndLengths)
     */
    areasAndLengths(
      url: string,
      areasAndLengthsParameters: AreasAndLengthsParameters,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * The Auto Complete operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#autoComplete)
     */
    autoComplete(url: string, polygons: Polygon[], polylines: Polyline[], requestOptions?: any): Promise<Polygon[]>;
    /**
     * Creates buffer polygons at a specified distance around the given geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#buffer)
     */
    buffer(url: string, bufferParameters: BufferParameters, requestOptions?: any): Promise<Polygon[]>;
    /**
     * The convexHull operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#convexHull)
     */
    convexHull(url: string, geometries: Geometry[], requestOptions?: any): Promise<Geometry>;
    /**
     * The cut operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#cut)
     */
    cut(url: string, geometries: Geometry[], cutter: Polyline, requestOptions?: any): Promise<any>;
    /**
     * The densify operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#densify)
     */
    densify(url: string, densifyParameters: DensifyParameters, requestOptions?: any): Promise<Geometry[]>;
    /**
     * The difference operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#difference)
     */
    difference(url: string, geometries: Geometry[], geometry: Geometry, requestOptions?: any): Promise<Geometry>;
    /**
     * Measures the planar or geodesic distance between geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#distance)
     */
    distance(url: string, distanceParameters: DistanceParameters, requestOptions?: any): Promise<number>;
    /**
     * Converts an array of well-known strings into xy-coordinates based on the conversion type and spatial reference supplied by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#fromGeoCoordinateString)
     */
    fromGeoCoordinateString(
      url: string,
      params: geometryServiceFromGeoCoordinateStringParams,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Generalizes the input geometries using the Douglas-Peucker algorithm.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#generalize)
     */
    generalize(url: string, generalizeParameters: GeneralizeParameters, requestOptions?: any): Promise<Geometry[]>;
    /**
     * The intersect operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#intersect)
     */
    intersect(url: string, geometries: Geometry[], intersector: Geometry, requestOptions?: any): Promise<Geometry[]>;
    /**
     * Calculates an interior point for each polygon specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#labelPoints)
     */
    labelPoints(url: string, polygons: Polygon[], requestOptions?: any): Promise<Point[]>;
    /**
     * Gets the lengths for a [Geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html) when the geometry type is [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#lengths)
     */
    lengths(url: string, lengthsParameters: LengthsParameters, requestOptions?: any): Promise<any>;
    /**
     * Constructs the offset of the input geometries based on a planar distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#offset)
     */
    offset(url: string, offsetParameters: OffsetParameters, requestOptions?: any): Promise<Geometry[]>;
    /**
     * Projects a set of geometries to a new spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#project)
     */
    project(url: string, projectParameters: ProjectParameters, requestOptions?: any): Promise<Geometry[]>;
    /**
     * Computes the set of pairs of geometries from the input geometry arrays that belong to the specified relation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#relation)
     */
    relation(url: string, relationParameters: RelationParameters, requestOptions?: any): Promise<Polygon[]>;
    /**
     * The reshape operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#reshape)
     */
    reshape(url: string, geometry: Geometry, reshaper: Polyline, requestOptions?: any): Promise<Geometry>;
    /**
     * Alters the given geometries to make their definitions topologically legal with respect to their geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#simplify)
     */
    simplify(url: string, geometries: Geometry[], requestOptions?: any): Promise<Geometry[]>;
    /**
     * Converts an array of XY-coordinates into well-known strings based on the conversion type and spatial reference supplied by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    toGeoCoordinateString(
      url: string,
      params: geometryServiceToGeoCoordinateStringParams,
      requestOptions?: any,
    ): Promise<string[]>;
    /**
     * Trims or extends the input polylines using the user specified guide polyline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#trimExtend)
     */
    trimExtend(url: string, trimExtendParameters: TrimExtendParameters, requestOptions?: any): Promise<Polyline[]>;
    /**
     * The union operation is performed on a geometry service resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#union)
     */
    union(url: string, geometries: Geometry[], requestOptions?: any): Promise<Geometry>;
  }

  export const geometryService: geometryService;

  export interface geometryServiceFromGeoCoordinateStringParams {
    /**
     * An array of formatted strings as specified by `conversionType`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#fromGeoCoordinateString)
     */
    strings: string[];
    /**
     * The spatial reference or well-known ID to convert the input string coordinates to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#fromGeoCoordinateString)
     */
    sr: SpatialReference | string | number;
    /**
     * The conversion type of the input strings.
     *
     * @default mrgs
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#fromGeoCoordinateString)
     */
    conversionType?: "mrgs" | "usng" | "utm" | "geo-ref" | "gars" | "dms" | "ddm" | "dd";
    /**
     * Conversion options for mrgs, utm and gars conversion types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#fromGeoCoordinateString)
     */
    conversionMode?: string;
  }

  export interface geometryServiceToGeoCoordinateStringParams {
    /**
     * The spatial reference (or WKID of the spatial reference) of the XY-coordinates to be converted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    sr: SpatialReference | string | number;
    /**
     * An array of XY-coordinates (in JSON format) to be converted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    coordinates: number[][];
    /**
     * The conversion type of the input strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    conversionType: "mgrs" | "usng" | "utm" | "geo-ref" | "gars" | "dms" | "ddm" | "dd";
    /**
     * Conversion options for mgrs and utm conversion types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    conversionMode?: string;
    /**
     * The number of digits to output for each of the numerical portions in the string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    numOfDigits?: number;
    /**
     * If `true`, then numeric portions of the string are rounded to the nearest whole magnitude as specified by `numOfDigits`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    rounding?: boolean;
    /**
     * If `true`, then spaces are added between components of the string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#toGeoCoordinateString)
     */
    addSpaces?: boolean;
  }

  /**
   * Represents a GP resource exposed by the ArcGIS REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor.html)
   */
  interface geoprocessor {
    /**
     * Sends a request to the server to execute a synchronous GP task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor.html#execute)
     */
    execute(url: string, params?: any, options?: GPOptions, requestOptions?: any): Promise<any>;
    /**
     * Submits a job to the server for asynchronous processing by the GP task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor.html#submitJob)
     */
    submitJob(url: string, params?: any, options?: GPOptions, requestOptions?: any): Promise<JobInfo>;
  }

  export const geoprocessor: geoprocessor;

  /**
   * A convenience module for providing input options for the geoprocessing service return values.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html)
   */
  interface GPOptions {
    /**
     * The spatial reference of the output geometries.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * ProcessExtent, if specified, will only process features that overlap this extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#processExtent)
     */
    processExtent: Extent;
    /**
     * The spatial reference that the model will use to perform geometry operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#processSpatialReference)
     */
    processSpatialReference: SpatialReference;
    /**
     * When `true`, individual values in a value table will be prefixed with the associated column name.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#returnColumnName)
     */
    returnColumnName: boolean;
    /**
     * When `true`, a parameter value of type `"feature-record-set-layer"` will be returned as a [feature layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) rather than a [feature set](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#returnFeatureCollection)
     */
    returnFeatureCollection: boolean;
    /**
     * If `true`, m-values will be included in the results if the features have m-values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#returnM)
     */
    returnM: boolean;
    /**
     * If `true`, z-values will be included in the results if the features have z-values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor-GPOptions.html#returnZ)
     */
    returnZ: boolean;
  }

  export const GPOptions: GPOptions;

  /**
   * Performs an identify operation on the layers of a map service exposed by the ArcGIS Server REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html)
   */
  interface identify {
    /**
     * Sends a request to the ArcGIS REST map service resource to identify features based on the [IdentifyParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html) specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html#identify)
     */
    identify(url: string, params: IdentifyParameters, requestOptions?: any): Promise<any>;
  }

  export const identify: identify;

  /**
   * Performs various operations on an image service resource:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html)
   */
  interface imageService {
    /**
     * Computes the rotation angle of a [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) at a given location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computeAngles)
     */
    computeAngles(
      url: string,
      parameters: ImageAngleParameters | ImageAngleParametersProperties,
      requestOptions?: any,
    ): Promise<ImageAngleResult>;
    /**
     * Computes histograms based on the provided [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computeHistograms)
     */
    computeHistograms(
      url: string,
      parameters: ImageHistogramParameters | ImageHistogramParametersProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Computes the corresponding pixel location in columns and rows for an [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) based on input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computePixelSpaceLocations)
     */
    computePixelSpaceLocations(
      url: string,
      parameters: ImagePixelLocationParameters,
      requestOptions?: any,
    ): Promise<ImagePixelLocationResult>;
    /**
     * Computes [statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterBandStatistics) and [histograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#RasterHistogram) for the provided [ImageHistogramParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computeStatisticsHistograms)
     */
    computeStatisticsHistograms(
      url: string,
      parameters: ImageHistogramParameters | ImageHistogramParametersProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Finds images based on the provided [FindImagesParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#findImages)
     */
    findImages(
      url: string,
      parameters: FindImagesParameters | FindImagesParametersProperties,
      requestOptions?: any,
    ): Promise<FindImagesResult>;
    /**
     * Retrieves an image's url using the provided [ImageUrlParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#getImageUrl)
     */
    getImageUrl(
      url: string,
      parameters: ImageUrlParameters | ImageUrlParametersProperties,
      requestOptions?: any,
    ): Promise<ImageUrlResult>;
    /**
     * Returns sample point locations, pixel values and corresponding resolutions of the source data for a given geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#getSamples)
     */
    getSamples(
      url: string,
      parameters: ImageSampleParameters | ImageSampleParametersProperties,
      requestOptions?: any,
    ): Promise<ImageSampleResult>;
    /**
     * Sends a request to the ArcGIS REST image service resource to identify content based on the [ImageIdentifyParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html) specified in the `params` argument.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#identify)
     */
    identify(url: string, params: ImageIdentifyParameters, requestOptions?: any): Promise<ImageIdentifyResult>;
    /**
     * Convert a geometry from an image space to a map space using the provided [ImageToMapParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#imageToMap)
     */
    imageToMap(
      url: string,
      parameters: ImageToMapParameters | ImageToMapParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Creates a map space geometry from multiray image space geometries using the provided [ImageToMapMultirayParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#imageToMapMultiray)
     */
    imageToMapMultiray(
      url: string,
      parameters: ImageToMapMultirayParameters | ImageToMapMultirayParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Converts a given geometry from a map space to an image space using the provided [MapToImageParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#mapToImage)
     */
    mapToImage(
      url: string,
      parameters: MapToImageParameters | MapToImageParametersProperties,
      requestOptions?: any,
    ): Promise<Geometry>;
    /**
     * Calculates the area and perimeter of a given geometry on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaAndPerimeter)
     */
    measureAreaAndPerimeter(
      url: string,
      parameters: ImageAreaParameters | ImageAreaParametersProperties,
      requestOptions?: any,
    ): Promise<ImageAreaResult>;
    /**
     * Measures the area and the perimeter of a polygon in an image space on a selected raster when the following conditions are met:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaFromImage)
     */
    measureAreaFromImage(
      url: string,
      parameters: MeasureFromImageParameters | MeasureFromImageParametersProperties,
      requestOptions?: any,
    ): Promise<MeasureAreaFromImageResult>;
    /**
     * Calculates the distance and angle between two points on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureDistanceAndAngle)
     */
    measureDistanceAndAngle(
      url: string,
      parameters: ImageDistanceParameters | ImageDistanceParametersProperties,
      requestOptions?: any,
    ): Promise<ImageDistanceResult>;
    /**
     * Calculates the height of an object between two points on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureHeight)
     */
    measureHeight(
      url: string,
      parameters: ImageHeightParameters | ImageHeightParametersProperties,
      requestOptions?: any,
    ): Promise<ImageHeightResult>;
    /**
     * Measures the length of a polyline in an image space on a selected raster when the following conditions are met:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureLengthFromImage)
     */
    measureLengthFromImage(
      url: string,
      parameters: MeasureFromImageParameters | MeasureFromImageParametersProperties,
      requestOptions?: any,
    ): Promise<MeasureLengthFromImageResult>;
    /**
     * Returns the location for a given point or centroid of a given area on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measurePointOrCentroid)
     */
    measurePointOrCentroid(
      parameters: ImagePointParameters | ImagePointParametersProperties,
      requestOptions?: any,
    ): Promise<ImagePointResult>;
    /**
     * Returns the boundary of an image for the provided [ImageBoundaryParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryBoundary)
     */
    queryBoundary(
      url: string,
      parameters: ImageBoundaryParameters | ImageBoundaryParametersProperties,
      requestOptions?: any,
    ): Promise<ImageBoundaryResult>;
    /**
     * Returns GPS information for the provided [ImageGPSInfoParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryGPSInfo)
     */
    queryGPSInfo(
      url: string,
      parameters: ImageGPSInfoParameters | ImageGPSInfoParametersProperties,
      requestOptions?: any,
    ): Promise<ImageGPSInfoResult>;
  }

  export const imageService: imageService;

  export interface DataModel extends Accessor, JSONSupport {}

  export class DataModel {
    /**
     * The data model defines the [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) in a knowledge graph service  as well as some additional settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html)
     */

    constructor(properties?: DataModelProperties);

    /**
     * Indicates if the data in the graph is managed by ArcGIS Knowledge.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#arcgisManaged)
     */
    readonly arcgisManaged: boolean;
    /**
     * A list of the [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) contained in the knowledge graph including their associated properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#entityTypes)
     */
    readonly entityTypes: EntityType[];
    /**
     * Information about the global ID for the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    readonly identifierInfo: DataModelIdentifierInfo;
    /**
     * A list of the meta entity types in the knowledge graph and their associated properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#metaEntityTypes)
     */
    readonly metaEntityTypes: EntityType[];
    /**
     * A list of the [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) in the knowledge graph including their associated properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#relationshipTypes)
     */
    readonly relationshipTypes: RelationshipType[];
    /**
     * List of the search indexes in the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#searchIndexes)
     */
    readonly searchIndexes: SearchIndex[];
    /**
     * Specifies [spatial reference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) information for the knowledge graph.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * Indicates whether users can make changes to the data model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#strict)
     */
    readonly strict: boolean;
    /**
     * The date the data model was last updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#timestamp)
     */
    readonly timestamp: Date;

    static fromJSON(json: any): DataModel;
  }

  interface DataModelProperties {}

  export interface DataModelIdentifierInfo {
    /**
     * Information on the type and format of the universally unique identifier
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    identifierMappingInfo: DataModelIdentifierInfoIdentifierMappingInfo;
    /**
     * Information on how the unique identifier is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    identifierGenerationInfo: DataModelIdentifierInfoIdentifierGenerationInfo;
  }

  export interface DataModelIdentifierInfoIdentifierGenerationInfo {
    /**
     * Information on the format of the unique identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    uuidMethodHint: "esriMethodHintUNSPECIFIED" | "esriUUIDESRI" | "esriUUIDRFC4122";
  }

  export interface DataModelIdentifierInfoIdentifierMappingInfo {
    /**
     * Indicates the type of the ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    identifierInfoType:
      | "esriIdentifierInfoTypeUNSPECIFIED"
      | "esriIdentifierInfoTypeUniformProperty"
      | "esriIdentifierInfoTypeDatabaseNative";
    /**
     * Properties of database native identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    databaseNativeIdentifier: any;
    /**
     * Properties of the user defined unique identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    uniformPropertyIdentifier: DataModelIdentifierInfoIdentifierMappingInfoUniformPropertyIdentifier;
  }

  export interface DataModelIdentifierInfoIdentifierMappingInfoUniformPropertyIdentifier {
    /**
     * The name of the unique identifier property for all records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html#identifierInfo)
     */
    identifierPropertyName: string;
  }

  export class Entity extends GraphNamedObject {
    /**
     * An entity is a specific instance of an [EntityType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) that can exist in the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html)
     */

    constructor(properties?: EntityProperties);

    static fromJSON(json: any): Entity;
  }

  interface EntityProperties extends GraphNamedObjectProperties {}

  export class EntityType extends GraphObjectType {
    /**
     * An entity type defines a homogeneous collection of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) with a common set of properties and a spatial feature type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html)
     */

    constructor(properties?: EntityTypeProperties);

    static fromJSON(json: any): EntityType;
  }

  interface EntityTypeProperties extends GraphObjectTypeProperties {}

  export interface FieldIndex extends Accessor, JSONSupport {}

  export class FieldIndex {
    /**
     * Defines an index on fields associated with an [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html)
     */

    constructor(properties?: FieldIndexProperties);

    /**
     * Specifies if the field is indexed in ascending order.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#ascending)
     */
    ascending: boolean;
    /**
     * Description of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#description)
     */
    description: string;
    /**
     * The ordered field names included in this index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#fieldNames)
     */
    fieldNames: string[];
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#name)
     */
    name: string;
    /**
     * Specifies if the values in the field are unique (no duplicate values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#unique)
     */
    unique: boolean;

    static fromJSON(json: any): FieldIndex;
  }

  interface FieldIndexProperties {
    /**
     * Specifies if the field is indexed in ascending order.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#ascending)
     */
    ascending?: boolean;
    /**
     * Description of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#description)
     */
    description?: string;
    /**
     * The ordered field names included in this index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#fieldNames)
     */
    fieldNames?: string[];
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#name)
     */
    name?: string;
    /**
     * Specifies if the values in the field are unique (no duplicate values).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html#unique)
     */
    unique?: boolean;
  }

  export class GraphApplyEdits extends Accessor {
    /**
     * This class defines [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to add, delete, and update in a knowledge graph service's [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html)
     */

    constructor(properties?: GraphApplyEditsProperties);

    /**
     * A list of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) to add to the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityAdds)
     */
    entityAdds: Entity[];
    /**
     * A list of objects containing an [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and the ids of the [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) of that type to delete.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityDeletes)
     */
    entityDeletes: GraphNamedObjectDeletes[];
    /**
     * A list of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) with the modified properties to update in the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityUpdates)
     */
    entityUpdates: Entity[];
    /**
     * Additional options to set an [input quantization](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html) for any geometries being added to the graph and to automatically delete all relationships associated with a deleted entity.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#options)
     */
    options: GraphApplyEditsOptions;
    /**
     * A list of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to add to the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipAdds)
     */
    relationshipAdds: knowledgeGraphRelationship[];
    /**
     * A list of objects containing a [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html), and the ids of the [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) of that type to delete.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipDeletes)
     */
    relationshipDeletes: GraphNamedObjectDeletes[];
    /**
     * A list of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) with modified properties to update in the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipUpdates)
     */
    relationshipUpdates: knowledgeGraphRelationship[];
  }

  interface GraphApplyEditsProperties {
    /**
     * A list of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) to add to the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityAdds)
     */
    entityAdds?: EntityProperties[];
    /**
     * A list of objects containing an [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and the ids of the [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) of that type to delete.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityDeletes)
     */
    entityDeletes?: GraphNamedObjectDeletes[];
    /**
     * A list of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) with the modified properties to update in the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#entityUpdates)
     */
    entityUpdates?: EntityProperties[];
    /**
     * Additional options to set an [input quantization](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html) for any geometries being added to the graph and to automatically delete all relationships associated with a deleted entity.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#options)
     */
    options?: GraphApplyEditsOptions;
    /**
     * A list of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) to add to the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipAdds)
     */
    relationshipAdds?: knowledgeGraphRelationshipProperties[];
    /**
     * A list of objects containing a [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html), and the ids of the [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) of that type to delete.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipDeletes)
     */
    relationshipDeletes?: GraphNamedObjectDeletes[];
    /**
     * A list of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) with modified properties to update in the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#relationshipUpdates)
     */
    relationshipUpdates?: knowledgeGraphRelationshipProperties[];
  }

  export interface GraphApplyEditsOptions {
    /**
     * Custom [quantization parameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html) for input geometry that compresses geometry for transfer to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#options)
     */
    inputQuantizationParameters?: InputQuantizationParameters;
    /**
     * If `true`, deleting an entity will automatically delete all relationships connected to that entity.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#options)
     */
    cascadeDelete?: boolean;
  }

  /**
   * GraphNamedObjectDeletes represents the list of [GraphNamedObjects](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) ([entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html)) of a specific type that will be deleted from a knowledge graph.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#GraphNamedObjectDeletes)
   */
  export interface GraphNamedObjectDeletes {
    /**
     * The name of the [EntityType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) that the entities belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#GraphNamedObjectDeletes)
     */
    typeName: string;
    /**
     * A list of the ids of the specified type to delete.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEdits.html#GraphNamedObjectDeletes)
     */
    ids: string[];
  }

  export class GraphApplyEditsResult extends Accessor {
    /**
     * The result of an [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits) performed on a knowledge graph service's [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html)
     */

    constructor(properties?: GraphApplyEditsResultProperties);

    /**
     * Returns a list of objects for each [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) that added, updated or deleted records by [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResults)
     */
    editResults: EditResultsObject[];
    /**
     * The error message explaining information about why [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits) failed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#error)
     */
    error: GraphApplyEditsResultError;
    /**
     * If `true` there was an error processing [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#hasError)
     */
    hasError: any;
  }

  interface GraphApplyEditsResultProperties {
    /**
     * Returns a list of objects for each [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) that added, updated or deleted records by [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResults)
     */
    editResults?: EditResultsObject[];
    /**
     * The error message explaining information about why [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits) failed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#error)
     */
    error?: GraphApplyEditsResultError;
    /**
     * If `true` there was an error processing [executeApplyEdits()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#hasError)
     */
    hasError?: any;
  }

  /**
   * editResultsObject returns a list of all [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) of each [type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html) that was added, updated or deleted from the [KnowledgeGraph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) as well as any errors that occurred during the operation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResultsObject)
   */
  export interface EditResultsObject {
    /**
     * The name of the [EntityType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [RelationshipType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) that had changed items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResultsObject)
     */
    typeName: string;
    /**
     * A list of objects containing the id and error information for every added entity or relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResultsObject)
     */
    adds: NamedObjectEditResults[];
    /**
     * A list of objects containing the id and error information for every updated entity or relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResultsObject)
     */
    updates: NamedObjectEditResults[];
    /**
     * A list of objects containing the id and error information for every deleted entity or relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#editResultsObject)
     */
    deletes: NamedObjectEditResults[];
  }

  export interface GraphApplyEditsResultError {
    /**
     * Error code returned from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#error)
     */
    errorCode: any;
    /**
     * Description of the error.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#error)
     */
    errorMessage: string;
  }

  /**
   * NamedObjectEditResults contains the `id` of the [GraphNamedObject](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html) ([Entity](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [Relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html)) that was added, updated or deleted from the [KnowledgeGraph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) as well as any errors that occurred during the operation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#NamedObjectEditResults)
   */
  export interface NamedObjectEditResults {
    /**
     * The id of the [Entity](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [Relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) that was added, updated or deleted from the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#NamedObjectEditResults)
     */
    id: string;
    /**
     * Indicates any errors caused during the operation applied to this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#NamedObjectEditResults)
     */
    error?: NamedObjectEditResultsError;
  }

  export interface NamedObjectEditResultsError {
    /**
     * Error code returned from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#NamedObjectEditResults)
     */
    errorCode: any;
    /**
     * Description of the error.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphApplyEditsResult.html#NamedObjectEditResults)
     */
    errorMessage: string;
  }

  export class GraphNamedObject extends GraphObject {
    /**
     * The parent class for an [Entity](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [Relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) that is defined in the graph schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html)
     */

    constructor(properties?: GraphNamedObjectProperties);

    /**
     * The unique ID of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html#id)
     */
    id: string;
    /**
     * Specifies the name for all similar types of objects ([entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html)) defined in the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html#typeName)
     */
    typeName: string;

    static fromJSON(json: any): GraphNamedObject;
  }

  interface GraphNamedObjectProperties extends GraphObjectProperties {
    /**
     * The unique ID of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html#id)
     */
    id?: string;
    /**
     * Specifies the name for all similar types of objects ([entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) or [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html)) defined in the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphNamedObject.html#typeName)
     */
    typeName?: string;
  }

  export interface GraphObject extends Accessor, JSONSupport {}

  export class GraphObject {
    /**
     * This is the parent class of all objects that can be represented on a graph structure or graph query - [Entity](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html), [Relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html), [Path](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Path.html), and anonymous object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObject.html)
     */

    constructor(properties?: GraphObjectProperties);

    /**
     * The properties of the graph object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObject.html#properties)
     */
    properties: any;

    static fromJSON(json: any): GraphObject;
  }

  interface GraphObjectProperties {
    /**
     * The properties of the graph object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObject.html#properties)
     */
    properties?: any;
  }

  export interface GraphObjectType extends Accessor, JSONSupport {}

  export class GraphObjectType {
    /**
     * Parent object for [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) defined in the graph schema.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html)
     */

    constructor(properties?: GraphObjectTypeProperties);

    /**
     * The display name of the graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#alias)
     */
    alias: string;
    /**
     * Specifies [index fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html) for a graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#fieldIndexes)
     */
    fieldIndexes: FieldIndex[];
    /**
     * The name of the graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#name)
     */
    name: string;
    /**
     * Specifies the [properties](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html) of a graph object type such as an [EntityType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [RelationshipType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#properties)
     */
    properties: GraphProperty[];
    /**
     * The object type's role in the knowledge graph.
     *
     * @default "Regular"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#role)
     */
    role: string;

    static fromJSON(json: any): GraphObjectType;
  }

  interface GraphObjectTypeProperties {
    /**
     * The display name of the graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#alias)
     */
    alias?: string;
    /**
     * Specifies [index fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-FieldIndex.html) for a graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#fieldIndexes)
     */
    fieldIndexes?: FieldIndexProperties[];
    /**
     * The name of the graph object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#name)
     */
    name?: string;
    /**
     * Specifies the [properties](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html) of a graph object type such as an [EntityType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) or [RelationshipType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#properties)
     */
    properties?: GraphPropertyProperties[];
    /**
     * The object type's role in the knowledge graph.
     *
     * @default "Regular"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphObjectType.html#role)
     */
    role?: string;
  }

  export interface GraphProperty extends Accessor, JSONSupport {}

  export class GraphProperty {
    /**
     * Specifies the properties of for [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) and [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) defined in the graph schema by the [data model](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html)
     */

    constructor(properties?: GraphPropertyProperties);

    /**
     * The display name for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#alias)
     */
    alias: string;
    /**
     * Specifies a default value for the object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#defaultValue)
     */
    defaultValue: any;
    /**
     * Specifies whether the property is visible by default.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#defaultVisibility)
     */
    defaultVisibility: boolean;
    /**
     * Specifies whether the property is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#editable)
     */
    editable: boolean;
    /**
     * Specifies the field type for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#fieldType)
     */
    fieldType:
      | "esriFieldTypeSmallInteger"
      | "esriFieldTypeInteger"
      | "esriFieldTypeSingle"
      | "esriFieldTypeDouble"
      | "esriFieldTypeString"
      | "esriFieldTypeDate"
      | "esriFieldTypeOID"
      | "esriFieldTypeGeometry"
      | "esriFieldTypeBlob"
      | "esriFieldTypeRaster"
      | "esriFieldTypeGUID"
      | "esriFieldTypeGlobalID"
      | "esriFieldTypeXML"
      | "esriFieldTypeBigInteger"
      | "esriFieldTypeTimestampOffset"
      | "esriFieldTypeTimeOnly"
      | "esriFieldTypeDateOnly";
    /**
     * Specifies the geometry type for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#geometryType)
     */
    geometryType: string;
    /**
     * Specifies whether the property has an m-value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#hasM)
     */
    hasM: boolean;
    /**
     * Specifies whether property has a z-value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#hasZ)
     */
    hasZ: boolean;
    /**
     * The name of the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#name)
     */
    name: string;
    /**
     * Specifies whether the property can be `null`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#nullable)
     */
    nullable: boolean;
    /**
     * Specifies whether the property is required.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#required)
     */
    required: boolean;
    /**
     * Specifies the role of the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#role)
     */
    role:
      | "esriGraphPropertyUNSPECIFIED"
      | "esriGraphPropertyRegular"
      | "esriGraphPropertyDocumentName"
      | "esriGraphPropertyDocumentTitle"
      | "esriGraphPropertyDocumentUrl"
      | "esriGraphPropertyDocumentText"
      | "esriGraphPropertyDocumentKeywords"
      | "esriGraphPropertyDocumentContentType"
      | "esriGraphPropertyDocumentMetadata"
      | "esriGraphPropertyDocumentFileExtension"
      | "esriGraphPropertyProvenanceInstanceId"
      | "esriGraphPropertyProvenanceSourceType"
      | "esriGraphPropertyProvenanceSourceName"
      | "esriGraphPropertyProvenanceSource"
      | "esriGraphPropertyProvenanceComment"
      | "esriGraphPropertyProvenanceTypeName"
      | "esriGraphPropertyProvenancePropertyName";
    /**
     * Specifies if the property is system maintained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#systemMaintained)
     */
    systemMaintained: boolean;

    static fromJSON(json: any): GraphProperty;
  }

  interface GraphPropertyProperties {
    /**
     * The display name for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#alias)
     */
    alias?: string;
    /**
     * Specifies a default value for the object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#defaultValue)
     */
    defaultValue?: any;
    /**
     * Specifies whether the property is visible by default.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#defaultVisibility)
     */
    defaultVisibility?: boolean;
    /**
     * Specifies whether the property is editable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#editable)
     */
    editable?: boolean;
    /**
     * Specifies the field type for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#fieldType)
     */
    fieldType?:
      | "esriFieldTypeSmallInteger"
      | "esriFieldTypeInteger"
      | "esriFieldTypeSingle"
      | "esriFieldTypeDouble"
      | "esriFieldTypeString"
      | "esriFieldTypeDate"
      | "esriFieldTypeOID"
      | "esriFieldTypeGeometry"
      | "esriFieldTypeBlob"
      | "esriFieldTypeRaster"
      | "esriFieldTypeGUID"
      | "esriFieldTypeGlobalID"
      | "esriFieldTypeXML"
      | "esriFieldTypeBigInteger"
      | "esriFieldTypeTimestampOffset"
      | "esriFieldTypeTimeOnly"
      | "esriFieldTypeDateOnly";
    /**
     * Specifies the geometry type for the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#geometryType)
     */
    geometryType?: string;
    /**
     * Specifies whether the property has an m-value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#hasM)
     */
    hasM?: boolean;
    /**
     * Specifies whether property has a z-value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#hasZ)
     */
    hasZ?: boolean;
    /**
     * The name of the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#name)
     */
    name?: string;
    /**
     * Specifies whether the property can be `null`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#nullable)
     */
    nullable?: boolean;
    /**
     * Specifies whether the property is required.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#required)
     */
    required?: boolean;
    /**
     * Specifies the role of the property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#role)
     */
    role?:
      | "esriGraphPropertyUNSPECIFIED"
      | "esriGraphPropertyRegular"
      | "esriGraphPropertyDocumentName"
      | "esriGraphPropertyDocumentTitle"
      | "esriGraphPropertyDocumentUrl"
      | "esriGraphPropertyDocumentText"
      | "esriGraphPropertyDocumentKeywords"
      | "esriGraphPropertyDocumentContentType"
      | "esriGraphPropertyDocumentMetadata"
      | "esriGraphPropertyDocumentFileExtension"
      | "esriGraphPropertyProvenanceInstanceId"
      | "esriGraphPropertyProvenanceSourceType"
      | "esriGraphPropertyProvenanceSourceName"
      | "esriGraphPropertyProvenanceSource"
      | "esriGraphPropertyProvenanceComment"
      | "esriGraphPropertyProvenanceTypeName"
      | "esriGraphPropertyProvenancePropertyName";
    /**
     * Specifies if the property is system maintained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphProperty.html#systemMaintained)
     */
    systemMaintained?: boolean;
  }

  export class GraphQuery extends Accessor {
    /**
     * Defines the query operation performed on a knowledge graph service's [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQuery.html)
     */

    constructor(properties?: GraphQueryProperties);

    /**
     * The Esri implementation of [openCypher](https://opencypher.org/) query to be executed against the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQuery.html#openCypherQuery)
     */
    openCypherQuery: string;
  }

  interface GraphQueryProperties {
    /**
     * The Esri implementation of [openCypher](https://opencypher.org/) query to be executed against the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQuery.html#openCypherQuery)
     */
    openCypherQuery?: string;
  }

  export class GraphQueryResult extends Accessor {
    /**
     * The results of a [executeQuery()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQuery) or [executeSearch()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearch) on a knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryResult.html)
     */

    constructor(properties?: GraphQueryResultProperties);

    /**
     * An array of the result types that are returned from a search or query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryResult.html#resultRows)
     */
    resultRows: GraphAnyValue[][];
  }

  interface GraphQueryResultProperties {
    /**
     * An array of the result types that are returned from a search or query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryResult.html#resultRows)
     */
    resultRows?: (
      | PathProperties
      | GraphObjectProperties
      | GeometryProperties
      | DateProperties
      | string
      | number
      | GraphAnyValue[]
    )[][];
  }

  export type GraphAnyValue = Path | GraphObject | Geometry | Date | string | number | GraphAnyValue[];

  export class GraphQueryStreaming extends GraphQuery {
    /**
     * Defines a streaming query operation performed on a [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html)
     */

    constructor(properties?: GraphQueryStreamingProperties);

    /**
     * Custom quantization parameters for input geometry that compresses geometry for transfer to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#bindGeometryQuantizationParameters)
     */
    bindGeometryQuantizationParameters: InputQuantizationParameters;
    /**
     * Specifies a set of parameters containing data to be included in the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#bindParameters)
     */
    bindParameters: HashMap<
      HashMap<any> | number | bigint | string | Date | Geometry | (HashMap<any> | number | string | Date | Geometry)[]
    >;
    /**
     * Used to project the geometry onto a virtual grid, likely representing pixels on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#outputQuantizationParameters)
     */
    outputQuantizationParameters: OutputQuantizationParameters;
    /**
     * Custom output spatial reference parameters for output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#outputSpatialReference)
     */
    outputSpatialReference: SpatialReference;
    /**
     * Enables provenance to be used and returned in the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#provenanceBehavior)
     */
    provenanceBehavior: "exclude" | "include";
  }

  interface GraphQueryStreamingProperties extends GraphQueryProperties {
    /**
     * Custom quantization parameters for input geometry that compresses geometry for transfer to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#bindGeometryQuantizationParameters)
     */
    bindGeometryQuantizationParameters?: InputQuantizationParametersProperties;
    /**
     * Specifies a set of parameters containing data to be included in the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#bindParameters)
     */
    bindParameters?: HashMap<
      | HashMap<any>
      | number
      | bigint
      | string
      | DateProperties
      | GeometryProperties
      | (HashMap<any> | number | string | DateProperties | GeometryProperties)[]
    >;
    /**
     * Used to project the geometry onto a virtual grid, likely representing pixels on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#outputQuantizationParameters)
     */
    outputQuantizationParameters?: OutputQuantizationParametersProperties;
    /**
     * Custom output spatial reference parameters for output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#outputSpatialReference)
     */
    outputSpatialReference?: SpatialReferenceProperties;
    /**
     * Enables provenance to be used and returned in the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreaming.html#provenanceBehavior)
     */
    provenanceBehavior?: "exclude" | "include";
  }

  export class GraphQueryStreamingResult extends Accessor {
    /**
     * The result of a [executeSearchStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearchStreaming) or [executeQueryStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQueryStreaming) on a knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreamingResult.html)
     */

    constructor(properties?: GraphQueryStreamingResultProperties);

    /**
     * Readable stream of objects matching [executeSearchStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearchStreaming) or [executeQueryStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQueryStreaming) criteria.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreamingResult.html#resultRowsStream)
     */
    resultRowsStream: any;
  }

  interface GraphQueryStreamingResultProperties {
    /**
     * Readable stream of objects matching [executeSearchStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearchStreaming) or [executeQueryStreaming()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQueryStreaming) criteria.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphQueryStreamingResult.html#resultRowsStream)
     */
    resultRowsStream?: any;
  }

  export class GraphSearch extends Accessor {
    /**
     * The search operation is performed on a [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearch.html)
     */

    constructor(properties?: GraphSearchProperties);

    /**
     * The text to search for in the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearch.html#searchQuery)
     */
    searchQuery: string;
    /**
     * Specifies whether to search [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html), [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html), or both.
     *
     * @default "both"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearch.html#typeCategoryFilter)
     */
    typeCategoryFilter: "entity" | "relationship" | "both";
  }

  interface GraphSearchProperties {
    /**
     * The text to search for in the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearch.html#searchQuery)
     */
    searchQuery?: string;
    /**
     * Specifies whether to search [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html), [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html), or both.
     *
     * @default "both"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearch.html#typeCategoryFilter)
     */
    typeCategoryFilter?: "entity" | "relationship" | "both";
  }

  export class GraphSearchStreaming extends GraphSearch {
    /**
     * The search operation is performed on a knowledge graph service's [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html)
     */

    constructor(properties?: GraphSearchStreamingProperties);

    /**
     * Specifies list of IDs to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#idsFilter)
     */
    idsFilter: string[];
    /**
     * Specifies list of names of [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html#name) or [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#name) to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#namedTypesFilter)
     */
    namedTypesFilter: string[];
    /**
     * The maximum number of results returned from the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#num)
     */
    num: number;
    /**
     * If `true`, returns the IDs of objects that match the search, the names of the properties that matched the search term, scores and highlights of the result set.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#returnSearchContext)
     */
    returnSearchContext: boolean;
    /**
     * The index of the first result to return.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#start)
     */
    start: number;
  }

  interface GraphSearchStreamingProperties extends GraphSearchProperties {
    /**
     * Specifies list of IDs to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#idsFilter)
     */
    idsFilter?: string[];
    /**
     * Specifies list of names of [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html#name) or [relationship types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#name) to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#namedTypesFilter)
     */
    namedTypesFilter?: string[];
    /**
     * The maximum number of results returned from the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#num)
     */
    num?: number;
    /**
     * If `true`, returns the IDs of objects that match the search, the names of the properties that matched the search term, scores and highlights of the result set.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#returnSearchContext)
     */
    returnSearchContext?: boolean;
    /**
     * The index of the first result to return.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-GraphSearchStreaming.html#start)
     */
    start?: number;
  }

  export class InputQuantizationParameters extends Accessor {
    /**
     * Custom quantization parameters for input geometry that compresses geometry for transfer to the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html)
     */

    constructor(properties?: InputQuantizationParametersProperties);

    /**
     * Origin of M-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#mFalseOrigin)
     */
    mFalseOrigin: number;
    /**
     * Number of significant digits for M-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#mResolution)
     */
    mResolution: number;
    /**
     * False origin of x values of the quantization grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#xFalseOrigin)
     */
    xFalseOrigin: number;
    /**
     * Number of significant digits for the x and y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#xyResolution)
     */
    xyResolution: number;
    /**
     * False origin for y-values of the quantization grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#yFalseOrigin)
     */
    yFalseOrigin: number;
    /**
     * The false origin of the Z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#zFalseOrigin)
     */
    zFalseOrigin: number;
    /**
     * Number of significant digits of the Z-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#zResolution)
     */
    zResolution: number;
  }

  interface InputQuantizationParametersProperties {
    /**
     * Origin of M-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#mFalseOrigin)
     */
    mFalseOrigin?: number;
    /**
     * Number of significant digits for M-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#mResolution)
     */
    mResolution?: number;
    /**
     * False origin of x values of the quantization grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#xFalseOrigin)
     */
    xFalseOrigin?: number;
    /**
     * Number of significant digits for the x and y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#xyResolution)
     */
    xyResolution?: number;
    /**
     * False origin for y-values of the quantization grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#yFalseOrigin)
     */
    yFalseOrigin?: number;
    /**
     * The false origin of the Z-values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#zFalseOrigin)
     */
    zFalseOrigin?: number;
    /**
     * Number of significant digits of the Z-Values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-InputQuantizationParameters.html#zResolution)
     */
    zResolution?: number;
  }

  export interface KnowledgeGraph extends Accessor, JSONSupport {}

  export class KnowledgeGraph {
    /**
     * The knowledge graph associated with the [knowledgeGraphService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html)
     */

    constructor(properties?: KnowledgeGraphProperties);

    /**
     * The [data model](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html) of the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html#dataModel)
     */
    readonly dataModel: DataModel;
    /**
     * The [service definition](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html) of the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html#serviceDefinition)
     */
    readonly serviceDefinition: ServiceDefinition;
    /**
     * The url to a hosted knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html#url)
     */
    url: string;

    static fromJSON(json: any): KnowledgeGraph;
  }

  interface KnowledgeGraphProperties {
    /**
     * The url to a hosted knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html#url)
     */
    url?: string;
  }

  export class OutputQuantizationParameters extends Accessor {
    /**
     * Used to project the geometry onto a virtual grid, likely representing pixels on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html)
     */

    constructor(properties?: OutputQuantizationParametersProperties);

    /**
     * An extent defining the quantization grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    extent: OutputQuantizationParametersExtent;
    /**
     * Geometry coordinates are optimized for viewing and displaying of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#quantizeMode)
     */
    quantizeMode: "view" | "edit";
    /**
     * The size of one pixel in the units of the [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#tolerance)
     */
    tolerance: number;
  }

  interface OutputQuantizationParametersProperties {
    /**
     * An extent defining the quantization grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    extent?: OutputQuantizationParametersExtent;
    /**
     * Geometry coordinates are optimized for viewing and displaying of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#quantizeMode)
     */
    quantizeMode?: "view" | "edit";
    /**
     * The size of one pixel in the units of the [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#tolerance)
     */
    tolerance?: number;
  }

  export interface OutputQuantizationParametersExtent {
    /**
     * Maximum horizontal grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    xmax: number;
    /**
     * Minimum horizontal grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    xmin: number;
    /**
     * Maximum vertical grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    ymax: number;
    /**
     * Minimum vertical grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-OutputQuantizationParameters.html#extent)
     */
    ymin: number;
  }

  export interface Path extends Accessor, JSONSupport {}

  export class Path {
    /**
     * An object containing all [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) required to traverse a graph from one entity to another.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Path.html)
     */

    constructor(properties?: PathProperties);

    /**
     * Array of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) where the first element is the starting entity of the path, the final element is the ending entity of the path, and intermediary elements are entities and relationships connecting the two.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Path.html#path)
     */
    path: GraphNamedObject[];

    static fromJSON(json: any): Path;
  }

  interface PathProperties {
    /**
     * Array of [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) where the first element is the starting entity of the path, the final element is the ending entity of the path, and intermediary elements are entities and relationships connecting the two.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Path.html#path)
     */
    path?: GraphNamedObjectProperties[];
  }

  export class knowledgeGraphRelationship extends GraphNamedObject {
    /**
     * A relationship is an instance of a [relationship type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html) that  defines an association between two [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html)
     */

    constructor(properties?: knowledgeGraphRelationshipProperties);

    /**
     * The ID of the destination entity of the relationship (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html#destinationId)
     */
    destinationId: string;
    /**
     * The ID of the origin entity of the relationship (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html#originId)
     */
    originId: string;

    static fromJSON(json: any): knowledgeGraphRelationship;
  }

  interface knowledgeGraphRelationshipProperties extends GraphNamedObjectProperties {
    /**
     * The ID of the destination entity of the relationship (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html#destinationId)
     */
    destinationId?: string;
    /**
     * The ID of the origin entity of the relationship (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html#originId)
     */
    originId?: string;
  }

  export class RelationshipType extends GraphObjectType {
    /**
     * A relationship type defines a homogenous collection of [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) that can exist between two [entity types](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html), with a common set of properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html)
     */

    constructor(properties?: RelationshipTypeProperties);

    /**
     * Specifies valid origin and destination [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) pairs for this relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#endPoints)
     */
    endPoints: RelationshipTypeEndPoints[];

    static fromJSON(json: any): RelationshipType;
  }

  interface RelationshipTypeProperties extends GraphObjectTypeProperties {
    /**
     * Specifies valid origin and destination [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html) pairs for this relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#endPoints)
     */
    endPoints?: RelationshipTypeEndPoints[];
  }

  export interface RelationshipTypeEndPoints {
    /**
     * The `name` of the origin [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#endPoints)
     */
    originEntityType: string;
    /**
     * The `name` of the destination [entity type](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-EntityType.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-RelationshipType.html#endPoints)
     */
    destinationEntityType: string;
  }

  export class SearchIndex extends Accessor {
    /**
     * Defines a search index for a [KnowledgeGraph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html)
     */

    constructor(properties?: SearchIndexProperties);

    /**
     * Additional analyzers for string processing during search such as support for languages.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#analyzers)
     */
    readonly analyzers: SearchIndexAnalyzers[];
    /**
     * The name of the search index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#name)
     */
    readonly name: string;
    /**
     * A map of the named types and their properties that are included in the search index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#searchProperties)
     */
    readonly searchProperties: globalThis.Map<string, SearchIndexSearchProperties>;
    /**
     * Specifies if the index supports searching entities, relationships or both.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#supportedCategory)
     */
    readonly supportedCategory: "esriTypeUNSPECIFIED" | "esriTypeEntity" | "esriTypeRelationship" | "esriTypeBoth";
  }

  interface SearchIndexProperties {}

  export interface SearchIndexAnalyzers {
    /**
     * The name of the analyzer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#analyzers)
     */
    name: string;
  }

  /**
   * Search properties.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#SearchProperties)
   */
  export interface SearchIndexSearchProperties {
    /**
     * The properties of the named type that are included in the search index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-SearchIndex.html#SearchProperties)
     */
    propertyNames: string[];
  }

  export interface ServiceDefinition extends Accessor, JSONSupport {}

  export class ServiceDefinition {
    /**
     * Outlines the service capabilities for a [knowledgeGraphService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html)
     */

    constructor(properties?: ServiceDefinitionProperties);

    /**
     * Indicates if geometry data in the graph can be modified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#allowGeometryUpdates)
     */
    readonly allowGeometryUpdates: boolean;
    /**
     * Specifies the operational capabilities of the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#capabilities)
     */
    readonly capabilities: string[];
    /**
     * Copyright information for the knowledge graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#copyrightText)
     */
    readonly copyrightText: string;
    /**
     * The version number of the ArcGIS Enterprise instance where the knowledge graph service is hosted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#currentVersion)
     */
    readonly currentVersion: number;
    /**
     * Indicates if data editing is not supported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#dataEditingNotSupported)
     */
    readonly dataEditingNotSupported: boolean;
    /**
     * Specifies the timezone for dateTime fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#dateFieldsTimeReference)
     */
    readonly dateFieldsTimeReference: ServiceDefinitionDateFieldsTimeReference;
    /**
     * The description of the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#description)
     */
    readonly description: string;
    /**
     * The maximum number of records returned by a [query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQuery) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#maxRecordCount)
     */
    readonly maxRecordCount: number;
    /**
     * Indicates if the knowledge graph schema (data model) can be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#schemaEditingNotSupported)
     */
    readonly schemaEditingNotSupported: boolean;
    /**
     * The maximum number of records returned by a [search](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearch) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#searchMaxRecordCount)
     */
    readonly searchMaxRecordCount: number;
    /**
     * specifies the details of the index, edit, search and  geometry capabilities of the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    readonly serviceCapabilities: ServiceDefinitionServiceCapabilities;
    /**
     * The ArcGIS Enterprise Portal item id of the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceItemId)
     */
    readonly serviceItemId: string;
    /**
     * Specifies the [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) information for the knowledge graph service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#spatialReference)
     */
    readonly spatialReference: SpatialReference;
    /**
     * The query formats supported by the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#supportedQueryFormats)
     */
    readonly supportedQueryFormats: string[];
    /**
     * Indicates if the service supports entity types with a document role.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#supportsDocuments)
     */
    readonly supportsDocuments: boolean;
    /**
     * Indicates if the service supports provenance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#supportsProvenance)
     */
    readonly supportsProvenance: boolean;
    /**
     * Indicates if the services supports search operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#supportsSearch)
     */
    readonly supportsSearch: boolean;
    /**
     * The units for the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#units)
     */
    readonly units: string;

    static fromJSON(json: any): ServiceDefinition;
  }

  interface ServiceDefinitionProperties {}

  export interface ServiceDefinitionDateFieldsTimeReference {
    /**
     * Indicates if datetime fields respects daylight savings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#dateFieldsTimeReference)
     */
    respectsDaylightSaving: boolean;
    /**
     * the timezone for all datetime fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#dateFieldsTimeReference)
     */
    timeZone: string;
  }

  export interface ServiceDefinitionServiceCapabilities {
    /**
     * Information on the supported capabilities of indexes in the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    indexCapabilities: ServiceDefinitionServiceCapabilitiesIndexCapabilities;
    /**
     * Information on the supported capabilities of the the [executeApplyEdits](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    applyEditsCapabilities: ServiceDefinitionServiceCapabilitiesApplyEditsCapabilities;
    /**
     * Information on the supported capabilities of [search](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearch) operations in the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    searchCapabilities: ServiceDefinitionServiceCapabilitiesSearchCapabilities;
    /**
     * Information on the supported capabilities of geometries in the knowledge graph service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    geometryCapabilities: ServiceDefinitionServiceCapabilitiesGeometryCapabilities;
  }

  export interface ServiceDefinitionServiceCapabilitiesApplyEditsCapabilities {
    /**
     * If `true`, when an apply edits operation fails, the data will revert to it's state before the change.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    graphDefaultRollbackOnFailure: boolean;
  }

  export interface ServiceDefinitionServiceCapabilitiesGeometryCapabilities {
    /**
     * The maximum bounding rectangle size along the x-axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    geometryMaxBoundingRectangleSizeX: number;
    /**
     * The maximum bounding rectangle size along the y-axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    geometryMaxBoundingRectangleSizeY: number;
    /**
     * Specifies if the knowledge graph service supports m-values on geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportsMValues: boolean;
    /**
     * Specifies if the knowledge graph service supports z-values on geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportsZValues: boolean;
    /**
     * Specifies what geometry types are supported by the graph such as `Point`, `Multipoint`, `Polyline`, `Polygon`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportedGeometryTypes: string[];
  }

  export interface ServiceDefinitionServiceCapabilitiesIndexCapabilities {
    /**
     * Specifies if the service supports indexes in descending order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportsDescendingIndex: boolean;
    /**
     * Specifies if the system supports creating unique constraints for relationship types
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportsUniqueRelationshipConstraint: boolean;
    /**
     * Specifies if the service supports indexes on relationships.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    supportsRelationshipIndex: boolean;
  }

  export interface ServiceDefinitionServiceCapabilitiesSearchCapabilities {
    /**
     * The search category filters supported by this service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    searchTypeFilterCapabilities: ("esriTypeEntity" | "esriTypeRelationship" | "esriTypeBoth")[];
    /**
     * Specifies if search strings can contain a leading wildcard operator (`*`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-ServiceDefinition.html#serviceCapabilities)
     */
    allowLeadingWildcardQueries: boolean;
  }

  /**
   * A knowledge graph service is associated with several resources.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html)
   */
  interface knowledgeGraphService {
    /**
     * Add, delete, or update [entities](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Entity.html) and [relationships](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-Relationship.html) in a [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeApplyEdits)
     */
    executeApplyEdits(
      graph: KnowledgeGraph,
      edits: GraphApplyEdits,
      requestOptions?: RequestOptions,
    ): Promise<GraphApplyEditsResult>;
    /**
     * Executes a query on [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource using the Esri implementation of [openCypher](https://opencypher.org/) and returns the results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQuery)
     */
    executeQuery(
      graph: KnowledgeGraph,
      queryArguments: GraphQuery,
      requestOptions?: RequestOptions,
    ): Promise<GraphQueryResult>;
    /**
     * Execute a streaming query on a [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeQueryStreaming)
     */
    executeQueryStreaming(
      graph: KnowledgeGraph,
      queryArguments: GraphQueryStreaming,
      requestOptions?: RequestOptions,
    ): Promise<GraphQueryStreamingResult>;
    /**
     * Search the knowledge graph using a full-text index that is automatically built on the property values in the graph and returns the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearch)
     */
    executeSearch(
      graph: KnowledgeGraph,
      searchArguments: GraphSearch,
      requestOptions?: RequestOptions,
    ): Promise<GraphQueryResult>;
    /**
     * Execute a streaming search on a [knowledge graph service's](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html) [graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html) resource.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#executeSearchStreaming)
     */
    executeSearchStreaming(
      graph: KnowledgeGraph,
      searchArguments: GraphSearchStreaming,
      requestOptions?: any,
    ): Promise<GraphQueryStreamingResult>;
    /**
     * Retrieves the knowledge graph service based on the URL provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#fetchKnowledgeGraph)
     */
    fetchKnowledgeGraph(url: string): Promise<KnowledgeGraph>;
    /**
     * Refreshes the [data model](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-DataModel.html) of the [knowledge graph](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraph-KnowledgeGraph.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-knowledgeGraphService.html#refreshDataModel)
     */
    refreshDataModel(graph: KnowledgeGraph): void;
  }

  export const knowledgeGraphService: knowledgeGraphService;

  /**
   * A convenience module for importing [locator](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html) functions when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html)
   */
  interface locator {
    /**
     * Find address candidates for multiple input addresses.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    addressesToLocations(
      url: string,
      params: locatorAddressesToLocationsParams,
      requestOptions?: any,
    ): Promise<AddressCandidate[]>;
    /**
     * Sends a request to the ArcGIS REST geocode resource to find candidates for a single address specified in the address parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    addressToLocations(
      url: string,
      params: locatorAddressToLocationsParams,
      requestOptions?: any,
    ): Promise<AddressCandidate[]>;
    /**
     * Locates an address based on a given point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#locationToAddress)
     */
    locationToAddress(
      url: string,
      params: locatorLocationToAddressParams,
      requestOptions?: any,
    ): Promise<AddressCandidate>;
    /**
     * Get character by character auto complete suggestions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#suggestLocations)
     */
    suggestLocations(
      url: string,
      params: locatorSuggestLocationsParams,
      requestOptions?: any,
    ): Promise<SuggestionResult[]>;
  }

  export const locator: locator;

  export interface locatorAddressesToLocationsParams {
    /**
     * The input addresses in the format supported by the geocode service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    addresses: any[];
    /**
     * Limits the results to only search in the country provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    countryCode?: string;
    /**
     * Limit result to one or more categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    categories?: string[];
    /**
     * Define the type of location, either `"street"` or `"rooftop"`, of the point returned from the [World Geocoding Service](https://developers.arcgis.com/rest/geocode/api-reference/geocoding-category-filtering.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    locationType?: string;
    /**
     * The spatial reference of the output geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressesToLocations)
     */
    outSpatialReference?: SpatialReference;
  }

  export interface locatorAddressToLocationsParams {
    /**
     * The address argument is data object that contains properties representing the various address fields accepted by the corresponding geocode service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    address: any;
    /**
     * Limit result to one or more categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    categories?: string[];
    /**
     * Limit result to a specific country.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    countryCode?: string;
    /**
     * Allows the results of single geocode transactions to be persisted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    forStorage?: boolean;
    /**
     * Used to weight returned results for a specified area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    location?: Point;
    /**
     * Define the type of location, either `"street"` or `"rooftop"`, of the point returned from the [World Geocoding Service](https://developers.arcgis.com/rest/geocode/api-reference/geocoding-category-filtering.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    locationType?: string;
    /**
     * A `suggestLocations` result ID (magicKey).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    magicKey?: string;
    /**
     * Maximum results to return from the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    maxLocations?: number;
    /**
     * The list of fields included in the returned result set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    outFields?: string[];
    /**
     * The spatial reference of the output geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    outSpatialReference?: SpatialReference;
    /**
     * Defines the extent within which the geocode server will search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations)
     */
    searchExtent?: Extent;
  }

  export interface locatorLocationToAddressParams {
    /**
     * The point at which to search for the closest address.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#locationToAddress)
     */
    location: Point;
    /**
     * Define the type of location, either `"street"` or `"rooftop"`, of the point returned from the [World Geocoding Service](https://developers.arcgis.com/rest/geocode/api-reference/geocoding-category-filtering.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#locationToAddress)
     */
    locationType?: string;
    /**
     * The spatial reference of the output geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#locationToAddress)
     */
    outSpatialReference?: SpatialReference;
  }

  export interface locatorSuggestLocationsParams {
    /**
     * A place or address type which can be used to filter suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#suggestLocations)
     */
    categories?: string[];
    /**
     * Defines a normalized location point that is used to sort geocoding candidates based upon their proximity to the given location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#suggestLocations)
     */
    location: Point;
    /**
     * The input text entered by a user which is used by the suggest operation to generate a list of possible matches.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#suggestLocations)
     */
    text: string;
  }

  /**
   * Describes the object representing the result of the [suggestLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#suggestLocations) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#SuggestionResult)
   */
  export interface SuggestionResult {
    /**
     * Indicates if the result is a Collection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#SuggestionResult)
     */
    isCollection: boolean;
    /**
     * ID used in combination with the `text` property to uniquely identify a suggestion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#SuggestionResult)
     */
    magicKey: string;
    /**
     * The string name of the suggested location to geocode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#SuggestionResult)
     */
    text: string;
  }

  /**
   * The utility network associations model connectivity, containment, and structure relations between assets.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-queryAssociations.html)
   */
  interface queryAssociations {
    /**
     * Returns all associations filtered by the [QueryAssociationsParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html) in a utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-queryAssociations.html#queryAssociations)
     */
    queryAssociations(
      url: string,
      props: QueryAssociationsParameters,
      requestOptions?: any,
    ): Promise<QueryAssociationsResult>;
  }

  export const queryAssociations: queryAssociations;

  export interface AggregatedGeometry extends Accessor, JSONSupport {}

  export class AggregatedGeometry {
    /**
     * A class that defines an aggregation of geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html)
     */

    constructor(properties?: AggregatedGeometryProperties);

    /**
     * A [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry representing a union of all polyline features encountered during the trace and specified by the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#line)
     */
    line: Polyline;
    /**
     * A [multi-point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) geometry representing a union of all point features encountered during the trace and specified by the output The aggregated geometries will only include geometries that belong to features with `assetgroups/assettypes` specified in the trace output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#multipoint)
     */
    multipoint: Multipoint;
    /**
     * A [polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometry representing a union of all polygon features encountered during the trace and specified by the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#polygon)
     */
    polygon: Polygon;

    static fromJSON(json: any): AggregatedGeometry;
  }

  interface AggregatedGeometryProperties {
    /**
     * A [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry representing a union of all polyline features encountered during the trace and specified by the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#line)
     */
    line?: PolylineProperties;
    /**
     * A [multi-point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) geometry representing a union of all point features encountered during the trace and specified by the output The aggregated geometries will only include geometries that belong to features with `assetgroups/assettypes` specified in the trace output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#multipoint)
     */
    multipoint?: MultipointProperties;
    /**
     * A [polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometry representing a union of all polygon features encountered during the trace and specified by the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html#polygon)
     */
    polygon?: PolygonProperties;
  }

  export interface Association extends Accessor, JSONSupport {}

  export class Association {
    /**
     * The utility network associations model connectivity, containment and structure relations between assets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html)
     */

    constructor(properties?: AssociationProperties);

    /**
     * The type of association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#associationType)
     */
    associationType: string;
    /**
     * Error code returned from the server for a failed associations query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#errorCode)
     */
    errorCode: number;
    /**
     * Message returned from the server for a failed associations query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#errorMessage)
     */
    errorMessage: string;
    /**
     * The from side network element of the association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#fromNetworkElement)
     */
    fromNetworkElement: NetworkElement;
    /**
     * The synthesized [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry created between the two network elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#geometry)
     */
    geometry: Polyline;
    /**
     * The globalId (UUID) of the association record, uniquely identifies this association row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#globalId)
     */
    globalId: string;
    /**
     * Specifies the content visibility on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#isContentVisible)
     */
    isContentVisible: boolean;
    /**
     * This double parameter of value of 0-1 indicates a percentage along the line of where the trace location is placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#percentAlong)
     */
    percentAlong: number;
    /**
     * Indicates the type of association a feature or object participates in, the role the network feature plays in the association relationship, and any properties that are set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#status)
     */
    status: number;
    /**
     * The to side network element of the association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#toNetworkElement)
     */
    toNetworkElement: NetworkElement;

    static fromJSON(json: any): Association;
  }

  interface AssociationProperties {
    /**
     * The type of association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#associationType)
     */
    associationType?: string;
    /**
     * Error code returned from the server for a failed associations query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#errorCode)
     */
    errorCode?: number;
    /**
     * Message returned from the server for a failed associations query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#errorMessage)
     */
    errorMessage?: string;
    /**
     * The from side network element of the association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#fromNetworkElement)
     */
    fromNetworkElement?: NetworkElementProperties;
    /**
     * The synthesized [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry created between the two network elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#geometry)
     */
    geometry?: PolylineProperties;
    /**
     * The globalId (UUID) of the association record, uniquely identifies this association row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#globalId)
     */
    globalId?: string;
    /**
     * Specifies the content visibility on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#isContentVisible)
     */
    isContentVisible?: boolean;
    /**
     * This double parameter of value of 0-1 indicates a percentage along the line of where the trace location is placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#percentAlong)
     */
    percentAlong?: number;
    /**
     * Indicates the type of association a feature or object participates in, the role the network feature plays in the association relationship, and any properties that are set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#status)
     */
    status?: number;
    /**
     * The to side network element of the association.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html#toNetworkElement)
     */
    toNetworkElement?: NetworkElementProperties;
  }

  export interface AssociationGeometriesResult extends Accessor, JSONSupport {}

  export class AssociationGeometriesResult {
    /**
     * This class defines the results of the function [synthesizeAssociationGeometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-synthesizeAssociationGeometries.html) which takes an extent and returns an [AggregatedGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html) of all [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) within the extent based on the defined parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AssociationGeometriesResult.html)
     */

    constructor(properties?: AssociationGeometriesResultProperties);

    /**
     * List of returned [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) filtered based on the input parameters in [SynthesizeAssociationGeometriesParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AssociationGeometriesResult.html#associations)
     */
    associations: Association[];
    /**
     * Indicates whether the [maxGeometryCount](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#maxGeometryCount) has exceeded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AssociationGeometriesResult.html#maxGeometryCountExceeded)
     */
    maxGeometryCountExceeded: boolean;

    static fromJSON(json: any): AssociationGeometriesResult;
  }

  interface AssociationGeometriesResultProperties {
    /**
     * List of returned [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) filtered based on the input parameters in [SynthesizeAssociationGeometriesParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AssociationGeometriesResult.html#associations)
     */
    associations?: AssociationProperties[];
    /**
     * Indicates whether the [maxGeometryCount](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#maxGeometryCount) has exceeded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AssociationGeometriesResult.html#maxGeometryCountExceeded)
     */
    maxGeometryCountExceeded?: boolean;
  }

  export interface FunctionResult extends Accessor, JSONSupport {}

  export class FunctionResult {
    /**
     * A trace can optionally return a aggregated function result if the trace configuration asked for it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-FunctionResult.html)
     */

    constructor(properties?: FunctionResultProperties);

    /**
     * The aggregate function type used in the supplied trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-FunctionResult.html#functionType)
     */
    readonly functionType: "add" | "subtract" | "average" | "count" | "min" | "max";
    /**
     * The name of the function result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-FunctionResult.html#networkAttributeName)
     */
    readonly networkAttributeName: string;
    /**
     * Actual result of the aggregate function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-FunctionResult.html#result)
     */
    readonly result: number;

    static fromJSON(json: any): FunctionResult;
  }

  interface FunctionResultProperties {}

  export interface NetworkElement extends Accessor, JSONSupport {}

  export class NetworkElement {
    /**
     * The network element is a representation of how the network topology defines its graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html)
     */

    constructor(properties?: NetworkElementProperties);

    /**
     * The asset group code that this network element represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#assetGroupCode)
     */
    assetGroupCode: number;
    /**
     * The asset type code discriminator this network element represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#assetTypeCode)
     */
    assetTypeCode: number;
    /**
     * The globalId of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#globalId)
     */
    globalId: string;
    /**
     * The network source Id of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#networkSourceId)
     */
    networkSourceId: number;
    /**
     * The objectId of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#objectId)
     */
    objectId: number;
    /**
     * Applicable to edge elements, represents a double value (0 to 1) where this edge element starts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#positionFrom)
     */
    positionFrom: number;
    /**
     * Applicable to edge elements, represents a double value (0 to 1) where this edge element ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#positionTo)
     */
    positionTo: number;
    /**
     * The terminal id defined at the network element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#terminalId)
     */
    terminalId: number;

    static fromJSON(json: any): NetworkElement;
  }

  interface NetworkElementProperties {
    /**
     * The asset group code that this network element represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#assetGroupCode)
     */
    assetGroupCode?: number;
    /**
     * The asset type code discriminator this network element represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#assetTypeCode)
     */
    assetTypeCode?: number;
    /**
     * The globalId of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#globalId)
     */
    globalId?: string;
    /**
     * The network source Id of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#networkSourceId)
     */
    networkSourceId?: number;
    /**
     * The objectId of the feature the network element belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#objectId)
     */
    objectId?: number;
    /**
     * Applicable to edge elements, represents a double value (0 to 1) where this edge element starts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#positionFrom)
     */
    positionFrom?: number;
    /**
     * Applicable to edge elements, represents a double value (0 to 1) where this edge element ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#positionTo)
     */
    positionTo?: number;
    /**
     * The terminal id defined at the network element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html#terminalId)
     */
    terminalId?: number;
  }

  export interface QueryAssociationsParameters extends Accessor, JSONSupport {}

  export class QueryAssociationsParameters {
    /**
     * QueryAssociationsParameters describes the parameters required to execute the [queryAssociations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-queryAssociations.html) function, which returns a list of [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) filtered by the parameters set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html)
     */

    constructor(properties?: QueryAssociationsParametersProperties);

    /**
     * The [NetworkElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html) for which the association is queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#elements)
     */
    elements: NetworkElement[];
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The date/timestamp (in UTC) to execute the function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#moment)
     */
    moment: Date;
    /**
     * Specifies whether to return logically deleted associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#returnDeletes)
     */
    returnDeletes: boolean;
    /**
     * Specifies an array of [Association](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) types to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#types)
     */
    types: string[];

    static fromJSON(json: any): QueryAssociationsParameters;
  }

  interface QueryAssociationsParametersProperties {
    /**
     * The [NetworkElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-NetworkElement.html) for which the association is queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#elements)
     */
    elements?: NetworkElementProperties[];
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The date/timestamp (in UTC) to execute the function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#moment)
     */
    moment?: DateProperties;
    /**
     * Specifies whether to return logically deleted associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#returnDeletes)
     */
    returnDeletes?: boolean;
    /**
     * Specifies an array of [Association](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) types to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html#types)
     */
    types?: string[];
  }

  export interface QueryAssociationsResult extends Accessor, JSONSupport {}

  export class QueryAssociationsResult {
    /**
     * Defines the results of the [queryAssociations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-queryAssociations.html) function which takes in [QueryAssociationsParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html) and returns an [QueryAssociationsResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsResult.html) of all [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) filtered by the parameters set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsResult.html)
     */

    constructor(properties?: QueryAssociationsResultProperties);

    /**
     * List of [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) returned filtered based on the input parameters in [QueryAssociationsParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsResult.html#associations)
     */
    associations: Association[];

    static fromJSON(json: any): QueryAssociationsResult;
  }

  interface QueryAssociationsResultProperties {
    /**
     * List of [associations](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-Association.html) returned filtered based on the input parameters in [QueryAssociationsParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-QueryAssociationsResult.html#associations)
     */
    associations?: AssociationProperties[];
  }

  export interface SynthesizeAssociationGeometriesParameters extends Accessor, JSONSupport {}

  export class SynthesizeAssociationGeometriesParameters {
    /**
     * This class describes the parameters required to execute the synthesizeAssociationGeometries function which synthesizes and returns the associations geometries in a given extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html)
     */

    constructor(properties?: SynthesizeAssociationGeometriesParametersProperties);

    /**
     * The extent used to execute a spatial query to retrieve the associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#extent)
     */
    extent: Extent;
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * A number that indicates the maximum associations that should be synthesized after which the operation should immediately return.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#maxGeometryCount)
     */
    maxGeometryCount: number;
    /**
     * The date/timestamp (in UTC) to execute the function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#moment)
     */
    moment: Date;
    /**
     * The spatial reference of the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#outSpatialReference)
     */
    outSpatialReference: any;
    /**
     * Indicates whether to synthesize and return structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnAttachmentAssociations)
     */
    returnAttachmentAssociations: boolean;
    /**
     * Indicates whether to synthesize and return connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnConnectivityAssociations)
     */
    returnConnectivityAssociations: boolean;
    /**
     * Indicates whether to synthesize and return containment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnContainmentAssociations)
     */
    returnContainmentAssociations: boolean;

    static fromJSON(json: any): SynthesizeAssociationGeometriesParameters;
  }

  interface SynthesizeAssociationGeometriesParametersProperties {
    /**
     * The extent used to execute a spatial query to retrieve the associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * A number that indicates the maximum associations that should be synthesized after which the operation should immediately return.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#maxGeometryCount)
     */
    maxGeometryCount?: number;
    /**
     * The date/timestamp (in UTC) to execute the function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#moment)
     */
    moment?: DateProperties;
    /**
     * The spatial reference of the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#outSpatialReference)
     */
    outSpatialReference?: any;
    /**
     * Indicates whether to synthesize and return structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnAttachmentAssociations)
     */
    returnAttachmentAssociations?: boolean;
    /**
     * Indicates whether to synthesize and return connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnConnectivityAssociations)
     */
    returnConnectivityAssociations?: boolean;
    /**
     * Indicates whether to synthesize and return containment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-SynthesizeAssociationGeometriesParameters.html#returnContainmentAssociations)
     */
    returnContainmentAssociations?: boolean;
  }

  export interface TraceLocation extends Accessor, JSONSupport {}

  export class TraceLocation {
    /**
     * To perform the trace analytic, users can optionally supply a list of locations in forms of globalIds (UUID) and terminals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html)
     */

    constructor(properties?: TraceLocationProperties);

    /**
     * The globalId (UUID) of the feature to start or stop the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#globalId)
     */
    globalId: string;
    /**
     * This indicates whether this barrier starting location should be skipped (filtered) when a trace attempts to find upstream controllers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#isFilterBarrier)
     */
    isFilterBarrier: boolean;
    /**
     * This double parameter of value of 0-1 indicates a percentage along the line of where the trace location is placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#percentAlong)
     */
    percentAlong: number;
    /**
     * The terminal Id to place the starting location at.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#terminalId)
     */
    terminalId: number;
    /**
     * The type of the trace location; `starting-point` defines where the trace should start and `barrier` defines where the trace should stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#type)
     */
    type: "starting-point" | "barrier";

    static fromJSON(json: any): TraceLocation;
  }

  interface TraceLocationProperties {
    /**
     * The globalId (UUID) of the feature to start or stop the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#globalId)
     */
    globalId?: string;
    /**
     * This indicates whether this barrier starting location should be skipped (filtered) when a trace attempts to find upstream controllers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#isFilterBarrier)
     */
    isFilterBarrier?: boolean;
    /**
     * This double parameter of value of 0-1 indicates a percentage along the line of where the trace location is placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#percentAlong)
     */
    percentAlong?: number;
    /**
     * The terminal Id to place the starting location at.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#terminalId)
     */
    terminalId?: number;
    /**
     * The type of the trace location; `starting-point` defines where the trace should start and `barrier` defines where the trace should stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceLocation.html#type)
     */
    type?: "starting-point" | "barrier";
  }

  export interface TraceParameters extends Accessor, JSONSupport {}

  export class TraceParameters {
    /**
     * The list of parameters that are needed to determine how the trace analytic will be executed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html)
     */

    constructor(properties?: TraceParametersProperties);

    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The date/timestamp (in UTC) to execute the function at a given time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#moment)
     */
    moment: Date;
    /**
     * The globalId (UUID) of the named trace configuration persisted in the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#namedTraceConfigurationGlobalId)
     */
    namedTraceConfigurationGlobalId: string;
    /**
     * The spatial reference that should be used to project the [aggregated geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html) returned by the trace (if applicable).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * Parameter specifying the types of results to return after running a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#resultTypes)
     */
    resultTypes: ResultTypeJSON[];
    /**
     * Defines the properties of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceConfiguration)
     */
    traceConfiguration: UNTraceConfiguration;
    /**
     * The list of starting points and barriers that will define where the trace starts and stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceLocations)
     */
    traceLocations: TraceLocation[];
    /**
     * The trace type defined in this trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceType)
     */
    traceType:
      | "connected"
      | "upstream"
      | "downstream"
      | "shortest-path"
      | "subnetwork"
      | "subnetwork-controller"
      | "loops"
      | "isolation";

    static fromJSON(json: any): TraceParameters;
  }

  interface TraceParametersProperties {
    /**
     * The geodatabase version to execute the function against.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The date/timestamp (in UTC) to execute the function at a given time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#moment)
     */
    moment?: DateProperties;
    /**
     * The globalId (UUID) of the named trace configuration persisted in the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#namedTraceConfigurationGlobalId)
     */
    namedTraceConfigurationGlobalId?: string;
    /**
     * The spatial reference that should be used to project the [aggregated geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-AggregatedGeometry.html) returned by the trace (if applicable).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * Parameter specifying the types of results to return after running a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#resultTypes)
     */
    resultTypes?: ResultTypeJSON[];
    /**
     * Defines the properties of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceConfiguration)
     */
    traceConfiguration?: UNTraceConfiguration;
    /**
     * The list of starting points and barriers that will define where the trace starts and stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceLocations)
     */
    traceLocations?: TraceLocationProperties[];
    /**
     * The trace type defined in this trace configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#traceType)
     */
    traceType?:
      | "connected"
      | "upstream"
      | "downstream"
      | "shortest-path"
      | "subnetwork"
      | "subnetwork-controller"
      | "loops"
      | "isolation";
  }

  /**
   * ResultTypeJSON specifies the type of results to return after running a trace.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
   */
  export interface ResultTypeJSON {
    /**
     * Specifies whether to return type `"elements"` or `"aggregatedGeometry"`.
     *
     * [Read more...](global.html#type)
     */
    type: "elements" | "aggregatedGeometry";
    /**
     * Specifies whether to return geometry values in resultTypes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
     */
    includeGeometry: boolean;
    /**
     * Specifies whether to return propagated values in resultTypes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
     */
    includePropagatedValues: boolean;
    /**
     * Specifies the network attribute names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
     */
    networkAttributeNames: string[];
    /**
     * Specifies the name of a predefined Diagram Template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
     */
    diagramTemplateName: string;
    /**
     * Specifies the types of output fields that can be generated from geoprocessing tools.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceParameters.html#ResultTypeJSON)
     */
    resultTypeFields: any[];
  }

  export interface TraceResult extends Accessor, JSONSupport {}

  export class TraceResult {
    /**
     * This class defines the collection of results returned from the trace function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html)
     */

    constructor(properties?: TraceResultProperties);

    /**
     * This property defines an aggregation of geometries returned by the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#aggregatedGeometry)
     */
    aggregatedGeometry: AggregatedGeometry;
    /**
     * An array of network elements returned by the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#elements)
     */
    elements: NetworkElement[];
    /**
     * Returns an array of function aggregation results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#globalFunctionResults)
     */
    globalFunctionResults: FunctionResult[];
    /**
     * This parameter is specific to the K-Nearest Neighbors Algorithm, when the nearest filter is provided in the trace configuration.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#kFeaturesForKNNFound)
     */
    kFeaturesForKNNFound: boolean;
    /**
     * Returns `true` if the starting points in the network trace operation are ignored.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#startingPointsIgnored)
     */
    startingPointsIgnored: boolean;
    /**
     * Returns any warnings encountered by the trace operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#warnings)
     */
    warnings: string[];

    static fromJSON(json: any): TraceResult;
  }

  interface TraceResultProperties {
    /**
     * This property defines an aggregation of geometries returned by the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#aggregatedGeometry)
     */
    aggregatedGeometry?: AggregatedGeometryProperties;
    /**
     * An array of network elements returned by the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#elements)
     */
    elements?: NetworkElementProperties[];
    /**
     * Returns an array of function aggregation results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#globalFunctionResults)
     */
    globalFunctionResults?: FunctionResultProperties[];
    /**
     * This parameter is specific to the K-Nearest Neighbors Algorithm, when the nearest filter is provided in the trace configuration.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#kFeaturesForKNNFound)
     */
    kFeaturesForKNNFound?: boolean;
    /**
     * Returns `true` if the starting points in the network trace operation are ignored.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#startingPointsIgnored)
     */
    startingPointsIgnored?: boolean;
    /**
     * Returns any warnings encountered by the trace operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-TraceResult.html#warnings)
     */
    warnings?: string[];
  }

  export interface ValidateNetworkTopologyParameters extends Accessor, JSONSupport {}

  export class ValidateNetworkTopologyParameters {
    /**
     * ValidateNetworkTopologyParameters describes the optional and required parameters for validating a network topology.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html)
     */

    constructor(properties?: ValidateNetworkTopologyParametersProperties);

    /**
     * Specifies the geodatabase version name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The spatial reference of the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#outSpatialReference)
     */
    outSpatialReference: any;
    /**
     * Specifies the GUID used to lock the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#sessionID)
     */
    sessionID: string;
    /**
     * Specifies the envelope of the area to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validateArea)
     */
    validateArea: Extent;
    /**
     * Specifies the set of features and objects to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validationSet)
     */
    validationSet: ValidationSetItemJSON[];
    /**
     * Specifies the validation to perform.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validationType)
     */
    validationType: "normal" | "rebuild" | "force-rebuild";

    static fromJSON(json: any): ValidateNetworkTopologyParameters;
  }

  interface ValidateNetworkTopologyParametersProperties {
    /**
     * Specifies the geodatabase version name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The spatial reference of the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#outSpatialReference)
     */
    outSpatialReference?: any;
    /**
     * Specifies the GUID used to lock the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#sessionID)
     */
    sessionID?: string;
    /**
     * Specifies the envelope of the area to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validateArea)
     */
    validateArea?: ExtentProperties;
    /**
     * Specifies the set of features and objects to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validationSet)
     */
    validationSet?: ValidationSetItemJSON[];
    /**
     * Specifies the validation to perform.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#validationType)
     */
    validationType?: "normal" | "rebuild" | "force-rebuild";
  }

  /**
   * ValidationSetItemJSON represents the set of features and objects to validate.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#ValidationSetItemJSON)
   */
  export interface ValidationSetItemJSON {
    /**
     * The network source id of the.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#ValidationSetItemJSON)
     */
    sourceId: number;
    /**
     * An array of globalId strings to validate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyParameters.html#ValidationSetItemJSON)
     */
    globalIds: string[];
  }

  /**
   * Class that holds the returned object after running the [Network.validateTopology()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#validateTopology) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html)
   */
  interface ValidateNetworkTopologyResult {
    /**
     * Returns dirty area count if using Utility Network Version 3, otherwise will return 0.
     *
     * @deprecated since version 4.28. Dirty area count was implemented in the original version of utility network, but as of
     * schema version 4 of the utility network, this is no longer supported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#dirtyAreaCount)
     */
    dirtyAreaCount: number;
    /**
     * Contains information about the subnetworks that are marked as dirty during the validation process, along with the domain network and tier containing the subnetwork.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#discoveredSubnetworks)
     */
    discoveredSubnetworks: Subnetwork[];
    /**
     * If `maxRecordCount` is configured for a layer, `exceededTransferLimit` will be `true` if a query matches more than the `maxRecordCount` features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#exceededTransferLimit)
     */
    exceededTransferLimit: boolean;
    /**
     * If the current feature service contains no dirty areas, full update will return true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#fullUpdate)
     */
    fullUpdate: boolean;
    /**
     * The date/timestamp (in UTC) when the validate process was executed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#moment)
     */
    moment: Date;
    /**
     * The serviceEdits returns full features including the original features prior to delete, the original and current features for updates, and the current rows for inserts, which may contain implicit changes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#serviceEdits)
     */
    serviceEdits: ValidateServiceEdits[];
    /**
     * If error features are created during the validation process validateErrorsCreated will be true.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#validateErrorsCreated)
     */
    validateErrorsCreated: boolean;
  }

  export const ValidateNetworkTopologyResult: ValidateNetworkTopologyResult;

  /**
   * The Subnetwork object represents the domain name, tier name, and subnetwork via fulfilling the promise returned from running the [Network.validateTopology()](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-Network.html#validateTopology) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#Subnetwork)
   */
  export interface Subnetwork {
    /**
     * The domain name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#Subnetwork)
     */
    domain: string;
    /**
     * The tier name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#Subnetwork)
     */
    tier: string;
    /**
     * The subnetwork name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#Subnetwork)
     */
    subnetwork: string;
  }

  /**
   * ValidateServiceEdits represents the layerId and editedFeatures.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#ValidateServiceEdits)
   */
  export interface ValidateServiceEdits {
    /**
     * The layerId of the layer with ServiceEdits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#ValidateServiceEdits)
     */
    layerId: number;
    /**
     * The editedFeatures object returns full features including the original features prior to delete, the original and current features for updates, and the current rows for inserts, which may contain implicit changes (for example, as a result of a calculation rule).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-support-ValidateNetworkTopologyResult.html#ValidateServiceEdits)
     */
    editedFeatures?: any;
  }

  /**
   * The utility network associations model connectivity, containment, and structure relations between assets.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-synthesizeAssociationGeometries.html)
   */
  interface synthesizeAssociationGeometries {
    /**
     * Given an extent, returns all associations within this extent and their synthesized geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-synthesizeAssociationGeometries.html#synthesizeAssociationGeometries)
     */
    synthesizeAssociationGeometries(
      url: string,
      params: SynthesizeAssociationGeometriesParameters,
      requestOptions?: any,
    ): Promise<AssociationGeometriesResult>;
  }

  export const synthesizeAssociationGeometries: synthesizeAssociationGeometries;

  /**
   * Trace is the core analytic of the utility network, by providing a set of parameters, you can start the trace in one or more features and traverse the network topology satisfying the parameters until the trace stops at end points or when the definition of the trace condition barriers are met.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-trace.html)
   */
  interface trace {
    /**
     * The trace function takes a set of parameters, executes the trace analytic on the backend, and returns trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networks-trace.html#trace)
     */
    trace(url: string, params: TraceParameters, requestOptions?: any): Promise<TraceResult>;
  }

  export const trace: trace;

  /**
   * This modules contains functions for working with network services.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html)
   */
  interface networkService {
    /**
     * Retrieve a description of the network service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html#fetchServiceDescription)
     */
    fetchServiceDescription(url: string, apiKey?: string, requestOptions?: any): Promise<ServiceDescription>;
  }

  export const networkService: networkService;

  /**
   * The network service description.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html#ServiceDescription)
   */
  export interface ServiceDescription {
    /**
     * Server version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html#ServiceDescription)
     */
    currentVersion: number;
    /**
     * Default travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html#ServiceDescription)
     */
    defaultTravelMode: TravelMode;
    /**
     * Available travel modes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-networkService.html#ServiceDescription)
     */
    supportedTravelModes: TravelMode[];
  }

  /**
   * Find [places](/documentation/mapping-apis-and-services/places/) within a search distance of a geographic point or within an extent, or find more information about specific places.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html)
   */
  interface places {
    /**
     * Get place details, including name, address, description, and other attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#fetchPlace)
     */
    fetchPlace(params: FetchPlaceParameters, requestOptions?: any): Promise<any>;
    /**
     * A nearby search that finds places within a given radius of a location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesNearPoint)
     */
    queryPlacesNearPoint(params: PlacesQueryParameters, requestOptions?: any): Promise<PlacesQueryResult>;
    /**
     * A search that finds places within a given extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesWithinExtent)
     */
    queryPlacesWithinExtent(params: PlacesQueryParameters, requestOptions?: any): Promise<PlacesQueryResult>;
  }

  export const places: places;

  /**
   * The print module provides an `executePrint` method that generates a printer-ready version of the map using an [Export Web Map Task](https://developers.arcgis.com/rest/services-reference/export-web-map-task.htm) available with ArGIS Server 10.1 and later.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html)
   */
  interface print {
    /**
     * Sends a request to the print service to create a printable static image of the map using the options specified in the [PrintParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html#execute)
     */
    execute(url: string, params: PrintParameters, requestOptions?: any): Promise<PrintResponse>;
    /**
     * Request the mode for the print request service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html#getMode)
     */
    getMode(url: string): Promise<string>;
  }

  export const print: print;

  /**
   * Represents the response of the [execute()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html#execute) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html#PrintResponse)
   */
  export interface PrintResponse {
    /**
     * URL pointing to the location of the generated printout of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html#PrintResponse)
     */
    url: string;
  }

  /**
   * Executes different types of query operations on a layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html)
   */
  interface query {
    /**
     * Query information about attachments associated with features from a [feature layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) specified in the url parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeAttachmentQuery)
     */
    executeAttachmentQuery(
      url: string,
      attachmentQuery: AttachmentQuery | AttachmentQueryProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Gets a count of the number of features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForCount)
     */
    executeForCount(url: string, query: Query | QueryProperties, requestOptions?: any): Promise<number>;
    /**
     * Gets the extent of the features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForExtent)
     */
    executeForExtent(url: string, params: Query, requestOptions?: any): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer specified in the url parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForIds)
     */
    executeForIds(url: string, query: Query | QueryProperties, requestOptions?: any): Promise<number[]>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns the count of features or records that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForTopCount)
     */
    executeForTopCount(url: string, topFeaturesQuery: TopFeaturesQuery, requestOptions?: any): Promise<number>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForTopExtents)
     */
    executeForTopExtents(
      url: string,
      topFeaturesQuery: TopFeaturesQuery,
      outSpatialReference: SpatialReference,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns an array of Object IDs of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeForTopIds)
     */
    executeForTopIds(
      url: string,
      topFeaturesQuery: TopFeaturesQuery,
      outSpatialReference: SpatialReference,
      requestOptions?: any,
    ): Promise<number[]>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer specified in the url parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeQueryJSON)
     */
    executeQueryJSON(url: string, query: Query | QueryProperties, requestOptions?: any): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against the layer specified in the url parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeQueryPBF)
     */
    executeQueryPBF(url: string, query: Query | QueryProperties, requestOptions?: any): Promise<FeatureSet>;
    /**
     * Executes a [RelationshipQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html) against the layer or table specified in the url parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeRelationshipQuery)
     */
    executeRelationshipQuery(
      url: string,
      relationshipQuery: RelationshipQuery | RelationshipQueryProperties,
      requestOptions?: any,
    ): Promise<any>;
    /**
     * Executes a [TopFeaturesQuery](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html) against a feature service and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) once the promise resolves.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeTopFeaturesQuery)
     */
    executeTopFeaturesQuery(
      url: string,
      topFeaturesQuery: TopFeaturesQuery,
      outSpatialReference: SpatialReference,
      requestOptions?: any,
    ): Promise<FeatureSet>;
  }

  export const query: query;

  export interface AttachmentInfo extends Accessor, JSONSupport {}

  export class AttachmentInfo {
    /**
     * The `AttachmentInfo` class returns information about attachments associated with a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html)
     */

    constructor(properties?: AttachmentInfoProperties);

    /**
     * The content type of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#contentType)
     */
    contentType: string;
    /**
     * An array of [ExifInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo) for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#exifInfo)
     */
    exifInfo: ExifInfo[];
    /**
     * The global identifier for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#globalId)
     */
    globalId: string;
    /**
     * The identifier for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#id)
     */
    id: number;
    /**
     * Keywords used for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#keywords)
     */
    keywords: string;
    /**
     * String value indicating the name of the file attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#name)
     */
    name: string;
    /**
     * The [OrientationInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#OrientationInfo) for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#orientationInfo)
     */
    readonly orientationInfo: OrientationInfo;
    /**
     * The parent or the feature global id of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#parentGlobalId)
     */
    parentGlobalId: string;
    /**
     * The parent or the feature object id of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#parentObjectId)
     */
    parentObjectId: number | string;
    /**
     * The file size of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#size)
     */
    size: number;
    /**
     * The URL of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#url)
     */
    url: string;

    /**
     * Creates a deep clone of the AttachmentInfo class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#clone)
     */
    clone(): AttachmentInfo;

    static fromJSON(json: any): AttachmentInfo;
  }

  interface AttachmentInfoProperties {
    /**
     * The content type of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#contentType)
     */
    contentType?: string;
    /**
     * An array of [ExifInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo) for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#exifInfo)
     */
    exifInfo?: ExifInfo[];
    /**
     * The global identifier for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#globalId)
     */
    globalId?: string;
    /**
     * The identifier for the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#id)
     */
    id?: number;
    /**
     * Keywords used for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#keywords)
     */
    keywords?: string;
    /**
     * String value indicating the name of the file attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#name)
     */
    name?: string;
    /**
     * The parent or the feature global id of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#parentGlobalId)
     */
    parentGlobalId?: string;
    /**
     * The parent or the feature object id of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#parentObjectId)
     */
    parentObjectId?: number | string;
    /**
     * The file size of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#size)
     */
    size?: number;
    /**
     * The URL of the attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#url)
     */
    url?: string;
  }

  /**
   * An array of [Exchangeable image file format](https://en.wikipedia.org/wiki/Exif) information for the attachment.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
   */
  export interface ExifInfo {
    /**
     * The file name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
     */
    name?: string;
    /**
     * Array of tag objects containing the following properties:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
     */
    tags?: ExifInfoTags[];
  }

  /**
   * An object containing properties specific to the orientation of an image attachment.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#OrientationInfo)
   */
  export interface OrientationInfo {
    /**
     * The identifier for the orientation info.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#OrientationInfo)
     */
    id?: number;
    /**
     * The rotation value for the attached image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#OrientationInfo)
     */
    rotation?: number;
    /**
     * Indicates whether the image displays mirrored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#OrientationInfo)
     */
    mirrored?: boolean;
  }

  export interface ExifInfoTags {
    /**
     * The tag name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
     */
    name: string;
    /**
     * The tag description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
     */
    description: string;
    /**
     * The value of the tag.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#ExifInfo)
     */
    value: any;
  }

  /**
   * Find routes between two or more locations and optionally get driving directions.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html)
   */
  interface route {
    /**
     * Solves the route against the route layer with the route parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve)
     */
    solve(url: string, params: RouteParameters, requestOptions?: any): Promise<RouteSolveResult>;
  }

  export const route: route;

  /**
   * With the Service area service, you can find the area that can be reached from the input location within a given travel time or travel distance.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-serviceArea.html)
   */
  interface serviceArea {
    /**
     * Determines the service area based on a set of parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-serviceArea.html#solve)
     */
    solve(url: string, params: ServiceAreaParameters, requestOptions?: any): Promise<ServiceAreaSolveResult>;
  }

  export const serviceArea: serviceArea;

  export interface AddressCandidate extends Accessor, JSONSupport {}

  export class AddressCandidate {
    /**
     * Represents the result of a geocode service operation as a list of address candidates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html)
     */

    constructor(properties?: AddressCandidateProperties);

    /**
     * Address of the candidate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#address)
     */
    address: string;
    /**
     * Name value pairs of field name and field value as defined in `outFields` in [locator.addressToLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#attributes)
     */
    attributes: any;
    /**
     * The minimum and maximum X and Y coordinates of a bounding box of the address candidate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#extent)
     */
    extent: Extent;
    /**
     * The [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) object representing the location of the [address](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#address).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#location)
     */
    location: Point;
    /**
     * Numeric score between `0` and `100` for geocode candidates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#score)
     */
    score: number;

    static fromJSON(json: any): AddressCandidate;
  }

  interface AddressCandidateProperties {
    /**
     * Address of the candidate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#address)
     */
    address?: string;
    /**
     * Name value pairs of field name and field value as defined in `outFields` in [locator.addressToLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html#addressToLocations).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#attributes)
     */
    attributes?: any;
    /**
     * The minimum and maximum X and Y coordinates of a bounding box of the address candidate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * The [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) object representing the location of the [address](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#address).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#location)
     */
    location?: PointProperties;
    /**
     * Numeric score between `0` and `100` for geocode candidates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AddressCandidate.html#score)
     */
    score?: number;
  }

  export class AlgorithmicColorRamp extends ColorRamp {
    /**
     * Creates a color ramp for use in a raster renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html)
     */

    constructor(properties?: AlgorithmicColorRampProperties);

    /**
     * The algorithm used to generate the colors between the `fromColor` and `toColor`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#algorithm)
     */
    algorithm: "cie-lab" | "lab-lch" | "hsv";
    /**
     * The first color in the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#fromColor)
     */
    fromColor: Color;
    /**
     * The last color in the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#toColor)
     */
    toColor: Color;
    /**
     * A string value representing the color ramp type.
     *
     * @default "algorithmic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#type)
     */
    readonly type: "algorithmic";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#clone)
     */
    clone(): AlgorithmicColorRamp;

    static fromJSON(json: any): AlgorithmicColorRamp;
  }

  interface AlgorithmicColorRampProperties extends ColorRampProperties {
    /**
     * The algorithm used to generate the colors between the `fromColor` and `toColor`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#algorithm)
     */
    algorithm?: "cie-lab" | "lab-lch" | "hsv";
    /**
     * The first color in the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#fromColor)
     */
    fromColor?: Color;
    /**
     * The last color in the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AlgorithmicColorRamp.html#toColor)
     */
    toColor?: Color;
  }

  export interface ArealUnit extends Accessor, JSONSupport {}

  export class ArealUnit {
    /**
     * A geoprocessing object that contains area information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ArealUnit.html)
     */

    constructor(properties?: ArealUnitProperties);

    /**
     * Specifies the numerical area.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ArealUnit.html#area)
     */
    area: number;
    /**
     * Specifies the unit of area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ArealUnit.html#units)
     */
    units:
      | "international-acres"
      | "us-acres"
      | "ares"
      | "hectares"
      | "square-centimeters"
      | "square-decimeters"
      | "square-international-feet"
      | "square-us-feet"
      | "square-international-inches"
      | "square-us-inches"
      | "square-kilometers"
      | "square-meters"
      | "square-international-miles"
      | "square-us-miles"
      | "square-millimeters"
      | "square-international-yards"
      | "square-us-yards"
      | "unknown";

    static fromJSON(json: any): ArealUnit;
  }

  interface ArealUnitProperties {
    /**
     * Specifies the numerical area.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ArealUnit.html#area)
     */
    area?: number;
    /**
     * Specifies the unit of area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ArealUnit.html#units)
     */
    units?:
      | "international-acres"
      | "us-acres"
      | "ares"
      | "hectares"
      | "square-centimeters"
      | "square-decimeters"
      | "square-international-feet"
      | "square-us-feet"
      | "square-international-inches"
      | "square-us-inches"
      | "square-kilometers"
      | "square-meters"
      | "square-international-miles"
      | "square-us-miles"
      | "square-millimeters"
      | "square-international-yards"
      | "square-us-yards"
      | "unknown";
  }

  export interface AreasAndLengthsParameters extends Accessor, JSONSupport {}

  export class AreasAndLengthsParameters {
    /**
     * Input parameters for the areasAndLengths() method on the GeometryService.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html)
     */

    constructor(properties?: AreasAndLengthsParametersProperties);

    /**
     * The area unit in which areas of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#areaUnit)
     */
    areaUnit:
      | "acres"
      | "ares"
      | "hectares"
      | "square-miles"
      | "square-kilometers"
      | "square-meters"
      | "square-feet"
      | "square-yards";
    /**
     * Defines the type of calculation for the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#calculationType)
     */
    calculationType: "planar" | "geodesic" | "preserve-shape";
    /**
     * The length unit in which perimeters of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#lengthUnit)
     */
    lengthUnit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * Polygon geometries for which to compute areas and lengths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#polygons)
     */
    polygons: Polygon[];

    static fromJSON(json: any): AreasAndLengthsParameters;
  }

  interface AreasAndLengthsParametersProperties {
    /**
     * The area unit in which areas of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#areaUnit)
     */
    areaUnit?:
      | "acres"
      | "ares"
      | "hectares"
      | "square-miles"
      | "square-kilometers"
      | "square-meters"
      | "square-feet"
      | "square-yards";
    /**
     * Defines the type of calculation for the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#calculationType)
     */
    calculationType?: "planar" | "geodesic" | "preserve-shape";
    /**
     * The length unit in which perimeters of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#lengthUnit)
     */
    lengthUnit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * Polygon geometries for which to compute areas and lengths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AreasAndLengthsParameters.html#polygons)
     */
    polygons?: PolygonProperties[];
  }

  export interface AttachmentQuery extends Accessor, JSONSupport {}

  export class AttachmentQuery {
    /**
     * This class defines parameters for executing queries for feature attachments from a [feature layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html)
     */

    constructor(properties?: AttachmentQueryProperties);

    /**
     * The where clause to be applied to attachment queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#attachmentsWhere)
     */
    attachmentsWhere: string;
    /**
     * The file format that is supported by query attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#attachmentTypes)
     */
    attachmentTypes: string[];
    /**
     * Indicates if the service should cache the attachment query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#cacheHint)
     */
    cacheHint: boolean;
    /**
     * An array of globalIds for the features in the layer being queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#globalIds)
     */
    globalIds: string[];
    /**
     * Used to query for attachments that match the provided `keywords`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#keywords)
     */
    keywords: string[];
    /**
     * Used to query for attachments that match this `name`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#name)
     */
    name: string;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#num)
     */
    num: number;
    /**
     * An array of objectIds of the features to be queried for attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#objectIds)
     */
    objectIds: number[];
    /**
     * If `true`, the [Exchangeable image file format](https://en.wikipedia.org/wiki/Exif) for the attachment will be included in [attachmentInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#exifInfo).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#returnMetadata)
     */
    returnMetadata: boolean;
    /**
     * The file size of the attachment is specified in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#size)
     */
    size: number[];
    /**
     * This option fetches query results by skipping a specified number of records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#start)
     */
    start: number;
    /**
     * The `where` clause to be applied to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#where)
     */
    where: string;

    /**
     * Creates a deep clone of AttachmentQuery object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#clone)
     */
    clone(): AttachmentQuery;

    static fromJSON(json: any): AttachmentQuery;
  }

  interface AttachmentQueryProperties {
    /**
     * The where clause to be applied to attachment queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#attachmentsWhere)
     */
    attachmentsWhere?: string;
    /**
     * The file format that is supported by query attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#attachmentTypes)
     */
    attachmentTypes?: string[];
    /**
     * Indicates if the service should cache the attachment query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#cacheHint)
     */
    cacheHint?: boolean;
    /**
     * An array of globalIds for the features in the layer being queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#globalIds)
     */
    globalIds?: string[];
    /**
     * Used to query for attachments that match the provided `keywords`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#keywords)
     */
    keywords?: string[];
    /**
     * Used to query for attachments that match this `name`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#name)
     */
    name?: string;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#num)
     */
    num?: number;
    /**
     * An array of objectIds of the features to be queried for attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#objectIds)
     */
    objectIds?: number[];
    /**
     * If `true`, the [Exchangeable image file format](https://en.wikipedia.org/wiki/Exif) for the attachment will be included in [attachmentInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html#exifInfo).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#returnMetadata)
     */
    returnMetadata?: boolean;
    /**
     * The file size of the attachment is specified in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#size)
     */
    size?: number[];
    /**
     * This option fetches query results by skipping a specified number of records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#start)
     */
    start?: number;
    /**
     * The `where` clause to be applied to the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-AttachmentQuery.html#where)
     */
    where?: string;
  }

  export class BaseImageMeasureParameters {
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) on how individual images should be mosaicked when the measure is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureParameters.html#mosaicRule)
     */
    mosaicRule: MosaicRule;
    /**
     * Specifies the pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureParameters.html#pixelSize)
     */
    pixelSize: Point;
  }

  interface BaseImageMeasureParametersProperties {
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) on how individual images should be mosaicked when the measure is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureParameters.html#mosaicRule)
     */
    mosaicRule?: MosaicRuleProperties;
    /**
     * Specifies the pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureParameters.html#pixelSize)
     */
    pixelSize?: PointProperties;
  }

  export class BaseImageMeasureResult {
    /**
     * Name of the raster dataset used in the area and height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureResult.html#name)
     */
    name: string;
    /**
     * Sensor name of the raster dataset used in the area and height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureResult.html#sensorName)
     */
    sensorName: string;
  }

  interface BaseImageMeasureResultProperties {
    /**
     * Name of the raster dataset used in the area and height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureResult.html#name)
     */
    name?: string;
    /**
     * Sensor name of the raster dataset used in the area and height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BaseImageMeasureResult.html#sensorName)
     */
    sensorName?: string;
  }

  export interface BufferParameters extends Accessor, JSONSupport {}

  export class BufferParameters {
    /**
     * Sets the distances, units, and other parameters for the [buffer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#buffer) method on the [geometryService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html)
     */

    constructor(properties?: BufferParametersProperties);

    /**
     * The spatial reference in which the geometries are buffered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#bufferSpatialReference)
     */
    bufferSpatialReference: SpatialReference;
    /**
     * The distances the input features are buffered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#distances)
     */
    distances: number[];
    /**
     * If the input geometries are in a geographic coordinate system, set geodesic to `true` to generate a buffer polygon using a geodesic distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#geodesic)
     */
    geodesic: boolean;
    /**
     * The input geometries to buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * The spatial reference for the returned geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * If `true`, all geometries buffered at a given distance are unioned into a single (possibly multipart) polygon, and the unioned geometry is placed in the output array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#unionResults)
     */
    unionResults: boolean;
    /**
     * The units for calculating each buffer distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#unit)
     */
    unit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";

    static fromJSON(json: any): BufferParameters;
  }

  interface BufferParametersProperties {
    /**
     * The spatial reference in which the geometries are buffered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#bufferSpatialReference)
     */
    bufferSpatialReference?: SpatialReferenceProperties;
    /**
     * The distances the input features are buffered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#distances)
     */
    distances?: number[];
    /**
     * If the input geometries are in a geographic coordinate system, set geodesic to `true` to generate a buffer polygon using a geodesic distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#geodesic)
     */
    geodesic?: boolean;
    /**
     * The input geometries to buffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * The spatial reference for the returned geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * If `true`, all geometries buffered at a given distance are unioned into a single (possibly multipart) polygon, and the unioned geometry is placed in the output array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#unionResults)
     */
    unionResults?: boolean;
    /**
     * The units for calculating each buffer distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-BufferParameters.html#unit)
     */
    unit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
  }

  export class CameraInfo extends JSONSupport {
    /**
     * Camera information returned as a result of running [ImageryLayer.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryGPSInfo) or [imageService.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryGPSInfo) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html)
     */

    constructor(properties?: CameraInfoProperties);

    /**
     * Camera's columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#cols)
     */
    cols: number;
    /**
     * Camera's focal length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#focalLength)
     */
    focalLength: number;
    /**
     * Camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#id)
     */
    id: string;
    /**
     * Camera's manufacturer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#make)
     */
    make: string;
    /**
     * Camera's model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#model)
     */
    model: string;
    /**
     * Camera's pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#pixelSize)
     */
    pixelSize: number;
    /**
     * Camera's rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#rows)
     */
    rows: number;

    static fromJSON(json: any): CameraInfo;
  }

  interface CameraInfoProperties {
    /**
     * Camera's columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#cols)
     */
    cols?: number;
    /**
     * Camera's focal length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#focalLength)
     */
    focalLength?: number;
    /**
     * Camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#id)
     */
    id?: string;
    /**
     * Camera's manufacturer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#make)
     */
    make?: string;
    /**
     * Camera's model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#model)
     */
    model?: string;
    /**
     * Camera's pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#pixelSize)
     */
    pixelSize?: number;
    /**
     * Camera's rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-CameraInfo.html#rows)
     */
    rows?: number;
  }

  export interface ClosestFacilityParameters extends Accessor, JSONSupport, Clonable {}

  export class ClosestFacilityParameters {
    /**
     * ClosestFacilityParameters provides the input parameters for a [closestFacility](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html)
     */

    constructor(properties?: ClosestFacilityParametersProperties);

    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#accumulateAttributes)
     */
    accumulateAttributes: string[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#apiKey)
     */
    apiKey: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#attributeParameterValues)
     */
    attributeParameterValues: AttributeParameterValue[];
    /**
     * The travel time or travel distance value at which to stop searching for facilities for a given incident.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#defaultCutoff)
     */
    defaultCutoff: number;
    /**
     * The number of closest facilities to find per incident.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#defaultTargetFacilityCount)
     */
    defaultTargetFacilityCount: number;
    /**
     * The language that will be used when generating travel directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsLanguage)
     */
    directionsLanguage: string;
    /**
     * Specify the units for displaying travel distance in the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsLengthUnits)
     */
    directionsLengthUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Define the content and verbosity of the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsOutputType)
     */
    directionsOutputType:
      | "complete"
      | "complete-no-events"
      | "featuresets"
      | "instructions-only"
      | "standard"
      | "summary-only";
    /**
     * Specify the name of the formatting style for the directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsStyleName)
     */
    directionsStyleName: "desktop" | "navigation" | "campus";
    /**
     * Set the time-based impedance attribute to display the duration of a maneuver, such as "Go northwest on Alvorado St.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsTimeAttribute)
     */
    directionsTimeAttribute: "travel-time" | "truck-travel-time" | "walk-time" | "minutes" | "truck-time" | string;
    /**
     * One or more locations that are searched for when finding the closest location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#facilities)
     */
    facilities: DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#impedanceAttribute)
     */
    impedanceAttribute: string;
    /**
     * One or more locations from which the service searches for the nearby locations
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#incidents)
     */
    incidents: DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputLines)
     */
    outputLines: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#overrides)
     */
    overrides: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#pointBarriers)
     */
    pointBarriers: Collection<PointBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polygonBarriers)
     */
    polygonBarriers: Collection<PolygonBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polylineBarriers)
     */
    polylineBarriers: Collection<PolylineBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#preserveObjectID)
     */
    preserveObjectID: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#restrictionAttributes)
     */
    restrictionAttributes: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Specifies how U-Turns should be handled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#restrictUTurns)
     */
    restrictUTurns: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Specify whether the operation should generate driving directions for each route.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnDirections)
     */
    returnDirections: boolean;
    /**
     * Determines if facilities will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnFacilities)
     */
    returnFacilities: boolean;
    /**
     * Determines if incidents will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnIncidents)
     */
    returnIncidents: boolean;
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPointBarriers)
     */
    returnPointBarriers: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers: boolean;
    /**
     * Use this property to specify if the operation should return routes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnRoutes)
     */
    returnRoutes: boolean;
    /**
     * Specify whether traversed edges will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedEdges)
     */
    returnTraversedEdges: boolean;
    /**
     * Specify whether traversed junctions will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedJunctions)
     */
    returnTraversedJunctions: boolean;
    /**
     * Specify whether traversed turns will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedTurns)
     */
    returnTraversedTurns: boolean;
    /**
     * Include z values for the returned geometries if supported by the underlying network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnZ)
     */
    returnZ: boolean;
    /**
     * Specify the time and date to depart from or arrive at incidents or facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay)
     */
    timeOfDay: Date | "now";
    /**
     * Specify the time zone or zones of the [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDayIsUTC)
     */
    timeOfDayIsUTC: boolean;
    /**
     * The [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay) parameter value represents the arrival or departure time for the routes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDayUsage)
     */
    timeOfDayUsage: "start" | "end" | "not-used";
    /**
     * Specifies how the travel direction for the closest facility search will be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#travelDirection)
     */
    travelDirection: "from-facility" | "to-facility";
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#travelMode)
     */
    travelMode: TravelMode;
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#useHierarchy)
     */
    useHierarchy: boolean;

    static fromJSON(json: any): ClosestFacilityParameters;
  }

  interface ClosestFacilityParametersProperties {
    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#accumulateAttributes)
     */
    accumulateAttributes?: string[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#apiKey)
     */
    apiKey?: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#attributeParameterValues)
     */
    attributeParameterValues?: AttributeParameterValue[];
    /**
     * The travel time or travel distance value at which to stop searching for facilities for a given incident.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#defaultCutoff)
     */
    defaultCutoff?: number;
    /**
     * The number of closest facilities to find per incident.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#defaultTargetFacilityCount)
     */
    defaultTargetFacilityCount?: number;
    /**
     * The language that will be used when generating travel directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsLanguage)
     */
    directionsLanguage?: string;
    /**
     * Specify the units for displaying travel distance in the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsLengthUnits)
     */
    directionsLengthUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Define the content and verbosity of the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsOutputType)
     */
    directionsOutputType?:
      | "complete"
      | "complete-no-events"
      | "featuresets"
      | "instructions-only"
      | "standard"
      | "summary-only";
    /**
     * Specify the name of the formatting style for the directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsStyleName)
     */
    directionsStyleName?: "desktop" | "navigation" | "campus";
    /**
     * Set the time-based impedance attribute to display the duration of a maneuver, such as "Go northwest on Alvorado St.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsTimeAttribute)
     */
    directionsTimeAttribute?: "travel-time" | "truck-travel-time" | "walk-time" | "minutes" | "truck-time" | string;
    /**
     * One or more locations that are searched for when finding the closest location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#facilities)
     */
    facilities?:
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ?: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations?: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#impedanceAttribute)
     */
    impedanceAttribute?: string;
    /**
     * One or more locations from which the service searches for the nearby locations
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#incidents)
     */
    incidents?:
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision?: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outputLines)
     */
    outputLines?: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#overrides)
     */
    overrides?: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#pointBarriers)
     */
    pointBarriers?:
      | CollectionProperties<PointBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polygonBarriers)
     */
    polygonBarriers?:
      | CollectionProperties<PolygonBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polylineBarriers)
     */
    polylineBarriers?:
      | CollectionProperties<PolylineBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#preserveObjectID)
     */
    preserveObjectID?: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#restrictionAttributes)
     */
    restrictionAttributes?: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Specifies how U-Turns should be handled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#restrictUTurns)
     */
    restrictUTurns?: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Specify whether the operation should generate driving directions for each route.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnDirections)
     */
    returnDirections?: boolean;
    /**
     * Determines if facilities will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnFacilities)
     */
    returnFacilities?: boolean;
    /**
     * Determines if incidents will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnIncidents)
     */
    returnIncidents?: boolean;
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPointBarriers)
     */
    returnPointBarriers?: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers?: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers?: boolean;
    /**
     * Use this property to specify if the operation should return routes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnRoutes)
     */
    returnRoutes?: boolean;
    /**
     * Specify whether traversed edges will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedEdges)
     */
    returnTraversedEdges?: boolean;
    /**
     * Specify whether traversed junctions will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedJunctions)
     */
    returnTraversedJunctions?: boolean;
    /**
     * Specify whether traversed turns will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnTraversedTurns)
     */
    returnTraversedTurns?: boolean;
    /**
     * Include z values for the returned geometries if supported by the underlying network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * Specify the time and date to depart from or arrive at incidents or facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay)
     */
    timeOfDay?: DateProperties | "now";
    /**
     * Specify the time zone or zones of the [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDayIsUTC)
     */
    timeOfDayIsUTC?: boolean;
    /**
     * The [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDay) parameter value represents the arrival or departure time for the routes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#timeOfDayUsage)
     */
    timeOfDayUsage?: "start" | "end" | "not-used";
    /**
     * Specifies how the travel direction for the closest facility search will be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#travelDirection)
     */
    travelDirection?: "from-facility" | "to-facility";
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#travelMode)
     */
    travelMode?: TravelModeProperties;
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#useHierarchy)
     */
    useHierarchy?: boolean;
  }

  /**
   * An object describing the parameter values for the [attributeParameterValues](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#attributeParameterValues) property of [ClosestFacilityParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#AttributeParameterValue)
   */
  export interface AttributeParameterValue {
    /**
     * The name of the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#AttributeParameterValue)
     */
    attributeName: string;
    /**
     * The name of the parameter associated with the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#AttributeParameterValue)
     */
    parameterName: string;
    /**
     * The value for `parameterName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#AttributeParameterValue)
     */
    value: string | number;
  }

  export interface ClosestFacilitySolveResult extends Accessor, JSONSupport {}

  export class ClosestFacilitySolveResult {
    /**
     * The result from [closestFacility](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html)
     */

    constructor(properties?: ClosestFacilitySolveResultProperties);

    /**
     * Direction lines contains a set of line features for each segment of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directionLines)
     */
    directionLines: FeatureSet;
    /**
     * Direction points contains a set of point features representing the direction maneuvers such as arriving to or departing from a stop, turning left or right, and other events along your route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directionPoints)
     */
    directionPoints: FeatureSet;
    /**
     * Directions are returned if [ClosestFacilityParameters.returnDirections](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnDirections) is set to `true` and [ClosestFacilityParameters.directionsOutputType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsOutputType) is set to `complete`, `complete-no-events`, `instructions-only', `standard` or `summary-only` in a [closestFacility.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html#solve) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directions)
     */
    directions: DirectionsFeatureSet[];
    /**
     * This provides access to the output facilities from a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#facilities)
     */
    facilities: FeatureSet;
    /**
     * This provides access to the locations used as starting or ending points in a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#incidents)
     */
    incidents: FeatureSet;
    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#messages)
     */
    messages: NAMessage[];
    /**
     * A set of features representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#pointBarriers)
     */
    pointBarriers: FeatureSet;
    /**
     * A set of features representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#polygonBarriers)
     */
    polygonBarriers: FeatureSet;
    /**
     * A set of features representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#polylineBarriers)
     */
    polylineBarriers: FeatureSet;
    /**
     * A set of features representing routes between the facilities and the incidents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#routes)
     */
    routes: FeatureSet;
    /**
     * This provides access to the edges that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedEdges)
     */
    traversedEdges: FeatureSet;
    /**
     * This provides access to the junctions that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedJunctions)
     */
    traversedJunctions: FeatureSet;
    /**
     * This provides access to the turns that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedTurns)
     */
    traversedTurns: FeatureSet;

    static fromJSON(json: any): ClosestFacilitySolveResult;
  }

  interface ClosestFacilitySolveResultProperties {
    /**
     * Direction lines contains a set of line features for each segment of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directionLines)
     */
    directionLines?: FeatureSetProperties;
    /**
     * Direction points contains a set of point features representing the direction maneuvers such as arriving to or departing from a stop, turning left or right, and other events along your route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directionPoints)
     */
    directionPoints?: FeatureSetProperties;
    /**
     * Directions are returned if [ClosestFacilityParameters.returnDirections](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#returnDirections) is set to `true` and [ClosestFacilityParameters.directionsOutputType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilityParameters.html#directionsOutputType) is set to `complete`, `complete-no-events`, `instructions-only', `standard` or `summary-only` in a [closestFacility.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-closestFacility.html#solve) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#directions)
     */
    directions?: DirectionsFeatureSetProperties[];
    /**
     * This provides access to the output facilities from a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#facilities)
     */
    facilities?: FeatureSetProperties;
    /**
     * This provides access to the locations used as starting or ending points in a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#incidents)
     */
    incidents?: FeatureSetProperties;
    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#messages)
     */
    messages?: NAMessageProperties[];
    /**
     * A set of features representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#pointBarriers)
     */
    pointBarriers?: FeatureSetProperties;
    /**
     * A set of features representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#polygonBarriers)
     */
    polygonBarriers?: FeatureSetProperties;
    /**
     * A set of features representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#polylineBarriers)
     */
    polylineBarriers?: FeatureSetProperties;
    /**
     * A set of features representing routes between the facilities and the incidents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#routes)
     */
    routes?: FeatureSetProperties;
    /**
     * This provides access to the edges that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedEdges)
     */
    traversedEdges?: FeatureSetProperties;
    /**
     * This provides access to the junctions that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedJunctions)
     */
    traversedJunctions?: FeatureSetProperties;
    /**
     * This provides access to the turns that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ClosestFacilitySolveResult.html#traversedTurns)
     */
    traversedTurns?: FeatureSetProperties;
  }

  export interface ColorRamp extends Accessor, JSONSupport {}

  export class ColorRamp {
    /**
     * A ColorRamp object is used to specify a range of colors that are applied to a group of symbols or pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ColorRamp.html)
     */

    constructor(properties?: ColorRampProperties);

    /**
     * A string value representing the color ramp type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ColorRamp.html#type)
     */
    readonly type: "algorithmic" | "multipart";

    static fromJSON(json: any): ColorRamp;
  }

  interface ColorRampProperties {}

  export interface DataFile extends Accessor, JSONSupport {}

  export class DataFile {
    /**
     * A geoprocessing data object containing a data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataFile.html)
     */

    constructor(properties?: DataFileProperties);

    /**
     * The ID of the uploaded file returned as a result of the upload operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataFile.html#itemId)
     */
    itemId: string;
    /**
     * URL to the location of the data file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataFile.html#url)
     */
    url: string;

    static fromJSON(json: any): DataFile;
  }

  interface DataFileProperties {
    /**
     * The ID of the uploaded file returned as a result of the upload operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataFile.html#itemId)
     */
    itemId?: string;
    /**
     * URL to the location of the data file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataFile.html#url)
     */
    url?: string;
  }

  export interface DataLayer extends Accessor, JSONSupport {}

  export class DataLayer {
    /**
     * Input for properties of ClosestFacilityParameters, RouteParameters or ServiceAreaParameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html)
     */

    constructor(properties?: DataLayerProperties);

    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements: boolean;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometry)
     */
    geometry: Geometry;
    /**
     * The type of geometry specified by the [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometry) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometryType)
     */
    geometryType: "point" | "polyline" | "polygon" | "envelope" | "multipoint";
    /**
     * The name of the data layer in the map service that is being referenced.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#name)
     */
    name: string;
    /**
     * The spatial relationship to be applied on the input geometry while performing the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#spatialRelationship)
     */
    spatialRelationship:
      | "intersects"
      | "contains"
      | "crosses"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";

    type: "layer";
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#where)
     */
    where: string;

    static fromJSON(json: any): DataLayer;
  }

  interface DataLayerProperties {
    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements?: boolean;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * The type of geometry specified by the [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometry) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#geometryType)
     */
    geometryType?: "point" | "polyline" | "polygon" | "envelope" | "multipoint";
    /**
     * The name of the data layer in the map service that is being referenced.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#name)
     */
    name?: string;
    /**
     * The spatial relationship to be applied on the input geometry while performing the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#spatialRelationship)
     */
    spatialRelationship?:
      | "intersects"
      | "contains"
      | "crosses"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";

    type?: "layer";
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DataLayer.html#where)
     */
    where?: string;
  }

  export class DensifyParameters extends Accessor {
    /**
     * Input parameters for the densify() method on the GeometryService.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html)
     */

    constructor(properties?: DensifyParametersProperties);

    /**
     * If `true`, Geographic Coordinate System spatial references are used or densify geodesic will be performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#geodesic)
     */
    geodesic: boolean;
    /**
     * The array of geometries to be densified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * The length unit of `maxSegmentLength`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#lengthUnit)
     */
    lengthUnit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * All segments longer than `maxSegmentLength` are replaced with sequences of lines no longer than `maxSegmentLength.`
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#maxSegmentLength)
     */
    maxSegmentLength: number;

    /**
     * Converts an instance of  this class to its ArcGIS portal JSON representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#toJSON)
     */
    toJSON(): any;
  }

  interface DensifyParametersProperties {
    /**
     * If `true`, Geographic Coordinate System spatial references are used or densify geodesic will be performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#geodesic)
     */
    geodesic?: boolean;
    /**
     * The array of geometries to be densified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * The length unit of `maxSegmentLength`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#lengthUnit)
     */
    lengthUnit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * All segments longer than `maxSegmentLength` are replaced with sequences of lines no longer than `maxSegmentLength.`
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DensifyParameters.html#maxSegmentLength)
     */
    maxSegmentLength?: number;
  }

  export interface DirectionLine extends Accessor, JSONSupport {}

  export class DirectionLine {
    /**
     * The DirectionLine represents polylines associated with individual direction items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html)
     */

    constructor(properties?: DirectionLineProperties);

    /**
     * The type of line which is defined by esriDirectionLineType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#directionLineType)
     */
    directionLineType:
      | "unknown"
      | "segment"
      | "maneuver-segment"
      | "restriction-violation"
      | "scaled-cost-barrier"
      | "heavy-traffic"
      | "slow-traffic"
      | "moderate-traffic";
    /**
     * Length of the line measured in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#distance)
     */
    distance: number;
    /**
     * Time of the line measured in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#duration)
     */
    duration: number;
    /**
     * Polyline representing the direction's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#geometry)
     */
    geometry: Polyline;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;

    readonly type: "direction-line";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [DirectionLine](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [DirectionLine](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): DirectionLine;
    static fromJSON(json: any): DirectionLine;
  }

  interface DirectionLineProperties {
    /**
     * The type of line which is defined by esriDirectionLineType.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#directionLineType)
     */
    directionLineType?:
      | "unknown"
      | "segment"
      | "maneuver-segment"
      | "restriction-violation"
      | "scaled-cost-barrier"
      | "heavy-traffic"
      | "slow-traffic"
      | "moderate-traffic";
    /**
     * Length of the line measured in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#distance)
     */
    distance?: number;
    /**
     * Time of the line measured in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#duration)
     */
    duration?: number;
    /**
     * Polyline representing the direction's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#geometry)
     */
    geometry?: PolylineProperties;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionLine.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
  }

  export interface DirectionPoint extends Accessor, JSONSupport {}

  export class DirectionPoint {
    /**
     * The DirectionPoint represents direction items as points with various display information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html)
     */

    constructor(properties?: DirectionPointProperties);

    /**
     * Time when the action happens.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#arrivalTime)
     */
    arrivalTime: Date;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#arrivalTimeOffset)
     */
    arrivalTimeOffset: number;
    /**
     * The type of directions event or maneuver described by the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#directionPointType)
     */
    directionPointType:
      | "header"
      | "arrive"
      | "depart"
      | "straight"
      | "on-ferry"
      | "off-ferry"
      | "central-fork"
      | "roundabout"
      | "u-turn"
      | "door"
      | "stairs"
      | "elevator"
      | "escalator"
      | "pedestrian-ramp"
      | "left-fork"
      | "left-ramp"
      | "clockwise-roundabout"
      | "left-handed-u-turn"
      | "bear-left"
      | "left-turn"
      | "sharp-left"
      | "left-turn-and-immediate-left-turn"
      | "left-turn-and-immediate-right-turn"
      | "right-fork"
      | "right-ramp"
      | "counter-clockwise-roundabout"
      | "right-handed-u-turn"
      | "bear-right"
      | "right-turn"
      | "sharp-right"
      | "right-turn-and-immediate-left-turn"
      | "right-turn-and-immediate-right-turn"
      | "up-elevator"
      | "up-escalator"
      | "up-stairs"
      | "down-elevator"
      | "down-escalator"
      | "down-stairs"
      | "general-event"
      | "landmark"
      | "lane"
      | "time-zone-change"
      | "traffic-event"
      | "scaled-cost-barrier-event"
      | "boundary-crossing"
      | "restriction-violation";
    /**
     * The direction item text to dispay on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#displayText)
     */
    displayText: string;
    /**
     * Point representing the direction's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#geometry)
     */
    geometry: Point;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Sequence of the Direction items, starting with 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#sequence)
     */
    sequence: number;

    readonly type: "direction-point";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [DirectionPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [DirectionPoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): DirectionPoint;
    static fromJSON(json: any): DirectionPoint;
  }

  interface DirectionPointProperties {
    /**
     * Time when the action happens.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#arrivalTime)
     */
    arrivalTime?: DateProperties;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#arrivalTimeOffset)
     */
    arrivalTimeOffset?: number;
    /**
     * The type of directions event or maneuver described by the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#directionPointType)
     */
    directionPointType?:
      | "header"
      | "arrive"
      | "depart"
      | "straight"
      | "on-ferry"
      | "off-ferry"
      | "central-fork"
      | "roundabout"
      | "u-turn"
      | "door"
      | "stairs"
      | "elevator"
      | "escalator"
      | "pedestrian-ramp"
      | "left-fork"
      | "left-ramp"
      | "clockwise-roundabout"
      | "left-handed-u-turn"
      | "bear-left"
      | "left-turn"
      | "sharp-left"
      | "left-turn-and-immediate-left-turn"
      | "left-turn-and-immediate-right-turn"
      | "right-fork"
      | "right-ramp"
      | "counter-clockwise-roundabout"
      | "right-handed-u-turn"
      | "bear-right"
      | "right-turn"
      | "sharp-right"
      | "right-turn-and-immediate-left-turn"
      | "right-turn-and-immediate-right-turn"
      | "up-elevator"
      | "up-escalator"
      | "up-stairs"
      | "down-elevator"
      | "down-escalator"
      | "down-stairs"
      | "general-event"
      | "landmark"
      | "lane"
      | "time-zone-change"
      | "traffic-event"
      | "scaled-cost-barrier-event"
      | "boundary-crossing"
      | "restriction-violation";
    /**
     * The direction item text to dispay on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#displayText)
     */
    displayText?: string;
    /**
     * Point representing the direction's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#geometry)
     */
    geometry?: PointProperties;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Sequence of the Direction items, starting with 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionPoint.html#sequence)
     */
    sequence?: number;
  }

  export interface DirectionsEvent extends Accessor, JSONSupport {}

  export class DirectionsEvent {
    /**
     * DirectionsEvent represent [events](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeature.html#events) for features in a [DirectionsFeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html)
     */

    constructor(properties?: DirectionsEventProperties);

    /**
     * The date and time value indicating the arrival time at the Directions Event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#arriveTime)
     */
    arriveTime: Date;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#arriveTimeOffset)
     */
    arriveTimeOffset: number;
    /**
     * The point location of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#geometry)
     */
    geometry: Point;
    /**
     * An array of [direction strings](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#strings)
     */
    strings: DirectionsString[];

    static fromJSON(json: any): DirectionsEvent;
  }

  interface DirectionsEventProperties {
    /**
     * The date and time value indicating the arrival time at the Directions Event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#arriveTime)
     */
    arriveTime?: DateProperties;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#arriveTimeOffset)
     */
    arriveTimeOffset?: number;
    /**
     * The point location of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#geometry)
     */
    geometry?: PointProperties;
    /**
     * An array of [direction strings](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsEvent.html#strings)
     */
    strings?: DirectionsString[];
  }

  export class DirectionsFeature {
    /**
     * Name-value pairs of fields and field values associated with the Directions Feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeature.html#attributes)
     */
    attributes: any;
    /**
     * An array of Direction Events.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeature.html#events)
     */
    events: DirectionsEvent[];
    /**
     * The geometry of the Directions Feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeature.html#geometry)
     */
    geometry: Polyline;
    /**
     * An array of [direction strings](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeature.html#strings)
     */
    strings: DirectionsString[];

    constructor(properties?: any);
  }

  export interface DirectionsFeatureSet extends FeatureSet, Accessor {}

  export class DirectionsFeatureSet {
    /**
     * DirectionsFeatureSet is a subclass of [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) that contains street directions for a [solved route](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html)
     */

    constructor(properties?: DirectionsFeatureSetProperties);

    /**
     * The extent of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#extent)
     */
    extent: Extent;
    /**
     * The geometry type of the Directions FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#geometryType)
     */
    geometryType: "polyline";
    /**
     * A single polyline representing the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#mergedGeometry)
     */
    mergedGeometry: Polyline;
    /**
     * The ID of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#routeId)
     */
    routeId: number;
    /**
     * The name of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#routeName)
     */
    routeName: string;
    /**
     * A flattened array of all [direction strings](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#strings)
     */
    strings: DirectionsString[];
    /**
     * The total drive time for the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalDriveTime)
     */
    totalDriveTime: number;
    /**
     * The total length of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalLength)
     */
    totalLength: number;
    /**
     * The total time for the route including wait and service time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalTime)
     */
    totalTime: number;

    static fromJSON(json: any): DirectionsFeatureSet;
  }

  interface DirectionsFeatureSetProperties extends FeatureSetProperties {
    /**
     * The extent of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * The geometry type of the Directions FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#geometryType)
     */
    geometryType?: "polyline";
    /**
     * A single polyline representing the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#mergedGeometry)
     */
    mergedGeometry?: PolylineProperties;
    /**
     * The ID of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#routeId)
     */
    routeId?: number;
    /**
     * The name of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#routeName)
     */
    routeName?: string;
    /**
     * A flattened array of all [direction strings](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#strings)
     */
    strings?: DirectionsString[];
    /**
     * The total drive time for the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalDriveTime)
     */
    totalDriveTime?: number;
    /**
     * The total length of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalLength)
     */
    totalLength?: number;
    /**
     * The total time for the route including wait and service time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#totalTime)
     */
    totalTime?: number;
  }

  /**
   * An event string object with name and type.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString)
   */
  export interface DirectionsString {
    /**
     * The direction text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DirectionsFeatureSet.html#DirectionsString)
     */
    text: string;
    /**
     * The type of event.
     *
     * [Read more...](global.html#type)
     */
    type:
      | "alt-name"
      | "arrive"
      | "branch"
      | "cross-street"
      | "cumulative-length"
      | "depart"
      | "estimated-arrival-time"
      | "exit"
      | "general"
      | "length"
      | "service-time"
      | "street-name"
      | "summary"
      | "time"
      | "time-window"
      | "toward"
      | "violation-time"
      | "wait-time";
  }

  export class DistanceParameters extends Accessor {
    /**
     * Defines the input parameters when calling [geometryService.distance()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#distance).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html)
     */

    constructor(properties?: DistanceParametersProperties);

    /**
     * Specifies the units for measuring distance between [geometry1](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1) and [geometry2](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#distanceUnit)
     */
    distanceUnit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * When `true`, the geodesic distance between [geometry1](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1) and [geometry2](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2) is measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geodesic)
     */
    geodesic: boolean;
    /**
     * The geometry from which the distance is to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1)
     */
    geometry1: Geometry;
    /**
     * The geometry to which the distance is to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2)
     */
    geometry2: Geometry;

    /**
     * Converts an instance of  this class to its ArcGIS portal JSON representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#toJSON)
     */
    toJSON(): any;
  }

  interface DistanceParametersProperties {
    /**
     * Specifies the units for measuring distance between [geometry1](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1) and [geometry2](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#distanceUnit)
     */
    distanceUnit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
    /**
     * When `true`, the geodesic distance between [geometry1](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1) and [geometry2](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2) is measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geodesic)
     */
    geodesic?: boolean;
    /**
     * The geometry from which the distance is to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry1)
     */
    geometry1?: GeometryProperties;
    /**
     * The geometry to which the distance is to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-DistanceParameters.html#geometry2)
     */
    geometry2?: GeometryProperties;
  }

  export interface FeatureSet extends Accessor, JSONSupport {}

  export class FeatureSet {
    /**
     * A collection of features returned from ArcGIS Server or used as input to methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html)
     */

    constructor(properties?: FeatureSetProperties);

    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#displayFieldName)
     */
    displayFieldName: string;
    /**
     * Typically, a layer has a limit on the number of features (i.e., records) returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#exceededTransferLimit)
     */
    exceededTransferLimit: boolean;
    /**
     * The array of graphics returned from a task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#features)
     */
    features: Graphic[];
    /**
     * Information about each field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#fields)
     */
    fields: Field[];
    /**
     * The geometry type of features in the FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "extent" | "mesh";
    /**
     * The [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometry) used to query the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry)
     */
    queryGeometry: Geometry;
    /**
     * When a FeatureSet is used as input to Geoprocessor, the spatial reference is set to the map's spatial reference by default.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#spatialReference)
     */
    spatialReference: SpatialReference;

    static fromJSON(json: any): FeatureSet;
  }

  interface FeatureSetProperties {
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#displayFieldName)
     */
    displayFieldName?: string;
    /**
     * Typically, a layer has a limit on the number of features (i.e., records) returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#exceededTransferLimit)
     */
    exceededTransferLimit?: boolean;
    /**
     * The array of graphics returned from a task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#features)
     */
    features?: GraphicProperties[];
    /**
     * Information about each field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#fields)
     */
    fields?: FieldProperties[];
    /**
     * The geometry type of features in the FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#geometryType)
     */
    geometryType?: "point" | "multipoint" | "polyline" | "polygon" | "extent" | "mesh";
    /**
     * The [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometry) used to query the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry)
     */
    queryGeometry?: GeometryProperties;
    /**
     * When a FeatureSet is used as input to Geoprocessor, the spatial reference is set to the map's spatial reference by default.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
  }

  export class FetchPlaceParameters extends PlacesParameters {
    /**
     * The following properties define the parameters for the [fetchPlace()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#fetchPlace) method pointing to a [url](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html#url) that represents a places service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html)
     */

    constructor(properties?: FetchPlaceParametersProperties);

    /**
     * The Id of the place that you want to fetch additional details.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html#placeId)
     */
    placeId: string;
    /**
     * The array of fields that define the attributes to return for a place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html#requestedFields)
     */
    requestedFields: string[];
  }

  interface FetchPlaceParametersProperties extends PlacesParametersProperties {
    /**
     * The Id of the place that you want to fetch additional details.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html#placeId)
     */
    placeId?: string;
    /**
     * The array of fields that define the attributes to return for a place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FetchPlaceParameters.html#requestedFields)
     */
    requestedFields?: string[];
  }

  export class FindImagesParameters extends JSONSupport {
    /**
     * Input parameter for the [ImageryLayer.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#findImages) or [imageService.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#findImages) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html)
     */

    constructor(properties?: FindImagesParametersProperties);

    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry with `z` value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#fromGeometry)
     */
    fromGeometry: Point;
    /**
     * The maximum image count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#maxCount)
     */
    maxCount: number;
    /**
     * An array of ObjectIDs to filter images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#objectIds)
     */
    objectIds: (number | string)[];
    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry with `z` value that defines the target geometry's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#toGeometry)
     */
    toGeometry: Point;
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#where)
     */
    where: string;

    static fromJSON(json: any): FindImagesParameters;
  }

  interface FindImagesParametersProperties {
    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry with `z` value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#fromGeometry)
     */
    fromGeometry?: PointProperties;
    /**
     * The maximum image count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#maxCount)
     */
    maxCount?: number;
    /**
     * An array of ObjectIDs to filter images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#objectIds)
     */
    objectIds?: (number | string)[];
    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry with `z` value that defines the target geometry's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#toGeometry)
     */
    toGeometry?: PointProperties;
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#where)
     */
    where?: string;
  }

  export class FindImagesResult extends JSONSupport {
    /**
     * Results for the [ImageryLayer.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#findImages) or [imageService.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#findImages) methods containing images that meet the search requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesResult.html)
     */

    constructor(properties?: FindImagesResultProperties);

    /**
     * An array of image inspection information found between [fromGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#fromGeometry) and [toGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#toGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesResult.html#images)
     */
    images: ImageInspectionInfo[];

    static fromJSON(json: any): FindImagesResult;
  }

  interface FindImagesResultProperties {
    /**
     * An array of image inspection information found between [fromGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#fromGeometry) and [toGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesParameters.html#toGeometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindImagesResult.html#images)
     */
    images?: ImageInspectionInfoProperties[];
  }

  export interface FindParameters extends Accessor, JSONSupport {}

  export class FindParameters {
    /**
     * Input parameters for [find](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-find.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html)
     */

    constructor(properties?: FindParametersProperties);

    /**
     * Determines whether to look for an exact match of the search text or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#contains)
     */
    contains: boolean;
    /**
     * Specify the geodatabase version to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * Specify the number of decimal places for the geometries returned by the task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * The layers to perform the find operation on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#layerIds)
     */
    layerIds: number[];
    /**
     * The maximum allowable offset used for generalizing geometries returned by the find operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#maxAllowableOffset)
     */
    maxAllowableOffset: number;
    /**
     * The spatial reference of the output geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * If `true`, the output will include the geometry associated with each result.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * The names of the fields of a layer to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchFields)
     */
    searchFields: string[];
    /**
     * The text that is searched across the layers and the fields as specified in the `layers` and [searchFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchFields) properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchText)
     */
    searchText: string;

    static fromJSON(json: any): FindParameters;
  }

  interface FindParametersProperties {
    /**
     * Determines whether to look for an exact match of the search text or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#contains)
     */
    contains?: boolean;
    /**
     * Specify the geodatabase version to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * Specify the number of decimal places for the geometries returned by the task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * The layers to perform the find operation on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#layerIds)
     */
    layerIds?: number[];
    /**
     * The maximum allowable offset used for generalizing geometries returned by the find operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#maxAllowableOffset)
     */
    maxAllowableOffset?: number;
    /**
     * The spatial reference of the output geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * If `true`, the output will include the geometry associated with each result.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * The names of the fields of a layer to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchFields)
     */
    searchFields?: string[];
    /**
     * The text that is searched across the layers and the fields as specified in the `layers` and [searchFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchFields) properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindParameters.html#searchText)
     */
    searchText?: string;
  }

  export interface FindResult extends Accessor, JSONSupport {}

  export class FindResult {
    /**
     * The result from [find](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-find.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html)
     */

    constructor(properties?: FindResultProperties);

    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#displayFieldName)
     */
    displayFieldName: string;
    /**
     * The found feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#feature)
     */
    feature: Graphic;
    /**
     * The name of the field that contains the search text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#foundFieldName)
     */
    foundFieldName: string;
    /**
     * Unique ID of the layer that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#layerId)
     */
    layerId: number;
    /**
     * The layer name that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#layerName)
     */
    layerName: string;
    /**
     * The value of the `foundFieldName` in the feature's attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#value)
     */
    value: string | number;

    static fromJSON(json: any): FindResult;
  }

  interface FindResultProperties {
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#displayFieldName)
     */
    displayFieldName?: string;
    /**
     * The found feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#feature)
     */
    feature?: GraphicProperties;
    /**
     * The name of the field that contains the search text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#foundFieldName)
     */
    foundFieldName?: string;
    /**
     * Unique ID of the layer that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#layerId)
     */
    layerId?: number;
    /**
     * The layer name that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#layerName)
     */
    layerName?: string;
    /**
     * The value of the `foundFieldName` in the feature's attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FindResult.html#value)
     */
    value?: string | number;
  }

  export interface GeneralizeParameters extends Accessor, JSONSupport {}

  export class GeneralizeParameters {
    /**
     * Sets the geometries, maximum deviation and units for the [generalize](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#generalize) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html)
     */

    constructor(properties?: GeneralizeParametersProperties);

    /**
     * The maximum deviation unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#deviationUnit)
     */
    deviationUnit:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "decimal-degrees";
    /**
     * The array of input geometries to generalize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * The maximum deviation for constructing a generalized geometry based on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#maxDeviation)
     */
    maxDeviation: number;

    static fromJSON(json: any): GeneralizeParameters;
  }

  interface GeneralizeParametersProperties {
    /**
     * The maximum deviation unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#deviationUnit)
     */
    deviationUnit?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "decimal-degrees";
    /**
     * The array of input geometries to generalize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * The maximum deviation for constructing a generalized geometry based on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GeneralizeParameters.html#maxDeviation)
     */
    maxDeviation?: number;
  }

  export interface GPMessage extends Accessor, JSONSupport {}

  export class GPMessage {
    /**
     * Represents a message generated during the execution of a module:esri/rest/Geoprocessor method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GPMessage.html)
     */

    constructor(properties?: GPMessageProperties);

    /**
     * The geoprocessing message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GPMessage.html#description)
     */
    description: string;
    /**
     * The geoprocessing message type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GPMessage.html#type)
     */
    type:
      | "informative"
      | "process-definition"
      | "process-start"
      | "process-stop"
      | "warning"
      | "error"
      | "empty"
      | "abort";

    static fromJSON(json: any): GPMessage;
  }

  interface GPMessageProperties {
    /**
     * The geoprocessing message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GPMessage.html#description)
     */
    description?: string;
    /**
     * The geoprocessing message type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-GPMessage.html#type)
     */
    type?:
      | "informative"
      | "process-definition"
      | "process-start"
      | "process-stop"
      | "warning"
      | "error"
      | "empty"
      | "abort";
  }

  export interface IdentifyParameters extends Accessor, JSONSupport {}

  export class IdentifyParameters {
    /**
     * Input parameters for the [identify](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html)
     */

    constructor(properties?: IdentifyParametersProperties);

    /**
     * Resolution of the current map view in dots per inch.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#dpi)
     */
    dpi: number;
    /**
     * Specify the geodatabase version to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The geometry used to select features during the Identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#geometry)
     */
    geometry: Geometry;
    /**
     * Specify the number of decimal places for the geometries returned by the task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * Height of the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) in pixels.
     *
     * @default 400
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#height)
     */
    height: number;
    /**
     * The historic moment to identify.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#historicMoment)
     */
    historicMoment: Date;
    /**
     * The layers on which to perform the identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#layerIds)
     */
    layerIds: number[];
    /**
     * Specifies which layers to use when using Identify.
     *
     * @default "top"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#layerOption)
     */
    layerOption: "top" | "visible" | "all" | "popup";
    /**
     * The Extent or bounding box of the current map view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#mapExtent)
     */
    mapExtent: Extent;
    /**
     * The maximum allowable offset used for generalizing geometries returned by the identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#maxAllowableOffset)
     */
    maxAllowableOffset: number;
    /**
     * If `true`, field names will be returned instead of field aliases.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnFieldName)
     */
    returnFieldName: boolean;
    /**
     * If `true`, the result set includes the geometry associated with each result.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * When `true`, indicates that M values will be returned.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnM)
     */
    returnM: boolean;
    /**
     * If `true`, the values in the result will not be formatted i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnUnformattedValues)
     */
    returnUnformattedValues: boolean;
    /**
     * When `true`, indicates that z-values will be returned.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnZ)
     */
    returnZ: boolean;
    /**
     * The spatial reference of the input and output geometries as well as of the [mapExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#mapExtent).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * An [Collection](esri-core-Collection.html) of [Sublayer](esri-layers-support-Sublayer.html) objects used to change the layer ordering and/or rendering, or redefine the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#sublayers)
     */
    sublayers: Collection<Sublayer>;
    /**
     * Specify the time extent used by identify.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The distance in screen pixels from the specified geometry within which the identify should be performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#tolerance)
     */
    tolerance: number;
    /**
     * Width of the current map view in pixels.
     *
     * @default 400
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#width)
     */
    width: number;

    static fromJSON(json: any): IdentifyParameters;
  }

  interface IdentifyParametersProperties {
    /**
     * Resolution of the current map view in dots per inch.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#dpi)
     */
    dpi?: number;
    /**
     * Specify the geodatabase version to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The geometry used to select features during the Identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * Specify the number of decimal places for the geometries returned by the task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * Height of the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) in pixels.
     *
     * @default 400
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#height)
     */
    height?: number;
    /**
     * The historic moment to identify.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#historicMoment)
     */
    historicMoment?: DateProperties;
    /**
     * The layers on which to perform the identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#layerIds)
     */
    layerIds?: number[];
    /**
     * Specifies which layers to use when using Identify.
     *
     * @default "top"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#layerOption)
     */
    layerOption?: "top" | "visible" | "all" | "popup";
    /**
     * The Extent or bounding box of the current map view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#mapExtent)
     */
    mapExtent?: ExtentProperties;
    /**
     * The maximum allowable offset used for generalizing geometries returned by the identify operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#maxAllowableOffset)
     */
    maxAllowableOffset?: number;
    /**
     * If `true`, field names will be returned instead of field aliases.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnFieldName)
     */
    returnFieldName?: boolean;
    /**
     * If `true`, the result set includes the geometry associated with each result.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * When `true`, indicates that M values will be returned.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnM)
     */
    returnM?: boolean;
    /**
     * If `true`, the values in the result will not be formatted i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnUnformattedValues)
     */
    returnUnformattedValues?: boolean;
    /**
     * When `true`, indicates that z-values will be returned.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * The spatial reference of the input and output geometries as well as of the [mapExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#mapExtent).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * An [Collection](esri-core-Collection.html) of [Sublayer](esri-layers-support-Sublayer.html) objects used to change the layer ordering and/or rendering, or redefine the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#sublayers)
     */
    sublayers?: CollectionProperties<SublayerProperties>;
    /**
     * Specify the time extent used by identify.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The distance in screen pixels from the specified geometry within which the identify should be performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#tolerance)
     */
    tolerance?: number;
    /**
     * Width of the current map view in pixels.
     *
     * @default 400
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyParameters.html#width)
     */
    width?: number;
  }

  export interface IdentifyResult extends Accessor, JSONSupport {}

  export class IdentifyResult {
    /**
     * The result from [identify](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-identify.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html)
     */

    constructor(properties?: IdentifyResultProperties);

    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#displayFieldName)
     */
    displayFieldName: string;
    /**
     * An identified feature from the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#feature)
     */
    feature: Graphic;
    /**
     * Unique ID of the layer that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#layerId)
     */
    layerId: number;
    /**
     * The layer name that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#layerName)
     */
    layerName: string;

    static fromJSON(json: any): IdentifyResult;
  }

  interface IdentifyResultProperties {
    /**
     * The name of the layer's primary display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#displayFieldName)
     */
    displayFieldName?: string;
    /**
     * An identified feature from the map service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#feature)
     */
    feature?: GraphicProperties;
    /**
     * Unique ID of the layer that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#layerId)
     */
    layerId?: number;
    /**
     * The layer name that contains the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-IdentifyResult.html#layerName)
     */
    layerName?: string;
  }

  export interface ImageAngleParameters extends Accessor, JSONSupport {}

  export class ImageAngleParameters {
    /**
     * Input parameters for [ImageryLayer.computeAngles()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeAngles) or [imageService.computeAngles()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computeAngles) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html)
     */

    constructor(properties?: ImageAngleParametersProperties);

    /**
     * Angle names to be computed.
     *
     * @default ["north","up"]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#angleNames)
     */
    angleNames: string[];
    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry that defines the reference point of rotation to compute the angle direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#point)
     */
    point: Point;
    /**
     * The rasterId of a raster catalog in the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#rasterId)
     */
    rasterId: number;
    /**
     * The spatial reference used to compute the angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#spatialReference)
     */
    spatialReference: SpatialReference;

    static fromJSON(json: any): ImageAngleParameters;
  }

  interface ImageAngleParametersProperties {
    /**
     * Angle names to be computed.
     *
     * @default ["north","up"]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#angleNames)
     */
    angleNames?: string[];
    /**
     * A [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry that defines the reference point of rotation to compute the angle direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#point)
     */
    point?: PointProperties;
    /**
     * The rasterId of a raster catalog in the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#rasterId)
     */
    rasterId?: number;
    /**
     * The spatial reference used to compute the angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleParameters.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
  }

  export interface ImageAngleResult extends Accessor, JSONSupport {}

  export class ImageAngleResult {
    /**
     * The results from [imageService.computeAngles()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computeAngles) or [ImageryLayer.computeAngles()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeAngles) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html)
     */

    constructor(properties?: ImageAngleResultProperties);

    /**
     * The computed north angle after rotating the map so the top of the image is oriented toward north.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#north)
     */
    north: number;
    /**
     * The spatial reference used to compute the angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The computed up angle after rotating the map so the top of the image is always oriented to the direction of the sensor when it acquired the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#up)
     */
    up: number;

    static fromJSON(json: any): ImageAngleResult;
  }

  interface ImageAngleResultProperties {
    /**
     * The computed north angle after rotating the map so the top of the image is oriented toward north.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#north)
     */
    north?: number;
    /**
     * The spatial reference used to compute the angles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The computed up angle after rotating the map so the top of the image is always oriented to the direction of the sensor when it acquired the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAngleResult.html#up)
     */
    up?: number;
  }

  export interface ImageAreaParameters extends Accessor, JSONSupport, BaseImageMeasureParameters {}

  export class ImageAreaParameters {
    /**
     * Input parameters used by  the [ImageryLayer.measureAreaAndPerimeter()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureAreaAndPerimeter) and [imageService.measureAreaAndPerimeter()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaAndPerimeter) methods to perform imagery area and perimeter mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html)
     */

    constructor(properties?: ImageAreaParametersProperties);

    /**
     * The area unit used for an area calculation.
     *
     * @default "square-meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#areaUnit)
     */
    areaUnit:
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * The extent or polygon geometry used to perform the area and perimeter measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#geometry)
     */
    geometry: Extent | Polygon;
    /**
     * When `true`, this method calculates the 3D measurements for the area and perimeter of a given geometry on an image service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#is3D)
     */
    is3D: boolean;
    /**
     * Linear unit used for a perimeter calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#linearUnit)
     */
    linearUnit: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * The string value representing the type of imagery mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#type)
     */
    readonly type: "area-perimeter";

    static fromJSON(json: any): ImageAreaParameters;
  }

  interface ImageAreaParametersProperties extends BaseImageMeasureParametersProperties {
    /**
     * The area unit used for an area calculation.
     *
     * @default "square-meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#areaUnit)
     */
    areaUnit?:
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * The extent or polygon geometry used to perform the area and perimeter measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#geometry)
     */
    geometry?: (ExtentProperties & { type: "extent" }) | (PolygonProperties & { type: "polygon" });
    /**
     * When `true`, this method calculates the 3D measurements for the area and perimeter of a given geometry on an image service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#is3D)
     */
    is3D?: boolean;
    /**
     * Linear unit used for a perimeter calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaParameters.html#linearUnit)
     */
    linearUnit?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
  }

  export interface ImageAreaResult extends Accessor, JSONSupport, BaseImageMeasureResult {}

  export class ImageAreaResult {
    /**
     * Image service area and perimeter measurement result returned when the [ImageryLayer.measureAreaAndPerimeter()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureAreaAndPerimeter) or [imageService.measureAreaAndPerimeter()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaAndPerimeter) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaResult.html)
     */

    constructor(properties?: ImageAreaResultProperties);

    /**
     * An object containing results of the area measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaResult.html#area)
     */
    area: MeasurementValue;
    /**
     * An object containing results of the perimeter measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaResult.html#perimeter)
     */
    perimeter: MeasurementValue;

    static fromJSON(json: any): ImageAreaResult;
  }

  interface ImageAreaResultProperties extends BaseImageMeasureResultProperties {
    /**
     * An object containing results of the area measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaResult.html#area)
     */
    area?: MeasurementValue;
    /**
     * An object containing results of the perimeter measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageAreaResult.html#perimeter)
     */
    perimeter?: MeasurementValue;
  }

  export interface ImageBoundaryParameters extends Accessor, JSONSupport {}

  export class ImageBoundaryParameters {
    /**
     * Image boundary query parameters for [ImageryLayer.queryBoundary()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryBoundary) or [imageService.queryBoundary()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryBoundary) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryParameters.html)
     */

    constructor(properties?: ImageBoundaryParametersProperties);

    /**
     * The spatial reference for the output boundary geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;

    static fromJSON(json: any): ImageBoundaryParameters;
  }

  interface ImageBoundaryParametersProperties {
    /**
     * The spatial reference for the output boundary geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
  }

  export interface ImageBoundaryResult extends Accessor, JSONSupport {}

  export class ImageBoundaryResult {
    /**
     * Image boundary query result for [ImageryLayer.queryBoundary()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryBoundary) or [imageService.queryBoundary()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryBoundary) methods containing boundary geometry of an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryResult.html)
     */

    constructor(properties?: ImageBoundaryResultProperties);

    /**
     * The area of the boundary geometry in square meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryResult.html#area)
     */
    area: number;
    /**
     * The geometry that defines the boundary of the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryResult.html#geometry)
     */
    geometry: Polygon | Extent;

    static fromJSON(json: any): ImageBoundaryResult;
  }

  interface ImageBoundaryResultProperties {
    /**
     * The area of the boundary geometry in square meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryResult.html#area)
     */
    area?: number;
    /**
     * The geometry that defines the boundary of the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageBoundaryResult.html#geometry)
     */
    geometry?: (PolygonProperties & { type: "polygon" }) | (ExtentProperties & { type: "extent" });
  }

  export interface ImageDistanceParameters extends Accessor, JSONSupport, BaseImageMeasureParameters {}

  export class ImageDistanceParameters {
    /**
     * Input parameters used by the [ImageryLayer.measureDistanceAndAngle()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureDistanceAndAngle) or [imageService.measureDistanceAndAngle()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureDistanceAndAngle) methods to perform imagery distance and angle mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html)
     */

    constructor(properties?: ImageDistanceParametersProperties);

    /**
     * The angular unit used for angle calculation.
     *
     * @default "degrees"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#angularUnit)
     */
    angularUnit: "degrees" | "radians";
    /**
     * A point that defines the from location of the distance and angle measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#fromGeometry)
     */
    fromGeometry: Point;
    /**
     * When `true`, this method calculates the 3D measurements for the distance and angle between two points on an image service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#is3D)
     */
    is3D: boolean;
    /**
     * The linear unit used for distance calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#linearUnit)
     */
    linearUnit: LengthUnit;
    /**
     * A point that defines the to location of the distance and angle measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#toGeometry)
     */
    toGeometry: Point;
    /**
     * The string value representing the type of imagery mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#type)
     */
    readonly type: "distance-angle";

    static fromJSON(json: any): ImageDistanceParameters;
  }

  interface ImageDistanceParametersProperties extends BaseImageMeasureParametersProperties {
    /**
     * The angular unit used for angle calculation.
     *
     * @default "degrees"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#angularUnit)
     */
    angularUnit?: "degrees" | "radians";
    /**
     * A point that defines the from location of the distance and angle measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#fromGeometry)
     */
    fromGeometry?: PointProperties;
    /**
     * When `true`, this method calculates the 3D measurements for the distance and angle between two points on an image service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#is3D)
     */
    is3D?: boolean;
    /**
     * The linear unit used for distance calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#linearUnit)
     */
    linearUnit?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * A point that defines the to location of the distance and angle measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceParameters.html#toGeometry)
     */
    toGeometry?: PointProperties;
  }

  export interface ImageDistanceResult extends Accessor, JSONSupport, BaseImageMeasureResult {}

  export class ImageDistanceResult {
    /**
     * Image service distance and angle measurement result returned when the [ImageryLayer.measureDistanceAndAngle()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureDistanceAndAngle) or [imageService.measureDistanceAndAngle()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureDistanceAndAngle) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html)
     */

    constructor(properties?: ImageDistanceResultProperties);

    /**
     * An object containing the azimuth angle measurement values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#azimuthAngle)
     */
    azimuthAngle: MeasurementValue;
    /**
     * An object containing results of the distance measurement between two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#distance)
     */
    distance: MeasurementValue;
    /**
     * An object containing elevation angle measurement values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#elevationAngle)
     */
    elevationAngle: MeasurementValue;

    static fromJSON(json: any): ImageDistanceResult;
  }

  interface ImageDistanceResultProperties extends BaseImageMeasureResultProperties {
    /**
     * An object containing the azimuth angle measurement values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#azimuthAngle)
     */
    azimuthAngle?: MeasurementValue;
    /**
     * An object containing results of the distance measurement between two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#distance)
     */
    distance?: MeasurementValue;
    /**
     * An object containing elevation angle measurement values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageDistanceResult.html#elevationAngle)
     */
    elevationAngle?: MeasurementValue;
  }

  export class ImageGPSInfo extends JSONSupport {
    /**
     * Image GPS information for images returned as a result of the [ImageryLayer.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryGPSInfo) or [imageService.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryGPSInfo) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html)
     */

    constructor(properties?: ImageGPSInfoProperties);

    /**
     * Image acquisition date represented using Linux Epoch time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#acquisitionDate)
     */
    acquisitionDate: Date;
    /**
     * Image's camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#cameraID)
     */
    cameraID: string;
    /**
     * Image's center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#center)
     */
    center: Point;
    /**
     * Image's GPS location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#gps)
     */
    gps: any;
    /**
     * Image id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#id)
     */
    id: number;
    /**
     * Image name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#name)
     */
    name: string;
    /**
     * Image's orientation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#orientation)
     */
    orientation: any;

    static fromJSON(json: any): ImageGPSInfo;
  }

  interface ImageGPSInfoProperties {
    /**
     * Image acquisition date represented using Linux Epoch time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#acquisitionDate)
     */
    acquisitionDate?: DateProperties;
    /**
     * Image's camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#cameraID)
     */
    cameraID?: string;
    /**
     * Image's center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#center)
     */
    center?: PointProperties;
    /**
     * Image's GPS location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#gps)
     */
    gps?: any;
    /**
     * Image id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#id)
     */
    id?: number;
    /**
     * Image name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#name)
     */
    name?: string;
    /**
     * Image's orientation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfo.html#orientation)
     */
    orientation?: any;
  }

  export class ImageGPSInfoParameters extends JSONSupport {
    /**
     * Image GPS info parameters for [ImageryLayer.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryGPSInfo) or [imageService.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryGPSInfo) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html)
     */

    constructor(properties?: ImageGPSInfoParametersProperties);

    /**
     * The geometry to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#geometry)
     */
    geometry: Extent | Multipoint | Point | Polygon | Polyline;
    /**
     * An array of ObjectIDs to be used to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#objectIds)
     */
    objectIds: (number | string)[];
    /**
     * For spatial queries, this parameter defines the spatial relationship to query image footprints in the layer against the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#geometry).
     *
     * @default "intersects"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#spatialRelationship)
     */
    spatialRelationship:
      | "intersects"
      | "contains"
      | "crosses"
      | "disjoint"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";
    /**
     * A time extent for a temporal query to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#where)
     */
    where: string;

    static fromJSON(json: any): ImageGPSInfoParameters;
  }

  interface ImageGPSInfoParametersProperties {
    /**
     * The geometry to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#geometry)
     */
    geometry?:
      | (ExtentProperties & { type: "extent" })
      | (MultipointProperties & { type: "multipoint" })
      | (PointProperties & { type: "point" })
      | (PolygonProperties & { type: "polygon" })
      | (PolylineProperties & { type: "polyline" });
    /**
     * An array of ObjectIDs to be used to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#objectIds)
     */
    objectIds?: (number | string)[];
    /**
     * For spatial queries, this parameter defines the spatial relationship to query image footprints in the layer against the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#geometry).
     *
     * @default "intersects"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#spatialRelationship)
     */
    spatialRelationship?:
      | "intersects"
      | "contains"
      | "crosses"
      | "disjoint"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";
    /**
     * A time extent for a temporal query to query images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoParameters.html#where)
     */
    where?: string;
  }

  export class ImageGPSInfoResult extends JSONSupport {
    /**
     * Result for [ImageryLayer.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#queryGPSInfo) or [imageService.queryGPSInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#queryGPSInfo) methods containing camera and image GPS information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoResult.html)
     */

    constructor(properties?: ImageGPSInfoResultProperties);

    /**
     * An array of camera information for the queried images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoResult.html#cameras)
     */
    cameras: CameraInfo[];
    /**
     * An array of GPS information for the queried images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoResult.html#images)
     */
    images: ImageGPSInfo[];

    static fromJSON(json: any): ImageGPSInfoResult;
  }

  interface ImageGPSInfoResultProperties {
    /**
     * An array of camera information for the queried images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoResult.html#cameras)
     */
    cameras?: CameraInfoProperties[];
    /**
     * An array of GPS information for the queried images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageGPSInfoResult.html#images)
     */
    images?: ImageGPSInfoProperties[];
  }

  export interface ImageHeightParameters extends Accessor, JSONSupport, BaseImageMeasureParameters {}

  export class ImageHeightParameters {
    /**
     * Input parameters used by the [ImageryLayer.measureHeight()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureHeight) or [imageService.measureHeight()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureHeight) methods to perform imagery height mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html)
     */

    constructor(properties?: ImageHeightParametersProperties);

    /**
     * A point that defines the from location of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#fromGeometry)
     */
    fromGeometry: Point;
    /**
     * Linear unit used for height calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#linearUnit)
     */
    linearUnit: LengthUnit;
    /**
     * Determines how the height will be measured when the sensor info is available.
     *
     * @default "base-and-top"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#operationType)
     */
    operationType: "base-and-top" | "base-and-top-shadow" | "top-and-top-shadow";
    /**
     * A point that defines the to location of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#toGeometry)
     */
    toGeometry: Point;
    /**
     * The string value representing the type of imagery mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#type)
     */
    readonly type: "height";

    static fromJSON(json: any): ImageHeightParameters;
  }

  interface ImageHeightParametersProperties extends BaseImageMeasureParametersProperties {
    /**
     * A point that defines the from location of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#fromGeometry)
     */
    fromGeometry?: PointProperties;
    /**
     * Linear unit used for height calculation.
     *
     * @default "meters"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#linearUnit)
     */
    linearUnit?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * Determines how the height will be measured when the sensor info is available.
     *
     * @default "base-and-top"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#operationType)
     */
    operationType?: "base-and-top" | "base-and-top-shadow" | "top-and-top-shadow";
    /**
     * A point that defines the to location of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightParameters.html#toGeometry)
     */
    toGeometry?: PointProperties;
  }

  export interface ImageHeightResult extends Accessor, JSONSupport, BaseImageMeasureResult {}

  export class ImageHeightResult {
    /**
     * Image service height mensuration result returned when the [ImageryLayer.measureHeight()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureHeight) or [imageService.measureHeight()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureHeight) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightResult.html)
     */

    constructor(properties?: ImageHeightResultProperties);

    /**
     * An object containing results of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightResult.html#height)
     */
    height: MeasurementValue;

    static fromJSON(json: any): ImageHeightResult;
  }

  interface ImageHeightResultProperties extends BaseImageMeasureResultProperties {
    /**
     * An object containing results of the height measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHeightResult.html#height)
     */
    height?: MeasurementValue;
  }

  export interface ImageHistogramParameters extends Accessor, JSONSupport {}

  export class ImageHistogramParameters {
    /**
     * Input parameters for the [computeHistograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeHistograms) or [computeStatisticsHistograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computeStatisticsHistograms) method on [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), or the [computeStatisticsHistograms](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#computeStatisticsHistograms) method on [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html)
     */

    constructor(properties?: ImageHistogramParametersProperties);

    /**
     * Input geometry that defines the area of interest for which the histograms and statistics will be computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#geometry)
     */
    geometry: Extent | Polygon;
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) on how individual images should be mosaicked when the histogram is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#mosaicRule)
     */
    mosaicRule: MosaicRule;
    /**
     * Specifies the pixel size (or the resolution).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#pixelSize)
     */
    pixelSize: Point;
    /**
     * Specifies the [raster function](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html) from which to compute the statistics and histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#rasterFunction)
     */
    rasterFunction: RasterFunction;
    /**
     * The [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) for which to compute the statistics and histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#timeExtent)
     */
    timeExtent: TimeExtent;

    static fromJSON(json: any): ImageHistogramParameters;
  }

  interface ImageHistogramParametersProperties {
    /**
     * Input geometry that defines the area of interest for which the histograms and statistics will be computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#geometry)
     */
    geometry?: (ExtentProperties & { type: "extent" }) | (PolygonProperties & { type: "polygon" });
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) on how individual images should be mosaicked when the histogram is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#mosaicRule)
     */
    mosaicRule?: MosaicRuleProperties;
    /**
     * Specifies the pixel size (or the resolution).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#pixelSize)
     */
    pixelSize?: PointProperties;
    /**
     * Specifies the [raster function](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RasterFunction.html) from which to compute the statistics and histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#rasterFunction)
     */
    rasterFunction?: RasterFunctionProperties;
    /**
     * The [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) for which to compute the statistics and histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageHistogramParameters.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
  }

  export interface ImageIdentifyParameters extends Accessor, JSONSupport {}

  export class ImageIdentifyParameters {
    /**
     * Input parameters for [imageService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html)
     */

    constructor(properties?: ImageIdentifyParametersProperties);

    /**
     * Input geometry that defines the location to be identified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#geometry)
     */
    geometry: Point | Polygon | Extent;
    /**
     * Controls the maximum number of returned catalog items, set to 1 to return the top most raster only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#maxItemCount)
     */
    maxItemCount: number;
    /**
     * Specifies the mosaic rules defining the image sorting order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#mosaicRule)
     */
    mosaicRule: MosaicRule;
    /**
     * Specifies the pixel level being identified on the x and y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#pixelSize)
     */
    pixelSize: Point;
    /**
     * When `true`, the request is processed for all variables and dimensions.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#processAsMultidimensional)
     */
    processAsMultidimensional: boolean;
    /**
     * Specifies the raster function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunction)
     */
    rasterFunction: RasterFunction;
    /**
     * An array the [raster functions](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunction) to retrieve multiple processed pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunctions)
     */
    rasterFunctions: RasterFunction[];
    /**
     * If `true`, returns both geometry and attributes of the catalog items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnCatalogItems)
     */
    returnCatalogItems: boolean;
    /**
     * When `true`, each feature in the catalog items includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * If `true`, the pixel values of all raster catalog items under the requested geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnPixelValues)
     */
    returnPixelValues: boolean;
    /**
     * A time extent for a temporal data against [time-aware imagery layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#timeExtent)
     */
    timeExtent: TimeExtent;

    static fromJSON(json: any): ImageIdentifyParameters;
  }

  interface ImageIdentifyParametersProperties {
    /**
     * Input geometry that defines the location to be identified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#geometry)
     */
    geometry?:
      | (PointProperties & { type: "point" })
      | (PolygonProperties & { type: "polygon" })
      | (ExtentProperties & { type: "extent" });
    /**
     * Controls the maximum number of returned catalog items, set to 1 to return the top most raster only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#maxItemCount)
     */
    maxItemCount?: number;
    /**
     * Specifies the mosaic rules defining the image sorting order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#mosaicRule)
     */
    mosaicRule?: MosaicRuleProperties;
    /**
     * Specifies the pixel level being identified on the x and y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#pixelSize)
     */
    pixelSize?: PointProperties;
    /**
     * When `true`, the request is processed for all variables and dimensions.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#processAsMultidimensional)
     */
    processAsMultidimensional?: boolean;
    /**
     * Specifies the raster function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunction)
     */
    rasterFunction?: RasterFunctionProperties;
    /**
     * An array the [raster functions](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunction) to retrieve multiple processed pixel values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#rasterFunctions)
     */
    rasterFunctions?: RasterFunctionProperties[];
    /**
     * If `true`, returns both geometry and attributes of the catalog items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnCatalogItems)
     */
    returnCatalogItems?: boolean;
    /**
     * When `true`, each feature in the catalog items includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * If `true`, the pixel values of all raster catalog items under the requested geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#returnPixelValues)
     */
    returnPixelValues?: boolean;
    /**
     * A time extent for a temporal data against [time-aware imagery layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
  }

  /**
   * Specifies the pixel size (or the resolution).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
   */
  export interface PixelSize {
    /**
     * Represents the size of one pixel in map units along the x axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
     */
    x: number;
    /**
     * Represents the size of one pixel in map units along the y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
     */
    y: number;
    /**
     * Spatial reference to be used for the request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
     */
    spatialReference?: PixelSizeSpatialReference;
  }

  export interface PixelSizeSpatialReference {
    /**
     * The wkid of the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
     */
    wkid?: number;
    /**
     * The Well known text or wkt of the spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyParameters.html#PixelSize)
     */
    wkt?: number;
  }

  export interface ImageIdentifyResult extends Accessor, JSONSupport {}

  export class ImageIdentifyResult {
    /**
     * The results from [imageService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html)
     */

    constructor(properties?: ImageIdentifyResultProperties);

    /**
     * The set of catalog items that overlap the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#catalogItems)
     */
    catalogItems: FeatureSet;
    /**
     * The set of visible areas for the identified catalog items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#catalogItemVisibilities)
     */
    catalogItemVisibilities: number[];
    /**
     * The identified location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#location)
     */
    location: Point;
    /**
     * The identify property name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#name)
     */
    name: string;
    /**
     * The identify property id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#objectId)
     */
    objectId: number;
    /**
     * The attributes of the identified object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#properties)
     */
    properties: any;
    /**
     * The identify image service pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#value)
     */
    value: string;

    static fromJSON(json: any): ImageIdentifyResult;
  }

  interface ImageIdentifyResultProperties {
    /**
     * The set of catalog items that overlap the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#catalogItems)
     */
    catalogItems?: FeatureSetProperties;
    /**
     * The set of visible areas for the identified catalog items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#catalogItemVisibilities)
     */
    catalogItemVisibilities?: number[];
    /**
     * The identified location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#location)
     */
    location?: PointProperties;
    /**
     * The identify property name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#name)
     */
    name?: string;
    /**
     * The identify property id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#objectId)
     */
    objectId?: number;
    /**
     * The attributes of the identified object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#properties)
     */
    properties?: any;
    /**
     * The identify image service pixel value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageIdentifyResult.html#value)
     */
    value?: string;
  }

  export class ImageInspectionInfo extends JSONSupport {
    /**
     * Image information for images returned as a result of running [ImageryLayer.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#findImages) or [imageService.findImages()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#findImages) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html)
     */

    constructor(properties?: ImageInspectionInfoProperties);

    /**
     * Image acquisition date represented in Linux Epoch time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#acquisitionDate)
     */
    acquisitionDate: Date;
    /**
     * Image's camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#cameraID)
     */
    cameraID: string;
    /**
     * Image's center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#center)
     */
    center: Point;
    /**
     * Camera's columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#cols)
     */
    cols: number;
    /**
     * Camera's focal length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#focalLength)
     */
    focalLength: number;
    /**
     * Image id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#id)
     */
    id: number;
    /**
     * Camera's manufacturer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#make)
     */
    make: string;
    /**
     * Camera's model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#model)
     */
    model: string;
    /**
     * Image's orientation along x, y, z axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    orientation: ImageInspectionInfoOrientation;
    /**
     * Perspective center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#perspectiveCenter)
     */
    perspectiveCenter: Point;
    /**
     * Camera's pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#pixelSize)
     */
    pixelSize: number;
    /**
     * The relative reference to the image's uri.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#referenceUri)
     */
    referenceUri: string;
    /**
     * Camera's rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#rows)
     */
    rows: number;

    static fromJSON(json: any): ImageInspectionInfo;
  }

  interface ImageInspectionInfoProperties {
    /**
     * Image acquisition date represented in Linux Epoch time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#acquisitionDate)
     */
    acquisitionDate?: DateProperties;
    /**
     * Image's camera id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#cameraID)
     */
    cameraID?: string;
    /**
     * Image's center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#center)
     */
    center?: PointProperties;
    /**
     * Camera's columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#cols)
     */
    cols?: number;
    /**
     * Camera's focal length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#focalLength)
     */
    focalLength?: number;
    /**
     * Image id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#id)
     */
    id?: number;
    /**
     * Camera's manufacturer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#make)
     */
    make?: string;
    /**
     * Camera's model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#model)
     */
    model?: string;
    /**
     * Image's orientation along x, y, z axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    orientation?: ImageInspectionInfoOrientation;
    /**
     * Perspective center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#perspectiveCenter)
     */
    perspectiveCenter?: PointProperties;
    /**
     * Camera's pixel size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#pixelSize)
     */
    pixelSize?: number;
    /**
     * The relative reference to the image's uri.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#referenceUri)
     */
    referenceUri?: string;
    /**
     * Camera's rows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#rows)
     */
    rows?: number;
  }

  export interface ImageInspectionInfoOrientation {
    /**
     * Image rotation along the x axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    omega: number;
    /**
     * Image rotation along the y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    phi: number;
    /**
     * Image rotation along the z axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    kappa: number;
    /**
     * Indicates if the rotation is clockwise.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageInspectionInfo.html#orientation)
     */
    clockwise: boolean;
  }

  export class ImageParameters extends Accessor {
    /**
     * Represents the image parameter options used when calling [JobInfo.fetchResultImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultImage) and [JobInfo.fetchResultMapImageLayer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultMapImageLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html)
     */

    constructor(properties?: ImageParametersProperties);

    /**
     * Dots per inch setting for [JobInfo.fetchResultMapImageLayer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultMapImageLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#dpi)
     */
    dpi: number;
    /**
     * Extent of map to be exported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#extent)
     */
    extent: Extent;
    /**
     * Map image format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#format)
     */
    format: "png" | "png8" | "png24" | "png32" | "jpg" | "pdf" | "bmp" | "gif" | "svg";
    /**
     * Requested image height in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#height)
     */
    height: number;
    /**
     * Spatial reference of exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#imageSpatialReference)
     */
    imageSpatialReference: SpatialReference;
    /**
     * Array of layer definition expressions that allows you to filter the features of individual layers in the exported map image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerDefinitions)
     */
    layerDefinitions: string[];
    /**
     * A list of layer IDs, that represent which layers to include in the exported map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerIds)
     */
    layerIds: number[];
    /**
     * This property determines how the layers specified by [layerIds](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerIds) are treated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerOption)
     */
    layerOption: "show" | "hide" | "include" | "exclude";
    /**
     * Indicates whether or not the background of the dynamic image is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#transparent)
     */
    transparent: boolean;
    /**
     * Requested image width in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#width)
     */
    width: number;

    /**
     * Converts an instance of this class to its ArcGIS portal JSON representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#toJSON)
     */
    toJSON(): any;
  }

  interface ImageParametersProperties {
    /**
     * Dots per inch setting for [JobInfo.fetchResultMapImageLayer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultMapImageLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#dpi)
     */
    dpi?: number;
    /**
     * Extent of map to be exported.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Map image format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#format)
     */
    format?: "png" | "png8" | "png24" | "png32" | "jpg" | "pdf" | "bmp" | "gif" | "svg";
    /**
     * Requested image height in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#height)
     */
    height?: number;
    /**
     * Spatial reference of exported image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#imageSpatialReference)
     */
    imageSpatialReference?: SpatialReferenceProperties;
    /**
     * Array of layer definition expressions that allows you to filter the features of individual layers in the exported map image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerDefinitions)
     */
    layerDefinitions?: string[];
    /**
     * A list of layer IDs, that represent which layers to include in the exported map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerIds)
     */
    layerIds?: number[];
    /**
     * This property determines how the layers specified by [layerIds](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerIds) are treated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#layerOption)
     */
    layerOption?: "show" | "hide" | "include" | "exclude";
    /**
     * Indicates whether or not the background of the dynamic image is transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#transparent)
     */
    transparent?: boolean;
    /**
     * Requested image width in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageParameters.html#width)
     */
    width?: number;
  }

  export interface ImagePixelLocationParameters extends Accessor, JSONSupport {}

  export class ImagePixelLocationParameters {
    /**
     * Input parameters for the [ImageryLayer.computePixelSpaceLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computePixelSpaceLocations) or [imageService.computePixelSpaceLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computePixelSpaceLocations) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationParameters.html)
     */

    constructor(properties?: ImagePixelLocationParametersProperties);

    /**
     * An array of [points](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) that defines pixel locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationParameters.html#geometries)
     */
    geometries: Point[];
    /**
     * The rasterId of a raster catalog in the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationParameters.html#rasterId)
     */
    rasterId: number;

    static fromJSON(json: any): ImagePixelLocationParameters;
  }

  interface ImagePixelLocationParametersProperties {
    /**
     * An array of [points](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) that defines pixel locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationParameters.html#geometries)
     */
    geometries?: PointProperties[];
    /**
     * The rasterId of a raster catalog in the image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationParameters.html#rasterId)
     */
    rasterId?: number;
  }

  export interface ImagePixelLocationResult extends Accessor, JSONSupport {}

  export class ImagePixelLocationResult {
    /**
     * The results from [imageService.computePixelSpaceLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#computePixelSpaceLocations) or [ImageryLayer.computePixelSpaceLocations()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#computePixelSpaceLocations) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationResult.html)
     */

    constructor(properties?: ImagePixelLocationResultProperties);

    /**
     * An array of objects containing pixel space x,y coordinates and the pixel's z values of the pixel location geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationResult.html#geometries)
     */
    geometries: any[];

    static fromJSON(json: any): ImagePixelLocationResult;
  }

  interface ImagePixelLocationResultProperties {
    /**
     * An array of objects containing pixel space x,y coordinates and the pixel's z values of the pixel location geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePixelLocationResult.html#geometries)
     */
    geometries?: any[];
  }

  export interface ImagePointParameters extends Accessor, JSONSupport, BaseImageMeasureParameters {}

  export class ImagePointParameters {
    /**
     * Input parameters used by the [ImageryLayer.measurePointOrCentroid()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurePointOrCentroid) or [imageService.measurePointOrCentroid()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measurePointOrCentroid) methods to perform imagery point or centroid mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html)
     */

    constructor(properties?: ImagePointParametersProperties);

    /**
     * Input geometry to determine a a point location or a centroid of a given area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html#geometry)
     */
    geometry: Point | Extent | Polygon;
    /**
     * When `true`, this method calculates the z-value for the returned point geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html#is3D)
     */
    is3D: boolean;
    /**
     * The string value representing the type of imagery mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html#type)
     */
    readonly type: "point";

    static fromJSON(json: any): ImagePointParameters;
  }

  interface ImagePointParametersProperties extends BaseImageMeasureParametersProperties {
    /**
     * Input geometry to determine a a point location or a centroid of a given area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html#geometry)
     */
    geometry?:
      | (PointProperties & { type: "point" })
      | (ExtentProperties & { type: "extent" })
      | (PolygonProperties & { type: "polygon" });
    /**
     * When `true`, this method calculates the z-value for the returned point geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointParameters.html#is3D)
     */
    is3D?: boolean;
  }

  export interface ImagePointResult extends Accessor, JSONSupport, BaseImageMeasureResult {}

  export class ImagePointResult {
    /**
     * Image service point or centroid measurement result returned when the [ImageryLayer.measurePointOrCentroid()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measurePointOrCentroid) or [imageService.measurePointOrCentroid()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measurePointOrCentroid) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointResult.html)
     */

    constructor(properties?: ImagePointResultProperties);

    /**
     * The measured point on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointResult.html#point)
     */
    point: Point;

    static fromJSON(json: any): ImagePointResult;
  }

  interface ImagePointResultProperties extends BaseImageMeasureResultProperties {
    /**
     * The measured point on an image service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImagePointResult.html#point)
     */
    point?: PointProperties;
  }

  export interface ImageSample extends Accessor, JSONSupport {}

  export class ImageSample {
    /**
     * The [getSamples](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getSamples) method on [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) returns [ImageSampleResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleResult.html) containing array of this class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html)
     */

    constructor(properties?: ImageSampleProperties);

    /**
     * Name-value pairs of fields and field values associated with the [sample location](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#location).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#attributes)
     */
    attributes: any;
    /**
     * The sample location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#location)
     */
    location: Point;
    /**
     * The location id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#locationId)
     */
    locationId: number;
    /**
     * The pixel value associated with the sampled location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#pixelValue)
     */
    pixelValue: number[];
    /**
     * The raster id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#rasterId)
     */
    rasterId: number;
    /**
     * The resolution representing the average of source raster's resolutions in x and y axes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#resolution)
     */
    resolution: number;

    static fromJSON(json: any): ImageSample;
  }

  interface ImageSampleProperties {
    /**
     * Name-value pairs of fields and field values associated with the [sample location](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#location).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#attributes)
     */
    attributes?: any;
    /**
     * The sample location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#location)
     */
    location?: PointProperties;
    /**
     * The location id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#locationId)
     */
    locationId?: number;
    /**
     * The pixel value associated with the sampled location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#pixelValue)
     */
    pixelValue?: number[];
    /**
     * The raster id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#rasterId)
     */
    rasterId?: number;
    /**
     * The resolution representing the average of source raster's resolutions in x and y axes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html#resolution)
     */
    resolution?: number;
  }

  export interface ImageSampleParameters extends Accessor, JSONSupport {}

  export class ImageSampleParameters {
    /**
     * Input parameters for the [getSamples()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getSamples) method on [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html)
     */

    constructor(properties?: ImageSampleParametersProperties);

    /**
     * Input geometry that defines the locations to be sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry)
     */
    geometry: Point | Multipoint | Polyline | Extent | Polygon;
    /**
     * Defines how to interpolate pixel values.
     *
     * @default "nearest"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#interpolation)
     */
    interpolation: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * When sampling multiple locations, you can use an array of [points](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instead of providing a [multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) for the [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#locations)
     */
    locations: Point[];
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) defining the image sort order and selection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#mosaicRule)
     */
    mosaicRule: MosaicRule;
    /**
     * The list of fields associated with the rasters to be included in the response.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#outFields)
     */
    outFields: string[];
    /**
     * Specifies the pixel size (or the resolution) that will be used for the sampling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#pixelSize)
     */
    pixelSize: Point;
    /**
     * When `true`, returns the first valid pixel value that meets specified conditions at each sampling point location.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#returnFirstValueOnly)
     */
    returnFirstValueOnly: boolean;
    /**
     * Specifies the approximate number of locations to sample from the provided geometry when the input geometry is [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html), [polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) or [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sampleCount)
     */
    sampleCount: number;
    /**
     * Specifies the distance interval to sample points from the provided [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry) when input geometry is [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sampleDistance)
     */
    sampleDistance: number;
    /**
     * Specifies the slice id of a multidimensional raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sliceId)
     */
    sliceId: number;
    /**
     * The [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) for which to perform sampling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#timeExtent)
     */
    timeExtent: TimeExtent;

    static fromJSON(json: any): ImageSampleParameters;
  }

  interface ImageSampleParametersProperties {
    /**
     * Input geometry that defines the locations to be sampled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry)
     */
    geometry?:
      | (PointProperties & { type: "point" })
      | (MultipointProperties & { type: "multipoint" })
      | (PolylineProperties & { type: "polyline" })
      | (ExtentProperties & { type: "extent" })
      | (PolygonProperties & { type: "polygon" });
    /**
     * Defines how to interpolate pixel values.
     *
     * @default "nearest"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#interpolation)
     */
    interpolation?: "nearest" | "bilinear" | "cubic" | "majority";
    /**
     * When sampling multiple locations, you can use an array of [points](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instead of providing a [multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) for the [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#locations)
     */
    locations?: PointProperties[];
    /**
     * Specifies the [mosaic rule](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-MosaicRule.html) defining the image sort order and selection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#mosaicRule)
     */
    mosaicRule?: MosaicRuleProperties;
    /**
     * The list of fields associated with the rasters to be included in the response.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#outFields)
     */
    outFields?: string[];
    /**
     * Specifies the pixel size (or the resolution) that will be used for the sampling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#pixelSize)
     */
    pixelSize?: PointProperties;
    /**
     * When `true`, returns the first valid pixel value that meets specified conditions at each sampling point location.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#returnFirstValueOnly)
     */
    returnFirstValueOnly?: boolean;
    /**
     * Specifies the approximate number of locations to sample from the provided geometry when the input geometry is [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html), [polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) or [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sampleCount)
     */
    sampleCount?: number;
    /**
     * Specifies the distance interval to sample points from the provided [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#geometry) when input geometry is [polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sampleDistance)
     */
    sampleDistance?: number;
    /**
     * Specifies the slice id of a multidimensional raster.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#sliceId)
     */
    sliceId?: number;
    /**
     * The [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) for which to perform sampling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleParameters.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
  }

  export interface ImageSampleResult extends Accessor, JSONSupport {}

  export class ImageSampleResult {
    /**
     * The result from the [getSamples](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getSamples) method on [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) containing array of [ImageSample](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSample.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleResult.html)
     */

    constructor(properties?: ImageSampleResultProperties);

    /**
     * An array of image sample results returned in response to [ImageryLayer.getSamples()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getSamples) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleResult.html#samples)
     */
    samples: ImageSample[];

    static fromJSON(json: any): ImageSampleResult;
  }

  interface ImageSampleResultProperties {
    /**
     * An array of image sample results returned in response to [ImageryLayer.getSamples()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getSamples) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageSampleResult.html#samples)
     */
    samples?: ImageSampleProperties[];
  }

  export class ImageToMapMultirayParameters extends JSONSupport {
    /**
     * Input parameters for the [ImageryLayer.imageToMapMultiray()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#imageToMapMultiray) or [imageService.imageToMapMultiray()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#imageToMapMultiray) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html)
     */

    constructor(properties?: ImageToMapMultirayParametersProperties);

    /**
     * An array of geometries in image space coordinates representing the same map location on different images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * The spatial reference for the output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * The rasterIds of the raster items that correspond to the input [geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#geometries).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#rasterIds)
     */
    rasterIds: number[];

    static fromJSON(json: any): ImageToMapMultirayParameters;
  }

  interface ImageToMapMultirayParametersProperties {
    /**
     * An array of geometries in image space coordinates representing the same map location on different images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * The spatial reference for the output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * The rasterIds of the raster items that correspond to the input [geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#geometries).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapMultirayParameters.html#rasterIds)
     */
    rasterIds?: number[];
  }

  export class ImageToMapParameters extends JSONSupport {
    /**
     * Input parameters for the [ImageryLayer.imageToMap()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#imageToMap) or [imageService.imageToMap()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#imageToMap) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html)
     */

    constructor(properties?: ImageToMapParametersProperties);

    /**
     * When `true`, it will adjust the background vertices to be in the foreground.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#adjust)
     */
    adjust: boolean;
    /**
     * The depth offset.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#depthOffset)
     */
    depthOffset: number;
    /**
     * The geometry in an image space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#geometry)
     */
    geometry: Multipoint | Point | Polygon | Polyline;
    /**
     * The spatial reference for the output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * The corresponding raster id for the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#geometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#rasterId)
     */
    rasterId: number;

    static fromJSON(json: any): ImageToMapParameters;
  }

  interface ImageToMapParametersProperties {
    /**
     * When `true`, it will adjust the background vertices to be in the foreground.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#adjust)
     */
    adjust?: boolean;
    /**
     * The depth offset.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#depthOffset)
     */
    depthOffset?: number;
    /**
     * The geometry in an image space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#geometry)
     */
    geometry?:
      | (MultipointProperties & { type: "multipoint" })
      | (PointProperties & { type: "point" })
      | (PolygonProperties & { type: "polygon" })
      | (PolylineProperties & { type: "polyline" });
    /**
     * The spatial reference for the output geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * The corresponding raster id for the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#geometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageToMapParameters.html#rasterId)
     */
    rasterId?: number;
  }

  export class ImageUrlParameters extends JSONSupport {
    /**
     * Input parameters for the [ImageryLayer.getImageUrl()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getImageUrl) or [imageService.getImageUrl()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#getImageUrl) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html)
     */

    constructor(properties?: ImageUrlParametersProperties);

    /**
     * The raster's object id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html#rasterId)
     */
    rasterId: number;
    /**
     * The relative reference to the image's uri.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html#referenceUri)
     */
    referenceUri: string;

    static fromJSON(json: any): ImageUrlParameters;
  }

  interface ImageUrlParametersProperties {
    /**
     * The raster's object id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html#rasterId)
     */
    rasterId?: number;
    /**
     * The relative reference to the image's uri.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlParameters.html#referenceUri)
     */
    referenceUri?: string;
  }

  export interface ImageUrlResult extends Accessor, JSONSupport {}

  export class ImageUrlResult {
    /**
     * The result returned when the [ImageryLayer.getImageUrl()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#getImageUrl) or [imageService.getImageUrl()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#getImageUrl) method resolves successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlResult.html)
     */

    constructor(properties?: ImageUrlResultProperties);

    /**
     * Image's url
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlResult.html#url)
     */
    url: string;

    static fromJSON(json: any): ImageUrlResult;
  }

  interface ImageUrlResultProperties {
    /**
     * Image's url
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ImageUrlResult.html#url)
     */
    url?: string;
  }

  export interface JobInfo extends Accessor, JSONSupport {}

  export class JobInfo {
    /**
     * Represents information pertaining to the execution of an asynchronous [geoprocessor](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor.html) request on the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html)
     */

    constructor(properties?: JobInfoProperties);

    /**
     * The unique job ID assigned by geoprocessing server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#jobId)
     */
    jobId: string;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#jobStatus)
     */
    jobStatus:
      | "job-cancelled"
      | "job-cancelling"
      | "job-deleted"
      | "job-deleting"
      | "job-timed-out"
      | "job-executing"
      | "job-failed"
      | "job-new"
      | "job-submitted"
      | "job-succeeded"
      | "job-waiting";
    /**
     * An array of messages that include a type and description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#messages)
     */
    messages: GPMessage[];
    /**
     * Displays the progress of the geoprocessing job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#progress)
     */
    readonly progress: JobInfoProgress;
    /**
     * The [options](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#esriRequest) to be used for data requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#requestOptions)
     */
    requestOptions: any;
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the geoprocessing request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#sourceUrl)
     */
    sourceUrl: string;

    /**
     * Cancels an asynchronous geoprocessing job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#cancelJob)
     */
    cancelJob(requestOptions?: any): Promise<JobInfo>;
    /**
     * Sends a request for the current state of this job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#checkJobStatus)
     */
    checkJobStatus(requestOptions?: any): Promise<JobInfo>;
    /**
     * Stop monitoring this job for status updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#destroy)
     */
    destroy(): void;
    /**
     * Sends a request to the GP Task to get the task result identified by `resultName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultData)
     */
    fetchResultData(resultName: string, gpOptions?: GPOptions, requestOptions?: any): Promise<ParameterValue>;
    /**
     * Sends a request to the GP Task to get the task result identified by `jobId` and `resultName` as an image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultImage)
     */
    fetchResultImage(resultName: string, imageParams: ImageParameters, requestOptions?: any): Promise<ParameterValue>;
    /**
     * Get the task result identified by `jobId` as an [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#fetchResultMapImageLayer)
     */
    fetchResultMapImageLayer(): Promise<MapImageLayer>;
    /**
     * Resolves when an asynchronous job has completed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#waitForJobCompletion)
     */
    waitForJobCompletion(options?: JobInfoWaitForJobCompletionOptions): Promise<JobInfo>;

    static fromJSON(json: any): JobInfo;
  }

  interface JobInfoProperties {
    /**
     * The unique job ID assigned by geoprocessing server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#jobId)
     */
    jobId?: string;
    /**
     * The job status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#jobStatus)
     */
    jobStatus?:
      | "job-cancelled"
      | "job-cancelling"
      | "job-deleted"
      | "job-deleting"
      | "job-timed-out"
      | "job-executing"
      | "job-failed"
      | "job-new"
      | "job-submitted"
      | "job-succeeded"
      | "job-waiting";
    /**
     * An array of messages that include a type and description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#messages)
     */
    messages?: GPMessageProperties[];
    /**
     * The [options](https://developers.arcgis.com/javascript/latest/api-reference/esri-request.html#esriRequest) to be used for data requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#requestOptions)
     */
    requestOptions?: any;
    /**
     * ArcGIS Server Rest API endpoint to the resource that receives the geoprocessing request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#sourceUrl)
     */
    sourceUrl?: string;
  }

  export interface JobInfoProgress {
    /**
     * Progress message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#progress)
     */
    message: string;
    /**
     * Progress percentage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#progress)
     */
    percent?: number;
  }

  export interface JobInfoWaitForJobCompletionOptions {
    /**
     * The time in millisecond between remote job status requests.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#waitForJobCompletion)
     */
    interval?: any;
    /**
     * [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) allows for cancelable asynchronous job.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#waitForJobCompletion)
     */
    signal?: any;
    /**
     * Callback function that is called at the specified interval.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-JobInfo.html#waitForJobCompletion)
     */
    statusCallback?: any;
  }

  export class LegendLayer extends Accessor {
    /**
     * Define layer properties for the legend layers associated with a [PrintTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html)
     */

    constructor(properties?: LegendLayerProperties);

    /**
     * The id of the operational layer to include in the printout's legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#layerId)
     */
    layerId: string;
    /**
     * The ids of the sublayers to include in the printout's legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#subLayerIds)
     */
    subLayerIds: string[];
    /**
     * The title of the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#title)
     */
    title: string;
  }

  interface LegendLayerProperties {
    /**
     * The id of the operational layer to include in the printout's legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#layerId)
     */
    layerId?: string;
    /**
     * The ids of the sublayers to include in the printout's legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#subLayerIds)
     */
    subLayerIds?: string[];
    /**
     * The title of the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html#title)
     */
    title?: string;
  }

  export interface LengthsParameters extends Accessor, JSONSupport {}

  export class LengthsParameters {
    /**
     * Sets the length units and other parameters for the [geometryService.lengths()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#lengths) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html)
     */

    constructor(properties?: LengthsParametersProperties);

    /**
     * Defines the type of calculation for the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#calculationType)
     */
    calculationType: "planar" | "geodesic" | "preserve-shape";
    /**
     * If polylines are in a geographic coordinate system, then geodesic needs to be set to `true` in order to calculate the ellipsoidal shortest path distance between each pair of the vertices in the polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#geodesic)
     */
    geodesic: boolean;
    /**
     * The length unit in which perimeters of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#lengthUnit)
     */
    lengthUnit: number | string;
    /**
     * The array of polylines whose lengths are to be computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#polylines)
     */
    polylines: Polyline[];

    static fromJSON(json: any): LengthsParameters;
  }

  interface LengthsParametersProperties {
    /**
     * Defines the type of calculation for the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#calculationType)
     */
    calculationType?: "planar" | "geodesic" | "preserve-shape";
    /**
     * If polylines are in a geographic coordinate system, then geodesic needs to be set to `true` in order to calculate the ellipsoidal shortest path distance between each pair of the vertices in the polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#geodesic)
     */
    geodesic?: boolean;
    /**
     * The length unit in which perimeters of polygons will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#lengthUnit)
     */
    lengthUnit?: number | string;
    /**
     * The array of polylines whose lengths are to be computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LengthsParameters.html#polylines)
     */
    polylines?: PolylineProperties[];
  }

  export interface LinearUnit extends Accessor, JSONSupport {}

  export class LinearUnit {
    /**
     * A geoprocessing object that describes a distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LinearUnit.html)
     */

    constructor(properties?: LinearUnitProperties);

    /**
     * Specifies the value of the linear distance.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LinearUnit.html#distance)
     */
    distance: number;
    /**
     * Specifies the unit type of the linear distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LinearUnit.html#units)
     */
    units:
      | "centimeters"
      | "decimal-degrees"
      | "decimeters"
      | "feet"
      | "inches"
      | "international-feet"
      | "international-inches"
      | "international-miles"
      | "international-nautical-miles"
      | "international-yards"
      | "kilometers"
      | "meters"
      | "miles"
      | "millimeters"
      | "nautical-miles"
      | "points"
      | "yards"
      | "unknown"
      | "us-feet";

    static fromJSON(json: any): LinearUnit;
  }

  interface LinearUnitProperties {
    /**
     * Specifies the value of the linear distance.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LinearUnit.html#distance)
     */
    distance?: number;
    /**
     * Specifies the unit type of the linear distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LinearUnit.html#units)
     */
    units?:
      | "centimeters"
      | "decimal-degrees"
      | "decimeters"
      | "feet"
      | "inches"
      | "international-feet"
      | "international-inches"
      | "international-miles"
      | "international-nautical-miles"
      | "international-yards"
      | "kilometers"
      | "meters"
      | "miles"
      | "millimeters"
      | "nautical-miles"
      | "points"
      | "yards"
      | "unknown"
      | "us-feet";
  }

  export class MapToImageParameters extends JSONSupport {
    /**
     * Input parameters for the [ImageryLayer.mapToImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#mapToImage) or [imageService.mapToImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#mapToImage) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html)
     */

    constructor(properties?: MapToImageParametersProperties);

    /**
     * The geometry in a map space (coordinates).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#geometry)
     */
    geometry: Extent | Multipoint | Point | Polygon | Polyline;
    /**
     * The raster id of the raster item corresponding to the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#geometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#rasterId)
     */
    rasterId: number;
    /**
     * When `true`, the operation will return an empty geometry if the input geometry is not visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#visibleOnly)
     */
    visibleOnly: boolean;

    static fromJSON(json: any): MapToImageParameters;
  }

  interface MapToImageParametersProperties {
    /**
     * The geometry in a map space (coordinates).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#geometry)
     */
    geometry?:
      | (ExtentProperties & { type: "extent" })
      | (MultipointProperties & { type: "multipoint" })
      | (PointProperties & { type: "point" })
      | (PolygonProperties & { type: "polygon" })
      | (PolylineProperties & { type: "polyline" });
    /**
     * The raster id of the raster item corresponding to the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#geometry).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#rasterId)
     */
    rasterId?: number;
    /**
     * When `true`, the operation will return an empty geometry if the input geometry is not visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MapToImageParameters.html#visibleOnly)
     */
    visibleOnly?: boolean;
  }

  export interface MeasureAreaFromImageResult extends Accessor, JSONSupport {}

  export class MeasureAreaFromImageResult {
    /**
     * The area and length result from a selected image's measurement in an image space when the [ImageryLayer.measureAreaFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureAreaFromImage) or [imageService.measureAreaFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaFromImage) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html)
     */

    constructor(properties?: MeasureAreaFromImageResultProperties);

    /**
     * The area of a polygon in square meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#area)
     */
    area: number;
    /**
     * The polygon's center in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#center)
     */
    center: Point;
    /**
     * The polygon geometry in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#geometry)
     */
    geometry: Polygon;
    /**
     * The perimeter of a polygon in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#length)
     */
    length: number;

    static fromJSON(json: any): MeasureAreaFromImageResult;
  }

  interface MeasureAreaFromImageResultProperties {
    /**
     * The area of a polygon in square meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#area)
     */
    area?: number;
    /**
     * The polygon's center in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#center)
     */
    center?: PointProperties;
    /**
     * The polygon geometry in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#geometry)
     */
    geometry?: PolygonProperties;
    /**
     * The perimeter of a polygon in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureAreaFromImageResult.html#length)
     */
    length?: number;
  }

  export interface MeasureFromImageParameters extends Accessor, JSONSupport {}

  export class MeasureFromImageParameters {
    /**
     * Input parameters used by the [ImageryLayer.measureLengthFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureLengthFromImage), [ImageryLayer.measureAreaFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureAreaFromImage), [imageService.measureLengthFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureLengthFromImage) and [imageService.measureAreaFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureAreaFromImage) methods to perform imagery area, perimeter and length mensuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureFromImageParameters.html)
     */

    constructor(properties?: MeasureFromImageParametersProperties);

    /**
     * The polyline or polygon geometry used to perform length or area measurements in an image space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureFromImageParameters.html#geometry)
     */
    geometry: Polyline | Polygon;
    /**
     * The id of the raster to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureFromImageParameters.html#rasterId)
     */
    rasterId: number;

    static fromJSON(json: any): MeasureFromImageParameters;
  }

  interface MeasureFromImageParametersProperties {
    /**
     * The polyline or polygon geometry used to perform length or area measurements in an image space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureFromImageParameters.html#geometry)
     */
    geometry?: (PolylineProperties & { type: "polyline" }) | (PolygonProperties & { type: "polygon" });
    /**
     * The id of the raster to be measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureFromImageParameters.html#rasterId)
     */
    rasterId?: number;
  }

  export interface MeasureLengthFromImageResult extends Accessor, JSONSupport {}

  export class MeasureLengthFromImageResult {
    /**
     * The length result from a selected image's measurement in an image space when the [ImageryLayer.measureLengthFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html#measureLengthFromImage) or [imageService.measureLengthFromImage()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-imageService.html#measureLengthFromImage) methods resolve successfully.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureLengthFromImageResult.html)
     */

    constructor(properties?: MeasureLengthFromImageResultProperties);

    /**
     * Polyline geometry in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureLengthFromImageResult.html#geometry)
     */
    geometry: Polyline;
    /**
     * The length of a polyline in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureLengthFromImageResult.html#length)
     */
    length: number;

    static fromJSON(json: any): MeasureLengthFromImageResult;
  }

  interface MeasureLengthFromImageResultProperties {
    /**
     * Polyline geometry in a map space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureLengthFromImageResult.html#geometry)
     */
    geometry?: PolylineProperties;
    /**
     * The length of a polyline in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MeasureLengthFromImageResult.html#length)
     */
    length?: number;
  }

  export class MultipartColorRamp extends ColorRamp {
    /**
     * Creates a multipart color ramp to combine multiple continuous color ramps for use in raster renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MultipartColorRamp.html)
     */

    constructor(properties?: MultipartColorRampProperties);

    /**
     * Define an array of algorithmic color ramps used to generate the multi part ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MultipartColorRamp.html#colorRamps)
     */
    colorRamps: AlgorithmicColorRamp[];
    /**
     * A string value representing the color ramp type.
     *
     * @default "multipart"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MultipartColorRamp.html#type)
     */
    readonly type: "multipart";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MultipartColorRamp.html#clone)
     */
    clone(): MultipartColorRamp;

    static fromJSON(json: any): MultipartColorRamp;
  }

  interface MultipartColorRampProperties extends ColorRampProperties {
    /**
     * Define an array of algorithmic color ramps used to generate the multi part ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-MultipartColorRamp.html#colorRamps)
     */
    colorRamps?: AlgorithmicColorRampProperties[];
  }

  export interface NAMessage extends Accessor, JSONSupport {}

  export class NAMessage {
    /**
     * Represents a message generated during the execution of a network analyst task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html)
     */

    constructor(properties?: NAMessageProperties);

    /**
     * A description of the network analyst message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html#description)
     */
    description: string;
    /**
     * The network analyst message type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html#type)
     */
    type:
      | "informative"
      | "process-definition"
      | "process-start"
      | "process-stop"
      | "warning"
      | "error"
      | "empty"
      | "abort";

    static fromJSON(json: any): NAMessage;
  }

  interface NAMessageProperties {
    /**
     * A description of the network analyst message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html#description)
     */
    description?: string;
    /**
     * The network analyst message type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html#type)
     */
    type?:
      | "informative"
      | "process-definition"
      | "process-start"
      | "process-stop"
      | "warning"
      | "error"
      | "empty"
      | "abort";
  }

  export interface NetworkFeatureSet extends FeatureSet, Accessor {}

  export class NetworkFeatureSet {
    /**
     * A subclass of FeaureSet that can be used as an input in the Route, Closest Facility, and Service Area solvers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkFeatureSet.html)
     */

    constructor(properties?: NetworkFeatureSetProperties);

    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkFeatureSet.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements: boolean;

    static fromJSON(json: any): NetworkFeatureSet;
  }

  interface NetworkFeatureSetProperties extends FeatureSetProperties {
    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkFeatureSet.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements?: boolean;
  }

  export interface NetworkUrl extends Accessor, JSONSupport {}

  export class NetworkUrl {
    /**
     * An input type for Route, Closest Facility, and Service Area solvers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkUrl.html)
     */

    constructor(properties?: NetworkUrlProperties);

    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkUrl.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements: boolean;
    /**
     * A url to any ArcGIS Server feature, map, or geoprocessing service that returns a JSON feature set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkUrl.html#url)
     */
    url: string;

    static fromJSON(json: any): NetworkUrl;
  }

  interface NetworkUrlProperties {
    /**
     * If true, restricted network elements should be considered when finding network locations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkUrl.html#doNotLocateOnRestrictedElements)
     */
    doNotLocateOnRestrictedElements?: boolean;
    /**
     * A url to any ArcGIS Server feature, map, or geoprocessing service that returns a JSON feature set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NetworkUrl.html#url)
     */
    url?: string;
  }

  export interface OffsetParameters extends Accessor, JSONSupport {}

  export class OffsetParameters {
    /**
     * Sets the offset distance, type and other parameters for the [geometryService.offset](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#offset) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html)
     */

    constructor(properties?: OffsetParametersProperties);

    /**
     * The `bevelRatio` is multiplied by the offset distance and the result determines how far a mitered offset intersection can be located before it is beveled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#bevelRatio)
     */
    bevelRatio: number;
    /**
     * The array of geometries to be offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * Specifies the planar distance for constructing an offset based on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetDistance)
     */
    offsetDistance: number;
    /**
     * Options that determine how the ends intersect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetHow)
     */
    offsetHow: "bevelled" | "mitered" | "rounded";
    /**
     * The offset distance unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetUnit)
     */
    offsetUnit: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";

    static fromJSON(json: any): OffsetParameters;
  }

  interface OffsetParametersProperties {
    /**
     * The `bevelRatio` is multiplied by the offset distance and the result determines how far a mitered offset intersection can be located before it is beveled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#bevelRatio)
     */
    bevelRatio?: number;
    /**
     * The array of geometries to be offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * Specifies the planar distance for constructing an offset based on the input geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetDistance)
     */
    offsetDistance?: number;
    /**
     * Options that determine how the ends intersect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetHow)
     */
    offsetHow?: "bevelled" | "mitered" | "rounded";
    /**
     * The offset distance unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-OffsetParameters.html#offsetUnit)
     */
    offsetUnit?: "feet" | "kilometers" | "meters" | "miles" | "nautical-miles" | "yards";
  }

  export interface ParameterValue extends Accessor, JSONSupport {}

  export class ParameterValue {
    /**
     * Represents the output parameters of a [geoprocessor](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geoprocessor.html) method and their properties and values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html)
     */

    constructor(properties?: ParameterValueProperties);

    /**
     * Specifies the parameter's data type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#dataType)
     */
    dataType:
      | "areal-unit"
      | "boolean"
      | "data-file"
      | "date"
      | "double"
      | "feature-record-set-layer"
      | "field"
      | "linear-unit"
      | "long"
      | "raster-data"
      | "raster-data-layer"
      | "record-set"
      | "string"
      | "value-table"
      | "multi-value-areal-unit"
      | "multi-value-boolean"
      | "multi-value-composite"
      | "multi-value-data-file"
      | "multi-value-date"
      | "multi-value-double"
      | "multi-value-feature-record-set-layer"
      | "multi-value-field"
      | "multi-value-linear-unit"
      | "multi-value-long"
      | "multi-value-raster-data"
      | "multi-value-raster-data-layer"
      | "multi-value-record-set"
      | "multi-value-string";
    /**
     * The name of the parameter the value is for.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#paramName)
     */
    paramName: string;
    /**
     * The value of the parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#value)
     */
    value:
      | ArealUnit
      | boolean
      | DataFile
      | Date
      | number
      | FeatureSet
      | Field
      | LinearUnit
      | RasterData
      | string
      | FeatureLayer
      | MapImage
      | (
          | ArealUnit
          | boolean
          | DataFile
          | Date
          | number
          | FeatureSet
          | Field
          | LinearUnit
          | RasterData
          | string
          | FeatureLayer
          | MapImage
        )[];

    static fromJSON(json: any): ParameterValue;
  }

  interface ParameterValueProperties {
    /**
     * Specifies the parameter's data type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#dataType)
     */
    dataType?:
      | "areal-unit"
      | "boolean"
      | "data-file"
      | "date"
      | "double"
      | "feature-record-set-layer"
      | "field"
      | "linear-unit"
      | "long"
      | "raster-data"
      | "raster-data-layer"
      | "record-set"
      | "string"
      | "value-table"
      | "multi-value-areal-unit"
      | "multi-value-boolean"
      | "multi-value-composite"
      | "multi-value-data-file"
      | "multi-value-date"
      | "multi-value-double"
      | "multi-value-feature-record-set-layer"
      | "multi-value-field"
      | "multi-value-linear-unit"
      | "multi-value-long"
      | "multi-value-raster-data"
      | "multi-value-raster-data-layer"
      | "multi-value-record-set"
      | "multi-value-string";
    /**
     * The name of the parameter the value is for.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#paramName)
     */
    paramName?: string;
    /**
     * The value of the parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ParameterValue.html#value)
     */
    value?:
      | ArealUnitProperties
      | boolean
      | DataFileProperties
      | DateProperties
      | number
      | FeatureSetProperties
      | FieldProperties
      | LinearUnitProperties
      | RasterDataProperties
      | string
      | (FeatureLayerProperties & { type: "feature" })
      | MapImageProperties
      | (
          | ArealUnitProperties
          | boolean
          | DataFileProperties
          | DateProperties
          | number
          | FeatureSetProperties
          | FieldProperties
          | LinearUnitProperties
          | RasterDataProperties
          | string
          | (FeatureLayerProperties & { type: "feature" })
          | MapImageProperties
        )[];
  }

  export class PlaceResult extends Accessor {
    /**
     * The `PlaceResult` object includes a single place that satisfies the search and either: the distance from the [search point](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#point) in meters, or falls within the [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#extent) of the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html)
     */

    constructor(properties?: PlaceResultProperties);

    /**
     * An array of category objects for a place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#categories)
     */
    categories: Category[];
    /**
     * The distance, in meters, from the place to the search point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#distance)
     */
    distance: number;
    /**
     * Url for an icon for this place in either cim, png or svg format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#icon)
     */
    icon: any;
    /**
     * A location defined by X and Y coordinates in WGS84.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#location)
     */
    location: Point;
    /**
     * The name of the place, or point of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#name)
     */
    name: string;
    /**
     * The unique Id of this place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#placeId)
     */
    placeId: string;
  }

  interface PlaceResultProperties {
    /**
     * An array of category objects for a place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#categories)
     */
    categories?: Category[];
    /**
     * The distance, in meters, from the place to the search point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#distance)
     */
    distance?: number;
    /**
     * Url for an icon for this place in either cim, png or svg format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#icon)
     */
    icon?: any;
    /**
     * A location defined by X and Y coordinates in WGS84.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#location)
     */
    location?: PointProperties;
    /**
     * The name of the place, or point of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#name)
     */
    name?: string;
    /**
     * The unique Id of this place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#placeId)
     */
    placeId?: string;
  }

  /**
   * Each category object has a categoryId and a label.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#Category)
   */
  export interface Category {
    /**
     * The Id of the category of the place.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#Category)
     */
    categoryId: number;
    /**
     * The name of the the category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlaceResult.html#Category)
     */
    label: string;
  }

  export class PlacesParameters extends Accessor {
    /**
     * The following properties define common properties for use with [places](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html)
     */

    constructor(properties?: PlacesParametersProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#apiKey)
     */
    apiKey: string;
    /**
     * Determines whether icons are returned and the type of icons returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#icon)
     */
    icon: "cim" | "png" | "svg";
    /**
     * URL to the places service.
     *
     * @default "https://places-api.arcgis.com/arcgis/rest/services/places-service/v1"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#url)
     */
    url: string;
  }

  interface PlacesParametersProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#apiKey)
     */
    apiKey?: string;
    /**
     * Determines whether icons are returned and the type of icons returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#icon)
     */
    icon?: "cim" | "png" | "svg";
    /**
     * URL to the places service.
     *
     * @default "https://places-api.arcgis.com/arcgis/rest/services/places-service/v1"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesParameters.html#url)
     */
    url?: string;
  }

  export class PlacesQueryParameters extends PlacesParameters {
    /**
     * The following properties define the parameters for the [queryPlacesNearPoint()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesNearPoint) and [queryPlacesWithinExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesWithinExtent) methods pointing to a [url](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#url) that represents a places service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html)
     */

    constructor(properties?: PlacesQueryParametersProperties);

    /**
     * Filters places to those that match the category Ids.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#categoryIds)
     */
    categoryIds: string[];
    /**
     * The extent of the bounding box to be searched inside.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#extent)
     */
    extent: Extent;
    /**
     * Request results starting from the given offset.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#offset)
     */
    offset: number;
    /**
     * The number of places that should be sent in the response for a single request.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#pageSize)
     */
    pageSize: number;
    /**
     * A location defined by X and Y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#point)
     */
    point: Point;
    /**
     * The radius in meters to search for places, measured from the supplied [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#point).
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#radius)
     */
    radius: number;
    /**
     * Free search text for places against names, categories, etc.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#searchText)
     */
    searchText: string;

    /**
     * Creates a deep clone of the instance of PlacesQueryParameters that this method is called on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#clone)
     */
    clone(): PlacesQueryParameters;
  }

  interface PlacesQueryParametersProperties extends PlacesParametersProperties {
    /**
     * Filters places to those that match the category Ids.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#categoryIds)
     */
    categoryIds?: string[];
    /**
     * The extent of the bounding box to be searched inside.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Request results starting from the given offset.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#offset)
     */
    offset?: number;
    /**
     * The number of places that should be sent in the response for a single request.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#pageSize)
     */
    pageSize?: number;
    /**
     * A location defined by X and Y coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#point)
     */
    point?: PointProperties;
    /**
     * The radius in meters to search for places, measured from the supplied [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#point).
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#radius)
     */
    radius?: number;
    /**
     * Free search text for places against names, categories, etc.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryParameters.html#searchText)
     */
    searchText?: string;
  }

  export class PlacesQueryResult extends Accessor {
    /**
     * The `PlacesQueryResult` object includes the results from the [queryPlacesNearPoint()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesNearPoint) and [queryPlacesWithinExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesWithinExtent) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html)
     */

    constructor(properties?: PlacesQueryResultProperties);

    /**
     * The query parameters for the next set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#nextQueryParams)
     */
    nextQueryParams: PlacesQueryParameters;
    /**
     * The query parameters for the previous set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#previousQueryParams)
     */
    previousQueryParams: PlacesQueryParameters;
    /**
     * An array of results from searching for places using the [queryPlacesNearPoint()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesNearPoint) and [queryPlacesWithinExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesWithinExtent) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#results)
     */
    results: PlaceResult[];
  }

  interface PlacesQueryResultProperties {
    /**
     * The query parameters for the next set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#nextQueryParams)
     */
    nextQueryParams?: PlacesQueryParametersProperties;
    /**
     * The query parameters for the previous set of results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#previousQueryParams)
     */
    previousQueryParams?: PlacesQueryParametersProperties;
    /**
     * An array of results from searching for places using the [queryPlacesNearPoint()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesNearPoint) and [queryPlacesWithinExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html#queryPlacesWithinExtent) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PlacesQueryResult.html#results)
     */
    results?: PlaceResultProperties[];
  }

  export interface PointBarrier extends Accessor, JSONSupport {}

  export class PointBarrier {
    /**
     * A point barrier to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html)
     */

    constructor(properties?: PointBarrierProperties);

    /**
     * Specify if the point barrier restricts travel completely or adds time or distance when it is crossed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#barrierType)
     */
    barrierType: "added-cost" | "restriction" | "scaled-cost";
    /**
     * The direction of traffic that is affected by the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#curbApproach)
     */
    readonly curbApproach: "either-side-of-vehicle" | "right-side-of-vehicle" | "left-side-of-vehicle" | "no-u-turn";
    /**
     * Point barriers are applied to the edge elements during the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#fullEdge)
     */
    fullEdge: "permit" | "restrict";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#geometry)
     */
    geometry: Point;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#name)
     */
    name: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;

    readonly type: "point-barrier";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [PointBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [PointBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): PointBarrier;
    static fromJSON(json: any): PointBarrier;
  }

  interface PointBarrierProperties {
    /**
     * Specify if the point barrier restricts travel completely or adds time or distance when it is crossed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#barrierType)
     */
    barrierType?: "added-cost" | "restriction" | "scaled-cost";
    /**
     * Point barriers are applied to the edge elements during the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#fullEdge)
     */
    fullEdge?: "permit" | "restrict";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#geometry)
     */
    geometry?: PointProperties;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#name)
     */
    name?: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PointBarrier.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
  }

  export interface PolygonBarrier extends Accessor, JSONSupport {}

  export class PolygonBarrier {
    /**
     * A polygon barrier to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html)
     */

    constructor(properties?: PolygonBarrierProperties);

    /**
     * Use this parameter to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#barrierType)
     */
    barrierType: "added-cost" | "restriction" | "scaled-cost";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#geometry)
     */
    geometry: Polygon;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#name)
     */
    name: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;

    readonly type: "polygon-barrier";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [PolygonBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [PolygonBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): PolygonBarrier;
    static fromJSON(json: any): PolygonBarrier;
  }

  interface PolygonBarrierProperties {
    /**
     * Use this parameter to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#barrierType)
     */
    barrierType?: "added-cost" | "restriction" | "scaled-cost";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#geometry)
     */
    geometry?: PolygonProperties;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#name)
     */
    name?: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolygonBarrier.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
  }

  export interface PolylineBarrier extends Accessor, JSONSupport {}

  export class PolylineBarrier {
    /**
     * A polyline barrier to restrict travel along a street network when using a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html)
     */

    constructor(properties?: PolylineBarrierProperties);

    /**
     * Specify whether the barrier restricts travel completely or scales time or distance when it is crossed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#barrierType)
     */
    barrierType: "added-cost" | "restriction" | "scaled-cost";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#geometry)
     */
    geometry: Polyline;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#name)
     */
    name: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;

    readonly type: "polyline-barrier";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [PolylineBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [PolylineBarrier](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): PolylineBarrier;
    static fromJSON(json: any): PolylineBarrier;
  }

  interface PolylineBarrierProperties {
    /**
     * Specify whether the barrier restricts travel completely or scales time or distance when it is crossed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#barrierType)
     */
    barrierType?: "added-cost" | "restriction" | "scaled-cost";
    /**
     * The point location of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#geometry)
     */
    geometry?: PolylineProperties;
    /**
     * The name of the barrier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#name)
     */
    name?: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PolylineBarrier.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
  }

  export class PrintParameters extends Accessor {
    /**
     * Input parameters for [print](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html)
     */

    constructor(properties?: PrintParametersProperties);

    /**
     * Additional parameters for the print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#extraParameters)
     */
    extraParameters: any;
    /**
     * Specify the output spatial reference for the printout.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * Defines the layout template used for the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#template)
     */
    template: PrintTemplate;
    /**
     * The view to print.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#view)
     */
    view: MapView;
  }

  interface PrintParametersProperties {
    /**
     * Additional parameters for the print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#extraParameters)
     */
    extraParameters?: any;
    /**
     * Specify the output spatial reference for the printout.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * Defines the layout template used for the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#template)
     */
    template?: PrintTemplateProperties;
    /**
     * The view to print.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintParameters.html#view)
     */
    view?: MapViewProperties;
  }

  export class PrintTemplate extends Accessor {
    /**
     * Defines the layout template options used by [print](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-print.html) and the [PrintViewModel.print()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#print) method to generate the print page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html)
     */

    constructor(properties?: PrintTemplateProperties);

    /**
     * When `false`, the attribution is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#attributionVisible)
     */
    attributionVisible: boolean;
    /**
     * Define the map width, height and dpi.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#exportOptions)
     */
    exportOptions: PrintTemplateExportOptions;
    /**
     * When true, the feature's attributes are included in feature collection layers even when they are not needed for rendering.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#forceFeatureAttributes)
     */
    forceFeatureAttributes: boolean;
    /**
     * The output format for the printed map.
     *
     * @default "png32"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#format)
     */
    format: "gif" | "jpg" | "png8" | "png32" | "tiff" | "pdf" | "svg" | "svgz" | "aix" | "eps";
    /**
     * When `true`, charts will be included in the printout request.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#includeCharts)
     */
    includeCharts: boolean;
    /**
     * When `true`, tables will be included in the printout request.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#includeTables)
     */
    includeTables: boolean;
    /**
     * The layout used for the print output.
     *
     * @default "map-only"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layout)
     */
    layout:
      | "map-only"
      | "a3-landscape"
      | "a3-portrait"
      | "a4-landscape"
      | "a4-portrait"
      | "letter-ansi-a-landscape"
      | "letter-ansi-a-portrait"
      | "tabloid-ansi-b-landscape"
      | "tabloid-ansi-b-portrait";
    /**
     * A custom layout hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutItem)
     */
    layoutItem: PortalItem;
    /**
     * Defines the layout elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    layoutOptions: PrintTemplateLayoutOptions;
    /**
     * The optional map scale of the printed map.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#outScale)
     */
    outScale: number;
    /**
     * The name of the report template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#report)
     */
    report:
      | "attribute-list-letter-ansi-a-landscape"
      | "attribute-list-letter-ansi-a-portrait"
      | "attribute-list-with-group-letter-ansi-a-landscape"
      | "attribute-list-with-group-letter-ansi-a-portrait"
      | "page-per-feature-letter-ansi-a-landscape"
      | "page-per-feature-letter-ansi-a-portrait";
    /**
     * A custom report template hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) for report printing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#reportItem)
     */
    reportItem: PortalItem;
    /**
     * This object links the various report elements to their data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#reportOptions)
     */
    reportOptions: any;
    /**
     * Define whether the printed map should preserve map scale or map extent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#scalePreserved)
     */
    scalePreserved: boolean;
    /**
     * When `true`, labels will be shown on the layout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#showLabels)
     */
    showLabels: boolean;
  }

  interface PrintTemplateProperties {
    /**
     * When `false`, the attribution is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#attributionVisible)
     */
    attributionVisible?: boolean;
    /**
     * Define the map width, height and dpi.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#exportOptions)
     */
    exportOptions?: PrintTemplateExportOptions;
    /**
     * When true, the feature's attributes are included in feature collection layers even when they are not needed for rendering.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#forceFeatureAttributes)
     */
    forceFeatureAttributes?: boolean;
    /**
     * The output format for the printed map.
     *
     * @default "png32"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#format)
     */
    format?: "gif" | "jpg" | "png8" | "png32" | "tiff" | "pdf" | "svg" | "svgz" | "aix" | "eps";
    /**
     * When `true`, charts will be included in the printout request.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#includeCharts)
     */
    includeCharts?: boolean;
    /**
     * When `true`, tables will be included in the printout request.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#includeTables)
     */
    includeTables?: boolean;
    /**
     * The layout used for the print output.
     *
     * @default "map-only"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layout)
     */
    layout?:
      | "map-only"
      | "a3-landscape"
      | "a3-portrait"
      | "a4-landscape"
      | "a4-portrait"
      | "letter-ansi-a-landscape"
      | "letter-ansi-a-portrait"
      | "tabloid-ansi-b-landscape"
      | "tabloid-ansi-b-portrait";
    /**
     * A custom layout hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutItem)
     */
    layoutItem?: PortalItemProperties;
    /**
     * Defines the layout elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    layoutOptions?: PrintTemplateLayoutOptions;
    /**
     * The optional map scale of the printed map.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#outScale)
     */
    outScale?: number;
    /**
     * The name of the report template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#report)
     */
    report?:
      | "attribute-list-letter-ansi-a-landscape"
      | "attribute-list-letter-ansi-a-portrait"
      | "attribute-list-with-group-letter-ansi-a-landscape"
      | "attribute-list-with-group-letter-ansi-a-portrait"
      | "page-per-feature-letter-ansi-a-landscape"
      | "page-per-feature-letter-ansi-a-portrait";
    /**
     * A custom report template hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html) for report printing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#reportItem)
     */
    reportItem?: PortalItemProperties;
    /**
     * This object links the various report elements to their data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#reportOptions)
     */
    reportOptions?: any;
    /**
     * Define whether the printed map should preserve map scale or map extent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#scalePreserved)
     */
    scalePreserved?: boolean;
    /**
     * When `true`, labels will be shown on the layout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#showLabels)
     */
    showLabels?: boolean;
  }

  export interface PrintTemplateExportOptions {
    /**
     * Map width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#exportOptions)
     */
    width?: number;
    /**
     * Map height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#exportOptions)
     */
    height?: number;
    /**
     * Resolution in dots per inch.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#exportOptions)
     */
    dpi?: number;
  }

  export interface PrintTemplateLayoutOptions {
    /**
     * The text used for the map title if the specified layout contains a title text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    titleText?: string;
    /**
     * The text used for the author if the specified layout contains an author text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    authorText?: string;
    /**
     * The text used for the copyright if the specified layout contains a copyright text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    copyrightText?: string;
    /**
     * The unit used for the scalebar.
     *
     * @default Miles
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    scalebarUnit?: "Miles" | "Kilometers" | "Meters" | "Feet";
    /**
     * An array of [LegendLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-LegendLayer.html) containing the ids of the layers that will be included in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    legendLayers?: LegendLayer[];
    /**
     * An array of name-value pair objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    customTextElements?: any[];
    /**
     * An object containing optional elements from the print service that can be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-PrintTemplate.html#layoutOptions)
     */
    elementOverrides?: any;
  }

  export class ProjectParameters extends Accessor {
    /**
     * Defines the projection parameters used when calling the [geometryService.project()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#project).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html)
     */

    constructor(properties?: ProjectParametersProperties);

    /**
     * The input geometries to project.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#geometries)
     */
    geometries: Geometry[];
    /**
     * The spatial reference to which you are projecting the geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * The well-known id {wkid:number} or well-known text {wkt:string} of the datum transformation to be applied to the projected geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformation)
     */
    transformation: ProjectParametersTransformation;
    /**
     * Indicates whether to transform forward or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformForward)
     */
    transformForward: boolean;

    /**
     * Converts an instance of  this class to its ArcGIS portal JSON representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#toJSON)
     */
    toJSON(): any;
  }

  interface ProjectParametersProperties {
    /**
     * The input geometries to project.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#geometries)
     */
    geometries?: GeometryProperties[];
    /**
     * The spatial reference to which you are projecting the geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * The well-known id {wkid:number} or well-known text {wkt:string} of the datum transformation to be applied to the projected geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformation)
     */
    transformation?: ProjectParametersTransformation;
    /**
     * Indicates whether to transform forward or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformForward)
     */
    transformForward?: boolean;
  }

  export interface ProjectParametersTransformation {
    /**
     * The well-known ID of the datum transformation to apply to the projection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformation)
     */
    wkid?: number;
    /**
     * The well-known text that defines a spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ProjectParameters.html#transformation)
     */
    wkt?: string;
  }

  export interface Query extends Accessor, JSONSupport, QueryMixin {}

  export class Query {
    /**
     * This class defines parameters for executing queries for features from a layer or layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html)
     */

    constructor(properties?: QueryProperties);

    /**
     * An array of Object IDs representing [aggregate](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#isAggregate) (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#aggregateIds)
     */
    aggregateIds: (number | string)[];
    /**
     * Datum transformation used for projecting geometries in the query results when [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outSpatialReference) is different than the layer's spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#datumTransformation)
     */
    datumTransformation: number | SimpleTransformation | CompositeTransformation;
    /**
     * Specifies the geodatabase version to display for feature service queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * Specifies the number of decimal places for geometries returned by the [JSON query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeQueryJSON).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * Used only in [statistical queries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#statistic).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics)
     */
    groupByFieldsForStatistics: string[];
    /**
     * A condition used with [outStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outStatistics) and [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) to limit query results to groups that satisfy the aggregation function(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having)
     */
    having: string;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#historicMoment)
     */
    historicMoment: Date;
    /**
     * The maximum distance in units of [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outSpatialReference) used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#maxAllowableOffset)
     */
    maxAllowableOffset: number;
    /**
     * When set, the maximum number of features returned by the query will equal the `maxRecordCount` of the service multiplied by this factor.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#maxRecordCountFactor)
     */
    maxRecordCountFactor: number;
    /**
     * Parameter dictates how the geometry of a multipatch feature will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#multipatchOption)
     */
    multipatchOption: string;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#num)
     */
    num: number;
    /**
     * An array of ObjectIDs to be used to query for features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#objectIds)
     */
    objectIds: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#orderByFields)
     */
    orderByFields: string[];
    /**
     * Attribute fields to include in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outFields)
     */
    outFields: string[];
    /**
     * The definitions for one or more field-based statistics to be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outStatistics)
     */
    outStatistics: StatisticDefinition[];
    /**
     * Filters features from the layer based on pre-authored parameterized filters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#parameterValues)
     */
    parameterValues: HashMap<string | string[] | number | number[] | Date | Date[]>;
    /**
     * Specifies the pixel level to be identified on the X and Y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#pixelSize)
     */
    pixelSize: Point;
    /**
     * Used to project the geometry onto a virtual grid, likely representing pixels on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    quantizationParameters: QueryQuantizationParameters;
    /**
     * Filters features from the layer that are within the specified range values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#rangeValues)
     */
    rangeValues: QueryRangeValues[];
    /**
     * The Dimensionally Extended 9 Intersection Model (DE-9IM) matrix relation (encoded as a string) to query the spatial relationship of the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometry) to the layer's features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#relationParameter)
     */
    relationParameter: string;
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) will be returned with a centroid.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnCentroid)
     */
    returnCentroid: boolean;
    /**
     * If `true` then the query returns distinct values based on the field(s) specified in [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outFields).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnDistinctValues)
     */
    returnDistinctValues: boolean;
    /**
     * If `true`, then all features are returned for each tile request, even if they exceed the maximum record limit per query indicated on the service by `maxRecordCount`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnExceededLimitFeatures)
     */
    returnExceededLimitFeatures: boolean;
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnM)
     */
    returnM: boolean;
    /**
     * If `true`, the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry) will be returned with the query results.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnQueryGeometry)
     */
    returnQueryGeometry: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnZ)
     */
    returnZ: boolean;
    /**
     * This parameter can be either standard SQL92 `standard` or it can use the native SQL of the underlying datastore `native`.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#sqlFormat)
     */
    sqlFormat: "none" | "standard" | "native";
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#start)
     */
    start: number;
    /**
     * Shorthand for a where clause using "like".
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#text)
     */
    text: string;
    /**
     * A time extent for a temporal query against [time-aware layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#timeExtent)
     */
    timeExtent: TimeExtent;

    /**
     * Creates a deep clone of Query object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#clone)
     */
    clone(): Query;

    static fromJSON(json: any): Query;
  }

  interface QueryProperties extends QueryMixinProperties {
    /**
     * An array of Object IDs representing [aggregate](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html#isAggregate) (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#aggregateIds)
     */
    aggregateIds?: (number | string)[];
    /**
     * Datum transformation used for projecting geometries in the query results when [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outSpatialReference) is different than the layer's spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#datumTransformation)
     */
    datumTransformation?: number | SimpleTransformation | CompositeTransformation;
    /**
     * Specifies the geodatabase version to display for feature service queries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * Specifies the number of decimal places for geometries returned by the [JSON query operation](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query.html#executeQueryJSON).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * Used only in [statistical queries](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#statistic).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics)
     */
    groupByFieldsForStatistics?: string[];
    /**
     * A condition used with [outStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outStatistics) and [groupByFieldsForStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#groupByFieldsForStatistics) to limit query results to groups that satisfy the aggregation function(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#having)
     */
    having?: string;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#historicMoment)
     */
    historicMoment?: DateProperties;
    /**
     * The maximum distance in units of [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outSpatialReference) used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#maxAllowableOffset)
     */
    maxAllowableOffset?: number;
    /**
     * When set, the maximum number of features returned by the query will equal the `maxRecordCount` of the service multiplied by this factor.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#maxRecordCountFactor)
     */
    maxRecordCountFactor?: number;
    /**
     * Parameter dictates how the geometry of a multipatch feature will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#multipatchOption)
     */
    multipatchOption?: string;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#num)
     */
    num?: number;
    /**
     * An array of ObjectIDs to be used to query for features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#objectIds)
     */
    objectIds?: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#orderByFields)
     */
    orderByFields?: string[];
    /**
     * Attribute fields to include in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outFields)
     */
    outFields?: string[];
    /**
     * The definitions for one or more field-based statistics to be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outStatistics)
     */
    outStatistics?: StatisticDefinitionProperties[];
    /**
     * Filters features from the layer based on pre-authored parameterized filters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#parameterValues)
     */
    parameterValues?: HashMap<string | string[] | number | number[] | DateProperties | DateProperties[]>;
    /**
     * Specifies the pixel level to be identified on the X and Y axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#pixelSize)
     */
    pixelSize?: PointProperties;
    /**
     * Used to project the geometry onto a virtual grid, likely representing pixels on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    quantizationParameters?: QueryQuantizationParameters;
    /**
     * Filters features from the layer that are within the specified range values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#rangeValues)
     */
    rangeValues?: QueryRangeValues[];
    /**
     * The Dimensionally Extended 9 Intersection Model (DE-9IM) matrix relation (encoded as a string) to query the spatial relationship of the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#geometry) to the layer's features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#relationParameter)
     */
    relationParameter?: string;
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) will be returned with a centroid.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnCentroid)
     */
    returnCentroid?: boolean;
    /**
     * If `true` then the query returns distinct values based on the field(s) specified in [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outFields).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnDistinctValues)
     */
    returnDistinctValues?: boolean;
    /**
     * If `true`, then all features are returned for each tile request, even if they exceed the maximum record limit per query indicated on the service by `maxRecordCount`.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnExceededLimitFeatures)
     */
    returnExceededLimitFeatures?: boolean;
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnM)
     */
    returnM?: boolean;
    /**
     * If `true`, the [query geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html#queryGeometry) will be returned with the query results.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnQueryGeometry)
     */
    returnQueryGeometry?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * This parameter can be either standard SQL92 `standard` or it can use the native SQL of the underlying datastore `native`.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#sqlFormat)
     */
    sqlFormat?: "none" | "standard" | "native";
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#start)
     */
    start?: number;
    /**
     * Shorthand for a where clause using "like".
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#text)
     */
    text?: string;
    /**
     * A time extent for a temporal query against [time-aware layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
  }

  export interface CompositeTransformation {
    geoTransforms: CompositeTransformationGeoTransforms[];
  }

  export interface QueryQuantizationParameters {
    /**
     * An extent defining the quantization grid bounds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    extent?: Extent;
    /**
     * Geometry coordinates are optimized for viewing and displaying of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    mode?: "view" | "edit";
    /**
     * The integer's coordinates will be returned relative to the origin position defined by this property value.
     *
     * @default upper-left
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    originPosition?: "upper-left" | "lower-left";
    /**
     * The size of one pixel in the units of the [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#outSpatialReference).
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#quantizationParameters)
     */
    tolerance?: number;
  }

  export interface QueryRangeValues {
    /**
     * The range id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#rangeValues)
     */
    name: string;
    /**
     * Single value or value range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#rangeValues)
     */
    value: number | number[];
  }

  export interface SimpleTransformation {
    /**
     * the spatial reference well known id
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#SimpleTransformation)
     */
    wkid: number;
  }

  export interface CompositeTransformationGeoTransforms {
    /**
     * the spatial reference well known id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#CompositeTransformation)
     */
    wkid: number;
    /**
     * indicates whether to transform forward or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html#CompositeTransformation)
     */
    transformForward: boolean;
  }

  export class QueryMixin {
    /**
     * Indicates if the service should cache the query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#cacheHint)
     */
    cacheHint: boolean;
    /**
     * Specifies a search distance from a given [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry) in a spatial query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#distance)
     */
    distance: number;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry)
     */
    geometry: Geometry;
    /**
     * The spatial reference for the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * For spatial queries, this parameter defines the spatial relationship to query features in the layer or layer view against the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry).
     *
     * @default intersects
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#spatialRelationship)
     */
    spatialRelationship:
      | "intersects"
      | "contains"
      | "crosses"
      | "disjoint"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";
    /**
     * The unit for calculating the buffer distance when [distance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#distance) is specified in spatial queries.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#units)
     */
    units: "feet" | "miles" | "nautical-miles" | "us-nautical-miles" | "meters" | "kilometers";
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#where)
     */
    where: string;
  }

  interface QueryMixinProperties {
    /**
     * Indicates if the service should cache the query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#cacheHint)
     */
    cacheHint?: boolean;
    /**
     * Specifies a search distance from a given [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry) in a spatial query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#distance)
     */
    distance?: number;
    /**
     * The geometry to apply to the spatial filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * The spatial reference for the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * For spatial queries, this parameter defines the spatial relationship to query features in the layer or layer view against the input [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#geometry).
     *
     * @default intersects
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#spatialRelationship)
     */
    spatialRelationship?:
      | "intersects"
      | "contains"
      | "crosses"
      | "disjoint"
      | "envelope-intersects"
      | "index-intersects"
      | "overlaps"
      | "touches"
      | "within"
      | "relation";
    /**
     * The unit for calculating the buffer distance when [distance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#distance) is specified in spatial queries.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#units)
     */
    units?: "feet" | "miles" | "nautical-miles" | "us-nautical-miles" | "meters" | "kilometers";
    /**
     * A where clause for the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-QueryMixin.html#where)
     */
    where?: string;
  }

  export interface RasterData extends Accessor, JSONSupport {}

  export class RasterData {
    /**
     * A geoprocessing data object containing a raster data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html)
     */

    constructor(properties?: RasterDataProperties);

    /**
     * Specifies the format of the raster data, such as "jpg", "tif", etc.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#format)
     */
    format: string;
    /**
     * The ID of the uploaded file returned as a result of the upload operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#itemId)
     */
    itemId: string;
    /**
     * URL to the location of the raster data file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#url)
     */
    url: string;

    static fromJSON(json: any): RasterData;
  }

  interface RasterDataProperties {
    /**
     * Specifies the format of the raster data, such as "jpg", "tif", etc.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#format)
     */
    format?: string;
    /**
     * The ID of the uploaded file returned as a result of the upload operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#itemId)
     */
    itemId?: string;
    /**
     * URL to the location of the raster data file.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RasterData.html#url)
     */
    url?: string;
  }

  export interface RelationParameters extends Accessor, JSONSupport {}

  export class RelationParameters {
    /**
     * Sets the relation and other parameters for the [geometryService.relation()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#relation) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html)
     */

    constructor(properties?: RelationParametersProperties);

    /**
     * The first array of geometries to compute the relation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#geometries1)
     */
    geometries1: Geometry[];
    /**
     * The second array of geometries to compute the relation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#geometries2)
     */
    geometries2: Geometry[];
    /**
     * The spatial relationship to be tested between the two input geometry arrays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#relation)
     */
    relation:
      | "cross"
      | "disjoint"
      | "in"
      | "interior-intersection"
      | "intersection"
      | "line-coincidence"
      | "line-touch"
      | "overlap"
      | "point-touch"
      | "relation"
      | "touch"
      | "within";
    /**
     * The string describes the spatial relationship to be tested when `RelationParameters.relation = 'relation'`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#relationParameter)
     */
    relationParameter: string;

    static fromJSON(json: any): RelationParameters;
  }

  interface RelationParametersProperties {
    /**
     * The first array of geometries to compute the relation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#geometries1)
     */
    geometries1?: GeometryProperties[];
    /**
     * The second array of geometries to compute the relation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#geometries2)
     */
    geometries2?: GeometryProperties[];
    /**
     * The spatial relationship to be tested between the two input geometry arrays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#relation)
     */
    relation?:
      | "cross"
      | "disjoint"
      | "in"
      | "interior-intersection"
      | "intersection"
      | "line-coincidence"
      | "line-touch"
      | "overlap"
      | "point-touch"
      | "relation"
      | "touch"
      | "within";
    /**
     * The string describes the spatial relationship to be tested when `RelationParameters.relation = 'relation'`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationParameters.html#relationParameter)
     */
    relationParameter?: string;
  }

  export interface RelationshipQuery extends Accessor, JSONSupport {}

  export class RelationshipQuery {
    /**
     * This class defines parameters for executing queries for related records from a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html)
     */

    constructor(properties?: RelationshipQueryProperties);

    /**
     * Indicates if the service should cache the relationship query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#cacheHint)
     */
    cacheHint: boolean;
    /**
     * Specify the geodatabase version to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * Specify the number of decimal places for the geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#historicMoment)
     */
    historicMoment: Date;
    /**
     * The maximum allowable offset used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#maxAllowableOffset)
     */
    maxAllowableOffset: number;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#num)
     */
    num: number;
    /**
     * An array of objectIds for the features in the layer/table being queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#objectIds)
     */
    objectIds: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#orderByFields)
     */
    orderByFields: string[];
    /**
     * Attribute fields to include in the FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#outFields)
     */
    outFields: string[];
    /**
     * The spatial reference for the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * The ID of the relationship to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#relationshipId)
     */
    relationshipId: number;
    /**
     * If `true`, each feature in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnM)
     */
    returnM: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnZ)
     */
    returnZ: boolean;
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#start)
     */
    start: number;
    /**
     * The definition expression to be applied to the related table or layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#where)
     */
    where: string;

    /**
     * Creates a deep clone of RelationshipQuery object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#clone)
     */
    clone(): RelationshipQuery;

    static fromJSON(json: any): RelationshipQuery;
  }

  interface RelationshipQueryProperties {
    /**
     * Indicates if the service should cache the relationship query results.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#cacheHint)
     */
    cacheHint?: boolean;
    /**
     * Specify the geodatabase version to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * Specify the number of decimal places for the geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * The historic moment to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#historicMoment)
     */
    historicMoment?: DateProperties;
    /**
     * The maximum allowable offset used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#maxAllowableOffset)
     */
    maxAllowableOffset?: number;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#num)
     */
    num?: number;
    /**
     * An array of objectIds for the features in the layer/table being queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#objectIds)
     */
    objectIds?: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#orderByFields)
     */
    orderByFields?: string[];
    /**
     * Attribute fields to include in the FeatureSet.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#outFields)
     */
    outFields?: string[];
    /**
     * The spatial reference for the returned geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * The ID of the relationship to be queried.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#relationshipId)
     */
    relationshipId?: number;
    /**
     * If `true`, each feature in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnM)
     */
    returnM?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#start)
     */
    start?: number;
    /**
     * The definition expression to be applied to the related table or layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RelationshipQuery.html#where)
     */
    where?: string;
  }

  export interface RouteInfo extends Accessor, JSONSupport {}

  export class RouteInfo {
    /**
     * A RouteInfo contains information about a solved route including the routes geometry and overall distance and time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html)
     */

    constructor(properties?: RouteInfoProperties);

    /**
     * The end time of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#endTime)
     */
    endTime: Date;
    /**
     * The local time offset (in minutes) for the end time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#endTimeOffset)
     */
    endTimeOffset: number;
    /**
     * Polyline representing the route's geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#geometry)
     */
    geometry: Polyline;
    /**
     * User specified route name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#name)
     */
    name: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * The start time of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#startTime)
     */
    startTime: Date;
    /**
     * The local time offset (in minutes) for the start time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#startTimeOffset)
     */
    startTimeOffset: number;
    /**
     * Total distance traveled in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#totalDistance)
     */
    totalDistance: number;
    /**
     * Total time in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#totalDuration)
     */
    totalDuration: number;

    readonly type: "route-info";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [RouteInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [RouteInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): RouteInfo;
    static fromJSON(json: any): RouteInfo;
  }

  interface RouteInfoProperties {
    /**
     * The end time of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#endTime)
     */
    endTime?: DateProperties;
    /**
     * The local time offset (in minutes) for the end time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#endTimeOffset)
     */
    endTimeOffset?: number;
    /**
     * Polyline representing the route's geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#geometry)
     */
    geometry?: PolylineProperties;
    /**
     * User specified route name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#name)
     */
    name?: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * The start time of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#startTime)
     */
    startTime?: DateProperties;
    /**
     * The local time offset (in minutes) for the start time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#startTimeOffset)
     */
    startTimeOffset?: number;
    /**
     * Total distance traveled in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#totalDistance)
     */
    totalDistance?: number;
    /**
     * Total time in minutes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteInfo.html#totalDuration)
     */
    totalDuration?: number;
  }

  export interface RouteParameters extends Accessor, JSONSupport, Clonable {}

  export class RouteParameters {
    /**
     * Input parameters for [route](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html)
     */

    constructor(properties?: RouteParametersProperties);

    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#accumulateAttributes)
     */
    accumulateAttributes: string[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#apiKey)
     */
    apiKey: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#attributeParameterValues)
     */
    attributeParameterValues: RouteParametersAttributeParameterValue[];
    /**
     * The language that will be used when generating travel directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsLanguage)
     */
    directionsLanguage: string;
    /**
     * Specify the units for displaying travel distance in the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsLengthUnits)
     */
    directionsLengthUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Define the content and verbosity of the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsOutputType)
     */
    directionsOutputType:
      | "complete"
      | "complete-no-events"
      | "featuresets"
      | "instructions-only"
      | "standard"
      | "summary-only";
    /**
     * Specify the name of the formatting style for the directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsStyleName)
     */
    directionsStyleName: "desktop" | "navigation" | "campus";
    /**
     * Set the time-based impedance attribute to display the duration of a maneuver, such as "Go northwest on Alvorado St.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsTimeAttribute)
     */
    directionsTimeAttribute: string;
    /**
     * Use this property to specify whether the operation should reorder stops to find the optimized route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#findBestSequence)
     */
    findBestSequence: boolean;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#impedanceAttribute)
     */
    impedanceAttribute:
      | "kilometers"
      | "miles"
      | "minutes"
      | "travel-time"
      | "truck-minutes"
      | "truck-travel-time"
      | "walk-time"
      | string;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * @default "true-shape"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputLines)
     */
    outputLines: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#overrides)
     */
    overrides: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#pointBarriers)
     */
    pointBarriers: Collection<PointBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polygonBarriers)
     */
    polygonBarriers: Collection<PolygonBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polylineBarriers)
     */
    polylineBarriers: Collection<PolylineBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to indicate whether the operation should keep the first stop fixed when reordering the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveFirstStop)
     */
    preserveFirstStop: boolean;
    /**
     * Use this property to indicate whether the operation should keep the last stop fixed when reordering the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveLastStop)
     */
    preserveLastStop: boolean;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveObjectID)
     */
    preserveObjectID: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#restrictionAttributes)
     */
    restrictionAttributes: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Use this property to restrict or permit the route from making U-turns at junctions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#restrictUTurns)
     */
    restrictUTurns: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnBarriers)
     */
    returnBarriers: boolean;
    /**
     * Specify whether the operation should generate driving directions for each route.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnDirections)
     */
    returnDirections: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers: boolean;
    /**
     * Use this property to specify if the operation should return routes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnRoutes)
     */
    returnRoutes: boolean;
    /**
     * Use this property to specify whether [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnStops)
     */
    returnStops: boolean;
    /**
     * Specify whether traversed edges will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedEdges)
     */
    returnTraversedEdges: boolean;
    /**
     * Specify whether traversed junctions will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedJunctions)
     */
    returnTraversedJunctions: boolean;
    /**
     * Specify whether traversed turns will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedTurns)
     */
    returnTraversedTurns: boolean;
    /**
     * Include z values for the returned geometries if supported by the underlying network.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnZ)
     */
    returnZ: boolean;
    /**
     * Indicates the time that travel should begin.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTime)
     */
    startTime: Date | "now";
    /**
     * Specify the time zone or zones of the [startTime](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTime) property.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTimeIsUTC)
     */
    startTimeIsUTC: boolean;
    /**
     * Specifies the locations the output route or routes will visit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops)
     */
    stops: Collection<Stop> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Specify whether the timeWindowStart and timeWindowEnd property values on [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) are specified in UTC or geographically local time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#timeWindowsAreUTC)
     */
    timeWindowsAreUTC: boolean;
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#travelMode)
     */
    travelMode: TravelMode;
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#useHierarchy)
     */
    useHierarchy: boolean;
    /**
     * Use this property to indicate whether the operation should consider time windows specified on the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) when finding the best route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#useTimeWindows)
     */
    useTimeWindows: boolean;

    static fromJSON(json: any): RouteParameters;
  }

  interface RouteParametersProperties {
    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#accumulateAttributes)
     */
    accumulateAttributes?: string[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#apiKey)
     */
    apiKey?: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#attributeParameterValues)
     */
    attributeParameterValues?: RouteParametersAttributeParameterValue[];
    /**
     * The language that will be used when generating travel directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsLanguage)
     */
    directionsLanguage?: string;
    /**
     * Specify the units for displaying travel distance in the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsLengthUnits)
     */
    directionsLengthUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Define the content and verbosity of the driving directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsOutputType)
     */
    directionsOutputType?:
      | "complete"
      | "complete-no-events"
      | "featuresets"
      | "instructions-only"
      | "standard"
      | "summary-only";
    /**
     * Specify the name of the formatting style for the directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsStyleName)
     */
    directionsStyleName?: "desktop" | "navigation" | "campus";
    /**
     * Set the time-based impedance attribute to display the duration of a maneuver, such as "Go northwest on Alvorado St.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsTimeAttribute)
     */
    directionsTimeAttribute?: string;
    /**
     * Use this property to specify whether the operation should reorder stops to find the optimized route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#findBestSequence)
     */
    findBestSequence?: boolean;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ?: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations?: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#impedanceAttribute)
     */
    impedanceAttribute?:
      | "kilometers"
      | "miles"
      | "minutes"
      | "travel-time"
      | "truck-minutes"
      | "truck-travel-time"
      | "walk-time"
      | string;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision?: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * @default "true-shape"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outputLines)
     */
    outputLines?: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#overrides)
     */
    overrides?: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#pointBarriers)
     */
    pointBarriers?:
      | CollectionProperties<PointBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polygonBarriers)
     */
    polygonBarriers?:
      | CollectionProperties<PolygonBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polylineBarriers)
     */
    polylineBarriers?:
      | CollectionProperties<PolylineBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to indicate whether the operation should keep the first stop fixed when reordering the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveFirstStop)
     */
    preserveFirstStop?: boolean;
    /**
     * Use this property to indicate whether the operation should keep the last stop fixed when reordering the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveLastStop)
     */
    preserveLastStop?: boolean;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#preserveObjectID)
     */
    preserveObjectID?: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#restrictionAttributes)
     */
    restrictionAttributes?: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Use this property to restrict or permit the route from making U-turns at junctions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#restrictUTurns)
     */
    restrictUTurns?: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnBarriers)
     */
    returnBarriers?: boolean;
    /**
     * Specify whether the operation should generate driving directions for each route.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnDirections)
     */
    returnDirections?: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers?: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers?: boolean;
    /**
     * Use this property to specify if the operation should return routes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnRoutes)
     */
    returnRoutes?: boolean;
    /**
     * Use this property to specify whether [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnStops)
     */
    returnStops?: boolean;
    /**
     * Specify whether traversed edges will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedEdges)
     */
    returnTraversedEdges?: boolean;
    /**
     * Specify whether traversed junctions will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedJunctions)
     */
    returnTraversedJunctions?: boolean;
    /**
     * Specify whether traversed turns will be returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnTraversedTurns)
     */
    returnTraversedTurns?: boolean;
    /**
     * Include z values for the returned geometries if supported by the underlying network.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * Indicates the time that travel should begin.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTime)
     */
    startTime?: DateProperties | "now";
    /**
     * Specify the time zone or zones of the [startTime](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTime) property.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#startTimeIsUTC)
     */
    startTimeIsUTC?: boolean;
    /**
     * Specifies the locations the output route or routes will visit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops)
     */
    stops?:
      | CollectionProperties<StopProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Specify whether the timeWindowStart and timeWindowEnd property values on [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) are specified in UTC or geographically local time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#timeWindowsAreUTC)
     */
    timeWindowsAreUTC?: boolean;
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#travelMode)
     */
    travelMode?: TravelModeProperties;
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#useHierarchy)
     */
    useHierarchy?: boolean;
    /**
     * Use this property to indicate whether the operation should consider time windows specified on the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops) when finding the best route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#useTimeWindows)
     */
    useTimeWindows?: boolean;
  }

  /**
   * An object describing the parameter values for the [attributeParameterValues](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#attributeParameterValues) property of [RouteParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#AttributeParameterValue)
   */
  export interface RouteParametersAttributeParameterValue {
    /**
     * The name of the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#AttributeParameterValue)
     */
    attributeName: string;
    /**
     * The name of the parameter associated with the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#AttributeParameterValue)
     */
    parameterName: string;
    /**
     * The value for `parameterName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#AttributeParameterValue)
     */
    value: string | number;
  }

  export interface RouteResult extends Accessor, JSONSupport {}

  export class RouteResult {
    /**
     * The route result from [route.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve) as part of a [RouteSolveResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html)
     */

    constructor(properties?: RouteResultProperties);

    /**
     * Direction lines contains a set of line features for each segment of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directionLines)
     */
    directionLines: FeatureSet;
    /**
     * Direction points contains a set of point features representing the direction maneuvers such as arriving to or departing from a stop, turning left or right, and other events along your route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directionPoints)
     */
    directionPoints: FeatureSet;
    /**
     * Direction are returned if [RouteParameters.returnDirections](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnDirections) is set to `true` and [RouteParameters.directionsOutputType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsOutputType) is set to `complete`, `complete-no-events`, `instructions-only', `standard` or `summary-only` in a [route.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directions)
     */
    directions: DirectionsFeatureSet;
    /**
     * This property returns a Graphic that represent the overall path with attributes containing the total cost.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#route)
     */
    route: Graphic;
    /**
     * The name of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#routeName)
     */
    routeName: string;
    /**
     * Array of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#stops)
     */
    stops: Graphic[];
    /**
     * This provides access to the edges that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedEdges)
     */
    traversedEdges: FeatureSet;
    /**
     * This provides access to the junctions that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedJunctions)
     */
    traversedJunctions: FeatureSet;
    /**
     * This provides access to the turns that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedTurns)
     */
    traversedTurns: FeatureSet;

    static fromJSON(json: any): RouteResult;
  }

  interface RouteResultProperties {
    /**
     * Direction lines contains a set of line features for each segment of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directionLines)
     */
    directionLines?: FeatureSetProperties;
    /**
     * Direction points contains a set of point features representing the direction maneuvers such as arriving to or departing from a stop, turning left or right, and other events along your route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directionPoints)
     */
    directionPoints?: FeatureSetProperties;
    /**
     * Direction are returned if [RouteParameters.returnDirections](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#returnDirections) is set to `true` and [RouteParameters.directionsOutputType](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#directionsOutputType) is set to `complete`, `complete-no-events`, `instructions-only', `standard` or `summary-only` in a [route.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#directions)
     */
    directions?: DirectionsFeatureSetProperties;
    /**
     * This property returns a Graphic that represent the overall path with attributes containing the total cost.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#route)
     */
    route?: GraphicProperties;
    /**
     * The name of the route.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#routeName)
     */
    routeName?: string;
    /**
     * Array of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#stops)
     */
    stops?: GraphicProperties[];
    /**
     * This provides access to the edges that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedEdges)
     */
    traversedEdges?: FeatureSetProperties;
    /**
     * This provides access to the junctions that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedJunctions)
     */
    traversedJunctions?: FeatureSetProperties;
    /**
     * This provides access to the turns that are traversed while solving a network analysis layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html#traversedTurns)
     */
    traversedTurns?: FeatureSetProperties;
  }

  export interface RouteSolveResult extends Accessor, JSONSupport {}

  export class RouteSolveResult {
    /**
     * The results from [route.solve()](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-route.html#solve) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html)
     */

    constructor(properties?: RouteSolveResultProperties);

    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#messages)
     */
    messages: NAMessage[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#pointBarriers)
     */
    pointBarriers: Graphic[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#polygonBarriers)
     */
    polygonBarriers: Graphic[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#polylineBarriers)
     */
    polylineBarriers: Graphic[];
    /**
     * An array of [route results](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#routeResults)
     */
    routeResults: RouteResult[];

    static fromJSON(json: any): RouteSolveResult;
  }

  interface RouteSolveResultProperties {
    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#messages)
     */
    messages?: NAMessageProperties[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#pointBarriers)
     */
    pointBarriers?: GraphicProperties[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#polygonBarriers)
     */
    polygonBarriers?: GraphicProperties[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#polylineBarriers)
     */
    polylineBarriers?: GraphicProperties[];
    /**
     * An array of [route results](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteResult.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteSolveResult.html#routeResults)
     */
    routeResults?: RouteResultProperties[];
  }

  export interface ServiceAreaParameters extends Accessor, JSONSupport, Clonable {}

  export class ServiceAreaParameters {
    /**
     * ServiceAreaParameters provides the input parameters for a [serviceArea](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-serviceArea.html) request.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html)
     */

    constructor(properties?: ServiceAreaParametersProperties);

    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#accumulateAttributes)
     */
    accumulateAttributes: (
      | "kilometers"
      | "miles"
      | "minutes"
      | "travel-time"
      | "truck-minutes"
      | "truck-travel-time"
      | "walk-time"
      | string
    )[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#apiKey)
     */
    apiKey: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#attributeParameterValues)
     */
    attributeParameterValues: ServiceAreaParametersAttributeParameterValue[];
    /**
     * Use this parameter to specify the size and number of service areas to generate for each facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#defaultBreaks)
     */
    defaultBreaks: number[];
    /**
     * An array of network source names to not use when generating polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#excludeSourcesFromPolygons)
     */
    excludeSourcesFromPolygons: string[];
    /**
     * The input locations around which service areas are generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#facilities)
     */
    facilities: DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#impedanceAttribute)
     */
    impedanceAttribute: string;
    /**
     * Use this parameter to specify whether the service areas from different facilities that have the same break value should be joined together or split at break values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#mergeSimilarPolygonRanges)
     */
    mergeSimilarPolygonRanges: boolean;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputLines)
     */
    outputLines: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * This parameter sets whether service area polygons should be returned and to what detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputPolygons)
     */
    outputPolygons: "none" | "simplified" | "detailed";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outSpatialReference)
     */
    outSpatialReference: SpatialReference;
    /**
     * Specifies whether the service area lines from different facilities can overlap each other.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overlapLines)
     */
    overlapLines: boolean;
    /**
     * Specifies whether the service areas from different facilities can overlap each other.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overlapPolygons)
     */
    overlapPolygons: boolean;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overrides)
     */
    overrides: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#pointBarriers)
     */
    pointBarriers: Collection<PointBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polygonBarriers)
     */
    polygonBarriers: Collection<PolygonBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polylineBarriers)
     */
    polylineBarriers: Collection<PolylineBarrier> | DataLayer | FeatureSet | NetworkFeatureSet | NetworkUrl;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#preserveObjectID)
     */
    preserveObjectID: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#restrictionAttributes)
     */
    restrictionAttributes: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Specifies how U-Turns should be handled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#restrictUTurns)
     */
    restrictUTurns: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Determines if facilities will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnFacilities)
     */
    returnFacilities: boolean;
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPointBarriers)
     */
    returnPointBarriers: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers: boolean;
    /**
     * Specifies whether the service area lines should be split at break values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#splitLinesAtBreaks)
     */
    splitLinesAtBreaks: boolean;
    /**
     * Specifies whether multiple service areas around a facility are created as disks or rings.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#splitPolygonsAtBreaks)
     */
    splitPolygonsAtBreaks: boolean;
    /**
     * Specify the time and date to depart from or arrive at incidents or facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDay)
     */
    timeOfDay: Date | "now";
    /**
     * Specify the time zone or zones of the [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDay) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDayIsUTC)
     */
    timeOfDayIsUTC: boolean;
    /**
     * Specifies whether the direction of travel used to generate the service area polygons is toward or away from the facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#travelDirection)
     */
    travelDirection: "from-facility" | "to-facility";
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#travelMode)
     */
    travelMode: TravelMode;
    /**
     * Specifies whether the service areas are trimmed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimOuterPolygon)
     */
    trimOuterPolygon: boolean;
    /**
     * The property defines the distance that can be reached from the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistance)
     */
    trimPolygonDistance: number;
    /**
     * Specifies the units of the value specified [trimPolygonDistance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistance).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistanceUnits)
     */
    trimPolygonDistanceUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#useHierarchy)
     */
    useHierarchy: boolean;

    static fromJSON(json: any): ServiceAreaParameters;
  }

  interface ServiceAreaParametersProperties {
    /**
     * Use this property to specify whether the operation should accumulate values other than the value specified for [impedanceAttribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#impedanceAttribute).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#accumulateAttributes)
     */
    accumulateAttributes?: (
      | "kilometers"
      | "miles"
      | "minutes"
      | "travel-time"
      | "truck-minutes"
      | "truck-travel-time"
      | "walk-time"
      | string
    )[];
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#apiKey)
     */
    apiKey?: string;
    /**
     * Use this property to specify additional values required by an attribute or restriction, such as to specify whether the restriction prohibits, avoids, or prefers travel on restricted roads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#attributeParameterValues)
     */
    attributeParameterValues?: ServiceAreaParametersAttributeParameterValue[];
    /**
     * Use this parameter to specify the size and number of service areas to generate for each facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#defaultBreaks)
     */
    defaultBreaks?: number[];
    /**
     * An array of network source names to not use when generating polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#excludeSourcesFromPolygons)
     */
    excludeSourcesFromPolygons?: string[];
    /**
     * The input locations around which service areas are generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#facilities)
     */
    facilities?:
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecisionM)
     */
    geometryPrecisionM?: number;
    /**
     * Use this property to specify the number of decimal places in the response geometries returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#geometryPrecisionZ)
     */
    geometryPrecisionZ?: number;
    /**
     * Specify whether invalid input locations should be ignored when finding the best solution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#ignoreInvalidLocations)
     */
    ignoreInvalidLocations?: boolean;
    /**
     * Specifies the impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#impedanceAttribute)
     */
    impedanceAttribute?: string;
    /**
     * Use this parameter to specify whether the service areas from different facilities that have the same break value should be joined together or split at break values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#mergeSimilarPolygonRanges)
     */
    mergeSimilarPolygonRanges?: boolean;
    /**
     * Use this property to specify by how much you want to simplify the route geometry returned by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecision)
     */
    outputGeometryPrecision?: number;
    /**
     * Use this property to specify the units for the value specified for the [outputGeometryPrecision](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecision) parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputGeometryPrecisionUnits)
     */
    outputGeometryPrecisionUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Use this property to specify the type of route features that are output by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputLines)
     */
    outputLines?: "none" | "straight" | "true-shape" | "true-shape-with-measure";
    /**
     * This parameter sets whether service area polygons should be returned and to what detail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outputPolygons)
     */
    outputPolygons?: "none" | "simplified" | "detailed";
    /**
     * Use this property to specify the spatial reference of the geometries, such as line or point features, returned by a solve operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#outSpatialReference)
     */
    outSpatialReference?: SpatialReferenceProperties;
    /**
     * Specifies whether the service area lines from different facilities can overlap each other.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overlapLines)
     */
    overlapLines?: boolean;
    /**
     * Specifies whether the service areas from different facilities can overlap each other.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overlapPolygons)
     */
    overlapPolygons?: boolean;
    /**
     * Specify additional settings that can influence the behavior of the solver when finding solutions for the network analysis problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#overrides)
     */
    overrides?: any;
    /**
     * Use this property to specify one or more points that will act as temporary restrictions or represent additional time or distance that may be required to travel on the underlying streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#pointBarriers)
     */
    pointBarriers?:
      | CollectionProperties<PointBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify polygons that either completely restrict travel or proportionately scale the time or distance required to travel on the streets intersected by the polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polygonBarriers)
     */
    polygonBarriers?:
      | CollectionProperties<PolygonBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify one or more lines that prohibit travel anywhere the lines intersect the streets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polylineBarriers)
     */
    polylineBarriers?:
      | CollectionProperties<PolylineBarrierProperties>
      | (DataLayerProperties & { type: "layer" })
      | FeatureSetProperties
      | NetworkFeatureSetProperties
      | NetworkUrlProperties;
    /**
     * Use this property to specify if the Object IDs specified for input locations such as stops or barriers should be preserved when the input locations are returned as output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#preserveObjectID)
     */
    preserveObjectID?: boolean;
    /**
     * Use this property to specify which restrictions should be honored by the operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#restrictionAttributes)
     */
    restrictionAttributes?: (
      | "any-hazmat-prohibited"
      | "avoid-carpool-roads"
      | "avoid-express-lanes"
      | "avoid-ferries"
      | "avoid-gates"
      | "avoid-limited-access-roads"
      | "avoid-private-roads"
      | "avoid-roads-unsuitable-for-pedestrians"
      | "avoid-stairways"
      | "avoid-toll-roads"
      | "avoid-toll-roads-for-trucks"
      | "avoid-truck-restricted-roads"
      | "avoid-unpaved-roads"
      | "axle-count-restriction"
      | "driving-a-bus"
      | "driving-a-taxi"
      | "driving-a-truck"
      | "driving-an-automobile"
      | "driving-an-emergency-vehicle"
      | "height-restriction"
      | "kingpin-to-rear-axle-length-restriction"
      | "length-restriction"
      | "preferred-for-pedestrians"
      | "riding-a-motorcycle"
      | "roads-under-construction-prohibited"
      | "semi-or-tractor-with-one-or-more-trailers-prohibited"
      | "single-axle-vehicles-prohibited"
      | "tandem-axle-vehicles-prohibited"
      | "through-traffic-prohibited"
      | "truck-with-trailers-restriction"
      | "use-preferred-hazmat-routes"
      | "use-preferred-truck-routes"
      | "walking"
      | "weight-restriction"
      | string
    )[];
    /**
     * Specifies how U-Turns should be handled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#restrictUTurns)
     */
    restrictUTurns?: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
    /**
     * Determines if facilities will be returned by the service.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnFacilities)
     */
    returnFacilities?: boolean;
    /**
     * Specify whether [point barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#pointBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPointBarriers)
     */
    returnPointBarriers?: boolean;
    /**
     * Specify whether [polygon barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polygonBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPolygonBarriers)
     */
    returnPolygonBarriers?: boolean;
    /**
     * Specify whether [polyline barriers](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#polylineBarriers) will be returned by the routing operation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#returnPolylineBarriers)
     */
    returnPolylineBarriers?: boolean;
    /**
     * Specifies whether the service area lines should be split at break values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#splitLinesAtBreaks)
     */
    splitLinesAtBreaks?: boolean;
    /**
     * Specifies whether multiple service areas around a facility are created as disks or rings.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#splitPolygonsAtBreaks)
     */
    splitPolygonsAtBreaks?: boolean;
    /**
     * Specify the time and date to depart from or arrive at incidents or facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDay)
     */
    timeOfDay?: DateProperties | "now";
    /**
     * Specify the time zone or zones of the [timeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDay) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#timeOfDayIsUTC)
     */
    timeOfDayIsUTC?: boolean;
    /**
     * Specifies whether the direction of travel used to generate the service area polygons is toward or away from the facilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#travelDirection)
     */
    travelDirection?: "from-facility" | "to-facility";
    /**
     * A travel mode represents a means of transportation, such as driving or walking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#travelMode)
     */
    travelMode?: TravelModeProperties;
    /**
     * Specifies whether the service areas are trimmed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimOuterPolygon)
     */
    trimOuterPolygon?: boolean;
    /**
     * The property defines the distance that can be reached from the network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistance)
     */
    trimPolygonDistance?: number;
    /**
     * Specifies the units of the value specified [trimPolygonDistance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistance).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#trimPolygonDistanceUnits)
     */
    trimPolygonDistanceUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Specify whether hierarchy should be used when finding the shortest paths.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#useHierarchy)
     */
    useHierarchy?: boolean;
  }

  /**
   * An object describing the parameter values for the [attributeParameterValues](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#attributeParameterValues) property of [ServiceAreaParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#AttributeParameterValue)
   */
  export interface ServiceAreaParametersAttributeParameterValue {
    /**
     * The name of the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#AttributeParameterValue)
     */
    attributeName: string;
    /**
     * The name of the parameter associated with the restriction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#AttributeParameterValue)
     */
    parameterName: string;
    /**
     * The value for `parameterName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaParameters.html#AttributeParameterValue)
     */
    value: string | number;
  }

  export interface ServiceAreaSolveResult extends Accessor, JSONSupport {}

  export class ServiceAreaSolveResult {
    /**
     * The result from [serviceArea](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-serviceArea.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html)
     */

    constructor(properties?: ServiceAreaSolveResultProperties);

    /**
     * This provides access to the output facilities from a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#facilities)
     */
    facilities: FeatureSet;
    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#messages)
     */
    messages: NAMessage[];
    /**
     * A set of features representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#pointBarriers)
     */
    pointBarriers: FeatureSet;
    /**
     * A set of features representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#polygonBarriers)
     */
    polygonBarriers: FeatureSet;
    /**
     * A set of features representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#polylineBarriers)
     */
    polylineBarriers: FeatureSet;
    /**
     * A feature set containing polygon features that represent areas that can be reached from the input locations with a given travel time, travel distance, or travel cost.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#serviceAreaPolygons)
     */
    serviceAreaPolygons: FeatureSet;
    /**
     * A feature set containing linear features and covers the streets, or network edges, that can be reached within the given time, distance, or other travel-cost cutoff.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#serviceAreaPolylines)
     */
    serviceAreaPolylines: FeatureSet;

    static fromJSON(json: any): ServiceAreaSolveResult;
  }

  interface ServiceAreaSolveResultProperties {
    /**
     * This provides access to the output facilities from a closest facility analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#facilities)
     */
    facilities?: FeatureSetProperties;
    /**
     * An array of processing [messages](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-NAMessage.html) generated by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#messages)
     */
    messages?: NAMessageProperties[];
    /**
     * A set of features representing point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#pointBarriers)
     */
    pointBarriers?: FeatureSetProperties;
    /**
     * A set of features representing polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#polygonBarriers)
     */
    polygonBarriers?: FeatureSetProperties;
    /**
     * A set of features representing polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#polylineBarriers)
     */
    polylineBarriers?: FeatureSetProperties;
    /**
     * A feature set containing polygon features that represent areas that can be reached from the input locations with a given travel time, travel distance, or travel cost.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#serviceAreaPolygons)
     */
    serviceAreaPolygons?: FeatureSetProperties;
    /**
     * A feature set containing linear features and covers the streets, or network edges, that can be reached within the given time, distance, or other travel-cost cutoff.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-ServiceAreaSolveResult.html#serviceAreaPolylines)
     */
    serviceAreaPolylines?: FeatureSetProperties;
  }

  export interface StatisticDefinition extends Accessor, JSONSupport {}

  export class StatisticDefinition {
    /**
     * This class defines the parameters for querying a layer or layer view for statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html)
     */

    constructor(properties?: StatisticDefinitionProperties);

    /**
     * Defines the field for which statistics will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#onStatisticField)
     */
    onStatisticField: string;
    /**
     * Specifies the output field name for the requested statistic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#outStatisticFieldName)
     */
    outStatisticFieldName: string;
    /**
     * The parameters for [percentile statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticParameters)
     */
    statisticParameters: StatisticDefinitionStatisticParameters;
    /**
     * Defines the type of statistic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType)
     */
    statisticType:
      | "count"
      | "sum"
      | "min"
      | "max"
      | "avg"
      | "stddev"
      | "var"
      | "exceedslimit"
      | "percentile-continuous"
      | "percentile-discrete"
      | "envelope-aggregate"
      | "centroid-aggregate"
      | "convex-hull-aggregate";

    /**
     * Creates a deep clone of StatisticDefinition object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#clone)
     */
    clone(): StatisticDefinition;

    static fromJSON(json: any): StatisticDefinition;
  }

  interface StatisticDefinitionProperties {
    /**
     * Defines the field for which statistics will be calculated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#onStatisticField)
     */
    onStatisticField?: string;
    /**
     * Specifies the output field name for the requested statistic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#outStatisticFieldName)
     */
    outStatisticFieldName?: string;
    /**
     * The parameters for [percentile statistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticParameters)
     */
    statisticParameters?: StatisticDefinitionStatisticParameters;
    /**
     * Defines the type of statistic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticType)
     */
    statisticType?:
      | "count"
      | "sum"
      | "min"
      | "max"
      | "avg"
      | "stddev"
      | "var"
      | "exceedslimit"
      | "percentile-continuous"
      | "percentile-discrete"
      | "envelope-aggregate"
      | "centroid-aggregate"
      | "convex-hull-aggregate";
  }

  export interface StatisticDefinitionStatisticParameters {
    /**
     * Percentile value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticParameters)
     */
    value: number;
    /**
     * Specify `ASC` (ascending) or `DESC` (descending) to control the order of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-StatisticDefinition.html#statisticParameters)
     */
    orderBy?: "ASC" | "DESC";
  }

  export interface Stop extends Accessor, JSONSupport {}

  export class Stop {
    /**
     * A stop respresents the start, end, or midpoint of a route in [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#stops) or [RouteParameters](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-RouteParameters.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html)
     */

    constructor(properties?: StopProperties);

    /**
     * The date and time value indicating the arrival time at the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#arriveTime)
     */
    arriveTime: Date;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#arriveTimeOffset)
     */
    arriveTimeOffset: number;
    /**
     * Specify the direction a vehicle may arrive at and depart from the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#curbApproach)
     */
    curbApproach: "either-side-of-vehicle" | "right-side-of-vehicle" | "left-side-of-vehicle" | "no-u-turn";
    /**
     * The date and time value indicating the departure time from the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#departTime)
     */
    departTime: Date;
    /**
     * The local time offset in minutes for the departure time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#departTimeOffset)
     */
    departTimeOffset: number;
    /**
     * The point location of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#geometry)
     */
    geometry: Point;

    locationType: "stop" | "waypoint" | "break";
    /**
     * The name of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#name)
     */
    name: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * If the findBestSequence parameter is set to false, the output routes will visit the stops in the order you specify with this attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#sequence)
     */
    sequence: number;
    /**
     * Indicates the status of the point with respect to its location on the network and the outcome of the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#status)
     */
    status:
      | "ok"
      | "not-located"
      | "network-element-not-located"
      | "element-not-traversable"
      | "invalid-field-values"
      | "not-reached"
      | "time-window-violation"
      | "not-located-on-closest";
    /**
     * The latest time the route can visit the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#timeWindowEnd)
     */
    timeWindowEnd: Date;
    /**
     * The earliest time the route can visit the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#timeWindowStart)
     */
    timeWindowStart: Date;

    readonly type: "stop";

    /**
     * Creates a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) from the [Stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#toGraphic)
     */
    toGraphic(): Graphic;

    /**
     * Creates a [Stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html) from the parsed [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#fromGraphic)
     */
    static fromGraphic(graphic?: Graphic): Stop;
    static fromJSON(json: any): Stop;
  }

  interface StopProperties {
    /**
     * The date and time value indicating the arrival time at the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#arriveTime)
     */
    arriveTime?: DateProperties;
    /**
     * The local time offset (in minutes) for the arrival time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#arriveTimeOffset)
     */
    arriveTimeOffset?: number;
    /**
     * Specify the direction a vehicle may arrive at and depart from the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#curbApproach)
     */
    curbApproach?: "either-side-of-vehicle" | "right-side-of-vehicle" | "left-side-of-vehicle" | "no-u-turn";
    /**
     * The date and time value indicating the departure time from the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#departTime)
     */
    departTime?: DateProperties;
    /**
     * The local time offset in minutes for the departure time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#departTimeOffset)
     */
    departTimeOffset?: number;
    /**
     * The point location of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#geometry)
     */
    geometry?: PointProperties;

    locationType?: "stop" | "waypoint" | "break";
    /**
     * The name of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#name)
     */
    name?: string;
    /**
     * The template for displaying content in a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when the graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * If the findBestSequence parameter is set to false, the output routes will visit the stops in the order you specify with this attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#sequence)
     */
    sequence?: number;
    /**
     * Indicates the status of the point with respect to its location on the network and the outcome of the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#status)
     */
    status?:
      | "ok"
      | "not-located"
      | "network-element-not-located"
      | "element-not-traversable"
      | "invalid-field-values"
      | "not-reached"
      | "time-window-violation"
      | "not-located-on-closest";
    /**
     * The latest time the route can visit the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#timeWindowEnd)
     */
    timeWindowEnd?: DateProperties;
    /**
     * The earliest time the route can visit the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Stop.html#timeWindowStart)
     */
    timeWindowStart?: DateProperties;
  }

  export interface TopFeaturesQuery extends Accessor, JSONSupport, QueryMixin {}

  export class TopFeaturesQuery {
    /**
     * This class defines parameters for executing [top features queries](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#queryTopFeatures) from a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html)
     */

    constructor(properties?: TopFeaturesQueryProperties);

    /**
     * Specifies the number of decimal places for geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#geometryPrecision)
     */
    geometryPrecision: number;
    /**
     * The maximum distance in units of [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#outSpatialReference) used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#maxAllowableOffset)
     */
    maxAllowableOffset: number;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#num)
     */
    num: number;
    /**
     * An array of ObjectIDs to be used to query for features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#objectIds)
     */
    objectIds: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#orderByFields)
     */
    orderByFields: string[];
    /**
     * Attribute fields to include in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#outFields)
     */
    outFields: string[];
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry)
     */
    returnGeometry: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnM)
     */
    returnM: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnZ)
     */
    returnZ: boolean;
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#start)
     */
    start: number;
    /**
     * A time extent for a temporal query against [time-aware layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The `topFilter` parameter is used to set the [groupByFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#groupByFields), [orderByFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#orderByFields), and [topCount](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#topCount) criteria used in generating the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#topFilter)
     */
    topFilter: TopFilter;

    /**
     * Creates a deep clone of TopFeaturesQuery object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#clone)
     */
    clone(): TopFeaturesQuery;

    static fromJSON(json: any): TopFeaturesQuery;
  }

  interface TopFeaturesQueryProperties extends QueryMixinProperties {
    /**
     * Specifies the number of decimal places for geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#geometryPrecision)
     */
    geometryPrecision?: number;
    /**
     * The maximum distance in units of [outSpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#outSpatialReference) used for generalizing geometries returned by the query operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#maxAllowableOffset)
     */
    maxAllowableOffset?: number;
    /**
     * The number of features to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#num)
     */
    num?: number;
    /**
     * An array of ObjectIDs to be used to query for features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#objectIds)
     */
    objectIds?: (number | string)[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#orderByFields)
     */
    orderByFields?: string[];
    /**
     * Attribute fields to include in the [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#outFields)
     */
    outFields?: string[];
    /**
     * If `true`, each feature in the returned [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) includes the geometry.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry)
     */
    returnGeometry?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry) is `true`, then m-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnM)
     */
    returnM?: boolean;
    /**
     * If `true`, and [returnGeometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnGeometry) is `true`, then z-values are included in the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#returnZ)
     */
    returnZ?: boolean;
    /**
     * The zero-based index indicating where to begin retrieving features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#start)
     */
    start?: number;
    /**
     * A time extent for a temporal query against [time-aware layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#timeInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The `topFilter` parameter is used to set the [groupByFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#groupByFields), [orderByFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#orderByFields), and [topCount](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#topCount) criteria used in generating the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFeaturesQuery.html#topFilter)
     */
    topFilter?: TopFilterProperties;
  }

  export interface TopFilter extends Accessor, JSONSupport {}

  export class TopFilter {
    /**
     * This class defines the top filter parameters for executing top features queries for features from a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html)
     */

    constructor(properties?: TopFilterProperties);

    /**
     * When one or more field names are provided in this property, the output result will be grouped based on unique values from those fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#groupByFields)
     */
    groupByFields: string[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#orderByFields)
     */
    orderByFields: string[];
    /**
     * Defines the number of features to be returned from the top features query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#topCount)
     */
    topCount: number;

    /**
     * Creates a deep clone of TopFilter object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#clone)
     */
    clone(): TopFilter;

    static fromJSON(json: any): TopFilter;
  }

  interface TopFilterProperties {
    /**
     * When one or more field names are provided in this property, the output result will be grouped based on unique values from those fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#groupByFields)
     */
    groupByFields?: string[];
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#orderByFields)
     */
    orderByFields?: string[];
    /**
     * Defines the number of features to be returned from the top features query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TopFilter.html#topCount)
     */
    topCount?: number;
  }

  export interface TravelMode extends Accessor, JSONSupport, Clonable {}

  export class TravelMode {
    /**
     * A TravelMode is a set of characteristics that define how an object like a vehicle, bicycle, or pedestrian moves along a street network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html)
     */

    constructor(properties?: TravelModeProperties);

    /**
     * Lists the parameterized attributes used by the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#attributeParameterValues)
     */
    attributeParameterValues: TravelModeAttributeParameterValues[];
    /**
     * A short text description of the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#description)
     */
    description: string;
    /**
     * Indicates the distance-based cost attribute for reporting directions and for solving vehicle routing problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#distanceAttributeName)
     */
    distanceAttributeName: string;
    /**
     * The unique identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#id)
     */
    id: string;
    /**
     * The network cost attribute used as impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#impedanceAttributeName)
     */
    impedanceAttributeName: string;
    /**
     * The unique name of the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#name)
     */
    name: string;
    /**
     * The list of the restriction attributes used when solving network analysis problems with this travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#restrictionAttributeNames)
     */
    restrictionAttributeNames: string[];
    /**
     * Specifies whether the travel mode generalizes the geometry of analysis results and by how much.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationTolerance)
     */
    simplificationTolerance: number;
    /**
     * The linear units associated with [simplificationTolerance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationTolerance).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationToleranceUnits)
     */
    simplificationToleranceUnits: LengthUnit | "decimal-degrees" | "points" | "unknown";
    /**
     * Indicates the time-based cost attribute for reporting directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#timeAttributeName)
     */
    timeAttributeName: string;
    /**
     * Indicates the category of travel or vehicle represented by this travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#type)
     */
    type: "automobile" | "truck" | "walk" | "other";
    /**
     * Indicates whether the travel mode uses a hierarchy attribute while performing the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#useHierarchy)
     */
    useHierarchy: boolean;
    /**
     * Indicates how the U-turns at junctions that could occur during network traversal are handled by the solver.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#uturnAtJunctions)
     */
    uturnAtJunctions: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";

    static fromJSON(json: any): TravelMode;
  }

  interface TravelModeProperties {
    /**
     * Lists the parameterized attributes used by the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#attributeParameterValues)
     */
    attributeParameterValues?: TravelModeAttributeParameterValues[];
    /**
     * A short text description of the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#description)
     */
    description?: string;
    /**
     * Indicates the distance-based cost attribute for reporting directions and for solving vehicle routing problems.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#distanceAttributeName)
     */
    distanceAttributeName?: string;
    /**
     * The unique identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#id)
     */
    id?: string;
    /**
     * The network cost attribute used as impedance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#impedanceAttributeName)
     */
    impedanceAttributeName?: string;
    /**
     * The unique name of the travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#name)
     */
    name?: string;
    /**
     * The list of the restriction attributes used when solving network analysis problems with this travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#restrictionAttributeNames)
     */
    restrictionAttributeNames?: string[];
    /**
     * Specifies whether the travel mode generalizes the geometry of analysis results and by how much.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationTolerance)
     */
    simplificationTolerance?: number;
    /**
     * The linear units associated with [simplificationTolerance](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationTolerance).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#simplificationToleranceUnits)
     */
    simplificationToleranceUnits?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
      | "decimal-degrees"
      | "points"
      | "unknown";
    /**
     * Indicates the time-based cost attribute for reporting directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#timeAttributeName)
     */
    timeAttributeName?: string;
    /**
     * Indicates the category of travel or vehicle represented by this travel mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#type)
     */
    type?: "automobile" | "truck" | "walk" | "other";
    /**
     * Indicates whether the travel mode uses a hierarchy attribute while performing the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#useHierarchy)
     */
    useHierarchy?: boolean;
    /**
     * Indicates how the U-turns at junctions that could occur during network traversal are handled by the solver.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#uturnAtJunctions)
     */
    uturnAtJunctions?: "allow-backtrack" | "at-dead-ends-only" | "no-backtrack" | "at-dead-ends-and-intersections";
  }

  export interface TravelModeAttributeParameterValues {
    /**
     * The name of the attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#attributeParameterValues)
     */
    attributeName: string;
    /**
     * The parameter name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#attributeParameterValues)
     */
    parameterName: string;
    /**
     * The parameter value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html#attributeParameterValues)
     */
    value: string | number;
  }

  export interface TrimExtendParameters extends Accessor, JSONSupport {}

  export class TrimExtendParameters {
    /**
     * Used to set the parameters for the [geometryService.trimExtend](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-geometryService.html#trimExtend) operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html)
     */

    constructor(properties?: TrimExtendParametersProperties);

    /**
     * A flag used with the `trimExtend` operation.
     *
     * @default "default-curve-extension"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#extendHow)
     */
    extendHow:
      | "default-curve-extension"
      | "relocate-ends"
      | "keep-end-attributes"
      | "no-end-attributes"
      | "no-extend-at-from"
      | "no-extend-at-to";
    /**
     * The array of polylines to trim or extend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#polylines)
     */
    polylines: Polyline[];
    /**
     * A polyline used as a guide for trimming or extending input polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#trimExtendTo)
     */
    trimExtendTo: Polyline;

    static fromJSON(json: any): TrimExtendParameters;
  }

  interface TrimExtendParametersProperties {
    /**
     * A flag used with the `trimExtend` operation.
     *
     * @default "default-curve-extension"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#extendHow)
     */
    extendHow?:
      | "default-curve-extension"
      | "relocate-ends"
      | "keep-end-attributes"
      | "no-end-attributes"
      | "no-extend-at-from"
      | "no-extend-at-to";
    /**
     * The array of polylines to trim or extend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#polylines)
     */
    polylines?: PolylineProperties[];
    /**
     * A polyline used as a guide for trimming or extending input polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TrimExtendParameters.html#trimExtendTo)
     */
    trimExtendTo?: PolylineProperties;
  }

  /**
   * Represents [symbol service](https://developers.arcgis.com/rest/services-reference/enterprise/symbol-server.htm) resources exposed by the ArcGIS REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html)
   */
  interface symbolService {
    /**
     * Converts a [Scalable Vector Graphic](https://pro.arcgis.com/en/pro-app/latest/help/mapping/layer-properties/scalable-vector-graphics-support.htm) (SVG) to a [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#generateSymbol)
     */
    generateSymbol(
      url: string,
      params: GenerateSymbolParameters,
      requestOptions?: RequestOptions,
    ): Promise<GenerateSymbolResponse>;
  }

  export const symbolService: symbolService;

  /**
   * Input parameters for the [generateSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#generateSymbol) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#GenerateSymbolParameters)
   */
  export interface GenerateSymbolParameters {
    /**
     * The SVG image to convert.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#GenerateSymbolParameters)
     */
    svgImage?: string | FormData | HTMLFormElement;
  }

  /**
   * The response of the [generateSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#generateSymbol) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#GenerateSymbolResponse)
   */
  export interface GenerateSymbolResponse {
    /**
     * The CIMSymbol created from the [generateSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#generateSymbol) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-symbolService.html#GenerateSymbolResponse)
     */
    symbol: CIMSymbol;
  }

  /**
   * Function that suggests a [fixedBinLevel](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fixedBinLevel) in a [FeatureReductionBinning](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html) visualization.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-binLevel.html)
   */
  interface binLevel {
    /**
     * Generates a suggested [fixedBinLevel](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#fixedBinLevel) for a binning visualization based on the view scale at the time this method is called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-binLevel.html#binLevel)
     */
    binLevel(params: binLevelBinLevelParams): Promise<number>;
  }

  export const binLevel: binLevel;

  export interface binLevelBinLevelParams {
    /**
     * The [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) where the layer will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
  }

  /**
   * Function for determining suggested [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#minScale) and [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#maxScale) scale ranges for an input layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html)
   */
  interface scaleRange {
    /**
     * Generates a suggested scale range (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    scaleRange(params: scaleRangeScaleRangeParams): Promise<ScaleRangeResult>;
  }

  const __scaleRangeMapped: scaleRange;
  export const scaleRange: typeof __scaleRangeMapped.scaleRange;

  /**
   * Suggested `min` and `max` scales to apply to the input layer for the [scaleRange()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#ScaleRangeResult)
   */
  export interface ScaleRangeResult {
    /**
     * The suggested [minScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#minScale) to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#ScaleRangeResult)
     */
    minScale: number;
    /**
     * The suggested [maxScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#maxScale) to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#ScaleRangeResult)
     */
    maxScale: number;
  }

  export interface scaleRangeScaleRangeParams {
    /**
     * The layer for which to generate a suggested min/max scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | OGCFeatureLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) where the layer will be rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The number of features in the `layer` to sample for spatial statistics.
     *
     * @default 500
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    sampleSize?: number;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    filter?: FeatureFilter;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-scaleRange.html#scaleRange)
     */
    signal?: AbortSignal;
  }

  /**
   * Function for determining the suggested `minSize` and `maxSize` of a [scale-dependent size visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html)
   */
  interface sizeRange {
    /**
     * Generates a suggested size range (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange)
     */
    sizeRange(params: sizeRangeSizeRangeParams): Promise<SizeRangeResult>;
  }

  const __sizeRangeMapped: sizeRange;
  export const sizeRange: typeof __sizeRangeMapped.sizeRange;

  /**
   * The suggested `minSize` and `maxSize` [size stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentStops) generated from the [sizeRange()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange) function to apply to the [scale-dependent size visual variable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#ScaleDependentIcons).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#SizeRangeResult)
   */
  export interface SizeRangeResult {
    /**
     * The suggested sizes of the smallest symbols at various scales in a layer with a [size visualization](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#SizeRangeResult)
     */
    minSize: ScaleDependentStops;
    /**
     * The suggested sizes of the largest symbols at various scales in a layer with a [size visualization](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#SizeRangeResult)
     */
    maxSize: ScaleDependentStops;
  }

  export interface sizeRangeSizeRangeParams {
    /**
     * The layer for which to generate a suggested min/max size range based on scale for a size visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | OGCFeatureLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) where the layer will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange)
     */
    filter?: FeatureFilter;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-heuristics-sizeRange.html#sizeRange)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains a helper method for generating default labels to be set on a layer's [binning configuration](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html)
   */
  interface bins {
    /**
     * Generates default [labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo) schemes to be set on a FeatureLayer's [featureReduction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#featureReduction) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#getLabelSchemes)
     */
    getLabelSchemes(params: binsGetLabelSchemesParams): Promise<Schemes>;
  }

  export const bins: bins;

  export interface binsGetLabelSchemesParams {
    /**
     * The point layer that has or will have binning enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#getLabelSchemes)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer;
    /**
     * The name of the [AggregateField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#name) to use in the primary label scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#getLabelSchemes)
     */
    field?: string;
    /**
     * The renderer to set on `layer.featureReduction.renderer` when binning is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#getLabelSchemes)
     */
    renderer?: Renderer;
  }

  /**
   * Contains suggested labelingInfo to be set on the layer's [featureReduction.labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Scheme)
   */
  export interface Scheme {
    /**
     * The name of the generated binning labeling scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Scheme)
     */
    name: string;
    /**
     * The name of the aggregate field used in the labeling scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Scheme)
     */
    fieldName: string;
    /**
     * An array of LabelClass objects to set on the layer's [featureReduction.labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html#labelingInfo) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Scheme)
     */
    labelingInfo: LabelClass[];
  }

  /**
   * The return object of the [getLabelSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#getLabelSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Schemes)
   */
  export interface Schemes {
    /**
     * Includes the primary labeling scheme suggested for the input layer's bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Schemes)
     */
    primaryScheme: Scheme;
    /**
     * Includes secondary labeling schemes suggested for the input layer's bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-bins.html#Schemes)
     */
    secondarySchemes: Scheme[];
  }

  /**
   * This object contains a helper method for generating default labels to be set on a layer's [cluster configuration](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html)
   */
  interface clusters {
    /**
     * Generates default [labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo) schemes to be set on a FeatureLayer's [featureReduction](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#featureReduction) configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#getLabelSchemes)
     */
    getLabelSchemes(params: clustersGetLabelSchemesParams): Promise<clustersSchemes>;
  }

  export const clusters: clusters;

  export interface clustersGetLabelSchemesParams {
    /**
     * The point layer that has or will have clustering enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#getLabelSchemes)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer;
    /**
     * The view where the input layer will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
    /**
     * The name of the [AggregateField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-AggregateField.html#name) to use in the primary label scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#getLabelSchemes)
     */
    field?: string;
    /**
     * The renderer to set on the input layer when clustering is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#getLabelSchemes)
     */
    renderer?: Renderer;
  }

  /**
   * Contains a suggested labelingInfo to be set on the layer's [featureReduction.labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Scheme)
   */
  export interface clustersScheme {
    /**
     * The name of the generated cluster labeling scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Scheme)
     */
    name: string;
    /**
     * The name of the aggregate field the labeling scheme is based on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Scheme)
     */
    fieldName: string;
    /**
     * An array of LabelClass objects to set on the layer's [featureReduction.labelingInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#labelingInfo) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Scheme)
     */
    labelingInfo: LabelClass[];
    /**
     * The suggested [featureReduction.clusterMinSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#clusterMinSize) required to fit labels within clusters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Scheme)
     */
    clusterMinSize: number;
  }

  /**
   * The return object of the [getLabelSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#getLabelSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Schemes)
   */
  export interface clustersSchemes {
    /**
     * Includes the primary labeling scheme suggested for the input layer's clusters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Schemes)
     */
    primaryScheme: clustersScheme;
    /**
     * Includes secondary labeling schemes suggested for the input layer's clusters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-labels-clusters.html#Schemes)
     */
    secondarySchemes: clustersScheme[];
  }

  /**
   * This object contains helper methods for generating popup templates to be set on a layer's [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupTemplate).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-clusters.html)
   */
  interface popupClusters {
    /**
     * Returns one or more suggested default [popupTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for a given layer's [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html#popupTemplate) configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-clusters.html#getTemplates)
     */
    getTemplates(params: clustersGetTemplatesParams): Promise<Templates>;
  }

  export const popupClusters: popupClusters;

  export interface clustersGetTemplatesParams {
    /**
     * The point layer that is or will be clustered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-clusters.html#getTemplates)
     */
    layer: FeatureLayer | CSVLayer | GeoJSONLayer | WFSLayer | OGCFeatureLayer | StreamLayer | OrientedImageryLayer;
    /**
     * Specify the renderer to be used on the layer when `featureReduction` is enabled if it will be different than the renderer already set on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-clusters.html#getTemplates)
     */
    renderer?: Renderer;
  }

  /**
   * This object contains helper methods for generating popup templates to be set on a [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#popupTemplate).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html)
   */
  interface templates {
    /**
     * Returns one or more suggested [popupTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for a given layer based on its renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#getTemplates)
     */
    getTemplates(params: templatesGetTemplatesParams): Promise<Templates>;
  }

  export const templates: templates;

  /**
   * Defines a suggested PopupTemplate with a given name and title describing the content and purpose of the PopupTemplate.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Template)
   */
  export interface Template {
    /**
     * The name of the auto-generated PopupTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Template)
     */
    name: string;
    /**
     * The title of the PopupTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Template)
     */
    title: string;
    /**
     * The suggested PopupTemplate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Template)
     */
    value: PopupTemplate;
  }

  /**
   * The return object of the [getTemplates()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#getTemplates) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Templates)
   */
  export interface Templates {
    /**
     * Includes the primary PopupTemplate suggested for the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Templates)
     */
    primaryTemplate: Template;
    /**
     * Includes secondary PopupTemplates that may be applied to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#Templates)
     */
    secondaryTemplates: Template[];
  }

  export interface templatesGetTemplatesParams {
    /**
     * The layer to which the suggested popup templates can be applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#getTemplates)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * Specify the renderer to be used on the layer when if it will be different than the renderer already set on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-popup-templates.html#getTemplates)
     */
    renderer?: Renderer;
  }

  /**
   * This object contains helper methods for generating class breaks visualizations for raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html)
   */
  interface classBreaks {
    /**
     * Generates a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html) used to render imagery data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    createRenderer(params: classBreaksCreateRendererParams): Promise<RasterClassBreaksResult>;
  }

  export const classBreaks: classBreaks;

  export interface classBreaksCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer | WCSLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * The field whose data will be used in the classification.
     *
     * @default value
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    field?: string;
    /**
     * The classification method used for generating breaks.
     *
     * @default natural-breaks
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    classificationMethod?: "equal-interval" | "natural-breaks" | "quantile" | "standard-deviation" | "defined-interval";
    /**
     * Only applicable to multidimensional datasets where a raster layer can contain more than one variable (such as temperature, humidity, wind speed) with different statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    variableName?: string;
    /**
     * The number of class breaks to generate for the classification.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    numClasses?: number;
    /**
     * The color ramp to apply to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    colorRamp?: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * The colors to apply to each class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    colors?: number[][];
    /**
     * Only applicable when `classificationMethod` is `defined-interval`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    definedInterval?: number;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#RasterClassBreaksResult)
   */
  export interface RasterClassBreaksResult {
    /**
     * The renderer object configured to best match the given basemap and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#RasterClassBreaksResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * This object describes class breaks generated from data in a layer for a given field with a specified classification method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-classBreaks.html#RasterClassBreaksResult)
     */
    classBreaksResult: ClassBreaksResult;
  }

  /**
   * This object contains helper methods for generating a [RasterColormapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html) for raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html)
   */
  interface colormap {
    /**
     * Generates a [RasterColormapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterColormapRenderer.html) used to render imagery data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer)
     */
    createRenderer(params: colormapCreateRendererParams): Promise<RasterColormapResult>;
  }

  export const colormap: colormap;

  export interface colormapCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#RasterColormapResult)
   */
  export interface RasterColormapResult {
    /**
     * The colormap renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-colormap.html#RasterColormapResult)
     */
    renderer: RasterColormapRenderer;
  }

  /**
   * This object contains helper methods for generating a [FlowRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html) for a `Vector-UV` or `Vector-MagDir` [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html)
   */
  interface flow {
    /**
     * Generates a [FlowRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html) to display raster data with stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    createRenderer(params: flowCreateRendererParams): Promise<FlowRendererResult>;
  }

  export const flow: flow;

  export interface flowCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: MapView;
    /**
     * Determines how flow lines will render.
     *
     * @default "flow-line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    theme?: "flow-line" | "wave-front";
    /**
     * When `true`, the color of stream lines will vary depending on the magnitude of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    includeColorVariable?: boolean;
    /**
     * When `true`, the width of stream lines will vary depending on the magnitude of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    includeSizeVariable?: boolean;
    /**
     * When `true`, the opacity of stream lines will vary depending on the magnitude of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    includeOpacityVariable?: boolean;
    /**
     * Provides options for setting a title to the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    legendOptions?: flowCreateRendererParamsLegendOptions;
    /**
     * The flow scheme used to set colors and sizes to the flow lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    flowScheme?: FlowScheme;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * Defines the flow direction of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    flowRepresentation?: "flow-from" | "flow-to";
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface flowCreateRendererParamsLegendOptions {
    /**
     * The title used to represent the data in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer)
     */
    title?: string;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
   */
  export interface FlowRendererResult {
    /**
     * The FlowRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    renderer: FlowRenderer;
    /**
     * The flow scheme used to set colors and sizes to the flow lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    flowScheme: FlowScheme;
    /**
     * Visual variables included in the output renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    visualVariables: VisualVariable[];
    /**
     * Summary statistics of the raster layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal color of the flow lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    basemapTheme: string;
    /**
     * A suggested layer effect to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-flow.html#FlowRendererResult)
     */
    layerEffect: string;
  }

  /**
   * This object contains helper methods for generating an RGB [stretch visualization](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html) for raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html)
   */
  interface rgb {
    /**
     * Generates an RGB [RasterStretchRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html) to render three selected bands into red, green, and blue color channels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    createRenderer(params: rgbCreateRendererParams): Promise<RasterRGBResult>;
  }

  export const rgb: rgb;

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#RasterRGBResult)
   */
  export interface RasterRGBResult {
    /**
     * The RGB RasterStretchRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#RasterRGBResult)
     */
    renderer: RasterStretchRenderer;
    /**
     * The RGB band indexes following the order of red, green, and blue channels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#RasterRGBResult)
     */
    rgbBandIds: number[];
  }

  export interface rgbCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer | WCSLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * A preferred stretch type can be provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    stretchType?: "none" | "min-max" | "standard-deviation" | "histogram-equalization" | "percent-clip" | "sigmoid";
    /**
     * The RGB band indexes following the order of red, green, and blue channels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    rgbBandIds?: number[];
    /**
     * The gamma values to be used if `useGamma` is `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    gamma?: number[];
    /**
     * Indicates whether the `gamma` values should be used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    useGamma?: boolean;
    /**
     * When `true`, calculates the renderer's statistics based on the current display extent and recalculates them as you zoom and pan around the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * Useful in scenarios where an image service does not have statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    estimateStatistics?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains helper methods for generating a [RasterShadedReliefRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html) to render elevation values in raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html)
   */
  interface shadedRelief {
    /**
     * Generates a [RasterShadedReliefRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterShadedReliefRenderer.html) to render elevation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    createRenderer(params: shadedReliefCreateRendererParams): Promise<RasterShadedReliefResult>;
  }

  export const shadedRelief: shadedRelief;

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#RasterShadedReliefResult)
   */
  export interface RasterShadedReliefResult {
    /**
     * The RasterShadedReliefRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#RasterShadedReliefResult)
     */
    renderer: RasterShadedReliefRenderer;
  }

  export interface shadedReliefCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * The preferred hillshade type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    hillshadeType?: "traditional" | "multi-directional";
    /**
     * Applies a constant or adjusted z-factor based on resolution changes.
     *
     * @default none
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    scalingType?: "none" | "adjusted";
    /**
     * The color ramp to apply to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    colorRamp?: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-shadedRelief.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains helper methods for generating a single-band [stretch visualization](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html) for raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html)
   */
  interface stretch {
    /**
     * Generates a [RasterStretchRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-RasterStretchRenderer.html) to render data from a single raster band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    createRenderer(params: stretchCreateRendererParams): Promise<RasterStretchColorrampResult>;
  }

  export const stretch: stretch;

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#RasterStretchColorrampResult)
   */
  export interface RasterStretchColorrampResult {
    /**
     * The RasterStretchRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#RasterStretchColorrampResult)
     */
    renderer: RasterStretchRenderer;
    /**
     * The zero-based index of the band represented by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#RasterStretchColorrampResult)
     */
    bandId: number;
  }

  export interface stretchCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer | WCSLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * A preferred stretch type can be provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    stretchType?: "none" | "min-max" | "standard-deviation" | "histogram-equalization" | "percent-clip" | "sigmoid";
    /**
     * The 0-based index of a selected band.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    bandId?: number;
    /**
     * The color ramp to apply to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    colorRamp?: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * Only applicable to multidimensional datasets where a raster layer can contain more than one variable (such as temperature, humidity, wind speed) with different statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    variableName?: string;
    /**
     * The gamma values to be used if `useGamma` is `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    gamma?: number[];
    /**
     * Indicates whether the `gamma` values should be used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    useGamma?: boolean;
    /**
     * When `true`, calculates the renderer's statistics based on the current display extent and recalculates them as you zoom and pan around the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    dynamicRangeAdjustment?: boolean;
    /**
     * Useful in scenarios where an image service does not have statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    estimateStatistics?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains helper methods for generating a [UniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html) for raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html)
   */
  interface uniqueValue {
    /**
     * Generates a [UniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html) to render thematic imagery.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    createRenderer(params: uniqueValueCreateRendererParams): Promise<RasterUniqueValuesResult>;
  }

  export const uniqueValue: uniqueValue;

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#RasterUniqueValuesResult)
   */
  export interface RasterUniqueValuesResult {
    /**
     * The UniqueValueRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#RasterUniqueValuesResult)
     */
    renderer: UniqueValueRenderer;
    /**
     * The class field name from which the unique value infos were generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#RasterUniqueValuesResult)
     */
    classFieldName: string;
  }

  export interface uniqueValueCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * A preferred class field name used for calculating unique values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    classFieldName?: string;
    /**
     * The colors to apply to each unique value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    colors?: number[][];
    /**
     * The color ramp to apply to the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    colorRamp?: AlgorithmicColorRamp | MultipartColorRamp;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-uniqueValue.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains helper methods for generating a [VectorFieldRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html) for a `Vector-UV` or `Vector-MagDir` [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) or [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html)
   */
  interface vectorField {
    /**
     * Generates a [VectorFieldRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-VectorFieldRenderer.html) to display raster data with vector symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    createRenderer(params: vectorFieldCreateRendererParams): Promise<VectorFieldRendererResult>;
  }

  export const vectorField: vectorField;

  export interface vectorFieldCreateRendererParams {
    /**
     * The input layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    layer: ImageryLayer | ImageryTileLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    rasterFunction?: RasterFunction;
    /**
     * Defines the flow direction of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    flowRepresentation?: "flow-from" | "flow-to";
    /**
     * Defines the origin and direction of rotation depending on how the angle of rotation was measured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    rotationType?: "geographic" | "arithmetic";
    /**
     * The predefined symbol styles used to represent the vector flow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    style?:
      | "beaufort-ft"
      | "beaufort-km"
      | "beaufort-kn"
      | "beaufort-m"
      | "beaufort-mi"
      | "classified-arrow"
      | "ocean-current-kn"
      | "ocean-current-m"
      | "simple-scalar"
      | "single-arrow"
      | "wind-barb";
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#VectorFieldRendererResult)
   */
  export interface VectorFieldRendererResult {
    /**
     * The VectorFieldRenderer renderer to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-vectorField.html#VectorFieldRendererResult)
     */
    renderer: VectorFieldRenderer;
  }

  /**
   * This module provides convenience methods for querying color ramps (arrays of colors) used for raster renderers in the `module:esri/smartMapping/raster/color/renderers` modules.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html)
   */
  interface colorRamps {
    /**
     * Returns all color ramps designed for raster renderers, each as an array of colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#all)
     */
    all(): colorRampsColorRamp[];
    /**
     * Returns the color ramp matching the given name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#byName)
     */
    byName(name: string): colorRampsColorRamp;
    /**
     * Creates a raster color ramp from a list of colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#createColorRamp)
     */
    createColorRamp(options: colorRampsCreateColorRampOptions): MultipartColorRamp | AlgorithmicColorRamp;
    /**
     * Returns a raster color ramp's name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#getColorRampName)
     */
    getColorRampName(colorRamp: MultipartColorRamp | AlgorithmicColorRamp): ColorRampName;
    /**
     * Returns the names of all raster color ramps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#names)
     */
    names(): string[];
  }

  export const colorRamps: colorRamps;

  /**
   * Defines a ramp of colors to use for visualizing raster data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRamp)
   */
  export interface colorRampsColorRamp {
    /**
     * The unique name of the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRamp)
     */
    name: string;
    /**
     * An array of colors intended for displaying a continuous color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRamp)
     */
    colors: Color[];
  }

  /**
   * The name of a color ramp and whether it is inverted.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRampName)
   */
  export interface ColorRampName {
    /**
     * The unique name of the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRampName)
     */
    name: string;
    /**
     * Indicates if the ramp's colors are inverted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#ColorRampName)
     */
    inverted: boolean;
  }

  export interface colorRampsCreateColorRampOptions {
    /**
     * An array of colors to convert to a multi-part or algorithmic color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#createColorRamp)
     */
    colors: Color[];
    /**
     * The algorithm used to interpolate colors between stops.
     *
     * @default cie-lab
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-colorRamps.html#createColorRamp)
     */
    algorithm?: "cie-lab" | "lab-lch" | "hsv";
  }

  /**
   * This object contains utility methods for obtaining information about supported renderers of raster layers (i.e.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html)
   */
  interface supportUtils {
    /**
     * Returns default band ids used by a multispectral imagery layer when using a raster [rgb](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-rgb.html) or [stretch](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-renderers-stretch.html) renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getDefaultBandCombination)
     */
    getDefaultBandCombination(params: utilsGetDefaultBandCombinationParams): Promise<number[]>;
    /**
     * Returns supported raster renderer information of an imagery layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo)
     */
    getSupportedRendererInfo(params: utilsGetSupportedRendererInfoParams): Promise<SupportedRendererInfo>;
  }

  export const supportUtils: supportUtils;

  /**
   * The result object of the [getSupportedRendererInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#SupportedRendererInfo)
   */
  export interface SupportedRendererInfo {
    /**
     * The default renderer of the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#SupportedRendererInfo)
     */
    defaultRenderer:
      | ClassBreaksRenderer
      | RasterColormapRenderer
      | UniqueValueRenderer
      | RasterStretchRenderer
      | VectorFieldRenderer
      | RasterShadedReliefRenderer
      | FlowRenderer;
    /**
     * A list of supported renderer types that may be applied to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#SupportedRendererInfo)
     */
    supportedTypes: SupportedRendererType[];
  }

  /**
   * Supported renderer types for imagery layers.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#SupportedRendererType)
   */
  export type SupportedRendererType =
    | "raster-stretch"
    | "raster-unique-value"
    | "raster-rgb"
    | "raster-class-breaks"
    | "raster-colormap"
    | "raster-shaded-relief"
    | "raster-vector-field"
    | "raster-flow";

  export interface utilsGetDefaultBandCombinationParams {
    /**
     * Layer for which the default band ids are returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getDefaultBandCombination)
     */
    layer: ImageryLayer | ImageryTileLayer | WCSLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getDefaultBandCombination)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getDefaultBandCombination)
     */
    rasterFunction?: RasterFunction;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getDefaultBandCombination)
     */
    signal?: AbortSignal;
  }

  export interface utilsGetSupportedRendererInfoParams {
    /**
     * The input layer for which the renderer info is returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo)
     */
    layer: ImageryLayer | ImageryTileLayer | WCSLayer;
    /**
     * Deprecated since 4.27.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo)
     */
    renderingRule?: RasterFunction;
    /**
     * Specifies the function for how the requested image should be processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo)
     */
    rasterFunction?: RasterFunction;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-raster-support-utils.html#getSupportedRendererInfo)
     */
    signal?: AbortSignal;
  }

  /**
   * This object contains helper methods for generating data-driven visualizations with continuous color or class breaks based on a field value or expression from features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html)
   */
  interface color {
    /**
     * Generates a continuous color [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) representing the age of features based on one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    createAgeRenderer(params: colorCreateAgeRendererParams): Promise<AgeRendererResult>;
    /**
     * Generates a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html) that may be applied directly to the layer used to call this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    createClassBreaksRenderer(params: colorCreateClassBreaksRendererParams): Promise<ClassBreaksRendererResult>;
    /**
     * Generates a [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) that may be applied directly to the layer used to call this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    createContinuousRenderer(params: colorCreateContinuousRendererParams): Promise<ContinuousRendererResult>;
    /**
     * Generates a [PointCloudStretchRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudStretchRenderer.html) with a color scheme best-suited for the view's background based on statistics returned from a given field of a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    createPCContinuousRenderer(params: colorCreatePCContinuousRendererParams): Promise<PCContinuousRendererResult>;
    /**
     * Generates a [PointCloudRGBRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudRGBRenderer.html) based on the `RGB` field of a given [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer)
     */
    createPCTrueColorRenderer(params: colorCreatePCTrueColorRendererParams): Promise<PCTrueColorRendererResult>;
    /**
     * This method generates a color visual variable with default stops that are optimally chosen based on the statistics queried for the indicated field or expression and colors determined by the view's background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    createVisualVariable(params: colorCreateVisualVariableParams): Promise<VisualVariableResult>;
  }

  export const color: color;

  /**
   * The result object of the [createAgeRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
   */
  export interface AgeRendererResult {
    /**
     * The renderer object configured to best match the background of the view and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * A color visual variable configured based on the statistics of the data and the background of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    visualVariable: ColorVariable;
    /**
     * The time unit used to represent age in the output `renderer`.
     *
     * [Read more...](global.html#unit)
     */
    unit: string;
    /**
     * The color scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    colorScheme: ColorScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for data from the given field name or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#AgeRendererResult)
     */
    basemapTheme: string;
  }

  /**
   * The result object of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
   */
  export interface ClassBreaksRendererResult {
    /**
     * The renderer object configured to best match the background of the view and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * This object describes class breaks generated from data in a layer for a given field with a specified classification method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    classBreaksResult: ClassBreaksResult;
    /**
     * The color scheme used for the class breaks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    colorScheme: ColorScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ClassBreaksRendererResult)
     */
    basemapTheme: string;
  }

  export interface colorCreateAgeRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view: View;
    /**
     * The name of the field, or a date value representing the start time in the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    startTime: Date | string | number;
    /**
     * The name of the field, or a date value representing the end time in the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    endTime: Date | string | number;
    /**
     * The time unit used to calculate the difference between `endTime` and `startTime`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: "years" | "months" | "days" | "hours" | "minutes" | "seconds";
    /**
     * Sets a maximum age for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    maxValue?: number;
    /**
     * Sets a minimum age for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    minValue?: number;
    /**
     * Determines which values will be emphasized in the continuous ramp and the map.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    theme?: "high-to-low" | "above" | "below" | "above-and-below" | "centered-on" | "extremes";
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    filter?: FeatureFilter;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * For point and polyline layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    legendOptions?: colorCreateAgeRendererParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    colorScheme?: ColorScheme;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    colorMixMode?: string;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    signal?: AbortSignal;
  }

  export interface colorCreateAgeRendererParamsLegendOptions {
    /**
     * The title used to represent the age color ramp in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    title?: string;
    /**
     * Indicates whether to include the age renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer)
     */
    showLegend?: boolean;
  }

  export interface colorCreateClassBreaksRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and classified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    normalizationField?: string;
    /**
     * Indicates how the data is normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    normalizationType?: "field" | "log" | "percent-of-total";
    /**
     * When `normalizationType` is `percent-of-total`, this property contains the total of all data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    normalizationTotal?: number;
    /**
     * The classification method used for generating breaks.
     *
     * @default equal-interval
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    classificationMethod?: "equal-interval" | "natural-breaks" | "quantile" | "standard-deviation";
    /**
     * If a `standard-deviation` classification method is used, then this indicates the interval by which to generate class breaks.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    standardDeviationInterval?: number;
    /**
     * The number of class breaks to generate.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    numClasses?: number;
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    colorScheme?: ColorScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    filter?: FeatureFilter;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Provides options for setting a title to describe a field instead of using the field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    legendOptions?: colorCreateClassBreaksRendererParamsLegendOptions;
    /**
     * A minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    minValue?: number;
    /**
     * A maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    maxValue?: number;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value and features that do not fall within the configured data range.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    signal?: AbortSignal;
  }

  export interface colorCreateClassBreaksRendererParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer)
     */
    title?: string;
  }

  export interface colorCreateContinuousRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    normalizationField?: string;
    /**
     * Determines which values will be emphasized in the continuous ramp and the map.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    theme?: "high-to-low" | "above" | "below" | "above-and-below" | "centered-on" | "extremes";
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    colorScheme?: ColorScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    filter?: FeatureFilter;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * For point and polyline layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    legendOptions?: colorCreateContinuousRendererParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    maxValue?: number;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value and features that do not fall within the configured data range.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    signal?: AbortSignal;
  }

  export interface colorCreateContinuousRendererParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    title?: string;
    /**
     * Indicates whether to describe the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer)
     */
    showLegend?: boolean;
  }

  export interface colorCreatePCContinuousRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    layer: PointCloudLayer;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    field: string;
    /**
     * The [named string](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap) or basemap object of the Esri basemap that will be paired with the output visualization.
     *
     * @default gray
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    basemap?: string | Basemap;
    /**
     * The size of each point expressed as a percentage.
     *
     * @default 100%
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    size?: string;
    /**
     * The number of points per inch in the view.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    density?: number;
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    colorScheme?: ColorScheme;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer)
     */
    signal?: AbortSignal;
  }

  export interface colorCreatePCTrueColorRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer)
     */
    layer: PointCloudLayer;
    /**
     * The size of each point expressed as a percentage.
     *
     * @default 100%
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer)
     */
    size?: string;
    /**
     * The number of points per inch in the view.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer)
     */
    density?: number;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer)
     */
    signal?: AbortSignal;
  }

  export interface colorCreateVisualVariableParams {
    /**
     * The layer for which the visual variable is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    normalizationField?: string;
    /**
     * Determines which values will be emphasized in the continuous ramp and the map.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    theme?: "high-to-low" | "above" | "below" | "above-and-below" | "centered-on" | "extremes";
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    colorScheme?: ColorScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    valueExpressionTitle?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    legendOptions?: colorCreateVisualVariableParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    maxValue?: number;
    /**
     * Indicates if the size units of the symbols will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    worldScale?: boolean;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    signal?: AbortSignal;
  }

  export interface colorCreateVisualVariableParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    title?: string;
    /**
     * Indicates whether to describe the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable)
     */
    showLegend?: boolean;
  }

  /**
   * The result object of the [createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
   */
  export interface ContinuousRendererResult {
    /**
     * The renderer object configured to best match the view's background and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * A color visual variable configured based on the statistics of the data and the view's background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    visualVariable: ColorVariable;
    /**
     * The color scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    colorScheme: ColorScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for data from the given field name or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult)
     */
    basemapTheme: string;
  }

  /**
   * The result object of the [createPCContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCContinuousRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
   */
  export interface PCContinuousRendererResult {
    /**
     * The renderer object configured to best match the background of the view and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    renderer: PointCloudStretchRenderer;
    /**
     * The color scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    colorScheme: ColorScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for data from the given field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal color scheme for the output renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCContinuousRendererResult)
     */
    basemapTheme: string;
  }

  /**
   * The result object of the [createPCTrueColorRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createPCTrueColorRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCTrueColorRendererResult)
   */
  export interface PCTrueColorRendererResult {
    /**
     * The renderer object configured to represent the true color of each point in the point cloud.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#PCTrueColorRendererResult)
     */
    renderer: PointCloudRGBRenderer;
  }

  /**
   * The result object of the [createVisualVariable()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createVisualVariable) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
   */
  export interface VisualVariableResult {
    /**
     * A color visual variable configured based on the statistics of the data and the view's background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    visualVariable: ColorVariable;
    /**
     * The color scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    colorScheme: ColorScheme;
    /**
     * Basic statistics returned from a query to the service for the given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    defaultValuesUsed: boolean;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    basemapTheme: string;
    /**
     * Authoring information related to the creation of the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#VisualVariableResult)
     */
    authoringInfo: AuthoringInfo;
  }

  /**
   * This object contains a helper method for generating a [dot density visualization](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html)
   */
  interface dotDensity {
    /**
     * Generates a [DotDensityRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-DotDensityRenderer.html) based on one or more complementary numeric fields and/or Arcade expressions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    createRenderer(params: dotDensityCreateRendererParams): Promise<RendererResult>;
  }

  export const dotDensity: dotDensity;

  export interface dotDensityCreateRendererParams {
    /**
     * The **polygon** layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The MapView instance in which the visualization will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
    /**
     * A set of complementary numeric fields/expressions used as the basis of the dot density visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    attributes: dotDensityCreateRendererParamsAttributes[];
    /**
     * Indicates whether to vary the value of each dot by the view's scale.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    dotValueOptimizationEnabled?: boolean;
    /**
     * Indicates whether to enable color blending of different colored dots rendered at the same pixel.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    dotBlendingEnabled?: boolean;
    /**
     * Indicates whether the polygon outline width should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    legendOptions?: dotDensityCreateRendererParamsLegendOptions;
    /**
     * In authoring apps, the user may select a pre-defined dot density scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    dotDensityScheme?: DotDensityScheme;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface dotDensityCreateRendererParamsAttributes {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    field?: string;
    /**
     * The label describing the field name (or category) in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    label?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer)
     */
    valueExpressionTitle?: string;
  }

  export interface dotDensityCreateRendererParamsLegendOptions {
    /**
     * Indicates the unit represented by each dot in the legend.
     *
     * [Read more...](global.html#unit)
     */
    unit?: string;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#RendererResult)
   */
  export interface RendererResult {
    /**
     * The object representing the dot density visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#RendererResult)
     */
    renderer: DotDensityRenderer;
    /**
     * The dot density scheme used by the renderer based on the view's background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#RendererResult)
     */
    dotDensityScheme: DotDensityScheme;
    /**
     * The ID of the basemap used to determine the optimal colors of the dots.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html#RendererResult)
     */
    basemapTheme: string;
  }

  /**
   * This object contains a helper method for generating a [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) for a point [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html)
   */
  interface heatmap {
    /**
     * Generates a [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) that may be applied directly to the layer used to call this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    createRenderer(params: heatmapCreateRendererParams): Promise<HeatmapRendererResult>;
    /**
     * Allows you to update the [colorStops](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#colorStops) of a [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) with opacity, making the low density areas of the heat map to fade out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#updateRenderer)
     */
    updateRenderer(params: heatmapUpdateRendererParams): HeatmapRenderer;
  }

  export const heatmap: heatmap;

  export interface heatmapCreateRendererParams {
    /**
     * The point layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view instance in which the visualization will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView | SceneView;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    field?: string;
    /**
     * In authoring apps, the user may select a pre-defined heatmap scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    heatmapScheme?: HeatmapScheme;
    /**
     * A statistics object generated from the [heatmapStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    statistics?: HeatmapStatisticsResult;
    /**
     * Indicates how much to fade the lower color stops with transparency to create a fuzzy boundary on the edge of the heatmap.
     *
     * @default 0.2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    fadeRatio?: number;
    /**
     * Indicates whether the heatmap should fade its colors to transparent.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    fadeToTransparent?: boolean;
    /**
     * The [radius](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#radius) in points that determines the area of influence of each point.
     *
     * @default 18
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    radius?: number;
    /**
     * The minimum [ratio](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio) used in the [HeatmapRenderer.colorStops](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#colorStops) of the output renderer.
     *
     * @default 0.01
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    minRatio?: number;
    /**
     * The maximum [ratio](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-support-HeatmapColorStop.html#ratio) used in the [HeatmapRenderer.colorStops](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html#colorStops) of the output renderer.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    maxRatio?: number;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
   */
  export interface HeatmapRendererResult {
    /**
     * The renderer object configured to best match the view's background and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    renderer: HeatmapRenderer;
    /**
     * The color scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    scheme: HeatmapScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics required for generating a renderer with optimal values for the given layer and view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    statistics: HeatmapStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal color stops of the heatmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult)
     */
    basemapTheme: string;
  }

  export interface heatmapUpdateRendererParams {
    /**
     * The heatmap renderer for which to add or remove opacity.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#updateRenderer)
     */
    renderer: HeatmapRenderer;
    /**
     * Indicates how much to fade the lower color stops with transparency to create a fuzzy boundary on the edge of the heatmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#updateRenderer)
     */
    fadeRatio?: number;
  }

  /**
   * This object contains helper methods for generating location-only visualizations (not data-driven) in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html)
   */
  interface location {
    /**
     * Generates a [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) that may be applied directly to a supported [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    createRenderer(params: locationCreateRendererParams): Promise<locationRendererResult>;
  }

  export const location: location;

  export interface locationCreateRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * In authoring apps, the user may select a pre-defined location scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    locationScheme?: LocationScheme;
    /**
     * For point and polyline layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#RendererResult)
   */
  export interface locationRendererResult {
    /**
     * A simple renderer configured with a single color best suited to match the background of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#RendererResult)
     */
    renderer: SimpleRenderer;
    /**
     * The location scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#RendererResult)
     */
    locationScheme: LocationScheme;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-location.html#RendererResult)
     */
    basemapTheme: string;
  }

  /**
   * This object contains a helper method for generating data-driven visualizations with continuous opacity based on data returned from a field value or expression in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html)
   */
  interface opacity {
    /**
     * This method generates an opacity visual variable with default alpha values optimally mapped to data values based on the statistics queried for the indicated field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    createVisualVariable(params: opacityCreateVisualVariableParams): Promise<opacityVisualVariableResult>;
  }

  export const opacity: opacity;

  export interface opacityCreateVisualVariableParams {
    /**
     * The layer for which the visual variable is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    normalizationField?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    valueExpressionTitle?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    legendOptions?: opacityCreateVisualVariableParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    maxValue?: number;
    /**
     * A [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) instance is required when a `valueExpression` is specified.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    signal?: AbortSignal;
  }

  export interface opacityCreateVisualVariableParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable)
     */
    title?: string;
  }

  /**
   * The result object of the [createVisualVariable()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult)
   */
  export interface opacityVisualVariableResult {
    /**
     * An opacity visual variable configured based on the statistics of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult)
     */
    visualVariable: OpacityVariable;
    /**
     * Basic statistics returned from a query to the service for the given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Authoring information related to the creation of the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult)
     */
    authoringInfo: AuthoringInfo;
  }

  /**
   * This object contains a helper method for generating a pie chart for every feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html)
   */
  interface pieChart {
    /**
     * Generates a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html) based on a set of numeric fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    createRenderer(params: pieChartCreateRendererParams): Promise<pieChartRendererResult>;
    /**
     * Generates a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html) to use for a [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualization based off an input layer's [UniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-UniqueValueRenderer.html) or [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    createRendererForClustering(params: pieChartCreateRendererForClusteringParams): Promise<ClusterRendererResult>;
  }

  export const pieChart: pieChart;

  /**
   * The result object of the [createRendererForClustering()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#ClusterRendererResult)
   */
  export interface ClusterRendererResult {
    /**
     * The generated pie chart renderer to use for clustering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#ClusterRendererResult)
     */
    renderer: PieChartRenderer;
    /**
     * The aggregate fields used by the `renderer`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#ClusterRendererResult)
     */
    fields: AggregateField[];
  }

  export interface pieChartCreateRendererForClusteringParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * Determines whether to create a pie chart or a donut chart.
     *
     * @default "pie"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    shape?: "pie" | "donut";
    /**
     * Includes the `defaultSymbol` defined in the input layer's renderer in the output pie chart renderer.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    legendOptions?: pieChartCreateRendererForClusteringParamsLegendOptions;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    signal?: AbortSignal;
  }

  export interface pieChartCreateRendererForClusteringParamsLegendOptions {
    /**
     * The title used to describe the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRendererForClustering)
     */
    title?: string;
  }

  export interface pieChartCreateRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer;
    /**
     * The view instance in which the visualization will be rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
    /**
     * A set of complementary numeric fields/expressions used to create the charts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    attributes: pieChartCreateRendererParamsAttributes[];
    /**
     * Determines whether to create a pie chart or a donut chart.
     *
     * @default "pie"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    shape?: "pie" | "donut";
    /**
     * Indicates whether to include data-driven size in the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    includeSizeVariable?: boolean;
    /**
     * Only for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Indicates whether chart sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    legendOptions?: pieChartCreateRendererParamsLegendOptions;
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    pieChartScheme?: PieChartScheme;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface pieChartCreateRendererParamsAttributes {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    field?: string;
    /**
     * The label describing the field name (or pie slice) in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    label?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    valueExpressionTitle?: string;
  }

  export interface pieChartCreateRendererParamsLegendOptions {
    /**
     * The title used to describe the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer)
     */
    title?: string;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
   */
  export interface pieChartRendererResult {
    /**
     * The generated pie chart renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    renderer: PieChartRenderer;
    /**
     * A size visual variable representing the sum of all attributes included in the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    size?: sizeVisualVariableResult;
    /**
     * The color scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    pieChartScheme: PieChartScheme;
    /**
     * The ID of the basemap used to determine the optimal color schemes for the charts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    basemapTheme: string;
    /**
     * Contains the total counts of each attribute used in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html#RendererResult)
     */
    statistics: UniqueValuesResult;
  }

  /**
   * This object contains a helper method for generating a predominance visualization.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html)
   */
  interface predominance {
    /**
     * Generates a predominance renderer based on a set of competing numeric fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    createRenderer(params: predominanceCreateRendererParams): Promise<predominanceRendererResult>;
  }

  export const predominance: predominance;

  export interface predominanceCreateRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view instance in which the visualization will be rendered.
     *
     * [Read more...](global.html)
     */
    view: View;
    /**
     * A set of competing numeric fields used as the basis of the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    fields: predominanceCreateRendererParamsFields[];
    /**
     * Indicates whether to include data-driven opacity in the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    includeOpacityVariable?: boolean;
    /**
     * Indicates whether to include data-driven size in the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    includeSizeVariable?: boolean;
    /**
     * Only for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    legendOptions?: predominanceCreateRendererParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * Indicates how values should be sorted in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * @default count
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    sortBy?: "count" | "value";
    /**
     * In authoring apps, the user may select a pre-defined predominance scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    predominanceScheme?: PredominanceScheme;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface predominanceCreateRendererParamsFields {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    name: string;
    /**
     * The label describing the field name (or category) in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    label?: string;
  }

  export interface predominanceCreateRendererParamsLegendOptions {
    /**
     * The title used to describe the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    title?: string;
    /**
     * Indicates whether to include the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer)
     */
    showLegend?: boolean;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
   */
  export interface predominanceRendererResult {
    /**
     * The unique value renderer object representing the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    renderer: UniqueValueRenderer;
    /**
     * An array of objects describing the value, symbol, and count of each unique type or category represented in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    predominantCategoryInfos: typeUniqueValueInfo[];
    /**
     * An array of objects describing the values or categories excluded from consideration in visualizing data from the given fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    excludedCategoryInfos: any[];
    /**
     * A size visual variable representing the total count of all fields included in the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    size?: sizeVisualVariableResult;
    /**
     * An opacity visual variable representing the percentage of the total count comprised by the predominant value of all fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    opacity?: opacityVisualVariableResult;
    /**
     * The predominance scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    predominanceScheme: PredominanceScheme;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html#RendererResult)
     */
    basemapTheme: string;
  }

  /**
   * This object contains a helper method for creating a renderer for exploring the relationship between two numeric attributes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html)
   */
  interface relationship {
    /**
     * Generates a relationship renderer (bivariate choropleth) based on a set of competing numeric fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    createRenderer(params: relationshipCreateRendererParams): Promise<relationshipRendererResult>;
    /**
     * Updates a relationship renderer (bivariate choropleth) generated from [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer) based on the given input parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    updateRenderer(params: relationshipUpdateRendererParams): Promise<UniqueValueRenderer>;
  }

  export const relationship: relationship;

  export interface relationshipCreateRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view instance in which the visualization will be rendered.
     *
     * [Read more...](global.html)
     */
    view: View;
    /**
     * A numeric field that will be used to explore its relationship with `field2`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    field1: relationshipCreateRendererParamsField1;
    /**
     * A numeric field that will be used to explore its relationship with `field1`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    field2: relationshipCreateRendererParamsField2;
    /**
     * The method for classifying each field's data values.
     *
     * @default quantile
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    classificationMethod?: "quantile" | "equal-interval" | "natural-breaks";
    /**
     * Determines the orientation of the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    focus?: string;
    /**
     * Indicates the number of classes by which to break up the values of each field.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    numClasses?: number;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * For point and polyline layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    legendOptions?: relationshipCreateRendererParamsLegendOptions;
    /**
     * In authoring apps, the user may select a pre-defined relationship scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    relationshipScheme?: RelationshipScheme;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value or that fall outside of the prescribed class breaks.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface relationshipCreateRendererParamsField1 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    normalizationField?: string;
    /**
     * The maximum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    maxValue?: number;
    /**
     * The minimum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    minValue?: number;
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    label?: string;
  }

  export interface relationshipCreateRendererParamsField2 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    normalizationField?: string;
    /**
     * The maximum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    maxValue?: number;
    /**
     * The minimum bound of values to visualize in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    minValue?: number;
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    label?: string;
  }

  export interface relationshipCreateRendererParamsLegendOptions {
    /**
     * The title used to describe the renderer in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    title?: string;
    /**
     * Indicates whether to include the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)
     */
    showLegend?: boolean;
  }

  export interface relationshipUpdateRendererParams {
    /**
     * A numeric field used to explore its relationship with `field2`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    field1: relationshipUpdateRendererParamsField1;
    /**
     * A numeric field used to explore its relationship with `field1`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    field2: relationshipUpdateRendererParamsField2;
    /**
     * Determines the orientation of the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    focus?: string;
    /**
     * Indicates the number of classes by which to break up the values of each field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    numClasses: number;
    /**
     * The array of colors to be used by the symbols in the uniqueValueInfos of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    colors: Color[];
    /**
     * The relationship renderer (generated from [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer)) to update with the other parameters provided in this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    renderer: UniqueValueRenderer;
  }

  export interface relationshipUpdateRendererParamsField1 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    normalizationField?: string;
    /**
     * The class breaks for the first field used in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    classBreakInfos: ClassBreak[];
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    label?: string;
  }

  export interface relationshipUpdateRendererParamsField2 {
    /**
     * The name of a numeric field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    field: string;
    /**
     * The name of a numeric field used to normalize the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    normalizationField?: string;
    /**
     * The class breaks for the second field used in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    classBreakInfos: ClassBreak[];
    /**
     * The label used to describe the field or variable in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#updateRenderer)
     */
    label?: string;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
   */
  export interface relationshipRendererResult {
    /**
     * The renderer object representing the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    renderer: UniqueValueRenderer;
    /**
     * Describes each class break for both fields considered in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    classBreaks: RendererResultClassBreaks;
    /**
     * Describes each container or bin used in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    uniqueValueInfos: typeUniqueValueInfo[];
    /**
     * The relationship scheme used by the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    relationshipScheme: RelationshipScheme;
    /**
     * The ID of the basemap used to determine the optimal fill colors of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    basemapTheme: string;
  }

  export interface RendererResultClassBreaks {
    /**
     * The class breaks generated for values in the `field1` parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    field1: ClassBreaksResult;
    /**
     * The class breaks generated for values in the `field2` parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html#RendererResult)
     */
    field2: ClassBreaksResult;
  }

  /**
   * This object contains helper methods for generating data-driven visualizations with continuous size or class breaks based on a field value or expression from features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html)
   */
  interface size {
    /**
     * Generates a continuous size [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) representing the age of features based on one or more fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    createAgeRenderer(params: sizeCreateAgeRendererParams): Promise<sizeAgeRendererResult>;
    /**
     * Generates a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html) that may be applied directly to the layer used to call this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    createClassBreaksRenderer(params: sizeCreateClassBreaksRendererParams): Promise<sizeClassBreaksRendererResult>;
    /**
     * Generates a [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) that may be applied directly to the layer used to call this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    createContinuousRenderer(params: sizeCreateContinuousRendererParams): Promise<sizeContinuousRendererResult>;
    /**
     * This method generates an array of size visual variables with default stops that are optimally chosen based on the statistics queried for the indicated field or expression and mapped to appropriate sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    createVisualVariables(params: sizeCreateVisualVariablesParams): Promise<sizeVisualVariableResult>;
    /**
     * Updates a renderer generated from [createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) that uses a specialty reference size variable (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    updateRendererWithReferenceSize(
      params: sizeUpdateRendererWithReferenceSizeParams,
    ): Promise<ClassBreaksRenderer | UniqueValueRenderer>;
  }

  export const size: size;

  /**
   * The result object of the [createAgeRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
   */
  export interface sizeAgeRendererResult {
    /**
     * The renderer object configured to best match the view's background and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * Size visual variables configured based on the statistics of the data and the size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    visualVariables: SizeVariable[];
    /**
     * The time unit used to represent age in the output `renderer`.
     *
     * [Read more...](global.html#unit)
     */
    unit: string;
    /**
     * The size scheme used by the visual variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    sizeScheme: SizeScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query for the generated expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#AgeRendererResult)
     */
    basemapTheme: string;
  }

  /**
   * The result object of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
   */
  export interface sizeClassBreaksRendererResult {
    /**
     * The renderer object configured to best match the view's background and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * This object describes class breaks generated from data in a layer for a given field with a specified classification method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    classBreaksResult: ClassBreaksResult;
    /**
     * The size scheme used by the visual variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    sizeScheme: SizeScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ClassBreaksRendererResult)
     */
    basemapTheme: string;
  }

  /**
   * The result object of the [createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
   */
  export interface sizeContinuousRendererResult {
    /**
     * The renderer object configured to best match the background of the view and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * Size visual variables configured based on the statistics of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    visualVariables: SizeVariable[];
    /**
     * The size scheme used by the visual variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    sizeScheme: SizeScheme;
    /**
     * Indicates whether default values are used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for data from the given field name or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    basemapTheme: string;
    /**
     * _Since 4.30_ Indicates if the polygon layer used to generate the renderer was determined to be gridded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult)
     */
    isGrid?: boolean;
  }

  export interface sizeCreateAgeRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view: View;
    /**
     * The name of the field, or a date value representing the start time in the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    startTime: Date | string | number;
    /**
     * The name of the field, or a date value representing the end time in the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    endTime: Date | string | number;
    /**
     * The time unit used to calculate the difference between `endTime` and `startTime`.
     *
     * [Read more...](global.html#unit)
     */
    unit?: "years" | "months" | "days" | "hours" | "minutes" | "seconds";
    /**
     * Sets the size stops based on meaningful data values.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    theme?: "high-to-low" | "above" | "below";
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Sets a maximum age for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    maxValue?: number;
    /**
     * Sets a minimum age for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    minValue?: number;
    /**
     * Only for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Provides options for modifying [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) properties describing the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    legendOptions?: sizeCreateAgeRendererParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    sizeScheme?: SizeScheme;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    signal?: AbortSignal;
  }

  export interface sizeCreateAgeRendererParamsLegendOptions {
    /**
     * The title used to represent the age size ramp in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    title?: string;
    /**
     * Indicates whether to include the age renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createAgeRenderer)
     */
    showLegend?: boolean;
  }

  export interface sizeCreateClassBreaksRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and classified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    normalizationField?: string;
    /**
     * Indicates how the data is normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    normalizationType?: "field" | "log" | "percent-of-total";
    /**
     * When `normalizationType` is `percent-of-total`, this property contains the total of all data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    normalizationTotal?: number;
    /**
     * The classification method used for generating breaks.
     *
     * @default equal-interval
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    classificationMethod?: "equal-interval" | "natural-breaks" | "quantile" | "standard-deviation";
    /**
     * If a `standard-deviation` classification method is used, then this indicates the interval by which to generate class breaks.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    standardDeviationInterval?: number;
    /**
     * The number of class breaks to generate.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    numClasses?: number;
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    sizeScheme?: SizeScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Only for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Provides options for setting a title to describe a field instead of using the field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    legendOptions?: sizeCreateClassBreaksRendererParamsLegendOptions;
    /**
     * A minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    minValue?: number;
    /**
     * A maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    maxValue?: number;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value and features that do not fall within the configured data range.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    signal?: AbortSignal;
  }

  export interface sizeCreateClassBreaksRendererParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer)
     */
    title?: string;
  }

  export interface sizeCreateContinuousRendererParams {
    /**
     * The layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    normalizationField?: string;
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    sizeScheme?: SizeScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * Defines the size theme.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    theme?: "high-to-low" | "above" | "below" | "reference-size";
    /**
     * _Since 4.30_ Options for specifying the symbol used in a reference size visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    referenceSizeOptions?: sizeCreateContinuousRendererParamsReferenceSizeOptions;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    filter?: FeatureFilter;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    sqlWhere?: string;
    /**
     * Only for polygon layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    legendOptions?: sizeCreateContinuousRendererParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    maxValue?: number;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value and features that do not fall within the configured data range.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    signal?: AbortSignal;
  }

  export interface sizeCreateContinuousRendererParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    title?: string;
    /**
     * Indicates whether to describe the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    showLegend?: boolean;
  }

  export interface sizeCreateContinuousRendererParamsReferenceSizeOptions {
    /**
     * Defines the symbol shape to use in a reference size visualization.
     *
     * @default "circle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer)
     */
    symbolStyle?: "circle" | "diamond" | "hexagon-flat" | "hexagon-pointy" | "square";
  }

  export interface sizeCreateVisualVariablesParams {
    /**
     * The layer for which the visual variables are generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    field: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    normalizationField?: string;
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    sizeScheme?: SizeScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    valueExpressionTitle?: string;
    /**
     * Sets the size stops based on meaningful data values.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    theme?: "high-to-low" | "above" | "below";
    /**
     * _Since 4.30_ When defined, creates a proportional size renderer with an outer ring that functions as a reference line representing a maximum data value (or threshold).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    referenceSizeOptions?: sizeCreateVisualVariablesParamsReferenceSizeOptions;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    filter?: FeatureFilter;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    sqlWhere?: string;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    legendOptions?: sizeCreateVisualVariablesParamsLegendOptions;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    maxValue?: number;
    /**
     * Indicates if the size units of the symbols will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    worldScale?: boolean;
    /**
     * When set to `all`, a single size variable that scales uniformly in all dimensions is generated.
     *
     * @default all
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    axis?: "all" | "height";
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    signal?: AbortSignal;
  }

  export interface sizeCreateVisualVariablesParamsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    title?: string;
    /**
     * Indicates whether to describe the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    showLegend?: boolean;
  }

  export interface sizeCreateVisualVariablesParamsReferenceSizeOptions {
    /**
     * Defines the symbol shape to use in a reference size visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables)
     */
    symbolStyle?: "circle" | "diamond" | "hexagon-flat" | "hexagon-pointy" | "square";
  }

  export interface sizeUpdateRendererWithReferenceSizeParams {
    /**
     * The layer for which the renderer is updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The renderer instance to update if not already set on the `layer`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    renderer?: ClassBreaksRenderer | UniqueValueRenderer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view: View;
    /**
     * The name of a numeric field used to dynamically change the inner dot size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    field?: string;
    /**
     * The name of a numeric field use to normalize the data in the input `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    normalizationField?: string;
    /**
     * The size stops with which to update the reference size visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    sizeStops: SizeStop[];
    /**
     * Updates the symbol shape used in the reference size visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    referenceSizeOptions?: sizeUpdateRendererWithReferenceSizeParamsReferenceSizeOptions;
    /**
     * Use this property to update the color of the symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    sizeScheme?: SizeScheme;
    /**
     * Only valid for reference size visualizations implemented in a UniqueValueRenderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    typeScheme?: TypeScheme;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * Indicates whether the renderer is intended for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    forBinning?: boolean;
  }

  export interface sizeUpdateRendererWithReferenceSizeParamsReferenceSizeOptions {
    /**
     * Defines the symbol shape to use in a reference size visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#updateRendererWithReferenceSize)
     */
    symbolStyle?: "circle" | "diamond" | "hexagon-flat" | "hexagon-pointy" | "square";
  }

  /**
   * The result object of the [createVisualVariables()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createVisualVariables) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
   */
  export interface sizeVisualVariableResult {
    /**
     * An array of size visual variables configured based on the statistics of the data and the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    visualVariables: SizeVariable[];
    /**
     * The size scheme used by the main visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    sizeScheme: SizeScheme;
    /**
     * Indicates whether default values were used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for the given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    basemapTheme: string;
    /**
     * Authoring information related to the creation of the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    authoringInfo: AuthoringInfo;
    /**
     * _Since 4.30_ Indicates if the polygon layer used to generate the renderer was determined to be gridded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#VisualVariableResult)
     */
    isGrid?: boolean;
  }

  /**
   * This module provides a function for regenerating renderers based on the current state of the layer and view.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html)
   */
  interface rendererUtils {
    /**
     * Regenerates the renderer for a layer based on the current state of the layer and view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html#regenerateRenderer)
     */
    regenerateRenderer(params: rendererUtilsRegenerateRendererParams): Promise<Renderer>;
  }

  export const rendererUtils: rendererUtils;

  export interface rendererUtilsRegenerateRendererParams {
    /**
     * The layer for which to regenerate the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html#regenerateRenderer)
     */
    layer: FeatureLayer;
    /**
     * The view in which the layer is rendered.
     *
     * [Read more...](global.html)
     */
    view: MapView;
    /**
     * The renderer to update.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html#regenerateRenderer)
     */
    renderer?: Renderer;
    /**
     * Indicates whether to regenerate the binning renderer or the renderer set on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html#regenerateRenderer)
     */
    forBinning?: boolean;
    /**
     * When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-support-rendererUtils.html#regenerateRenderer)
     */
    filter?: FeatureFilter;
  }

  /**
   * This object contains helper methods for generating data-driven visualizations with unique types (or categories) based on a field value from features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html)
   */
  interface type {
    /**
     * Generates a [PointCloudUniqueValueRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PointCloudUniqueValueRenderer.html) based on a given field of a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    createPCClassRenderer(params: typeCreatePCClassRendererParams): Promise<PCClassRendererResult>;
    /**
     * Generates a [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) that may be applied directly to a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) that supports renderers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    createRenderer(params: typeCreateRendererParams): Promise<typeRendererResult>;
  }

  export const type: type;

  /**
   * The result object of the [createPCClassRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#PCClassRendererResult)
   */
  export interface PCClassRendererResult {
    /**
     * The renderer object configured to represent the class codes in the point cloud.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#PCClassRendererResult)
     */
    renderer: PointCloudUniqueValueRenderer;
  }

  /**
   * The result object of the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
   */
  export interface typeRendererResult {
    /**
     * The renderer object configured to best match the background of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    renderer: UniqueValueRenderer;
    /**
     * An array of objects describing the value, symbol, and count of each unique type or category represented in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    uniqueValueInfos: typeUniqueValueInfo[];
    /**
     * An array of objects describing the values or categories excluded from consideration in visualizing data from the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    excludedUniqueValueInfos: any[];
    /**
     * The scheme used to represent each category within the `uniqueValueInfos`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    typeScheme: TypeScheme;
    /**
     * The ID of the basemap used to determine the optimal color scheme to represent the categorical variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#RendererResult)
     */
    basemapTheme: string;
  }

  export interface typeCreatePCClassRendererParams {
    /**
     * The layer for which the visualization is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    layer: PointCloudLayer;
    /**
     * The name of the field containing classification data for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    field: string;
    /**
     * The size of each point expressed as a percentage.
     *
     * @default 100%
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    size?: string;
    /**
     * The number of points per inch in the view.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    density?: number;
    /**
     * In authoring apps, the user may select a pre-defined type scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    typeScheme?: TypeSchemeForPoint;
    /**
     * A statistics object generated from the [uniqueValues](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    statistics?: UniqueValuesResult;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createPCClassRenderer)
     */
    signal?: AbortSignal;
  }

  export interface typeCreateRendererParams {
    /**
     * The layer for which the renderer is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field from which to extract unique values that will be used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    field?: string;
    /**
     * Specifies the name of a second attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    field2?: string;
    /**
     * Specifies the name of a third attribute field used to categorize features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    field3?: string;
    /**
     * The number of types (or categories) displayed by the renderer.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    numTypes?: number;
    /**
     * Indicates how values should be sorted in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * @default count
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    sortBy?: "count" | "value" | "none";
    /**
     * In authoring apps, the user may select a pre-defined type scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    typeScheme?: TypeScheme;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * For polygon layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    outlineOptimizationEnabled?: boolean;
    /**
     * For point and polyline layers only.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    sizeOptimizationEnabled?: boolean;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    filter?: FeatureFilter;
    /**
     * Provides options for setting a title to a field that will override the field alias defined in the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    legendOptions?: typeCreateRendererParamsLegendOptions;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * A statistics object generated from the [uniqueValues](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    statistics?: UniqueValuesResult;
    /**
     * **This option only applies to generating renderers for mesh SceneLayers**.
     *
     * @default replace
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    colorMixMode?: string;
    /**
     * Indicates that all domain codes should be returned if the given field has domain values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    returnAllCodedValues?: boolean;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    signal?: AbortSignal;
  }

  export interface typeCreateRendererParamsLegendOptions {
    /**
     * The title used to represent the given field in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer)
     */
    title: string;
  }

  /**
   * Describes the symbol, label, and value of a unique value generated by the [createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#createRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#UniqueValueInfo)
   */
  export interface typeUniqueValueInfo {
    /**
     * A unique value representing a type or category of features in a layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#UniqueValueInfo)
     */
    value: string | number;
    /**
     * The number of features with the given `value` (or belonging to the given category).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#UniqueValueInfo)
     */
    count: number;
    /**
     * The label describing the value or category in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#UniqueValueInfo)
     */
    label: string;
    /**
     * The symbol used to represent features belonging to the given category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-type.html#UniqueValueInfo)
     */
    symbol: Symbol;
  }

  /**
   * This object contains helper methods for generating data-driven univariate visualizations using both continuous color and continuous size based on a single field value or expression from features in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html)
   */
  interface univariateColorSize {
    /**
     * Generates a [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) that may be applied directly to a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    createContinuousRenderer(
      params: univariateColorSizeCreateContinuousRendererParams,
    ): Promise<univariateColorSizeContinuousRendererResult>;
    /**
     * This method generates color and size visual variables, both based on the same given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    createVisualVariables(params: univariateColorSizeCreateVisualVariablesParams): Promise<VisualVariablesResult>;
  }

  export const univariateColorSize: univariateColorSize;

  /**
   * The result object of the [createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
   */
  export interface univariateColorSizeContinuousRendererResult {
    /**
     * The renderer object configured to best match the view's background and the spread of the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    renderer: ClassBreaksRenderer;
    /**
     * Object containing visual variable and scheme information for the color portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    color: ContinuousRendererResultColor;
    /**
     * Object containing visual variable and scheme information for the size portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    size: ContinuousRendererResultSize;
    /**
     * Indicates whether default values were used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for the given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    basemapTheme: string;
  }

  export interface univariateColorSizeCreateContinuousRendererParams {
    /**
     * The layer for which the visual variable is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    normalizationField?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    valueExpressionTitle?: string;
    /**
     * Sets the size stops and colors based on meaningful data values.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    theme?: "high-to-low" | "above" | "below" | "above-and-below";
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    filter?: FeatureFilter;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    maxValue?: number;
    /**
     * Enables the `defaultSymbol` on the renderer and assigns it to features with no value and features that do not fall within the configured data range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    defaultSymbolEnabled?: boolean;
    /**
     * Options for configuring the color portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    colorOptions?: univariateColorSizeCreateContinuousRendererParamsColorOptions;
    /**
     * Options for configuring the size portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    sizeOptions?: univariateColorSizeCreateContinuousRendererParamsSizeOptions;
    /**
     * Options for setting symbols for the `above-and-below` theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    symbolOptions?: univariateColorSizeCreateContinuousRendererParamsSymbolOptions;
    /**
     * Provides options for configuring the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) representing the renderer generated from this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    legendOptions?: univariateColorSizeCreateContinuousRendererParamsLegendOptions;
    /**
     * The type of symbol to generate.
     *
     * @default 2d
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    symbolType?: "2d" | "3d-flat" | "3d-volumetric" | "3d-volumetric-uniform";
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    signal?: AbortSignal;
  }

  export interface univariateColorSizeCreateContinuousRendererParamsColorOptions {
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    colorScheme?: ColorScheme;
    /**
     * Only applies to the `above-and-below` theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    isContinuous?: boolean;
  }

  export interface univariateColorSizeCreateContinuousRendererParamsLegendOptions {
    /**
     * The title used to represent the renderer generated from this method in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    title?: string;
    /**
     * Indicates if the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget should display content describing the generated renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    showLegend?: boolean;
  }

  export interface univariateColorSizeCreateContinuousRendererParamsSizeOptions {
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    sizeScheme?: SizeScheme;
    /**
     * Indicates whether symbol sizes should vary based on view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    sizeOptimizationEnabled?: boolean;
  }

  export interface univariateColorSizeCreateContinuousRendererParamsSymbolOptions {
    /**
     * Sets above and below symbols based on pre-defined named symbol pairs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    symbolStyle?:
      | "caret"
      | "circle-caret"
      | "arrow"
      | "circle-arrow"
      | "plus-minus"
      | "circle-plus-minus"
      | "square"
      | "circle"
      | "triangle"
      | "happy-sad"
      | "thumb";
    /**
     * Allows you to specify custom symbols in the `above-and-below` theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    symbols?: univariateColorSizeCreateContinuousRendererParamsSymbolOptionsSymbols;
  }

  export interface univariateColorSizeCreateContinuousRendererParamsSymbolOptionsSymbols {
    /**
     * The symbol to use for the above portion of the `above-and-below` visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    above: Symbol;
    /**
     * The symbol to use for the below portion of the `above-and-below` visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer)
     */
    below: Symbol;
  }

  export interface univariateColorSizeCreateVisualVariablesParams {
    /**
     * The layer for which the visual variable is generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The view where the input layer is rendered.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * The name of the field whose data will be queried for statistics and used for the basis of the data-driven visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    field?: string;
    /**
     * The name of the field to normalize the values of the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    normalizationField?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    valueExpression?: string;
    /**
     * Text describing the value returned from the `valueExpression`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    valueExpressionTitle?: string;
    /**
     * Sets the size stops and colors based on meaningful data values.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    theme?: "high-to-low" | "above" | "below" | "above-and-below";
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    sqlWhere?: string;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    filter?: FeatureFilter;
    /**
     * A statistics object generated from the [summaryStatistics](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html) function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    statistics?: SummaryStatisticsResult;
    /**
     * A custom minimum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    minValue?: number;
    /**
     * A custom maximum value set by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    maxValue?: number;
    /**
     * Options for configuring the color portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    colorOptions?: univariateColorSizeCreateVisualVariablesParamsColorOptions;
    /**
     * Options for configuring the size portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    sizeOptions?: univariateColorSizeCreateVisualVariablesParamsSizeOptions;
    /**
     * Indicates if the size units of the symbols will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    worldScale?: boolean;
    /**
     * Indicates whether the generated renderer is for a binning visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    signal?: AbortSignal;
  }

  export interface univariateColorSizeCreateVisualVariablesParamsColorOptions {
    /**
     * Determines which values will be emphasized in the continuous ramp and the map.
     *
     * @default high-to-low
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    theme?: "high-to-low" | "above-and-below" | "centered-on" | "extremes";
    /**
     * In authoring apps, the user may select a pre-defined color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    colorScheme?: ColorScheme;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    legendOptions?: univariateColorSizeCreateVisualVariablesParamsColorOptionsLegendOptions;
    /**
     * Only applies to the `above-and-below` theme.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    isContinuous?: boolean;
  }

  export interface univariateColorSizeCreateVisualVariablesParamsColorOptionsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    title: string;
  }

  export interface univariateColorSizeCreateVisualVariablesParamsSizeOptions {
    /**
     * When set to `all`, a single size variable that scales uniformly in all dimensions is generated.
     *
     * @default all
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    axis?: "all" | "height";
    /**
     * In authoring apps, the user may select a pre-defined size scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    sizeScheme?: SizeScheme;
    /**
     * Provides options for setting a title to a field when an expression is provided instead of a field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    legendOptions?: univariateColorSizeCreateVisualVariablesParamsSizeOptionsLegendOptions;
  }

  export interface univariateColorSizeCreateVisualVariablesParamsSizeOptionsLegendOptions {
    /**
     * The title used to represent the given field or expression in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables)
     */
    title: string;
  }

  /**
   * The result object of the [createVisualVariables()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createVisualVariables) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
   */
  export interface VisualVariablesResult {
    /**
     * Object containing visual variable and scheme information for the color portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    color: VisualVariablesResultColor;
    /**
     * Object containing visual variable and scheme information for the size portion of the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    size: VisualVariablesResultSize;
    /**
     * Indicates whether default values were used in the absence of sufficient data and/or statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    defaultValuesUsed: boolean;
    /**
     * Basic statistics returned from a query to the service for the given field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    statistics: SummaryStatisticsResult;
    /**
     * The ID of the basemap used to determine the optimal fill color of the features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    basemapId: string;
    /**
     * Indicates whether the average color of the input view's basemap is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    basemapTheme: string;
    /**
     * Authoring information related to the creation of the visual variables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    authoringInfo: AuthoringInfo;
  }

  export interface ContinuousRendererResultColor {
    /**
     * A color visual variable configured based on the statistics of the data and the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    visualVariable: ColorVariable;
    /**
     * The color scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    colorScheme: ColorScheme;
  }

  export interface ContinuousRendererResultSize {
    /**
     * The size visual variable(s) configured based on the statistics of the data and the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    visualVariables: SizeVariable[];
    /**
     * The size scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult)
     */
    sizeScheme: SizeScheme;
  }

  export interface VisualVariablesResultColor {
    /**
     * A color visual variable configured based on the statistics of the data and the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    visualVariable: ColorVariable;
    /**
     * The color scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    colorScheme: ColorScheme;
  }

  export interface VisualVariablesResultSize {
    /**
     * The size visual variable(s) configured based on the statistics of the data and the view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    visualVariables: SizeVariable[];
    /**
     * The size scheme used by the visual variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#VisualVariablesResult)
     */
    sizeScheme: SizeScheme;
  }

  /**
   * Function for generating class breaks for an input field in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) based on a given classification method and normalization type.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html)
   */
  interface statisticsClassBreaks {
    /**
     * Generates class breaks for an input field (or expression) of a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) based on a given classification method and normalization type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    classBreaks(params: classBreaksClassBreaksParams): Promise<ClassBreaksResult>;
  }

  const __statisticsClassBreaksMapped: statisticsClassBreaks;
  export const statisticsClassBreaks: typeof __statisticsClassBreaksMapped.classBreaks;

  /**
   * An object describing a single class break generated from the [classBreaks()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreak)
   */
  export interface ClassBreak {
    /**
     * The label describing the given class break for use in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreak)
     */
    label: string;
    /**
     * The lower bound of the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreak)
     */
    minValue: number;
    /**
     * The upper bound of the class break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreak)
     */
    maxValue: number;
  }

  export interface classBreaksClassBreaksParams {
    /**
     * The layer from which to generate class breaks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The class breaks will be generated based on values of this field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    field?: string;
    /**
     * The field by which to normalize the values returned from the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    normalizationField?: string;
    /**
     * The method for classifying the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    classificationMethod?: "equal-interval" | "natural-breaks" | "quantile" | "standard-deviation";
    /**
     * When `classificationMethod = "standard-deviation"`, this sets the interval at which each class break should be set (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    standardDeviationInterval?: number;
    /**
     * The minimum bounding value for the class breaks definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    minValue?: number;
    /**
     * The maximum bounding value for the class breaks definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    maxValue?: number;
    /**
     * Indicates the number of classes to generate for the class breaks definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    numClasses?: number;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    valueExpression?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    sqlWhere?: string;
    /**
     * A [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) instance is required when a `valueExpression` is specified.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * A feature filter used to filter statistic queries by geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    filter?: FeatureFilter;
    /**
     * A subset of features for which to generate the class breaks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    features?: Graphic[];
    /**
     * Only applicable when the input `layer` is a service-backed FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    useFeaturesInView?: boolean;
    /**
     * Indicates whether the generated statistics are for a binning or clustering visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks)
     */
    signal?: AbortSignal;
  }

  /**
   * Object returned from the [classBreaks()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreaksResult)
   */
  export interface ClassBreaksResult {
    /**
     * An array of objects describing the class breaks generated from the [classBreaks()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#classBreaks) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreaksResult)
     */
    classBreakInfos: ClassBreak[];
    /**
     * The minimum value of features in the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreaksResult)
     */
    minValue: number;
    /**
     * The maximum value of features in the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreaksResult)
     */
    maxValue: number;
    /**
     * The normalization total if `normalizationType` is set to `percent-of-total` when generating class breaks with [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-classBreaks.html#ClassBreaksResult)
     */
    normalizationTotal: number;
  }

  /**
   * Function for generating statistics from a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) for a [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) visualization.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html)
   */
  interface heatmapStatistics {
    /**
     * Returns an object containing various statistics describing the intensity values for a heatmap visualization of a given point [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics)
     */
    heatmapStatistics(params: heatmapStatisticsHeatmapStatisticsParams): Promise<HeatmapStatisticsResult>;
  }

  const __heatmapStatisticsMapped: heatmapStatistics;
  export const heatmapStatistics: typeof __heatmapStatisticsMapped.heatmapStatistics;

  export interface heatmapStatisticsHeatmapStatisticsParams {
    /**
     * The layer from which to generate statistics for the pixel intensity values and a given `field` if provided.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics)
     */
    layer:
      | FeatureLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * A view instance used to calculate pixel intensity values based on the features in the current view.
     *
     * [Read more...](global.html)
     */
    view: MapView | SceneView;
    /**
     * The name of the numeric field for which the statistics will be generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics)
     */
    field?: string;
    /**
     * The area of influence for each point as a radius in points from the point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics)
     */
    radius?: number;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics)
     */
    signal?: AbortSignal;
  }

  /**
   * The statistics returned from the [heatmapStatistics()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#heatmapStatistics) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#HeatmapStatisticsResult)
   */
  export interface HeatmapStatisticsResult {
    /**
     * The maximum kernel density value of all pixels in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#HeatmapStatisticsResult)
     */
    max: number;
    /**
     * The minimum kernel density value of all pixels in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-heatmapStatistics.html#HeatmapStatisticsResult)
     */
    min: number;
  }

  /**
   * Generates a histogram based on data in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) for a given field.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html)
   */
  interface histogram {
    /**
     * Generates a histogram for data returned from a `field` in a given `layer`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    histogram(params: histogramHistogramParams): Promise<HistogramResult>;
  }

  const __histogramMapped: histogram;
  export const histogram: typeof __histogramMapped.histogram;

  export interface histogramHistogramParams {
    /**
     * The layer for which to generate a histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | CatalogLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The name of the numeric field for which the histogram will be generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    field?: string;
    /**
     * Determines how the provided `field` values will be normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    normalizationType?: "field" | "log" | "percent-of-total" | "natural-log" | "square-root";
    /**
     * The field by which to normalize the values returned from the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    normalizationField?: string;
    /**
     * Only applies if `normalizationType` is `percent-of-total`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    normalizationTotal?: number;
    /**
     * The method for classifying the data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    classificationMethod?: "equal-interval" | "natural-breaks" | "quantile" | "standard-deviation";
    /**
     * When `classificationMethod = "standard-deviation"`, this sets the interval at which each class break should be set (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    standardDeviationInterval?: number;
    /**
     * The minimum bounding value for the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    minValue?: number;
    /**
     * The maximum bounding value for the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    maxValue?: number;
    /**
     * Indicates the number of classes to generate for the histogram.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    numBins?: number;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    valueExpression?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    sqlWhere?: string;
    /**
     * A [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) instance is required when a `valueExpression` is specified.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * A feature filter used to filter statistic queries by geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    filter?: FeatureFilter;
    /**
     * A subset of features for which to generate the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    features?: Graphic[];
    /**
     * Only applicable when the input `layer` is a service-backed FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    useFeaturesInView?: boolean;
    /**
     * Indicates whether the generated statistics are for a binning or clustering visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram)
     */
    signal?: AbortSignal;
  }

  /**
   * The result returned from the [histogram()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#histogram) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#HistogramResult)
   */
  export interface HistogramResult {
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#HistogramResult)
     */
    bins: Bin[];
    /**
     * The minimum value returned by the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#HistogramResult)
     */
    minValue: number;
    /**
     * The maximum value returned by the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#HistogramResult)
     */
    maxValue: number;
    /**
     * The number used to normalize all values when `percent-of-total` is specified as the `normalizationType`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html#HistogramResult)
     */
    normalizationTotal?: number;
  }

  /**
   * Function for generating category statistics for a [predominance](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html) renderer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html)
   */
  interface predominantCategories {
    /**
     * Determines predominant categories for a layer based on a given set of competing numeric fields and returns the number of features belonging to each category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories)
     */
    predominantCategories(
      params: predominantCategoriesPredominantCategoriesParams,
    ): Promise<PredominantCategoriesResult>;
  }

  const __predominantCategoriesMapped: predominantCategories;
  export const predominantCategories: typeof __predominantCategoriesMapped.predominantCategories;

  export interface predominantCategoriesPredominantCategoriesParams {
    /**
     * The layer from which to generate predominant categories for the given set of `fields`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * An array of numeric fields from which to determine predominant categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories)
     */
    fields: string[];
    /**
     * Indicates whether the generated statistics are for a binning or clustering visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories)
     */
    forBinning?: boolean;
    /**
     * The view in which features will be rendered.
     *
     * [Read more...](global.html)
     */
    view?: MapView | SceneView;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories)
     */
    signal?: AbortSignal;
  }

  /**
   * The statistics returned from the [predominantCategories()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#predominantCategories) function.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#PredominantCategoriesResult)
   */
  export interface PredominantCategoriesResult {
    /**
     * An array of objects describing the count of each predominant category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#PredominantCategoriesResult)
     */
    predominantCategoryInfos: PredominantCategoriesResultPredominantCategoryInfos[];
  }

  export interface PredominantCategoriesResultPredominantCategoryInfos {
    /**
     * The value or category referenced in a predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#PredominantCategoriesResult)
     */
    value: string | number;
    /**
     * The number of features belonging to the predominant category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#PredominantCategoriesResult)
     */
    count: number;
    /**
     * The label describing the predominant category, or field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-predominantCategories.html#PredominantCategoriesResult)
     */
    label?: string;
  }

  /**
   * Function for generating attribute statistics in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) based on values returned from a given field.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html)
   */
  interface summaryStatistics {
    /**
     * Returns an object containing statistics describing a set of values returned from a field (or expression) in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    summaryStatistics(params: summaryStatisticsSummaryStatisticsParams): Promise<SummaryStatisticsResult>;
  }

  const __summaryStatisticsMapped: summaryStatistics;
  export const summaryStatistics: typeof __summaryStatisticsMapped.summaryStatistics;

  /**
   * The type of statistic to include or exclude in the summary statistics calculation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#OutStatisticType)
   */
  export type OutStatisticType =
    | "avg"
    | "count"
    | "max"
    | "median"
    | "min"
    | "stddev"
    | "sum"
    | "variance"
    | "nullcount";

  /**
   * The statistics returned from the [summaryStatistics()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics) query.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
   */
  export interface SummaryStatisticsResult {
    /**
     * The average of all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    avg: number;
    /**
     * The total number of features with a non-null value for the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    count: number;
    /**
     * The maximum of all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    max: number;
    /**
     * The median of all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    median: number;
    /**
     * The minimum of all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    min: number;
    /**
     * The standard deviation calculated from values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    stddev: number;
    /**
     * The sum of all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    sum: number;
    /**
     * The calculated variance from all values returned from the field or expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    variance: number;
    /**
     * The number of null values stored in the given field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#SummaryStatisticsResult)
     */
    nullcount?: number;
  }

  export interface summaryStatisticsSummaryStatisticsParams {
    /**
     * The layer from which to generate statistics for the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | CatalogLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The name of the numeric or string field for which the summary statistics will be generated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    field?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    valueExpression?: string;
    /**
     * A SQL expression evaluating to a number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    sqlWhere?: string;
    /**
     * Determines how the provided `field` values will be normalized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    normalizationType?: "field" | "log" | "percent-of-total" | "natural-log" | "square-root";
    /**
     * The field by which to normalize the values returned from the given `field`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    normalizationField?: string;
    /**
     * Only applies if `normalizationType` is `percent-of-total`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    normalizationTotal?: number;
    /**
     * The minimum bounding value for the statistics calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    minValue?: number;
    /**
     * The maximum bounding value for the statistics calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    maxValue?: number;
    /**
     * A [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) instance is required when a `valueExpression` is specified.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * _Since 4.31_ Indicates the statistics to calculate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    outStatisticTypes?: summaryStatisticsSummaryStatisticsParamsOutStatisticTypes;
    /**
     * _Since 4.25_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    filter?: FeatureFilter;
    /**
     * A subset of features for which to calculate the statistics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    features?: Graphic[];
    /**
     * Only applicable when the input `layer` is a service-backed FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    useFeaturesInView?: boolean;
    /**
     * Indicates whether the generated statistics are for a binning or clustering visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    signal?: AbortSignal;
  }

  export interface summaryStatisticsSummaryStatisticsParamsOutStatisticTypes {
    /**
     * The statistics to include in the calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    include?: OutStatisticType[];
    /**
     * The statistics to exclude from the calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatistics.html#summaryStatistics)
     */
    exclude?: OutStatisticType[];
  }

  /**
   * Function for generating statistics for the age of features in a layer based on a given start time and end time.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html)
   */
  interface summaryStatisticsForAge {
    /**
     * Returns an object containing various statistics describing an age value (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    summaryStatisticsForAge(
      params: summaryStatisticsForAgeSummaryStatisticsForAgeParams,
    ): Promise<SummaryStatisticsResult>;
  }

  const __summaryStatisticsForAgeMapped: summaryStatisticsForAge;
  export const summaryStatisticsForAge: typeof __summaryStatisticsForAgeMapped.summaryStatisticsForAge;

  export interface summaryStatisticsForAgeSummaryStatisticsForAgeParams {
    /**
     * The layer from which to generate age statistics for the given `startTime` and `endTime`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The start time for the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    startTime: Date | string | number;
    /**
     * The end time for the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    endTime: Date | string | number;
    /**
     * The desired units of the age result.
     *
     * [Read more...](global.html#unit)
     */
    unit: "years" | "months" | "days" | "hours" | "minutes" | "seconds";
    /**
     * The view in which features will be rendered.
     *
     * [Read more...](global.html)
     */
    view?: MapView | SceneView;
    /**
     * _Since 4.31_ Indicates the statistics to calculate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    outStatisticTypes?: summaryStatisticsForAgeSummaryStatisticsForAgeParamsOutStatisticTypes;
    /**
     * _Since 4.31_ When defined, only features included in the filter are considered in the attribute and spatial statistics calculations when determining the final renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    filter?: FeatureFilter;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    signal?: AbortSignal;
  }

  export interface summaryStatisticsForAgeSummaryStatisticsForAgeParamsOutStatisticTypes {
    /**
     * The statistics to include in the calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    include?: OutStatisticType[];
    /**
     * The statistics to exclude from the calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-summaryStatisticsForAge.html#summaryStatisticsForAge)
     */
    exclude?: OutStatisticType[];
  }

  /**
   * Provides utility functions for generating [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expressions intended for use in an [age renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createAgeRenderer).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html)
   */
  interface ageUtils {
    /**
     * Returns an [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression for visualizing the age of a feature based on a given start time and end time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#getAgeExpressions)
     */
    getAgeExpressions(params: ageUtilsGetAgeExpressionsParams): AgeExpressionsResult;
  }

  export const ageUtils: ageUtils;

  /**
   * The result object returned from the [getAgeExpressions()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#getAgeExpressions) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#AgeExpressionsResult)
   */
  export interface AgeExpressionsResult {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#AgeExpressionsResult)
     */
    valueExpression: string;
    /**
     * A SQL expression and where clause that matches the generated `valueExpression` used to query statistics from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#AgeExpressionsResult)
     */
    statisticsQuery: any;
    /**
     * A SQL expression and where clause that matches the generated `valueExpression` used to query for a histogram from the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#AgeExpressionsResult)
     */
    histogramQuery: any;
  }

  export interface ageUtilsGetAgeExpressionsParams {
    /**
     * The layer from which to generate age statistics for the given `startTime` and `endTime`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#getAgeExpressions)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The start time for the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#getAgeExpressions)
     */
    startTime: Date | string | number;
    /**
     * The end time for the age calculation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-ageUtils.html#getAgeExpressions)
     */
    endTime: Date | string | number;
    /**
     * The desired units of the age result.
     *
     * [Read more...](global.html#unit)
     */
    unit?: "years" | "months" | "days" | "hours" | "minutes" | "seconds";
  }

  /**
   * Provides utility functions for generating [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) and SQL expressions intended for use in an [predominance renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html)
   */
  interface predominanceUtils {
    /**
     * Returns [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) and SQL expressions used for visualizing the predominant category (or field) among a set of competing fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#getPredominanceExpressions)
     */
    getPredominanceExpressions(
      layer:
        | FeatureLayer
        | SceneLayer
        | CSVLayer
        | GeoJSONLayer
        | WFSLayer
        | OGCFeatureLayer
        | StreamLayer
        | OrientedImageryLayer
        | CatalogFootprintLayer
        | KnowledgeGraphSublayer
        | SubtypeGroupLayer
        | SubtypeSublayer,
      fieldNames: string[],
    ): PredominantExpressions;
  }

  export const predominanceUtils: predominanceUtils;

  /**
   * Contains the Arcade and SQL expressions used to query feature services and client-side layers for the strength of the predominant category compared with all other categories.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#OpacityExpressionInfo)
   */
  export interface OpacityExpressionInfo {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#OpacityExpressionInfo)
     */
    valueExpression: string;
    /**
     * The SQL expressions used for querying the percentage the predominant value comprises among of a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#OpacityExpressionInfo)
     */
    statisticsQuery: SQLExpressionInfo;
    /**
     * The SQL expressions used for querying the histogram using the percentage the predominant value comprises among of a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#OpacityExpressionInfo)
     */
    histogramQuery: SQLExpressionInfo;
  }

  /**
   * The result object returned from the [getAgeExpressions()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#getAgeExpressions) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#PredominantExpressions)
   */
  export interface PredominantExpressions {
    /**
     * The Arcade expression used to get the predominant category among a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#PredominantExpressions)
     */
    predominantCategory: PredominantExpressionsPredominantCategory;
    /**
     * The Arcade and SQL expressions used to query the server or client-side layer for statistics of the sum of all competing fields considered for the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#PredominantExpressions)
     */
    size: SizeExpressionInfo;
    /**
     * The Arcade and SQL expressions used to query the server or client-side layer for statistics of the percentage among all competing fields comprised by the predominant category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#PredominantExpressions)
     */
    opacity: OpacityExpressionInfo;
  }

  /**
   * Contains the Arcade and SQL expressions used to query feature services and client-side layers for the sum of all values considered in a predominance visualization.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SizeExpressionInfo)
   */
  export interface SizeExpressionInfo {
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SizeExpressionInfo)
     */
    valueExpression: string;
    /**
     * The SQL expressions used for querying the sum of a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SizeExpressionInfo)
     */
    statisticsQuery: SQLExpressionInfo;
    /**
     * The SQL expressions used for querying the histogram for the sum of a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SizeExpressionInfo)
     */
    histogramQuery: SQLExpressionInfo;
  }

  /**
   * SQL expressions used to query feature services for predominant values and related statistics given a set of fields.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SQLExpressionInfo)
   */
  export interface SQLExpressionInfo {
    /**
     * The SQL expression used for querying the predominant category given a set of fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SQLExpressionInfo)
     */
    sqlExpression: string;
    /**
     * The SQL where clause used to query the predominant value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#SQLExpressionInfo)
     */
    sqlWhere: string;
  }

  export interface PredominantExpressionsPredominantCategory {
    /**
     * valueExpression - An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-support-predominanceUtils.html#PredominantExpressions)
     */
    valueExpression: string;
  }

  /**
   * A function that queries for unique values from a field in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html)
   */
  interface uniqueValues {
    /**
     * Returns an object containing an array of unique values queried from a given field (or values returned from an expression) in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) along with the total count of features that belong to the given category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    uniqueValues(params: uniqueValuesUniqueValuesParams): Promise<UniqueValuesResult>;
  }

  const __uniqueValuesMapped: uniqueValues;
  export const uniqueValues: typeof __uniqueValuesMapped.uniqueValues;

  /**
   * An object that contains the unique values returned from the [uniqueValues()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues) query of a layer's field.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#UniqueValuesResult)
   */
  export interface UniqueValuesResult {
    /**
     * An array of objects, each containing a unique value/type/category present in the field specified in the [uniqueValues()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues) query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#UniqueValuesResult)
     */
    uniqueValueInfos: UniqueValuesResultUniqueValueInfos[];
  }

  export interface uniqueValuesUniqueValuesParams {
    /**
     * The layer from which to query for unique values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    layer:
      | FeatureLayer
      | SceneLayer
      | CSVLayer
      | PointCloudLayer
      | GeoJSONLayer
      | WFSLayer
      | OGCFeatureLayer
      | StreamLayer
      | OrientedImageryLayer
      | CatalogFootprintLayer
      | CatalogLayer
      | KnowledgeGraphSublayer
      | SubtypeGroupLayer
      | SubtypeSublayer;
    /**
     * The name of the numeric or string field from which the unique values will be obtained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    field?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    valueExpression?: string;
    /**
     * A SQL expression evaluating to a number or string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    sqlExpression?: string;
    /**
     * A SQL where clause used to filter features for the statistics query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    sqlWhere?: string;
    /**
     * Indicates that all domain codes should be returned if the given field has domain values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    returnAllCodedValues?: boolean;
    /**
     * A [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) instance is required when a `valueExpression` is specified.
     *
     * [Read more...](global.html)
     */
    view?: View;
    /**
     * A feature filter used to filter statistic queries by geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    filter?: FeatureFilter;
    /**
     * A subset of features for which to generate the unique values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    features?: Graphic[];
    /**
     * Only applicable when the input `layer` is a service-backed FeatureLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    useFeaturesInView?: boolean;
    /**
     * Indicates whether the generated statistics are for a binning or clustering visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    forBinning?: boolean;
    /**
     * Allows for cancelable requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#uniqueValues)
     */
    signal?: AbortSignal;
  }

  export interface UniqueValuesResultUniqueValueInfos {
    /**
     * A unique value representing a type or category of features in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#UniqueValuesResult)
     */
    value: string | number;
    /**
     * The number of features assigned the given `value` (or belonging to the given category).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-uniqueValues.html#UniqueValuesResult)
     */
    count: number;
  }

  /**
   * Object containing helper methods for generating optimal symbols for data-driven color visualizations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html)
   */
  interface symbologyColor {
    /**
     * Clones a color scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#cloneScheme)
     */
    cloneScheme(scheme: ColorScheme): ColorScheme;
    /**
     * Flips the colors in reverse order of the input color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#flipColors)
     */
    flipColors(scheme: ColorScheme): ColorScheme;
    /**
     * Returns all schemes matching the given color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getMatchingSchemes)
     */
    getMatchingSchemes(params: colorGetMatchingSchemesParams): ColorScheme[];
    /**
     * Returns a color scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    getSchemeByName(params: colorGetSchemeByNameParams): ColorScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for data-driven color visualizations in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes)
     */
    getSchemes(params: colorGetSchemesParams): ColorSchemes;
    /**
     * Returns an array of color schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    getSchemesByTag(params: colorGetSchemesByTagParams): ColorScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyColor: symbologyColor;

  export interface colorGetMatchingSchemesParams {
    /**
     * The name of the theme from which to get color schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getMatchingSchemes)
     */
    theme: "high-to-low" | "above-and-below" | "centered-on" | "extremes";
    /**
     * An array of colors used to match with schemes for the given theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getMatchingSchemes)
     */
    colors: Color[];
    /**
     * The geometry type of the layer for which the color scheme will be applied.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: string;
    /**
     * Indicates whether 3D volumetric symbols will be used in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getMatchingSchemes)
     */
    worldScale?: boolean;
    /**
     * The [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) instance used to calculate the suggested sizes of the symbols.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface colorGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The theme determines which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    theme: "high-to-low" | "above-and-below" | "centered-on" | "extremes";
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemeByName)
     */
    worldScale?: boolean;
  }

  export interface colorGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The theme determines which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    theme: "high-to-low" | "above-and-below" | "centered-on" | "extremes";
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemesByTag)
     */
    worldScale?: boolean;
  }

  export interface colorGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The theme determines which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes)
     */
    theme: "high-to-low" | "above-and-below" | "centered-on" | "extremes";
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes)
     */
    worldScale?: boolean;
  }

  /**
   * Properties defining the symbology scheme used to visualize features with attribute data-driven color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorScheme)
   */
  export type ColorScheme = ColorSchemeForPoint | ColorSchemeForPolyline | ColorSchemeForPolygon | ColorSchemeForMesh;

  /**
   * Properties defining the symbology scheme used to visualize mesh features with attribute data-driven color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
   */
  export interface ColorSchemeForMesh {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to use in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    id: string;
    /**
     * Indicates which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    theme: string;
    /**
     * The continuous color ramp that will be applied to the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    colors: Color[];
    /**
     * The color of the symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    noDataColor: Color;
    /**
     * Predefined classed color schemes to use for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    colorsForClassBreaks: ColorSchemeForMeshColorsForClassBreaks[];
    /**
     * The opacity of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize point features with attribute data-driven color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
   */
  export interface ColorSchemeForPoint {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to use in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    id: string;
    /**
     * Indicates which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    theme: string;
    /**
     * The continuous color ramp that will be applied to the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    colors: Color[];
    /**
     * The color of the symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    noDataColor: Color;
    /**
     * Predefined classed color schemes to use for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    colorsForClassBreaks: ColorSchemeForPointColorsForClassBreaks[];
    /**
     * Properties for defining the outline of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    outline: ColorSchemeForPointOutline;
    /**
     * The default size of the symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    size: number;
    /**
     * The opacity of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polygon features with attribute data-driven color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
   */
  export interface ColorSchemeForPolygon {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to use in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    id: string;
    /**
     * Indicates which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    theme: string;
    /**
     * The continuous color ramp that will be applied to the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    colors: Color[];
    /**
     * The color of the symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    noDataColor: Color;
    /**
     * Predefined classed color schemes to use for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    colorsForClassBreaks: ColorSchemeForPolygonColorsForClassBreaks[];
    /**
     * Properties for defining the outline of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    outline: ColorSchemeForPolygonOutline;
    /**
     * The opacity of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polyline features with attribute data-driven color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
   */
  export interface ColorSchemeForPolyline {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to use in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    id: string;
    /**
     * Indicates which values will be emphasized in the continuous ramp and the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    theme: string;
    /**
     * The continuous color ramp that will be applied to the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    colors: Color[];
    /**
     * The color of the symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    noDataColor: Color;
    /**
     * Predefined classed color schemes to use for the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    colorsForClassBreaks: ColorSchemeForPolylineColorsForClassBreaks[];
    /**
     * The default width of the symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    width: number;
    /**
     * The opacity of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemes)
   */
  export interface ColorSchemes {
    /**
     * The color scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemes)
     */
    primaryScheme: ColorScheme;
    /**
     * Additional color schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemes)
     */
    secondarySchemes: ColorScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemes)
     */
    basemapTheme: string;
  }

  /**
   * Describes a theme that pairs well with the given basemaps.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#Theme)
   */
  export interface colorTheme {
    /**
     * The name of the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#Theme)
     */
    name: string;
    /**
     * The label identifying the theme in UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#Theme)
     */
    label: string;
    /**
     * Text describing the theme in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#Theme)
     */
    description: string;
    /**
     * The Esri basemaps that pair well with the given theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#Theme)
     */
    basemaps: string[];
  }

  export interface ColorSchemeForMeshColorsForClassBreaks {
    /**
     * An array of hex color strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    colors: Color[];
    /**
     * The number of classes used in the classed color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForMesh)
     */
    numClasses: number;
  }

  export interface ColorSchemeForPointColorsForClassBreaks {
    /**
     * An array of hex color strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    colors: Color[];
    /**
     * The number of classes used in the classed color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    numClasses: number;
  }

  export interface ColorSchemeForPointOutline {
    /**
     * The outline color of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPoint)
     */
    width: number;
  }

  export interface ColorSchemeForPolygonColorsForClassBreaks {
    /**
     * An array of hex color strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    colors: Color[];
    /**
     * The number of classes used in the classed color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    numClasses: number;
  }

  export interface ColorSchemeForPolygonOutline {
    /**
     * The outline color of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolygon)
     */
    width: number;
  }

  export interface ColorSchemeForPolylineColorsForClassBreaks {
    /**
     * An array of hex color strings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    colors: Color[];
    /**
     * The number of classes used in the classed color visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html#ColorSchemeForPolyline)
     */
    numClasses: number;
  }

  /**
   * Object containing helper methods for getting optimal symbol schemes used to create [dot density visualizations](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-dotDensity.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html)
   */
  interface symbologyDotDensity {
    /**
     * Clones a dot density scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#cloneScheme)
     */
    cloneScheme(scheme: DotDensityScheme): DotDensityScheme;
    /**
     * Returns a dot density scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemeByName)
     */
    getSchemeByName(params: dotDensityGetSchemeByNameParams): DotDensityScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for dot density-based data-driven visualizations in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemes)
     */
    getSchemes(params: dotDensityGetSchemesParams): DotDensitySchemes;
    /**
     * Returns an array of dot density schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    getSchemesByTag(params: dotDensityGetSchemesByTagParams): DotDensityScheme[];
  }

  export const symbologyDotDensity: symbologyDotDensity;

  export interface dotDensityGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The number of colors to visualize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemeByName)
     */
    numColors: number;
  }

  export interface dotDensityGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The number of colors to visualize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemesByTag)
     */
    numColors: number;
  }

  export interface dotDensityGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The number of colors to visualize.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemes)
     */
    numColors: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize predominance for polygon features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
   */
  export interface DotDensityScheme {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    tags: string[];
    /**
     * The dot color for each attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    colors: Color[];
    /**
     * Properties for defining the outline of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    outline: DotDensitySchemeOutline;
    /**
     * The opacity of the symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensitySchemes)
   */
  export interface DotDensitySchemes {
    /**
     * The dot density scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensitySchemes)
     */
    primaryScheme: DotDensityScheme;
    /**
     * Additional dot density schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensitySchemes)
     */
    secondarySchemes: DotDensityScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensitySchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensitySchemes)
     */
    basemapTheme: string;
  }

  export interface DotDensitySchemeOutline {
    /**
     * The outline color of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    color: Color;
    /**
     * The width of the symbol's outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-dotDensity.html#DotDensityScheme)
     */
    width: number;
  }

  /**
   * Object containing helper methods for generating optimal settings for [FlowRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html)
   */
  interface symbologyFlow {
    /**
     * Clones a flow scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#cloneScheme)
     */
    cloneScheme(scheme: FlowScheme): FlowScheme;
    /**
     * Returns the flow scheme with the given name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemeByName)
     */
    getSchemeByName(params: flowGetSchemeByNameParams): FlowScheme;
    /**
     * Returns a primary scheme and secondary schemes defining properties for flow visualizations in a [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) and [ImageryTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemes)
     */
    getSchemes(params: flowGetSchemesParams): FlowSchemes;
    /**
     * Returns the flow schemes filtered by tags included and excluded from the paramters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    getSchemesByTag(params: flowGetSchemesByTagParams): FlowScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): flowTheme[];
  }

  export const symbologyFlow: symbologyFlow;

  export interface flowGetSchemeByNameParams {
    /**
     * The name of the desired scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemeByName)
     */
    name: string;
    /**
     * Determines how flow lines will render.
     *
     * @default "flow-line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemeByName)
     */
    theme?: "flow-line" | "wave-front";
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
  }

  export interface flowGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes excluding all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * Determines how flow lines will render.
     *
     * @default "flow-line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    theme?: "flow-line" | "wave-front";
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
  }

  export interface flowGetSchemesParams {
    /**
     * Determines how flow lines will render.
     *
     * @default "flow-line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemes)
     */
    theme?: "flow-line" | "wave-front";
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
  }

  /**
   * Suggested property values for defining the scheme used to visualize flow lines.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
   */
  export interface FlowScheme {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to use in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    id: string;
    /**
     * Determines how flow lines will render.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    theme: "flow-line" | "wave-front";
    /**
     * The suggested color of the flow lines if a color visual variable is not desired.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    color: Color;
    /**
     * The suggested colors to use if a [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) is set on the flow renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    colors: Color[];
    /**
     * The suggested [density](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#density) of the stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    density: number;
    /**
     * The suggested [flowSpeed](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#flowSpeed) of the stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    flowSpeed: number;
    /**
     * The suggested [length](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailLength) of the stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    trailLength: number;
    /**
     * The suggested [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailWidth) of the stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    trailWidth: number;
    /**
     * The suggested [cap style](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-FlowRenderer.html#trailCap) of the stream lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    trailCap: number;
    /**
     * The suggested [effect](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryTileLayer.html#effect) to apply to the input layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    layerEffect: string;
    /**
     * The suggested size to apply to small magnitude values if a [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) is set on the flow renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    minWidth: number;
    /**
     * The suggested size to apply to large magnitude values if a [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) is set on the flow renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    maxWidth: number;
    /**
     * The suggested opacity to apply to small magnitude values if an [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) is set on the flow renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    minOpacity: number;
    /**
     * The suggested opacity to apply to large magnitude values if an [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) is set on the flow renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowScheme)
     */
    maxOpacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowSchemes)
   */
  export interface FlowSchemes {
    /**
     * The flow scheme best suited for the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowSchemes)
     */
    primaryScheme: FlowScheme;
    /**
     * Additional flow schemes that may be used to visualize stream lines overlaid on the given basemap or basemap theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowSchemes)
     */
    secondarySchemes: FlowScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#FlowSchemes)
     */
    basemapTheme: string;
  }

  /**
   * Describes a theme that pairs well with the given basemaps.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#Theme)
   */
  export interface flowTheme {
    /**
     * The name of the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#Theme)
     */
    name: string;
    /**
     * The label identifying the theme in UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#Theme)
     */
    label: string;
    /**
     * Text describing the theme in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#Theme)
     */
    description: string;
    /**
     * The Esri basemaps that pair well with the given theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-flow.html#Theme)
     */
    basemaps: string[];
  }

  /**
   * Object containing helper methods for generating optimal colors for heatmap visualizations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html)
   */
  interface symbologyHeatmap {
    /**
     * Clones a heatmap scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#cloneScheme)
     */
    cloneScheme(scheme: HeatmapScheme): HeatmapScheme;
    /**
     * Returns a heatmap scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemeByName)
     */
    getSchemeByName(params: heatmapGetSchemeByNameParams): HeatmapScheme;
    /**
     * Returns a primary scheme and secondary schemes defining properties for heatmap visualizations in a point [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemes)
     */
    getSchemes(params: heatmapGetSchemesParams): HeatmapSchemes;
    /**
     * Returns an array of heatmap schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemesByTag)
     */
    getSchemesByTag(params: heatmapGetSchemesByTagParams): HeatmapScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyHeatmap: symbologyHeatmap;

  export interface heatmapGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
  }

  export interface heatmapGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
  }

  export interface heatmapGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
  }

  /**
   * Properties defining the symbology scheme used to visualize point features as a heatmap.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
   */
  export interface HeatmapScheme {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
     */
    tags: string[];
    /**
     * The unique id of the color scheme to usee in the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
     */
    id: string;
    /**
     * A continuous color ramp used to construct a heatmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
     */
    colors: Color[];
    /**
     * The opacity of the heatmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapScheme)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapSchemes)
   */
  export interface HeatmapSchemes {
    /**
     * The heatmap color scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapSchemes)
     */
    primaryScheme: HeatmapScheme;
    /**
     * Additional heatmap color schemes that may be used to visualize data overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapSchemes)
     */
    secondarySchemes: HeatmapScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-heatmap.html#HeatmapSchemes)
     */
    basemapTheme: string;
  }

  /**
   * Object containing helper methods for generating optimal symbols for location-only visualizations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html)
   */
  interface symbologyLocation {
    /**
     * Clones a location scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#cloneScheme)
     */
    cloneScheme(scheme: LocationScheme): LocationScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for location-only visualizions in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getSchemes)
     */
    getSchemes(params: locationGetSchemesParams): LocationSchemes;
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyLocation: symbologyLocation;

  export interface locationGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getSchemes)
     */
    worldScale?: boolean;
  }

  /**
   * Defines the symbology scheme used to visualize default location-based symbols depending on the layer's geometry type.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationScheme)
   */
  export type LocationScheme =
    | LocationSchemeForPoint
    | LocationSchemeForPolyline
    | LocationSchemeForPolygon
    | LocationSchemeForMesh;

  /**
   * Properties defining the location-only symbology scheme used to visualize mesh features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForMesh)
   */
  export interface LocationSchemeForMesh {
    /**
     * The color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForMesh)
     */
    color: Color;
    /**
     * The opacity of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForMesh)
     */
    opacity: number;
  }

  /**
   * Properties defining the location-only symbology scheme used to visualize point features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
   */
  export interface LocationSchemeForPoint {
    /**
     * The fill color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    color: Color;
    /**
     * Properties for defining the outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    outline: LocationSchemeForPointOutline;
    /**
     * The size of the marker symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    size: number;
    /**
     * The opacity of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    opacity: number;
  }

  /**
   * Properties defining the location-only symbology scheme used to visualize polygon features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
   */
  export interface LocationSchemeForPolygon {
    /**
     * The color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
     */
    color: Color;
    /**
     * Properties for defining the outline of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
     */
    outline: LocationSchemeForPolygonOutline;
    /**
     * The opacity of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
     */
    opacity: number;
  }

  /**
   * Properties defining the location-only symbology scheme used to visualize polyline features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolyline)
   */
  export interface LocationSchemeForPolyline {
    /**
     * The color of the line symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolyline)
     */
    color: Color;
    /**
     * The width of the line symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolyline)
     */
    width: number;
    /**
     * The opacity of the line symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolyline)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemes)
   */
  export interface LocationSchemes {
    /**
     * The location scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemes)
     */
    primaryScheme: LocationScheme;
    /**
     * Additional location schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemes)
     */
    secondarySchemes: LocationScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemes)
     */
    basemapTheme: string;
  }

  export interface LocationSchemeForPointOutline {
    /**
     * The outline color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the marker symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPoint)
     */
    width: number;
  }

  export interface LocationSchemeForPolygonOutline {
    /**
     * The outline color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the fill symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-location.html#LocationSchemeForPolygon)
     */
    width: number;
  }

  /**
   * Object containing helper methods for getting optimal color schemes used to create [pie chart visualizations](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-pieChart.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html)
   */
  interface symbologyPieChart {
    /**
     * Clones a pie chart scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#cloneScheme)
     */
    cloneScheme(scheme: PieChartScheme): PieChartScheme;
    /**
     * Returns a pie chart scheme with the given name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemeByName)
     */
    getSchemeByName(params: pieChartGetSchemeByNameParams): PieChartScheme;
    /**
     * Returns a primary scheme and secondary schemes defining properties for pie chart visualizations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemes)
     */
    getSchemes(params: pieChartGetSchemesParams): PieChartSchemes;
    /**
     * Returns an array of pie chart schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    getSchemesByTag(params: pieChartGetSchemesByTagParams): PieChartScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyPieChart: symbologyPieChart;

  export interface pieChartGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "polygon";
    /**
     * The number of values to include in the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemeByName)
     */
    numColors: number;
  }

  export interface pieChartGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "polygon";
    /**
     * The number of categories to include in the pie chart visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemesByTag)
     */
    numColors: number;
  }

  export interface pieChartGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "polygon";
    /**
     * The number of unique values to include in the pie chart visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemes)
     */
    numColors: number;
  }

  /**
   * Properties defining the colors and schemes used to visualize pie charts.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
   */
  export interface PieChartScheme {
    /**
     * The unique name of the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    name: string;
    /**
     * Tags associated with the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    tags: string[];
    /**
     * The colors used to visualize pie slices in the chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    colors: Color[];
    /**
     * The color of the "others" category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    colorForOthersCategory: Color;
    /**
     * Properties for defining the outline of the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    outline: PieChartSchemeOutline;
    /**
     * The scheme defining the size visual variable used to vary chart size per feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    sizeScheme: SizeScheme;
    /**
     * The default size of the pie charts in points or pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    size: number | string;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartSchemes)
   */
  export interface PieChartSchemes {
    /**
     * The suggest default chart scheme based on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartSchemes)
     */
    primaryScheme: PieChartScheme;
    /**
     * Additional chart schemes that may be used to visualize data overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartSchemes)
     */
    secondarySchemes: PieChartScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartSchemes)
     */
    basemapTheme: string;
  }

  export interface PieChartSchemeOutline {
    /**
     * The outline color of the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    color: Color;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-pieChart.html#PieChartScheme)
     */
    width: number | string;
  }

  /**
   * Object containing helper methods for getting optimal symbol schemes used to create [predominance visualizations](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-predominance.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html)
   */
  interface symbologyPredominance {
    /**
     * Clones a predominance scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#cloneScheme)
     */
    cloneScheme(scheme: PredominanceScheme): PredominanceScheme;
    /**
     * Returns a predominance scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    getSchemeByName(params: predominanceGetSchemeByNameParams): PredominanceScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for predominance-based data-driven visualizations in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    getSchemes(params: predominanceGetSchemesParams): PredominanceSchemes;
    /**
     * Returns an array of predominance schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    getSchemesByTag(params: predominanceGetSchemesByTagParams): PredominanceScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyPredominance: symbologyPredominance;

  export interface predominanceGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The number of unique values to include in the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    numColors: number;
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemeByName)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface predominanceGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The number of unique values to include in the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    numColors: number;
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemesByTag)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface predominanceGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * The number of unique values to include in the predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    numColors: number;
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  /**
   * Defines the symbology scheme used to visualize predominance based on the layer's geometry type.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceScheme)
   */
  export type PredominanceScheme =
    | PredominanceSchemeForPoint
    | PredominanceSchemeForPolyline
    | PredominanceSchemeForPolygon
    | PredominanceSchemeForMesh;

  /**
   * Properties defining the symbology scheme used to visualize predominance for mesh features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
   */
  export interface PredominanceSchemeForMesh {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
     */
    tags: string[];
    /**
     * The fill colors of the mesh symbols used for each category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
     */
    colors: Color[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
     */
    noDataColor: Color;
    /**
     * The opacity of the symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForMesh)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize predominance for point features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
   */
  export interface PredominanceSchemeForPoint {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    tags: string[];
    /**
     * The fill colors of the point symbols used for each category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    colors: Color[];
    /**
     * The color of the point symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    outline: PredominanceSchemeForPointOutline;
    /**
     * The opacity of the point symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    opacity: number;
    /**
     * The scheme defining the size visual variable if included in a predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    sizeScheme: SizeSchemeForPoint;
  }

  /**
   * Properties defining the symbology scheme used to visualize predominance for polygon features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
   */
  export interface PredominanceSchemeForPolygon {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    tags: string[];
    /**
     * The fill colors of the polygon symbols used for each category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    colors: Color[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    outline: PredominanceSchemeForPolygonOutline;
    /**
     * The opacity of the symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    opacity: number;
    /**
     * The scheme defining the size visual variable if included in a predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    sizeScheme: SizeSchemeForPolygon;
  }

  /**
   * Properties defining the symbology scheme used to visualize predominance for polyline features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
   */
  export interface PredominanceSchemeForPolyline {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    tags: string[];
    /**
     * The colors of the line symbols used for each category.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    colors: Color[];
    /**
     * The color of the line symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    noDataColor: Color;
    /**
     * The default width of the line symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    width: number;
    /**
     * The opacity of the line symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    opacity: number;
    /**
     * The scheme defining the size visual variable if included in a predominance visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolyline)
     */
    sizeScheme: SizeSchemeForPolyline;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemes)
   */
  export interface PredominanceSchemes {
    /**
     * The predominance scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemes)
     */
    primaryScheme: PredominanceScheme;
    /**
     * Additional predominance schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemes)
     */
    secondarySchemes: PredominanceScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemes)
     */
    basemapTheme: string;
  }

  export interface PredominanceSchemeForPointOutline {
    /**
     * The outline color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the marker symbol in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPoint)
     */
    width: number;
  }

  export interface PredominanceSchemeForPolygonOutline {
    /**
     * The outline color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the fill symbol's outline in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-predominance.html#PredominanceSchemeForPolygon)
     */
    width: number;
  }

  /**
   * Object containing helper methods for getting optimal symbol schemes used to create [relationship (bivariate choropleth) visualizations](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-relationship.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html)
   */
  interface symbologyRelationship {
    /**
     * Clones a relationship (bivariate color) scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#cloneScheme)
     */
    cloneScheme(scheme: RelationshipScheme): RelationshipScheme;
    /**
     * Returns a relationship scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    getSchemeByName(params: relationshipGetSchemeByNameParams): RelationshipScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for relationship-based (bivariate choropleth) data-driven visualizations in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes)
     */
    getSchemes(params: relationshipGetSchemesParams): RelationshipSchemes;
    /**
     * Returns an array of relationship schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    getSchemesByTag(params: relationshipGetSchemesByTagParams): RelationshipScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyRelationship: symbologyRelationship;

  export interface relationshipGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemeByName)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface relationshipGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemesByTag)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface relationshipGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines which set of primary and secondary color schemes to return.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes)
     */
    theme?: "default";
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  /**
   * Defines the symbology scheme used to visualize relationship renderers (bivariate color) based on the layer's geometry type.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipScheme)
   */
  export type RelationshipScheme =
    | RelationshipSchemeForPoint
    | RelationshipSchemeForPolyline
    | RelationshipSchemeForPolygon
    | RelationshipSchemeForMesh;

  /**
   * Properties defining the symbology scheme used to visualize relationship renderers (bivariate color) for mesh features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
   */
  export interface RelationshipSchemeForMesh {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    tags: string[];
    /**
     * The ID uniquely identifying the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    id: string;
    /**
     * Defines the colors to use in each bin of the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    colorsForClassBreaks: RelationshipSchemeForMeshColorsForClassBreaks[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    noDataColor: Color;
    /**
     * The opacity of the fill symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize relationship renderers (bivariate color) for point features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
   */
  export interface RelationshipSchemeForPoint {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    tags: string[];
    /**
     * The ID uniquely identifying the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    id: string;
    /**
     * Defines the colors to use in each bin of the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    colorsForClassBreaks: RelationshipSchemeForPointColorsForClassBreaks[];
    /**
     * The color of the point symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    outline: RelationshipSchemeForPointOutline;
    /**
     * The opacity of the point symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    opacity: number;
    /**
     * The size of the point symbol in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    size: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize relationship renderers (bivariate color) for polygon features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
   */
  export interface RelationshipSchemeForPolygon {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    tags: string[];
    /**
     * The ID uniquely identifying the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    id: string;
    /**
     * Defines the colors to use in each bin of the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    colorsForClassBreaks: RelationshipSchemeForPolygonColorsForClassBreaks[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    outline: RelationshipSchemeForPolygonOutline;
    /**
     * The opacity of the fill symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize relationship renderers (bivariate color) for polyline features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
   */
  export interface RelationshipSchemeForPolyline {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    tags: string[];
    /**
     * The ID uniquely identifying the color scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    id: string;
    /**
     * Defines the colors to use in each bin of the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    colorsForClassBreaks: RelationshipSchemeForPolylineColorsForClassBreaks[];
    /**
     * The color of the line symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    noDataColor: Color;
    /**
     * The opacity of the line symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    opacity: number;
    /**
     * The width of the line symbol in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    width: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemes)
   */
  export interface RelationshipSchemes {
    /**
     * The relationship scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemes)
     */
    primaryScheme: RelationshipScheme;
    /**
     * Additional relationship schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemes)
     */
    secondarySchemes: RelationshipScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemes)
     */
    basemapTheme: string;
  }

  export interface RelationshipSchemeForMeshColorsForClassBreaks {
    /**
     * The colors of the fill symbols used for each bin in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    colors: Color[][];
    /**
     * The number of breaks (rows/columns) for each variable in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForMesh)
     */
    numClasses: number;
  }

  export interface RelationshipSchemeForPointColorsForClassBreaks {
    /**
     * The fill colors of the point symbols used for each bin in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    colors: Color[][];
    /**
     * The number of breaks (rows/columns) for each variable in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    numClasses: number;
  }

  export interface RelationshipSchemeForPointOutline {
    /**
     * The outline color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPoint)
     */
    width: number;
  }

  export interface RelationshipSchemeForPolygonColorsForClassBreaks {
    /**
     * The colors of the fill symbols used for each bin in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    colors: Color[][];
    /**
     * The number of breaks (rows/columns) for each variable in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    numClasses: number;
  }

  export interface RelationshipSchemeForPolygonOutline {
    /**
     * The outline color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolygon)
     */
    width: number;
  }

  export interface RelationshipSchemeForPolylineColorsForClassBreaks {
    /**
     * The colors of the line symbols used for each bin in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    colors: Color[][];
    /**
     * The number of breaks (rows/columns) for each variable in the relationship visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-relationship.html#RelationshipSchemeForPolyline)
     */
    numClasses: number;
  }

  /**
   * Object containing helper methods for generating optimal symbols for data-driven size visualizations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html)
   */
  interface symbologySize {
    /**
     * Clones a size scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#cloneScheme)
     */
    cloneScheme(scheme: SizeScheme): SizeScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for size-based data-driven visualizations in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getSchemes)
     */
    getSchemes(params: sizeGetSchemesParams): SizeSchemes;
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologySize: symbologySize;

  export interface sizeGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon";
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getSchemes)
     */
    worldScale?: boolean;
  }

  /**
   * Properties defining the symbology scheme used to visualize features with attribute data-driven size.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeScheme)
   */
  export type SizeScheme = SizeSchemeForPoint | SizeSchemeForPolyline | SizeSchemeForPolygon;

  /**
   * Properties defining the symbology scheme used to visualize point features driven by attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
   */
  export interface SizeSchemeForPoint {
    /**
     * The fill color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    color: Color;
    /**
     * The color of the marker symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    outline: SizeSchemeForPointOutline;
    /**
     * The default size of the marker symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    size: number;
    /**
     * The size of the marker symbol (in pixels or points) used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    noDataSize: number;
    /**
     * The minimum size of a marker symbol in pixels or points representing features with low data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    minSize: number;
    /**
     * The maximum size of a marker symbol in pixels or points representing features with high data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    maxSize: number;
    /**
     * The opacity of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polygon features driven by attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
   */
  export interface SizeSchemeForPolygon {
    /**
     * The scheme defining the symbology of the marker symbol used to represent polygon features by size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    marker: SizeSchemeForPoint;
    /**
     * The schema defining the backgound symbol of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    background: SizeSchemeForPolygonBackground;
    /**
     * The opacity of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polyline features driven by attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
   */
  export interface SizeSchemeForPolyline {
    /**
     * The color of the line symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    color: Color;
    /**
     * The color of the line symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    noDataColor: Color;
    /**
     * The default width of the line symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    width: number;
    /**
     * The width of the line symbol in pixels or points used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    noDataWidth: number;
    /**
     * The minimum width of a line symbol in pixels or points representing features with low data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    minWidth: number;
    /**
     * The maximum width of a line symbol in pixels or points representing features with high data values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    maxWidth: number;
    /**
     * The opacity of the line symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolyline)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemes)
   */
  export interface SizeSchemes {
    /**
     * The size scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemes)
     */
    primaryScheme: SizeScheme;
    /**
     * Additional size schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemes)
     */
    secondarySchemes: SizeScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemes)
     */
    basemapTheme: string;
  }

  export interface SizeSchemeForPointOutline {
    /**
     * The outline color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the marker symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPoint)
     */
    width: number;
  }

  export interface SizeSchemeForPolygonBackground {
    /**
     * The color of the fill symbol representing the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    color: Color;
    /**
     * Properties for defining the outline of the background fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    outline: SizeSchemeForPolygonBackgroundOutline;
  }

  export interface SizeSchemeForPolygonBackgroundOutline {
    /**
     * The outline color of the background fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the background fill symbol's outline in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-size.html#SizeSchemeForPolygon)
     */
    width: number;
  }

  /**
   * This module provides convenience methods for querying color ramps (arrays of colors) used in the smart mapping [color symbology](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html) module.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html)
   */
  interface supportColorRamps {
    /**
     * Returns all color ramps available in smartMapping [color schemes](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#all)
     */
    all(): supportColorRampsColorRamp[];
    /**
     * Returns the color ramp available in the smartMapping [color schemes](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html) matching the given name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#byName)
     */
    byName(name: string): supportColorRampsColorRamp;
    /**
     * Returns the color ramps available in the smartMapping [color schemes](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html) matching the given tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#byTag)
     */
    byTag(params: colorRampsByTagParams): supportColorRampsColorRamp[];
    /**
     * Returns the names of all color ramps available in the smartMapping [color schemes](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-color.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#names)
     */
    names(): string[];
  }

  export const supportColorRamps: supportColorRamps;

  /**
   * Defines a ramp of colors to use for visualizing data in a chart, [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html), or [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
   */
  export interface supportColorRampsColorRamp {
    /**
     * The unique name of the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    name: string;
    /**
     * Tags associated with the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    tags: string[];
    /**
     * An array of five colors intended for displaying a continuous color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    colors: Color[];
    /**
     * Contains an array of ramps ranging in size from 1 color to 10 colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    colorsForClassBreaks: ColorRampColorsForClassBreaks[];
  }

  export interface colorRampsByTagParams {
    /**
     * When provided, only ramps containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#byTag)
     */
    includedTags?: string[];
    /**
     * When provided, only ramps missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#byTag)
     */
    excludedTags?: string[];
  }

  export interface ColorRampColorsForClassBreaks {
    /**
     * An array of 1 to 10 colors intended for displaying as a ramp with distinct or unique colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    colors: Color[];
    /**
     * The number of classes or colors used in the ramp of discrete colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-support-colorRamps.html#ColorRamp)
     */
    numClasses: number;
  }

  /**
   * Object containing helper methods for getting optimal symbol themes used to create data-driven visualizations of unique values or types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html)
   */
  interface symbologyType {
    /**
     * Clones a type scheme object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#cloneScheme)
     */
    cloneScheme(scheme: TypeScheme): TypeScheme;
    /**
     * Returns a type scheme with the provided name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    getSchemeByName(params: typeGetSchemeByNameParams): TypeScheme;
    /**
     * Returns a primary scheme and secondary schemes defining symbol properties for type-based data-driven visualizations in a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    getSchemes(params: typeGetSchemesParams): TypeSchemes;
    /**
     * Returns an array of type schemes with the provided tags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    getSchemesByTag(params: typeGetSchemesByTagParams): TypeScheme[];
    /**
     * Returns metadata for the available themes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getThemes)
     */
    getThemes(basemap?: string | Basemap): colorTheme[];
  }

  export const symbologyType: symbologyType;

  export interface typeGetSchemeByNameParams {
    /**
     * The name of the scheme to retrieve.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    name: string;
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines whether to return a standardized  unique value scheme for point cloud class renderers (i.e.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    theme?: "point-cloud-class" | "default";
    /**
     * _Since 4.31_ Indicates the number of colors to include in the color ramp within the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    numColors?: number;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemeByName)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface typeGetSchemesByTagParams {
    /**
     * When provided, only schemes containing all the matching tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    includedTags?: string[];
    /**
     * When provided, only schemes missing all the provided tags will be returned.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    excludedTags?: string[];
    /**
     * The basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines whether to return a standardized  unique value scheme for point cloud class renderers (i.e.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    theme?: "point-cloud-class" | "default";
    /**
     * _Since 4.31_ Indicates the number of colors to include in the color ramp within the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    numColors?: number;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemesByTag)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  export interface typeGetSchemesParams {
    /**
     * The Esri basemap to pair with the visualization.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    basemap?: string | Basemap;
    /**
     * If you have a non-Esri basemap (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    basemapTheme?: "light" | "dark";
    /**
     * The geometry type of the features to visualize.
     *
     * [Read more...](global.html#geometryType)
     */
    geometryType: "point" | "multipoint" | "polyline" | "polygon" | "mesh" | "multipatch";
    /**
     * Determines whether to return a standardized  unique value scheme for point cloud class renderers (i.e.
     *
     * @default default
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    theme?: "point-cloud-class" | "default";
    /**
     * _Since 4.31_ Indicates the number of colors to include in the color ramp within the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    numColors?: number;
    /**
     * Indicates if the size units of the scheme will be in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes)
     */
    worldScale?: boolean;
    /**
     * The SceneView instance in which the scheme will be used.
     *
     * [Read more...](global.html)
     */
    view?: SceneView;
  }

  /**
   * Properties defining the symbology scheme used to visualize features with attribute data-driven types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeScheme)
   */
  export type TypeScheme = TypeSchemeForPoint | TypeSchemeForPolyline | TypeSchemeForPolygon | TypeSchemeForMesh;

  /**
   * Properties defining the symbology scheme used to visualize mesh features driven by type-based attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
   */
  export interface TypeSchemeForMesh {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
     */
    tags: string[];
    /**
     * The fill colors of the symbols comprising the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
     */
    colors: Color[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
     */
    noDataColor: Color;
    /**
     * The opacity of the symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForMesh)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize point features driven by type-based attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
   */
  export interface TypeSchemeForPoint {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    tags: string[];
    /**
     * The fill colors of the point symbols comprising the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    colors: Color[];
    /**
     * The color of the point symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    outline: TypeSchemeForPointOutline;
    /**
     * The default size of the point symbol in pixels, points, or real-world units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    size: number;
    /**
     * The opacity of the point symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polygon features driven by type-based attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
   */
  export interface TypeSchemeForPolygon {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    tags: string[];
    /**
     * The fill colors of the symbols comprising the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    colors: Color[];
    /**
     * The color of the fill symbol used to indicate features with no data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    noDataColor: Color;
    /**
     * Properties for defining the outline of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    outline: TypeSchemeForPolygonOutline;
    /**
     * The opacity of the symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    opacity: number;
  }

  /**
   * Properties defining the symbology scheme used to visualize polyline features driven by type-based attribute data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
   */
  export interface TypeSchemeForPolyline {
    /**
     * The unique name of the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    name: string;
    /**
     * Tags associated with the scheme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    tags: string[];
    /**
     * The colors of the line symbols comprising the theme.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    colors: Color[];
    /**
     * The color of the line symbol used to indicate features with no data and features that are out of range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    noDataColor: Color;
    /**
     * The default width of the line symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    width: number;
    /**
     * The opacity of the line symbol (0 - 1).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolyline)
     */
    opacity: number;
  }

  /**
   * The return object of the [getSchemes()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#getSchemes) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemes)
   */
  export interface TypeSchemes {
    /**
     * The type scheme best suited for the given basemap and geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemes)
     */
    primaryScheme: TypeScheme;
    /**
     * Additional type schemes that may be used to visualize data of the given geometry type overlaid on the given basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemes)
     */
    secondarySchemes: TypeScheme[];
    /**
     * The ID of the basemap associated with the given schemes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemes)
     */
    basemapId: string;
    /**
     * Indicates whether the average basemap color is `light` or `dark`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemes)
     */
    basemapTheme: string;
  }

  export interface TypeSchemeForPointOutline {
    /**
     * The outline color of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    color: Color;
    /**
     * The width of the marker symbol in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPoint)
     */
    width: number;
  }

  export interface TypeSchemeForPolygonOutline {
    /**
     * The outline color of the fill symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    color: Color;
    /**
     * The width of the fill symbol's outline in pixels or points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-symbology-type.html#TypeSchemeForPolygon)
     */
    width: number;
  }

  export interface ActionBase extends Accessor, Identifiable {}

  export class ActionBase {
    /**
     * Actions are customizable behavior which can be executed in certain widgets such as [Popups](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), a [BasemapLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html), or a [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html)
     */

    constructor(properties?: ActionBaseProperties);

    /**
     * Set this property to `true` to display a spinner icon.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#active)
     */
    active: boolean;
    /**
     * This adds a CSS class to the [ActionButton's](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#className)
     */
    className: string;
    /**
     * Indicates whether this action is disabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#disabled)
     */
    disabled: boolean;
    /**
     * Calcite icon used for the action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#icon)
     */
    icon: string;
    /**
     * The name of the ID assigned to this action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#id)
     */
    id: string;
    /**
     * The title of the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#title)
     */
    title: string;
    /**
     * Specifies the type of action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#type)
     */
    type: "button" | "slider" | "toggle";
    /**
     * Indicates if the action is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#visible)
     */
    visible: boolean;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#clone)
     */
    clone(): ActionBase;
  }

  interface ActionBaseProperties {
    /**
     * Set this property to `true` to display a spinner icon.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#active)
     */
    active?: boolean;
    /**
     * This adds a CSS class to the [ActionButton's](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#className)
     */
    className?: string;
    /**
     * Indicates whether this action is disabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#disabled)
     */
    disabled?: boolean;
    /**
     * Calcite icon used for the action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#icon)
     */
    icon?: string;
    /**
     * The name of the ID assigned to this action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#id)
     */
    id?: string;
    /**
     * The title of the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#title)
     */
    title?: string;
    /**
     * Specifies the type of action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#type)
     */
    type?: "button" | "slider" | "toggle";
    /**
     * Indicates if the action is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionBase.html#visible)
     */
    visible?: boolean;
  }

  export class ActionButton extends ActionBase {
    /**
     * A customizable button that performs a specific action(s) used in widgets such as the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html), [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html), and [BasemapLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html)
     */

    constructor(properties?: ActionButtonProperties);

    /**
     * The URL to an image that will be used to represent the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html#image)
     */
    image: string;
    /**
     * Specifies the type of action.
     *
     * @default "button"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html#type)
     */
    readonly type: "button";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html#clone)
     */
    clone(): ActionButton;
  }

  interface ActionButtonProperties extends ActionBaseProperties {
    /**
     * The URL to an image that will be used to represent the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html#image)
     */
    image?: string;
  }

  export class ActionToggle extends ActionBase {
    /**
     * A customizable toggle used in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget that performs a specific action(s) which can be toggled on/off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html)
     */

    constructor(properties?: ActionToggleProperties);

    /**
     * Specifies the type of action.
     *
     * @default "toggle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html#type)
     */
    readonly type: "toggle";
    /**
     * Indicates the value of whether the action is toggled on/off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html#value)
     */
    value: boolean;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html#clone)
     */
    clone(): ActionToggle;
  }

  interface ActionToggleProperties extends ActionBaseProperties {
    /**
     * Indicates the value of whether the action is toggled on/off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html#value)
     */
    value?: boolean;
  }

  export class BasemapStyle extends Accessor {
    /**
     * The style of the basemap from the [basemap styles service (v2)](https://developers.arcgis.com/rest/basemap-styles/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html)
     */

    constructor(properties?: BasemapStyleProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#apiKey)
     */
    apiKey: string;
    /**
     * The id of the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#id)
     */
    id: string;
    /**
     * The language of the place labels in the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#language)
     */
    language: string;
    /**
     * Indicates whether to display [places](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html) with the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#places)
     */
    places: "all" | "attributed" | "none";
    /**
     * The [URL](https://developers.arcgis.com/rest/basemap-styles/#service-url) to the basemap styles service.
     *
     * @default "https://basemapstyles-api.arcgis.com/arcgis/rest/services/styles/v2/webmaps"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#serviceUrl)
     */
    serviceUrl: string;
    /**
     * Displays country boundaries and labels based on a specific view of a country.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#worldview)
     */
    worldview: string;
  }

  interface BasemapStyleProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#apiKey)
     */
    apiKey?: string;
    /**
     * The id of the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#id)
     */
    id?: string;
    /**
     * The language of the place labels in the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#language)
     */
    language?: string;
    /**
     * Indicates whether to display [places](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-places.html) with the basemap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#places)
     */
    places?: "all" | "attributed" | "none";
    /**
     * The [URL](https://developers.arcgis.com/rest/basemap-styles/#service-url) to the basemap styles service.
     *
     * @default "https://basemapstyles-api.arcgis.com/arcgis/rest/services/styles/v2/webmaps"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#serviceUrl)
     */
    serviceUrl?: string;
    /**
     * Displays country boundaries and labels based on a specific view of a country.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-BasemapStyle.html#worldview)
     */
    worldview?: string;
  }

  export class LayersMixin {
    /**
     * A collection of operational [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#layers)
     */
    layers: Collection<Layer>;

    /**
     * Adds a layer to the [layers collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#add)
     */
    add(layer: Layer | Promise<any>, index?: number): void;
    /**
     * Adds a layer or an array of layers to the [layers collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#addMany)
     */
    addMany(layers: Layer[], index?: number): void;
    /**
     * Returns a layer based on the given layer ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#findLayerById)
     */
    findLayerById(layerId: string): Layer;
    /**
     * Removes the specified layer from the layers collection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#remove)
     */
    remove(layer: Layer): Layer;
    /**
     * Removes all layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#removeAll)
     */
    removeAll(): Layer[];
    /**
     * Removes the specified layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#removeMany)
     */
    removeMany(layers: Layer[]): Layer[];
    /**
     * Changes the layer order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#reorder)
     */
    reorder(layer: Layer, index: number): Layer;
  }

  interface LayersMixinProperties {
    /**
     * A collection of operational [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-LayersMixin.html#layers)
     */
    layers?: CollectionProperties<LayerProperties> | LayerProperties[];
  }

  export interface MapFloorInfo extends Accessor, JSONSupport {}

  export class MapFloorInfo {
    /**
     * Floor-aware maps facilitate visualizing, editing, and analyzing indoor data and they allow for fast navigation of buildings in maps and scenes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html)
     */

    constructor(properties?: MapFloorInfoProperties);

    /**
     * Contains the facility features of a floor plan, which describe the footprints of managed buildings and other structures.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#facilityLayer)
     */
    facilityLayer: FacilityLayerInfo;
    /**
     * Contains the level features of a floor plan, which describe the footprint of each occupiable floor contained in a managed facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#levelLayer)
     */
    levelLayer: LevelLayerInfo;
    /**
     * Contains the site features of a floor plan, which describe the boundaries of managed sites and is used for visualization in mapmaking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#siteLayer)
     */
    siteLayer: SiteLayerInfo;

    static fromJSON(json: any): MapFloorInfo;
  }

  interface MapFloorInfoProperties {
    /**
     * Contains the facility features of a floor plan, which describe the footprints of managed buildings and other structures.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#facilityLayer)
     */
    facilityLayer?: FacilityLayerInfoProperties;
    /**
     * Contains the level features of a floor plan, which describe the footprint of each occupiable floor contained in a managed facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#levelLayer)
     */
    levelLayer?: LevelLayerInfoProperties;
    /**
     * Contains the site features of a floor plan, which describe the boundaries of managed sites and is used for visualization in mapmaking.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-MapFloorInfo.html#siteLayer)
     */
    siteLayer?: SiteLayerInfoProperties;
  }

  /**
   * Various utils for working with [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widget functionality.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html)
   */
  interface popupUtils {
    /**
     * Creates an array of [fieldInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-FieldInfo.html) used for populating [FieldsContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createFieldInfos)
     */
    createFieldInfos(config: FieldInfosConfig, options?: CreatePopupTemplateOptions): FieldInfo[];
    /**
     * Creates [fields](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html) content used for populating a [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createFieldsContent)
     */
    createFieldsContent(config: FieldInfosConfig, options?: CreatePopupTemplateOptions): FieldsContent;
    /**
     * Creates a popup template given the specified [config](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config) information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createPopupTemplate)
     */
    createPopupTemplate(config: Config, options?: CreatePopupTemplateOptions): PopupTemplate;
    /**
     * Creates a default popup template to use for [FeatureReductionBinning](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionBinning.html) or [FeatureReductionCluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) visualizations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createPopupTemplateForFeatureReduction)
     */
    createPopupTemplateForFeatureReduction(
      params: popupUtilsCreatePopupTemplateForFeatureReductionParams,
    ): PopupTemplate;
  }

  export const popupUtils: popupUtils;

  /**
   * A configuration object containing properties for creating a [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
   */
  export interface Config {
    /**
     * The display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
     */
    displayField?: string;
    /**
     * The title for the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
     */
    title: string;
    /**
     * The fields that record who adds or edits data in the feature service and when the edit is made.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
     */
    editFieldsInfo?: EditFieldsInfo;
    /**
     * The fields displayed within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
     */
    fields: Field[];
    /**
     * The object id field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#Config)
     */
    objectIdField?: string;
  }

  /**
   * Options for creating the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#CreatePopupTemplateOptions)
   */
  export interface CreatePopupTemplateOptions {
    /**
     * An array of field types to ignore when creating the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#CreatePopupTemplateOptions)
     */
    ignoreFieldTypes?: string[];
    /**
     * An array of field names set to be visible within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#CreatePopupTemplateOptions)
     */
    visibleFieldNames?: Set<string>;
  }

  /**
   * A configuration object containing field properties for creating [FieldsContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html) for a [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#FieldInfosConfig)
   */
  export interface FieldInfosConfig {
    /**
     * The fields that record who adds or edits data in the feature service and when the edit is made.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#FieldInfosConfig)
     */
    editFieldsInfo?: EditFieldsInfo;
    /**
     * The fields displayed within the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#FieldInfosConfig)
     */
    fields: Field[];
    /**
     * The object id field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#FieldInfosConfig)
     */
    objectIdField?: string;
  }

  export interface popupUtilsCreatePopupTemplateForFeatureReductionParams {
    /**
     * The FeatureReduction object for which to create a default popup template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createPopupTemplateForFeatureReduction)
     */
    featureReduction: FeatureReductionBinning | FeatureReductionCluster;
    /**
     * The fields of the layer aggregated during the feature reduction process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createPopupTemplateForFeatureReduction)
     */
    fields?: Field[];
    /**
     * The title to display in the popup template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-popupUtils.html#createPopupTemplateForFeatureReduction)
     */
    title?: string;
  }

  export class TablesMixin {
    /**
     * A collection of [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) instances that are tables saved in a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) and/or a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-TablesMixin.html#tables)
     */
    tables: Collection<Layer>;

    /**
     * Returns a table based on the given table ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-TablesMixin.html#findTableById)
     */
    findTableById(tableId: string): Layer;
  }

  interface TablesMixinProperties {
    /**
     * A collection of [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) instances that are tables saved in a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) and/or a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-TablesMixin.html#tables)
     */
    tables?: CollectionProperties<LayerProperties> | LayerProperties[];
  }

  /**
   * Provides utility methods for working with [dates](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-timeUtils.html)
   */
  interface timeUtils {
    /**
     * Returns the time extent of all layers which are "feature based" time-aware and/or have a "layer based" time-aware with [Layer.visibilityTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#visibilityTimeExtent) set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-timeUtils.html#getTimeExtentFromLayers)
     */
    getTimeExtentFromLayers(layers: Layer[] | Collection<Layer>, signal?: AbortSignal): Promise<TimeExtent>;
    /**
     * Extracts time slider settings from a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html) if the document contains a time slider widget definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-timeUtils.html#getTimeSliderSettingsFromWebDocument)
     */
    getTimeSliderSettingsFromWebDocument(document: WebMap | WebScene, signal?: AbortSignal): Promise<any>;
    /**
     * Extracts time slider settings from a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) if the webmap contains a time slider widget definition.
     *
     * @deprecated since 4.30. Use {@link module:esri/support/timeUtils#getTimeSliderSettingsFromWebDocument getTimeSliderSettingsFromWebDocument} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-timeUtils.html#getTimeSliderSettingsFromWebMap)
     */
    getTimeSliderSettingsFromWebMap(webMap: WebMap, signal?: AbortSignal): Promise<any>;
  }

  export const timeUtils: timeUtils;

  /**
   * A convenience module for importing [Symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html)
   */
  namespace symbols {
    /**
     * CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#CIMSymbol)
     */
    export type CIMSymbol = __esri.CIMSymbol;
    export const CIMSymbol: typeof __esri.CIMSymbol;

    /**
     * ExtrudeSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#ExtrudeSymbol3DLayer)
     */
    export type ExtrudeSymbol3DLayer = __esri.ExtrudeSymbol3DLayer;
    export const ExtrudeSymbol3DLayer: typeof __esri.ExtrudeSymbol3DLayer;

    /**
     * FillSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#FillSymbol3DLayer)
     */
    export type FillSymbol3DLayer = __esri.FillSymbol3DLayer;
    export const FillSymbol3DLayer: typeof __esri.FillSymbol3DLayer;

    /**
     * WaterSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#WaterSymbol3DLayer)
     */
    export type WaterSymbol3DLayer = __esri.WaterSymbol3DLayer;
    export const WaterSymbol3DLayer: typeof __esri.WaterSymbol3DLayer;

    /**
     * Font.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Font)
     */
    export type Font = __esri.Font;
    export const Font: typeof __esri.Font;

    /**
     * IconSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#IconSymbol3DLayer)
     */
    export type IconSymbol3DLayer = __esri.IconSymbol3DLayer;
    export const IconSymbol3DLayer: typeof __esri.IconSymbol3DLayer;

    /**
     * LabelSymbol3D.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LabelSymbol3D)
     */
    export type LabelSymbol3D = __esri.LabelSymbol3D;
    export const LabelSymbol3D: typeof __esri.LabelSymbol3D;

    /**
     * LineSymbol3D.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LineSymbol3D)
     */
    export type LineSymbol3D = __esri.LineSymbol3D;
    export const LineSymbol3D: typeof __esri.LineSymbol3D;

    /**
     * LineSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LineSymbol3DLayer)
     */
    export type LineSymbol3DLayer = __esri.LineSymbol3DLayer;
    export const LineSymbol3DLayer: typeof __esri.LineSymbol3DLayer;

    /**
     * MeshSymbol3D.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#MeshSymbol3D)
     */
    export type MeshSymbol3D = __esri.MeshSymbol3D;
    export const MeshSymbol3D: typeof __esri.MeshSymbol3D;

    /**
     * ObjectSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#ObjectSymbol3DLayer)
     */
    export type ObjectSymbol3DLayer = __esri.ObjectSymbol3DLayer;
    export const ObjectSymbol3DLayer: typeof __esri.ObjectSymbol3DLayer;

    /**
     * PathSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PathSymbol3DLayer)
     */
    export type PathSymbol3DLayer = __esri.PathSymbol3DLayer;
    export const PathSymbol3DLayer: typeof __esri.PathSymbol3DLayer;

    /**
     * PictureFillSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PictureFillSymbol)
     */
    export type PictureFillSymbol = __esri.PictureFillSymbol;
    export const PictureFillSymbol: typeof __esri.PictureFillSymbol;

    /**
     * PictureMarkerSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PictureMarkerSymbol)
     */
    export type PictureMarkerSymbol = __esri.PictureMarkerSymbol;
    export const PictureMarkerSymbol: typeof __esri.PictureMarkerSymbol;

    /**
     * PointSymbol3D.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PointSymbol3D)
     */
    export type PointSymbol3D = __esri.PointSymbol3D;
    export const PointSymbol3D: typeof __esri.PointSymbol3D;

    /**
     * PolygonSymbol3D.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PolygonSymbol3D)
     */
    export type PolygonSymbol3D = __esri.PolygonSymbol3D;
    export const PolygonSymbol3D: typeof __esri.PolygonSymbol3D;

    /**
     * SimpleFillSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleFillSymbol)
     */
    export type SimpleFillSymbol = __esri.SimpleFillSymbol;
    export const SimpleFillSymbol: typeof __esri.SimpleFillSymbol;

    /**
     * SimpleLineSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleLineSymbol)
     */
    export type SimpleLineSymbol = __esri.SimpleLineSymbol;
    export const SimpleLineSymbol: typeof __esri.SimpleLineSymbol;

    /**
     * SimpleMarkerSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleMarkerSymbol)
     */
    export type SimpleMarkerSymbol = __esri.SimpleMarkerSymbol;
    export const SimpleMarkerSymbol: typeof __esri.SimpleMarkerSymbol;

    /**
     * TextSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#TextSymbol)
     */
    export type TextSymbol = __esri.TextSymbol;
    export const TextSymbol: typeof __esri.TextSymbol;

    /**
     * TextSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#TextSymbol3DLayer)
     */
    export type TextSymbol3DLayer = __esri.TextSymbol3DLayer;
    export const TextSymbol3DLayer: typeof __esri.TextSymbol3DLayer;

    /**
     * WebStyleSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#WebStyleSymbol)
     */
    export type WebStyleSymbol = __esri.WebStyleSymbol;
    export const WebStyleSymbol: typeof __esri.WebStyleSymbol;

    /**
     * FillSymbol types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#FillSymbol)
     */
    export type FillSymbol = __esri.PictureFillSymbol | __esri.SimpleFillSymbol;

    /**
     * MarkerSymbol types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#MarkerSymbol)
     */
    export type MarkerSymbol = __esri.PictureMarkerSymbol | __esri.SimpleMarkerSymbol;

    /**
     * Symbol2D types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol2D)
     */
    export type Symbol2D =
      | __esri.PictureFillSymbol
      | __esri.PictureMarkerSymbol
      | __esri.SimpleFillSymbol
      | __esri.SimpleLineSymbol
      | __esri.SimpleMarkerSymbol
      | __esri.TextSymbol
      | __esri.CIMSymbol;

    /**
     * Symbol3DLayer types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol3DLayer)
     */
    export type Symbol3DLayer =
      | __esri.ExtrudeSymbol3DLayer
      | __esri.FillSymbol3DLayer
      | __esri.WaterSymbol3DLayer
      | __esri.IconSymbol3DLayer
      | __esri.LineSymbol3DLayer
      | __esri.ObjectSymbol3DLayer
      | __esri.PathSymbol3DLayer
      | __esri.TextSymbol3DLayer
      | __esri.WaterSymbol3DLayer;

    /**
     * Symbol3D types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol3D)
     */
    export type Symbol3D =
      | __esri.LabelSymbol3D
      | __esri.LineSymbol3D
      | __esri.MeshSymbol3D
      | __esri.PointSymbol3D
      | __esri.PolygonSymbol3D;

    /**
     * Symbol2D3D types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol2D3D)
     */
    export type Symbol2D3D = __esri.Symbol2D | __esri.symbolsSymbol3D;

    /**
     * Symbol types.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol)
     */
    export type Symbol = __esri.Symbol2D3D | __esri.WebStyleSymbol;
  }

  export interface Callout3D extends Accessor, JSONSupport {}

  export class Callout3D {
    /**
     * When symbols have an offset from their position, it's important to still see what the real location is.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-Callout3D.html)
     */

    constructor(properties?: Callout3DProperties);

    /**
     * Creates a deep clone of the callout.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-Callout3D.html#clone)
     */
    clone(): Callout3D;

    static fromJSON(json: any): Callout3D;
  }

  interface Callout3DProperties {}

  export class LineCallout3D extends Callout3D {
    /**
     * This type of callout displays a line to connect a symbol or a label with its actual location in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html)
     */

    constructor(properties?: LineCallout3DProperties);

    /**
     * The border settings of the callout line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#border)
     */
    border: LineCallout3DBorder;
    /**
     * The color of the callout line.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#color)
     */
    color: Color;
    /**
     * The width of the callout line in points.
     *
     * @default 1px
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#size)
     */
    size: number;

    type: "line";

    /**
     * Creates a deep clone of the callout.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#clone)
     */
    clone(): LineCallout3D;

    static fromJSON(json: any): LineCallout3D;
  }

  interface LineCallout3DProperties extends Callout3DProperties {
    /**
     * The border settings of the callout line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#border)
     */
    border?: LineCallout3DBorderProperties;
    /**
     * The color of the callout line.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#color)
     */
    color?: Color | number[] | string;
    /**
     * The width of the callout line in points.
     *
     * @default 1px
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#size)
     */
    size?: number | string;

    type?: "line";
  }

  export interface LineCallout3DBorderProperties {
    /**
     * The color of the callout line border.
     *
     * @default white
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#border)
     */
    color?: Color | number[] | string;
  }

  export interface LineCallout3DBorder extends AnonymousAccessor {
    /**
     * The color of the callout line border.
     *
     * @default white
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-callouts-LineCallout3D.html#border)
     */
    color?: Color;
  }

  export class CIMSymbol extends Symbol {
    /**
     * CIMSymbols are high quality, scalable, multi-layer vector symbols for features and graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html)
     */

    constructor(properties?: CIMSymbolProperties);

    /**
     * The JSON payload of the [CIMSymbolReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#data)
     */
    data: CIMSymbolReference;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#type)
     */
    readonly type: "cim";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#clone)
     */
    clone(): CIMSymbol;

    static fromJSON(json: any): CIMSymbol;
  }

  interface CIMSymbolProperties extends SymbolProperties {
    /**
     * The JSON payload of the [CIMSymbolReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#data)
     */
    data?: CIMSymbolReference;
  }

  /**
   * Represents animated symbol properties, a collection of symbol properties that apply when the symbol layer has animation data.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
   */
  export interface CIMAnimatedSymbolProperties {
    /**
     * Indicates whether the animated symbol should play its animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    playAnimation?: boolean;
    /**
     * Indicates whether the animated symbol should play back in reverse.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    reverseAnimation?: boolean;
    /**
     * Indicates whether to generate a randomized start offset in seconds to apply to the symbol animation of each feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    randomizeStartTime?: boolean;
    /**
     * The starting value for generating a random number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    randomizeStartSeed?: number;
    /**
     * The time offset in seconds to use as the starting point of the symbol animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    startTimeOffset?: number;
    /**
     * The time (in seconds) it take to play through the symbol's animation once.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    duration?: number;
    /**
     * Determines how to repeat the animation of a symbol when the animation cycle ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    repeatType?: "None" | "Loop" | "Oscillate";
    /**
     * The number of seconds to delay before repeating an animation cycle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    repeatDelay?: number;
    /**
     * _Since 4.31_ The easing method to use when animating the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMAnimatedSymbolProperties)
     */
    easing?: "Linear" | "EaseIn" | "EaseOut" | "EaseInOut";
  }

  /**
   * Represents a background callout for a [CIMTextSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMBackgroundCallout)
   */
  export interface CIMBackgroundCallout {
    /**
     * The callout type.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMBackgroundCallout";
    /**
     * The symbol used to draw the background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMBackgroundCallout)
     */
    backgroundSymbol: CIMPolygonSymbol;
  }

  /**
   * Represents color substitution, an ordered list of color substitutes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMColorSubstitution)
   */
  export interface CIMColorSubstitution {
    /**
     * The old color (the color that will be substituted).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMColorSubstitution)
     */
    oldColor: number[];
    /**
     * The new color that will replace the old color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMColorSubstitution)
     */
    newColor: number[];
  }

  /**
   * Represents a geometric effect.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffect)
   */
  export type CIMGeometricEffect =
    | CIMGeometricEffectArrow
    | CIMGeometricEffectBuffer
    | CIMGeometricEffectControlMeasureLine
    | CIMGeometricEffectCut
    | CIMGeometricEffectDashes
    | CIMGeometricEffectDonut
    | CIMGeometricEffectJog
    | CIMGeometricEffectMove
    | CIMGeometricEffectOffset
    | CIMGeometricEffectRotate
    | CIMGeometricEffectScale
    | CIMGeometricEffectWave;

  /**
   * Represents the arrow geometric effect which creates a dynamic line along a line feature with an arrow of a specified arrow type and width.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectArrow)
   */
  export interface CIMGeometricEffectArrow {
    type: "CIMGeometricEffectArrow";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectArrow)
     */
    primitiveName?: string;
    /**
     * The type of arrow to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectArrow)
     */
    geometricEffectArrowType: "OpenEnded" | "Block" | "Crossed";
    /**
     * The distance between the lines that construct the body of the arrow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectArrow)
     */
    width: number;
  }

  /**
   * Represents the buffer geometric effect which creates a dynamic polygon with a specified distance around features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectBuffer)
   */
  export interface CIMGeometricEffectBuffer {
    type: "CIMGeometricEffectBuffer";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectBuffer)
     */
    primitiveName?: string;
    /**
     * The distance from the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectBuffer)
     */
    size: number;
  }

  /**
   * Represents the control measure line geometric effect.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectControlMeasureLine)
   */
  export interface CIMGeometricEffectControlMeasureLine {
    type: "CIMGeometricEffectControlMeasureLine";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectControlMeasureLine)
     */
    primitiveName?: string;
    /**
     * The style of effect.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectControlMeasureLine)
     */
    rule: string;
  }

  /**
   * Represents the cut geometric effect which creates a dynamic line that is shorter on one or both ends than the line feature or polygon outline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
   */
  export interface CIMGeometricEffectCut {
    type: "CIMGeometricEffectCut";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
     */
    primitiveName?: string;
    /**
     * The distance from the beginning of a line that the display of the stroke starts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
     */
    beginCut?: number;
    /**
     * The distance from the end of a line that the display of the stroke ends.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
     */
    endCut?: number;
    /**
     * The distance around the middle of a line that the display of the stroke is interrupted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
     */
    middleCut?: number;
    /**
     * A value indicating whether the effect should be applied in the opposite manner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectCut)
     */
    invert?: boolean;
  }

  /**
   * Represents the dashes geometric effect which creates a dynamic multipart line geometry from a line feature or the outline of a polygon based on a template.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
   */
  export interface CIMGeometricEffectDashes {
    type: "CIMGeometricEffectDashes";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
     */
    primitiveName?: string;
    /**
     * Where the pattern should end relative to the ending point of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
     */
    customEndingOffset?: number;
    /**
     * The distance for each dash and gap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
     */
    dashTemplate: number[];
    /**
     * The setting which determines how the strokes with dash patterns are handled at the end points of the line geometry's segments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
     */
    lineDashEnding: "NoConstraint" | "HalfPattern" | "HalfGap" | "FullPattern" | "FullGap" | "Custom";
    /**
     * The position where the pattern should begin relative to the starting point of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDashes)
     */
    offsetAlongLine: number;
  }

  /**
   * Represents the donut geometric effect which creates a dynamic polygon ring of a specified width in relation to the outline of polygon features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDonut)
   */
  export interface CIMGeometricEffectDonut {
    type: "CIMGeometricEffectDonut";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDonut)
     */
    primitiveName?: string;
    /**
     * The option for the way the symbol handles complex geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDonut)
     */
    option: "Fast" | "Accurate";
    /**
     * The distance from the edge of the polygon that the fill symbol is to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectDonut)
     */
    width: number;
  }

  /**
   * Represents the jog geometric effect which creates a dynamic line with a jog of a specified angle, position, and width in the line.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectJog)
   */
  export interface CIMGeometricEffectJog {
    type: "CIMGeometricEffectJog";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectJog)
     */
    primitiveName?: string;
    /**
     * The angle of the job in the line which is measured in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectJog)
     */
    angle: number;
    /**
     * The length of the segment that forms the jog in the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectJog)
     */
    length: number;
    /**
     * The location of the center of the jog, as a percentage measured from the start of the input geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectJog)
     */
    position: number;
  }

  /**
   * Represents the move geometric effect which creates a point, line or polygon that is offset a specified distance in X and Y.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectMove)
   */
  export interface CIMGeometricEffectMove {
    type: "CIMGeometricEffectMove";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectMove)
     */
    primitiveName?: string;
    /**
     * The distance to move the symbol along the X-axis of the feature geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectMove)
     */
    offsetX?: number;
    /**
     * The distance to move the symbol along the Y-axis of the feature geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectMove)
     */
    offsetY?: number;
  }

  /**
   * Represents the offset geometric effect which creates a dynamic line or polygon offset at a specified distance perpendicularly from a feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectOffset)
   */
  export interface CIMGeometricEffectOffset {
    type: "CIMGeometricEffectOffset";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectOffset)
     */
    primitiveName?: string;
    /**
     * The way strokes or fills are displayed at corners.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectOffset)
     */
    method: "Mitered" | "Bevelled" | "Rounded" | "Square";
    /**
     * The distance of the symbol perpendicular to the feature geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectOffset)
     */
    offset: number;
    /**
     * The way the symbol handles complex geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectOffset)
     */
    option: "Fast" | "Accurate";
  }

  /**
   * Represents the rotate geometric effect which creates a dynamic line or polygon rotated a specified angle from the feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectRotate)
   */
  export interface CIMGeometricEffectRotate {
    type: "CIMGeometricEffectRotate";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectRotate)
     */
    primitiveName?: string;
    /**
     * The amount of rotation for the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectRotate)
     */
    angle: number;
  }

  /**
   * Represents the scale geometric effect which creates a dynamic line or polygon scaled by a specified factor.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectScale)
   */
  export interface CIMGeometricEffectScale {
    type: "CIMGeometricEffectScale";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectScale)
     */
    primitiveName?: string;
    /**
     * The amount of change in size of a symbol in the x-axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectScale)
     */
    xScaleFactor: number;
    /**
     * The amount of change in size of a symbol in the y-axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectScale)
     */
    yScaleFactor: number;
  }

  /**
   * Represents the wave geometric effect which creates a dynamic line or polygon along a feature with a repeating wave pattern.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
   */
  export interface CIMGeometricEffectWave {
    type: "CIMGeometricEffectWave";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
     */
    primitiveName?: string;
    /**
     * The distance perpendicular to a feature to display the curves for the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
     */
    amplitude: number;
    /**
     * The distance along the line or polygon to display the curves for the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
     */
    period: number;
    /**
     * The starting value for generating a random number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
     */
    seed?: number;
    /**
     * The shape of the curves to be displayed along the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMGeometricEffectWave)
     */
    waveform: "Sinus" | "Square" | "Triangle" | "Random";
  }

  /**
   * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
   */
  export interface CIMHatchFill {
    type: "CIMHatchFill";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    primitiveName?: string;
    /**
     * The line symbol that is used to draw the hatch lines in the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    lineSymbol: CIMLineSymbol;
    /**
     * How much to move the stroke to a new X-position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    offsetX?: number;
    /**
     * How much to move the stroke to a new Y-position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    offsetY?: number;
    /**
     * The angle of rotation for all the strokes, in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    rotation?: number;
    /**
     * The distance between the line symbols in the hatch pattern.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMHatchFill)
     */
    separation: number;
  }

  /**
   * Represents a line symbol used to draw polyline features and graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMLineSymbol)
   */
  export interface CIMLineSymbol {
    type: "CIMLineSymbol";
    /**
     * The symbol layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMLineSymbol)
     */
    symbolLayers: CIMSymbolLayer[];
    /**
     * The geometric effects applied to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMLineSymbol)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol size properties are rendered using real world units or page units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMLineSymbol)
     */
    useRealWorldSymbolSizes?: boolean;
  }

  /**
   * Represents a marker graphic which is used to define vector graphics in a vector marker.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerGraphic)
   */
  export interface CIMMarkerGraphic {
    type: "CIMMarkerGraphic";
    /**
     * The geometry of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerGraphic)
     */
    geometry: any;
    /**
     * The symbol used to draw the marker graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerGraphic)
     */
    symbol: CIMPointSymbol | CIMLineSymbol | CIMPolygonSymbol | CIMTextSymbol;
    /**
     * The text that is defined within the marker if drawn with a text symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerGraphic)
     */
    textString?: string;
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerGraphic)
     */
    primitiveName?: string;
  }

  /**
   * Represents marker placement along the line which places markers that are the same size evenly along a line or polygon outline.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
   */
  export interface CIMMarkerPlacementAlongLineSameSize {
    type: "CIMMarkerPlacementAlongLineSameSize";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    placePerPart?: boolean;
    /**
     * A value indicating whether to angle the marker to the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    angleToLine?: boolean;
    /**
     * The offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    offset?: number;
    /**
     * Where the pattern should end relative to the ending point of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    customEndingOffset?: number;
    /**
     * How markers are placed at the end points of a line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    endings?: "NoConstraint" | "WithMarkers" | "WithFullGap" | "WithHalfGap" | "Custom";
    /**
     * Where the pattern should begin relative to the starting point of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    offsetAlongLine?: number;
    /**
     * The numeric pattern that defines the sequence of placed markers and the length of space between them.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAlongLineSameSize)
     */
    placementTemplate: number[];
  }

  /**
   * Represents marker placement at extremities which places markers at only one or both endpoints of a line.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
   */
  export interface CIMMarkerPlacementAtExtremities {
    type: "CIMMarkerPlacementAtExtremities";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    placePerPart?: boolean;
    /**
     * A value indicating whether to angle the marker to the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    angleToLine?: boolean;
    /**
     * The offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    offset?: number;
    /**
     * Which ends of the line a marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    extremityPlacement?: "Both" | "JustBegin" | "JustEnd" | "None";
    /**
     * The distance from the ends of a line that the marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtExtremities)
     */
    offsetAlongLine?: number;
  }

  /**
   * Represents marker placement at ratio positions which places a set number of markers along the line or the outline of a polygon.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
   */
  export interface CIMMarkerPlacementAtRatioPositions {
    type: "CIMMarkerPlacementAtRatioPositions";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    placePerPart?: boolean;
    /**
     * A value indicating whether to angle the marker to the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    angleToLine?: boolean;
    /**
     * The offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    offset?: number;
    /**
     * The distance from the beginning of a line that the marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    beginPosition?: number;
    /**
     * The distance from the end of a line that the marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    endPosition?: number;
    /**
     * A value indicating whether only the first marker will be rotated 180 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    flipFirst?: boolean;
    /**
     * The array of positions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementAtRatioPositions)
     */
    positionArray: number[];
  }

  /**
   * Represents marker placement inside a polygon which defines how a polygon is filled with a pattern of markers.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
   */
  export interface CIMMarkerPlacementInsidePolygon {
    type: "CIMMarkerPlacementInsidePolygon";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    placePerPart?: boolean;
    /**
     * The orientation angle that the markers are placed on within the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    gridAngle?: number;
    /**
     * The grid type which defines how markers are placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    gridType?: "Fixed" | "Random";
    /**
     * The marker row offset horizontally.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    offsetX?: number;
    /**
     * The marker row offset vertically.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    offsetY?: number;
    /**
     * A value indicating whether every other row of markers should be shifted to create an offset grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    shiftOddRows?: boolean;
    /**
     * The distance between each marker on the X-axis of the grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    stepX?: number;
    /**
     * The distance between each marker on the Y-axis of the grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    stepY?: number;
    /**
     * _Since 4.24_ A percentage that determines the randomness of markers within the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    randomness?: number;
    /**
     * _Since 4.24_ The starting value for generating a random pattern.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    seed?: number;
    /**
     * The clipping option which specifies how markers should be clipped at the polygon boundary.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementInsidePolygon)
     */
    clipping?: "ClipAtBoundary";
  }

  /**
   * Represents a marker placement on the line.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
   */
  export interface CIMMarkerPlacementOnLine {
    type: "CIMMarkerPlacementOnLine";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    placePerPart?: boolean;
    /**
     * A value indicating whether to angle the marker to the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    angleToLine?: boolean;
    /**
     * The offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    offset?: number;
    /**
     * The location on a line where a marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    relativeTo: "LineMiddle" | "LineBeginning" | "LineEnd" | "SegmentMidpoint";
    /**
     * The distances from a specified location on a line that a marker will be placed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnLine)
     */
    startPointOffset?: number;
  }

  /**
   * Represents a marker placement on vertices which places a single marker on a line or polygon outline at a set distance from the middle or one of the endpoints.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
   */
  export interface CIMMarkerPlacementOnVertices {
    type: "CIMMarkerPlacementOnVertices";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    placePerPart?: boolean;
    /**
     * A value indicating whether to angle the marker to the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    angleToLine?: boolean;
    /**
     * The offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    offset?: number;
    /**
     * A value indicating whether a marker will be placed on the control points of the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    placeOnControlPoints?: boolean;
    /**
     * A value indicating whether a marker will be placed on the endpoints of the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    placeOnEndPoints?: boolean;
    /**
     * A value indicating whether a marker will be placed on the vertices of the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementOnVertices)
     */
    placeOnRegularVertices?: boolean;
  }

  /**
   * Represents marker placement polygon center which defines how a single marker will be placed within the polygon.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
   */
  export interface CIMMarkerPlacementPolygonCenter {
    type: "CIMMarkerPlacementPolygonCenter";
    /**
     * The primitive name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    primitiveName?: string;
    /**
     * A value indicating whether to consider individual geometry parts or the whole geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    placePerPart?: boolean;
    /**
     * The method used to determine the polygon center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    method?: "OnPolygon" | "CenterOfMass" | "BoundingBoxCenter";
    /**
     * The value which offsets the marker horizontally from the center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    offsetX?: number;
    /**
     * The value which offsets the marker vertically from the center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    offsetY?: number;
    /**
     * A value indicating whether the marker should be clipped if it extends pasts the boundary of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMMarkerPlacementPolygonCenter)
     */
    clipAtBoundary?: boolean;
  }

  /**
   * Represents a picture fill which fills polygonal geometry with a picture.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
   */
  export interface CIMPictureFill {
    type: "CIMPictureFill";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    primitiveName?: string;
    /**
     * The url of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    url: string;
    /**
     * The distance that the image is offset in the horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    offsetX?: number;
    /**
     * The distance that the image is offset in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    offsetY?: number;
    /**
     * The angle of the image within the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    rotation?: number;
    /**
     * The width of the symbol without changing the height, as a ratio.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    scaleX?: number;
    /**
     * The height of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    height?: number;
    /**
     * The color substitutions which allows colors in the image to be substituted with a different color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    colorSubstitutions?: CIMColorSubstitution[];
    /**
     * The color that is applied as a tint to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureFill)
     */
    tintColor?: number[];
  }

  /**
   * Represents a picture marker created from a raster (bitmapped) image file.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
   */
  export interface CIMPictureMarker {
    type: "CIMPictureMarker";
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    enable: boolean;
    /**
     * The url of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    url: string;
    /**
     * The height of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    size: number;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    colorLocked?: boolean;
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    effects?: CIMGeometricEffect[];
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    primitiveName?: string;
    /**
     * The specified location where all transformation property operations originate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    anchorPoint?: CIMPictureMarkerAnchorPoint;
    /**
     * A value which specifies if the anchor point location is considered a percentage of the size or as an absolute distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    anchorPointUnits?: "Relative" | "Absolute";
    /**
     * The distance that the image is offset in the horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    offsetX?: number;
    /**
     * The distance that the image is offset in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    offsetY?: number;
    /**
     * A value indicating whether the rotation is applied clockwise or counterclockwise to the marker layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    rotateClockwise?: boolean;
    /**
     * The angle of the image within the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    rotation?: number;
    /**
     * Determines how markers are placed along a line or within a polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    markerPlacement?: MarkerPlacement;
    /**
     * The width of the symbol without changing the height, as a ratio.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    scaleX?: number;
    /**
     * The height of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    height?: number;
    /**
     * The color substitutions which allows colors in the image to be substituted with a different color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    colorSubstitutions?: CIMColorSubstitution[];
    /**
     * The color that is applied as a tint to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    tintColor?: number[];
    /**
     * _Since 4.24_ The collection of symbol properties that apply when the symbol layer has animation data (animated GIF or PNG).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
    /**
     * _Since 4.31_ The animations applied to the picure marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    animations?: CIMSymbolAnimation[];
  }

  /**
   * Represents a picture stroke which draws linear geometry with a repeating image file.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
   */
  export interface CIMPictureStroke {
    type: "CIMPictureStroke";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    primitiveName?: string;
    /**
     * The url of the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    url: string;
    /**
     * The color substitutions which allows colors in the image to be substituted with a different color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    colorSubstitutions?: CIMColorSubstitution[];
    /**
     * The color that is applied as a tint to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    tintColor?: number[];
    /**
     * The width of the stroke.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    width: number;
    /**
     * Determines how the stroke should draw at the ends of the geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    capStyle?: "Butt" | "Round" | "Square";
    /**
     * Determines how the symbol is drawn at the stroke segment connections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    joinStyle?: "Bevel" | "Round" | "Miter";
    /**
     * The maximum 'sharpness' that is allowed for miter joins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureStroke)
     */
    miterLimit?: number;
  }

  /**
   * Represents a point symbol used to draw point features and graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
   */
  export interface CIMPointSymbol {
    type: "CIMPointSymbol";
    /**
     * The symbol layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    symbolLayers?: CIMSymbolLayer[];
    /**
     * The amount of variation applied to the symbol, measured in degrees, propagated cumulatively to all marker symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    angle?: number;
    /**
     * Indicates whether point symbols align to the map or to the display when a rotation is applied to the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    angleAlignment?: "Display" | "Map";
    /**
     * The geometric effects applied to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    effects?: CIMGeometricEffect[];
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    primitiveName?: string;
    /**
     * The X scale which changes the width of the symbol without changing the height, as a ratio.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    scaleX?: number;
    /**
     * A value indicating whether the symbol size properties are rendered using real world units or page units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    useRealWorldSymbolSizes?: boolean;
    /**
     * _Since 4.31_ The animations applied globally to the entire symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPointSymbol)
     */
    animations?: CIMSymbolAnimation[];
  }

  /**
   * Represents a polygon symbol used to draw polygon features and graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPolygonSymbol)
   */
  export interface CIMPolygonSymbol {
    type: "CIMPolygonSymbol";
    /**
     * The symbol layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPolygonSymbol)
     */
    symbolLayers: CIMSymbolLayer[];
    /**
     * The geometric effects applied to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPolygonSymbol)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol size properties are rendered using real world units or page units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPolygonSymbol)
     */
    useRealWorldSymbolSizes?: boolean;
  }

  /**
   * Represents a solid fill which fills polygonal geometry with a single solid color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
   */
  export interface CIMSolidFill {
    type: "CIMSolidFill";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    primitiveName?: string;
    /**
     * The color that is applied to the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    color: number[];
    /**
     * _Since 4.31_ The animations applied to the solid fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidFill)
     */
    animations?: CIMSymbolAnimation[];
  }

  /**
   * Represents a solid stroke which draws linear geometry with a single solid color.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
   */
  export interface CIMSolidStroke {
    type: "CIMSolidStroke";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    primitiveName?: string;
    /**
     * The color that is applied to the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    color: number[];
    /**
     * The width of the stroke.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    width: number;
    /**
     * Determines how the stroke should draw at the ends of the geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    capStyle?: "Butt" | "Round" | "Square";
    /**
     * Determines how the symbol is drawn at the stroke segment connections.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    joinStyle?: "Bevel" | "Round" | "Miter";
    /**
     * The maximum 'sharpness' that is allowed for miter joins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    miterLimit?: number;
    /**
     * _Since 4.31_ The animations applied to the solid stroke.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSolidStroke)
     */
    animations?: CIMSymbolAnimation[];
  }

  /**
   * Represents a symbol animation used to animate the appearance of a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimation)
   */
  export type CIMSymbolAnimation =
    | CIMSymbolAnimationSize
    | CIMSymbolAnimationScale
    | CIMSymbolAnimationColor
    | CIMSymbolAnimationTransparency
    | CIMSymbolAnimationRotation
    | CIMSymbolAnimationOffset;

  /**
   * Represents a symbol animation that changes the color of a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationColor)
   */
  export interface CIMSymbolAnimationColor {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationColor";
    /**
     * The color of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationColor)
     */
    toColor: Color;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationColor)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol animation that offsets a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationOffset)
   */
  export interface CIMSymbolAnimationOffset {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationOffset";
    /**
     * The x offset of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationOffset)
     */
    offsetX?: number;
    /**
     * The y offset of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationOffset)
     */
    offsetY?: number;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationOffset)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol animation that rotates a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationRotation)
   */
  export interface CIMSymbolAnimationRotation {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationRotation";
    /**
     * The rotation of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationRotation)
     */
    toRotation: number;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationRotation)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol animation that scales a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationScale)
   */
  export interface CIMSymbolAnimationScale {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationScale";
    /**
     * The scale of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationScale)
     */
    scaleFactor: number;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationScale)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol animation that changes the size of a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationSize)
   */
  export interface CIMSymbolAnimationSize {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationSize";
    /**
     * The size of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationSize)
     */
    toSize: number;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationSize)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol animation that changes the transparency of a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationTransparency)
   */
  export interface CIMSymbolAnimationTransparency {
    /**
     * The type of animation.
     *
     * [Read more...](global.html#type)
     */
    type: "CIMSymbolAnimationTransparency";
    /**
     * The transparency of the symbol to animate to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationTransparency)
     */
    toTransparency?: number;
    /**
     * The properties that apply when the symbol layer has animation data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolAnimationTransparency)
     */
    animatedSymbolProperties?: CIMAnimatedSymbolProperties;
  }

  /**
   * Represents a symbol layer, the component that makes up a symbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolLayer)
   */
  export type CIMSymbolLayer =
    | CIMHatchFill
    | CIMPictureFill
    | CIMPictureMarker
    | CIMPictureStroke
    | CIMSolidFill
    | CIMSolidStroke
    | CIMVectorMarker;

  /**
   * Represents a symbol reference from the [data](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#data) property.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference)
   */
  export interface CIMSymbolReference {
    type: "CIMSymbolReference";
    /**
     * The primitive overrides.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference)
     */
    primitiveOverrides?: PrimitiveOverride[];
    /**
     * The symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference)
     */
    symbol?: CIMPointSymbol | CIMLineSymbol | CIMPolygonSymbol;
    /**
     * _Since 4.20_ The minimum scale the symbol reference should be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference)
     */
    minScale?: number;
    /**
     * _Since 4.20_ The maximum scale the symbol reference should be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMSymbolReference)
     */
    maxScale?: number;
  }

  /**
   * Represents a text symbol which is used to draw text graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
   */
  export interface CIMTextSymbol {
    type: "CIMTextSymbol";
    /**
     * The amount of rotation applied to the text symbol, measured in degrees, around the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    angle?: number;
    /**
     * The font family name of the font (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    fontFamilyName: string;
    /**
     * The style name for the font family.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    fontStyleName?: "Regular" | "Bold" | "Italic";
    /**
     * The size of the halo that extends beyond the symbol shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    haloSize?: number;
    /**
     * The polygon symbol that is used to draw the halo for a text symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    haloSymbol?: CIMPolygonSymbol;
    /**
     * The size of the text in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    height: number;
    /**
     * The alignment type used to align the text to the geometry horizontally.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    horizontalAlignment?: "Left" | "Right" | "Center";
    /**
     * The X offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    offsetX?: number;
    /**
     * The Y offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    offsetY?: number;
    /**
     * A value indicating whether to draw the text with a strike through it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    strikethrough?: boolean;
    /**
     * The polygon symbol that is used to draw the glyphs of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    symbol: CIMPolygonSymbol;
    /**
     * A value indicating whether to draw the text with an underline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    underline?: boolean;
    /**
     * The vertical alignment of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    verticalAlignment?: "Top" | "Center" | "Baseline" | "Bottom";
    /**
     * _Since 4.26_ The background of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMTextSymbol)
     */
    callout?: CIMBackgroundCallout;
  }

  /**
   * Represents a vector marker which can represent vector graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
   */
  export interface CIMVectorMarker {
    type: "CIMVectorMarker";
    /**
     * The geometric effects applied to the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    effects?: CIMGeometricEffect[];
    /**
     * A value indicating whether the symbol layer is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    enable: boolean;
    /**
     * A value indicating whether the color can be changed for this symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    colorLocked?: boolean;
    /**
     * The primitive name of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    primitiveName?: string;
    /**
     * The height of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    size: number;
    /**
     * The specified location where all transformation property operations originate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    anchorPoint?: CIMVectorMarkerAnchorPoint;
    /**
     * A value which specifies if the anchor point location is considered a percentage of the size or as an absolute distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    anchorPointUnits?: "Relative" | "Absolute";
    /**
     * The distance that the image is offset in the horizontal direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    offsetX?: number;
    /**
     * The distance that the image is offset in the vertical direction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    offsetY?: number;
    /**
     * A value indicating whether the rotation is applied clockwise or counterclockwise to the marker layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    rotateClockwise?: boolean;
    /**
     * The angle of the image within the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    rotation?: number;
    /**
     * Determines how markers are placed along a line or within a polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    markerPlacement?: MarkerPlacement;
    /**
     * The outer boundary of the entire vector marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    frame: Envelope;
    /**
     * The vector graphics that define the shape of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    markerGraphics: CIMMarkerGraphic[];
    /**
     * A value indicating whether the strokes and or fills of a marker are scaled proportionally when the symbol size is changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    scaleSymbolsProportionally?: boolean;
    /**
     * A value indicating whether the frame of the vector marker should be honored when drawing the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    respectFrame?: boolean;
    /**
     * _Since 4.31_ The animations applied to the vector marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    animations?: CIMSymbolAnimation[];
  }

  /**
   * An envelope is a rectangle defined by a range of values for each coordinate and attribute.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#Envelope)
   */
  export interface Envelope {
    /**
     * The minimum X value in the area of the envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#Envelope)
     */
    xmin: number;
    /**
     * The maximum X value in the area of the envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#Envelope)
     */
    xmax: number;
    /**
     * The minimum Y value in the area of the envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#Envelope)
     */
    ymin: number;
    /**
     * The maximum Y value in the area of the envelope.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#Envelope)
     */
    ymax: number;
  }

  /**
   * Represents a marker placement.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#MarkerPlacement)
   */
  export type MarkerPlacement =
    | CIMMarkerPlacementAlongLineSameSize
    | CIMMarkerPlacementAtExtremities
    | CIMMarkerPlacementAtRatioPositions
    | CIMMarkerPlacementInsidePolygon
    | CIMMarkerPlacementOnLine
    | CIMMarkerPlacementOnVertices
    | CIMMarkerPlacementPolygonCenter;

  /**
   * Represents a primitive override.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
   */
  export interface PrimitiveOverride {
    type?: "CIMPrimitiveOverride";
    /**
     * The primitive name this override applies to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    primitiveName: string;
    /**
     * The name of the property in the primitive this override applies to (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    propertyName: string;
    /**
     * Contains the Arcade expression that returns a value to be applied to the property on the primitive.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    valueExpressionInfo?: PrimitiveOverrideValueExpressionInfo;
  }

  export interface CIMPictureMarkerAnchorPoint {
    /**
     * The x coordinate of the anchor point, measured along the east/west axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    x?: any;
    /**
     * The y coordinate of the anchor point, measured along the north/south axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMPictureMarker)
     */
    y?: any;
  }

  export interface CIMVectorMarkerAnchorPoint {
    /**
     * The x coordinate of the anchor point, measured along the east/west axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    x?: any;
    /**
     * The y coordinate of the anchor point, measured along the north/south axis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#CIMVectorMarker)
     */
    y?: any;
  }

  export interface PrimitiveOverrideValueExpressionInfo {
    type: "CIMExpressionInfo";
    /**
     * The human readable text that describes the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    title?: string;
    /**
     * An [Arcade](https://developers.arcgis.com/javascript/latest/arcade/) expression following the specification defined by the [Arcade Visualization Profile](https://developers.arcgis.com/javascript/latest/arcade/#visualization).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    expression: string;
    /**
     * The name of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    name?: string;
    /**
     * The return type of the expression.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html#PrimitiveOverride)
     */
    returnType?: "Default" | "String" | "Numeric";
  }

  export interface Edges3D extends Accessor, JSONSupport {}

  export class Edges3D {
    /**
     * Edges3D is the base class for symbol classes that add edge rendering visualization to existing symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html)
     */

    constructor(properties?: Edges3DProperties);

    /**
     * The color of the edges.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#color)
     */
    color: Color;
    /**
     * A size in points by which to extend edges beyond their original end points.
     *
     * @default "0"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#extensionLength)
     */
    extensionLength: number;
    /**
     * The size of the edges in points.
     *
     * @default 1px
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#size)
     */
    size: number;

    /**
     * Creates a deep clone of the edges.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#clone)
     */
    clone(): Edges3D;

    static fromJSON(json: any): Edges3D;
  }

  interface Edges3DProperties {
    /**
     * The color of the edges.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#color)
     */
    color?: Color | number[] | string;
    /**
     * A size in points by which to extend edges beyond their original end points.
     *
     * @default "0"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#extensionLength)
     */
    extensionLength?: number | string;
    /**
     * The size of the edges in points.
     *
     * @default 1px
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html#size)
     */
    size?: number | string;
  }

  export class SketchEdges3D extends Edges3D {
    /**
     * SketchEdges3D is a symbol type that visualizes edges of [3D Object SceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), extruded polygons or mesh geometries with a sketched line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SketchEdges3D.html)
     */

    constructor(properties?: SketchEdges3DProperties);

    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SketchEdges3D.html#type)
     */
    type: "sketch";

    /**
     * Creates a deep clone of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SketchEdges3D.html#clone)
     */
    clone(): SketchEdges3D;

    static fromJSON(json: any): SketchEdges3D;
  }

  interface SketchEdges3DProperties extends Edges3DProperties {
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SketchEdges3D.html#type)
     */
    type?: "sketch";
  }

  export class SolidEdges3D extends Edges3D {
    /**
     * SolidEdges3D is a symbol type that visualizes edges of [3D Object SceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), extruded polygons or mesh geometries with a solid line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SolidEdges3D.html)
     */

    constructor(properties?: SolidEdges3DProperties);

    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SolidEdges3D.html#type)
     */
    type: "solid";

    /**
     * Creates a deep clone of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SolidEdges3D.html#clone)
     */
    clone(): SolidEdges3D;

    static fromJSON(json: any): SolidEdges3D;
  }

  interface SolidEdges3DProperties extends Edges3DProperties {
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-SolidEdges3D.html#type)
     */
    type?: "solid";
  }

  export class ExtrudeSymbol3DLayer extends Symbol3DLayer {
    /**
     * ExtrudeSymbol3DLayer is used to render [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometries by extruding them upward from the ground, creating a 3D volumetric object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html)
     */

    constructor(properties?: ExtrudeSymbol3DLayerProperties);

    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#castShadows)
     */
    castShadows: boolean;
    /**
     * Sets the contour edges on polygons symbolized with ExtrudeSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#edges)
     */
    edges: Edges3D;
    /**
     * The material used to shade the extrusion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#material)
     */
    material: ExtrudeSymbol3DLayerMaterial;
    /**
     * The height of the extrusion in meters.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#size)
     */
    size: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#type)
     */
    readonly type: "extrude";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#clone)
     */
    clone(): ExtrudeSymbol3DLayer;

    static fromJSON(json: any): ExtrudeSymbol3DLayer;
  }

  interface ExtrudeSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#castShadows)
     */
    castShadows?: boolean;
    /**
     * Sets the contour edges on polygons symbolized with ExtrudeSymbol3DLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#edges)
     */
    edges?: Edges3DProperties;
    /**
     * The material used to shade the extrusion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#material)
     */
    material?: ExtrudeSymbol3DLayerMaterialProperties;
    /**
     * The height of the extrusion in meters.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#size)
     */
    size?: number;
  }

  export interface ExtrudeSymbol3DLayerMaterialProperties {
    /**
     * The fill color of the extrusion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface ExtrudeSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The fill color of the extrusion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export class FillSymbol extends Symbol {
    /**
     * Fill symbols are used to draw [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) graphics in a [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) or a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html)
     */

    constructor(properties?: FillSymbolProperties);

    /**
     * The outline of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html#outline)
     */
    outline: SimpleLineSymbol;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html#type)
     */
    type: "simple-fill" | "picture-fill";

    static fromJSON(json: any): FillSymbol;
  }

  interface FillSymbolProperties extends SymbolProperties {
    /**
     * The outline of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html#outline)
     */
    outline?: SimpleLineSymbolProperties;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol.html#type)
     */
    type?: "simple-fill" | "picture-fill";
  }

  export class FillSymbol3DLayer extends Symbol3DLayer {
    /**
     * FillSymbol3DLayer is used to render the surfaces of flat 2D [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometries and 3D volumetric meshes in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html)
     */

    constructor(properties?: FillSymbol3DLayerProperties);

    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#castShadows)
     */
    castShadows: boolean;
    /**
     * Sets the contour edges on 3D Objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#edges)
     */
    edges: Edges3D;
    /**
     * The material defines the final color of the graphic, by blending the `color` property set in the material with the feature's geometry color/texture information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    material: FillSymbol3DLayerMaterial;
    /**
     * The outline used to draw a line around the filled geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    outline: FillSymbol3DLayerOutline;
    /**
     * The pattern used to render the polygon fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#pattern)
     */
    pattern: StylePattern3D;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#type)
     */
    readonly type: "fill";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#clone)
     */
    clone(): FillSymbol3DLayer;

    static fromJSON(json: any): FillSymbol3DLayer;
  }

  interface FillSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#castShadows)
     */
    castShadows?: boolean;
    /**
     * Sets the contour edges on 3D Objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#edges)
     */
    edges?: Edges3DProperties;
    /**
     * The material defines the final color of the graphic, by blending the `color` property set in the material with the feature's geometry color/texture information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    material?: FillSymbol3DLayerMaterialProperties;
    /**
     * The outline used to draw a line around the filled geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    outline?: FillSymbol3DLayerOutlineProperties;
    /**
     * The pattern used to render the polygon fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#pattern)
     */
    pattern?: StylePattern3DProperties;
  }

  export interface FillSymbol3DLayerMaterialProperties {
    /**
     * The color of the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
    /**
     * Specifies how the material `color` is applied to the geometry color/texture information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    colorMixMode?: "tint" | "replace" | "multiply";
  }

  export interface FillSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The color of the fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    color?: Color;
    /**
     * Specifies how the material `color` is applied to the geometry color/texture information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#material)
     */
    colorMixMode?: "tint" | "replace" | "multiply";
  }

  export interface FillSymbol3DLayerOutlineProperties {
    /**
     * The color of the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    color?: Color | number[] | string;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    size?: number | string;
    /**
     * The stroke pattern used to render the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    pattern?: LineStylePattern3DProperties;
    /**
     * The style applied to the tips of each pattern segment along the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    patternCap?: "butt" | "round" | "square";
  }

  export interface FillSymbol3DLayerOutline extends AnonymousAccessor {
    /**
     * The color of the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    color?: Color;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    size?: number;
    /**
     * The stroke pattern used to render the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    pattern?: LineStylePattern3D;
    /**
     * The style applied to the tips of each pattern segment along the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-FillSymbol3DLayer.html#outline)
     */
    patternCap?: "butt" | "round" | "square";
  }

  export interface Font extends Accessor, JSONSupport {}

  export class Font {
    /**
     * The font used to display [2D text symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html) and [3D text symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html)
     */

    constructor(properties?: FontProperties);

    /**
     * The text decoration.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#decoration)
     */
    decoration: "underline" | "line-through" | "none";
    /**
     * The font family of the text.
     *
     * @default "sans-serif"
     *
     * The font family of the text. The possible values are dependent upon the layer type, and if you are working with a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html). See the [Labeling guide page](https://developers.arcgis.com/javascript/latest/labeling/) for detailed explanation, or click the `Read more` below.  >>> esri-read-more **Font families for 3D SceneViews**  The supported font families for 3D [SceneViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) are dependent upon the fonts installed on the user's computer and web browser. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`. See these references for instructions on how to install a new font on [Windows](https://support.microsoft.com/en-us/help/314960/how-to-install-or-remove-a-font-in-windows) or [macOS](https://support.apple.com/en-us/ht201749).  Fonts that are not installed locally can also be loaded from a url by defining a [@font-face](https://developer.mozilla.org/en-US/docs/Web/CSS/@font-face) in a css file and referencing it from the `family` property in a symbol layer.  **Font families for 2D MapImageLayer**  The supported font families for [MapImageLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) are dependent upon the fonts installed on the [ArcGIS Server](https://enterprise.arcgis.com/en/server/latest/get-started/windows/what-is-arcgis-for-server-.htm) that published the layer. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`.  **Font families for 2D FeatureLayer**, CSVLayer, StreamLayer, and TextSymbol  The supported font families are based on hosted fonts files in `.pbf` format. By default, the fonts available are mostly the same ones used by the Esri Vector Basemaps. These fonts are available via `https://static.arcgis.com/fonts`. The URL can be configured to point to your own font resources by setting the [esriConfig.fontsUrl](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#fontsUrl) property. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`. This uses the `Arial Unicode MS` font file.  > A preview of the fonts listed in the following table are available in the [Labeling](https://developers.arcgis.com/javascript/latest/labeling/) overview page.  List of fonts currently supported in a 2D MapView:  [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#family)
     */
    family: string;
    /**
     * The font size in points.
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#size)
     */
    size: number;
    /**
     * The text style.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#style)
     */
    style: "normal" | "italic" | "oblique";
    /**
     * The text weight.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#weight)
     */
    weight: "normal" | "bold" | "bolder" | "lighter";

    /**
     * Creates a deep clone of the font object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#clone)
     */
    clone(): Font;

    static fromJSON(json: any): Font;
  }

  interface FontProperties {
    /**
     * The text decoration.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#decoration)
     */
    decoration?: "underline" | "line-through" | "none";
    /**
     * The font family of the text.
     *
     * @default "sans-serif"
     *
     * The font family of the text. The possible values are dependent upon the layer type, and if you are working with a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html). See the [Labeling guide page](https://developers.arcgis.com/javascript/latest/labeling/) for detailed explanation, or click the `Read more` below.  >>> esri-read-more **Font families for 3D SceneViews**  The supported font families for 3D [SceneViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) are dependent upon the fonts installed on the user's computer and web browser. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`. See these references for instructions on how to install a new font on [Windows](https://support.microsoft.com/en-us/help/314960/how-to-install-or-remove-a-font-in-windows) or [macOS](https://support.apple.com/en-us/ht201749).  Fonts that are not installed locally can also be loaded from a url by defining a [@font-face](https://developer.mozilla.org/en-US/docs/Web/CSS/@font-face) in a css file and referencing it from the `family` property in a symbol layer.  **Font families for 2D MapImageLayer**  The supported font families for [MapImageLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) are dependent upon the fonts installed on the [ArcGIS Server](https://enterprise.arcgis.com/en/server/latest/get-started/windows/what-is-arcgis-for-server-.htm) that published the layer. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`.  **Font families for 2D FeatureLayer**, CSVLayer, StreamLayer, and TextSymbol  The supported font families are based on hosted fonts files in `.pbf` format. By default, the fonts available are mostly the same ones used by the Esri Vector Basemaps. These fonts are available via `https://static.arcgis.com/fonts`. The URL can be configured to point to your own font resources by setting the [esriConfig.fontsUrl](https://developers.arcgis.com/javascript/latest/api-reference/esri-config.html#fontsUrl) property. If an app uses a font that is not installed, the Font class implements a fallback mechanism that will use the default font family value, which is `sans-serif`. This uses the `Arial Unicode MS` font file.  > A preview of the fonts listed in the following table are available in the [Labeling](https://developers.arcgis.com/javascript/latest/labeling/) overview page.  List of fonts currently supported in a 2D MapView:  [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#family)
     */
    family?: string;
    /**
     * The font size in points.
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#size)
     */
    size?: number | string;
    /**
     * The text style.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#style)
     */
    style?: "normal" | "italic" | "oblique";
    /**
     * The text weight.
     *
     * @default "normal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html#weight)
     */
    weight?: "normal" | "bold" | "bolder" | "lighter";
  }

  export class IconSymbol3DLayer extends Symbol3DLayer {
    /**
     * IconSymbol3DLayer is used to render [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometries using a flat 2D icon (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html)
     */

    constructor(properties?: IconSymbol3DLayerProperties);

    /**
     * The positioning of the icon relative to the geometry.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchor)
     */
    anchor:
      | "center"
      | "left"
      | "right"
      | "top"
      | "bottom"
      | "top-left"
      | "top-right"
      | "bottom-left"
      | "bottom-right"
      | "relative";
    /**
     * Defines the [anchor](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchor) relative to the center of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchorPosition)
     */
    anchorPosition: IconSymbol3DLayerAnchorPosition;
    /**
     * The clockwise rotation angle of the icon in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#angle)
     */
    angle: number;
    /**
     * The material used to shade the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#material)
     */
    material: IconSymbol3DLayerMaterial;
    /**
     * The outline of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    outline: IconSymbol3DLayerOutline;
    /**
     * The shape (`primitive`) or image URL (`href`) used to visualize the features.
     *
     * @default { primitive: "circle" }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#resource)
     */
    resource: IconSymbol3DLayerResource;
    /**
     * The size of the icon in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#size)
     */
    size: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#type)
     */
    readonly type: "icon";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#clone)
     */
    clone(): IconSymbol3DLayer;

    static fromJSON(json: any): IconSymbol3DLayer;
  }

  interface IconSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * The positioning of the icon relative to the geometry.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchor)
     */
    anchor?:
      | "center"
      | "left"
      | "right"
      | "top"
      | "bottom"
      | "top-left"
      | "top-right"
      | "bottom-left"
      | "bottom-right"
      | "relative";
    /**
     * Defines the [anchor](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchor) relative to the center of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchorPosition)
     */
    anchorPosition?: IconSymbol3DLayerAnchorPosition;
    /**
     * The clockwise rotation angle of the icon in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#angle)
     */
    angle?: number;
    /**
     * The material used to shade the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#material)
     */
    material?: IconSymbol3DLayerMaterialProperties;
    /**
     * The outline of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    outline?: IconSymbol3DLayerOutlineProperties;
    /**
     * The shape (`primitive`) or image URL (`href`) used to visualize the features.
     *
     * @default { primitive: "circle" }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#resource)
     */
    resource?: IconSymbol3DLayerResource;
    /**
     * The size of the icon in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#size)
     */
    size?: number | string;
  }

  export interface IconSymbol3DLayerAnchorPosition {
    /**
     * Value defining the position relative to the icon's width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchorPosition)
     */
    x: number;
    /**
     * Value defining the position relative to the icon's height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#anchorPosition)
     */
    y: number;
  }

  export interface IconSymbol3DLayerMaterialProperties {
    /**
     * The fill color of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface IconSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The fill color of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export interface IconSymbol3DLayerOutlineProperties {
    /**
     * The color of the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    color?: Color | number[] | string;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    size?: number | string;
  }

  export interface IconSymbol3DLayerOutline extends AnonymousAccessor {
    /**
     * The color of the outline.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    color?: Color;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#outline)
     */
    size?: number;
  }

  export interface IconSymbol3DLayerResource {
    /**
     * Uses a built-in shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#resource)
     */
    primitive?: string;
    /**
     * The URL or data URI for the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-IconSymbol3DLayer.html#resource)
     */
    href?: string;
  }

  export class LabelSymbol3D extends Symbol3D {
    /**
     * LabelSymbol3D is used to render labels for features from a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html)
     */

    constructor(properties?: LabelSymbol3DProperties);

    /**
     * Settings for adding a callout visualization to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#callout)
     */
    callout: Callout3D;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<TextSymbol3DLayer>;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#type)
     */
    readonly type: "label-3d";
    /**
     * Shifts the symbol along the vertical world axis by a given height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#verticalOffset)
     */
    verticalOffset: Symbol3DVerticalOffset;

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#clone)
     */
    clone(): LabelSymbol3D;

    static fromJSON(json: any): LabelSymbol3D;
  }

  interface LabelSymbol3DProperties extends Symbol3DProperties {
    /**
     * Settings for adding a callout visualization to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#callout)
     */
    callout?: Callout3DProperties;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<TextSymbol3DLayerProperties>;
    /**
     * Shifts the symbol along the vertical world axis by a given height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LabelSymbol3D.html#verticalOffset)
     */
    verticalOffset?: Symbol3DVerticalOffsetProperties;
  }

  export interface LineStyleMarker3D extends Accessor, Clonable {}

  export class LineStyleMarker3D {
    /**
     * LineStyleMarker3D is used for rendering a simple marker graphic on a [LineSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html)
     */

    constructor(properties?: LineStyleMarker3DProperties);

    /**
     * The color of the marker.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#color)
     */
    color: Color;
    /**
     * Indicates where the marker is placed.
     *
     * @default "begin-end"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#placement)
     */
    placement: "begin" | "end" | "begin-end";
    /**
     * Style of the marker.
     *
     * @default "arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#style)
     */
    style: "arrow" | "circle" | "square" | "diamond" | "cross" | "x";
    /**
     * The type of marker applied to a line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#type)
     */
    readonly type: "style";
  }

  interface LineStyleMarker3DProperties {
    /**
     * The color of the marker.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#color)
     */
    color?: Color | number[] | string;
    /**
     * Indicates where the marker is placed.
     *
     * @default "begin-end"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#placement)
     */
    placement?: "begin" | "end" | "begin-end";
    /**
     * Style of the marker.
     *
     * @default "arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#style)
     */
    style?: "arrow" | "circle" | "square" | "diamond" | "cross" | "x";
    /**
     * The type of marker applied to a line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineStyleMarker3D.html#type)
     */
    type?: "style";
  }

  export class LineSymbol extends Symbol {
    /**
     * Line symbols are used to draw [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) features in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol.html)
     */

    constructor(properties?: LineSymbolProperties);

    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol.html#type)
     */
    type: "simple-line";
    /**
     * The width of the symbol in points.
     *
     * @default 0.75
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol.html#width)
     */
    width: number;

    static fromJSON(json: any): LineSymbol;
  }

  interface LineSymbolProperties extends SymbolProperties {
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol.html#type)
     */
    type?: "simple-line";
    /**
     * The width of the symbol in points.
     *
     * @default 0.75
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol.html#width)
     */
    width?: number | string;
  }

  export class LineSymbol3D extends Symbol3D {
    /**
     * LineSymbol3D is used to render features with [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html)
     */

    constructor(properties?: LineSymbol3DProperties);

    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<LineSymbol3DLayer | PathSymbol3DLayer>;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html#type)
     */
    readonly type: "line-3d";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html#clone)
     */
    clone(): LineSymbol3D;

    static fromJSON(json: any): LineSymbol3D;
  }

  interface LineSymbol3DProperties extends Symbol3DProperties {
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<
      (LineSymbol3DLayerProperties & { type: "line" }) | (PathSymbol3DLayerProperties & { type: "path" })
    >;
  }

  export class LineSymbol3DLayer extends Symbol3DLayer {
    /**
     * LineSymbol3DLayer renders [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometries using a flat 2D line with a [LineSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html) in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html)
     */

    constructor(properties?: LineSymbol3DLayerProperties);

    /**
     * The style used to draw the endpoint of a line.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#cap)
     */
    cap: "butt" | "round" | "square";
    /**
     * The style used to draw the intersection of two line segments within a line geometry.
     *
     * @default "miter"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#join)
     */
    join: "miter" | "round" | "bevel";
    /**
     * Optional markers to be placed at the start and/or end of each line geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#marker)
     */
    marker: LineStyleMarker3D;
    /**
     * The material used to shade the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#material)
     */
    material: LineSymbol3DLayerMaterial;
    /**
     * The pattern used to render the line stroke.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#pattern)
     */
    pattern: LineStylePattern3D;
    /**
     * The width of the line in points.
     *
     * @default "1px"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#size)
     */
    size: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#type)
     */
    readonly type: "line";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#clone)
     */
    clone(): LineSymbol3DLayer;

    static fromJSON(json: any): LineSymbol3DLayer;
  }

  interface LineSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * The style used to draw the endpoint of a line.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#cap)
     */
    cap?: "butt" | "round" | "square";
    /**
     * The style used to draw the intersection of two line segments within a line geometry.
     *
     * @default "miter"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#join)
     */
    join?: "miter" | "round" | "bevel";
    /**
     * Optional markers to be placed at the start and/or end of each line geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#marker)
     */
    marker?: LineStyleMarker3DProperties;
    /**
     * The material used to shade the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#material)
     */
    material?: LineSymbol3DLayerMaterialProperties;
    /**
     * The pattern used to render the line stroke.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#pattern)
     */
    pattern?: LineStylePattern3DProperties;
    /**
     * The width of the line in points.
     *
     * @default "1px"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#size)
     */
    size?: number | string;
  }

  export interface LineSymbol3DLayerMaterialProperties {
    /**
     * The color of the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface LineSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The color of the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export class LineSymbolMarker extends Accessor {
    /**
     * LineSymbolMarker is used for rendering a simple marker graphic on a [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html)
     */

    constructor(properties?: LineSymbolMarkerProperties);

    /**
     * The color of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#color)
     */
    color: Color;
    /**
     * The placement of the marker(s) on the line.
     *
     * @default "begin-end"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#placement)
     */
    placement: "begin" | "end" | "begin-end";
    /**
     * The marker style.
     *
     * @default "arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#style)
     */
    style: "arrow" | "circle" | "square" | "diamond" | "cross" | "x";
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#type)
     */
    readonly type: "line-marker";
  }

  interface LineSymbolMarkerProperties {
    /**
     * The color of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#color)
     */
    color?: Color | number[] | string;
    /**
     * The placement of the marker(s) on the line.
     *
     * @default "begin-end"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#placement)
     */
    placement?: "begin" | "end" | "begin-end";
    /**
     * The marker style.
     *
     * @default "arrow"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbolMarker.html#style)
     */
    style?: "arrow" | "circle" | "square" | "diamond" | "cross" | "x";
  }

  export class MarkerSymbol extends Symbol {
    /**
     * Marker symbols are used to draw [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) graphics in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or individual [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html)
     */

    constructor(properties?: MarkerSymbolProperties);

    /**
     * The angle of the marker relative to the screen in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#angle)
     */
    angle: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#type)
     */
    type: "simple-marker" | "picture-marker";
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#xoffset)
     */
    xoffset: number;
    /**
     * The offset on the y-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#yoffset)
     */
    yoffset: number;

    static fromJSON(json: any): MarkerSymbol;
  }

  interface MarkerSymbolProperties extends SymbolProperties {
    /**
     * The angle of the marker relative to the screen in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#angle)
     */
    angle?: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#type)
     */
    type?: "simple-marker" | "picture-marker";
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#xoffset)
     */
    xoffset?: number | string;
    /**
     * The offset on the y-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MarkerSymbol.html#yoffset)
     */
    yoffset?: number | string;
  }

  export class MeshSymbol3D extends Symbol3D {
    /**
     * MeshSymbol3D is used to render 3D mesh features in a [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MeshSymbol3D.html)
     */

    constructor(properties?: MeshSymbol3DProperties);

    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MeshSymbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<FillSymbol3DLayer>;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MeshSymbol3D.html#type)
     */
    readonly type: "mesh-3d";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MeshSymbol3D.html#clone)
     */
    clone(): MeshSymbol3D;

    static fromJSON(json: any): MeshSymbol3D;
  }

  interface MeshSymbol3DProperties extends Symbol3DProperties {
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-MeshSymbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<FillSymbol3DLayerProperties>;
  }

  export class ObjectSymbol3DLayer extends Symbol3DLayer {
    /**
     * ObjectSymbol3DLayer is used to render [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometries using a volumetric 3D shape (e.g., a sphere or cylinder) with a [PointSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html)
     */

    constructor(properties?: ObjectSymbol3DLayerProperties);

    /**
     * The positioning of the symbol relative to the geometry.
     *
     * @default "origin"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchor)
     */
    anchor: "center" | "top" | "bottom" | "origin" | "relative";
    /**
     * Defines the [anchor](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchor) relative to the center of the [symbol layer resource](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchorPosition)
     */
    anchorPosition: ObjectSymbol3DLayerAnchorPosition;
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#castShadows)
     */
    castShadows: boolean;
    /**
     * The depth, or diameter from north to south, of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#depth)
     */
    depth: number;
    /**
     * The clockwise rotation of the symbol in the horizontal plane (i.e., around the z axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#heading)
     */
    heading: number;
    /**
     * The height of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#height)
     */
    height: number;
    /**
     * The material used to shade the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#material)
     */
    material: ObjectSymbol3DLayerMaterial;
    /**
     * The primitive shape (`primitive`) or external 3D model (`href`) used to visualize the points.
     *
     * @default { primitive: "sphere" }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource)
     */
    resource: ObjectSymbol3DLayerResource;
    /**
     * The rotation of the symbol in the lateral vertical plane (i.e., around the y axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#roll)
     */
    roll: number;
    /**
     * The rotation of the symbol in the longitudinal vertical plane (i.e., around the x axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#tilt)
     */
    tilt: number;
    /**
     * The object type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#type)
     */
    readonly type: "object";
    /**
     * The width, or diameter from east to west, of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#width)
     */
    width: number;

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#clone)
     */
    clone(): ObjectSymbol3DLayer;

    static fromJSON(json: any): ObjectSymbol3DLayer;
  }

  interface ObjectSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * The positioning of the symbol relative to the geometry.
     *
     * @default "origin"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchor)
     */
    anchor?: "center" | "top" | "bottom" | "origin" | "relative";
    /**
     * Defines the [anchor](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchor) relative to the center of the [symbol layer resource](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchorPosition)
     */
    anchorPosition?: ObjectSymbol3DLayerAnchorPosition;
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#castShadows)
     */
    castShadows?: boolean;
    /**
     * The depth, or diameter from north to south, of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#depth)
     */
    depth?: number;
    /**
     * The clockwise rotation of the symbol in the horizontal plane (i.e., around the z axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#heading)
     */
    heading?: number;
    /**
     * The height of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#height)
     */
    height?: number;
    /**
     * The material used to shade the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#material)
     */
    material?: ObjectSymbol3DLayerMaterialProperties;
    /**
     * The primitive shape (`primitive`) or external 3D model (`href`) used to visualize the points.
     *
     * @default { primitive: "sphere" }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource)
     */
    resource?: ObjectSymbol3DLayerResource;
    /**
     * The rotation of the symbol in the lateral vertical plane (i.e., around the y axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#roll)
     */
    roll?: number;
    /**
     * The rotation of the symbol in the longitudinal vertical plane (i.e., around the x axis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#tilt)
     */
    tilt?: number;
    /**
     * The width, or diameter from east to west, of the object in meters.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#width)
     */
    width?: number;
  }

  export interface ObjectSymbol3DLayerAnchorPosition {
    /**
     * Value defining the position relative to the x axis of the bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchorPosition)
     */
    x: number;
    /**
     * Value defining the position relative to the y axis of the bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchorPosition)
     */
    y: number;
    /**
     * Value defining the position relative to the z axis of the bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#anchorPosition)
     */
    z: number;
  }

  export interface ObjectSymbol3DLayerMaterialProperties {
    /**
     * The fill color of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface ObjectSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The fill color of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export interface ObjectSymbol3DLayerResource {
    /**
     * Uses a built-in shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource)
     */
    primitive?: string;
    /**
     * The URL to the 3D model in [glTF format](https://www.khronos.org/gltf/).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ObjectSymbol3DLayer.html#resource)
     */
    href?: string;
  }

  export class PathSymbol3DLayer extends Symbol3DLayer {
    /**
     * PathSymbol3DLayer renders [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometries by extruding a 2D profile along the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html)
     */

    constructor(properties?: PathSymbol3DLayerProperties);

    /**
     * Defines offset of the path cross section relative to the [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#anchor)
     */
    anchor: "center" | "bottom" | "top";
    /**
     * Controls the shape at the start and end point of the path.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#cap)
     */
    cap: "none" | "butt" | "square" | "round";
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#castShadows)
     */
    castShadows: boolean;
    /**
     * The vertical dimension of the cross-section of the path in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#height)
     */
    height: number;
    /**
     * Controls the shape of the connection between two segments of the path.
     *
     * @default "miter"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#join)
     */
    join: "miter" | "bevel" | "round";
    /**
     * The material used to shade the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#material)
     */
    material: PathSymbol3DLayerMaterial;
    /**
     * Cross-section profile of the path geometry.
     *
     * @default "circle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profile)
     */
    profile: "circle" | "quad";
    /**
     * Defines how the [profile](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profile) is rotated as it is extruded along the [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profileRotation)
     */
    profileRotation: "heading" | "all";
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#type)
     */
    readonly type: "path";
    /**
     * The horizontal dimension of the cross-section of the path in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#width)
     */
    width: number;

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#clone)
     */
    clone(): PathSymbol3DLayer;

    static fromJSON(json: any): PathSymbol3DLayer;
  }

  interface PathSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * Defines offset of the path cross section relative to the [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#anchor)
     */
    anchor?: "center" | "bottom" | "top";
    /**
     * Controls the shape at the start and end point of the path.
     *
     * @default "butt"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#cap)
     */
    cap?: "none" | "butt" | "square" | "round";
    /**
     * Indicates whether the symbol layer geometry casts shadows in the scene.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#castShadows)
     */
    castShadows?: boolean;
    /**
     * The vertical dimension of the cross-section of the path in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#height)
     */
    height?: number;
    /**
     * Controls the shape of the connection between two segments of the path.
     *
     * @default "miter"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#join)
     */
    join?: "miter" | "bevel" | "round";
    /**
     * The material used to shade the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#material)
     */
    material?: PathSymbol3DLayerMaterialProperties;
    /**
     * Cross-section profile of the path geometry.
     *
     * @default "circle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profile)
     */
    profile?: "circle" | "quad";
    /**
     * Defines how the [profile](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profile) is rotated as it is extruded along the [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#profileRotation)
     */
    profileRotation?: "heading" | "all";
    /**
     * The horizontal dimension of the cross-section of the path in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#width)
     */
    width?: number;
  }

  export interface PathSymbol3DLayerMaterialProperties {
    /**
     * The color of the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface PathSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The color of the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PathSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export interface LineStylePattern3D extends Accessor, JSONSupport {}

  export class LineStylePattern3D {
    /**
     * Renders lines with predefined style patterns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-LineStylePattern3D.html)
     */

    constructor(properties?: LineStylePattern3DProperties);

    /**
     * The pattern style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-LineStylePattern3D.html#style)
     */
    style:
      | "dash"
      | "dash-dot"
      | "dot"
      | "long-dash"
      | "long-dash-dot"
      | "long-dash-dot-dot"
      | "none"
      | "short-dash"
      | "short-dash-dot"
      | "short-dash-dot-dot"
      | "short-dot"
      | "solid";
    /**
     * The pattern type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-LineStylePattern3D.html#type)
     */
    readonly type: "style";

    /**
     * Creates a deep clone of the pattern.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-LineStylePattern3D.html#clone)
     */
    clone(): LineStylePattern3D;

    static fromJSON(json: any): LineStylePattern3D;
  }

  interface LineStylePattern3DProperties {
    /**
     * The pattern style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-LineStylePattern3D.html#style)
     */
    style?:
      | "dash"
      | "dash-dot"
      | "dot"
      | "long-dash"
      | "long-dash-dot"
      | "long-dash-dot-dot"
      | "none"
      | "short-dash"
      | "short-dash-dot"
      | "short-dash-dot-dot"
      | "short-dot"
      | "solid";
  }

  export interface StylePattern3D extends Accessor, JSONSupport {}

  export class StylePattern3D {
    /**
     * Renders polygons with predefined style pattern fills.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-StylePattern3D.html)
     */

    constructor(properties?: StylePattern3DProperties);

    /**
     * The fill style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-StylePattern3D.html#style)
     */
    style:
      | "backward-diagonal"
      | "cross"
      | "diagonal-cross"
      | "forward-diagonal"
      | "horizontal"
      | "none"
      | "solid"
      | "vertical";
    /**
     * The pattern type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-StylePattern3D.html#type)
     */
    readonly type: "style";

    /**
     * Creates a deep clone of the pattern.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-StylePattern3D.html#clone)
     */
    clone(): StylePattern3D;

    static fromJSON(json: any): StylePattern3D;
  }

  interface StylePattern3DProperties {
    /**
     * The fill style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-patterns-StylePattern3D.html#style)
     */
    style?:
      | "backward-diagonal"
      | "cross"
      | "diagonal-cross"
      | "forward-diagonal"
      | "horizontal"
      | "none"
      | "solid"
      | "vertical";
  }

  export class PictureFillSymbol extends FillSymbol {
    /**
     * PictureFillSymbol uses an image in a repeating pattern to symbolize polygon features in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html)
     */

    constructor(properties?: PictureFillSymbolProperties);

    /**
     * The height of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#height)
     */
    height: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#type)
     */
    readonly type: "picture-fill";
    /**
     * The URL to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#url)
     */
    url: string;
    /**
     * The width of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#width)
     */
    width: number;
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#xoffset)
     */
    xoffset: number;
    /**
     * The scale factor on the x axis of the symbol.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#xscale)
     */
    xscale: number;
    /**
     * The offset on the y-axis in pixels or points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#yoffset)
     */
    yoffset: number;
    /**
     * The scale factor on the y axis of the symbol.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#yscale)
     */
    yscale: number;

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#clone)
     */
    clone(): PictureFillSymbol;

    static fromJSON(json: any): PictureFillSymbol;
  }

  interface PictureFillSymbolProperties extends FillSymbolProperties {
    /**
     * The height of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#height)
     */
    height?: number | string;
    /**
     * The URL to the image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#url)
     */
    url?: string;
    /**
     * The width of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#width)
     */
    width?: number | string;
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#xoffset)
     */
    xoffset?: number | string;
    /**
     * The scale factor on the x axis of the symbol.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#xscale)
     */
    xscale?: number;
    /**
     * The offset on the y-axis in pixels or points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#yoffset)
     */
    yoffset?: number | string;
    /**
     * The scale factor on the y axis of the symbol.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureFillSymbol.html#yscale)
     */
    yscale?: number;
  }

  export class PictureMarkerSymbol extends MarkerSymbol {
    /**
     * PictureMarkerSymbol renders [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) graphics in either a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) using an image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html)
     */

    constructor(properties?: PictureMarkerSymbolProperties);

    /**
     * The height of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#height)
     */
    height: number;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#type)
     */
    readonly type: "picture-marker";
    /**
     * The URL to an image or SVG document.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#url)
     */
    url: string;
    /**
     * The width of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#width)
     */
    width: number;

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#clone)
     */
    clone(): PictureMarkerSymbol;

    static fromJSON(json: any): PictureMarkerSymbol;
  }

  interface PictureMarkerSymbolProperties extends MarkerSymbolProperties {
    /**
     * The height of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#height)
     */
    height?: number | string;
    /**
     * The URL to an image or SVG document.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#url)
     */
    url?: string;
    /**
     * The width of the image in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PictureMarkerSymbol.html#width)
     */
    width?: number | string;
  }

  export class PointSymbol3D extends Symbol3D {
    /**
     * PointSymbol3D is used to render features with [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html)
     */

    constructor(properties?: PointSymbol3DProperties);

    /**
     * Settings for adding a callout visualization to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#callout)
     */
    callout: LineCallout3D;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<IconSymbol3DLayer | ObjectSymbol3DLayer | TextSymbol3DLayer>;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#type)
     */
    readonly type: "point-3d";
    /**
     * Shifts the symbol along the vertical world axis by a given height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#verticalOffset)
     */
    verticalOffset: Symbol3DVerticalOffset;

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#clone)
     */
    clone(): PointSymbol3D;

    static fromJSON(json: any): PointSymbol3D;
  }

  interface PointSymbol3DProperties extends Symbol3DProperties {
    /**
     * Settings for adding a callout visualization to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#callout)
     */
    callout?: LineCallout3DProperties;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<
      | (IconSymbol3DLayerProperties & { type: "icon" })
      | (ObjectSymbol3DLayerProperties & { type: "object" })
      | (TextSymbol3DLayerProperties & { type: "text" })
    >;
    /**
     * Shifts the symbol along the vertical world axis by a given height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html#verticalOffset)
     */
    verticalOffset?: Symbol3DVerticalOffsetProperties;
  }

  export class PolygonSymbol3D extends Symbol3D {
    /**
     * PolygonSymbol3D is used to render features with [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometry in a 3D [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html)
     */

    constructor(properties?: PolygonSymbol3DProperties);

    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<
      | ExtrudeSymbol3DLayer
      | FillSymbol3DLayer
      | IconSymbol3DLayer
      | LineSymbol3DLayer
      | ObjectSymbol3DLayer
      | TextSymbol3DLayer
      | WaterSymbol3DLayer
    >;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html#type)
     */
    readonly type: "polygon-3d";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html#clone)
     */
    clone(): PolygonSymbol3D;

    static fromJSON(json: any): PolygonSymbol3D;
  }

  interface PolygonSymbol3DProperties extends Symbol3DProperties {
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<
      | (ExtrudeSymbol3DLayerProperties & { type: "extrude" })
      | (FillSymbol3DLayerProperties & { type: "fill" })
      | (IconSymbol3DLayerProperties & { type: "icon" })
      | (LineSymbol3DLayerProperties & { type: "line" })
      | (ObjectSymbol3DLayerProperties & { type: "object" })
      | (TextSymbol3DLayerProperties & { type: "text" })
      | (WaterSymbol3DLayerProperties & { type: "water" })
    >;
  }

  export class SimpleFillSymbol extends FillSymbol {
    /**
     * SimpleFillSymbol is used for rendering 2D polygons in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html)
     */

    constructor(properties?: SimpleFillSymbolProperties);

    /**
     * The fill style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html#style)
     */
    style:
      | "backward-diagonal"
      | "cross"
      | "diagonal-cross"
      | "forward-diagonal"
      | "horizontal"
      | "none"
      | "solid"
      | "vertical";
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html#type)
     */
    readonly type: "simple-fill";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html#clone)
     */
    clone(): SimpleFillSymbol;

    static fromJSON(json: any): SimpleFillSymbol;
  }

  interface SimpleFillSymbolProperties extends FillSymbolProperties {
    /**
     * The fill style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html#style)
     */
    style?:
      | "backward-diagonal"
      | "cross"
      | "diagonal-cross"
      | "forward-diagonal"
      | "horizontal"
      | "none"
      | "solid"
      | "vertical";
  }

  export class SimpleLineSymbol extends LineSymbol {
    /**
     * SimpleLineSymbol is used for rendering 2D [polyline geometries](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html)
     */

    constructor(properties?: SimpleLineSymbolProperties);

    /**
     * Specifies the cap style.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#cap)
     */
    cap: "butt" | "round" | "square";
    /**
     * Specifies the join style.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#join)
     */
    join: "miter" | "round" | "bevel";
    /**
     * Specifies the color, style, and placement of a symbol marker on the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#marker)
     */
    marker: LineSymbolMarker;
    /**
     * Maximum allowed ratio of the width of a miter join to the line width.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#miterLimit)
     */
    miterLimit: number;
    /**
     * Specifies the line style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#style)
     */
    style:
      | "dash"
      | "dash-dot"
      | "dot"
      | "long-dash"
      | "long-dash-dot"
      | "long-dash-dot-dot"
      | "none"
      | "short-dash"
      | "short-dash-dot"
      | "short-dash-dot-dot"
      | "short-dot"
      | "solid";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#clone)
     */
    clone(): SimpleLineSymbol;

    static fromJSON(json: any): SimpleLineSymbol;
  }

  interface SimpleLineSymbolProperties extends LineSymbolProperties {
    /**
     * Specifies the cap style.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#cap)
     */
    cap?: "butt" | "round" | "square";
    /**
     * Specifies the join style.
     *
     * @default "round"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#join)
     */
    join?: "miter" | "round" | "bevel";
    /**
     * Specifies the color, style, and placement of a symbol marker on the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#marker)
     */
    marker?: LineSymbolMarkerProperties;
    /**
     * Maximum allowed ratio of the width of a miter join to the line width.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#miterLimit)
     */
    miterLimit?: number;
    /**
     * Specifies the line style.
     *
     * @default "solid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html#style)
     */
    style?:
      | "dash"
      | "dash-dot"
      | "dot"
      | "long-dash"
      | "long-dash-dot"
      | "long-dash-dot-dot"
      | "none"
      | "short-dash"
      | "short-dash-dot"
      | "short-dash-dot-dot"
      | "short-dot"
      | "solid";
  }

  export class SimpleMarkerSymbol extends MarkerSymbol {
    /**
     * SimpleMarkerSymbol is used for rendering 2D [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometries with a simple shape and [color](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#color) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html)
     */

    constructor(properties?: SimpleMarkerSymbolProperties);

    /**
     * The outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#outline)
     */
    outline: SimpleLineSymbol;
    /**
     * The SVG path of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#path)
     */
    path: string;
    /**
     * The size of the marker in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#size)
     */
    size: number;
    /**
     * The marker style.
     *
     * @default "circle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#style)
     */
    style: "circle" | "square" | "cross" | "x" | "diamond" | "triangle" | "path";
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#type)
     */
    readonly type: "simple-marker";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#clone)
     */
    clone(): SimpleMarkerSymbol;

    static fromJSON(json: any): SimpleMarkerSymbol;
  }

  interface SimpleMarkerSymbolProperties extends MarkerSymbolProperties {
    /**
     * The outline of the marker symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#outline)
     */
    outline?: SimpleLineSymbolProperties;
    /**
     * The SVG path of the icon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#path)
     */
    path?: string;
    /**
     * The size of the marker in points.
     *
     * @default 12
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#size)
     */
    size?: number | string;
    /**
     * The marker style.
     *
     * @default "circle"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html#style)
     */
    style?: "circle" | "square" | "cross" | "x" | "diamond" | "triangle" | "path";
  }

  /**
   * Provides utility functions for converting simple symbols and picture symbols to [CIMSymbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimConversionUtils.html)
   */
  interface cimConversionUtils {
    /**
     * Converts a given symbol to a CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimConversionUtils.html#convertToCIMSymbol)
     */
    convertToCIMSymbol(
      symbol: SimpleMarkerSymbol | SimpleLineSymbol | SimpleFillSymbol | PictureMarkerSymbol | PictureFillSymbol,
    ): CIMSymbol;
  }

  export const cimConversionUtils: cimConversionUtils;

  /**
   * Provides utility functions for [CIMSymbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html)
   */
  interface cimSymbolUtils {
    /**
     * Sets the color of the symbol layers of a CIMSymbol to a given value if the symbol layer is not color locked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#applyCIMSymbolColor)
     */
    applyCIMSymbolColor(symbol: CIMSymbol, color: Color, options?: cimSymbolUtilsApplyCIMSymbolColorOptions): void;
    /**
     * Sets the rotation value of a CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#applyCIMSymbolRotation)
     */
    applyCIMSymbolRotation(symbol: CIMSymbol, rotation: number, clockwise?: boolean): void;
    /**
     * Returns the first color of the symbol layers in a CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#getCIMSymbolColor)
     */
    getCIMSymbolColor(symbol: CIMSymbol): Color;
    /**
     * Returns the rotation value of a CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#getCIMSymbolRotation)
     */
    getCIMSymbolRotation(symbol: CIMSymbol, clockwise?: boolean): number;
    /**
     * Returns the size of a given CIMSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#getCIMSymbolSize)
     */
    getCIMSymbolSize(symbol: CIMSymbol): number;
    /**
     * Scales the largest layer of a CIMSymbol to a given size.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#scaleCIMSymbolTo)
     */
    scaleCIMSymbolTo(symbol: CIMSymbol, size: number, options?: cimSymbolUtilsScaleCIMSymbolToOptions): void;
  }

  export const cimSymbolUtils: cimSymbolUtils;

  export interface cimSymbolUtilsApplyCIMSymbolColorOptions {
    /**
     * The symbol layers to apply the color to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#applyCIMSymbolColor)
     */
    layersToColor?: "fill" | "outline" | "fill-and-outline";
  }

  export interface cimSymbolUtilsScaleCIMSymbolToOptions {
    /**
     * When `true`, this property will preserve the outline width of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-cimSymbolUtils.html#scaleCIMSymbolTo)
     */
    preserveOutlineWidth?: boolean;
  }

  /**
   * Provides a utility method used to deserialize a JSON symbol object returned by the REST API.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-jsonUtils.html)
   */
  interface symbolsSupportJsonUtils {
    /**
     * Creates a new instance of an appropriate [Symbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-jsonUtils.html#fromJSON)
     */
    fromJSON(json: any): Symbol;
  }

  export const symbolsSupportJsonUtils: symbolsSupportJsonUtils;

  export class StyleOrigin extends Accessor {
    /**
     * The style origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html)
     */

    constructor(properties?: StyleOriginProperties);

    /**
     * Name of the symbol in the style referenced by styleName or styleUrl.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#name)
     */
    name: string;
    /**
     * The portal of the style origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#portal)
     */
    portal: Portal;
    /**
     * The well-known esri-provided style, such as `EsriThematicShapesStyle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#styleName)
     */
    styleName: string;
    /**
     * A url to a style definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#styleUrl)
     */
    styleUrl: string;

    /**
     * Creates a deep clone of the style origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#clone)
     */
    clone(): StyleOrigin;
  }

  interface StyleOriginProperties {
    /**
     * Name of the symbol in the style referenced by styleName or styleUrl.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#name)
     */
    name?: string;
    /**
     * The portal of the style origin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#portal)
     */
    portal?: PortalProperties;
    /**
     * The well-known esri-provided style, such as `EsriThematicShapesStyle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#styleName)
     */
    styleName?: string;
    /**
     * A url to a style definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-StyleOrigin.html#styleUrl)
     */
    styleUrl?: string;
  }

  export class Symbol3DVerticalOffset extends Accessor {
    /**
     * Shifts a symbol along the vertical world axis by a given height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html)
     */

    constructor(properties?: Symbol3DVerticalOffsetProperties);

    /**
     * The maximum vertical symbol offset in world units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#maxWorldLength)
     */
    maxWorldLength: number;
    /**
     * The minimum vertical symbol offset in world units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#minWorldLength)
     */
    minWorldLength: number;
    /**
     * Vertical symbol offset in screen size.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#screenLength)
     */
    screenLength: number;

    /**
     * Creates a deep clone of the vertical offset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#clone)
     */
    clone(): Symbol3DVerticalOffset;
  }

  interface Symbol3DVerticalOffsetProperties {
    /**
     * The maximum vertical symbol offset in world units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#maxWorldLength)
     */
    maxWorldLength?: number;
    /**
     * The minimum vertical symbol offset in world units.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#minWorldLength)
     */
    minWorldLength?: number;
    /**
     * Vertical symbol offset in screen size.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-Symbol3DVerticalOffset.html#screenLength)
     */
    screenLength?: number | string;
  }

  /**
   * Utilities for generating small preview images of symbols.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html)
   */
  interface symbolUtils {
    /**
     * Returns a color representing the input [graphic's](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    getDisplayedColor(graphic: Graphic, options?: symbolUtilsGetDisplayedColorOptions): Promise<Color>;
    /**
     * Returns a symbol representing the input [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    getDisplayedSymbol(graphic: Graphic, options?: symbolUtilsGetDisplayedSymbolOptions): Promise<Symbol>;
    /**
     * Returns a label corresponding to the symbol of the input [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) as displayed in the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    getLegendLabel(
      graphic: Graphic,
      view?: MapView | SceneView,
      options?: symbolUtilsGetLegendLabelOptions,
    ): Promise<string>;
    /**
     * Generates a preview image of a color ramp to display in a DOM element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderColorRampPreviewHTML)
     */
    renderColorRampPreviewHTML(colors: Color[], options?: symbolUtilsRenderColorRampPreviewHTMLOptions): HTMLElement;
    /**
     * Generates an HTML element representing the legend of a [PieChartRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-PieChartRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPieChartPreviewHTML)
     */
    renderPieChartPreviewHTML(colors: Color[], options?: symbolUtilsRenderPieChartPreviewHTMLOptions): HTMLElement;
    /**
     * Generates a preview image of a given symbol that can be displayed in a DOM element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    renderPreviewHTML(symbol: Symbol, options?: symbolUtilsRenderPreviewHTMLOptions): Promise<HTMLElement>;
    /**
     * Generates an HTML element representing the square or diamond legend element of a relationship renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderRelationshipRampPreviewHTML)
     */
    renderRelationshipRampPreviewHTML(
      renderer: UniqueValueRenderer,
      options?: symbolUtilsRenderRelationshipRampPreviewHTMLOptions,
    ): HTMLElement;
  }

  export const symbolUtils: symbolUtils;

  export interface symbolUtilsGetDisplayedColorOptions {
    /**
     * The [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which the graphic is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    scale?: number;
    /**
     * The [viewingMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewingMode) of the view, if the graphic is displayed in a SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    viewingMode?: string;
    /**
     * The spatial reference of the view in which the graphic is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    spatialReference?: SpatialReference;
    /**
     * The renderer of the layer associated with the `graphic`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    renderer?: Renderer;
    /**
     * The [resolution](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#resolution) of the view at which the graphic is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedColor)
     */
    resolution?: number;
  }

  export interface symbolUtilsGetDisplayedSymbolOptions {
    /**
     * The [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    scale?: number;
    /**
     * The [viewingMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewingMode) of the view, if the symbol is displayed in a SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    viewingMode?: string;
    /**
     * The spatial reference of the view in which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    spatialReference?: SpatialReference;
    /**
     * The renderer of the layer associated with the `graphic`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    renderer?: Renderer;
    /**
     * The [resolution](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#resolution) of the view at which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getDisplayedSymbol)
     */
    resolution?: number;
  }

  export interface symbolUtilsGetLegendLabelOptions {
    /**
     * The [view scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) at which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    scale?: number;
    /**
     * The [viewingMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewingMode) of the view, if the symbol is displayed in a SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    viewingMode?: string;
    /**
     * The spatial reference of the view in which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    spatialReference?: SpatialReference;
    /**
     * The renderer of the layer associated with the `graphic`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    renderer?: Renderer;
    /**
     * The [resolution](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#resolution) of the view at which the symbol is displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#getLegendLabel)
     */
    resolution?: number;
  }

  export interface symbolUtilsRenderColorRampPreviewHTMLOptions {
    /**
     * Specifies the alignment of the color ramp.
     *
     * @default vertical
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderColorRampPreviewHTML)
     */
    align?: "horizontal" | "vertical";
    /**
     * Indicates whether to render the color ramp with a continuous gradient.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderColorRampPreviewHTML)
     */
    gradient?: boolean;
    /**
     * The width of the ramp in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderColorRampPreviewHTML)
     */
    width?: number;
    /**
     * The height of the ramp in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderColorRampPreviewHTML)
     */
    height?: number;
  }

  export interface symbolUtilsRenderPieChartPreviewHTMLOptions {
    /**
     * The radius of the pie chart in pixels.
     *
     * @default 40
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPieChartPreviewHTML)
     */
    radius?: number;
    /**
     * The hole percentage of the chart on a scale of 0-1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPieChartPreviewHTML)
     */
    holePercentage?: number;
    /**
     * The values used to indicate the area each color should represent as a percentage when creating the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPieChartPreviewHTML)
     */
    values?: number[];
    /**
     * The outline of the pie chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPieChartPreviewHTML)
     */
    outline?: SimpleLineSymbol;
  }

  export interface symbolUtilsRenderPreviewHTMLOptions {
    /**
     * The parent node to append to the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    node?: HTMLElement;
    /**
     * Use a number to set the size (in points) of the symbol preview for any symbol representing a point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    size?: number | symbolUtilsRenderPreviewHTMLOptionsSize;
    /**
     * The maximum size of the symbol preview in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    maxSize?: number;
    /**
     * The opacity of the layer represented by the `symbol`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    opacity?: number;
    /**
     * When `true` the size of the symbol preview will include the outline in the measurement of the entire symbol.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    scale?: boolean;
    /**
     * Indicates whether to disable upsampling for raster images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    disableUpsampling?: boolean;
    /**
     * Options for setting the shape of a fill symbol preview.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    symbolConfig?: "tall" | symbolUtilsRenderPreviewHTMLOptionsSymbolConfig;
    /**
     * The rotation of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    rotation?: number;
    /**
     * _Since 4.25_ The text that will be displayed in the symbol preview of a TextSymbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    overrideText?: string;
  }

  export interface symbolUtilsRenderPreviewHTMLOptionsSize {
    /**
     * The width of the symbol preview in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    width?: number;
    /**
     * The height of the symbol preview in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    height?: number;
  }

  export interface symbolUtilsRenderPreviewHTMLOptionsSymbolConfig {
    /**
     * Set to `true` for "tall" symbols in portrait view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    isTall?: boolean;
    /**
     * Set to `true` to render the preview as a square instead of a generic polygon shape.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderPreviewHTML)
     */
    isSquareFill?: boolean;
  }

  export interface symbolUtilsRenderRelationshipRampPreviewHTMLOptions {
    /**
     * The container node in which to place the generated relationship element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderRelationshipRampPreviewHTML)
     */
    node?: HTMLElement;
    /**
     * The opacity of the legend element.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-support-symbolUtils.html#renderRelationshipRampPreviewHTML)
     */
    opacity?: number;
  }

  export interface Symbol extends Accessor, JSONSupport {}

  export class Symbol {
    /**
     * Symbol is the base class for all symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html)
     */

    constructor(properties?: SymbolProperties);

    /**
     * The color of the symbol.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html#color)
     */
    color: Color;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html#type)
     */
    readonly type:
      | "simple-marker"
      | "picture-marker"
      | "simple-line"
      | "simple-fill"
      | "picture-fill"
      | "text"
      | "shield-label-symbol"
      | "point-3d"
      | "line-3d"
      | "polygon-3d"
      | "web-style"
      | "mesh-3d"
      | "label-3d"
      | "cim";

    static fromJSON(json: any): Symbol;
  }

  interface SymbolProperties {
    /**
     * The color of the symbol.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html#color)
     */
    color?: Color | number[] | string;
  }

  export class Symbol3D extends Symbol {
    /**
     * Symbol3D is the base class for all 3D symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html)
     */

    constructor(properties?: Symbol3DProperties);

    /**
     * The origin of the style from which the symbol was originally referenced.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html#styleOrigin)
     */
    styleOrigin: StyleOrigin;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html#symbolLayers)
     */
    symbolLayers: Collection<Symbol3DLayer>;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html#type)
     */
    readonly type: "point-3d" | "line-3d" | "polygon-3d" | "mesh-3d" | "label-3d";

    static fromJSON(json: any): Symbol3D;
  }

  interface Symbol3DProperties extends SymbolProperties {
    /**
     * The origin of the style from which the symbol was originally referenced.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html#styleOrigin)
     */
    styleOrigin?: StyleOriginProperties;
    /**
     * A Collection of [Symbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html) objects used to visualize the graphic or feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html#symbolLayers)
     */
    symbolLayers?: CollectionProperties<Symbol3DLayerProperties>;
  }

  export interface Symbol3DLayer extends Accessor, JSONSupport {}

  export class Symbol3DLayer {
    /**
     * Symbol layers are used to define the visualization of [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html), [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html), and mesh geometries rendered with [3D symbols](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3D.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html)
     */

    constructor(properties?: Symbol3DLayerProperties);

    /**
     * The 3D symbol layer type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol3DLayer.html#type)
     */
    readonly type: "icon" | "object" | "line" | "path" | "fill" | "water" | "extrude" | "text";

    static fromJSON(json: any): Symbol3DLayer;
  }

  interface Symbol3DLayerProperties {}

  export class TextSymbol extends Symbol {
    /**
     * Text symbols are used to define the graphic for displaying labels on a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [Sublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Sublayer.html), and [StreamLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html) in a 2D [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html)
     */

    constructor(properties?: TextSymbolProperties);

    /**
     * The angle of the text.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#angle)
     */
    angle: number;
    /**
     * The background color of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#backgroundColor)
     */
    backgroundColor: Color;
    /**
     * The border color of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#borderLineColor)
     */
    borderLineColor: Color;
    /**
     * The border size or width of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#borderLineSize)
     */
    borderLineSize: number;
    /**
     * The [Font](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html) used to style the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#font)
     */
    font: Font;
    /**
     * The color of the text symbol's halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#haloColor)
     */
    haloColor: Color;
    /**
     * The size in points of the text symbol's halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#haloSize)
     */
    haloSize: number;
    /**
     * Adjusts the horizontal alignment of the text in multi-lines.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#horizontalAlignment)
     */
    horizontalAlignment: "left" | "right" | "center";
    /**
     * Determines whether to adjust the spacing between characters in the text string.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#kerning)
     */
    kerning: boolean;
    /**
     * The height of the space between each line of text.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#lineHeight)
     */
    lineHeight: number;
    /**
     * The maximum length in points for each line of text.
     *
     * @default 192
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#lineWidth)
     */
    lineWidth: number;
    /**
     * Determines whether every character in the text string is rotated.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#rotated)
     */
    rotated: boolean;
    /**
     * The text string to display in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#text)
     */
    text: string;

    readonly type: "text";
    /**
     * Adjusts the vertical alignment of the text.
     *
     * @default "baseline"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#verticalAlignment)
     */
    verticalAlignment: "baseline" | "top" | "middle" | "bottom";
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#xoffset)
     */
    xoffset: number;
    /**
     * The offset on the y-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#yoffset)
     */
    yoffset: number;

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#clone)
     */
    clone(): TextSymbol;

    static fromJSON(json: any): TextSymbol;
  }

  interface TextSymbolProperties extends SymbolProperties {
    /**
     * The angle of the text.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#angle)
     */
    angle?: number;
    /**
     * The background color of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#backgroundColor)
     */
    backgroundColor?: Color | number[] | string;
    /**
     * The border color of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#borderLineColor)
     */
    borderLineColor?: Color | number[] | string;
    /**
     * The border size or width of the label's bounding box.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#borderLineSize)
     */
    borderLineSize?: number;
    /**
     * The [Font](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Font.html) used to style the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#font)
     */
    font?: FontProperties;
    /**
     * The color of the text symbol's halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#haloColor)
     */
    haloColor?: Color | number[] | string;
    /**
     * The size in points of the text symbol's halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#haloSize)
     */
    haloSize?: number | string;
    /**
     * Adjusts the horizontal alignment of the text in multi-lines.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#horizontalAlignment)
     */
    horizontalAlignment?: "left" | "right" | "center";
    /**
     * Determines whether to adjust the spacing between characters in the text string.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#kerning)
     */
    kerning?: boolean;
    /**
     * The height of the space between each line of text.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#lineHeight)
     */
    lineHeight?: number;
    /**
     * The maximum length in points for each line of text.
     *
     * @default 192
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#lineWidth)
     */
    lineWidth?: number | string;
    /**
     * Determines whether every character in the text string is rotated.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#rotated)
     */
    rotated?: boolean;
    /**
     * The text string to display in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#text)
     */
    text?: string;
    /**
     * Adjusts the vertical alignment of the text.
     *
     * @default "baseline"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#verticalAlignment)
     */
    verticalAlignment?: "baseline" | "top" | "middle" | "bottom";
    /**
     * The offset on the x-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#xoffset)
     */
    xoffset?: number | string;
    /**
     * The offset on the y-axis in points.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html#yoffset)
     */
    yoffset?: number | string;
  }

  export class TextSymbol3DLayer extends Symbol3DLayer {
    /**
     * TextSymbol3DLayer is used to draw text labels for features of any geometry type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html)
     */

    constructor(properties?: TextSymbol3DLayerProperties);

    /**
     * The background of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#background)
     */
    background: TextSymbol3DLayerBackground;
    /**
     * The font of the text label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#font)
     */
    font: Font;
    /**
     * The halo surrounding the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    halo: TextSymbol3DLayerHalo;
    /**
     * Adjusts the horizontal alignment of the text in multi-lines.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#horizontalAlignment)
     */
    horizontalAlignment: "left" | "right" | "center";
    /**
     * The height of the space between each line of text.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#lineHeight)
     */
    lineHeight: number;
    /**
     * The material used to color the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#material)
     */
    material: TextSymbol3DLayerMaterial;
    /**
     * Size of the text label in points.
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#size)
     */
    size: number;
    /**
     * The text to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#text)
     */
    text: string;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#type)
     */
    readonly type: "text";
    /**
     * Adjusts the vertical alignment of the text.
     *
     * @default "baseline"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#verticalAlignment)
     */
    verticalAlignment: "baseline" | "top" | "middle" | "bottom";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#clone)
     */
    clone(): TextSymbol3DLayer;

    static fromJSON(json: any): TextSymbol3DLayer;
  }

  interface TextSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * The background of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#background)
     */
    background?: TextSymbol3DLayerBackgroundProperties;
    /**
     * The font of the text label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#font)
     */
    font?: FontProperties;
    /**
     * The halo surrounding the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    halo?: TextSymbol3DLayerHaloProperties;
    /**
     * Adjusts the horizontal alignment of the text in multi-lines.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#horizontalAlignment)
     */
    horizontalAlignment?: "left" | "right" | "center";
    /**
     * The height of the space between each line of text.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#lineHeight)
     */
    lineHeight?: number;
    /**
     * The material used to color the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#material)
     */
    material?: TextSymbol3DLayerMaterialProperties;
    /**
     * Size of the text label in points.
     *
     * @default 9
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#size)
     */
    size?: number | string;
    /**
     * The text to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#text)
     */
    text?: string;
    /**
     * Adjusts the vertical alignment of the text.
     *
     * @default "baseline"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#verticalAlignment)
     */
    verticalAlignment?: "baseline" | "top" | "middle" | "bottom";
  }

  export interface TextSymbol3DLayerBackgroundProperties {
    /**
     * The color of the background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#background)
     */
    color?: Color | number[] | string;
  }

  export interface TextSymbol3DLayerBackground extends AnonymousAccessor {
    /**
     * The color of the background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#background)
     */
    color?: Color;
  }

  export interface TextSymbol3DLayerHaloProperties {
    /**
     * The color of the halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    color?: Color | number[] | string;
    /**
     * The size of the halo in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    size?: number | string;
  }

  export interface TextSymbol3DLayerHalo extends AnonymousAccessor {
    /**
     * The color of the halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    color?: Color;
    /**
     * The size of the halo in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#halo)
     */
    size?: number;
  }

  export interface TextSymbol3DLayerMaterialProperties {
    /**
     * The color of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#material)
     */
    color?: Color | number[] | string;
  }

  export interface TextSymbol3DLayerMaterial extends AnonymousAccessor {
    /**
     * The color of the text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol3DLayer.html#material)
     */
    color?: Color;
  }

  export class WaterSymbol3DLayer extends Symbol3DLayer {
    /**
     * WaterSymbol3DLayer is used to render [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometries as realistic, animated water surfaces, therefore it can only be used inside a [PolygonSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html)
     */

    constructor(properties?: WaterSymbol3DLayerProperties);

    /**
     * The dominant color used to shade the water.
     *
     * @default "#0077be"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#color)
     */
    color: Color;
    /**
     * The symbol type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#type)
     */
    readonly type: "water";
    /**
     * Indicates the size of the waterbody which is represented by the symbol layer.
     *
     * @default "medium"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waterbodySize)
     */
    waterbodySize: "small" | "medium" | "large";
    /**
     * Indicates the direction in which the waves travel.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waveDirection)
     */
    waveDirection: number;
    /**
     * Indicates the shape and intensity of the waves.
     *
     * @default "moderate"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waveStrength)
     */
    waveStrength: "calm" | "rippled" | "slight" | "moderate";

    /**
     * Creates a deep clone of the symbol layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#clone)
     */
    clone(): WaterSymbol3DLayer;

    static fromJSON(json: any): WaterSymbol3DLayer;
  }

  interface WaterSymbol3DLayerProperties extends Symbol3DLayerProperties {
    /**
     * The dominant color used to shade the water.
     *
     * @default "#0077be"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#color)
     */
    color?: Color | number[] | string;
    /**
     * Indicates the size of the waterbody which is represented by the symbol layer.
     *
     * @default "medium"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waterbodySize)
     */
    waterbodySize?: "small" | "medium" | "large";
    /**
     * Indicates the direction in which the waves travel.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waveDirection)
     */
    waveDirection?: number;
    /**
     * Indicates the shape and intensity of the waves.
     *
     * @default "moderate"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WaterSymbol3DLayer.html#waveStrength)
     */
    waveStrength?: "calm" | "rippled" | "slight" | "moderate";
  }

  export class WebStyleSymbol extends Symbol {
    /**
     * WebStyleSymbol is a class used to conveniently create vector 2D symbols and realistic and thematic 3D symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html)
     */

    constructor(properties?: WebStyleSymbolProperties);

    /**
     * The name of the symbol within the web style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#name)
     */
    name: string;
    /**
     * The portal that contains the web style this symbol refers to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#portal)
     */
    portal: Portal;
    /**
     * A registered web style name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#styleName)
     */
    styleName: string;
    /**
     * URL that points to the web style definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#styleUrl)
     */
    styleUrl: string;

    readonly type: "web-style";

    /**
     * Creates a deep clone of the symbol.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#clone)
     */
    clone(): WebStyleSymbol;
    /**
     * Returns the [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) instance the WebStyleSymbol refers to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#fetchCIMSymbol)
     */
    fetchCIMSymbol(options?: WebStyleSymbolFetchCIMSymbolOptions): Promise<CIMSymbol>;
    /**
     * Downloads and returns the [PointSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html) instance the WebStyleSymbol refers to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#fetchSymbol)
     */
    fetchSymbol(options?: WebStyleSymbolFetchSymbolOptions): Promise<PointSymbol3D>;

    static fromJSON(json: any): WebStyleSymbol;
  }

  interface WebStyleSymbolProperties extends SymbolProperties {
    /**
     * The name of the symbol within the web style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#name)
     */
    name?: string;
    /**
     * The portal that contains the web style this symbol refers to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#portal)
     */
    portal?: PortalProperties;
    /**
     * A registered web style name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#styleName)
     */
    styleName?: string;
    /**
     * URL that points to the web style definition.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#styleUrl)
     */
    styleUrl?: string;
  }

  export interface WebStyleSymbolFetchCIMSymbolOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#fetchCIMSymbol)
     */
    signal?: AbortSignal;
  }

  export interface WebStyleSymbolFetchSymbolOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html#fetchSymbol)
     */
    signal?: AbortSignal;
  }

  /**
   * CIMSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#CIMSymbol)
   */
  export type symbolsCIMSymbol = CIMSymbol;

  /**
   * ExtrudeSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#ExtrudeSymbol3DLayer)
   */
  export type symbolsExtrudeSymbol3DLayer = ExtrudeSymbol3DLayer;

  /**
   * FillSymbol types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#FillSymbol)
   */
  export type symbolsFillSymbol = PictureFillSymbol | SimpleFillSymbol;

  /**
   * FillSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#FillSymbol3DLayer)
   */
  export type symbolsFillSymbol3DLayer = FillSymbol3DLayer;

  /**
   * Font.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Font)
   */
  export type symbolsFont = Font;

  /**
   * IconSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#IconSymbol3DLayer)
   */
  export type symbolsIconSymbol3DLayer = IconSymbol3DLayer;

  /**
   * LabelSymbol3D.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LabelSymbol3D)
   */
  export type symbolsLabelSymbol3D = LabelSymbol3D;

  /**
   * LineSymbol3D.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LineSymbol3D)
   */
  export type symbolsLineSymbol3D = LineSymbol3D;

  /**
   * LineSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#LineSymbol3DLayer)
   */
  export type symbolsLineSymbol3DLayer = LineSymbol3DLayer;

  /**
   * MarkerSymbol types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#MarkerSymbol)
   */
  export type symbolsMarkerSymbol = PictureMarkerSymbol | SimpleMarkerSymbol;

  /**
   * MeshSymbol3D.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#MeshSymbol3D)
   */
  export type symbolsMeshSymbol3D = MeshSymbol3D;

  /**
   * ObjectSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#ObjectSymbol3DLayer)
   */
  export type symbolsObjectSymbol3DLayer = ObjectSymbol3DLayer;

  /**
   * PathSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PathSymbol3DLayer)
   */
  export type symbolsPathSymbol3DLayer = PathSymbol3DLayer;

  /**
   * PictureFillSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PictureFillSymbol)
   */
  export type symbolsPictureFillSymbol = PictureFillSymbol;

  /**
   * PictureMarkerSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PictureMarkerSymbol)
   */
  export type symbolsPictureMarkerSymbol = PictureMarkerSymbol;

  /**
   * PointSymbol3D.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PointSymbol3D)
   */
  export type symbolsPointSymbol3D = PointSymbol3D;

  /**
   * PolygonSymbol3D.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#PolygonSymbol3D)
   */
  export type symbolsPolygonSymbol3D = PolygonSymbol3D;

  /**
   * SimpleFillSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleFillSymbol)
   */
  export type symbolsSimpleFillSymbol = SimpleFillSymbol;

  /**
   * SimpleLineSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleLineSymbol)
   */
  export type symbolsSimpleLineSymbol = SimpleLineSymbol;

  /**
   * SimpleMarkerSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#SimpleMarkerSymbol)
   */
  export type symbolsSimpleMarkerSymbol = SimpleMarkerSymbol;

  /**
   * Symbol types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol)
   */
  export type symbolsSymbol = Symbol2D3D | WebStyleSymbol;

  /**
   * Symbol2D types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol2D)
   */
  export type Symbol2D =
    | PictureFillSymbol
    | PictureMarkerSymbol
    | SimpleFillSymbol
    | SimpleLineSymbol
    | SimpleMarkerSymbol
    | TextSymbol
    | CIMSymbol;

  /**
   * Symbol2D3D types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol2D3D)
   */
  export type Symbol2D3D = Symbol2D | symbolsSymbol3D;

  /**
   * Symbol3D types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol3D)
   */
  export type symbolsSymbol3D = LabelSymbol3D | LineSymbol3D | MeshSymbol3D | PointSymbol3D | PolygonSymbol3D;

  /**
   * Symbol3DLayer types.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#Symbol3DLayer)
   */
  export type symbolsSymbol3DLayer =
    | ExtrudeSymbol3DLayer
    | FillSymbol3DLayer
    | WaterSymbol3DLayer
    | IconSymbol3DLayer
    | LineSymbol3DLayer
    | ObjectSymbol3DLayer
    | PathSymbol3DLayer
    | TextSymbol3DLayer
    | WaterSymbol3DLayer;

  /**
   * TextSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#TextSymbol)
   */
  export type symbolsTextSymbol = TextSymbol;

  /**
   * TextSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#TextSymbol3DLayer)
   */
  export type symbolsTextSymbol3DLayer = TextSymbol3DLayer;

  /**
   * WaterSymbol3DLayer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#WaterSymbol3DLayer)
   */
  export type symbolsWaterSymbol3DLayer = WaterSymbol3DLayer;

  /**
   * WebStyleSymbol.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols.html#WebStyleSymbol)
   */
  export type symbolsWebStyleSymbol = WebStyleSymbol;

  export interface AttributeTableTemplate extends Accessor, JSONSupport {}

  export class AttributeTableTemplate {
    /**
     * An `AttributeTableTemplate` manages the configuration of how the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget displays its columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html)
     */

    constructor(properties?: AttributeTableTemplateProperties);

    /**
     * An array of attribute table element objects that represent an ordered list of table elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#elements)
     */
    elements: (
      | AttributeTableAttachmentElement
      | AttributeTableFieldElement
      | AttributeTableGroupElement
      | AttributeTableRelationshipElement
    )[];
    /**
     * An Array of [FieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#FieldOrder) objects indicating the records' sort order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#orderByFields)
     */
    orderByFields: FieldOrder[];

    /**
     * Creates a deep clone of the AttributeTableTemplate class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#clone)
     */
    clone(): AttributeTableTemplate;

    static fromJSON(json: any): AttributeTableTemplate;
  }

  interface AttributeTableTemplateProperties {
    /**
     * An array of attribute table element objects that represent an ordered list of table elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#elements)
     */
    elements?: (
      | (AttributeTableAttachmentElementProperties & { type: "attachment" })
      | (AttributeTableFieldElementProperties & { type: "field" })
      | (AttributeTableGroupElementProperties & { type: "group" })
      | (AttributeTableRelationshipElementProperties & { type: "relationship" })
    )[];
    /**
     * An Array of [FieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#FieldOrder) objects indicating the records' sort order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#orderByFields)
     */
    orderByFields?: FieldOrder[];
  }

  /**
   * This object sets a field and sort order for how records display within the table.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#FieldOrder)
   */
  export interface FieldOrder {
    /**
     * Name of a field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#FieldOrder)
     */
    field?: string;
    /**
     * Indicates whether features are sorted in ascending or descending order of the field values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#FieldOrder)
     */
    order?: "asc" | "desc";
  }

  export class AttributeTableAttachmentElement extends AttributeTableElement {
    /**
     * An `AttributeTableAttachmentElement` defines how attachments display within a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html)
     */

    constructor(properties?: AttributeTableAttachmentElementProperties);

    /**
     * The display type of the attachment element in the table.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html#displayType)
     */
    displayType: "auto";
    /**
     * The type of the attribute table [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html#type)
     */
    readonly type: "attachment";

    /**
     * Creates a deep clone of the AttributeTableAttachmentElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html#clone)
     */
    clone(): AttributeTableAttachmentElement;

    static fromJSON(json: any): AttributeTableAttachmentElement;
  }

  interface AttributeTableAttachmentElementProperties extends AttributeTableElementProperties {
    /**
     * The display type of the attachment element in the table.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html#displayType)
     */
    displayType?: "auto";
  }

  export interface AttributeTableElement extends Accessor, JSONSupport {}

  export class AttributeTableElement {
    /**
     * Attribute table elements define what should display within the [AttributeTableTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html)
     */

    constructor(properties?: AttributeTableElementProperties);

    /**
     * The table element's description which provides the purpose behind it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html#description)
     */
    description: string;
    /**
     * A string value containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html#label)
     */
    label: string;
    /**
     * The type of table element to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html#type)
     */
    readonly type: "attachment" | "field" | "group" | "relationship";

    static fromJSON(json: any): AttributeTableElement;
  }

  interface AttributeTableElementProperties {
    /**
     * The table element's description which provides the purpose behind it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html#description)
     */
    description?: string;
    /**
     * A string value containing the field alias.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html#label)
     */
    label?: string;
  }

  export class AttributeTableFieldElement extends AttributeTableElement {
    /**
     * An `AttributeTableFieldElement` table element defines how a layer's [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) displays in the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html)
     */

    constructor(properties?: AttributeTableFieldElementProperties);

    /**
     * The field name as defined by the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html#fieldName)
     */
    fieldName: string;
    /**
     * Indicates the type of attribute table [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html#type)
     */
    readonly type: "field";

    /**
     * Creates a deep clone of the AttributeTableFieldElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html#clone)
     */
    clone(): AttributeTableFieldElement;

    static fromJSON(json: any): AttributeTableFieldElement;
  }

  interface AttributeTableFieldElementProperties extends AttributeTableElementProperties {
    /**
     * The field name as defined by the feature layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html#fieldName)
     */
    fieldName?: string;
  }

  export class AttributeTableGroupElement extends AttributeTableElement {
    /**
     * An `AttributeTableGroupElement` table element defines a container that holds a set of [table elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-AttributeTableTemplate.html#elements).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableGroupElement.html)
     */

    constructor(properties?: AttributeTableGroupElementProperties);

    /**
     * An array of [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html), [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html), and [attachment](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html) elements to display as grouped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableGroupElement.html#elements)
     */
    elements: (AttributeTableFieldElement | AttributeTableRelationshipElement | AttributeTableAttachmentElement)[];
    /**
     * The type of the attribute table [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableGroupElement.html#type)
     */
    readonly type: "group";

    /**
     * Creates a deep clone of the AttributeTableGroupElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableGroupElement.html#clone)
     */
    clone(): AttributeTableGroupElement;

    static fromJSON(json: any): AttributeTableGroupElement;
  }

  interface AttributeTableGroupElementProperties extends AttributeTableElementProperties {
    /**
     * An array of [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableFieldElement.html), [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html), and [attachment](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableAttachmentElement.html) elements to display as grouped.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableGroupElement.html#elements)
     */
    elements?: (
      | (AttributeTableFieldElementProperties & { type: "field" })
      | (AttributeTableRelationshipElementProperties & { type: "relationship" })
      | (AttributeTableAttachmentElementProperties & { type: "attachment" })
    )[];
  }

  export class AttributeTableRelationshipElement extends AttributeTableElement {
    /**
     * An `AttributeTableRelationshipElement` element defines how a relationship between [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) and [tables](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#tables) participates and displays within a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html)
     */

    constructor(properties?: AttributeTableRelationshipElementProperties);

    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html#relationshipId)
     */
    relationshipId: number;
    /**
     * Indicates the type of attribute table [element](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html#type)
     */
    readonly type: "relationship";

    /**
     * Creates a deep clone of the AttributeTableRelationshipElement class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html#clone)
     */
    clone(): AttributeTableRelationshipElement;

    static fromJSON(json: any): AttributeTableRelationshipElement;
  }

  interface AttributeTableRelationshipElementProperties extends AttributeTableElementProperties {
    /**
     * The numeric id value for the defined relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableRelationshipElement.html#relationshipId)
     */
    relationshipId?: number;
  }

  /**
   * A convenience module for importing [AttributeTableElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-elements-AttributeTableElement.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-tables-support-elements.html)
   */
  interface supportElements {}

  export const supportElements: supportElements;

  export interface TimeExtent extends Accessor, JSONSupport {}

  export class TimeExtent {
    /**
     * A period of time with a definitive [start](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#start) and [end](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#end) date.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html)
     */

    constructor(properties?: TimeExtentProperties);

    /**
     * The end time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#end)
     */
    end: Date;
    /**
     * The start time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#start)
     */
    start: Date;

    /**
     * Creates a deep clone of TimeExtent object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#clone)
     */
    clone(): TimeExtent;
    /**
     * Expands the TimeExtent so that the start and end dates are rounded down and up, respectively, to the parsed time unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#expandTo)
     */
    expandTo(
      unit:
        | "milliseconds"
        | "seconds"
        | "minutes"
        | "hours"
        | "days"
        | "weeks"
        | "months"
        | "years"
        | "decades"
        | "centuries",
      timeZone: "system" | "unknown" | string,
    ): TimeExtent;

    static fromJSON(json: any): TimeExtent;
  }

  interface TimeExtentProperties {
    /**
     * The end time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#end)
     */
    end?: DateProperties;
    /**
     * The start time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html#start)
     */
    start?: DateProperties;
  }

  export interface TimeInterval extends Accessor, JSONSupport, Clonable {}

  export class TimeInterval {
    /**
     * TimeInterval is a class that describes a length of time in one of ten temporal units such as seconds, days, or years.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html)
     */

    constructor(properties?: TimeIntervalProperties);

    /**
     * Temporal units.
     *
     * @default "milliseconds"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html#unit)
     */
    unit:
      | "milliseconds"
      | "seconds"
      | "minutes"
      | "hours"
      | "days"
      | "weeks"
      | "months"
      | "years"
      | "decades"
      | "centuries";
    /**
     * The numerical value of the time extent.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html#value)
     */
    value: number;

    static fromJSON(json: any): TimeInterval;
  }

  interface TimeIntervalProperties {
    /**
     * Temporal units.
     *
     * @default "milliseconds"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html#unit)
     */
    unit?:
      | "milliseconds"
      | "seconds"
      | "minutes"
      | "hours"
      | "days"
      | "weeks"
      | "months"
      | "years"
      | "decades"
      | "centuries";
    /**
     * The numerical value of the time extent.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html#value)
     */
    value?: number;
  }

  export interface esriTimeExtent extends Accessor, JSONSupport {}

  export class esriTimeExtent {
    constructor(properties?: esriTimeExtentProperties);

    /**
     * The end time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#end)
     */
    end: Date;
    /**
     * The start time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#start)
     */
    start: Date;

    /**
     * Creates a deep clone of TimeExtent object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#clone)
     */
    clone(): TimeExtent;
    /**
     * Expands the TimeExtent so that the start and end dates are rounded down and up, respectively, to the parsed time unit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#expandTo)
     */
    expandTo(
      unit:
        | "milliseconds"
        | "seconds"
        | "minutes"
        | "hours"
        | "days"
        | "weeks"
        | "months"
        | "years"
        | "decades"
        | "centuries",
      timeZone: "system" | "unknown" | string,
    ): TimeExtent;

    /**
     * Returns the time extent resulting from the intersection of a given time extent and parsed time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#intersection)
     */
    static intersection(timeExtent: TimeExtent): TimeExtent;
    /**
     * Returns the time extent resulting from the union of the current time extent and a given time extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#union)
     */
    static union(timeExtent: TimeExtent): TimeExtent;
    static fromJSON(json: any): esriTimeExtent;
  }

  interface esriTimeExtentProperties {
    /**
     * The end time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#end)
     */
    end?: DateProperties;
    /**
     * The start time of the time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeExtent.html#start)
     */
    start?: DateProperties;
  }

  export interface esriTimeInterval extends Accessor, JSONSupport, Clonable {}

  export class esriTimeInterval {
    constructor(properties?: esriTimeIntervalProperties);

    /**
     * Temporal units.
     *
     * @default "milliseconds"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeInterval.html#unit)
     */
    unit:
      | "milliseconds"
      | "seconds"
      | "minutes"
      | "hours"
      | "days"
      | "weeks"
      | "months"
      | "years"
      | "decades"
      | "centuries";
    /**
     * The numerical value of the time extent.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeInterval.html#value)
     */
    value: number;

    static fromJSON(json: any): esriTimeInterval;
  }

  interface esriTimeIntervalProperties {
    /**
     * Temporal units.
     *
     * @default "milliseconds"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeInterval.html#unit)
     */
    unit?:
      | "milliseconds"
      | "seconds"
      | "minutes"
      | "hours"
      | "days"
      | "weeks"
      | "months"
      | "years"
      | "decades"
      | "centuries";
    /**
     * The numerical value of the time extent.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-TimeInterval.html#value)
     */
    value?: number;
  }

  /**
   * Provides utility methods for creating [VersioningStates](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-utils.html)
   */
  interface versionManagementUtils {
    /**
     * Used to generate an instance of [VersioningState](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-utils.html#createVersioningStates)
     */
    createVersioningStates(
      input: WebMap | VersionAdapter,
      usePersistentReadSessions: boolean,
    ): Promise<Collection<VersioningState>>;
    /**
     * Used to get an instance of [VersioningState](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html) from a [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-utils.html#getVersioningStates)
     */
    getVersioningStates(view: View, usePersistentReadSessions: boolean): Promise<Collection<VersioningState>>;
  }

  export const versionManagementUtils: versionManagementUtils;

  /**
   * Provides utility methods for creating [VersionAdapter](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-versionAdapters-utils.html)
   */
  interface versionAdaptersUtils {
    /**
     * Used to generate an instance of VersionsAdapter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-versionAdapters-utils.html#createVersionAdapter)
     */
    createVersionAdapter(input: Network | FeatureLayer | SubtypeGroupLayer): VersionAdapter;
    /**
     * Used to generate an instances of VersionsAdapter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-versionAdapters-utils.html#createVersionAdapters)
     */
    createVersionAdapters(input: Network[] | FeatureLayer[] | SubtypeGroupLayer[]): VersionAdapter[];
  }

  export const versionAdaptersUtils: versionAdaptersUtils;

  export class VersioningState {
    /**
     * The current version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#currentVersion)
     */
    currentVersion: VersionManagementServiceVersionIdentifier | Date;
    /**
     * Contains information on the current version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#currentVersionInfo)
     */
    currentVersionInfo: VersionInfo;
    /**
     * Contains the default version's name and guid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#defaultVersionIdentifier)
     */
    readonly defaultVersionIdentifier: VersionManagementServiceVersionIdentifier;
    /**
     * The absolute URL of the REST endpoint for the feature service housing the version management service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#featureServiceUrl)
     */
    featureServiceUrl: string;
    /**
     * If true, current version is the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#isDefault)
     */
    readonly isDefault: boolean;
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * The state of the current version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#state)
     */
    state: "lock-none" | "lock-read" | "lock-write";
    /**
     * The absolute URL of the REST endpoint for the version management service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#url)
     */
    url: string;
    /**
     * If the set to `true`, the current version will have a read lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#usePersistentReadSessions)
     */
    usePersistentReadSessions: boolean;
    /**
     * Contains a collection of all versionable in the versioning state class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#versionableItems)
     */
    versionableItems: Collection<VersionAdapter>;
    /**
     * Contains list of all available versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#versionInfos)
     */
    versionInfos: VersionInfo[];
    /**
     * Contains metadata about the version management service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#versionManagementService)
     */
    versionManagementService: VersionManagementService;

    /**
     * The alter operation allows you to change the geodatabase version's name, description, owner, and access permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#alterVersion)
     */
    alterVersion(
      versionIdentifier: VersionManagementServiceVersionIdentifier,
      props: VersioningStateAlterVersionProps,
    ): Promise<boolean>;
    /**
     * Method used to change a layer's version/moment or network's version/moment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#changeVersion)
     */
    changeVersion(
      toVersion: Date | VersionManagementServiceVersionIdentifier,
    ): Promise<globalThis.Map<VersionAdapter, ServiceResult>>;
    /**
     * Deletes a version given the following parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#deleteVersion)
     */
    deleteVersion(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<boolean>;
    /**
     * Method used to get extended information about a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#getVersionInfoExtended)
     */
    getVersionInfoExtended(): Promise<VersionInfoExtendedJSON>;
    /**
     * Method used to get a list of versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#getVersionInfos)
     */
    getVersionInfos(refresh: boolean): Promise<VersionInfo[]>;
    /**
     * Triggers the loading of the version management service instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#load)
     */
    load(): Promise<any>;
    /**
     * This method redos the last edit made while in an edit session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#redo)
     */
    redo(): Promise<ServiceResult>;
    /**
     * Using the specified session ID an exclusive lock is obtained for the session on the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#startEditing)
     */
    startEditing(): Promise<ServiceResult>;
    /**
     * Using the specified session id, the exclusive lock for the version is downgraded to a shared lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#stopEditing)
     */
    stopEditing(saveEdits: boolean): Promise<ServiceResult>;
    /**
     * This method undos the last edit made while in an edit session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#undo)
     */
    undo(): Promise<ServiceResult>;

    constructor(properties?: any);
  }

  /**
   * This contains basic information about a given version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
   */
  export interface VersionInfo {
    /**
     * The identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    versionIdentifier: VersionInfoVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    description?: string;
    /**
     * Access type accepts values of private, public, hidden, and protected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    access?: string;
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    creationDate?: number;
    /**
     * The date when the version was last modified or changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    modifiedDate?: number;
    /**
     * The date when the version was last reconciled with the parent version or the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    reconcileDate?: number;
    /**
     * The date when the version was last evaluated for conflicts or discrepancies during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    evaluationDate?: number;
    /**
     * The date of the most recent ancestor version, if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    previousAncestorDate?: number;
    /**
     * The date of the common ancestor version, which is the shared ancestor between two or more versions during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    commonAncestorDate?: number;
  }

  /**
   * This contains extended information about a given version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
   */
  export interface VersionInfoExtendedJSON {
    /**
     * The identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    versionIdentifier: VersionInfoExtendedJSONVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    description?: string;
    /**
     * The access type permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    access?: "hidden" | "private" | "protected" | "public";
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    creationDate?: number;
    /**
     * The date when the version was last modified or changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    modifiedDate?: number;
    /**
     * The date when the version was last reconciled with the parent version or the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    reconcileDate?: number;
    /**
     * The date when the version was last evaluated for conflicts or discrepancies during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    evaluationDate?: number;
    /**
     * The date of the most recent ancestor version, if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    previousAncestorDate?: number;
    /**
     * The date of the common ancestor version, which is the shared ancestor between two or more versions during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    commonAncestorDate?: number;
    /**
     * Specifies if the version is currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    isBeingEdited?: boolean;
    /**
     * Specifies if the version is currently being read.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    isBeingRead?: boolean;
    /**
     * Specifies if the version is has conflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    hasConflicts?: boolean;
    /**
     * Specifies if the version has UninspectedConflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    hasUninspectedConflicts?: boolean;
    /**
     * Specifies if the version currently has a lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    isLocked?: boolean;
    /**
     * Specifies which user is the current lock owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    lockOwner?: string;
    /**
     * Specifies the date at which the version was locked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    lockDate?: number;
  }

  export interface VersioningStateAlterVersionProps {
    /**
     * The new owner name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#alterVersion)
     */
    ownerName?: string;
    /**
     * The new name for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#alterVersion)
     */
    versionName?: string;
    /**
     * The new description for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#alterVersion)
     */
    description?: string;
    /**
     * The new access permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#alterVersion)
     */
    access?: "hidden" | "private" | "protected" | "public";
  }

  export interface VersionInfoVersionIdentifier {
    /**
     * The name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    name: string;
    /**
     * The guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfo)
     */
    guid: string;
  }

  export interface VersionInfoExtendedJSONVersionIdentifier {
    /**
     * The name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    name: string;
    /**
     * The guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersioningState.html#VersionInfoExtendedJSON)
     */
    guid: string;
  }

  export class VersionManagementService {
    /**
     * Describes the version management service's supported capabilities.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#capabilities)
     */
    capabilities: VersionManagementServiceCapabilities;
    /**
     * Contains the value for the default version's name and GUID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#defaultVersionIdentifier)
     */
    defaultVersionIdentifier: VersionManagementServiceVersionIdentifier;
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * The name of the version management service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#name)
     */
    name: string;
    /**
     * The absolute URL of the REST endpoint for the version management service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#url)
     */
    url: string;

    /**
     * The alter operation allows you to change the geodatabase version's name, description, owner, and access permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#alterVersion)
     */
    alterVersion(
      versionIdentifier: VersionManagementServiceVersionIdentifier,
      props: VersionManagementServiceAlterVersionProps,
    ): Promise<boolean>;
    /**
     * `canRedo` may be used to verify that a [redo](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#redo) operation is possible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#canRedo)
     */
    canRedo(versionIdentifier: VersionManagementServiceVersionIdentifier): boolean;
    /**
     * `canUndo` may be used to verify that an [undo](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#undo) operation is possible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#canUndo)
     */
    canUndo(versionIdentifier: VersionManagementServiceVersionIdentifier): boolean;
    /**
     * Method used to change a layer's version/moment or network's version/moment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#changeVersion)
     */
    changeVersion(
      input: WebMap | Collection<FeatureLayer> | FeatureLayer[] | Network,
      fromVersion: Date | VersionManagementServiceVersionIdentifier,
      toVersion: Date | VersionManagementServiceVersionIdentifier,
    ): Promise<boolean>;
    /**
     * Method used to change a layer's version/moment or network's version/moment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#changeVersionWithResult)
     */
    changeVersionWithResult(
      input: WebMap | Collection<VersionAdapter> | VersionManagementService[],
      fromVersion: Date | VersionManagementServiceVersionIdentifier,
      toVersion: Date | VersionManagementServiceVersionIdentifier,
    ): Promise<globalThis.Map<VersionAdapter, ServiceResult>>;
    /**
     * Creates a new version given the following parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#createVersion)
     */
    createVersion(
      props?: VersionManagementServiceCreateVersionProps,
    ): Promise<VersionManagementServiceVersionInfoExtendedJSON>;
    /**
     * Deletes a version given the following parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#deleteVersion)
     */
    deleteVersion(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<boolean>;
    /**
     * Returns the current client lock type on a given version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getLockType)
     */
    getLockType(versionIdentifier: VersionManagementServiceVersionIdentifier): string;
    /**
     * This method will return the [versionIdentifier](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#versionIdentifier) given the guid of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionIdentifierFromGuid)
     */
    getVersionIdentifierFromGuid(guid: string): VersionManagementServiceVersionIdentifier;
    /**
     * This method will return the [versionIdentifier](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#versionIdentifier) given the name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionIdentifierFromName)
     */
    getVersionIdentifierFromName(name: string): VersionManagementServiceVersionIdentifier;
    /**
     * Method used to get extended information about a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionInfoExtended)
     */
    getVersionInfoExtended(
      versionIdentifier: VersionManagementServiceVersionIdentifier,
    ): Promise<VersionManagementServiceVersionInfoExtendedJSON>;
    /**
     * Returns information about a version or versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionInfos)
     */
    getVersionInfos(props?: VersionManagementServiceGetVersionInfosProps): Promise<VersionInfoJSON[]>;
    /**
     * Triggers the loading of the version management service instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#load)
     */
    load(): Promise<any>;
    /**
     * The Post operation allows the client to post the changes in their version to the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#post)
     */
    post(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<PostResult>;
    /**
     * Use the reconcile operation to reconcile a branch version with the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#reconcile)
     */
    reconcile(
      versionIdentifier: VersionManagementServiceVersionIdentifier,
      props: VersionManagementServiceReconcileProps,
    ): Promise<ReconcileResult>;
    /**
     * This method redos the last edit made while in an edit session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#redo)
     */
    redo(versionIdentifier: VersionManagementServiceVersionIdentifier): void;
    /**
     * Using the specified session ID an exclusive lock is obtained for the session on the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#startEditing)
     */
    startEditing(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<boolean>;
    /**
     * Using the specified session ID an exclusive lock is obtained for the session on the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#startEditingWithResult)
     */
    startEditingWithResult(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<ServiceResult>;
    /**
     * Using the specified client generated session ID, a shared lock is obtained for the session if the version is not already being edited by another user/session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#startReading)
     */
    startReading(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<boolean>;
    /**
     * Using the specified client generated session ID, a shared lock is obtained for the session if the version is not already being edited by another user/session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#startReadingWithResult)
     */
    startReadingWithResult(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<ServiceResult>;
    /**
     * Using the specified session id, the exclusive lock for the version is downgraded to a shared lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#stopEditing)
     */
    stopEditing(versionIdentifier: VersionManagementServiceVersionIdentifier, saveEdits: boolean): Promise<boolean>;
    /**
     * Using the specified session id, the exclusive lock for the version is downgraded to a shared lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#stopEditingWithResult)
     */
    stopEditingWithResult(
      versionIdentifier: VersionManagementServiceVersionIdentifier,
      saveEdits: boolean,
    ): Promise<ServiceResult>;
    /**
     * The shared lock is released on the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#stopReading)
     */
    stopReading(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<boolean>;
    /**
     * The shared lock is released on the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#stopReadingWithResult)
     */
    stopReadingWithResult(versionIdentifier: VersionManagementServiceVersionIdentifier): Promise<ServiceResult>;
    /**
     * This method undos the last edit made while in an edit session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#undo)
     */
    undo(versionIdentifier: VersionManagementServiceVersionIdentifier): void;

    constructor(properties?: any);
  }

  /**
   * This contains information about the post result.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#PostResult)
   */
  export interface PostResult {
    /**
     * Returns data moments of the post.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#PostResult)
     */
    moment: Date;
    /**
     * Specifies if the post rest call was successful.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#PostResult)
     */
    success: boolean;
  }

  /**
   * This contains information about the reconcile result.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ReconcileResult)
   */
  export interface ReconcileResult {
    /**
     * Specifies if the reconcile returned with conflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ReconcileResult)
     */
    hasConflicts: boolean;
    /**
     * Returns data moment of the reconcile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ReconcileResult)
     */
    moment: Date;
    /**
     * Specifies if the updates from the version were posted.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ReconcileResult)
     */
    didPost: boolean;
    /**
     * Specifies if the reconcile rest call was successful.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ReconcileResult)
     */
    success: boolean;
  }

  /**
   * The response returned after performing an operation using the VersionManagementService.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ServiceResult)
   */
  export interface ServiceResult {
    /**
     * If true then the operation was successful.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ServiceResult)
     */
    success: boolean;
    /**
     * Contains the error returned by the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ServiceResult)
     */
    error: ServiceResultError;
  }

  /**
   * Contains information on versionable items such as type, current gdbVersion, current historic moment, and the featureService url of the versionable item.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter)
   */
  export interface VersionAdapter {
    /**
     * Contains items with the `gdbVersion` property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter)
     */
    versionableItem: FeatureLayer | Network | SubtypeGroupLayer;
    /**
     * The type of versionable item.
     *
     * [Read more...](global.html#type)
     */
    type: "feature-layer" | "subtype-group-layer" | "network";
    /**
     * The current gdbVersion of a versionable item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter)
     */
    gdbVersion?: string;
    /**
     * The current moment of a versionable item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter)
     */
    historicMoment?: Date;
    /**
     * A read only property that contains the [FeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html) url of a versionable item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionAdapter)
     */
    featureServiceUrl: string;
  }

  /**
   * Holds the name and guid of a version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
   */
  export interface VersionIdentifier {
    /**
     * The version name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
     */
    name: string;
    /**
     * The version GUID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
     */
    guid: string;
  }

  /**
   * Holds the name and guid of a version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
   */
  export interface VersionManagementServiceVersionIdentifier {
    /**
     * The version name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
     */
    name: string;
    /**
     * The version GUID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionIdentifier)
     */
    guid: string;
  }

  /**
   * This contains extended information about a given version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
   */
  export interface VersionManagementServiceVersionInfoExtendedJSON {
    /**
     * The identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    versionIdentifier: VersionManagementServiceVersionInfoExtendedJSONVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    description?: string;
    /**
     * The access type permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    access?: "hidden" | "private" | "protected" | "public";
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    creationDate?: number;
    /**
     * The date when the version was last modified or changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    modifiedDate?: number;
    /**
     * The date when the version was last reconciled with the parent version or the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    reconcileDate?: number;
    /**
     * The date when the version was last evaluated for conflicts or discrepancies during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    evaluationDate?: number;
    /**
     * The date of the most recent ancestor version, if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    previousAncestorDate?: number;
    /**
     * The date of the common ancestor version, which is the shared ancestor between two or more versions during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    commonAncestorDate?: number;
    /**
     * Specifies if the version is currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    isBeingEdited?: boolean;
    /**
     * Specifies if the version is currently being read.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    isBeingRead?: boolean;
    /**
     * Specifies if the version is has conflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    hasConflicts?: boolean;
    /**
     * Specifies if the version has UninspectedConflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    hasUninspectedConflicts?: boolean;
    /**
     * Specifies if the version currently has a lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    isLocked?: boolean;
    /**
     * Specifies which user is the current lock owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    lockOwner?: string;
    /**
     * Specifies the date at which the version was locked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    lockDate?: number;
  }

  /**
   * This contains basic information about a given version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
   */
  export interface VersionInfoJSON {
    /**
     * The identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    versionIdentifier: VersionInfoJSONVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    description?: string;
    /**
     * Access type accepts values of private, public, hidden, and protected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    access?: string;
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    creationDate?: number;
    /**
     * The date when the version was last modified or changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    modifiedDate?: number;
    /**
     * The date when the version was last reconciled with the parent version or the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    reconcileDate?: number;
    /**
     * The date when the version was last evaluated for conflicts or discrepancies during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    evaluationDate?: number;
    /**
     * The date of the most recent ancestor version, if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    previousAncestorDate?: number;
    /**
     * The date of the common ancestor version, which is the shared ancestor between two or more versions during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    commonAncestorDate?: number;
  }

  export interface VersionManagementServiceAlterVersionProps {
    /**
     * The new owner name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#alterVersion)
     */
    ownerName?: string;
    /**
     * The new name for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#alterVersion)
     */
    versionName?: string;
    /**
     * The new description for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#alterVersion)
     */
    description?: string;
    /**
     * The new access permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#alterVersion)
     */
    access?: "hidden" | "private" | "protected" | "public";
  }

  /**
   * Describes the version management service's supported capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
   */
  export interface VersionManagementServiceCapabilities {
    /**
     * Indicates whether the service supports conflict detection based on attribute values during editing operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsConflictDetectionByAttribute: boolean;
    /**
     * Indicates whether the service supports partial posting, allowing users to post only a subset of edits instead of the entire set.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsPartialPost: boolean;
    /**
     * Indicates whether the service supports retrieving differences in data from a specific moment in time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsDifferencesFromMoment: boolean;
    /**
     * Indicates whether the service supports retrieving differences in data between different layers or feature classes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsDifferencesWithLayers: boolean;
    /**
     * Indicates whether the service supports asynchronous reconcile operations, allowing users to reconcile conflicting edits asynchronously.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsAsyncReconcile: boolean;
    /**
     * Indicates whether the service supports asynchronous posting of edits, allowing users to post edits asynchronously.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsAsyncPost: boolean;
    /**
     * Indicates whether the service supports asynchronous retrieval of differences in data, allowing users to retrieve differences asynchronously.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionManagementServiceCapabilities)
     */
    supportsAsyncDifferences: boolean;
  }

  export interface VersionManagementServiceCreateVersionProps {
    /**
     * The name of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#createVersion)
     */
    versionName: string;
    /**
     * The description of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#createVersion)
     */
    description?: string;
    /**
     * The access type of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#createVersion)
     */
    access?: "hidden" | "private" | "protected" | "public";
  }

  export interface VersionManagementServiceGetVersionInfosProps {
    /**
     * Filter to query versions by owner name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionInfos)
     */
    ownerFilter: string;
    /**
     * Specifies whether to include hidden versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#getVersionInfos)
     */
    includeHidden: boolean;
  }

  export interface VersionManagementServiceReconcileProps {
    /**
     * Specifies whether the reconcile should be aborted if conflicts are found.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#reconcile)
     */
    abortIfConflicts: boolean;
    /**
     * Specifies the conditions required for conflicts to occur are defined by object (row) or attribute (column).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#reconcile)
     */
    conflictDetection: "by-attribute" | "by-object";
    /**
     * Specifies if post will occur after calling reconcile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#reconcile)
     */
    withPost: boolean;
  }

  export interface ServiceResultError {
    /**
     * Contains the error message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ServiceResult)
     */
    message: string;
    /**
     * Contains the error code number.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#ServiceResult)
     */
    extendedCode: number;
  }

  export interface VersionManagementServiceVersionInfoExtendedJSONVersionIdentifier {
    /**
     * The name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    name: string;
    /**
     * The guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoExtendedJSON)
     */
    guid: string;
  }

  export interface VersionInfoJSONVersionIdentifier {
    /**
     * The name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    name: string;
    /**
     * The guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-versionManagement-VersionManagementService.html#VersionInfoJSON)
     */
    guid: string;
  }

  export interface Viewpoint extends Accessor, JSONSupport {}

  export class Viewpoint {
    /**
     * Describes a point of view for a 2D or 3D view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html)
     */

    constructor(properties?: ViewpointProperties);

    /**
     * The viewpoint camera (3D only).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#camera)
     */
    camera: Camera;
    /**
     * The rotation of due north in relation to the top of the view in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#rotation)
     */
    rotation: number;
    /**
     * The scale of the viewpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#scale)
     */
    scale: number;
    /**
     * The target geometry framed by the viewpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#targetGeometry)
     */
    targetGeometry: Geometry;

    /**
     * Create a deep clone of the viewpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#clone)
     */
    clone(): Viewpoint;

    static fromJSON(json: any): Viewpoint;
  }

  interface ViewpointProperties {
    /**
     * The viewpoint camera (3D only).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#camera)
     */
    camera?: CameraProperties;
    /**
     * The rotation of due north in relation to the top of the view in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#rotation)
     */
    rotation?: number;
    /**
     * The scale of the viewpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#scale)
     */
    scale?: number;
    /**
     * The target geometry framed by the viewpoint.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html#targetGeometry)
     */
    targetGeometry?: GeometryProperties;
  }

  export class BaseLayerView2D extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) with a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html)
     */

    constructor(properties?: BaseLayerView2DProperties);

    /**
     * The array of [Tile](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile) objects computed to cover the MapView's visible area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#tiles)
     */
    tiles: Tile[];
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) this [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#view)
     */
    view: MapView;

    /**
     * Method called when after the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) is created and right before it's asked to draw the layer's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#attach)
     */
    attach(): void;
    /**
     * Method called after the layer is removed and the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) is about to be removed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#detach)
     */
    detach(): void;
    /**
     * Method to implement that is responsible for providing objects hit at the specified screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#hitTest)
     */
    hitTest(mapPoint: Point, screenPoint: ScreenPoint): Promise<Graphic[]>;
    /**
     * The method to implement that is responsible of drawing the content of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#render)
     */
    render(renderParameters: BaseLayerView2DRenderRenderParameters): void;
    /**
     * The [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) can call this method to ask the MapView to schedule a new rendering frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#requestRender)
     */
    requestRender(): void;
    /**
     * Method to implement, which notifies of tiles being added or removed for the current view state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#tilesChanged)
     */
    tilesChanged(added: Tile[], removed: Tile[]): void;
  }

  interface BaseLayerView2DProperties extends LayerViewProperties {
    /**
     * The array of [Tile](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile) objects computed to cover the MapView's visible area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#tiles)
     */
    tiles?: Tile[];
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) this [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#view)
     */
    view?: MapViewProperties;
  }

  export interface BaseLayerView2DRenderRenderParameters {
    /**
     * The [canvas 2D context](https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D) in which to draw content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#render)
     */
    context: CanvasRenderingContext2D;
    /**
     * The stationary state of the `MapView`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#render)
     */
    stationary: boolean;
    /**
     * The object that describes view state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#render)
     */
    state: ViewState;
  }

  /**
   * An object representing a point on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#ScreenPoint)
   */
  export interface ScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#ScreenPoint)
     */
    y: number;
  }

  /**
   * Represents a tile reference.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
   */
  export interface Tile {
    /**
     * The tile string identifier in the format `level/row/col/world`
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    id: string;
    /**
     * The level identifier of the [LOD](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html) to which the tile belongs
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    level: number;
    /**
     * The row identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    row: number;
    /**
     * The column identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    col: number;
    /**
     * When the projection allows world wrapping (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    world: number;
    /**
     * The number of map units per pixel in the tile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    resolution: number;
    /**
     * The map scale at the tile's level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    scale: number;
    /**
     * The coordinates of the top-left corner of the tile as an array of two numbers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    coords: number[];
    /**
     * The bounds of the tile as an array of four numbers that be readily converted to an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerView2D.html#Tile)
     */
    bounds: number[];
  }

  export class BaseLayerViewGL2D extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) with a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html)
     */

    constructor(properties?: BaseLayerViewGL2DProperties);

    /**
     * The WebGL rendering context associated to this layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#context)
     */
    context: WebGL2RenderingContext;
    /**
     * The array of module:esri/views/2d/layers/BaseLayerViewGL2D#Tile objects computed to cover the MapView's visible area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tiles)
     */
    tiles: BaseLayerViewGL2DTile[];
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) this [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#view)
     */
    view: MapView;

    /**
     * Method called after the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) is created and right before it starts drawing the layer's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#attach)
     */
    attach(): void;
    /**
     * Bind the designated rendering output surface and restore the correct viewport.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#bindRenderTarget)
     */
    bindRenderTarget(): void;
    /**
     * Method called after the layer is removed and the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) is about to be removed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#detach)
     */
    detach(): void;
    /**
     * Get the designated rendering output surface and corresponding viewport configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#getRenderTarget)
     */
    getRenderTarget(): RenderTarget;
    /**
     * Method to implement that is responsible for providing objects hit at the specified screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#hitTest)
     */
    hitTest(mapPoint: Point, screenPoint: BaseLayerViewGL2DScreenPoint): Promise<Graphic[]>;
    /**
     * The method to implement that is responsible of drawing the content of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render)
     */
    render(renderParameters: BaseLayerViewGL2DRenderRenderParameters): void;
    /**
     * The [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) can call this method to ask the MapView to schedule a new rendering frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#requestRender)
     */
    requestRender(): void;
    /**
     * Tessellate an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) into a rectangle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tessellateExtent)
     */
    tessellateExtent(extent: Extent): Promise<TessellatedMesh>;
    /**
     * Tessellate a [Multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) into quads (markers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tessellateMultipoint)
     */
    tessellateMultipoint(multipoint: Multipoint, footprint: Rect): Promise<TessellatedMesh>;
    /**
     * Tessellate a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) into a quad (marker).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tessellatePoint)
     */
    tessellatePoint(point: Point, footprint: Rect): Promise<TessellatedMesh>;
    /**
     * Tessellate a [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) into triangles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tessellatePolygon)
     */
    tessellatePolygon(polygon: Polygon): Promise<TessellatedMesh>;
    /**
     * Tessellate a [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) into triangles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tessellatePolyline)
     */
    tessellatePolyline(polyline: Polyline, width: number): Promise<TessellatedMesh>;
    /**
     * Method to implement, which notifies of tiles being added or removed for the current view state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tilesChanged)
     */
    tilesChanged(added: BaseLayerViewGL2DTile[], removed: BaseLayerViewGL2DTile[]): void;
  }

  interface BaseLayerViewGL2DProperties extends LayerViewProperties {
    /**
     * The WebGL rendering context associated to this layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#context)
     */
    context?: WebGL2RenderingContext;
    /**
     * The array of module:esri/views/2d/layers/BaseLayerViewGL2D#Tile objects computed to cover the MapView's visible area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#tiles)
     */
    tiles?: BaseLayerViewGL2DTile[];
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) this [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) belongs to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#view)
     */
    view?: MapViewProperties;
  }

  export interface BaseLayerViewGL2DRenderRenderParameters {
    /**
     * The WebGL or WebGL 2 context.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render)
     */
    context: WebGL2RenderingContext;
    /**
     * The stationary state of the `MapView`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render)
     */
    stationary: boolean;
    /**
     * The object that describes view state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render)
     */
    state: ViewState;
  }

  /**
   * A vertex of a tessellated mesh.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
   */
  export interface MeshVertex {
    /**
     * The `x`-coordinate, expressed in the same units as the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    x: number;
    /**
     * The `y`-coordinate, expressed in the same units as the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    y: number;
    /**
     * The `x`-offset in screen space; this is used to extrude points (into quads) and polylines, and is 0 for polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    xOffset: number;
    /**
     * The `y`-offset in screen space; this is used to extrude points (into quads) and polylines, and is 0 for polygons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    yOffset: number;
    /**
     * The `u`-coordinate for texture mapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    uTexcoord: number;
    /**
     * The `v`-coordinate for texture mapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    vTexcoord: number;
    /**
     * The distance parameter for this vertex; this only applies when tessellating polylines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#MeshVertex)
     */
    distance: number;
  }

  /**
   * A rectangle in screen-space.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Rect)
   */
  export interface Rect {
    /**
     * The `x`-coordinate of the upper-left corner of the rectangle, relative to the anchor of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Rect)
     */
    x: number;
    /**
     * The `y`-coordinate of the upper-left corner of the rectangle, relative to the anchor of the marker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Rect)
     */
    y: number;
    /**
     * Width of the rectangle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Rect)
     */
    width: number;
    /**
     * Height of the rectangle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Rect)
     */
    height: number;
  }

  /**
   * The destination to which [render()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render) should direct its output.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#RenderTarget)
   */
  export interface RenderTarget {
    /**
     * The framebuffer where the [render()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#render) method should direct its output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#RenderTarget)
     */
    framebuffer: any;
    /**
     * A viewport that fully covers `framebuffer`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#RenderTarget)
     */
    viewport: number[];
  }

  /**
   * An object representing a point on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#ScreenPoint)
   */
  export interface BaseLayerViewGL2DScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#ScreenPoint)
     */
    y: number;
  }

  /**
   * The triangle mesh resulting from a tessellation operation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#TessellatedMesh)
   */
  export interface TessellatedMesh {
    /**
     * The vertices that make up the mesh.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#TessellatedMesh)
     */
    vertices: MeshVertex[];
    /**
     * The indices of the triangles that connect vertices together; each consecutive triplet of indices denotes a triangle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#TessellatedMesh)
     */
    indices: number[];
  }

  /**
   * Represents a tile reference.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
   */
  export interface BaseLayerViewGL2DTile {
    /**
     * The tile string identifier in the format `level/row/col/world`
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    id: string;
    /**
     * The level identifier of the [LOD](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html) to which the tile belongs
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    level: number;
    /**
     * The row identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    row: number;
    /**
     * The column identifier.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    col: number;
    /**
     * When the projection allows world wrapping (e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    world: number;
    /**
     * The number of map units per pixel in the tile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    resolution: number;
    /**
     * The map scale at the tile's level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    scale: number;
    /**
     * The coordinates of the top-left corner of the tile as an array of two numbers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    coords: number[];
    /**
     * The bounds of the tile as an array of four numbers that be readily converted to an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-layers-BaseLayerViewGL2D.html#Tile)
     */
    bounds: number[];
  }

  export interface ViewState extends Accessor, JSONSupport {}

  export class ViewState {
    /**
     * Object that holds information about the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html)
     */

    constructor(properties?: ViewStateProperties);

    /**
     * Represents the view's center point as an array of two numbers `[x, y]`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#center)
     */
    readonly center: number[];
    /**
     * The extent represents the visible portion of a map within the view as an instance of Extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#extent)
     */
    readonly extent: Extent;
    /**
     * Represents the size of one pixel in map units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#resolution)
     */
    readonly resolution: number;
    /**
     * The clockwise rotation of due north in relation to the top of the view in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#rotation)
     */
    readonly rotation: number;
    /**
     * Represents the map scale at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#scale)
     */
    readonly scale: number;
    /**
     * Represents the width and height of the view in pixels, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#size)
     */
    readonly size: number[];

    /**
     * Creates a deep clone of ViewState object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#clone)
     */
    clone(): ViewState;
    /**
     * Copies the properties from a given view state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#copy)
     */
    copy(state: ViewState): ViewState;
    /**
     * Converts the x and y screen coordinates to map coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#toMap)
     */
    toMap(out: number[], x: number, y: number): number[];
    /**
     * Converts the x and y map coordinates to screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#toScreen)
     */
    toScreen(out: number[], x: number, y: number): number[];
    /**
     * Converts the x and y map coordinates to screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-2d-ViewState.html#toScreenNoRotation)
     */
    toScreenNoRotation(out: number[], x: number, y: number): number[];

    static fromJSON(json: any): ViewState;
  }

  interface ViewStateProperties {}

  export class AreaMeasurementAnalysisView3D {
    /**
     * The area measurement analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#analysis)
     */
    readonly analysis: AreaMeasurementAnalysis;
    /**
     * Result of the area measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#result)
     */
    result: AreaMeasurementAnalysisResult;
    /**
     * The analysis view type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#type)
     */
    readonly type: "area-measurement-view-3d";
    /**
     * When `true`, the analysis is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#visible)
     */
    visible: boolean;

    constructor(properties?: any);
  }

  /**
   * Result obtained from an [AreaMeasurementAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-AreaMeasurementAnalysis.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#AreaMeasurementAnalysisResult)
   */
  export interface AreaMeasurementAnalysisResult {
    /**
     * Describes the mode in which the measurement is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#AreaMeasurementAnalysisResult)
     */
    mode: "euclidean" | "geodesic";
    /**
     * Area of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#AreaMeasurementAnalysisResult)
     */
    area: Area;
    /**
     * Perimeter length of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-AreaMeasurementAnalysisView3D.html#AreaMeasurementAnalysisResult)
     */
    perimeter: Length;
  }

  export class DirectLineMeasurementAnalysisView3D {
    /**
     * The direct line measurement analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#analysis)
     */
    readonly analysis: DirectLineMeasurementAnalysis;
    /**
     * Result of the direct line measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#result)
     */
    result: DirectLineMeasurementAnalysisResult;
    /**
     * The analysis view type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#type)
     */
    readonly type: "direct-line-measurement-view-3d";
    /**
     * When `true`, the analysis is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#visible)
     */
    visible: boolean;

    constructor(properties?: any);
  }

  /**
   * Result obtained from a [DirectLineMeasurementAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DirectLineMeasurementAnalysis.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
   */
  export interface DirectLineMeasurementAnalysisResult {
    /**
     * Describes how the `horizontalDistance` is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
     */
    mode: "euclidean" | "geodesic";
    /**
     * DEPRECATED since 4.29: Use `directDistance` and `horizontalDistance` - Measured distance between the start and end points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
     */
    distance: Length;
    /**
     * The 3D distance between the two points that is computed in a Euclidean manner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
     */
    directDistance: Length;
    /**
     * The 2D distance between the two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
     */
    horizontalDistance: Length;
    /**
     * The elevation difference between the two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-DirectLineMeasurementAnalysisView3D.html#DirectLineMeasurementAnalysisResult)
     */
    verticalDistance: Length;
  }

  export class LineOfSightAnalysisResult {
    /**
     * The first [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) intersected by the line of sight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisResult.html#intersectedGraphic)
     */
    readonly intersectedGraphic: Graphic;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location where the line of sight first intersects the ground or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisResult.html#intersectedLocation)
     */
    readonly intersectedLocation: Point;
    /**
     * Represents a target in a [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html) which is used to calculate the visibility from the [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html#observer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisResult.html#target)
     */
    target: LineOfSightAnalysisTarget;
    /**
     * Whether the target is visible from the observer or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisResult.html#visible)
     */
    readonly visible: boolean;

    constructor(properties?: any);
  }

  export class LineOfSightAnalysisView3D {
    /**
     * The line of sight analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#analysis)
     */
    readonly analysis: LineOfSightAnalysis;
    /**
     * Set to `true` to enable interactivity for the associated [LineOfSightAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LineOfSightAnalysis.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#interactive)
     */
    interactive: boolean;
    /**
     * Analysis results for each target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#results)
     */
    readonly results: Collection<LineOfSightAnalysisResult>;
    /**
     * The analysis view type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#type)
     */
    readonly type: "line-of-sight-view-3d";
    /**
     * When `true`, the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#analysis) is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-LineOfSightAnalysisView3D.html#visible)
     */
    visible: boolean;

    constructor(properties?: any);
  }

  export class SliceAnalysisView3D {
    /**
     * Only one [SliceAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html) at a time can be active in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-SliceAnalysisView3D.html#active)
     */
    active: boolean;
    /**
     * The slice analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-SliceAnalysisView3D.html#analysis)
     */
    readonly analysis: SliceAnalysis;
    /**
     * Set to `true` to enable interactivity for the associated [SliceAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-SliceAnalysis.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-SliceAnalysisView3D.html#interactive)
     */
    interactive: boolean;

    readonly type: "slice-view-3d";
    /**
     * When `true`, the analysis is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-SliceAnalysisView3D.html#visible)
     */
    visible: boolean;

    constructor(properties?: any);
  }

  export class ViewshedAnalysisView3D {
    /**
     * The viewshed analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#analysis)
     */
    readonly analysis: ViewshedAnalysis;
    /**
     * Set to `true` to enable interactivity for the associated [ViewshedAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-ViewshedAnalysis.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#interactive)
     */
    interactive: boolean;
    /**
     * The selected viewshed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#selectedViewshed)
     */
    selectedViewshed: Viewshed;
    /**
     * The analysis view type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#type)
     */
    readonly type: "viewshed-view-3d";
    /**
     * When `true`, the analysis is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#visible)
     */
    visible: boolean;

    /**
     * Starts the interactive creation of new viewsheds and adds them to the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#analysis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#createViewsheds)
     */
    createViewsheds(options?: ViewshedAnalysisView3DCreateViewshedsOptions): Promise<any>;

    constructor(properties?: any);
  }

  export interface ViewshedAnalysisView3DCreateViewshedsOptions {
    /**
     * Abort signal which can be used to cancel creation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-analysis-ViewshedAnalysisView3D.html#createViewsheds)
     */
    signal?: AbortSignal;
  }

  export class CloudyWeather extends Accessor {
    /**
     * The CloudyWeather class allows you to change the weather conditions in the scene to cloudy weather.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-CloudyWeather.html)
     */

    constructor(properties?: CloudyWeatherProperties);

    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-CloudyWeather.html#cloudCover)
     */
    cloudCover: number;
    /**
     * The type of weather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-CloudyWeather.html#type)
     */
    readonly type: "cloudy";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-CloudyWeather.html#clone)
     */
    clone(): CloudyWeather;
  }

  interface CloudyWeatherProperties {
    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-CloudyWeather.html#cloudCover)
     */
    cloudCover?: number;
  }

  export class FoggyWeather extends Accessor {
    /**
     * The FoggyWeather class allows you to change the weather conditions in the scene to foggy weather.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-FoggyWeather.html)
     */

    constructor(properties?: FoggyWeatherProperties);

    /**
     * Specifies the amount of fog used in the scene.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-FoggyWeather.html#fogStrength)
     */
    fogStrength: number;
    /**
     * The type of Weather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-FoggyWeather.html#type)
     */
    readonly type: "foggy";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-FoggyWeather.html#clone)
     */
    clone(): FoggyWeather;
  }

  interface FoggyWeatherProperties {
    /**
     * Specifies the amount of fog used in the scene.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-FoggyWeather.html#fogStrength)
     */
    fogStrength?: number;
  }

  export class RainyWeather extends Accessor {
    /**
     * The RainyWeather class allows you to change the weather conditions in the scene to rainy weather.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html)
     */

    constructor(properties?: RainyWeatherProperties);

    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#cloudCover)
     */
    cloudCover: number;
    /**
     * Specifies the amount of falling rain.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#precipitation)
     */
    precipitation: number;
    /**
     * The type of Weather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#type)
     */
    readonly type: "rainy";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#clone)
     */
    clone(): RainyWeather;
  }

  interface RainyWeatherProperties {
    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#cloudCover)
     */
    cloudCover?: number;
    /**
     * Specifies the amount of falling rain.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-RainyWeather.html#precipitation)
     */
    precipitation?: number;
  }

  export class SnowyWeather extends Accessor {
    /**
     * The SnowyWeather class allows you to change the weather conditions in the scene to snowy weather.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html)
     */

    constructor(properties?: SnowyWeatherProperties);

    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#cloudCover)
     */
    cloudCover: number;
    /**
     * Specifies the amount of falling snow.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#precipitation)
     */
    precipitation: number;
    /**
     * Display surfaces covered with snow.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#snowCover)
     */
    snowCover: "enabled" | "disabled";
    /**
     * The type of Weather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#type)
     */
    readonly type: "snowy";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#clone)
     */
    clone(): SnowyWeather;
  }

  interface SnowyWeatherProperties {
    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#cloudCover)
     */
    cloudCover?: number;
    /**
     * Specifies the amount of falling snow.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#precipitation)
     */
    precipitation?: number;
    /**
     * Display surfaces covered with snow.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SnowyWeather.html#snowCover)
     */
    snowCover?: "enabled" | "disabled";
  }

  export class SunLighting extends websceneSunLighting {
    /**
     * The SunLighting class allows you to change the lighting in the scene to sunlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunLighting.html)
     */

    constructor(properties?: SunLightingProperties);

    /**
     * Indicates whether the date and time of the simulated sun is automatically updated to maintain the current time of day while the camera changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunLighting.html#cameraTrackingEnabled)
     */
    cameraTrackingEnabled: boolean;

    /**
     * Create a clone of this while partially applying the provided webscene lighting settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunLighting.html#cloneWithWebsceneLighting)
     */
    cloneWithWebsceneLighting(): void;
  }

  interface SunLightingProperties extends websceneSunLightingProperties {
    /**
     * Indicates whether the date and time of the simulated sun is automatically updated to maintain the current time of day while the camera changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunLighting.html#cameraTrackingEnabled)
     */
    cameraTrackingEnabled?: boolean;
  }

  export class SunnyWeather extends Accessor {
    /**
     * The SunnyWeather class allows you to change the weather conditions in the scene to sunny weather.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunnyWeather.html)
     */

    constructor(properties?: SunnyWeatherProperties);

    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunnyWeather.html#cloudCover)
     */
    cloudCover: number;
    /**
     * The type of weather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunnyWeather.html#type)
     */
    readonly type: "sunny";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunnyWeather.html#clone)
     */
    clone(): SunnyWeather;
  }

  interface SunnyWeatherProperties {
    /**
     * Specifies the amount of cloud cover in the sky for a certain weather type.
     *
     * @default 0.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-SunnyWeather.html#cloudCover)
     */
    cloudCover?: number;
  }

  export class VirtualLighting extends websceneVirtualLighting {
    /**
     * The VirtualLighting class allows you to change the lighting in the scene to virtual light.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-VirtualLighting.html)
     */

    constructor(properties?: VirtualLightingProperties);

    /**
     * Create a clone of this while partially applying the provided webscene lighting settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-environment-VirtualLighting.html#cloneWithWebsceneLighting)
     */
    cloneWithWebsceneLighting(): void;
  }

  interface VirtualLightingProperties extends websceneVirtualLightingProperties {}

  /**
   * render the map/scene on screen.
   *
   * @deprecated since 4.29. Use the new {@link module:esri/views/3d/webgl/RenderNode RenderNode} instead.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html)
   */
  interface externalRenderers {
    add(view: SceneView, renderer: ExternalRenderer): void;

    fromRenderCoordinates(
      view: SceneView,
      srcCoordinates: number[] | Float32Array | Float64Array,
      srcStart: number,
      destCoordinates: number[] | Float32Array | Float64Array,
      destStart: number,
      destSpatialReference: SpatialReference,
      count: number,
    ): number[] | Float32Array | Float64Array;

    getRenderCamera(view: SceneView): RenderCamera;

    remove(view: SceneView, renderer: ExternalRenderer): void;

    renderCoordinateTransformAt(
      view: SceneView,
      origin: number[] | Float32Array | Float64Array,
      srcSpatialReference?: SpatialReference,
      dest?: number[] | Float32Array,
    ): number[] | Float32Array;

    requestRender(view: SceneView): void;

    toRenderCoordinates(
      view: SceneView,
      srcCoordinates: number[] | Float32Array | Float64Array,
      srcStart: number,
      srcSpatialReference: SpatialReference,
      destCoordinates: number[] | Float32Array | Float64Array,
      destStart: number,
      count: number,
    ): number[] | Float32Array | Float64Array;
  }

  export const externalRenderers: externalRenderers;

  /**
   * Defines an external renderer using callbacks and properties.
   *
   * @deprecated since 4.29. Use new {@link module:esri/views/3d/webgl/RenderNode RenderNode} instead.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#ExternalRenderer)
   */
  export interface ExternalRenderer {
    /**
     * Typically called once after adding the external renderer to a view, or whenever the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) becomes [ready](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ready).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#ExternalRenderer)
     */
    setup?: RenderContextCallback;
    /**
     * Called in every frame to execute the state update and draw calls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#ExternalRenderer)
     */
    render?: RenderContextCallback;
    /**
     * Called when the external renderer is removed from a view, or when the [ready](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ready) state of the view turns false.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#ExternalRenderer)
     */
    dispose?: RenderContextCallback;
  }

  /**
   * The object passed as a parameter to every call to `setup` and `render` the external renderer.
   *
   * @deprecated since 4.29. Use new {@link module:esri/views/3d/webgl/RenderNode RenderNode} instead.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
   */
  export interface RenderContext {
    /**
     * The WebGL rendering context.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
     */
    gl: WebGL2RenderingContext;
    /**
     * The camera used to render the current frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
     */
    camera: RenderCamera;
    /**
     * The lighting used by [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) to render the current frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
     */
    sunLight: SunLight;

    /**
     * A convenience function provided to completely reset the WebGL state after using it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
     */
    resetWebGLState(): void;
    /**
     * Binds the color and depth buffers an external renderer is supposed to render into.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-externalRenderers.html#RenderContext)
     */
    bindRenderTarget(): void;
  }

  export type RenderContextCallback = (context?: RenderContext) => void;

  /**
   * This class contains performance information like memory usage and number of features for a specific layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html)
   */
  interface LayerPerformanceInfo {
    /**
     * The number of features displayed in the SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html#displayedNumberOfFeatures)
     */
    displayedNumberOfFeatures: number;
    /**
     * The layer corresponding to the memory usage information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html#layer)
     */
    layer: Layer;
    /**
     * The maximum number of features that can be displayed in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html#maximumNumberOfFeatures)
     */
    maximumNumberOfFeatures: number;
    /**
     * An estimate of the memory currently in use by the layer, in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html#memory)
     */
    memory: number;
    /**
     * The total number of features contained in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-LayerPerformanceInfo.html#totalNumberOfFeatures)
     */
    totalNumberOfFeatures: number;
  }

  export const LayerPerformanceInfo: LayerPerformanceInfo;

  /**
   * This class contains information about performance in the scene like global memory usage and additional details for layers about memory consumption and number of features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html)
   */
  interface SceneViewPerformanceInfo {
    /**
     * An estimate of the memory used by the [Edges3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-edges-Edges3D.html) feature when added to 3D objects in [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) or to polygons with [ExtrudeSymbol3DLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-ExtrudeSymbol3DLayer.html) symbols.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#edgesMemory)
     */
    edgesMemory: number;
    /**
     * An array representing detailed information about memory usage of most layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#layerPerformanceInfos)
     */
    layerPerformanceInfos: LayerPerformanceInfo[];
    /**
     * Quality level as a number between 0 and 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#quality)
     */
    quality: number;
    /**
     * An estimate of the memory currently in use by [ElevationLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html) and [tiled layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-TileLayer.html), in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#terrainMemory)
     */
    terrainMemory: number;
    /**
     * The total memory available to the SceneView, in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#totalMemory)
     */
    totalMemory: number;
    /**
     * An estimate of the memory that is in use by the SceneView, in bytes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-support-SceneViewPerformanceInfo.html#usedMemory)
     */
    usedMemory: number;
  }

  export const SceneViewPerformanceInfo: SceneViewPerformanceInfo;

  /**
   * A convenience module for importing [RenderNode](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html) classes when developing with [TypeScript](https://developers.arcgis.com/javascript/latest/typescript-setup/).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html)
   */
  interface webgl {
    /**
     * Transforms positions from the internal rendering coordinate system to the output spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#fromRenderCoordinates)
     */
    fromRenderCoordinates(
      view: SceneView,
      srcCoordinates: number[] | Float32Array | Float64Array,
      srcStart: number,
      destCoordinates: number[] | Float32Array | Float64Array,
      destStart: number,
      destSpatialReference: SpatialReference,
      count: number,
    ): number[] | Float32Array | Float64Array;
    /**
     * Computes a 4x4 affine transformation matrix that constitutes a linear coordinate transformation from a local Cartesian coordinate system to the virtual world coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#renderCoordinateTransformAt)
     */
    renderCoordinateTransformAt(
      view: SceneView,
      origin: number[] | Float32Array | Float64Array,
      srcSpatialReference?: SpatialReference,
      dest?: number[] | Float32Array,
    ): number[] | Float32Array;
    /**
     * Transforms positions from the given spatial reference to the internal rendering coordinate system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#toRenderCoordinates)
     */
    toRenderCoordinates(
      view: SceneView,
      srcCoordinates: number[] | Float32Array | Float64Array,
      srcStart: number,
      srcSpatialReference: SpatialReference,
      destCoordinates: number[] | Float32Array | Float64Array,
      destStart: number,
      count: number,
    ): number[] | Float32Array | Float64Array;
  }

  export const webgl: webgl;

  /**
   * ManagedFBO is an interface to represent a framebuffer object resource of the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html)
   */
  interface ManagedFBO {
    /**
     * The name of the managed fbo target
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#name)
     */
    readonly name: RenderNodeInput;

    /**
     * Acquire and attach a new color texture to this framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#acquireColor)
     */
    acquireColor(attachment: ColorAttachment): ManagedFBO;
    /**
     * Acquire and attach a new depth buffer to this framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#acquireDepth)
     */
    acquireDepth(): ManagedFBO;
    /**
     * Attach a color buffer texture to this framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#attachColor)
     */
    attachColor(color: ManagedColorAttachment, attachment: ColorAttachment): ManagedFBO;
    /**
     * Attach a depth buffer texture to this framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#attachDepth)
     */
    attachDepth(depth: ManagedDepthAttachment): ManagedFBO;
    /**
     * Returns a managed color attachment linked to this framebuffer object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#getAttachment)
     */
    getAttachment(attachment?: ColorAttachment | DepthAttachment): ManagedColorAttachment | ManagedDepthAttachment;
    /**
     * Return the texture for a given color or depth attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#getTexture)
     */
    getTexture(attachment?: ColorAttachment | DepthAttachment): FBOTexture;
    /**
     * Release this managed framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#release)
     */
    release(): void;
    /**
     * Increase reference counting on this managed framebuffer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#retain)
     */
    retain(): void;
  }

  export const ManagedFBO: ManagedFBO;

  /**
   * Describes a color attachment point of WebGL2.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ColorAttachment)
   */
  export interface ColorAttachment {}

  /**
   * Describes a depth attachment point of WebGL2.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#DepthAttachment)
   */
  export interface DepthAttachment {}

  /**
   * Encapsulates a WebGL texture.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#FBOTexture)
   */
  export interface FBOTexture {
    /**
     * the WebGL name content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#FBOTexture)
     */
    glName: string;
  }

  /**
   * Describes a single attachment of a managed framebuffer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedColorAttachment)
   */
  export interface ManagedColorAttachment {
    /**
     * Increase reference counting on this managed attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedColorAttachment)
     */
    retain(): void;
    /**
     * Release this managed attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedColorAttachment)
     */
    release(): void;
  }

  /**
   * Describes the depth attachment of a managed framebuffer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedDepthAttachment)
   */
  export interface ManagedDepthAttachment {
    /**
     * Increase reference counting on this managed attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedDepthAttachment)
     */
    retain(): void;
    /**
     * Release this managed attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-ManagedFBO.html#ManagedDepthAttachment)
     */
    release(): void;
  }

  /**
   * This is the RenderCamera interface used by the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html)
   */
  interface RenderCamera {
    /**
     * The camera target ("look at") position in the internal Cartesian rendering coordinate system represented by a vector with 3 components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#center)
     */
    readonly center: readonly number[];
    /**
     * The position of the camera in the internal Cartesian rendering coordinate system represented by a vector with 3 components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#eye)
     */
    readonly eye: readonly number[];
    /**
     * The distance to the far plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#far)
     */
    readonly far: number;
    /**
     * The horizontal field of view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#fovX)
     */
    readonly fovX: number;
    /**
     * The vertical field of view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#fovY)
     */
    readonly fovY: number;
    /**
     * The distance to the near plane.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#near)
     */
    readonly near: number;
    /**
     * The render pixel ratio.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#pixelRatio)
     */
    readonly pixelRatio: number;
    /**
     * A 4x4 matrix that defines the perspective projection transformation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#projectionMatrix)
     */
    readonly projectionMatrix: readonly number[];
    /**
     * The camera up vector with 3 components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#up)
     */
    readonly up: readonly number[];
    /**
     * A 4x4 matrix representing the inverse transpose of `viewMatrix`, used to transform normals
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#viewInverseTransposeMatrix)
     */
    readonly viewInverseTransposeMatrix: readonly number[];
    /**
     * A 4x4 matrix that transforms coordinates from world space to camera space.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#viewMatrix)
     */
    readonly viewMatrix: readonly number[];
    /**
     * The viewport expressed as vector with 4 components (x, y, width, height).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderCamera.html#viewport)
     */
    readonly viewport: readonly number[];
  }

  export const RenderCamera: RenderCamera;

  export class RenderNode extends Accessor {
    /**
     * The RenderNode provides low level access to the render pipeline of the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) to create custom visualizations and effects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html)
     */

    constructor(properties?: RenderNodeProperties);

    /**
     * Get the render representation of the current camera of a view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#camera)
     */
    camera: RenderCamera;
    /**
     * Declare which inputs are needed from the engine for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#consumes)
     */
    consumes: ConsumedNodes;
    /**
     * Returns the current WebGL2RenderingContext instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#gl)
     */
    gl: WebGL2RenderingContext;
    /**
     * Define the output produced by the render function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#produces)
     */
    produces: RenderNodeOutput;
    /**
     * The lighting used by [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) to render the current frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#sunLight)
     */
    sunLight: SunLight;
    /**
     * The SceneView linked to this render node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#view)
     */
    view: SceneView;

    /**
     * Acquires and binds a managed framebuffer object to be written to and returned by the render function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#acquireOutputFramebuffer)
     */
    acquireOutputFramebuffer(): ManagedFBO;
    /**
     * Bind the color and depth buffers to render into and return the ManagedFBO.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#bindRenderTarget)
     */
    bindRenderTarget(): ManagedFBO;
    /**
     * The render function is called whenever a frame is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#render)
     */
    render(inputs: ManagedFBO[]): ManagedFBO;
    /**
     * Request the SceneView to be redrawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#requestRender)
     */
    requestRender(): void;
    /**
     * Reset WebGL to a well-defined state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#resetWebGLState)
     */
    resetWebGLState(): void;
  }

  interface RenderNodeProperties {
    /**
     * Get the render representation of the current camera of a view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#camera)
     */
    camera?: RenderCamera;
    /**
     * Declare which inputs are needed from the engine for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#consumes)
     */
    consumes?: ConsumedNodes;
    /**
     * Returns the current WebGL2RenderingContext instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#gl)
     */
    gl?: WebGL2RenderingContext;
    /**
     * Define the output produced by the render function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#produces)
     */
    produces?: "composite-color" | "opaque-color" | "transparent-color" | "final-color";
    /**
     * The lighting used by [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) to render the current frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#sunLight)
     */
    sunLight?: SunLight;
    /**
     * The SceneView linked to this render node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#view)
     */
    view?: SceneViewProperties;
  }

  /**
   * Tuple of an RGB color value and an intensity value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#ColorAndIntensity)
   */
  export interface ColorAndIntensity {
    /**
     * RGB color with values between 0 and 1.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#ColorAndIntensity)
     */
    color: number[];
    /**
     * Scalar intensity value by which the color should be scaled for compositing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#ColorAndIntensity)
     */
    intensity: number;
  }

  /**
   * Describes the lighting used by [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html), derived from its sun lighting model.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#SunLight)
   */
  export interface SunLight {
    /**
     * The incident light direction in render coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#SunLight)
     */
    direction: number[];
    /**
     * The diffuse light color and intensity.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#SunLight)
     */
    diffuse: ColorAndIntensity;
    /**
     * The ambient light color and intensity.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl-RenderNode.html#SunLight)
     */
    ambient: ColorAndIntensity;
  }

  /**
   * Describes the input framebuffer objects of a custom RenderNode.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#ConsumedNodes)
   */
  export interface ConsumedNodes {
    /**
     * One or more required inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#ConsumedNodes)
     */
    required: RenderNodeInput[];
    /**
     * One or more optional inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#ConsumedNodes)
     */
    optional?: RenderNodeInput[];
  }

  /**
   * Describes a specific RenderNodeInput.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#RenderNodeInput)
   */
  export type RenderNodeInput =
    | "composite-color"
    | "opaque-color"
    | "transparent-color"
    | "final-color"
    | "normals"
    | "highlights";

  /**
   * Describes a specific RenderNodeOutput.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-3d-webgl.html#RenderNodeOutput)
   */
  export type RenderNodeOutput = "composite-color" | "opaque-color" | "transparent-color" | "final-color";

  export class DimensionAnalysisView extends Accessor {
    /**
     * Represents the analysis view of a [DimensionAnalysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-DimensionAnalysis.html) after it has been added to [SceneView.analyses](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#analyses).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html)
     */

    constructor(properties?: DimensionAnalysisViewProperties);

    /**
     * The dimension analysis object associated with the analysis view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis)
     */
    readonly analysis: DimensionAnalysis;
    /**
     * Set to `true` to enable interactivity for the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#interactive)
     */
    interactive: boolean;
    /**
     * Results for each dimension in the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#results)
     */
    readonly results: Collection<LengthDimensionResult>;
    /**
     * The selected dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#selectedDimension)
     */
    selectedDimension: LengthDimension;
    /**
     * When `true`, the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis) is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#visible)
     */
    visible: boolean;

    /**
     * Starts the interactive creation of new dimensions and adds them to the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#createLengthDimensions)
     */
    createLengthDimensions(options?: DimensionAnalysisViewCreateLengthDimensionsOptions): Promise<any>;
  }

  interface DimensionAnalysisViewProperties {
    /**
     * Set to `true` to enable interactivity for the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#interactive)
     */
    interactive?: boolean;
    /**
     * The selected dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#selectedDimension)
     */
    selectedDimension?: LengthDimensionProperties;
    /**
     * When `true`, the [analysis](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#analysis) is visualized in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#visible)
     */
    visible?: boolean;
  }

  export interface DimensionAnalysisViewCreateLengthDimensionsOptions {
    /**
     * Abort signal which can be used to cancel creation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-DimensionAnalysisView.html#createLengthDimensions)
     */
    signal?: AbortSignal;
  }

  /**
   * Represents the result of a LengthDimension.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-LengthDimensionResult.html)
   */
  interface LengthDimensionResult {
    /**
     * The [LengthDimension](https://developers.arcgis.com/javascript/latest/api-reference/esri-analysis-LengthDimension.html) that this result corresponds to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-LengthDimensionResult.html#dimension)
     */
    dimension: LengthDimension;
    /**
     * The length currently being measured by the [dimension](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-LengthDimensionResult.html#dimension).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-analysis-LengthDimensionResult.html#length)
     */
    length: Length;
  }

  export const LengthDimensionResult: LengthDimensionResult;

  export class BasemapView extends Accessor {
    /**
     * Represents the view for a single basemap after it has been added to either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html)
     */

    constructor(properties?: BasemapViewProperties);

    /**
     * A collection containing a hierarchical list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of the base layers in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html#baseLayerViews)
     */
    readonly baseLayerViews: Collection<LayerView>;
    /**
     * A collection containing a hierarchical list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of the reference layers in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html#referenceLayerViews)
     */
    readonly referenceLayerViews: Collection<LayerView>;
    /**
     * Value is `true` when the basemap is updating; for example, if it is in the process of fetching data.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html#updating)
     */
    readonly updating: boolean;
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html#view)
     */
    view: MapView | SceneView;
  }

  interface BasemapViewProperties {
    /**
     * References the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BasemapView.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class BreakpointsOwner {
    /**
     * A convenience property used for defining the breakpoints on the [height](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#height) and [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#width) of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    breakpoints: BreakpointsOwnerBreakpoints;
    /**
     * A convenience property indicating the general size of the view's height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint)
     */
    heightBreakpoint: "xsmall" | "small" | "medium" | "large" | "xlarge";
    /**
     * A convenience property indicating the view's orientation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#orientation)
     */
    readonly orientation: "landscape" | "portrait";
    /**
     * A convenience property indicating the general size of the view's width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint)
     */
    widthBreakpoint: "xsmall" | "small" | "medium" | "large" | "xlarge";
  }

  interface BreakpointsOwnerProperties {
    /**
     * A convenience property used for defining the breakpoints on the [height](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#height) and [width](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#width) of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    breakpoints?: BreakpointsOwnerBreakpoints;
    /**
     * A convenience property indicating the general size of the view's height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint)
     */
    heightBreakpoint?: "xsmall" | "small" | "medium" | "large" | "xlarge";
    /**
     * A convenience property indicating the general size of the view's width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint)
     */
    widthBreakpoint?: "xsmall" | "small" | "medium" | "large" | "xlarge";
  }

  export interface BreakpointsOwnerBreakpoints {
    /**
     * Sets the `xsmall` breakpoint in pixels used by [widthBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint) and [heightBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint).
     *
     * @default 544
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    xsmall?: number;
    /**
     * Sets the `small` breakpoint in pixels used by [widthBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint) and [heightBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint).
     *
     * @default 768
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    small?: number;
    /**
     * Sets the `medium` breakpoint in pixels used by [widthBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint) and [heightBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint).
     *
     * @default 992
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    medium?: number;
    /**
     * Sets the `large` breakpoint in pixels used by [widthBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint) and [heightBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint).
     *
     * @default 1200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    large?: number;
    /**
     * Sets the `xlarge` breakpoint in pixels used by [widthBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#widthBreakpoint) and [heightBreakpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#heightBreakpoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-BreakpointsOwner.html#breakpoints)
     */
    xlarge?: number;
  }

  export class DOMContainer {
    /**
     * The `id` or node representing the DOM element containing the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#container)
     */
    container: HTMLDivElement;
    /**
     * Indicates if the browser focus is on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#focused)
     */
    readonly focused: boolean;
    /**
     * The height of the view in pixels read from the view container element.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#height)
     */
    readonly height: number;
    /**
     * Indicates if the view is being resized.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#resizing)
     */
    readonly resizing: boolean;
    /**
     * An array containing the width and height of the view in pixels, e.g.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#size)
     */
    readonly size: number[];
    /**
     * Indicates if the view is visible on the page.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#suspended)
     */
    readonly suspended: boolean;
    /**
     * Exposes the default widgets available in the view and allows you to toggle them on and off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#ui)
     */
    ui: DefaultUI;
    /**
     * The width of the view in pixels read from the view container element.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#width)
     */
    readonly width: number;
  }

  interface DOMContainerProperties {
    /**
     * The `id` or node representing the DOM element containing the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#container)
     */
    container?: HTMLDivElement | string;
    /**
     * Exposes the default widgets available in the view and allows you to toggle them on and off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-DOMContainer.html#ui)
     */
    ui?: DefaultUIProperties;
  }

  export class Draw extends Accessor {
    /**
     * The Draw class provides advanced drawing capabilities for developers who need complete control over creating temporary graphics with different geometries.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html)
     */

    constructor(properties?: DrawProperties);

    /**
     * A reference to the active [draw action](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#activeAction)
     */
    activeAction: DrawAction;
    /**
     * The view in which geometries will be drawn by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#view)
     */
    view: MapView | SceneView;

    /**
     * Complete the current active drawing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#complete)
     */
    complete(): void;
    /**
     * Creates an instance of the requested draw action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#create)
     */
    create(
      drawAction: "point" | "multipoint" | "polyline" | "polygon" | "rectangle" | "circle" | "ellipse",
      drawOptions?: DrawCreateDrawOptions,
    ): DrawAction;
    /**
     * Resets the drawing by clearing the active action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#reset)
     */
    reset(): void;
  }

  interface DrawProperties {
    /**
     * A reference to the active [draw action](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#activeAction)
     */
    activeAction?: DrawActionProperties;
    /**
     * The view in which geometries will be drawn by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface DrawCreateDrawOptions {
    /**
     * The drawing mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html#create)
     */
    mode?: "hybrid" | "freehand" | "click";
  }

  export interface DrawAction extends Accessor, Evented {}

  export class DrawAction {
    /**
     * DrawAction is the base class for all draw actions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html)
     */

    constructor(properties?: DrawActionProperties);

    /**
     * Controls whether the created geometry will have z coordinates or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#hasZ)
     */
    hasZ: boolean;
    /**
     * Two-dimensional array of numbers representing the coordinates of each vertex comprising the geometry being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#vertices)
     */
    readonly vertices: number[][];
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#view)
     */
    view: MapView | SceneView;

    /**
     * Indicates if the [redo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#redo) method can be called on the action instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#canRedo)
     */
    canRedo(): boolean;
    /**
     * Indicates if the [undo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#undo) method can be called on the action instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#canUndo)
     */
    canUndo(): boolean;
    /**
     * Maps the given screen point to a map point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#getCoordsAndPointFromScreenPoint)
     */
    getCoordsAndPointFromScreenPoint(screenPoint: DrawActionScreenPoint): FromScreenPointResult;
    /**
     * Maps the given screen point to a map point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#getCoordsFromScreenPoint)
     */
    getCoordsFromScreenPoint(screenPoint: DrawActionScreenPoint): number[];
    /**
     * Incrementally redo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#redo)
     */
    redo(): void;
    /**
     * Maps the given screen point to a map point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#screenToMap)
     */
    screenToMap(screenPoint: DrawActionScreenPoint): Point;
    /**
     * Incrementally undo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#undo)
     */
    undo(): void;
  }

  interface DrawActionProperties {
    /**
     * Controls whether the created geometry will have z coordinates or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#hasZ)
     */
    hasZ?: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The result object of the [getCoordsAndPointFromScreenPoint()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#getCoordsAndPointFromScreenPoint) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#FromScreenPointResult)
   */
  export interface FromScreenPointResult {
    /**
     * Array of x, y, and z component (if hasZ is enabled).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#FromScreenPointResult)
     */
    vertex: number[];
    /**
     * The map point of the point associated with the given screen point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#FromScreenPointResult)
     */
    mapPoint: Point;
  }

  /**
   * An object representing a point on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#ScreenPoint)
   */
  export interface DrawActionScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-DrawAction.html#ScreenPoint)
     */
    y: number;
  }

  export class MultipointDrawAction extends DrawAction {
    /**
     * This class uses the view events to generate a set of coordinates to create a new [Multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) geometry using [Draw](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-MultipointDrawAction.html)
     */

    constructor(properties?: MultipointDrawActionProperties);

    /**
     * Completes drawing the [multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) geometry and fires the [draw-complete](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-MultipointDrawAction.html#event-draw-complete) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-MultipointDrawAction.html#complete)
     */
    complete(): void;

    on(name: "cursor-update", eventHandler: MultipointDrawActionCursorUpdateEventHandler): IHandle;

    on(name: "vertex-add", eventHandler: MultipointDrawActionVertexAddEventHandler): IHandle;

    on(name: "vertex-remove", eventHandler: MultipointDrawActionVertexRemoveEventHandler): IHandle;

    on(name: "draw-complete", eventHandler: MultipointDrawActionDrawCompleteEventHandler): IHandle;

    on(name: "undo", eventHandler: MultipointDrawActionUndoEventHandler): IHandle;

    on(name: "redo", eventHandler: MultipointDrawActionRedoEventHandler): IHandle;
  }

  interface MultipointDrawActionProperties extends DrawActionProperties {}

  export interface MultipointDrawActionCursorUpdateEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "cursor-update";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface MultipointDrawActionDrawCompleteEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "draw-complete";

    vertices: number[][];
  }

  export interface MultipointDrawActionRedoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface MultipointDrawActionUndoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface MultipointDrawActionVertexAddEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-add";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface MultipointDrawActionVertexRemoveEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-remove";

    vertexIndex: number;

    vertices: number[][];
  }

  export class PointDrawAction extends DrawAction {
    /**
     * This class uses the view events to generate a set of coordinates to create a new [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry using [Draw](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PointDrawAction.html)
     */

    constructor(properties?: PointDrawActionProperties);

    /**
     * Completes drawing the [point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) geometry and fires the [draw-complete](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PointDrawAction.html#event-draw-complete) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PointDrawAction.html#complete)
     */
    complete(): void;

    on(name: "cursor-update", eventHandler: PointDrawActionCursorUpdateEventHandler): IHandle;

    on(name: "draw-complete", eventHandler: PointDrawActionDrawCompleteEventHandler): IHandle;
  }

  interface PointDrawActionProperties extends DrawActionProperties {}

  export interface PointDrawActionCursorUpdateEvent {
    coordinates: number[];

    defaultPrevented: boolean;

    preventDefault: Function;

    type: "cursor-update";
  }

  export interface PointDrawActionDrawCompleteEvent {
    coordinates: number[];

    defaultPrevented: boolean;

    preventDefault: Function;

    type: "draw-complete";
  }

  export class PolygonDrawAction extends DrawAction {
    /**
     * This class uses different [events](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html#events-summary) to generate a set of vertices to create a new [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html) geometry using [Draw](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html)
     */

    constructor(properties?: PolygonDrawActionProperties);

    /**
     * The drawing mode.
     *
     * @default "hybrid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html#mode)
     */
    mode: "hybrid" | "freehand" | "click";

    /**
     * Completes drawing the polygon geometry and fires the [draw-complete](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html#event-draw-complete) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html#complete)
     */
    complete(): void;

    on(name: "cursor-update", eventHandler: PolygonDrawActionCursorUpdateEventHandler): IHandle;

    on(name: "vertex-add", eventHandler: PolygonDrawActionVertexAddEventHandler): IHandle;

    on(name: "vertex-remove", eventHandler: PolygonDrawActionVertexRemoveEventHandler): IHandle;

    on(name: "draw-complete", eventHandler: PolygonDrawActionDrawCompleteEventHandler): IHandle;

    on(name: "undo", eventHandler: PolygonDrawActionUndoEventHandler): IHandle;

    on(name: "redo", eventHandler: PolygonDrawActionRedoEventHandler): IHandle;
  }

  interface PolygonDrawActionProperties extends DrawActionProperties {
    /**
     * The drawing mode.
     *
     * @default "hybrid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolygonDrawAction.html#mode)
     */
    mode?: "hybrid" | "freehand" | "click";
  }

  export interface PolygonDrawActionCursorUpdateEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "cursor-update";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolygonDrawActionDrawCompleteEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "draw-complete";

    vertices: number[][];
  }

  export interface PolygonDrawActionRedoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolygonDrawActionUndoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolygonDrawActionVertexAddEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-add";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolygonDrawActionVertexRemoveEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-remove";

    vertexIndex: number;

    vertices: number[][];
  }

  export class PolylineDrawAction extends DrawAction {
    /**
     * This class uses different [events](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html#events-summary) to generate a set of vertices to create a new [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html) geometry using [Draw](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-Draw.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html)
     */

    constructor(properties?: PolylineDrawActionProperties);

    /**
     * The drawing mode.
     *
     * @default "hybrid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html#mode)
     */
    mode: "hybrid" | "freehand" | "click";

    /**
     * Completes drawing the polyline geometry and fires the [draw-complete](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html#event-draw-complete) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html#complete)
     */
    complete(): void;

    on(name: "cursor-update", eventHandler: PolylineDrawActionCursorUpdateEventHandler): IHandle;

    on(name: "vertex-add", eventHandler: PolylineDrawActionVertexAddEventHandler): IHandle;

    on(name: "vertex-remove", eventHandler: PolylineDrawActionVertexRemoveEventHandler): IHandle;

    on(name: "draw-complete", eventHandler: PolylineDrawActionDrawCompleteEventHandler): IHandle;

    on(name: "undo", eventHandler: PolylineDrawActionUndoEventHandler): IHandle;

    on(name: "redo", eventHandler: PolylineDrawActionRedoEventHandler): IHandle;
  }

  interface PolylineDrawActionProperties extends DrawActionProperties {
    /**
     * The drawing mode.
     *
     * @default "hybrid"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-PolylineDrawAction.html#mode)
     */
    mode?: "hybrid" | "freehand" | "click";
  }

  export interface PolylineDrawActionCursorUpdateEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "cursor-update";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolylineDrawActionDrawCompleteEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "draw-complete";

    vertices: number[][];
  }

  export interface PolylineDrawActionRedoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolylineDrawActionUndoEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: string;

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolylineDrawActionVertexAddEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-add";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface PolylineDrawActionVertexRemoveEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-remove";

    vertexIndex: number;

    vertices: number[][];
  }

  export class SegmentDrawAction extends DrawAction {
    /**
     * This class uses different [events](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#events-summary) to generate a set of vertices to create a geometry with [drag mode](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#mode) or with two clicks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html)
     */

    constructor(properties?: SegmentDrawActionProperties);

    /**
     * The drawing mode.
     *
     * @default "freehand"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#mode)
     */
    mode: "freehand" | "click";

    /**
     * Completes drawing the polygon geometry and fires the [draw-complete](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#event-draw-complete) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#complete)
     */
    complete(): void;

    on(name: "cursor-update", eventHandler: SegmentDrawActionCursorUpdateEventHandler): IHandle;

    on(name: "draw-complete", eventHandler: SegmentDrawActionDrawCompleteEventHandler): IHandle;

    on(name: "vertex-add", eventHandler: SegmentDrawActionVertexAddEventHandler): IHandle;
  }

  interface SegmentDrawActionProperties extends DrawActionProperties {
    /**
     * The drawing mode.
     *
     * @default "freehand"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-draw-SegmentDrawAction.html#mode)
     */
    mode?: "freehand" | "click";
  }

  export interface SegmentDrawActionCursorUpdateEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "cursor-update";

    vertexIndex: number;

    vertices: number[][];
  }

  export interface SegmentDrawActionDrawCompleteEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "draw-complete";

    vertices: number[][];
  }

  export interface SegmentDrawActionVertexAddEvent {
    defaultPrevented: boolean;

    preventDefault: Function;

    type: "vertex-add";

    vertexIndex: number;

    vertices: number[][];
  }

  export class GroundView extends Accessor {
    /**
     * This class represents the view for the [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html) of a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-GroundView.html)
     */

    constructor(properties?: GroundViewProperties);

    /**
     * An elevation sampler that may be used to sample and query elevation values from the ground surface that is currently being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-GroundView.html#elevationSampler)
     */
    readonly elevationSampler: ElevationSampler;
    /**
     * The extent represents the visible portion of ground within the view as an instance of [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-GroundView.html#extent)
     */
    readonly extent: Extent;
    /**
     * A collection containing a hierarchical list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of the [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers) in the ground.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-GroundView.html#layerViews)
     */
    readonly layerViews: Collection<LayerView>;
    /**
     * Value is `true` when any of the ground layer views are updating.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-GroundView.html#updating)
     */
    readonly updating: boolean;
  }

  interface GroundViewProperties {}

  export class GamepadInputDevice extends Accessor {
    /**
     * Properties and configuration of a gamepad.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadInputDevice.html)
     */

    constructor(properties?: GamepadInputDeviceProperties);

    /**
     * The native Gamepad object exposed by the browser.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadInputDevice.html#native)
     */
    readonly native: Gamepad;
  }

  interface GamepadInputDeviceProperties {}

  export class GamepadSettings extends Accessor {
    /**
     * Gamepad input specific configuration settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadSettings.html)
     */

    constructor(properties?: GamepadSettingsProperties);

    /**
     * A readonly collection of all gamepads detected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadSettings.html#devices)
     */
    readonly devices: Collection<GamepadInputDevice>;
    /**
     * Determines what focus is required for gamepad events to be dispatched.
     *
     * @default "document"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadSettings.html#enabledFocusMode)
     */
    enabledFocusMode: "document" | "view" | "none";
  }

  interface GamepadSettingsProperties {
    /**
     * Determines what focus is required for gamepad events to be dispatched.
     *
     * @default "document"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-gamepad-GamepadSettings.html#enabledFocusMode)
     */
    enabledFocusMode?: "document" | "view" | "none";
  }

  export class Input extends Accessor {
    /**
     * Input device related configuration settings on the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-Input.html)
     */

    constructor(properties?: InputProperties);

    /**
     * Gamepad input specific configuration settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-input-Input.html#gamepad)
     */
    readonly gamepad: GamepadSettings;
  }

  interface InputProperties {}

  export class SketchLabelOptions extends Accessor {
    /**
     * The `SketchLabelOptions` allows users to configure the labels which are shown next to each segment of a graphic while sketching and editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchLabelOptions.html)
     */

    constructor(properties?: SketchLabelOptionsProperties);

    /**
     * Whether labels are shown next to each segment of the graphic being sketched.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchLabelOptions.html#enabled)
     */
    enabled: boolean;
  }

  interface SketchLabelOptionsProperties {
    /**
     * Whether labels are shown next to each segment of the graphic being sketched.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchLabelOptions.html#enabled)
     */
    enabled?: boolean;
  }

  export class SketchTooltipOptions extends Accessor {
    /**
     * The `SketchTooltipOptions` allows users to configure the tooltips which are shown while sketching and editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html)
     */

    constructor(properties?: SketchTooltipOptionsProperties);

    /**
     * Whether tooltips are shown while sketching and editing.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#enabled)
     */
    enabled: boolean;
    /**
     * A custom help message that is displayed at the bottom of the tooltip.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#helpMessage)
     */
    helpMessage: string;
    /**
     * An icon that is displayed next to the help message at the bottom of the tooltip.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#helpMessageIcon)
     */
    helpMessageIcon: string;
    /**
     * Whether users can focus the tooltip and input constraint values.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#inputEnabled)
     */
    inputEnabled: boolean;
    /**
     * The elements that are displayed within the tooltip while sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#visibleElements)
     */
    visibleElements: VisibleElements;
  }

  interface SketchTooltipOptionsProperties {
    /**
     * Whether tooltips are shown while sketching and editing.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#enabled)
     */
    enabled?: boolean;
    /**
     * A custom help message that is displayed at the bottom of the tooltip.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#helpMessage)
     */
    helpMessage?: string;
    /**
     * An icon that is displayed next to the help message at the bottom of the tooltip.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#helpMessageIcon)
     */
    helpMessageIcon?: string;
    /**
     * Whether users can focus the tooltip and input constraint values.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#inputEnabled)
     */
    inputEnabled?: boolean;
    /**
     * The elements that are displayed within the tooltip while sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#visibleElements)
     */
    visibleElements?: VisibleElements;
  }

  /**
   * The elements that are displayed within the tooltip while sketching and editing.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
   */
  export interface VisibleElements {
    /**
     * When set to `false`, the area is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    area?: boolean;
    /**
     * When set to `false`, the coordinate is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    coordinates?: boolean;
    /**
     * When set to `false`, the direction (or deflection angle) is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    direction?: boolean;
    /**
     * When set to `false`, the distance is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    distance?: boolean;
    /**
     * When set to `false`, the elevation is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    elevation?: boolean;
    /**
     * When set to `false`, the header at the top of the tooltip is not shown, when in input mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    header?: boolean;
    /**
     * When set to `false`, the help message at the bottom of the tooltip is not shown.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    helpMessage?: boolean;
    /**
     * When set to `false`, the orientation is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    orientation?: boolean;
    /**
     * When set to `false`, the radius is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    radius?: boolean;
    /**
     * When set to `false`, the rotation is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    rotation?: boolean;
    /**
     * When set to `false`, the scale is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    scale?: boolean;
    /**
     * When set to `false`, the size is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    size?: boolean;
    /**
     * When set to `false`, the total length is not shown in the tooltip.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchTooltipOptions.html#VisibleElements)
     */
    totalLength?: boolean;
  }

  export class SketchValueOptions extends Accessor {
    /**
     * The `SketchValueOptions` allows users to configure how numerical values and constraints behave while sketching and editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html)
     */

    constructor(properties?: SketchValueOptionsProperties);

    /**
     * How direction values are displayed and, in case of input, how they are interpreted.
     *
     * @default "relative"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#directionMode)
     */
    directionMode: "relative" | "absolute";
    /**
     * Units used for displaying values in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#displayUnits)
     */
    displayUnits: Units;
    /**
     * Units used for interpreting input values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#inputUnits)
     */
    inputUnits: Units;
  }

  interface SketchValueOptionsProperties {
    /**
     * How direction values are displayed and, in case of input, how they are interpreted.
     *
     * @default "relative"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#directionMode)
     */
    directionMode?: "relative" | "absolute";
    /**
     * Units used for displaying values in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#displayUnits)
     */
    displayUnits?: Units;
    /**
     * Units used for interpreting input values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#inputUnits)
     */
    inputUnits?: Units;
  }

  /**
   * Units used for displaying or interpreting values in the UI.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#Units)
   */
  export interface Units {
    /**
     * Length units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#Units)
     */
    length?: SystemOrLengthUnit;
    /**
     * Vertical length units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#Units)
     */
    verticalLength?: SystemOrLengthUnit;
    /**
     * Area units.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-sketch-SketchValueOptions.html#Units)
     */
    area?: SystemOrAreaUnit;
  }

  export class FeatureSnappingLayerSource extends Accessor {
    /**
     * The FeatureSnappingLayerSource specifies which layers will be utilized for snapping in the [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html)
     */

    constructor(properties?: FeatureSnappingLayerSourceProperties);

    /**
     * Indicates whether feature snapping is turned on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html#enabled)
     */
    enabled: boolean;
    /**
     * The source layer used for snapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html#layer)
     */
    readonly layer:
      | BuildingSceneLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | GraphicsLayer
      | MapNotesLayer
      | SceneLayer
      | SubtypeSublayer
      | WFSLayer;
  }

  interface FeatureSnappingLayerSourceProperties {
    /**
     * Indicates whether feature snapping is turned on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html#enabled)
     */
    enabled?: boolean;
    /**
     * The source layer used for snapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html#layer)
     */
    layer?:
      | (BuildingSceneLayerProperties & { type: "building-scene" })
      | (CSVLayerProperties & { type: "csv" })
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (GraphicsLayerProperties & { type: "graphics" })
      | (MapNotesLayerProperties & { type: "map-notes" })
      | (SceneLayerProperties & { type: "scene" })
      | (SubtypeSublayerProperties & { type: "subtype-sublayer" })
      | (WFSLayerProperties & { type: "wfs" });
  }

  export class SnappingOptions extends Accessor {
    /**
     * The `SnappingOptions` allows users to configure snapping for their editing or drawing experience in both the [Sketch](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#snappingOptions) and [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#snappingOptions) widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html)
     */

    constructor(properties?: SnappingOptionsProperties);

    /**
     * When true, enables support for attribute rule-based snapping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#attributeRulesEnabled)
     */
    attributeRulesEnabled: boolean;
    /**
     * Snapping distance for snapping in pixels.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#distance)
     */
    distance: number;
    /**
     * Global configuration to turn snapping on or off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#enabled)
     */
    enabled: boolean;
    /**
     * Global configuration option to turn feature snapping on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#featureEnabled)
     */
    featureEnabled: boolean;
    /**
     * List of sources for feature snapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#featureSources)
     */
    featureSources: Collection<FeatureSnappingLayerSource>;
    /**
     * Turns the grid on or off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#gridEnabled)
     */
    gridEnabled: boolean;
    /**
     * Global configuration option to turn self snapping (within one feature while either drawing or reshaping) on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#selfEnabled)
     */
    selfEnabled: boolean;
  }

  interface SnappingOptionsProperties {
    /**
     * When true, enables support for attribute rule-based snapping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#attributeRulesEnabled)
     */
    attributeRulesEnabled?: boolean;
    /**
     * Snapping distance for snapping in pixels.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#distance)
     */
    distance?: number;
    /**
     * Global configuration to turn snapping on or off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#enabled)
     */
    enabled?: boolean;
    /**
     * Global configuration option to turn feature snapping on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#featureEnabled)
     */
    featureEnabled?: boolean;
    /**
     * List of sources for feature snapping.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#featureSources)
     */
    featureSources?: CollectionProperties<FeatureSnappingLayerSourceProperties>;
    /**
     * Turns the grid on or off.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#gridEnabled)
     */
    gridEnabled?: boolean;
    /**
     * Global configuration option to turn self snapping (within one feature while either drawing or reshaping) on or off.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html#selfEnabled)
     */
    selfEnabled?: boolean;
  }

  export class Tooltip extends Accessor {
    /**
     * Object which represents a tooltip that is shown while sketching and editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-Tooltip.html)
     */

    constructor(properties?: TooltipProperties);

    /**
     * The current mode the tooltip is in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-Tooltip.html#mode)
     */
    readonly mode: "feedback" | "input";
  }

  interface TooltipProperties {}

  export class BuildingComponentSublayerView extends Accessor {
    /**
     * Represents the sublayer view of a [BuildingComponentSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html)
     */

    constructor(properties?: BuildingComponentSublayerViewProperties);

    /**
     * A list of attribute fields fetched for each feature including fields required for layer [rendering](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#renderer) and additional fields defined on the [BuildingComponentSublayer.outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#outFields) or [BuildingSceneLayer.outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#availableFields)
     */
    readonly availableFields: string[];
    /**
     * Applies a client-side [FeatureFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) to the displayed data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#filter)
     */
    filter: FeatureFilter;
    /**
     * The sublayer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#sublayer)
     */
    readonly sublayer: BuildingComponentSublayer;
    /**
     * Value is `true` if the sublayer is suspended (i.e., sublayer will not redraw or update itself when the extent changes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#suspended)
     */
    readonly suspended: boolean;
    /**
     * Value is `true` when the sublayer is updating; for example, if it is in the process of fetching data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#updating)
     */
    readonly updating: boolean;

    /**
     * Creates a query parameter object that can be used to fetch features as they are being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Highlights the given feature(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#highlight)
     */
    highlight(target?: Graphic | Graphic[] | number | number[]): Handle;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the 3D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the 3D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query. If query parameters are not provided, the extent and count of all loaded features are returned.  Read more about queries in the Query section of the [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) class description.  To query for the extent of features directly from a Scene Service rather than those loaded for the current view, you must use the [BuildingComponentSublayer.queryExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryExtent) method.  For making attribute based queries on a BuildingComponentSublayerView you need to specify the required fields in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#outFields) property of the BuildingComponentSublayer to ensure that attribute values are available on the client for querying. You can use [availableFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#availableFields) to inspect which fields are available on the client. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryExtent)
     */
    queryExtent(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerViewQueryExtentOptions,
    ): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the number of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the number of features that satisfy the query. If query parameters are not provided, the count of all loaded features is returned.  To query for the count of features directly from a Scene Service rather than those loaded for the current view, you must use the [BuildingComponentSublayer.queryFeatureCount()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#queryFeatureCount) method.  For making attribute based queries on a BuildingSceneLayerView you need to specify the required fields in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html#outFields) property of the BuildingComponentSublayer to ensure that attribute values are available on the client for querying. You can use [availableFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#availableFields) to inspect which fields are available on the client. Read more about queries in the Query section of the [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) class description. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerViewQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerViewQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns an array of the ObjectIDs of features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: BuildingComponentSublayerViewQueryObjectIdsOptions,
    ): Promise<number[]>;
  }

  interface BuildingComponentSublayerViewProperties {
    /**
     * Applies a client-side [FeatureFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) to the displayed data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#filter)
     */
    filter?: FeatureFilterProperties;
  }

  export interface BuildingComponentSublayerViewQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerViewQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerViewQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface BuildingComponentSublayerViewQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export class BuildingSceneLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingSceneLayerView.html)
     */

    constructor(properties?: BuildingSceneLayerViewProperties);

    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingSceneLayerView.html#layer)
     */
    readonly layer: BuildingSceneLayer;
    /**
     * Collection of [sublayer views](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingComponentSublayerView.html) for all the component sublayers of the BuildingSceneLayer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingSceneLayerView.html#sublayerViews)
     */
    readonly sublayerViews: Collection<BuildingComponentSublayerView>;

    /**
     * Highlights the given feature(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-BuildingSceneLayerView.html#highlight)
     */
    highlight(target?: Graphic | Graphic[]): Handle;
  }

  interface BuildingSceneLayerViewProperties extends LayerViewProperties {}

  export class CatalogDynamicGroupLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [CatalogLayer's dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer) after the CatalogLayer has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogDynamicGroupLayerView.html)
     */

    constructor(properties?: CatalogDynamicGroupLayerViewProperties);

    /**
     * A collection of layer views within the CatalogDynamicGroupLayerView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogDynamicGroupLayerView.html#layerViews)
     */
    readonly layerViews: Collection<LayerView>;
  }

  interface CatalogDynamicGroupLayerViewProperties extends LayerViewProperties {}

  export interface CatalogFootprintLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class CatalogFootprintLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [CatalogLayer's footprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#footprintLayer) after the CatalogLayer has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogFootprintLayerView.html)
     */

    constructor(properties?: CatalogFootprintLayerViewProperties);
  }

  interface CatalogFootprintLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class CatalogLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogLayerView.html)
     */

    constructor(properties?: CatalogLayerViewProperties);

    /**
     * The [CatalogDynamicGroupLayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogDynamicGroupLayerView.html) of the [CatalogLayer's dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogLayerView.html#dynamicGroupLayerView)
     */
    readonly dynamicGroupLayerView: CatalogDynamicGroupLayerView;
    /**
     * The [CatalogFootprintLayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogFootprintLayerView.html) of the [CatalogLayer's footprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#footprintLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogLayerView.html#footprintLayerView)
     */
    readonly footprintLayerView: CatalogFootprintLayerView;
    /**
     * A collection of [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) instances associated with this layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CatalogLayerView.html#layerViews)
     */
    readonly layerViews: Collection<LayerView>;
  }

  interface CatalogLayerViewProperties extends LayerViewProperties {}

  export interface CSVLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class CSVLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-CSVLayerView.html)
     */

    constructor(properties?: CSVLayerViewProperties);
  }

  interface CSVLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class DimensionLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [DimensionLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html)
     */

    constructor(properties?: DimensionLayerViewProperties);

    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#interactive)
     */
    interactive: boolean;
    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#layer)
     */
    readonly layer: DimensionLayer;
    /**
     * Results for each dimension in the layer [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#results)
     */
    readonly results: Collection<LengthDimensionResult>;
    /**
     * The selected dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#selectedDimension)
     */
    selectedDimension: LengthDimension;

    /**
     * Starts the interactive creation of new dimensions and adds them to the layer [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-DimensionLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#createLengthDimensions)
     */
    createLengthDimensions(options?: DimensionLayerViewCreateLengthDimensionsOptions): Promise<any>;
  }

  interface DimensionLayerViewProperties extends LayerViewProperties {
    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#interactive)
     */
    interactive?: boolean;
    /**
     * The selected dimension.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#selectedDimension)
     */
    selectedDimension?: LengthDimensionProperties;
  }

  export interface DimensionLayerViewCreateLengthDimensionsOptions {
    /**
     * Abort signal which can be used to cancel creation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-DimensionLayerView.html#createLengthDimensions)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class FeatureLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerView.html)
     */

    constructor(properties?: FeatureLayerViewProperties);

    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerView.html#layer)
     */
    readonly layer: FeatureLayer;
  }

  interface FeatureLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class FeatureLayerViewMixin {
    /**
     * A list of attribute fields fetched for each feature including fields required for layer's `renderer` `labelingInfo`, `elevationInfo`, and additional fields defined on the `outFields` properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#availableFields)
     */
    readonly availableFields: string[];
    /**
     * Indicates whether the layer view is currently fetching new features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#dataUpdating)
     */
    readonly dataUpdating: boolean;
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#featureEffect)
     */
    featureEffect: FeatureEffect;
    /**
     * The [attribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where), [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry), and [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent) filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#filter)
     */
    filter: FeatureFilter;
    /**
     * Indicates whether the layer view contains all available features from the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#hasAllFeatures)
     */
    readonly hasAllFeatures: boolean;
    /**
     * Indicates whether the layer view has successfully queried all of its features for the current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#hasAllFeaturesInView)
     */
    readonly hasAllFeaturesInView: boolean;
    /**
     * Indicates whether the layer view has geometries at full resolution.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#hasFullGeometries)
     */
    readonly hasFullGeometries: boolean;
    /**
     * The maximum number of features that can be displayed at a time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#maximumNumberOfFeatures)
     */
    maximumNumberOfFeatures: number;
    /**
     * Signifies whether the maximum number of features has been exceeded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#maximumNumberOfFeaturesExceeded)
     */
    maximumNumberOfFeaturesExceeded: boolean;

    /**
     * Creates query parameter object that can be used to [fetch aggregate features](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryAggregates) as they are being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#createAggregateQuery)
     */
    createAggregateQuery(): Query;
    /**
     * Creates a query parameter object that can be used to fetch features as they are being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against aggregate features (i.e.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryAggregates)
     */
    queryAggregates(
      query?: Query | QueryProperties,
      options?: FeatureLayerViewMixinQueryAggregatesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: FeatureLayerViewMixinQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: FeatureLayerViewMixinQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryFeatures)
     */
    queryFeatures(
      query?: Query | QueryProperties,
      options?: FeatureLayerViewMixinQueryFeaturesOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns array of the ObjectIDs of features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryObjectIds)
     */
    queryObjectIds(
      query?: Query | QueryProperties,
      options?: FeatureLayerViewMixinQueryObjectIdsOptions,
    ): Promise<number[]>;
  }

  interface FeatureLayerViewMixinProperties {
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#featureEffect)
     */
    featureEffect?: FeatureEffectProperties;
    /**
     * The [attribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where), [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry), and [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent) filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#filter)
     */
    filter?: FeatureFilterProperties;
    /**
     * The maximum number of features that can be displayed at a time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#maximumNumberOfFeatures)
     */
    maximumNumberOfFeatures?: number;
    /**
     * Signifies whether the maximum number of features has been exceeded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#maximumNumberOfFeaturesExceeded)
     */
    maximumNumberOfFeaturesExceeded?: boolean;
  }

  export interface FeatureLayerViewMixinQueryAggregatesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryAggregates)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerViewMixinQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerViewMixinQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerViewMixinQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface FeatureLayerViewMixinQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-FeatureLayerViewMixin.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface GeoJSONLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class GeoJSONLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GeoJSONLayerView.html)
     */

    constructor(properties?: GeoJSONLayerViewProperties);
  }

  interface GeoJSONLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class GeoRSSLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [GeoRSSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoRSSLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GeoRSSLayerView.html)
     */

    constructor(properties?: GeoRSSLayerViewProperties);
  }

  interface GeoRSSLayerViewProperties extends LayerViewProperties {}

  export interface GraphicsLayerView extends LayerView, HighlightLayerViewMixin {}

  export class GraphicsLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GraphicsLayerView.html)
     */

    constructor(properties?: GraphicsLayerViewProperties);

    /**
     * Returns all graphics available for drawing in the layer view as a [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GraphicsLayerView.html#queryGraphics)
     */
    queryGraphics(): Promise<Collection<Graphic>>;
  }

  interface GraphicsLayerViewProperties extends LayerViewProperties, HighlightLayerViewMixinProperties {}

  export class GroupLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [GroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GroupLayerView.html)
     */

    constructor(properties?: GroupLayerViewProperties);

    /**
     * A collection of layer views within the GroupLayerView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GroupLayerView.html#layerViews)
     */
    layerViews: Collection;
  }

  interface GroupLayerViewProperties extends LayerViewProperties {
    /**
     * A collection of layer views within the GroupLayerView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-GroupLayerView.html#layerViews)
     */
    layerViews?: CollectionProperties;
  }

  export class HighlightLayerViewMixin {
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-HighlightLayerViewMixin.html#highlightOptions)
     */
    highlightOptions: HighlightOptions;

    /**
     * Highlights the given feature(s) in a layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-HighlightLayerViewMixin.html#highlight)
     */
    highlight(target?: Graphic | Graphic[] | number | number[] | string | string[]): Handle;
  }

  interface HighlightLayerViewMixinProperties {
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-HighlightLayerViewMixin.html#highlightOptions)
     */
    highlightOptions?: HighlightOptions;
  }

  export class ImageryLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of an [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html)
     */

    constructor(properties?: ImageryLayerViewProperties);

    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#highlightOptions)
     */
    highlightOptions: HighlightOptions;
    /**
     * An object that provides the user access to [pixels](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels) and their values in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#pixelData)
     */
    pixelData: PixelData;

    /**
     * Highlights the given feature(s) in an ImageryLayerView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#highlight)
     */
    highlight(target?: Graphic | Graphic[]): Handle;
  }

  interface ImageryLayerViewProperties extends LayerViewProperties {
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#highlightOptions)
     */
    highlightOptions?: HighlightOptions;
    /**
     * An object that provides the user access to [pixels](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-PixelBlock.html#pixels) and their values in the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ImageryLayerView.html#pixelData)
     */
    pixelData?: PixelData;
  }

  export class KMLLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [KMLLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KMLLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html)
     */

    constructor(properties?: KMLLayerViewProperties);

    /**
     * A collection of all the [MapImages](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage) from visible sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#allVisibleMapImages)
     */
    readonly allVisibleMapImages: Collection<KMLLayerViewMapImage>;
    /**
     * A collection of graphics representing all the points from visible sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#allVisiblePoints)
     */
    readonly allVisiblePoints: Collection<Graphic>;
    /**
     * A collection of graphics representing all the polygons from visible sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#allVisiblePolygons)
     */
    readonly allVisiblePolygons: Collection<Graphic>;
    /**
     * A collection of graphics representing all the polylines from visible sublayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#allVisiblePolylines)
     */
    readonly allVisiblePolylines: Collection<Graphic>;
  }

  interface KMLLayerViewProperties extends LayerViewProperties {}

  /**
   * **MapImage** represents an image overlay draped onto the terrain.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage)
   */
  export interface KMLLayerViewMapImage {
    /**
     * Map image id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage)
     */
    id: number;
    /**
     * URL to the map image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage)
     */
    href: string;
    /**
     * The [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of the map image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage)
     */
    Extent: Extent;
    /**
     * Rotation of the map image about its center, in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-KMLLayerView.html#MapImage)
     */
    rotation: number;
  }

  export interface LayerView extends Accessor, corePromise, Identifiable {}

  export class LayerView {
    /**
     * Represents the view for a single layer after it has been added to either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html)
     */

    constructor(properties?: LayerViewProperties);

    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#layer)
     */
    readonly layer: Layer;
    /**
     * Indicates if the  [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#spatialReference) of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) is supported by the layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#spatialReferenceSupported)
     */
    readonly spatialReferenceSupported: boolean;
    /**
     * Value is `true` if the layer is suspended (i.e., layer will not redraw or update itself when the extent changes).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#suspended)
     */
    readonly suspended: boolean;
    /**
     * Indicates if the layer view is making any updates that will impact what is displayed on the map.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#updating)
     */
    readonly updating: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) associated with the layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#view)
     */
    readonly view: MapView | SceneView;
    /**
     * When `true`, the layer is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#visible)
     */
    visible: boolean;
    /**
     * When `true`, the layer is visible in the view at the current scale.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#visibleAtCurrentScale)
     */
    readonly visibleAtCurrentScale: boolean;
    /**
     * When `true`, the layer is visible in the view's [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#visibleAtCurrentTimeExtent)
     */
    readonly visibleAtCurrentTimeExtent: boolean;
  }

  interface LayerViewProperties {
    /**
     * When `true`, the layer is visible in the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html#visible)
     */
    visible?: boolean;
  }

  export class MediaLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html)
     */

    constructor(properties?: MediaLayerViewProperties);

    /**
     * Options for when the layer view is interactive.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactionOptions)
     */
    interactionOptions: MediaLayerViewInteractionOptions;
    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactive)
     */
    interactive: boolean;
    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#layer)
     */
    readonly layer: MediaLayer;
    /**
     * The selected element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#selectedElement)
     */
    selectedElement: ImageElement | VideoElement;
  }

  interface MediaLayerViewProperties extends LayerViewProperties {
    /**
     * Options for when the layer view is interactive.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactionOptions)
     */
    interactionOptions?: MediaLayerViewInteractionOptionsProperties;
    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactive)
     */
    interactive?: boolean;
    /**
     * The selected element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#selectedElement)
     */
    selectedElement?: (ImageElementProperties & { type: "image" }) | (VideoElementProperties & { type: "video" });
  }

  export interface MediaLayerViewInteractionOptionsProperties {
    /**
     * The type of tool to use to interact with media elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactionOptions)
     */
    tool?: "transform" | "reshape";
  }

  export interface MediaLayerViewInteractionOptions extends AnonymousAccessor {
    /**
     * The type of tool to use to interact with media elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-MediaLayerView.html#interactionOptions)
     */
    tool?: "transform" | "reshape";
  }

  export interface OGCFeatureLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class OGCFeatureLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [OGCFeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-OGCFeatureLayerView.html)
     */

    constructor(properties?: OGCFeatureLayerViewProperties);
  }

  interface OGCFeatureLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class PointCloudLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [PointCloudLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-PointCloudLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-PointCloudLayerView.html)
     */

    constructor(properties?: PointCloudLayerViewProperties);

    /**
     * A list of attribute fields fetched for each feature including fields required for layer [rendering](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#renderer) and additional fields defined on the [SceneLayer.outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-PointCloudLayerView.html#availableFields)
     */
    readonly availableFields: string[];

    /**
     * Highlights the given point(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-PointCloudLayerView.html#highlight)
     */
    highlight(target?: Graphic | Graphic[]): Handle;
  }

  interface PointCloudLayerViewProperties extends LayerViewProperties {}

  export class SceneLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html)
     */

    constructor(properties?: SceneLayerViewProperties);

    /**
     * A list of attribute fields fetched for each feature including fields required for layer [rendering](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#renderer) and additional fields defined on the [SceneLayer.outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#availableFields)
     */
    readonly availableFields: string[];
    /**
     * Applies a client-side [FeatureFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) to the displayed data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#filter)
     */
    filter: FeatureFilter;
    /**
     * The scene layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#layer)
     */
    readonly layer: SceneLayer;
    /**
     * The maximum number of features that can be displayed at a time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#maximumNumberOfFeatures)
     */
    maximumNumberOfFeatures: number;
    /**
     * Signifies whether the maximum number of features has been exceeded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#maximumNumberOfFeaturesExceeded)
     */
    maximumNumberOfFeaturesExceeded: boolean;

    /**
     * Creates query parameter object that can be used to fetch features as they are being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Highlights the given feature(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#highlight)
     */
    highlight(target?: Graphic | Graphic[] | number | number[]): Handle;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the 3D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the 3D [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query. If query parameters are not provided, the extent and count of all loaded features are returned.  Read more about queries in the Query section of the [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) class description.  To query for the extent of features directly from a Scene Service rather than those loaded for the current view, you must use the [SceneLayer.queryExtent()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryExtent) method.  For making attribute based queries on a SceneLayerView you need to specify the required fields in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields) property of the SceneLayer to ensure that attribute values are available on the client for querying. You can use [availableFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#availableFields) to inspect which fields are available on the client. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: SceneLayerViewQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the number of features that satisfy the query.
     *
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns the number of features that satisfy the query. If query parameters are not provided, the count of all loaded features is returned. Read more about queries in the Query section of the [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) class description.  To query for the count of features directly from a Scene Service rather than those loaded for the current view, you must use the [SceneLayer.queryFeatureCount()](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#queryFeatureCount) method.  For making attribute based queries on a SceneLayerView you need to specify the required fields in the [outFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html#outFields) property of the SceneLayer to ensure that attribute values are available on the client for querying. You can use [availableFields](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#availableFields) to inspect which fields are available on the client. [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: SceneLayerViewQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: SceneLayerViewQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features in the layer view and returns an array of the ObjectIDs of features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: SceneLayerViewQueryObjectIdsOptions): Promise<number[]>;
  }

  interface SceneLayerViewProperties extends LayerViewProperties {
    /**
     * Applies a client-side [FeatureFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html) to the displayed data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#filter)
     */
    filter?: FeatureFilterProperties;
    /**
     * The maximum number of features that can be displayed at a time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#maximumNumberOfFeatures)
     */
    maximumNumberOfFeatures?: number;
    /**
     * Signifies whether the maximum number of features has been exceeded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#maximumNumberOfFeaturesExceeded)
     */
    maximumNumberOfFeaturesExceeded?: boolean;
  }

  export interface SceneLayerViewQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerViewQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerViewQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface SceneLayerViewQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-SceneLayerView.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerView extends LayerView, HighlightLayerViewMixin, Evented {}

  export class StreamLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [StreamLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html)
     */

    constructor(properties?: StreamLayerViewProperties);

    /**
     * The error that explains an unsuccessful attempt to connect to the stream service or an unexpected disconnection from the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#connectionError)
     */
    readonly connectionError: Error;
    /**
     * The status of the Web Socket connection with the stream service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#connectionStatus)
     */
    readonly connectionStatus: "connected" | "disconnected" | "paused";
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#featureEffect)
     */
    featureEffect: FeatureEffect;
    /**
     * The [attribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where), [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry), and [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent) filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#filter)
     */
    filter: FeatureFilter;

    /**
     * Connects to a stream service web socket.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#connect)
     */
    connect(): void;
    /**
     * Creates a query parameter object that can be used to fetch features as they are being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#createQuery)
     */
    createQuery(): Query;
    /**
     * Disconnects from a stream service web socket.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#disconnect)
     */
    disconnect(): void;
    /**
     * Pauses the connection and stops new observations from being applied until [resume()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#resume) is called.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#pause)
     */
    pause(): void;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns the [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryExtent)
     */
    queryExtent(query?: Query | QueryProperties, options?: StreamLayerViewQueryExtentOptions): Promise<any>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns the number of features that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryFeatureCount)
     */
    queryFeatureCount(
      query?: Query | QueryProperties,
      options?: StreamLayerViewQueryFeatureCountOptions,
    ): Promise<number>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryFeatures)
     */
    queryFeatures(query?: Query | QueryProperties, options?: StreamLayerViewQueryFeaturesOptions): Promise<FeatureSet>;
    /**
     * If a [trackIdField](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-TimeInfo.html#trackIdField) is specified on the stream service, this method executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns a [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html) of the latest observations for each `trackId` that satisfy the query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryLatestObservations)
     */
    queryLatestObservations(
      query?: Query | QueryProperties,
      options?: StreamLayerViewQueryLatestObservationsOptions,
    ): Promise<FeatureSet>;
    /**
     * Executes a [Query](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-Query.html) against features available for drawing in the layerView and returns array of the ObjectIDs of features that satisfy the input query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryObjectIds)
     */
    queryObjectIds(query?: Query | QueryProperties, options?: StreamLayerViewQueryObjectIdsOptions): Promise<number[]>;
    /**
     * Resumes the connection and the new observations will be applied.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#resume)
     */
    resume(): void;

    on(name: "data-received", eventHandler: StreamLayerViewDataReceivedEventHandler): IHandle;

    on(name: "message-received", eventHandler: StreamLayerViewMessageReceivedEventHandler): IHandle;

    on(name: "update-rate", eventHandler: StreamLayerViewUpdateRateEventHandler): IHandle;
  }

  interface StreamLayerViewProperties extends LayerViewProperties, HighlightLayerViewMixinProperties {
    /**
     * The featureEffect can be used to draw attention features of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#featureEffect)
     */
    featureEffect?: FeatureEffectProperties;
    /**
     * The [attribute](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#where), [geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#geometry), and [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureFilter.html#timeExtent) filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#filter)
     */
    filter?: FeatureFilterProperties;
  }

  export interface StreamLayerViewDataReceivedEvent {
    attributes: any;

    geometry: any;
  }

  export interface StreamLayerViewMessageReceivedEvent {
    message: any;
  }

  export interface StreamLayerViewQueryExtentOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryExtent)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerViewQueryFeatureCountOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryFeatureCount)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerViewQueryFeaturesOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryFeatures)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerViewQueryLatestObservationsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryLatestObservations)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerViewQueryObjectIdsOptions {
    /**
     * Signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-StreamLayerView.html#queryObjectIds)
     */
    signal?: AbortSignal;
  }

  export interface StreamLayerViewUpdateRateEvent {
    client: number;

    websocket: number;
  }

  export class VectorTileLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [VectorTileLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VectorTileLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-VectorTileLayerView.html)
     */

    constructor(properties?: VectorTileLayerViewProperties);

    /**
     * When `false`, it indicates that tiles have no visible features within the current extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-VectorTileLayerView.html#hasVisibleFeatures)
     */
    readonly hasVisibleFeatures: boolean;
  }

  interface VectorTileLayerViewProperties extends LayerViewProperties {}

  export class ViewshedLayerView extends LayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [ViewshedLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html)
     */

    constructor(properties?: ViewshedLayerViewProperties);

    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#interactive)
     */
    interactive: boolean;
    /**
     * The layer being viewed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#layer)
     */
    readonly layer: ViewshedLayer;
    /**
     * The selected viewshed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#selectedViewshed)
     */
    selectedViewshed: Viewshed;

    /**
     * Starts the interactive creation of new viewsheds and adds them to the layer [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ViewshedLayer.html#source).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#createViewsheds)
     */
    createViewsheds(options?: ViewshedLayerViewCreateViewshedsOptions): Promise<any>;
  }

  interface ViewshedLayerViewProperties extends LayerViewProperties {
    /**
     * Set to `true` to enable interactivity for the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#layer).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#interactive)
     */
    interactive?: boolean;
    /**
     * The selected viewshed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#selectedViewshed)
     */
    selectedViewshed?: ViewshedProperties;
  }

  export interface ViewshedLayerViewCreateViewshedsOptions {
    /**
     * Abort signal which can be used to cancel creation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-ViewshedLayerView.html#createViewsheds)
     */
    signal?: AbortSignal;
  }

  export interface WFSLayerView extends LayerView, FeatureLayerViewMixin, HighlightLayerViewMixin {}

  export class WFSLayerView {
    /**
     * Represents the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of a [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) after it has been added to a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-WFSLayerView.html)
     */

    constructor(properties?: WFSLayerViewProperties);
  }

  interface WFSLayerViewProperties
    extends LayerViewProperties,
      FeatureLayerViewMixinProperties,
      HighlightLayerViewMixinProperties {}

  export class LinkChartView extends View2D {
    /**
     * LinkChartView is a [2D view](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html) that displays a [WebLinkChart](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebLinkChart.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-LinkChartView.html)
     */

    constructor(properties?: LinkChartViewProperties);

    /**
     * The [WebLinkChart](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebLinkChart.html) displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-LinkChartView.html#map)
     */
    map: WebLinkChart;
  }

  interface LinkChartViewProperties extends View2DProperties {
    /**
     * The [WebLinkChart](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebLinkChart.html) displayed in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-LinkChartView.html#map)
     */
    map?: WebLinkChartProperties;
  }

  export class Magnifier extends Accessor {
    /**
     * The Magnifier allows end users to show a portion of the view as a magnified image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html)
     */

    constructor(properties?: MagnifierProperties);

    /**
     * Controls the amount of magnification to display.
     *
     * @default 1.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#factor)
     */
    factor: number;
    /**
     * Indicates whether the mask image is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#maskEnabled)
     */
    maskEnabled: boolean;
    /**
     * The mask url points to an image that determines the visible area of the magnified image [(alpha channel)](https://developer.mozilla.org/en-US/docs/Glossary/Alpha).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#maskUrl)
     */
    maskUrl: string;
    /**
     * The offset of the magnifier in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#offset)
     */
    offset: MagnifierScreenPoint;
    /**
     * Indicates whether the overlay image is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#overlayEnabled)
     */
    overlayEnabled: boolean;
    /**
     * The overlay url points to an image that is displayed on top of the magnified image.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#overlayUrl)
     */
    overlayUrl: string;
    /**
     * The position of the magnifier in pixels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#position)
     */
    position: MagnifierScreenPoint;
    /**
     * The size of the magnifier in pixels.
     *
     * @default 120
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#size)
     */
    size: number;
    /**
     * Indicates whether the magnifier is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#visible)
     */
    visible: boolean;
  }

  interface MagnifierProperties {
    /**
     * Controls the amount of magnification to display.
     *
     * @default 1.5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#factor)
     */
    factor?: number;
    /**
     * Indicates whether the mask image is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#maskEnabled)
     */
    maskEnabled?: boolean;
    /**
     * The mask url points to an image that determines the visible area of the magnified image [(alpha channel)](https://developer.mozilla.org/en-US/docs/Glossary/Alpha).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#maskUrl)
     */
    maskUrl?: string;
    /**
     * The offset of the magnifier in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#offset)
     */
    offset?: MagnifierScreenPoint;
    /**
     * Indicates whether the overlay image is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#overlayEnabled)
     */
    overlayEnabled?: boolean;
    /**
     * The overlay url points to an image that is displayed on top of the magnified image.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#overlayUrl)
     */
    overlayUrl?: string;
    /**
     * The position of the magnifier in pixels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#position)
     */
    position?: MagnifierScreenPoint;
    /**
     * The size of the magnifier in pixels.
     *
     * @default 120
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#size)
     */
    size?: number;
    /**
     * Indicates whether the magnifier is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#visible)
     */
    visible?: boolean;
  }

  /**
   * An object representing the location on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#ScreenPoint)
   */
  export interface MagnifierScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Magnifier.html#ScreenPoint)
     */
    y: number;
  }

  export class MapView extends View2D {
    /**
     * A MapView displays a 2D view of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html)
     */

    constructor(properties?: MapViewProperties);

    /**
     * Returns [hit test results](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult) from each layer that intersects the specified screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest)
     */
    hitTest(screenPoint: MapViewScreenPoint | MouseEvent, options?: MapViewHitTestOptions): Promise<HitTestResult>;
    /**
     * Create a screenshot of the current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    takeScreenshot(options?: MapViewTakeScreenshotOptions): Promise<Screenshot>;
  }

  interface MapViewProperties extends View2DProperties {}

  /**
   * Animation options for the [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
   */
  export interface GoToOptions2D {
    /**
     * Indicates whether the transition to the new viewpoint is animated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    animate?: boolean;
    /**
     * **Since 4.30.** The strategy for deciding whether the transition will animate when `animate` is not `false`, and the user has not expressed a preference for reduced motion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    animationMode?: "always" | "auto";
    /**
     * **Since 4.30.** Increases or decreases the animation speed by the specified factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    speedFactor?: number;
    /**
     * The specified duration of the animation, in milliseconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    duration?: number;
    /**
     * **Since 4.30.** The maximum allowed duration (in milliseconds) of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    maxDuration?: number;
    /**
     * The easing function used for the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    easing?: "linear" | "ease" | "ease-in" | "ease-out" | "ease-in-out" | Function;
    /**
     * **Since 4.28.** Indicates if the target geometry is normalized based on the [center](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#center) of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    pickClosestTarget?: boolean;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToOptions2D)
     */
    signal?: AbortSignal;
  }

  /**
   * The target location/viewpoint to animate to in the [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GoToTarget2D)
   */
  export type GoToTarget2D =
    | number[]
    | Geometry
    | Geometry[]
    | Collection<Geometry>
    | Graphic
    | Graphic[]
    | Collection<Graphic>
    | Viewpoint
    | any;

  /**
   * Object specification for the graphic hit result returned in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GraphicHit)
   */
  export interface MapViewGraphicHit {
    type: "graphic";
    /**
     * A graphic representing a feature in the view that intersects the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GraphicHit)
     */
    graphic: Graphic;
    /**
     * The layer that contains the feature/graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GraphicHit)
     */
    layer: Layer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#GraphicHit)
     */
    mapPoint: Point;
  }

  /**
   * A layer, graphic, or collection of layers or graphics to be used in the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) filter options.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestItem)
   */
  export type HitTestItem = Layer | Graphic | Collection<Layer> | Collection<Graphic> | Layer[] | Graphic[];

  /**
   * Object specification for the result of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult)
   */
  export interface HitTestResult {
    /**
     * An array of result objects returned from the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult)
     */
    results: MapViewViewHit[];
    /**
     * The screen coordinates (or native mouse event) of the click on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult)
     */
    screenPoint: MapViewScreenPoint;
  }

  export interface MapViewHitTestOptions {
    /**
     * A list of layers and graphics to include in the hitTest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest)
     */
    include?: HitTestItem[] | Collection<HitTestItem> | Layer | Graphic;
    /**
     * A list of layers and graphics to exclude from the hitTest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest)
     */
    exclude?: HitTestItem[] | Collection<HitTestItem> | Layer | Graphic;
  }

  export interface MapViewTakeScreenshotOptions {
    /**
     * The format of the resulting encoded data url.
     *
     * @default png
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    format?: "jpg" | "png";
    /**
     * When used, only the visible layers in this list will be included in the output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    layers?: Layer[];
    /**
     * The quality (0 to 100) of the encoded image when encoding as `jpg`.
     *
     * @default 98
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    quality?: number;
    /**
     * The width of the screenshot (defaults to the area width).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    width?: number;
    /**
     * The height of the screenshot (defaults to the area height).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    height?: number;
    /**
     * Specifies whether to take a screenshot of a specific area of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    area?: MapViewTakeScreenshotOptionsArea;
    /**
     * Indicates whether to ignore the background color set in the [initial view properties](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#initialViewProperties) of the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    ignoreBackground?: boolean;
    /**
     * Indicates whether view padding should be ignored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    ignorePadding?: boolean;
  }

  export interface MapViewTakeScreenshotOptionsArea {
    /**
     * The x coordinate of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    x?: number;
    /**
     * The y coordinate of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    y?: number;
    /**
     * The width of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    width?: number;
    /**
     * The height of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot)
     */
    height?: number;
  }

  /**
   * Object specification for the media hit results returned from [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#MediaHit)
   */
  export interface MapViewMediaHit {
    type: "media";
    /**
     * An element representing a media element in [MediaLayer.source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source) that intersects the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#MediaHit)
     */
    element: ImageElement | VideoElement;
    /**
     * The media layer that contains the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#MediaHit)
     */
    layer: MediaLayer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#MediaHit)
     */
    mapPoint: Point;
    /**
     * _Since 4.28_ An object representing a point on the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#MediaHit)
     */
    sourcePoint: SourcePoint;
  }

  /**
   * Object specification for the route hit results returned from [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#RouteHit)
   */
  export interface MapViewRouteHit {
    type: "route";
    /**
     * The route layer that contains the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#RouteHit)
     */
    layer: RouteLayer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#RouteHit)
     */
    mapPoint: Point;
    /**
     * The route hit test will contain all intersecting network elements, which includes one of the following: DirectionLine, DirectionPoint, PointBarrier, PolylineBarrier, PolygonBarrier, Stop, or RouteInfo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#RouteHit)
     */
    networkFeature: DirectionLine | DirectionPoint | PointBarrier | PolygonBarrier | PolygonBarrier | RouteInfo | Stop;
  }

  /**
   * An object representing a point on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ScreenPoint)
   */
  export interface MapViewScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ScreenPoint)
     */
    y: number;
  }

  /**
   * Object returned when [takeScreenshot()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#takeScreenshot) promise resolves:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#Screenshot)
   */
  export interface Screenshot {
    /**
     * A data url representing the screenshot.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#Screenshot)
     */
    dataUrl: string;
    /**
     * The raw RGBA image data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#Screenshot)
     */
    data: ImageData;
  }

  /**
   * Options for the [toScreen()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#toScreen) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ToScreenOptions2D)
   */
  export interface ToScreenOptions2D {
    /**
     * Indicates if the point will be normalized based on the [center](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#center) of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ToScreenOptions2D)
     */
    pickClosestTarget?: boolean;
  }

  /**
   * Object specification for the result returned in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#ViewHit)
   */
  export type MapViewViewHit = MapViewGraphicHit | MapViewMediaHit | MapViewRouteHit;

  export class gamepadGamepadSettings extends Accessor {
    /**
     * Gamepad navigation specific configuration settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html)
     */

    constructor(properties?: gamepadGamepadSettingsProperties);

    /**
     * Use this property to explicitly select the gamepad device for map and scene navigation.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#device)
     */
    device: GamepadInputDevice;
    /**
     * Whether gamepad navigation is enabled on the View.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#enabled)
     */
    enabled: boolean;
    /**
     * This setting controls the behavior of forward and back movement of the left stick.
     *
     * @default "pan"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#mode)
     */
    mode: "pan" | "zoom";
    /**
     * Determines whether pressing the tilt axis forwards make the view tilt down (towards the nadir), or up (towards the zenith).
     *
     * @default "forward-down"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#tiltDirection)
     */
    tiltDirection: "forward-down" | "forward-up";
  }

  interface gamepadGamepadSettingsProperties {
    /**
     * Use this property to explicitly select the gamepad device for map and scene navigation.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#device)
     */
    device?: GamepadInputDeviceProperties;
    /**
     * Whether gamepad navigation is enabled on the View.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#enabled)
     */
    enabled?: boolean;
    /**
     * This setting controls the behavior of forward and back movement of the left stick.
     *
     * @default "pan"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#mode)
     */
    mode?: "pan" | "zoom";
    /**
     * Determines whether pressing the tilt axis forwards make the view tilt down (towards the nadir), or up (towards the zenith).
     *
     * @default "forward-down"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-gamepad-GamepadSettings.html#tiltDirection)
     */
    tiltDirection?: "forward-down" | "forward-up";
  }

  export class Navigation extends Accessor {
    /**
     * Navigation related configuration settings on the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html)
     */

    constructor(properties?: NavigationProperties);

    /**
     * Indicates if single finger touch [drag](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#event-drag) events are enabled or disabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#browserTouchPanEnabled)
     */
    browserTouchPanEnabled: boolean;
    /**
     * Gamepad navigation specific configuration settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#gamepad)
     */
    gamepad: gamepadGamepadSettings;
    /**
     * When `true`, the view will temporarily continue to pan after the pointer (e.g.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#momentumEnabled)
     */
    momentumEnabled: boolean;
    /**
     * Indicates whether the view can be zoomed in or out with the mouse wheel.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#mouseWheelZoomEnabled)
     */
    mouseWheelZoomEnabled: boolean;
  }

  interface NavigationProperties {
    /**
     * Indicates if single finger touch [drag](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#event-drag) events are enabled or disabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#browserTouchPanEnabled)
     */
    browserTouchPanEnabled?: boolean;
    /**
     * Gamepad navigation specific configuration settings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#gamepad)
     */
    gamepad?: gamepadGamepadSettingsProperties;
    /**
     * When `true`, the view will temporarily continue to pan after the pointer (e.g.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#momentumEnabled)
     */
    momentumEnabled?: boolean;
    /**
     * Indicates whether the view can be zoomed in or out with the mouse wheel.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-navigation-Navigation.html#mouseWheelZoomEnabled)
     */
    mouseWheelZoomEnabled?: boolean;
  }

  export class PopupView {
    /**
     * A [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) object that displays general content or attributes from [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) in the [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#map).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#popup)
     */
    popup: Popup;
    /**
     * Controls whether the popup opens when users click on the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#popupEnabled)
     */
    popupEnabled: boolean;

    /**
     * Closes the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#closePopup)
     */
    closePopup(): void;
    /**
     * Opens the popup at the given location with content defined either explicitly with `content` or driven from the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) of input features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    openPopup(options?: PopupViewOpenPopupOptions): Promise<any>;
  }

  interface PopupViewProperties {
    /**
     * A [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) object that displays general content or attributes from [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) in the [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#map).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#popup)
     */
    popup?: PopupProperties;
    /**
     * Controls whether the popup opens when users click on the view.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#popupEnabled)
     */
    popupEnabled?: boolean;
  }

  export interface PopupViewOpenPopupOptions {
    /**
     * Sets the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    title?: string;
    /**
     * Sets the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#content) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    content?: string | HTMLElement | Widget;
    /**
     * Sets the popup's [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location), which is the geometry used to position the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    location?: Point;
    /**
     * When `true`, indicates the popup should fetch the content of this feature and display it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    fetchFeatures?: boolean;
    /**
     * Sets the popup's [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features), which populate the title and content of the popup based on each graphic's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    features?: Graphic[];
    /**
     * Sets pending [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#promises) on the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    promises?: Promise<any>[];
    /**
     * This property enables multiple features in a popup to display in a list rather than displaying the first selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    featureMenuOpen?: boolean;
    /**
     * When `true`, indicates the popup should update its [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location) for each paginated feature based on the [selectedFeature's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeature) geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    updateLocationEnabled?: boolean;
    /**
     * When `true`, indicates that only the popup header will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    collapsed?: boolean;
    /**
     * When `true`, indicates that the focus should be on the popup after it has been opened.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-PopupView.html#openPopup)
     */
    shouldFocus?: boolean;
  }

  export interface SceneView extends View, BreakpointsOwner, PopupView {}

  export class SceneView {
    /**
     * A SceneView displays a 3D view of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html) instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html)
     */

    constructor(properties?: SceneViewProperties);

    /**
     * Allows the view to be partially or fully transparent when composited with the webpage elements behind it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#alphaCompositingEnabled)
     */
    alphaCompositingEnabled: boolean;
    /**
     * Allows for adding analyses directly to the default analyses in the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#analyses)
     */
    analyses: Collection<
      | DimensionAnalysis
      | DirectLineMeasurementAnalysis
      | AreaMeasurementAnalysis
      | SliceAnalysis
      | LineOfSightAnalysis
      | ViewshedAnalysis
    >;
    /**
     * The observation point from which the visible portion (or perspective) of the SceneView is determined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#camera)
     */
    camera: Camera;
    /**
     * Represents the view's center point; when setting the center you may pass a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instance or an array of numbers representing at longitude/latitude pair (`[-100.4593, 36.9014]`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#center)
     */
    center: Point;
    /**
     * Represents an optional clipping area used to define the visible [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of a local scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#clippingArea)
     */
    clippingArea: Extent;
    /**
     * Specifies constraints for [Camera tilt](esri-Camera.html#tilt) and altitude that may be applied to the SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    constraints: SceneViewConstraints;
    /**
     * Specifies various properties of the environment's visualization in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    environment: SceneViewEnvironment;
    /**
     * The extent represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#extent)
     */
    extent: Extent;
    /**
     * Applies a display filter on the view for a specific set of floor levels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#floors)
     */
    floors: Collection<string>;
    /**
     * The view for the ground of the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#groundView)
     */
    readonly groundView: GroundView;
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions)
     */
    highlightOptions: HighlightOptions;
    /**
     * This property contains performance information in a SceneView like global memory usage and additional details for layers about memory consumption and number of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#performanceInfo)
     */
    readonly performanceInfo: SceneViewPerformanceInfo;
    /**
     * SceneView can draw scenes in three different quality modes: `high`, `medium` and `low`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#qualityProfile)
     */
    qualityProfile: "low" | "medium" | "high";
    /**
     * Represents an approximation of the map scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#scale)
     */
    scale: number;
    /**
     * The tiling scheme information of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#tileInfo)
     */
    readonly tileInfo: TileInfo;
    /**
     * The type of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#type)
     */
    readonly type: "3d";
    /**
     * The viewing mode (`local` or `global`).
     *
     * @default "global"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewingMode)
     */
    viewingMode: "global" | "local";
    /**
     * Represents the current view as a [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or point of observation on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewpoint)
     */
    viewpoint: Viewpoint;
    /**
     * The visibleArea represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of a [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#visibleArea)
     */
    readonly visibleArea: Polygon;
    /**
     * Represents the level of detail (LOD) at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#zoom)
     */
    zoom: number;

    /**
     * Sets the view to a given target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo)
     */
    goTo(target: GoToTarget3D, options?: GoToOptions3D): Promise<any>;
    /**
     * Returns [hit test results](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult) from each layer that intersects the specified screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest)
     */
    hitTest(
      screenPoint: SceneViewScreenPoint | MouseEvent,
      options?: SceneViewHitTestOptions,
    ): Promise<SceneViewHitTestResult>;
    /**
     * Create a screenshot of the current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    takeScreenshot(options?: SceneViewTakeScreenshotOptions): Promise<SceneViewScreenshot>;
    /**
     * Converts the given screen point to a [map point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#toMap)
     */
    toMap(screenPoint: SceneViewScreenPoint | MouseEvent, options?: SceneViewToMapOptions): Point;
    /**
     * Converts the given [map point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) to a screen point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#toScreen)
     */
    toScreen(point: Point): SceneViewScreenPoint;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: LineOfSightAnalysis): Promise<LineOfSightAnalysisView3D>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: DirectLineMeasurementAnalysis): Promise<DirectLineMeasurementAnalysisView3D>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: AreaMeasurementAnalysis): Promise<AreaMeasurementAnalysisView3D>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: SliceAnalysis): Promise<SliceAnalysisView3D>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: DimensionAnalysis): Promise<DimensionAnalysisView>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(analysis: ViewshedAnalysis): Promise<ViewshedAnalysisView3D>;
    /**
     * Gets the analysis view created for the given analysis object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#whenAnalysisView)
     */
    whenAnalysisView(
      analysis:
        | AreaMeasurementAnalysis
        | DimensionAnalysis
        | DirectLineMeasurementAnalysis
        | LineOfSightAnalysis
        | SliceAnalysis
        | ViewshedAnalysis,
    ): Promise<
      | AreaMeasurementAnalysisView3D
      | DimensionAnalysisView
      | DirectLineMeasurementAnalysisView3D
      | LineOfSightAnalysisView3D
      | SliceAnalysisView3D
      | ViewshedAnalysisView3D
    >;
  }

  interface SceneViewProperties extends ViewProperties, BreakpointsOwnerProperties, PopupViewProperties {
    /**
     * Allows the view to be partially or fully transparent when composited with the webpage elements behind it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#alphaCompositingEnabled)
     */
    alphaCompositingEnabled?: boolean;
    /**
     * Allows for adding analyses directly to the default analyses in the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#analyses)
     */
    analyses?: CollectionProperties<
      | DimensionAnalysisProperties
      | DirectLineMeasurementAnalysisProperties
      | AreaMeasurementAnalysisProperties
      | (SliceAnalysisProperties & { type: "slice" })
      | LineOfSightAnalysisProperties
      | (ViewshedAnalysisProperties & { type: "viewshed" })
    >;
    /**
     * The observation point from which the visible portion (or perspective) of the SceneView is determined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#camera)
     */
    camera?: CameraProperties;
    /**
     * Represents the view's center point; when setting the center you may pass a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instance or an array of numbers representing at longitude/latitude pair (`[-100.4593, 36.9014]`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#center)
     */
    center?: (PointProperties & { type: "point" }) | number[];
    /**
     * Represents an optional clipping area used to define the visible [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of a local scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#clippingArea)
     */
    clippingArea?: ExtentProperties;
    /**
     * Specifies constraints for [Camera tilt](esri-Camera.html#tilt) and altitude that may be applied to the SceneView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    constraints?: SceneViewConstraintsProperties;
    /**
     * Specifies various properties of the environment's visualization in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    environment?: SceneViewEnvironmentProperties;
    /**
     * The extent represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of an [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Applies a display filter on the view for a specific set of floor levels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#floors)
     */
    floors?: CollectionProperties<string>;
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions)
     */
    highlightOptions?: HighlightOptions;
    /**
     * SceneView can draw scenes in three different quality modes: `high`, `medium` and `low`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#qualityProfile)
     */
    qualityProfile?: "low" | "medium" | "high";
    /**
     * Represents an approximation of the map scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#scale)
     */
    scale?: number;
    /**
     * The viewing mode (`local` or `global`).
     *
     * @default "global"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewingMode)
     */
    viewingMode?: "global" | "local";
    /**
     * Represents the current view as a [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or point of observation on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
    /**
     * Represents the level of detail (LOD) at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#zoom)
     */
    zoom?: number;
  }

  export type EasingFunction = (t: number, duration: number) => number;

  /**
   * Animation options for the [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
   */
  export interface GoToOptions3D {
    /**
     * Indicates if the transition to the new view should be animated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    animate?: boolean;
    /**
     * Increases or decreases the animation speed by the specified factor.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    speedFactor?: number;
    /**
     * Set the exact duration (in milliseconds) of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    duration?: number;
    /**
     * The maximum allowed duration (in milliseconds) of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    maxDuration?: number;
    /**
     * The easing function to use for the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    easing?:
      | "linear"
      | "in-cubic"
      | "out-cubic"
      | "in-out-cubic"
      | "in-expo"
      | "out-expo"
      | "in-out-expo"
      | "in-out-coast-quadratic"
      | EasingFunction;
    /**
     * An [AbortSignal](https://developer.mozilla.org/en-US/docs/Web/API/AbortSignal) to abort the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToOptions3D)
     */
    signal?: AbortSignal;
  }

  /**
   * The target location/viewpoint to animate to in the [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GoToTarget3D)
   */
  export type GoToTarget3D =
    | number[]
    | Geometry
    | Geometry[]
    | Collection<Geometry>
    | Graphic
    | Graphic[]
    | Collection<Graphic>
    | Camera
    | Viewpoint
    | any;

  /**
   * Object specification for the graphic hit result returned in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GraphicHit)
   */
  export interface GraphicHit {
    type: "graphic";
    /**
     * A graphic present in the view that intersects the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GraphicHit)
     */
    graphic: Graphic;
    /**
     * The distance from the camera position to the point geometry hit on this graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GraphicHit)
     */
    distance: number;
    /**
     * The layer that contains the feature/graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GraphicHit)
     */
    layer: Layer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#GraphicHit)
     */
    mapPoint: Point;
  }

  /**
   * Object specification for the result of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
   */
  export interface SceneViewHitTestResult {
    /**
     * An array of result objects returned from the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) when the location of the input screen coordinates intersect features in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
     */
    results: ViewHit[];
    /**
     * Ground intersection result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
     */
    ground: HitTestResultGround;
    /**
     * The screen coordinates (or native mouse event) of the click on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
     */
    screenPoint: SceneViewScreenPoint | MouseEvent;
  }

  /**
   * A layer or graphic to be used in [toMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#toMap) and [hitTest](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) filter options.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#IntersectItem)
   */
  export type IntersectItem = Graphic | Layer | BuildingSublayer;

  /**
   * Object specification for the media hit results returned from [MediaLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html) in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
   */
  export interface MediaHit {
    type: "media";
    /**
     * An element representing a media element in [MediaLayer.source](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MediaLayer.html#source) that intersects the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
     */
    element: ImageElement | VideoElement;
    /**
     * The distance from the camera position to the point geometry hit on this graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
     */
    distance: number;
    /**
     * The media layer that contains the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
     */
    layer: MediaLayer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
     */
    mapPoint: Point;
    /**
     * _Since 4.28_ An object representing a point on the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#MediaHit)
     */
    sourcePoint: SourcePoint;
  }

  /**
   * Object specification for the route hit results returned from [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#RouteHit)
   */
  export interface RouteHit {
    type: "route";
    /**
     * The route layer that contains the element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#RouteHit)
     */
    layer: RouteLayer;
    /**
     * The point geometry in the spatial reference of the view corresponding with the input screen coordinates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#RouteHit)
     */
    mapPoint: Point;
    /**
     * The route hit test will contain all intersecting network elements which, includes one of the following: DirectionLine, DirectionPoint, PointBarrier, PolylineBarrier, PolygonBarrier, Stop, or RouteInfo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#RouteHit)
     */
    networkFeature: DirectionLine | DirectionPoint | PointBarrier | PolygonBarrier | PolygonBarrier | RouteInfo | Stop;
  }

  export interface SceneViewConstraintsProperties {
    /**
     * Specifies a constraint on the minimum and maximum allowed camera altitude.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    altitude?: SceneViewConstraintsAltitudeProperties;
    /**
     * Specifies the near and far webgl clip distances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    clipDistance?: SceneViewConstraintsClipDistanceProperties;
    /**
     * Specifies a constraint on the amount of allowed tilting of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    tilt?: SceneViewConstraintsTiltProperties;
  }

  export interface SceneViewConstraints extends AnonymousAccessor {
    /**
     * Specifies a constraint on the minimum and maximum allowed camera altitude.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    altitude?: SceneViewConstraintsAltitude;
    /**
     * Specifies the near and far webgl clip distances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    clipDistance?: SceneViewConstraintsClipDistance;
    /**
     * Specifies a constraint on the amount of allowed tilting of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    tilt?: SceneViewConstraintsTilt;
  }

  export interface SceneViewConstraintsAltitudeProperties {
    /**
     * The minimum allowed camera altitude (in meters).
     *
     * @default -200_000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    min?: number;
    /**
     * The maximum allowed camera altitude (in meters).
     *
     * @default EARTH_RADIUS * 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    max?: number;
  }

  export interface SceneViewConstraintsAltitude extends AnonymousAccessor {
    /**
     * The minimum allowed camera altitude (in meters).
     *
     * @default -200_000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    min?: number;
    /**
     * The maximum allowed camera altitude (in meters).
     *
     * @default EARTH_RADIUS * 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    max?: number;
  }

  export interface SceneViewConstraintsClipDistanceProperties {
    /**
     * The near clip distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    near?: number;
    /**
     * The far clip distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    far?: number;
    /**
     * Specifies the mode of the constraint which is either `auto` or `manual`.
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    mode?: "auto" | "manual";
  }

  export interface SceneViewConstraintsClipDistance extends AnonymousAccessor {
    /**
     * The near clip distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    near?: number;
    /**
     * The far clip distance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    far?: number;
    /**
     * Specifies the mode of the constraint which is either `auto` or `manual`.
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    mode?: "auto" | "manual";
  }

  export interface SceneViewConstraintsTiltProperties {
    /**
     * Specifies the maximum amount of tilt (in degrees) allowed in the view and may range from 0.5 to 179.5 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    max?: number;
    /**
     * Specifies the mode of the constraint.
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    mode?: "auto" | "manual";
  }

  export interface SceneViewConstraintsTilt extends AnonymousAccessor {
    /**
     * Specifies the maximum amount of tilt (in degrees) allowed in the view and may range from 0.5 to 179.5 degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    max?: number;
    /**
     * Specifies the mode of the constraint.
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#constraints)
     */
    mode?: "auto" | "manual";
  }

  export interface SceneViewEnvironmentProperties {
    /**
     * Specifies how the background of the scene (which lies behind sky, stars and atmosphere) should be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    background?: BackgroundProperties;
    /**
     * Indicates the type of lighting in the scene.
     *
     * @default SunLighting
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    lighting?: (SunLightingProperties & { type?: "sun" }) | (VirtualLightingProperties & { type: "virtual" });
    /**
     * Indicates whether atmosphere visualization is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    atmosphereEnabled?: boolean;
    /**
     * Indicates the type of weather visualization in the scene.
     *
     * @default SunnyWeather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    weather?:
      | (SunnyWeatherProperties & { type: "sunny" })
      | (CloudyWeatherProperties & { type: "cloudy" })
      | (RainyWeatherProperties & { type: "rainy" })
      | (SnowyWeatherProperties & { type: "snowy" })
      | (FoggyWeatherProperties & { type: "foggy" });
    /**
     * Indicates whether stars visualization is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    starsEnabled?: boolean;
  }

  export interface SceneViewEnvironment extends AnonymousAccessor {
    /**
     * Specifies how the background of the scene (which lies behind sky, stars and atmosphere) should be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    background?: Background;
    /**
     * Indicates the type of lighting in the scene.
     *
     * @default SunLighting
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    lighting?: SunLighting | VirtualLighting;
    /**
     * Indicates whether atmosphere visualization is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    atmosphereEnabled?: boolean;
    /**
     * Indicates the type of weather visualization in the scene.
     *
     * @default SunnyWeather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    weather?: SunnyWeather | CloudyWeather | RainyWeather | SnowyWeather | FoggyWeather;
    /**
     * Indicates whether stars visualization is enabled.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#environment)
     */
    starsEnabled?: boolean;
  }

  export interface SceneViewHitTestOptions {
    /**
     * A list of layers and graphics to include for intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest)
     */
    include?:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
    /**
     * A list of layers and graphics to include for intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest)
     */
    exclude?:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
  }

  export interface SceneViewTakeScreenshotOptions {
    /**
     * The format of the resulting encoded data url.
     *
     * @default png
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    format?: "jpg" | "png";
    /**
     * The quality (0 to 100) of the encoded image when format is `jpg`.
     *
     * @default 98
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    quality?: number;
    /**
     * The width of the screenshot (defaults to the area width).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    width?: number;
    /**
     * The height of the screenshot (defaults to the area height).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    height?: number;
    /**
     * Specifies whether to take a screenshot of a specific area of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    area?: SceneViewTakeScreenshotOptionsArea;
    /**
     * Indicates whether view padding should be ignored.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    ignorePadding?: boolean;
  }

  export interface SceneViewTakeScreenshotOptionsArea {
    /**
     * The x coordinate of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    x?: number;
    /**
     * The y coordinate of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    y?: number;
    /**
     * The width of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    width?: number;
    /**
     * The height of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot)
     */
    height?: number;
  }

  export interface SceneViewToMapOptions {
    /**
     * A list of layers and graphics to include for intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#toMap)
     */
    include?:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
    /**
     * A list of layers and graphics to exclude for intersection testing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#toMap)
     */
    exclude?:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
  }

  /**
   * An object representing a point on the screen.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ScreenPoint)
   */
  export interface SceneViewScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ScreenPoint)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ScreenPoint)
     */
    y: number;
  }

  /**
   * Object returned when [takeScreenshot()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#takeScreenshot) promise resolves:
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#Screenshot)
   */
  export interface SceneViewScreenshot {
    /**
     * A data url representing the screenshot.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#Screenshot)
     */
    dataUrl: string;
    /**
     * The raw RGBA image data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#Screenshot)
     */
    data: ImageData;
  }

  /**
   * Object specification for the result returned in [HitTestResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult) of the [hitTest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#ViewHit)
   */
  export type ViewHit = GraphicHit | MediaHit | RouteHit;

  export interface HitTestResultGround {
    /**
     * The point at which the ground was hit while performing the hitTest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
     */
    mapPoint: Point;
    /**
     * The distance from camera position to the ground hit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#HitTestResult)
     */
    distance: number;
  }

  /**
   * Contains utilities for working with colors in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-support-colorUtils.html)
   */
  interface colorUtils {
    /**
     * Returns the average color of the view's background within the view's [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#extent).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-support-colorUtils.html#getBackgroundColor)
     */
    getBackgroundColor(view: MapView | SceneView): Promise<Color>;
    /**
     * This method inspects the basemap and background of a MapView and returns either `light` or `dark` as the theme of the background depending on if the average color of the basemap and the view's background is light or dark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-support-colorUtils.html#getBackgroundColorTheme)
     */
    getBackgroundColorTheme(view: MapView | SceneView): Promise<"light" | "dark">;
  }

  export const colorUtils: colorUtils;

  export class Theme extends Accessor {
    /**
     * This class is used by the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to define the base colors used by widgets and components to render temporary graphics and labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Theme.html)
     */

    constructor(properties?: ThemeProperties);

    /**
     * The base color used to render temporary graphics in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Theme.html#accentColor)
     */
    accentColor: Color;
    /**
     * The base color used to render temporary labels in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Theme.html#textColor)
     */
    textColor: Color;
  }

  interface ThemeProperties {
    /**
     * The base color used to render temporary graphics in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Theme.html#accentColor)
     */
    accentColor?: Color | number[] | string;
    /**
     * The base color used to render temporary labels in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-Theme.html#textColor)
     */
    textColor?: Color | number[] | string;
  }

  export class DefaultUI extends UI {
    /**
     * The DefaultUI class exposes the default [widget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html) [components](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-DefaultUI.html#components) available in either a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-DefaultUI.html)
     */

    constructor(properties?: DefaultUIProperties);

    /**
     * An array of strings representing the default widgets visible when a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-DefaultUI.html#components)
     */
    components: string[];
  }

  interface DefaultUIProperties extends UIProperties {
    /**
     * An array of strings representing the default widgets visible when a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-DefaultUI.html#components)
     */
    components?: string[];
  }

  export class UI extends Accessor {
    /**
     * This class provides a simple interface for [adding](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add), [moving](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move) and [removing](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#remove) components from a view's user interface (UI).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html)
     */

    constructor(properties?: UIProperties);

    /**
     * The HTML Element that contains the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#container)
     */
    container: HTMLElement;
    /**
     * The height of the UI container.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#height)
     */
    readonly height: number;
    /**
     * Defines the padding for the UI from the top, left, right, and bottom sides of the container or [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default { left: 15, top: 15, right: 15, bottom: 15 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#padding)
     */
    padding: any | number;
    /**
     * The view associated with the UI components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#view)
     */
    view: MapView | SceneView;
    /**
     * The width of the UI container.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#width)
     */
    readonly width: number;

    /**
     * Adds one or more HTML component(s) or [widgets](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html) to the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    add(component: Widget | HTMLElement | string | any[] | UIAddComponent, position?: string | UIAddPosition): void;
    /**
     * Removes all components from a given position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#empty)
     */
    empty(
      position?:
        | "bottom-leading"
        | "bottom-left"
        | "bottom-right"
        | "bottom-trailing"
        | "top-leading"
        | "top-left"
        | "top-right"
        | "top-trailing"
        | "manual",
    ): void;
    /**
     * Find a component by widget or DOM ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#find)
     */
    find(id: string): HTMLElement | Widget;
    /**
     * Returns all widgets and/or HTML elements in the view, or only components at specified positions in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#getComponents)
     */
    getComponents(position?: UIPosition | UIPosition[]): (Widget | HTMLElement)[];
    /**
     * Moves one or more UI component(s) to the specified position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    move(component: Widget | HTMLElement | string | any[] | UIMoveComponent, position?: string | UIMovePosition): void;
    /**
     * Removes one or more HTML component(s) or [widgets](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html) from the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#remove)
     */
    remove(component: Widget | HTMLElement | string | any[]): void;
  }

  interface UIProperties {
    /**
     * The HTML Element that contains the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#container)
     */
    container?: HTMLElement;
    /**
     * Defines the padding for the UI from the top, left, right, and bottom sides of the container or [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * @default { left: 15, top: 15, right: 15, bottom: 15 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#padding)
     */
    padding?: any | number;
    /**
     * The view associated with the UI components.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface UIAddComponent {
    /**
     * The component to add to the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    component: Widget | HTMLElement | string;
    /**
     * The position in the view at which to add the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    position?:
      | "bottom-leading"
      | "bottom-left"
      | "bottom-right"
      | "bottom-trailing"
      | "top-leading"
      | "top-left"
      | "top-right"
      | "top-trailing"
      | "manual";
    /**
     * The placement index of the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    index?: number;
  }

  export interface UIAddPosition {
    /**
     * The position in the view at which to add the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    position?:
      | "bottom-leading"
      | "bottom-left"
      | "bottom-right"
      | "bottom-trailing"
      | "top-leading"
      | "top-left"
      | "top-right"
      | "top-trailing"
      | "manual";
    /**
     * The placement index of the component(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#add)
     */
    index?: number;
  }

  export interface UIMoveComponent {
    /**
     * The component to move.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    component: Widget | HTMLElement | string;
    /**
     * The destination position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    position?:
      | "bottom-leading"
      | "bottom-left"
      | "bottom-right"
      | "bottom-trailing"
      | "top-leading"
      | "top-left"
      | "top-right"
      | "top-trailing"
      | "manual";
    /**
     * The placement index of the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    index?: number;
  }

  export interface UIMovePosition {
    /**
     * The position in the view at which to add the component.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    position?:
      | "bottom-leading"
      | "bottom-left"
      | "bottom-right"
      | "bottom-trailing"
      | "top-leading"
      | "top-left"
      | "top-right"
      | "top-trailing"
      | "manual";
    /**
     * The placement index of the component(s).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#move)
     */
    index?: number;
  }

  /**
   * The UI position of an element in the view.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ui-UI.html#UIPosition)
   */
  export type UIPosition =
    | "bottom-leading"
    | "bottom-left"
    | "bottom-right"
    | "bottom-trailing"
    | "top-leading"
    | "top-left"
    | "top-right"
    | "top-trailing"
    | "manual";

  export interface View extends Accessor, corePromise, Evented, DOMContainer, PopupView {}

  export class View {
    /**
     * A view provides the means of viewing and interacting with the components of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html)
     */

    constructor(properties?: ViewProperties);

    /**
     * Collection containing a flat list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) related to the basemap, operational layers, and group layers in this view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#allLayerViews)
     */
    allLayerViews: Collection<LayerView>;
    /**
     * Represents an ongoing view animation initialized by [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#animation)
     */
    animation: ViewAnimation;
    /**
     * Represents the view for a single basemap after it has been added to the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#basemapView)
     */
    basemapView: BasemapView;
    /**
     * A fatal [error](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html) returned when the view loses its WebGL context.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#fatalError)
     */
    fatalError: Error;
    /**
     * Allows for adding [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) directly to the default graphics in the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#graphics)
     */
    graphics: Collection<Graphic>;
    /**
     * Options to configure input handling of the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#input)
     */
    readonly input: Input;
    /**
     * Indication whether the view is being interacted with (for example when panning or by an interactive tool).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#interacting)
     */
    readonly interacting: boolean;
    /**
     * A collection containing a hierarchical list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of the [operational layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#layerViews)
     */
    layerViews: Collection<LayerView>;
    /**
     * The magnifier allows for showing a portion of the view as a magnifier image on top of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#magnifier)
     */
    readonly magnifier: Magnifier;
    /**
     * An instance of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) object to display in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#map)
     */
    map: Map;
    /**
     * Indication whether the view is being navigated (for example when panning).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#navigating)
     */
    readonly navigating: boolean;
    /**
     * Options to configure the navigation behavior of the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#navigation)
     */
    navigation: Navigation;
    /**
     * Use the padding property to make the [center](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#center), and [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#extent), etc.
     *
     * @default {left: 0, top: 0, right: 0, bottom: 0}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    padding: ViewPadding;
    /**
     * When `true`, this property indicates whether the view successfully satisfied all dependencies, signaling that the following conditions are met.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#ready)
     */
    readonly ready: boolean;
    /**
     * Represents the current value of one pixel in the unit of the view's [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#spatialReference).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#resolution)
     */
    readonly resolution: number;
    /**
     * The spatial reference of the view.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * Indication whether the view is animating, being navigated with or resizing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#stationary)
     */
    readonly stationary: boolean;
    /**
     * This property specifies the base colors used by some widgets and components to render graphics and labels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#theme)
     */
    theme: Theme;
    /**
     * The view's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The type of the view is either `2d` (indicating a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html)) or `3d` (indicating a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html)).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#type)
     */
    readonly type: string;
    /**
     * Indicates whether the view is being updated by additional data requests to the network, or by processing received data.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#updating)
     */
    readonly updating: boolean;

    /**
     * Destroys the view, and any associated resources, including its [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#map), [popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#popup), and [UI](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#ui) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#destroy)
     */
    destroy(): void;
    /**
     * Sets the focus on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#focus)
     */
    focus(): void;
    /**
     * Call this method to clear any [fatal errors](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#fatalError) resulting from a lost WebGL context.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#tryFatalErrorRecovery)
     */
    tryFatalErrorRecovery(): void;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: PointCloudLayer): Promise<PointCloudLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: StreamLayer): Promise<StreamLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: WFSLayer): Promise<WFSLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: OGCFeatureLayer): Promise<OGCFeatureLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: FeatureLayer): Promise<FeatureLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: CSVLayer): Promise<CSVLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: GeoJSONLayer): Promise<GeoJSONLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: GeoRSSLayer): Promise<GeoRSSLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: GraphicsLayer): Promise<GraphicsLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: ImageryLayer): Promise<ImageryLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: KMLLayer): Promise<KMLLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: SceneLayer): Promise<SceneLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: DimensionLayer): Promise<DimensionLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: MediaLayer): Promise<MediaLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: ViewshedLayer): Promise<ViewshedLayerView>;
    /**
     * Gets the [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) created on the view for the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#whenLayerView)
     */
    whenLayerView(layer: Layer): Promise<LayerView>;

    on(name: "resize", eventHandler: ViewResizeEventHandler): IHandle;

    on(name: "layerview-create", eventHandler: ViewLayerviewCreateEventHandler): IHandle;

    on(name: "layerview-create-error", eventHandler: ViewLayerviewCreateErrorEventHandler): IHandle;

    on(name: "layerview-destroy", eventHandler: ViewLayerviewDestroyEventHandler): IHandle;

    on(name: "click", eventHandler: ViewClickEventHandler): IHandle;

    on(name: "click", modifiers: string[], eventHandler: ViewClickEventHandler): IHandle;

    on(name: "double-click", eventHandler: ViewDoubleClickEventHandler): IHandle;

    on(name: "double-click", modifiers: string[], eventHandler: ViewDoubleClickEventHandler): IHandle;

    on(name: "immediate-double-click", eventHandler: ViewImmediateDoubleClickEventHandler): IHandle;

    on(
      name: "immediate-double-click",
      modifiers: string[],
      eventHandler: ViewImmediateDoubleClickEventHandler,
    ): IHandle;

    on(name: "immediate-click", eventHandler: ViewImmediateClickEventHandler): IHandle;

    on(name: "immediate-click", modifiers: string[], eventHandler: ViewImmediateClickEventHandler): IHandle;

    on(name: "hold", eventHandler: ViewHoldEventHandler): IHandle;

    on(name: "hold", modifiers: string[], eventHandler: ViewHoldEventHandler): IHandle;

    on(name: "drag", eventHandler: ViewDragEventHandler): IHandle;

    on(name: "drag", modifiers: string[], eventHandler: ViewDragEventHandler): IHandle;

    on(name: "mouse-wheel", eventHandler: ViewMouseWheelEventHandler): IHandle;

    on(name: "mouse-wheel", modifiers: string[], eventHandler: ViewMouseWheelEventHandler): IHandle;

    on(name: "key-down", eventHandler: ViewKeyDownEventHandler): IHandle;

    on(name: "key-down", modifiers: string[], eventHandler: ViewKeyDownEventHandler): IHandle;

    on(name: "key-up", eventHandler: ViewKeyUpEventHandler): IHandle;

    on(name: "key-up", modifiers: string[], eventHandler: ViewKeyUpEventHandler): IHandle;

    on(name: "pointer-down", eventHandler: ViewPointerDownEventHandler): IHandle;

    on(name: "pointer-down", modifiers: string[], eventHandler: ViewPointerDownEventHandler): IHandle;

    on(name: "pointer-move", eventHandler: ViewPointerMoveEventHandler): IHandle;

    on(name: "pointer-move", modifiers: string[], eventHandler: ViewPointerMoveEventHandler): IHandle;

    on(name: "pointer-up", eventHandler: ViewPointerUpEventHandler): IHandle;

    on(name: "pointer-up", modifiers: string[], eventHandler: ViewPointerUpEventHandler): IHandle;

    on(name: "pointer-enter", eventHandler: ViewPointerEnterEventHandler): IHandle;

    on(name: "pointer-enter", modifiers: string[], eventHandler: ViewPointerEnterEventHandler): IHandle;

    on(name: "pointer-leave", eventHandler: ViewPointerLeaveEventHandler): IHandle;

    on(name: "pointer-leave", modifiers: string[], eventHandler: ViewPointerLeaveEventHandler): IHandle;

    on(name: "focus", eventHandler: ViewFocusEventHandler): IHandle;

    on(name: "blur", eventHandler: ViewBlurEventHandler): IHandle;

    /**
     * Contains the collection of active views on the page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#views)
     */
    static readonly views: Collection<View>;
  }

  interface ViewProperties extends DOMContainerProperties, PopupViewProperties {
    /**
     * Collection containing a flat list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) related to the basemap, operational layers, and group layers in this view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#allLayerViews)
     */
    allLayerViews?: CollectionProperties<LayerViewProperties>;
    /**
     * Represents an ongoing view animation initialized by [goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#animation)
     */
    animation?: ViewAnimationProperties;
    /**
     * Represents the view for a single basemap after it has been added to the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#basemapView)
     */
    basemapView?: BasemapViewProperties;
    /**
     * A fatal [error](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html) returned when the view loses its WebGL context.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#fatalError)
     */
    fatalError?: Error;
    /**
     * Allows for adding [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) directly to the default graphics in the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#graphics)
     */
    graphics?: CollectionProperties<GraphicProperties>;
    /**
     * A collection containing a hierarchical list of all the created [LayerViews](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) of the [operational layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#layers) in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#layerViews)
     */
    layerViews?: CollectionProperties<LayerViewProperties>;
    /**
     * An instance of a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) object to display in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#map)
     */
    map?: MapProperties;
    /**
     * Options to configure the navigation behavior of the View.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#navigation)
     */
    navigation?: NavigationProperties;
    /**
     * Use the padding property to make the [center](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#center), and [extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#extent), etc.
     *
     * @default {left: 0, top: 0, right: 0, bottom: 0}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    padding?: ViewPadding;
    /**
     * The spatial reference of the view.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * This property specifies the base colors used by some widgets and components to render graphics and labels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#theme)
     */
    theme?: ThemeProperties;
    /**
     * The view's time extent.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
  }

  export interface ViewBlurEvent {
    native: any;

    target: View;
  }

  export interface ViewClickEvent {
    button: number;

    buttons: 0 | 1 | 2;

    mapPoint: Point;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "click";

    x: number;

    y: number;
  }

  export interface ViewDoubleClickEvent {
    button: number;

    buttons: 0 | 1 | 2;

    mapPoint: Point;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "double-click";

    x: number;

    y: number;
  }

  export interface ViewDragEvent {
    action: "start" | "added" | "update" | "removed" | "end";

    angle: number;

    button: 0 | 1 | 2;

    buttons: number;

    native: any;

    origin: ViewDragEventOrigin;

    radius: number;

    stopPropagation: Function;

    timestamp: number;

    type: "drag";

    x: number;

    y: number;
  }

  export interface ViewFocusEvent {
    native: any;

    target: View;
  }

  /**
   * Options for configuring the highlight on [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#highlightOptions), [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions) or on layer views.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
   */
  export interface HighlightOptions {
    /**
     * The color of the highlight fill.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    color?: Color;
    /**
     * The color of the halo surrounding the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    haloColor?: Color;
    /**
     * The opacity of the highlight halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    haloOpacity?: number;
    /**
     * The opacity of the fill (area within the halo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    fillOpacity?: number;
    /**
     * The color of the highlighted feature's shadow in [3D SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    shadowColor?: Color;
    /**
     * The opacity of the highlighted feature's shadow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    shadowOpacity?: number;
    /**
     * Defines the intensity of the shadow area obtained by overlapping the shadow of the highlighted feature and the shadow of other objects in [3D SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#HighlightOptions)
     */
    shadowDifference?: number;
  }

  export interface ViewHoldEvent {
    button: 0 | 1 | 2;

    buttons: number;

    mapPoint: Point;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "hold";

    x: number;

    y: number;
  }

  export interface ViewImmediateClickEvent {
    button: 0 | 1 | 2;

    buttons: number;

    mapPoint: Point;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "immediate-click";

    x: number;

    y: number;
  }

  export interface ViewImmediateDoubleClickEvent {
    button: 0 | 1 | 2;

    buttons: number;

    mapPoint: Point;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "immediate-double-click";

    x: number;

    y: number;
  }

  export interface ViewKeyDownEvent {
    key: string;

    native: any;

    repeat: boolean;

    stopPropagation: Function;

    timestamp: number;

    type: "key-down";
  }

  export interface ViewKeyUpEvent {
    key: string;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "key-up";
  }

  export interface ViewLayerviewCreateErrorEvent {
    error: Error;

    layer: Layer;
  }

  export interface ViewLayerviewCreateEvent {
    layer: Layer;

    layerView: LayerView;
  }

  export interface ViewLayerviewDestroyEvent {
    layer: Layer;

    layerView: LayerView;
  }

  export interface ViewMouseWheelEvent {
    deltaY: number;

    native: any;

    stopPropagation: Function;

    timestamp: number;

    type: "mouse-wheel";

    x: number;

    y: number;
  }

  export interface ViewPointerDownEvent {
    button: number;

    buttons: number;

    native: any;

    pointerId: number;

    pointerType: "mouse" | "touch";

    stopPropagation: Function;

    timestamp: number;

    type: "pointer-down";

    x: number;

    y: number;
  }

  export interface ViewPointerEnterEvent {
    button: number;

    buttons: number;

    native: any;

    pointerId: number;

    pointerType: "mouse" | "touch";

    stopPropagation: Function;

    timestamp: number;

    type: "pointer-enter";

    x: number;

    y: number;
  }

  export interface ViewPointerLeaveEvent {
    button: number;

    buttons: number;

    native: any;

    pointerId: number;

    pointerType: "mouse" | "touch";

    stopPropagation: Function;

    timestamp: number;

    type: "pointer-leave";

    x: number;

    y: number;
  }

  export interface ViewPointerMoveEvent {
    button: number;

    buttons: number;

    native: any;

    pointerId: number;

    pointerType: "mouse" | "touch";

    stopPropagation: Function;

    timestamp: number;

    type: "pointer-move";

    x: number;

    y: number;
  }

  export interface ViewPointerUpEvent {
    button: number;

    buttons: number;

    native: any;

    pointerId: number;

    pointerType: "mouse" | "touch";

    stopPropagation: Function;

    timestamp: number;

    type: "pointer-up";

    x: number;

    y: number;
  }

  export interface ViewResizeEvent {
    height: number;

    oldHeight: number;

    oldWidth: number;

    width: number;
  }

  export interface ViewPadding {
    /**
     * The left padding (in pixels).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    left?: number;
    /**
     * The top padding (in pixels).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    top?: number;
    /**
     * The right padding (in pixels).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    right?: number;
    /**
     * The bottom padding (in pixels).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html#padding)
     */
    bottom?: number;
  }

  export interface View2D extends View, BreakpointsOwner, PopupView {}

  export class View2D {
    /**
     * View2D is the base class for all 2D views.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html)
     */

    constructor(properties?: View2DProperties);

    /**
     * The background color of the MapView.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#background)
     */
    background: ColorBackground;
    /**
     * Represents the view's center point; when setting the center, you may pass a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instance or an array of numbers representing a longitude/latitude pair (`[-100.4593, 36.9014]`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#center)
     */
    center: Point;
    /**
     * Specifies constraints to [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale), [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom), and [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#rotation) that may be applied to the MapView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    constraints: View2DConstraints;
    /**
     * The extent represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#extent)
     */
    extent: Extent;
    /**
     * Applies a display filter on the view for a specific set of floor levels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#floors)
     */
    floors: Collection<string>;
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#highlightOptions)
     */
    highlightOptions: HighlightOptions;
    /**
     * Defines which anchor stays still while resizing the browser window.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#resizeAlign)
     */
    resizeAlign:
      | "center"
      | "left"
      | "right"
      | "top"
      | "bottom"
      | "top-left"
      | "top-right"
      | "bottom-left"
      | "bottom-right";
    /**
     * The clockwise rotation of due north in relation to the top of the view in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#rotation)
     */
    rotation: number;
    /**
     * Represents the map scale at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale)
     */
    scale: number;
    /**
     * Indicates if the MapView's [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#spatialReference) can be changed after it is initialized.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#spatialReferenceLocked)
     */
    spatialReferenceLocked: boolean;
    /**
     * The tiling scheme information of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#tileInfo)
     */
    readonly tileInfo: TileInfo;
    /**
     * Defines the time zone of the view.
     *
     * @default "system"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#timeZone)
     */
    timeZone: string;
    /**
     * The dimension of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#type)
     */
    readonly type: "2d";
    /**
     * Represents the current view as a [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or point of observation on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#viewpoint)
     */
    viewpoint: Viewpoint;
    /**
     * The visibleArea represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of a [Polygon](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polygon.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#visibleArea)
     */
    readonly visibleArea: Polygon;
    /**
     * Represents the level of detail (LOD) at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom)
     */
    zoom: number;

    /**
     * Sets the view to a given target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#goTo)
     */
    goTo(target: GoToTarget2D, options?: GoToOptions2D): Promise<any>;
    /**
     * Converts the given screen point to a [map point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#toMap)
     */
    toMap(screenPoint: MapViewScreenPoint | MouseEvent): Point;
    /**
     * Converts the given [map point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) to a screen point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#toScreen)
     */
    toScreen(point: Point, options?: ToScreenOptions2D): MapViewScreenPoint;
  }

  interface View2DProperties extends ViewProperties, BreakpointsOwnerProperties, PopupViewProperties {
    /**
     * The background color of the MapView.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#background)
     */
    background?: ColorBackgroundProperties;
    /**
     * Represents the view's center point; when setting the center, you may pass a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) instance or an array of numbers representing a longitude/latitude pair (`[-100.4593, 36.9014]`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#center)
     */
    center?: (PointProperties & { type: "point" }) | number[];
    /**
     * Specifies constraints to [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale), [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom), and [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#rotation) that may be applied to the MapView.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    constraints?: View2DConstraints;
    /**
     * The extent represents the visible portion of a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) within the view as an instance of [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#extent)
     */
    extent?: ExtentProperties;
    /**
     * Applies a display filter on the view for a specific set of floor levels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#floors)
     */
    floors?: CollectionProperties<string>;
    /**
     * Options for configuring the highlight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#highlightOptions)
     */
    highlightOptions?: HighlightOptions;
    /**
     * Defines which anchor stays still while resizing the browser window.
     *
     * @default "center"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#resizeAlign)
     */
    resizeAlign?:
      | "center"
      | "left"
      | "right"
      | "top"
      | "bottom"
      | "top-left"
      | "top-right"
      | "bottom-left"
      | "bottom-right";
    /**
     * The clockwise rotation of due north in relation to the top of the view in degrees.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#rotation)
     */
    rotation?: number;
    /**
     * Represents the map scale at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale)
     */
    scale?: number;
    /**
     * Indicates if the MapView's [spatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#spatialReference) can be changed after it is initialized.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#spatialReferenceLocked)
     */
    spatialReferenceLocked?: boolean;
    /**
     * Defines the time zone of the view.
     *
     * @default "system"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#timeZone)
     */
    timeZone?: string;
    /**
     * Represents the current view as a [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or point of observation on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
    /**
     * Represents the level of detail (LOD) at the center of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom)
     */
    zoom?: number;
  }

  export interface View2DConstraints {
    /**
     * An array of [LODs](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-LOD.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    lods?: LOD[];
    /**
     * The area in which the user is allowed to navigate laterally.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    geometry?: Geometry;
    /**
     * The minimum [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale) the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    minScale?: number;
    /**
     * The maximum [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale) the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    maxScale?: number;
    /**
     * The minimum [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom) level the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    minZoom?: number;
    /**
     * The maximum [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom) level the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    maxZoom?: number;
    /**
     * When `true`, the view snaps to the next LOD when zooming in or out.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    snapToZoom?: boolean;
    /**
     * Indicates whether the user can rotate the map.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    rotationEnabled?: boolean;
    /**
     * A read-only property that specifies the levels of  detail (LODs) read from the [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    effectiveLODs?: LOD[];
    /**
     * A read-only property that specifies the minimum [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom) level the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    effectiveMinZoom?: number;
    /**
     * A read-only property that specifies the maximum [zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#zoom) level the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    effectiveMaxZoom?: number;
    /**
     * A read-only property that specifies the minimum [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale) the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    effectiveMinScale?: number;
    /**
     * A read-only property that specifies the maximum [scale](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#scale) the user is allowed to zoom to within the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View2D.html#constraints)
     */
    effectiveMaxScale?: number;
  }

  export interface ViewAnimation extends Accessor, corePromise {}

  export class ViewAnimation {
    /**
     * Contains a [state](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#state) property used for checking the state of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html)
     */

    constructor(properties?: ViewAnimationProperties);

    /**
     * The state of the animation.
     *
     * @default "running"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#state)
     */
    readonly state: "running" | "finished" | "stopped" | "waiting-for-target";
    /**
     * The target of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#target)
     */
    target: Viewpoint | Promise<Viewpoint>;

    /**
     * Finishes the view animation by immediately going to the target and sets the state of the animation to `finished`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#finish)
     */
    finish(): void;
    /**
     * Stops the view animation at its current state and sets the state of the animation to `stopped`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#stop)
     */
    stop(): void;
  }

  interface ViewAnimationProperties {
    /**
     * The target of the animation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-ViewAnimation.html#target)
     */
    target?: ViewpointProperties | Promise<ViewpointProperties>;
  }

  export interface ViewDragEventOrigin {
    x: number;

    y: number;
  }

  export class Search extends Accessor {
    /**
     * Represents the search parameters set within the web scene or the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html)
     */

    constructor(properties?: SearchProperties);

    /**
     * Indicates whether the place finder is enabled in the web scene or the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#addressSearchEnabled)
     */
    addressSearchEnabled: boolean;
    /**
     * Whether search functionality is enabled in the web scene or the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#enabled)
     */
    enabled: boolean;
    /**
     * The hint provided with the search dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#hintText)
     */
    hintText: string;
    /**
     * A collection of layers to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#layers)
     */
    layers: Collection<SearchLayer>;
    /**
     * A collection of tables to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#tables)
     */
    tables: Collection<SearchTable>;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#clone)
     */
    clone(): Search;
  }

  interface SearchProperties {
    /**
     * Indicates whether the place finder is enabled in the web scene or the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#addressSearchEnabled)
     */
    addressSearchEnabled?: boolean;
    /**
     * Whether search functionality is enabled in the web scene or the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#enabled)
     */
    enabled?: boolean;
    /**
     * The hint provided with the search dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#hintText)
     */
    hintText?: string;
    /**
     * A collection of layers to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#layers)
     */
    layers?: CollectionProperties<SearchLayerProperties>;
    /**
     * A collection of tables to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Search.html#tables)
     */
    tables?: CollectionProperties<SearchTableProperties>;
  }

  export class SearchLayer extends Accessor {
    /**
     * Represents a layer to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html)
     */

    constructor(properties?: SearchLayerProperties);

    /**
     * The field to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#field)
     */
    field: SearchLayerField;
    /**
     * The id of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#id)
     */
    id: string;
    /**
     * The sub layer index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#subLayer)
     */
    subLayer: number;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#clone)
     */
    clone(): SearchLayer;
  }

  interface SearchLayerProperties {
    /**
     * The field to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#field)
     */
    field?: SearchLayerFieldProperties;
    /**
     * The id of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#id)
     */
    id?: string;
    /**
     * The sub layer index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayer.html#subLayer)
     */
    subLayer?: number;
  }

  export class SearchLayerField extends Accessor {
    /**
     * Represents the field of a layer to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html)
     */

    constructor(properties?: SearchLayerFieldProperties);

    /**
     * Whether or not the field is an exact match.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#exactMatch)
     */
    exactMatch: boolean;
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#name)
     */
    name: string;
    /**
     * The data type of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#type)
     */
    type:
      | "small-integer"
      | "integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#clone)
     */
    clone(): SearchLayerField;
  }

  interface SearchLayerFieldProperties {
    /**
     * Whether or not the field is an exact match.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#exactMatch)
     */
    exactMatch?: boolean;
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#name)
     */
    name?: string;
    /**
     * The data type of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchLayerField.html#type)
     */
    type?:
      | "small-integer"
      | "integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";
  }

  export class SearchTable extends Accessor {
    /**
     * Represents a table to be included in search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html)
     */

    constructor(properties?: SearchTableProperties);

    /**
     * The field to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html#field)
     */
    field: SearchTableField;
    /**
     * The id of the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html#id)
     */
    id: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html#clone)
     */
    clone(): SearchTable;
  }

  interface SearchTableProperties {
    /**
     * The field to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html#field)
     */
    field?: SearchTableFieldProperties;
    /**
     * The id of the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html#id)
     */
    id?: string;
  }

  export class SearchTableField extends Accessor {
    /**
     * Represents the field of a table to use for search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html)
     */

    constructor(properties?: SearchTableFieldProperties);

    /**
     * Whether or not the field is an exact match.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#exactMatch)
     */
    exactMatch: boolean;
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#name)
     */
    name: string;
    /**
     * The data type of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#type)
     */
    type:
      | "small-integer"
      | "integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#clone)
     */
    clone(): SearchTableField;
  }

  interface SearchTableFieldProperties {
    /**
     * Whether or not the field is an exact match.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#exactMatch)
     */
    exactMatch?: boolean;
    /**
     * The name of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#name)
     */
    name?: string;
    /**
     * The data type of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTableField.html#type)
     */
    type?:
      | "small-integer"
      | "integer"
      | "single"
      | "double"
      | "long"
      | "string"
      | "date"
      | "oid"
      | "geometry"
      | "blob"
      | "raster"
      | "guid"
      | "global-id"
      | "xml";
  }

  export class Viewing extends Accessor {
    /**
     * Represents view-specific properties of application and UI elements for the web map or web scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Viewing.html)
     */

    constructor(properties?: ViewingProperties);

    /**
     * An object specifying the search parameters set within the web scene or web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Viewing.html#search)
     */
    search: Search;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Viewing.html#clone)
     */
    clone(): Viewing;
  }

  interface ViewingProperties {
    /**
     * An object specifying the search parameters set within the web scene or web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-Viewing.html#search)
     */
    search?: SearchProperties;
  }

  export interface GeotriggersInfo extends Accessor, JSONSupport, Clonable {}

  export class GeotriggersInfo {
    /**
     * Information relating to a list of Geotriggers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-GeotriggersInfo.html)
     */

    constructor(properties?: GeotriggersInfoProperties);

    /**
     * A list of Geotriggers.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-GeotriggersInfo.html#geotriggers)
     */
    geotriggers: Geotrigger[];

    static fromJSON(json: any): GeotriggersInfo;
  }

  interface GeotriggersInfoProperties {
    /**
     * A list of Geotriggers.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-GeotriggersInfo.html#geotriggers)
     */
    geotriggers?: GeotriggerProperties[];
  }

  export interface DeviceLocationFeed extends Accessor, JSONSupport, Clonable {}

  export class DeviceLocationFeed {
    /**
     * A Geotrigger feed which uses the device location to provide updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-DeviceLocationFeed.html)
     */

    constructor(properties?: DeviceLocationFeedProperties);

    /**
     * An optional Arcade expression that controls whether a location update will be used by a geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-DeviceLocationFeed.html#filterExpression)
     */
    filterExpression: geotriggersInfoExpressionInfo;
    /**
     * String indicating the type of Geotrigger feed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-DeviceLocationFeed.html#type)
     */
    readonly type: "device-location";

    static fromJSON(json: any): DeviceLocationFeed;
  }

  interface DeviceLocationFeedProperties {
    /**
     * An optional Arcade expression that controls whether a location update will be used by a geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-DeviceLocationFeed.html#filterExpression)
     */
    filterExpression?: geotriggersInfoExpressionInfoProperties;
  }

  export interface geotriggersInfoExpressionInfo extends Accessor, JSONSupport, Clonable {}

  export class geotriggersInfoExpressionInfo {
    /**
     * Defines a script expression that can be used to compute values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html)
     */

    constructor(properties?: geotriggersInfoExpressionInfoProperties);

    /**
     * Optional expression in the [Arcade expression](https://developers.arcgis.com/arcade/) language.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#expression)
     */
    expression: string;
    /**
     * Optional return type of the Arcade expression.
     *
     * @default "string"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#returnType)
     */
    returnType: "number" | "string";
    /**
     * Optional title of the expression.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#title)
     */
    title: string;

    static fromJSON(json: any): geotriggersInfoExpressionInfo;
  }

  interface geotriggersInfoExpressionInfoProperties {
    /**
     * Optional expression in the [Arcade expression](https://developers.arcgis.com/arcade/) language.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#expression)
     */
    expression?: string;
    /**
     * Optional return type of the Arcade expression.
     *
     * @default "string"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#returnType)
     */
    returnType?: "number" | "string";
    /**
     * Optional title of the expression.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-ExpressionInfo.html#title)
     */
    title?: string;
  }

  export interface FeatureFenceParameters extends Accessor, JSONSupport, Clonable {}

  export class FeatureFenceParameters {
    /**
     * Fence parameters for a Geotrigger that uses feature data from an online feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html)
     */

    constructor(properties?: FeatureFenceParametersProperties);

    /**
     * An optional buffer distance to apply to fence features in meters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#bufferDistance)
     */
    bufferDistance: number;
    /**
     * An object defining the source for a feature layer to be used as fences.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#fenceSource)
     */
    fenceSource: FeatureLayerSource;
    /**
     * An optional filter to reduce the features used for the parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#filter)
     */
    filter: geotriggersInfoFeatureFilter;
    /**
     * String indicating the fence parameters type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#type)
     */
    readonly type: "features";

    static fromJSON(json: any): FeatureFenceParameters;
  }

  interface FeatureFenceParametersProperties {
    /**
     * An optional buffer distance to apply to fence features in meters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#bufferDistance)
     */
    bufferDistance?: number;
    /**
     * An object defining the source for a feature layer to be used as fences.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#fenceSource)
     */
    fenceSource?: FeatureLayerSourceProperties;
    /**
     * An optional filter to reduce the features used for the parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFenceParameters.html#filter)
     */
    filter?: geotriggersInfoFeatureFilterProperties;
  }

  export interface geotriggersInfoFeatureFilter extends Accessor, JSONSupport, Clonable {}

  export class geotriggersInfoFeatureFilter {
    /**
     * Description of spatial and attribute filters that will be applied to Feature data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFilter.html)
     */

    constructor(properties?: geotriggersInfoFeatureFilterProperties);

    /**
     * An optional geometry used to filter the features from a feature table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFilter.html#geometry)
     */
    geometry: Geometry;
    /**
     * An optional SQL-based definition expression string that narrows the data to be used.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFilter.html#where)
     */
    where: string;

    static fromJSON(json: any): geotriggersInfoFeatureFilter;
  }

  interface geotriggersInfoFeatureFilterProperties {
    /**
     * An optional geometry used to filter the features from a feature table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFilter.html#geometry)
     */
    geometry?: GeometryProperties;
    /**
     * An optional SQL-based definition expression string that narrows the data to be used.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureFilter.html#where)
     */
    where?: string;
  }

  export interface FeatureLayerSource extends Accessor, JSONSupport, Clonable {}

  export class FeatureLayerSource {
    /**
     * The source for a feature layer to be used as fences for Geotriggers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html)
     */

    constructor(properties?: FeatureLayerSourceProperties);

    /**
     * A unique identifying string that must match the `id` property of a feature layer in an associated map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html#layerId)
     */
    layerId: string;
    /**
     * A URL to a service that will be used for all queries against the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html#layerUrl)
     */
    layerUrl: string;
    /**
     * String indicating the type of source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html#type)
     */
    readonly type: "feature-layer";

    static fromJSON(json: any): FeatureLayerSource;
  }

  interface FeatureLayerSourceProperties {
    /**
     * A unique identifying string that must match the `id` property of a feature layer in an associated map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html#layerId)
     */
    layerId?: string;
    /**
     * A URL to a service that will be used for all queries against the layer.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FeatureLayerSource.html#layerUrl)
     */
    layerUrl?: string;
  }

  export interface FenceGeotrigger extends Geotrigger, JSONSupport, Clonable {}

  export class FenceGeotrigger {
    /**
     * A condition which monitors the dynamic elements of the geotrigger feed for enter/exit against the fences defined by the Fence Parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html)
     */

    constructor(properties?: FenceGeotriggerProperties);

    /**
     * The rule that determines whether a fence polygon has been entered or exited by the geometry from a feed.
     *
     * @default "enter-contains-and-exit-does-not-intersect"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#enterExitRule)
     */
    enterExitRule:
      | "enter-contains-and-exit-does-not-contain"
      | "enter-contains-and-exit-does-not-intersect"
      | "enter-intersects-and-exit-does-not-intersect";
    /**
     * The feed for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#feed)
     */
    feed: DeviceLocationFeed;
    /**
     * Indicates how the geotrigger will use accuracy information from a feed.
     *
     * @default "use-geometry"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#feedAccuracyMode)
     */
    feedAccuracyMode: "use-geometry" | "use-geometry-with-accuracy";
    /**
     * Indicates the type of event that will trigger notifications for the Fence Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#fenceNotificationRule)
     */
    fenceNotificationRule: "enter" | "enter-or-exit" | "exit";
    /**
     * An object defining the fences to use for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#fenceParameters)
     */
    fenceParameters: FeatureFenceParameters;
    /**
     * The name for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#name)
     */
    name: string;
    /**
     * Options that control the notification information sent to a client app when a Geotrigger condition is met.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#notificationOptions)
     */
    notificationOptions: GeotriggerNotificationOptions;

    static fromJSON(json: any): FenceGeotrigger;
  }

  interface FenceGeotriggerProperties extends GeotriggerProperties {
    /**
     * The rule that determines whether a fence polygon has been entered or exited by the geometry from a feed.
     *
     * @default "enter-contains-and-exit-does-not-intersect"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#enterExitRule)
     */
    enterExitRule?:
      | "enter-contains-and-exit-does-not-contain"
      | "enter-contains-and-exit-does-not-intersect"
      | "enter-intersects-and-exit-does-not-intersect";
    /**
     * The feed for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#feed)
     */
    feed?: DeviceLocationFeedProperties;
    /**
     * Indicates how the geotrigger will use accuracy information from a feed.
     *
     * @default "use-geometry"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#feedAccuracyMode)
     */
    feedAccuracyMode?: "use-geometry" | "use-geometry-with-accuracy";
    /**
     * Indicates the type of event that will trigger notifications for the Fence Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#fenceNotificationRule)
     */
    fenceNotificationRule?: "enter" | "enter-or-exit" | "exit";
    /**
     * An object defining the fences to use for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#fenceParameters)
     */
    fenceParameters?: FeatureFenceParametersProperties;
    /**
     * The name for this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#name)
     */
    name?: string;
    /**
     * Options that control the notification information sent to a client app when a Geotrigger condition is met.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-FenceGeotrigger.html#notificationOptions)
     */
    notificationOptions?: GeotriggerNotificationOptionsProperties;
  }

  export interface Geotrigger extends Accessor, JSONSupport, Clonable {}

  export class Geotrigger {
    /**
     * A Geotrigger is a condition that will be monitored against dynamic GIS data - for example using a spatial operation to check for enter/exit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-Geotrigger.html)
     */

    constructor(properties?: GeotriggerProperties);

    /**
     * String indicating the Geotrigger condition type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-Geotrigger.html#type)
     */
    readonly type: "fence";

    static fromJSON(json: any): Geotrigger;
  }

  interface GeotriggerProperties {}

  export interface GeotriggerNotificationOptions extends Accessor, JSONSupport, Clonable {}

  export class GeotriggerNotificationOptions {
    /**
     * Options that control the notification information sent to a client app client when a Geotrigger condition is met.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-GeotriggerNotificationOptions.html)
     */

    constructor(properties?: GeotriggerNotificationOptionsProperties);

    /**
     * An optional Arcade expression which can be used to configure notification information when the Geotrigger condition is met.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-GeotriggerNotificationOptions.html#expressionInfo)
     */
    expressionInfo: geotriggersInfoExpressionInfo;
    /**
     * An optional list of strings indicating the set of possible actions resulting from this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-GeotriggerNotificationOptions.html#requestedActions)
     */
    requestedActions: string[];

    static fromJSON(json: any): GeotriggerNotificationOptions;
  }

  interface GeotriggerNotificationOptionsProperties {
    /**
     * An optional Arcade expression which can be used to configure notification information when the Geotrigger condition is met.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-GeotriggerNotificationOptions.html#expressionInfo)
     */
    expressionInfo?: geotriggersInfoExpressionInfoProperties;
    /**
     * An optional list of strings indicating the set of possible actions resulting from this Geotrigger.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-geotriggersInfo-GeotriggerNotificationOptions.html#requestedActions)
     */
    requestedActions?: string[];
  }

  export interface PositioningService extends Accessor, JSONSupport, Clonable {}

  export class PositioningService {
    /**
     * Provides indoor positioning data service information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-ips-PositioningService.html)
     */

    constructor(properties?: PositioningServiceProperties);

    /**
     * Feature Service item representing indoor positioning data service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-ips-PositioningService.html#portalItem)
     */
    portalItem: PortalItem;

    static fromJSON(json: any): PositioningService;
  }

  interface PositioningServiceProperties {
    /**
     * Feature Service item representing indoor positioning data service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-ips-PositioningService.html#portalItem)
     */
    portalItem?: PortalItemProperties;
  }

  export interface IPSInfo extends Accessor, JSONSupport, Clonable {}

  export class IPSInfo {
    /**
     * The indoor positioning system (IPS) information for a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-IPSInfo.html)
     */

    constructor(properties?: IPSInfoProperties);

    /**
     * Defines the portal item for the positioning data service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-IPSInfo.html#poisitioningService)
     */
    poisitioningService: PositioningService;

    static fromJSON(json: any): IPSInfo;
  }

  interface IPSInfoProperties {
    /**
     * Defines the portal item for the positioning data service.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-IPSInfo.html#poisitioningService)
     */
    poisitioningService?: PositioningServiceProperties;
  }

  export interface TimeSlider extends Accessor, JSONSupport {}

  export class TimeSlider {
    /**
     * Time animation is controlled by a configurable [time slider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html)
     */

    constructor(properties?: TimeSliderProperties);

    /**
     * The current time extent of the time slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#currentTimeExtent)
     */
    currentTimeExtent: TimeExtent;
    /**
     * The temporal extent for the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#fullTimeExtent)
     */
    fullTimeExtent: TimeExtent;
    /**
     * When `true`, the time slider will play its animation in a loop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#loop)
     */
    loop: boolean;
    /**
     * The number of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#numStops)
     */
    numStops: number;
    /**
     * The thumb count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#numThumbs)
     */
    numThumbs: number;
    /**
     * The time rate in milliseconds between animation steps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stopDelay)
     */
    stopDelay: number;
    /**
     * Defines regularly spaced stops on the time slider from a [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stopInterval)
     */
    stopInterval: TimeInterval;
    /**
     * An array of dates for the time slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stops)
     */
    stops: Date[];

    static fromJSON(json: any): TimeSlider;
  }

  interface TimeSliderProperties {
    /**
     * The current time extent of the time slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#currentTimeExtent)
     */
    currentTimeExtent?: TimeExtentProperties;
    /**
     * The temporal extent for the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#fullTimeExtent)
     */
    fullTimeExtent?: TimeExtentProperties;
    /**
     * When `true`, the time slider will play its animation in a loop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#loop)
     */
    loop?: boolean;
    /**
     * The number of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#numStops)
     */
    numStops?: number;
    /**
     * The thumb count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#numThumbs)
     */
    numThumbs?: number;
    /**
     * The time rate in milliseconds between animation steps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stopDelay)
     */
    stopDelay?: number;
    /**
     * Defines regularly spaced stops on the time slider from a [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html) object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stopInterval)
     */
    stopInterval?: TimeIntervalProperties;
    /**
     * An array of dates for the time slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-widgets-TimeSlider.html#stops)
     */
    stops?: DateProperties[];
  }

  export interface WebDocument2D extends Map, corePromise {}

  export class WebDocument2D {
    /**
     * WebDocument2D is the base class for all 2D web documents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html)
     */

    constructor(properties?: WebDocument2DProperties);

    /**
     * The applicationProperties contains the viewing properties of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#applicationProperties)
     */
    applicationProperties: ApplicationProperties;
    /**
     * The name of the application that authored the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#authoringApp)
     */
    authoringApp: string;
    /**
     * The version of the application that authored the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#authoringAppVersion)
     */
    authoringAppVersion: string;
    /**
     * An array of saved geographic extents that allow end users to quickly navigate to a particular area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#bookmarks)
     */
    bookmarks: Collection<Bookmark>;
    /**
     * When a web map is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#floorInfo)
     */
    floorInfo: MapFloorInfo;
    /**
     * The initial view of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#initialViewProperties)
     */
    initialViewProperties: InitialViewProperties;
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * The portal item from which the WebMap is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * The URL to the thumbnail used for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#thumbnailUrl)
     */
    thumbnailUrl: string;
    /**
     * The widgets object contains widgets that are exposed to the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#widgets)
     */
    widgets: Widgets;

    /**
     * Triggers the loading of the WebMap instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#load)
     */
    load(): Promise<any>;
    /**
     * Loads all the externally loadable resources associated with the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#loadAll)
     */
    loadAll(): Promise<WebMap>;
    /**
     * Update properties of the WebMap related to the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    updateFrom(view: MapView, options?: WebDocument2DUpdateFromOptions): Promise<any>;
  }

  interface WebDocument2DProperties extends MapProperties {
    /**
     * The applicationProperties contains the viewing properties of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#applicationProperties)
     */
    applicationProperties?: ApplicationPropertiesProperties;
    /**
     * The name of the application that authored the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#authoringApp)
     */
    authoringApp?: string;
    /**
     * The version of the application that authored the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#authoringAppVersion)
     */
    authoringAppVersion?: string;
    /**
     * An array of saved geographic extents that allow end users to quickly navigate to a particular area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#bookmarks)
     */
    bookmarks?: CollectionProperties<BookmarkProperties>;
    /**
     * When a web map is configured as floor-aware, it has a floorInfo property defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#floorInfo)
     */
    floorInfo?: MapFloorInfoProperties;
    /**
     * The initial view of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#initialViewProperties)
     */
    initialViewProperties?: InitialViewPropertiesProperties;
    /**
     * The portal item from which the WebMap is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * The URL to the thumbnail used for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#thumbnailUrl)
     */
    thumbnailUrl?: string;
    /**
     * The widgets object contains widgets that are exposed to the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#widgets)
     */
    widgets?: Widgets;
  }

  export interface WebDocument2DUpdateFromOptions {
    /**
     * When `true`, the view's background will not be updated for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    backgroundExcluded?: boolean;
    /**
     * When `true`, the initial [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#viewpoint) of the view will not be updated for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    viewpointExcluded?: boolean;
    /**
     * When `true`, the view's scale will be updated for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    scalePreserved?: boolean;
    /**
     * When `true`, the thumbnail will not be updated for the webmap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    thumbnailExcluded?: boolean;
    /**
     * The size of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    thumbnailSize?: WebDocument2DUpdateFromOptionsThumbnailSize;
    /**
     * When `true`, the webmap's [widgets](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#widgets) property will not be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    widgetsExcluded?: boolean;
  }

  export interface WebDocument2DUpdateFromOptionsThumbnailSize {
    /**
     * The width of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    width: number;
    /**
     * The height of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebDocument2D.html#updateFrom)
     */
    height: number;
  }

  export class WebLinkChart extends WebDocument2D {
    /**
     * The WebLinkChart class contains properties and methods for storing, managing, and overlaying layers in a link chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebLinkChart.html)
     */

    constructor(properties?: WebLinkChartProperties);
  }

  interface WebLinkChartProperties extends WebDocument2DProperties {}

  export class WebMap extends WebDocument2D {
    /**
     * Loads a [WebMap](https://doc.arcgis.com/en/arcgis-online/create-maps/make-your-first-map.htm) from [ArcGIS Online](https://www.arcgis.com/home/) or [ArcGIS Enterprise portal](https://enterprise.arcgis.com/en/portal/latest/administer/windows/what-is-portal-for-arcgis-.htm) into a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html)
     */

    constructor(properties?: WebMapProperties);

    /**
     * Information relating to a list of Geotriggers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#geotriggersInfo)
     */
    geotriggersInfo: GeotriggersInfo;
    /**
     * Contains indoor positioning system information for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#ipsInfo)
     */
    ipsInfo: IPSInfo;
    /**
     * Provides multiple slides.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#presentation)
     */
    presentation: any;
    /**
     * The version of the source document from which the WebMap was read.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#sourceVersion)
     */
    readonly sourceVersion: WebMapSourceVersion;
    /**
     * The utilityNetworks object contains a collection of [UtilityNetworks](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html) saved on the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#utilityNetworks)
     */
    utilityNetworks: Collection<UtilityNetwork>;

    /**
     * Saves the webmap to its associated portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#save)
     */
    save(options?: WebMapSaveOptions): Promise<PortalItem>;
    /**
     * Saves the webmap to a new [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: WebMapSaveAsOptions): Promise<PortalItem>;

    /**
     * Creates a new instance of this class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#fromJSON)
     */
    static fromJSON(json: any): any;
  }

  interface WebMapProperties extends WebDocument2DProperties {
    /**
     * Information relating to a list of Geotriggers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#geotriggersInfo)
     */
    geotriggersInfo?: GeotriggersInfoProperties;
    /**
     * Contains indoor positioning system information for the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#ipsInfo)
     */
    ipsInfo?: IPSInfoProperties;
    /**
     * Provides multiple slides.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#presentation)
     */
    presentation?: any;
    /**
     * The utilityNetworks object contains a collection of [UtilityNetworks](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html) saved on the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#utilityNetworks)
     */
    utilityNetworks?: CollectionProperties<UtilityNetworkProperties>;
  }

  export class ApplicationProperties extends Accessor {
    /**
     * Represents configuration of application and UI elements of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-ApplicationProperties.html)
     */

    constructor(properties?: ApplicationPropertiesProperties);

    /**
     * View-specific properties of application and UI elements for the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-ApplicationProperties.html#viewing)
     */
    viewing: Viewing;
  }

  interface ApplicationPropertiesProperties {
    /**
     * View-specific properties of application and UI elements for the web map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-ApplicationProperties.html#viewing)
     */
    viewing?: ViewingProperties;
  }

  export interface ColorBackground extends Accessor, JSONSupport {}

  export class ColorBackground {
    /**
     * Represents the background color of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) when displayed in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-background-ColorBackground.html)
     */

    constructor(properties?: ColorBackgroundProperties);

    /**
     * The color of the background.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-background-ColorBackground.html#color)
     */
    color: Color;

    /**
     * Creates a deep clone of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-background-ColorBackground.html#clone)
     */
    clone(): ColorBackground;

    static fromJSON(json: any): ColorBackground;
  }

  interface ColorBackgroundProperties {
    /**
     * The color of the background.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-background-ColorBackground.html#color)
     */
    color?: Color | number[] | string;
  }

  export interface Bookmark extends Accessor, JSONSupport, Identifiable {}

  export class Bookmark {
    /**
     * A bookmark is a saved map extent that allows end users to quickly navigate to a particular area of interest using the [Bookmarks](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html)
     */

    constructor(properties?: BookmarkProperties);

    /**
     * The name of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#name)
     */
    name: string;
    /**
     * The URL for a thumbnail image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#thumbnail)
     */
    thumbnail: BookmarkThumbnail;
    /**
     * The time extent of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The viewpoint of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#viewpoint)
     */
    viewpoint: Viewpoint;

    static fromJSON(json: any): Bookmark;
  }

  interface BookmarkProperties {
    /**
     * The name of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#name)
     */
    name?: string;
    /**
     * The URL for a thumbnail image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#thumbnail)
     */
    thumbnail?: BookmarkThumbnail;
    /**
     * The time extent of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The viewpoint of the bookmark item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
  }

  export interface BookmarkThumbnail {
    /**
     * URL to a thumbnail to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#thumbnail)
     */
    url?: string;
  }

  export interface InitialViewProperties extends Accessor, Clonable {}

  export class InitialViewProperties {
    /**
     * Represents the initial viewing state of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) when displayed in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html)
     */

    constructor(properties?: InitialViewPropertiesProperties);

    /**
     * The background color of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#background)
     */
    background: ColorBackground;
    /**
     * The spatial reference of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The initial [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The initial time zone of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#timeZone)
     */
    timeZone: string;
    /**
     * The initial viewpoint of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#viewpoint)
     */
    viewpoint: Viewpoint;
  }

  interface InitialViewPropertiesProperties {
    /**
     * The background color of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#background)
     */
    background?: ColorBackgroundProperties;
    /**
     * The spatial reference of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The initial [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The initial time zone of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#timeZone)
     */
    timeZone?: string;
    /**
     * The initial viewpoint of the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-InitialViewProperties.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
  }

  /**
   * Floor filtering is controlled by a configurable [floor filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
   */
  export interface WebMapFloorFilter {
    /**
     * Indicates whether the FloorFilter is active and filtering the displayed content according to the [FloorFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html) selection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    enabled: boolean;
    /**
     * Indicates whether the levels list is showing the long names from longNameField.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    longNames: boolean;
    /**
     * Indicates whether the floor filter has been minimized to show only the levels list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    minimized: boolean;
    /**
     * Indicates whether the levels portion of the floor filter has been pinned to show the levels list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    pinnedLevels: boolean;
    /**
     * Contains the site ID for the initially selected [site](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#site) in the floor filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    site: string;
    /**
     * Contains the facility ID for the initially selected [facility](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#facility) in the floor filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    facility: string;
    /**
     * Contains the level ID for the initially selected floor, which is used when filtering layers by their configured floor-aware properties.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter)
     */
    level: string;
  }

  export interface WebMapSaveAsOptions {
    /**
     * The folder in which to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * Allow the webmap to be saved even in the case it contains unsupported content (layers, renderers, symbols).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface WebMapSaveOptions {
    /**
     * When `true`, the webmap will save even if it contains unsupported content (layers, renderers, symbols).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export interface WebMapSourceVersion {
    /**
     * The major version of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#sourceVersion)
     */
    major: number;
    /**
     * The minor version of the WebMap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#sourceVersion)
     */
    minor: number;
  }

  /**
   * The widgets object contains widgets that are exposed to the user.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#Widgets)
   */
  export interface Widgets {
    /**
     * Time animation is controlled by a configurable [time slider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#Widgets)
     */
    timeSlider: TimeSlider;
    /**
     * Floor filtering is controlled by a configurable [floor filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#Widgets)
     */
    floorFilter: WebMapFloorFilter;
  }

  export interface WebScene extends Map, corePromise {}

  export class WebScene {
    /**
     * The web scene is the core element of 3D mapping across ArcGIS.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html)
     */

    constructor(properties?: WebSceneProperties);

    /**
     * Configuration of application and UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#applicationProperties)
     */
    applicationProperties: websceneApplicationProperties;
    /**
     * The name of the application that authored the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#authoringApp)
     */
    authoringApp: string;
    /**
     * The version of the application that authored the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#authoringAppVersion)
     */
    authoringAppVersion: string;
    /**
     * *This property only applies to local scenes.* Represents an optional clipping area used to define the bounds or [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of a local scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingArea)
     */
    clippingArea: Extent;
    /**
     * *This property only applies to local scenes.* Determines whether clipping using the [clippingArea](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingArea) is enabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingEnabled)
     */
    clippingEnabled: boolean;
    /**
     * When a web scene is configured as floor-aware, it has a floorInfo property defined.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#floorInfo)
     */
    floorInfo: MapFloorInfo;
    /**
     * The height model info of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#heightModelInfo)
     */
    heightModelInfo: HeightModelInfo;
    /**
     * The initial view of the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#initialViewProperties)
     */
    initialViewProperties: websceneInitialViewProperties;
    /**
     * Indicates whether the instance has loaded.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#loaded)
     */
    readonly loaded: boolean;
    /**
     * The Error object returned if an error occurred while loading.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#loadError)
     */
    readonly loadError: Error;
    /**
     * Represents the status of a [load](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#load) operation.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#loadStatus)
     */
    readonly loadStatus: "not-loaded" | "loading" | "failed" | "loaded";
    /**
     * The portal item from which the WebScene is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#portalItem)
     */
    portalItem: PortalItem;
    /**
     * Provides a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of slides that act as bookmarks for saving predefined [viewpoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) and visible layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#presentation)
     */
    presentation: Presentation;
    /**
     * The version of the source document from which the WebScene was read.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#sourceVersion)
     */
    readonly sourceVersion: WebSceneSourceVersion;
    /**
     * The URL to the thumbnail used for the web scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#thumbnailUrl)
     */
    thumbnailUrl: string;
    /**
     * The widgets object contains widgets that are exposed to the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#widgets)
     */
    widgets: WebSceneWidgets;

    /**
     * Triggers the loading of the WebScene instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#load)
     */
    load(): Promise<any>;
    /**
     * Loads all the externally loadable resources associated with the webscene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#loadAll)
     */
    loadAll(): Promise<WebScene>;
    /**
     * Saves the webscene to its associated portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#save)
     */
    save(options?: WebSceneSaveOptions): Promise<PortalItem>;
    /**
     * Saves the webscene to a new portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#saveAs)
     */
    saveAs(portalItem: PortalItem | PortalItemProperties, options?: WebSceneSaveAsOptions): Promise<PortalItem>;
    /**
     * Converts an instance of this class to its ArcGIS portal JSON representation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#toJSON)
     */
    toJSON(): any;
    /**
     * Update properties of the WebScene related to the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    updateFrom(view: SceneView, options?: WebSceneUpdateFromOptions): Promise<any>;

    /**
     * The latest supported webscene spec version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#version)
     */
    static readonly version: string;

    /**
     * Creates a new instance of this class and initializes it with values from a JSON object generated from an ArcGIS product.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#fromJSON)
     */
    static fromJSON(json: any): WebScene;
  }

  interface WebSceneProperties extends MapProperties {
    /**
     * Configuration of application and UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#applicationProperties)
     */
    applicationProperties?: websceneApplicationPropertiesProperties;
    /**
     * The name of the application that authored the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#authoringApp)
     */
    authoringApp?: string;
    /**
     * The version of the application that authored the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#authoringAppVersion)
     */
    authoringAppVersion?: string;
    /**
     * *This property only applies to local scenes.* Represents an optional clipping area used to define the bounds or [Extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Extent.html) of a local scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingArea)
     */
    clippingArea?: ExtentProperties;
    /**
     * *This property only applies to local scenes.* Determines whether clipping using the [clippingArea](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingArea) is enabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#clippingEnabled)
     */
    clippingEnabled?: boolean;
    /**
     * When a web scene is configured as floor-aware, it has a floorInfo property defined.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#floorInfo)
     */
    floorInfo?: MapFloorInfoProperties;
    /**
     * The height model info of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#heightModelInfo)
     */
    heightModelInfo?: HeightModelInfoProperties;
    /**
     * The initial view of the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#initialViewProperties)
     */
    initialViewProperties?: websceneInitialViewPropertiesProperties;
    /**
     * The portal item from which the WebScene is loaded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#portalItem)
     */
    portalItem?: PortalItemProperties;
    /**
     * Provides a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of slides that act as bookmarks for saving predefined [viewpoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) and visible layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#presentation)
     */
    presentation?: PresentationProperties;
    /**
     * The URL to the thumbnail used for the web scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#thumbnailUrl)
     */
    thumbnailUrl?: string;
    /**
     * The widgets object contains widgets that are exposed to the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#widgets)
     */
    widgets?: WebSceneWidgets;
  }

  export class websceneApplicationProperties extends Accessor {
    /**
     * Represents configuration of application and UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-ApplicationProperties.html)
     */

    constructor(properties?: websceneApplicationPropertiesProperties);

    /**
     * View-specific properties of application and UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-ApplicationProperties.html#viewing)
     */
    viewing: Viewing;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-ApplicationProperties.html#clone)
     */
    clone(): websceneApplicationProperties;
  }

  interface websceneApplicationPropertiesProperties {
    /**
     * View-specific properties of application and UI elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-ApplicationProperties.html#viewing)
     */
    viewing?: ViewingProperties;
  }

  export interface Background extends Accessor, JSONSupport {}

  export class Background {
    /**
     * The background of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) that lies behind the sky, atmosphere and stars.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-Background.html)
     */

    constructor(properties?: BackgroundProperties);

    /**
     * Creates a deep clone of the Background.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-Background.html#clone)
     */
    clone(): Background;

    static fromJSON(json: any): Background;
  }

  interface BackgroundProperties {}

  export interface backgroundColorBackground extends Background, JSONSupport {}

  export class backgroundColorBackground {
    /**
     * This type of [Background](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-Background.html) allows to set a [Color](https://developers.arcgis.com/javascript/latest/api-reference/esri-Color.html) as the background of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-ColorBackground.html)
     */

    constructor(properties?: backgroundColorBackgroundProperties);

    /**
     * The color of the background.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-ColorBackground.html#color)
     */
    color: Color;

    /**
     * Creates a deep clone of the object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-ColorBackground.html#clone)
     */
    clone(): backgroundColorBackground;

    static fromJSON(json: any): backgroundColorBackground;
  }

  interface backgroundColorBackgroundProperties extends BackgroundProperties {
    /**
     * The color of the background.
     *
     * @default "black"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-background-ColorBackground.html#color)
     */
    color?: Color | number[] | string;
  }

  export class Environment extends Accessor {
    /**
     * Represents settings that affect the environment in which the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html) is displayed (such as lighting).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html)
     */

    constructor(properties?: EnvironmentProperties);

    /**
     * Specifies whether the atmosphere should be displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#atmosphereEnabled)
     */
    atmosphereEnabled: boolean;
    /**
     * Specifies how the background of the scene (which lies behind sky, stars and atmosphere) should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#background)
     */
    background: Background;
    /**
     * Settings for defining the lighting of the scene.
     *
     * @default SunLighting
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#lighting)
     */
    lighting: SunLighting | VirtualLighting;
    /**
     * Specifies whether stars should be displayed in the sky.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#starsEnabled)
     */
    starsEnabled: boolean;
    /**
     * Indicates the type of weather visualization in the scene.
     *
     * @default SunnyWeather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#weather)
     */
    weather: SunnyWeather | CloudyWeather | RainyWeather | SnowyWeather | FoggyWeather;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#clone)
     */
    clone(): Environment;
  }

  interface EnvironmentProperties {
    /**
     * Specifies whether the atmosphere should be displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#atmosphereEnabled)
     */
    atmosphereEnabled?: boolean;
    /**
     * Specifies how the background of the scene (which lies behind sky, stars and atmosphere) should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#background)
     */
    background?: BackgroundProperties;
    /**
     * Settings for defining the lighting of the scene.
     *
     * @default SunLighting
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#lighting)
     */
    lighting?: (SunLightingProperties & { type?: "sun" }) | (VirtualLightingProperties & { type: "virtual" });
    /**
     * Specifies whether stars should be displayed in the sky.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#starsEnabled)
     */
    starsEnabled?: boolean;
    /**
     * Indicates the type of weather visualization in the scene.
     *
     * @default SunnyWeather
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html#weather)
     */
    weather?:
      | (SunnyWeatherProperties & { type: "sunny" })
      | (CloudyWeatherProperties & { type: "cloudy" })
      | (RainyWeatherProperties & { type: "rainy" })
      | (SnowyWeatherProperties & { type: "snowy" })
      | (FoggyWeatherProperties & { type: "foggy" });
  }

  export class websceneInitialViewProperties extends Accessor {
    /**
     * Represents the initial viewing state of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html) when displayed in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html)
     */

    constructor(properties?: websceneInitialViewPropertiesProperties);

    /**
     * The initial environment settings of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#environment)
     */
    environment: Environment;
    /**
     * The spatial reference of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The initial time extent of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The viewing mode of the scene.
     *
     * @default "global"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#viewingMode)
     */
    viewingMode: "global" | "local";
    /**
     * The initial viewpoint of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#viewpoint)
     */
    viewpoint: Viewpoint;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#clone)
     */
    clone(): websceneInitialViewProperties;
  }

  interface websceneInitialViewPropertiesProperties {
    /**
     * The initial environment settings of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#environment)
     */
    environment?: EnvironmentProperties;
    /**
     * The spatial reference of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The initial time extent of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The viewing mode of the scene.
     *
     * @default "global"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#viewingMode)
     */
    viewingMode?: "global" | "local";
    /**
     * The initial viewpoint of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-InitialViewProperties.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
  }

  export class Presentation extends Accessor {
    /**
     * A presentation contains a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [slides](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html) that allows users to quickly navigate to predefined settings of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html)
     */

    constructor(properties?: PresentationProperties);

    /**
     * A collection of [slides](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html) that bookmark [viewpoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), visible layers, and other settings previously defined in a [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides)
     */
    slides: Collection<Slide>;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#clone)
     */
    clone(): Presentation;
  }

  interface PresentationProperties {
    /**
     * A collection of [slides](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html) that bookmark [viewpoints](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), visible layers, and other settings previously defined in a [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides)
     */
    slides?: CollectionProperties<SlideProperties>;
  }

  export class Slide extends Accessor {
    /**
     * A slide stores a snapshot of several pre-set properties of the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html) and [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html), such as the [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#basemap), [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#viewpoint) and [visible layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html)
     */

    constructor(properties?: SlideProperties);

    /**
     * The basemap of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#basemap)
     */
    basemap: Basemap;
    /**
     * The description of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#description)
     */
    description: SlideDescription;
    /**
     * Represents settings that affect the environment in which the WebScene is displayed (such as lighting).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#environment)
     */
    environment: SlideEnvironment;
    /**
     * Ground properties for this slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#ground)
     */
    ground: SlideGround;
    /**
     * The visibility of a slide in a presentation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#hidden)
     */
    hidden: boolean;
    /**
     * The unique id of a slide within the [slides property](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides) of a [Presentation](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#id)
     */
    id: string;
    /**
     * Layout of the slide.
     *
     * @default "caption"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#layout)
     */
    layout: "caption" | "cover" | "none";
    /**
     * A data URI encoded thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#thumbnail)
     */
    thumbnail: SlideThumbnail;
    /**
     * The time extent of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * The title of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#title)
     */
    title: SlideTitle;
    /**
     * The viewpoint of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#viewpoint)
     */
    viewpoint: Viewpoint;
    /**
     * The visible layers of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    visibleLayers: Collection<SlideVisibleLayers>;

    /**
     * Applies a slide's settings to a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#applyTo)
     */
    applyTo(view: SceneView, optionsWithDefaults?: any): Promise<Slide>;
    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#clone)
     */
    clone(): Slide;
    /**
     * Updates a slide from a [WebScene's slides](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    updateFrom(view: SceneView, options?: SlideUpdateFromOptions): Promise<Slide>;

    /**
     * Creates a slide from a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html), which may be added to the [slides](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides) in the WebScene's [presentation](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#presentation).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    static createFrom(view: SceneView, options?: SlideCreateFromOptions): Promise<Slide>;
  }

  interface SlideProperties {
    /**
     * The basemap of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#basemap)
     */
    basemap?: BasemapProperties | string;
    /**
     * The description of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#description)
     */
    description?: SlideDescriptionProperties | string;
    /**
     * Represents settings that affect the environment in which the WebScene is displayed (such as lighting).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#environment)
     */
    environment?: SlideEnvironmentProperties;
    /**
     * Ground properties for this slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#ground)
     */
    ground?: SlideGroundProperties;
    /**
     * The visibility of a slide in a presentation.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#hidden)
     */
    hidden?: boolean;
    /**
     * The unique id of a slide within the [slides property](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html#slides) of a [Presentation](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Presentation.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#id)
     */
    id?: string;
    /**
     * Layout of the slide.
     *
     * @default "caption"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#layout)
     */
    layout?: "caption" | "cover" | "none";
    /**
     * A data URI encoded thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#thumbnail)
     */
    thumbnail?: SlideThumbnailProperties | string;
    /**
     * The time extent of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * The title of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#title)
     */
    title?: SlideTitleProperties | string;
    /**
     * The viewpoint of the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
    /**
     * The visible layers of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    visibleLayers?: CollectionProperties<SlideVisibleLayersProperties>;
  }

  export interface SlideCreateFromOptions {
    /**
     * Screenshot options to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    screenshot?: SlideCreateFromOptionsScreenshot;
  }

  export interface SlideCreateFromOptionsScreenshot {
    /**
     * The image format.
     *
     * @default png
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    format?: string;
    /**
     * The image quality (due to compression).
     *
     * @default 80
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    quality?: number;
    /**
     * The image width.
     *
     * @default 120
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    width?: number;
    /**
     * The image height.
     *
     * @default 75
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#createFrom)
     */
    height?: number;
  }

  export interface SlideDescriptionProperties {
    /**
     * The description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#description)
     */
    text?: string;
  }

  export interface SlideDescription extends AnonymousAccessor {
    /**
     * The description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#description)
     */
    text?: string;
  }

  export interface SlideEnvironmentProperties {
    /**
     * Settings for defining the lighting of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#environment)
     */
    lighting?:
      | (websceneSunLightingProperties & { type?: "sun" })
      | (websceneVirtualLightingProperties & { type: "virtual" });
  }

  export interface SlideEnvironment extends AnonymousAccessor {
    /**
     * Settings for defining the lighting of the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#environment)
     */
    lighting?: websceneSunLighting | websceneVirtualLighting;
  }

  export interface SlideGroundProperties {
    /**
     * Ground opacity
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#ground)
     */
    opacity?: number;
  }

  export interface SlideGround extends AnonymousAccessor {
    /**
     * Ground opacity
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#ground)
     */
    opacity?: number;
  }

  export interface SlideThumbnailProperties {
    /**
     * The URI pointing to the thumbnail image representing the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#thumbnail)
     */
    url?: string;
  }

  export interface SlideThumbnail extends AnonymousAccessor {
    /**
     * The URI pointing to the thumbnail image representing the slide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#thumbnail)
     */
    url?: string;
  }

  export interface SlideTitleProperties {
    /**
     * The title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#title)
     */
    text?: string;
  }

  export interface SlideTitle extends AnonymousAccessor {
    /**
     * The title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#title)
     */
    text?: string;
  }

  export interface SlideUpdateFromOptions {
    /**
     * Screenshot options to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    screenshot: SlideUpdateFromOptionsScreenshot;
  }

  export interface SlideUpdateFromOptionsScreenshot {
    /**
     * The image format.
     *
     * @default png
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    format?: string;
    /**
     * The image quality (due to compression).
     *
     * @default 80
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    quality?: number;
    /**
     * The image width.
     *
     * @default 120
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    width?: number;
    /**
     * The image height.
     *
     * @default 75
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#updateFrom)
     */
    height?: number;
  }

  export interface SlideVisibleLayersProperties {
    /**
     * The ID of a [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#id) in the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#layers) or [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers) that is made `visible` in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) when the slide is applied to the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    id?: string;
    /**
     * The service ids of the visible sublayers on the layer specified by the layer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    sublayerIds?: number[];
  }

  export interface SlideVisibleLayers extends AnonymousAccessor {
    /**
     * The ID of a [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#id) in the [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#layers) or [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers) that is made `visible` in the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) when the slide is applied to the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    id: string;
    /**
     * The service ids of the visible sublayers on the layer specified by the layer id.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Slide.html#visibleLayers)
     */
    sublayerIds: number[];
  }

  export class websceneSunLighting extends Accessor {
    /**
     * The sun lighting object is part of the [webscene/Environment](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html) and contains information relating to how a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) is lit by the sun.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html)
     */

    constructor(properties?: websceneSunLightingProperties);

    /**
     * The current date and time of the simulated sun.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#date)
     */
    date: Date;
    /**
     * Indicates whether to show shadows cast by the sun.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#directShadowsEnabled)
     */
    directShadowsEnabled: boolean;
    /**
     * The UTC time zone offset in hours that should be displayed in the UI to represent the date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#displayUTCOffset)
     */
    displayUTCOffset: number;
    /**
     * Indicates that the light source is simulated position of the sun.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#type)
     */
    readonly type: "sun";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#clone)
     */
    clone(): websceneSunLighting;
  }

  interface websceneSunLightingProperties {
    /**
     * The current date and time of the simulated sun.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#date)
     */
    date?: DateProperties;
    /**
     * Indicates whether to show shadows cast by the sun.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#directShadowsEnabled)
     */
    directShadowsEnabled?: boolean;
    /**
     * The UTC time zone offset in hours that should be displayed in the UI to represent the date.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-SunLighting.html#displayUTCOffset)
     */
    displayUTCOffset?: number;
  }

  export class websceneVirtualLighting extends Accessor {
    /**
     * The virtual lighting object is part of the [webscene/Environment](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-Environment.html) and contains information relating to how a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) is lit with a virtual light.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-VirtualLighting.html)
     */

    constructor(properties?: websceneVirtualLightingProperties);

    /**
     * Indicates whether to show shadows cast by the light source.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-VirtualLighting.html#directShadowsEnabled)
     */
    directShadowsEnabled: boolean;
    /**
     * Indicates that the light source is virtual light.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-VirtualLighting.html#type)
     */
    readonly type: "virtual";

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-VirtualLighting.html#clone)
     */
    clone(): websceneVirtualLighting;
  }

  interface websceneVirtualLightingProperties {
    /**
     * Indicates whether to show shadows cast by the light source.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-webscene-VirtualLighting.html#directShadowsEnabled)
     */
    directShadowsEnabled?: boolean;
  }

  export interface WebSceneSaveAsOptions {
    /**
     * the folder in which to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#saveAs)
     */
    folder?: PortalFolder;
    /**
     * allow the scene to be saved even in the case it contains unsupported content (layers, renderers, symbols).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#saveAs)
     */
    ignoreUnsupported?: boolean;
  }

  export interface WebSceneSaveOptions {
    /**
     * When `true`, the scene will save even if it contains unsupported content (layers, renderers, symbols).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#save)
     */
    ignoreUnsupported?: boolean;
  }

  export interface WebSceneSourceVersion {
    /**
     * The major version of the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#sourceVersion)
     */
    major: number;
    /**
     * The minor version of the WebScene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#sourceVersion)
     */
    minor: number;
  }

  export interface WebSceneUpdateFromOptions {
    /**
     * Do not update the initial environment from the view, defaults to false.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    environmentExcluded?: boolean;
    /**
     * Do not update the initial viewpoint from the view, defaults to false.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    viewpointExcluded?: boolean;
    /**
     * Do not update the thumbnail from the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    thumbnailExcluded?: boolean;
    /**
     * The size of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    thumbnailSize?: WebSceneUpdateFromOptionsThumbnailSize;
    /**
     * When `true`, the webscene's [widgets](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#widgets) property will not be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    widgetsExcluded?: boolean;
  }

  export interface WebSceneUpdateFromOptionsThumbnailSize {
    /**
     * The width of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    width: number;
    /**
     * The height of the thumbnail.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#updateFrom)
     */
    height: number;
  }

  /**
   * The widgets object contains widgets that are exposed to the user.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#Widgets)
   */
  export interface WebSceneWidgets {
    /**
     * Time animation is controlled by a configurable [time slider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html#Widgets)
     */
    timeSlider: TimeSlider;
  }

  export class AreaMeasurement2D extends Widget {
    /**
     * The AreaMeasurement2D widget calculates and displays the area and perimeter of a polygon in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html)
     */

    constructor(properties?: AreaMeasurement2DProperties);

    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#unit)
     */
    unit: SystemOrAreaUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#unitOptions)
     */
    unitOptions: SystemOrAreaUnit[];
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#viewModel)
     */
    viewModel: AreaMeasurement2DViewModel;
  }

  interface AreaMeasurement2DProperties extends WidgetProperties {
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares"
    )[];
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D.html#viewModel)
     */
    viewModel?: AreaMeasurement2DViewModel;
  }

  export class AreaMeasurement2DViewModel {
    /**
     * Threshold (in meters) which determines the method for computing and displaying areas and perimeters in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold: number;
    /**
     * The area and perimeter of the measurement polygon in square meters and meters respectively.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurement)
     */
    readonly measurement: AreaMeasurement2DViewModelMeasurement;
    /**
     * This property returns the locale specific representation of the area and perimeter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurementLabel)
     */
    readonly measurementLabel: AreaMeasurement2DViewModelMeasurementLabel;
    /**
     * The ViewModel's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "measuring" | "measured";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#unit)
     */
    unit: SystemOrAreaUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#unitOptions)
     */
    unitOptions: SystemOrAreaUnit[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#view)
     */
    view: MapView;

    /**
     * Clears the current measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#clear)
     */
    clear(): void;
    /**
     * Starts a new measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#start)
     */
    start(): void;

    constructor(properties?: any);
  }

  export interface AreaMeasurement2DViewModelMeasurement {
    /**
     * The area (m²).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurement)
     */
    area: number;
    /**
     * The perimeter (m).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurement)
     */
    perimeter: number;
    /**
     * Measurement area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurement)
     */
    geometry: any;
  }

  export interface AreaMeasurement2DViewModelMeasurementLabel {
    /**
     * The area (m²).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurementLabel)
     */
    area: number;
    /**
     * The perimeter (m).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement2D-AreaMeasurement2DViewModel.html#measurementLabel)
     */
    perimeter: number;
  }

  export class AreaMeasurement3D extends Widget {
    /**
     * The AreaMeasurement3D widget calculates and displays the area and perimeter of a polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html)
     */

    constructor(properties?: AreaMeasurement3DProperties);

    /**
     * The area measurement analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#analysis)
     */
    readonly analysis: AreaMeasurementAnalysis;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#unit)
     */
    unit: SystemOrAreaUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#unitOptions)
     */
    unitOptions: SystemOrAreaUnit[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#viewModel)
     */
    viewModel: AreaMeasurement3DViewModel;
  }

  interface AreaMeasurement3DProperties extends WidgetProperties {
    /**
     * The area measurement analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#analysis)
     */
    analysis?: AreaMeasurementAnalysisProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares"
    )[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html#viewModel)
     */
    viewModel?: AreaMeasurement3DViewModelProperties;
  }

  export class AreaMeasurement3DViewModel extends Accessor {
    /**
     * Provides the logic for the [AreaMeasurement3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html)
     */

    constructor(properties?: AreaMeasurement3DViewModelProperties);

    /**
     * The area measurement analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#analysis)
     */
    readonly analysis: AreaMeasurementAnalysis;
    /**
     * The current measurement of the area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#measurement)
     */
    readonly measurement: AreaMeasurement3DViewModelMeasurement;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "measuring" | "measured";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#unit)
     */
    unit: SystemOrAreaUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#unitOptions)
     */
    unitOptions: SystemOrAreaUnit[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#view)
     */
    view: SceneView;

    /**
     * Clears the current measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#clear)
     */
    clear(): void;
    /**
     * Starts a new measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#start)
     */
    start(): void;
  }

  interface AreaMeasurement3DViewModelProperties {
    /**
     * The area measurement analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#analysis)
     */
    analysis?: AreaMeasurementAnalysisProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares"
    )[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export interface AreaMeasurement3DViewModelMeasurement {
    /**
     * Describes the mode in which the measurement is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#measurement)
     */
    mode: "euclidean" | "geodesic";
    /**
     * Area of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#measurement)
     */
    area: AreaMeasurement3DViewModelMeasurementValue;
    /**
     * Perimeter length of the polygon.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#measurement)
     */
    perimeterLength: AreaMeasurement3DViewModelMeasurementValue;
  }

  /**
   * Measurement value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#MeasurementValue)
   */
  export interface AreaMeasurement3DViewModelMeasurementValue {
    /**
     * Textual representation of the measured value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#MeasurementValue)
     */
    text: string;
    /**
     * State of the measured value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-AreaMeasurement3D-AreaMeasurement3DViewModel.html#MeasurementValue)
     */
    state: string;
  }

  export class Attachments extends Widget {
    /**
     * This widget allows users to view and edit attachments associated with a feature and is considered a standalone experience that can be utilized in widgets such [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) and [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html)
     */

    constructor(properties?: AttachmentsProperties);

    /**
     * Configures the attachment editing functionality that can be performed by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#capabilities)
     */
    capabilities: AttachmentsCapabilities;
    /**
     * A string value indicating how to display an attachment.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#displayType)
     */
    displayType: "auto" | "preview" | "list";
    /**
     * The graphic for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#graphic)
     */
    graphic: Graphic;
    /**
     * Indicates whether there is currently an attachment being added, updated or deleted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#submitting)
     */
    readonly submitting: boolean;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#viewModel)
     */
    viewModel: AttachmentsViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#visibleElements)
     */
    visibleElements: AttachmentsVisibleElements;
  }

  interface AttachmentsProperties extends WidgetProperties {
    /**
     * Configures the attachment editing functionality that can be performed by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#capabilities)
     */
    capabilities?: AttachmentsCapabilities;
    /**
     * A string value indicating how to display an attachment.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#displayType)
     */
    displayType?: "auto" | "preview" | "list";
    /**
     * The graphic for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#viewModel)
     */
    viewModel?: AttachmentsViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#visibleElements)
     */
    visibleElements?: AttachmentsVisibleElements;
  }

  export class AttachmentsViewModel extends Accessor {
    /**
     * Provides the logic for the [Attachments](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html)
     */

    constructor(properties?: AttachmentsViewModelProperties);

    /**
     * The current [AttachmentInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html) being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#activeAttachmentInfo)
     */
    activeAttachmentInfo: AttachmentInfo;
    /**
     * A collection of [AttachmentInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html) defined on a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#attachmentInfos)
     */
    readonly attachmentInfos: AttachmentInfo[];
    /**
     * Configures the attachment editing functionality that can be performed by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#capabilities)
     */
    capabilities: AttachmentsCapabilities;
    /**
     * The graphic for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#graphic)
     */
    graphic: Graphic;
    /**
     * The current mode performed by the user.
     *
     * @default "view"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#mode)
     */
    mode: "view" | "add" | "edit";
    /**
     * The current state of the widget.
     *
     * @default "ready"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#state)
     */
    readonly state: "ready" | "disabled" | "loading";
    /**
     * Defines whether or not the feature supports resizing attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#supportsResizeAttachments)
     */
    supportsResizeAttachments: boolean;

    /**
     * Queries for the attachments on a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#getAttachments)
     */
    getAttachments(): Promise<AttachmentInfo[]>;
  }

  interface AttachmentsViewModelProperties {
    /**
     * The current [AttachmentInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-query-support-AttachmentInfo.html) being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#activeAttachmentInfo)
     */
    activeAttachmentInfo?: AttachmentInfoProperties;
    /**
     * Configures the attachment editing functionality that can be performed by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#capabilities)
     */
    capabilities?: AttachmentsCapabilities;
    /**
     * The graphic for the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * The current mode performed by the user.
     *
     * @default "view"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#mode)
     */
    mode?: "view" | "add" | "edit";
    /**
     * Defines whether or not the feature supports resizing attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#supportsResizeAttachments)
     */
    supportsResizeAttachments?: boolean;
  }

  /**
   * An object that specifies the available attachments capabilities.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
   */
  export interface AttachmentsCapabilities {
    /**
     * Indicates whether to allow attachment editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
     */
    editing?: boolean;
    /**
     * Describes the configurable attachments operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
     */
    operations?: AttachmentsCapabilitiesOperations[];
  }

  export interface AttachmentsCapabilitiesOperations {
    /**
     * Indicates whether to allow adding attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
     */
    add?: boolean;
    /**
     * Indicates whether to allow updating an existing attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
     */
    update?: boolean;
    /**
     * Indicates whether to allow deleting an existing attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html#AttachmentsCapabilities)
     */
    delete?: boolean;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
   */
  export interface AttachmentsVisibleElements {
    /**
     * Indicates whether to display the `Add` button which prompts the dialog to add a new attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    addButton?: boolean;
    /**
     * Indicates whether to display the `add` button after selecting the attachment to add.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    addSubmitButton?: boolean;
    /**
     * Indicates whether to display the `cancel` button after selecting the attachment to add.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    cancelAddButton?: boolean;
    /**
     * Indicates whether to display the `cancel` button after selecting an attachment to update an existing attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    cancelUpdateButton?: boolean;
    /**
     * Indicates whether to display the `delete` button to delete an existing attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    deleteButton?: boolean;
    /**
     * Indicates whether to display an error message if adding or updating an attachment results in errors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    errorMessage?: boolean;
    /**
     * Indicates whether to display a progress bar when adding an attachment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    progressBar?: boolean;
    /**
     * Indicates whether to display an `update` button to allow updating on existing attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#VisibleElements)
     */
    updateButton?: boolean;
  }

  export class Attribution extends Widget {
    /**
     * The Attribution widget displays attribution text for the layers in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html)
     */

    constructor(properties?: AttributionProperties);

    /**
     * Full attribution text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#attributionText)
     */
    readonly attributionText: string;
    /**
     * Text used to split attribution by [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)
     *
     * @default |
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#itemDelimiter)
     */
    itemDelimiter: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#viewModel)
     */
    viewModel: AttributionViewModel;
  }

  interface AttributionProperties extends WidgetProperties {
    /**
     * Text used to split attribution by [layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)
     *
     * @default |
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#itemDelimiter)
     */
    itemDelimiter?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html#viewModel)
     */
    viewModel?: AttributionViewModelProperties;
  }

  export class AttributionViewModel extends Accessor {
    /**
     * Provides the logic for the [Attribution](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html)
     */

    constructor(properties?: AttributionViewModelProperties);

    /**
     * A collection of [AttributionItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#AttributionItem)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#items)
     */
    readonly items: Collection<AttributionItem>;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "loading";
    /**
     * The view from which the view model will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface AttributionViewModelProperties {
    /**
     * The view from which the view model will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The following properties define an attribution item that contains the attribution text for a layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#AttributionItem)
   */
  export interface AttributionItem {
    /**
     * The attribution text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#AttributionItem)
     */
    text: string;
    /**
     * The layer that has the attribution text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attribution-AttributionViewModel.html#AttributionItem)
     */
    layer: Layer;
  }

  export class BasemapGallery extends Widget {
    /**
     * The BasemapGallery widget displays a collection of images representing basemaps from ArcGIS.com or a user-defined set of map or image services.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html)
     */

    constructor(properties?: BasemapGalleryProperties);

    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#activeBasemap)
     */
    activeBasemap: Basemap;
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates the heading level to use for the message "No basemaps available" when no basemaps are available in the BasemapGallery.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The source for basemaps that the widget will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#source)
     */
    source: LocalBasemapsSource | PortalBasemapsSource;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#viewModel)
     */
    viewModel: BasemapGalleryViewModel;
  }

  interface BasemapGalleryProperties extends WidgetProperties {
    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#activeBasemap)
     */
    activeBasemap?: BasemapProperties;
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates the heading level to use for the message "No basemaps available" when no basemaps are available in the BasemapGallery.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The source for basemaps that the widget will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#source)
     */
    source?:
      | LocalBasemapsSourceProperties
      | PortalBasemapsSourceProperties
      | CollectionProperties<BasemapProperties>
      | BasemapProperties[]
      | PortalProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html#viewModel)
     */
    viewModel?: BasemapGalleryViewModelProperties;
  }

  export class BasemapGalleryViewModel extends Accessor {
    /**
     * Provides the logic for the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html)
     */

    constructor(properties?: BasemapGalleryViewModelProperties);

    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#activeBasemap)
     */
    activeBasemap: Basemap;
    /**
     * A collection of [BasemapGalleryItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html)s representing basemaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#items)
     */
    items: Collection<BasemapGalleryItem>;
    /**
     * The source for basemaps that the widget will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#source)
     */
    source: LocalBasemapsSource | PortalBasemapsSource;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#state)
     */
    readonly state: "ready" | "disabled" | "loading" | "unsupported";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * A convenience function to check basemap equality.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#basemapEquals)
     */
    basemapEquals(basemap1: Basemap, basemap2: Basemap): boolean;
  }

  interface BasemapGalleryViewModelProperties {
    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#activeBasemap)
     */
    activeBasemap?: BasemapProperties;
    /**
     * A collection of [BasemapGalleryItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html)s representing basemaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#items)
     */
    items?: CollectionProperties<BasemapGalleryItemProperties>;
    /**
     * The source for basemaps that the widget will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#source)
     */
    source?:
      | LocalBasemapsSourceProperties
      | PortalBasemapsSourceProperties
      | CollectionProperties<BasemapProperties>
      | BasemapProperties[]
      | PortalProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class BasemapGalleryItem extends Accessor {
    /**
     * The BasemapGalleryItem class represents one of the [items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#items) in the [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html)
     */

    constructor(properties?: BasemapGalleryItemProperties);

    /**
     * The item's associated basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#basemap)
     */
    basemap: Basemap;
    /**
     * The Error object returned if an error occurred.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#error)
     */
    readonly error: Error;
    /**
     * The item's state.
     *
     * @default "loading"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#state)
     */
    readonly state: "loading" | "ready" | "error";
    /**
     * The view associated with this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#view)
     */
    view: MapView | SceneView;
  }

  interface BasemapGalleryItemProperties {
    /**
     * The item's associated basemap.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#basemap)
     */
    basemap?: BasemapProperties;
    /**
     * The view associated with this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-BasemapGalleryItem.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class LocalBasemapsSource extends Accessor {
    /**
     * The LocalBasemapsSource class is a Collection-driven [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#source) in the [BasemapGalleryViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html) or [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-LocalBasemapsSource.html)
     */

    constructor(properties?: LocalBasemapsSourceProperties);

    /**
     * A collection of [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-LocalBasemapsSource.html#basemaps)
     */
    basemaps: Collection<Basemap>;
    /**
     * The source's state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-LocalBasemapsSource.html#state)
     */
    readonly state: string;
  }

  interface LocalBasemapsSourceProperties {
    /**
     * A collection of [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-LocalBasemapsSource.html#basemaps)
     */
    basemaps?: CollectionProperties<BasemapProperties>;
  }

  export class PortalBasemapsSource extends LocalBasemapsSource {
    /**
     * The PortalBasemapsSource class is a Portal-driven [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html#source) in the [BasemapGalleryViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-BasemapGalleryViewModel.html) or [BasemapGallery](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html)
     */

    constructor(properties?: PortalBasemapsSourceProperties);

    /**
     * Function used to filter basemaps after being fetched from the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#filterFunction)
     */
    filterFunction: BasemapFilter;
    /**
     * The Portal from which to fetch basemaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#portal)
     */
    portal: Portal;
    /**
     * An object with key-value pairs used to create a custom basemap gallery group query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#query)
     */
    query: any | string;
    /**
     * The source's state.
     *
     * @default "not-loaded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#state)
     */
    readonly state: "not-loaded" | "loading" | "ready";
    /**
     * Callback for updating basemaps after being fetched and filtered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#updateBasemapsCallback)
     */
    updateBasemapsCallback: UpdateBasemapsCallback;

    /**
     * Refreshes basemaps by fetching them from the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#refresh)
     */
    refresh(): Promise<any>;
  }

  interface PortalBasemapsSourceProperties extends LocalBasemapsSourceProperties {
    /**
     * Function used to filter basemaps after being fetched from the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#filterFunction)
     */
    filterFunction?: BasemapFilter;
    /**
     * The Portal from which to fetch basemaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#portal)
     */
    portal?: PortalProperties;
    /**
     * An object with key-value pairs used to create a custom basemap gallery group query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#query)
     */
    query?: any | string;
    /**
     * Callback for updating basemaps after being fetched and filtered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapGallery-support-PortalBasemapsSource.html#updateBasemapsCallback)
     */
    updateBasemapsCallback?: UpdateBasemapsCallback;
  }

  export type BasemapFilter = (item: Basemap, index: number, array: Basemap[]) => boolean | Promise<boolean>;

  export type UpdateBasemapsCallback = (items: Basemap[]) => Basemap[] | Promise<Basemap[]>;

  export class BasemapLayerList extends Widget {
    /**
     * The BasemapLayerList widget provides a way to display a list of [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) layers and switch on/off their visibility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html)
     */

    constructor(properties?: BasemapLayerListProperties);

    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#baseFilterText)
     */
    baseFilterText: string;
    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing the [baseLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#baseItems)
     */
    readonly baseItems: Collection<ListItem>;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing a base layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#baseListItemCreatedFunction)
     */
    baseListItemCreatedFunction: ListItemCreatedHandler;
    /**
     * The current basemap's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#basemapTitle)
     */
    basemapTitle: string;
    /**
     * The [CatalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html) widget instance that displays a catalog layer's [dynamic group layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogLayerList)
     */
    readonly catalogLayerList: CatalogLayerList;
    /**
     * [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    catalogOptions: BasemapLayerListCatalogOptions;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#collapsed)
     */
    collapsed: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#dragEnabled)
     */
    dragEnabled: boolean;
    /**
     * Indicates whether the basemap’s title, layer order and layer grouping can be edited by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/BasemapLayerList#selectionMode selectionMode}, {@link module:esri/widgets/BasemapLayerList#VisibleElements visibleElements.editTitleButton}, and {@link module:esri/widgets/BasemapLayerList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * Indicates whether the form to edit the basemap's title is currently visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#editingTitle)
     */
    editingTitle: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#filterPlaceholder)
     */
    filterPlaceholder: string;
    /**
     * Indicates the heading level to use for the widget title (i.e.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#minFilterItems)
     */
    minFilterItems: number;
    /**
     * Indicates whether more than one list item may be selected by the user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/BasemapLayerList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled: boolean;
    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#referenceFilterText)
     */
    referenceFilterText: string;
    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing the [referenceLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#referenceLayers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#referenceItems)
     */
    readonly referenceItems: Collection<ListItem>;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing a reference layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#referenceListItemCreatedFunction)
     */
    referenceListItemCreatedFunction: ListItemCreatedHandler;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing basemap layers selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#selectedItems)
     */
    selectedItems: Collection<ListItem>;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#selectionMode)
     */
    selectionMode: "multiple" | "none" | "single" | "single-persist";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#viewModel)
     */
    viewModel: BasemapLayerListViewModel;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibilityAppearance)
     */
    visibilityAppearance: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements)
     */
    visibleElements: BasemapLayerListVisibleElements;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: ListItem): void;

    on(name: "trigger-action", eventHandler: BasemapLayerListTriggerActionEventHandler): IHandle;
  }

  interface BasemapLayerListProperties extends WidgetProperties {
    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#baseFilterText)
     */
    baseFilterText?: string;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing a base layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#baseListItemCreatedFunction)
     */
    baseListItemCreatedFunction?: ListItemCreatedHandler;
    /**
     * The current basemap's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#basemapTitle)
     */
    basemapTitle?: string;
    /**
     * [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    catalogOptions?: BasemapLayerListCatalogOptions;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#collapsed)
     */
    collapsed?: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#dragEnabled)
     */
    dragEnabled?: boolean;
    /**
     * Indicates whether the basemap’s title, layer order and layer grouping can be edited by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/BasemapLayerList#selectionMode selectionMode}, {@link module:esri/widgets/BasemapLayerList#VisibleElements visibleElements.editTitleButton}, and {@link module:esri/widgets/BasemapLayerList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * Indicates whether the form to edit the basemap's title is currently visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#editingTitle)
     */
    editingTitle?: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#filterPlaceholder)
     */
    filterPlaceholder?: string;
    /**
     * Indicates the heading level to use for the widget title (i.e.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#minFilterItems)
     */
    minFilterItems?: number;
    /**
     * Indicates whether more than one list item may be selected by the user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/BasemapLayerList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#referenceFilterText)
     */
    referenceFilterText?: string;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing a reference layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#referenceListItemCreatedFunction)
     */
    referenceListItemCreatedFunction?: ListItemCreatedHandler;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing basemap layers selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#selectedItems)
     */
    selectedItems?: CollectionProperties<ListItemProperties>;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#selectionMode)
     */
    selectionMode?: "multiple" | "none" | "single" | "single-persist";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#viewModel)
     */
    viewModel?: BasemapLayerListViewModelProperties;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibilityAppearance)
     */
    visibilityAppearance?: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#visibleElements)
     */
    visibleElements?: BasemapLayerListVisibleElements;
  }

  export class BasemapLayerListViewModel extends Accessor {
    /**
     * Provides logic for the [BasemapLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html)
     */

    constructor(properties?: BasemapLayerListViewModelProperties);

    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing the [baseLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#baseLayers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#baseItems)
     */
    readonly baseItems: Collection<ListItem>;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#baseListItemCreatedFunction)
     */
    baseListItemCreatedFunction: ListItemCreatedHandler;
    /**
     * The current basemap's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#basemapTitle)
     */
    basemapTitle: string;
    /**
     * Whether to provide an indication if a layer is being published in the [BasemapLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled: boolean;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all layers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#listModeDisabled)
     */
    listModeDisabled: boolean;
    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing the [referenceLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html#referenceLayers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#referenceItems)
     */
    readonly referenceItems: Collection<ListItem>;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing reference layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#referenceListItemCreatedFunction)
     */
    referenceListItemCreatedFunction: ListItemCreatedHandler;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#state)
     */
    readonly state: "ready" | "loading" | "disabled";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: ListItem): void;
  }

  interface BasemapLayerListViewModelProperties {
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#baseListItemCreatedFunction)
     */
    baseListItemCreatedFunction?: ListItemCreatedHandler;
    /**
     * The current basemap's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#basemapTitle)
     */
    basemapTitle?: string;
    /**
     * Whether to provide an indication if a layer is being published in the [BasemapLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled?: boolean;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all layers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#listModeDisabled)
     */
    listModeDisabled?: boolean;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing reference layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#referenceListItemCreatedFunction)
     */
    referenceListItemCreatedFunction?: ListItemCreatedHandler;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList-BasemapLayerListViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface BasemapLayerListCatalogOptions {
    /**
     * Placeholder text used in the filter input box.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    filterPlaceholder?: string;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    listItemCreatedFunction?: CatalogLayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to display the filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    minFilterItems?: number;
    /**
     * The selection mode for the catalog layer list.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    selectionMode?: string;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    visibilityAppearance?: string;
    /**
     * The visible elements that are displayed within the catalog layer list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    visibleElements?: BasemapLayerListCatalogOptionsVisibleElements;
  }

  export interface BasemapLayerListCatalogOptionsVisibleElements {
    /**
     * Indicates whether to display errors in the catalog layer list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    errors?: boolean;
    /**
     * Indicates whether to display the filter input box in the catalog layer list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    filter?: boolean;
    /**
     * Indicates whether to display status indicators in the catalog layer list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    statusIndicators?: boolean;
    /**
     * Indicates whether temporary layer indicators will be displayed for layers with [layer.persistenceEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled) set to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#catalogOptions)
     */
    temporaryLayerIndicators?: boolean;
  }

  export type ListItemCreatedHandler = (event: any) => void;

  export interface BasemapLayerListTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: ListItem;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
   */
  export interface BasemapLayerListVisibleElements {
    /**
     * Indicates whether to the base layers will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    baseLayers?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a close button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a collapse button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a button in the header to edit the basemap title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    editTitleButton?: boolean;
    /**
     * **Since 4.25** Indicates whether layer load errors will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    errors?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a filter input box when then number of list items is equal to or greater than the value set in [minFilterItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#minFilterItems), allowing users to filter layers by their title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    filter?: boolean;
    /**
     * **Since 4.29** Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * **Since 4.29** Indicates whether the widget displays a heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * Indicates whether to the reference layers will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    referenceLayers?: boolean;
    /**
     * Indicates whether the status indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    statusIndicators?: boolean;
    /**
     * **Since 4.29** Indicates whether temporary layer indicators will be displayed for layers with [layer.persistenceEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled) set to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapLayerList.html#VisibleElements)
     */
    temporaryLayerIndicators?: boolean;
  }

  export class BasemapToggle extends Widget {
    /**
     * The BasemapToggle provides a widget which allows an end-user to switch between two basemaps.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html)
     */

    constructor(properties?: BasemapToggleProperties);

    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#activeBasemap)
     */
    readonly activeBasemap: Basemap;
    /**
     * The next basemap for toggling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#nextBasemap)
     */
    nextBasemap: Basemap;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#viewModel)
     */
    viewModel: BasemapToggleViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#visibleElements)
     */
    visibleElements: BasemapToggleVisibleElements;

    /**
     * Toggles to the [next basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#nextBasemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#toggle)
     */
    toggle(): Promise<any>;
  }

  interface BasemapToggleProperties extends WidgetProperties {
    /**
     * The next basemap for toggling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#nextBasemap)
     */
    nextBasemap?: BasemapProperties | string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#viewModel)
     */
    viewModel?: BasemapToggleViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#visibleElements)
     */
    visibleElements?: BasemapToggleVisibleElements;
  }

  export class BasemapToggleViewModel extends Accessor {
    /**
     * Provides the logic for the [BasemapToggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html)
     */

    constructor(properties?: BasemapToggleViewModelProperties);

    /**
     * The map's [basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html#basemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#activeBasemap)
     */
    readonly activeBasemap: Basemap;
    /**
     * The next basemap for toggling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#nextBasemap)
     */
    nextBasemap: Basemap;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#state)
     */
    readonly state: "ready" | "incompatible-next-basemap" | "loading" | "disabled";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Helper method to find a basemap's thumbnail URL.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#getThumbnailUrl)
     */
    getThumbnailUrl(basemap: Basemap): string;
    /**
     * Toggles to the [next basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#nextBasemap).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#toggle)
     */
    toggle(): Promise<any>;
  }

  interface BasemapToggleViewModelProperties {
    /**
     * The next basemap for toggling.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#nextBasemap)
     */
    nextBasemap?: BasemapProperties | string;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle-BasemapToggleViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#VisibleElements)
   */
  export interface BasemapToggleVisibleElements {
    /**
     * Indicates whether to the title will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BasemapToggle.html#VisibleElements)
     */
    title?: boolean;
  }

  export class BatchAttributeForm extends Widget {
    /**
     * BatchAttributeForm allows editing the attributes of multiple features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html)
     */

    constructor(properties?: BatchAttributeFormProperties);

    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#disabled)
     */
    disabled: boolean;
    /**
     * The associated features containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#features)
     */
    features: Collection<Graphic>;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#title) of the form.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#headingLevel)
     */
    headingLevel: number;
    /**
     * When `true`, sets the widget to a readOnly state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#readOnly)
     */
    readOnly: boolean;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#viewModel)
     */
    viewModel: BatchAttributeFormViewModel;
  }

  interface BatchAttributeFormProperties extends WidgetProperties {
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#disabled)
     */
    disabled?: boolean;
    /**
     * The associated features containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#features)
     */
    features?: CollectionProperties<GraphicProperties>;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#title) of the form.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * When `true`, sets the widget to a readOnly state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#readOnly)
     */
    readOnly?: boolean;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html#viewModel)
     */
    viewModel?: BatchAttributeFormViewModel;
  }

  /**
   * Provides the logic for the [BatchAttributeForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm.html) widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BatchAttributeForm-BatchAttributeFormViewModel.html)
   */
  interface BatchAttributeFormViewModel {}

  export const BatchAttributeFormViewModel: BatchAttributeFormViewModel;

  export interface Bookmarks extends Widget, GoTo {}

  export class Bookmarks {
    /**
     * The Bookmarks widget allows end users to quickly navigate to a particular area of interest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html)
     */

    constructor(properties?: BookmarksProperties);

    /**
     * A collection of [Bookmark](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#bookmarks)
     */
    bookmarks: Collection<Bookmark>;
    /**
     * Specifies how new bookmarks will be created if [visibleElements.addBookmarkButton](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#defaultCreateOptions)
     */
    defaultCreateOptions: BookmarkOptions;
    /**
     * Specifies how bookmarks will be edited, if [visibleElements.editBookmarkButton](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#defaultEditOptions)
     */
    defaultEditOptions: BookmarkOptions;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates if a Bookmark is able to be dragged in order to update its position in the list.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#dragEnabled)
     */
    dragEnabled: boolean;
    /**
     * Indicates whether the bookmarks are able to be edited.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/Bookmarks#VisibleElements visibleElements.editBookmarkButton}, {@link module:esri/widgets/Bookmarks#VisibleElements visibleElements.addBookmarkButton}, and {@link module:esri/widgets/Bookmarks#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * Defines the text used as a placeholder when visibleElements.filter is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#filterPlaceholder)
     */
    filterPlaceholder: string;
    /**
     * Defines the text used to filter the bookmarks when visibleElements.filter is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#filterText)
     */
    filterText: string;
    /**
     * Indicates the heading level to use for the message "No bookmarks" when no bookmarks are available in this widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#viewModel)
     */
    viewModel: BookmarksViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#visibleElements)
     */
    visibleElements: BookmarksVisibleElements;

    /**
     * Zoom to a specific bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#goTo)
     */
    goTo(Bookmark: Bookmark): Promise<any>;

    on(name: "bookmark-select", eventHandler: BookmarksBookmarkSelectEventHandler): IHandle;

    on(name: "bookmark-edit", eventHandler: BookmarksBookmarkEditEventHandler): IHandle;
  }

  interface BookmarksProperties extends WidgetProperties, GoToProperties {
    /**
     * A collection of [Bookmark](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#bookmarks)
     */
    bookmarks?: CollectionProperties<BookmarkProperties>;
    /**
     * Specifies how new bookmarks will be created if [visibleElements.addBookmarkButton](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#defaultCreateOptions)
     */
    defaultCreateOptions?: BookmarkOptions;
    /**
     * Specifies how bookmarks will be edited, if [visibleElements.editBookmarkButton](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements) is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#defaultEditOptions)
     */
    defaultEditOptions?: BookmarkOptions;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates if a Bookmark is able to be dragged in order to update its position in the list.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#dragEnabled)
     */
    dragEnabled?: boolean;
    /**
     * Indicates whether the bookmarks are able to be edited.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/Bookmarks#VisibleElements visibleElements.editBookmarkButton}, {@link module:esri/widgets/Bookmarks#VisibleElements visibleElements.addBookmarkButton}, and {@link module:esri/widgets/Bookmarks#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * Defines the text used as a placeholder when visibleElements.filter is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#filterPlaceholder)
     */
    filterPlaceholder?: string;
    /**
     * Defines the text used to filter the bookmarks when visibleElements.filter is set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#filterText)
     */
    filterText?: string;
    /**
     * Indicates the heading level to use for the message "No bookmarks" when no bookmarks are available in this widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#viewModel)
     */
    viewModel?: BookmarksViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#visibleElements)
     */
    visibleElements?: BookmarksVisibleElements;
  }

  export interface BookmarksViewModel extends Accessor, GoTo {}

  export class BookmarksViewModel {
    /**
     * Provides the logic for the [Bookmarks](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html)
     */

    constructor(properties?: BookmarksViewModelProperties);

    /**
     * The [Bookmark](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html) that is being navigated to.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#activeBookmark)
     */
    readonly activeBookmark: Bookmark;
    /**
     * A collection of [Bookmark](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#bookmarks)
     */
    bookmarks: Collection<Bookmark>;
    /**
     * Defines the capabilities of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#capabilities)
     */
    capabilities: BookmarksCapabilities;
    /**
     * Specifies how new bookmarks will be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#defaultCreateOptions)
     */
    defaultCreateOptions: BookmarkOptions;
    /**
     * Specifies how bookmarks will be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#defaultEditOptions)
     */
    defaultEditOptions: BookmarkOptions;
    /**
     * The view model's state.
     *
     * @default "ready"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#state)
     */
    readonly state: "loading" | "ready";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Creates a new bookmark from the [defaultCreateOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#defaultCreateOptions), unless otherwise specified.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#createBookmark)
     */
    createBookmark(options?: BookmarkOptions): Promise<Bookmark>;
    /**
     * Edits the given bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#editBookmark)
     */
    editBookmark(bookmark: Bookmark, options?: BookmarkOptions): Promise<Bookmark>;
    /**
     * Zoom to a specific bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#goTo)
     */
    goTo(bookmark: Bookmark): Promise<any>;
  }

  interface BookmarksViewModelProperties extends GoToProperties {
    /**
     * A collection of [Bookmark](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html)s.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#bookmarks)
     */
    bookmarks?: CollectionProperties<BookmarkProperties>;
    /**
     * Defines the capabilities of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#capabilities)
     */
    capabilities?: BookmarksCapabilities;
    /**
     * Specifies how new bookmarks will be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#defaultCreateOptions)
     */
    defaultCreateOptions?: BookmarkOptions;
    /**
     * Specifies how bookmarks will be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#defaultEditOptions)
     */
    defaultEditOptions?: BookmarkOptions;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * Specifies how bookmarks will be created or modified.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
   */
  export interface BookmarkOptions {
    /**
     * Indicates whether a screenshot is taken when a new bookmark is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    takeScreenshot?: boolean;
    /**
     * _Since 4.17_ Indicates whether the viewpoint of the current view will become the [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#viewpoint) of a newly created or modified bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    captureViewpoint?: boolean;
    /**
     * _Since 4.17_ Indicates whether the rotation of the current view will be saved in the [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#viewpoint) of a newly created or modified bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    captureRotation?: boolean;
    /**
     * _Since 4.17_ Indicates whether the scale of the current view will be saved in the [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#viewpoint) of a newly created or modified bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    captureScale?: boolean;
    /**
     * _Since 4.22_ Indicates whether the time extent of the current view will be saved in the [timeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#timeExtent) of a newly created or modified bookmark.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    captureTimeExtent?: boolean;
    /**
     * An object that specifies the settings of the screenshot that will be used to create the bookmark's [thumbnail](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#thumbnail).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    screenshotSettings?: BookmarkOptionsScreenshotSettings;
  }

  /**
   * Specifies the abilities for the Bookmarks widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarksCapabilities)
   */
  export interface BookmarksCapabilities {
    /**
     * Indicates whether the time capability is enabled in the Bookmarks widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarksCapabilities)
     */
    time?: boolean;
  }

  export interface BookmarkOptionsScreenshotSettings {
    /**
     * The width (in pixels) of the screenshot.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    width?: number;
    /**
     * The height (in pixels) of the screenshot.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    height?: number;
    /**
     * Used to take a screenshot of a subregion of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    area?: BookmarkOptionsScreenshotSettingsArea;
    /**
     * An optional list of layers to be included in the screenshot.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    layers?: Layer[];
  }

  export interface BookmarkOptionsScreenshotSettingsArea {
    /**
     * The x value of the screenshot area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    x?: number;
    /**
     * The y value of the screenshot area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    y?: number;
    /**
     * The width of the screenshot area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    width?: number;
    /**
     * The height of the screenshot area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks-BookmarksViewModel.html#BookmarkOptions)
     */
    height?: number;
  }

  export interface BookmarksBookmarkEditEvent {
    bookmark: Bookmark;
  }

  export interface BookmarksBookmarkSelectEvent {
    bookmark: Bookmark;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
   */
  export interface BookmarksVisibleElements {
    /**
     * _Deprecated since 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    addBookmark?: boolean;
    /**
     * _Since 4.29._ Indicates whether the button to add a new bookmark displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    addBookmarkButton?: boolean;
    /**
     * _Since 4.29._ Indicates whether to display a close button at the top of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * _Since 4.29._ Indicates whether to display a collapse button at the top of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * _Since 4.29._ Indicates whether the button to edit a bookmark displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    editBookmarkButton?: boolean;
    /**
     * _Since 4.29._ Indicates whether the bookmarks filter displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    filter?: boolean;
    /**
     * _Since 4.29._ Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * _Since 4.29._ Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * _Since 4.13._ Indicates whether the thumbnail associated with the bookmark displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    thumbnail?: boolean;
    /**
     * _Since 4.22._ Indicates whether the time (`h:m:s`) is displayed alongside the date if the bookmark has a [time extent](https://developers.arcgis.com/javascript/latest/api-reference/esri-webmap-Bookmark.html#timeExtent) defined.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Bookmarks.html#VisibleElements)
     */
    time?: boolean;
  }

  export class BuildingExplorer extends Widget {
    /**
     * The BuildingExplorer widget is used to filter and explore the various components of [BuildingSceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html)
     */

    constructor(properties?: BuildingExplorerProperties);

    /**
     * Indicates the heading level to use for the headings in the widget.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of layers of type [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) that are added to the widget for exploration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#layers)
     */
    layers: Collection<BuildingSceneLayer>;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#viewModel)
     */
    viewModel: BuildingExplorerViewModel;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#visibleElements)
     */
    visibleElements: BuildingExplorerVisibleElements;
  }

  interface BuildingExplorerProperties extends WidgetProperties {
    /**
     * Indicates the heading level to use for the headings in the widget.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of layers of type [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) that are added to the widget for exploration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#layers)
     */
    layers?: CollectionProperties<BuildingSceneLayerProperties>;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#viewModel)
     */
    viewModel?: BuildingExplorerViewModelProperties;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#visibleElements)
     */
    visibleElements?: BuildingExplorerVisibleElements;
  }

  export class BuildingExplorerViewModel extends Accessor {
    /**
     * Provides the logic for the [BuildingExplorer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html)
     */

    constructor(properties?: BuildingExplorerViewModelProperties);

    /**
     * A [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of layers of type [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) that are added to the widget for exploration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#layers)
     */
    layers: Collection<BuildingSceneLayer>;
    /**
     * Contains information about the level filter, such as the value selected by the user in the Level element or the minimum and maximum allowed values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#level)
     */
    readonly level: BuildingLevel;
    /**
     * Contains information about the construction phase filter, such as the value selected by the user in the Construction phases element and the minimum and maximum allowed values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#phase)
     */
    readonly phase: BuildingPhase;
    /**
     * The current state of the view model that can be used for rendering the UI of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#state)
     */
    readonly state: "disabled" | "loading" | "ready" | "failed";
    /**
     * The view in which the BuildingExplorer is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#view)
     */
    view: SceneView;
  }

  interface BuildingExplorerViewModelProperties {
    /**
     * A [collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of layers of type [BuildingSceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-BuildingSceneLayer.html) that are added to the widget for exploration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#layers)
     */
    layers?: CollectionProperties<BuildingSceneLayerProperties>;
    /**
     * The view in which the BuildingExplorer is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingExplorerViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export class BuildingLevel extends Accessor {
    /**
     * Provides information for the building level filter, such as the value selected by the user or the minimum and maximum allowed values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html)
     */

    constructor(properties?: BuildingLevelProperties);

    /**
     * List of all the values which are allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#allowedValues)
     */
    allowedValues: number[];
    /**
     * Whether the filter is enabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#enabled)
     */
    readonly enabled: boolean;
    /**
     * Whether the next value can be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#hasNext)
     */
    readonly hasNext: boolean;
    /**
     * Whether the previous value can be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#hasPrevious)
     */
    readonly hasPrevious: boolean;
    /**
     * The maximum value allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#max)
     */
    readonly max: number;
    /**
     * The minimum value allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#min)
     */
    readonly min: number;
    /**
     * The value which is currently set on the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#value)
     */
    readonly value: number;

    /**
     * Removes the filter by setting [enabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#enabled) to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#clear)
     */
    clear(): void;
    /**
     * If the field that is used for filtering has a coded value domain, the label for the value can be used to be displayed in a tooltip or in the widget UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#getValueLabel)
     */
    getValueLabel(value: number): string;
    /**
     * Selects the next value, if available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#next)
     */
    next(): void;
    /**
     * Selects the previous value, if available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#previous)
     */
    previous(): void;
    /**
     * Selects the specified value for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#select)
     */
    select(value: number): void;
  }

  interface BuildingLevelProperties {
    /**
     * List of all the values which are allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingLevel.html#allowedValues)
     */
    allowedValues?: number[];
  }

  export class BuildingPhase extends Accessor {
    /**
     * BuildingPhase provides information for the construction phase filter, such as the value selected by the user or the minimum and maximum allowed values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html)
     */

    constructor(properties?: BuildingPhaseProperties);

    /**
     * List of all the values which are allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#allowedValues)
     */
    allowedValues: number[];
    /**
     * Whether the filter is enabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#enabled)
     */
    readonly enabled: boolean;
    /**
     * Whether the next value can be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#hasNext)
     */
    readonly hasNext: boolean;
    /**
     * Whether the previous value can be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#hasPrevious)
     */
    readonly hasPrevious: boolean;
    /**
     * The maximum value allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#max)
     */
    readonly max: number;
    /**
     * The minimum value allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#min)
     */
    readonly min: number;
    /**
     * The value which is currently set on the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#value)
     */
    readonly value: number;

    /**
     * Removes the filter by setting [enabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#enabled) to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#clear)
     */
    clear(): void;
    /**
     * If the field that is used for filtering has a coded value domain, the label for the value can be used to be displayed in a tooltip or in the widget UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#getValueLabel)
     */
    getValueLabel(value: number): string;
    /**
     * Selects the next value, if available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#next)
     */
    next(): void;
    /**
     * Selects the previous value, if available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#previous)
     */
    previous(): void;
    /**
     * Selects the specified value for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#select)
     */
    select(value: number): void;
  }

  interface BuildingPhaseProperties {
    /**
     * List of all the values which are allowed for the filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer-BuildingPhase.html#allowedValues)
     */
    allowedValues?: number[];
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#VisibleElements)
   */
  export interface BuildingExplorerVisibleElements {
    /**
     * When set to `false`, the building levels filter is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#VisibleElements)
     */
    levels?: boolean;
    /**
     * When set to `false`, the construction phases filter is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#VisibleElements)
     */
    phases?: boolean;
    /**
     * When set to `false`, the disciplines and categories sublayer list is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-BuildingExplorer.html#VisibleElements)
     */
    disciplines?: boolean;
  }

  export class CatalogLayerList extends Widget {
    /**
     * The CatalogLayerList widget provides a way to display and interact with [CatalogLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html)
     */

    constructor(properties?: CatalogLayerListProperties);

    /**
     * The collection of [ListItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing the [catalogLayer's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogLayer) [dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogItems)
     */
    readonly catalogItems: Collection<ListItem>;
    /**
     * The [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) to display in the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogLayer)
     */
    catalogLayer: CatalogLayer;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#collapsed)
     */
    collapsed: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#filterPlaceholder)
     */
    filterPlaceholder: string;
    /**
     * The value of the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#filterText)
     */
    filterText: string;
    /**
     * Indicates the heading level to use for the heading of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: CatalogLayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#minFilterItems)
     */
    minFilterItems: number;
    /**
     * A collection of selected [ListItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing [catalogItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogItems) selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#selectedItems)
     */
    selectedItems: Collection<ListItem>;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#selectionMode)
     */
    selectionMode: "multiple" | "none" | "single" | "single-persist";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#viewModel)
     */
    viewModel: CatalogLayerListViewModel;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibilityAppearance)
     */
    visibilityAppearance: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements)
     */
    visibleElements: CatalogLayerListVisibleElements;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#triggerAction)
     */
    triggerAction(__0: ActionButton | ActionToggle, __1: ListItem): void;

    on(name: "trigger-action", eventHandler: CatalogLayerListTriggerActionEventHandler): IHandle;
  }

  interface CatalogLayerListProperties extends WidgetProperties {
    /**
     * The [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) to display in the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogLayer)
     */
    catalogLayer?: CatalogLayerProperties;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#collapsed)
     */
    collapsed?: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#filterPlaceholder)
     */
    filterPlaceholder?: string;
    /**
     * The value of the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#filterText)
     */
    filterText?: string;
    /**
     * Indicates the heading level to use for the heading of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: CatalogLayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#minFilterItems)
     */
    minFilterItems?: number;
    /**
     * A collection of selected [ListItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing [catalogItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#catalogItems) selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#selectedItems)
     */
    selectedItems?: CollectionProperties<ListItemProperties>;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#selectionMode)
     */
    selectionMode?: "multiple" | "none" | "single" | "single-persist";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#viewModel)
     */
    viewModel?: CatalogLayerListViewModelProperties;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibilityAppearance)
     */
    visibilityAppearance?: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#visibleElements)
     */
    visibleElements?: CatalogLayerListVisibleElements;
  }

  export class CatalogLayerListViewModel extends Accessor {
    /**
     * Provides the logic for the [CatalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html)
     */

    constructor(properties?: CatalogLayerListViewModelProperties);

    /**
     * The collection of [ListItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) representing the [catalogLayer's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#catalogLayer) [dynamicGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#catalogItems)
     */
    readonly catalogItems: Collection<ListItem>;
    /**
     * The [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) to display in the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#catalogLayer)
     */
    catalogLayer: CatalogLayer;
    /**
     * Whether to provide an indication if a layer is being published in the [CatalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled: boolean;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: CatalogLayerListViewModelListItemCreatedHandler;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#state)
     */
    readonly state: "loading" | "ready" | "disabled";
    /**
     * The view from which the widget will operate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#triggerAction)
     */
    triggerAction(__0: ActionButton | ActionToggle, __1: ListItem): void;

    on(name: "trigger-action", eventHandler: CatalogLayerListViewModelTriggerActionEventHandler): IHandle;
  }

  interface CatalogLayerListViewModelProperties {
    /**
     * The [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) to display in the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#catalogLayer)
     */
    catalogLayer?: CatalogLayerProperties;
    /**
     * Whether to provide an indication if a layer is being published in the [CatalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled?: boolean;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: CatalogLayerListViewModelListItemCreatedHandler;
    /**
     * The view from which the widget will operate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList-CatalogLayerListViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export type CatalogLayerListViewModelListItemCreatedHandler = (event: any) => void;

  export interface CatalogLayerListViewModelTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: ListItem;
  }

  export type CatalogLayerListListItemCreatedHandler = (event: any) => void;

  export interface CatalogLayerListTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: ListItem;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
   */
  export interface CatalogLayerListVisibleElements {
    /**
     * Indicates whether to display a close button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * Indicates whether to display a collapse button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * Indicates whether layer load errors will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    errors?: boolean;
    /**
     * Indicates whether to display a filter input box when then number of list items is equal to or greater than the value set in [minFilterItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#minFilterItems), allowing users to filter layers by their title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    filter?: boolean;
    /**
     * Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * Indicates whether the status indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    statusIndicators?: boolean;
    /**
     * Indicates whether temporary layer indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html#VisibleElements)
     */
    temporaryLayerIndicators?: boolean;
  }

  export interface Compass extends Widget, GoTo {}

  export class Compass {
    /**
     * The Compass widget indicates where north is in relation to the current view [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) or [camera heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html)
     */

    constructor(properties?: CompassProperties);

    /**
     * The view in which the Compass obtains and indicates camera [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading), using a (SceneView) or [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) (MapView).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html#viewModel)
     */
    viewModel: CompassViewModel;

    /**
     * If working in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html), sets the view's [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) to `0`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html#reset)
     */
    reset(): void;
  }

  interface CompassProperties extends WidgetProperties, GoToProperties {
    /**
     * The view in which the Compass obtains and indicates camera [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading), using a (SceneView) or [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) (MapView).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html#viewModel)
     */
    viewModel?: CompassViewModelProperties;
  }

  export interface CompassViewModel extends Accessor, GoTo {}

  export class CompassViewModel {
    /**
     * Provides the logic for the [Compass](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html)
     */

    constructor(properties?: CompassViewModelProperties);

    /**
     * The z axis orientation.
     *
     * @default { z: 0 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#orientation)
     */
    readonly orientation: Orientation;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#state)
     */
    readonly state: "compass" | "rotation" | "disabled";
    /**
     * The view in which the Compass obtains and indicates camera [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading), using a (SceneView) or [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) (MapView).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * If working in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html), sets the view's [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) is to `0`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#reset)
     */
    reset(): void;
  }

  interface CompassViewModelProperties extends GoToProperties {
    /**
     * The view in which the Compass obtains and indicates camera [heading](https://developers.arcgis.com/javascript/latest/api-reference/esri-Camera.html#heading), using a (SceneView) or [rotation](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#rotation) (MapView).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The z axis orientation.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#Orientation)
   */
  export interface Orientation {
    /**
     * z axis orientation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Compass-CompassViewModel.html#Orientation)
     */
    z?: number;
  }

  export interface CoordinateConversion extends Widget, GoTo {}

  export class CoordinateConversion {
    /**
     * The CoordinateConversion widget provides a way to display user cursor position either as map coordinates or as any of several popular coordinate notations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html)
     */

    constructor(properties?: CoordinateConversionProperties);

    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Conversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html) that the widget is currently displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#conversions)
     */
    conversions: Collection<Conversion>;
    /**
     * Describes the location of the coordinates currently displayed by the widget as a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#currentLocation)
     */
    readonly currentLocation: Point;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) that the widget is capable of displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#formats)
     */
    formats: Collection<Format>;
    /**
     * Indicates the heading level to use for the coordinate input and coordinate settings headings.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#headingLevel)
     */
    headingLevel: number;
    /**
     * This symbol is used to visualize the location currently described by the widget when `capture` mode is active.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#locationSymbol)
     */
    locationSymbol: SimpleMarkerSymbol | PictureMarkerSymbol;
    /**
     * Describes the current mode of the widget.
     *
     * @default "live"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#mode)
     */
    mode: "live" | "capture";
    /**
     * If this property is set to `true`, multiple conversions can be displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#multipleConversions)
     */
    multipleConversions: boolean;
    /**
     * Determines whether the widget should expand up or down.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#orientation)
     */
    orientation: "auto" | "expand-up" | "expand-down";
    /**
     * If this property is set to `true`, sessionStorage or localStorage (depending on [storageType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageType)) will be used to hydrate and persist the CoordinateConversion widget's state.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageEnabled)
     */
    storageEnabled: boolean;
    /**
     * This property determines whether sessionStorage or localStorage will be used to store widget state.
     *
     * @default "session"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageType)
     */
    storageType: "session" | "local";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#viewModel)
     */
    viewModel: CoordinateConversionViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#visibleElements)
     */
    visibleElements: CoordinateConversionVisibleElements;

    /**
     * Attempt to convert a string into a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#reverseConvert)
     */
    reverseConvert(coordinate: string, format: Format): Promise<Point>;
  }

  interface CoordinateConversionProperties extends WidgetProperties, GoToProperties {
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Conversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html) that the widget is currently displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#conversions)
     */
    conversions?: CollectionProperties<ConversionProperties>;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) that the widget is capable of displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#formats)
     */
    formats?: CollectionProperties<FormatProperties>;
    /**
     * Indicates the heading level to use for the coordinate input and coordinate settings headings.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * This symbol is used to visualize the location currently described by the widget when `capture` mode is active.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#locationSymbol)
     */
    locationSymbol?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" });
    /**
     * Describes the current mode of the widget.
     *
     * @default "live"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#mode)
     */
    mode?: "live" | "capture";
    /**
     * If this property is set to `true`, multiple conversions can be displayed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#multipleConversions)
     */
    multipleConversions?: boolean;
    /**
     * Determines whether the widget should expand up or down.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#orientation)
     */
    orientation?: "auto" | "expand-up" | "expand-down";
    /**
     * If this property is set to `true`, sessionStorage or localStorage (depending on [storageType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageType)) will be used to hydrate and persist the CoordinateConversion widget's state.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageEnabled)
     */
    storageEnabled?: boolean;
    /**
     * This property determines whether sessionStorage or localStorage will be used to store widget state.
     *
     * @default "session"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#storageType)
     */
    storageType?: "session" | "local";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#viewModel)
     */
    viewModel?: CoordinateConversionViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#visibleElements)
     */
    visibleElements?: CoordinateConversionVisibleElements;
  }

  export interface CoordinateConversionViewModel extends Accessor, GoTo {}

  export class CoordinateConversionViewModel {
    /**
     * Provides the logic for the [CoordinateConversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html)
     */

    constructor(properties?: CoordinateConversionViewModelProperties);

    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Conversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html) that the widget is currently displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#conversions)
     */
    conversions: Collection<Conversion>;
    /**
     * Describes the location of the coordinates currently displayed by the widget as a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#currentLocation)
     */
    currentLocation: Point;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) that the widget is capable of displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#formats)
     */
    formats: Collection<Format>;
    /**
     * This symbol is used to visualize the location currently described by the widget when `capture` mode is active.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#locationSymbol)
     */
    locationSymbol: SimpleMarkerSymbol | PictureMarkerSymbol;
    /**
     * Describes the current mode of the widget.
     *
     * @default "live"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#mode)
     */
    mode: "live" | "capture";
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#state)
     */
    readonly state: "ready" | "loading" | "disabled";
    /**
     * If this property is set to `true`, sessionStorage or localStorage (depending on [storageType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageType)) will be used to hydrate and persist the CoordinateConversion widget's state.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageEnabled)
     */
    storageEnabled: boolean;
    /**
     * This property determines whether sessionStorage or localStorage will be used to store widget state.
     *
     * @default "session"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageType)
     */
    storageType: "session" | "local";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Attempt to convert a point into a [Position](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#Position).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#convert)
     */
    convert(point: Point, format: Format): Promise<Position>;
    /**
     * Attempt to convert a string into a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#reverseConvert)
     */
    reverseConvert(coordinate: string, format: Format): Promise<Point>;
    /**
     * Update the input conversions based on the input point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#updateConversions)
     */
    updateConversions(conversions: Conversion[], location: Point): Promise<Conversion[]>;
  }

  interface CoordinateConversionViewModelProperties extends GoToProperties {
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Conversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html) that the widget is currently displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#conversions)
     */
    conversions?: CollectionProperties<ConversionProperties>;
    /**
     * Describes the location of the coordinates currently displayed by the widget as a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#currentLocation)
     */
    currentLocation?: PointProperties;
    /**
     * A [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) containing every [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) that the widget is capable of displaying.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#formats)
     */
    formats?: CollectionProperties<FormatProperties>;
    /**
     * This symbol is used to visualize the location currently described by the widget when `capture` mode is active.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#locationSymbol)
     */
    locationSymbol?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PictureMarkerSymbolProperties & { type: "picture-marker" });
    /**
     * Describes the current mode of the widget.
     *
     * @default "live"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#mode)
     */
    mode?: "live" | "capture";
    /**
     * If this property is set to `true`, sessionStorage or localStorage (depending on [storageType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageType)) will be used to hydrate and persist the CoordinateConversion widget's state.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageEnabled)
     */
    storageEnabled?: boolean;
    /**
     * This property determines whether sessionStorage or localStorage will be used to store widget state.
     *
     * @default "session"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#storageType)
     */
    storageType?: "session" | "local";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * Describes a point in terms of a location, a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), and a coordinate, a string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#Position)
   */
  export interface Position {
    /**
     * A point geometry representing the location described by the conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#Position)
     */
    location: Point;
    /**
     * A string representing the location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-CoordinateConversionViewModel.html#Position)
     */
    coordinate: string;
  }

  export class Conversion extends Accessor {
    /**
     * The Conversion class represents one of the [conversions](esri-widgets-CoordinateConversion.html#conversions) in the [Coordinate Conversion widget](esri-widgets-CoordinateConversion.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html)
     */

    constructor(properties?: ConversionProperties);

    /**
     * A formatted string based on the current [position](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#position) and display information on the [format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#format).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#displayCoordinate)
     */
    readonly displayCoordinate: string;
    /**
     * The [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) for this conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#format)
     */
    format: Format;
    /**
     * The position property contains the location information for this conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#position)
     */
    position: ConversionPosition;
  }

  interface ConversionProperties {
    /**
     * The [Format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html) for this conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#format)
     */
    format?: FormatProperties;
    /**
     * The position property contains the location information for this conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#position)
     */
    position?: ConversionPosition;
  }

  /**
   * Describes a point in terms of a location, a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), and a coordinate, a string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#Position)
   */
  export interface ConversionPosition {
    /**
     * A point geometry representing the location described by the conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#Position)
     */
    location: Point;
    /**
     * A string representing the location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Conversion.html#Position)
     */
    coordinate: string;
  }

  export class Format extends Accessor {
    /**
     * The Format class represents one of the [formats](esri-widgets-CoordinateConversion.html#formats) in the [Coordinate Conversion widget](esri-widgets-CoordinateConversion.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html)
     */

    constructor(properties?: FormatProperties);

    /**
     * Contains information that describes how this Format should be converted.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#conversionInfo)
     */
    conversionInfo: ConversionInfo;
    /**
     * A array of [Coordinate Segments](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#coordinateSegments)
     */
    coordinateSegments: CoordinateSegment[];
    /**
     * A string that serves as a pattern describing how this Format should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#currentPattern)
     */
    currentPattern: string;
    /**
     * The default pattern describing how this Format should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#defaultPattern)
     */
    defaultPattern: string;
    /**
     * A string that is used to label this Format in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#label)
     */
    label: string;
    /**
     * The name of the coordinate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#name)
     */
    name: string;
    /**
     * The [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) for this format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The view model of the [CoordinateConversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html) widget that is utilizing this format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#viewModel)
     */
    viewModel: CoordinateConversionViewModel;
  }

  interface FormatProperties {
    /**
     * Contains information that describes how this Format should be converted.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#conversionInfo)
     */
    conversionInfo?: ConversionInfo;
    /**
     * A array of [Coordinate Segments](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#coordinateSegments)
     */
    coordinateSegments?: CoordinateSegment[];
    /**
     * A string that serves as a pattern describing how this Format should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#currentPattern)
     */
    currentPattern?: string;
    /**
     * The default pattern describing how this Format should be displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#defaultPattern)
     */
    defaultPattern?: string;
    /**
     * A string that is used to label this Format in the UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#label)
     */
    label?: string;
    /**
     * The name of the coordinate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#name)
     */
    name?: string;
    /**
     * The [SpatialReference](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-SpatialReference.html) for this format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * The view model of the [CoordinateConversion](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html) widget that is utilizing this format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#viewModel)
     */
    viewModel?: CoordinateConversionViewModelProperties;
  }

  /**
   * This object describes how a Format should be projected and formatted for display.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#ConversionInfo)
   */
  export interface ConversionInfo {
    /**
     * A function that takes a point and returns a [position](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#Position).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#ConversionInfo)
     */
    convert?: ConvertFunction;
    /**
     * A function that takes a string and returns a Point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#ConversionInfo)
     */
    reverseConvert?: ReverseConvertFunction;
  }

  export type ConvertFunction = (point: Point) => FormatPosition;

  /**
   * A coordinate segment represents one piece of a coordinate string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment)
   */
  export interface CoordinateSegment {
    /**
     * A string that provides the name for this segment of a coordinate string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment)
     */
    alias: string;
    /**
     * A string that describes the role of this segment.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment)
     */
    description: string;
    /**
     * A regular expression that should extract this segment from the complete coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#CoordinateSegment)
     */
    searchPattern: RegExp;
  }

  /**
   * Describes a point in terms of a location, a [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), and a coordinate, a string.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#Position)
   */
  export interface FormatPosition {
    /**
     * A point geometry representing the location described by the conversion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#Position)
     */
    location: Point;
    /**
     * A string representing the location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion-support-Format.html#Position)
     */
    coordinate: string;
  }

  export type ReverseConvertFunction = (coordinate: string) => Point;

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#VisibleElements)
   */
  export interface CoordinateConversionVisibleElements {
    /**
     * Indicates whether the settings button will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#VisibleElements)
     */
    settingsButton?: boolean;
    /**
     * Indicates whether the input coordinate button will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#VisibleElements)
     */
    editButton?: boolean;
    /**
     * Indicates whether the expand and retract buttons will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#VisibleElements)
     */
    expandButton?: boolean;
    /**
     * Indicates whether the capture mode button will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CoordinateConversion.html#VisibleElements)
     */
    captureButton?: boolean;
  }

  export class Daylight extends Widget {
    /**
     * The Daylight widget can be used to manipulate the lighting conditions of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html)
     */

    constructor(properties?: DaylightProperties);

    /**
     * Controls whether the widget displays a date or a season picker.
     *
     * @default "date"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#dateOrSeason)
     */
    dateOrSeason: "season" | "date";
    /**
     * Indicates the heading level to use for the widget title.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Controls the speed of the daytime and date animation.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#playSpeedMultiplier)
     */
    playSpeedMultiplier: number;
    /**
     * Sets steps, or intervals, on the time slider to restrict the times of the day that can be selected when dragging the thumb.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#timeSliderSteps)
     */
    timeSliderSteps: number | number[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#view)
     */
    view: SceneView;
    /**
     * The view model for the Daylight widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#viewModel)
     */
    viewModel: DaylightViewModel;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#visibleElements)
     */
    visibleElements: DaylightVisibleElements;
  }

  interface DaylightProperties extends WidgetProperties {
    /**
     * Controls whether the widget displays a date or a season picker.
     *
     * @default "date"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#dateOrSeason)
     */
    dateOrSeason?: "season" | "date";
    /**
     * Indicates the heading level to use for the widget title.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Controls the speed of the daytime and date animation.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#playSpeedMultiplier)
     */
    playSpeedMultiplier?: number;
    /**
     * Sets steps, or intervals, on the time slider to restrict the times of the day that can be selected when dragging the thumb.
     *
     * @default 5
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#timeSliderSteps)
     */
    timeSliderSteps?: number | number[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for the Daylight widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#viewModel)
     */
    viewModel?: DaylightViewModelProperties;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#visibleElements)
     */
    visibleElements?: DaylightVisibleElements;
  }

  export class DaylightViewModel extends Accessor {
    /**
     * Provides the logic for the [Daylight](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html)
     */

    constructor(properties?: DaylightViewModelProperties);

    /**
     * A season can be set instead of a date.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#currentSeason)
     */
    currentSeason: "spring" | "summer" | "fall" | "winter";
    /**
     * Starts or pauses the daytime animation cycling through the minutes of the day.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#dayPlaying)
     */
    dayPlaying: boolean;
    /**
     * Indicates whether to show shadows cast by the sun.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#directShadowsEnabled)
     */
    directShadowsEnabled: boolean;
    /**
     * The calendar date in the timezone given by [utcOffset](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#localDate)
     */
    localDate: Date;
    /**
     * Controls the daytime and date animation speed.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#playSpeedMultiplier)
     */
    playSpeedMultiplier: number;
    /**
     * Indicates whether date and time are used to determine position of the light source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#sunLightingEnabled)
     */
    sunLightingEnabled: boolean;
    /**
     * Slider position for the time of day in the timezone given by [utcOffset](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#timeSliderPosition)
     */
    timeSliderPosition: number;
    /**
     * The difference in hours between UTC time and the time displayed in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset)
     */
    utcOffset: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#view)
     */
    view: SceneView;
    /**
     * Starts or pauses the date animation cycling through the months of the year.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#yearPlaying)
     */
    yearPlaying: boolean;
  }

  interface DaylightViewModelProperties {
    /**
     * A season can be set instead of a date.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#currentSeason)
     */
    currentSeason?: "spring" | "summer" | "fall" | "winter";
    /**
     * Starts or pauses the daytime animation cycling through the minutes of the day.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#dayPlaying)
     */
    dayPlaying?: boolean;
    /**
     * Indicates whether to show shadows cast by the sun.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#directShadowsEnabled)
     */
    directShadowsEnabled?: boolean;
    /**
     * The calendar date in the timezone given by [utcOffset](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#localDate)
     */
    localDate?: DateProperties;
    /**
     * Controls the daytime and date animation speed.
     *
     * @default 1.0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#playSpeedMultiplier)
     */
    playSpeedMultiplier?: number;
    /**
     * Indicates whether date and time are used to determine position of the light source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#sunLightingEnabled)
     */
    sunLightingEnabled?: boolean;
    /**
     * Slider position for the time of day in the timezone given by [utcOffset](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#timeSliderPosition)
     */
    timeSliderPosition?: number;
    /**
     * The difference in hours between UTC time and the time displayed in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#utcOffset)
     */
    utcOffset?: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#view)
     */
    view?: SceneViewProperties;
    /**
     * Starts or pauses the date animation cycling through the months of the year.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight-DaylightViewModel.html#yearPlaying)
     */
    yearPlaying?: boolean;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
   */
  export interface DaylightVisibleElements {
    /**
     * When set to `false`, the header is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    header?: boolean;
    /**
     * When set to `false`, neither of the play buttons are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    playButtons?: boolean;
    /**
     * When set to `false`, neither the date nor the season picker are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    datePicker?: boolean;
    /**
     * When set to `false`, the timezone selector is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    timezone?: boolean;
    /**
     * When set to `false`, the sun lighting checkbox is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    sunLightingToggle?: boolean;
    /**
     * When set to `false`, the shadows checkbox is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Daylight.html#VisibleElements)
     */
    shadowsToggle?: boolean;
  }

  export class DirectionalPad extends Widget {
    /**
     * A directional pad (D-Pad) widget can be used to control the position and rotation of the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html)
     */

    constructor(properties?: DirectionalPadProperties);

    /**
     * Indicates whether interaction is allowed on the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#disabled)
     */
    disabled: boolean;
    /**
     * The view which the directional pad will control.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#view)
     */
    view: MapView;
    /**
     * The view model for the DirectionalPad widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#viewModel)
     */
    viewModel: DirectionalPadViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#visibleElements)
     */
    visibleElements: DirectionalPadVisibleElements;
    /**
     * Determines the size of directional pad buttons and the slider.
     *
     * @default "s"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#visualScale)
     */
    visualScale: "s" | "m" | "l";
  }

  interface DirectionalPadProperties extends WidgetProperties {
    /**
     * Indicates whether interaction is allowed on the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#disabled)
     */
    disabled?: boolean;
    /**
     * The view which the directional pad will control.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for the DirectionalPad widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#viewModel)
     */
    viewModel?: DirectionalPadViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#visibleElements)
     */
    visibleElements?: DirectionalPadVisibleElements;
    /**
     * Determines the size of directional pad buttons and the slider.
     *
     * @default "s"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#visualScale)
     */
    visualScale?: "s" | "m" | "l";
  }

  export class DirectionalPadViewModel extends Accessor {
    /**
     * Provides the logic for the [DirectionalPad](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html)
     */

    constructor(properties?: DirectionalPadViewModelProperties);

    /**
     * The closest angle that matches the current mouse position relative to the widget's center.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#angle)
     */
    readonly angle: number;
    /**
     * Indicates whether interaction is allowed on the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#disabled)
     */
    disabled: boolean;
    /**
     * The angle of rotation of the map.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#rotation)
     */
    rotation: number;
    /**
     * The state of the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "moving";

    /**
     * Starts movement in the correct direction according to pointer location, and listens for pointer move (to change direction) or release (to stop movement)  This method should be called in response to pointerdown event on a button
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#beginFollowingPointer)
     */
    beginFollowingPointer(pointerLocation: Vector, widgetCenter: Vector): void;
    /**
     * Initialize a brief movement in a desired direction
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#moveOnce)
     */
    moveOnce(angle: number): void;
  }

  interface DirectionalPadViewModelProperties {
    /**
     * Indicates whether interaction is allowed on the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#disabled)
     */
    disabled?: boolean;
    /**
     * The angle of rotation of the map.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#rotation)
     */
    rotation?: number;
  }

  /**
   * An x/y coordinate pair.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#Vector)
   */
  export interface Vector {
    /**
     * X coordinate
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#Vector)
     */
    x?: number;
    /**
     * Y coordinate
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad-DirectionalPadViewModel.html#Vector)
     */
    y?: number;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#VisibleElements)
   */
  export interface DirectionalPadVisibleElements {
    /**
     * Indicates whether to display the directional pad buttons.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#VisibleElements)
     */
    directionalButtons?: boolean;
    /**
     * Indicates whether to display the map rotation slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#VisibleElements)
     */
    rotationSlider?: boolean;
    /**
     * Indicates whether to display the button to reset the rotation angle.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectionalPad.html#VisibleElements)
     */
    rotationResetButton?: boolean;
  }

  export interface Directions extends Widget, GoTo {}

  export class Directions {
    /**
     * The Directions widget provides a way to calculate directions, between two or more input locations with a [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html), using ArcGIS Online and custom Network Analysis Route services.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html)
     */

    constructor(properties?: DirectionsProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#apiKey)
     */
    apiKey: string;
    /**
     * Indicates the heading level to use for the origin and destination addresses (i.e.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The most recent route result.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    readonly lastRoute: DirectionsLastRoute;
    /**
     * The [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) associated with the Directions widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#layer)
     */
    layer: RouteLayer;
    /**
     * The maximum number of stops allowed for routing.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#maxStops)
     */
    maxStops: number;
    /**
     * Controls the default properties used when [searching](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#searchProperties)
     */
    searchProperties: DirectionsSearchProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#unit)
     */
    unit:
      | "imperial"
      | "metric"
      | "centimeters"
      | "decimeters"
      | "feet"
      | "inches"
      | "kilometers"
      | "meters"
      | "miles"
      | "millimeters"
      | "nautical-miles"
      | "yards";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#viewModel)
     */
    viewModel: DirectionsViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#visibleElements)
     */
    visibleElements: DirectionsVisibleElements;

    /**
     * Computes a route and directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#getDirections)
     */
    getDirections(): Promise<RouteLayerSolveResult>;
    /**
     * Saves the RouteLayer associated with the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the RouteLayer associated with the view model as a new portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#saveAs)
     */
    saveAs(portalItem: Promise<PortalItem>, options?: DirectionsSaveAsOptions): Promise<PortalItem>;
    /**
     * Zoom so that the full route is displayed within the current map extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#zoomToRoute)
     */
    zoomToRoute(): void;
  }

  interface DirectionsProperties extends WidgetProperties, GoToProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#apiKey)
     */
    apiKey?: string;
    /**
     * Indicates the heading level to use for the origin and destination addresses (i.e.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) associated with the Directions widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#layer)
     */
    layer?: RouteLayerProperties;
    /**
     * The maximum number of stops allowed for routing.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#maxStops)
     */
    maxStops?: number;
    /**
     * Controls the default properties used when [searching](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#searchProperties)
     */
    searchProperties?: DirectionsSearchProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "centimeters"
      | "decimeters"
      | "feet"
      | "inches"
      | "kilometers"
      | "meters"
      | "miles"
      | "millimeters"
      | "nautical-miles"
      | "yards";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#viewModel)
     */
    viewModel?: DirectionsViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#visibleElements)
     */
    visibleElements?: DirectionsVisibleElements;
  }

  export interface DirectionsViewModel extends Accessor, GoTo {}

  export class DirectionsViewModel {
    /**
     * Provides the communication and data manipulation logic for the [Directions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html)
     */

    constructor(properties?: DirectionsViewModelProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#apiKey)
     */
    apiKey: string;
    /**
     * The network attribute name to be used as the impedance attribute in the analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#impedanceAttribute)
     */
    readonly impedanceAttribute: DirectionsViewModelImpedanceAttribute;
    /**
     * The most recent route result.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    readonly lastRoute: DirectionsViewModelLastRoute;
    /**
     * The [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) associated with the DirectionsViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#layer)
     */
    layer: RouteLayer;
    /**
     * The maximum number of stops allowed for routing.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#maxStops)
     */
    maxStops: number;
    /**
     * Route Parameters object used to call the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#routeParameters)
     */
    routeParameters: RouteParameters;
    /**
     * The travel mode that will be used by [getDirections()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#getDirections) when requesting the route and directions.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#selectedTravelMode)
     */
    selectedTravelMode: TravelMode;
    /**
     * The Service Description object returned by the Route REST Endpoint.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#serviceDescription)
     */
    readonly serviceDescription: ServiceDescription;
    /**
     * The current state of the view model.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * The name of the network attribute to use for the drive time when computing directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#timeAttribute)
     */
    readonly timeAttribute: DirectionsViewModelTimeAttribute;
    /**
     * An array of available [travel modes](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-TravelMode.html) from the associated [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#layer)'s routing service (see [RouteLayer.url](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html#url)).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#travelModes)
     */
    readonly travelModes: TravelMode[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Centers the map at the specified maneuver or stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#centerAt)
     */
    centerAt(stopOrManeuver: Graphic): void;
    /**
     * Clears any highlighted route segments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#clearHighlights)
     */
    clearHighlights(): void;
    /**
     * Removes the route directions from the directions list, leaving the inputs untouched.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#clearResults)
     */
    clearResults(): void;
    /**
     * Returns the cost attribute associated with the parsed name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#getCostAttribute)
     */
    getCostAttribute(attributeName: string): any;
    /**
     * Computes a route and directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#getDirections)
     */
    getDirections(): Promise<RouteLayerSolveResult>;
    /**
     * Highlights the specified network feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#highlight)
     */
    highlight(
      networkFeature:
        | DirectionLine
        | DirectionPoint
        | PointBarrier
        | PolygonBarrier
        | PolylineBarrier
        | RouteInfo
        | Stop,
    ): Promise<any>;
    /**
     * This method should be called to load the view model's routing resources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#load)
     */
    load(): Promise<any>;
    /**
     * Resets the state of the ViewModel, clearing all the input stops and results in the widget and in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#reset)
     */
    reset(): void;
    /**
     * Saves the RouteLayer associated with the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#save)
     */
    save(): Promise<PortalItem>;
    /**
     * Saves the RouteLayer associated with the view model as a new portal item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#saveAs)
     */
    saveAs(portalItem: Promise<PortalItem>, options?: DirectionsViewModelSaveAsOptions): Promise<PortalItem>;
    /**
     * Zoom so that the full route is displayed within the current map extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#zoomToRoute)
     */
    zoomToRoute(): void;
  }

  interface DirectionsViewModelProperties extends GoToProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#apiKey)
     */
    apiKey?: string;
    /**
     * The [RouteLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-RouteLayer.html) associated with the DirectionsViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#layer)
     */
    layer?: RouteLayerProperties;
    /**
     * The maximum number of stops allowed for routing.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#maxStops)
     */
    maxStops?: number;
    /**
     * Route Parameters object used to call the service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#routeParameters)
     */
    routeParameters?: RouteParametersProperties;
    /**
     * The travel mode that will be used by [getDirections()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#getDirections) when requesting the route and directions.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#selectedTravelMode)
     */
    selectedTravelMode?: TravelModeProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface DirectionsViewModelImpedanceAttribute {
    /**
     * Name of the impedance network attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#impedanceAttribute)
     */
    name: string;
    /**
     * Units of this network attribute.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#impedanceAttribute)
     */
    units: string;
  }

  export interface DirectionsViewModelLastRoute extends RouteLayerSolveResult {
    /**
     * Collection of direction polylines associated with line segments between turns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    directionLines: Collection<DirectionLine>;
    /**
     * Collection of direction items as points with various display information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    directionPoints: Collection<DirectionPoint>;
    /**
     * Collection of point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    pointBarriers: Collection<PointBarrier>;
    /**
     * Collection of polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    polygonBarriers: Collection<PolygonBarrier>;
    /**
     * Collection of polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    polylineBarriers: Collection<PolylineBarrier>;
    /**
     * Information about a solved route including the route's geometry and overall distance and time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    routeInfo: RouteInfo;
    /**
     * Collection of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#lastRoute)
     */
    stops: Collection<Stop>;
  }

  export interface DirectionsViewModelSaveAsOptions {
    /**
     * The folder in which to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#saveAs)
     */
    folder?: PortalFolder;
  }

  export interface DirectionsViewModelTimeAttribute {
    /**
     * The name of the network attribute to use for the drive time when computing directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#timeAttribute)
     */
    name: string;
    /**
     * The units to use when calculating drive time for directions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions-DirectionsViewModel.html#timeAttribute)
     */
    units: string;
  }

  export interface DirectionsLastRoute extends RouteLayerSolveResult {
    /**
     * Collection of direction polylines associated with line segments between turns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    directionLines: Collection<DirectionLine>;
    /**
     * Collection of direction items as points with various display information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    directionPoints: Collection<DirectionPoint>;
    /**
     * Collection of point barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    pointBarriers: Collection<PointBarrier>;
    /**
     * Collection of polygon barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    polygonBarriers: Collection<PolygonBarrier>;
    /**
     * Collection of polyline barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    polylineBarriers: Collection<PolylineBarrier>;
    /**
     * Information about a solved route including the route's geometry and overall distance and time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    routeInfo: RouteInfo;
    /**
     * Collection of stops.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#lastRoute)
     */
    stops: Collection<Stop>;
  }

  export interface DirectionsSaveAsOptions {
    /**
     * The folder in which to save the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#saveAs)
     */
    folder?: PortalFolder;
  }

  /**
   * Configurable Search properties of the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
   */
  export interface DirectionsSearchProperties {
    /**
     * Number index indicating the current selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    activeSourceIndex?: number;
    /**
     * String value used as a hint for input text when searching on multiple sources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    allPlaceholder?: string;
    /**
     * Indicates whether to automatically navigate to the selected result once selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    autoNavigate?: boolean;
    /**
     * Indicates whether to automatically select and zoom to the first geocoded result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    autoSelect?: boolean;
    /**
     * Indicates whether or not to include [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) in the Search UI.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    includeDefaultSources?: boolean | Function;
    /**
     * Define the type of location, either `"street"` or `"rooftop"`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    locationType?: string;
    /**
     * Indicates the maximum number of search results to return.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    maxResults?: number;
    /**
     * Indicates the maximum number of suggestions to return for the widget's input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    maxSuggestions?: number;
    /**
     * Indicates the minimum number of characters required before querying for a suggestion.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    minSuggestCharacters?: number;
    /**
     * Indicates whether to display a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when a selected result is clicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    popupEnabled?: boolean;
    /**
     * A customized PopupTemplate for the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    popupTemplate?: PopupTemplate;
    /**
     * Indicates whether to show a graphic on the map for the selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    resultGraphicEnabled?: boolean;
    /**
     * Indicates whether to display the option to search all sources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    searchAllEnabled?: boolean;
    /**
     * The value of the search box input text string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    searchTerm?: string;
    /**
     * Specifies the sources to search in the [view](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#view).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    sources?: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * Indicates whether to display suggestions as the user enters input text in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    suggestionsEnabled?: boolean;
    /**
     * The view of the widget.
     *
     * [Read more...](global.html)
     */
    view?: MapView | SceneView;
    /**
     * The Search widget's view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#SearchProperties)
     */
    viewModel?: SearchViewModel;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#VisibleElements)
   */
  export interface DirectionsVisibleElements {
    /**
     * Indicates whether to a link to the route layer portal-item (if any) will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#VisibleElements)
     */
    layerDetails?: boolean;
    /**
     * Indicates whether to the Save As button will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#VisibleElements)
     */
    saveAsButton?: boolean;
    /**
     * Indicates whether to the Save button will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Directions.html#VisibleElements)
     */
    saveButton?: boolean;
  }

  export class DirectLineMeasurement3D extends Widget {
    /**
     * The DirectLineMeasurement3D widget calculates and displays the 3D distance between two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html)
     */

    constructor(properties?: DirectLineMeasurement3DProperties);

    /**
     * The direct line measurement analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#analysis)
     */
    readonly analysis: DirectLineMeasurementAnalysis;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of unit systems (imperial, metric) and specific units for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#viewModel)
     */
    viewModel: DirectLineMeasurement3DViewModel;
  }

  interface DirectLineMeasurement3DProperties extends WidgetProperties {
    /**
     * The direct line measurement analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#analysis)
     */
    analysis?: DirectLineMeasurementAnalysisProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of unit systems (imperial, metric) and specific units for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html#viewModel)
     */
    viewModel?: DirectLineMeasurement3DViewModelProperties;
  }

  export class DirectLineMeasurement3DViewModel extends Accessor {
    /**
     * Provides the logic for the [DirectLineMeasurement3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html)
     */

    constructor(properties?: DirectLineMeasurement3DViewModelProperties);

    /**
     * The direct line measurement analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#analysis)
     */
    readonly analysis: DirectLineMeasurementAnalysis;
    /**
     * The current measurement calculated between the two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#measurement)
     */
    readonly measurement: DirectLineMeasurement3DViewModelMeasurement;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "measuring" | "measured";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of unit systems (imperial, metric) and specific units for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#view)
     */
    view: SceneView;

    /**
     * Clears the current measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#clear)
     */
    clear(): void;
    /**
     * Starts a new measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#start)
     */
    start(): void;
  }

  interface DirectLineMeasurement3DViewModelProperties {
    /**
     * The direct line measurement analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#analysis)
     */
    analysis?: DirectLineMeasurementAnalysisProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of unit systems (imperial, metric) and specific units for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export interface DirectLineMeasurement3DViewModelMeasurement {
    /**
     * Describes how the `horizontalDistance` is computed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#measurement)
     */
    mode: "euclidean" | "geodesic";
    /**
     * The 3D distance between the two points that is computed in a Euclidean manner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#measurement)
     */
    directDistance: DirectLineMeasurement3DViewModelMeasurementValue;
    /**
     * The 2D distance between the two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#measurement)
     */
    horizontalDistance: DirectLineMeasurement3DViewModelMeasurementValue;
    /**
     * The elevation difference between the two points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#measurement)
     */
    verticalDistance: DirectLineMeasurement3DViewModelMeasurementValue;
  }

  /**
   * Measurement value.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#MeasurementValue)
   */
  export interface DirectLineMeasurement3DViewModelMeasurementValue {
    /**
     * Textual representation of the measured value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#MeasurementValue)
     */
    text: string;
    /**
     * State of the measured value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DirectLineMeasurement3D-DirectLineMeasurement3DViewModel.html#MeasurementValue)
     */
    state: string;
  }

  export class DistanceMeasurement2D extends Widget {
    /**
     * The DistanceMeasurement2D widget calculates and displays the distance between two or more points in a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html)
     */

    constructor(properties?: DistanceMeasurement2DProperties);

    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#viewModel)
     */
    viewModel: DistanceMeasurement2DViewModel;
  }

  interface DistanceMeasurement2DProperties extends WidgetProperties {
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html#viewModel)
     */
    viewModel?: DistanceMeasurement2DViewModelProperties;
  }

  export class DistanceMeasurement2DViewModel extends Accessor {
    /**
     * Provides the logic for the [DistanceMeasurement2D](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html)
     */

    constructor(properties?: DistanceMeasurement2DViewModelProperties);

    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold: number;
    /**
     * The length and geometry of the measurement polyline in meters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#measurement)
     */
    readonly measurement: DistanceMeasurement2DViewModelMeasurement;
    /**
     * This property returns the locale specific representation of the length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#measurementLabel)
     */
    readonly measurementLabel: string;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "measuring" | "measured";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#view)
     */
    view: MapView;

    /**
     * Clears the current measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#clear)
     */
    clear(): void;
    /**
     * Starts a new measurement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#start)
     */
    start(): void;
  }

  interface DistanceMeasurement2DViewModelProperties {
    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold?: number;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  export interface DistanceMeasurement2DViewModelMeasurement {
    /**
     * Line length (m).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#measurement)
     */
    length: number;
    /**
     * Measurement line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-DistanceMeasurement2D-DistanceMeasurement2DViewModel.html#measurement)
     */
    geometry: any;
  }

  export class Editor extends Widget {
    /**
     * This widget provides an out-of-the-box editing experience to help streamline editing within a web application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html)
     */

    constructor(properties?: EditorProperties);

    /**
     * A property indicating the current active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#activeWorkflow)
     */
    readonly activeWorkflow: CreateFeaturesWorkflow | UpdateWorkflow;
    /**
     * An array of string values which specifies what end users are allowed to edit.
     *
     * @deprecated since version 4.29. Use {@link module:esri/widgets/Editor#visibleElements Editor.visibleElements} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#allowedWorkflows)
     */
    allowedWorkflows: "create-features" | "update";
    /**
     * Indicates the heading level to use for title of the widget.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#labelOptions)
     */
    labelOptions: SketchLabelOptions;
    /**
     * An array of editing configurations for individual layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#layerInfos)
     */
    layerInfos: EditorLayerInfo[];
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * This property allows customization of supporting Editor widgets and their default behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#supportingWidgetDefaults)
     */
    supportingWidgetDefaults: SupportingWidgetDefaults;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#tooltipOptions)
     */
    tooltipOptions: SketchTooltipOptions;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#valueOptions)
     */
    valueOptions: SketchValueOptions;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#viewModel)
     */
    viewModel: EditorViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#visibleElements)
     */
    visibleElements: EditorVisibleElements;

    /**
     * Cancels any active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#cancelWorkflow)
     */
    cancelWorkflow(): Promise<void>;
    /**
     * This is applicable if there is an active [UpdateWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#deleteFeatureFromWorkflow)
     */
    deleteFeatureFromWorkflow(): Promise<void>;
    /**
     * Initiates the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) by displaying the panel where feature(s) creation begins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startCreateFeaturesWorkflowAtFeatureCreation)
     */
    startCreateFeaturesWorkflowAtFeatureCreation(creationInfo: CreationInfo): Promise<void>;
    /**
     * This method starts the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) at the "creating-features" step with the provided feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startCreateFeaturesWorkflowAtFeatureEdit)
     */
    startCreateFeaturesWorkflowAtFeatureEdit(
      params: EditorStartCreateFeaturesWorkflowAtFeatureEditParams,
    ): Promise<void>;
    /**
     * Initiates the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) by displaying the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startCreateFeaturesWorkflowAtFeatureTypeSelection)
     */
    startCreateFeaturesWorkflowAtFeatureTypeSelection(): Promise<void>;
    /**
     * Starts the [UpdateWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html) at the attribute editing panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startUpdateWorkflowAtFeatureEdit)
     */
    startUpdateWorkflowAtFeatureEdit(feature: Graphic): Promise<void>;
    /**
     * Starts the [UpdateWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html) using the current selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startUpdateWorkflowAtFeatureSelection)
     */
    startUpdateWorkflowAtFeatureSelection(): Promise<void>;
    /**
     * This method starts the [UpdateWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html) where it waits for multiple features to be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startUpdateWorkflowAtMultipleFeatureSelection)
     */
    startUpdateWorkflowAtMultipleFeatureSelection(candidates: Graphic[]): Promise<void>;

    on(name: "sketch-create", eventHandler: EditorSketchCreateEventHandler): IHandle;

    on(name: "sketch-update", eventHandler: EditorSketchUpdateEventHandler): IHandle;
  }

  interface EditorProperties extends WidgetProperties {
    /**
     * An array of string values which specifies what end users are allowed to edit.
     *
     * @deprecated since version 4.29. Use {@link module:esri/widgets/Editor#visibleElements Editor.visibleElements} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#allowedWorkflows)
     */
    allowedWorkflows?: "create-features" | "update";
    /**
     * Indicates the heading level to use for title of the widget.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#labelOptions)
     */
    labelOptions?: SketchLabelOptionsProperties;
    /**
     * An array of editing configurations for individual layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#layerInfos)
     */
    layerInfos?: EditorLayerInfo[];
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for editing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * This property allows customization of supporting Editor widgets and their default behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#supportingWidgetDefaults)
     */
    supportingWidgetDefaults?: SupportingWidgetDefaults;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#tooltipOptions)
     */
    tooltipOptions?: SketchTooltipOptionsProperties;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#valueOptions)
     */
    valueOptions?: SketchValueOptionsProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#viewModel)
     */
    viewModel?: EditorViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#visibleElements)
     */
    visibleElements?: EditorVisibleElements;
  }

  export class CreateFeaturesWorkflow extends Workflow {
    /**
     * A read-only class containing the logic used when creating features using the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html)
     */

    constructor(properties?: CreateFeaturesWorkflowProperties);

    /**
     * Indicates the current feature state during creation.
     *
     * @default "create-new"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html#createFeatureState)
     */
    readonly createFeatureState: "create-new" | "update-pending";
    /**
     * The shared workflow data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html#data)
     */
    readonly data: CreateFeaturesWorkflowData;
    /**
     * Returns the number of pending features of an active [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html#numPendingFeatures)
     */
    readonly numPendingFeatures: number;
    /**
     * Returns a collection of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing features that are currently pending in the [CreateFeaturesWorkflowData](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html#pendingFeatures)
     */
    readonly pendingFeatures: Collection<GraphicsLayer>;

    /**
     * Moves the pending feature into update mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html#updatePendingFeature)
     */
    updatePendingFeature(feature: Graphic): Promise<void>;
  }

  interface CreateFeaturesWorkflowProperties extends WorkflowProperties {}

  export class CreateFeaturesWorkflowData extends Accessor {
    /**
     * This object supports the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html)
     */

    constructor(properties?: CreateFeaturesWorkflowDataProperties);

    /**
     * This provides the feature template and layer when creating a new feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#creationInfo)
     */
    creationInfo: CreationInfo;
    /**
     * Returns a collection of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing features that have been created during the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#pendingFeatures)
     */
    pendingFeatures: Collection<Graphic>;
    /**
     * The associated [EditorViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html) for this workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#viewModel)
     */
    viewModel: EditorViewModel;
  }

  interface CreateFeaturesWorkflowDataProperties {
    /**
     * This provides the feature template and layer when creating a new feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#creationInfo)
     */
    creationInfo?: CreationInfo;
    /**
     * Returns a collection of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing features that have been created during the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#pendingFeatures)
     */
    pendingFeatures?: CollectionProperties<GraphicProperties>;
    /**
     * The associated [EditorViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html) for this workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflowData.html#viewModel)
     */
    viewModel?: EditorViewModelProperties;
  }

  export class EditorViewModel extends Accessor {
    /**
     * Provides the logic for the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html)
     */

    constructor(properties?: EditorViewModelProperties);

    /**
     * A property indicating the current active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#activeWorkflow)
     */
    readonly activeWorkflow: CreateFeaturesWorkflow | UpdateWorkflow;
    /**
     * An array of string values which specifies what end users are allowed to edit.
     *
     * @deprecated since version 4.29. Use {@link module:esri/widgets/Editor#visibleElements Editor.visibleElements} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#allowedWorkflows)
     */
    allowedWorkflows: "create-features" | "update";
    /**
     * The [AttachmentsViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#attachmentsViewModel)
     */
    attachmentsViewModel: AttachmentsViewModel;
    /**
     * Convenience property that indicates at least one layer supports a `create-features` workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#canCreate)
     */
    readonly canCreate: boolean;
    /**
     * Convenience property that indicates at least one layer supports a `update` workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#canUpdate)
     */
    readonly canUpdate: boolean;
    /**
     * A read-only collection of editable item objects that match the feature being updated.
     *
     * @deprecated since 4.29. Use [editorItems](#editorItems) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#editableItems)
     */
    readonly editableItems: Collection<EditableItem>;
    /**
     * A predominantly read-only collection of [editor items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html) that corresponds to the feature being updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#editorItems)
     */
    readonly editorItems: Collection<EditorItem>;
    /**
     * An array of objects containing information specific to any failed editing operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#failures)
     */
    readonly failures: EditorViewModelFailures[];
    /**
     * The [FeatureFormViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#featureFormViewModel)
     */
    featureFormViewModel: FeatureFormViewModel;
    /**
     * The [FeatureTemplatesViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#featureTemplatesViewModel)
     */
    featureTemplatesViewModel: FeatureTemplatesViewModel;
    /**
     * Options to configure the labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#labelOptions)
     */
    labelOptions: SketchLabelOptions;
    /**
     * An array of editing configurations for individual layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#layerInfos)
     */
    layerInfos: EditorLayerInfo[];
    /**
     * The [SketchViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#sketchViewModel)
     */
    sketchViewModel: SketchViewModel;
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The widget's state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#state)
     */
    readonly state:
      | "adding-attachment"
      | "awaiting-feature-creation-info"
      | "awaiting-feature-to-create"
      | "awaiting-feature-to-update"
      | "awaiting-update-feature-candidate"
      | "creating-features"
      | "disabled"
      | "editing-attributes"
      | "editing-attachment"
      | "editing-existing-feature"
      | "editing-new-feature"
      | "awaiting-update-feature-candidate"
      | "ready";
    /**
     * Indicates if there is at least one edit request being processed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#syncing)
     */
    readonly syncing: boolean;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#tooltipOptions)
     */
    tooltipOptions: SketchTooltipOptions;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#valueOptions)
     */
    valueOptions: SketchValueOptions;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Cancels any active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#cancelWorkflow)
     */
    cancelWorkflow(): Promise<void>;
    /**
     * This is applicable if there is an active update workflow with an active child workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#deleteFeatureFromWorkflow)
     */
    deleteFeatureFromWorkflow(): Promise<void>;
    /**
     * Initiates the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) by displaying the panel where feature creation begins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startCreateFeaturesWorkflowAtFeatureCreation)
     */
    startCreateFeaturesWorkflowAtFeatureCreation(creationInfo: CreationInfo): Promise<void>;
    /**
     * This method starts the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) at the "creating-features" step with the provided feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startCreateFeaturesWorkflowAtFeatureEdit)
     */
    startCreateFeaturesWorkflowAtFeatureEdit(
      params: EditorViewModelStartCreateFeaturesWorkflowAtFeatureEditParams,
    ): Promise<void>;
    /**
     * Initiates the [CreateFeaturesWorkflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) by displaying the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startCreateFeaturesWorkflowAtFeatureTypeSelection)
     */
    startCreateFeaturesWorkflowAtFeatureTypeSelection(): Promise<void>;
    /**
     * Starts the update workflow at the feature geometry and attribute editing panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startUpdateWorkflowAtFeatureEdit)
     */
    startUpdateWorkflowAtFeatureEdit(feature: Graphic): Promise<void>;
    /**
     * Starts the `update` workflow using the current selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startUpdateWorkflowAtFeatureSelection)
     */
    startUpdateWorkflowAtFeatureSelection(): Promise<void>;
    /**
     * Starts the Editor workflow where it waits for multiple features to be selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startUpdateWorkflowAtMultipleFeatureSelection)
     */
    startUpdateWorkflowAtMultipleFeatureSelection(): Promise<void>;
    /**
     * Toggles the `UpdateWorkflow`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#toggleUpdateWorkflow)
     */
    toggleUpdateWorkflow(): void;

    on(name: "sketch-create", eventHandler: EditorViewModelSketchCreateEventHandler): IHandle;

    on(name: "sketch-update", eventHandler: EditorViewModelSketchUpdateEventHandler): IHandle;
  }

  interface EditorViewModelProperties {
    /**
     * An array of string values which specifies what end users are allowed to edit.
     *
     * @deprecated since version 4.29. Use {@link module:esri/widgets/Editor#visibleElements Editor.visibleElements} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#allowedWorkflows)
     */
    allowedWorkflows?: "create-features" | "update";
    /**
     * The [AttachmentsViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments-AttachmentsViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#attachmentsViewModel)
     */
    attachmentsViewModel?: AttachmentsViewModelProperties;
    /**
     * The [FeatureFormViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#featureFormViewModel)
     */
    featureFormViewModel?: FeatureFormViewModelProperties;
    /**
     * The [FeatureTemplatesViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#featureTemplatesViewModel)
     */
    featureTemplatesViewModel?: FeatureTemplatesViewModelProperties;
    /**
     * Options to configure the labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#labelOptions)
     */
    labelOptions?: SketchLabelOptionsProperties;
    /**
     * An array of editing configurations for individual layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#layerInfos)
     */
    layerInfos?: EditorLayerInfo[];
    /**
     * The [SketchViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html) for supporting the editor widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#sketchViewModel)
     */
    sketchViewModel?: SketchViewModelProperties;
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#tooltipOptions)
     */
    tooltipOptions?: SketchTooltipOptionsProperties;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#valueOptions)
     */
    valueOptions?: SketchValueOptionsProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The editable item available within the map.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
   */
  export interface EditableItem {
    /**
     * The layer associated with the editable item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    layer: FeatureLayer | SceneLayer | SubtypeSublayer;
    /**
     * The workflow type that this item supports.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    supports: string;
    /**
     * Indicates whether this layer has attachments on it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    hasAttachments: boolean;
    /**
     * Indicates whether this layer enables or disables geometry updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    geometryUpdatesEnabled: boolean;
    /**
     * Indicates whether this layer supports attribute editing while in the `update` workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    attributeUpdatesEnabled: boolean;
    /**
     * Indicates whether this layer supports the ability to work with attachments while creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    attachmentsOnCreateEnabled: boolean;
    /**
     * Indicates whether this layer supports the ability to work with attachments while updating or deleting features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    attachmentsOnUpdateEnabled: boolean;
    /**
     * Indicates whether this layer's [formTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) is invalid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#EditableItem)
     */
    hasInvalidFormTemplate: boolean;
  }

  export interface EditorViewModelFailures {
    /**
     * The read-only error message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#failures)
     */
    error: Error;
  }

  export interface EditorViewModelStartCreateFeaturesWorkflowAtFeatureEditParams {
    /**
     * The initial feature used when you begin creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html#startCreateFeaturesWorkflowAtFeatureEdit)
     */
    initialFeature: Graphic;
  }

  export interface EditorViewModelSketchCreateEvent {
    detail: EditorViewModelSketchCreateEventDetail;

    layer: FeatureLayer | SceneLayer | SubtypeSublayer;

    type: "sketch-create";
  }

  export interface EditorViewModelSketchUpdateEvent {
    detail: EditorViewModelSketchUpdateEventDetail;

    layer: FeatureLayer | SceneLayer | SubtypeSublayer;

    toolEventInfo: UpdateToolEventInfo;

    type: "sketch-update";
  }

  export interface EditorViewModelSketchCreateEventDetail {
    graphic: Graphic;

    state: "start" | "active" | "complete" | "cancel";

    tool: "point" | "multipoint" | "polyline" | "polygon" | "circle" | "rectangle" | "mesh";

    toolEventInfo: CreateToolEventInfo;

    type: "create";
  }

  export interface EditorViewModelSketchUpdateEventDetail {
    aborted: boolean;

    graphics: Graphic[];

    state: "start" | "active" | "complete";

    tool: "move" | "transform" | "reshape";

    type: "update";
  }

  export class Edits extends Accessor {
    /**
     * This class supports the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Edits.html)
     */

    constructor(properties?: EditsProperties);

    /**
     * The feature to be edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Edits.html#feature)
     */
    readonly feature: Graphic;
    /**
     * When true, the feature has been modified from its original state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Edits.html#modified)
     */
    readonly modified: boolean;
  }

  interface EditsProperties {}

  export class EditorItem extends Accessor {
    /**
     * A predominantly read-only editable item that corresponds to the feature being updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html)
     */

    constructor(properties?: EditorItemProperties);

    /**
     * The current [editing capabilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities) for the provided layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#capabilities)
     */
    readonly capabilities: EditorEditingCapabilities;
    /**
     * Indicates whether to override [editable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#editable) to `false`.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether at least one edit operation is supported in [capabilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#capabilities).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#editable)
     */
    readonly editable: boolean;
    /**
     * A reference to the primary [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) in use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#formTemplate)
     */
    readonly formTemplate: FormTemplate;
    /**
     * Indicates whether the primary [FormTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) has an unknown (invalid) field.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#hasInvalidFormTemplate)
     */
    readonly hasInvalidFormTemplate: boolean;
    /**
     * Indicates whether the associated layer is of type `Table`.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#isTable)
     */
    isTable: boolean;
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#layer)
     */
    layer: FeatureLayer | SubtypeSublayer;
    /**
     * The [layerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo) for the provided layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#layerInfo)
     */
    layerInfo: EditorLayerInfo;
    /**
     * Indicates whether the item supports starting a new [create features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-CreateFeaturesWorkflow.html) workflow based on specific conditions set within the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#supportsCreateFeaturesWorkflow)
     */
    readonly supportsCreateFeaturesWorkflow: boolean;
    /**
     * Indicates whether the item supports starting a new [update workflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html) based on specific conditions set within the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#supportsUpdateWorkflow)
     */
    readonly supportsUpdateWorkflow: boolean;
    /**
     * Indicates whether the associated layer and its parent are visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#visible)
     */
    readonly visible: boolean;
  }

  interface EditorItemProperties {
    /**
     * Indicates whether to override [editable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#editable) to `false`.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether the associated layer is of type `Table`.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#isTable)
     */
    isTable?: boolean;
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#layer)
     */
    layer?: (FeatureLayerProperties & { type: "feature" }) | (SubtypeSublayerProperties & { type: "subtype-sublayer" });
    /**
     * The [layerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo) for the provided layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#layerInfo)
     */
    layerInfo?: EditorLayerInfo;
  }

  /**
   * The editing capabilities for attachments.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#AttachmentCapabilities)
   */
  export interface AttachmentCapabilities {
    /**
     * Indicates whether attachments are enabled on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#AttachmentCapabilities)
     */
    enabled: boolean;
  }

  /**
   * Specific permissions for create operations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
   */
  export interface CreateCapabilities {
    /**
     * The editing capabilities for attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
     */
    attachments: AttachmentCapabilities;
    /**
     * Indicates whether attributes can be created on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
     */
    attributes: boolean;
    /**
     * Indicates whether create operations are enabled on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
     */
    enabled: boolean;
    /**
     * Indicates whether a geometry can be created on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
     */
    geometry: boolean;
    /**
     * Indicates when editing is enabled and is *only* due to administrative permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#CreateCapabilities)
     */
    reliesOnOwnerAdminPrivileges: boolean;
  }

  /**
   * The default editing capabilities for the provided layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
   */
  export interface DefaultEditingCapabilities {
    /**
     * The editing capabilities for attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    attachments: AttachmentCapabilities;
    /**
     * The editing capabilities for creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    create: CreateCapabilities;
    /**
     * The editing capabilities for deleting features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    delete: DeleteCapabilities;
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [SubtypeSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-SubtypeSublayer.html) currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    layer: FeatureLayer | SubtypeSublayer;
    /**
     * Indicates when editing is enabled and is *only* due to administrative permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    reliesOnOwnerAdminPrivileges: boolean;
    /**
     * The editing capabilities for updating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DefaultEditingCapabilities)
     */
    update: UpdateCapabilities;
  }

  /**
   * Specific permissions for delete operations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DeleteCapabilities)
   */
  export interface DeleteCapabilities {
    /**
     * Indicates whether delete operations are enabled on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DeleteCapabilities)
     */
    enabled: boolean;
    /**
     * Indicates when editing is enabled and is *only* due to administrative permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#DeleteCapabilities)
     */
    reliesOnOwnerAdminPrivileges: boolean;
  }

  /**
   * The current editing capabilities for the provided layer and read by the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
   */
  export interface EditorEditingCapabilities {
    /**
     * Specific permissions for attachment operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    attachments: AttachmentCapabilities;
    /**
     * Specific permissions for create operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    create: CreateCapabilities;
    /**
     * The default editing capabilities for the provided layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    defaults: DefaultEditingCapabilities;
    /**
     * Specific permissions for delete operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    delete: DeleteCapabilities;
    /**
     * The primary form template in use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    formTemplate: FormTemplate;
    /**
     * The associated layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    layer: FeatureLayer | SubtypeSublayer;
    /**
     * The layer's associated [layerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    layerInfo: EditorLayerInfo;
    /**
     * Specific permissions for relationship operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    relationship: RelationshipCapabilities;
    /**
     * Indicates when editing is enabled *only* due to administrative permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    reliesOnOwnerAdminPrivileges: boolean;
    /**
     * Specific permission`s for update operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#EditorEditingCapabilities)
     */
    update: UpdateCapabilities;
  }

  /**
   * Specific permissions for relationship operations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#RelationshipCapabilities)
   */
  export interface RelationshipCapabilities {
    /**
     * Indicates whether the layer contains at least one relationship, in addition to having child [workflow](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html) permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#RelationshipCapabilities)
     */
    enabled: boolean;
  }

  /**
   * Specific permissions for update operations.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
   */
  export interface UpdateCapabilities {
    /**
     * The editing capabilities for attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
     */
    attachments: AttachmentCapabilities;
    /**
     * Indicates whether attributes can be updated on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
     */
    attributes: boolean;
    /**
     * Indicates whether update operations are enabled on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
     */
    enabled: boolean;
    /**
     * Indicates whether a geometry can be updated on the editable layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
     */
    geometry: boolean;
    /**
     * Indicates when editing is enabled and is *only* due to administrative permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-support-EditorItem.html#UpdateCapabilities)
     */
    reliesOnOwnerAdminPrivileges: boolean;
  }

  export class UpdateWorkflow extends Workflow {
    /**
     * A read-only class containing the logic used when updating and/or deleting features using the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html)
     */

    constructor(properties?: UpdateWorkflowProperties);

    /**
     * The shared workflow data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html#data)
     */
    readonly data: UpdateWorkflowData;
    /**
     * The type of workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflow.html#type)
     */
    readonly type: "update";
  }

  interface UpdateWorkflowProperties extends WorkflowProperties {}

  export class UpdateWorkflowData extends Accessor {
    /**
     * This object supports the [Editor](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflowData.html)
     */

    constructor(properties?: UpdateWorkflowDataProperties);

    /**
     * An array of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflowData.html#candidates)
     */
    candidates: Graphic[];
    /**
     * The associated [EditorViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html) for this workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflowData.html#viewModel)
     */
    viewModel: EditorViewModel;
  }

  interface UpdateWorkflowDataProperties {
    /**
     * An array of features to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflowData.html#candidates)
     */
    candidates?: GraphicProperties[];
    /**
     * The associated [EditorViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-EditorViewModel.html) for this workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-UpdateWorkflowData.html#viewModel)
     */
    viewModel?: EditorViewModelProperties;
  }

  export class Workflow extends Accessor {
    /**
     * The read-only `Workflow` class helps manage different stages of an [editing](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html) workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html)
     */

    constructor(properties?: WorkflowProperties);

    /**
     * The shared workflow data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#data)
     */
    readonly data: CreateFeaturesWorkflowData | UpdateWorkflowData;
    /**
     * This property indicates whether there is a next step in the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#hasNextStep)
     */
    readonly hasNextStep: boolean;
    /**
     * This property indicates if there is a previous step in the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#hasPreviousStep)
     */
    readonly hasPreviousStep: boolean;
    /**
     * Indicates whether the workflow is considered active.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#started)
     */
    readonly started: boolean;
    /**
     * The name of the current step in the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#stepId)
     */
    readonly stepId: string;
    /**
     * Value indicating the workflow type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#type)
     */
    readonly type: "create" | "create-features" | "update";

    /**
     * Cancels the active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#cancel)
     */
    cancel(options?: WorkflowCancelOptions): Promise<void>;
    /**
     * Call this method when the workflow is considered finished.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#commit)
     */
    commit(): Promise<void>;
    /**
     * Moves to the next step in the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#next)
     */
    next(): Promise<void>;
    /**
     * Moves to the previous step in the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#previous)
     */
    previous(options?: WorkflowPreviousOptions): Promise<void>;
    /**
     * Resets the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#reset)
     */
    reset(): Promise<void>;
    /**
     * Starts the workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#start)
     */
    start(): Promise<any>;
  }

  interface WorkflowProperties {}

  export interface WorkflowCancelOptions {
    /**
     * The returned Error object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#cancel)
     */
    error?: Error;
    /**
     * Indicates whether to force cancel the active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#cancel)
     */
    force?: boolean;
    /**
     * Indicates whether to display a prompt indicating canceling the active workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#cancel)
     */
    notify?: boolean;
  }

  export interface WorkflowPreviousOptions {
    /**
     * Cancels the current workflow when calling this method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor-Workflow.html#previous)
     */
    cancelCurrentStep: boolean;
  }

  /**
   * This object provides required and optional properties needed when creating a new feature.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
   */
  export interface CreationInfo {
    /**
     * The associated layer for newly-created feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
     */
    layer: FeatureLayer | OrientedImageryLayer | SceneLayer | SubtypeSublayer;
    /**
     * The associated feature template used to create new features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
     */
    template: FeatureTemplate;
    /**
     * The associated [Mesh](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Mesh.html) geometry for the newly-created feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
     */
    geometryToPlace?: Mesh;
    /**
     * The maximum amount of feature to create in one batch editing operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
     */
    maxFeatures?: number;
    /**
     * The initial feature used when you begin creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#CreationInfo)
     */
    initialFeature?: Graphic;
  }

  export interface EditorStartCreateFeaturesWorkflowAtFeatureEditParams {
    /**
     * The initial feature used when you begin creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#startCreateFeaturesWorkflowAtFeatureEdit)
     */
    initialFeature: Graphic;
  }

  /**
   * Configurations used for setting the layer's editable preferences within the Editor.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
   */
  export interface EditorLayerInfo {
    /**
     * The associated layer containing the fields to edit.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    layer: FeatureLayer | OrientedImageryLayer | SceneLayer | SubtypeSublayer | SubtypeGroupLayer;
    /**
     * Indicates whether to enable editing on the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    enabled?: boolean;
    /**
     * Indicates whether to enable the ability to create new features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    addEnabled?: boolean;
    /**
     * Indicates whether to enable the ability to update existing features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    updateEnabled?: boolean;
    /**
     * Indicates whether to enable the ability to delete features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    deleteEnabled?: boolean;
    /**
     * Indicates whether to enable or disable attribute updates from the feature's form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    attributeUpdatesEnabled?: boolean;
    /**
     * _(Since 4.26)_ Indicates whether to enable or disable attachments while creating features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    attachmentsOnCreateEnabled?: boolean;
    /**
     * _(Since 4.26)_ Indicates whether to enable or disable attachments while updating existing features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    attachmentsOnUpdateEnabled?: boolean;
    /**
     * Indicates whether to enable or disable geometry updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    geometryUpdatesEnabled?: boolean;
    /**
     * A template that defines the feature's form content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#LayerInfo)
     */
    formTemplate?: FormTemplate;
  }

  export interface EditorSketchCreateEvent {
    detail: EditorSketchCreateEventDetail;

    layer: FeatureLayer | SceneLayer | SubtypeSublayer;

    type: "sketch-create";
  }

  export interface EditorSketchUpdateEvent {
    detail: EditorSketchUpdateEventDetail;

    layer: FeatureLayer | SceneLayer | SubtypeSublayer;

    type: "sketch-update";
  }

  /**
   * Set this to customize any supporting Editor widget's default behavior.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
   */
  export interface SupportingWidgetDefaults {
    /**
     * An object containing properties specific for customizing the [Attachments](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    attachments?: SupportingWidgetDefaultsAttachments;
    /**
     * An object containing properties specific for customizing the [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    featureForm?: SupportingWidgetDefaultsFeatureForm;
    /**
     * An object containing properties specific for customizing the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    featureTemplates?: SupportingWidgetDefaultsFeatureTemplates;
    /**
     * An object containing properties specific for customizing the [Sketch](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    sketch?: SupportingWidgetDefaultsSketch;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
   */
  export interface EditorVisibleElements {
    /**
     * _Since 4.28_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    createFeaturesSection?: boolean;
    /**
     * _Since 4.28_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    editFeaturesSection?: boolean;
    /**
     * _Since 4.29_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    labelsToggle?: boolean;
    /**
     * _Since 4.29_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    tooltipsToggle?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    settingsMenu?: boolean;
    /**
     * Indicates whether to display the `SnappingControls` widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    snappingControls?: boolean;
    /**
     * The available [SnappingControls](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html) elements within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    snappingControlsElements?: VisibleElementsSnappingControlsElements;
  }

  export interface EditorSketchCreateEventDetail {
    graphic: Graphic;

    state: "start" | "active" | "complete" | "cancel";

    tool: "point" | "multipoint" | "polyline" | "polygon" | "circle" | "rectangle" | "mesh";

    toolEventInfo: CreateToolEventInfo;

    type: "create";
  }

  export interface EditorSketchUpdateEventDetail {
    aborted: boolean;

    graphics: Graphic[];

    state: "start" | "active" | "complete";

    tool: "move" | "transform" | "reshape";

    toolEventInfo: UpdateToolEventInfo;

    type: "update";
  }

  export interface SupportingWidgetDefaultsAttachments {
    /**
     * String indicating how to [display](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Attachments.html#displayType) the attachments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    displayType?: string;
    /**
     * The capabilities needed for the attachments widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    capabilities?: AttachmentsCapabilities;
  }

  export interface SupportingWidgetDefaultsFeatureForm {
    /**
     * String indicating the [groupDisplay](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#groupDisplay) and how they will be displayed to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    groupDisplay?: string;
  }

  export interface SupportingWidgetDefaultsFeatureTemplates {
    /**
     * Aids in managing various template items and how they display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    groupBy?: string | GroupByFunction;

    visibleElements: SupportingWidgetDefaultsFeatureTemplatesVisibleElements;
    /**
     * (Since 4.28) Indicates whether the list of [feature template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) should scroll within the containing element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    enableListScroll?: boolean;
  }

  export interface SupportingWidgetDefaultsFeatureTemplatesVisibleElements {
    /**
     * Indicates whether the [templates filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#visibleElements) displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    filter?: boolean;
  }

  export interface SupportingWidgetDefaultsSketch {
    /**
     * An object containing the `defaultUpdateOptions` for the [Sketch](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    defaultUpdateOptions?: any;
    /**
     * The marker symbol used to symbolize any point feature updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    markerSymbol?: SimpleMarkerSymbol;
    /**
     * The polygon symbol used to symbolize any polygon feature updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    polygonSymbol?: SimpleFillSymbol;
    /**
     * The line symbol used to symbolize any line feature updates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#SupportingWidgetDefaults)
     */
    polylineSymbol?: SimpleLineSymbol;
  }

  export interface VisibleElementsSnappingControlsElements {
    /**
     * Indicates whether to display the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    header?: boolean;
    /**
     * Indicates whether to display the `enabledToggle` (Enable snapping).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    enabledToggle?: boolean;
    /**
     * Indicates whether to display the `selfEnabledToggle` (Geometry guides).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    selfEnabledToggle?: boolean;
    /**
     * Indicates whether to display the `featureEnabledToggle` (Feature to feature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    featureEnabledToggle?: boolean;
    /**
     * Indicates whether to display the [FeatureSnappingLayerSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html) layerList.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    layerList?: boolean;
    /**
     * Indicates whether to display the “Enable all” or “Disable all” button to enable / disable snapping for all the layers in the list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    layerListToggleLayersButton?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    gridControls?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    gridControlsElements?: VisibleElementsSnappingControlsElementsGridControlsElements;
  }

  export interface VisibleElementsSnappingControlsElementsGridControlsElements {
    /**
     * Indicates whether to display the grid snapping toggle embedded in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    gridSnapEnabledToggle?: boolean;
    /**
     * Indicates whether to display the grid display toggle embedded in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * Indicates whether to display the color/theme selection component of grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    colorSelection?: boolean;
    /**
     * Indicates whether to display the dynamic scaling toggle in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    dynamicScaleToggle?: boolean;
    /**
     * Indicates whether to display the 'rotate with map' toggle in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    rotateWithMapToggle?: boolean;
    /**
     * Indicates whether to display the spacing and rotation inputs in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    numericInputs?: boolean;
    /**
     * Indicates whether to display the major line interval input in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    lineIntervalInput?: boolean;
    /**
     * Indicates whether to display the interactive placement buttons in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    placementButtons?: boolean;
    /**
     * Indicates whether to display a notice when the grid is not visible because it is out of scale and dynamic scaling is turned off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Editor.html#VisibleElements)
     */
    outOfScaleWarning?: boolean;
  }

  export class ElevationProfile extends Widget {
    /**
     * The ElevationProfile widget is used to generate and display an elevation profile from an [input line graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#input).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html)
     */

    constructor(properties?: ElevationProfileProperties);

    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold: number;
    /**
     * The input line graphic along which elevation will be queried in order to generate an elevation profile.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#input)
     */
    input: Graphic;
    /**
     * Collection of elevation profile lines which are to be generated and displayed in the widget's chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#profiles)
     */
    profiles: Collection<
      ElevationProfileLineGround | ElevationProfileLineInput | ElevationProfileLineQuery | ElevationProfileLineView
    >;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#view)
     */
    view: SceneView | MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#viewModel)
     */
    viewModel: ElevationProfileViewModel;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#visibleElements)
     */
    visibleElements: ElevationProfileVisibleElements;
  }

  interface ElevationProfileProperties extends WidgetProperties {
    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold?: number;
    /**
     * The input line graphic along which elevation will be queried in order to generate an elevation profile.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#input)
     */
    input?: GraphicProperties;
    /**
     * Collection of elevation profile lines which are to be generated and displayed in the widget's chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#profiles)
     */
    profiles?: CollectionProperties<
      | (ElevationProfileLineGroundProperties & { type: "ground" })
      | (ElevationProfileLineInputProperties & { type: "input" })
      | (ElevationProfileLineQueryProperties & { type: "query" })
      | (ElevationProfileLineViewProperties & { type: "view" })
    >;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html) or the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#view)
     */
    view?: (SceneViewProperties & { type: "3d" }) | (MapViewProperties & { type: "2d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#viewModel)
     */
    viewModel?: ElevationProfileViewModelProperties;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#visibleElements)
     */
    visibleElements?: ElevationProfileVisibleElements;
  }

  export class ElevationProfileLine extends Accessor {
    /**
     * Common interface for all the elevation profile lines.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html)
     */

    constructor(properties?: ElevationProfileLineProperties);

    /**
     * Color of the line on the chart and the hovered points in the view.
     *
     * @default "#000000"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#color)
     */
    color: Color;
    /**
     * Point being hovered in the chart, in the view's spatial reference.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#hoveredPoint)
     */
    readonly hoveredPoint: Point;
    /**
     * Unique identifier for the profile line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#id)
     */
    id: string;
    /**
     * How far along the generation of this profile is.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#progress)
     */
    readonly progress: number;
    /**
     * List of samples that make up the elevation profile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#samples)
     */
    readonly samples: ElevationProfileSample[];
    /**
     * Statistics about the generated elevation profile, if available.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#statistics)
     */
    readonly statistics: ElevationProfileStatistics;
    /**
     * Title of the line, to be displayed in the chart tooltip and in the chart legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#title)
     */
    title: string;
    /**
     * The line type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#type)
     */
    readonly type: "ground" | "input" | "query" | "view";
    /**
     * Whether a line visualization representing [elevationSamples](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#samples) should be added to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#viewVisualizationEnabled)
     */
    viewVisualizationEnabled: boolean;
    /**
     * Whether the line should be computed and shown in the chart.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#visible)
     */
    visible: boolean;
  }

  interface ElevationProfileLineProperties {
    /**
     * Color of the line on the chart and the hovered points in the view.
     *
     * @default "#000000"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#color)
     */
    color?: Color | number[] | string;
    /**
     * Unique identifier for the profile line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#id)
     */
    id?: string;
    /**
     * Title of the line, to be displayed in the chart tooltip and in the chart legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#title)
     */
    title?: string;
    /**
     * Whether a line visualization representing [elevationSamples](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#samples) should be added to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#viewVisualizationEnabled)
     */
    viewVisualizationEnabled?: boolean;
    /**
     * Whether the line should be computed and shown in the chart.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#visible)
     */
    visible?: boolean;
  }

  /**
   * Represents an elevation sample in the profile.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
   */
  export interface ElevationProfileSample {
    /**
     * The x coordinate of the sample, in the spatial reference of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
     */
    x: number;
    /**
     * The y coordinate of the sample, in the spatial reference of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
     */
    y: number;
    /**
     * The z coordinate of the sample, in the spatial reference of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
     */
    z?: number;
    /**
     * Horizontal (2D) distance from the beginning of the path, in the effective unit selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
     */
    distance: number;
    /**
     * The elevation of the sample, in the effective units selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileSample)
     */
    elevation?: number;
  }

  /**
   * Represents the statistics for the generated profile line.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
   */
  export interface ElevationProfileStatistics {
    /**
     * The maximum horizontal (2D) distance of the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    maxDistance: number;
    /**
     * The minimum elevation along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    minElevation: number;
    /**
     * The maximum elevation along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    maxElevation: number;
    /**
     * The average elevation along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    avgElevation: number;
    /**
     * The cumulative elevation gain along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    elevationGain: number;
    /**
     * The cumulative elevation loss along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    elevationLoss: number;
    /**
     * The maximum slope along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    maxSlope: number;
    /**
     * The average slope along the path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLine.html#ElevationProfileStatistics)
     */
    avgSlope: number;
  }

  export class ElevationProfileLineGround extends ElevationProfileLine {
    /**
     * Profile line which samples elevation from the [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html) of the [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) currently set in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineGround.html)
     */

    constructor(properties?: ElevationProfileLineGroundProperties);

    /**
     * The line type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineGround.html#type)
     */
    readonly type: "ground";
  }

  interface ElevationProfileLineGroundProperties extends ElevationProfileLineProperties {}

  export class ElevationProfileLineInput extends ElevationProfileLine {
    /**
     * Profile line which samples elevation from the geometry of the input [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) itself, typically used on input lines with z values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineInput.html)
     */

    constructor(properties?: ElevationProfileLineInputProperties);

    /**
     * The line type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineInput.html#type)
     */
    readonly type: "input";
  }

  interface ElevationProfileLineInputProperties extends ElevationProfileLineProperties {}

  export class ElevationProfileLineQuery extends ElevationProfileLine {
    /**
     * Profile line which samples elevation from a custom elevation source, for example by creating a new [ElevationLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html), or by using an elevation layer from [ground.layers](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html#layers).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineQuery.html)
     */

    constructor(properties?: ElevationProfileLineQueryProperties);

    /**
     * Elevation source used to sample elevation when generating the profile, for example an [ElevationLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineQuery.html#source)
     */
    source: ElevationProfileLineQuerySource;
    /**
     * The line type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineQuery.html#type)
     */
    readonly type: "query";
  }

  interface ElevationProfileLineQueryProperties extends ElevationProfileLineProperties {
    /**
     * Elevation source used to sample elevation when generating the profile, for example an [ElevationLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ElevationLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineQuery.html#source)
     */
    source?: ElevationProfileLineQuerySource;
  }

  export interface ElevationProfileLineQuerySource {
    /**
     * Function used to query elevation values for a geometry ([Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html), [Multipoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Multipoint.html) or [Polyline](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Polyline.html)).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineQuery.html#source)
     */
    queryElevation: QueryElevation;
  }

  export type QueryElevation = (
    geometry: Point | Multipoint | Polyline,
    options?: any,
  ) => Promise<ElevationLayerElevationQueryResult>;

  export class ElevationProfileLineView extends ElevationProfileLine {
    /**
     * Profile line which samples elevation directly from the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html)
     */

    constructor(properties?: ElevationProfileLineViewProperties);

    /**
     * Items which are to be excluded when querying elevation from view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html#exclude)
     */
    exclude:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
    /**
     * Items which are to be hit when querying elevation from view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html#include)
     */
    include:
      | (IntersectItem | Collection<IntersectItem> | IntersectItem[] | Ground)[]
      | Collection<IntersectItem>
      | IntersectItem;
    /**
     * The line type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html#type)
     */
    readonly type: "view";
  }

  interface ElevationProfileLineViewProperties extends ElevationProfileLineProperties {
    /**
     * Items which are to be excluded when querying elevation from view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html#exclude)
     */
    exclude?:
      | (
          | GraphicProperties
          | LayerProperties
          | BuildingSublayerProperties
          | CollectionProperties<GraphicProperties | LayerProperties | BuildingSublayerProperties>
          | (GraphicProperties | LayerProperties | BuildingSublayerProperties)[]
          | GroundProperties
        )[]
      | CollectionProperties<GraphicProperties | LayerProperties | BuildingSublayerProperties>
      | GraphicProperties
      | LayerProperties
      | BuildingSublayerProperties;
    /**
     * Items which are to be hit when querying elevation from view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileLineView.html#include)
     */
    include?:
      | (
          | GraphicProperties
          | LayerProperties
          | BuildingSublayerProperties
          | CollectionProperties<GraphicProperties | LayerProperties | BuildingSublayerProperties>
          | (GraphicProperties | LayerProperties | BuildingSublayerProperties)[]
          | GroundProperties
        )[]
      | CollectionProperties<GraphicProperties | LayerProperties | BuildingSublayerProperties>
      | GraphicProperties
      | LayerProperties
      | BuildingSublayerProperties;
  }

  export class ElevationProfileViewModel extends Accessor {
    /**
     * Provides the logic for the [ElevationProfile](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html)
     */

    constructor(properties?: ElevationProfileViewModelProperties);

    /**
     * Units which have been selected according to the magnitude of the elevations and distances that are to be displayed in the widget, according to the selected unit or unit system.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#effectiveUnits)
     */
    readonly effectiveUnits: EffectiveUnits;
    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold: number;
    /**
     * Whether the graphic used as [input](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#input) is highlighted.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#highlightEnabled)
     */
    highlightEnabled: boolean;
    /**
     * The position, in the range [0, 1], being hovered in the graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#hoveredChartPosition)
     */
    hoveredChartPosition: number;
    /**
     * The input path along which elevation will be queried in order to generate an elevation profile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#input)
     */
    input: Graphic;
    /**
     * Collection of elevation profile lines which are to be generated and displayed in the widget's chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#profiles)
     */
    profiles: Collection<
      ElevationProfileLineGround | ElevationProfileLineInput | ElevationProfileLineQuery | ElevationProfileLineView
    >;
    /**
     * The progress, between 0 and 1 of generating all the configured elevation profiles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#progress)
     */
    readonly progress: number;
    /**
     * The current state of the view model that can be used for rendering the UI of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "creating" | "created" | "selecting" | "selected";
    /**
     * Whether the chart should use a uniform scale for the X and Y axes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#uniformChartScaling)
     */
    uniformChartScaling: boolean;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#unit)
     */
    unit: SystemOrLengthUnit;
    /**
     * List of available units and unit systems (imperial, metric) for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#unitOptions)
     */
    unitOptions: SystemOrLengthUnit[];
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#view)
     */
    view: SceneView | MapView;

    /**
     * Stops a creation/selection operation and restores the previously configured input path.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#cancel)
     */
    cancel(): void;
    /**
     * Clears the existing profile and stops any interaction.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#clear)
     */
    clear(): void;
    /**
     * If mode is "sketch" (the default), switches to the "creating" state in which the user can draw a new line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#start)
     */
    start(options?: ElevationProfileViewModelStartOptions): void;
    /**
     * Stops a creation/selection operation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#stop)
     */
    stop(): void;
  }

  interface ElevationProfileViewModelProperties {
    /**
     * Threshold (in meters) which determines the method for computing and displaying distances in non-Web Mercator projected coordinate systems (PCS).
     *
     * @default null
     * @deprecated since version 4.29.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#geodesicDistanceThreshold)
     */
    geodesicDistanceThreshold?: number;
    /**
     * Whether the graphic used as [input](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#input) is highlighted.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#highlightEnabled)
     */
    highlightEnabled?: boolean;
    /**
     * The position, in the range [0, 1], being hovered in the graph.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#hoveredChartPosition)
     */
    hoveredChartPosition?: number;
    /**
     * The input path along which elevation will be queried in order to generate an elevation profile.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#input)
     */
    input?: GraphicProperties;
    /**
     * Collection of elevation profile lines which are to be generated and displayed in the widget's chart.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#profiles)
     */
    profiles?: CollectionProperties<
      | (ElevationProfileLineGroundProperties & { type: "ground" })
      | (ElevationProfileLineInputProperties & { type: "input" })
      | (ElevationProfileLineQueryProperties & { type: "query" })
      | (ElevationProfileLineViewProperties & { type: "view" })
    >;
    /**
     * Whether the chart should use a uniform scale for the X and Y axes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#uniformChartScaling)
     */
    uniformChartScaling?: boolean;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#unit)
     */
    unit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * List of available units and unit systems (imperial, metric) for displaying the elevation and distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#unitOptions)
     */
    unitOptions?: (
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet"
    )[];
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#view)
     */
    view?: (SceneViewProperties & { type: "3d" }) | (MapViewProperties & { type: "2d" });
  }

  /**
   * The units which have been selected according to the magnitude of the elevations and distances that are to be displayed in the widget, according to the selected unit or unit system.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#EffectiveUnits)
   */
  export interface EffectiveUnits {
    /**
     * Units used for displaying distance or length values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#EffectiveUnits)
     */
    distance: LengthUnit;
    /**
     * Units used for displaying elevation values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#EffectiveUnits)
     */
    elevation: LengthUnit;
  }

  export interface ElevationProfileViewModelStartOptions {
    /**
     * The mode that the widget will start with.
     *
     * @default "sketch"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile-ElevationProfileViewModel.html#start)
     */
    mode?: "sketch" | "select";
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
   */
  export interface ElevationProfileVisibleElements {
    /**
     * When set to `false`, the legend (which includes statistics) is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    legend?: boolean;
    /**
     * When set to `false`, the chart is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    chart?: boolean;
    /**
     * When set to `false` the button used to clear the current elevation profile is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    clearButton?: boolean;
    /**
     * When set to `false`, the button used to open the settings popup is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    settingsButton?: boolean;
    /**
     * When set to `false`, the button used to start drawing/sketching is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    sketchButton?: boolean;
    /**
     * When set to `false`, the button used to select a path is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    selectButton?: boolean;
    /**
     * When set to `false`, the dropdown used to select the units is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    unitSelector?: boolean;
    /**
     * When set to `false`, the element used to toggle uniform chart scaling on or off is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ElevationProfile.html#VisibleElements)
     */
    uniformChartScalingToggle?: boolean;
  }

  export class Expand extends Widget {
    /**
     * The Expand widget acts as a clickable button for opening a widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html)
     */

    constructor(properties?: ExpandProperties);

    /**
     * Automatically collapses the expand widget instance when the view's viewpoint updates.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#autoCollapse)
     */
    autoCollapse: boolean;
    /**
     * When true, the Expand widget will close after the Escape key is pressed when the keyboard focus is within its content.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#closeOnEsc)
     */
    closeOnEsc: boolean | Function;
    /**
     * Calcite icon used to style the Expand button when the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#content) can be collapsed.
     *
     * @default "chevrons-right"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#collapseIcon)
     */
    collapseIcon: string;
    /**
     * Tooltip to display to indicate Expand widget can be collapsed.
     *
     * @default "Collapse" (English locale)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#collapseTooltip)
     */
    collapseTooltip: string;
    /**
     * The content to display within the expanded Expand widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#content)
     */
    content: Node | string | Widget;
    /**
     * Indicates whether the widget is currently expanded or not.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expanded)
     */
    expanded: boolean;
    /**
     * Calcite icon used when the widget is collapsed.
     *
     * @default "chevrons-left"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expandIcon)
     */
    expandIcon: string;
    /**
     * Tooltip to display to indicate Expand widget can be expanded.
     *
     * @default "Expand" (English locale)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expandTooltip)
     */
    expandTooltip: string;
    /**
     * This value associates two or more Expand widget instances with each other, allowing one instance to auto collapse when another instance in the same group is expanded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#group)
     */
    group: string;
    /**
     * A number to display at the corner of the widget to indicate the number of, for example, open issues or unread notices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#iconNumber)
     */
    iconNumber: number;
    /**
     * The mode in which the widget displays.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#mode)
     */
    mode: "auto" | "floating" | "drawer";
    /**
     * The placement used by the [calcite popover](https://developers.arcgis.com/calcite-design-system/components/popover/) when the widget is expanded.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#placement)
     */
    placement:
      | "auto"
      | "auto-start"
      | "auto-end"
      | "top"
      | "top-start"
      | "top-end"
      | "bottom"
      | "bottom-start"
      | "bottom-end"
      | "right"
      | "right-start"
      | "right-end"
      | "left"
      | "left-start"
      | "left-end"
      | "leading-start"
      | "leading"
      | "leading-end"
      | "trailing-end"
      | "trailing"
      | "trailing-start";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#viewModel)
     */
    viewModel: ExpandViewModel;

    /**
     * Collapse the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#collapse)
     */
    collapse(): void;
    /**
     * Expand the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expand)
     */
    expand(): void;
    /**
     * Toggle the widget by expanding it if it's collapsed, or collapsing it if it's expanded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#toggle)
     */
    toggle(): void;
  }

  interface ExpandProperties extends WidgetProperties {
    /**
     * Automatically collapses the expand widget instance when the view's viewpoint updates.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#autoCollapse)
     */
    autoCollapse?: boolean;
    /**
     * When true, the Expand widget will close after the Escape key is pressed when the keyboard focus is within its content.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#closeOnEsc)
     */
    closeOnEsc?: boolean | Function;
    /**
     * Calcite icon used to style the Expand button when the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#content) can be collapsed.
     *
     * @default "chevrons-right"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#collapseIcon)
     */
    collapseIcon?: string;
    /**
     * Tooltip to display to indicate Expand widget can be collapsed.
     *
     * @default "Collapse" (English locale)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#collapseTooltip)
     */
    collapseTooltip?: string;
    /**
     * The content to display within the expanded Expand widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#content)
     */
    content?: Node | string | WidgetProperties;
    /**
     * Indicates whether the widget is currently expanded or not.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expanded)
     */
    expanded?: boolean;
    /**
     * Calcite icon used when the widget is collapsed.
     *
     * @default "chevrons-left"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expandIcon)
     */
    expandIcon?: string;
    /**
     * Tooltip to display to indicate Expand widget can be expanded.
     *
     * @default "Expand" (English locale)
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#expandTooltip)
     */
    expandTooltip?: string;
    /**
     * This value associates two or more Expand widget instances with each other, allowing one instance to auto collapse when another instance in the same group is expanded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#group)
     */
    group?: string;
    /**
     * A number to display at the corner of the widget to indicate the number of, for example, open issues or unread notices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#iconNumber)
     */
    iconNumber?: number;
    /**
     * The mode in which the widget displays.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#mode)
     */
    mode?: "auto" | "floating" | "drawer";
    /**
     * The placement used by the [calcite popover](https://developers.arcgis.com/calcite-design-system/components/popover/) when the widget is expanded.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#placement)
     */
    placement?:
      | "auto"
      | "auto-start"
      | "auto-end"
      | "top"
      | "top-start"
      | "top-end"
      | "bottom"
      | "bottom-start"
      | "bottom-end"
      | "right"
      | "right-start"
      | "right-end"
      | "left"
      | "left-start"
      | "left-end"
      | "leading-start"
      | "leading"
      | "leading-end"
      | "trailing-end"
      | "trailing"
      | "trailing-start";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html#viewModel)
     */
    viewModel?: ExpandViewModelProperties;
  }

  export class ExpandViewModel extends Accessor {
    /**
     * Provides the logic for the [Expand](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html)
     */

    constructor(properties?: ExpandViewModelProperties);

    /**
     * Automatically collapses the expand widget instance when the view's viewpoint updates.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#autoCollapse)
     */
    autoCollapse: boolean;
    /**
     * Whether the widget is currently expanded or not.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#expanded)
     */
    expanded: boolean;
    /**
     * This value associates two or more Expand widget instances with each other, allowing one instance to auto collapse when another instance in the same group is expanded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#group)
     */
    group: string;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * The view in which the Expand is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface ExpandViewModelProperties {
    /**
     * Automatically collapses the expand widget instance when the view's viewpoint updates.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#autoCollapse)
     */
    autoCollapse?: boolean;
    /**
     * Whether the widget is currently expanded or not.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#expanded)
     */
    expanded?: boolean;
    /**
     * This value associates two or more Expand widget instances with each other, allowing one instance to auto collapse when another instance in the same group is expanded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#group)
     */
    group?: string;
    /**
     * The view in which the Expand is used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Expand-ExpandViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class Feature extends Widget {
    /**
     * The Feature widget displays a graphic according to its [PopupTemplate](esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html)
     */

    constructor(properties?: FeatureProperties);

    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled: boolean;
    /**
     * The [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#graphic)
     */
    graphic: Graphic;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#title) of the feature widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The location of the graphic to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#location)
     */
    location: Point;
    /**
     * A map is required when the input [graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#graphic) has a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#map)
     */
    map: Map;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#timeZone)
     */
    timeZone: string;
    /**
     * The title for the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#title)
     */
    readonly title: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#viewModel)
     */
    viewModel: FeatureViewModel;
    /**
     * The visible elements that are displayed within the widget's [graphic.popupTemplate.content](esri-PopupTemplate.html#content).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#visibleElements)
     */
    visibleElements: FeatureVisibleElements;

    /**
     * Paginates to the next [media](esri-popup-content-MediaContent.html) info.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#nextMedia)
     */
    nextMedia(contentElementIndex: number): void;
    /**
     * Paginates to the previous [media](esri-popup-content-MediaContent.html) info in the specified [media](esri-popup-content-MediaContent.html) content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#previousMedia)
     */
    previousMedia(contentElementIndex: number): void;
    /**
     * Paginates to a specified [media](esri-popup-content-MediaContent.html) info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#setActiveMedia)
     */
    setActiveMedia(contentElementIndex: number, mediaInfoIndex: number): void;
  }

  interface FeatureProperties extends WidgetProperties {
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled?: boolean;
    /**
     * The [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#title) of the feature widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The location of the graphic to be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#location)
     */
    location?: PointProperties;
    /**
     * A map is required when the input [graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#graphic) has a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#map)
     */
    map?: MapProperties;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#timeZone)
     */
    timeZone?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#viewModel)
     */
    viewModel?: FeatureViewModelProperties;
    /**
     * The visible elements that are displayed within the widget's [graphic.popupTemplate.content](esri-PopupTemplate.html#content).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#visibleElements)
     */
    visibleElements?: FeatureVisibleElements;
  }

  export class FeatureViewModel extends Accessor {
    /**
     * Provides the logic for the [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html)
     */

    constructor(properties?: FeatureViewModelProperties);

    /**
     * Defines the specific [abilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities) that the [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets should use when querying and displaying its content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#abilities)
     */
    abilities: Abilities;
    /**
     * The [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#content) of the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#content)
     */
    readonly content: Content[] | Widget | HTMLElement | string;
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled: boolean;
    /**
     * The formatted attributes calculated from `fieldInfo` [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) content.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#formattedAttributes)
     */
    readonly formattedAttributes: FeatureViewModelFormattedAttributes;
    /**
     * The [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#graphic)
     */
    graphic: Graphic;
    /**
     * A read-only property containing metadata regarding the last edit performed on a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#lastEditInfo)
     */
    readonly lastEditInfo: FeatureViewModelLastEditInfo;
    /**
     * The [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) interaction used to trigger the opening of the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#location)
     */
    location: Point;
    /**
     * A map is required when the input [graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#graphic) has a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#map)
     */
    map: Map;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#state)
     */
    readonly state: "ready" | "disabled" | "error" | "loading";
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#timeZone)
     */
    timeZone: string;
    /**
     * The title for the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#title)
     */
    readonly title: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#view)
     */
    view: MapView | SceneView;
    /**
     * Indicates whether the feature is currently waiting for all of its content to finish loading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#waitingForContent)
     */
    readonly waitingForContent: boolean;

    /**
     * Paginates to the next [media](esri-popup-content-MediaContent.html) info.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#nextMedia)
     */
    nextMedia(contentElementIndex: number): void;
    /**
     * Paginates to the previous [media](esri-popup-content-MediaContent.html) info in the specified [media](esri-popup-content-MediaContent.html) content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#previousMedia)
     */
    previousMedia(contentElementIndex: number): void;
    /**
     * Paginates to a specified [media](esri-popup-content-MediaContent.html) info object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#setActiveMedia)
     */
    setActiveMedia(contentElementIndex: number, mediaInfoIndex: number): void;
  }

  interface FeatureViewModelProperties {
    /**
     * Defines the specific [abilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities) that the [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets should use when querying and displaying its content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#abilities)
     */
    abilities?: Abilities;
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled?: boolean;
    /**
     * The [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used to represent the feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * The [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) interaction used to trigger the opening of the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#location)
     */
    location?: PointProperties;
    /**
     * A map is required when the input [graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#graphic) has a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#map)
     */
    map?: MapProperties;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#timeZone)
     */
    timeZone?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * This object defines specific abilities for the [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) and [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widgets.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
   */
  export interface Abilities {
    /**
     * Indicate whether the widget should query and display [attachments](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html) associated with the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    attachmentsContent?: boolean;
    /**
     * Indicate whether the widget should display animations for charts residing in [media content elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    chartAnimation?: boolean;
    /**
     * Indicate whether the widget should display any [custom content elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-CustomContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    customContent?: boolean;
    /**
     * Indicate whether the widget should display any [fields content elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    fieldsContent?: boolean;
    /**
     * Indicate whether the widget should display any [media content elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    mediaContent?: boolean;
    /**
     * Indicate whether the widget should display any [text content elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities)
     */
    textContent?: boolean;
  }

  export interface FeatureViewModelFormattedAttributes {
    /**
     * The formatted attributes specified at the root level of the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) using the [PopupTemplate.fieldInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html#fieldInfos) property.
     *
     * [Read more...](global.html)
     */
    global?: any;
    /**
     * The formatted attributes for an individual content element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#formattedAttributes)
     */
    content?: any;
  }

  export interface FeatureViewModelLastEditInfo {
    /**
     * User who performed the last edit on a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#lastEditInfo)
     */
    user: string;
    /**
     * Date that the edit was performed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#lastEditInfo)
     */
    date: string;
    /**
     * The type of edit that was performed on the feature.
     *
     * [Read more...](global.html#type)
     */
    type: "edit" | "create";
  }

  export interface VisibleContentElements {
    /**
     * Indicates whether to display any [AttachmentsContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-AttachmentsContent.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements)
     */
    attachments?: boolean;
    /**
     * Indicates whether to display any [FieldsContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-FieldsContent.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements)
     */
    fields?: boolean;
    /**
     * Indicates whether to display any [MediaContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-MediaContent.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements)
     */
    media?: boolean;
    /**
     * Indicates whether to display any [TextContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-TextContent.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements)
     */
    text?: boolean;
    /**
     * Indicates whether to display any [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements)
     */
    expression?: boolean;
  }

  export interface FeatureVisibleElements {
    /**
     * Indicates whether the title associated with the feature displays.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleElements)
     */
    title?: boolean;
    /**
     * Indicates whether content for the Feature displays, can also indicate the specific types of content elements by setting it via [VisibleContentElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleContentElements).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleElements)
     */
    content?: boolean | VisibleContentElements;
    /**
     * Indicates whether [lastEditInfo](esri-widgets-Feature-FeatureViewModel.html#lastEditInfo) is displayed within the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html#VisibleElements)
     */
    lastEditedInfo?: boolean;
  }

  export class FeatureForm extends Widget {
    /**
     * The FeatureForm widget displays attributes of a feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html)
     */

    constructor(properties?: FeatureFormProperties);

    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#disabled)
     */
    disabled: boolean;
    /**
     * The associated feature containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#feature)
     */
    feature: Graphic;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used for the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#formTemplate)
     */
    formTemplate: FormTemplate;
    /**
     * Defines how groups will be displayed to the user.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#groupDisplay)
     */
    groupDisplay: "all" | "sequential";
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#title) of the form.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Layer containing the editable feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#layer)
     */
    layer: FeatureLayer | SubtypeSublayer;
    /**
     * A reference to the associated [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#map)
     */
    map: Map;
    /**
     * The timezone displayed within the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#timeZone)
     */
    timeZone: string;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#viewModel)
     */
    viewModel: FeatureFormViewModel;

    /**
     * Returns all of the field values, regardless of update status.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#getValues)
     */
    getValues(): any;
    /**
     * Fires the [submit](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#event-submit) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#submit)
     */
    submit(): void;

    on(name: "value-change", eventHandler: FeatureFormValueChangeEventHandler): IHandle;

    on(name: "submit", eventHandler: FeatureFormSubmitEventHandler): IHandle;
  }

  interface FeatureFormProperties extends WidgetProperties {
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#disabled)
     */
    disabled?: boolean;
    /**
     * The associated feature containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#feature)
     */
    feature?: GraphicProperties;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used for the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#formTemplate)
     */
    formTemplate?: FormTemplateProperties;
    /**
     * Defines how groups will be displayed to the user.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#groupDisplay)
     */
    groupDisplay?: "all" | "sequential";
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#title) of the form.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Layer containing the editable feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#layer)
     */
    layer?: (FeatureLayerProperties & { type: "feature" }) | (SubtypeSublayerProperties & { type: "subtype-sublayer" });
    /**
     * A reference to the associated [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#map)
     */
    map?: MapProperties;
    /**
     * The timezone displayed within the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#timeZone)
     */
    timeZone?: string;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html#viewModel)
     */
    viewModel?: FeatureFormViewModelProperties;
  }

  export class EditableInput {}

  export interface FeatureFormViewModel extends Accessor, Evented {}

  export class FeatureFormViewModel {
    /**
     * Provides the logic for the [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html)
     */

    constructor(properties?: FeatureFormViewModelProperties);

    /**
     * The associated feature containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#feature)
     */
    feature: Graphic;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used for the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#formTemplate)
     */
    formTemplate: FormTemplate;
    /**
     * The field, group, or relationship inputs that make up the form [FeatureForm](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#inputs)
     */
    readonly inputs: (FieldInput | GroupInput | RelationshipInput)[];
    /**
     * Layer containing the editable feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#layer)
     */
    layer: FeatureLayer | SceneLayer | SubtypeSublayer;
    /**
     * A reference to the associated [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#map)
     */
    map: Map;
    /**
     * The widget's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * Indicates if the field's value can be submitted without introducing data validation issues.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#submittable)
     */
    submittable: boolean;
    /**
     * The timezone displayed within the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#timeZone)
     */
    timeZone: string;
    /**
     * Indicates whether the form is currently updating.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#updating)
     */
    readonly updating: boolean;
    /**
     * Indicates whether all of the input fields are valid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#valid)
     */
    readonly valid: boolean;

    /**
     * Convenience method to find field inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#findField)
     */
    findField(fieldName: string): FieldInput;
    /**
     * Returns all of the field values, regardless of whether or not they were updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#getValues)
     */
    getValues(): any;
    /**
     * The method used to set the updated field value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#setValue)
     */
    setValue(fieldName: string, value: number | string): void;
    /**
     * Fires the [submit](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#event-submit) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#submit)
     */
    submit(): void;
    /**
     * Validates whether a feature's attribute values conform to the defined contingent values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#validateContingencyConstraints)
     */
    validateContingencyConstraints(
      values: string,
      options?: FeatureFormViewModelValidateContingencyConstraintsOptions,
    ): any[];

    on(name: "value-change", eventHandler: FeatureFormViewModelValueChangeEventHandler): IHandle;

    on(name: "submit", eventHandler: FeatureFormViewModelSubmitEventHandler): IHandle;
  }

  interface FeatureFormViewModelProperties {
    /**
     * The associated feature containing the editable attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#feature)
     */
    feature?: GraphicProperties;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-form-FormTemplate.html) used for the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#formTemplate)
     */
    formTemplate?: FormTemplateProperties;
    /**
     * Layer containing the editable feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#layer)
     */
    layer?:
      | (FeatureLayerProperties & { type: "feature" })
      | (SceneLayerProperties & { type: "scene" })
      | (SubtypeSublayerProperties & { type: "subtype-sublayer" });
    /**
     * A reference to the associated [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#map)
     */
    map?: MapProperties;
    /**
     * Indicates if the field's value can be submitted without introducing data validation issues.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#submittable)
     */
    submittable?: boolean;
    /**
     * The timezone displayed within the form.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#timeZone)
     */
    timeZone?: string;
  }

  export interface FeatureFormViewModelValidateContingencyConstraintsOptions {
    /**
     * If `true`, return contingency violations for field groups that are invalid because values have not yet been specified for all their fields.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FeatureFormViewModel.html#validateContingencyConstraints)
     */
    includeIncompleteViolations?: boolean;
  }

  export interface FeatureFormViewModelSubmitEvent {
    invalid: string[];

    valid: string[];

    values: any;
  }

  export interface FeatureFormViewModelValueChangeEvent {
    feature: Graphic;

    fieldName: string;

    layer: FeatureLayer;

    valid: boolean;

    value: number | string;
  }

  export interface FieldInput extends Accessor, EditableInput, InputBase {}

  export class FieldInput {
    /**
     * This is a read-only support class that represents a field's input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html)
     */

    constructor(properties?: FieldInputProperties);

    /**
     * The type of data displayed by the field input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#dataType)
     */
    readonly dataType: "number" | "text" | "date" | "unsupported";
    /**
     * The input value's domain.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#domain)
     */
    readonly domain: CodedValueDomain | RangeDomain;
    /**
     * Indicates if the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#editable)
     */
    readonly editable: boolean;
    /**
     * If the input field's value is invalid, this property returns a validation error code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#error)
     */
    readonly error: string;
    /**
     * The associated field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#field)
     */
    readonly field: Field;
    /**
     * The group containing the field input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#group)
     */
    readonly group: GroupInput;
    /**
     * A hint for the field's value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#hint)
     */
    readonly hint: string;
    /**
     * Indicates whether date information is included for date inputs.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#includeDate)
     */
    readonly includeDate: boolean;
    /**
     * Indicates whether time information is included for date inputs.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#includeTime)
     */
    readonly includeTime: string;
    /**
     * Indicates whether timestamp information is included for date inputs.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#includeTimeOffset)
     */
    readonly includeTimeOffset: boolean;
    /**
     * The type of editor used when working with `string` fields.
     *
     * @default "text-box"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#inputType)
     */
    readonly inputType: "text-box" | "text-area";
    /**
     * The field's label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#label)
     */
    readonly label: string;
    /**
     * Restricts the input length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#maxLength)
     */
    readonly maxLength: number;
    /**
     * Restricts the input length.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#minLength)
     */
    readonly minLength: number;
    /**
     * The associated field name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#name)
     */
    readonly name: string;
    /**
     * Indicates whether the field is required.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#required)
     */
    readonly required: boolean;
    /**
     * Indicates if the field's value can be submitted without introducing data validation issues.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#submittable)
     */
    readonly submittable: boolean;
    /**
     * The type of input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#type)
     */
    readonly type: "field";
    /**
     * Indicates if the field is updating.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#updating)
     */
    readonly updating: boolean;
    /**
     * Indicates if the input value is valid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#valid)
     */
    readonly valid: boolean;
    /**
     * The field input's value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#value)
     */
    readonly value: number | string;
    /**
     * Indicates if the field is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-FieldInput.html#visible)
     */
    readonly visible: boolean;
  }

  interface FieldInputProperties extends InputBaseProperties {}

  export interface GroupInput extends Accessor, InputBase {}

  export class GroupInput {
    /**
     * This is a support class that represents a group of field inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html)
     */

    constructor(properties?: GroupInputProperties);

    /**
     * Defines if the group should be expanded or collapsed when the form is initially displayed.
     *
     * @default "expanded"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#initialState)
     */
    readonly initialState: "expanded" | "collapsed";
    /**
     * The field inputs contained within the group.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#inputs)
     */
    readonly inputs: FieldInput[] | RelationshipInput[];
    /**
     * The group's label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#label)
     */
    readonly label: string;
    /**
     * Indicates whether or not the group is open, ie.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#open)
     */
    open: boolean;
    /**
     * The type of input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#type)
     */
    readonly type: "group";
    /**
     * The group's visibility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#visible)
     */
    readonly visible: boolean;
  }

  interface GroupInputProperties extends InputBaseProperties {
    /**
     * Indicates whether or not the group is open, ie.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-GroupInput.html#open)
     */
    open?: boolean;
  }

  export class InputBase {
    /**
     * The input's description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-InputBase.html#description)
     */
    readonly description: string;
  }

  interface InputBaseProperties {}

  export interface RelationshipInput extends Accessor, EditableInput, InputBase {}

  export class RelationshipInput {
    /**
     * This is a read-only support class that represents a relationship input field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html)
     */

    constructor(properties?: RelationshipInputProperties);

    /**
     * A numeric value indicating the maximum number of related features to display in the list of related records.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#displayCount)
     */
    readonly displayCount: number;
    /**
     * A string value indicating how to display related records within the relationship content.
     *
     * @default "list"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#displayType)
     */
    readonly displayType: "list";
    /**
     * Indicates whether the input is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#editable)
     */
    readonly editable: boolean;
    /**
     * The group containing the relationship input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#group)
     */
    readonly group: GroupInput;
    /**
     * An array of [RelatedRecordsInfoFieldOrder](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-support-RelatedRecordsInfoFieldOrder.html) objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#orderByFields)
     */
    readonly orderByFields: RelatedRecordsInfoFieldOrder[];
    /**
     * The type of input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#type)
     */
    readonly type: "relationship";
    /**
     * Indicates whether the form is currently updating.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureForm-RelationshipInput.html#updating)
     */
    readonly updating: boolean;
  }

  interface RelationshipInputProperties extends InputBaseProperties {}

  export interface FeatureFormSubmitEvent {
    invalid: string[];

    valid: string[];

    values: any;
  }

  export interface FeatureFormValueChangeEvent {
    feature: Graphic;

    fieldName: string;

    layer: FeatureLayer;

    valid: boolean;

    value: number | string;
  }

  export interface Features extends Widget, GoTo, Evented {}

  export class Features {
    /**
     * The Features widget allows users to view a feature's popupTemplate content such as attributes, actions, related records, etc., without having to be tied to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html)
     */

    constructor(properties?: FeaturesProperties);

    /**
     * Indicates if the widget is active when it is visible and is not [waiting for results](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#waitingForResult).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#active)
     */
    readonly active: boolean;
    /**
     * Indicates whether the popup displays its content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#collapsed)
     */
    readonly collapsed: boolean;
    /**
     * The content of the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#content)
     */
    content: string | HTMLElement | Widget;
    /**
     * An array of features associated with the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features)
     */
    features: Graphic[];
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title) of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A map is required when the input [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features) have a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#map)
     */
    map: Map;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#promises)
     */
    promises: Promise<any>[];
    /**
     * The selected feature accessed by the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeature)
     */
    readonly selectedFeature: Graphic;
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeatureIndex)
     */
    selectedFeatureIndex: number;
    /**
     * Returns a reference to the current [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeatureWidget)
     */
    readonly selectedFeatureWidget: Feature;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#timeZone)
     */
    timeZone: string;
    /**
     * The title of the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title)
     */
    title: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#view)
     */
    view: MapView | SceneView;
    /**
     * This is a class that contains all the logic (properties and methods) that controls this widget's behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#viewModel)
     */
    viewModel: FeaturesViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#visibleElements)
     */
    visibleElements: FeaturesVisibleElements;

    /**
     * Use this method to remove focus from the Widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#blur)
     */
    blur(): void;
    /**
     * Removes [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#promises), [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features), [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#content), and [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title) from the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#clear)
     */
    clear(): void;
    /**
     * This is a convenience method to closes the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#close)
     */
    close(): void;
    /**
     * Use this method to return feature(s) at a given screen location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#fetchFeatures)
     */
    fetchFeatures(
      screenPoint: FeaturesFetchFeaturesScreenPoint,
      options?: FetchFeaturesOptions,
    ): Promise<FetchPopupFeaturesResult>;
    /**
     * Use this method to give focus to the Widget if the widget is able to be focused.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#focus)
     */
    focus(): void;
    /**
     * Selects the feature at the next index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#next)
     */
    next(): FeaturesViewModel;
    /**
     * Opens the Features widget in its [container](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#container) with content defined either explicitly with `content` or driven from the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) of input features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    open(options?: FeaturesOpenOptions): void;
    /**
     * Selects the feature at the previous index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#previous)
     */
    previous(): FeaturesViewModel;
    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#event-trigger-action) event and executes the [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions) at the specified index in the [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions) array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#triggerAction)
     */
    triggerAction(actionIndex: number): void;

    on(name: "trigger-action", eventHandler: FeaturesTriggerActionEventHandler): IHandle;
  }

  interface FeaturesProperties extends WidgetProperties, GoToProperties {
    /**
     * The content of the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#content)
     */
    content?: string | HTMLElement | WidgetProperties;
    /**
     * An array of features associated with the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features)
     */
    features?: GraphicProperties[];
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title) of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A map is required when the input [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features) have a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#map)
     */
    map?: MapProperties;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#promises)
     */
    promises?: Promise<any>[];
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeatureIndex)
     */
    selectedFeatureIndex?: number;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#timeZone)
     */
    timeZone?: string;
    /**
     * The title of the Features widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title)
     */
    title?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * This is a class that contains all the logic (properties and methods) that controls this widget's behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#viewModel)
     */
    viewModel?: FeaturesViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#visibleElements)
     */
    visibleElements?: FeaturesVisibleElements;
  }

  export interface FeaturesViewModel extends Accessor, Evented, GoTo {}

  export class FeaturesViewModel {
    /**
     * Provides the logic for the [Features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html)
     */

    constructor(properties?: FeaturesViewModelProperties);

    /**
     * [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions)
     */
    actions: Collection<ActionButton | ActionToggle>;
    /**
     * Indicates if the view model is active when it is visible and is not [waiting for results](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#waitingForResult).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#active)
     */
    readonly active: boolean;
    /**
     * A collection of [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggles](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#allActions)
     */
    readonly allActions: Collection<ActionButton | ActionToggle>;
    /**
     * This closes the container when the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) camera or [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#autoCloseEnabled)
     */
    autoCloseEnabled: boolean;
    /**
     * Indicates if the "Browse features" experience is active in a [cluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) popup.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#browseClusterEnabled)
     */
    browseClusterEnabled: boolean;
    /**
     * The information to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#content)
     */
    content: string | HTMLElement | Widget;
    /**
     * A read-only property that specifies a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of action [buttons](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) and/or [toggles](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#defaultActions)
     */
    readonly defaultActions: Collection<ActionButton | ActionToggle>;
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled: boolean;
    /**
     * The number of selected [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features) available.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featureCount)
     */
    readonly featureCount: number;
    /**
     * An array of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features)
     */
    features: Graphic[];
    /**
     * The number of features to fetch at one time.
     *
     * @default 20
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featuresPerPage)
     */
    featuresPerPage: number;
    /**
     * Defines the specific [abilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities) that can be used when querying and displaying content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featureViewModelAbilities)
     */
    featureViewModelAbilities: Abilities;
    /**
     * An array of [feature view model(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featureViewModels)
     */
    readonly featureViewModels: FeatureViewModel[];
    /**
     * Highlight the selected feature using the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#highlightOptions) set on the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions) set on the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#highlightEnabled)
     */
    highlightEnabled: boolean;
    /**
     * Indicates whether or not to include [defaultActions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#defaultActions).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#includeDefaultActions)
     */
    includeDefaultActions: boolean;
    /**
     * Geometry used to show the location of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#location)
     */
    location: Point;
    /**
     * A map is required when the input [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features) have a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#map)
     */
    map: Map;
    /**
     * The number of [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises) remaining to be resolved.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#pendingPromisesCount)
     */
    readonly pendingPromisesCount: number;
    /**
     * The number of selected [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises) available.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promiseCount)
     */
    readonly promiseCount: number;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises)
     */
    promises: Promise<any>[];
    /**
     * The graphic used to represent the cluster extent when the `Browse features` action is active.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedClusterBoundaryFeature)
     */
    readonly selectedClusterBoundaryFeature: Graphic;
    /**
     * The selected feature accessed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeature)
     */
    readonly selectedFeature: Graphic;
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeatureIndex)
     */
    selectedFeatureIndex: number;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#spatialReference)
     */
    spatialReference: SpatialReference;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * Dates and times will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#timeZone)
     */
    timeZone: string;
    /**
     * The title of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#title)
     */
    title: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#view)
     */
    view: MapView | SceneView;
    /**
     * Indicates whether the widget is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#visible)
     */
    visible: boolean;
    /**
     * Indicates whether a feature was found while resolving [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#waitingForResult)
     */
    readonly waitingForResult: boolean;

    /**
     * Removes all [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises), [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features), [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#content), and [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#title).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#clear)
     */
    clear(): void;
    /**
     * Use this method to return feature(s) at a given screen location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#fetchFeatures)
     */
    fetchFeatures(
      screenPoint: FeaturesViewModelFetchFeaturesScreenPoint,
      options?: PopupFetchFeaturesOptions,
    ): Promise<FetchPopupFeaturesResult>;
    /**
     * Selects the feature at the next index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#next)
     */
    next(): FeaturesViewModel;
    /**
     * Opens the widget at the given location with content defined either explicitly with `content` or driven from the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) of input features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    open(options?: FeaturesViewModelOpenOptions): void;
    /**
     * Selects the feature at the previous index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#previous)
     */
    previous(): FeaturesViewModel;
    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#event-trigger-action) event and executes the [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions) at the specified index in the [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions) array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#triggerAction)
     */
    triggerAction(actionIndex: number): void;

    on(name: "trigger-action", eventHandler: FeaturesViewModelTriggerActionEventHandler): IHandle;
  }

  interface FeaturesViewModelProperties extends GoToProperties {
    /**
     * [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions)
     */
    actions?: CollectionProperties<
      (ActionButtonProperties & { type: "button" }) | (ActionToggleProperties & { type: "toggle" })
    >;
    /**
     * This closes the container when the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) camera or [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#autoCloseEnabled)
     */
    autoCloseEnabled?: boolean;
    /**
     * Indicates if the "Browse features" experience is active in a [cluster](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureReductionCluster.html) popup.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#browseClusterEnabled)
     */
    browseClusterEnabled?: boolean;
    /**
     * The information to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#content)
     */
    content?: string | HTMLElement | WidgetProperties;
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled?: boolean;
    /**
     * An array of features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features)
     */
    features?: GraphicProperties[];
    /**
     * The number of features to fetch at one time.
     *
     * @default 20
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featuresPerPage)
     */
    featuresPerPage?: number;
    /**
     * Defines the specific [abilities](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature-FeatureViewModel.html#Abilities) that can be used when querying and displaying content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#featureViewModelAbilities)
     */
    featureViewModelAbilities?: Abilities;
    /**
     * Highlight the selected feature using the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#highlightOptions) set on the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions) set on the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#highlightEnabled)
     */
    highlightEnabled?: boolean;
    /**
     * Indicates whether or not to include [defaultActions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#defaultActions).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#includeDefaultActions)
     */
    includeDefaultActions?: boolean;
    /**
     * Geometry used to show the location of the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#location)
     */
    location?: PointProperties;
    /**
     * A map is required when the input [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features) have a popupTemplate that contains [Arcade](https://developers.arcgis.com/arcade) expressions in [ExpressionInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-ExpressionInfo.html) or [ExpressionContent](https://developers.arcgis.com/javascript/latest/api-reference/esri-popup-content-ExpressionContent.html) that may use the `$map` profile variable to access data from layers within a map.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#map)
     */
    map?: MapProperties;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises)
     */
    promises?: Promise<any>[];
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeatureIndex)
     */
    selectedFeatureIndex?: number;
    /**
     * The spatial reference used for [Arcade](https://developers.arcgis.com/arcade) operations.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#spatialReference)
     */
    spatialReference?: SpatialReferenceProperties;
    /**
     * Dates and times will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#timeZone)
     */
    timeZone?: string;
    /**
     * The title of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#title)
     */
    title?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * Indicates whether the widget is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#visible)
     */
    visible?: boolean;
  }

  export interface FeaturesViewModelFetchFeaturesScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#fetchFeatures)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#fetchFeatures)
     */
    y: number;
  }

  export interface FeaturesViewModelOpenOptions {
    /**
     * Sets the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#title) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    title?: string;
    /**
     * Sets the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#content) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    content?: string;
    /**
     * **Since:** 4.12  When `true`, indicates the feature's content should be fetched and displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    fetchFeatures?: boolean;
    /**
     * Sets the [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#location), which is the geometry used to position the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    location?: Point;
    /**
     * Sets the [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#features), which populates the title and content based on each graphic's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    features?: Graphic[];
    /**
     * Sets pending [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#promises).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    promises?: Promise<any>[];
    /**
     * When `true`, indicates the [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#location) property will be updated for each paginated feature based on the [selected feature's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#selectedFeature) geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#open)
     */
    updateLocationEnabled?: boolean;
  }

  export interface FeaturesViewModelTriggerActionEvent {
    action: ActionButton | ActionToggle;
  }

  export interface FeaturesFetchFeaturesScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#fetchFeatures)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#fetchFeatures)
     */
    y: number;
  }

  export interface FeaturesOpenOptions {
    /**
     * This property allows you to specify a collection of [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) to the action bar when the Features widget opens.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    actions?: Collection<ActionButton>;
    /**
     * Indicates whether the widget displays its content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    collapsed?: boolean;
    /**
     * Sets the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#content) of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    content?: string | HTMLElement | Widget;
    /**
     * Sets the widget's [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#features), which populate the title and content of the widget based on each graphic's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    features?: Graphic[];
    /**
     * This property enables multiple features in the widget to display in a list rather than displaying the first selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    featureMenuOpen?: boolean;
    /**
     * When `true`, indicates the widget should fetch the content of this feature and display it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    fetchFeatures?: boolean;
    /**
     * The geometry used to query for the features when `fetchFeatures=true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    location?: Point;
    /**
     * Sets pending [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#promises) on the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    promises?: Promise<any>[];
    /**
     * Sets the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#title) of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    title?: string;
    /**
     * Indicates whether to update the location when the [selectedFeatureIndex](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#selectedFeatureIndex) changes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#open)
     */
    updateLocationEnabled?: string;
  }

  /**
   * Optional properties to use with the [fetchFeatures](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#fetchFeatures) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#FetchFeaturesOptions)
   */
  export interface FetchFeaturesOptions {
    /**
     * The `click` event for either the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#FetchFeaturesOptions)
     */
    event?: any;
    /**
     * The signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#FetchFeaturesOptions)
     */
    signal?: AbortSignal;
  }

  export interface FeaturesTriggerActionEvent {
    action: ActionButton;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
   */
  export interface FeaturesVisibleElements {
    /**
     * Indicates whether to display the action bar that holds the feature's [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features-FeaturesViewModel.html#actions) will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    actionBar?: boolean;
    /**
     * Indicates whether to display a close button on the widget dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * **Since 4.28.** Indicates whether to display the collapse button on the widget dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * Indicates whether pagination for feature navigation will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    featureNavigation?: boolean;
    /**
     * **Since 4.30.** Indicates whether to display the group heading for a list of multiple features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    featureListLayerTitle?: boolean;
    /**
     * **Since 4.29.** Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * Indicates whether to display the widget's loading spinner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Features.html#VisibleElements)
     */
    spinner?: boolean;
  }

  export class FeatureTable extends Widget {
    /**
     * The FeatureTable's functionality is described in the following sections:
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html)
     */

    constructor(properties?: FeatureTableProperties);

    /**
     * Configuration for the [ActionColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#actionColumnConfig)
     */
    actionColumnConfig: ActionColumnConfig;
    /**
     * A read-only property indicating the type of filter used by the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#activeFilters)
     */
    readonly activeFilters: Collection<GeometryFilter | SelectionFilter>;
    /**
     * Use this read-only property if needing to query features while retaining a column's sort order.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#activeSortOrders)
     */
    readonly activeSortOrders: ColumnSortOrder[];
    /**
     * Indicates the table is displaying all related tables in `show all` mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#allRelatedTablesVisible)
     */
    readonly allRelatedTablesVisible: boolean;
    /**
     * Indicates whether to display the `Attachments` field in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#attachmentsEnabled)
     */
    attachmentsEnabled: boolean;
    /**
     * Use this property to configure how columns display within the table in regard to visibility, column order, and sorting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Indicates whether the table should automatically refresh when the underlying data changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#autoRefreshEnabled)
     */
    autoRefreshEnabled: boolean;
    /**
     * Indicates whether the table should allow reordering of columns.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#columnReorderingEnabled)
     */
    columnReorderingEnabled: boolean;
    /**
     * A read-only collection of [column](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html), [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html), [group](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-GroupColumn.html), [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html), [attachment](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html), and [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html) columns that are displayed within the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#columns)
     */
    readonly columns: Collection<
      Column | FieldColumn | GroupColumn | ActionColumn | AttachmentsColumn | RelationshipColumn
    >;
    /**
     * Text displayed in the table header, under the title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#description)
     */
    description: string | Function;
    /**
     * Indicates whether the table is disabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether [editing](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#editing-in-featuretable) is enabled on the data within the feature table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * The total number of records displayed in the table's current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#effectiveSize)
     */
    readonly effectiveSize: number;
    /**
     * Indicates whether the table only displays rows that are considered selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#filterBySelectionEnabled)
     */
    filterBySelectionEnabled: boolean;
    /**
     * Set this property to filter the features displayed in the table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#filterGeometry)
     */
    filterGeometry: Geometry;
    /**
     * A collection of string values which indicate [field.names](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) that should be hidden within the table.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#hiddenFields)
     */
    hiddenFields: Collection<string>;
    /**
     * Indicates whether to highlight the associated feature when a row is selected by checking the corresponding checkbox.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#highlightEnabled)
     */
    highlightEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#highlightIds)
     */
    highlightIds: Collection<number | string>;
    /**
     * The user-provided number of total features accessed from the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#initialSize)
     */
    initialSize: number;
    /**
     * Indicates if the table is querying or syncing data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#isQueryingOrSyncing)
     */
    readonly isQueryingOrSyncing: boolean;
    /**
     * Indicates if the table is syncing attachment edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#isSyncingAttachments)
     */
    readonly isSyncingAttachments: boolean;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#layer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * An array of layers listed within the [dropdown component](https://developers.arcgis.com/calcite-design-system/components/dropdown/) of the table's header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#layers)
     */
    readonly layers: (
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer
    )[];
    /**
     * The [layer view](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) associated with the table's [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#layerView)
     */
    readonly layerView: LayerView;
    /**
     * This property is applicable when working with layers that contain a large number of features, as it provides the ability to limit the displayed total feature count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#maxSize)
     */
    maxSize: number;
    /**
     * Set this object to customize the feature table's menu content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#menuConfig)
     */
    menuConfig: TableMenuConfig;
    /**
     * Controls whether the table allows multiple selected rows.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled: boolean;
    /**
     * Indicates whether sorting multiple columns is supported within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multiSortEnabled)
     */
    multiSortEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#objectIds)
     */
    objectIds: Collection<number | string>;
    /**
     * An array of field names from the table's data source to include when the table requests data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#outFields)
     */
    outFields: string[];
    /**
     * Number of pages of features to be displayed in the table, based on the total [number of features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#size) and configured [pageSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageSize).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageCount)
     */
    readonly pageCount: number;
    /**
     * Represents the index of the page of the feature currently being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageIndex)
     */
    pageIndex: number;
    /**
     * The default page size used when displaying features within the table.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageSize)
     */
    pageSize: number;
    /**
     * Controls whether the table should only display a single page of features at any time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#paginationEnabled)
     */
    paginationEnabled: boolean;
    /**
     * Indicates whether to display any related records associated with rows within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedRecordsEnabled)
     */
    relatedRecordsEnabled: boolean;
    /**
     * A nested table instance which represents a relationship to another table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedTable)
     */
    relatedTable: FeatureTable;
    /**
     * A collection of nested table instances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedTables)
     */
    relatedTables: Collection<FeatureTable>;
    /**
     * Indicates whether to fetch geometries for the corresponding features displayed in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnGeometryEnabled)
     */
    returnGeometryEnabled: boolean;
    /**
     * Indicates whether geometries fetched for the corresponding features contain M values, if supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnMEnabled)
     */
    returnMEnabled: boolean;
    /**
     * Indicates whether the fetched features' geometries contain Z values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnZEnabled)
     */
    returnZEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#rowHighlightIds)
     */
    rowHighlightIds: Collection<number | string>;
    /**
     * Total number of records currently displayed in the table.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#size)
     */
    readonly size: number;
    /**
     * The state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#state)
     */
    readonly state: "disabled" | "loading" | "loaded" | "ready" | "error";
    /**
     * A reference to top-level controller table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableController)
     */
    tableController: FeatureTable;
    /**
     * A reference to the instance of a table widget that is directly associated with this table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableParent)
     */
    tableParent: FeatureTable;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html) used for the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableTemplate)
     */
    tableTemplate: TableTemplate;
    /**
     * The [TimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) in which to filter and display data within the FeatureTable widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * Dates and times displayed in the widget will be in terms of this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeZone)
     */
    timeZone: string;
    /**
     * Replacement title for the table widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#title)
     */
    title: string | Function;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#viewModel)
     */
    viewModel: FeatureTableViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#visibleElements)
     */
    visibleElements: FeatureTableVisibleElements;

    /**
     * Clears the current selection filter within the table.
     *
     * @deprecated since version 4.30. Use [`filterBySelectionEnabled`](#filterBySelectionEnabled) or [`objectIds`](#objectIds) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#clearSelectionFilter)
     */
    clearSelectionFilter(): void;
    /**
     * Deletes all the selected rows from the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#deleteSelection)
     */
    deleteSelection(showWarningPrompt?: boolean): Promise<void>;
    /**
     * Exports features associated with currently selected rows to a CSV file and displays a download prompt.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#exportSelectionToCSV)
     */
    exportSelectionToCSV(includeGeometry?: boolean): Promise<void>;
    /**
     * Filters the table using the current row selection and displays only the selected table rows.
     *
     * @deprecated since version 4.30. Use [`filterBySelectionEnabled`](#filterBySelectionEnabled) or [`objectIds`](#objectIds) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#filterBySelection)
     */
    filterBySelection(): void;
    /**
     * Finds the specified column within the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#findColumn)
     */
    findColumn(fieldName: string): Column | GroupColumn | ActionColumn | AttachmentsColumn | RelationshipColumn;
    /**
     * Instructs the table to scroll to or display a specific page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#goToPage)
     */
    goToPage(index: number): void;
    /**
     * Hides the specified column from the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#hideColumn)
     */
    hideColumn(fieldName: string): void;
    /**
     * Instructs the table to scroll to or display the next page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#nextPage)
     */
    nextPage(): void;
    /**
     * Instructs the table to scroll to or display the previous page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#previousPage)
     */
    previousPage(): void;
    /**
     * Refreshes the table contents while maintaining the current scroll position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#refresh)
     */
    refresh(): Promise<void>;
    /**
     * Re-renders the cell's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#refreshCellContent)
     */
    refreshCellContent(): void;
    /**
     * Resets the horizontal scroll position of the table to the default view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#scrollLeft)
     */
    scrollLeft(): void;
    /**
     * Scrolls the table to the bottom row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#scrollToBottom)
     */
    scrollToBottom(): void;
    /**
     * Scrolls the table to a row based on a specified index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#scrollToIndex)
     */
    scrollToIndex(index: number): void;
    /**
     * Scrolls the table to a row based on a specified object ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#scrollToRow)
     */
    scrollToRow(objectId: number): void;
    /**
     * Scrolls the table to the top row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#scrollToTop)
     */
    scrollToTop(): void;
    /**
     * Shows all of the columns in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#showAllColumns)
     */
    showAllColumns(): void;
    /**
     * Shows the specified column within the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#showColumn)
     */
    showColumn(fieldName: string): void;
    /**
     * Sorts the column in either ascending ("asc") or descending ("desc") order.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#sortColumn)
     */
    sortColumn(fieldName: string, direction: "asc" | "desc"): void;
    /**
     * Toggles current visibility of the provided column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#toggleColumnVisibility)
     */
    toggleColumnVisibility(fieldName: string): void;
    /**
     * Zooms the view to the extent of the current row selection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#zoomToSelection)
     */
    zoomToSelection(): void;

    on(name: "cell-click", eventHandler: FeatureTableCellClickEventHandler): IHandle;

    on(name: "cell-dblclick", eventHandler: FeatureTableCellDblclickEventHandler): IHandle;

    on(name: "cell-pointerout", eventHandler: FeatureTableCellPointeroutEventHandler): IHandle;

    on(name: "cell-pointerover", eventHandler: FeatureTableCellPointeroverEventHandler): IHandle;

    on(name: "cell-keydown", eventHandler: FeatureTableCellKeydownEventHandler): IHandle;

    on(name: "column-reorder", eventHandler: FeatureTableColumnReorderEventHandler): IHandle;
  }

  interface FeatureTableProperties extends WidgetProperties {
    /**
     * Configuration for the [ActionColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#actionColumnConfig)
     */
    actionColumnConfig?: ActionColumnConfig;
    /**
     * Indicates whether to display the `Attachments` field in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#attachmentsEnabled)
     */
    attachmentsEnabled?: boolean;
    /**
     * Use this property to configure how columns display within the table in regard to visibility, column order, and sorting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Indicates whether the table should automatically refresh when the underlying data changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#autoRefreshEnabled)
     */
    autoRefreshEnabled?: boolean;
    /**
     * Indicates whether the table should allow reordering of columns.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#columnReorderingEnabled)
     */
    columnReorderingEnabled?: boolean;
    /**
     * Text displayed in the table header, under the title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#description)
     */
    description?: string | Function;
    /**
     * Indicates whether the table is disabled.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether [editing](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#editing-in-featuretable) is enabled on the data within the feature table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * Indicates whether the table only displays rows that are considered selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#filterBySelectionEnabled)
     */
    filterBySelectionEnabled?: boolean;
    /**
     * Set this property to filter the features displayed in the table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#filterGeometry)
     */
    filterGeometry?: GeometryProperties;
    /**
     * A collection of string values which indicate [field.names](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) that should be hidden within the table.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#hiddenFields)
     */
    hiddenFields?: CollectionProperties<string>;
    /**
     * Indicates whether to highlight the associated feature when a row is selected by checking the corresponding checkbox.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#highlightEnabled)
     */
    highlightEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#highlightIds)
     */
    highlightIds?: CollectionProperties<number | string>;
    /**
     * The user-provided number of total features accessed from the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#initialSize)
     */
    initialSize?: number;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#layer)
     */
    layer?:
      | (CatalogFootprintLayerProperties & { type: "catalog-footprint" })
      | (CSVLayerProperties & { type: "csv" })
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (ImageryLayerProperties & { type: "imagery" })
      | (KnowledgeGraphSublayerProperties & { type: "knowledge-graph-sublayer" })
      | (SceneLayerProperties & { type: "scene" })
      | (WFSLayerProperties & { type: "wfs" });
    /**
     * This property is applicable when working with layers that contain a large number of features, as it provides the ability to limit the displayed total feature count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#maxSize)
     */
    maxSize?: number;
    /**
     * Set this object to customize the feature table's menu content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#menuConfig)
     */
    menuConfig?: TableMenuConfig;
    /**
     * Controls whether the table allows multiple selected rows.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates whether sorting multiple columns is supported within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multiSortEnabled)
     */
    multiSortEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#objectIds)
     */
    objectIds?: CollectionProperties<number | string>;
    /**
     * An array of field names from the table's data source to include when the table requests data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#outFields)
     */
    outFields?: string[];
    /**
     * Represents the index of the page of the feature currently being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageIndex)
     */
    pageIndex?: number;
    /**
     * The default page size used when displaying features within the table.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#pageSize)
     */
    pageSize?: number;
    /**
     * Controls whether the table should only display a single page of features at any time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#paginationEnabled)
     */
    paginationEnabled?: boolean;
    /**
     * Indicates whether to display any related records associated with rows within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedRecordsEnabled)
     */
    relatedRecordsEnabled?: boolean;
    /**
     * A nested table instance which represents a relationship to another table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedTable)
     */
    relatedTable?: FeatureTableProperties;
    /**
     * A collection of nested table instances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#relatedTables)
     */
    relatedTables?: CollectionProperties<FeatureTableProperties>;
    /**
     * Indicates whether to fetch geometries for the corresponding features displayed in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnGeometryEnabled)
     */
    returnGeometryEnabled?: boolean;
    /**
     * Indicates whether geometries fetched for the corresponding features contain M values, if supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnMEnabled)
     */
    returnMEnabled?: boolean;
    /**
     * Indicates whether the fetched features' geometries contain Z values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#returnZEnabled)
     */
    returnZEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#rowHighlightIds)
     */
    rowHighlightIds?: CollectionProperties<number | string>;
    /**
     * A reference to top-level controller table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableController)
     */
    tableController?: FeatureTableProperties;
    /**
     * A reference to the instance of a table widget that is directly associated with this table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableParent)
     */
    tableParent?: FeatureTableProperties;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html) used for the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#tableTemplate)
     */
    tableTemplate?: TableTemplateProperties;
    /**
     * The [TimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) in which to filter and display data within the FeatureTable widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * Dates and times displayed in the widget will be in terms of this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeZone)
     */
    timeZone?: string;
    /**
     * Replacement title for the table widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#title)
     */
    title?: string | Function;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#viewModel)
     */
    viewModel?: FeatureTableViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#visibleElements)
     */
    visibleElements?: FeatureTableVisibleElements;
  }

  export class ActionColumn extends Accessor {
    /**
     * The [ActionColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html) class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) and is designed specifically for displaying a singular [Calcite Action](https://developers.arcgis.com/calcite-design-system/components/action/) component for each row in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html)
     */

    constructor(properties?: ActionColumnProperties);

    /**
     * Function invoked when an individual action is clicked or pressed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#callback)
     */
    callback: ActionColumnCallback;
    /**
     * Indicates if the action should appear disabled.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#disabled)
     */
    disabled: ActionColumnDisabledFunction | boolean;
    /**
     * Icon used by each action.
     *
     * @default "pencil"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#icon)
     */
    icon: string;
  }

  interface ActionColumnProperties {
    /**
     * Function invoked when an individual action is clicked or pressed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#callback)
     */
    callback?: ActionColumnCallback;
    /**
     * Indicates if the action should appear disabled.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#disabled)
     */
    disabled?: ActionColumnDisabledFunction | boolean;
    /**
     * Icon used by each action.
     *
     * @default "pencil"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html#icon)
     */
    icon?: string;
  }

  export type ActionColumnCallback = (params: any) => void;

  export type ActionColumnDisabledFunction = (params: any) => boolean;

  export class AttachmentsColumn extends Accessor {
    /**
     * The `AttachmentsColumn` class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) to display attachment counts for each feature in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html)
     */

    constructor(properties?: AttachmentsColumnProperties);

    /**
     * The sanitized label displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the attachments column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#effectiveLabel)
     */
    readonly effectiveLabel: string;
    /**
     * The [Calcite Icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the attachments column.
     *
     * @default "attachment"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#icon)
     */
    icon: string;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#layer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * Indicates whether the attachments column is sortable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#sortable)
     */
    readonly sortable: boolean;
    /**
     * The text alignment of the attachments column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#textAlign)
     */
    readonly textAlign: string;
    /**
     * Attempts to display attachment thumbnail previews as images instead of generic icons.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailAppearance)
     */
    thumbnailAppearance: boolean;
    /**
     * Controls the number of attachment thumbnail previews to display in each cell.
     *
     * @default 8
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailCount)
     */
    thumbnailCount: number;
    /**
     * Controls the scale of attachment thumbnail icons.
     *
     * @default s
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailIconScale)
     */
    thumbnailIconScale: string;
    /**
     * Controls if attachments thumbnail previews are rendered in each cell.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailsEnabled)
     */
    thumbnailsEnabled: boolean;
  }

  interface AttachmentsColumnProperties {
    /**
     * The [Calcite Icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the attachments column.
     *
     * @default "attachment"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#icon)
     */
    icon?: string;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#layer)
     */
    layer?:
      | (CatalogFootprintLayerProperties & { type: "catalog-footprint" })
      | (CSVLayerProperties & { type: "csv" })
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (ImageryLayerProperties & { type: "imagery" })
      | (KnowledgeGraphSublayerProperties & { type: "knowledge-graph-sublayer" })
      | (SceneLayerProperties & { type: "scene" })
      | (WFSLayerProperties & { type: "wfs" });
    /**
     * Attempts to display attachment thumbnail previews as images instead of generic icons.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailAppearance)
     */
    thumbnailAppearance?: boolean;
    /**
     * Controls the number of attachment thumbnail previews to display in each cell.
     *
     * @default 8
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailCount)
     */
    thumbnailCount?: number;
    /**
     * Controls the scale of attachment thumbnail icons.
     *
     * @default s
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailIconScale)
     */
    thumbnailIconScale?: string;
    /**
     * Controls if attachments thumbnail previews are rendered in each cell.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html#thumbnailsEnabled)
     */
    thumbnailsEnabled?: boolean;
  }

  export class FeatureTableViewModel extends Accessor {
    /**
     * Provides the logic for the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget, which allows users to view content from feature attributes in a tabular format.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html)
     */

    constructor(properties?: FeatureTableViewModelProperties);

    /**
     * Configuration for the [ActionColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#actionColumnConfig)
     */
    actionColumnConfig: ActionColumnConfig;
    /**
     * A read-only property indicating the type of filter used by the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#activeFilters)
     */
    readonly activeFilters: Collection<GeometryFilter | SelectionFilter>;
    /**
     * Use this read-only property if needing to query features while retaining a column's sort order.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#activeSortOrders)
     */
    readonly activeSortOrders: ColumnSortOrder[];
    /**
     * Indicates the table is displaying all related tables in `show all` mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#allRelatedTablesVisible)
     */
    readonly allRelatedTablesVisible: boolean;
    /**
     * Indicates whether to display the `Attachments` field in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#attachmentsEnabled)
     */
    attachmentsEnabled: boolean;
    /**
     * Use this property to configure how columns display within the table in regard to visibility, column order, and sorting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#attributeTableTemplate)
     */
    attributeTableTemplate: AttributeTableTemplate;
    /**
     * Indicates whether the table should automatically refresh when the underlying data changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#autoRefreshEnabled)
     */
    autoRefreshEnabled: boolean;
    /**
     * A read-only collection of [column](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html), [field](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html), [group](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-GroupColumn.html), [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html), [attachment](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-AttachmentsColumn.html), and [relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html) columns that are displayed within the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#columns)
     */
    readonly columns: Collection<
      Column | FieldColumn | GroupColumn | ActionColumn | AttachmentsColumn | RelationshipColumn
    >;
    /**
     * Indicates whether editing is enabled on the data within the feature table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#editingEnabled)
     */
    editingEnabled: boolean;
    /**
     * The total number of records displayed in the table's current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#effectiveSize)
     */
    readonly effectiveSize: number;
    /**
     * Indicates whether the table only displays rows that are considered selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterBySelectionEnabled)
     */
    filterBySelectionEnabled: boolean;
    /**
     * Set this property to filter the features displayed in the table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterGeometry)
     */
    filterGeometry: Geometry;
    /**
     * A collection of string [field.names](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) that are to remain hidden within the table.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#hiddenFields)
     */
    hiddenFields: Collection<string>;
    /**
     * Indicates whether to highlight the associated feature when a row is selected.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#highlightEnabled)
     */
    highlightEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#highlightIds)
     */
    highlightIds: Collection<number | string>;
    /**
     * The user-provided number of total features accessed from the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#initialSize)
     */
    initialSize: number;
    /**
     * Indicates if the table is querying or syncing data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#isQueryingOrSyncing)
     */
    readonly isQueryingOrSyncing: boolean;
    /**
     * Indicates if the table is syncing attachment edits.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#isSyncingAttachments)
     */
    readonly isSyncingAttachments: boolean;
    /**
     * The associated  [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * The [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) displaying data for the table's associated [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layerView)
     */
    readonly layerView: LayerView;
    /**
     * This property is applicable when working with layers that contain a large number of features, as it provides the ability to limit the displayed total feature count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#maxSize)
     */
    maxSize: number;
    /**
     * Controls whether the table allows multiple selected rows.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled: boolean;
    /**
     * Indicates whether sorting multiple columns is supported within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#multiSortEnabled)
     */
    multiSortEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#objectIds)
     */
    objectIds: Collection<number | string>;
    /**
     * An array of field names from the table's data source to include when the table requests data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#outFields)
     */
    outFields: string[];
    /**
     * Number of pages of features to be displayed in the table, based on the total [number of features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#size) and configured [pageSize](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageSize).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageCount)
     */
    readonly pageCount: number;
    /**
     * Represents the index of the page of the feature currently being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageIndex)
     */
    pageIndex: number;
    /**
     * The default page size used when displaying features within the table.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageSize)
     */
    pageSize: number;
    /**
     * Controls whether the table should only display a single page of features at any time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#paginationEnabled)
     */
    paginationEnabled: boolean;
    /**
     * Indicates whether to display any related records associated with rows within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedRecordsEnabled)
     */
    relatedRecordsEnabled: boolean;
    /**
     * A nested table instance which represents a relationship to another table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedTable)
     */
    relatedTable: FeatureTable;
    /**
     * A collection of nested table instances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedTables)
     */
    relatedTables: Collection<FeatureTable>;
    /**
     * An array of relationships that exist on the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relationships)
     */
    relationships: Relationship[];
    /**
     * Indicates whether to fetch geometries for the corresponding features displayed in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnGeometryEnabled)
     */
    returnGeometryEnabled: boolean;
    /**
     * Indicates whether geometries fetched for the corresponding features contain M values, if supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnMEnabled)
     */
    returnMEnabled: boolean;
    /**
     * Indicates whether the fetched features' geometries contain Z values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnZEnabled)
     */
    returnZEnabled: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#rowHighlightIds)
     */
    rowHighlightIds: Collection<number | string>;
    /**
     * Total number of records currently displayed in the table.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#size)
     */
    readonly size: number;
    /**
     * The state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#state)
     */
    readonly state: "disabled" | "loading" | "loaded" | "ready" | "error";
    /**
     * Indicates whether the table and associated layer support adding attachments with the current configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#supportsAddAttachments)
     */
    readonly supportsAddAttachments: boolean;
    /**
     * Indicates whether the table supports viewing attachments with the current configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#supportsAttachments)
     */
    readonly supportsAttachments: boolean;
    /**
     * Indicates whether the table and associated layer support deleting attachments with the current configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#supportsDeleteAttachments)
     */
    readonly supportsDeleteAttachments: boolean;
    /**
     * Indicates whether the table and associated layer support resizing attachments with the current configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#supportsResizeAttachments)
     */
    readonly supportsResizeAttachments: boolean;
    /**
     * Indicates whether the table and associated layer support updating attachments with the current configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#supportsUpdateAttachments)
     */
    readonly supportsUpdateAttachments: boolean;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html) used for the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#tableTemplate)
     */
    tableTemplate: TableTemplate;
    /**
     * The [TimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) in which to filter and display data within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * Dates and times displayed in the widget will be in terms of this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#timeZone)
     */
    timeZone: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Clears the current [filterBySelection](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterBySelection) so that the table displays all of the table rows.
     *
     * @deprecated since version 4.30. Use [`filterBySelectionEnabled`](#filterBySelectionEnabled) or [`objectIds()`](#objectIds) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#clearSelectionFilter)
     */
    clearSelectionFilter(): void;
    /**
     * Deletes all the selected rows from the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#deleteSelection)
     */
    deleteSelection(): Promise<void>;
    /**
     * Exports features associated with currently selected rows to a CSV file and displays a download prompt.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#exportSelectionToCSV)
     */
    exportSelectionToCSV(includeGeometry?: boolean): Promise<void>;
    /**
     * Filters the table using the current row selection and displays only those selected table rows.
     *
     * @deprecated since version 4.30. Use [`filterBySelectionEnabled`](#filterBySelectionEnabled) or [`objectIds`](#objectIds) instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterBySelection)
     */
    filterBySelection(): void;
    /**
     * Returns the current row index for the associated feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#getObjectIdIndex)
     */
    getObjectIdIndex(objectId: number | string): number;
    /**
     * Returns a field value given the specified feature `object` and an associated `fieldName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#getValue)
     */
    getValue(objectId: number, fieldName: string): string | number;
    /**
     * Instructs the table to scroll to or display a specific page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#goToPage)
     */
    goToPage(): void;
    /**
     * Internal method used to hide the attachments view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#hideAttachmentsView)
     */
    hideAttachmentsView(): void;
    /**
     * Instructs the table to scroll to or display the next page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#nextPage)
     */
    nextPage(): void;
    /**
     * Instructs the table to scroll to or display the previous page of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#previousPage)
     */
    previousPage(): void;
    /**
     * Refreshes the table contents while maintaining the current scroll position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#refresh)
     */
    refresh(): void;
    /**
     * Re-renders the cell's content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#refreshCellContent)
     */
    refreshCellContent(): void;
    /**
     * Performs a light refresh of cell content, emptying the store and fetching fresh data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#refreshPageCache)
     */
    refreshPageCache(): void;
    /**
     * Performs a full reset of the entire table resulting in the table scrolling to the top-most row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#reset)
     */
    reset(): void;
    /**
     * Resets the horizontal scroll position of the table to the default view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#scrollLeft)
     */
    scrollLeft(): void;
    /**
     * Scrolls the table to the bottom row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#scrollToBottom)
     */
    scrollToBottom(): void;
    /**
     * Scrolls the table to a row based on a specified index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#scrollToIndex)
     */
    scrollToIndex(index: number): void;
    /**
     * Scrolls the table to a row based on a specified object ID.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#scrollToRow)
     */
    scrollToRow(objectId: number): void;
    /**
     * Scrolls the table to the top row.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#scrollToTop)
     */
    scrollToTop(): void;
    /**
     * Toggles current visibility of the provided column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#toggleColumnVisibility)
     */
    toggleColumnVisibility(fieldName: string): void;
    /**
     * Zooms the view to the extent of the current row selection.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#zoomToSelection)
     */
    zoomToSelection(): void;

    on(name: "cell-click", eventHandler: FeatureTableViewModelCellClickEventHandler): IHandle;

    on(name: "cell-dblclick", eventHandler: FeatureTableViewModelCellDblclickEventHandler): IHandle;

    on(name: "cell-pointerout", eventHandler: FeatureTableViewModelCellPointeroutEventHandler): IHandle;

    on(name: "cell-pointerover", eventHandler: FeatureTableViewModelCellPointeroverEventHandler): IHandle;

    on(name: "cell-keydown", eventHandler: FeatureTableViewModelCellKeydownEventHandler): IHandle;

    on(name: "column-reorder", eventHandler: FeatureTableViewModelColumnReorderEventHandler): IHandle;
  }

  interface FeatureTableViewModelProperties {
    /**
     * Configuration for the [ActionColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-ActionColumn.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#actionColumnConfig)
     */
    actionColumnConfig?: ActionColumnConfig;
    /**
     * Indicates whether to display the `Attachments` field in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#attachmentsEnabled)
     */
    attachmentsEnabled?: boolean;
    /**
     * Use this property to configure how columns display within the table in regard to visibility, column order, and sorting.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#attributeTableTemplate)
     */
    attributeTableTemplate?: AttributeTableTemplateProperties;
    /**
     * Indicates whether the table should automatically refresh when the underlying data changes.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#autoRefreshEnabled)
     */
    autoRefreshEnabled?: boolean;
    /**
     * Indicates whether editing is enabled on the data within the feature table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#editingEnabled)
     */
    editingEnabled?: boolean;
    /**
     * Indicates whether the table only displays rows that are considered selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterBySelectionEnabled)
     */
    filterBySelectionEnabled?: boolean;
    /**
     * Set this property to filter the features displayed in the table.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#filterGeometry)
     */
    filterGeometry?: GeometryProperties;
    /**
     * A collection of string [field.names](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) that are to remain hidden within the table.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#hiddenFields)
     */
    hiddenFields?: CollectionProperties<string>;
    /**
     * Indicates whether to highlight the associated feature when a row is selected.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#highlightEnabled)
     */
    highlightEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#highlightIds)
     */
    highlightIds?: CollectionProperties<number | string>;
    /**
     * The user-provided number of total features accessed from the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#initialSize)
     */
    initialSize?: number;
    /**
     * The associated  [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layer)
     */
    layer?:
      | (CatalogFootprintLayerProperties & { type: "catalog-footprint" })
      | (CSVLayerProperties & { type: "csv" })
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (ImageryLayerProperties & { type: "imagery" })
      | (KnowledgeGraphSublayerProperties & { type: "knowledge-graph-sublayer" })
      | (SceneLayerProperties & { type: "scene" })
      | (WFSLayerProperties & { type: "wfs" });
    /**
     * This property is applicable when working with layers that contain a large number of features, as it provides the ability to limit the displayed total feature count.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#maxSize)
     */
    maxSize?: number;
    /**
     * Controls whether the table allows multiple selected rows.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates whether sorting multiple columns is supported within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#multiSortEnabled)
     */
    multiSortEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#objectIds)
     */
    objectIds?: CollectionProperties<number | string>;
    /**
     * An array of field names from the table's data source to include when the table requests data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#outFields)
     */
    outFields?: string[];
    /**
     * Represents the index of the page of the feature currently being displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageIndex)
     */
    pageIndex?: number;
    /**
     * The default page size used when displaying features within the table.
     *
     * @default 50
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#pageSize)
     */
    pageSize?: number;
    /**
     * Controls whether the table should only display a single page of features at any time.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#paginationEnabled)
     */
    paginationEnabled?: boolean;
    /**
     * Indicates whether to display any related records associated with rows within the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedRecordsEnabled)
     */
    relatedRecordsEnabled?: boolean;
    /**
     * A nested table instance which represents a relationship to another table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedTable)
     */
    relatedTable?: FeatureTableProperties;
    /**
     * A collection of nested table instances.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relatedTables)
     */
    relatedTables?: CollectionProperties<FeatureTableProperties>;
    /**
     * An array of relationships that exist on the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#relationships)
     */
    relationships?: RelationshipProperties[];
    /**
     * Indicates whether to fetch geometries for the corresponding features displayed in the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnGeometryEnabled)
     */
    returnGeometryEnabled?: boolean;
    /**
     * Indicates whether geometries fetched for the corresponding features contain M values, if supported.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnMEnabled)
     */
    returnMEnabled?: boolean;
    /**
     * Indicates whether the fetched features' geometries contain Z values.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#returnZEnabled)
     */
    returnZEnabled?: boolean;
    /**
     * This property accepts and returns a collection of feature object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#rowHighlightIds)
     */
    rowHighlightIds?: CollectionProperties<number | string>;
    /**
     * The associated [template](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html) used for the feature table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#tableTemplate)
     */
    tableTemplate?: TableTemplateProperties;
    /**
     * The [TimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeExtent.html) in which to filter and display data within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * Dates and times displayed in the widget will be in terms of this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#timeZone)
     */
    timeZone?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * Configuration for the FeatureTable's actionColumn.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
   */
  export interface ActionColumnConfig {
    /**
     * The string value indicating the [Calcite icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed for the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
     */
    icon?: string;
    /**
     * The accessible label to display when hovering over the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
     */
    label: string;
    /**
     * Callback function invoked when the action is clicked or activated via the keyboard.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
     */
    callback: ActionColumnCallback;
    /**
     * Indicates the column should be frozen to the end of the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
     */
    frozenToEnd?: boolean;
    /**
     * Indicates if a specific action should be conditionally disabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ActionColumnConfig)
     */
    disabled?: ActionColumnDisabledFunction | boolean;
  }

  export interface FeatureTableViewModelCellClickEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-click";
  }

  export interface FeatureTableViewModelCellDblclickEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-dblclick";
  }

  export interface FeatureTableViewModelCellKeydownEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-keydown";
  }

  export interface FeatureTableViewModelCellPointeroutEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-pointerout";
  }

  export interface FeatureTableViewModelCellPointeroverEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-pointerover";
  }

  export interface FeatureTableViewModelColumnReorderEvent {
    type: "column-reorder";
  }

  /**
   * An array of objects containing a column's name and sort direction.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ColumnSortOrder)
   */
  export interface ColumnSortOrder {
    /**
     * The field name as defined by the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ColumnSortOrder)
     */
    fieldName: string;
    /**
     * Controls the sort order of the column given the set `fieldName`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#ColumnSortOrder)
     */
    direction: "asc" | "desc";
  }

  /**
   * Use this for spatial filtering, it is the [Geometry](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Geometry.html) used to filter out data within the table.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#GeometryFilter)
   */
  export interface GeometryFilter {
    /**
     * The type of the filter used.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The geometry used to filter out the table's data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#GeometryFilter)
     */
    geometry: Geometry;
  }

  /**
   * Use this for selecting rows within the table based off of their object ID.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#SelectionFilter)
   */
  export interface SelectionFilter {
    /**
     * The type of the filter used.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * An array of numbers indicating the row's object IDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FeatureTableViewModel.html#SelectionFilter)
     */
    objectIds: number[];
  }

  export interface FieldColumn extends Accessor, Column {}

  export class FieldColumn {
    /**
     * The [FieldColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html) class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) and provides the underlying logic for column behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html)
     */

    constructor(properties?: FieldColumnProperties);

    /**
     * The display name for the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#alias)
     */
    readonly alias: string;
    /**
     * The default value set for the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#defaultValue)
     */
    readonly defaultValue: number | string;
    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#editable)
     */
    editable: boolean;
    /**
     * The [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) associated with this column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#field)
     */
    field: Field;
    /**
     * Custom function for rendering cell content that is called when the column is rendered in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#formatFunction)
     */
    formatFunction: FieldValueFormatFunction;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#layer)
     */
    layer:
      | CatalogFootprintLayer
      | CSVLayer
      | FeatureLayer
      | GeoJSONLayer
      | ImageryLayer
      | KnowledgeGraphSublayer
      | SceneLayer
      | WFSLayer;
    /**
     * Restricts the input length for the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#maxLength)
     */
    readonly maxLength: number;
    /**
     * Restricts the input length for the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#minLength)
     */
    readonly minLength: number;
    /**
     * The name of the field.
     *
     * @deprecated since version 4.30, use {@link module:esri/widgets/FeatureTable/FieldColumn#fieldName FieldColumn.fieldName} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#name)
     */
    readonly name: string;
    /**
     * Indicates if the field can accept `null` values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#nullable)
     */
    readonly nullable: boolean;
    /**
     * Indicates whether a value is required when editing feature attributes within the table's cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#required)
     */
    required: boolean;
    /**
     * A configurable template for the table column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#template)
     */
    template: FieldColumnTemplate;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#view)
     */
    view: MapView | SceneView;
  }

  interface FieldColumnProperties extends ColumnProperties {
    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#editable)
     */
    editable?: boolean;
    /**
     * The [Field](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Field.html) associated with this column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#field)
     */
    field?: FieldProperties;
    /**
     * Custom function for rendering cell content that is called when the column is rendered in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#formatFunction)
     */
    formatFunction?: FieldValueFormatFunction;
    /**
     * The associated [CatalogFootprintLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-catalog-CatalogFootprintLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html), [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [ImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-ImageryLayer.html), [KnowledgeGraphSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-knowledgeGraph-KnowledgeGraphSublayer.html), [SceneLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html), or [WFSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WFSLayer.html) containing the fields and attributes to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#layer)
     */
    layer?:
      | (CatalogFootprintLayerProperties & { type: "catalog-footprint" })
      | (CSVLayerProperties & { type: "csv" })
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (ImageryLayerProperties & { type: "imagery" })
      | (KnowledgeGraphSublayerProperties & { type: "knowledge-graph-sublayer" })
      | (SceneLayerProperties & { type: "scene" })
      | (WFSLayerProperties & { type: "wfs" });
    /**
     * Indicates whether a value is required when editing feature attributes within the table's cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#required)
     */
    required?: boolean;
    /**
     * A configurable template for the table column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#template)
     */
    template?: FieldColumnTemplateProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export type FieldValueFormatFunction = (params: any) => string | number | HTMLElement;

  export interface Column extends Accessor, Evented {}

  export class Column {
    /**
     * The `Column` class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) and provides much of the underlying logic for column behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html)
     */

    constructor(properties?: ColumnProperties);

    /**
     * Indicates if the column width will automatically adjust to account for large content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#autoWidth)
     */
    autoWidth: boolean;
    /**
     * Contains information describing the purpose of each column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#description)
     */
    description: string;
    /**
     * Controls the sort order of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#direction)
     */
    direction: "asc" | "desc";
    /**
     * The sanitized [description](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#description) string, describing the purpose of each column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#effectiveDescription)
     */
    effectiveDescription: string;
    /**
     * The sanitized [label](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#label) string safe for display in the header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#effectiveLabel)
     */
    effectiveLabel: string;
    /**
     * The unique field name as defined by the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#fieldName)
     */
    readonly fieldName: string;
    /**
     * Controls the [flex-grow](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-grow) property for the column.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#flexGrow)
     */
    flexGrow: number;
    /**
     * Custom function for rendering cell content that is called when the column is rendered in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#formatFunction)
     */
    readonly formatFunction: FormatFunction;
    /**
     * Indicates if the the column is frozen in place at the beginning of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#frozen)
     */
    readonly frozen: boolean;
    /**
     * Indicates if the column is frozen in place at the end of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#frozenToEnd)
     */
    readonly frozenToEnd: boolean;
    /**
     * The string value indicating the [icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed in the header cell of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#icon)
     */
    icon: string;
    /**
     * Indicates the initial sort priority of the column when first rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#initialSortPriority)
     */
    readonly initialSortPriority: number;
    /**
     * Indicates whether the column is in an invalid state.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#invalid)
     */
    invalid: boolean;
    /**
     * The default label displayed in the column header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#label)
     */
    label: string;
    /**
     * The element representing the field column's menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#menu)
     */
    readonly menu: HTMLElement;
    /**
     * Configuration options for the column's menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#menuConfig)
     */
    readonly menuConfig: ColumnTableMenuConfig;
    /**
     * Indicates whether the column's menu is currently open.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#menuIsOpen)
     */
    readonly menuIsOpen: boolean;
    /**
     * Indicates whether the Column's menu will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#menuIsVisible)
     */
    readonly menuIsVisible: boolean;
    /**
     * Indicates whether the column is resizable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#resizable)
     */
    readonly resizable: boolean;
    /**
     * Indicates whether the column is sortable.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#sortable)
     */
    sortable: boolean;
    /**
     * The [timeZone](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeZone) of the parent table widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#tableTimeZone)
     */
    tableTimeZone: string;
    /**
     * Aligns the columns cell content horizontally.
     *
     * @default "start"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#textAlign)
     */
    textAlign: "start" | "center" | "end";
    /**
     * Indicates cell content should be wrapped and displayed on multiple lines within the cell.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#textWrap)
     */
    textWrap: boolean;
    /**
     * The [storage IANA time zone](https://www.iana.org/time-zones) of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#timeZone)
     */
    timeZone: string;
    /**
     * The [visible elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements) of the column's associated FeatureTable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#visibleElements)
     */
    visibleElements: FeatureTableVisibleElements;
    /**
     * Width of the column in pixels.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#width)
     */
    width: number | string;

    /**
     * Convenience method for closing the column menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#closeMenu)
     */
    closeMenu(): void;
    /**
     * Convenience method for opening the column menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#openMenu)
     */
    openMenu(): void;
    /**
     * Convenience method for sorting the current column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#sort)
     */
    sort(): void;
  }

  interface ColumnProperties {
    /**
     * Indicates if the column width will automatically adjust to account for large content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#autoWidth)
     */
    autoWidth?: boolean;
    /**
     * Contains information describing the purpose of each column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#description)
     */
    description?: string;
    /**
     * Controls the sort order of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#direction)
     */
    direction?: "asc" | "desc";
    /**
     * The sanitized [description](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#description) string, describing the purpose of each column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#effectiveDescription)
     */
    effectiveDescription?: string;
    /**
     * The sanitized [label](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#label) string safe for display in the header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#effectiveLabel)
     */
    effectiveLabel?: string;
    /**
     * The unique field name as defined by the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#fieldName)
     */
    fieldName?: string;
    /**
     * Controls the [flex-grow](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-grow) property for the column.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#flexGrow)
     */
    flexGrow?: number;
    /**
     * Indicates if the the column is frozen in place at the beginning of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#frozen)
     */
    frozen?: boolean;
    /**
     * Indicates if the column is frozen in place at the end of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#frozenToEnd)
     */
    frozenToEnd?: boolean;
    /**
     * The string value indicating the [icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed in the header cell of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#icon)
     */
    icon?: string;
    /**
     * Indicates the initial sort priority of the column when first rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#initialSortPriority)
     */
    initialSortPriority?: number;
    /**
     * Indicates whether the column is in an invalid state.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#invalid)
     */
    invalid?: boolean;
    /**
     * The default label displayed in the column header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#label)
     */
    label?: string;
    /**
     * Indicates whether the column is sortable.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#sortable)
     */
    sortable?: boolean;
    /**
     * The [timeZone](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#timeZone) of the parent table widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#tableTimeZone)
     */
    tableTimeZone?: string;
    /**
     * Aligns the columns cell content horizontally.
     *
     * @default "start"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#textAlign)
     */
    textAlign?: "start" | "center" | "end";
    /**
     * Indicates cell content should be wrapped and displayed on multiple lines within the cell.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#textWrap)
     */
    textWrap?: boolean;
    /**
     * The [storage IANA time zone](https://www.iana.org/time-zones) of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#timeZone)
     */
    timeZone?: string;
    /**
     * The [visible elements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements) of the column's associated FeatureTable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#visibleElements)
     */
    visibleElements?: FeatureTableVisibleElements;
    /**
     * Width of the column in pixels.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#width)
     */
    width?: number | string;
  }

  export interface ColumnTableMenuConfig {
    /**
     * Indicates whether the menu is disabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#ColumnTableMenuConfig)
     */
    disabled?: boolean;
    /**
     * The menu items within the table menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#ColumnTableMenuConfig)
     */
    items: TableMenuItemConfig[];
    /**
     * Indicates whether the menu is open.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#ColumnTableMenuConfig)
     */
    open?: boolean;
    /**
     * The string value indicating the [icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed in the header cell of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#ColumnTableMenuConfig)
     */
    icon?: string;
    /**
     * The selection mode of the menu items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#ColumnTableMenuConfig)
     */
    selectionMode?: "none" | "single" | "multiple";
  }

  export type FormatFunction = (params: any) => string | number | HTMLElement;

  /**
   * The related records for the associated feature used in the [Columns's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#formatFunction) and [FieldColumn's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html#FieldValueFormatFunction) `formatFunction`.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#RelatedRecordInfo)
   */
  export interface RelatedRecordInfo {
    /**
     * The ID of the relationship defined in the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#RelatedRecordInfo)
     */
    relationshipId?: string;
    /**
     * The number of related records associated with the feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html#RelatedRecordInfo)
     */
    count?: number;
  }

  export interface GroupColumn extends Accessor, Column {}

  export class GroupColumn {
    /**
     * The `GroupColumn` class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) and provides the underlying logic for grouped column behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-GroupColumn.html)
     */

    constructor(properties?: GroupColumnProperties);

    /**
     * An array of [columns](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html) to display as grouped which represent an ordered list of [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-GroupColumn.html#columns)
     */
    readonly columns: Column[];
  }

  interface GroupColumnProperties extends ColumnProperties {}

  export class ButtonMenu extends Widget {
    /**
     * This widget provides the underlying menu functionality when working with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html)
     */

    constructor(properties?: ButtonMenuProperties);

    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#disabled)
     */
    disabled: boolean;
    /**
     * Adds a CSS class to the menu button's DOM node.
     *
     * @deprecated since 4.27. Use {@link module:esri/widgets/DirectionPad#icon icon} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#iconClass)
     */
    iconClass: string;
    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#items)
     */
    items: ButtonMenuItem[];
    /**
     * Indicates if the menu content is open and visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#open)
     */
    open: boolean;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#viewModel)
     */
    viewModel: ButtonMenuViewModel;
  }

  interface ButtonMenuProperties extends WidgetProperties {
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#disabled)
     */
    disabled?: boolean;
    /**
     * Adds a CSS class to the menu button's DOM node.
     *
     * @deprecated since 4.27. Use {@link module:esri/widgets/DirectionPad#icon icon} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#iconClass)
     */
    iconClass?: string;
    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#items)
     */
    items?: ButtonMenuItemProperties[];
    /**
     * Indicates if the menu content is open and visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#open)
     */
    open?: boolean;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html#viewModel)
     */
    viewModel?: ButtonMenuViewModelProperties;
  }

  export class ButtonMenuItem extends Accessor {
    /**
     * The `ButtonMenuItem` class provides the underlying menu functionality to create and customize new menu items within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget's [ButtonMenu](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenu.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html)
     */

    constructor(properties?: ButtonMenuItemProperties);

    /**
     * Indicates whether to automatically close the menu's item.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#autoCloseMenu)
     */
    autoCloseMenu: boolean;
    /**
     * A function that executes on the ButtonMenuItem's `click` event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#clickFunction)
     */
    clickFunction: ButtonMenuItemClickFunction;
    /**
     * Adds a CSS class to the menu button's DOM node.
     *
     * @deprecated since 4.27. Use {@link module:esri/widgets/DirectionPad#icon icon} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#iconClass)
     */
    iconClass: string;
    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#items)
     */
    items: ButtonMenuItem[];
    /**
     * The label of the menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#label)
     */
    label: string;
    /**
     * Indicates if the menu content is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#open)
     */
    open: boolean;
    /**
     * Indicates whether the menu item is selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#selected)
     */
    selected: boolean;
    /**
     * Indicates whether a toggled state should be applied to individual menu items.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#selectionEnabled)
     */
    selectionEnabled: boolean;
  }

  interface ButtonMenuItemProperties {
    /**
     * Indicates whether to automatically close the menu's item.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#autoCloseMenu)
     */
    autoCloseMenu?: boolean;
    /**
     * A function that executes on the ButtonMenuItem's `click` event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#clickFunction)
     */
    clickFunction?: ButtonMenuItemClickFunction;
    /**
     * Adds a CSS class to the menu button's DOM node.
     *
     * @deprecated since 4.27. Use {@link module:esri/widgets/DirectionPad#icon icon} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#iconClass)
     */
    iconClass?: string;
    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#items)
     */
    items?: ButtonMenuItemProperties[];
    /**
     * The label of the menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#label)
     */
    label?: string;
    /**
     * Indicates if the menu content is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#open)
     */
    open?: boolean;
    /**
     * Indicates whether the menu item is selected.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#selected)
     */
    selected?: boolean;
    /**
     * Indicates whether a toggled state should be applied to individual menu items.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html#selectionEnabled)
     */
    selectionEnabled?: boolean;
  }

  export type ButtonMenuItemClickFunction = (event: any) => void;

  export class ButtonMenuViewModel extends Accessor {
    /**
     * Provides the logic for the module:esri/widgets/ButtonMenuItemConfig widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuViewModel.html)
     */

    constructor(properties?: ButtonMenuViewModelProperties);

    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuViewModel.html#items)
     */
    items: ButtonMenuItem[];
    /**
     * Indicates if the menu content is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuViewModel.html#open)
     */
    open: boolean;
  }

  interface ButtonMenuViewModelProperties {
    /**
     * An array of individual [menu items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuViewModel.html#items)
     */
    items?: ButtonMenuItemProperties[];
    /**
     * Indicates if the menu content is visible.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-support-ButtonMenuViewModel.html#open)
     */
    open?: boolean;
  }

  export class RelationshipColumn extends Accessor {
    /**
     * The `RelationshipColumn` class works with the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) to display any associated related records for each feature in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html)
     */

    constructor(properties?: RelationshipColumnProperties);

    /**
     * Indicates whether the column displays collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#collapsed)
     */
    collapsed: boolean;
    /**
     * The sanitized label displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the relationship column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#effectiveLabel)
     */
    readonly effectiveLabel: string;
    /**
     * The [Calcite Icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the attachments column.
     *
     * @default "link"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#icon)
     */
    icon: string;
    /**
     * The layer that contains the features being displayed in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#layer)
     */
    readonly layer: FeatureLayer | SceneLayer;
    /**
     * Returns the [Relationship](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-Relationship.html) of the [relatedlayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relatedLayer) to find a relationship that has the same id as the [relationshipId](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relationshipId).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#originRelationship)
     */
    readonly originRelationship: Relationship;
    /**
     * The layer that contains the related features being displayed in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relatedLayer)
     */
    readonly relatedLayer: FeatureLayer | SceneLayer;
    /**
     * Returns the first relationship from [layer's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#layer) `relationships` that matches the [relationshipId](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relationshipId).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relationship)
     */
    readonly relationship: Relationship;
    /**
     * The unique ID for the relationship.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#relationshipId)
     */
    readonly relationshipId: number;
    /**
     * Indicates whether the relationship column can be resized.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#resizable)
     */
    readonly resizable: number;
    /**
     * The text alignment of the relationship column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#textAlign)
     */
    readonly textAlign: string;
    /**
     * Default width of the relationship column in pixels.
     *
     * @default "200px"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#width)
     */
    width: number | string;
  }

  interface RelationshipColumnProperties {
    /**
     * Indicates whether the column displays collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#collapsed)
     */
    collapsed?: boolean;
    /**
     * The [Calcite Icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) header for the attachments column.
     *
     * @default "link"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#icon)
     */
    icon?: string;
    /**
     * Default width of the relationship column in pixels.
     *
     * @default "200px"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-RelationshipColumn.html#width)
     */
    width?: number | string;
  }

  export class ColumnTemplate extends ColumnTemplateBase {
    /**
     * A ColumnTemplate formats and defines the structure of a [Column](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-Column.html) within a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html)
     */

    constructor(properties?: ColumnTemplateProperties);

    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html#editable)
     */
    editable: boolean;
    /**
     * Indicates whether a field requires a value.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html#required)
     */
    required: boolean;
    /**
     * The type of column that the template represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html#type)
     */
    readonly type: "column";

    static fromJSON(json: any): ColumnTemplate;
  }

  interface ColumnTemplateProperties extends ColumnTemplateBaseProperties {
    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html#editable)
     */
    editable?: boolean;
    /**
     * Indicates whether a field requires a value.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplate.html#required)
     */
    required?: boolean;
  }

  export interface ColumnTemplateBase extends Accessor, JSONSupport {}

  export class ColumnTemplateBase {
    /**
     * The base class for all column templates used by the [tableTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html) within the [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html)
     */

    constructor(properties?: ColumnTemplateBaseProperties);

    /**
     * Indicates if the column width will automatically adjust to account for large content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#autoWidth)
     */
    autoWidth: boolean;
    /**
     * A string description of the column to give context to what it represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#description)
     */
    description: string;
    /**
     * Controls the sort order of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#direction)
     */
    direction: "asc" | "desc";
    /**
     * The unique field name as defined by the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#fieldName)
     */
    fieldName: string;
    /**
     * Controls the [flex-grow](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-grow) property for the column.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#flexGrow)
     */
    flexGrow: number;
    /**
     * Custom function for rendering cell content that is called when the column is rendered in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#formatFunction)
     */
    formatFunction: FormatFunction;
    /**
     * Indicates if the the column is frozen in place at the beginning of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#frozen)
     */
    readonly frozen: boolean;
    /**
     * Indicates if the column is frozen in place at the end of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#frozenToEnd)
     */
    readonly frozenToEnd: boolean;
    /**
     * The string value indicating the [icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed in the header cell of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#icon)
     */
    icon: string;
    /**
     * Use this in combination with [FeatureTable.multiSortEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multiSortEnabled) and FeatureTable.direction properties to set sorting functionality on multiple columns.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#initialSortPriority)
     */
    initialSortPriority: number;
    /**
     * Indicates whether the column is in an invalid state.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#invalid)
     */
    invalid: boolean;
    /**
     * The default label displayed in the column header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#label)
     */
    label: string;
    /**
     * Configuration options for the column menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#menuConfig)
     */
    menuConfig: ColumnTableMenuConfig;
    /**
     * Indicates whether the column is resizable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#resizable)
     */
    resizable: boolean;
    /**
     * Indicates whether the column can be sorted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#sortable)
     */
    sortable: boolean;
    /**
     * Aligns the columns cell content horizontally.
     *
     * @default "start"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#textAlign)
     */
    textAlign: "start" | "center" | "end";
    /**
     * Indicates cell content should be wrapped and displayed on multiple lines within the cell.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#textWrap)
     */
    textWrap: boolean;
    /**
     * The time zone of the specific column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#timeZone)
     */
    timeZone: string;
    /**
     * The type of column that the template represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#type)
     */
    readonly type: "column" | "field" | "group" | "attachment" | "relationship";
    /**
     * Indicates whether the column is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#visible)
     */
    visible: boolean;
    /**
     * Default width of the column in pixels.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#width)
     */
    width: number | string;

    static fromJSON(json: any): ColumnTemplateBase;
  }

  interface ColumnTemplateBaseProperties {
    /**
     * Indicates if the column width will automatically adjust to account for large content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#autoWidth)
     */
    autoWidth?: boolean;
    /**
     * A string description of the column to give context to what it represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#description)
     */
    description?: string;
    /**
     * Controls the sort order of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#direction)
     */
    direction?: "asc" | "desc";
    /**
     * The unique field name as defined by the data source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#fieldName)
     */
    fieldName?: string;
    /**
     * Controls the [flex-grow](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-grow) property for the column.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#flexGrow)
     */
    flexGrow?: number;
    /**
     * Custom function for rendering cell content that is called when the column is rendered in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#formatFunction)
     */
    formatFunction?: FormatFunction;
    /**
     * Indicates if the the column is frozen in place at the beginning of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#frozen)
     */
    frozen?: boolean;
    /**
     * Indicates if the column is frozen in place at the end of the table.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#frozenToEnd)
     */
    frozenToEnd?: boolean;
    /**
     * The string value indicating the [icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed in the header cell of the column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#icon)
     */
    icon?: string;
    /**
     * Use this in combination with [FeatureTable.multiSortEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#multiSortEnabled) and FeatureTable.direction properties to set sorting functionality on multiple columns.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#initialSortPriority)
     */
    initialSortPriority?: number;
    /**
     * Indicates whether the column is in an invalid state.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#invalid)
     */
    invalid?: boolean;
    /**
     * The default label displayed in the column header cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#label)
     */
    label?: string;
    /**
     * Configuration options for the column menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#menuConfig)
     */
    menuConfig?: ColumnTableMenuConfig;
    /**
     * Indicates whether the column is resizable.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#resizable)
     */
    resizable?: boolean;
    /**
     * Indicates whether the column can be sorted.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#sortable)
     */
    sortable?: boolean;
    /**
     * Aligns the columns cell content horizontally.
     *
     * @default "start"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#textAlign)
     */
    textAlign?: "start" | "center" | "end";
    /**
     * Indicates cell content should be wrapped and displayed on multiple lines within the cell.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#textWrap)
     */
    textWrap?: boolean;
    /**
     * The time zone of the specific column.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#timeZone)
     */
    timeZone?: string;
    /**
     * Indicates whether the column is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#visible)
     */
    visible?: boolean;
    /**
     * Default width of the column in pixels.
     *
     * @default 200
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-ColumnTemplateBase.html#width)
     */
    width?: number | string;
  }

  export class FieldColumnTemplate extends ColumnTemplateBase {
    /**
     * A FieldColumnTemplate formats and defines the structure of a [FieldColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-FieldColumn.html) within a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html)
     */

    constructor(properties?: FieldColumnTemplateProperties);

    /**
     * The [coded value domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html) or a [range domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html) of the associated field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#domain)
     */
    domain: CodedValueDomain | RangeDomain;
    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#editable)
     */
    editable: boolean;
    /**
     * Custom function for rendering cell content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#formatFunction)
     */
    formatFunction: FieldColumnTemplateFieldValueFormatFunction;
    /**
     * Indicates whether a field requires a value.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#required)
     */
    required: boolean;
    /**
     * The type of column that the template represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#type)
     */
    readonly type: "field";

    static fromJSON(json: any): FieldColumnTemplate;
  }

  interface FieldColumnTemplateProperties extends ColumnTemplateBaseProperties {
    /**
     * The [coded value domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-CodedValueDomain.html) or a [range domain](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-RangeDomain.html) of the associated field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#domain)
     */
    domain?: (CodedValueDomainProperties & { type: "coded-value" }) | (RangeDomainProperties & { type: "range" });
    /**
     * Indicates whether the field is editable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#editable)
     */
    editable?: boolean;
    /**
     * Custom function for rendering cell content.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#formatFunction)
     */
    formatFunction?: FieldColumnTemplateFieldValueFormatFunction;
    /**
     * Indicates whether a field requires a value.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html#required)
     */
    required?: boolean;
  }

  export type FieldColumnTemplateFieldValueFormatFunction = (params: any) => string | number | HTMLElement;

  export class GroupColumnTemplate extends ColumnTemplateBase {
    /**
     * A GroupColumnTemplate formats and defines the structure of a [GroupColumn](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-Grid-GroupColumn.html) within a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html)
     */

    constructor(properties?: GroupColumnTemplateProperties);

    /**
     * A collection of [field column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html) and/or [group column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html) that represent an ordered list of column templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html#columnTemplates)
     */
    columnTemplates: (FieldColumnTemplate | GroupColumnTemplate)[];
    /**
     * The type of column that the template represents.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html#type)
     */
    readonly type: "group";

    static fromJSON(json: any): GroupColumnTemplate;
  }

  interface GroupColumnTemplateProperties extends ColumnTemplateBaseProperties {
    /**
     * A collection of [field column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html) and/or [group column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html) that represent an ordered list of column templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html#columnTemplates)
     */
    columnTemplates?: (
      | (FieldColumnTemplateProperties & { type: "field" })
      | (GroupColumnTemplateProperties & { type: "group" })
    )[];
  }

  export interface TableTemplate extends Accessor, JSONSupport {}

  export class TableTemplate {
    /**
     * A TableTemplate formats and defines the content of a [FeatureTable](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html)
     */

    constructor(properties?: TableTemplateProperties);

    /**
     * A collection of [field column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html) and/or [group column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html) that represent an ordered list of column templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html#columnTemplates)
     */
    columnTemplates: (FieldColumnTemplate | GroupColumnTemplate | ColumnTemplate)[];

    /**
     * Creates a deep clone of the TableTemplate class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html#clone)
     */
    clone(): TableTemplate;

    static fromJSON(json: any): TableTemplate;
  }

  interface TableTemplateProperties {
    /**
     * A collection of [field column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-FieldColumnTemplate.html) and/or [group column templates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-GroupColumnTemplate.html) that represent an ordered list of column templates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable-support-TableTemplate.html#columnTemplates)
     */
    columnTemplates?: (
      | (FieldColumnTemplateProperties & { type: "field" })
      | (GroupColumnTemplateProperties & { type: "group" })
      | (ColumnTemplateProperties & { type: "column" })
    )[];
  }

  export interface FeatureTableCellClickEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-click";
  }

  export interface FeatureTableCellDblclickEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-dblclick";
  }

  export interface FeatureTableCellKeydownEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-keydown";
  }

  export interface FeatureTableCellPointeroutEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-pointerout";
  }

  export interface FeatureTableCellPointeroverEvent {
    feature: Graphic;

    fieldName: string;

    index: number;

    native: MouseEvent | any | any;

    objectId: number | string;

    type: "cell-pointerover";
  }

  export interface FeatureTableColumnReorderEvent {
    type: "column-reorder";
  }

  /**
   * The configuration for the table's menu.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuConfig)
   */
  export interface TableMenuConfig {
    /**
     * Indicates whether the menu is disabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuConfig)
     */
    disabled?: boolean;
    /**
     * The menu items within the table menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuConfig)
     */
    items: TableMenuItemConfig[];
    /**
     * Indicates whether the menu is open.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuConfig)
     */
    open?: boolean;
  }

  /**
   * The configuration for an individual menu item within the table menu.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
   */
  export interface TableMenuItemConfig {
    /**
     * The text displayed for the menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    label: string;
    /**
     * Indicates whether the menu item is disabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    disabled?: boolean;
    /**
     * The string value indicating the [Calcite icon](https://developers.arcgis.com/calcite-design-system/icons/) displayed for the menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    icon?: string;
    /**
     * This is the CSS class for the icon displayed for the menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    iconClass?: string;
    /**
     * Indicates whether the menu item is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    selected?: boolean;

    /**
     * The function that executes when the menu item is clicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    clickFunction(): void;
    /**
     * A function that determines whether the menu item is hidden.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#TableMenuItemConfig)
     */
    hidden(): void;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
   */
  export interface FeatureTableVisibleElements {
    /**
     * _(Since 4.31)_ Indicates whether to display descriptions in the header cells of individual columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    columnDescriptions?: boolean;
    /**
     * _(Since 4.23)_ Indicates whether to display interactive menus in the header cells of individual columns.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    columnMenus?: boolean;
    /**
     * _(Since 4.30)_ The menu items within the column menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    columnMenuItems?: VisibleElementsColumnMenuItems;
    /**
     * Indicates whether to display the feature table's header information.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    header?: boolean;
    /**
     * _(Since 4.30)_ Indicates whether to display the feature table's layer switch drop down menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    layerDropdown?: boolean;
    /**
     * Indicates whether to display the feature table's menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    menu?: boolean;
    /**
     * The menu items within the feature table menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    menuItems?: VisibleElementsMenuItems;
    /**
     * _(Since 4.30)_ Indicates whether to display the progress indicator when the table is querying or syncing data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    progress?: boolean;
    /**
     * Indicates whether to display the selection column in the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    selectionColumn?: boolean;
    /**
     * _(Since 4.30)_ Indicates whether to display the tooltip for the Show/hide columns button.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    tooltips?: boolean;
  }

  export interface VisibleElementsColumnMenuItems {
    /**
     * _(Since 4.30)_ Indicates whether to display the `Sort Ascending` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    sortAscending?: boolean;
    /**
     * _(Since 4.30)_ Indicates whether to display the `Sort Descending` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    sortDescending?: boolean;
  }

  export interface VisibleElementsMenuItems {
    /**
     * Indicates whether to display the `Clear selection` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    clearSelection?: boolean;
    /**
     * _(Since 4.25)_ Indicates whether to display the `Delete Selection` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    deleteSelection?: boolean;
    /**
     * _(Since 4.31)_ Indicates whether to display the `Export selection to CSV` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    exportSelectionToCSV?: boolean;
    /**
     * Indicates whether to display the `Refresh data` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    refreshData?: boolean;
    /**
     * _(Since 4.23)_ Indicates whether to toggle between showing only selected records in the table to showing all of the records.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    selectedRecordsShowAllToggle?: boolean;
    /**
     * _(Since 4.23)_ Indicates whether to display the `Show selected records` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    selectedRecordsShowSelectedToggle?: boolean;
    /**
     * Indicates whether to enable toggling column visibility within the menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    toggleColumns?: boolean;
    /**
     * _(Since 4.23)_ Indicates whether to display the `Zoom to selected` menu item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTable.html#VisibleElements)
     */
    zoomToSelection?: boolean;
  }

  export class FeatureTemplates extends Widget {
    /**
     * The FeatureTemplates widget is part of the overall editing workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html)
     */

    constructor(properties?: FeatureTemplatesProperties);

    /**
     * Used to disable the widget user interface.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether the list of available feature [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) should scroll within its containing element.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#enableListScroll)
     */
    enableListScroll: boolean;
    /**
     * [Function](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#FilterFunction) can be defined to help filter [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#filterFunction)
     */
    filterFunction: FilterFunction;
    /**
     * Text used to filter items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#filterText)
     */
    filterText: string;
    /**
     * It is possible to group [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html).
     *
     * @default "layer"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#groupBy)
     */
    groupBy: string | GroupByFunction;
    /**
     * Indicates the heading level to use for the labels of grouped feature templates.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#headingLevel)
     */
    headingLevel: number;
    /**
     * An array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#layers)
     */
    layers: (FeatureLayer | GeoJSONLayer | SceneLayer | SubtypeSublayer | SubtypeGroupLayer)[];
    /**
     * Specifies the selection behavior of list items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#selectionMode)
     */
    selectionMode: "single" | "none";
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#viewModel)
     */
    viewModel: FeatureTemplatesViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#visibleElements)
     */
    visibleElements: FeatureTemplatesVisibleElements;

    /**
     * Selects the [template item](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#select)
     */
    select(item: TemplateItem): void;

    on(name: "select", eventHandler: FeatureTemplatesSelectEventHandler): IHandle;
  }

  interface FeatureTemplatesProperties extends WidgetProperties {
    /**
     * Used to disable the widget user interface.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether the list of available feature [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) should scroll within its containing element.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#enableListScroll)
     */
    enableListScroll?: boolean;
    /**
     * [Function](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#FilterFunction) can be defined to help filter [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#filterFunction)
     */
    filterFunction?: FilterFunction;
    /**
     * Text used to filter items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#filterText)
     */
    filterText?: string;
    /**
     * It is possible to group [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html).
     *
     * @default "layer"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#groupBy)
     */
    groupBy?: string | GroupByFunction;
    /**
     * Indicates the heading level to use for the labels of grouped feature templates.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * An array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) to display within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#layers)
     */
    layers?: (
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (SceneLayerProperties & { type: "scene" })
      | (SubtypeSublayerProperties & { type: "subtype-sublayer" })
      | (SubtypeGroupLayerProperties & { type: "subtype-group" })
    )[];
    /**
     * Specifies the selection behavior of list items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#selectionMode)
     */
    selectionMode?: "single" | "none";
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#viewModel)
     */
    viewModel?: FeatureTemplatesViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#visibleElements)
     */
    visibleElements?: FeatureTemplatesVisibleElements;
  }

  export interface FeatureTemplatesViewModel extends Accessor, Evented {}

  export class FeatureTemplatesViewModel {
    /**
     * Provides the logic for the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html)
     */

    constructor(properties?: FeatureTemplatesViewModelProperties);

    /**
     * Used to disable the associated user interface.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#disabled)
     */
    disabled: boolean;
    /**
     * [Function](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#FilterFunction) can be defined to help filter [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#filterFunction)
     */
    filterFunction: FilterFunction;
    /**
     * It is possible to group [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html).
     *
     * @default "layer"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#groupBy)
     */
    groupBy: string | GroupByFunction;
    /**
     * The template items or grouped template items.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#items)
     */
    readonly items: TemplateItem[] | TemplateItemGroup[];
    /**
     * An array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) that are associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#layers)
     */
    layers: (FeatureLayer | GeoJSONLayer | SceneLayer | SubtypeSublayer | SubtypeGroupLayer)[];
    /**
     * The widget's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#state)
     */
    readonly state: "ready" | "loading" | "disabled";

    /**
     * This method updates the [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) with the provided filter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#refresh)
     */
    refresh(): void;
    /**
     * Selects the [template item](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#select)
     */
    select(item: TemplateItem): void;

    on(name: "select", eventHandler: FeatureTemplatesViewModelSelectEventHandler): IHandle;
  }

  interface FeatureTemplatesViewModelProperties {
    /**
     * Used to disable the associated user interface.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#disabled)
     */
    disabled?: boolean;
    /**
     * [Function](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#FilterFunction) can be defined to help filter [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#filterFunction)
     */
    filterFunction?: FilterFunction;
    /**
     * It is possible to group [template items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html).
     *
     * @default "layer"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#groupBy)
     */
    groupBy?: string | GroupByFunction;
    /**
     * An array of [FeatureLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) that are associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-FeatureTemplatesViewModel.html#layers)
     */
    layers?: (
      | (FeatureLayerProperties & { type: "feature" })
      | (GeoJSONLayerProperties & { type: "geojson" })
      | (SceneLayerProperties & { type: "scene" })
      | (SubtypeSublayerProperties & { type: "subtype-sublayer" })
      | (SubtypeGroupLayerProperties & { type: "subtype-group" })
    )[];
  }

  export interface FeatureTemplatesViewModelSelectEvent {
    item: TemplateItem;

    template: FeatureTemplate;
  }

  export class TemplateItem extends Accessor {
    /**
     * The item displayed within the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html)
     */

    constructor(properties?: TemplateItemProperties);

    /**
     * The description that is displayed for the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#description)
     */
    description: string;
    /**
     * The label that is displayed for the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#label)
     */
    label: string;
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) associated with the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#layer)
     */
    layer: FeatureLayer;
    /**
     * The associated [FeatureTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html) for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#template)
     */
    template: FeatureTemplate;
    /**
     * An object used to create a thumbnail image that represents a feature type in the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#thumbnail)
     */
    thumbnail: HTMLElement;

    /**
     * Creates a deep clone of the template item object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#clone)
     */
    clone(): TemplateItem;

    /**
     * Fetches the thumbnail used to display the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#fetchThumbnail)
     */
    static fetchThumbnail(): Promise<any>;
  }

  interface TemplateItemProperties {
    /**
     * The description that is displayed for the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#description)
     */
    description?: string;
    /**
     * The label that is displayed for the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#label)
     */
    label?: string;
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) associated with the template item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#layer)
     */
    layer?: FeatureLayerProperties;
    /**
     * The associated [FeatureTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-support-FeatureTemplate.html) for the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#template)
     */
    template?: FeatureTemplateProperties;
    /**
     * An object used to create a thumbnail image that represents a feature type in the feature template.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html#thumbnail)
     */
    thumbnail?: HTMLElement;
  }

  export interface TemplateItemGroup extends Accessor, Identifiable {}

  export class TemplateItemGroup {
    /**
     * This is a read-only support class that represents a group of items displayed within the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItemGroup.html)
     */

    constructor(properties?: TemplateItemGroupProperties);

    /**
     * An array of [TemplateItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html) grouped to display in the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItemGroup.html#items)
     */
    readonly items: TemplateItem[];
    /**
     * The label displayed in the [FeatureTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html) widget indicating the grouped [TemplateItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItemGroup.html#label)
     */
    readonly label: string;

    /**
     * Notifies the tracking system that `filterFunction` has changed and that `items` may therefore need to be recomputed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates-TemplateItemGroup.html#reapplyFilter)
     */
    reapplyFilter(): void;
  }

  interface TemplateItemGroupProperties {}

  export type FilterFunction = (filterName: any) => boolean;

  export type GroupByFunction = (grouping: any) => string | any;

  export interface FeatureTemplatesSelectEvent {
    item: TemplateItem;

    template: FeatureTemplate;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#VisibleElements)
   */
  export interface FeatureTemplatesVisibleElements {
    /**
     * Indicates whether to the filter will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FeatureTemplates.html#VisibleElements)
     */
    filter?: boolean;
  }

  export class FloorFilter extends Widget {
    /**
     * The FloorFilter widget simplifies visualization of GIS data for a specific floor of a building in your application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html)
     */

    constructor(properties?: FloorFilterProperties);

    /**
     * The currently selected facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#facility)
     */
    facility: string;
    /**
     * Indicates the heading level to use for the headings separating floors in buildings.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#headingLevel)
     */
    headingLevel: number;
    /**
     * The currently selected floor level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#level)
     */
    level: string;
    /**
     * Determines if the widget is expanded or collapsed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#longNames)
     */
    longNames: boolean;
    /**
     * The currently selected site.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#site)
     */
    site: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#viewModel)
     */
    viewModel: FloorFilterViewModel;

    /**
     * Updates the [floorFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html#FloorFilter) widget definition in the provided [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#updateWebDocument)
     */
    updateWebDocument(webmap: WebMap): void;
  }

  interface FloorFilterProperties extends WidgetProperties {
    /**
     * The currently selected facility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#facility)
     */
    facility?: string;
    /**
     * Indicates the heading level to use for the headings separating floors in buildings.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * The currently selected floor level.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#level)
     */
    level?: string;
    /**
     * Determines if the widget is expanded or collapsed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#longNames)
     */
    longNames?: boolean;
    /**
     * The currently selected site.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#site)
     */
    site?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html#viewModel)
     */
    viewModel?: FloorFilterViewModelProperties;
  }

  export interface FloorFilterViewModel extends Accessor, GoTo {}

  export class FloorFilterViewModel {
    /**
     * Provides the logic for the [FloorFilter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html)
     */

    constructor(properties?: FloorFilterViewModelProperties);

    /**
     * The currently selected facility.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#facility)
     */
    facility: string;
    /**
     * The currently selected level.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#level)
     */
    level: string;
    /**
     * Determines if the widget is expanded or collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#longNames)
     */
    longNames: boolean;
    /**
     * The currently selected site.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#site)
     */
    site: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface FloorFilterViewModelProperties extends GoToProperties {
    /**
     * The currently selected facility.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#facility)
     */
    facility?: string;
    /**
     * The currently selected level.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#level)
     */
    level?: string;
    /**
     * Determines if the widget is expanded or collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#longNames)
     */
    longNames?: boolean;
    /**
     * The currently selected site.
     *
     * @default undefined
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#site)
     */
    site?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-FloorFilter-FloorFilterViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class Fullscreen extends Widget {
    /**
     * Provides a simple widget to present the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) or a user-defined [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) using the entire screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html)
     */

    constructor(properties?: FullscreenProperties);

    /**
     * The [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) to present in fullscreen mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#element)
     */
    element: HTMLElement;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#viewModel)
     */
    viewModel: FullscreenViewModel;
  }

  interface FullscreenProperties extends WidgetProperties {
    /**
     * The [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) to present in fullscreen mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#element)
     */
    element?: HTMLElement;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html#viewModel)
     */
    viewModel?: FullscreenViewModelProperties;
  }

  export class FullscreenViewModel extends Accessor {
    /**
     * Provides the logic for the [Fullscreen](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html)
     */

    constructor(properties?: FullscreenViewModelProperties);

    /**
     * The [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) to present in fullscreen mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#element)
     */
    element: HTMLElement;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#state)
     */
    readonly state: "active" | "ready" | "feature-unsupported" | "disabled";
    /**
     * The view associated with the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Enter fullscreen
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#enter)
     */
    enter(): void;
    /**
     * Exit fullscreen
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#exit)
     */
    exit(): void;
    /**
     * Toggle fullscreen
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#toggle)
     */
    toggle(): void;
  }

  interface FullscreenViewModelProperties {
    /**
     * The [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) to present in fullscreen mode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#element)
     */
    element?: HTMLElement;
    /**
     * The view associated with the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Fullscreen-FullscreenViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class Histogram extends Widget {
    /**
     * Renders a histogram to visualize the spread of a dataset based on [bins](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#bins) representing buckets, or sub-ranges, of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html)
     */

    constructor(properties?: HistogramProperties);

    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#average)
     */
    average: number;
    /**
     * Function for styling bars representing histogram bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#barCreatedFunction)
     */
    barCreatedFunction: BarCreatedFunction;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#bins)
     */
    bins: Bin[];
    /**
     * Function that fires each time a data line is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLineCreatedFunction)
     */
    dataLineCreatedFunction: DataLineCreatedFunction;
    /**
     * When set, renders lines on the histogram that indicate important or meaningful values to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLines)
     */
    dataLines: HistogramDataLines[];
    /**
     * A function used to format labels on the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#labelFormatFunction)
     */
    labelFormatFunction: LabelFormatter;
    /**
     * Determines the orientation of the Histogram widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#layout)
     */
    layout: "vertical" | "horizontal";
    /**
     * The maximum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#max)
     */
    max: number;
    /**
     * The minimum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#min)
     */
    min: number;
    /**
     * The state of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * The view model for the Histogram widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#viewModel)
     */
    viewModel: HistogramViewModel;

    /**
     * A convenience function used to create a Histogram widget instance from the result of the [histogram](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-statistics-histogram.html) statistics function.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#fromHistogramResult)
     */
    static fromHistogramResult(result: HistogramResult): Histogram;
  }

  interface HistogramProperties extends WidgetProperties {
    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#average)
     */
    average?: number;
    /**
     * Function for styling bars representing histogram bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#barCreatedFunction)
     */
    barCreatedFunction?: BarCreatedFunction;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#bins)
     */
    bins?: Bin[];
    /**
     * Function that fires each time a data line is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLineCreatedFunction)
     */
    dataLineCreatedFunction?: DataLineCreatedFunction;
    /**
     * When set, renders lines on the histogram that indicate important or meaningful values to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLines)
     */
    dataLines?: HistogramDataLines[];
    /**
     * A function used to format labels on the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#labelFormatFunction)
     */
    labelFormatFunction?: LabelFormatter;
    /**
     * Determines the orientation of the Histogram widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#layout)
     */
    layout?: "vertical" | "horizontal";
    /**
     * The maximum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#max)
     */
    max?: number;
    /**
     * The minimum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#min)
     */
    min?: number;
    /**
     * The view model for the Histogram widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#viewModel)
     */
    viewModel?: HistogramViewModelProperties;
  }

  export class HistogramViewModel extends Accessor {
    /**
     * Provides the logic for the [Histogram](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html)
     */

    constructor(properties?: HistogramViewModelProperties);

    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#average)
     */
    average: number;
    /**
     * The range of values for the histogram calculated from the bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#binRange)
     */
    readonly binRange: number;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#bins)
     */
    bins: Bin[];
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#labelFormatFunction)
     */
    labelFormatFunction: LabelFormatter;
    /**
     * The maximum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#max)
     */
    max: number;
    /**
     * The minimum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#min)
     */
    min: number;
    /**
     * The range of values for the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#range)
     */
    readonly range: number;
    /**
     * The current state of the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
  }

  interface HistogramViewModelProperties {
    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#average)
     */
    average?: number;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#bins)
     */
    bins?: Bin[];
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#labelFormatFunction)
     */
    labelFormatFunction?: LabelFormatter;
    /**
     * The maximum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#max)
     */
    max?: number;
    /**
     * The minimum value or bound of the entire histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram-HistogramViewModel.html#min)
     */
    min?: number;
  }

  export type BarCreatedFunction = (index: number, element: any) => void;

  /**
   * Represents the bin of a histogram.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#Bin)
   */
  export interface Bin {
    /**
     * The number of values in a dataset that are contained between the indicated `minValue` and `maxValue`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#Bin)
     */
    count: number;
    /**
     * The minimum value (or bound) of the bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#Bin)
     */
    minValue: number;
    /**
     * The maximum value (or bound) of the bin.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#Bin)
     */
    maxValue: number;
  }

  export type DataLineCreatedFunction = (lineElement: any, labelElement?: any, index?: number) => void;

  export interface HistogramDataLines {
    /**
     * The value on the data axis of the histogram where a line will be rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLines)
     */
    value: number;
    /**
     * The label associated with the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html#dataLines)
     */
    label?: string | number;
  }

  export type LabelFormatter = (value: number, type?: string, index?: number) => string;

  export class HistogramRangeSlider extends Widget {
    /**
     * A slider widget that can be used for filtering data or gathering numeric input from a user for a range of data.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html)
     */

    constructor(properties?: HistogramRangeSliderProperties);

    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#average)
     */
    average: number;
    /**
     * Function for styling bars representing histogram bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#barCreatedFunction)
     */
    barCreatedFunction: BarCreatedFunction;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#bins)
     */
    bins: Bin[];
    /**
     * Function that fires each time a data line is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLineCreatedFunction)
     */
    dataLineCreatedFunction: DataLineCreatedFunction;
    /**
     * When set, renders lines on the histogram that indicate important or meaningful values to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLines)
     */
    dataLines: HistogramRangeSliderDataLines[];
    /**
     * Sets the color of the histogram bars that are excluded based on the specified [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType).
     *
     * @default "#d7e5f0"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#excludedBarColor)
     */
    excludedBarColor: Color;
    /**
     * Sets the color of the histogram bars that are included in the specified [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType).
     *
     * @default "#599dd4"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#includedBarColor)
     */
    includedBarColor: Color;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#labelFormatFunction)
     */
    labelFormatFunction: SliderLabelFormatter;
    /**
     * The maximum value or upper bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#max)
     */
    max: number;
    /**
     * The minimum value or lower bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#min)
     */
    min: number;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#precision)
     */
    precision: number;
    /**
     * Indicates how the histogram bins should be rendered as the user slides the thumbs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType)
     */
    rangeType:
      | "equal"
      | "not-equal"
      | "less-than"
      | "greater-than"
      | "at-most"
      | "at-least"
      | "between"
      | "not-between";
    /**
     * Indicates the standard deviation of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#standardDeviation)
     */
    standardDeviation: number;
    /**
     * Indicates the number of standard deviation lines to render on the histogram from the [average].
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#standardDeviationCount)
     */
    standardDeviationCount: number;
    /**
     * An array of either one or two numbers representing thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#values)
     */
    values: number[];
    /**
     * The view model for the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#viewModel)
     */
    viewModel: HistogramRangeSliderViewModel;

    /**
     * Generates a SQL where clause based on a given field and the slider's [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#generateWhereClause)
     */
    generateWhereClause(field: string): string;

    on(name: "max-change", eventHandler: HistogramRangeSliderMaxChangeEventHandler): IHandle;

    on(name: "min-change", eventHandler: HistogramRangeSliderMinChangeEventHandler): IHandle;

    on(name: "segment-drag", eventHandler: HistogramRangeSliderSegmentDragEventHandler): IHandle;

    on(name: "thumb-change", eventHandler: HistogramRangeSliderThumbChangeEventHandler): IHandle;

    on(name: "thumb-drag", eventHandler: HistogramRangeSliderThumbDragEventHandler): IHandle;
  }

  interface HistogramRangeSliderProperties extends WidgetProperties {
    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#average)
     */
    average?: number;
    /**
     * Function for styling bars representing histogram bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#barCreatedFunction)
     */
    barCreatedFunction?: BarCreatedFunction;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#bins)
     */
    bins?: Bin[];
    /**
     * Function that fires each time a data line is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLineCreatedFunction)
     */
    dataLineCreatedFunction?: DataLineCreatedFunction;
    /**
     * When set, renders lines on the histogram that indicate important or meaningful values to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLines)
     */
    dataLines?: HistogramRangeSliderDataLines[];
    /**
     * Sets the color of the histogram bars that are excluded based on the specified [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType).
     *
     * @default "#d7e5f0"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#excludedBarColor)
     */
    excludedBarColor?: Color | number[] | string;
    /**
     * Sets the color of the histogram bars that are included in the specified [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType).
     *
     * @default "#599dd4"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#includedBarColor)
     */
    includedBarColor?: Color | number[] | string;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#labelFormatFunction)
     */
    labelFormatFunction?: SliderLabelFormatter;
    /**
     * The maximum value or upper bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#max)
     */
    max?: number;
    /**
     * The minimum value or lower bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#min)
     */
    min?: number;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#precision)
     */
    precision?: number;
    /**
     * Indicates how the histogram bins should be rendered as the user slides the thumbs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#rangeType)
     */
    rangeType?:
      | "equal"
      | "not-equal"
      | "less-than"
      | "greater-than"
      | "at-most"
      | "at-least"
      | "between"
      | "not-between";
    /**
     * Indicates the standard deviation of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#standardDeviation)
     */
    standardDeviation?: number;
    /**
     * Indicates the number of standard deviation lines to render on the histogram from the [average].
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#standardDeviationCount)
     */
    standardDeviationCount?: number;
    /**
     * An array of either one or two numbers representing thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#values)
     */
    values?: number[];
    /**
     * The view model for the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#viewModel)
     */
    viewModel?: HistogramRangeSliderViewModelProperties;
  }

  export class HistogramRangeSliderViewModel extends SliderViewModel {
    /**
     * Provides the logic for the [HistogramRangeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html)
     */

    constructor(properties?: HistogramRangeSliderViewModelProperties);

    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#average)
     */
    average: number;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#bins)
     */
    bins: Bin[];
    /**
     * Determines the SQL where clause generated in [generateWhereClause()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#generateWhereClause) for filtering purposes.
     *
     * @default "equal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#rangeType)
     */
    rangeType:
      | "equal"
      | "not-equal"
      | "less-than"
      | "greater-than"
      | "at-most"
      | "at-least"
      | "between"
      | "not-between";
    /**
     * Indicates the standard deviation of the dataset above and below the `average`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#standardDeviation)
     */
    standardDeviation: number;

    /**
     * Generates a SQL where clause based on a given field and the slider's [rangeType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#rangeType).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#generateWhereClause)
     */
    generateWhereClause(field: string): string;
  }

  interface HistogramRangeSliderViewModelProperties extends SliderViewModelProperties {
    /**
     * The statistical average of the data in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#average)
     */
    average?: number;
    /**
     * An array of objects representing each bin in the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#bins)
     */
    bins?: Bin[];
    /**
     * Determines the SQL where clause generated in [generateWhereClause()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#generateWhereClause) for filtering purposes.
     *
     * @default "equal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#rangeType)
     */
    rangeType?:
      | "equal"
      | "not-equal"
      | "less-than"
      | "greater-than"
      | "at-most"
      | "at-least"
      | "between"
      | "not-between";
    /**
     * Indicates the standard deviation of the dataset above and below the `average`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider-HistogramRangeSliderViewModel.html#standardDeviation)
     */
    standardDeviation?: number;
  }

  export interface HistogramRangeSliderDataLines {
    /**
     * The value on the data axis of the histogram where a line will be rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLines)
     */
    value: number;
    /**
     * The label associated with the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-HistogramRangeSlider.html#dataLines)
     */
    label?: string | number;
  }

  export interface HistogramRangeSliderMaxChangeEvent {
    oldValue: number;

    type: "max-change";

    value: number;
  }

  export interface HistogramRangeSliderMinChangeEvent {
    oldValue: number;

    type: "min-change";

    value: number;
  }

  export interface HistogramRangeSliderSegmentDragEvent {
    index: number;

    state: "start" | "drag";

    thumbIndices: number[];

    type: "segment-drag";
  }

  export interface HistogramRangeSliderThumbChangeEvent {
    index: number;

    oldValue: number;

    type: "thumb-change";

    value: number;
  }

  export interface HistogramRangeSliderThumbDragEvent {
    index: number;

    state: "start" | "drag";

    type: "thumb-drag";

    value: number;
  }

  export interface Home extends Widget, GoTo {}

  export class Home {
    /**
     * Provides a simple widget that switches the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to its initial [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or a previously defined [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewpoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html)
     */

    constructor(properties?: HomeProperties);

    /**
     * Overwrite localized strings for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#uiStrings)
     */
    uiStrings: any;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewModel)
     */
    viewModel: HomeViewModel;
    /**
     * The [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), or point of view, to zoom to when going home.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewpoint)
     */
    viewpoint: Viewpoint;

    /**
     * This function provides the ability to interrupt and cancel the process of navigating the view back to the view's initial extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#cancelGo)
     */
    cancelGo(): void;
    /**
     * Animates the view to the initial Viewpoint of the view or the value of [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewpoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#go)
     */
    go(): void;

    on(name: "go", eventHandler: HomeGoEventHandler): IHandle;
  }

  interface HomeProperties extends WidgetProperties, GoToProperties {
    /**
     * Overwrite localized strings for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#uiStrings)
     */
    uiStrings?: any;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewModel)
     */
    viewModel?: HomeViewModelProperties;
    /**
     * The [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), or point of view, to zoom to when going home.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
  }

  export interface HomeViewModel extends Accessor, Evented, GoTo {}

  export class HomeViewModel {
    /**
     * Provides the logic for the [Home](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home.html) widget that animates the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to its initial [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) or a previously defined [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#viewpoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html)
     */

    constructor(properties?: HomeViewModelProperties);

    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "going-home";
    /**
     * The view associated with the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#view)
     */
    view: MapView | SceneView;
    /**
     * The [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), or point of view, to zoom to when going home.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#viewpoint)
     */
    viewpoint: Viewpoint;

    /**
     * This function provides the ability to interrupt and cancel the process of navigating the view back to the view's initial extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#cancelGo)
     */
    cancelGo(): void;
    /**
     * Animates the view to the initial [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) of the view or the value of [viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#viewpoint).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#go)
     */
    go(): void;

    on(name: "go", eventHandler: HomeViewModelGoEventHandler): IHandle;
  }

  interface HomeViewModelProperties extends GoToProperties {
    /**
     * The view associated with the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html), or point of view, to zoom to when going home.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Home-HomeViewModel.html#viewpoint)
     */
    viewpoint?: ViewpointProperties;
  }

  export interface HomeViewModelGoEvent {}

  export interface HomeGoEvent {}

  export class LayerList extends Widget {
    /**
     * The LayerList widget provides a way to display a list of layers, and switch on/off their visibility.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html)
     */

    constructor(properties?: LayerListProperties);

    /**
     * The [CatalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-CatalogLayerList.html) widget instance that displays a catalog layer's [dynamic group layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html#dynamicGroupLayer).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogLayerList)
     */
    readonly catalogLayerList: CatalogLayerList;
    /**
     * [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    catalogOptions: LayerListCatalogOptions;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#collapsed)
     */
    collapsed: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#dragEnabled)
     */
    dragEnabled: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#filterPlaceholder)
     */
    filterPlaceholder: string;
    /**
     * The value of the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#filterText)
     */
    filterText: string;
    /**
     * Indicates the heading level to use for the heading of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#headingLevel)
     */
    headingLevel: number;
    /**
     * [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    knowledgeGraphOptions: LayerListKnowledgeGraphOptions;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: LayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to enable drag and drop reordering with [dragEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#dragEnabled).
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#minDragEnabledItems)
     */
    minDragEnabledItems: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#minFilterItems)
     */
    minFilterItems: number;
    /**
     * Indicates whether more than one list item may be selected by the user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/LayerList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled: boolean;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that are opened in a [catalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogLayerList) or [tableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#tableList) flow item.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#openedLayers)
     */
    openedLayers: Collection<Layer>;
    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing operational layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#operationalItems)
     */
    readonly operationalItems: Collection<ListItem>;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing operational layers selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectedItems)
     */
    selectedItems: Collection<ListItem>;
    /**
     * Indicates whether list items may be selected by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/LayerList#selectionMode selectionMode} and {@link module:esri/widgets/LayerList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectionEnabled)
     */
    selectionEnabled: boolean;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectionMode)
     */
    selectionMode: "multiple" | "none" | "single" | "single-persist";
    /**
     * The [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html) widget instance that displays the tables associated with a [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#tableList)
     */
    readonly tableList: TableList;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#viewModel)
     */
    viewModel: LayerListViewModel;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibilityAppearance)
     */
    visibilityAppearance: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements)
     */
    visibleElements: LayerListVisibleElements;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: ListItem): void;

    on(name: "trigger-action", eventHandler: LayerListTriggerActionEventHandler): IHandle;
  }

  interface LayerListProperties extends WidgetProperties {
    /**
     * [CatalogLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CatalogLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    catalogOptions?: LayerListCatalogOptions;
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#collapsed)
     */
    collapsed?: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#dragEnabled)
     */
    dragEnabled?: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#filterPlaceholder)
     */
    filterPlaceholder?: string;
    /**
     * The value of the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#filterText)
     */
    filterText?: string;
    /**
     * Indicates the heading level to use for the heading of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * [KnowledgeGraphLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-KnowledgeGraphLayer.html) specific properties.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    knowledgeGraphOptions?: LayerListKnowledgeGraphOptions;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: LayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to enable drag and drop reordering with [dragEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#dragEnabled).
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#minDragEnabledItems)
     */
    minDragEnabledItems?: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#minFilterItems)
     */
    minFilterItems?: number;
    /**
     * Indicates whether more than one list item may be selected by the user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/LayerList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that are opened in a [catalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogLayerList) or [tableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#tableList) flow item.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#openedLayers)
     */
    openedLayers?: CollectionProperties<LayerProperties>;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing operational layers selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectedItems)
     */
    selectedItems?: CollectionProperties<ListItemProperties>;
    /**
     * Indicates whether list items may be selected by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/LayerList#selectionMode selectionMode} and {@link module:esri/widgets/LayerList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectionEnabled)
     */
    selectionEnabled?: boolean;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#selectionMode)
     */
    selectionMode?: "multiple" | "none" | "single" | "single-persist";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#viewModel)
     */
    viewModel?: LayerListViewModelProperties;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibilityAppearance)
     */
    visibilityAppearance?: "default" | "checkbox";
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#visibleElements)
     */
    visibleElements?: LayerListVisibleElements;
  }

  export class LayerListViewModel extends Accessor {
    /**
     * Provides the logic for the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html)
     */

    constructor(properties?: LayerListViewModelProperties);

    /**
     * Whether to provide an indication if a layer is being published in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled: boolean;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: LayerListListItemCreatedHandler;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all layers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#listModeDisabled)
     */
    listModeDisabled: boolean;
    /**
     * A collection of [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)s representing operational layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#operationalItems)
     */
    readonly operationalItems: Collection<ListItem>;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#state)
     */
    readonly state: "loading" | "ready" | "disabled";
    /**
     * The view from which the widget will operate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Moves a list item from one position to another in the LayerList widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#moveListItem)
     */
    moveListItem(targetItem: ListItem, fromParentItem: ListItem, toParentItem: ListItem, newIndex: number): void;
    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: ListItem): void;

    on(name: "trigger-action", eventHandler: LayerListViewModelTriggerActionEventHandler): IHandle;
  }

  interface LayerListViewModelProperties {
    /**
     * Whether to provide an indication if a layer is being published in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled?: boolean;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: LayerListListItemCreatedHandler;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all layers.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#listModeDisabled)
     */
    listModeDisabled?: boolean;
    /**
     * The view from which the widget will operate.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface LayerListViewModelTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: ListItem;
  }

  export interface ListItem extends Accessor, Identifiable {}

  export class ListItem {
    /**
     * The ListItem class represents one of the [operationalItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html#operationalItems) in the [LayerListViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-LayerListViewModel.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html)
     */

    constructor(properties?: ListItemProperties);

    /**
     * Whether the actions panel is open in the LayerList.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#actionsOpen)
     */
    actionsOpen: boolean;
    /**
     * A nested 2-dimensional collection of actions that could be triggered on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#actionsSections)
     */
    actionsSections: Collection<Collection<ActionButton | ActionToggle>>;
    /**
     * When a layer contains sublayers, this property is a Collection of ListItem objects belonging to the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#children)
     */
    children: Collection<ListItem>;
    /**
     * Indicates if the children of a list item (or sublayers in a GroupLayer) can be sorted or moved/reordered.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#childrenSortable)
     */
    childrenSortable: boolean;
    /**
     * Only valid when the list item represents a [StreamLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-StreamLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#connectionStatus)
     */
    readonly connectionStatus: "connected" | "disconnected" | "paused";
    /**
     * The Error object returned if an error occurred.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#error)
     */
    readonly error: Error;
    /**
     * When `true`, hides the layer from the LayerList instance.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#hidden)
     */
    hidden: boolean;
    /**
     * The layer associated with the triggered action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#layer)
     */
    layer: Layer;
    /**
     * The [LayerView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-layers-LayerView.html) displaying data for the associated [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#layer).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#layerView)
     */
    readonly layerView: LayerView;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the child layers in the list item.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#listModeDisabled)
     */
    listModeDisabled: boolean;
    /**
     * Whether the layer is open in the LayerList.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#open)
     */
    open: boolean;
    /**
     * Allows you to display custom content for each ListItem in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#panel)
     */
    panel: ListItemPanel;
    /**
     * The parent of this item
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#parent)
     */
    parent: ListItem;
    /**
     * Value is `true` when the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#layer) is being published.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#publishing)
     */
    readonly publishing: boolean;
    /**
     * Indicates if the list item (or layer in the map) can be sorted or moved/reordered.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#sortable)
     */
    sortable: boolean;
    /**
     * The title of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#title)
     */
    title: string;
    /**
     * Value is `true` when the layer is updating; for example, if it is in the process of fetching data.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#updating)
     */
    readonly updating: boolean;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#view)
     */
    view: MapView | SceneView;
    /**
     * Indicates how to manage the visibility of the children layers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#visibilityMode)
     */
    readonly visibilityMode: string;
    /**
     * Indicates if the ListItem is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#visible)
     */
    visible: boolean;
    /**
     * Whether the layer is visible at the current scale or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#visibleAtCurrentScale)
     */
    readonly visibleAtCurrentScale: boolean;
    /**
     * Whether the layer is visible at the current time extent or not.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#visibleAtCurrentTimeExtent)
     */
    readonly visibleAtCurrentTimeExtent: boolean;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#clone)
     */
    clone(): ListItem;
  }

  interface ListItemProperties {
    /**
     * Whether the actions panel is open in the LayerList.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#actionsOpen)
     */
    actionsOpen?: boolean;
    /**
     * A nested 2-dimensional collection of actions that could be triggered on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#actionsSections)
     */
    actionsSections?:
      | CollectionProperties<
          CollectionProperties<
            (ActionButtonProperties & { type: "button" }) | (ActionToggleProperties & { type: "toggle" })
          >
        >
      | any[][];
    /**
     * When a layer contains sublayers, this property is a Collection of ListItem objects belonging to the given layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#children)
     */
    children?: CollectionProperties<ListItemProperties>;
    /**
     * Indicates if the children of a list item (or sublayers in a GroupLayer) can be sorted or moved/reordered.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#childrenSortable)
     */
    childrenSortable?: boolean;
    /**
     * When `true`, hides the layer from the LayerList instance.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#hidden)
     */
    hidden?: boolean;
    /**
     * The layer associated with the triggered action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#layer)
     */
    layer?: LayerProperties;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the child layers in the list item.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#listModeDisabled)
     */
    listModeDisabled?: boolean;
    /**
     * Whether the layer is open in the LayerList.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#open)
     */
    open?: boolean;
    /**
     * Allows you to display custom content for each ListItem in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#panel)
     */
    panel?: ListItemPanelProperties;
    /**
     * The parent of this item
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#parent)
     */
    parent?: ListItemProperties;
    /**
     * Indicates if the list item (or layer in the map) can be sorted or moved/reordered.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#sortable)
     */
    sortable?: boolean;
    /**
     * The title of the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#title)
     */
    title?: string;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * Indicates if the ListItem is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html#visible)
     */
    visible?: boolean;
  }

  export interface ListItemPanel extends Widget, Identifiable {}

  export class ListItemPanel {
    /**
     * This class allows you to display custom content for each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) in the [LayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html)
     */

    constructor(properties?: ListItemPanelProperties);

    /**
     * Adds a CSS class used to style a node that represents the panel.
     *
     * @default null
     * @deprecated since version 4.30. Use {@link module:esri/widgets/LayerList/ListItemPanel#icon icon}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#className)
     */
    className: string;
    /**
     * The content displayed in the ListItem panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#content)
     */
    content: LayerListItemPanelContent | LayerListItemPanelContent[];
    /**
     * If `true`, disables the ListItem's panel so the user cannot open or interact with it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether the panel's content should be rendered as a [Calcite Flow Item](https://developers.arcgis.com/calcite-design-system/components/flow-item/).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#flowEnabled)
     */
    flowEnabled: boolean;
    /**
     * The URL or data URI of an image used to represent the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#image)
     */
    image: string;
    /**
     * The panel's parent ListItem that represents a layer in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#listItem)
     */
    listItem: ListItem;
    /**
     * Indicates if the panel's content is open and visible to the user.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#open)
     */
    open: boolean;
    /**
     * The title of the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#title)
     */
    title: string;
  }

  interface ListItemPanelProperties extends WidgetProperties {
    /**
     * Adds a CSS class used to style a node that represents the panel.
     *
     * @default null
     * @deprecated since version 4.30. Use {@link module:esri/widgets/LayerList/ListItemPanel#icon icon}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#className)
     */
    className?: string;
    /**
     * The content displayed in the ListItem panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#content)
     */
    content?: WidgetProperties | HTMLElement | string | (WidgetProperties | HTMLElement | string)[];
    /**
     * If `true`, disables the ListItem's panel so the user cannot open or interact with it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether the panel's content should be rendered as a [Calcite Flow Item](https://developers.arcgis.com/calcite-design-system/components/flow-item/).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#flowEnabled)
     */
    flowEnabled?: boolean;
    /**
     * The URL or data URI of an image used to represent the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#image)
     */
    image?: string;
    /**
     * The panel's parent ListItem that represents a layer in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#listItem)
     */
    listItem?: ListItemProperties;
    /**
     * Indicates if the panel's content is open and visible to the user.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#open)
     */
    open?: boolean;
    /**
     * The title of the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#title)
     */
    title?: string;
  }

  /**
   * The content displayed in the ListItem panel.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItemPanel.html#LayerListItemPanelContent)
   */
  export type LayerListItemPanelContent = Widget | HTMLElement | string;

  export interface LayerListCatalogOptions {
    /**
     * Placeholder text used in the filter input box.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    filterPlaceholder?: string;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList-ListItem.html) is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    listItemCreatedFunction?: CatalogLayerListListItemCreatedHandler;
    /**
     * The minimum number of list items required to display the filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    minFilterItems?: number;
    /**
     * The selection mode for the catalog layer list.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    selectionMode?: string;
    /**
     * Determines the icons used to indicate visibility.
     *
     * @default "default"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    visibilityAppearance?: string;
    /**
     * The visible elements that are displayed within the catalog layer list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    visibleElements?: LayerListCatalogOptionsVisibleElements;
  }

  export interface LayerListCatalogOptionsVisibleElements {
    /**
     * Indicates whether to display errors in the catalog layer list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    errors?: boolean;
    /**
     * Indicates whether to display the filter input box in the catalog layer list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    filter?: boolean;
    /**
     * Indicates whether to display status indicators in the catalog layer list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    statusIndicators?: boolean;
    /**
     * Indicates whether temporary layer indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogOptions)
     */
    temporaryLayerIndicators?: boolean;
  }

  export interface LayerListKnowledgeGraphOptions {
    /**
     * Placeholder text used in the filter input box.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    filterPlaceholder?: string;
    /**
     * A function that executes each time a [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    listItemCreatedFunction?: TableListListItemCreatedHandler;
    /**
     * The minimum number of list items required to display the filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    minFilterItems?: number;
    /**
     * The selection mode for the table list.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    selectionMode?: string;
    /**
     * The visible elements that are displayed within the table list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    visibleElements?: LayerListKnowledgeGraphOptionsVisibleElements;
  }

  export interface LayerListKnowledgeGraphOptionsVisibleElements {
    /**
     * Indicates whether to display errors in the table list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    errors?: boolean;
    /**
     * Indicates whether to display the filter input box in the table list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    filter?: boolean;
    /**
     * Indicates whether to display status indicators in the table list.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#knowledgeGraphOptions)
     */
    statusIndicators?: boolean;
  }

  export type LayerListListItemCreatedHandler = (event: any) => void;

  export interface LayerListTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: ListItem;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
   */
  export interface LayerListVisibleElements {
    /**
     * **Since 4.30** Indicates whether to display the [catalogLayerList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#catalogLayerList).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    catalogLayerList?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a close button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a collapse button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * **Since 4.25** Indicates whether layer load errors will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    errors?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a filter input box when then number of list items is equal to or greater than the value set in [minFilterItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#minFilterItems), allowing users to filter layers by their title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    filter?: boolean;
    /**
     * **Since 4.29** Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * **Since 4.29** Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * Indicates whether the status indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    statusIndicators?: boolean;
    /**
     * **Since 4.29** Indicates whether temporary layer indicators will be displayed for layers with [layer.persistenceEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled) set to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LayerList.html#VisibleElements)
     */
    temporaryLayerIndicators?: boolean;
  }

  export class Legend extends Widget {
    /**
     * The Legend widget describes the symbols used to represent layers in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html)
     */

    constructor(properties?: LegendProperties);

    /**
     * Collection of [ActiveLayerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html) objects used by the legend view to display data in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#activeLayerInfos)
     */
    activeLayerInfos: Collection<ActiveLayerInfo>;
    /**
     * Indicates whether to show the [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) layers in the Legend.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#basemapLegendVisible)
     */
    basemapLegendVisible: boolean;
    /**
     * Indicates the heading level to use for the legend title.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#headingLevel)
     */
    headingLevel: number;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView: boolean;
    /**
     * Specifies a subset of the layers to display in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#layerInfos)
     */
    layerInfos: LegendLayerInfos[];
    /**
     * If a layer uses a unique value render, only features that satisfy the layer's [definition expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#definitionExpression) will be displayed in the legend when set to true.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#respectLayerDefinitionExpression)
     */
    respectLayerDefinitionExpression: boolean;
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#respectLayerVisibility)
     */
    respectLayerVisibility: boolean;
    /**
     * Indicates the style of the legend.
     *
     * @default "classic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#style)
     */
    style: string | LegendStyle;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#viewModel)
     */
    viewModel: LegendViewModel;
  }

  interface LegendProperties extends WidgetProperties {
    /**
     * Collection of [ActiveLayerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html) objects used by the legend view to display data in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#activeLayerInfos)
     */
    activeLayerInfos?: CollectionProperties<ActiveLayerInfoProperties>;
    /**
     * Indicates whether to show the [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) layers in the Legend.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#basemapLegendVisible)
     */
    basemapLegendVisible?: boolean;
    /**
     * Indicates the heading level to use for the legend title.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView?: boolean;
    /**
     * Specifies a subset of the layers to display in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#layerInfos)
     */
    layerInfos?: LegendLayerInfos[];
    /**
     * If a layer uses a unique value render, only features that satisfy the layer's [definition expression](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#definitionExpression) will be displayed in the legend when set to true.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#respectLayerDefinitionExpression)
     */
    respectLayerDefinitionExpression?: boolean;
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#respectLayerVisibility)
     */
    respectLayerVisibility?: boolean;
    /**
     * Indicates the style of the legend.
     *
     * @default "classic"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#style)
     */
    style?: string | LegendStyle;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#viewModel)
     */
    viewModel?: LegendViewModelProperties;
  }

  export class LegendViewModel extends Accessor {
    /**
     * Provides the logic for the [Legend](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html) widget, which displays a label and symbol for interpreting the [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html) of each layer in a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html)
     */

    constructor(properties?: LegendViewModelProperties);

    /**
     * Collection of [ActiveLayerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html) objects used by the legend view to display data in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#activeLayerInfos)
     */
    activeLayerInfos: Collection<ActiveLayerInfo>;
    /**
     * Indicates whether to show the [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) layers in the Legend.
     *
     * @default filterBasemaps
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#basemapLegendVisible)
     */
    basemapLegendVisible: boolean;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView: boolean;
    /**
     * Specifies a subset of the layers in the map to display in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#layerInfos)
     */
    layerInfos: LegendViewModelLayerInfos[];
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#respectLayerVisibility)
     */
    respectLayerVisibility: boolean;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface LegendViewModelProperties {
    /**
     * Collection of [ActiveLayerInfo](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html) objects used by the legend view to display data in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#activeLayerInfos)
     */
    activeLayerInfos?: CollectionProperties<ActiveLayerInfoProperties>;
    /**
     * Indicates whether to show the [Basemap](https://developers.arcgis.com/javascript/latest/api-reference/esri-Basemap.html) layers in the Legend.
     *
     * @default filterBasemaps
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#basemapLegendVisible)
     */
    basemapLegendVisible?: boolean;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView?: boolean;
    /**
     * Specifies a subset of the layers in the map to display in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#layerInfos)
     */
    layerInfos?: LegendViewModelLayerInfos[];
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#respectLayerVisibility)
     */
    respectLayerVisibility?: boolean;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface LegendViewModelLayerInfos {
    /**
     * Specify a title for the layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#layerInfos)
     */
    title?: string;
    /**
     * A layer to add to the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-LegendViewModel.html#layerInfos)
     */
    layer?: Layer;
  }

  export class ActiveLayerInfo extends Accessor {
    /**
     * ActiveLayerInfo is added to or removed from the collection of activeLayerInfos as layers become visible or invisible in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html)
     */

    constructor(properties?: ActiveLayerInfoProperties);

    /**
     * A collection of child activeLayerInfos.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#children)
     */
    children: Collection<ActiveLayerInfo>;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView: boolean;
    /**
     * Indicates if the legend's display of the layer's renderer is driven by the scale of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#isScaleDriven)
     */
    isScaleDriven: boolean;
    /**
     * The layer represented by the ActiveLayerInfo object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layer)
     */
    layer: Layer;
    /**
     * The layerView represented by the ActiveLayerInfo object's layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layerView)
     */
    layerView: LayerView;
    /**
     * The legendElements is constructed using the layer [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElements)
     */
    legendElements: LegendElement[];
    /**
     * The opacity of the layer or parent element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#opacity)
     */
    readonly opacity: number;
    /**
     * The ActiveLayerInfo of the parent module:esri/layers/support/ISublayer or [GroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#parent)
     */
    parent: ActiveLayerInfo;
    /**
     * Indicates if the activeLayerInfo is ready.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ready)
     */
    ready: boolean;
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#respectLayerVisibility)
     */
    respectLayerVisibility: boolean;
    /**
     * The scale of the view instance in which the Legend is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#scale)
     */
    scale: number;
    /**
     * Only applies if the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layer) is a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#sublayerIds)
     */
    sublayerIds: number[];
    /**
     * The text string that represents the legend's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#title)
     */
    title: string;
    /**
     * The version of the ActiveLayerInfo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#version)
     */
    version: number;
    /**
     * The view in which the Legend is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#view)
     */
    view: View;
  }

  interface ActiveLayerInfoProperties {
    /**
     * A collection of child activeLayerInfos.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#children)
     */
    children?: CollectionProperties<ActiveLayerInfoProperties>;
    /**
     * When `true`, layers will only be shown in the legend if they are visible in the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#hideLayersNotInCurrentView)
     */
    hideLayersNotInCurrentView?: boolean;
    /**
     * Indicates if the legend's display of the layer's renderer is driven by the scale of the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#isScaleDriven)
     */
    isScaleDriven?: boolean;
    /**
     * The layer represented by the ActiveLayerInfo object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layer)
     */
    layer?: LayerProperties;
    /**
     * The layerView represented by the ActiveLayerInfo object's layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layerView)
     */
    layerView?: LayerViewProperties;
    /**
     * The legendElements is constructed using the layer [Renderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-Renderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElements)
     */
    legendElements?: (
      | SymbolTableElement
      | ColorRampElement
      | OpacityRampElement
      | SizeRampElement
      | HeatmapRampElement
      | RelationshipRampElement
    )[];
    /**
     * The ActiveLayerInfo of the parent module:esri/layers/support/ISublayer or [GroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GroupLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#parent)
     */
    parent?: ActiveLayerInfoProperties;
    /**
     * Indicates if the activeLayerInfo is ready.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ready)
     */
    ready?: boolean;
    /**
     * Determines whether to respect the properties of the layers in the map that control the legend's visibility (`minScale`, `maxScale`, `legendEnabled`).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#respectLayerVisibility)
     */
    respectLayerVisibility?: boolean;
    /**
     * The scale of the view instance in which the Legend is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#scale)
     */
    scale?: number;
    /**
     * Only applies if the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#layer) is a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#sublayerIds)
     */
    sublayerIds?: number[];
    /**
     * The text string that represents the legend's title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#title)
     */
    title?: string;
    /**
     * The version of the ActiveLayerInfo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#version)
     */
    version?: number;
    /**
     * The view in which the Legend is rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#view)
     */
    view?: ViewProperties;
  }

  /**
   * Describes the schema of the ColorRampElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampElement)
   */
  export interface ColorRampElement {
    /**
     * The type of this element is always `color-ramp`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The title of the color ramp as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampElement)
     */
    title: string | RampTitle;
    /**
     * The individual color stops rendered in the legend that correspond to the color visual variable in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampElement)
     */
    infos: ColorRampStop[];
  }

  /**
   * Describes the schema of the ColorRampStop element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampStop)
   */
  export interface ColorRampStop {
    /**
     * The label in the legend describing features with the given `color` and `value`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampStop)
     */
    label: string;
    /**
     * The value of the color visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampStop)
     */
    value: number;
    /**
     * The color of the visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampStop)
     */
    color: Color;
    /**
     * A number between `0` and `1` describing the position of the label on the color ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ColorRampStop)
     */
    offset: number;
  }

  /**
   * Describes the schema of the HeatmapRampElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampElement)
   */
  export interface HeatmapRampElement {
    /**
     * The type of this element is always `heatmap-ramp`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The title of the heatmap ramp as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampElement)
     */
    title?: string | RampTitle;
    /**
     * The individual color stops rendered in the legend that correspond to the heatmap colorStops in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampElement)
     */
    infos: HeatmapRampStop[];
  }

  /**
   * Describes the schema of the HeatmapRampStop element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampStop)
   */
  export interface HeatmapRampStop {
    /**
     * The label of the color stop displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampStop)
     */
    label: string;
    /**
     * The ratio of a pixel's intensity value to the minPixelIntensity of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampStop)
     */
    ratio: number;
    /**
     * The color of the pixel corresponding to the appropriate pixel `ratio`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampStop)
     */
    color: Color;
    /**
     * A number between `0` and `1` describing the position of the label on the ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#HeatmapRampStop)
     */
    offset: number;
  }

  /**
   * Properties defining the scheme of each of the ImageSymbolTableElementInfo.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
   */
  export interface ImageSymbolTableElementInfo {
    /**
     * The label of the category corresponding to the given image.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
     */
    label?: string;
    /**
     * The data URI of the image representing a colored pixel in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
     */
    src: string;
    /**
     * The opacity of the image (`0` to `1`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
     */
    opacity: number;
    /**
     * The width of the image in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
     */
    width?: number;
    /**
     * The height of the image in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#ImageSymbolTableElementInfo)
     */
    height?: number;
  }

  /**
   * Properties defining the scheme of each of the supported [legendElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElements).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#LegendElement)
   */
  export type LegendElement =
    | SymbolTableElement
    | ColorRampElement
    | OpacityRampElement
    | SizeRampElement
    | HeatmapRampElement
    | RelationshipRampElement;

  /**
   * Describes the schema of the OpacityRampElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampElement)
   */
  export interface OpacityRampElement {
    /**
     * The type of this element is always `opacity-ramp`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The title of the opacity ramp as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampElement)
     */
    title?: string | RampTitle;
    /**
     * The individual opacity stops rendered in the legend that correspond to the opacity visual variable in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampElement)
     */
    infos: OpacityRampStop[];
  }

  /**
   * Describes the schema of the OpacityRampStop element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampStop)
   */
  export interface OpacityRampStop {
    /**
     * The label in the legend describing features with the given `color` and `value`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampStop)
     */
    label: string;
    /**
     * The value of the opacity visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampStop)
     */
    value: number;
    /**
     * The color of the visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampStop)
     */
    color: Color;
    /**
     * A number between `0` and `1` describing the position of the label on the ramp.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#OpacityRampStop)
     */
    offset: number;
  }

  /**
   * Describes the schema of the RampTitle element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
   */
  export interface RampTitle {
    /**
     * The field name used in a visual variable of a renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
     */
    field: string;
    /**
     * If provided, the normalization field name used in a visual variable of a renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
     */
    normField: string;
    /**
     * If `true`, the legend formats to display the field and normalization field as a ratio.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
     */
    ratio: boolean;
    /**
     * If `true`, the legend formats to display the field and normalization field as a percentage.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
     */
    ratioPercent: boolean;
    /**
     * If `true`, the legend formats to display the field and normalization field as a percentage of total.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RampTitle)
     */
    ratioPercentTotal: boolean;
  }

  /**
   * Describes the schema of the RelationshipLabels element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipLabels)
   */
  export interface RelationshipLabels {
    /**
     * The label corresponding with the top corner of the legend if the focus is set to `HH`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipLabels)
     */
    top: string;
    /**
     * The label corresponding with the bottom corner of the legend if the focus is set to `HH`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipLabels)
     */
    bottom: string;
    /**
     * The label corresponding with the left corner of the legend (if the focus is set to `HH`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipLabels)
     */
    left: string;
    /**
     * The label corresponding with the right corner of the legend (if the focus is set to `HH`).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipLabels)
     */
    right: string;
  }

  /**
   * Describes the schema of the RelationshipRampElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
   */
  export interface RelationshipRampElement {
    /**
     * The type of this element is always `relationship-ramp`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The number of classes for each field comprising the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    numClasses: number;
    /**
     * Determines the orientation of the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    focus: string;
    /**
     * A 2-dimensional array of colors as displayed in the legend grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    colors: Color[][];
    /**
     * The labels for each corner of the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    labels: RelationshipLabels;
    /**
     * The rotation of the legend in degrees (0-360).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    rotation: number;
    /**
     * The title of the renderer as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    title?: string;
    /**
     * Info objects associated with the relationship renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RelationshipRampElement)
     */
    infos?: any[];
  }

  /**
   * Describes the schema of the RendererTitle element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RendererTitle)
   */
  export interface RendererTitle {
    /**
     * The title of the renderer as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RendererTitle)
     */
    title?: string;
    /**
     * The field name used in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RendererTitle)
     */
    field: string;
    /**
     * If a normalization field is provided to the renderer, this field is also used in the generated title for the renderer in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RendererTitle)
     */
    normField: string;
    /**
     * Indicates if a percentage is used in the normalization of the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#RendererTitle)
     */
    normByPct: boolean;
  }

  /**
   * Describes the schema of the SizeRampElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampElement)
   */
  export interface SizeRampElement {
    /**
     * The type of this element is always `size-ramp`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The title of the size ramp as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampElement)
     */
    title?: string | RampTitle;
    /**
     * The individual size stops rendered in the legend that correspond to the size visual variable in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampElement)
     */
    infos: SizeRampStop[];
  }

  /**
   * Describes the schema of the SizeRampStop element.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
   */
  export interface SizeRampStop {
    /**
     * The label in the legend describing features with the given `symbol` and `value`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    label: string;
    /**
     * The value of the size visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    value?: any;
    /**
     * The symbol corresponding to the value of the stop in the renderer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    symbol: Symbol;
    /**
     * The size of the visual variable [stop](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html#stops) in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    size?: number;
    /**
     * The width of the outline in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    outlineSize?: number;
    /**
     * The HTML element rendered in the legend representing features with the given value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SizeRampStop)
     */
    preview?: HTMLElement;
  }

  /**
   * Describes the schema of the SymbolTableElement used as a [legendElement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#legendElement).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElement)
   */
  export interface SymbolTableElement {
    /**
     * The type of this element is always `symbol-table`.
     *
     * [Read more...](global.html#type)
     */
    type: string;
    /**
     * The title of the renderer as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElement)
     */
    title?: string | RendererTitle;
    /**
     * Only used for renderers set to ImageryLayers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElement)
     */
    legendType?: string;
    /**
     * An array of each symbol/category/break rendered for the symbol table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElement)
     */
    infos: SymbolTableElementType[];
  }

  /**
   * Properties defining the scheme of the SymbolTableElementInfo.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
   */
  export interface SymbolTableElementInfo {
    /**
     * The title of the renderer as displayed in the Legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
     */
    label: string | RampTitle;
    /**
     * The value corresponding with the given `symbol` in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
     */
    value?: any;
    /**
     * The symbol for the corresponding `value` in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
     */
    symbol: Symbol;
    /**
     * The size of the symbol in points.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
     */
    size?: number;
    /**
     * A preview of the symbol element as displayed in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementInfo)
     */
    preview?: HTMLElement;
  }

  /**
   * Properties defining the scheme of each SymbolTableElementType.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend-support-ActiveLayerInfo.html#SymbolTableElementType)
   */
  export type SymbolTableElementType = ImageSymbolTableElementInfo | SymbolTableElementInfo;

  export interface LegendLayerInfos {
    /**
     * Specifies a title for the layer to display above its symbols and descriptions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#layerInfos)
     */
    title?: string;
    /**
     * A layer to display in the legend.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#layerInfos)
     */
    layer: Layer;
    /**
     * Only applicable if the `layer` is a [MapImageLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html), [SubtypeGroupLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SubtypeGroupLayer.html) or [WMSLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-WMSLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#layerInfos)
     */
    sublayerIds?: number[];
  }

  export interface LegendStyle {
    /**
     * Specifies the style of the legend.
     *
     * [Read more...](global.html#type)
     */
    type: "classic" | "card";
    /**
     * When a `card` type is specified, you can specify one of the following layout options.
     *
     * @default stack
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Legend.html#style)
     */
    layout?: "auto" | "side-by-side" | "stack";
  }

  export class LineOfSight extends Widget {
    /**
     * The LineOfSight widget is a 3D analysis tool that allows you to perform visibility analysis in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html)
     */

    constructor(properties?: LineOfSightProperties);

    /**
     * The line of sight analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#analysis)
     */
    readonly analysis: LineOfSightAnalysis;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#viewModel)
     */
    viewModel: LineOfSightViewModel;
  }

  interface LineOfSightProperties extends WidgetProperties {
    /**
     * The line of sight analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#analysis)
     */
    analysis?: LineOfSightAnalysisProperties;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html#viewModel)
     */
    viewModel?: LineOfSightViewModelProperties;
  }

  export class LineOfSightTarget extends Accessor {
    /**
     * This class represents a target point for a [LineOfSight](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html) analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html)
     */

    constructor(properties?: LineOfSightTargetProperties);

    /**
     * The first [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) intersected by the line of sight.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html#intersectedGraphic)
     */
    readonly intersectedGraphic: Graphic;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location where the line of sight first intersects the ground or 3D objects in the scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html#intersectedLocation)
     */
    readonly intersectedLocation: Point;
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location of the target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html#location)
     */
    location: Point;
    /**
     * Whether the target is visible from the observer or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html#visible)
     */
    readonly visible: boolean;
  }

  interface LineOfSightTargetProperties {
    /**
     * A [Point](https://developers.arcgis.com/javascript/latest/api-reference/esri-geometry-Point.html) representing the location of the target.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html#location)
     */
    location?: PointProperties;
  }

  export class LineOfSightViewModel extends Accessor {
    /**
     * Provides the logic for the [LineOfSight](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html)
     */

    constructor(properties?: LineOfSightViewModelProperties);

    /**
     * The line of sight analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#analysis)
     */
    readonly analysis: LineOfSightAnalysis;
    /**
     * The observer's viewpoint from which lines of sight will be drawn towards the targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#observer)
     */
    observer: Point;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "creating" | "created";
    /**
     * A collection of [LineOfSightTarget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html) containing the target location and the analysis results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#targets)
     */
    targets: Collection<LineOfSightTarget>;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#view)
     */
    view: SceneView;

    /**
     * Clears the current analysis results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#clear)
     */
    clear(): void;
    /**
     * If stopped, this method continues the line of sight analysis and the user can add more targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#continue)
     */
    continue(): void;
    /**
     * Starts a new line of sight analysis.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#start)
     */
    start(): void;
    /**
     * Stops the current line of sight analysis, keeping the results in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#stop)
     */
    stop(): void;
  }

  interface LineOfSightViewModelProperties {
    /**
     * The line of sight analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#analysis)
     */
    analysis?: LineOfSightAnalysisProperties;
    /**
     * The observer's viewpoint from which lines of sight will be drawn towards the targets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#observer)
     */
    observer?: PointProperties;
    /**
     * A collection of [LineOfSightTarget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightTarget.html) containing the target location and the analysis results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#targets)
     */
    targets?: CollectionProperties<LineOfSightTargetProperties>;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-LineOfSight-LineOfSightViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export interface Locate extends Widget, GoTo {}

  export class Locate {
    /**
     * Provides a simple widget that animates the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to the user's current location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html)
     */

    constructor(properties?: LocateProperties);

    /**
     * The browser's Geolocation API Position options for locating.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#geolocationOptions)
     */
    geolocationOptions: any;
    /**
     * Indicates whether the widget should navigate the view to the position and scale of the geolocated result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#goToLocationEnabled)
     */
    goToLocationEnabled: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#graphic)
     */
    graphic: Graphic;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) of the result graphic from the [locate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#locate) method.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * Indicates whether the widget will automatically rotate to the device heading based on the Geolocation APIs [`GeolocationCoordinates.heading`](https://developer.mozilla.org/en-US/docs/Web/API/GeolocationCoordinates/heading) property.
     *
     * @default true
     * @deprecated since 4.29. Use Track widget instead
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#rotationEnabled)
     */
    rotationEnabled: boolean;
    /**
     * Indicates the scale to set on the view when navigating to the position of the geolocated result once a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#scale)
     */
    scale: number;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#viewModel)
     */
    viewModel: LocateViewModel;

    /**
     * This function provides the ability to interrupt and cancel the process of programmatically obtaining the location of the user's device.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#cancelLocate)
     */
    cancelLocate(): void;
    /**
     * Animates the view to the user's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#locate)
     */
    locate(): Promise<any>;

    on(name: "locate", eventHandler: LocateLocateEventHandler): IHandle;

    on(name: "locate-error", eventHandler: LocateLocateErrorEventHandler): IHandle;
  }

  interface LocateProperties extends WidgetProperties, GoToProperties {
    /**
     * The browser's Geolocation API Position options for locating.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#geolocationOptions)
     */
    geolocationOptions?: any;
    /**
     * Indicates whether the widget should navigate the view to the position and scale of the geolocated result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#goToLocationEnabled)
     */
    goToLocationEnabled?: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) of the result graphic from the [locate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#locate) method.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * Indicates whether the widget will automatically rotate to the device heading based on the Geolocation APIs [`GeolocationCoordinates.heading`](https://developer.mozilla.org/en-US/docs/Web/API/GeolocationCoordinates/heading) property.
     *
     * @default true
     * @deprecated since 4.29. Use Track widget instead
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#rotationEnabled)
     */
    rotationEnabled?: boolean;
    /**
     * Indicates the scale to set on the view when navigating to the position of the geolocated result once a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#scale)
     */
    scale?: number;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html#viewModel)
     */
    viewModel?: LocateViewModelProperties;
  }

  export interface LocateViewModel extends Accessor, Evented, GeolocationPositioning, GoTo {}

  export class LocateViewModel {
    /**
     * Provides the logic for the [Locate](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate.html) widget, which animates the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to the user's current location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html)
     */

    constructor(properties?: LocateViewModelProperties);

    /**
     * Error that caused the last [locate-error](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#event:locate-error) event to fire.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#error)
     */
    error: any | Error;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) of the result graphic from the [locate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#locate) method.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "locating" | "feature-unsupported" | "error";

    /**
     * This function provides the ability to interrupt and cancel the process of programmatically obtaining the location of the user's device.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#cancelLocate)
     */
    cancelLocate(): void;
    /**
     * Animates the view to the user's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#locate)
     */
    locate(): Promise<any>;

    on(name: "locate", eventHandler: LocateViewModelLocateEventHandler): IHandle;

    on(name: "locate-error", eventHandler: LocateViewModelLocateErrorEventHandler): IHandle;
  }

  interface LocateViewModelProperties extends GeolocationPositioningProperties, GoToProperties {
    /**
     * Error that caused the last [locate-error](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#event:locate-error) event to fire.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#error)
     */
    error?: any | Error;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) of the result graphic from the [locate()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#locate) method.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Locate-LocateViewModel.html#popupEnabled)
     */
    popupEnabled?: boolean;
  }

  export interface LocateViewModelLocateErrorEvent {
    error: any | Error;
  }

  export interface LocateViewModelLocateEvent {
    position: any;
  }

  export interface LocateLocateErrorEvent {
    error: Error;
  }

  export interface LocateLocateEvent {
    position: any;
  }

  export class Measurement extends Widget {
    /**
     * The Measurement widget groups and manages multiple measurement tools and allows you to easily switch between them using the [activeTool](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#activeTool) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html)
     */

    constructor(properties?: MeasurementProperties);

    /**
     * Specifies the current measurement tool to display.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#activeTool)
     */
    activeTool: "area" | "distance" | "direct-line";
    /**
     * The measurement widget that is currently being used.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#activeWidget)
     */
    readonly activeWidget: AreaMeasurement2D | AreaMeasurement3D | DirectLineMeasurement3D | DistanceMeasurement2D;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#areaUnit)
     */
    areaUnit: SystemOrAreaUnit;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#linearUnit)
     */
    linearUnit: SystemOrLengthUnit;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#viewModel)
     */
    viewModel: MeasurementViewModel;

    /**
     * Removes all measurement widgets and associated graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#clear)
     */
    clear(): void;
    /**
     * Starts a new measurement for the active measurement widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#startMeasurement)
     */
    startMeasurement(): void;
  }

  interface MeasurementProperties extends WidgetProperties {
    /**
     * Specifies the current measurement tool to display.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#activeTool)
     */
    activeTool?: "area" | "distance" | "direct-line";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#areaUnit)
     */
    areaUnit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#linearUnit)
     */
    linearUnit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html#viewModel)
     */
    viewModel?: MeasurementViewModelProperties;
  }

  export class MeasurementViewModel extends Accessor {
    /**
     * Provides the logic for the [Measurement](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html)
     */

    constructor(properties?: MeasurementViewModelProperties);

    /**
     * Specifies the current measurement tool to display.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#activeTool)
     */
    activeTool: "area" | "distance" | "direct-line";
    /**
     * View model of the active measurement widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#activeViewModel)
     */
    activeViewModel:
      | AreaMeasurement2DViewModel
      | AreaMeasurement3DViewModel
      | DirectLineMeasurement3DViewModel
      | DistanceMeasurement2DViewModel;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#areaUnit)
     */
    areaUnit: SystemOrAreaUnit;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#linearUnit)
     */
    linearUnit: SystemOrLengthUnit;
    /**
     * The ViewModel's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "measuring" | "measured";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface MeasurementViewModelProperties {
    /**
     * Specifies the current measurement tool to display.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#activeTool)
     */
    activeTool?: "area" | "distance" | "direct-line";
    /**
     * View model of the active measurement widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#activeViewModel)
     */
    activeViewModel?:
      | AreaMeasurement2DViewModel
      | AreaMeasurement3DViewModelProperties
      | DirectLineMeasurement3DViewModelProperties
      | DistanceMeasurement2DViewModelProperties;
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the area values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#areaUnit)
     */
    areaUnit?:
      | "imperial"
      | "metric"
      | "square-millimeters"
      | "square-centimeters"
      | "square-decimeters"
      | "square-meters"
      | "square-kilometers"
      | "square-inches"
      | "square-feet"
      | "square-yards"
      | "square-miles"
      | "square-us-feet"
      | "acres"
      | "ares"
      | "hectares";
    /**
     * Unit system (imperial, metric) or specific unit used for displaying the distance values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#linearUnit)
     */
    linearUnit?:
      | "imperial"
      | "metric"
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Measurement-MeasurementViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class NavigationToggle extends Widget {
    /**
     * Provides two simple buttons for toggling the [navigation mode](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#navigationMode) of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html)
     */

    constructor(properties?: NavigationToggleProperties);

    /**
     * Sets the layout of the widget to either `horizontal` or `vertical`.
     *
     * @default "vertical"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#layout)
     */
    layout: "vertical" | "horizontal";
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#viewModel)
     */
    viewModel: NavigationToggleViewModel;

    /**
     * Toggles the navigation mode of the [view](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#view) from `pan` to `rotate` or vice versa.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#toggle)
     */
    toggle(): void;
  }

  interface NavigationToggleProperties extends WidgetProperties {
    /**
     * Sets the layout of the widget to either `horizontal` or `vertical`.
     *
     * @default "vertical"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#layout)
     */
    layout?: "vertical" | "horizontal";
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html#viewModel)
     */
    viewModel?: NavigationToggleViewModelProperties;
  }

  export class NavigationToggleViewModel extends Accessor {
    /**
     * Provides the logic for the [NavigationToggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle.html) widget, which provides two simple buttons for toggling the [navigation mode](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#navigationMode) of a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html)
     */

    constructor(properties?: NavigationToggleViewModelProperties);

    /**
     * The navigation mode of the [view](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#view).
     *
     * @default "pan"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#navigationMode)
     */
    navigationMode: "pan" | "rotate";
    /**
     * The state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * The view associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#view)
     */
    view: SceneView;

    /**
     * Toggles the navigation mode of the [view](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#view) from `pan` to `rotate` or vice versa.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#toggle)
     */
    toggle(): void;
  }

  interface NavigationToggleViewModelProperties {
    /**
     * The navigation mode of the [view](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#view).
     *
     * @default "pan"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#navigationMode)
     */
    navigationMode?: "pan" | "rotate";
    /**
     * The view associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-NavigationToggle-NavigationToggleViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export class OrientedImageryViewer extends Widget {
    /**
     * The oriented imagery viewer widget allows the user to explore and use their oriented images.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html)
     */

    constructor(properties?: OrientedImageryViewerProperties);

    /**
     * Toggles the visibility of the current footprint polygon associated with the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#currentCoverageVisible)
     */
    currentCoverageVisible: boolean;
    /**
     * When `false`, the view click interaction is enabled for the oriented imagery viewer widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates if the image gallery functionality is available in the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#galleryOpened)
     */
    galleryOpened: boolean;
    /**
     * Indicates if the image enhancement tool is active.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#imageEnhancementToolActive)
     */
    imageEnhancementToolActive: boolean;
    /**
     * When `true`, the image gallery functionality is available in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#imageGalleryEnabled)
     */
    readonly imageGalleryEnabled: boolean;
    /**
     * Indicates if the additional footprint polygons are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#isAdditionalCoverageVisible)
     */
    isAdditionalCoverageVisible: boolean;
    /**
     * Indicates if the additional camera locations are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#isAdditionalPointSourcesVisible)
     */
    isAdditionalPointSourcesVisible: boolean;
    /**
     * Specifies the current [OrientedImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#layer)
     */
    layer: OrientedImageryLayer;
    /**
     * Indicates if the map-image location tool is available in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#mapImageConversionToolState)
     */
    mapImageConversionToolState: boolean;
    /**
     * When `true`, the navigation tool is available in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#navigationToolActive)
     */
    navigationToolActive: boolean;
    /**
     * This property provides the location of an image pixel in Map coordinates when using map-image conversion tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#referencePoint)
     */
    readonly referencePoint: Point;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#viewModel)
     */
    viewModel: OrientedImageryViewerViewModel;
  }

  interface OrientedImageryViewerProperties extends WidgetProperties {
    /**
     * Toggles the visibility of the current footprint polygon associated with the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#currentCoverageVisible)
     */
    currentCoverageVisible?: boolean;
    /**
     * When `false`, the view click interaction is enabled for the oriented imagery viewer widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates if the image gallery functionality is available in the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#galleryOpened)
     */
    galleryOpened?: boolean;
    /**
     * Indicates if the image enhancement tool is active.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#imageEnhancementToolActive)
     */
    imageEnhancementToolActive?: boolean;
    /**
     * Indicates if the additional footprint polygons are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#isAdditionalCoverageVisible)
     */
    isAdditionalCoverageVisible?: boolean;
    /**
     * Indicates if the additional camera locations are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#isAdditionalPointSourcesVisible)
     */
    isAdditionalPointSourcesVisible?: boolean;
    /**
     * Specifies the current [OrientedImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html) to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#layer)
     */
    layer?: OrientedImageryLayerProperties;
    /**
     * Indicates if the map-image location tool is available in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#mapImageConversionToolState)
     */
    mapImageConversionToolState?: boolean;
    /**
     * When `true`, the navigation tool is available in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#navigationToolActive)
     */
    navigationToolActive?: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html#viewModel)
     */
    viewModel?: OrientedImageryViewerViewModelProperties;
  }

  export class OrientedImageryViewerViewModel extends Accessor {
    /**
     * Provides the logic for the [OrientedImageryViewer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html)
     */

    constructor(properties?: OrientedImageryViewerViewModelProperties);

    /**
     * Changes the image brightness loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#brightness)
     */
    brightness: number;
    /**
     * Changes the image contrast loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#contrast)
     */
    contrast: number;
    /**
     * Toggles the visibility of the current footprint polygon associated with the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#currentCoverageVisible)
     */
    currentCoverageVisible: boolean;
    /**
     * When `false`, the view click interaction enabled for the oriented imagery viewer widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#disabled)
     */
    disabled: boolean;
    /**
     * When `true`, the image gallery functionality is available in the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#imageGalleryEnabled)
     */
    imageGalleryEnabled: boolean;
    /**
     * Indicates if the image has been loaded onto the oriented imagery viewer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#imageLoaded)
     */
    imageLoaded: boolean;
    /**
     * Indicates if the additional footprint polygons are visible for the selected image loaded in the viewer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#isAdditionalCoverageVisible)
     */
    isAdditionalCoverageVisible: boolean;
    /**
     * Indicates if the additional camera locations are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#isAdditionalPointSourcesVisible)
     */
    isAdditionalPointSourcesVisible: boolean;
    /**
     * [OrientedImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html) associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#layer)
     */
    layer: OrientedImageryLayer;
    /**
     * Indicates if the map-image location tool available in the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#mapImageConversionToolState)
     */
    mapImageConversionToolState: boolean;
    /**
     * Changes the image sharpness loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#sharpness)
     */
    sharpness: number;
  }

  interface OrientedImageryViewerViewModelProperties {
    /**
     * Changes the image brightness loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#brightness)
     */
    brightness?: number;
    /**
     * Changes the image contrast loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#contrast)
     */
    contrast?: number;
    /**
     * Toggles the visibility of the current footprint polygon associated with the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#currentCoverageVisible)
     */
    currentCoverageVisible?: boolean;
    /**
     * When `false`, the view click interaction enabled for the oriented imagery viewer widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#disabled)
     */
    disabled?: boolean;
    /**
     * When `true`, the image gallery functionality is available in the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#imageGalleryEnabled)
     */
    imageGalleryEnabled?: boolean;
    /**
     * Indicates if the image has been loaded onto the oriented imagery viewer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#imageLoaded)
     */
    imageLoaded?: boolean;
    /**
     * Indicates if the additional footprint polygons are visible for the selected image loaded in the viewer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#isAdditionalCoverageVisible)
     */
    isAdditionalCoverageVisible?: boolean;
    /**
     * Indicates if the additional camera locations are visible for the selected image loaded in the viewer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#isAdditionalPointSourcesVisible)
     */
    isAdditionalPointSourcesVisible?: boolean;
    /**
     * [OrientedImageryLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OrientedImageryLayer.html) associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#layer)
     */
    layer?: OrientedImageryLayerProperties;
    /**
     * Indicates if the map-image location tool available in the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#mapImageConversionToolState)
     */
    mapImageConversionToolState?: boolean;
    /**
     * Changes the image sharpness loaded in the viewer.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-OrientedImageryViewer-OrientedImageryViewerViewModel.html#sharpness)
     */
    sharpness?: number;
  }

  export interface Popup extends Widget, GoTo, Evented {}

  export class Popup {
    /**
     * The Popup widget allows users to view content from feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html)
     */

    constructor(properties?: PopupProperties);

    /**
     * [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions)
     */
    actions: Collection<ActionButton | ActionToggle>;
    /**
     * Indicates if the widget is active when it is visible and is not [waiting for results](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup-PopupViewModel.html#waitingForResult).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#active)
     */
    readonly active: boolean;
    /**
     * Position of the popup in relation to the selected feature.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#alignment)
     */
    alignment: "auto" | "top-center" | "top-right" | "bottom-left" | "bottom-center" | "bottom-right" | Function;
    /**
     * This closes the popup when the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) camera or [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#autoCloseEnabled)
     */
    autoCloseEnabled: boolean;
    /**
     * Indicates whether the popup displays its content.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#collapsed)
     */
    readonly collapsed: boolean;
    /**
     * Indicates whether to enable collapse functionality for the popup.
     *
     * @default true
     * @deprecated since 4.29. Use {@link module:esri/widgets/Popup~VisibleElements PopupVisibleElements.collapseButton} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#collapseEnabled)
     */
    collapseEnabled: boolean;
    /**
     * The content of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#content)
     */
    content: string | HTMLElement | Widget;
    /**
     * Dock position in the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#currentDockPosition)
     */
    readonly currentDockPosition:
      | "auto"
      | "top-center"
      | "top-right"
      | "top-left"
      | "bottom-left"
      | "bottom-center"
      | "bottom-right";
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled: boolean;
    /**
     * Indicates whether the placement of the popup is docked to the side of the view.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockEnabled)
     */
    dockEnabled: boolean;
    /**
     * Docking the popup allows for a better user experience, particularly when opening popups in apps on mobile devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    dockOptions: PopupDockOptions;
    /**
     * The number of selected [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features) available to the popup.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#featureCount)
     */
    readonly featureCount: number;
    /**
     * An array of features associated with the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features)
     */
    features: Graphic[];
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title) of the popup.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Highlight the selected popup feature using the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#highlightOptions) set on the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions) set on the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#highlightEnabled)
     */
    highlightEnabled: boolean;
    /**
     * Point used to position the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location)
     */
    location: Point;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#promises)
     */
    promises: Promise<any>[];
    /**
     * The selected feature accessed by the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeature)
     */
    readonly selectedFeature: Graphic;
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeatureIndex)
     */
    selectedFeatureIndex: number;
    /**
     * Returns a reference to the current [Feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Feature.html) that the Popup is using.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeatureWidget)
     */
    readonly selectedFeatureWidget: Feature;
    /**
     * Indicates whether to display a spinner at the popup location prior to its display when it has pending promises.
     *
     * @deprecated since 4.29. Use {@link module:esri/widgets/Popup~VisibleElements PopupVisibleElements.spinner} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#spinnerEnabled)
     */
    spinnerEnabled: boolean;
    /**
     * The title of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title)
     */
    title: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#view)
     */
    view: MapView | SceneView;
    /**
     * This is a class that contains all the logic (properties and methods) that controls this widget's behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#viewModel)
     */
    viewModel: PopupViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#visibleElements)
     */
    visibleElements: PopupVisibleElements;

    /**
     * Use this method to remove focus from the Widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#blur)
     */
    blur(): void;
    /**
     * Removes [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#promises), [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features), [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#content), [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title) and [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location) from the Popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#clear)
     */
    clear(): void;
    /**
     * Closes the popup by setting its [visible](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#visible) property to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#close)
     */
    close(): void;
    /**
     * Use this method to return feature(s) at a given screen location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#fetchFeatures)
     */
    fetchFeatures(
      screenPoint: PopupFetchFeaturesScreenPoint,
      options?: PopupFetchFeaturesOptions,
    ): Promise<FetchPopupFeaturesResult>;
    /**
     * Use this method to give focus to the Widget if the widget is able to be focused.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#focus)
     */
    focus(): void;
    /**
     * Selects the feature at the next index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#next)
     */
    next(): PopupViewModel;
    /**
     * Opens the popup at the given location with content defined either explicitly with `content` or driven from the [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) of input features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    open(options?: PopupOpenOptions): void;
    /**
     * Selects the feature at the previous index in relation to the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#previous)
     */
    previous(): PopupViewModel;
    /**
     * Positions the popup on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#reposition)
     */
    reposition(): void;
    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#event-trigger-action) event and executes the [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions) at the specified index in the [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions) array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#triggerAction)
     */
    triggerAction(actionIndex: number): void;

    on(name: "trigger-action", eventHandler: PopupTriggerActionEventHandler): IHandle;
  }

  interface PopupProperties extends WidgetProperties, GoToProperties {
    /**
     * [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html) objects.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions)
     */
    actions?: CollectionProperties<
      (ActionButtonProperties & { type: "button" }) | (ActionToggleProperties & { type: "toggle" })
    >;
    /**
     * Position of the popup in relation to the selected feature.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#alignment)
     */
    alignment?: "auto" | "top-center" | "top-right" | "bottom-left" | "bottom-center" | "bottom-right" | Function;
    /**
     * This closes the popup when the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) camera or [Viewpoint](https://developers.arcgis.com/javascript/latest/api-reference/esri-Viewpoint.html) changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#autoCloseEnabled)
     */
    autoCloseEnabled?: boolean;
    /**
     * Indicates whether to enable collapse functionality for the popup.
     *
     * @default true
     * @deprecated since 4.29. Use {@link module:esri/widgets/Popup~VisibleElements PopupVisibleElements.collapseButton} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#collapseEnabled)
     */
    collapseEnabled?: boolean;
    /**
     * The content of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#content)
     */
    content?: string | HTMLElement | WidgetProperties;
    /**
     * Enables automatic creation of a popup template for layers that have popups enabled but no popupTemplate defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#defaultPopupTemplateEnabled)
     */
    defaultPopupTemplateEnabled?: boolean;
    /**
     * Indicates whether the placement of the popup is docked to the side of the view.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockEnabled)
     */
    dockEnabled?: boolean;
    /**
     * Docking the popup allows for a better user experience, particularly when opening popups in apps on mobile devices.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    dockOptions?: PopupDockOptions;
    /**
     * An array of features associated with the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features)
     */
    features?: GraphicProperties[];
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title) of the popup.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Highlight the selected popup feature using the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#highlightOptions) set on the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or the [highlightOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#highlightOptions) set on the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#highlightEnabled)
     */
    highlightEnabled?: boolean;
    /**
     * Point used to position the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location)
     */
    location?: PointProperties;
    /**
     * An array of pending Promises that have not yet been fulfilled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#promises)
     */
    promises?: Promise<any>[];
    /**
     * Index of the feature that is [selected](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeatureIndex)
     */
    selectedFeatureIndex?: number;
    /**
     * Indicates whether to display a spinner at the popup location prior to its display when it has pending promises.
     *
     * @deprecated since 4.29. Use {@link module:esri/widgets/Popup~VisibleElements PopupVisibleElements.spinner} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#spinnerEnabled)
     */
    spinnerEnabled?: boolean;
    /**
     * The title of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title)
     */
    title?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * This is a class that contains all the logic (properties and methods) that controls this widget's behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#viewModel)
     */
    viewModel?: PopupViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#visibleElements)
     */
    visibleElements?: PopupVisibleElements;
  }

  export class PopupViewModel extends FeaturesViewModel {
    /**
     * Provides the logic for the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) widget, which allows users to view content from feature attributes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup-PopupViewModel.html)
     */

    constructor(properties?: PopupViewModelProperties);
  }

  interface PopupViewModelProperties extends FeaturesViewModelProperties {}

  /**
   * Optional properties to use with the [fetchFeatures](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#fetchFeatures) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchFeaturesOptions)
   */
  export interface PopupFetchFeaturesOptions {
    /**
     * The `click` event for either the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchFeaturesOptions)
     */
    event?: any;
    /**
     * The signal object that can be used to abort the asynchronous task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchFeaturesOptions)
     */
    signal?: AbortSignal;
  }

  /**
   * The resulting features returned from the [fetchFeatures](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#fetchFeatures) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchPopupFeaturesResult)
   */
  export interface FetchPopupFeaturesResult {
    /**
     * An array of promises containing graphics from the selected location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchPopupFeaturesResult)
     */
    allGraphicsPromise?: Promise<Graphic[]>;
    /**
     * The resulting location of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#hitTest) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#hitTest)'s' `hitTest`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#FetchPopupFeaturesResult)
     */
    location?: Point;
  }

  export interface PopupDockOptions {
    /**
     * Defines the dimensions of the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) at which to dock the popup.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    breakpoint?: boolean | PopupDockOptionsBreakpoint;
    /**
     * If `true`, displays the dock button.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    buttonEnabled?: boolean;
    /**
     * The position in the view at which to dock the popup.
     *
     * @default auto
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    position?: string | Function;
  }

  export interface PopupDockOptionsBreakpoint {
    /**
     * The maximum width of the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) at which the popup will be set to dockEnabled automatically.
     *
     * @default 544
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    width?: number;
    /**
     * The maximum height of the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) at which the popup will be set to dockEnabled automatically.
     *
     * @default 544
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#dockOptions)
     */
    height?: number;
  }

  export interface PopupFetchFeaturesScreenPoint {
    /**
     * The x coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#fetchFeatures)
     */
    x: number;
    /**
     * The y coordinate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#fetchFeatures)
     */
    y: number;
  }

  export interface PopupOpenOptions {
    /**
     * Sets the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#title) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    title?: string;
    /**
     * Sets the [content](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#content) of the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    content?: string | HTMLElement | Widget;
    /**
     * Sets the popup's [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location), which is the geometry used to position the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    location?: Point;
    /**
     * When `true`, indicates the popup should fetch the content of this feature and display it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    fetchFeatures?: boolean;
    /**
     * Sets the popup's [features](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#features), which populate the title and content of the popup based on each graphic's [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    features?: Graphic[];
    /**
     * Sets pending [promises](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#promises) on the popup.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    promises?: Promise<any>[];
    /**
     * **Since:** 4.5    This property enables multiple features in a popup to display in a list rather than displaying the first selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    featureMenuOpen?: boolean;
    /**
     * When `true` indicates the popup should update its [location](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#location) for each paginated feature based on the [selected feature's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#selectedFeature) geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    updateLocationEnabled?: boolean;
    /**
     * **Since:** 4.5    When `true`, indicates that only the popup header will display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    collapsed?: boolean;
    /**
     * **Since:** 4.23    When `true`, indicates that the focus should be on the popup after it has been opened.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#open)
     */
    shouldFocus?: boolean;
  }

  export interface PopupTriggerActionEvent {
    action: ActionButton | ActionToggle;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
   */
  export interface PopupVisibleElements {
    /**
     * **Since 4.29.** Indicates whether to display the action bar that holds the feature's [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#actions).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    actionBar?: boolean;
    /**
     * Indicates whether to display a close button on the widget dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * **Since 4.29.** Indicates whether to display the collapse button on the widget dialog.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * Indicates whether pagination for feature navigation will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    featureNavigation?: boolean;
    /**
     * **Since 4.30.** Indicates whether to display the group heading for a list of multiple features.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    featureListLayerTitle?: boolean;
    /**
     * **Since 4.29.** Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * **Since 4.29.** Indicates whether to display the widget's loading spinner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html#VisibleElements)
     */
    spinner?: boolean;
  }

  export class Print extends Widget {
    /**
     * The Print widget connects your application with a [printing service](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-the-portal-to-print-maps.htm) to allow the map to be printed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html)
     */

    constructor(properties?: PrintProperties);

    /**
     * Specify the print output file format(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#allowedFormats)
     */
    allowedFormats: string | string[];
    /**
     * Specify the print output layout(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#allowedLayouts)
     */
    allowedLayouts: string | string[];
    /**
     * The collection of links exported from the Print widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#exportedLinks)
     */
    exportedLinks: Collection<FileLink>;
    /**
     * This option allows passing extra parameters (in addition to [templateOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateOptions)) to the print (export webmap) requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#extraParameters)
     */
    extraParameters: any;
    /**
     * Indicates the heading level to use for the "Exported files" text where users can access the exported map printout.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Indicates whether or not to include [defaultTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#defaultTemplates).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#includeDefaultTemplates)
     */
    includeDefaultTemplates: boolean;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#portal)
     */
    portal: Portal;
    /**
     * The URL of the REST endpoint of the Export Web Map Task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#printServiceUrl)
     */
    printServiceUrl: string;
    /**
     * The initial state of the print area toggle in the Print widget UI.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#showPrintAreaEnabled)
     */
    showPrintAreaEnabled: boolean;
    /**
     * An object containing an array of `customTextElements` name-value pair objects for each print template in a custom print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateCustomTextElements)
     */
    templateCustomTextElements: any[];
    /**
     * Defines the layout template options used by the [Print](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) widget to generate the print page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateOptions)
     */
    templateOptions: TemplateOptions;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#viewModel)
     */
    viewModel: PrintViewModel;

    on(name: "complete", eventHandler: PrintCompleteEventHandler): IHandle;

    on(name: "submit", eventHandler: PrintSubmitEventHandler): IHandle;
  }

  interface PrintProperties extends WidgetProperties {
    /**
     * Specify the print output file format(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#allowedFormats)
     */
    allowedFormats?: string | string[];
    /**
     * Specify the print output layout(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#allowedLayouts)
     */
    allowedLayouts?: string | string[];
    /**
     * The collection of links exported from the Print widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#exportedLinks)
     */
    exportedLinks?: CollectionProperties<FileLink>;
    /**
     * This option allows passing extra parameters (in addition to [templateOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateOptions)) to the print (export webmap) requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#extraParameters)
     */
    extraParameters?: any;
    /**
     * Indicates the heading level to use for the "Exported files" text where users can access the exported map printout.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Indicates whether or not to include [defaultTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#defaultTemplates).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#includeDefaultTemplates)
     */
    includeDefaultTemplates?: boolean;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#portal)
     */
    portal?: PortalProperties;
    /**
     * The URL of the REST endpoint of the Export Web Map Task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#printServiceUrl)
     */
    printServiceUrl?: string;
    /**
     * The initial state of the print area toggle in the Print widget UI.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#showPrintAreaEnabled)
     */
    showPrintAreaEnabled?: boolean;
    /**
     * An object containing an array of `customTextElements` name-value pair objects for each print template in a custom print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateCustomTextElements)
     */
    templateCustomTextElements?: any[];
    /**
     * Defines the layout template options used by the [Print](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) widget to generate the print page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#templateOptions)
     */
    templateOptions?: TemplateOptionsProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#viewModel)
     */
    viewModel?: PrintViewModelProperties;
  }

  export class CustomTemplate extends Accessor {
    /**
     * Defines the custom layout template options used by the [Print](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) widget to generate the print page.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html)
     */

    constructor(properties?: CustomTemplateProperties);

    /**
     * The template's description.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#description)
     */
    readonly description: string;
    /**
     * The output format for the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#format)
     */
    format: "pdf" | "png32" | "png8" | "jpg" | "gif" | "eps" | "svg" | "svgz";
    /**
     * The text that appears inside the [Print Widget's](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) `Select template` button.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#label)
     */
    readonly label: string;
    /**
     * The layout used for the print output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#layout)
     */
    readonly layout:
      | "map-only"
      | "a3-landscape"
      | "a3-portrait"
      | "a4-landscape"
      | "a4-portrait"
      | "letter-ansi-a-landscape"
      | "letter-ansi-a-portrait"
      | "tabloid-ansi-b-landscape"
      | "tabloid-ansi-b-portrait";
    /**
     * Defines the layout elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#layoutOptions)
     */
    readonly layoutOptions: CustomTemplateLayoutOptions;
  }

  interface CustomTemplateProperties {
    /**
     * The output format for the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#format)
     */
    format?: "pdf" | "png32" | "png8" | "jpg" | "gif" | "eps" | "svg" | "svgz";
  }

  export interface CustomTemplateLayoutOptions {
    /**
     * Indicates whether the legend will be included in the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#layoutOptions)
     */
    legend?: boolean;
    /**
     * Indicates whether the north arrow will be included in the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-CustomTemplate.html#layoutOptions)
     */
    northArrow?: boolean;
  }

  export class PrintViewModel extends Accessor {
    /**
     * Provides the logic for the [Print](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html)
     */

    constructor(properties?: PrintViewModelProperties);

    /**
     * Specify the print output file format(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#allowedFormats)
     */
    allowedFormats: string | string[];
    /**
     * Specify the print output layout(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#allowedLayouts)
     */
    allowedLayouts: string | string[];
    /**
     * A collection of print templates defined on the Portal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#defaultTemplates)
     */
    readonly defaultTemplates: Collection<CustomTemplate>;
    /**
     * The effective URL of the REST endpoint of the Export Web Map Task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#effectivePrintServiceUrl)
     */
    readonly effectivePrintServiceUrl: string;
    /**
     * Returns an array of objects of all print templates available on the custom print service to see which templates were published with `customTextElements`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#effectiveTemplateCustomTextElements)
     */
    readonly effectiveTemplateCustomTextElements: any[];
    /**
     * This option allows passing extra parameters to the print (export webmap) requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#extraParameters)
     */
    extraParameters: any;
    /**
     * Indicates whether or not to include [defaultTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#defaultTemplates).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#includeDefaultTemplates)
     */
    includeDefaultTemplates: boolean;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#portal)
     */
    portal: Portal;
    /**
     * The URL of the REST endpoint of the Export Web Map Task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#printServiceUrl)
     */
    printServiceUrl: string;
    /**
     * Print timeout value in milliseconds.
     *
     * @default 120000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#printTimeout)
     */
    printTimeout: number;
    /**
     * The initial state of the print area toggle in the [Print widget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) UI.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#showPrintAreaEnabled)
     */
    showPrintAreaEnabled: boolean;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#state)
     */
    readonly state: "disabled" | "initializing" | "ready" | "error";
    /**
     * An object containing an array of `customTextElements` name-value pair objects for each print template in a custom print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#templateCustomTextElements)
     */
    templateCustomTextElements: any[];
    /**
     * The service metadata that contains the [format](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#format) and [layout](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layout) information for the printout.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#templatesInfo)
     */
    readonly templatesInfo: any;
    /**
     * The time interval in milliseconds between each job status request sent to an asynchronous GP task.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#updateDelay)
     */
    updateDelay: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#view)
     */
    view: MapView;

    /**
     * Returns a CustomTemplate with the specified `id`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#getLayoutTemplateById)
     */
    getLayoutTemplateById(id: string): CustomTemplate;
    /**
     * This method should be called to load the view model's printing resources.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#load)
     */
    load(): Promise<any>;
    /**
     * Prints (exports) the current MapView according to selected options.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#print)
     */
    print(template: PrintTemplate): Promise<any>;
  }

  interface PrintViewModelProperties {
    /**
     * Specify the print output file format(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#allowedFormats)
     */
    allowedFormats?: string | string[];
    /**
     * Specify the print output layout(s) that the user can select based on the options available from the print service.
     *
     * @default "all"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#allowedLayouts)
     */
    allowedLayouts?: string | string[];
    /**
     * This option allows passing extra parameters to the print (export webmap) requests.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#extraParameters)
     */
    extraParameters?: any;
    /**
     * Indicates whether or not to include [defaultTemplates](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#defaultTemplates).
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#includeDefaultTemplates)
     */
    includeDefaultTemplates?: boolean;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#portal)
     */
    portal?: PortalProperties;
    /**
     * The URL of the REST endpoint of the Export Web Map Task.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#printServiceUrl)
     */
    printServiceUrl?: string;
    /**
     * Print timeout value in milliseconds.
     *
     * @default 120000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#printTimeout)
     */
    printTimeout?: number;
    /**
     * The initial state of the print area toggle in the [Print widget](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) UI.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#showPrintAreaEnabled)
     */
    showPrintAreaEnabled?: boolean;
    /**
     * An object containing an array of `customTextElements` name-value pair objects for each print template in a custom print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#templateCustomTextElements)
     */
    templateCustomTextElements?: any[];
    /**
     * The time interval in milliseconds between each job status request sent to an asynchronous GP task.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#updateDelay)
     */
    updateDelay?: number;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-PrintViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  export class TemplateOptions extends Accessor {
    /**
     * Defines the layout template options used by the [Print](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html) widget to generate printed output.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html)
     */

    constructor(properties?: TemplateOptionsProperties);

    /**
     * When `false`, the attribution is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#attributionEnabled)
     */
    attributionEnabled: boolean;
    /**
     * The text used for the author if the specified layout contains an author text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#author)
     */
    author: string;
    /**
     * The text used for the copyright if the specified layout contains an copyright text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#copyright)
     */
    copyright: string;
    /**
     * Use this prop to display text fields for custom text elements from the print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#customTextElements)
     */
    customTextElements: any[];
    /**
     * Resolution in dots per inch.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#dpi)
     */
    dpi: string;
    /**
     * This property only applies when the [layout](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layout) value is `map-only`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#fileName)
     */
    fileName: string;
    /**
     * When true, the feature's attributes are included in feature collection layers even when they are not needed for rendering.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#forceFeatureAttributes)
     */
    forceFeatureAttributes: boolean;
    /**
     * The output format for the printed map.
     *
     * @default "pdf"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#format)
     */
    format: "pdf" | "png32" | "png8" | "jpg" | "gif" | "eps" | "svg" | "svgz";
    /**
     * Map height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#height)
     */
    height: number;
    /**
     * When `true`, tables will be included in the printout request.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#includeTables)
     */
    includeTables: boolean;
    /**
     * The layout used for the print output.
     *
     * @default "letter-ansi-a-landscape"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layout)
     */
    layout:
      | "map-only"
      | "a3-landscape"
      | "a3-portrait"
      | "a4-landscape"
      | "a4-portrait"
      | "letter-ansi-a-landscape"
      | "letter-ansi-a-portrait"
      | "tabloid-ansi-b-landscape"
      | "tabloid-ansi-b-portrait";
    /**
     * A custom layout hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layoutItem)
     */
    layoutItem: PortalItem;
    /**
     * When `false`, the legend is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#legendEnabled)
     */
    legendEnabled: boolean;
    /**
     * When `true`, the north arrow will be included on the printout.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#northArrowEnabled)
     */
    northArrowEnabled: boolean;
    /**
     * The map scale of the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#scale)
     */
    scale: number;
    /**
     * Define whether the printed map should preserve map scale or map extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#scaleEnabled)
     */
    scaleEnabled: boolean;
    /**
     * The text used for the map title if the specified layout contains a title text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#title)
     */
    title: string;
    /**
     * Map width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#width)
     */
    width: number;
  }

  interface TemplateOptionsProperties {
    /**
     * When `false`, the attribution is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#attributionEnabled)
     */
    attributionEnabled?: boolean;
    /**
     * The text used for the author if the specified layout contains an author text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#author)
     */
    author?: string;
    /**
     * The text used for the copyright if the specified layout contains an copyright text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#copyright)
     */
    copyright?: string;
    /**
     * Use this prop to display text fields for custom text elements from the print service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#customTextElements)
     */
    customTextElements?: any[];
    /**
     * Resolution in dots per inch.
     *
     * @default 96
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#dpi)
     */
    dpi?: string;
    /**
     * This property only applies when the [layout](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layout) value is `map-only`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#fileName)
     */
    fileName?: string;
    /**
     * When true, the feature's attributes are included in feature collection layers even when they are not needed for rendering.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#forceFeatureAttributes)
     */
    forceFeatureAttributes?: boolean;
    /**
     * The output format for the printed map.
     *
     * @default "pdf"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#format)
     */
    format?: "pdf" | "png32" | "png8" | "jpg" | "gif" | "eps" | "svg" | "svgz";
    /**
     * Map height.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#height)
     */
    height?: number;
    /**
     * When `true`, tables will be included in the printout request.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#includeTables)
     */
    includeTables?: boolean;
    /**
     * The layout used for the print output.
     *
     * @default "letter-ansi-a-landscape"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layout)
     */
    layout?:
      | "map-only"
      | "a3-landscape"
      | "a3-portrait"
      | "a4-landscape"
      | "a4-portrait"
      | "letter-ansi-a-landscape"
      | "letter-ansi-a-portrait"
      | "tabloid-ansi-b-landscape"
      | "tabloid-ansi-b-portrait";
    /**
     * A custom layout hosted as a [portal item](https://developers.arcgis.com/javascript/latest/api-reference/esri-portal-PortalItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#layoutItem)
     */
    layoutItem?: PortalItemProperties;
    /**
     * When `false`, the legend is not displayed on the printout.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#legendEnabled)
     */
    legendEnabled?: boolean;
    /**
     * When `true`, the north arrow will be included on the printout.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#northArrowEnabled)
     */
    northArrowEnabled?: boolean;
    /**
     * The map scale of the printed map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#scale)
     */
    scale?: number;
    /**
     * Define whether the printed map should preserve map scale or map extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#scaleEnabled)
     */
    scaleEnabled?: boolean;
    /**
     * The text used for the map title if the specified layout contains a title text element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#title)
     */
    title?: string;
    /**
     * Map width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#width)
     */
    width?: number;
  }

  export interface PrintCompleteEvent {
    results: PrintCompleteEventResults;
  }

  /**
   * Represents an exported map request from the result of the Print widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
   */
  export interface FileLink {
    /**
     * The location of the FileLink element in the array.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    count: number;
    /**
     * The [error](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Error.html) from the Print widget, if any.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    error: string;
    /**
     * The [file](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#format) type of the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    extension: string;
    /**
     * The [fileName](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#fileName) or [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print-TemplateOptions.html#title) of the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    name: string;
    /**
     * The state of the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    state: string;
    /**
     * The [printServiceUrl](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#printServiceUrl) of the print-out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Print.html#FileLink)
     */
    url: string;
  }

  export interface PrintSubmitEvent {
    results: PrintSubmitEventResults;
  }

  export interface PrintCompleteEventResults {
    link: Collection<FileLink>;
  }

  export interface PrintSubmitEventResults {
    link: Collection<FileLink>;
  }

  export class ScaleBar extends Widget {
    /**
     * The ScaleBar widget displays a scale bar on the map or in a specified HTML node.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html)
     */

    constructor(properties?: ScaleBarProperties);

    /**
     * The style for the scale bar.
     *
     * @default "line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#style)
     */
    style: "ruler" | "line";
    /**
     * Units to use for the scale bar.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#unit)
     */
    unit: "metric" | "imperial" | "dual";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#viewModel)
     */
    viewModel: ScaleBarViewModel;
  }

  interface ScaleBarProperties extends WidgetProperties {
    /**
     * The style for the scale bar.
     *
     * @default "line"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#style)
     */
    style?: "ruler" | "line";
    /**
     * Units to use for the scale bar.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#unit)
     */
    unit?: "metric" | "imperial" | "dual";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html#viewModel)
     */
    viewModel?: ScaleBarViewModelProperties;
  }

  export class ScaleBarViewModel extends Accessor {
    /**
     * Provides the logic for the [ScaleBar](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar-ScaleBarViewModel.html)
     */

    constructor(properties?: ScaleBarViewModelProperties);

    /**
     * The current state of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar-ScaleBarViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar-ScaleBarViewModel.html#view)
     */
    view: MapView;

    /**
     * Computes the size and units of the scale bar widget given a base length in pixels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar-ScaleBarViewModel.html#getScaleBarProperties)
     */
    getScaleBarProperties(length: number, measurementSystem: "imperial" | "metric"): any;
  }

  interface ScaleBarViewModelProperties {
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleBar-ScaleBarViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  export class ScaleRangeSlider extends Widget {
    /**
     * The ScaleRangeSlider widget allows the user to set a minimum and maximum scale based on named scale ranges.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html)
     */

    constructor(properties?: ScaleRangeSliderProperties);

    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#disabled)
     */
    disabled: boolean;
    /**
     * When provided, the initial [minScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScale) and [maxScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScale) values will match the layer's.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#layer)
     */
    layer: Layer;
    /**
     * The maximum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScale)
     */
    maxScale: number;
    /**
     * The lowest possible maximum scale value on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScaleLimit)
     */
    maxScaleLimit: number;
    /**
     * The minimum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScale)
     */
    minScale: number;
    /**
     * The highest possible minimum scale value on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScaleLimit)
     */
    minScaleLimit: number;
    /**
     * The mode of the widget, indicating which slider thumbs can be adjusted.
     *
     * @default "range"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#mode)
     */
    mode: "range" | "max-scale-only" | "min-scale-only";
    /**
     * The region that the scale thumbnails will focus on.
     *
     * @default "US"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#region)
     */
    region: SupportedRegion;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#viewModel)
     */
    viewModel: ScaleRangeSliderViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#visibleElements)
     */
    visibleElements: ScaleRangeSliderVisibleElements;
  }

  interface ScaleRangeSliderProperties extends WidgetProperties {
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#disabled)
     */
    disabled?: boolean;
    /**
     * When provided, the initial [minScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScale) and [maxScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScale) values will match the layer's.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#layer)
     */
    layer?: LayerProperties;
    /**
     * The maximum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScale)
     */
    maxScale?: number;
    /**
     * The lowest possible maximum scale value on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#maxScaleLimit)
     */
    maxScaleLimit?: number;
    /**
     * The minimum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScale)
     */
    minScale?: number;
    /**
     * The highest possible minimum scale value on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#minScaleLimit)
     */
    minScaleLimit?: number;
    /**
     * The mode of the widget, indicating which slider thumbs can be adjusted.
     *
     * @default "range"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#mode)
     */
    mode?: "range" | "max-scale-only" | "min-scale-only";
    /**
     * The region that the scale thumbnails will focus on.
     *
     * @default "US"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#region)
     */
    region?:
      | "AE"
      | "AR"
      | "AT"
      | "AU"
      | "BE"
      | "BG"
      | "BO"
      | "BR"
      | "CA"
      | "CH"
      | "CI"
      | "CL"
      | "CN"
      | "CO"
      | "CR"
      | "CZ"
      | "DE"
      | "DK"
      | "EE"
      | "EG"
      | "ES"
      | "FI"
      | "FR"
      | "GB"
      | "GL"
      | "GR"
      | "GT"
      | "HK"
      | "ID"
      | "IE"
      | "IL"
      | "IN"
      | "IQ"
      | "IS"
      | "IT"
      | "JP"
      | "KE"
      | "KR"
      | "KW"
      | "LI"
      | "LT"
      | "LU"
      | "LV"
      | "MA"
      | "MG"
      | "ML"
      | "MO"
      | "MX"
      | "MY"
      | "NI"
      | "NL"
      | "NO"
      | "NZ"
      | "PE"
      | "PL"
      | "PR"
      | "PT"
      | "RO"
      | "RU"
      | "RW"
      | "SE"
      | "SG"
      | "SK"
      | "SR"
      | "SV"
      | "TH"
      | "TN"
      | "TW"
      | "US"
      | "VE"
      | "VI"
      | "ZA";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#viewModel)
     */
    viewModel?: ScaleRangeSliderViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#visibleElements)
     */
    visibleElements?: ScaleRangeSliderVisibleElements;
  }

  export class ScaleRanges extends Accessor {
    /**
     * The ScaleRanges class represents the ranges of scales for the [ScaleRangeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html)
     */

    constructor(properties?: ScaleRangesProperties);

    /**
     * Clamps the scale to the closest minScale or maxScale on the scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#clampScale)
     */
    clampScale(scale: number): number;
    /**
     * Determines whether the given scale is within the current scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#contains)
     */
    contains(scale: number): boolean;
    /**
     * Finds the scale range name at a given index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#findScaleRangeByIndex)
     */
    findScaleRangeByIndex(index: number): NamedScaleRange;
    /**
     * Helper to create a ScaleRanges object from a minimum and maximum scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#fromScaleRange)
     */
    fromScaleRange(minScale: number, maxScale: number): ScaleRanges;
    /**
     * Determines if the input scale value can be considered to be at the smallest scale range edge.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#isMaxScaleEdge)
     */
    isMaxScaleEdge(scale: void): boolean;
    /**
     * Determines if the input scale value can be considered to be at the largest scale range edge.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#isMinScaleEdge)
     */
    isMinScaleEdge(scale: void): boolean;
  }

  interface ScaleRangesProperties {}

  /**
   * The NamedScaleRange provides the minimum and maximum scale of an named scale id.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#NamedScaleRange)
   */
  export interface NamedScaleRange {
    /**
     * The named scale range, such as "room" or "metropolitan area".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#NamedScaleRange)
     */
    id: string;
    /**
     * The minimum scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#NamedScaleRange)
     */
    minScale: number;
    /**
     * The maximum scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRanges.html#NamedScaleRange)
     */
    maxScale: number;
  }

  export class ScaleRangeSliderViewModel extends Accessor {
    /**
     * Provides the logic for the [ScaleRangeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html)
     */

    constructor(properties?: ScaleRangeSliderViewModelProperties);

    /**
     * When provided, the initial [minScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScale) and [maxScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScale) values will match the layer's.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#layer)
     */
    layer: Layer;
    /**
     * The maximum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScale)
     */
    maxScale: number;
    /**
     * The lowest possible maximum scale value from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScaleLimit)
     */
    maxScaleLimit: number;
    /**
     * The minimum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScale)
     */
    minScale: number;
    /**
     * The highest possible minimum scale value from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScaleLimit)
     */
    minScaleLimit: number;
    /**
     * The valid scale ranges available based on the slider position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#scaleRanges)
     */
    readonly scaleRanges: ScaleRanges;
    /**
     * The [SliderViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html) for supporting the scale range slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#sliderViewModel)
     */
    sliderViewModel: SliderViewModel;
    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Utility method for converting scale-to-slider values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#mapScaleToSlider)
     */
    mapScaleToSlider(scale: number): number;
    /**
     * Utility method for converting slider-to-scale values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#mapSliderToScale)
     */
    mapSliderToScale(value: number): number;
  }

  interface ScaleRangeSliderViewModelProperties {
    /**
     * When provided, the initial [minScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScale) and [maxScale](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScale) values will match the layer's.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#layer)
     */
    layer?: LayerProperties;
    /**
     * The maximum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScale)
     */
    maxScale?: number;
    /**
     * The lowest possible maximum scale value from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#maxScaleLimit)
     */
    maxScaleLimit?: number;
    /**
     * The minimum scale of the active scale range.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScale)
     */
    minScale?: number;
    /**
     * The highest possible minimum scale value from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#minScaleLimit)
     */
    minScaleLimit?: number;
    /**
     * The [SliderViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html) for supporting the scale range slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#sliderViewModel)
     */
    sliderViewModel?: SliderViewModelProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider-ScaleRangeSliderViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The region that the scale thumbnails will focus on.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#SupportedRegion)
   */
  export type SupportedRegion =
    | "AE"
    | "AR"
    | "AT"
    | "AU"
    | "BE"
    | "BG"
    | "BO"
    | "BR"
    | "CA"
    | "CH"
    | "CI"
    | "CL"
    | "CN"
    | "CO"
    | "CR"
    | "CZ"
    | "DE"
    | "DK"
    | "EE"
    | "EG"
    | "ES"
    | "FI"
    | "FR"
    | "GB"
    | "GL"
    | "GR"
    | "GT"
    | "HK"
    | "ID"
    | "IE"
    | "IL"
    | "IN"
    | "IQ"
    | "IS"
    | "IT"
    | "JP"
    | "KE"
    | "KR"
    | "KW"
    | "LI"
    | "LT"
    | "LU"
    | "LV"
    | "MA"
    | "MG"
    | "ML"
    | "MO"
    | "MX"
    | "MY"
    | "NI"
    | "NL"
    | "NO"
    | "NZ"
    | "PE"
    | "PL"
    | "PR"
    | "PT"
    | "RO"
    | "RU"
    | "RW"
    | "SE"
    | "SG"
    | "SK"
    | "SR"
    | "SV"
    | "TH"
    | "TN"
    | "TW"
    | "US"
    | "VE"
    | "VI"
    | "ZA";

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#VisibleElements)
   */
  export interface ScaleRangeSliderVisibleElements {
    /**
     * Indicates whether the preview thumbnail of the scale is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#VisibleElements)
     */
    preview?: boolean;
    /**
     * _Since 4.24_ Indicates whether the scale dropdown menus are visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#VisibleElements)
     */
    scaleMenus?: boolean | VisibleElementsScaleMenus;
  }

  export interface VisibleElementsScaleMenus {
    /**
     * Indicates whether the minimum scale dropdown menu is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#VisibleElements)
     */
    minScaleMenu?: boolean;
    /**
     * Indicates whether the maximum scale dropdown menu is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ScaleRangeSlider.html#VisibleElements)
     */
    maxScaleMenu?: boolean;
  }

  export interface widgetsSearch extends Widget, GoTo {}

  export class widgetsSearch {
    /**
     * The Search widget provides a way to perform search operations on [locator service(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html), [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html)/[feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) service feature layer(s), [SceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) with an associated feature layer, [BuildingComponentSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html) with an associated feature layer, [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html), [OGCFeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html), and/or [table(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html)
     */

    constructor(properties?: widgetsSearchProperties);

    /**
     * The current active menu of the Search widget.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#activeMenu)
     */
    activeMenu: "none" | "suggestion" | "source" | "warning";
    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) object currently selected.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#activeSource)
     */
    readonly activeSource: LayerSearchSource | LocatorSearchSource;
    /**
     * The selected source's index.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#activeSourceIndex)
     */
    activeSourceIndex: number;
    /**
     * String value used as a hint for input text when searching on multiple sources.
     *
     * @default "Find address or place"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#allPlaceholder)
     */
    allPlaceholder: string;
    /**
     * The combined collection of [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) and [sources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#allSources)
     */
    readonly allSources: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * Indicates whether to automatically select and zoom to the first geocoded result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#autoSelect)
     */
    autoSelect: boolean;
    /**
     * A read-only property that is a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [LayerSearchSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html) and/or [LocatorSearchSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#defaultSources)
     */
    readonly defaultSources: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether or not to include [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) in the Search UI.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#includeDefaultSources)
     */
    includeDefaultSources: boolean | Function;
    /**
     * Enables location services within the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#locationEnabled)
     */
    locationEnabled: boolean;
    /**
     * The maximum number of results returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#maxResults)
     */
    maxResults: number;
    /**
     * The maximum number of suggestions returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#maxSuggestions)
     */
    maxSuggestions: number;
    /**
     * The minimum number of characters needed for the search if not specified by the source.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#minSuggestCharacters)
     */
    minSuggestCharacters: number;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) on feature click.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * A customized [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * It is possible to search a specified portal instance's [locator services](http://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm) Use this property to set this [ArcGIS Portal](https://enterprise.arcgis.com/en/portal/) instance to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#portal)
     */
    portal: Portal;
    /**
     * The graphic used to highlight the resulting feature or location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#resultGraphic)
     */
    readonly resultGraphic: Graphic;
    /**
     * Indicates if the [resultGraphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#resultGraphic) will display at the location of the selected feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#resultGraphicEnabled)
     */
    resultGraphicEnabled: boolean;
    /**
     * An array of objects, each containing a [SearchResult](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult) from the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#results)
     */
    readonly results: any[];
    /**
     * Indicates whether to display the option to search all sources.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#searchAllEnabled)
     */
    searchAllEnabled: boolean;
    /**
     * The value of the search box input text string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#searchTerm)
     */
    searchTerm: string;
    /**
     * The result selected from a search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#selectedResult)
     */
    readonly selectedResult: SearchResult;
    /**
     * The Search widget may be used to search features in a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html)/[feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) service feature layer(s), [SceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) with an associated feature layer, [BuildingComponentSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html) with an associated feature layer, [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html) or [OGCFeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html), or [table](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html), or geocode locations with a [locator](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources)
     */
    sources: Collection<SearchSource>;
    /**
     * An array of results from the [suggest method](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggest).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggestions)
     */
    readonly suggestions: SuggestResult[];
    /**
     * Enable suggestions for the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggestionsEnabled)
     */
    suggestionsEnabled: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#viewModel)
     */
    viewModel: SearchViewModel;

    /**
     * Unfocuses the widget's text input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#blur)
     */
    blur(): void;
    /**
     * Clears the current searchTerm, search results, suggest results, graphic, and graphics layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#clear)
     */
    clear(): void;
    /**
     * Brings focus to the widget's text input.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#focus)
     */
    focus(): void;
    /**
     * Depending on the sources specified, search() queries the feature layer(s) and/or performs address matching using any specified [locator(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html) and returns any applicable results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#search)
     */
    search(searchTerm?: string | Geometry | SuggestResult | number[][]): Promise<SearchResponse>;
    /**
     * Performs a suggest() request on the active Locator.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggest)
     */
    suggest(value?: string): Promise<SuggestResponse>;

    on(name: "search-blur", eventHandler: SearchSearchBlurEventHandler): IHandle;

    on(name: "search-focus", eventHandler: SearchSearchFocusEventHandler): IHandle;

    on(name: "search-clear", eventHandler: SearchSearchClearEventHandler): IHandle;

    on(name: "search-start", eventHandler: SearchSearchStartEventHandler): IHandle;

    on(name: "suggest-start", eventHandler: SearchSuggestStartEventHandler): IHandle;

    on(name: "search-complete", eventHandler: SearchSearchCompleteEventHandler): IHandle;

    on(name: "select-result", eventHandler: SearchSelectResultEventHandler): IHandle;

    on(name: "suggest-complete", eventHandler: SearchSuggestCompleteEventHandler): IHandle;
  }

  interface widgetsSearchProperties extends WidgetProperties, GoToProperties {
    /**
     * The current active menu of the Search widget.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#activeMenu)
     */
    activeMenu?: "none" | "suggestion" | "source" | "warning";
    /**
     * The selected source's index.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#activeSourceIndex)
     */
    activeSourceIndex?: number;
    /**
     * String value used as a hint for input text when searching on multiple sources.
     *
     * @default "Find address or place"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#allPlaceholder)
     */
    allPlaceholder?: string;
    /**
     * Indicates whether to automatically select and zoom to the first geocoded result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#autoSelect)
     */
    autoSelect?: boolean;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether or not to include [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) in the Search UI.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#includeDefaultSources)
     */
    includeDefaultSources?: boolean | Function;
    /**
     * Enables location services within the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#locationEnabled)
     */
    locationEnabled?: boolean;
    /**
     * The maximum number of results returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#maxResults)
     */
    maxResults?: number;
    /**
     * The maximum number of suggestions returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#maxSuggestions)
     */
    maxSuggestions?: number;
    /**
     * The minimum number of characters needed for the search if not specified by the source.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#minSuggestCharacters)
     */
    minSuggestCharacters?: number;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) on feature click.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * A customized [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for the selected feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * It is possible to search a specified portal instance's [locator services](http://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm) Use this property to set this [ArcGIS Portal](https://enterprise.arcgis.com/en/portal/) instance to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#portal)
     */
    portal?: PortalProperties;
    /**
     * Indicates if the [resultGraphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#resultGraphic) will display at the location of the selected feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#resultGraphicEnabled)
     */
    resultGraphicEnabled?: boolean;
    /**
     * Indicates whether to display the option to search all sources.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#searchAllEnabled)
     */
    searchAllEnabled?: boolean;
    /**
     * The value of the search box input text string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#searchTerm)
     */
    searchTerm?: string;
    /**
     * The Search widget may be used to search features in a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html)/[feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) service feature layer(s), [SceneLayers](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-SceneLayer.html) with an associated feature layer, [BuildingComponentSublayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-buildingSublayers-BuildingComponentSublayer.html) with an associated feature layer, [GeoJSONLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GeoJSONLayer.html), [CSVLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-CSVLayer.html) or [OGCFeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-OGCFeatureLayer.html), or [table](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html), or geocode locations with a [locator](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources)
     */
    sources?: CollectionProperties<SearchSourceProperties>;
    /**
     * Enable suggestions for the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggestionsEnabled)
     */
    suggestionsEnabled?: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#viewModel)
     */
    viewModel?: SearchViewModelProperties;
  }

  export interface LayerSearchSource extends SearchSource, JSONSupport {}

  export class LayerSearchSource {
    /**
     * The following properties define a [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)-based [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) whose features may be searched by a [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html)
     */

    constructor(properties?: LayerSearchSourceProperties);

    /**
     * The results are displayed using this field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#displayField)
     */
    displayField: string;
    /**
     * Indicates to only return results that match the search value exactly.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#exactMatch)
     */
    exactMatch: boolean;
    /**
     * The layer queried in the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#layer)
     */
    layer: Layer;
    /**
     * The name of the source for display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#name)
     */
    name: string;
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#orderByFields)
     */
    orderByFields: string[];
    /**
     * An array of string values representing the names of fields in the feature layer to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#searchFields)
     */
    searchFields: string[];
    /**
     * A template string used to display multiple fields in a defined order when results are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#searchTemplate)
     */
    searchTemplate: string;
    /**
     * A template string used to display multiple fields in a defined order when suggestions are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#suggestionTemplate)
     */
    suggestionTemplate: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#clone)
     */
    clone(): LayerSearchSource;

    static fromJSON(json: any): LayerSearchSource;
  }

  interface LayerSearchSourceProperties extends SearchSourceProperties {
    /**
     * The results are displayed using this field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#displayField)
     */
    displayField?: string;
    /**
     * Indicates to only return results that match the search value exactly.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#exactMatch)
     */
    exactMatch?: boolean;
    /**
     * The layer queried in the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#layer)
     */
    layer?: LayerProperties;
    /**
     * The name of the source for display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#name)
     */
    name?: string;
    /**
     * One or more field names used to order the query results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#orderByFields)
     */
    orderByFields?: string[];
    /**
     * An array of string values representing the names of fields in the feature layer to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#searchFields)
     */
    searchFields?: string[];
    /**
     * A template string used to display multiple fields in a defined order when results are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#searchTemplate)
     */
    searchTemplate?: string;
    /**
     * A template string used to display multiple fields in a defined order when suggestions are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html#suggestionTemplate)
     */
    suggestionTemplate?: string;
  }

  export interface LocatorSearchSource extends SearchSource, JSONSupport {}

  export class LocatorSearchSource {
    /**
     * The following properties define a [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) pointing to a [url](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#url) that represents a locator service, which may be used to geocode locations with a [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html)
     */

    constructor(properties?: LocatorSearchSourceProperties);

    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#apiKey)
     */
    apiKey: string;
    /**
     * A string array which limits the results to one or more categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#categories)
     */
    categories: string[];
    /**
     * Constricts search results to a specified country code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#countryCode)
     */
    countryCode: string;
    /**
     * Sets the scale of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#scale) for the resulting search result, if the locator service doesn’t return an extent with a scale.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#defaultZoomScale)
     */
    defaultZoomScale: number;
    /**
     * This property controls prioritization of [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget result candidates depending on the view scale.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#localSearchDisabled)
     */
    localSearchDisabled: boolean;
    /**
     * Defines the type of location, either `street` or `rooftop`, of the point returned from the [World Geocoding Service](https://developers.arcgis.com/rest/geocode/api-reference/geocoding-category-filtering.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#locationType)
     */
    locationType: "rooftop" | "street";
    /**
     * The name of the source for display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#name)
     */
    name: string;
    /**
     * A template string used to display multiple fields in a defined order when results are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#searchTemplate)
     */
    searchTemplate: string;
    /**
     * The field name of the Single Line Address Field in the REST services directory for the locator service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#singleLineFieldName)
     */
    singleLineFieldName: string;
    /**
     * URL to the ArcGIS Server REST resource that represents a locator service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#url)
     */
    url: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#clone)
     */
    clone(): LocatorSearchSource;

    static fromJSON(json: any): LocatorSearchSource;
  }

  interface LocatorSearchSourceProperties extends SearchSourceProperties {
    /**
     * An authorization string used to access a resource or service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#apiKey)
     */
    apiKey?: string;
    /**
     * A string array which limits the results to one or more categories.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#categories)
     */
    categories?: string[];
    /**
     * Constricts search results to a specified country code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#countryCode)
     */
    countryCode?: string;
    /**
     * Sets the scale of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#scale) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#scale) for the resulting search result, if the locator service doesn’t return an extent with a scale.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#defaultZoomScale)
     */
    defaultZoomScale?: number;
    /**
     * This property controls prioritization of [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget result candidates depending on the view scale.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#localSearchDisabled)
     */
    localSearchDisabled?: boolean;
    /**
     * Defines the type of location, either `street` or `rooftop`, of the point returned from the [World Geocoding Service](https://developers.arcgis.com/rest/geocode/api-reference/geocoding-category-filtering.htm).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#locationType)
     */
    locationType?: "rooftop" | "street";
    /**
     * The name of the source for display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#name)
     */
    name?: string;
    /**
     * A template string used to display multiple fields in a defined order when results are displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#searchTemplate)
     */
    searchTemplate?: string;
    /**
     * The field name of the Single Line Address Field in the REST services directory for the locator service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#singleLineFieldName)
     */
    singleLineFieldName?: string;
    /**
     * URL to the ArcGIS Server REST resource that represents a locator service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html#url)
     */
    url?: string;
  }

  export class SearchResultRenderer extends Widget {
    /**
     * The `SearchResultRenderer` renders the [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget results and allows expanding a DOM element to show alternative matches.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchResultRenderer.html)
     */

    constructor(properties?: SearchResultRendererProperties);

    /**
     * Indicates whether to display the `Show more results` link within the search result's popup.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchResultRenderer.html#showMoreResultsOpen)
     */
    showMoreResultsOpen: boolean;
    /**
     * The view model for this Search widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchResultRenderer.html#viewModel)
     */
    viewModel: SearchViewModel;
  }

  interface SearchResultRendererProperties extends WidgetProperties {
    /**
     * Indicates whether to display the `Show more results` link within the search result's popup.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchResultRenderer.html#showMoreResultsOpen)
     */
    showMoreResultsOpen?: boolean;
    /**
     * The view model for this Search widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchResultRenderer.html#viewModel)
     */
    viewModel?: SearchViewModelProperties;
  }

  export interface SearchSource extends Accessor, JSONSupport, Identifiable {}

  export class SearchSource {
    /**
     * The following properties define generic [sources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) properties for use in the [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html)
     */

    constructor(properties?: SearchSourceProperties);

    /**
     * Indicates whether to automatically navigate to the selected result once selected.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#autoNavigate)
     */
    autoNavigate: boolean;
    /**
     * For filtering suggests or search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#filter)
     */
    filter: SearchSourceFilter;
    /**
     * Function used to get search results.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#getResults)
     */
    getResults: GetResultsHandler;
    /**
     * Function used to get search suggestions.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#getSuggestions)
     */
    getSuggestions: GetSuggestionsParameters;
    /**
     * Indicates the maximum number of search results to return.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#maxResults)
     */
    maxResults: number;
    /**
     * Indicates the maximum number of suggestions to return for the widget's input.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#maxSuggestions)
     */
    maxSuggestions: number;
    /**
     * Indicates the minimum number of characters required before querying for a suggestion.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#minSuggestCharacters)
     */
    minSuggestCharacters: number;
    /**
     * Specifies the fields returned with the search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#outFields)
     */
    outFields: string[];
    /**
     * Used as a hint for the source input text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#placeholder)
     */
    placeholder: string;
    /**
     * Indicates whether to display a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when a selected result is clicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * The popup template used to display search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * Specify this to prefix the user's input of the search text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#prefix)
     */
    prefix: string;
    /**
     * Indicates whether to show a graphic on the map for the selected source using the [resultSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultGraphicEnabled)
     */
    resultGraphicEnabled: boolean;
    /**
     * The symbol used to display the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultSymbol)
     */
    resultSymbol: Symbol;
    /**
     * Specify this to add a suffix to the user's input for the search value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#suffix)
     */
    suffix: string;
    /**
     * Indicates whether to display suggestions as the user enters input text in the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#suggestionsEnabled)
     */
    suggestionsEnabled: boolean;
    /**
     * Indicates whether to constrain the search results to the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#withinViewEnabled)
     */
    withinViewEnabled: boolean;
    /**
     * The set zoom scale for the resulting search result.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#zoomScale)
     */
    zoomScale: number;

    static fromJSON(json: any): SearchSource;
  }

  interface SearchSourceProperties {
    /**
     * Indicates whether to automatically navigate to the selected result once selected.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#autoNavigate)
     */
    autoNavigate?: boolean;
    /**
     * For filtering suggests or search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#filter)
     */
    filter?: SearchSourceFilter;
    /**
     * Function used to get search results.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#getResults)
     */
    getResults?: GetResultsHandler;
    /**
     * Function used to get search suggestions.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#getSuggestions)
     */
    getSuggestions?: GetSuggestionsParameters;
    /**
     * Indicates the maximum number of search results to return.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#maxResults)
     */
    maxResults?: number;
    /**
     * Indicates the maximum number of suggestions to return for the widget's input.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#maxSuggestions)
     */
    maxSuggestions?: number;
    /**
     * Indicates the minimum number of characters required before querying for a suggestion.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#minSuggestCharacters)
     */
    minSuggestCharacters?: number;
    /**
     * Specifies the fields returned with the search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#outFields)
     */
    outFields?: string[];
    /**
     * Used as a hint for the source input text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#placeholder)
     */
    placeholder?: string;
    /**
     * Indicates whether to display a [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) when a selected result is clicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * The popup template used to display search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * Specify this to prefix the user's input of the search text.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#prefix)
     */
    prefix?: string;
    /**
     * Indicates whether to show a graphic on the map for the selected source using the [resultSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultSymbol).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultGraphicEnabled)
     */
    resultGraphicEnabled?: boolean;
    /**
     * The symbol used to display the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#resultSymbol)
     */
    resultSymbol?: SymbolProperties;
    /**
     * Specify this to add a suffix to the user's input for the search value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#suffix)
     */
    suffix?: string;
    /**
     * Indicates whether to display suggestions as the user enters input text in the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#suggestionsEnabled)
     */
    suggestionsEnabled?: boolean;
    /**
     * Indicates whether to constrain the search results to the view's extent.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#withinViewEnabled)
     */
    withinViewEnabled?: boolean;
    /**
     * The set zoom scale for the resulting search result.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#zoomScale)
     */
    zoomScale?: number;
  }

  export type GetResultsHandler = (params: any) => Promise<SearchResult[]>;

  export type GetSuggestionsParameters = (params: any) => Promise<SuggestResult[]>;

  export interface SearchSourceFilter {
    /**
     * The where clause specified for filtering suggests or search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#filter)
     */
    where?: string;
    /**
     * The filter geometry for suggests or search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchSource.html#filter)
     */
    geometry?: Geometry;
  }

  export interface SearchViewModel extends Accessor, Evented, GoTo {}

  export class SearchViewModel {
    /**
     * Provides the logic for the [Search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html) widget, which performs search operations on [locator service(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html), [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-MapImageLayer.html)/[feature](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) service feature layer(s), and/or [table(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html)
     */

    constructor(properties?: SearchViewModelProperties);

    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources) object currently selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#activeSource)
     */
    readonly activeSource: LayerSearchSource | LocatorSearchSource;
    /**
     * The selected source's index.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#activeSourceIndex)
     */
    activeSourceIndex: number;
    /**
     * String value used as a hint for input text when searching on multiple sources.
     *
     * @default "Find address or place"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#allPlaceholder)
     */
    allPlaceholder: string;
    /**
     * The combined collection of [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) and [sources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#allSources)
     */
    readonly allSources: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * Indicates whether to automatically select and zoom to the first geocoded result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#autoSelect)
     */
    autoSelect: boolean;
    /**
     * A read-only property that is a [Collection](https://developers.arcgis.com/javascript/latest/api-reference/esri-core-Collection.html) of [LayerSearchSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LayerSearchSource.html) and/or [LocatorSearchSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-LocatorSearchSource.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources)
     */
    readonly defaultSources: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * The default [symbol(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) for the search result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSymbols)
     */
    defaultSymbols: SearchViewModelDefaultSymbols;
    /**
     * Indicates whether or not to include [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) in the Search UI.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#includeDefaultSources)
     */
    includeDefaultSources: boolean | Function;
    /**
     * Indicates whether location services are enabled within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#locationEnabled)
     */
    locationEnabled: boolean;
    /**
     * The maximum character length of the search text.
     *
     * @default 128
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxInputLength)
     */
    maxInputLength: number;
    /**
     * The maximum number of results returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxResults)
     */
    maxResults: number;
    /**
     * The maximum number of suggestions returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxSuggestions)
     */
    maxSuggestions: number;
    /**
     * The minimum number of characters needed for the search if not specified by the source.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#minSuggestCharacters)
     */
    minSuggestCharacters: number;
    /**
     * The placeholder used by the [activeSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#activeSource).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#placeholder)
     */
    readonly placeholder: string;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) on feature click.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#popupEnabled)
     */
    popupEnabled: boolean;
    /**
     * A customized [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for the selected feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#popupTemplate)
     */
    popupTemplate: PopupTemplate;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm) Use this property to set this [ArcGIS Portal](https://enterprise.arcgis.com/en/portal/) instance to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#portal)
     */
    portal: Portal;
    /**
     * The graphic used to highlight the resulting feature or location.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#resultGraphic)
     */
    readonly resultGraphic: Graphic;
    /**
     * Indicates if the [resultGraphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#resultGraphic) will display at the location of the selected feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#resultGraphicEnabled)
     */
    resultGraphicEnabled: boolean;
    /**
     * An array of current results from the search.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#results)
     */
    readonly results: any[];
    /**
     * Indicates whether to display the option to search all sources.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#searchAllEnabled)
     */
    searchAllEnabled: boolean;
    /**
     * The value of the search box input text string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#searchTerm)
     */
    searchTerm: string;
    /**
     * The result selected from a search.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#selectedResult)
     */
    readonly selectedResult: any;
    /**
     * The selected SuggestResult.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#selectedSuggestion)
     */
    readonly selectedSuggestion: Geometry | SearchViewModelSuggestResult;
    /**
     * The Search widget may be used to search features in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [table](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html), or geocode locations with a [locator](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources)
     */
    sources: Collection<LayerSearchSource | LocatorSearchSource>;
    /**
     * The current state of the widget.
     *
     * @default "ready"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#state)
     */
    readonly state: "ready" | "disabled" | "searching" | "loading";
    /**
     * The millisecond delay after keyup and before making a [suggest](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest) network request.
     *
     * @default 350
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggestionDelay)
     */
    suggestionDelay: number;
    /**
     * An array of results from the [suggest method](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggestions)
     */
    readonly suggestions: SearchViewModelSuggestResult[];
    /**
     * Enable suggestions for the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggestionsEnabled)
     */
    suggestionsEnabled: boolean;
    /**
     * Indicates whether the `View` or `Portal` is loading resources prior to use.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#updating)
     */
    readonly updating: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Clears the current value, search results, suggest results, graphic, and graphics layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#clear)
     */
    clear(): void;
    /**
     * Depending on the sources specified, `search()` queries the feature layer(s) and/or performs address matching using any specified [Locator(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html) and returns the applicable results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#search)
     */
    search(
      searchTerm?: string | Geometry | SearchViewModelSuggestResult | number[][],
    ): Promise<SearchViewModelSearchResponse>;
    /**
     * Returns search results near your current location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#searchNearby)
     */
    searchNearby(): Promise<SearchViewModelSearchResponse>;
    /**
     * Performs a suggest() request on the active Locator.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest)
     */
    suggest(value?: string): Promise<SearchViewModelSuggestResponse>;

    on(name: "search-clear", eventHandler: SearchViewModelSearchClearEventHandler): IHandle;

    on(name: "search-start", eventHandler: SearchViewModelSearchStartEventHandler): IHandle;

    on(name: "suggest-start", eventHandler: SearchViewModelSuggestStartEventHandler): IHandle;

    on(name: "search-complete", eventHandler: SearchViewModelSearchCompleteEventHandler): IHandle;

    on(name: "select-result", eventHandler: SearchViewModelSelectResultEventHandler): IHandle;

    on(name: "suggest-complete", eventHandler: SearchViewModelSuggestCompleteEventHandler): IHandle;
  }

  interface SearchViewModelProperties extends GoToProperties {
    /**
     * The selected source's index.
     *
     * @default 0
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#activeSourceIndex)
     */
    activeSourceIndex?: number;
    /**
     * String value used as a hint for input text when searching on multiple sources.
     *
     * @default "Find address or place"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#allPlaceholder)
     */
    allPlaceholder?: string;
    /**
     * Indicates whether to automatically select and zoom to the first geocoded result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#autoSelect)
     */
    autoSelect?: boolean;
    /**
     * The default [symbol(s)](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-Symbol.html) for the search result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSymbols)
     */
    defaultSymbols?: SearchViewModelDefaultSymbols;
    /**
     * Indicates whether or not to include [defaultSources](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSources) in the Search UI.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#includeDefaultSources)
     */
    includeDefaultSources?: boolean | Function;
    /**
     * Indicates whether location services are enabled within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#locationEnabled)
     */
    locationEnabled?: boolean;
    /**
     * The maximum character length of the search text.
     *
     * @default 128
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxInputLength)
     */
    maxInputLength?: number;
    /**
     * The maximum number of results returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxResults)
     */
    maxResults?: number;
    /**
     * The maximum number of suggestions returned by the widget if not specified by the source.
     *
     * @default 6
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#maxSuggestions)
     */
    maxSuggestions?: number;
    /**
     * The minimum number of characters needed for the search if not specified by the source.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#minSuggestCharacters)
     */
    minSuggestCharacters?: number;
    /**
     * Indicates whether to display the [Popup](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Popup.html) on feature click.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#popupEnabled)
     */
    popupEnabled?: boolean;
    /**
     * A customized [PopupTemplate](https://developers.arcgis.com/javascript/latest/api-reference/esri-PopupTemplate.html) for the selected feature.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#popupTemplate)
     */
    popupTemplate?: PopupTemplateProperties;
    /**
     * It is possible to search a specified portal instance's [locator services](https://enterprise.arcgis.com/en/portal/latest/administer/windows/configure-portal-to-geocode-addresses.htm) Use this property to set this [ArcGIS Portal](https://enterprise.arcgis.com/en/portal/) instance to search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#portal)
     */
    portal?: PortalProperties;
    /**
     * Indicates if the [resultGraphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#resultGraphic) will display at the location of the selected feature.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#resultGraphicEnabled)
     */
    resultGraphicEnabled?: boolean;
    /**
     * Indicates whether to display the option to search all sources.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#searchAllEnabled)
     */
    searchAllEnabled?: boolean;
    /**
     * The value of the search box input text string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#searchTerm)
     */
    searchTerm?: string;
    /**
     * The Search widget may be used to search features in a [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) or [table](https://developers.arcgis.com/javascript/latest/api-reference/esri-webdoc-applicationProperties-SearchTable.html), or geocode locations with a [locator](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-locator.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources)
     */
    sources?: CollectionProperties<LayerSearchSourceProperties | LocatorSearchSourceProperties>;
    /**
     * The millisecond delay after keyup and before making a [suggest](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest) network request.
     *
     * @default 350
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggestionDelay)
     */
    suggestionDelay?: number;
    /**
     * Enable suggestions for the widget.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggestionsEnabled)
     */
    suggestionsEnabled?: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface SearchViewModelSearchClearEvent {}

  export interface SearchViewModelSearchCompleteEvent {
    activeSourceIndex: number;

    errors: Error[];

    numResults: number;

    results: SearchViewModelSearchCompleteEventResults[];

    searchTerm: string;
  }

  /**
   * When resolved, returns this response after calling [search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#search).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
   */
  export interface SearchViewModelSearchResponse {
    /**
     * The index of the source from which the search result was obtained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    activeSourceIndex: number;
    /**
     * An array of error objects returned from the search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    errors: Error[];
    /**
     * The number of search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    numResults: number;
    /**
     * The searched expression
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    searchTerm: string;
    /**
     * An array of objects representing the results of search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    results: SearchViewModelSearchResponseResults[];
  }

  /**
   * The result object returned from the [search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#search).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResult)
   */
  export interface SearchViewModelSearchResult {
    /**
     * The extent, or bounding box, of the returned feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResult)
     */
    extent: Extent;
    /**
     * The resulting feature or location obtained from the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResult)
     */
    feature: Graphic;
    /**
     * The name of the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResult)
     */
    name: string;
  }

  export interface SearchViewModelSearchStartEvent {}

  export interface SearchViewModelDefaultSymbols {
    /**
     * A [SimpleMarkerSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html), [PointSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html), [TextSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html), [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html), or [WebStyleSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html) used for representing the point geometry that is being drawn for the search result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSymbols)
     */
    point?: SimpleMarkerSymbol | PointSymbol3D | TextSymbol | CIMSymbol | WebStyleSymbol;
    /**
     * A [SimpleFillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html), [PolygonSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polygon geometry that is being drawn for the search result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSymbols)
     */
    polygon?: SimpleFillSymbol | PolygonSymbol3D | CIMSymbol;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html), [LineSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polyline geometry that is being drawn for the search result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#defaultSymbols)
     */
    polyline?: SimpleLineSymbol | LineSymbol3D | CIMSymbol;
  }

  export interface SearchViewModelSelectResultEvent {
    result: SearchViewModelSelectResultEventResult;

    source: any;

    sourceIndex: number;
  }

  export interface SearchViewModelSuggestCompleteEvent {
    activeSourceIndex: number;

    errors: Error[];

    numResults: number;

    results: SearchViewModelSuggestCompleteEventResults[];

    searchTerm: string;
  }

  /**
   * When resolved, returns this response after calling [suggest](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
   */
  export interface SearchViewModelSuggestResponse {
    /**
     * The index of the source from which suggestions are obtained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    activeSourceIndex: number;
    /**
     * An array of error objects returned from the suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    errors: Error[];
    /**
     * The number of suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    numResults: number;
    /**
     * The search expression used for the suggest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    searchTerm: string;
    /**
     * An array of objects representing the results of suggest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    results: SearchViewModelSuggestResponseResults[];
  }

  /**
   * The result object returned from a [suggest](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#suggest).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResult)
   */
  export interface SearchViewModelSuggestResult {
    /**
     * The key related to the suggest result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResult)
     */
    key: string;
    /**
     * The string name of the suggested location to geocode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResult)
     */
    text: string;
    /**
     * The index of the currently selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResult)
     */
    sourceIndex: number;
  }

  export interface SearchViewModelSuggestStartEvent {}

  export interface SearchViewModelSearchCompleteEventResults {
    results: SearchResult[];

    sourceIndex: number;

    source: any[];
  }

  export interface SearchViewModelSearchResponseResults {
    /**
     * An array of search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    results: SearchViewModelSearchResult[];
    /**
     * The index of the currently selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    sourceIndex: number;
    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources) of the selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SearchResponse)
     */
    source: any;
  }

  export interface SearchViewModelSelectResultEventResult {
    extent: Extent;

    feature: Graphic;

    name: string;
  }

  export interface SearchViewModelSuggestCompleteEventResults {
    results: SearchViewModelSuggestResult[];

    sourceIndex: number;

    source: any;
  }

  export interface SearchViewModelSuggestResponseResults {
    /**
     * An array of suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    results: SearchViewModelSuggestResult[];
    /**
     * The index of the currently selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    sourceIndex: number;
    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#sources) of the selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search-SearchViewModel.html#SuggestResponse)
     */
    source: any;
  }

  export interface SearchSearchBlurEvent {}

  export interface SearchSearchClearEvent {}

  export interface SearchSearchCompleteEvent {
    activeSourceIndex: number;

    errors: Error[];

    numResults: number;

    results: SearchSearchCompleteEventResults[];

    searchTerm: string;
  }

  export interface SearchSearchFocusEvent {}

  /**
   * When resolved, returns this response after calling [search](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#search).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
   */
  export interface SearchResponse {
    /**
     * The index of the source from which the search result was obtained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    activeSourceIndex: number;
    /**
     * An array of error objects returned from the search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    errors: Error[];
    /**
     * The number of search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    numResults: number;
    /**
     * The searched expression
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    searchTerm: string;
    /**
     * An array of objects representing the results of search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    results: SearchResponseResults[];
  }

  /**
   * The result object returned from a [search()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#search).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult)
   */
  export interface SearchResult {
    /**
     * The extent, or bounding box, of the returned feature.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult)
     */
    extent: Extent;
    /**
     * The resulting feature or location obtained from the search.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult)
     */
    feature: Graphic;
    /**
     * The name of the result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult)
     */
    name: string;
    /**
     * The target of the result, which is a [Graphic](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) used for either [MapView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo) or [SceneView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo) navigation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResult)
     */
    target: Graphic;
  }

  export interface SearchSearchStartEvent {}

  export interface SearchSelectResultEvent {
    result: SearchSelectResultEventResult;

    source: any;

    sourceIndex: number;
  }

  export interface SearchSuggestCompleteEvent {
    activeSourceIndex: number;

    errors: Error[];

    numResults: number;

    results: SearchSuggestCompleteEventResults[];

    searchTerm: string;
  }

  /**
   * When resolved, returns this response after calling [suggest](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggest).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
   */
  export interface SuggestResponse {
    /**
     * The index of the source from which suggestions are obtained.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    activeSourceIndex: number;
    /**
     * An array of error objects returned from the suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    errors: Error[];
    /**
     * The number of suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    numResults: number;
    /**
     * The search expression used for the suggest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    searchTerm: string;
    /**
     * An array of objects representing the results of suggest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    results: SuggestResponseResults[];
  }

  /**
   * The result object returned from a [suggest()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#suggest).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResult)
   */
  export interface SuggestResult {
    /**
     * The key related to the suggest result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResult)
     */
    key: string;
    /**
     * The string name of the suggested location to geocode.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResult)
     */
    text: string;
    /**
     * The index of the currently selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResult)
     */
    sourceIndex: number;
  }

  export interface SearchSuggestStartEvent {}

  export interface SearchSearchCompleteEventResults {
    results: SearchResult[];

    sourceIndex: number;

    source: any;
  }

  export interface SearchResponseResults {
    /**
     * An array of search results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    results: SearchResult[];
    /**
     * The index of the currently selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    sourceIndex: number;
    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) of the selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SearchResponse)
     */
    source: any;
  }

  export interface SearchSelectResultEventResult {
    extent: Extent;

    feature: Graphic;

    name: string;
  }

  export interface SearchSuggestCompleteEventResults {
    results: SuggestResult[];

    sourceIndex: number;

    source: any;
  }

  export interface SuggestResponseResults {
    /**
     * An array of suggest results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    results: SuggestResult[];
    /**
     * The index of the currently selected source.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    sourceIndex: number;
    /**
     * The [source](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#sources) of the selected result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Search.html#SuggestResponse)
     */
    source: any;
  }

  export class ShadowCast extends Widget {
    /**
     * The ShadowCast widget displays the cumulative shadows of 3D features in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html)
     */

    constructor(properties?: ShadowCastProperties);

    /**
     * Indicates the heading level to use for the titles "Time range" and "Visualization".
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#viewModel)
     */
    viewModel: ShadowCastViewModel;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#visibleElements)
     */
    visibleElements: ShadowCastVisibleElements;
  }

  interface ShadowCastProperties extends WidgetProperties {
    /**
     * Indicates the heading level to use for the titles "Time range" and "Visualization".
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#viewModel)
     */
    viewModel?: ShadowCastViewModelProperties;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#visibleElements)
     */
    visibleElements?: ShadowCastVisibleElements;
  }

  export class ShadowCastViewModel extends Accessor {
    /**
     * Provides the logic for the [ShadowCast](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html)
     */

    constructor(properties?: ShadowCastViewModelProperties);

    /**
     * The calendar date used to calculate the shadow cast.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)
     */
    date: Date;
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "discrete".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    discreteOptions: ShadowCastViewModelDiscreteOptions;
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "duration".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    durationOptions: ShadowCastViewModelDurationOptions;
    /**
     * Time (in milliseconds from midnight of the [date](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)) when the shadow cast computation should stop.
     *
     * @default 16 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#endTimeOfDay)
     */
    endTimeOfDay: number;
    /**
     * Time (in milliseconds from midnight of the [date](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)) when the shadow cast computation should start.
     *
     * @default 10 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#startTimeOfDay)
     */
    startTimeOfDay: number;
    /**
     * The current state of the view model that can be used for rendering the UI of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "threshold".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    thresholdOptions: ShadowCastViewModelThresholdOptions;
    /**
     * The difference in hours between UTC time and the times displayed in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#utcOffset)
     */
    utcOffset: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#view)
     */
    view: SceneView;
    /**
     * Type of visualization to use when showing the shadows.
     *
     * @default "threshold"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType)
     */
    visualizationType: "threshold" | "duration" | "discrete";

    /**
     * Returns the time (in milliseconds) spent in shadow for a certain point on the screen.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#getDuration)
     */
    getDuration(point: SceneViewScreenPoint): number;
    /**
     * Starts the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#start)
     */
    start(): void;
    /**
     * Stops the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#stop)
     */
    stop(): void;
  }

  interface ShadowCastViewModelProperties {
    /**
     * The calendar date used to calculate the shadow cast.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)
     */
    date?: DateProperties;
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "discrete".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    discreteOptions?: ShadowCastViewModelDiscreteOptionsProperties;
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "duration".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    durationOptions?: ShadowCastViewModelDurationOptionsProperties;
    /**
     * Time (in milliseconds from midnight of the [date](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)) when the shadow cast computation should stop.
     *
     * @default 16 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#endTimeOfDay)
     */
    endTimeOfDay?: number;
    /**
     * Time (in milliseconds from midnight of the [date](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#date)) when the shadow cast computation should start.
     *
     * @default 10 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#startTimeOfDay)
     */
    startTimeOfDay?: number;
    /**
     * The configuration used when the widget's [visualizationType](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType) is set to "threshold".
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    thresholdOptions?: ShadowCastViewModelThresholdOptionsProperties;
    /**
     * The difference in hours between UTC time and the times displayed in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#utcOffset)
     */
    utcOffset?: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#view)
     */
    view?: SceneViewProperties;
    /**
     * Type of visualization to use when showing the shadows.
     *
     * @default "threshold"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#visualizationType)
     */
    visualizationType?: "threshold" | "duration" | "discrete";
  }

  export interface ShadowCastViewModelDiscreteOptionsProperties {
    /**
     * Color of the shadow visualization.
     *
     * @default [50, 50, 50, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    color?: Color;
    /**
     * Individual shadows are displayed at this time interval, starting with the [startTimeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#startTimeOfDay).
     *
     * @default 15 * 60 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    interval?: number;
    /**
     * Values (in milliseconds) selectable in the UI for the interval used to display shadows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    intervalOptions?: CollectionProperties<number>;
  }

  export interface ShadowCastViewModelDiscreteOptions extends AnonymousAccessor {
    /**
     * Color of the shadow visualization.
     *
     * @default [50, 50, 50, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    color?: Color;
    /**
     * Individual shadows are displayed at this time interval, starting with the [startTimeOfDay](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#startTimeOfDay).
     *
     * @default 15 * 60 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    interval?: number;
    /**
     * Values (in milliseconds) selectable in the UI for the interval used to display shadows.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#discreteOptions)
     */
    intervalOptions?: Collection<number>;
  }

  export interface ShadowCastViewModelDurationOptionsProperties {
    /**
     * Color of the shadow cast.
     *
     * @default [0, 0, 255, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    color?: Color;
    /**
     * Mode in which the cumulative shadow duration should be displayed: as a continuous surface or as an hourly aggregation of values.
     *
     * @default "continuous"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    mode?: "continuous" | "hourly";
  }

  export interface ShadowCastViewModelDurationOptions extends AnonymousAccessor {
    /**
     * Color of the shadow cast.
     *
     * @default [0, 0, 255, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    color?: Color;
    /**
     * Mode in which the cumulative shadow duration should be displayed: as a continuous surface or as an hourly aggregation of values.
     *
     * @default "continuous"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#durationOptions)
     */
    mode?: "continuous" | "hourly";
  }

  export interface ShadowCastViewModelThresholdOptionsProperties {
    /**
     * Color of the shadow visualization.
     *
     * @default [255, 0 , 0, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    color?: Color;
    /**
     * Time period in milliseconds.
     *
     * @default 4 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    value?: number;
    /**
     * The minimum time period (in milliseconds) selectable in the UI for the threshold value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    minValue?: number;
    /**
     * The maximum time period (in milliseconds) selectable in the UI for the threshold value.
     *
     * @default 8 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    maxValue?: number;
  }

  export interface ShadowCastViewModelThresholdOptions extends AnonymousAccessor {
    /**
     * Color of the shadow visualization.
     *
     * @default [255, 0 , 0, 0.7]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    color?: Color;
    /**
     * Time period in milliseconds.
     *
     * @default 4 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    value?: number;
    /**
     * The minimum time period (in milliseconds) selectable in the UI for the threshold value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    minValue?: number;
    /**
     * The maximum time period (in milliseconds) selectable in the UI for the threshold value.
     *
     * @default 8 * 3600 * 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast-ShadowCastViewModel.html#thresholdOptions)
     */
    maxValue?: number;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
   */
  export interface ShadowCastVisibleElements {
    /**
     * When set to `false`, the slider used to select a time range for the analysis is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    timeRangeSlider?: boolean;
    /**
     * When set to `false`, the dropdown used to select a timezone for the time range is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    timezone?: boolean;
    /**
     * When set to `false`, the date picker is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    datePicker?: boolean;
    /**
     * When set to `false`, the options for the various visualization modes are not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    visualizationOptions?: boolean;
    /**
     * When set to `false`, the color picker is not displayed in the options for any of the visualization modes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    colorPicker?: boolean;
    /**
     * When set to `false`, the tooltip with the accumulated shadow time is not displayed when hovering the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ShadowCast.html#VisibleElements)
     */
    tooltip?: boolean;
  }

  export class Sketch extends Widget {
    /**
     * Sketch widget provides a simple UI for creating and updating graphics on a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html)
     */

    constructor(properties?: SketchProperties);

    /**
     * When creating new graphics (for example after [create()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create) has been called), this property reflects the create tool being used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#activeTool)
     */
    readonly activeTool:
      | "point"
      | "polyline"
      | "polygon"
      | "circle"
      | "rectangle"
      | "move"
      | "transform"
      | "reshape"
      | "rectangle-selection"
      | "lasso-selection";
    /**
     * The tooltip currently being displayed for the [activeTool](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTool).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#activeTooltip)
     */
    readonly activeTooltip: Tooltip;
    /**
     * Property controlling the visibility and order of create tool buttons.
     *
     * @default ["point", "polyline", "polygon", "rectangle", "circle"]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#availableCreateTools)
     */
    availableCreateTools: string[];
    /**
     * The graphic that is being created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#createGraphic)
     */
    readonly createGraphic: Graphic;
    /**
     * Defines the default behavior once the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create) operation is completed.
     *
     * @default "continuous"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#creationMode)
     */
    creationMode: "single" | "continuous" | "update";
    /**
     * Default create options set for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    defaultCreateOptions: SketchDefaultCreateOptions;
    /**
     * Default update options set for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    defaultUpdateOptions: SketchDefaultUpdateOptions;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#labelOptions)
     */
    labelOptions: SketchLabelOptions;
    /**
     * The [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) associated with the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#layer)
     */
    layer: GraphicsLayer;
    /**
     * Determines the layout/orientation of the Sketch widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#layout)
     */
    layout: "vertical" | "horizontal";
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The Sketch widget's state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#state)
     */
    readonly state: "ready" | "disabled" | "active";
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#tooltipOptions)
     */
    tooltipOptions: SketchTooltipOptions;
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) that are being updated by the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#updateGraphics)
     */
    readonly updateGraphics: Collection<Graphic>;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#valueOptions)
     */
    valueOptions: SketchValueOptions;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#viewModel)
     */
    viewModel: SketchViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#visibleElements)
     */
    visibleElements: SketchVisibleElements;

    /**
     * Cancels the active operation and fires the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-create) or [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#cancel)
     */
    cancel(): void;
    /**
     * Completes the active operation and fires the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-create) or [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event and changes the event's state to `complete`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#complete)
     */
    complete(): void;
    /**
     * Create a graphic with the geometry specified in the `tool` parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create)
     */
    create(
      tool: "point" | "polyline" | "polygon" | "rectangle" | "circle",
      createOptions?: SketchCreateCreateOptions,
    ): Promise<void>;
    /**
     * Deletes the selected graphics used in the update workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#delete)
     */
    delete(): void;
    /**
     * Duplicates current graphics used in the update workflow and automatically adds them to the associated layer
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#duplicate)
     */
    duplicate(): void;
    /**
     * Incrementally redo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#redo)
     */
    redo(): void;
    /**
     * Incrementally undo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#undo)
     */
    undo(): void;
    /**
     * Initializes an update operation for the specified graphic(s) and fires [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    update(graphics: Graphic | Graphic[], updateOptions?: SketchUpdateUpdateOptions): Promise<void>;

    on(name: "delete", eventHandler: SketchDeleteEventHandler): IHandle;

    on(name: "create", eventHandler: SketchCreateEventHandler): IHandle;

    on(name: "update", eventHandler: SketchUpdateEventHandler): IHandle;

    on(name: "redo", eventHandler: SketchRedoEventHandler): IHandle;

    on(name: "undo", eventHandler: SketchUndoEventHandler): IHandle;
  }

  interface SketchProperties extends WidgetProperties {
    /**
     * Property controlling the visibility and order of create tool buttons.
     *
     * @default ["point", "polyline", "polygon", "rectangle", "circle"]
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#availableCreateTools)
     */
    availableCreateTools?: string[];
    /**
     * Defines the default behavior once the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create) operation is completed.
     *
     * @default "continuous"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#creationMode)
     */
    creationMode?: "single" | "continuous" | "update";
    /**
     * Default create options set for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    defaultCreateOptions?: SketchDefaultCreateOptions;
    /**
     * Default update options set for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    defaultUpdateOptions?: SketchDefaultUpdateOptions;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#labelOptions)
     */
    labelOptions?: SketchLabelOptionsProperties;
    /**
     * The [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) associated with the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#layer)
     */
    layer?: GraphicsLayerProperties;
    /**
     * Determines the layout/orientation of the Sketch widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#layout)
     */
    layout?: "vertical" | "horizontal";
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#tooltipOptions)
     */
    tooltipOptions?: SketchTooltipOptionsProperties;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#valueOptions)
     */
    valueOptions?: SketchValueOptionsProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for the Sketch widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#viewModel)
     */
    viewModel?: SketchViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#visibleElements)
     */
    visibleElements?: SketchVisibleElements;
  }

  export interface SketchViewModel extends Accessor, Evented {}

  export class SketchViewModel {
    /**
     * Provides the logic for the [Sketch](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html)
     */

    constructor(properties?: SketchViewModelProperties);

    /**
     * The [SimpleFillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html) displayed when actively creating a new `polygon` graphic using the [`polygon`](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTool) tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeFillSymbol)
     */
    activeFillSymbol: SimpleFillSymbol;
    /**
     * When creating new graphics (for example after [create()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create) has been called), this property reflects the create tool being used.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTool)
     */
    readonly activeTool:
      | "point"
      | "multipoint"
      | "polyline"
      | "polygon"
      | "circle"
      | "mesh"
      | "rectangle"
      | "move"
      | "transform"
      | "reshape";
    /**
     * The tooltip currently being displayed for the [activeTool](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTool).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTooltip)
     */
    readonly activeTooltip: Tooltip;
    /**
     * The graphic that is being created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#createGraphic)
     */
    readonly createGraphic: Graphic;
    /**
     * Default create options set for the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    defaultCreateOptions: SketchViewModelDefaultCreateOptions;
    /**
     * Default update options set for the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    defaultUpdateOptions: SketchViewModelDefaultUpdateOptions;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#labelOptions)
     */
    labelOptions: SketchLabelOptions;
    /**
     * The [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) associated with the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#layer)
     */
    layer: GraphicsLayer;
    /**
     * A [SimpleMarkerSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html), [PointSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html), [TextSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html), [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html), or [WebStyleSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html) used for representing the point geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#pointSymbol)
     */
    pointSymbol: SimpleMarkerSymbol | PointSymbol3D | TextSymbol | CIMSymbol | WebStyleSymbol;
    /**
     * A [SimpleFillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html), [PolygonSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polygon geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#polygonSymbol)
     */
    polygonSymbol: SimpleFillSymbol | PolygonSymbol3D | CIMSymbol;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html), [LineSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polyline geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#polylineSymbol)
     */
    polylineSymbol: SimpleLineSymbol | LineSymbol3D | CIMSymbol;
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The sketch view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#state)
     */
    readonly state: "ready" | "disabled" | "active";
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#tooltipOptions)
     */
    tooltipOptions: SketchTooltipOptions;
    /**
     * An array of graphics that are being updated by the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#updateGraphics)
     */
    readonly updateGraphics: Collection<Graphic>;
    /**
     * Indicates if a graphic can be selected to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#updateOnGraphicClick)
     */
    updateOnGraphicClick: boolean;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#valueOptions)
     */
    valueOptions: SketchValueOptions;
    /**
     * The view in which geometries will be sketched by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Cancels the active operation and fires the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#event-create) or [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#event-update) event If called in the middle of a create operation, `cancel()` discards the partially created graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#cancel)
     */
    cancel(): void;
    /**
     * Indicates if it is possible to perform a [redo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#redo) action in the current update session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#canRedo)
     */
    canRedo(): boolean;
    /**
     * Indicates if it is possible to perform an [undo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#undo) action in the current update session.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#canUndo)
     */
    canUndo(): boolean;
    /**
     * Completes the active operation and fires the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#event-create) or [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#event-update) event and changes the event's state to `complete`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#complete)
     */
    complete(): void;
    /**
     * Create a graphic with the geometry specified in the `tool` parameter.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create)
     */
    create(
      tool: "point" | "multipoint" | "polyline" | "polygon" | "rectangle" | "circle" | "mesh",
      createOptions?: SketchViewModelCreateCreateOptions,
    ): void;
    /**
     * Deletes the selected graphics used in the update workflow.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#delete)
     */
    delete(): void;
    /**
     * Duplicates current graphics used in the update workflow and automatically adds them to the associated layer
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#duplicate)
     */
    duplicate(): void;
    /**
     * Allows creation of a graphic similar to create with the difference that the geometry can be provided directly for the graphic being created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#place)
     */
    place(geometry: Mesh): void;
    /**
     * Incrementally redo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#redo)
     */
    redo(): void;
    /**
     * Incrementally undo actions recorded in the stack.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#undo)
     */
    undo(): void;
    /**
     * Initializes an update operation for the specified graphic(s) and fires [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#event-update) event.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    update(graphics: Graphic | Graphic[], updateOptions?: SketchViewModelUpdateUpdateOptions): Promise<void>;

    on(name: "delete", eventHandler: SketchViewModelDeleteEventHandler): IHandle;

    on(name: "create", eventHandler: SketchViewModelCreateEventHandler): IHandle;

    on(name: "update", eventHandler: SketchViewModelUpdateEventHandler): IHandle;

    on(name: "redo", eventHandler: SketchViewModelRedoEventHandler): IHandle;

    on(name: "undo", eventHandler: SketchViewModelUndoEventHandler): IHandle;
  }

  interface SketchViewModelProperties {
    /**
     * The [SimpleFillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html) displayed when actively creating a new `polygon` graphic using the [`polygon`](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeTool) tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#activeFillSymbol)
     */
    activeFillSymbol?: SimpleFillSymbolProperties;
    /**
     * Default create options set for the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    defaultCreateOptions?: SketchViewModelDefaultCreateOptions;
    /**
     * Default update options set for the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    defaultUpdateOptions?: SketchViewModelDefaultUpdateOptions;
    /**
     * Options to configure the sketch labels shown next to each segment of the geometry being created or updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#labelOptions)
     */
    labelOptions?: SketchLabelOptionsProperties;
    /**
     * The [GraphicsLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-GraphicsLayer.html) associated with the SketchViewModel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#layer)
     */
    layer?: GraphicsLayerProperties;
    /**
     * A [SimpleMarkerSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleMarkerSymbol.html), [PointSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PointSymbol3D.html), [TextSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-TextSymbol.html), [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html), or [WebStyleSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-WebStyleSymbol.html) used for representing the point geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#pointSymbol)
     */
    pointSymbol?:
      | (SimpleMarkerSymbolProperties & { type: "simple-marker" })
      | (PointSymbol3DProperties & { type: "point-3d" })
      | (TextSymbolProperties & { type: "text" })
      | (CIMSymbolProperties & { type: "cim" })
      | (WebStyleSymbolProperties & { type: "web-style" });
    /**
     * A [SimpleFillSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleFillSymbol.html), [PolygonSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-PolygonSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polygon geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#polygonSymbol)
     */
    polygonSymbol?:
      | (SimpleFillSymbolProperties & { type: "simple-fill" })
      | (PolygonSymbol3DProperties & { type: "polygon-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html), [LineSymbol3D](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-LineSymbol3D.html), or [CIMSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-CIMSymbol.html) used for representing the polyline geometry that is being drawn.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#polylineSymbol)
     */
    polylineSymbol?:
      | (SimpleLineSymbolProperties & { type: "simple-line" })
      | (LineSymbol3DProperties & { type: "line-3d" })
      | (CIMSymbolProperties & { type: "cim" });
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * Options to configure the tooltip shown next to the cursor when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#tooltipOptions)
     */
    tooltipOptions?: SketchTooltipOptionsProperties;
    /**
     * Indicates if a graphic can be selected to be updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#updateOnGraphicClick)
     */
    updateOnGraphicClick?: boolean;
    /**
     * Options to configure how values are displayed and input when creating or updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#valueOptions)
     */
    valueOptions?: SketchValueOptionsProperties;
    /**
     * The view in which geometries will be sketched by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export interface SketchViewModelCreateEvent {
    graphic: Graphic;

    state: "start" | "active" | "complete" | "cancel";

    tool: "point" | "multipoint" | "polyline" | "polygon" | "rectangle" | "circle" | "mesh";

    toolEventInfo: CreateToolEventInfo;

    type: "create";
  }

  export interface SketchViewModelDeleteEvent {
    graphics: Graphic[];

    tool: "move" | "reshape" | "transform";

    type: "delete";
  }

  export interface SketchViewModelRedoEvent {
    graphics: Graphic[];

    tool:
      | "point"
      | "multipoint"
      | "polyline"
      | "polygon"
      | "rectangle"
      | "circle"
      | "mesh"
      | "move"
      | "transform"
      | "reshape";

    type: "redo";
  }

  export interface SketchViewModelCreateCreateOptions {
    /**
     * The default z-value of the newly created geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create)
     */
    defaultZ?: number;
    /**
     * Controls whether the created geometry has z-values or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create)
     */
    hasZ?: boolean;
    /**
     * Specifies how the graphic can be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create)
     */
    mode?: "hybrid" | "freehand" | "click";
    /**
     * Controls whether or not the width and height of the drawn geometry are kept equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#create)
     */
    preserveAspectRatio?: boolean;
  }

  export interface SketchViewModelDefaultCreateOptions {
    /**
     * The default z-value of the newly created geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    defaultZ?: number;
    /**
     * Controls whether the created geometry has z-values or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    hasZ?: boolean;
    /**
     * Create operation mode how the graphic can be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    mode?: "hybrid" | "freehand" | "click";
    /**
     * Controls whether or not the width and height of the drawn geometry are kept equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultCreateOptions)
     */
    preserveAspectRatio?: boolean;
  }

  export interface SketchViewModelDefaultUpdateOptions {
    /**
     * Name of the update tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    tool?: "move" | "transform" | "reshape";
    /**
     * Indicates if the `rotation` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    enableRotation?: boolean;
    /**
     * Indicates if the `scale` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    enableScaling?: boolean;
    /**
     * Indicates if z-values can be modified when updating the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    enableZ?: boolean;
    /**
     * Indicates whether more than one selection can be made at once.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates if the uniform scale operation will be enabled when updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    preserveAspectRatio?: boolean;
    /**
     * Indicates if the graphic being updated can be toggled between `transform` and `reshape` update options.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    toggleToolOnClick?: boolean;
    /**
     * Changes the behavior for the `reshape` tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    reshapeOptions?: SketchViewModelDefaultUpdateOptionsReshapeOptions;
    /**
     * Options that control when to display or hide highlights for update operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    highlightOptions?: SketchViewModelDefaultUpdateOptionsHighlightOptions;
  }

  export interface SketchViewModelDefaultUpdateOptionsHighlightOptions {
    /**
     * Indicates if highlighting is enabled for update operations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    enabled?: boolean;
  }

  export interface SketchViewModelDefaultUpdateOptionsReshapeOptions {
    /**
     * Sets the reshape operation on the edge.
     *
     * @default "split"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    edgeOperation?: "none" | "split" | "offset";
    /**
     * Sets the move constraints for the whole shape.
     *
     * @default "move"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    shapeOperation?: "none" | "move" | "move-xy";
    /**
     * Sets the move constraints for the vertex.
     *
     * @default "move"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#defaultUpdateOptions)
     */
    vertexOperation?: "move" | "move-xy";
  }

  export interface SketchViewModelUpdateUpdateOptions {
    /**
     * Name of the update tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    tool?: "transform" | "reshape" | "move";
    /**
     * Indicates if the `rotation` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    enableRotation?: boolean;
    /**
     * Indicates if the `scale` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    enableScaling?: boolean;
    /**
     * Indicates if z-values can be modified when updating the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    enableZ?: boolean;
    /**
     * Indicates whether more than one selection can be made at once.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates if the uniform scale operation will be enabled when updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    preserveAspectRatio?: boolean;
    /**
     * Indicates if the graphic being updated can be toggled between `transform` and `reshape` update options.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch-SketchViewModel.html#update)
     */
    toggleToolOnClick?: boolean;
  }

  export interface SketchViewModelUndoEvent {
    graphics: Graphic[];

    tool:
      | "point"
      | "multipoint"
      | "polyline"
      | "polygon"
      | "rectangle"
      | "circle"
      | "mesh"
      | "move"
      | "transform"
      | "reshape";

    type: "undo";
  }

  export interface SketchViewModelUpdateEvent {
    aborted: boolean;

    graphics: Graphic[];

    state: "start" | "active" | "complete";

    tool: "move" | "transform" | "reshape";

    toolEventInfo: UpdateToolEventInfo;

    type: "update";
  }

  export interface SketchCreateEvent {
    graphic: Graphic;

    state: "start" | "active" | "complete" | "cancel";

    tool: "point" | "polyline" | "polygon" | "rectangle" | "circle";

    toolEventInfo: CreateToolEventInfo;

    type: "create";
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-create) event when the graphic is being created.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#CreateToolEventInfo)
   */
  export type CreateToolEventInfo = VertexAddEventInfo | CursorUpdateEventInfo;

  /**
   * This information is returned as `toolEventInfo` parameter for the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-create) event when the user moves the cursor on the view while the graphic is being created.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#CursorUpdateEventInfo)
   */
  export interface CursorUpdateEventInfo {
    type: "cursor-update";
    /**
     * An array of numbers representing the coordinates of the cursor location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#CursorUpdateEventInfo)
     */
    coordinates: number[];
  }

  export interface SketchDeleteEvent {
    graphics: Graphic[];

    tool: "move" | "reshape" | "transform";

    type: "delete";
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event while the user is moving the graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#MoveEventInfo)
   */
  export interface MoveEventInfo {
    /**
     * Returns information indicating the stage of the move operation.
     *
     * [Read more...](global.html#type)
     */
    type: "move-start" | "move" | "move-stop";
    /**
     * Number of pixels moved on the x-axis from the last known position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#MoveEventInfo)
     */
    dx: number;
    /**
     * Number of pixels moved on the y-axis from the last known position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#MoveEventInfo)
     */
    dy: number;
    /**
     * The instance of the graphic that is being moved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#MoveEventInfo)
     */
    mover: Graphic;
  }

  export interface SketchRedoEvent {
    graphics: Graphic[];

    tool: "point" | "polyline" | "polygon" | "rectangle" | "circle" | "move" | "transform" | "reshape";

    type: "redo";
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event while the user is reshaping the graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#ReshapeEventInfo)
   */
  export interface ReshapeEventInfo {
    /**
     * Returns information indicating the stage of the reshape operation.
     *
     * [Read more...](global.html#type)
     */
    type: "reshape-start" | "reshape" | "reshape-stop";
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event while the user is rotating the graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#RotateEventInfo)
   */
  export interface RotateEventInfo {
    /**
     * Returns information indicating the stage of the rotate operation.
     *
     * [Read more...](global.html#type)
     */
    type: "rotate-start" | "rotate" | "rotate-stop";
    /**
     * Angle of rotation in degrees.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#RotateEventInfo)
     */
    angle: number;
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event while the user is scaling or resizing the graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#ScaleEventInfo)
   */
  export interface ScaleEventInfo {
    /**
     * Returns information indicating the stage of the scale operation.
     *
     * [Read more...](global.html#type)
     */
    type: "scale-start" | "scale" | "scale-stop";
    /**
     * The x scale factor used to enlarge or shrink the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#ScaleEventInfo)
     */
    xScale: number;
    /**
     * The y scale factor used to enlarge or shrink the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#ScaleEventInfo)
     */
    yScale: number;
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event while the user is selecting or deselecting graphics using `Shift+Left-click`.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#SelectionChangeEventInfo)
   */
  export interface SelectionChangeEventInfo {
    type: "selection-change";
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing the latest graphic selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#SelectionChangeEventInfo)
     */
    added: Graphic[];
    /**
     * An array of [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-Graphic.html) representing the latest graphic deselected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#SelectionChangeEventInfo)
     */
    removed: Graphic[];
  }

  export interface SketchCreateCreateOptions {
    /**
     * The default z-value of the newly created geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create)
     */
    defaultZ?: number;
    /**
     * Controls whether the created geometry has z-values or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create)
     */
    hasZ?: boolean;
    /**
     * Specifies how the graphic can be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create)
     */
    mode?: "hybrid" | "freehand" | "click";
    /**
     * Controls whether or not the width and height of the drawn geometry are kept equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#create)
     */
    preserveAspectRatio?: boolean;
  }

  export interface SketchDefaultCreateOptions {
    /**
     * The default z-value of the newly created geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    defaultZ?: number;
    /**
     * Controls whether the created geometry has z-values or not.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    hasZ?: boolean;
    /**
     * Create operation mode how the graphic can be created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    mode?: "hybrid" | "freehand" | "click";
    /**
     * Controls whether or not the width and height of the drawn geometry are kept equal.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultCreateOptions)
     */
    preserveAspectRatio?: boolean;
  }

  export interface SketchDefaultUpdateOptions {
    /**
     * Name of the update tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    tool?: "move" | "transform" | "reshape";
    /**
     * Indicates if the `rotation` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    enableRotation?: boolean;
    /**
     * Indicates if the `scale` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    enableScaling?: boolean;
    /**
     * Indicates if z-values can be modified when updating the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    enableZ?: boolean;
    /**
     * Indicates whether more than one selection can be made at once.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates if the uniform scale operation will be enabled when updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    preserveAspectRatio?: boolean;
    /**
     * Indicates if the graphic being updated can be toggled between `transform` and `reshape` update options.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    toggleToolOnClick?: boolean;
    /**
     * Changes the behavior for the `reshape` tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    reshapeOptions?: SketchDefaultUpdateOptionsReshapeOptions;
    /**
     * Options that control when to display or hide highlights for update operations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    highlightOptions?: SketchDefaultUpdateOptionsHighlightOptions;
  }

  export interface SketchDefaultUpdateOptionsHighlightOptions {
    /**
     * Indicates if highlighting is enabled for update operations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    enabled?: boolean;
  }

  export interface SketchDefaultUpdateOptionsReshapeOptions {
    /**
     * Sets the reshape operation on the edge.
     *
     * @default "split"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    edgeOperation?: "none" | "split" | "offset";
    /**
     * Sets the move constraints for the whole shape.
     *
     * @default "move"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    shapeOperation?: "none" | "move" | "move-xy";
    /**
     * Sets the move constraints for the vertex.
     *
     * @default "move"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#defaultUpdateOptions)
     */
    vertexOperation?: "move" | "move-xy";
  }

  export interface SketchUpdateUpdateOptions {
    /**
     * Name of the update tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    tool?: "transform" | "reshape" | "move";
    /**
     * Indicates if the `rotation` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    enableRotation?: boolean;
    /**
     * Indicates if the `scale` operation will be enabled when updating graphics.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    enableScaling?: boolean;
    /**
     * Indicates if z-values can be modified when updating the graphic.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    enableZ?: boolean;
    /**
     * Indicates whether more than one selection can be made at once.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * Indicates if the uniform scale operation will be enabled when updating graphics.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    preserveAspectRatio?: boolean;
    /**
     * Indicates if the graphic being updated can be toggled between `transform` and `reshape` update options.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#update)
     */
    toggleToolOnClick?: boolean;
  }

  export interface SketchUndoEvent {
    graphics: Graphic[];

    tool: "point" | "polyline" | "polygon" | "rectangle" | "circle" | "move" | "transform" | "reshape";

    type: "undo";
  }

  export interface SketchUpdateEvent {
    aborted: boolean;

    graphics: Graphic[];

    state: "start" | "active" | "complete";

    tool: "move" | "transform" | "reshape";

    toolEventInfo: UpdateToolEventInfo;

    type: "update";
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event when the user is updating graphics.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#UpdateToolEventInfo)
   */
  export type UpdateToolEventInfo =
    | MoveEventInfo
    | ReshapeEventInfo
    | RotateEventInfo
    | ScaleEventInfo
    | SelectionChangeEventInfo
    | VertexAddEventInfo
    | VertexRemoveEventInfo;

  /**
   * This information is returned as `toolEventInfo` parameter for the [create](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-create) or [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event when the user adds vertices to the graphic being created or updated.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
   */
  export interface VertexAddEventInfo {
    type: "vertex-add";
    /**
     * An array of x,y coordinates representing the vertices added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
     */
    added: number[];
    /**
     * Contains the details of the added vertices to track changes in topology of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
     */
    vertices: VertexAddEventInfoVertices[];
  }

  /**
   * This information is returned as `toolEventInfo` parameter for the [update](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#event-update) event when the user is removing vertices from the graphic.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
   */
  export interface VertexRemoveEventInfo {
    type: "vertex-remove";
    /**
     * An array of x,y coordinates representing the vertices removed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
     */
    removed: number[];
    /**
     * Contains the details of the removed vertices to track changes in topology of the geometry.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
     */
    vertices: VertexRemoveEventInfoVertices[];
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
   */
  export interface SketchVisibleElements {
    /**
     * The available sketch tools within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    createTools?: VisibleElementsCreateTools;
    /**
     * Indicates whether to display the 'duplicate' button while a graphic is selected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    duplicateButton?: boolean;
    /**
     * The available selection tools within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    selectionTools?: VisibleElementsSelectionTools;
    /**
     * Indicates whether to display the settings menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    settingsMenu?: boolean;
    /**
     * _Since 4.29_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    labelsToggle?: boolean;
    /**
     * _Since 4.29_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    tooltipsToggle?: boolean;
    /**
     * Indicates whether to display the `SnappingControls` widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    snappingControls?: boolean;
    /**
     * The available [SnappingControls](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html) elements within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    snappingControlsElements?: SketchVisibleElementsSnappingControlsElements;
    /**
     * Indicates whether to display the undo/redo menu within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    undoRedoMenu?: boolean;
  }

  export interface VertexAddEventInfoVertices {
    /**
     * An array of x,y coordinates representing the vertices added.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
     */
    coordinates: number[];
    /**
     * The ring/path index of the added vertex.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
     */
    componentIndex: number;
    /**
     * The index of the vertex position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexAddEventInfo)
     */
    vertexIndex: number;
  }

  export interface VertexRemoveEventInfoVertices {
    /**
     * An array of x,y coordinates representing the vertices removed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
     */
    coordinates: number[];
    /**
     * The ring/path index of the removed vertex.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
     */
    componentIndex: number;
    /**
     * The index of the vertex position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VertexRemoveEventInfo)
     */
    vertexIndex: number;
  }

  export interface VisibleElementsCreateTools {
    /**
     * Indicates whether to display the point sketch tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    point?: boolean;
    /**
     * Indicates whether to display the polyline sketch tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    polyline?: boolean;
    /**
     * Indicates whether to display the polygon sketch tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    polygon?: boolean;
    /**
     * Indicates whether to display the rectangle sketch tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    rectangle?: boolean;
    /**
     * Indicates whether to display the circle sketch tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    circle?: boolean;
  }

  export interface VisibleElementsSelectionTools {
    /**
     * Indicates whether to display the `"rectangle-selection"` tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    "rectangle-selection"?: boolean;
    /**
     * Indicates whether to display the `"lasso-selection"` tool.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    "lasso-selection"?: boolean;
  }

  export interface SketchVisibleElementsSnappingControlsElements {
    /**
     * Indicates whether to display the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    header?: boolean;
    /**
     * Indicates whether to display the `enabledToggle` (Enable snapping).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    enabledToggle?: boolean;
    /**
     * Indicates whether to display the `selfEnabledToggle` (Geometry guides).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    selfEnabledToggle?: boolean;
    /**
     * Indicates whether to display the `featureEnabledToggle` (Feature to feature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    featureEnabledToggle?: boolean;
    /**
     * Indicates whether to display the [FeatureSnappingLayerSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html) layerList.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    layerList?: boolean;
    /**
     * Indicates whether to display the “Enable all” or “Disable all” button to enable / disable snapping for all the layers in the list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    layerListToggleLayersButton?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    gridControls?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    gridControlsElements?: SketchVisibleElementsSnappingControlsElementsGridControlsElements;
  }

  export interface SketchVisibleElementsSnappingControlsElementsGridControlsElements {
    /**
     * Indicates whether to display the grid snapping toggle embedded in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    gridSnapEnabledToggle?: boolean;
    /**
     * Indicates whether to display the grid display toggle embedded in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * Indicates whether to display the color/theme selection component of grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    colorSelection?: boolean;
    /**
     * Indicates whether to display the dynamic scaling toggle in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    dynamicScaleToggle?: boolean;
    /**
     * Indicates whether to display the 'rotate with map' toggle in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    rotateWithMapToggle?: boolean;
    /**
     * Indicates whether to display the spacing and rotation inputs in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    numericInputs?: boolean;
    /**
     * Indicates whether to display the major line interval input in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    lineIntervalInput?: boolean;
    /**
     * Indicates whether to display the interactive placement buttons in grid controls.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    placementButtons?: boolean;
    /**
     * Indicates whether to display a notice when the grid is not visible because it is out of scale and dynamic scaling is turned off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Sketch.html#VisibleElements)
     */
    outOfScaleWarning?: boolean;
  }

  export class Slice extends Widget {
    /**
     * The Slice widget is a 3D analysis tool that can be used to reveal occluded content in a {@link module:esri/views/SceneView}.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html)
     */

    constructor(properties?: SliceProperties);

    /**
     * The slice analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#analysis)
     */
    readonly analysis: SliceAnalysis;
    /**
     * Indicates the heading level to use for the "Excluded layers" heading.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#viewModel)
     */
    viewModel: SliceViewModel;
  }

  interface SliceProperties extends WidgetProperties {
    /**
     * The slice analysis object being created or modified by the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#analysis)
     */
    analysis?: SliceAnalysisProperties;
    /**
     * Indicates the heading level to use for the "Excluded layers" heading.
     *
     * @default 3
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A reference to the [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html#viewModel)
     */
    viewModel?: SliceViewModelProperties;
  }

  export class SliceViewModel extends Accessor {
    /**
     * Provides the logic for the [Slice](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html)
     */

    constructor(properties?: SliceViewModelProperties);

    /**
     * The slice analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#analysis)
     */
    readonly analysis: SliceAnalysis;
    /**
     * Add layers to this collection to exclude them from the slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#excludedLayers)
     */
    excludedLayers: Collection<Layer | BuildingComponentSublayer>;
    /**
     * Indicates whether the [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html) and layers that are draped on the ground surface are excluded from the slice.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#excludeGroundSurface)
     */
    excludeGroundSurface: boolean;
    /**
     * The shape used to slice elements in a 3D scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#shape)
     */
    shape: SlicePlane;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "slicing" | "sliced";
    /**
     * Enable tilting the slice shape.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#tiltEnabled)
     */
    tiltEnabled: boolean;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#view)
     */
    view: SceneView;

    /**
     * Clears the [shape](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#shape) of the slice, effectively removing it from the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#clear)
     */
    clear(): void;
    /**
     * Starts the interactive creation of a new slice, clearing the previous [shape](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#shape).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#start)
     */
    start(): void;
  }

  interface SliceViewModelProperties {
    /**
     * The slice analysis object being created or modified by the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#analysis)
     */
    analysis?: SliceAnalysisProperties;
    /**
     * Add layers to this collection to exclude them from the slice.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#excludedLayers)
     */
    excludedLayers?: CollectionProperties<
      LayerProperties | (BuildingComponentSublayerProperties & { type: "building-component" })
    >;
    /**
     * Indicates whether the [Ground](https://developers.arcgis.com/javascript/latest/api-reference/esri-Ground.html) and layers that are draped on the ground surface are excluded from the slice.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#excludeGroundSurface)
     */
    excludeGroundSurface?: boolean;
    /**
     * The shape used to slice elements in a 3D scene.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#shape)
     */
    shape?: SlicePlaneProperties;
    /**
     * Enable tilting the slice shape.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#tiltEnabled)
     */
    tiltEnabled?: boolean;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slice-SliceViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  export class Slider extends Widget {
    /**
     * A slider widget that can be used for filtering data, or gathering numeric input from a user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html)
     */

    constructor(properties?: SliderProperties);

    /**
     * When `true`, sets the slider to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates if the user can drag the segment between thumbs to update thumb positions.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#draggableSegmentsEnabled)
     */
    draggableSegmentsEnabled: boolean;
    /**
     * When set, the user is restricted from moving slider thumbs to positions higher than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMax)
     */
    effectiveMax: number;
    /**
     * When set, the user is restricted from moving slider thumbs to positions less than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMin)
     */
    effectiveMin: number;
    /**
     * The HTML Element nodes representing the slider segment between the [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min) and [effectiveMin](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMin), and the segment between the [effectiveMax](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMax) and [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveSegmentElements)
     */
    readonly effectiveSegmentElements: Collection<HTMLElement>;
    /**
     * A function that provides the developer with access to the input elements when [rangeLabelInputsEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#rangeLabelInputsEnabled) and/or [labelInputsEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelInputsEnabled) are set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputCreatedFunction)
     */
    inputCreatedFunction: InputCreatedFunction;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputFormatFunction)
     */
    inputFormatFunction: SliderLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputParseFunction)
     */
    inputParseFunction: InputParser;
    /**
     * The HTML Element nodes representing [labels](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labels) attached to slider thumbs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelElements)
     */
    readonly labelElements: Collection<HTMLElement>;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelFormatFunction)
     */
    labelFormatFunction: SliderLabelFormatter;
    /**
     * Indicates whether to enable editing input values via keyboard input when the user clicks a label.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelInputsEnabled)
     */
    labelInputsEnabled: boolean;
    /**
     * An array of strings associated with 'values' generated using an internal label formatter or the values returned from [labelFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labels)
     */
    readonly labels: string[];
    /**
     * Determines the layout/orientation of the Slider widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#layout)
     */
    layout: "horizontal" | "horizontal-reversed" | "vertical" | "vertical-reversed";
    /**
     * The maximum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max)
     */
    max: number;
    /**
     * The HTML Element node representing the [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max) value label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#maxLabelElement)
     */
    readonly maxLabelElement: HTMLElement;
    /**
     * The minimum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min)
     */
    min: number;
    /**
     * The HTML Element node representing the [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min) value label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#minLabelElement)
     */
    readonly minLabelElement: HTMLElement;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#precision)
     */
    precision: number;
    /**
     * Indicates whether to enable editing range values via keyboard input when the user clicks a [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min) or [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max) label.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#rangeLabelInputsEnabled)
     */
    rangeLabelInputsEnabled: boolean;
    /**
     * The HTML Element nodes representing interactive slider segments.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#segmentElements)
     */
    readonly segmentElements: Collection<HTMLElement>;
    /**
     * Indicates if the closest thumb will snap to the clicked location on the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#snapOnClickEnabled)
     */
    snapOnClickEnabled: boolean;
    /**
     * The current state of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#state)
     */
    readonly state: "ready" | "disabled" | "editing" | "dragging";
    /**
     * Sets steps, or intervals, on the slider that restrict user input to specific values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#steps)
     */
    steps: number | number[];
    /**
     * When `true`, all segments will sync together in updating thumb values when the user drags any segment.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#syncedSegmentsEnabled)
     */
    syncedSegmentsEnabled: boolean;
    /**
     * Function that executes each time a thumb is created on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#thumbCreatedFunction)
     */
    thumbCreatedFunction: ThumbCreatedFunction;
    /**
     * The HTML Element nodes representing slider thumbs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#thumbElements)
     */
    readonly thumbElements: Collection<HTMLElement>;
    /**
     * When `false`, the user can freely move any slider thumb to any position along the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#thumbsConstrained)
     */
    thumbsConstrained: boolean;
    /**
     * When set, renders ticks along the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#tickConfigs)
     */
    tickConfigs: TickConfig[];
    /**
     * The HTML Element nodes representing slider ticks and their associated labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#tickElements)
     */
    readonly tickElements: Collection<Collection<TickElementGroup>>;
    /**
     * The HTML Element node representing the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#trackElement)
     */
    trackElement: HTMLElement;
    /**
     * An array of numbers representing absolute thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#values)
     */
    values: number[];
    /**
     * The view model for the Slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#viewModel)
     */
    viewModel: SliderViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#visibleElements)
     */
    visibleElements: SliderVisibleElements;

    on(name: "max-change", eventHandler: SliderMaxChangeEventHandler): IHandle;

    on(name: "min-change", eventHandler: SliderMinChangeEventHandler): IHandle;

    on(name: "segment-click", eventHandler: SliderSegmentClickEventHandler): IHandle;

    on(name: "segment-drag", eventHandler: SliderSegmentDragEventHandler): IHandle;

    on(name: "thumb-change", eventHandler: SliderThumbChangeEventHandler): IHandle;

    on(name: "thumb-click", eventHandler: SliderThumbClickEventHandler): IHandle;

    on(name: "thumb-drag", eventHandler: SliderThumbDragEventHandler): IHandle;

    on(name: "track-click", eventHandler: SliderTrackClickEventHandler): IHandle;

    on(name: "max-click", eventHandler: SliderMaxClickEventHandler): IHandle;

    on(name: "min-click", eventHandler: SliderMinClickEventHandler): IHandle;

    on(name: "tick-click", eventHandler: SliderTickClickEventHandler): IHandle;
  }

  interface SliderProperties extends WidgetProperties {
    /**
     * When `true`, sets the slider to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates if the user can drag the segment between thumbs to update thumb positions.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#draggableSegmentsEnabled)
     */
    draggableSegmentsEnabled?: boolean;
    /**
     * When set, the user is restricted from moving slider thumbs to positions higher than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMax)
     */
    effectiveMax?: number;
    /**
     * When set, the user is restricted from moving slider thumbs to positions less than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#effectiveMin)
     */
    effectiveMin?: number;
    /**
     * A function that provides the developer with access to the input elements when [rangeLabelInputsEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#rangeLabelInputsEnabled) and/or [labelInputsEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelInputsEnabled) are set to `true`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputCreatedFunction)
     */
    inputCreatedFunction?: InputCreatedFunction;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputFormatFunction)
     */
    inputFormatFunction?: SliderLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#inputParseFunction)
     */
    inputParseFunction?: InputParser;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelFormatFunction)
     */
    labelFormatFunction?: SliderLabelFormatter;
    /**
     * Indicates whether to enable editing input values via keyboard input when the user clicks a label.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelInputsEnabled)
     */
    labelInputsEnabled?: boolean;
    /**
     * Determines the layout/orientation of the Slider widget.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#layout)
     */
    layout?: "horizontal" | "horizontal-reversed" | "vertical" | "vertical-reversed";
    /**
     * The maximum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max)
     */
    max?: number;
    /**
     * The minimum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min)
     */
    min?: number;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#precision)
     */
    precision?: number;
    /**
     * Indicates whether to enable editing range values via keyboard input when the user clicks a [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min) or [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max) label.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#rangeLabelInputsEnabled)
     */
    rangeLabelInputsEnabled?: boolean;
    /**
     * Indicates if the closest thumb will snap to the clicked location on the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#snapOnClickEnabled)
     */
    snapOnClickEnabled?: boolean;
    /**
     * Sets steps, or intervals, on the slider that restrict user input to specific values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#steps)
     */
    steps?: number | number[];
    /**
     * When `true`, all segments will sync together in updating thumb values when the user drags any segment.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#syncedSegmentsEnabled)
     */
    syncedSegmentsEnabled?: boolean;
    /**
     * Function that executes each time a thumb is created on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#thumbCreatedFunction)
     */
    thumbCreatedFunction?: ThumbCreatedFunction;
    /**
     * When `false`, the user can freely move any slider thumb to any position along the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#thumbsConstrained)
     */
    thumbsConstrained?: boolean;
    /**
     * When set, renders ticks along the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#tickConfigs)
     */
    tickConfigs?: TickConfig[];
    /**
     * The HTML Element node representing the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#trackElement)
     */
    trackElement?: HTMLElement;
    /**
     * An array of numbers representing absolute thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#values)
     */
    values?: number[];
    /**
     * The view model for the Slider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#viewModel)
     */
    viewModel?: SliderViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#visibleElements)
     */
    visibleElements?: SliderVisibleElements;
  }

  export interface SliderViewModel extends Accessor, Evented {}

  export class SliderViewModel {
    /**
     * Provides the logic for the [Slider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html)
     */

    constructor(properties?: SliderViewModelProperties);

    /**
     * When set, the user is restricted from moving slider thumbs to positions higher than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#effectiveMax)
     */
    effectiveMax: number;
    /**
     * When set, the user is restricted from moving slider thumbs to positions less than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#effectiveMin)
     */
    effectiveMin: number;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputFormatFunction)
     */
    inputFormatFunction: SliderLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputParseFunction)
     */
    inputParseFunction: InputParser;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#labelFormatFunction)
     */
    labelFormatFunction: SliderLabelFormatter;
    /**
     * An array of strings associated with [values](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#values) generated using an internal label formatter or the values returned from [labelFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#labelFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#labels)
     */
    readonly labels: string[];
    /**
     * The maximum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#max)
     */
    max: number;
    /**
     * The minimum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#min)
     */
    min: number;
    /**
     * Defines how slider values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#precision)
     */
    precision: number;
    /**
     * The state of the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * When `false`, the user can freely move any slider thumb to any position along the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#thumbsConstrained)
     */
    thumbsConstrained: boolean;
    /**
     * An array of numbers representing absolute thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#values)
     */
    values: number[];

    /**
     * The default input format function available for use as a fallback in custom formatting implementations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#defaultInputFormatFunction)
     */
    defaultInputFormatFunction(value: number): string;
    /**
     * The default input parsing function available for use as a fallback in custom parsing implementations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#defaultInputParseFunction)
     */
    defaultInputParseFunction(value: string): number;
    /**
     * The default label format function, available for use as a fallback in custom formatting implementations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#defaultLabelFormatFunction)
     */
    defaultLabelFormatFunction(value: number): string;
    /**
     * Returns the effective bounds of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#getBounds)
     */
    getBounds(): Bounds;
    /**
     * Returns the min and max bounds for a 'value' at the provided index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#getBoundsForValueAtIndex)
     */
    getBoundsForValueAtIndex(index: number): any;
    /**
     * Returns the formatted label for a provided value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#getLabelForValue)
     */
    getLabelForValue(value: number, type?: "min" | "max" | "tick" | "values", index?: number): string;
    /**
     * Updates a thumb [value](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#values) based on the provided index.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#setValue)
     */
    setValue(index: number, value: number): void;
    /**
     * Rounds the given value to the number of decimal places specified in the [precision](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#precision) property set on the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#toPrecision)
     */
    toPrecision(value: number): string;
  }

  interface SliderViewModelProperties {
    /**
     * When set, the user is restricted from moving slider thumbs to positions higher than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#effectiveMax)
     */
    effectiveMax?: number;
    /**
     * When set, the user is restricted from moving slider thumbs to positions less than this value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#effectiveMin)
     */
    effectiveMin?: number;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputFormatFunction)
     */
    inputFormatFunction?: SliderLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#inputParseFunction)
     */
    inputParseFunction?: InputParser;
    /**
     * A function used to format labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#labelFormatFunction)
     */
    labelFormatFunction?: SliderLabelFormatter;
    /**
     * The maximum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#max)
     */
    max?: number;
    /**
     * The minimum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#min)
     */
    min?: number;
    /**
     * Defines how slider values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#precision)
     */
    precision?: number;
    /**
     * When `false`, the user can freely move any slider thumb to any position along the track.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#thumbsConstrained)
     */
    thumbsConstrained?: boolean;
    /**
     * An array of numbers representing absolute thumb positions on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider-SliderViewModel.html#values)
     */
    values?: number[];
  }

  /**
   * Represents the effective bounds of the slider.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#Bounds)
   */
  export interface Bounds {
    /**
     * The effective min value of the slider, or the minimum value a user can set a thumb on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#Bounds)
     */
    min: number;
    /**
     * The effective max value of the slider, or the maximum value a user can set a thumb on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#Bounds)
     */
    max: number;
  }

  export type InputCreatedFunction = (inputElement: any, type: "max" | "min" | "thumb", thumbIndex?: number) => void;

  export type InputParser = (
    value: string,
    type?: "average" | "min" | "max" | "tick" | "value",
    index?: number,
  ) => number;

  export type SliderLabelFormatter = (
    value: number,
    type?: "average" | "min" | "max" | "tick" | "value",
    index?: number,
  ) => string;

  export interface SliderMaxChangeEvent {
    oldValue: number;

    type: "max-change";

    value: number;
  }

  export interface SliderMaxClickEvent {
    type: "max-click";

    value: number;
  }

  export interface SliderMinChangeEvent {
    oldValue: number;

    type: "min-change";

    value: number;
  }

  export interface SliderMinClickEvent {
    type: "min-click";

    value: number;
  }

  export interface SliderSegmentClickEvent {
    index: number;

    thumbIndices: number[];

    type: "segment-click";

    value: number;
  }

  export interface SliderSegmentDragEvent {
    index: number;

    state: "start" | "drag";

    thumbIndices: number[];

    type: "segment-drag";
  }

  export interface SliderThumbChangeEvent {
    index: number;

    oldValue: number;

    type: "thumb-change";

    value: number;
  }

  export interface SliderThumbClickEvent {
    index: number;

    type: "thumb-click";

    value: number;
  }

  export type ThumbCreatedFunction = (
    index: number,
    value: number,
    thumbElement: HTMLElement,
    labelElement?: HTMLElement,
  ) => void;

  export interface SliderThumbDragEvent {
    index: number;

    state: "drag" | "start" | "stop";

    type: "thumb-drag";

    value: number;
  }

  export interface SliderTickClickEvent {
    configIndex: number;

    groupIndex: number;

    type: "tick-click";

    value: number;
  }

  /**
   * Object specification for configuring ticks on the slider.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
   */
  export interface TickConfig {
    /**
     * The mode or method of positioning ticks along the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
     */
    mode: "count" | "percent" | "position";
    /**
     * Indicates where ticks will be rendered below the track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
     */
    values: number | number[];
    /**
     * Indicates whether to render labels for the ticks.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
     */
    labelsVisible?: boolean;
    /**
     * Callback that fires for each tick.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
     */
    tickCreatedFunction?: TickCreatedFunction;
    /**
     * Callback for formatting tick labels.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickConfig)
     */
    labelFormatFunction?: SliderLabelFormatter;
  }

  export type TickCreatedFunction = (value: number, tickElement: HTMLElement, labelElement?: HTMLElement) => void;

  /**
   * The HTML Element nodes representing a single slider tick and its associated label.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickElementGroup)
   */
  export interface TickElementGroup {
    /**
     * The HTMLElement representing a tick.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickElementGroup)
     */
    tickElement?: HTMLElement;
    /**
     * The HTMLElement representing the label associated with the tick element.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#TickElementGroup)
     */
    labelElement?: HTMLElement;
  }

  export interface SliderTrackClickEvent {
    type: "track-click";

    value: number;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#VisibleElements)
   */
  export interface SliderVisibleElements {
    /**
     * Indicates whether to display labels for slider thumbs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#VisibleElements)
     */
    labels?: boolean;
    /**
     * Indicates whether to display [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#min) or [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#max) range values on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#VisibleElements)
     */
    rangeLabels?: boolean;
  }

  export class BinaryColorSizeSlider extends SmartMappingSliderBase {
    /**
     * The BinaryColorSizeSlider widget is intended for authoring and exploring diverging data-driven visualizations in any layer that can be rendered with an above and below theme for a [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html)
     */

    constructor(properties?: BinaryColorSizeSliderProperties);

    /**
     * This property indicates whether the position of the outside handles are synced with the middle, or primary, handle.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary: boolean;
    /**
     * When `true`, ensures symbol sizes in the `above` range are consistent with symbol sizes in the `below` range for all slider thumb positions.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#stops)
     */
    stops: SizeStop[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#style)
     */
    style: BinaryColorSizeSliderStyle;
    /**
     * The view model for the BinaryColorSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#viewModel)
     */
    viewModel: BinaryColorSizeSliderViewModel;

    /**
     * A convenience function used to update the properties of a BinaryColorSizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#updateFromRendererResult)
     */
    updateFromRendererResult(
      rendererResult: univariateColorSizeContinuousRendererResult,
      histogramResult?: HistogramResult,
    ): void;
    /**
     * A convenience function used to update the input renderer based on the values of the slider [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#stops).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#updateRenderer)
     */
    updateRenderer(renderer: ClassBreaksRenderer): ClassBreaksRenderer;

    /**
     * A convenience function used to create a BinaryColorSizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#fromRendererResult)
     */
    static fromRendererResult(
      rendererResult: univariateColorSizeContinuousRendererResult,
      histogramResult?: HistogramResult,
    ): BinaryColorSizeSlider;
  }

  interface BinaryColorSizeSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * This property indicates whether the position of the outside handles are synced with the middle, or primary, handle.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary?: boolean;
    /**
     * When `true`, ensures symbol sizes in the `above` range are consistent with symbol sizes in the `below` range for all slider thumb positions.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled?: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#stops)
     */
    stops?: SizeStopProperties[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#style)
     */
    style?: BinaryColorSizeSliderStyle;
    /**
     * The view model for the BinaryColorSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#viewModel)
     */
    viewModel?: BinaryColorSizeSliderViewModelProperties;
  }

  export class BinaryColorSizeSliderViewModel extends SizeSliderViewModel {
    /**
     * Provides the logic for the [BinaryColorSizeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider-BinaryColorSizeSliderViewModel.html)
     */

    constructor(properties?: BinaryColorSizeSliderViewModelProperties);
  }

  interface BinaryColorSizeSliderViewModelProperties extends SizeSliderViewModelProperties {}

  export interface BinaryColorSizeSliderStyle {
    /**
     * The color of the slider's track above the middle stop value.
     *
     * @default new Color([149, 149, 149])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#style)
     */
    trackAboveFillColor?: Color;
    /**
     * The color of the slider's track below the middle stop value.
     *
     * @default new Color([149, 149, 149])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#style)
     */
    trackBelowFillColor?: Color;
    /**
     * The background color of the slider's track.
     *
     * @default new Color([224, 224, 224])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-BinaryColorSizeSlider.html#style)
     */
    trackBackgroundColor?: Color;
  }

  export class ClassedColorSlider extends SmartMappingSliderBase {
    /**
     * The ClassedColorSlider widget is designed for authoring and exploring data-driven visualizations in any layer that can be rendered with color in a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html)
     */

    constructor(properties?: ClassedColorSliderProperties);

    /**
     * An array of class breaks with associated colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#breaks)
     */
    breaks: ClassedColorSliderBreaks[];
    /**
     * The view model for the ClassedColorSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#viewModel)
     */
    viewModel: ClassedColorSliderViewModel;

    /**
     * A convenience function used to update the [classBreakInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos) of a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html) associated with this slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#updateClassBreakInfos)
     */
    updateClassBreakInfos(breakInfos: ClassBreakInfo[]): ClassBreakInfo[];
    /**
     * A convenience function used to update the properties a ClassedColorSlider from the result of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#updateFromRendererResult)
     */
    updateFromRendererResult(rendererResult: ClassBreaksRendererResult, histogramResult?: HistogramResult): void;

    /**
     * A convenience function used to create a ClassedColorSlider widget from the result of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createClassBreaksRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#fromRendererResult)
     */
    static fromRendererResult(
      rendererResult: ClassBreaksRendererResult,
      histogramResult?: HistogramResult,
    ): ClassedColorSlider;
  }

  interface ClassedColorSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * An array of class breaks with associated colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#breaks)
     */
    breaks?: ClassedColorSliderBreaks[];
    /**
     * The view model for the ClassedColorSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#viewModel)
     */
    viewModel?: ClassedColorSliderViewModelProperties;
  }

  export class ClassedColorSliderViewModel extends SmartMappingSliderViewModel {
    /**
     * Provides the logic for the [ClassedColorSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html)
     */

    constructor(properties?: ClassedColorSliderViewModelProperties);

    /**
     * An array of class breaks with associated colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#breaks)
     */
    breaks: ClassedColorSliderViewModelBreaks[];

    /**
     * Generates the color ramp rendered on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#getStopInfo)
     */
    getStopInfo(): StopInfo[];
  }

  interface ClassedColorSliderViewModelProperties extends SmartMappingSliderViewModelProperties {
    /**
     * An array of class breaks with associated colors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#breaks)
     */
    breaks?: ClassedColorSliderViewModelBreaks[];
  }

  export interface ClassedColorSliderViewModelBreaks {
    /**
     * Features with values within the provided `min` and `max` will be rendered with this color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#breaks)
     */
    color: Color;
    /**
     * The max value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#breaks)
     */
    max: number;
    /**
     * The min value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#breaks)
     */
    min: number;
  }

  /**
   * The return object of the [getStopInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#getStopInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#StopInfo)
   */
  export interface StopInfo {
    /**
     * The color of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#StopInfo)
     */
    color: Color;
    /**
     * The offset of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider-ClassedColorSliderViewModel.html#StopInfo)
     */
    offset: number;
  }

  export interface ClassedColorSliderBreaks {
    /**
     * Features with values within the provided `min` and `max` will be rendered with this color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#breaks)
     */
    color: Color;
    /**
     * The max value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#breaks)
     */
    max: number;
    /**
     * The min value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedColorSlider.html#breaks)
     */
    min: number;
  }

  export class ClassedSizeSlider extends SmartMappingSliderBase {
    /**
     * The ClassedSizeSlider widget is designed for authoring and exploring data-driven visualizations in any layer that can be rendered with size in a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html)
     */

    constructor(properties?: ClassedSizeSliderProperties);

    /**
     * An array of class breaks with associated sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#breaks)
     */
    breaks: ClassedSizeSliderBreaks[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#style)
     */
    style: ClassedSizeSliderStyle;
    /**
     * The view model for the ClassedSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#viewModel)
     */
    viewModel: ClassedSizeSliderViewModel;

    /**
     * A convenience function used to update the [classBreakInfos](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html#classBreakInfos) of a [ClassBreaksRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-ClassBreaksRenderer.html) associated with this slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#updateClassBreakInfos)
     */
    updateClassBreakInfos(breakInfos: ClassBreakInfo[]): ClassBreakInfo[];
    /**
     * A convenience function used to update the properties a ClassedSizeSlider from the result of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#updateClassBreakInfos)
     */
    updateClassBreakInfos(rendererResult: sizeClassBreaksRendererResult, histogramResult?: HistogramResult): void;

    /**
     * A convenience function used to create a ClassedSizeSlider widget from the result of the [createClassBreaksRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createClassBreaksRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#fromRendererResult)
     */
    static fromRendererResult(
      rendererResult: sizeClassBreaksRendererResult,
      histogramResult?: HistogramResult,
    ): ClassedSizeSlider;
  }

  interface ClassedSizeSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * An array of class breaks with associated sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#breaks)
     */
    breaks?: ClassedSizeSliderBreaks[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#style)
     */
    style?: ClassedSizeSliderStyle;
    /**
     * The view model for the ClassedSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#viewModel)
     */
    viewModel?: ClassedSizeSliderViewModelProperties;
  }

  export class ClassedSizeSliderViewModel extends SmartMappingSliderViewModel {
    /**
     * Provides the logic for the [ClassedSizeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html)
     */

    constructor(properties?: ClassedSizeSliderViewModelProperties);

    /**
     * An array of class breaks with associated sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html#breaks)
     */
    breaks: ClassedSizeSliderViewModelBreaks[];
  }

  interface ClassedSizeSliderViewModelProperties extends SmartMappingSliderViewModelProperties {
    /**
     * An array of class breaks with associated sizes.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html#breaks)
     */
    breaks?: ClassedSizeSliderViewModelBreaks[];
  }

  export interface ClassedSizeSliderViewModelBreaks {
    /**
     * The size in points of the symbol representing features with values within the given `min` and `max`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html#breaks)
     */
    size: number;
    /**
     * The max value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html#breaks)
     */
    max: number;
    /**
     * The min value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider-ClassedSizeSliderViewModel.html#breaks)
     */
    min: number;
  }

  export interface ClassedSizeSliderBreaks {
    /**
     * The size in points of the symbol representing features with values within the given `min` and `max`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#breaks)
     */
    size: number;
    /**
     * The max value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#breaks)
     */
    max: number;
    /**
     * The min value of the break.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#breaks)
     */
    min: number;
  }

  export interface ClassedSizeSliderStyle {
    /**
     * The color of the slider's track.
     *
     * @default new Color([149, 149, 149])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#style)
     */
    trackFillColor?: Color;
    /**
     * The background color of the slider's track.
     *
     * @default new Color([224, 224, 224])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ClassedSizeSlider.html#style)
     */
    trackBackgroundColor?: Color;
  }

  export class ColorSizeSlider extends SmartMappingSliderBase {
    /**
     * The ColorSizeSlider widget is intended for authoring and exploring data-driven visualizations in any layer that can be rendered with a [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) and a [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html)
     */

    constructor(properties?: ColorSizeSliderProperties);

    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary: boolean;
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled: boolean;
    /**
     * The colors and sizes corresponding with data values in the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) and [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) of the renderer associated with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#stops)
     */
    stops: ColorSizeStop[];
    /**
     * The view model for the ColorSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#viewModel)
     */
    viewModel: ColorSizeSliderViewModel;

    /**
     * A convenience function used to update the properties of a ColorSizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult) of the [univariateColorSize.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#updateFromRendererResult)
     */
    updateFromRendererResult(
      rendererResult: univariateColorSizeContinuousRendererResult,
      histogramResult?: HistogramResult,
    ): void;
    /**
     * A convenience function used to update a renderer generated with the [univariateColorSize.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method with the values obtained from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#updateRenderer)
     */
    updateRenderer(renderer: ClassBreaksRenderer): ClassBreaksRenderer;
    /**
     * A convenience function used to update the visual variables of a renderer generated with the [univariateColorSize.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method with the values obtained from the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#updateVisualVariables)
     */
    updateVisualVariables(variables: (ColorVariable | SizeVariable)[]): (ColorVariable | SizeVariable)[];

    /**
     * A convenience function used to create a ColorSizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#ContinuousRendererResult) of the [univariateColorSize.createContinuousRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-univariateColorSize.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#fromRendererResult)
     */
    static fromRendererResult(
      rendererResult: univariateColorSizeContinuousRendererResult,
      histogramResult?: HistogramResult,
    ): ColorSizeSlider;
  }

  interface ColorSizeSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary?: boolean;
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled?: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled?: boolean;
    /**
     * The colors and sizes corresponding with data values in the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) and [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) of the renderer associated with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#stops)
     */
    stops?: ColorSizeStopProperties[];
    /**
     * The view model for the ColorSizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html#viewModel)
     */
    viewModel?: ColorSizeSliderViewModelProperties;
  }

  export class ColorSizeSliderViewModel extends SizeSliderViewModel {
    /**
     * Provides the logic for the [ColorSizeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html)
     */

    constructor(properties?: ColorSizeSliderViewModelProperties);

    /**
     * The colors and sizes corresponding with data values in the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) and [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) of the renderer associated with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#stops)
     */
    stops: ColorSizeStop[];

    /**
     * Generates the color ramp gradient rendered on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#getStopInfo)
     */
    getStopInfo(): ColorSizeSliderViewModelStopInfo[];
  }

  interface ColorSizeSliderViewModelProperties extends SizeSliderViewModelProperties {
    /**
     * The colors and sizes corresponding with data values in the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) and [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) of the renderer associated with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#stops)
     */
    stops?: ColorSizeStopProperties[];
  }

  /**
   * The return object of the [getStopInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#getStopInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#StopInfo)
   */
  export interface ColorSizeSliderViewModelStopInfo {
    /**
     * The color of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#StopInfo)
     */
    color: Color;
    /**
     * The offset of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSizeSlider-ColorSizeSliderViewModel.html#StopInfo)
     */
    offset: number;
  }

  export class ColorSlider extends SmartMappingSliderBase {
    /**
     * The ColorSlider widget is intended for authoring and exploring data-driven visualizations in any layer that can be rendered with a [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html)
     */

    constructor(properties?: ColorSliderProperties);

    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled: boolean;
    /**
     * The color stops from the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#stops)
     */
    stops: ColorStop[];
    /**
     * The view model for the ColorSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#viewModel)
     */
    viewModel: ColorSliderViewModel;

    /**
     * A convenience function used to update the properties of a ColorSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#updateFromRendererResult)
     */
    updateFromRendererResult(rendererResult: ContinuousRendererResult, histogramResult?: HistogramResult): void;

    /**
     * A convenience function used to create a ColorSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-color.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#fromRendererResult)
     */
    static fromRendererResult(rendererResult: ContinuousRendererResult, histogramResult?: HistogramResult): ColorSlider;
  }

  interface ColorSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary?: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled?: boolean;
    /**
     * The color stops from the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#stops)
     */
    stops?: ColorStopProperties[];
    /**
     * The view model for the ColorSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html#viewModel)
     */
    viewModel?: ColorSliderViewModelProperties;
  }

  export class ColorSliderViewModel extends SmartMappingPrimaryHandleSliderViewModel {
    /**
     * Provides the logic for the [ColorSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html)
     */

    constructor(properties?: ColorSliderViewModelProperties);

    /**
     * The color stops from the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#stops)
     */
    stops: ColorStop[];

    /**
     * Generates the color ramp gradient rendered on the slider track.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#getStopInfo)
     */
    getStopInfo(): ColorSliderViewModelStopInfo[];
  }

  interface ColorSliderViewModelProperties extends SmartMappingPrimaryHandleSliderViewModelProperties {
    /**
     * The color stops from the [ColorVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-ColorVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#stops)
     */
    stops?: ColorStopProperties[];
  }

  /**
   * The return object of the [getStopInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#getStopInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#StopInfo)
   */
  export interface ColorSliderViewModelStopInfo {
    /**
     * The color of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#StopInfo)
     */
    color: Color;
    /**
     * The offset of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-ColorSlider-ColorSliderViewModel.html#StopInfo)
     */
    offset: number;
  }

  export class HeatmapSlider extends Widget {
    /**
     * The HeatmapSlider widget is intended for authoring and exploring data-driven visualizations in any layer that can be rendered with a [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html)
     */

    constructor(properties?: HeatmapSliderProperties);

    /**
     * The colorStops of the [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) to associate with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html#stops)
     */
    stops: HeatmapColorStop[];
    /**
     * The view model for the Heatmap widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html#viewModel)
     */
    viewModel: HeatmapSliderViewModel;

    on(name: "thumb-change", eventHandler: HeatmapSliderThumbChangeEventHandler): IHandle;

    on(name: "thumb-drag", eventHandler: HeatmapSliderThumbDragEventHandler): IHandle;

    /**
     * A convenience function used to create a HeatmapSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#HeatmapRendererResult) of the [heatmapRendererCreator.createRenderer()](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-heatmap.html#createRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html#fromHeatmapRendererResult)
     */
    static fromHeatmapRendererResult(rendererResult: HeatmapRendererResult): HeatmapSlider;
  }

  interface HeatmapSliderProperties extends WidgetProperties {
    /**
     * The colorStops of the [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) to associate with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html#stops)
     */
    stops?: HeatmapColorStopProperties[];
    /**
     * The view model for the Heatmap widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html#viewModel)
     */
    viewModel?: HeatmapSliderViewModelProperties;
  }

  export class HeatmapSliderViewModel extends SmartMappingSliderViewModel {
    /**
     * Provides the logic for the [HeatmapSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html)
     */

    constructor(properties?: HeatmapSliderViewModelProperties);

    /**
     * The colorStops of the [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) to associate with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#stops)
     */
    stops: HeatmapColorStop[];

    /**
     * Generates the color ramp gradient rendered on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#getStopInfo)
     */
    getStopInfo(): HeatmapSliderViewModelStopInfo[];
  }

  interface HeatmapSliderViewModelProperties extends SmartMappingSliderViewModelProperties {
    /**
     * The colorStops of the [HeatmapRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-HeatmapRenderer.html) to associate with the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#stops)
     */
    stops?: HeatmapColorStopProperties[];
  }

  /**
   * The return object of the [getColorStopInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#getColorStopInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#StopInfo)
   */
  export interface HeatmapSliderViewModelStopInfo {
    /**
     * The color of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#StopInfo)
     */
    color: Color;
    /**
     * The offset of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-HeatmapSlider-HeatmapSliderViewModel.html#StopInfo)
     */
    offset: number;
  }

  export interface HeatmapSliderThumbChangeEvent {
    index: number;

    oldValue: number;

    type: "thumb-change";

    value: number;
  }

  export interface HeatmapSliderThumbDragEvent {
    index: number;

    state: "start" | "drag";

    type: "thumb-drag";

    value: number;
  }

  export class OpacitySlider extends SmartMappingSliderBase {
    /**
     * The OpacitySlider widget is intended for authoring and exploring data-driven visualizations in any layer that can be rendered with an [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html)
     */

    constructor(properties?: OpacitySliderProperties);

    /**
     * The opacity stops from the [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#stops)
     */
    stops: OpacityStop[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#style)
     */
    style: OpacitySliderStyle;
    /**
     * The view model for the OpacitySlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#viewModel)
     */
    viewModel: OpacitySliderViewModel;

    /**
     * A convenience function used to update the properties of an OpacitySlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult) of the [createVisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#updateFromVisualVariableResult)
     */
    updateFromVisualVariableResult(
      visualVariableResult: opacityVisualVariableResult,
      histogramResult?: HistogramResult,
    ): void;

    /**
     * A convenience function used to create an OpacitySlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#VisualVariableResult) of the [createVisualVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-opacity.html#createVisualVariable) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#fromVisualVariableResult)
     */
    static fromVisualVariableResult(
      visualVariableResult: opacityVisualVariableResult,
      histogramResult?: HistogramResult,
    ): OpacitySlider;
  }

  interface OpacitySliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * The opacity stops from the [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#stops)
     */
    stops?: OpacityStopProperties[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#style)
     */
    style?: OpacitySliderStyle;
    /**
     * The view model for the OpacitySlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#viewModel)
     */
    viewModel?: OpacitySliderViewModelProperties;
  }

  export class OpacitySliderViewModel extends SmartMappingSliderViewModel {
    /**
     * Provides the logic for the [OpacitySlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html)
     */

    constructor(properties?: OpacitySliderViewModelProperties);

    /**
     * The opacity stops from the [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#stops)
     */
    stops: OpacityStop[];

    /**
     * Generates the opacity ramp gradient rendered on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#getStopInfo)
     */
    getStopInfo(): OpacitySliderViewModelStopInfo[];
  }

  interface OpacitySliderViewModelProperties extends SmartMappingSliderViewModelProperties {
    /**
     * The opacity stops from the [OpacityVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-OpacityVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#stops)
     */
    stops?: OpacityStopProperties[];
  }

  /**
   * The return object of the [getStopInfo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#getStopInfo) method.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#StopInfo)
   */
  export interface OpacitySliderViewModelStopInfo {
    /**
     * The color of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#StopInfo)
     */
    color: Color;
    /**
     * The offset of the stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider-OpacitySliderViewModel.html#StopInfo)
     */
    offset: number;
  }

  export interface OpacitySliderStyle {
    /**
     * The color of the slider's track.
     *
     * @default new Color([0, 121, 193])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-OpacitySlider.html#style)
     */
    trackFillColor?: Color;
  }

  export class SizeSlider extends SmartMappingSliderBase {
    /**
     * The SizeSlider widget is intended for authoring and exploring data-driven visualizations in any layer that can be rendered with a [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html)
     */

    constructor(properties?: SizeSliderProperties);

    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary: boolean;
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#stops)
     */
    stops: SizeStop[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#style)
     */
    style: SizeSliderStyle;
    /**
     * The view model for the SizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#viewModel)
     */
    viewModel: SizeSliderViewModel;

    /**
     * A convenience function used to update the properties of a SizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#updateFromRendererResult)
     */
    updateFromRendererResult(rendererResult: sizeContinuousRendererResult, histogramResult?: HistogramResult): void;
    /**
     * A convenience function used to update the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to match the values of the [stops](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#stops) on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#updateVisualVariable)
     */
    updateVisualVariable(sizeVariable: SizeVariable): SizeVariable;

    /**
     * A convenience function used to create a SizeSlider widget instance from the [result](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#ContinuousRendererResult) of the [createContinuousRenderer](https://developers.arcgis.com/javascript/latest/api-reference/esri-smartMapping-renderers-size.html#createContinuousRenderer) method.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#fromRendererResult)
     */
    static fromRendererResult(
      rendererResult: sizeContinuousRendererResult,
      histogramResult?: HistogramResult,
    ): SizeSlider;
  }

  interface SizeSliderProperties extends SmartMappingSliderBaseProperties {
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary?: boolean;
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled?: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#primaryHandleEnabled)
     */
    primaryHandleEnabled?: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#stops)
     */
    stops?: SizeStopProperties[];
    /**
     * Exposes various properties of the widget that can be styled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#style)
     */
    style?: SizeSliderStyle;
    /**
     * The view model for the SizeSlider widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#viewModel)
     */
    viewModel?: SizeSliderViewModelProperties;
  }

  export class SizeSliderViewModel extends SmartMappingPrimaryHandleSliderViewModel {
    /**
     * Provides the logic for the [SizeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider-SizeSliderViewModel.html)
     */

    constructor(properties?: SizeSliderViewModelProperties);

    /**
     * This property is typically used in diverging, or `above-and-below` renderer configurations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider-SizeSliderViewModel.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider-SizeSliderViewModel.html#stops)
     */
    stops: SizeStop[];
  }

  interface SizeSliderViewModelProperties extends SmartMappingPrimaryHandleSliderViewModelProperties {
    /**
     * This property is typically used in diverging, or `above-and-below` renderer configurations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider-SizeSliderViewModel.html#persistSizeRangeEnabled)
     */
    persistSizeRangeEnabled?: boolean;
    /**
     * The size stops from the [SizeVariable](https://developers.arcgis.com/javascript/latest/api-reference/esri-renderers-visualVariables-SizeVariable.html) to link to the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider-SizeSliderViewModel.html#stops)
     */
    stops?: SizeStopProperties[];
  }

  export interface SizeSliderStyle {
    /**
     * The color of the slider's track.
     *
     * @default new Color([149, 149, 149])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#style)
     */
    trackFillColor?: Color;
    /**
     * The background color of the slider's track.
     *
     * @default new Color([224, 224, 224])
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SizeSlider.html#style)
     */
    trackBackgroundColor?: Color;
  }

  export class SmartMappingPrimaryHandleSliderViewModel extends SmartMappingSliderViewModel {
    /**
     * Shared properties for sliders that have a [primaryHandle](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html#primaryHandleEnabled) option.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html)
     */

    constructor(properties?: SmartMappingPrimaryHandleSliderViewModelProperties);

    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html#primaryHandleEnabled)
     */
    primaryHandleEnabled: boolean;
  }

  interface SmartMappingPrimaryHandleSliderViewModelProperties extends SmartMappingSliderViewModelProperties {
    /**
     * Only applicable when three thumbs (i.e.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html#handlesSyncedToPrimary)
     */
    handlesSyncedToPrimary?: boolean;
    /**
     * When `true`, the slider will render a third handle between the two handles already provided by default.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingPrimaryHandleSliderViewModel.html#primaryHandleEnabled)
     */
    primaryHandleEnabled?: boolean;
  }

  export class SmartMappingSliderBase extends Widget {
    /**
     * The base class for all Smart Mapping slider widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html)
     */

    constructor(properties?: SmartMappingSliderBaseProperties);

    /**
     * The [Histogram](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html) associated with the data represented on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#histogramConfig)
     */
    histogramConfig: HistogramConfig;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputFormatFunction)
     */
    inputFormatFunction: SmartMappingSliderBaseLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputParseFunction)
     */
    inputParseFunction: InputParser;
    /**
     * A modified version of [Slider.labelFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelFormatFunction), which is a custom function used to format labels on the thumbs, min, max, and average values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#labelFormatFunction)
     */
    labelFormatFunction: SmartMappingSliderBaseLabelFormatter;
    /**
     * The maximum value or upper bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#max)
     */
    max: number;
    /**
     * The minimum value or lower bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#min)
     */
    min: number;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#precision)
     */
    precision: number;
    /**
     * The state of the view model.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * When `true`, all segments will sync together in updating thumb values when the user drags any segment.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#syncedSegmentsEnabled)
     */
    syncedSegmentsEnabled: boolean;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#visibleElements)
     */
    visibleElements: SmartMappingSliderBaseVisibleElements;
    /**
     * Zooms the slider track to the bounds provided in this property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#zoomOptions)
     */
    zoomOptions: SmartMappingSliderBaseZoomOptions;

    on(name: "max-change", eventHandler: SmartMappingSliderBaseMaxChangeEventHandler): IHandle;

    on(name: "min-change", eventHandler: SmartMappingSliderBaseMinChangeEventHandler): IHandle;

    on(name: "segment-drag", eventHandler: SmartMappingSliderBaseSegmentDragEventHandler): IHandle;

    on(name: "thumb-change", eventHandler: SmartMappingSliderBaseThumbChangeEventHandler): IHandle;

    on(name: "thumb-drag", eventHandler: SmartMappingSliderBaseThumbDragEventHandler): IHandle;
  }

  interface SmartMappingSliderBaseProperties extends WidgetProperties {
    /**
     * The [Histogram](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html) associated with the data represented on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#histogramConfig)
     */
    histogramConfig?: HistogramConfig;
    /**
     * A function used to format user inputs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputFormatFunction)
     */
    inputFormatFunction?: SmartMappingSliderBaseLabelFormatter;
    /**
     * Function used to parse slider inputs formatted by the [inputFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputFormatFunction).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#inputParseFunction)
     */
    inputParseFunction?: InputParser;
    /**
     * A modified version of [Slider.labelFormatFunction](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Slider.html#labelFormatFunction), which is a custom function used to format labels on the thumbs, min, max, and average values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#labelFormatFunction)
     */
    labelFormatFunction?: SmartMappingSliderBaseLabelFormatter;
    /**
     * The maximum value or upper bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#max)
     */
    max?: number;
    /**
     * The minimum value or lower bound of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#min)
     */
    min?: number;
    /**
     * Defines how slider thumb values should be rounded.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#precision)
     */
    precision?: number;
    /**
     * When `true`, all segments will sync together in updating thumb values when the user drags any segment.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#syncedSegmentsEnabled)
     */
    syncedSegmentsEnabled?: boolean;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#visibleElements)
     */
    visibleElements?: SmartMappingSliderBaseVisibleElements;
    /**
     * Zooms the slider track to the bounds provided in this property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#zoomOptions)
     */
    zoomOptions?: SmartMappingSliderBaseZoomOptions;
  }

  /**
   * Configuration options for defining the slider's [Histogram](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Histogram.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
   */
  export interface HistogramConfig {
    /**
     * Indicates the average value of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    average?: number;
    /**
     * Function for styling bars representing histogram bins.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    barCreatedFunction?: BarCreatedFunction;
    /**
     * The bins of the histogram.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    bins?: Bin[];
    /**
     * When set, renders lines on the histogram that indicate important or meaningful values to the end user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    dataLines?: HistogramConfigDataLines[];
    /**
     * Function that fires each time a data line is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    dataLineCreatedFunction?: DataLineCreatedFunction;
    /**
     * Indicates the standard deviation of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    standardDeviation?: number;
    /**
     * Indicates the standard deviation of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    standardDeviationCount?: number;
  }

  export type SmartMappingSliderBaseLabelFormatter = (
    value: number,
    type?: "average" | "min" | "max" | "value",
    index?: number,
  ) => string;

  export interface SmartMappingSliderBaseMaxChangeEvent {
    oldValue: number;

    type: "max-change";

    value: number;
  }

  export interface SmartMappingSliderBaseMinChangeEvent {
    oldValue: number;

    type: "min-change";

    value: number;
  }

  export interface SmartMappingSliderBaseSegmentDragEvent {
    index: number;

    state: "start" | "drag";

    thumbIndices: number[];

    type: "segment-drag";
  }

  export interface SmartMappingSliderBaseVisibleElements {
    /**
     * When `true`, displays interactive segments on the track that maintain the interval between two slider thumbs/handles.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#visibleElements)
     */
    interactiveTrack?: boolean;
  }

  export interface SmartMappingSliderBaseZoomOptions {
    /**
     * The lower bound of the zoom.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#zoomOptions)
     */
    min?: number;
    /**
     * The upper bound of the zoom.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#zoomOptions)
     */
    max?: number;
  }

  export interface SmartMappingSliderBaseThumbChangeEvent {
    index: number;

    oldValue: number;

    type: "thumb-change";

    value: number;
  }

  export interface SmartMappingSliderBaseThumbDragEvent {
    index: number;

    state: "start" | "drag";

    type: "thumb-drag";

    value: number;
  }

  export interface HistogramConfigDataLines {
    /**
     * The value on the data axis of the histogram where a line will be rendered.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    value?: number;
    /**
     * The label associated with the line.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderBase.html#HistogramConfig)
     */
    label?: string | number;
  }

  export class SmartMappingSliderViewModel extends SliderViewModel {
    /**
     * Provides the shared base logic for the smart mapping slider view models.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html)
     */

    constructor(properties?: SmartMappingSliderViewModelProperties);

    /**
     * Enables zooming on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomingEnabled)
     */
    zoomingEnabled: boolean;
    /**
     * Zooms the slider track to the bounds provided in this property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomOptions)
     */
    zoomOptions: SmartMappingSliderViewModelZoomOptions;

    /**
     * Gets the [max](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#max) value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#getUnzoomedMax)
     */
    getUnzoomedMax(): void;
    /**
     * Gets the [min](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#min) value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#getUnzoomedMin)
     */
    getUnzoomedMin(): void;
  }

  interface SmartMappingSliderViewModelProperties extends SliderViewModelProperties {
    /**
     * Enables zooming on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomingEnabled)
     */
    zoomingEnabled?: boolean;
    /**
     * Zooms the slider track to the bounds provided in this property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomOptions)
     */
    zoomOptions?: SmartMappingSliderViewModelZoomOptions;
  }

  export interface SmartMappingSliderViewModelZoomOptions {
    /**
     * The lower bound of the zoom.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomOptions)
     */
    min?: number;
    /**
     * The upper bound of the zoom.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-SmartMappingSliderViewModel.html#zoomOptions)
     */
    max?: number;
  }

  /**
   * Various utils for tying Smart Mapping renderers to the Smart Mapping slider widgets.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-support-utils.html)
   */
  interface smartMappingSupportUtils {
    /**
     * Formats a UNIX timestamp to a basic date string.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-support-utils.html#formatDateLabel)
     */
    formatDateLabel(value: number): string;
    /**
     * Formats a numeric value for display as a label based on the current locale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-support-utils.html#formatNumberLabel)
     */
    formatNumberLabel(value: number): string;
    /**
     * Computes and returns standard deviation values based on the given average and standard deviation.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-smartMapping-support-utils.html#getDeviationValues)
     */
    getDeviationValues(standardDeviation: number, average: number, count: number): number[];
  }

  export const smartMappingSupportUtils: smartMappingSupportUtils;

  export class GeolocationPositioning {
    /**
     * An object used for setting optional position parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#geolocationOptions)
     */
    geolocationOptions: any;
    /**
     * Indicates whether to navigate the view to the position and scale of the geolocated result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#goToLocationEnabled)
     */
    goToLocationEnabled: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#graphic)
     */
    graphic: Graphic;
    /**
     * Indicates the scale to set on the view when navigating to the position of the geolocated result, after a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#scale)
     */
    scale: number;
    /**
     * The view associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#view)
     */
    view: MapView | SceneView;
  }

  interface GeolocationPositioningProperties {
    /**
     * An object used for setting optional position parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#geolocationOptions)
     */
    geolocationOptions?: any;
    /**
     * Indicates whether to navigate the view to the position and scale of the geolocated result.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#goToLocationEnabled)
     */
    goToLocationEnabled?: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * Indicates the scale to set on the view when navigating to the position of the geolocated result, after a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#scale)
     */
    scale?: number;
    /**
     * The view associated with the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GeolocationPositioning.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export class GoTo {
    /**
     * This function provides the ability to override either the [MapView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo) or [SceneView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GoTo.html#goToOverride)
     */
    goToOverride: GoToOverride;
  }

  interface GoToProperties {
    /**
     * This function provides the ability to override either the [MapView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#goTo) or [SceneView goTo()](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html#goTo) methods.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GoTo.html#goToOverride)
     */
    goToOverride?: GoToOverride;
  }

  export class GridControls extends Widget {
    /**
     * The `GridControls` can be configured using the [visibleElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#visibleElements) below.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html)
     */

    constructor(properties?: GridControlsProperties);

    /**
     * The custom color used for drawing the grid, if any.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#customColor)
     */
    customColor: Color;
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) that will be controlled by this widget if the snapping toggle is enabled for display by [visibleElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#visibleElements).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The color scheme used to display the grid.
     *
     * @default "light"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#theme)
     */
    theme: "light" | "dark" | "custom";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#viewModel)
     */
    viewModel: GridControlsViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#visibleElements)
     */
    visibleElements: GridControlsVisibleElements;
  }

  interface GridControlsProperties extends WidgetProperties {
    /**
     * The custom color used for drawing the grid, if any.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#customColor)
     */
    customColor?: Color;
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) that will be controlled by this widget if the snapping toggle is enabled for display by [visibleElements](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#visibleElements).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * The color scheme used to display the grid.
     *
     * @default "light"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#theme)
     */
    theme?: "light" | "dark" | "custom";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#viewModel)
     */
    viewModel?: GridControlsViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#visibleElements)
     */
    visibleElements?: GridControlsVisibleElements;
  }

  export class GridControlsViewModel extends Accessor {
    /**
     * Provides the logic for the [GridControls](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html)
     */

    constructor(properties?: GridControlsViewModelProperties);

    /**
     * Returns `true` if the grid is enabled for display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#displayEnabled)
     */
    readonly displayEnabled: boolean;
    /**
     * Returns `true` if the grid is set to scale dynamically.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling)
     */
    dynamicScaling: boolean;
    /**
     * Returns the effective spacing of grid lines after applying the [dynamicScaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling) setting at the current view scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#effectiveSpacingAfterDynamicScaling)
     */
    readonly effectiveSpacingAfterDynamicScaling: number;
    /**
     * Sets the color used for grid display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#gridColor)
     */
    gridColor: Color;
    /**
     * Returns `true` if the grid is in a valid state for manually setting grid properties or starting an interactive placement.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#gridControlsEnabled)
     */
    readonly gridControlsEnabled: boolean;
    /**
     * True if the grid is currently not displayed (and therefore also not a valid snap target), because [dynamicScaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling) is off and the grid cells are too small to render at the current scale.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#gridOutOfScale)
     */
    readonly gridOutOfScale: boolean;
    /**
     * Sets the interactive placement state, either starting or ending a draw operation that implicitly adjusts the grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#interactivePlacementState)
     */
    interactivePlacementState: "interactive" | "place" | "rotate";
    /**
     * Controls the number of grid lines shown at 50% opacity between opaque grid lines when using [dynamicScaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling).
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#majorLineInterval)
     */
    majorLineInterval: number;
    /**
     * True if the spacing input should be shown for the current view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#numericSpacingInputShouldBeVisible)
     */
    readonly numericSpacingInputShouldBeVisible: boolean;
    /**
     * Determines the behavior of the grid when the map's viewpoint is rotated.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#rotateWithMap)
     */
    rotateWithMap: boolean;
    /**
     * The grid's rotation in radians.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#rotation)
     */
    rotation: number;
    /**
     * Controls snapping behavior if [snappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingOptions) has been defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingEnabled)
     */
    snappingEnabled: boolean;
    /**
     * This is the snapping options object that will be configured by the 'enable snapping' property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * Length of a grid cell.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#spacing)
     */
    spacing: number;
    /**
     * Unit of measure (foot, meter, etc) used when defining the [spacing](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#spacing) grid cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#unit)
     */
    unit: LengthUnit;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#view)
     */
    view: MapView;

    /**
     * Try to enable or disable grid display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#trySetDisplayEnabled)
     */
    trySetDisplayEnabled(visible: boolean): void;
  }

  interface GridControlsViewModelProperties {
    /**
     * Returns `true` if the grid is set to scale dynamically.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling)
     */
    dynamicScaling?: boolean;
    /**
     * Sets the color used for grid display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#gridColor)
     */
    gridColor?: Color;
    /**
     * Sets the interactive placement state, either starting or ending a draw operation that implicitly adjusts the grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#interactivePlacementState)
     */
    interactivePlacementState?: "interactive" | "place" | "rotate";
    /**
     * Controls the number of grid lines shown at 50% opacity between opaque grid lines when using [dynamicScaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#dynamicScaling).
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#majorLineInterval)
     */
    majorLineInterval?: number;
    /**
     * Determines the behavior of the grid when the map's viewpoint is rotated.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#rotateWithMap)
     */
    rotateWithMap?: boolean;
    /**
     * The grid's rotation in radians.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#rotation)
     */
    rotation?: number;
    /**
     * Controls snapping behavior if [snappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingOptions) has been defined.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingEnabled)
     */
    snappingEnabled?: boolean;
    /**
     * This is the snapping options object that will be configured by the 'enable snapping' property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * Length of a grid cell.
     *
     * @default 1
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#spacing)
     */
    spacing?: number;
    /**
     * Unit of measure (foot, meter, etc) used when defining the [spacing](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#spacing) grid cell.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#unit)
     */
    unit?:
      | "millimeters"
      | "centimeters"
      | "decimeters"
      | "meters"
      | "kilometers"
      | "inches"
      | "feet"
      | "yards"
      | "miles"
      | "nautical-miles"
      | "us-feet";
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls-GridControlsViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
   */
  export interface GridControlsVisibleElements {
    /**
     * Indicates whether to display the buttons to change the color of the grid.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    colorSelection?: boolean;
    /**
     * Indicates whether to display the `dynamicScaleToggle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    dynamicScaleToggle?: boolean;
    /**
     * Indicates whether to display the `gridEnabledToggle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * Indicates whether to display the `numericInputs`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    numericInputs?: boolean;
    /**
     * Indicates whether to display the `gridSnapEnabledToggle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    gridSnapEnabledToggle?: boolean;
    /**
     * Indicates whether to display the `lineIntervalInput`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    lineIntervalInput?: boolean;
    /**
     * Indicates whether to display a notice when the grid is not visible because it is out of scale and dynamic scaling is turned off.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    outOfScaleWarning?: boolean;
    /**
     * Indicates whether to display `placementButtons`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    placementButtons?: boolean;
    /**
     * Indicates whether to display the `rotateWithMapToggle`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-GridControls.html#VisibleElements)
     */
    rotateWithMapToggle?: boolean;
  }

  export class SnappingControls extends Widget {
    /**
     * The SnappingControls widget provides a user interface to use alongside the [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) class.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html)
     */

    constructor(properties?: SnappingControlsProperties);

    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for the SnappingControls widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#viewModel)
     */
    viewModel: SnappingControlsViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#visibleElements)
     */
    visibleElements: SnappingControlsVisibleElements;
  }

  interface SnappingControlsProperties extends WidgetProperties {
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for the SnappingControls widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#viewModel)
     */
    viewModel?: SnappingControlsViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#visibleElements)
     */
    visibleElements?: SnappingControlsVisibleElements;
  }

  export class SnappingControlsViewModel extends Accessor {
    /**
     * Provides the logic for the [SnappingControls](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html)
     */

    constructor(properties?: SnappingControlsViewModelProperties);

    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html#snappingOptions)
     */
    snappingOptions: SnappingOptions;
    /**
     * The current state of the view model.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html#state)
     */
    readonly state: "ready" | "disabled";
    /**
     * The view associated with the SnappingControls widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html#view)
     */
    view: MapView | SceneView;
  }

  interface SnappingControlsViewModelProperties {
    /**
     * The [SnappingOptions](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-SnappingOptions.html) for sketching.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html#snappingOptions)
     */
    snappingOptions?: SnappingOptionsProperties;
    /**
     * The view associated with the SnappingControls widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls-SnappingControlsViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
   */
  export interface SnappingControlsVisibleElements {
    /**
     * Indicates whether to display the `enabledToggle` (Enable snapping).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    enabledToggle?: boolean;
    /**
     * Indicates whether to display the `featureEnabledToggle` (Feature to feature).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    featureEnabledToggle?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    gridControls?: boolean;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    gridControlsElements?: GridControlsVisibleElements;
    /**
     * _Since 4.30_.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    gridEnabledToggle?: boolean;
    /**
     * Indicates whether to display the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    header?: boolean;
    /**
     * Indicates whether to display the [FeatureSnappingLayerSource](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-interactive-snapping-FeatureSnappingLayerSource.html) layerList.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    layerList?: boolean;
    /**
     * Indicates whether to display the “Enable all” or “Disable all” button to enable / disable snapping for all the layers in the list.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    layerListToggleLayersButton?: boolean;
    /**
     * Indicates whether to display the `selfEnabledToggle` (Geometry guides).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-SnappingControls.html#VisibleElements)
     */
    selfEnabledToggle?: boolean;
  }

  namespace widget {
    /**
     * This convenience decorator is used to help simplify accessibility within the widget keyboard events.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#accessibleHandler)
     */
    function accessibleHandler(): Function;
    /**
     * Utility method used for creating CSS animation/transition functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#cssTransition)
     */
    function cssTransition(type: "enter" | "exit", className: string): Function;
    /**
     * Utility method used to determine if a pressed key should activate button behavior.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#isActivationKey)
     */
    function isActivationKey(key: string): boolean;
    /**
     * Utility method used to determine if the directionality of the text of the document is right-to-left.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#isRTL)
     */
    function isRTL(): boolean;
    /**
     * This convenience decorator is used to help simplify localization of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#messageBundle)
     */
    function messageBundle(bundleId?: string): Function;
    /**
     * This convenience method is used to assign an [HTMLElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement) DOM node reference to a variable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#storeNode)
     */
    function storeNode(node: HTMLElement): void;
    /**
     * This convenience decorator helps dispatch view model events on the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#vmEvent)
     */
    function vmEvent(eventNames: string | string[]): Function;

    namespace tsx {
      /**
       * This convenience method is used to render the JSX in the [widget.render()](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#render) method.
       *
       * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-support-widget.html#tsx)
       */
      function tsx(selector: string, properties?: any, children?: any): any;

      namespace JSX {
        interface IntrinsicElements {
          [elementName: string]: any;
        }

        interface Element {}
      }
    }
  }

  export type GoToOverride = (view: MapView | SceneView, goToParameters: any) => void;

  export class Swipe extends Widget {
    /**
     * The Swipe widget provides a tool to show a portion of a layer or layers on top of a map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html)
     */

    constructor(properties?: SwipeProperties);

    /**
     * The direction the Swipe widget moves across the view.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#direction)
     */
    direction: "horizontal" | "vertical";
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#disabled)
     */
    disabled: boolean;
    /**
     * The text that shows in a tooltip when hovering over the handle of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#dragLabel)
     */
    dragLabel: string;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the left or top side of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#leadingLayers)
     */
    leadingLayers: Collection<Layer>;
    /**
     * The position of the Swipe widget.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#position)
     */
    position: number;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the right or bottom side of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#trailingLayers)
     */
    trailingLayers: Collection<Layer>;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#viewModel)
     */
    viewModel: SwipeViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#visibleElements)
     */
    visibleElements: SwipeVisibleElements;
  }

  interface SwipeProperties extends WidgetProperties {
    /**
     * The direction the Swipe widget moves across the view.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#direction)
     */
    direction?: "horizontal" | "vertical";
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#disabled)
     */
    disabled?: boolean;
    /**
     * The text that shows in a tooltip when hovering over the handle of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#dragLabel)
     */
    dragLabel?: string;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the left or top side of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#leadingLayers)
     */
    leadingLayers?: CollectionProperties<LayerProperties>;
    /**
     * The position of the Swipe widget.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#position)
     */
    position?: number;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the right or bottom side of the Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#trailingLayers)
     */
    trailingLayers?: CollectionProperties<LayerProperties>;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#viewModel)
     */
    viewModel?: SwipeViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#visibleElements)
     */
    visibleElements?: SwipeVisibleElements;
  }

  export class SwipeViewModel extends Accessor {
    /**
     * Provides the logic for the [Swipe](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html)
     */

    constructor(properties?: SwipeViewModelProperties);

    /**
     * The direction the Swipe widget moves across the view.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#direction)
     */
    direction: "horizontal" | "vertical";
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the left or top side of Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#leadingLayers)
     */
    leadingLayers: Collection<Layer>;
    /**
     * The position of the Swipe widget.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#position)
     */
    position: number;
    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the right or bottom side of Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#trailingLayers)
     */
    trailingLayers: Collection<Layer>;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#view)
     */
    view: MapView;
  }

  interface SwipeViewModelProperties {
    /**
     * The direction the Swipe widget moves across the view.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#direction)
     */
    direction?: "horizontal" | "vertical";
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the left or top side of Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#leadingLayers)
     */
    leadingLayers?: CollectionProperties<LayerProperties>;
    /**
     * The position of the Swipe widget.
     *
     * @default 25
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#position)
     */
    position?: number;
    /**
     * A collection of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html)s that will show on the right or bottom side of Swipe widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#trailingLayers)
     */
    trailingLayers?: CollectionProperties<LayerProperties>;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe-SwipeViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#VisibleElements)
   */
  export interface SwipeVisibleElements {
    /**
     * Indicates whether the handle from which you drag the widget is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#VisibleElements)
     */
    handle?: boolean;
    /**
     * Indicates whether the divider between the leading and trailing layers is visible.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Swipe.html#VisibleElements)
     */
    divider?: boolean;
  }

  export class TableList extends Widget {
    /**
     * The TableList widget provides a way to display a list of tables associated with a [Map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html)
     */

    constructor(properties?: TableListProperties);

    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#collapsed)
     */
    collapsed: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#dragEnabled)
     */
    dragEnabled: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#filterPlaceholder)
     */
    filterPlaceholder: string;
    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#filterText)
     */
    filterText: string;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#title) of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#headingLevel)
     */
    headingLevel: number;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: TableListListItemCreatedHandler;
    /**
     * A reference to a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) containing the tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#map)
     */
    map: Map | WebMap;
    /**
     * The minimum number of list items required to enable drag and drop reordering with [dragEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#dragEnabled).
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#minDragEnabledItems)
     */
    minDragEnabledItems: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#minFilterItems)
     */
    minFilterItems: number;
    /**
     * Indicates whether more than one [list item](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) can be selected by a user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/TableList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled: boolean;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html)s representing table list items selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectedItems)
     */
    selectedItems: Collection<TableListListItem>;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) may be selected by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/TableList#selectionMode selectionMode} and {@link module:esri/widgets/TableList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectionEnabled)
     */
    selectionEnabled: boolean;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectionMode)
     */
    selectionMode: "multiple" | "none" | "single" | "single-persist";
    /**
     * The collection of table [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html)s displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#tableItems)
     */
    readonly tableItems: Collection<TableListListItem>;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#viewModel)
     */
    viewModel: TableListViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements)
     */
    visibleElements: TableListVisibleElements;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: TableListListItem): void;

    on(name: "trigger-action", eventHandler: TableListTriggerActionEventHandler): IHandle;
  }

  interface TableListProperties extends WidgetProperties {
    /**
     * Indicates whether the widget is collapsed.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#collapsed)
     */
    collapsed?: boolean;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) may be reordered within the list by dragging and dropping.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#dragEnabled)
     */
    dragEnabled?: boolean;
    /**
     * Placeholder text used in the filter input if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#filterPlaceholder)
     */
    filterPlaceholder?: string;
    /**
     * The value of the filter input text string if [visibleElements.filter](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements) is true.
     *
     * @default ""
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#filterText)
     */
    filterText?: string;
    /**
     * Indicates the heading level to use for the [title](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#title) of the widget.
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: TableListListItemCreatedHandler;
    /**
     * A reference to a [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) containing the tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#map)
     */
    map?: MapProperties | WebMapProperties;
    /**
     * The minimum number of list items required to enable drag and drop reordering with [dragEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#dragEnabled).
     *
     * @default 2
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#minDragEnabledItems)
     */
    minDragEnabledItems?: number;
    /**
     * The minimum number of list items required to display the visibleElements.filter input box.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#minFilterItems)
     */
    minFilterItems?: number;
    /**
     * Indicates whether more than one [list item](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) can be selected by a user at a single time.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/TableList#selectionMode selectionMode} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#multipleSelectionEnabled)
     */
    multipleSelectionEnabled?: boolean;
    /**
     * A collection of selected [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html)s representing table list items selected by the user.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectedItems)
     */
    selectedItems?: CollectionProperties<TableListListItem>;
    /**
     * Indicates whether [list items](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) may be selected by the user.
     *
     * @default false
     * @deprecated since 4.29. Use {@link module:esri/widgets/TableList#selectionMode selectionMode} and {@link module:esri/widgets/TableList#dragEnabled dragEnabled} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectionEnabled)
     */
    selectionEnabled?: boolean;
    /**
     * Specifies the selection mode.
     *
     * @default "none"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#selectionMode)
     */
    selectionMode?: "multiple" | "none" | "single" | "single-persist";
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#viewModel)
     */
    viewModel?: TableListViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#visibleElements)
     */
    visibleElements?: TableListVisibleElements;
  }

  export class TableListListItem extends Identifiable {
    /**
     * Indicates whether the actions panel is open in the TableList.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#actionsOpen)
     */
    actionsOpen: boolean;
    /**
     * A nested 2-dimensional collection of actions that could be triggered on the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#actionsSections)
     */
    actionsSections: Collection<Collection<ActionButton | ActionToggle>>;
    /**
     * The Error object returned if an error occurred.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#error)
     */
    readonly error: Error;
    /**
     * When `true`, hides the layer from the TableList instance.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#hidden)
     */
    hidden: boolean;
    /**
     * The layer associated with the triggered action.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#layer)
     */
    layer: Layer;
    /**
     * Allows you to display custom content for each ListItem in the [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#panel)
     */
    panel: TableListListItemPanel;
    /**
     * Value is `true` when the [layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#layer) is being published.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#publishing)
     */
    readonly publishing: boolean;
    /**
     * The title of the table.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#title)
     */
    title: string;

    /**
     * Creates a deep clone of this object.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html#clone)
     */
    clone(): TableListListItem;

    constructor();
  }

  export interface TableListListItemPanel extends Widget, Identifiable {}

  export class TableListListItemPanel {
    /**
     * This class allows you to display custom content for each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html) in the [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html)
     */

    constructor(properties?: TableListListItemPanelProperties);

    /**
     * Adds a CSS class used to style a node that represents the panel.
     *
     * @default null
     * @deprecated since version 4.30. Use {@link module:esri/widgets/TableList/ListItemPanel#icon icon}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#className)
     */
    className: string;
    /**
     * The content displayed in the ListItem panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#content)
     */
    content: TableListItemPanelContent | TableListItemPanelContent[];
    /**
     * If `true`, disables the ListItem's panel so the user cannot open or interact with it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates whether the panel's content should be rendered as a [Calcite Flow Item](https://developers.arcgis.com/calcite-design-system/components/flow-item/).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#flowEnabled)
     */
    flowEnabled: boolean;
    /**
     * The URL or data URI of an image used to represent the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#image)
     */
    image: string;
    /**
     * The panel's parent ListItem that represents a table in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#listItem)
     */
    listItem: TableListListItem;
    /**
     * Indicates if the panel's content is open and visible to the user.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#open)
     */
    open: boolean;
    /**
     * The title of the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#title)
     */
    title: string;
  }

  interface TableListListItemPanelProperties extends WidgetProperties {
    /**
     * Adds a CSS class used to style a node that represents the panel.
     *
     * @default null
     * @deprecated since version 4.30. Use {@link module:esri/widgets/TableList/ListItemPanel#icon icon}
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#className)
     */
    className?: string;
    /**
     * The content displayed in the ListItem panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#content)
     */
    content?: string | WidgetProperties | HTMLElement | (string | WidgetProperties | HTMLElement)[];
    /**
     * If `true`, disables the ListItem's panel so the user cannot open or interact with it.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates whether the panel's content should be rendered as a [Calcite Flow Item](https://developers.arcgis.com/calcite-design-system/components/flow-item/).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#flowEnabled)
     */
    flowEnabled?: boolean;
    /**
     * The URL or data URI of an image used to represent the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#image)
     */
    image?: string;
    /**
     * The panel's parent ListItem that represents a table in the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#listItem)
     */
    listItem?: TableListListItem;
    /**
     * Indicates if the panel's content is open and visible to the user.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#open)
     */
    open?: boolean;
    /**
     * The title of the panel.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#title)
     */
    title?: string;
  }

  /**
   * The content displayed in the ListItem panel.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItemPanel.html#TableListItemPanelContent)
   */
  export type TableListItemPanelContent = string | Widget | HTMLElement;

  export class TableListViewModel extends Accessor {
    /**
     * Provides the logic for the [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html)
     */

    constructor(properties?: TableListViewModelProperties);

    /**
     * Whether to provide an indication if a layer is being published in the [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled: boolean;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction: TableListViewModelListItemCreatedHandler;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all tables.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#listModeDisabled)
     */
    listModeDisabled: boolean;
    /**
     * A reference to the [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) containing the tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#map)
     */
    map: Map | WebMap;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#state)
     */
    readonly state: "loading" | "ready" | "disabled";
    /**
     * The collection of table [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html)s displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#tableItems)
     */
    readonly tableItems: Collection<TableListListItem>;

    /**
     * Triggers the [trigger-action](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#event-trigger-action) event and executes the given [action](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionButton.html) or [action toggle](https://developers.arcgis.com/javascript/latest/api-reference/esri-support-actions-ActionToggle.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#triggerAction)
     */
    triggerAction(action: ActionButton | ActionToggle, item: TableListListItem): void;

    on(name: "trigger-action", eventHandler: TableListViewModelTriggerActionEventHandler): IHandle;
  }

  interface TableListViewModelProperties {
    /**
     * Whether to provide an indication if a layer is being published in the [TableList](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html).
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#checkPublishStatusEnabled)
     */
    checkPublishStatusEnabled?: boolean;
    /**
     * Specifies a function that accesses each [ListItem](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-ListItem.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#listItemCreatedFunction)
     */
    listItemCreatedFunction?: TableListViewModelListItemCreatedHandler;
    /**
     * Specifies whether to ignore the [listMode](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#listMode) property of the layers to display all tables.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#listModeDisabled)
     */
    listModeDisabled?: boolean;
    /**
     * A reference to the [map](https://developers.arcgis.com/javascript/latest/api-reference/esri-Map.html) containing the tables.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList-TableListViewModel.html#map)
     */
    map?: MapProperties | WebMapProperties;
  }

  export type TableListViewModelListItemCreatedHandler = (event: any) => void;

  export interface TableListViewModelTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: TableListListItem;
  }

  export type TableListListItemCreatedHandler = (event: any) => void;

  export interface TableListTriggerActionEvent {
    action: ActionButton | ActionToggle;

    item: TableListListItem;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
   */
  export interface TableListVisibleElements {
    /**
     * **Since 4.29** Indicates whether to display a close button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    closeButton?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a collapse button in the header.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    collapseButton?: boolean;
    /**
     * **Since 4.25** Indicates whether layer load errors will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    errors?: boolean;
    /**
     * **Since 4.29** Indicates whether to display a filter input box when then number of list items is equal to or greater than the value set in [minFilterItems](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#minFilterItems), allowing users to filter tables by their title.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    filter?: boolean;
    /**
     * **Since 4.29** Determines whether the widget should be shown within its built-in [flow component](https://developers.arcgis.com/calcite-design-system/components/flow/) or if the flow component should be excluded.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    flow?: boolean;
    /**
     * **Since 4.29** Indicates whether to display the widget heading.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    heading?: boolean;
    /**
     * Indicates whether the status indicators will be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    statusIndicators?: boolean;
    /**
     * **Since 4.29** Indicates whether temporary table indicators will be displayed for tables with [layer.persistenceEnabled](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html#persistenceEnabled) set to `false`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TableList.html#VisibleElements)
     */
    temporaryTableIndicators?: boolean;
  }

  export class widgetsTimeSlider extends Widget {
    /**
     * The TimeSlider widget simplifies visualization of temporal data in your application.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html)
     */

    constructor(properties?: widgetsTimeSliderProperties);

    /**
     * Defines actions that will appear in a menu when the user clicks the ellipsis button ![timeSlider-actions-menu](https://developers.arcgis.com/javascript/latest/assets/img/apiref/widgets/timeslider/ellipsis.png) in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#actions)
     */
    actions: Collection<Action>;
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#disabled)
     */
    disabled: boolean;
    /**
     * Lists the specific locations on the timeline where handle(s) will snap to when manipulated.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#effectiveStops)
     */
    readonly effectiveStops: Date[];
    /**
     * The temporal extent of the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#fullTimeExtent)
     */
    fullTimeExtent: TimeExtent;
    /**
     * A function used to specify custom formatting and styling of the min, max, and extent labels of the TimeSlider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#labelFormatFunction)
     */
    labelFormatFunction: DateLabelFormatter;
    /**
     * Determines the layout used by the TimeSlider widget.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#layout)
     */
    layout: "auto" | "compact" | "wide";
    /**
     * When `true`, the time slider will play its animation in a loop.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#loop)
     */
    loop: boolean;
    /**
     * The time slider mode.
     *
     * @default "time-window"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#mode)
     */
    mode: "instant" | "time-window" | "cumulative-from-start" | "cumulative-from-end";
    /**
     * The time (in milliseconds) between animation steps.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#playRate)
     */
    playRate: number;
    /**
     * Defines specific locations on the time slider where thumbs will snap to when manipulated.
     *
     * @default { count : 10 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#stops)
     */
    stops: StopsByDates | StopsByCount | StopsByInterval;
    /**
     * When set, overrides the default TimeSlider ticks labelling system.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#tickConfigs)
     */
    tickConfigs: TickConfig[];
    /**
     * The current time extent of the time slider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * Shows/hides time in the display.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeVisible)
     */
    timeVisible: boolean;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeZone)
     */
    timeZone: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#viewModel)
     */
    viewModel: TimeSliderViewModel;

    /**
     * Incrementally moves the time extent forward one stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#next)
     */
    next(): void;
    /**
     * Initiates the time slider's temporal playback.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#play)
     */
    play(): void;
    /**
     * Incrementally moves the time extent back one stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#previous)
     */
    previous(): void;
    /**
     * Stops the time slider's temporal playback.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#stop)
     */
    stop(): void;
    /**
     * Updates the time slider widget definition in the provided [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#updateWebDocument)
     */
    updateWebDocument(document: WebMap | WebScene): void;

    on(name: "trigger-action", eventHandler: TimeSliderTriggerActionEventHandler): IHandle;

    /**
     * Extracts time slider settings from a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) if the webmap contains a time slider widget definition.
     *
     * @deprecated since 4.29. Use {@link module:esri/support/timeUtils#getTimeSliderSettingsFromWebDocument getTimeSliderSettingsFromWebDocument} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#getPropertiesFromWebMap)
     */
    static getPropertiesFromWebMap(webMap: WebMap, signal?: AbortSignal): Promise<any>;
  }

  interface widgetsTimeSliderProperties extends WidgetProperties {
    /**
     * Defines actions that will appear in a menu when the user clicks the ellipsis button ![timeSlider-actions-menu](https://developers.arcgis.com/javascript/latest/assets/img/apiref/widgets/timeslider/ellipsis.png) in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#actions)
     */
    actions?: CollectionProperties<Action>;
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#disabled)
     */
    disabled?: boolean;
    /**
     * The temporal extent of the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#fullTimeExtent)
     */
    fullTimeExtent?: TimeExtentProperties;
    /**
     * A function used to specify custom formatting and styling of the min, max, and extent labels of the TimeSlider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#labelFormatFunction)
     */
    labelFormatFunction?: DateLabelFormatter;
    /**
     * Determines the layout used by the TimeSlider widget.
     *
     * @default "auto"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#layout)
     */
    layout?: "auto" | "compact" | "wide";
    /**
     * When `true`, the time slider will play its animation in a loop.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#loop)
     */
    loop?: boolean;
    /**
     * The time slider mode.
     *
     * @default "time-window"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#mode)
     */
    mode?: "instant" | "time-window" | "cumulative-from-start" | "cumulative-from-end";
    /**
     * The time (in milliseconds) between animation steps.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#playRate)
     */
    playRate?: number;
    /**
     * Defines specific locations on the time slider where thumbs will snap to when manipulated.
     *
     * @default { count : 10 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#stops)
     */
    stops?: StopsByDates | StopsByCount | StopsByInterval;
    /**
     * When set, overrides the default TimeSlider ticks labelling system.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#tickConfigs)
     */
    tickConfigs?: TickConfig[];
    /**
     * The current time extent of the time slider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * Shows/hides time in the display.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeVisible)
     */
    timeVisible?: boolean;
    /**
     * Dates and times displayed in the widget will be displayed in this time zone.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#timeZone)
     */
    timeZone?: string;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#viewModel)
     */
    viewModel?: TimeSliderViewModelProperties;
  }

  export class TimeSliderViewModel extends Accessor {
    /**
     * Provides the logic for the [TimeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html)
     */

    constructor(properties?: TimeSliderViewModelProperties);

    /**
     * Defines actions that will appear in a menu when the user clicks the ellipsis button ![timeSlider-actions-menu](https://developers.arcgis.com/javascript/latest/assets/img/apiref/widgets/timeslider/ellipsis.png) in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#actions)
     */
    actions: Collection<Action>;
    /**
     * Defined specific locations on the timeline that the handles will snap to when manipulated.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#effectiveStops)
     */
    readonly effectiveStops: Date[];
    /**
     * The temporal extent of the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#fullTimeExtent)
     */
    fullTimeExtent: TimeExtent;
    /**
     * If animating, the time indicator(s) will restart if they reach the edge.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#loop)
     */
    loop: boolean;
    /**
     * The time slider mode.
     *
     * @default "time-window"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#mode)
     */
    mode: "instant" | "time-window" | "cumulative-from-start" | "cumulative-from-end";
    /**
     * The time (in milliseconds) between animation steps.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#playRate)
     */
    playRate: number;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "playing";
    /**
     * Defines specific locations on the time slider where thumbs will snap to when manipulated.
     *
     * @default { count : 10 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#stops)
     */
    stops: StopsByDates | StopsByCount | StopsByInterval;
    /**
     * The current time extent of the time slider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#timeExtent)
     */
    timeExtent: TimeExtent;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Incrementally moves the time extent forward one stop
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#next)
     */
    next(): void;
    /**
     * Initiates the time slider's temporal playback.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#play)
     */
    play(): void;
    /**
     * Incrementally moves the time extent back one stop.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#previous)
     */
    previous(): void;
    /**
     * Stops the time slider's temporal playback.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#stop)
     */
    stop(): void;
    /**
     * Updates the time slider widget definition in the provided [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) or [WebScene](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebScene.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#updateWebDocument)
     */
    updateWebDocument(document: WebMap | WebScene): void;

    on(name: "trigger-action", eventHandler: TimeSliderViewModelTriggerActionEventHandler): IHandle;

    /**
     * Extracts time slider settings from a [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) if the webmap contains a time slider widget definition.
     *
     * @deprecated since 4.29. Use {@link module:esri/support/timeUtils#getTimeSliderSettingsFromWebDocument getTimeSliderSettingsFromWebDocument} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#getPropertiesFromWebMap)
     */
    static getPropertiesFromWebMap(webMap: WebMap, signal?: AbortSignal): Promise<any>;
  }

  interface TimeSliderViewModelProperties {
    /**
     * Defines actions that will appear in a menu when the user clicks the ellipsis button ![timeSlider-actions-menu](https://developers.arcgis.com/javascript/latest/assets/img/apiref/widgets/timeslider/ellipsis.png) in the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#actions)
     */
    actions?: CollectionProperties<Action>;
    /**
     * The temporal extent of the entire slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#fullTimeExtent)
     */
    fullTimeExtent?: TimeExtentProperties;
    /**
     * If animating, the time indicator(s) will restart if they reach the edge.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#loop)
     */
    loop?: boolean;
    /**
     * The time slider mode.
     *
     * @default "time-window"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#mode)
     */
    mode?: "instant" | "time-window" | "cumulative-from-start" | "cumulative-from-end";
    /**
     * The time (in milliseconds) between animation steps.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#playRate)
     */
    playRate?: number;
    /**
     * Defines specific locations on the time slider where thumbs will snap to when manipulated.
     *
     * @default { count : 10 }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#stops)
     */
    stops?: StopsByDates | StopsByCount | StopsByInterval;
    /**
     * The current time extent of the time slider.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#timeExtent)
     */
    timeExtent?: TimeExtentProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  /**
   * Definition of an action that can be assigned to the [actions](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#actions) property on the [TimeSlider](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#actions) or the [TimeSliderViewModel](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#actions).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#action)
   */
  export interface Action {
    /**
     * The name of the icon to display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#action)
     */
    icon: string;
    /**
     * Unique identifier of the action.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#action)
     */
    id: string;
    /**
     * The title of the action that will be displayed in the menu.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider-TimeSliderViewModel.html#action)
     */
    title: string;
  }

  export interface TimeSliderViewModelTriggerActionEvent {
    action: Action;
  }

  export type DateLabelFormatter = (
    value: Date | Date[],
    type?: "min" | "max" | "extent",
    element?: HTMLElement,
    layout?: "compact" | "wide",
  ) => void;

  /**
   * Divides the time slider's [fullTimeExtent](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#fullTimeExtent) into equal parts.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByCount)
   */
  export interface StopsByCount {
    /**
     * Number of evenly spaced divisions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByCount)
     */
    count: number;
    /**
     * The time period to divide.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByCount)
     */
    timeExtent?: TimeExtent;
  }

  /**
   * Specifies an array of dates for the time slider widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByDates)
   */
  export interface StopsByDates {
    /**
     * Array of dates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByDates)
     */
    dates: Date[];
  }

  /**
   * Defines regularly spaced stops on the time slider from a [TimeInterval](https://developers.arcgis.com/javascript/latest/api-reference/esri-time-TimeInterval.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByInterval)
   */
  export interface StopsByInterval {
    /**
     * Specifies a granularity of temporal data and allows you to visualize the data at specified intervals.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByInterval)
     */
    interval: TimeInterval;
    /**
     * A period of time with definitive start and end dates.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeSlider.html#StopsByInterval)
     */
    timeExtent?: TimeExtent;
  }

  export interface TimeSliderTriggerActionEvent {
    action: Action;
  }

  export class TimeZoneLabel extends Widget {
    /**
     * TimeZoneLabel is widget for displaying the current [time zone](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#timeZone) of the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html)
     */

    constructor(properties?: TimeZoneLabelProperties);

    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#disabled)
     */
    disabled: boolean;
    /**
     * The direction the widget will expand.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#expandDirection)
     */
    expandDirection: "start" | "end";
    /**
     * The expanded state of the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#expanded)
     */
    expanded: boolean;
    /**
     * A reference to a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#view)
     */
    view: MapView;
  }

  interface TimeZoneLabelProperties extends WidgetProperties {
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#disabled)
     */
    disabled?: boolean;
    /**
     * The direction the widget will expand.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#expandDirection)
     */
    expandDirection?: "start" | "end";
    /**
     * The expanded state of the widget.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#expanded)
     */
    expanded?: boolean;
    /**
     * A reference to a [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-TimeZoneLabel.html#view)
     */
    view?: MapViewProperties;
  }

  export interface Track extends Widget, GoTo {}

  export class Track {
    /**
     * Provides a simple button that animates the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to the user's location when clicked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html)
     */

    constructor(properties?: TrackProperties);

    /**
     * An object used for setting optional position parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#geolocationOptions)
     */
    geolocationOptions: any;
    /**
     * Indicates whether the widget will automatically navigate the view to the user's position when a geolocation result is found.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#goToLocationEnabled)
     */
    goToLocationEnabled: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#graphic)
     */
    graphic: Graphic;
    /**
     * Indicates whether the widget will automatically rotate to the device heading based on the Geolocation APIs [`GeolocationCoordinates.heading`](https://developer.mozilla.org/en-US/docs/Web/API/GeolocationCoordinates/heading) property.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#rotationEnabled)
     */
    rotationEnabled: boolean;
    /**
     * Indicates the [scale](https://developers.arcgis.com/documentation/mapping-apis-and-services/reference/zoom-levels-and-scale/) to set on the view when navigating to the position of the geolocated result, after a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#scale)
     */
    scale: number;
    /**
     * Indicates whether the widget is watching for new positions.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#tracking)
     */
    readonly tracking: boolean;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#view)
     */
    view: MapView | SceneView;
    /**
     * The viewModel for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#viewModel)
     */
    viewModel: TrackViewModel;

    /**
     * When executed, the widget will start [tracking](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#tracking) the user's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#start)
     */
    start(): void;
    /**
     * Stops tracking the user's location when executed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#stop)
     */
    stop(): void;

    on(name: "track", eventHandler: TrackTrackEventHandler): IHandle;

    on(name: "track-error", eventHandler: TrackTrackErrorEventHandler): IHandle;
  }

  interface TrackProperties extends WidgetProperties, GoToProperties {
    /**
     * An object used for setting optional position parameters.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#geolocationOptions)
     */
    geolocationOptions?: any;
    /**
     * Indicates whether the widget will automatically navigate the view to the user's position when a geolocation result is found.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#goToLocationEnabled)
     */
    goToLocationEnabled?: boolean;
    /**
     * The graphic used to show the user's location on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#graphic)
     */
    graphic?: GraphicProperties;
    /**
     * Indicates whether the widget will automatically rotate to the device heading based on the Geolocation APIs [`GeolocationCoordinates.heading`](https://developer.mozilla.org/en-US/docs/Web/API/GeolocationCoordinates/heading) property.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#rotationEnabled)
     */
    rotationEnabled?: boolean;
    /**
     * Indicates the [scale](https://developers.arcgis.com/documentation/mapping-apis-and-services/reference/zoom-levels-and-scale/) to set on the view when navigating to the position of the geolocated result, after a location is returned from the [track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#event-track) event.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#scale)
     */
    scale?: number;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The viewModel for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html#viewModel)
     */
    viewModel?: TrackViewModelProperties;
  }

  export interface TrackViewModel extends Accessor, Evented, GeolocationPositioning, GoTo {}

  export class TrackViewModel {
    /**
     * Provides the logic for the [Track](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track.html) widget, which animates the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html) to the user's location when clicked and tracks it as the location is updated.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html)
     */

    constructor(properties?: TrackViewModelProperties);

    /**
     * Error that caused the last [track-error](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#event:track-error) event to fire.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#error)
     */
    error: any | Error;
    /**
     * A function that is used as an expression to evaluate geolocation points, and returns a boolean value.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#positionFilterFunction)
     */
    positionFilterFunction: Function;
    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#state)
     */
    readonly state: "disabled" | "ready" | "tracking" | "waiting" | "feature-unsupported" | "error";
    /**
     * Indicates whether new positions are being watched.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#tracking)
     */
    readonly tracking: boolean;

    /**
     * When executed, [tracking](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#tracking) starts at the user's location.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#start)
     */
    start(): void;
    /**
     * Stops tracking the user's location when executed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#stop)
     */
    stop(): void;

    on(name: "track", eventHandler: TrackViewModelTrackEventHandler): IHandle;

    on(name: "track-error", eventHandler: TrackViewModelTrackErrorEventHandler): IHandle;
  }

  interface TrackViewModelProperties extends GeolocationPositioningProperties, GoToProperties {
    /**
     * Error that caused the last [track-error](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#event:track-error) event to fire.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#error)
     */
    error?: any | Error;
    /**
     * A function that is used as an expression to evaluate geolocation points, and returns a boolean value.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Track-TrackViewModel.html#positionFilterFunction)
     */
    positionFilterFunction?: Function;
  }

  export interface TrackViewModelTrackErrorEvent {
    error: any | Error;
  }

  export interface TrackViewModelTrackEvent {
    position: any;
  }

  export interface TrackTrackErrorEvent {
    error: Error;
  }

  export interface TrackTrackEvent {
    position: any;
  }

  export class UtilityNetworkAssociations extends Widget {
    /**
     * The UtilityNetworkAssociations widget class, functioning as a part of the ArcGIS Maps SDK for JavaScript, simplifies the management and manipulation of associations within a utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html)
     */

    constructor(properties?: UtilityNetworkAssociationsProperties);

    /**
     * Indicates whether to show a toggle to automatically show associations every time the map extent changes, or an action button to show associations within the current map extent on demand.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#autoRefreshAssociations)
     */
    autoRefreshAssociations: boolean;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for connectivity associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#connectivityAssociationsLineSymbol)
     */
    connectivityAssociationsLineSymbol: SimpleLineSymbol;
    /**
     * Indicates whether to query and display connectivity associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#includeConnectivityAssociations)
     */
    includeConnectivityAssociations: boolean;
    /**
     * Indicates whether to query and display structural attachment associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#includeStructuralAttachmentAssociations)
     */
    includeStructuralAttachmentAssociations: boolean;
    /**
     * The maximum number of associations that can be returned from the server.
     *
     * @default 250
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociations)
     */
    maxAllowableAssociations: number;
    /**
     * The maximum value or upper bound of the Maximum allowable associations slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderMax)
     */
    maxAllowableAssociationsSliderMax: number;
    /**
     * The minimum value or lower bound of the Maximum allowable associations slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderMin)
     */
    maxAllowableAssociationsSliderMin: number;
    /**
     * Specifies the interval to move the maximum allowable associations slider with the up, or down keys.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderStep)
     */
    maxAllowableAssociationsSliderStep: number;
    /**
     * Indicates whether to show arrows for connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showArrowsConnectivity)
     */
    showArrowsConnectivity: boolean;
    /**
     * Indicates whether to show arrows for structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showArrowsStructuralAttachment)
     */
    showArrowsStructuralAttachment: boolean;
    /**
     * When `autoRefreshAssociations` is `true`, indicates whether to automatically show associations every time the current map extent changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showAssociationsEnabled)
     */
    showAssociationsEnabled: boolean;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for structural attachment associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#structuralAttachmentAssociationsLineSymbol)
     */
    structuralAttachmentAssociationsLineSymbol: SimpleLineSymbol;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view associated with the UtilityNetworkAssociations widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#viewModel)
     */
    viewModel: UtilityNetworkAssociationsViewModel;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#visibleElements)
     */
    visibleElements: UtilityNetworkAssociationsVisibleElements;
  }

  interface UtilityNetworkAssociationsProperties extends WidgetProperties {
    /**
     * Indicates whether to show a toggle to automatically show associations every time the map extent changes, or an action button to show associations within the current map extent on demand.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#autoRefreshAssociations)
     */
    autoRefreshAssociations?: boolean;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for connectivity associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#connectivityAssociationsLineSymbol)
     */
    connectivityAssociationsLineSymbol?: SimpleLineSymbolProperties;
    /**
     * Indicates whether to query and display connectivity associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#includeConnectivityAssociations)
     */
    includeConnectivityAssociations?: boolean;
    /**
     * Indicates whether to query and display structural attachment associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#includeStructuralAttachmentAssociations)
     */
    includeStructuralAttachmentAssociations?: boolean;
    /**
     * The maximum number of associations that can be returned from the server.
     *
     * @default 250
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociations)
     */
    maxAllowableAssociations?: number;
    /**
     * The maximum value or upper bound of the Maximum allowable associations slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderMax)
     */
    maxAllowableAssociationsSliderMax?: number;
    /**
     * The minimum value or lower bound of the Maximum allowable associations slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderMin)
     */
    maxAllowableAssociationsSliderMin?: number;
    /**
     * Specifies the interval to move the maximum allowable associations slider with the up, or down keys.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociationsSliderStep)
     */
    maxAllowableAssociationsSliderStep?: number;
    /**
     * Indicates whether to show arrows for connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showArrowsConnectivity)
     */
    showArrowsConnectivity?: boolean;
    /**
     * Indicates whether to show arrows for structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showArrowsStructuralAttachment)
     */
    showArrowsStructuralAttachment?: boolean;
    /**
     * When `autoRefreshAssociations` is `true`, indicates whether to automatically show associations every time the current map extent changes.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#showAssociationsEnabled)
     */
    showAssociationsEnabled?: boolean;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for structural attachment associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#structuralAttachmentAssociationsLineSymbol)
     */
    structuralAttachmentAssociationsLineSymbol?: SimpleLineSymbolProperties;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#utilityNetwork)
     */
    utilityNetwork?: UtilityNetworkProperties;
    /**
     * The view associated with the UtilityNetworkAssociations widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#viewModel)
     */
    viewModel?: UtilityNetworkAssociationsViewModelProperties;
    /**
     * The visible elements that are displayed within the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#visibleElements)
     */
    visibleElements?: UtilityNetworkAssociationsVisibleElements;
  }

  export class UtilityNetworkAssociationsViewModel extends Accessor {
    /**
     * Provides the logic for the [UtilityNetworkAssociations](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html)
     */

    constructor(properties?: UtilityNetworkAssociationsViewModelProperties);

    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for connectivity associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#connectivityAssociationsLineSymbol)
     */
    connectivityAssociationsLineSymbol: SimpleLineSymbol;
    /**
     * Indicates whether to query and display connectivity associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#includeConnectivityAssociations)
     */
    includeConnectivityAssociations: boolean;
    /**
     * Indicates whether to query and display structural attachment associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#includeStructuralAttachmentAssociations)
     */
    includeStructuralAttachmentAssociations: boolean;
    /**
     * The maximum number of associations that can be returned from the server.
     *
     * @default 250
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#maxAllowableAssociations)
     */
    maxAllowableAssociations: number;
    /**
     * Indicates whether to show arrows for connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#showArrowsConnectivity)
     */
    showArrowsConnectivity: boolean;
    /**
     * Indicates whether to show arrows for structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#showArrowsStructuralAttachment)
     */
    showArrowsStructuralAttachment: boolean;
    /**
     * The view model's state.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#state)
     */
    readonly state: "disabled" | "loading" | "ready" | "executing" | "warning";
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for structural attachment associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#structuralAttachmentAssociationsLineSymbol)
     */
    structuralAttachmentAssociationsLineSymbol: SimpleLineSymbol;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view associated with the UtilityNetworkAssociations widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#view)
     */
    view: MapView;

    /**
     * Removes all associations from the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#removeAssociations)
     */
    removeAssociations(): void;
    /**
     * Queries associations within the current map extent.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#showAssociations)
     */
    showAssociations(): Promise<void>;
  }

  interface UtilityNetworkAssociationsViewModelProperties {
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for connectivity associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#connectivityAssociationsLineSymbol)
     */
    connectivityAssociationsLineSymbol?: SimpleLineSymbolProperties;
    /**
     * Indicates whether to query and display connectivity associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#includeConnectivityAssociations)
     */
    includeConnectivityAssociations?: boolean;
    /**
     * Indicates whether to query and display structural attachment associations.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#includeStructuralAttachmentAssociations)
     */
    includeStructuralAttachmentAssociations?: boolean;
    /**
     * The maximum number of associations that can be returned from the server.
     *
     * @default 250
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#maxAllowableAssociations)
     */
    maxAllowableAssociations?: number;
    /**
     * Indicates whether to show arrows for connectivity associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#showArrowsConnectivity)
     */
    showArrowsConnectivity?: boolean;
    /**
     * Indicates whether to show arrows for structural attachment associations.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#showArrowsStructuralAttachment)
     */
    showArrowsStructuralAttachment?: boolean;
    /**
     * A [SimpleLineSymbol](https://developers.arcgis.com/javascript/latest/api-reference/esri-symbols-SimpleLineSymbol.html) used for representing the polyline geometry that is being drawn for structural attachment associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#structuralAttachmentAssociationsLineSymbol)
     */
    structuralAttachmentAssociationsLineSymbol?: SimpleLineSymbolProperties;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#utilityNetwork)
     */
    utilityNetwork?: UtilityNetworkProperties;
    /**
     * The view associated with the UtilityNetworkAssociations widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations-UtilityNetworkAssociationsViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
   */
  export interface UtilityNetworkAssociationsVisibleElements {
    /**
     * The connectivity associations' line symbol settings that can be further configured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    connectivityAssociationsSettings?: VisibleElementsConnectivityAssociationsSettings;
    /**
     * Indicates whether to display a slider that adjusts the [maxAllowableAssociations](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#maxAllowableAssociations).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    maxAllowableAssociationsSlider: boolean;
    /**
     * The structural attachment associations' line symbol settings that can be further configured.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    structuralAttachmentAssociationsSettings?: VisibleElementsStructuralAttachmentAssociationsSettings;
  }

  export interface VisibleElementsConnectivityAssociationsSettings {
    /**
     * Indicates whether to display a toggle to hide / show arrows for connectivity associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    arrowsToggle?: boolean;
    /**
     * Indicates whether to display the cap input to update the line symbol's cap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    capSelect?: boolean;
    /**
     * Indicates whether to display the color picker to update the line symbol's color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    colorPicker?: boolean;
    /**
     * Indicates whether to display the style input to update the line symbol's style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    stylePicker?: boolean;
    /**
     * Indicates whether to display the width input to update the line symbol's width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    widthInput?: boolean;
  }

  export interface VisibleElementsStructuralAttachmentAssociationsSettings {
    /**
     * Indicates whether to display a toggle to hide / show arrows for structural attachment associations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    arrowsToggle?: boolean;
    /**
     * Indicates whether to display the cap input to update the line symbol's cap style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    capSelect?: boolean;
    /**
     * Indicates whether to display the color picker to update the line symbol's color.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    colorPicker?: boolean;
    /**
     * Indicates whether to display the style input to update the line symbol's style.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    stylePicker?: boolean;
    /**
     * Indicates whether to display the width input to update the line symbol's width.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkAssociations.html#VisibleElements)
     */
    widthInput?: boolean;
  }

  export class UtilityNetworkTrace extends Widget {
    /**
     * The UtilityNetworkTrace widget provides a way to run traces in a Utility Network based on connectivity or traversability from set input flags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html)
     */

    constructor(properties?: UtilityNetworkTraceProperties);

    /**
     * The default color to assign the aggregated geometry of a trace result.
     *
     * @default {
     * color: [255, 255, 0, 0.6],
     * haloOpacity: 0.9,
     * fillOpacity: 0.2,
     * hex: "#FFFF00"
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#defaultGraphicColor)
     */
    defaultGraphicColor: GraphicColor;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#disabled)
     */
    disabled: boolean;
    /**
     * When `true`, provides the ability to show the convex hull or buffer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#enableResultArea)
     */
    enableResultArea: boolean;
    /**
     * An array of map points to load into the widget to lookup flags.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#flags)
     */
    flags: FlagProperty[];
    /**
     * The Geodatabase version to pass into the trace.
     *
     * @default "sde.DEFAULT"
     * @deprecated since 4.31, gdbVersion will be removed and the gdbVersion of the {@link module:esri/networks/UtilityNetwork} will be consumed directly.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * Custom labels, descriptions, and symbol for the input [flags](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#flags).
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#inputSettings)
     */
    inputSettings: InputSetting[];
    /**
     * The properties to determine the size and color of the result area convex hull or buffer, and determines if it displays on the map when the trace completes.
     *
     * @default {
     *  type: "convexhull",
     *  distance: 10,
     *  unit: "meters",
     *  areaUnit: "square-meters",
     *  color: {
     *    color: [255, 165, 0, 0.5],
     *    haloOpacity: 0.9,
     *    fillOpacity: 0.2,
     *    hex: "#ffa500"
     *  },
     *  show: false
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#resultAreaProperties)
     */
    resultAreaProperties: ResultAreaPropertiesExtend;
    /**
     * A list of global Ids of traces to select on load
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#selectedTraces)
     */
    selectedTraces: string[];
    /**
     * When true, the utility network elements are selected in the view when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#selectOnComplete)
     */
    selectOnComplete: boolean;
    /**
     * When true, a graphic layer is added to the view to highlight the utility network elements when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#showGraphicsOnComplete)
     */
    showGraphicsOnComplete: boolean;
    /**
     * Determines whether to show list of selection attributes
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#showSelectionAttributes)
     */
    showSelectionAttributes: boolean;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#viewModel)
     */
    viewModel: UtilityNetworkTraceViewModel;

    /**
     * Checks if the requirements to execute a trace are met.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#checkCanTrace)
     */
    checkCanTrace(): Promise<void>;
    /**
     * Prompts to clear all input flags, selected trace types, and the trace results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#confirmReset)
     */
    confirmReset(): void;

    on(name: "add-flag", eventHandler: UtilityNetworkTraceAddFlagEventHandler): IHandle;

    on(name: "add-flag-complete", eventHandler: UtilityNetworkTraceAddFlagCompleteEventHandler): IHandle;

    on(name: "add-flag-error", eventHandler: UtilityNetworkTraceAddFlagErrorEventHandler): IHandle;

    on(name: "add-result-area", eventHandler: UtilityNetworkTraceAddResultAreaEventHandler): IHandle;

    on(name: "create-result-area", eventHandler: UtilityNetworkTraceCreateResultAreaEventHandler): IHandle;

    on(name: "remove-result-area", eventHandler: UtilityNetworkTraceRemoveResultAreaEventHandler): IHandle;
  }

  interface UtilityNetworkTraceProperties extends WidgetProperties {
    /**
     * The default color to assign the aggregated geometry of a trace result.
     *
     * @default {
     * color: [255, 255, 0, 0.6],
     * haloOpacity: 0.9,
     * fillOpacity: 0.2,
     * hex: "#FFFF00"
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#defaultGraphicColor)
     */
    defaultGraphicColor?: GraphicColor;
    /**
     * When true, the widget is visually withdrawn and cannot be interacted with.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#disabled)
     */
    disabled?: boolean;
    /**
     * When `true`, provides the ability to show the convex hull or buffer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#enableResultArea)
     */
    enableResultArea?: boolean;
    /**
     * An array of map points to load into the widget to lookup flags.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#flags)
     */
    flags?: FlagProperty[];
    /**
     * The Geodatabase version to pass into the trace.
     *
     * @default "sde.DEFAULT"
     * @deprecated since 4.31, gdbVersion will be removed and the gdbVersion of the {@link module:esri/networks/UtilityNetwork} will be consumed directly.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * Custom labels, descriptions, and symbol for the input [flags](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#flags).
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#inputSettings)
     */
    inputSettings?: InputSetting[];
    /**
     * The properties to determine the size and color of the result area convex hull or buffer, and determines if it displays on the map when the trace completes.
     *
     * @default {
     *  type: "convexhull",
     *  distance: 10,
     *  unit: "meters",
     *  areaUnit: "square-meters",
     *  color: {
     *    color: [255, 165, 0, 0.5],
     *    haloOpacity: 0.9,
     *    fillOpacity: 0.2,
     *    hex: "#ffa500"
     *  },
     *  show: false
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#resultAreaProperties)
     */
    resultAreaProperties?: ResultAreaPropertiesExtend;
    /**
     * A list of global Ids of traces to select on load
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#selectedTraces)
     */
    selectedTraces?: string[];
    /**
     * When true, the utility network elements are selected in the view when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#selectOnComplete)
     */
    selectOnComplete?: boolean;
    /**
     * When true, a graphic layer is added to the view to highlight the utility network elements when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#showGraphicsOnComplete)
     */
    showGraphicsOnComplete?: boolean;
    /**
     * Determines whether to show list of selection attributes
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#showSelectionAttributes)
     */
    showSelectionAttributes?: boolean;
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#utilityNetwork)
     */
    utilityNetwork?: UtilityNetworkProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#viewModel)
     */
    viewModel?: UtilityNetworkTraceViewModelProperties;
  }

  export interface UtilityNetworkTraceViewModel extends Accessor, GoTo {}

  export class UtilityNetworkTraceViewModel {
    /**
     * Provides the logic for the [UtilityNetworkTrace](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html)
     */

    constructor(properties?: UtilityNetworkTraceViewModelProperties);

    /**
     * The default color to assign the aggregated geometry of a trace result.
     *
     * @default {
     * color: [255, 255, 0, 0.6],
     * haloOpacity: 0.9,
     * fillOpacity: 0.2,
     * hex: "#FFFF00"
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#defaultGraphicColor)
     */
    defaultGraphicColor: GraphicColor;
    /**
     * When true, provides the ability to show the convex hull or buffer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#enableResultArea)
     */
    enableResultArea: boolean;
    /**
     * An array of map points to load into the widget to lookup flags.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#flags)
     */
    flags: FlagProperty[];
    /**
     * The Geodatabase version to pass into the trace.
     *
     * @default "sde.DEFAULT"
     * @deprecated since 4.31, gdbVersion will be removed and the gdbVersion of the {@link module:esri/networks/UtilityNetwork} will be consumed directly.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#gdbVersion)
     */
    gdbVersion: string;
    /**
     * The properties to determine the size and color of the result area convex hull or buffer, and determines if it displays on the map when the trace completes.
     *
     * @default {
     *  type: "convexhull",
     *  distance: 10,
     *  unit: "meters",
     *  areaUnit: "square-meters",
     *  color: {
     *    color: [255, 165, 0, 0.5],
     *    haloOpacity: 0.9,
     *    fillOpacity: 0.2,
     *    hex: "#ffa500"
     *  },
     *  show: false
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#resultAreaProperties)
     */
    resultAreaProperties: ResultAreaPropertiesExtend;
    /**
     * An array of global Ids of traces to select on initial load.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectedTraces)
     */
    selectedTraces: string[];
    /**
     * When true, the utility network elements are selected in the view when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectOnComplete)
     */
    selectOnComplete: boolean;
    /**
     * When true, a graphic layer is added to the view to highlight the utility network elements when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#showGraphicsOnComplete)
     */
    showGraphicsOnComplete: boolean;
    /**
     * Determines whether to show the list of selected features from completed traces.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#showSelectionAttributes)
     */
    showSelectionAttributes: boolean;
    /**
     * The view model's state.
     *
     * @default "ready"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#state)
     */
    readonly state: "loading" | "ready";
    /**
     * Stores the result of completed traces.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#traceResults)
     */
    traceResults: TraceResultExtend[];
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#view)
     */
    view: MapView;

    /**
     * Adds a flag point graphic for the click event on the view if there is a feature to query.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#addFlagByHit)
     */
    addFlagByHit(type: string): Promise<boolean>;
    /**
     * Creates a graphic from the trace result and adds it to the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#addResultAreaToMap)
     */
    addResultAreaToMap(trace: TraceResultExtend): void;
    /**
     * Create a graphic on the view’s [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#graphics) for the aggregated results of all the features returned by the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#addResultGraphicToView)
     */
    addResultGraphicToView(trace: TraceResultExtend, color: GraphicColor): Promise<any>;
    /**
     * Adds the selected terminal to a flag point.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#addTerminal)
     */
    addTerminal(selectedTerminal: string, feature: FlagProperty): void;
    /**
     * Get all parameters for the trace type selected to be run before executing the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#callTrace)
     */
    callTrace(): Promise<boolean>;
    /**
     * Change the graphic color for the aggregated results of a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#changeResultGraphicColor)
     */
    changeResultGraphicColor(color: GraphicColor, trace: TraceResultExtend): void;
    /**
     * May be used to verify if all requirements are met to execute a trace (at least 1 starting point and at least 1 trace type selected).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#checkCanTrace)
     */
    checkCanTrace(): ValidSetup;
    /**
     * Indicates if any selection exists on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#checkSelectionExist)
     */
    checkSelectionExist(): boolean;
    /**
     * Removes a specific trace from the results.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#clearResult)
     */
    clearResult(trace: TraceItem): void;
    /**
     * Creates a buffer or convex hull graphic of the trace result.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#createResultAreaGraphic)
     */
    createResultAreaGraphic(traceResults: TraceResultExtend): Promise<Graphic>;
    /**
     * Executes the trace and returns all trace results as graphics and/or feature selections and functions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#executeTrace)
     */
    executeTrace(traceItem: TraceItem, url: string, params: TraceParameters): Promise<TraceResultExtend>;
    /**
     * Gets the valid edge and junction layers within the view that are part of the Utility Network and can be used for placing flags in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#getValidSources)
     */
    getValidSources(): (EdgeSourceJSON | JunctionSourceJSON)[];
    /**
     * Reads the web map and loads the [UtilityNetwork](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html) if it exists.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#loadUtilityNetwork)
     */
    loadUtilityNetwork(): Promise<UtilityNetwork>;
    /**
     * Enables or disables the filter barrier setting on barrier flags.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#manageFilterBarrier)
     */
    manageFilterBarrier(status: boolean, feature: FlagProperty): void;
    /**
     * Is used to merge the feature selections in the layer views when multiple traces are run.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#mergeSelection)
     */
    mergeSelection(status: boolean, trace: TraceItem): void;
    /**
     * Query the layers by ObjectID for more attributes not present on the trace result elements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#queryFeaturesById)
     */
    queryFeaturesById(dataItems: NetworkElement[]): Promise<FeatureSetInfo[]>;
    /**
     * Takes the result of a hit test to lookup the asset to create a flag for the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#queryFlagByHitTest)
     */
    queryFlagByHitTest(event: CreateToolEventInfo, flagType: string): Promise<boolean>;
    /**
     * Removes all the graphics from the result area graphic layer
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeAllResultAreaGraphics)
     */
    removeAllResultAreaGraphics(): void;
    /**
     * Removes the selected flag from the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeFlag)
     */
    removeFlag(flag: FlagProperty): void;
    /**
     * Removes the result area graphic from the `Map`.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeResultAreaFromMap)
     */
    removeResultAreaFromMap(trace: TraceResultExtend): void;
    /**
     * Removes a specific trace result graphic from the view’s [graphics](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html#graphics).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeResultGraphicFromView)
     */
    removeResultGraphicFromView(trace: TraceResultExtend): void;
    /**
     * Removes the selection from the layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeSelection)
     */
    removeSelection(): void;
    /**
     * Removes the selected terminal from the flag.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#removeTerminal)
     */
    removeTerminal(selectedTerminal: string, feature: FlagProperty): void;
    /**
     * Clears all inputs (flags, trace types) and all results (selections, graphics) in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#reset)
     */
    reset(): void;
    /**
     * Performs a selection on a layer view based on a list of ObjectIDs.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectFeaturesById)
     */
    selectFeaturesById(resultSet: NetworkElement[]): void;
    /**
     * Loops through the trace result elements to group them by network source id and selects them on the layer view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectResults)
     */
    selectResults(resultSet: NetworkElement[]): void;
    /**
     * Set the trace type to be run from the available trace configurations in the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectTraces)
     */
    selectTraces(state: boolean, traceId: string): void;
    /**
     * Preset a trace type to be run from the available [trace configurations](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-UtilityNetwork.html#sharedNamedTraceConfigurations) in the [WebMap](https://developers.arcgis.com/javascript/latest/api-reference/esri-WebMap.html) when the widget loads.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectTracesOnLoad)
     */
    selectTracesOnLoad(): void;
    /**
     * Zoom to a flag's feature or a result feature in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#zoomToAsset)
     */
    zoomToAsset(geometry: GoToTarget2D | GoToTarget3D): void;
  }

  interface UtilityNetworkTraceViewModelProperties extends GoToProperties {
    /**
     * The default color to assign the aggregated geometry of a trace result.
     *
     * @default {
     * color: [255, 255, 0, 0.6],
     * haloOpacity: 0.9,
     * fillOpacity: 0.2,
     * hex: "#FFFF00"
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#defaultGraphicColor)
     */
    defaultGraphicColor?: GraphicColor;
    /**
     * When true, provides the ability to show the convex hull or buffer.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#enableResultArea)
     */
    enableResultArea?: boolean;
    /**
     * An array of map points to load into the widget to lookup flags.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#flags)
     */
    flags?: FlagProperty[];
    /**
     * The Geodatabase version to pass into the trace.
     *
     * @default "sde.DEFAULT"
     * @deprecated since 4.31, gdbVersion will be removed and the gdbVersion of the {@link module:esri/networks/UtilityNetwork} will be consumed directly.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#gdbVersion)
     */
    gdbVersion?: string;
    /**
     * The properties to determine the size and color of the result area convex hull or buffer, and determines if it displays on the map when the trace completes.
     *
     * @default {
     *  type: "convexhull",
     *  distance: 10,
     *  unit: "meters",
     *  areaUnit: "square-meters",
     *  color: {
     *    color: [255, 165, 0, 0.5],
     *    haloOpacity: 0.9,
     *    fillOpacity: 0.2,
     *    hex: "#ffa500"
     *  },
     *  show: false
     * }
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#resultAreaProperties)
     */
    resultAreaProperties?: ResultAreaPropertiesExtend;
    /**
     * An array of global Ids of traces to select on initial load.
     *
     * @default []
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectedTraces)
     */
    selectedTraces?: string[];
    /**
     * When true, the utility network elements are selected in the view when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#selectOnComplete)
     */
    selectOnComplete?: boolean;
    /**
     * When true, a graphic layer is added to the view to highlight the utility network elements when traces are completed.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#showGraphicsOnComplete)
     */
    showGraphicsOnComplete?: boolean;
    /**
     * Determines whether to show the list of selected features from completed traces.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#showSelectionAttributes)
     */
    showSelectionAttributes?: boolean;
    /**
     * Stores the result of completed traces.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#traceResults)
     */
    traceResults?: TraceResultExtend[];
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#utilityNetwork)
     */
    utilityNetwork?: UtilityNetworkProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  /**
   * AssetGroupJSON represents the [asset group](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/utility-feature-classification.htm#GUID-A4CCEB96-C4F8-459B-A763-FF6A5791F663).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetGroupJSON)
   */
  export interface AssetGroupJSON {
    /**
     * The asset group code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetGroupJSON)
     */
    assetGroupCode: number;
    /**
     * The asset group name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetGroupJSON)
     */
    assetGroupName: string;
    /**
     * The asset type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetGroupJSON)
     */
    assetTypes: AssetTypeJSON[];
  }

  /**
   * AssetTypeJSON represents [asset type](https://pro.arcgis.com/en/pro-app/latest/help/data/utility-network/utility-feature-classification.htm#GUID-A4CCEB96-C4F8-459B-A763-FF6A5791F663).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetTypeJSON)
   */
  export interface AssetTypeJSON {
    /**
     * The asset type code.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetTypeJSON)
     */
    assetTypeCode: number;
    /**
     * The asset type name.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetTypeJSON)
     */
    assetTypeName: string;
    /**
     * The id of the terminal configuration.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#AssetTypeJSON)
     */
    terminalConfigurationId: number;
  }

  /**
   * DisplayField represents the attribute field used as a display label for flags and selected features.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#DisplayField)
   */
  export interface DisplayField {
    /**
     * The feature's display field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#DisplayField)
     */
    field: string;
    /**
     * The value of the field.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#DisplayField)
     */
    value: string;
  }

  /**
   * EdgeSourceJSON represents the line layers that participate in the utility network service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#EdgeSourceJSON)
   */
  export interface EdgeSourceJSON {
    /**
     * The layer id of the line layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#EdgeSourceJSON)
     */
    layerId: number;
    /**
     * The asset group attribute values for the line layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#EdgeSourceJSON)
     */
    assetGroups: AssetGroupJSON[];
    /**
     * The network source id of the line layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#EdgeSourceJSON)
     */
    sourceId: number;
  }

  /**
   * FeatureSetInfo represents the information for the results in the featureset.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FeatureSetInfo)
   */
  export interface FeatureSetInfo {
    /**
     * The [FeatureLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html) of the dataset.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FeatureSetInfo)
     */
    layer: FeatureLayer;
    /**
     * The [FeatureSet](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-support-FeatureSet.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FeatureSetInfo)
     */
    featureSet: FeatureSet;
  }

  /**
   * FlagProperty represents the properties to define each flag point (starting points and barriers).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
   */
  export interface FlagProperty {
    /**
     * The available [terminals](https://developers.arcgis.com/javascript/latest/api-reference/esri-networks-support-Terminal.html) to define the permissible paths based on the terminal configurations.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    allTerminals: TerminalConfiguration;
    /**
     * All the information returned by the hitTest.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    details: any;
    /**
     * The display field of the flag.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    displayField: DisplayField;
    /**
     * The id of the flag.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    id: number;
    /**
     * The flag graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    mapGraphic: Graphic;
    /**
     * The geometry point of the flag.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    mapPoint: Point;
    /**
     * The terminals that are selected for the flag (this can be the default terminal or terminals selected by the end-user).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#FlagProperty)
     */
    selectedTerminals: number[];
    /**
     * The flag type being set.
     *
     * [Read more...](global.html#type)
     */
    type: string;
  }

  /**
   * GraphicColor represents the color for the trace result graphic in the graphics layer.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#GraphicColor)
   */
  export interface GraphicColor {
    /**
     * The color of the trace result graphic in the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#GraphicColor)
     */
    color: number[];
    /**
     * The opacity of the graphic symbol's halo.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#GraphicColor)
     */
    haloOpacity: number;
    /**
     * The color of the trace result graphic in the color picker.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#GraphicColor)
     */
    hex: string;
  }

  /**
   * JunctionSourceJSON represents the point layers that participate in the utility network service.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#JunctionSourceJSON)
   */
  export interface JunctionSourceJSON {
    /**
     * The layer id of the point layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#JunctionSourceJSON)
     */
    layerId: number;
    /**
     * The asset group attribute values for the point layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#JunctionSourceJSON)
     */
    assetGroups: AssetGroupJSON[];
    /**
     * The network source id of the point layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#JunctionSourceJSON)
     */
    sourceId: number;
  }

  /**
   * ResultAreaPropertiesExtend represents the properties to determine the size and color of the result area convex hull or buffer, and whether it displays on the map when the trace completes.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ResultAreaPropertiesExtend)
   */
  export interface ResultAreaPropertiesExtend {
    /**
     * The geometry of the result area.
     *
     * [Read more...](global.html#type)
     */
    type: "buffer" | "convexhull";
    /**
     * The buffer or padding added to the result area.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ResultAreaPropertiesExtend)
     */
    distance: number;
    /**
     * Units for linear measurements.
     *
     * [Read more...](global.html#unit)
     */
    unit: LinearUnits;
    /**
     * Units for area measurements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ResultAreaPropertiesExtend)
     */
    areaUnit: AreaUnits;
    /**
     * The color for the result area graphic.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ResultAreaPropertiesExtend)
     */
    color: GraphicColor;
    /**
     * Determines if the area is shown on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ResultAreaPropertiesExtend)
     */
    show: boolean;
  }

  /**
   * TraceItem extends the named trace configuration and adds a property to manage the selection on the view.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceItem)
   */
  export interface TraceItem {
    /**
     * Returns `true` if there is a selection on the view for a trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceItem)
     */
    selected: boolean;
  }

  /**
   * TraceResultExtend organizes the results based on the trace configuration and the trace results.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
   */
  export interface TraceResultExtend {
    /**
     * TraceItem extends the named trace configuration and adds a property to manage the selection on the view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    TraceItem: TraceItem;
    /**
     * The collection of results returned from the trace.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    TraceResult: TraceResult;
    /**
     * Returns true if selection is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    selectionEnabled: boolean;
    /**
     * Returns true if graphic is enabled.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    graphicEnabled: boolean;
    /**
     * The color for the graphic of the trace results in the graphics layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    graphicColor: GraphicColor;
    /**
     * The current status of the trace to return errors from the server.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#TraceResultExtend)
     */
    status: string;
  }

  /**
   * ValidSetup verifies whether a trace meets all the requirements before it can execute.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ValidSetup)
   */
  export interface ValidSetup {
    /**
     * Is `true` if the requirements are met.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ValidSetup)
     */
    status: boolean;
    /**
     * An array of error messages for the missing requirements.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace-UtilityNetworkTraceViewModel.html#ValidSetup)
     */
    issues: string[];
  }

  export interface UtilityNetworkTraceAddFlagCompleteEvent {
    symbol: SimpleMarkerSymbol | PictureMarkerSymbol;

    type: "starting-point" | "barrier";
  }

  export interface UtilityNetworkTraceAddFlagErrorEvent {
    symbol: SimpleMarkerSymbol | PictureMarkerSymbol;

    type: "starting-point" | "barrier";
  }

  export interface UtilityNetworkTraceAddFlagEvent {
    type: "starting-point" | "barrier";
  }

  export interface UtilityNetworkTraceAddResultAreaEvent {
    graphic: Graphic;
  }

  export interface UtilityNetworkTraceCreateResultAreaEvent {
    graphic: Graphic;
  }

  /**
   * InputSetting represents the labels, descriptions, and symbols that can be overridden for the [flags](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#flags) user interface (UI) in the UtilityNetworkTrace widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#InputSetting)
   */
  export interface InputSetting {
    /**
     * The flag type indicates which section will have its settings overridden.
     *
     * [Read more...](global.html#type)
     */
    type: "starting-point" | "barrier";
    /**
     * The label that replaces either the default "Starting points" or "Barriers" headings depending on what flag type is passed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#InputSetting)
     */
    label: string;
    /**
     * The description that replaces the default sub-headings for either the starting points or barriers.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#InputSetting)
     */
    description: string;
    /**
     * The symbol displayed next to either the "Starting points" or "Barriers" headings.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkTrace.html#InputSetting)
     */
    symbol: SimpleMarkerSymbol | PictureMarkerSymbol;
  }

  export interface UtilityNetworkTraceRemoveResultAreaEvent {
    graphic: Graphic;
  }

  export class UtilityNetworkValidateTopology extends Widget {
    /**
     * The UtilityNetworkValidateTopology widget class, functioning as a part of the ArcGIS Maps SDK for JavaScript, simplifies the process of validating a DirtyArea within a utility network.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html)
     */

    constructor(properties?: UtilityNetworkValidateTopologyProperties);

    /**
     * Specifies the extent of the validation.
     *
     * @default "current"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#extentToValidate)
     */
    extentToValidate: "current" | "entire";
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#view)
     */
    view: MapView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#viewModel)
     */
    viewModel: UtilityNetworkValidateTopologyViewModel;
  }

  interface UtilityNetworkValidateTopologyProperties extends WidgetProperties {
    /**
     * Specifies the extent of the validation.
     *
     * @default "current"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#extentToValidate)
     */
    extentToValidate?: "current" | "entire";
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#utilityNetwork)
     */
    utilityNetwork?: UtilityNetworkProperties;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology.html#viewModel)
     */
    viewModel?: UtilityNetworkValidateTopologyViewModel;
  }

  export class UtilityNetworkValidateTopologyViewModel {
    /**
     * If the validation process fails, this property returns an error message.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#executionError)
     */
    readonly executionError: string;
    /**
     * Specifies the extent of the validation.
     *
     * @default "current"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#extentToValidate)
     */
    extentToValidate: "current" | "entire";
    /**
     * If an error occurs during during validation a loadError wil be displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#loadErrors)
     */
    loadErrors: Collection<string>;
    /**
     * The view model's state.
     *
     * @default "ready"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#state)
     */
    readonly state: "disabled" | "executing" | "failed" | "loading" | "ready" | "success";
    /**
     * Determines the utility network to use.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#utilityNetwork)
     */
    utilityNetwork: UtilityNetwork;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#view)
     */
    view: MapView;

    /**
     * Validates the network topology of the [utility network](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#utilityNetwork).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-UtilityNetworkValidateTopology-UtilityNetworkValidateTopologyViewModel.html#validateTopology)
     */
    validateTopology(): void;

    constructor(properties?: any);
  }

  export class ValuePicker extends Widget {
    /**
     * ValuePicker is a widget that allows users to step or play through a list of values.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html)
     */

    constructor(properties?: ValuePickerProperties);

    /**
     * Returns `true` if the ValuePicker can be advanced to the next position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canNext)
     */
    canNext: boolean;
    /**
     * Returns `true` if the ValuePicker can be played.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canPlay)
     */
    canPlay: boolean;
    /**
     * Returns `true` if the ValuePicker can moved to the previous item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canPrevious)
     */
    canPrevious: boolean;
    /**
     * An optional caption that appears on the ValuePicker widget to give context for the user.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#caption)
     */
    caption: string;
    /**
     * An optional component for presenting and managing data.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#component)
     */
    component: ValuePickerCollection | ValuePickerCombobox | ValuePickerLabel | ValuePickerSlider;
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#disabled)
     */
    disabled: boolean;
    /**
     * Indicates if the widget should be orientated horizontally (default) or vertically.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#layout)
     */
    layout: "horizontal" | "vertical";
    /**
     * If true, playback will restart when it reaches the end.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#loop)
     */
    loop: boolean;
    /**
     * The pause, in milliseconds between playback advancement.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#playRate)
     */
    playRate: number;
    /**
     * The current value of the ValuePicker.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#values)
     */
    values: number[] | string[] | any[];
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#visibleElements)
     */
    visibleElements: ValuePickerVisibleElements;

    /**
     * Select the next value or advance to next.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#next)
     */
    next(): void;
    /**
     * Pause playing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#pause)
     */
    pause(): void;
    /**
     * Start playing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#play)
     */
    play(): void;
    /**
     * Select the previous value.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#previous)
     */
    previous(): void;

    on(name: "animate", eventHandler: ValuePickerAnimateEventHandler): IHandle;

    on(name: "next", eventHandler: ValuePickerNextEventHandler): IHandle;

    on(name: "pause", eventHandler: ValuePickerPauseEventHandler): IHandle;

    on(name: "play", eventHandler: ValuePickerPlayEventHandler): IHandle;

    on(name: "previous", eventHandler: ValuePickerPreviousEventHandler): IHandle;
  }

  interface ValuePickerProperties extends WidgetProperties {
    /**
     * Returns `true` if the ValuePicker can be advanced to the next position.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canNext)
     */
    canNext?: boolean;
    /**
     * Returns `true` if the ValuePicker can be played.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canPlay)
     */
    canPlay?: boolean;
    /**
     * Returns `true` if the ValuePicker can moved to the previous item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#canPrevious)
     */
    canPrevious?: boolean;
    /**
     * An optional caption that appears on the ValuePicker widget to give context for the user.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#caption)
     */
    caption?: string;
    /**
     * An optional component for presenting and managing data.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#component)
     */
    component?:
      | (ValuePickerCollection & { type: "collection" })
      | (ValuePickerCombobox & { type: "combobox" })
      | (ValuePickerLabel & { type: "label" })
      | (ValuePickerSlider & { type: "slider" });
    /**
     * When `true`, sets the widget to a disabled state so the user cannot interact with it.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#disabled)
     */
    disabled?: boolean;
    /**
     * Indicates if the widget should be orientated horizontally (default) or vertically.
     *
     * @default "horizontal"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#layout)
     */
    layout?: "horizontal" | "vertical";
    /**
     * If true, playback will restart when it reaches the end.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#loop)
     */
    loop?: boolean;
    /**
     * The pause, in milliseconds between playback advancement.
     *
     * @default 1000
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#playRate)
     */
    playRate?: number;
    /**
     * The current value of the ValuePicker.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#values)
     */
    values?: number[] | string[] | any[];
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#visibleElements)
     */
    visibleElements?: ValuePickerVisibleElements;
  }

  export class ValuePickerCollection {
    /**
     * A collection of values that can be navigated or animated with the play, next, and previous buttons on the [ValuePicker](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html) widget..
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCollection.html#collection)
     */
    collection: Collection;

    readonly type: "collection";

    constructor(properties?: any);
  }

  export class ValuePickerCombobox {
    /**
     * An array of combobox items.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#items)
     */
    items: ComboboxItem[];
    /**
     * Combobox label.
     *
     * @default "ValuePickerCombobox"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#label)
     */
    label: string;
    /**
     * Combobox placeholder text.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#placeholder)
     */
    placeholder: string;

    readonly type: "combobox";

    constructor(properties?: any);
  }

  /**
   * Object used to define the combobox [item](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#items).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#ComboboxItem)
   */
  export interface ComboboxItem {
    /**
     * The value of the combobox item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#ComboboxItem)
     */
    value: string;
    /**
     * The label for this item that users will see in the combobox.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerCombobox.html#ComboboxItem)
     */
    label: string;
  }

  export class ValuePickerLabel {
    /**
     * An array of label items.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerLabel.html#items)
     */
    items: Labelitem[];

    readonly type: "label";

    constructor(properties?: any);
  }

  /**
   * Object used to define the label items.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerLabel.html#labelitem)
   */
  export interface Labelitem {
    /**
     * The value of the item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerLabel.html#labelitem)
     */
    value: string;
    /**
     * The label users will see for this item.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerLabel.html#labelitem)
     */
    label: string;
  }

  export class ValuePickerSlider {
    /**
     * A function used to format labels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#labelFormatFunction)
     */
    labelFormatFunction: SliderLabelFormatter;
    /**
     * Slider tick labels.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#labels)
     */
    labels: number[];
    /**
     * The positions of major ticks.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#majorTicks)
     */
    majorTicks: number[];
    /**
     * The maximum possible data/thumb value of the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#max)
     */
    max: number;
    /**
     * The minimum possible data/thumb value on the slider.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#min)
     */
    min: number;
    /**
     * The positions of minor ticks.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#minorTicks)
     */
    minorTicks: number[];
    /**
     * When true, slider values will ascend right to left and bottom to top when horizontal and vertical respectively.
     *
     * @default false
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#reversed)
     */
    reversed: number;
    /**
     * Positions along the slider that the thumb will snap to when interacted with.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#steps)
     */
    steps: number[];

    readonly type: "slider";
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#visibleElements)
     */
    visibleElements: ValuePickerSliderVisibleElements;

    constructor(properties?: any);
  }

  /**
   * The visible elements that are displayed within the slider component.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#VisibleElements)
   */
  export interface ValuePickerSliderVisibleElements {
    /**
     * When set to `true`, the current value is displayed in a permanent tooltip directly over the slider's thumb.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker-ValuePickerSlider.html#VisibleElements)
     */
    thumbTooltip?: boolean;
  }

  export interface ValuePickerAnimateEvent {}

  export interface ValuePickerNextEvent {}

  export interface ValuePickerPauseEvent {}

  export interface ValuePickerPlayEvent {}

  export interface ValuePickerPreviousEvent {}

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#VisibleElements)
   */
  export interface ValuePickerVisibleElements {
    /**
     * When set to `false`, the next button (or up button when vertical) is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#VisibleElements)
     */
    nextButton?: boolean;
    /**
     * When set to `false`, the play/pause button is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#VisibleElements)
     */
    playButton?: boolean;
    /**
     * When set to `false`, the previous button (or down button when vertical) is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-ValuePicker.html#VisibleElements)
     */
    previousButton?: boolean;
  }

  export interface VersionManagementViewModel extends Accessor, Evented {}

  export class VersionManagementViewModel {
    /**
     * This class allows you to manage versions from a variety of [feature services](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html)
     */

    constructor(properties?: VersionManagementViewModelProperties);

    /**
     * Map of Service URLs and user type extensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#advancedEditingUserTypeExtensionLookup)
     */
    advancedEditingUserTypeExtensionLookup: globalThis.Map<string, boolean>;
    /**
     * Displays execution errors.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#executionError)
     */
    readonly executionError: string;
    /**
     * A key-value pair of [FeatureServiceResourcesBundle](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#FeatureServiceResourcesBundle).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#featureServiceLookup)
     */
    featureServiceLookup: globalThis.Map<string, VersionManagementViewModelFeatureServiceResourcesBundle>;
    /**
     * Displays an error if loading fails.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#loadError)
     */
    readonly loadError: string;
    /**
     * Map of Service URLs and enterprise versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#serverVersionLookup)
     */
    serverVersionLookup: globalThis.Map<string, number>;
    /**
     * Map of Service URLs and service names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#serviceNameLookup)
     */
    serviceNameLookup: globalThis.Map<string, string>;
    /**
     * The viewModel's state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#state)
     */
    readonly state: "disabled" | "executing" | "failed" | "loading" | "ready";
    /**
     * Map of Service URLs and logged in users.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#userLookup)
     */
    userLookup: globalThis.Map<string, string>;
    /**
     * This property determines if a user has version admin privileges.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionAdministratorLookup)
     */
    versionAdministratorLookup: globalThis.Map<string, boolean>;
    /**
     * A Map of current version identifiers keyed on the map service url.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionIdentifierLookup)
     */
    versionIdentifierLookup: globalThis.Map<string, VersionManagementServiceVersionIdentifier>;
    /**
     * Contains information about versions contained in version management service such as name, guid, etc.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionInfoLookup)
     */
    versionInfoLookup: globalThis.Map<string, VersionManagementViewModelVersionInfo[]>;
    /**
     * This property contains metadata about the versioning state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versioningStateLookup)
     */
    versioningStateLookup: globalThis.Map<string, VersioningState>;

    versioningStates: Collection<VersioningState>;
    /**
     * This property contains metadata about the version management service.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionManagementServiceLookup)
     */
    versionManagementServiceLookup: globalThis.Map<string, VersionManagementService>;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#view)
     */
    view: MapView;

    /**
     * The alter operation allows you to change the geodatabase version's name, description, owner, and access permissions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#alterVersion)
     */
    alterVersion(parameters: AlterVersionParameters): Promise<boolean>;
    /**
     * Method used to change a version using a featureServerUrl, name, and guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#changeVersion)
     */
    changeVersion(
      featureServerUrl: string,
      toVersionName: string,
      toVersionGuid: string,
    ): Promise<globalThis.Map<VersionAdapter, ServiceResult>>;
    /**
     * Creates a new version given the following parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#createVersion)
     */
    createVersion(parameters: CreateVersionParameters): Promise<VersionManagementViewModelVersionInfoExtendedJSON>;
    /**
     * Deletes a version given the following parameters.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#deleteVersion)
     */
    deleteVersion(featureServerUrl: string, versionName: string, versionGuid: string): Promise<boolean>;
    /**
     * Returns an array of versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#getVersionInfos)
     */
    getVersionInfos(featureServerUrl: string): Promise<VersionManagementViewModelVersionInfo[]>;
  }

  interface VersionManagementViewModelProperties {
    /**
     * Map of Service URLs and user type extensions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#advancedEditingUserTypeExtensionLookup)
     */
    advancedEditingUserTypeExtensionLookup?: globalThis.Map<string, boolean>;
    /**
     * A key-value pair of [FeatureServiceResourcesBundle](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#FeatureServiceResourcesBundle).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#featureServiceLookup)
     */
    featureServiceLookup?: globalThis.Map<string, VersionManagementViewModelFeatureServiceResourcesBundle>;
    /**
     * Map of Service URLs and enterprise versions.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#serverVersionLookup)
     */
    serverVersionLookup?: globalThis.Map<string, number>;
    /**
     * Map of Service URLs and service names.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#serviceNameLookup)
     */
    serviceNameLookup?: globalThis.Map<string, string>;
    /**
     * Map of Service URLs and logged in users.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#userLookup)
     */
    userLookup?: globalThis.Map<string, string>;
    /**
     * This property determines if a user has version admin privileges.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionAdministratorLookup)
     */
    versionAdministratorLookup?: globalThis.Map<string, boolean>;
    /**
     * A Map of current version identifiers keyed on the map service url.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionIdentifierLookup)
     */
    versionIdentifierLookup?: globalThis.Map<string, VersionManagementServiceVersionIdentifier>;
    /**
     * Contains information about versions contained in version management service such as name, guid, etc.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionInfoLookup)
     */
    versionInfoLookup?: globalThis.Map<string, VersionManagementViewModelVersionInfo[]>;
    /**
     * This property contains metadata about the versioning state.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versioningStateLookup)
     */
    versioningStateLookup?: globalThis.Map<string, VersioningState>;

    versioningStates?: CollectionProperties<VersioningState>;
    /**
     * This property contains metadata about the version management service.
     *
     * @deprecated since version 4.30. Use {@link module:esri/versionManagement/VersioningState} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#versionManagementServiceLookup)
     */
    versionManagementServiceLookup?: globalThis.Map<string, VersionManagementService>;
    /**
     * The view from which the widget will operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  /**
   * Parameters used to alter a version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
   */
  export interface AlterVersionParameters {
    /**
     * The url of a feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    featureServerUrl: string;
    /**
     * The Identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    versionIdentifier: VersionManagementServiceVersionIdentifier;
    /**
     * The new owner name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    ownerName?: string;
    /**
     * The new name for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    versionName?: string;
    /**
     * The new access permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    access?: "hidden" | "private" | "protected" | "public";
    /**
     * The new description for the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#AlterVersionParameters)
     */
    description?: string;
  }

  /**
   * Parameters used to create a new version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
   */
  export interface CreateVersionParameters {
    /**
     * The url of a feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
     */
    featureServerUrl: string;
    /**
     * The name of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
     */
    versionName: string;
    /**
     * The access type of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
     */
    access: "hidden" | "private" | "protected" | "public";
    /**
     * The description of the new version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
     */
    description?: string;
    /**
     * The version owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#CreateVersionParameters)
     */
    ownerName?: string;
  }

  /**
   * Contains [FeatureService](https://developers.arcgis.com/javascript/latest/api-reference/esri-rest-featureService-FeatureService.html) and array of [Layer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-Layer.html).
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#FeatureServiceResourcesBundle)
   */
  export interface VersionManagementViewModelFeatureServiceResourcesBundle {
    /**
     * The feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#FeatureServiceResourcesBundle)
     */
    featureService: FeatureService;
    /**
     * An array of layers from the feature service.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#FeatureServiceResourcesBundle)
     */
    layers: Layer[];
  }

  /**
   * Contains info about a Version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
   */
  export interface VersionManagementViewModelVersionInfo {
    /**
     * The Identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
     */
    versionIdentifier: VersionManagementServiceVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
     */
    description?: string;
    /**
     * The access type permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
     */
    access?: "hidden" | "private" | "protected" | "public";
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfo)
     */
    creationDate?: number;
  }

  /**
   * This contains extended information about a given version.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
   */
  export interface VersionManagementViewModelVersionInfoExtendedJSON {
    /**
     * The identifier of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    versionIdentifier: VersionManagementViewModelVersionInfoExtendedJSONVersionIdentifier;
    /**
     * Description of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    description?: string;
    /**
     * The access type permissions of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    access?: "hidden" | "private" | "protected" | "public";
    /**
     * The id of the version that was edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    versionId?: string;
    /**
     * The date the version was created on.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    creationDate?: number;
    /**
     * The date when the version was last modified or changed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    modifiedDate?: number;
    /**
     * The date when the version was last reconciled with the parent version or the default version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    reconcileDate?: number;
    /**
     * The date when the version was last evaluated for conflicts or discrepancies during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    evaluationDate?: number;
    /**
     * The date of the most recent ancestor version, if applicable.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    previousAncestorDate?: number;
    /**
     * The date of the common ancestor version, which is the shared ancestor between two or more versions during the reconciliation process.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    commonAncestorDate?: number;
    /**
     * Specifies if the version is currently being edited.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    isBeingEdited?: boolean;
    /**
     * Specifies if the version is currently being read.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    isBeingRead?: boolean;
    /**
     * Specifies if the version is has conflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    hasConflicts?: boolean;
    /**
     * Specifies if the version has UninspectedConflicts.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    hasUninspectedConflicts?: boolean;
    /**
     * Specifies if the version currently has a lock.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    isLocked?: boolean;
    /**
     * Specifies which user is the current lock owner.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    lockOwner?: string;
    /**
     * Specifies the date at which the version was locked.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    lockDate?: number;
  }

  export interface VersionManagementViewModelVersionInfoExtendedJSONVersionIdentifier {
    /**
     * The name of the version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    name: string;
    /**
     * The guid of a version.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VersionManagement-VersionManagementViewModel.html#VersionInfoExtendedJSON)
     */
    guid: string;
  }

  export class VideoPlayer extends Widget {
    /**
     * The VideoPlayer widget provides a user interface for interacting with a [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html)
     */

    constructor(properties?: VideoPlayerProperties);

    /**
     * The [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html) to use as the data source for the video player.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#layer)
     */
    layer: VideoLayer;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#view)
     */
    view: MapView;
    /**
     * The view model for the VideoPlayer widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#viewModel)
     */
    viewModel: VideoPlayerViewModel;
  }

  interface VideoPlayerProperties extends WidgetProperties {
    /**
     * The [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html) to use as the data source for the video player.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#layer)
     */
    layer?: VideoLayerProperties;
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html).
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#view)
     */
    view?: MapViewProperties;
    /**
     * The view model for the VideoPlayer widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html#viewModel)
     */
    viewModel?: VideoPlayerViewModelProperties;
  }

  export class VideoPlayerViewModel extends Accessor {
    /**
     * The VideoPlayerViewModel class provides the logic for the [VideoPlayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html)
     */

    constructor(properties?: VideoPlayerViewModelProperties);

    /**
     * The amount of the video layer that has been buffered, in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#buffered)
     */
    readonly buffered: number;
    /**
     * The current time of the video layer, in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#currentTime)
     */
    readonly currentTime: number;
    /**
     * The duration of the video layer, in seconds.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#duration)
     */
    readonly duration: number;
    /**
     * Indicates whether the video layer has ended.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#ended)
     */
    readonly ended: boolean;
    /**
     * Determines which telemetry elements to follow when the video layer is playing.
     *
     * @default "follow-both"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#followingMode)
     */
    followingMode: "follow-sensor" | "follow-frame" | "follow-both";
    /**
     * The [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html) to play.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#layer)
     */
    layer: VideoLayer;
    /**
     * The metadata associated with the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#metadata)
     */
    readonly metadata: any;
    /**
     * Indicates whether the video layer is playing.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#playing)
     */
    readonly playing: boolean;
    /**
     * The number of seconds to seek forward or backward when the user clicks the seek forward or seek backward buttons.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekLength)
     */
    seekLength: number;
    /**
     * The state of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#state)
     */
    readonly state: "can-play" | "not-ready" | "paused" | "playing" | "ready" | "waiting" | "data-loaded" | "error";
    /**
     * The [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) in which the video layer is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#view)
     */
    view: MapView;

    /**
     * Changes the color of the graphics drawn on the map to represent telemetry elements such as sensor location and frame.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#changeGraphicsColor)
     */
    changeGraphicsColor(color: Color): void;
    /**
     * Changes the playback speed of the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#changePlaybackSpeed)
     */
    changePlaybackSpeed(speed: number): void;
    /**
     * Pauses the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#pause)
     */
    pause(): void;
    /**
     * Plays the video layer.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#play)
     */
    play(): void;
    /**
     * Seeks the video layer backward by the number of seconds specified by the [seekLength](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekLength) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekBackward)
     */
    seekBackward(): void;
    /**
     * Seeks the video layer forward by the number of seconds specified by the [seekLength](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekLength) property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekForward)
     */
    seekForward(): void;
    /**
     * Seeks the video layer to the specified time.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekTo)
     */
    seekTo(seekToTime: number): void;
    /**
     * Seeks the video layer to the beginning.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekToBeginning)
     */
    seekToBeginning(): void;
    /**
     * Seeks the video layer to the ending.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekToEnding)
     */
    seekToEnding(): void;
    /**
     * Toggles the frame center display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleFrameCenterDisplay)
     */
    toggleFrameCenterDisplay(): void;
    /**
     * Toggles the video frame image draped on the map.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleFrameDisplay)
     */
    toggleFrameDisplay(): void;
    /**
     * Toggles the frame outline display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleFrameOutlineDisplay)
     */
    toggleFrameOutlineDisplay(): void;
    /**
     * Toggles the sensor display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleSensorDisplay)
     */
    toggleSensorDisplay(): void;
    /**
     * Toggles the sensor sight line display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleSensorSightLineDisplay)
     */
    toggleSensorSightLineDisplay(): void;
    /**
     * Toggles the sensor trail display.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#toggleSensorTrailDisplay)
     */
    toggleSensorTrailDisplay(): void;
  }

  interface VideoPlayerViewModelProperties {
    /**
     * Determines which telemetry elements to follow when the video layer is playing.
     *
     * @default "follow-both"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#followingMode)
     */
    followingMode?: "follow-sensor" | "follow-frame" | "follow-both";
    /**
     * The [VideoLayer](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-VideoLayer.html) to play.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#layer)
     */
    layer?: VideoLayerProperties;
    /**
     * The number of seconds to seek forward or backward when the user clicks the seek forward or seek backward buttons.
     *
     * @default 10
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#seekLength)
     */
    seekLength?: number;
    /**
     * The [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) in which the video layer is displayed.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-VideoPlayer-VideoPlayerViewModel.html#view)
     */
    view?: MapViewProperties;
  }

  export class Weather extends Widget {
    /**
     * The Weather widget provides an interface for easily selecting and configuring the weather effects in a [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html)
     */

    constructor(properties?: WeatherProperties);

    /**
     * Indicates the heading level to use for the title of the widget.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#headingLevel)
     */
    headingLevel: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#view)
     */
    view: SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#viewModel)
     */
    viewModel: WeatherViewModel;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#visibleElements)
     */
    visibleElements: WeatherVisibleElements;
  }

  interface WeatherProperties extends WidgetProperties {
    /**
     * Indicates the heading level to use for the title of the widget.
     *
     * @default 4
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#headingLevel)
     */
    headingLevel?: number;
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#view)
     */
    view?: SceneViewProperties;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#viewModel)
     */
    viewModel?: WeatherViewModelProperties;
    /**
     * This property provides the ability to display or hide the individual elements of the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#visibleElements)
     */
    visibleElements?: WeatherVisibleElements;
  }

  export class WeatherViewModel extends Accessor {
    /**
     * Provides the logic for the [Weather](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather-WeatherViewModel.html)
     */

    constructor(properties?: WeatherViewModelProperties);

    /**
     * The current state of the view model that can be used for rendering the UI of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather-WeatherViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather-WeatherViewModel.html#view)
     */
    view: SceneView;

    /**
     * Sets the weather to the specified type.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather-WeatherViewModel.html#setWeatherByType)
     */
    setWeatherByType(type: "sunny" | "cloudy" | "rainy" | "snowy" | "foggy"): void;
  }

  interface WeatherViewModelProperties {
    /**
     * A reference to the [View](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-View.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather-WeatherViewModel.html#view)
     */
    view?: SceneViewProperties;
  }

  /**
   * The visible elements that are displayed within the widget.
   *
   * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#VisibleElements)
   */
  export interface WeatherVisibleElements {
    /**
     * When set to `false`, the header is not displayed.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Weather.html#VisibleElements)
     */
    header?: boolean;
  }

  export interface Widget extends Accessor, Evented {}

  export class Widget {
    /**
     * The base class for the SDK's out-of-the-box widgets.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html)
     */

    constructor(properties?: WidgetProperties);

    /**
     * The ID or node representing the DOM element containing the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#container)
     */
    container: string | HTMLElement;
    /**
     * Icon which represents the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#icon)
     */
    icon: string;
    /**
     * The unique ID assigned to the widget when the widget is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#id)
     */
    id: string;
    /**
     * The widget's label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#label)
     */
    label: string;
    /**
     * Indicates whether the widget is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#visible)
     */
    visible: boolean;

    /**
     * A utility method used for building the value for a widget's `class` property.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#classes)
     */
    classes(...classNames: (string | string[] | any)[]): string;
    /**
     * Destroys the widget instance.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#destroy)
     */
    destroy(): void;
    /**
     * `isFulfilled()` may be used to verify if creating an instance of the class is fulfilled (either resolved or rejected).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#isFulfilled)
     */
    isFulfilled(): boolean;
    /**
     * `isRejected()` may be used to verify if creating an instance of the class is rejected.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#isRejected)
     */
    isRejected(): boolean;
    /**
     * `isResolved()` may be used to verify if creating an instance of the class is resolved.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#isResolved)
     */
    isResolved(): boolean;
    /**
     * Adds one or more handles which are to be tied to the lifecycle of the widget.
     *
     * @deprecated since 4.28 Use {@link module:esri/widgets/Widget#addHandles addHandles()} instead.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#own)
     */
    own(handleOrHandles: WatchHandle | WatchHandle[]): void;
    /**
     * Executes after widget is ready for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#postInitialize)
     */
    postInitialize(): void;
    /**
     * This method is implemented by subclasses for rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#render)
     */
    render(): any;
    /**
     * Renders widget to the DOM immediately.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#renderNow)
     */
    renderNow(): void;
    /**
     * Schedules widget rendering.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#scheduleRender)
     */
    scheduleRender(): void;
    /**
     * `when()` may be leveraged once an instance of the class is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#when)
     */
    when(callback?: Function, errback?: Function): Promise<any>;
  }

  interface WidgetProperties {
    /**
     * The ID or node representing the DOM element containing the widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#container)
     */
    container?: string | HTMLElement;
    /**
     * Icon which represents the widget.
     *
     * @default null
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#icon)
     */
    icon?: string;
    /**
     * The unique ID assigned to the widget when the widget is created.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#id)
     */
    id?: string;
    /**
     * The widget's label.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#label)
     */
    label?: string;
    /**
     * Indicates whether the widget is visible.
     *
     * @default true
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Widget.html#visible)
     */
    visible?: boolean;
  }

  export class Zoom extends Widget {
    /**
     * The Zoom widget allows users to zoom in/out within a view.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html)
     */

    constructor(properties?: ZoomProperties);

    /**
     * Determines the layout/orientation of the Zoom widget.
     *
     * @default "vertical"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#layout)
     */
    layout: "vertical" | "horizontal";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#view)
     */
    view: MapView | SceneView;
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#viewModel)
     */
    viewModel: ZoomViewModel;

    /**
     * Zooms the view in by an LOD factor of 0.5.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#zoomIn)
     */
    zoomIn(): void;
    /**
     * Zooms the view out by an LOD factor of 2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#zoomOut)
     */
    zoomOut(): void;
  }

  interface ZoomProperties extends WidgetProperties {
    /**
     * Determines the layout/orientation of the Zoom widget.
     *
     * @default "vertical"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#layout)
     */
    layout?: "vertical" | "horizontal";
    /**
     * A reference to the [MapView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-MapView.html) or [SceneView](https://developers.arcgis.com/javascript/latest/api-reference/esri-views-SceneView.html).
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
    /**
     * The view model for this widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html#viewModel)
     */
    viewModel?: ZoomViewModelProperties;
  }

  export class ZoomViewModel extends Accessor {
    /**
     * Provides the logic for the [Zoom](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom.html) widget.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html)
     */

    constructor(properties?: ZoomViewModelProperties);

    /**
     * Indicates if the view can zoom in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#canZoomIn)
     */
    canZoomIn: boolean;
    /**
     * Indicates if the view can zoom out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#canZoomOut)
     */
    canZoomOut: boolean;
    /**
     * The current state of the widget.
     *
     * @default "disabled"
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#state)
     */
    readonly state: "disabled" | "ready";
    /**
     * The view from which to operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#view)
     */
    view: MapView | SceneView;

    /**
     * Zooms the view in by an LOD factor of 0.5.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#zoomIn)
     */
    zoomIn(): void;
    /**
     * Zooms the view out by an LOD factor of 2.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#zoomOut)
     */
    zoomOut(): void;
  }

  interface ZoomViewModelProperties {
    /**
     * Indicates if the view can zoom in.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#canZoomIn)
     */
    canZoomIn?: boolean;
    /**
     * Indicates if the view can zoom out.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#canZoomOut)
     */
    canZoomOut?: boolean;
    /**
     * The view from which to operate.
     *
     * [Read more...](https://developers.arcgis.com/javascript/latest/api-reference/esri-widgets-Zoom-ZoomViewModel.html#view)
     */
    view?: (MapViewProperties & { type: "2d" }) | (SceneViewProperties & { type: "3d" });
  }

  export type MultipointDrawActionCursorUpdateEventHandler = (event: MultipointDrawActionCursorUpdateEvent) => void;

  export type MultipointDrawActionDrawCompleteEventHandler = (event: MultipointDrawActionDrawCompleteEvent) => void;

  export type MultipointDrawActionRedoEventHandler = (event: MultipointDrawActionRedoEvent) => void;

  export type MultipointDrawActionUndoEventHandler = (event: MultipointDrawActionUndoEvent) => void;

  export type MultipointDrawActionVertexAddEventHandler = (event: MultipointDrawActionVertexAddEvent) => void;

  export type MultipointDrawActionVertexRemoveEventHandler = (event: MultipointDrawActionVertexRemoveEvent) => void;

  export type OGCFeatureLayerLayerviewCreateErrorEventHandler = (
    event: OGCFeatureLayerLayerviewCreateErrorEvent,
  ) => void;

  export type OGCFeatureLayerLayerviewCreateEventHandler = (event: OGCFeatureLayerLayerviewCreateEvent) => void;

  export type OGCFeatureLayerLayerviewDestroyEventHandler = (event: OGCFeatureLayerLayerviewDestroyEvent) => void;

  export type OGCFeatureLayerRefreshEventHandler = (event: OGCFeatureLayerRefreshEvent) => void;

  export type OpenStreetMapLayerLayerviewCreateErrorEventHandler = (
    event: OpenStreetMapLayerLayerviewCreateErrorEvent,
  ) => void;

  export type OpenStreetMapLayerLayerviewCreateEventHandler = (event: OpenStreetMapLayerLayerviewCreateEvent) => void;

  export type OpenStreetMapLayerLayerviewDestroyEventHandler = (event: OpenStreetMapLayerLayerviewDestroyEvent) => void;

  export type OpenStreetMapLayerRefreshEventHandler = (event: OpenStreetMapLayerRefreshEvent) => void;

  export type PointDrawActionCursorUpdateEventHandler = (event: PointDrawActionCursorUpdateEvent) => void;

  export type PointDrawActionDrawCompleteEventHandler = (event: PointDrawActionDrawCompleteEvent) => void;

  export type PolygonDrawActionCursorUpdateEventHandler = (event: PolygonDrawActionCursorUpdateEvent) => void;

  export type PolygonDrawActionDrawCompleteEventHandler = (event: PolygonDrawActionDrawCompleteEvent) => void;

  export type PolygonDrawActionRedoEventHandler = (event: PolygonDrawActionRedoEvent) => void;

  export type PolygonDrawActionUndoEventHandler = (event: PolygonDrawActionUndoEvent) => void;

  export type PolygonDrawActionVertexAddEventHandler = (event: PolygonDrawActionVertexAddEvent) => void;

  export type PolygonDrawActionVertexRemoveEventHandler = (event: PolygonDrawActionVertexRemoveEvent) => void;

  export type PolylineDrawActionCursorUpdateEventHandler = (event: PolylineDrawActionCursorUpdateEvent) => void;

  export type PolylineDrawActionDrawCompleteEventHandler = (event: PolylineDrawActionDrawCompleteEvent) => void;

  export type PolylineDrawActionRedoEventHandler = (event: PolylineDrawActionRedoEvent) => void;

  export type PolylineDrawActionUndoEventHandler = (event: PolylineDrawActionUndoEvent) => void;

  export type PolylineDrawActionVertexAddEventHandler = (event: PolylineDrawActionVertexAddEvent) => void;

  export type PolylineDrawActionVertexRemoveEventHandler = (event: PolylineDrawActionVertexRemoveEvent) => void;

  export type PopupTriggerActionEventHandler = (event: PopupTriggerActionEvent) => void;

  export type PrintCompleteEventHandler = (event: PrintCompleteEvent) => void;

  export type PrintSubmitEventHandler = (event: PrintSubmitEvent) => void;

  export type SceneLayerEditsEventHandler = (event: SceneLayerEditsEvent) => void;

  export type SceneLayerLayerviewCreateErrorEventHandler = (event: SceneLayerLayerviewCreateErrorEvent) => void;

  export type SceneLayerLayerviewCreateEventHandler = (event: SceneLayerLayerviewCreateEvent) => void;

  export type SceneLayerLayerviewDestroyEventHandler = (event: SceneLayerLayerviewDestroyEvent) => void;

  export type SearchSearchBlurEventHandler = (event: SearchSearchBlurEvent) => void;

  export type SearchSearchClearEventHandler = (event: SearchSearchClearEvent) => void;

  export type SearchSearchCompleteEventHandler = (event: SearchSearchCompleteEvent) => void;

  export type SearchSearchFocusEventHandler = (event: SearchSearchFocusEvent) => void;

  export type SearchSearchStartEventHandler = (event: SearchSearchStartEvent) => void;

  export type SearchSelectResultEventHandler = (event: SearchSelectResultEvent) => void;

  export type SearchSuggestCompleteEventHandler = (event: SearchSuggestCompleteEvent) => void;

  export type SearchSuggestStartEventHandler = (event: SearchSuggestStartEvent) => void;

  export type SearchViewModelSearchClearEventHandler = (event: SearchViewModelSearchClearEvent) => void;

  export type SearchViewModelSearchCompleteEventHandler = (event: SearchViewModelSearchCompleteEvent) => void;

  export type SearchViewModelSearchStartEventHandler = (event: SearchViewModelSearchStartEvent) => void;

  export type SearchViewModelSelectResultEventHandler = (event: SearchViewModelSelectResultEvent) => void;

  export type SearchViewModelSuggestCompleteEventHandler = (event: SearchViewModelSuggestCompleteEvent) => void;

  export type SearchViewModelSuggestStartEventHandler = (event: SearchViewModelSuggestStartEvent) => void;

  export type SegmentDrawActionCursorUpdateEventHandler = (event: SegmentDrawActionCursorUpdateEvent) => void;

  export type SegmentDrawActionDrawCompleteEventHandler = (event: SegmentDrawActionDrawCompleteEvent) => void;

  export type SegmentDrawActionVertexAddEventHandler = (event: SegmentDrawActionVertexAddEvent) => void;

  export type SketchCreateEventHandler = (event: SketchCreateEvent) => void;

  export type SketchDeleteEventHandler = (event: SketchDeleteEvent) => void;

  export type SketchRedoEventHandler = (event: SketchRedoEvent) => void;

  export type SketchUndoEventHandler = (event: SketchUndoEvent) => void;

  export type SketchUpdateEventHandler = (event: SketchUpdateEvent) => void;

  export type SketchViewModelCreateEventHandler = (event: SketchViewModelCreateEvent) => void;

  export type SketchViewModelDeleteEventHandler = (event: SketchViewModelDeleteEvent) => void;

  export type SketchViewModelRedoEventHandler = (event: SketchViewModelRedoEvent) => void;

  export type SketchViewModelUndoEventHandler = (event: SketchViewModelUndoEvent) => void;

  export type SketchViewModelUpdateEventHandler = (event: SketchViewModelUpdateEvent) => void;

  export type SliderMaxChangeEventHandler = (event: SliderMaxChangeEvent) => void;

  export type SliderMaxClickEventHandler = (event: SliderMaxClickEvent) => void;

  export type SliderMinChangeEventHandler = (event: SliderMinChangeEvent) => void;

  export type SliderMinClickEventHandler = (event: SliderMinClickEvent) => void;

  export type SliderSegmentClickEventHandler = (event: SliderSegmentClickEvent) => void;

  export type SliderSegmentDragEventHandler = (event: SliderSegmentDragEvent) => void;

  export type SliderThumbChangeEventHandler = (event: SliderThumbChangeEvent) => void;

  export type SliderThumbClickEventHandler = (event: SliderThumbClickEvent) => void;

  export type SliderThumbDragEventHandler = (event: SliderThumbDragEvent) => void;

  export type SliderTickClickEventHandler = (event: SliderTickClickEvent) => void;

  export type SliderTrackClickEventHandler = (event: SliderTrackClickEvent) => void;

  export type SmartMappingSliderBaseMaxChangeEventHandler = (event: SmartMappingSliderBaseMaxChangeEvent) => void;

  export type SmartMappingSliderBaseMinChangeEventHandler = (event: SmartMappingSliderBaseMinChangeEvent) => void;

  export type SmartMappingSliderBaseSegmentDragEventHandler = (event: SmartMappingSliderBaseSegmentDragEvent) => void;

  export type SmartMappingSliderBaseThumbChangeEventHandler = (event: SmartMappingSliderBaseThumbChangeEvent) => void;

  export type SmartMappingSliderBaseThumbDragEventHandler = (event: SmartMappingSliderBaseThumbDragEvent) => void;

  export type StreamConnectionDataReceivedEventHandler = (event: StreamConnectionDataReceivedEvent) => void;

  export type StreamLayerViewDataReceivedEventHandler = (event: StreamLayerViewDataReceivedEvent) => void;

  export type StreamLayerViewMessageReceivedEventHandler = (event: StreamLayerViewMessageReceivedEvent) => void;

  export type StreamLayerViewUpdateRateEventHandler = (event: StreamLayerViewUpdateRateEvent) => void;

  export type SubtypeGroupLayerEditsEventHandler = (event: SubtypeGroupLayerEditsEvent) => void;

  export type SubtypeGroupLayerLayerviewCreateErrorEventHandler = (
    event: SubtypeGroupLayerLayerviewCreateErrorEvent,
  ) => void;

  export type SubtypeGroupLayerLayerviewCreateEventHandler = (event: SubtypeGroupLayerLayerviewCreateEvent) => void;

  export type SubtypeGroupLayerLayerviewDestroyEventHandler = (event: SubtypeGroupLayerLayerviewDestroyEvent) => void;

  export type SubtypeGroupLayerRefreshEventHandler = (event: SubtypeGroupLayerRefreshEvent) => void;

  export type TableListTriggerActionEventHandler = (event: TableListTriggerActionEvent) => void;

  export type TableListViewModelTriggerActionEventHandler = (event: TableListViewModelTriggerActionEvent) => void;

  export type TileLayerLayerviewCreateErrorEventHandler = (event: TileLayerLayerviewCreateErrorEvent) => void;

  export type TileLayerLayerviewCreateEventHandler = (event: TileLayerLayerviewCreateEvent) => void;

  export type TileLayerLayerviewDestroyEventHandler = (event: TileLayerLayerviewDestroyEvent) => void;

  export type TileLayerRefreshEventHandler = (event: TileLayerRefreshEvent) => void;

  export type TimeSliderTriggerActionEventHandler = (event: TimeSliderTriggerActionEvent) => void;

  export type TimeSliderViewModelTriggerActionEventHandler = (event: TimeSliderViewModelTriggerActionEvent) => void;

  export type TrackTrackErrorEventHandler = (event: TrackTrackErrorEvent) => void;

  export type TrackTrackEventHandler = (event: TrackTrackEvent) => void;

  export type TrackViewModelTrackErrorEventHandler = (event: TrackViewModelTrackErrorEvent) => void;

  export type TrackViewModelTrackEventHandler = (event: TrackViewModelTrackEvent) => void;

  export type UtilityNetworkTraceAddFlagCompleteEventHandler = (event: UtilityNetworkTraceAddFlagCompleteEvent) => void;

  export type UtilityNetworkTraceAddFlagErrorEventHandler = (event: UtilityNetworkTraceAddFlagErrorEvent) => void;

  export type UtilityNetworkTraceAddFlagEventHandler = (event: UtilityNetworkTraceAddFlagEvent) => void;

  export type UtilityNetworkTraceAddResultAreaEventHandler = (event: UtilityNetworkTraceAddResultAreaEvent) => void;

  export type UtilityNetworkTraceCreateResultAreaEventHandler = (
    event: UtilityNetworkTraceCreateResultAreaEvent,
  ) => void;

  export type UtilityNetworkTraceRemoveResultAreaEventHandler = (
    event: UtilityNetworkTraceRemoveResultAreaEvent,
  ) => void;

  export type ValuePickerAnimateEventHandler = (event: ValuePickerAnimateEvent) => void;

  export type ValuePickerNextEventHandler = (event: ValuePickerNextEvent) => void;

  export type ValuePickerPauseEventHandler = (event: ValuePickerPauseEvent) => void;

  export type ValuePickerPlayEventHandler = (event: ValuePickerPlayEvent) => void;

  export type ValuePickerPreviousEventHandler = (event: ValuePickerPreviousEvent) => void;

  export type VectorTileLayerLayerviewCreateErrorEventHandler = (
    event: VectorTileLayerLayerviewCreateErrorEvent,
  ) => void;

  export type VectorTileLayerLayerviewCreateEventHandler = (event: VectorTileLayerLayerviewCreateEvent) => void;

  export type VectorTileLayerLayerviewDestroyEventHandler = (event: VectorTileLayerLayerviewDestroyEvent) => void;

  export type VectorTileLayerRefreshEventHandler = (event: VectorTileLayerRefreshEvent) => void;

  export type ViewBlurEventHandler = (event: ViewBlurEvent) => void;

  export type ViewClickEventHandler = (event: ViewClickEvent) => void;

  export type ViewDoubleClickEventHandler = (event: ViewDoubleClickEvent) => void;

  export type ViewDragEventHandler = (event: ViewDragEvent) => void;

  export type ViewFocusEventHandler = (event: ViewFocusEvent) => void;

  export type ViewHoldEventHandler = (event: ViewHoldEvent) => void;

  export type ViewImmediateClickEventHandler = (event: ViewImmediateClickEvent) => void;

  export type ViewImmediateDoubleClickEventHandler = (event: ViewImmediateDoubleClickEvent) => void;

  export type ViewKeyDownEventHandler = (event: ViewKeyDownEvent) => void;

  export type ViewKeyUpEventHandler = (event: ViewKeyUpEvent) => void;

  export type ViewLayerviewCreateErrorEventHandler = (event: ViewLayerviewCreateErrorEvent) => void;

  export type ViewLayerviewCreateEventHandler = (event: ViewLayerviewCreateEvent) => void;

  export type ViewLayerviewDestroyEventHandler = (event: ViewLayerviewDestroyEvent) => void;

  export type ViewMouseWheelEventHandler = (event: ViewMouseWheelEvent) => void;

  export type ViewPointerDownEventHandler = (event: ViewPointerDownEvent) => void;

  export type ViewPointerEnterEventHandler = (event: ViewPointerEnterEvent) => void;

  export type ViewPointerLeaveEventHandler = (event: ViewPointerLeaveEvent) => void;

  export type ViewPointerMoveEventHandler = (event: ViewPointerMoveEvent) => void;

  export type ViewPointerUpEventHandler = (event: ViewPointerUpEvent) => void;

  export type ViewResizeEventHandler = (event: ViewResizeEvent) => void;

  export type WebTileLayerLayerviewCreateErrorEventHandler = (event: WebTileLayerLayerviewCreateErrorEvent) => void;

  export type WebTileLayerLayerviewCreateEventHandler = (event: WebTileLayerLayerviewCreateEvent) => void;

  export type WebTileLayerLayerviewDestroyEventHandler = (event: WebTileLayerLayerviewDestroyEvent) => void;

  export type WebTileLayerRefreshEventHandler = (event: WebTileLayerRefreshEvent) => void;

  export type WFSLayerLayerviewCreateErrorEventHandler = (event: WFSLayerLayerviewCreateErrorEvent) => void;

  export type WFSLayerLayerviewCreateEventHandler = (event: WFSLayerLayerviewCreateEvent) => void;

  export type WFSLayerLayerviewDestroyEventHandler = (event: WFSLayerLayerviewDestroyEvent) => void;

  export type WFSLayerRefreshEventHandler = (event: WFSLayerRefreshEvent) => void;

  export type WMSLayerLayerviewCreateErrorEventHandler = (event: WMSLayerLayerviewCreateErrorEvent) => void;

  export type WMSLayerLayerviewCreateEventHandler = (event: WMSLayerLayerviewCreateEvent) => void;

  export type WMSLayerLayerviewDestroyEventHandler = (event: WMSLayerLayerviewDestroyEvent) => void;

  export type WMSLayerRefreshEventHandler = (event: WMSLayerRefreshEvent) => void;

  export namespace CIM {
    /**
     * Represents the properties required for authoring an Arcade expression.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMRenderers.md#enumberation-ExpressionReturnType)
     */
    export enum ExpressionReturnType {
      /**
       * The return type of the expression is determined by the consumer using the expression.
       */
      Default = "Default",
      /**
       * The return type of the expression is treated as a string by all consumers.
       */
      String = "String",
      /**
       * The return type of the expression is treated as a numeric value by all consumers.
       */
      Numeric = "Numeric",
    }

    /**
     * Represents 3D symbol properties, a collection of symbol properties that apply when the symbol is used in a 3D context.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-AngleAlignment)
     */
    export enum AngleAlignment {
      /**
       * Points remain aligned to the display when the map is rotated.
       */
      Display = "Display",
      /**
       * Points are rotated with the map.
       */
      Map = "Map",
    }

    /**
     * Represents a balloon callout. Balloon callouts are a filled background that is placed behind text. They may or may not have a leader line connecting the callout to an anchor point.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-BalloonCalloutStyle)
     */
    export enum BalloonCalloutStyle {
      /**
       * Rectangle style.
       */
      Rectangle = "Rectangle",
      /**
       * Rounded rectangle style.
       */
      RoundedRectangle = "RoundedRectangle",
      /**
       * Oval style.
       */
      Oval = "Oval",
    }

    /**
     * Represents a bar chart marker, a chart made of vertical bars displaying values.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-BlendingMode)
     */
    export enum BlendingMode {
      /**
       * No blending.
       */
      None = "None",
      /**
       * Alpha blending.
       */
      Alpha = "Alpha",
      /**
       * Screen.
       */
      Screen = "Screen",
      /**
       * Multiply.
       */
      Multiply = "Multiply",
      /**
       * Add.
       */
      Add = "Add",
    }

    /**
     * Represents a bar chart marker, a chart made of vertical bars displaying values.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-BlockProgression)
     */
    export enum BlockProgression {
      /**
       * Top To Bottom.
       */
      TTB = "TTB",
      /**
       * Right To Left (vertical text).
       */
      RTL = "RTL",
      /**
       * Bottom To Top.
       */
      BTT = "BTT",
    }

    /**
     * Represents a CGA attribute, the symbol attribute as specified by the CGA code in the rule package.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-CGAAttributeType)
     */
    export enum CGAAttributeType {
      /**
       * Float - Attribute is a numeric attribute that is a float value
       */
      Float = "Float",
      /**
       * String - Attribute is a string
       */
      String = "String",
      /**
       * Boolean - Attribute is a boolean
       */
      Boolean = "Boolean",
    }

    /**
     * Represents a vector marker clipping path.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-ClippingType)
     */
    export enum ClippingType {
      /**
       * Intersect.
       */
      Intersect = "Intersect",
      /**
       * Subtract.
       */
      Subtract = "Subtract",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-ExternalColorMixMode)
     */
    export enum ExternalColorMixMode {
      /**
       * Tint using external color.
       */
      Tint = "Tint",
      /**
       * Ignore external color.
       */
      Ignore = "Ignore",
      /**
       * Multiply components by components of external color.
       */
      Multiply = "Multiply",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-ExtremityPlacement)
     */
    export enum ExtremityPlacement {
      /**
       * Both - marker is placed at the beginning and end of the line.
       */
      Both = "Both",
      /**
       * JustBegin - marker is placed at the beginning of the line, determined by the direction that the line was digitized.
       */
      JustBegin = "JustBegin",
      /**
       * JustEnd - marker is placed at the end of the line, determined by the direction that the line was digitized.
       */
      JustEnd = "JustEnd",
      /**
       * None - no marker is placed at either end of the marker.
       */
      None = "None",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-FillMode)
     */
    export enum FillMode {
      /**
       * Mosaic fill.
       */
      Mosaic = "Mosaic",
      /**
       * Centered fill.
       */
      Centered = "Centered",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-FontEffects)
     */
    export enum FontEffects {
      /**
       * Normal text.
       */
      Normal = "Normal",
      /**
       * Superscript text.
       */
      Superscript = "Superscript",
      /**
       * Subscript text
       */
      Subscript = "Subscript",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-FontEncoding)
     */
    export enum FontEncoding {
      /**
       * Symbol encoding.
       */
      MSSymbol = "MSSymbol",
      /**
       * Unicode.
       */
      Unicode = "Unicode",
    }

    /**
     * Represents the text part position properties on a callout part.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-FontType)
     */
    export enum FontType {
      /**
       * Unspecified.
       */
      Unspecified = "Unspecified",
      /**
       * TrueType.
       */
      TrueType = "TrueType",
      /**
       * OpenType with CFF outlines.
       */
      PSOpenType = "PSOpenType",
      /**
       * OpenType with TrueType outlines.
       */
      TTOpenType = "TTOpenType",
      /**
       * Adobe Type 1.
       */
      Type1 = "Type1",
    }

    /**
     * Represents the arrow geometric effect which creates a dynamic line along a line feature with an arrow of a specified arrow type and width.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectArrowType)
     */
    export enum GeometricEffectArrowType {
      /**
       * An open ended arrow.
       */
      OpenEnded = "OpenEnded",
      /**
       * A block arrow.
       */
      Block = "Block",
      /**
       * A crossed arrow.
       */
      Crossed = "Crossed",
    }

    /**
     * Represents the donut geometric effect which creates a dynamic polygon ring of a specified width in relation to the outline of polygon features.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectDonutMethod)
     */
    export enum GeometricEffectDonutMethod {
      /**
       * Mitered - matches the exact shape around a convex corner of the polygon.
       */
      Mitered = "Mitered",
      /**
       * Bevelled - follows the shortest straight path across a convex corner of the polygon.
       */
      Bevelled = "Bevelled",
      /**
       * Rounded - follows a path of equal distance around a convex corner of the polygon.
       */
      Rounded = "Rounded",
      /**
       * Square - follows a straight path across the corner of a line or polygon.
       */
      Square = "Square",
      /**
       * TrueBuffer - uses the buffer algorithm to follow a path around convex corners.
       */
      TrueBuffer = "TrueBuffer",
    }

    /**
     * Represents the enclosing polygon geometric effect which creates a dynamic polygon from the spatial extent of a line or polygon feature.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectEnclosingPolygonMethod)
     */
    export enum GeometricEffectEnclosingPolygonMethod {
      /**
       * ClosePath - for polygon input, it generates a polygon that matches the geometry of a polygon feature. For line input, it generates a polygon that connects both ends of the line to each other.
       */
      ClosePath = "ClosePath",
      /**
       * ConvexHull - for polygon input, it generates a polygon with a minimum number of sides to surround the feature. For line input, it generates a polygon that approximates the shape of the line.
       */
      ConvexHull = "ConvexHull",
      /**
       * RectangularBox - generates a polygon equal to the spatial envelope of the feature.
       */
      RectangularBox = "RectangularBox",
    }

    /**
     * Represents the extension geometric effect which creates a dynamic line that is extended from either the beginning or the end of the line feature at a specified deflection angle and length.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectExtensionOrigin)
     */
    export enum GeometricEffectExtensionOrigin {
      /**
       * BeginningOfLine - extension is added to the end of the line.
       */
      BeginningOfLine = "BeginningOfLine",
      /**
       * EndOfLine - extension is added to the end of the line.
       */
      EndOfLine = "EndOfLine",
    }

    /**
     * Represents a geometric effect which creates a localizer feather for aeronautical charts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectLocalizerFeatherStyle)
     */
    export enum GeometricEffectLocalizerFeatherStyle {
      /**
       * Displays a complete localizer feather.
       */
      Complete = "Complete",
      /**
       * Displays the left side of a localizer feather.
       */
      Left = "Left",
      /**
       * Displays the right side of a localizer feather.
       */
      Right = "Right",
    }

    /**
     * Represents a geometric effect which creates a hatch pattern to depict special use airspace for aeronautical charts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectOffsetMethod)
     */
    export enum GeometricEffectOffsetMethod {
      /**
       * Mitered - matches the exact shape around a corner of the line or polygon.
       */
      Mitered = "Mitered",
      /**
       * Bevelled - follows the shortest straight path across a corner of the line or polygon.
       */
      Bevelled = "Bevelled",
      /**
       * Rounded - follows a path of equal distance around a corner of the line or polygon.
       */
      Rounded = "Rounded",
      /**
       * Square - follows a straight path across the corner of a line or polygon.
       */
      Square = "Square",
    }

    /**
     * Represents a geometric effect which creates a hatch pattern to depict special use airspace for aeronautical charts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectOffsetOption)
     */
    export enum GeometricEffectOffsetOption {
      /**
       * Fast - ignores complex geometries and applies a best fit to the symbol.
       */
      Fast = "Fast",
      /**
       * Accurate - accommodates complex geometries and applied a true fit to the symbol.
       */
      Accurate = "Accurate",
    }

    /**
     * Represents the offset tangent geometric effect which creates a dynamic line along a line feature offset in the direction defined by either the beginning or the end of the line.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectOffsetTangentMethod)
     */
    export enum GeometricEffectOffsetTangentMethod {
      /**
       * BeginningOfLine - the tangent offset is applied from the beginning of the line.
       */
      BeginningOfLine = "BeginningOfLine",
      /**
       * EndOfLine - the tangent offset is applied from the end of the line.
       */
      EndOfLine = "EndOfLine",
    }

    /**
     * Represents the wave geometric effect which creates a dynamic line or polygon along a feature with a repeating wave pattern.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GeometricEffectWaveform)
     */
    export enum GeometricEffectWaveform {
      /**
       * Sinus - displays a sinusoidal curve.
       */
      Sinus = "Sinus",
      /**
       * Square - displays a three-sided rectangular shape.
       */
      Square = "Square",
      /**
       * Triangle - displays a two-sided triangular shape.
       */
      Triangle = "Triangle",
      /**
       * Random - displays a sine curve with random variations in the period and amplitude
       */
      Random = "Random",
    }

    /**
     * Represents the wave geometric effect which creates a dynamic line or polygon along a feature with a repeating wave pattern.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GlyphHinting)
     */
    export enum GlyphHinting {
      /**
       * No glyph hinting.
       */
      None = "None",
      /**
       * Default glyph hinting according to the font's settings.
       */
      Default = "Default",
      /**
       * Force glyph hinting.
       */
      Force = "Force",
    }

    /**
     * Represents the wave geometric effect which creates a dynamic line or polygon along a feature with a repeating wave pattern.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GradientAlignment)
     */
    export enum GradientAlignment {
      /**
       * Buffered - Distributes the color ramp along the line's geometry from the outside in (similar to the effect of creating a buffer of the line, then using the "buffer" fill type).
       */
      Buffered = "Buffered",
      /**
       * Left - Progresses the color ramp from the line's centerline to the outside edge on the left. The gradient will follow any curvature in the line's geometry.
       */
      Left = "Left",
      /**
       * Right - Progresses the color ramp from the line's centerline to the outside edge on the right. The gradient will follow any curvature in the line's geometry.
       */
      Right = "Right",
      /**
       * Along line - Distributes the color ramp linearly along the line, following the curvature of the line.
       */
      AlongLine = "AlongLine",
    }

    /**
     * Represents a gradient fill which fills polygonal geometry with a specified color scheme.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GradientFillMethod)
     */
    export enum GradientFillMethod {
      /**
       * Linear - Change from one color to the next is linear across the geometry.
       */
      Linear = "Linear",
      /**
       * Rectangular - Change from one color to the next is rectangular, from the extent of the geometry.
       */
      Rectangular = "Rectangular",
      /**
       * Circular - Change from one color to the next is circular, from the extent of the geometry
       */
      Circular = "Circular",
      /**
       * Buffered - Color changes based on an internal buffering of the geometry outline
       */
      Buffered = "Buffered",
    }

    /**
     * Represents a gradient stroke which draws linear geometry with a specified color scheme.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-GradientStrokeType)
     */
    export enum GradientStrokeType {
      /**
       * Discrete gradient types have distinct lines of separation between each color.
       */
      Discrete = "Discrete",
      /**
       * Continuous gradients vary continuously along the color change with no distinct boundary between the colors.
       */
      Continuous = "Continuous",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-HorizontalAlignment)
     */
    export enum HorizontalAlignment {
      /**
       * Left aligned.
       */
      Left = "Left",
      /**
       * Right aligned.
       */
      Right = "Right",
      /**
       * Centered.
       */
      Center = "Center",
      /**
       * Justified alignment.
       */
      Justify = "Justify",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LeaderLineStyle)
     */
    export enum LeaderLineStyle {
      /**
       * The line callout leader is a single line originating from the closest corner of the text box with the gap applied. If the callout has an accent bar it is connected to the closest point at the either top or bottom of the accent bar.
       */
      Base = "Base",
      /**
       * The line callout leader is a single line originating from the midpoint of the left or right side of the text box with the gap applied or from the midpoint of the accent bar if the callout has one.
       */
      MidPoint = "MidPoint",
      /**
       * The line callout leader is a 3-point line originating from the midpoint of the left or right side of the text box with the gap applied or the midpoint of the accent bar if the callout has one.
       */
      ThreePoint = "ThreePoint",
      /**
       * The line callout leader is a 4-point line originating from the midpoint of the left or right side of the text box with the gap applied or the midpoint of the accent bar if the callout has one.
       */
      FourPoint = "FourPoint",
      /**
       * The line callout draws a line that connects to the closest of the four corners of the text with the gap applied. If the callout has an accent bar it is connected to the closest point at either the top or bottom of the accent bar. Additionally, either and underline or an "overline" is drawn along the closest side (bottom or top) of the text.
       */
      Underline = "Underline",
      /**
       * The line callout leader is curved (clockwise) from the anchor point to the closest corner of the text box with the gap applied. If the callout has an accent bar it is connected to the closest point at the either top or bottom of the accent bar.
       */
      CircularCW = "CircularCW",
      /**
       * The line callout leader is curved (counter-clockwise) from the anchor point to the closest corner of the text box with the gap applied. If the callout has an accent bar it is connected to the closest point at the either top or bottom of the accent bar.
       */
      CircularCCW = "CircularCCW",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LineCapStyle)
     */
    export enum LineCapStyle {
      /**
       * Stroke ends in butt caps.
       */
      Butt = "Butt",
      /**
       * Stroke ends in round caps.
       */
      Round = "Round",
      /**
       * Stroke ends in square caps.
       */
      Square = "Square",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LineDashEnding)
     */
    export enum LineDashEnding {
      /**
       * No Constraint - no constraint is applied to how the dash is placed.
       */
      NoConstraint = "NoConstraint",
      /**
       * Half Pattern - a half dash will be placed on either side of control points.
       */
      HalfPattern = "HalfPattern",
      /**
       * Half Gap - a space equal to the half the gap value will be placed on either side of control points.
       */
      HalfGap = "HalfGap",
      /**
       * Full Pattern - a full dash will be placed on either side of control points.
       */
      FullPattern = "FullPattern",
      /**
       * Full Gap - a space equal to the gap value will be placed on either side of control points.
       */
      FullGap = "FullGap",
      /**
       * Custom - the pattern is fit to the length of the feature by adjusting the gaps slightly.
       */
      Custom = "Custom",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LineDecorationStyle)
     */
    export enum LineDecorationStyle {
      /**
       * No decoration.
       */
      None = "None",
      /**
       * The decoration is defined in the Layers property as a set of SymbolReferences.
       */
      Custom = "Custom",
      /**
       * A circle is added at the end of the line.
       */
      Circle = "Circle",
      /**
       * An open arrow is added to the end of the line.
       */
      OpenArrow = "OpenArrow",
      /**
       * A closed arrow is added to the end of the line.
       */
      ClosedArrow = "ClosedArrow",
      /**
       * A diamond is added at the end of the line.
       */
      Diamond = "Diamond",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LineGapType)
     */
    export enum LineGapType {
      /**
       * Extra Leading - Adds the specified value to the line spacing that accommodates the largest font in the line
       */
      ExtraLeading = "ExtraLeading",
      /**
       * Multiple - Sets the line spacing based on a multiple of the line.
       */
      Multiple = "Multiple",
      /**
       * Exact - Sets a fixed line spacing
       */
      Exact = "Exact",
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-LineJoinStyle)
     */
    export enum LineJoinStyle {
      /**
       * The stroke join is beveled.
       */
      Bevel = "Bevel",
      /**
       * The line join is round.
       */
      Round = "Round",
      /**
       * The line join is mitered.
       */
      Miter = "Miter",
    }

    /**
     * Represents marker placement polygon center which defines how a single marker will be placed within the polygon.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-MarkerPlacementType)
     */
    export enum MarkerPlacementType {
      /**
       * Place inside the polygon.
       */
      InsidePolygon = "InsidePolygon",
      /**
       * Place inside the polygon at the center.
       */
      PolygonCenter = "PolygonCenter",
      /**
       * Place randomly inside the polygon.
       */
      RandomlyInsidePolygon = "RandomlyInsidePolygon",
    }

    /**
     * Represents marker placement polygon center which defines how a single marker will be placed within the polygon.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-MaterialMode)
     */
    export enum MaterialMode {
      /**
       * Tint materials and textures with color property.
       */
      Tint = "Tint",
      /**
       * Replace materials and textures with color property.
       */
      Replace = "Replace",
      /**
       * Multiply materials and textures with color property.
       */
      Multiply = "Multiply",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementClip)
     */
    export enum PlacementClip {
      /**
       * Markers are clipped at the boundary of the polygon.
       */
      ClipAtBoundary = "ClipAtBoundary",
      /**
       * Markers are not drawn if their center falls outside of the polygon.
       */
      RemoveIfCenterOutsideBoundary = "RemoveIfCenterOutsideBoundary",
      /**
       * Markers are not drawn if they touch the boundary of the polygon.
       */
      DoNotTouchBoundary = "DoNotTouchBoundary",
      /**
       * Markers are not clipped and may extend past the boundary of the polygon.
       */
      DoNotClip = "DoNotClip",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementEndings)
     */
    export enum PlacementEndings {
      /**
       * No constraint on how the markers are placed.
       */
      NoConstraint = "NoConstraint",
      /**
       * A marker is placed at the control point.
       */
      WithMarkers = "WithMarkers",
      /**
       * A space equal to the placement template will be placed at the control point.
       */
      WithFullGap = "WithFullGap",
      /**
       * A space equal to half the placement template will be placed at the control point.
       */
      WithHalfGap = "WithHalfGap",
      /**
       * Will fit the pattern to the length of the features by adjusting the gaps slightly.
       */
      Custom = "Custom",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementGridType)
     */
    export enum PlacementGridType {
      /**
       * Markers are placed on a uniform grid.
       */
      Fixed = "Fixed",
      /**
       * Markers are placed randomly in the polygon.
       */
      Random = "Random",
      /**
       * Markers are placed randomly with a fixed quantity (dot density).
       */
      RandomFixedQuantity = "RandomFixedQuantity",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementOnLineRelativeTo)
     */
    export enum PlacementOnLineRelativeTo {
      /**
       * Marker is placed in the middle of the line.
       */
      LineMiddle = "LineMiddle",
      /**
       * Marker is placed at the beginning of the line.
       */
      LineBeginning = "LineBeginning",
      /**
       * Marker is placed at the end of the line.
       */
      LineEnd = "LineEnd",
      /**
       * Marker is at the midpoint of each segment in a line feature.
       */
      SegmentMidpoint = "SegmentMidpoint",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementPolygonCenterMethod)
     */
    export enum PlacementPolygonCenterMethod {
      /**
       * Place on the polygon.
       */
      OnPolygon = "OnPolygon",
      /**
       * The centroid of the polygon is used.
       */
      CenterOfMass = "CenterOfMass",
      /**
       * The bounding box of the polygon is used.
       */
      BoundingBoxCenter = "BoundingBoxCenter",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementRandomlyAlongLineRandomization)
     */
    export enum PlacementRandomlyAlongLineRandomization {
      /**
       * A low amount of randomness is applied
       */
      Low = "Low",
      /**
       * A medium amount of randomness is applied
       */
      Medium = "Medium",
      /**
       * A high amount of randomness is applied
       */
      High = "High",
    }

    /**
     * Represents a pie chart marker which is a marker that draws numeric values arranged in a circle.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PlacementStepPosition)
     */
    export enum PlacementStepPosition {
      /**
       * The marker center.
       */
      MarkerCenter = "MarkerCenter",
      /**
       * The marker bounds.
       */
      MarkerBounds = "MarkerBounds",
    }

    /**
     * Represents a point symbol callout which draws a point symbol as the background and a line symbol for leaders. Often used for highway shields.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-PointSymbolCalloutScale)
     */
    export enum PointSymbolCalloutScale {
      /**
       * No scaling.
       */
      None = "None",
      /**
       * Uniform scaling.
       */
      PropUniform = "PropUniform",
      /**
       * Non-uniform scaling.
       */
      PropNonuniform = "PropNonuniform",
      /**
       * Uniform scaling.
       */
      DifUniform = "DifUniform",
      /**
       * Non-uniform scaling.
       */
      DifNonuniform = "DifNonuniform",
    }

    /**
     * Represents shape vertices.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-Simple3DLineStyle)
     */
    export enum Simple3DLineStyle {
      /**
       * Stroke draws as a tube where the width determines the diameter of the tube.
       */
      Tube = "Tube",
      /**
       * Stroke draws flat upon the surface.
       */
      Strip = "Strip",
      /**
       * Stroke is vertically oriented where Width determines the height of the wall.
       */
      Wall = "Wall",
    }

    /**
     * Represents a simple line callout for drawing basic leader lines.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-SizeVariationMethod)
     */
    export enum SizeVariationMethod {
      /**
       * Change in size is applied randomly.
       */
      Random = "Random",
      /**
       * Markers are drawn with a pattern where the markers increase in size.
       */
      Increasing = "Increasing",
      /**
       * Markers are drawn in a pattern where the markers decrease in size.
       */
      Decreasing = "Decreasing",
      /**
       * Markers are drawn in a pattern where the size increase and then decrease.
       */
      IncreasingThenDecreasing = "IncreasingThenDecreasing",
    }

    /**
     * Represents a stacked bar chart marker which is a chart made of vertical stacked bars displaying values.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-SymbolUnits)
     */
    export enum SymbolUnits {
      /**
       * Relative units.
       */
      Relative = "Relative",
      /**
       * Absolute units.
       */
      Absolute = "Absolute",
    }

    /**
     * Represents a stacked bar chart marker which is a chart made of vertical stacked bars displaying values.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-TextCase)
     */
    export enum TextCase {
      /**
       * Text is proper/mixed case.
       */
      Normal = "Normal",
      /**
       * Text is all lower case.
       */
      LowerCase = "LowerCase",
      /**
       * Text is all upper case.
       */
      Allcaps = "Allcaps",
    }

    /**
     * Represents a text margin which defines the margin to apply around text.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-TextReadingDirection)
     */
    export enum TextReadingDirection {
      /**
       * Text is drawn from left-to-right.
       */
      LTR = "LTR",
      /**
       * Text is drawn from right-to-left.
       */
      RTL = "RTL",
    }

    /**
     * Represents a text symbol which is used to draw text graphics, bleeds, and annotation. Text symbols do not contain any symbol layers but can have callouts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-TextureFilter)
     */
    export enum TextureFilter {
      /**
       * Low draft quality.
       */
      Draft = "Draft",
      /**
       * Higher quality, recommended for pictures.
       */
      Picture = "Picture",
      /**
       * Higher quality, recommended when it is important to preserve edges for zoomed in images.
       */
      Text = "Text",
    }

    /**
     * Represents a vector marker which can represent vector graphics. It's constructed from MarkerGraphics which are geometries and symbols used as building blocks for the marker.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-VerticalAlignment)
     */
    export enum VerticalAlignment {
      /**
       * Top of the highest ascender in the text symbol is used to align the text.
       */
      Top = "Top",
      /**
       * Text is centered on the geometry.
       */
      Center = "Center",
      /**
       * Text is aligned so that the geometry lines up with the baseline of the text symbol. Descenders will go past the baseline.
       */
      Baseline = "Baseline",
      /**
       * Bottom of the lowest descender is used to align the text.
       */
      Bottom = "Bottom",
    }

    /**
     * Represents a vector marker which can represent vector graphics. It's constructed from MarkerGraphics which are geometries and symbols used as building blocks for the marker.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-VerticalGlyphOrientation)
     */
    export enum VerticalGlyphOrientation {
      /**
       * Align right.
       */
      Right = "Right",
      /**
       * Align upright.
       */
      Upright = "Upright",
    }

    /**
     * Represents a water fill which fills polygonal geometry with animated water.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-WaterbodySize)
     */
    export enum WaterbodySize {
      /**
       * Small water body.
       */
      Small = "Small",
      /**
       * Medium water body.
       */
      Medium = "Medium",
      /**
       * Large water body.
       */
      Large = "Large",
    }

    /**
     * Represents a water fill which fills polygonal geometry with animated water.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#enumberation-WaveStrength)
     */
    export enum WaveStrength {
      /**
       * Calm glassy water with no waves.
       */
      Calm = "Calm",
      /**
       * Rippled water.
       */
      Rippled = "Rippled",
      /**
       * Slightly wavy water.
       */
      Slight = "Slight",
      /**
       * Moderately wavy water.
       */
      Moderate = "Moderate",
    }

    /**
     * Relative to the ground.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMEnumerations.md#enumberation-BillboardMode)
     */
    export enum BillboardMode {
      /**
       * Not billboarded.
       */
      None = "None",
      /**
       * The symbol always faces toward the viewer as though spinning on a vertical signpost. Viewed from above, the symbol does not face you; you see the top of the symbol.
       */
      SignPost = "SignPost",
      /**
       * The symbol always faces the viewer directly, regardless of view angle.
       */
      FaceNearPlane = "FaceNearPlane",
    }

    /**
     * Relative to the ground.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMEnumerations.md#enumberation-DominantSizeAxis)
     */
    export enum DominantSizeAxis {
      /**
       * Dominant on the Z axis.
       */
      Z = "Z",
      /**
       * Dominant on the X axis.
       */
      X = "X",
      /**
       * Dominant on the Y axis.
       */
      Y = "Y",
    }

    /**
     * Relative to the ground.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMEnumerations.md#enumberation-GradientStrokeMethod)
     */
    export enum GradientStrokeMethod {
      /**
       * A gradient across the line.
       */
      AcrossLine = "AcrossLine",
      /**
       * A gradient along the line.
       */
      AlongLine = "AlongLine",
    }

    /**
     * Relative to the ground.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMEnumerations.md#enumberation-RotationOrder)
     */
    export enum RotationOrder {
      /**
       * Rotate in XYZ order.
       */
      XYZ = "XYZ",
      /**
       * Rotate in ZYX order.
       */
      ZXY = "ZXY",
      /**
       * Rotate in YXZ order.
       */
      YXZ = "YXZ",
    }

    /**
     * Represents the properties required for authoring an Arcade expression.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMRenderers.md#CIMExpressionInfo)
     */
    export interface CIMExpressionInfo {
      type: "CIMExpressionInfo";
      /**
       * The human readable text that describes the expression.
       */
      title?: string;
      /**
       * The Arcade expression used to evaluate and return the value that a property expects.
       */
      expression?: string;
      /**
       * The Name of the expression.
       */
      name?: string;
      /**
       * The ReturnType of the expression.
       */
      returnType?: keyof typeof ExpressionReturnType;
    }

    /**
     * Represents a primitive override.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMRenderers.md#CIMPrimitiveOverride)
     */
    export interface CIMPrimitiveOverride {
      type: "CIMPrimitiveOverride";
      /**
       * The primitive name this override applies to.
       */
      primitiveName?: string;
      /**
       * The property name in the primitive this override applies to.
       */
      propertyName?: string;
      /**
       * The expression.
       */
      expression?: string;
      /**
       * ExpressionInfo that contains the Arcade expression that returns value as a number or a string depending on the PropertyName.
       */
      valueExpressionInfo?: CIMExpressionInfo;
    }

    /**
     * Represents the scale dependent size variations for a symbol reference. Applies to point symbols, line symbols and the outline of polygon symbols. When the symbol reference is rendered at an intermediate scale a linearly scaled size is used.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMRenderers.md#CIMScaleDependentSizeVariation)
     */
    export interface CIMScaleDependentSizeVariation {
      type: "CIMScaleDependentSizeVariation";
      /**
       * The scale the size is associated with.
       */
      scale?: number;
      /**
       * The size for the associated scale.
       */
      size?: number;
    }

    /**
     * Represents a symbol reference. Symbol references currently store the symbol in-line in the symbol property. Overrides here are used primarily by renderers to pass overrides through the drawing pipeline.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMRenderers.md#CIMSymbolReference)
     */
    export interface CIMSymbolReference {
      type: "CIMSymbolReference";
      /**
       * The primitive overrides. Typically set by renderers at draw time.
       */
      primitiveOverrides?: CIMPrimitiveOverride[];
      /**
       * The style path. Reserved for future use.
       */
      stylePath?: string;
      /**
       * The symbol.
       */
      symbol?: CIMSymbolType;
      /**
       * The symbol name.
       */
      symbolName?: string;
      /**
       * The minimum scale range the symbol reference should be displayed at.
       */
      minScale?: number;
      /**
       * The maximum scale range the symbol reference should be displayed at.
       */
      maxScale?: number;
      /**
       * An array of scale dependent sizes.
       */
      scaleDependentSizeVariation?: CIMScaleDependentSizeVariation[];
      /**
       * The minimum distance at which symbols are visible. Objects closer than this don't get rendered.
       */
      minDistance?: number;
      /**
       * The maximum distance at which symbols are visible. Objects beyond this point don't get rendered.
       */
      maxDistance?: number;
    }

    /**
     * Represents 3D symbol properties, a collection of symbol properties that apply when the symbol is used in a 3D context.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIM3DSymbolProperties)
     */
    export interface CIM3DSymbolProperties {
      type: "CIM3DSymbolProperties";
      /**
       * The dominant size axis.
       */
      dominantSizeAxis3D?: keyof typeof DominantSizeAxis;
      /**
       * The rotation order 3D.
       */
      rotationOrder3D?: keyof typeof RotationOrder;
      /**
       * The scale Z.
       */
      scaleZ?: number;
      /**
       * The scale Y.
       */
      scaleY?: number;
    }

    /**
     * CIMCalloutBase
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMCallout)
     */
    export interface CIMCalloutBase {
      type: string;
      /**
       * The leader tolerance which is the closest distance (in points) to the text the anchor point can be for the callout to draw.
       */
      leaderTolerance?: number;
      /**
       * The leader offset which is an offset value defining the distance (in points) between the anchor point and the beginning of the drawn leader.
       */
      leaderOffset?: number;
    }

    /**
     * CIMLineCallout
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMLineCallout)
     */
    export interface CIMLineCallout extends CIMCalloutBase {
      /**
       * The line symbol to draw leaders with.
       */
      leaderLineSymbol?: CIMLineSymbol;
      /**
       * The gap (in points) between the point symbol and the beginning of the leader line.
       */
      gap?: number;
      /**
       * The style of line to generate when a Point leader is drawn defined by an enumeration value. Line leaders will always be drawn with their own geometry.
       */
      lineStyle?: keyof typeof LeaderLineStyle;
    }

    /**
     * CIMSymbolLayerBase
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMSymbolLayer)
     */
    export interface CIMSymbolLayerBase {
      type: string;
      /**
       * Whether the geometric effects that are applied to the symbol layer. Effects dynamically alter the feature geometry when the symbology is applied. Multiple effects applied to a symbol layer are rendered sequentially.
       */
      effects?: CIMGeometricEffectType[];
      /**
       * A value indicating whether the symbol layer is visible. The symbol layer draws only when enabled. Currently, an invisible layer is not considered in any transformations when in a 3D context.
       */
      enable?: boolean;
      /**
       * The internal name of the symbol layer used for symbol level drawing.
       */
      name?: string;
      /**
       * A value indicating whether the color set at the basic properties level is applied to the symbol layer. If the symbol layer is color locked then changes made to the color in the basic properties will not be applied to the symbol layer.
       */
      colorLocked?: boolean;
      /**
       * The primitive name.
       */
      primitiveName?: string;
      /**
       * A value indicating whether or not the symbol layer should overprint in press printing.
       */
      overprint?: boolean;
    }

    /**
     * CIMMarker
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarker)
     */
    export interface CIMMarker extends CIMSymbolLayerBase {
      /**
       * The specified location where all transformation property operations originate.
       */
      anchorPoint?: ExternalReferencePoint;
      /**
       * A value which specifies if the anchor point location is considered a percentage of the size or as an absolute distance.
       */
      anchorPointUnits?: keyof typeof SymbolUnits;
      /**
       * The angle the marker is rotated around the X axis. This type of rotation is also referred to as tilt and is only applied in 3D. The order of how this is applied with respect to other rotations depends on the RotationOrder3D. The name in the user interface is Tilt.
       */
      angleX?: number;
      /**
       * The angle the marker is rotated around the Y axis. This type of rotation is also referred to as roll and is only applied in 3D. The order of how this is applied with respect to other rotations depends on the RotationOrder3D. The name in the user interface is Roll.
       */
      angleY?: number;
      /**
       * Which axis is considered as the Size in 3D. Only applicable when the marker layer is a 3DShapeMarker.
       */
      dominantSizeAxis3D?: keyof typeof DominantSizeAxis;
      /**
       * The value the marker is moved along the X axis from the anchor point. This is applied after all rotation, as opposed to anchor point which is applied before the rotation.
       */
      offsetX?: number;
      /**
       * The value the marker is moved along the Y axis from the anchor point. This is applied after all rotation, as opposed to anchor point which is applied before the rotation.
       */
      offsetY?: number;
      /**
       * The value the marker is moved along the Z axis from the anchor point. This is applied after all rotation, as opposed to anchor point which is applied before the rotation.
       */
      offsetZ?: number;
      /**
       * A value indicating whether the rotation is applied clockwise or counterclockwise to the marker layer.
       */
      rotateClockwise?: boolean;
      /**
       * The angle that the marker is rotated around the anchor point, in degrees.
       */
      rotation?: number;
      /**
       * The height of the marker. Modifying Size changes the marker's height to the specified size and the width is updated proportionally.
       */
      size?: number;
      /**
       * The billboard mode of the marker.
       */
      billboardMode3D?: keyof typeof BillboardMode;
      /**
       * Marker placements which determine how markers are placed along a line or within a polygon.
       */
      markerPlacement?: CIMMarkerPlacementType;
    }

    /**
     * Represents a CGA attribute, the symbol attribute as specified by the CGA code in the rule package.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMCGAAttribute)
     */
    export interface CIMCGAAttribute {
      type: "CIMCGAAttribute";
      /**
       * The name.
       */
      name?: string;
      /**
       * The CGA attribute type.
       */
      CGAAttributeType?: keyof typeof CGAAttributeType;
      /**
       * The value.
       */
      value?: any;
    }

    /**
     * Represents a character marker. The shape of a marker is defined by a glyph in a font. The marker is drawn using the specified size and color. Each marker has a defined frame around the glyph that is considered when the size is applied. As a result, character markers of the same size may appear different sizes if the glyphs frames are different sizes.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMCharacterMarker)
     */
    export interface CIMCharacterMarker extends CIMMarker {
      type: "CIMCharacterMarker";
      /**
       * The Unicode decimal value for the individual glyph of the font that defines the shape of the marker.
       */
      characterIndex?: number;
      /**
       * The depth of the marker when drawn in 3D.
       */
      depth3D?: number;
      /**
       * The font family name of the font. e.g. Comic Sans.
       */
      fontFamilyName?: string;
      /**
       * The style name for the font family. e.g. Regular, Bold, or Italic.
       */
      fontStyleName?: string;
      /**
       * The font type.
       */
      fontType?: keyof typeof FontType;
      /**
       * The width of the symbol without changing the height (or depth in 3D), as a ratio.
       */
      scaleX?: number;
      /**
       * The polygon symbol that is used to renderer the marker.
       */
      symbol?: CIMPolygonSymbol;
      /**
       * A value indicating whether the marker stands a marker upright as though locked in place. The marker can be viewed from all angles.
       */
      verticalOrientation3D?: boolean;
      /**
       * A value indicating whether the strokes and/or fills of a marker are scaled proportionally when the symbol size is changed. When enabled, the strokes for the outline or fill of the polygon symbol used to draw the marker will be scaled proportionally with changes to the symbol size. If this property is not enabled, the stroke will draw with the specified width regardless of the marker size.
       */
      scaleSymbolsProportionally?: boolean;
      /**
       * A value indicating whether the frame of the character marker should be honored when transforming the marker.
       */
      respectFrame?: boolean;
    }

    /**
     * Represents a vector marker clipping path.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMClippingPath)
     */
    export interface CIMClippingPath {
      type: "CIMClippingPath";
      /**
       * The clipping type.
       */
      clippingType?: keyof typeof ClippingType;
      /**
       * The clipping path.
       */
      path?: ExternalReferencePolygon;
    }

    /**
     * Represents color substitution, an ordered list of color substitutes.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMColorSubstitution)
     */
    export interface CIMColorSubstitution {
      type: "CIMColorSubstitution";
      /**
       * The old color (the color that will be substituted).
       */
      oldColor?: number[];
      /**
       * The new color that will replace the old color.
       */
      newColor?: number[];
    }

    /**
     * CIMGeometricEffectBase
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffect)
     */
    export interface CIMGeometricEffectBase {
      type: string;
      /**
       * The primitive name.
       */
      primitiveName?: string;
    }

    /**
     * Represents the add control points geometric effect. Dynamically adds geometry control points to a feature to dictate the placement of markers or other effect properties that leverage control points. Control points are placed at angles or deflection based on the AngleTolerance value.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectAddControlPoints)
     */
    export interface CIMGeometricEffectAddControlPoints extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectAddControlPoints";
      /**
       * The value below which a control point will be placed. The maximum amount of deflection from one segment to another at a vertex. Angle values between 180 and 360 are interpreted the same as values between 0 and 180. Angle values of 180 and 360 are the same as 0.
       */
      angleTolerance?: number;
    }

    /**
     * Represents the arrow geometric effect which creates a dynamic line along a line feature with an arrow of a specified arrow type and width.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectArrow)
     */
    export interface CIMGeometricEffectArrow extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectArrow";
      /**
       * The type of arrow to be displayed.
       */
      geometricEffectArrowType?: keyof typeof GeometricEffectArrowType;
      /**
       * The distance between the lines that construct the body of the arrow.
       */
      width?: number;
    }

    /**
     * Represents the buffer geometric effect which creates a dynamic polygon with a specified distance around features.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectBuffer)
     */
    export interface CIMGeometricEffectBuffer extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectBuffer";
      /**
       * The distance from the feature. This distance is either from the edge of the marker, the edge of the stroke or the edge of the polygon outline.
       */
      size?: number;
    }

    /**
     * Represents the cut geometric effect which creates a dynamic line that is shorter on one or both ends than the line feature or polygon outline.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectCut)
     */
    export interface CIMGeometricEffectCut extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectCut";
      /**
       * The distance from the beginning of a line that the display of the stroke starts. The beginning of the line is determined by the direction in which the line was digitized.
       */
      beginCut?: number;
      /**
       * The distance from the end of a line that the display of the stroke ends. The end of the line is determined by the direction in which the line was digitized.
       */
      endCut?: number;
      /**
       * The distance around the middle of a line that the display of the stroke is interrupted.
       */
      middleCut?: number;
      /**
       * A value indicating whether the effect should be applied in the opposite manner. This displays the stroke symbol only at the ends of the line and leaves the rest of the line unsymbolized.
       */
      invert?: boolean;
    }

    /**
     * Represents the dashes geometric effect which creates a dynamic multipart line geometry from a line feature or the outline of a polygon based on a template.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectDashes)
     */
    export interface CIMGeometricEffectDashes extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectDashes";
      /**
       * Where the pattern should end relative to the ending point of the geometry. Negative numbers indicate a shift to the left and positive numbers a shift to the right. This property is only applied if the LineDashEnding is set to Custom.
       */
      customEndingOffset?: number;
      /**
       * The distance for each dash and gap. There can be multiple dash and gap values to form a complex pattern.
       */
      dashTemplate?: number[];
      /**
       * The setting which determines how the strokes with dash patterns and other patterns (pictures, placement effects) are handled at the end points of the line geometry's segments.
       */
      lineDashEnding?: keyof typeof LineDashEnding;
      /**
       * The position where the pattern should begin relative to the starting point of the geometry. It shifts the entire pattern along the line the specified distance. Negative values indicate a shift to the left and positive numbers a shift to the right. This property is only applied if LineDashEnding is set to NoConstraint or Custom.
       */
      offsetAlongLine?: number;
      /**
       * The line dash ending position.
       */
      controlPointEnding?: keyof typeof LineDashEnding;
    }

    /**
     * Represents the donut geometric effect which creates a dynamic polygon ring of a specified width in relation to the outline of polygon features.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectDonut)
     */
    export interface CIMGeometricEffectDonut extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectDonut";
      /**
       * The method which specifies the way the strokes are displayed at convex corners of the polygon.
       */
      method?: keyof typeof GeometricEffectDonutMethod;
      /**
       * The option for the way the symbol handles complex geometries.
       */
      option?: keyof typeof GeometricEffectOffsetOption;
      /**
       * The distance from the edge of the polygon that the fill symbol is to be displayed.
       */
      width?: number;
    }

    /**
     * Represents the enclosing polygon geometric effect which creates a dynamic polygon from the spatial extent of a line or polygon feature.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectEnclosingPolygon)
     */
    export interface CIMGeometricEffectEnclosingPolygon extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectEnclosingPolygon";
      /**
       * The method which specifies the way in which the polygon geometry is generated around the feature geometry.
       */
      method?: keyof typeof GeometricEffectEnclosingPolygonMethod;
    }

    /**
     * Represents the extension geometric effect which creates a dynamic line that is extended from either the beginning or the end of the line feature at a specified deflection angle and length.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectExtension)
     */
    export interface CIMGeometricEffectExtension extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectExtension";
      /**
       * The deflection angle used for the extension. A value of 0 indicates no deflection.
       */
      deflection?: number;
      /**
       * The origin of the extension to add to the line. The beginning and end of the line is defined by the direction the line was digitized.
       */
      origin?: keyof typeof GeometricEffectExtensionOrigin;
      /**
       * The length of the extension that is dynamically created.
       */
      length?: number;
    }

    /**
     * Represents the jog geometric effect which creates a dynamic line with a jog of a specified angle, position, and width in the line.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectJog)
     */
    export interface CIMGeometricEffectJog extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectJog";
      /**
       * The angle of the jog in the line which is measured in degrees.
       */
      angle?: number;
      /**
       * The length of the segment that forms the jog in the line.
       */
      length?: number;
      /**
       * The location of the center of the jog, as a percentage measured from the start of the input geometry.
       */
      position?: number;
    }

    /**
     * Represents a geometric effect which creates a localizer feather for aeronautical charts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectLocalizerFeather)
     */
    export interface CIMGeometricEffectLocalizerFeather extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectLocalizerFeather";
      /**
       * The localizer feather style.
       */
      style?: keyof typeof GeometricEffectLocalizerFeatherStyle;
      /**
       * The length of the localizer feather.
       */
      length?: number;
      /**
       * The width of the localizer feather.
       */
      width?: number;
      /**
       * The angle of the localizer feather.
       */
      angle?: number;
    }

    /**
     * Represents the move geometric effect which creates a point, line or polygon that is offset a specified distance in X and Y.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectMove)
     */
    export interface CIMGeometricEffectMove extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectMove";
      /**
       * The distance to move the symbol along the X-axis of the feature geometry.
       */
      offsetX?: number;
      /**
       * The distance to move the symbol along the Y-axis of the feature geometry.
       */
      offsetY?: number;
    }

    /**
     * Represents the offset geometric effect which creates a dynamic line or polygon offset at a specified distance perpendicularly from a feature.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectOffset)
     */
    export interface CIMGeometricEffectOffset extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectOffset";
      /**
       * The way the strokes or fills are displayed at corners.
       */
      method?: keyof typeof GeometricEffectOffsetMethod;
      /**
       * The distance of the symbol perpendicular to the feature geometry.
       */
      offset?: number;
      /**
       * The way the symbol handles complex geometries.
       */
      option?: keyof typeof GeometricEffectOffsetOption;
    }

    /**
     * Represents a geometric effect which creates a hatch pattern to depict special use airspace for aeronautical charts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectOffsetHatch)
     */
    export interface CIMGeometricEffectOffsetHatch extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectOffsetHatch";
      /**
       * The length of the offset hatch.
       */
      length?: number;
      /**
       * The spacing of the offset hatch.
       */
      spacing?: number;
    }

    /**
     * Represents the offset tangent geometric effect which creates a dynamic line along a line feature offset in the direction defined by either the beginning or the end of the line.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectOffsetTangent)
     */
    export interface CIMGeometricEffectOffsetTangent extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectOffsetTangent";
      /**
       * The origin of the tangent offset for the line. The beginning and end of the lines are defined by how the line was digitized.
       */
      method?: keyof typeof GeometricEffectOffsetTangentMethod;
      /**
       * The distance the geometry is moved tangent.
       */
      offset?: number;
    }

    /**
     * Represents the radial geometric effect which creates a dynamic line of a specified length and angle originating from a point feature.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectRadial)
     */
    export interface CIMGeometricEffectRadial extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectRadial";
      /**
       * The orientation of the line from the marker. The angle is calculated in a counterclockwise manner with 0 degrees equal to due east.
       */
      angle?: number;
      /**
       * The distance of the line from end to end.
       */
      length?: number;
    }

    /**
     * Represents the regular polygon geometric effect which creates a dynamic polygon around a point feature with a specified number of edges. All edges are equal in length and all angles are equal.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectRegularPolygon)
     */
    export interface CIMGeometricEffectRegularPolygon extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectRegularPolygon";
      /**
       * The amount of rotation for the polygon.
       */
      angle?: number;
      /**
       * The number of sides for the polygon. Specifying a value less than 3 produces a circle.
       */
      edges?: number;
      /**
       * The distance from the center of the polygon.
       */
      radius?: number;
    }

    /**
     * Represents the reverse geometric effect which creates a dynamic polygon around a point feature with a specified number of edges. All edges are equal in length and all angles are equal.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectReverse)
     */
    export interface CIMGeometricEffectReverse extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectReverse";
      /**
       * A value indicating whether the dynamic output of a previous geometric effect is to be flipped or not.
       */
      reverse?: boolean;
    }

    /**
     * Represents the rotate geometric effect which creates a dynamic line or polygon rotated a specified angle from the feature.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectRotate)
     */
    export interface CIMGeometricEffectRotate extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectRotate";
      /**
       * The amount of rotation for the symbol.
       */
      angle?: number;
    }

    /**
     * Represents the rotate geometric effect which creates a dynamic line or polygon scaled by a specified factor. Vertices are moved in relation to the center point of a feature envelope. Values greater than 1 move vertices away from the center point. Values between 0 and 1 move vertices toward the center point. Values less than 0 draw an inverse dynamic line or polygon where the vertices have crossed to the other side of the center point.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectScale)
     */
    export interface CIMGeometricEffectScale extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectScale";
      /**
       * The amount of change in size of a symbol in the x-axis. The value is expressed in terms of a ratio/percentage.
       */
      XScaleFactor?: number;
      /**
       * The amount of change in size of a symbol in the y-axis. The value is expressed in terms of a ratio/percentage.
       */
      YScaleFactor?: number;
    }

    /**
     * Represents the suppress geometric effect which creates a dynamic line that hides sections of a stroke between pairs control points.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectSuppress)
     */
    export interface CIMGeometricEffectSuppress extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectSuppress";
      /**
       * A value indicating whether the portion of the stroke symbol between control points should be suppressed. Sections that are suppressed draw with no symbol.
       */
      suppress?: boolean;
      /**
       * A value indicating whether to invert the suppression process. If this value is true, portions of the stroke symbol between control points are kept and all other portions are suppressed.
       */
      invert?: boolean;
    }

    /**
     * Represents the tapered polygon geometric effect which creates a dynamic polygon along a line feature, whose width varies by two specified amounts along its length, as defined by a percentage of the line feature's length.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectTaperedPolygon)
     */
    export interface CIMGeometricEffectTaperedPolygon extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectTaperedPolygon";
      /**
       * The width at the start of the line to be used to generate a polygon.
       */
      fromWidth?: number;
      /**
       * The distance along the line to be used to generate the polygon.
       */
      length?: number;
      /**
       * The width at the end of the line to be used to generate the polygon.
       */
      toWidth?: number;
    }

    /**
     * Represents the wave geometric effect which creates a dynamic line or polygon along a feature with a repeating wave pattern.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGeometricEffectWave)
     */
    export interface CIMGeometricEffectWave extends CIMGeometricEffectBase {
      type: "CIMGeometricEffectWave";
      /**
       * The distance perpendicular to a feature to display the curves for the symbol.
       */
      amplitude?: number;
      /**
       * The distance along the line or polygon to display the curves for the symbol.
       */
      period?: number;
      /**
       * The staring value for generating a random number. This is only used when the Waveform is set to Random.
       */
      seed?: number;
      /**
       * The shape of the curves to be displayed along the symbol.
       */
      waveform?: keyof typeof GeometricEffectWaveform;
    }

    /**
     * CIMFill
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMFill)
     */
    export interface CIMFill extends CIMSymbolLayerBase {}

    /**
     * Represents a gradient fill which fills polygonal geometry with a specified color scheme.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGradientFill)
     */
    export interface CIMGradientFill extends CIMFill {
      type: "CIMGradientFill";
      /**
       * The angle of the gradient when the GradientMethod is set to Linear or Rectangular.
       */
      angle?: number;
      /**
       * The color scheme that is applied to the fill.
       */
      colorRamp?: any;
      /**
       * A value which specifies how the gradient is applied within the fill.
       */
      gradientMethod?: keyof typeof GradientFillMethod;
      /**
       * A value which determines how much of the feature is covered by the color scheme. This is either a percentage of the total area which the color scheme spans or the number of page units from the starting point at which the gradient displays.
       */
      gradientSize?: number;
      /**
       * A value which specifies whether GradientSize is applied with an absolute distance or a relative percentage.
       */
      gradientSizeUnits?: keyof typeof SymbolUnits;
      /**
       * A value which specifies if the gradient is applied with discrete intervals or if it is continuous.
       */
      gradientType?: keyof typeof GradientStrokeType;
      /**
       * How many bands draw when the GradientType is set to Discrete.
       */
      interval?: number;
    }

    /**
     * CIMStroke
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMStroke)
     */
    export interface CIMStroke extends CIMSymbolLayerBase {
      /**
       * How the stroke should draw at the ends of the geometries.
       */
      capStyle?: keyof typeof LineCapStyle;
      /**
       * How the symbol is drawn at the stroke segment connections.
       */
      joinStyle?: keyof typeof LineJoinStyle;
      /**
       * How strokes will be rendered in 3D.
       */
      lineStyle3D?: keyof typeof Simple3DLineStyle;
      /**
       * The maximum 'sharpness' that is allowed for Miter joins. If the spike created by the miter join exceeds the miter limit times the width of the stroke, the sharp angle will be clipped and rendered with a bevel join. This property is only applied to the symbol layer when the JoinType is set to Miter.
       */
      miterLimit?: number;
      /**
       * The width of the stroke.
       */
      width?: number;
      /**
       * A value indicating whether to close caps when drawing them in 3D. When set to false, the caps are hollow.
       */
      closeCaps3D?: boolean;
    }

    /**
     * Represents a gradient stroke which draws linear geometry with a specified color scheme.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMGradientStroke)
     */
    export interface CIMGradientStroke extends CIMStroke {
      type: "CIMGradientStroke";
      /**
       * The color scheme that is applied to the stroke.
       */
      colorRamp?: any;
      /**
       * How the gradient is applied along the stroke.
       */
      gradientMethod?: keyof typeof GradientStrokeMethod;
      /**
       * How much of the feature is covered by the color scheme. This is either a percentage of the total area which the color scheme spans or the number of page units from the starting point at which the gradient displays.
       */
      gradientSize?: number;
      /**
       * Whether GradientSize is applied with an absolute distance or a relative percentage.
       */
      gradientSizeUnits?: keyof typeof SymbolUnits;
      /**
       * Whether the gradient is applied with discrete or continuous intervals.
       */
      gradientType?: keyof typeof GradientStrokeType;
      /**
       * How many bands draw when the GradientType is set to Discrete.
       */
      interval?: number;
    }

    /**
     * Represents a hatch fill which fills polygonal geometry with a uniform series of parallel line symbols.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMHatchFill)
     */
    export interface CIMHatchFill extends CIMFill {
      type: "CIMHatchFill";
      /**
       * The line symbol that is used to draw the hatch lines in the fill.
       */
      lineSymbol?: CIMLineSymbol;
      /**
       * How much to move the stroke to a new X-position.
       */
      offsetX?: number;
      /**
       * The angle of rotation for all the strokes, in degrees.
       */
      rotation?: number;
      /**
       * The distance between the line symbols in the hatch pattern.
       */
      separation?: number;
      /**
       * How much to move the stroke to a new Y-position.
       */
      offsetY?: number;
    }

    /**
     * CIMSymbolBase
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMSymbol)
     */
    export interface CIMSymbolBase {
      type: string;
    }

    /**
     * CIMMultiLayerSymbol
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMultiLayerSymbol)
     */
    export interface CIMMultiLayerSymbol extends CIMSymbolBase {
      /**
       * The geometric effects that are applied to the symbol.
       */
      effects?: CIMGeometricEffectType[];
      /**
       * The symbol layers. Symbol layers are the components that make up a symbol. A symbol layer is represented by a stroke, fill, marker, or procedural symbol layer.
       */
      symbolLayers?: CIMSymbolLayerType[];
      /**
       * The representative image of the symbol.
       */
      thumbnailURI?: string;
      /**
       * A value indicating whether the symbol size properties are rendered using real world units or page units. When set to true the symbol will draw using real world units (e.g. meters).
       */
      useRealWorldSymbolSizes?: boolean;
    }

    /**
     * Represents a line symbol which is used to draw polyline features or graphics.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMLineSymbol)
     */
    export interface CIMLineSymbol extends CIMMultiLayerSymbol {
      type: "CIMLineSymbol";
    }

    /**
     * Represents a marker graphic which is used to define vector graphics in a vector marker.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerGraphic)
     */
    export interface CIMMarkerGraphic {
      type: "CIMMarkerGraphic";
      /**
       * The geometry of the marker.
       */
      geometry?: ExternalReferenceGeometry;
      /**
       * The symbol used to draw the marker graphic, can be a point, line, polygon, or text symbol.
       */
      symbol?: CIMSymbolType;
      /**
       * The text that is defined within the marker if drawn with a text symbol.
       */
      textString?: string;
      /**
       * The primitive name.
       */
      primitiveName?: string;
    }

    /**
     * CIMMarkerPlacementBase
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacement)
     */
    export interface CIMMarkerPlacementBase {
      type: string;
      /**
       * The primitive name.
       */
      primitiveName?: string;
    }

    /**
     * CIMMarkerStrokePlacement
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerStrokePlacement)
     */
    export interface CIMMarkerStrokePlacement extends CIMMarkerPlacementBase {
      /**
       * A value indicating whether to angle the marker to the line.
       */
      angleToLine?: boolean;
      /**
       * The offset.
       */
      offset?: number;
    }

    /**
     * CIMMarkerPlacementAlongLine
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAlongLine)
     */
    export interface CIMMarkerPlacementAlongLine extends CIMMarkerStrokePlacement {
      /**
       * How markers are placed at control points.
       */
      controlPointPlacement?: keyof typeof PlacementEndings;
      /**
       * Where the pattern should end relative to the ending point of the geometry. The entire pattern is shifted along the line for the specified distance. Negative numbers shift to the left and positive numbers shift to the right. This is only applied if the Endings property is set to Custom.
       */
      customEndingOffset?: number;
      /**
       * How markers are placed at the end points of a line.
       */
      endings?: keyof typeof PlacementEndings;
      /**
       * Where the pattern should begin relative to the starting point of the geometry. The entire pattern is shifted along the line for the specified distance. Negative numbers shift to the left and positive numbers shift to the right. This is only applied if the Endings property is set to No Constraint or Custom.
       */
      offsetAlongLine?: number;
      /**
       * The numeric pattern that defines the sequence of placed markers and the length of space between them.
       */
      placementTemplate?: number[];
    }

    /**
     * Represents marker placement along the line which places randomly sized markers evenly along a line or polygon outline.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAlongLineRandomSize)
     */
    export interface CIMMarkerPlacementAlongLineRandomSize extends CIMMarkerPlacementAlongLine {
      type: "CIMMarkerPlacementAlongLineRandomSize";
      /**
       * The amount of randomness to be used for the size and rotation of the markers on the line. The size and rotation of the marker will vary for individual markers.
       */
      randomization?: keyof typeof PlacementRandomlyAlongLineRandomization;
      /**
       * The starting value for generating a random number. This random number is used by the Randomization property to determine the marker shape.
       */
      seed?: number;
    }

    /**
     * Represents marker placement along the line which places markers that are the same size evenly along a line or polygon outline.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAlongLineSameSize)
     */
    export interface CIMMarkerPlacementAlongLineSameSize extends CIMMarkerPlacementAlongLine {
      type: "CIMMarkerPlacementAlongLineSameSize";
    }

    /**
     * Represents marker placement along the line which places markers in either increasing, decreasing or alternating gradations along a line or polygon outline.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAlongLineVariableSize)
     */
    export interface CIMMarkerPlacementAlongLineVariableSize extends CIMMarkerPlacementAlongLine {
      type: "CIMMarkerPlacementAlongLineVariableSize";
      /**
       * The maximum random offset.
       */
      maxRandomOffset?: number;
      /**
       * The largest size of the marker to be placed on the line. The value is expressed as a ratio.
       */
      maxZoom?: number;
      /**
       * The smallest size of the marker to be placed on the line. The value is expressed as a ratio.
       */
      minZoom?: number;
      /**
       * The number of different sizes of markers to be placed on the line.
       */
      numberOfSizes?: number;
      /**
       * The starting value for generating a random number. This random number is used by the Randomization property to determine which size a marker will receive. This is only used if the VariationMethod is set to Random.
       */
      seed?: number;
      /**
       * The order in which the change of size in the markers should occur.
       */
      variationMethod?: keyof typeof SizeVariationMethod;
    }

    /**
     * Represents marker placement at extremities which places markers at only one or both endpoints of a line.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAtExtremities)
     */
    export interface CIMMarkerPlacementAtExtremities extends CIMMarkerStrokePlacement {
      type: "CIMMarkerPlacementAtExtremities";
      /**
       * Which ends of the line a marker will be placed.
       */
      extremityPlacement?: keyof typeof ExtremityPlacement;
      /**
       * The distance from the ends of a line that the marker will be placed.
       */
      offsetAlongLine?: number;
    }

    /**
     * Represents marker placement at geometry M values.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAtMeasuredUnits)
     */
    export interface CIMMarkerPlacementAtMeasuredUnits extends CIMMarkerStrokePlacement {
      type: "CIMMarkerPlacementAtMeasuredUnits";
      /**
       * The interval of measured units used to place markers.
       */
      interval?: number;
      /**
       * The rate of markers to skip.
       */
      skipMarkerRate?: number;
      /**
       * A value indicating whether markers should be placed at extremities.
       */
      placeAtExtremities?: boolean;
    }

    /**
     * Represents marker placement at ratio positions which places a set number of markers along the line or the outline of a polygon.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementAtRatioPositions)
     */
    export interface CIMMarkerPlacementAtRatioPositions extends CIMMarkerStrokePlacement {
      type: "CIMMarkerPlacementAtRatioPositions";
      /**
       * The distance from the beginning of a line that the marker will be placed.
       */
      beginPosition?: number;
      /**
       * The distance from the end of a line that the marker will be placed. The ending of a line is determined by the direction in which the line was digitized.
       */
      endPosition?: number;
      /**
       * A value indicating whether only the first marker will be rotated 180 degrees.
       */
      flipFirst?: boolean;
      /**
       * The array of positions.
       */
      positionArray?: number[];
    }

    /**
     * CIMMarkerFillPlacement
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerFillPlacement)
     */
    export interface CIMMarkerFillPlacement extends CIMMarkerPlacementBase {}

    /**
     * Represents marker placement inside a polygon which defines how a polygon is filled with a pattern of markers.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementInsidePolygon)
     */
    export interface CIMMarkerPlacementInsidePolygon extends CIMMarkerFillPlacement {
      type: "CIMMarkerPlacementInsidePolygon";
      /**
       * The orientation angle that the markers are placed on within the polygon.
       */
      gridAngle?: number;
      /**
       * The grid type which defines how markers are placed.
       */
      gridType?: keyof typeof PlacementGridType;
      /**
       * The marker row offset horizontally.
       */
      offsetX?: number;
      /**
       * The randomness of the pattern when markers are placed randomly in a polygon.
       */
      randomness?: number;
      /**
       * The starting value for generating a random pattern.
       */
      seed?: number;
      /**
       * A value indicating whether every other row of markers should be shifted to create an offset grid.
       */
      shiftOddRows?: boolean;
      /**
       * The distance between each marker on the X-axis of the grid.
       */
      stepX?: number;
      /**
       * The distance between each marker on the Y-axis of the grid.
       */
      stepY?: number;
      /**
       * The marker row offset vertically.
       */
      offsetY?: number;
      /**
       * The clipping option which specifies how markers should be clipped at the polygon boundary.
       */
      clipping?: keyof typeof PlacementClip;
    }

    /**
     * Represents a marker placement on the line.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementOnLine)
     */
    export interface CIMMarkerPlacementOnLine extends CIMMarkerStrokePlacement {
      type: "CIMMarkerPlacementOnLine";
      /**
       * The location on a line where a marker will be placed. The direction of the line is determined by the direction in which the line was digitized.
       */
      relativeTo?: keyof typeof PlacementOnLineRelativeTo;
      /**
       * The distances from a specified location on a line that a marker will be placed.
       */
      startPointOffset?: number;
    }

    /**
     * Represents a marker placement on vertices which places a single marker on a line or polygon outline at a set distance from the middle or one of the endpoints.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementOnVertices)
     */
    export interface CIMMarkerPlacementOnVertices extends CIMMarkerStrokePlacement {
      type: "CIMMarkerPlacementOnVertices";
      /**
       * A value indicating whether a marker will be placed on the control points of the line.
       */
      placeOnControlPoints?: boolean;
      /**
       * A value indicating whether a marker will be placed on the endpoints of the line.
       */
      placeOnEndPoints?: boolean;
      /**
       * A value indicating whether a marker will be placed on the vertices of the line.
       */
      placeOnRegularVertices?: boolean;
    }

    /**
     * Represents marker placement polygon center which defines how a single marker will be placed within the polygon.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMarkerPlacementPolygonCenter)
     */
    export interface CIMMarkerPlacementPolygonCenter extends CIMMarkerFillPlacement {
      type: "CIMMarkerPlacementPolygonCenter";
      /**
       * The method used to determine the polygon center.
       */
      method?: keyof typeof PlacementPolygonCenterMethod;
      /**
       * The value which offsets the marker horizontally from the center.
       */
      offsetX?: number;
      /**
       * The value which offsets the marker vertically from the center.
       */
      offsetY?: number;
      /**
       * A value indicating whether the marker should be clipped if it extends pasts the boundary of the polygon.
       */
      clipAtBoundary?: boolean;
    }

    /**
     * Represents material properties.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMaterialProperties)
     */
    export interface CIMMaterialProperties {
      type: "CIMMaterialProperties";
      /**
       * The specular color.
       */
      specularColor?: number[];
      /**
       * The shininess.
       */
      shininess?: number;
      /**
       * How this material combines with externally defined colors.
       */
      externalColorMixMode?: keyof typeof ExternalColorMixMode;
    }

    /**
     * Represents a material which defines how the multipatch or mesh is drawn.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMMaterialSymbolLayer)
     */
    export interface CIMMaterialSymbolLayer extends CIMSymbolLayerBase {
      type: "CIMMaterialSymbolLayer";
      /**
       * The material color.
       */
      color?: number[];
      /**
       * The mode in which the material is applied.
       */
      materialMode?: keyof typeof MaterialMode;
    }

    /**
     * Represents a marker symbol for 3D objects.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMObjectMarker3D)
     */
    export interface CIMObjectMarker3D extends CIMMarker {
      type: "CIMObjectMarker3D";
      /**
       * The URI of the binary reference containing the "web resource".
       */
      modelURI?: string;
      /**
       * The marker width.
       */
      width?: number;
      /**
       * The marker depth.
       */
      depth?: number;
      /**
       * The color which defines the color that is applied to the marker.
       */
      tintColor?: number[];
      /**
       * A value indicating whether the model can be exported.
       */
      isRestricted?: boolean;
      /**
       * The representative image of the marker.
       */
      thumbnail?: string;
      /**
       * A value indicating whether or not to ignore the marker anchor point and insert the model directly at the data point.
       */
      useAnchorPoint?: boolean;
      /**
       * The array of levels of detail.
       */
      LODs?: CIMObjectMarker3DLOD[];
    }

    /**
     * Represents a level of detail of an object marker 3D.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMObjectMarker3DLOD)
     */
    export interface CIMObjectMarker3DLOD {
      type: "CIMObjectMarker3DLOD";
      /**
       * The total number of triangles in the geometry of this level of detail.
       */
      faceCount?: number;
      /**
       * The URI of the binary reference containing the "web resource" for this level of detail.
       */
      modelURI?: string;
    }

    /**
     * Represents a picture fill which fills polygonal geometry with a picture. Supported file types are .bmp, .jpg, .png, and .gif.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMPictureFill)
     */
    export interface CIMPictureFill extends CIMFill {
      type: "CIMPictureFill";
      /**
       * The URL of the image. Typically a base64 encoded image.
       */
      url?: string;
      /**
       * The distance that the image is offset in the horizontal direction.
       */
      offsetX?: number;
      /**
       * The distance that the image is offset in the vertical direction.
       */
      offsetY?: number;
      /**
       * Angle of the image within the fill.
       */
      rotation?: number;
      /**
       * The width of the symbol without changing the height (or depth in 3D), as a ratio.
       */
      scaleX?: number;
      /**
       * The height of the image.
       */
      height?: number;
      /**
       * How the image is resampled.
       */
      textureFilter?: keyof typeof TextureFilter;
      /**
       * The color substitutions which allows colors in the image to be substituted with a different color.
       */
      colorSubstitutions?: CIMColorSubstitution[];
      /**
       * The color that is applied as a tint to the image. The color is applied to the whole image. When the tint is set to white the image appears with its native colors.
       */
      tintColor?: number[];
    }

    /**
     * Represents a picture marker created from a raster (bitmapped) image file. The image can have color substitutions to replace one or more colors in the image or it can have a tint applied to the whole image depending on the picture type. Supported formats are .bmp, .jpg, .png, and .gif.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMPictureMarker)
     */
    export interface CIMPictureMarker extends CIMMarker {
      type: "CIMPictureMarker";
      /**
       * The color substitutions for the picture.
       */
      colorSubstitutions?: CIMColorSubstitution[];
      /**
       * The depth of the image when drawn in 3D.
       */
      depth3D?: number;
      /**
       * A value indicating whether the image is right-reading when viewed from behind.
       */
      invertBackfaceTexture?: boolean;
      /**
       * The scale X which changes the width of the symbol without changing the height (or depth in 3D), as a ratio.
       */
      scaleX?: number;
      /**
       * How the image is resampled.
       */
      textureFilter?: keyof typeof TextureFilter;
      /**
       * The color that is applied as a tint to the image. The color is applied to the whole image. When the tint is set to white the image appears with its native colors.
       */
      tintColor?: number[];
      /**
       * The image that is used in the symbol layer. Typically a base64 encoded image.
       */
      url?: string;
      /**
       * A value indicating whether the marker stands upright as though locked in place. The marker can be viewed from all angles.
       */
      verticalOrientation3D?: boolean;
    }

    /**
     * Represents a picture stroke which draws linear geometry with a repeating image file. Supported file types are .bmp, .jpg, .png, and .gif.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMPictureStroke)
     */
    export interface CIMPictureStroke extends CIMStroke {
      type: "CIMPictureStroke";
      /**
       * The color substitutions for the picture.
       */
      colorSubstitutions?: CIMColorSubstitution[];
      /**
       * How the image is resampled.
       */
      textureFilter?: keyof typeof TextureFilter;
      /**
       * The image that is used in the symbol layer. Typically a base64 encoded image.
       */
      url?: string;
      /**
       * The color that is applied as a tint to the image. The color is applied to the whole image. When the tint is set to white the image appears with its native colors.
       */
      tintColor?: number[];
    }

    /**
     * Represents a point symbol used to draw point features and point graphics.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMPointSymbol)
     */
    export interface CIMPointSymbol extends CIMMultiLayerSymbol {
      type: "CIMPointSymbol";
      /**
       * The callout of the point symbol.
       */
      callout?: CIMCalloutType;
      /**
       * The size of the halo that extends beyond the symbol shape.
       */
      haloSize?: number;
      /**
       * The polygon symbol that is used to draw the halo for a point symbol.
       */
      haloSymbol?: CIMPolygonSymbol;
      /**
       * The primitive name.
       */
      primitiveName?: string;
      /**
       * The X scale which changes the width of the symbol without changing the height (or depth in 3D), as a ratio.
       */
      scaleX?: number;
      /**
       * The collection of symbol properties that apply when the symbol is used in a 3D context.
       */
      symbol3DProperties?: CIM3DSymbolProperties;
      /**
       * The amount of variation applied to the symbol, measured in degrees, propagated cumulatively to all marker symbols.
       */
      angle?: number;
      /**
       * Whether point symbols align to the map or to the display when a rotation is applied to the map.
       */
      angleAlignment?: keyof typeof AngleAlignment;
    }

    /**
     * Represents a polygon symbol which is used to draw polygon features or polygon graphics.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMPolygonSymbol)
     */
    export interface CIMPolygonSymbol extends CIMMultiLayerSymbol {
      type: "CIMPolygonSymbol";
    }

    /**
     * Represents shape vertices.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMShapeVertices)
     */
    export interface CIMShapeVertices {
      type: "CIMShapeVertices";
      /**
       * The indices.
       */
      indices?: number;
      /**
       * The shape.
       */
      shapes?: string;
    }

    /**
     * Represents a solid fill which fills polygonal geometry with a single solid color.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMSolidFill)
     */
    export interface CIMSolidFill extends CIMFill {
      type: "CIMSolidFill";
      /**
       * The color that is applied to the fill.
       */
      color?: number[];
    }

    /**
     * Represents a solid stroke which draws linear geometry with a single solid color.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMSolidStroke)
     */
    export interface CIMSolidStroke extends CIMStroke {
      type: "CIMSolidStroke";
      /**
       * The color that is applied to the stroke.
       */
      color?: number[];
    }

    /**
     * Represents a text margin which defines the margin to apply around text.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMTextMargin)
     */
    export interface CIMTextMargin {
      type: "CIMTextMargin";
      /**
       * The left margin.
       */
      left?: number;
      /**
       * The right margin.
       */
      right?: number;
      /**
       * The top margin.
       */
      top?: number;
      /**
       * The bottom margin.
       */
      bottom?: number;
    }

    /**
     * Represents a text symbol which is used to draw text graphics, bleeds, and annotation. Text symbols do not contain any symbol layers but can have callouts.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMTextSymbol)
     */
    export interface CIMTextSymbol extends CIMSymbolBase {
      type: "CIMTextSymbol";
      /**
       * The amount of rotation applied to the text symbol, measured in degrees, around the geometry.
       */
      angle?: number;
      /**
       * The amount of rotation of the text symbol around the X axis, measured in degrees, around the geometry. This type of rotation is also referred to as tilt. It is applied in 3D.
       */
      angleX?: number;
      /**
       * The amount of rotation of the text symbol around the Y axis, measured in degrees, around the geometry. This type of rotation is also referred to as roll. It is applied in 3D.
       */
      angleY?: number;
      /**
       * The direction in which multi-line text is stacked.
       */
      blockProgression?: keyof typeof BlockProgression;
      /**
       * The callout or background of the text with optional leader lines.
       */
      callout?: CIMCalloutType;
      /**
       * A value indicating whether to draw the text in a fashion compatible with ArcMap.
       */
      compatibilityMode?: boolean;
      /**
       * The ISO code for the base country of the text.
       */
      countryISO?: string;
      /**
       * The depth of the glyph when drawn in 3D. This is an extrusion of the characters of the text in the Z axis.
       */
      depth3D?: number;
      /**
       * A value indicating whether fonts that are drawn as rasters at some scales to draw as vectors instead.
       */
      drawGlyphsAsGeometry?: boolean;
      /**
       * A value indicating whether soft hyphens should be drawn. Soft hyphens are invisible markers that indicate where a hyphenated break is allowed within the text. They are only drawn if there is word wrapping at the end of a line.
       */
      drawSoftHyphen?: boolean;
      /**
       * A value indicating whether the baseline of the text geometry should be expanded in the same manner as the existing geometry if the text extends beyond the baseline.
       */
      extrapolateBaselines?: boolean;
      /**
       * The angle (in degrees from vertical) at which point rotated text is flipped (mirrored) in place.
       */
      flipAngle?: number;
      /**
       * Whether the text is drawn as subscript or superscript.
       */
      fontEffects?: keyof typeof FontEffects;
      /**
       * The font encoding.
       */
      fontEncoding?: keyof typeof FontEncoding;
      /**
       * The font family name of the font. e.g. Comic Sans.
       */
      fontFamilyName?: string;
      /**
       * The style name for the font family. e.g. Regular, Bold, or Italic.
       */
      fontStyleName?: string;
      /**
       * The type of font that the font family/style name reference.
       */
      fontType?: keyof typeof FontType;
      /**
       * An additional rotation that is applied to the individual glyphs contained in the text. This is applied to the individual glyphs whereas Angle, AngleX and AngleY are affect how the entire text string is oriented.
       */
      glyphRotation?: number;
      /**
       * The size of the halo that extends beyond the symbol shape.
       */
      haloSize?: number;
      /**
       * The polygon symbol that is used to draw the halo for a text symbol.
       */
      haloSymbol?: CIMPolygonSymbol;
      /**
       * The size of the text in points.
       */
      height?: number;
      /**
       * If hinting from the font is used for text rendering. Hinting is information included with most fonts to effectively fit the vector glyphs of the font into the raster grid onto which they are displayed.
       */
      hinting?: keyof typeof GlyphHinting;
      /**
       * The alignment type used to align the text to the geometry horizontally. Affects which side of a point geometry the point text is drawn or which end of a line it is drawn close to. Commonly used to define how stacked text appears.
       */
      horizontalAlignment?: keyof typeof HorizontalAlignment;
      /**
       * How many points to indent the text back from the end of the baseline.
       */
      indentAfter?: number;
      /**
       * How many points to indent the text from the beginning of the baseline.
       */
      indentBefore?: number;
      /**
       * How many points to indent the text from the beginning of the baseline for the first line only.
       */
      indentFirstLine?: number;
      /**
       * A value indicating whether the text is drawn with metric kerning, which adjusts the spacing between individual letter forms.
       */
      kerning?: boolean;
      /**
       * Whether the ISO code for the base language of the text.
       */
      languageISO?: string;
      /**
       * The additional spacing that is added to each glyph beyond what is defined by its character box in the font. Value indicates the percentage of a glyph's width. Also known as tracking.
       */
      letterSpacing?: number;
      /**
       * The width that is added to each glyph beyond what is defined by its character box in its font. This is a percentage of the original glyph.
       */
      letterWidth?: number;
      /**
       * A value indicating whether text is to be drawn with ligatures, which occur when two or more letters or portions of letters are joined to form a single glyph.
       */
      ligatures?: boolean;
      /**
       * The spacing between lines of text. This is also known as leading or line spacing.
       */
      lineGap?: number;
      /**
       * The type of line gap that is applied.
       */
      lineGapType?: keyof typeof LineGapType;
      /**
       * The X offset.
       */
      offsetX?: number;
      /**
       * The Y offset.
       */
      offsetY?: number;
      /**
       * The Z offset.
       */
      offsetZ?: number;
      /**
       * The color of the shadow that is defined for the text symbol. The shadow is drawn as an offset copy of the text.
       */
      shadowColor?: number[];
      /**
       * The shadow offset from the text symbol in the horizontal direction. If X and Y are zero, no shadow is drawn.
       */
      shadowOffsetX?: number;
      /**
       * The shadow offset from the text symbol in the vertical direction. If X and Y are zero, no shadow is drawn.
       */
      shadowOffsetY?: number;
      /**
       * A value indicating whether the text should be drawn as Small Capitals, where lower case text is converted to small caps and upper case text is left as upper case.
       */
      smallCaps?: boolean;
      /**
       * A value indicating whether to draw the text with a strike through it.
       */
      strikethrough?: boolean;
      /**
       * The polygon symbol that is used to draw the glyphs of the text.
       */
      symbol?: CIMPolygonSymbol;
      /**
       * The collection of properties that are applied to the text symbol only in a 3D context.
       */
      symbol3DProperties?: CIM3DSymbolProperties;
      /**
       * The letter case used to draw the text.
       */
      textCase?: keyof typeof TextCase;
      /**
       * The base text direction to draw the text.
       */
      textDirection?: keyof typeof TextReadingDirection;
      /**
       * A value indicating whether to draw the text with an underline.
       */
      underline?: boolean;
      /**
       * The vertical alignment of the text.
       */
      verticalAlignment?: keyof typeof VerticalAlignment;
      /**
       * The orientation for the non-vertical text in a vertical layout. For example, an English fragment in a Japanese text.
       */
      verticalGlyphOrientation?: keyof typeof VerticalGlyphOrientation;
      /**
       * The additional spacing that is added to between the words of the text string. 100% indicates that regular spacing is used.
       */
      wordSpacing?: number;
      /**
       * The billboard mode of the text symbol.
       */
      billboardMode3D?: keyof typeof BillboardMode;
      /**
       * A value indicating whether or not the symbol should overprint in press printing.
       */
      overprint?: boolean;
    }

    /**
     * Represents a vector marker which can represent vector graphics. It's constructed from MarkerGraphics which are geometries and symbols used as building blocks for the marker.
     *
     * [Read more...](https://github.com/Esri/cim-spec/blob/master/docs/v2/CIMSymbols.md#CIMVectorMarker)
     */
    export interface CIMVectorMarker extends CIMMarker {
      type: "CIMVectorMarker";
      /**
       * The depth of the marker when drawn in 3D.
       */
      depth3D?: number;
      /**
       * The outer boundary of the entire vector marker.
       */
      frame?: ExternalReferenceEnvelope;
      /**
       * The vector graphics that define the shape of the marker.
       */
      markerGraphics?: CIMMarkerGraphic[];
      /**
       * A value indicating whether the marker stands upright as though locked in place. The marker can be viewed from all angles.
       */
      verticalOrientation3D?: boolean;
      /**
       * A value indicating whether the strokes and or fills of a marker are scaled proportionally when the symbol size is changed. When enabled, the strokes for the outline or fill of the polygon symbol used to draw the marker will be scaled proportionally with changes to the symbol size. If this property is not enabled, then the stroke will draw with the specified width regardless of the marker size.
       */
      scaleSymbolsProportionally?: boolean;
      /**
       * A value indicating whether the frame of the vector marker should be honored when drawing the marker.
       */
      respectFrame?: boolean;
      /**
       * A clipping path for the vector marker graphics.
       */
      clippingPath?: CIMClippingPath;
    }

    export type CIMMarkerPlacementType =
      | CIMMarkerPlacementAlongLineRandomSize
      | CIMMarkerPlacementAlongLineSameSize
      | CIMMarkerPlacementAlongLineVariableSize
      | CIMMarkerPlacementAtExtremities
      | CIMMarkerPlacementAtMeasuredUnits
      | CIMMarkerPlacementAtRatioPositions
      | CIMMarkerPlacementOnLine
      | CIMMarkerPlacementOnVertices
      | CIMMarkerPlacementInsidePolygon
      | CIMMarkerPlacementPolygonCenter;
    export type CIMSymbolType = CIMLineSymbol | CIMPointSymbol | CIMPolygonSymbol | CIMTextSymbol;
    export type CIMGeometricEffectType =
      | CIMGeometricEffectAddControlPoints
      | CIMGeometricEffectArrow
      | CIMGeometricEffectBuffer
      | CIMGeometricEffectCut
      | CIMGeometricEffectDashes
      | CIMGeometricEffectDonut
      | CIMGeometricEffectEnclosingPolygon
      | CIMGeometricEffectExtension
      | CIMGeometricEffectJog
      | CIMGeometricEffectLocalizerFeather
      | CIMGeometricEffectMove
      | CIMGeometricEffectOffset
      | CIMGeometricEffectOffsetHatch
      | CIMGeometricEffectOffsetTangent
      | CIMGeometricEffectRadial
      | CIMGeometricEffectRegularPolygon
      | CIMGeometricEffectReverse
      | CIMGeometricEffectRotate
      | CIMGeometricEffectScale
      | CIMGeometricEffectSuppress
      | CIMGeometricEffectTaperedPolygon
      | CIMGeometricEffectWave;
    export type CIMSymbolLayerType =
      | CIMCharacterMarker
      | CIMObjectMarker3D
      | CIMPictureMarker
      | CIMVectorMarker
      | CIMGradientFill
      | CIMHatchFill
      | CIMPictureFill
      | CIMSolidFill
      | CIMGradientStroke
      | CIMPictureStroke
      | CIMSolidStroke
      | CIMMaterialSymbolLayer;
    export type CIMCalloutType = CIMLineCallout;

    export interface ExternalReferencePoint {
      x?: number;
      y?: number;
      z?: number;
      m?: number;
      spatialReference?: ExternalReferenceSpatialReference;
    }

    export interface ExternalReferencePolyline {
      hasM?: boolean;
      hasZ?: boolean;
      paths?: number[][][];
      curvePath?: number[][][];
      spatialReference?: ExternalReferenceSpatialReference;
    }

    export interface ExternalReferencePolygon {
      hasM?: boolean;
      hasZ?: boolean;
      rings?: number[][][];
      curveRings?: number[][][];
      spatialReference?: ExternalReferenceSpatialReference;
    }

    export interface ExternalReferenceEnvelope {
      mmax?: number;
      mmin?: number;
      xmax?: number;
      xmin?: number;
      ymax?: number;
      ymin?: number;
      zmax?: number;
      zmin?: number;
      spatialReference?: ExternalReferenceSpatialReference;
    }

    export interface ExternalReferenceSpatialReference {
      wkid?: number;
      latestWkid?: number;
      vcsWkid?: number;
      latestVcsWkid?: number;
    }

    export type ExternalReferenceGeometry =
      | ExternalReferencePoint
      | ExternalReferencePolyline
      | ExternalReferencePolygon
      | ExternalReferenceEnvelope;
  }
}

declare module "esri/Basemap" {
  import Basemap = __esri.Basemap;
  export = Basemap;
}

declare module "esri/Camera" {
  import Camera = __esri.Camera;
  export = Camera;
}

declare module "esri/CameraLayout" {
  import CameraLayout = __esri.CameraLayout;
  export = CameraLayout;
}

declare module "esri/Color" {
  import Color = __esri.Color;
  export = Color;
}

declare module "esri/analysis/AreaMeasurementAnalysis" {
  import AreaMeasurementAnalysis = __esri.AreaMeasurementAnalysis;
  export = AreaMeasurementAnalysis;
}

declare module "esri/analysis/DimensionAnalysis" {
  import DimensionAnalysis = __esri.DimensionAnalysis;
  export = DimensionAnalysis;
}

declare module "esri/analysis/DimensionSimpleStyle" {
  import DimensionSimpleStyle = __esri.DimensionSimpleStyle;
  export = DimensionSimpleStyle;
}

declare module "esri/analysis/DirectLineMeasurementAnalysis" {
  import DirectLineMeasurementAnalysis = __esri.DirectLineMeasurementAnalysis;
  export = DirectLineMeasurementAnalysis;
}

declare module "esri/analysis/LengthDimension" {
  import LengthDimension = __esri.LengthDimension;
  export = LengthDimension;
}

declare module "esri/analysis/LineOfSightAnalysis" {
  import LineOfSightAnalysis = __esri.LineOfSightAnalysis;
  export = LineOfSightAnalysis;
}

declare module "esri/analysis/LineOfSightAnalysisObserver" {
  import LineOfSightAnalysisObserver = __esri.LineOfSightAnalysisObserver;
  export = LineOfSightAnalysisObserver;
}

declare module "esri/analysis/LineOfSightAnalysisTarget" {
  import LineOfSightAnalysisTarget = __esri.LineOfSightAnalysisTarget;
  export = LineOfSightAnalysisTarget;
}

declare module "esri/analysis/SliceAnalysis" {
  import SliceAnalysis = __esri.SliceAnalysis;
  export = SliceAnalysis;
}

declare module "esri/analysis/SlicePlane" {
  import SlicePlane = __esri.SlicePlane;
  export = SlicePlane;
}

declare module "esri/analysis/Viewshed" {
  import Viewshed = __esri.Viewshed;
  export = Viewshed;
}

declare module "esri/analysis/ViewshedAnalysis" {
  import ViewshedAnalysis = __esri.ViewshedAnalysis;
  export = ViewshedAnalysis;
}

declare module "esri/core/Accessor" {
  import Accessor = __esri.Accessor;
  export = Accessor;
}

declare module "esri/core/Collection" {
  import Collection = __esri.Collection;
  export = Collection;
}

declare module "esri/core/Error" {
  import Error = __esri.Error;
  export = Error;
}

declare module "esri/core/Handles" {
  import Handles = __esri.Handles;
  export = Handles;
}

declare module "esri/core/workers/Connection" {
  import Connection = __esri.Connection;
  export = Connection;
}

declare module "esri/form/elements/AttachmentElement" {
  import AttachmentElement = __esri.AttachmentElement;
  export = AttachmentElement;
}

declare module "esri/form/elements/Element" {
  import Element = __esri.Element;
  export = Element;
}

declare module "esri/form/elements/FieldElement" {
  import FieldElement = __esri.FieldElement;
  export = FieldElement;
}

declare module "esri/form/elements/GroupElement" {
  import GroupElement = __esri.GroupElement;
  export = GroupElement;
}

declare module "esri/form/elements/RelationshipElement" {
  import RelationshipElement = __esri.RelationshipElement;
  export = RelationshipElement;
}

declare module "esri/form/elements/TextElement" {
  import TextElement = __esri.TextElement;
  export = TextElement;
}

declare module "esri/form/elements/inputs/BarcodeScannerInput" {
  import BarcodeScannerInput = __esri.BarcodeScannerInput;
  export = BarcodeScannerInput;
}

declare module "esri/form/elements/inputs/ComboBoxInput" {
  import ComboBoxInput = __esri.ComboBoxInput;
  export = ComboBoxInput;
}

declare module "esri/form/elements/inputs/DatePickerInput" {
  import DatePickerInput = __esri.DatePickerInput;
  export = DatePickerInput;
}

declare module "esri/form/elements/inputs/DateTimeOffsetPickerInput" {
  import DateTimeOffsetPickerInput = __esri.DateTimeOffsetPickerInput;
  export = DateTimeOffsetPickerInput;
}

declare module "esri/form/elements/inputs/DateTimePickerInput" {
  import DateTimePickerInput = __esri.DateTimePickerInput;
  export = DateTimePickerInput;
}

declare module "esri/form/elements/inputs/RadioButtonsInput" {
  import RadioButtonsInput = __esri.RadioButtonsInput;
  export = RadioButtonsInput;
}

declare module "esri/form/elements/inputs/SwitchInput" {
  import SwitchInput = __esri.SwitchInput;
  export = SwitchInput;
}

declare module "esri/form/elements/inputs/TextAreaInput" {
  import TextAreaInput = __esri.TextAreaInput;
  export = TextAreaInput;
}

declare module "esri/form/elements/inputs/TextBoxInput" {
  import TextBoxInput = __esri.TextBoxInput;
  export = TextBoxInput;
}

declare module "esri/form/elements/inputs/TimePickerInput" {
  import TimePickerInput = __esri.TimePickerInput;
  export = TimePickerInput;
}

declare module "esri/form/elements/inputs/attachments/AttachmentInput" {
  import AttachmentInput = __esri.AttachmentInput;
  export = AttachmentInput;
}

declare module "esri/form/elements/inputs/attachments/AudioInput" {
  import AudioInput = __esri.AudioInput;
  export = AudioInput;
}

declare module "esri/form/elements/inputs/attachments/DocumentInput" {
  import DocumentInput = __esri.DocumentInput;
  export = DocumentInput;
}

declare module "esri/form/elements/inputs/attachments/ImageInput" {
  import ImageInput = __esri.ImageInput;
  export = ImageInput;
}

declare module "esri/form/elements/inputs/attachments/SignatureInput" {
  import SignatureInput = __esri.SignatureInput;
  export = SignatureInput;
}

declare module "esri/form/elements/inputs/attachments/VideoInput" {
  import VideoInput = __esri.VideoInput;
  export = VideoInput;
}

declare module "esri/form/ExpressionInfo" {
  import ExpressionInfo = __esri.ExpressionInfo;
  export = ExpressionInfo;
}

declare module "esri/form/FormTemplate" {
  import FormTemplate = __esri.FormTemplate;
  export = FormTemplate;
}

declare module "esri/geometry/Circle" {
  import Circle = __esri.Circle;
  export = Circle;
}

declare module "esri/geometry/Extent" {
  import Extent = __esri.Extent;
  export = Extent;
}

declare module "esri/geometry/Geometry" {
  import Geometry = __esri.Geometry;
  export = Geometry;
}

declare module "esri/geometry/HeightModelInfo" {
  import HeightModelInfo = __esri.HeightModelInfo;
  export = HeightModelInfo;
}

declare module "esri/geometry/Mesh" {
  import Mesh = __esri.Mesh;
  export = Mesh;
}

declare module "esri/geometry/Multipoint" {
  import Multipoint = __esri.Multipoint;
  export = Multipoint;
}

declare module "esri/geometry/Point" {
  import Point = __esri.Point;
  export = Point;
}

declare module "esri/geometry/Polygon" {
  import Polygon = __esri.Polygon;
  export = Polygon;
}

declare module "esri/geometry/Polyline" {
  import Polyline = __esri.Polyline;
  export = Polyline;
}

declare module "esri/geometry/SpatialReference" {
  import SpatialReference = __esri.SpatialReference;
  export = SpatialReference;
}

declare module "esri/geometry/operators/support/Transformation" {
  import Transformation = __esri.Transformation;
  export = Transformation;
}

declare module "esri/geometry/support/GeographicTransformation" {
  import GeographicTransformation = __esri.GeographicTransformation;
  export = GeographicTransformation;
}

declare module "esri/geometry/support/GeographicTransformationStep" {
  import GeographicTransformationStep = __esri.GeographicTransformationStep;
  export = GeographicTransformationStep;
}

declare module "esri/geometry/support/MeshComponent" {
  import MeshComponent = __esri.MeshComponent;
  export = MeshComponent;
}

declare module "esri/geometry/support/MeshGeoreferencedVertexSpace" {
  import MeshGeoreferencedVertexSpace = __esri.MeshGeoreferencedVertexSpace;
  export = MeshGeoreferencedVertexSpace;
}

declare module "esri/geometry/support/MeshLocalVertexSpace" {
  import MeshLocalVertexSpace = __esri.MeshLocalVertexSpace;
  export = MeshLocalVertexSpace;
}

declare module "esri/geometry/support/MeshMaterial" {
  import MeshMaterial = __esri.MeshMaterial;
  export = MeshMaterial;
}

declare module "esri/geometry/support/MeshMaterialMetallicRoughness" {
  import MeshMaterialMetallicRoughness = __esri.MeshMaterialMetallicRoughness;
  export = MeshMaterialMetallicRoughness;
}

declare module "esri/geometry/support/MeshTexture" {
  import MeshTexture = __esri.MeshTexture;
  export = MeshTexture;
}

declare module "esri/geometry/support/MeshTextureTransform" {
  import MeshTextureTransform = __esri.MeshTextureTransform;
  export = MeshTextureTransform;
}

declare module "esri/geometry/support/MeshTransform" {
  import MeshTransform = __esri.MeshTransform;
  export = MeshTransform;
}

declare module "esri/Graphic" {
  import Graphic = __esri.Graphic;
  export = Graphic;
}

declare module "esri/Ground" {
  import Ground = __esri.Ground;
  export = Ground;
}

declare module "esri/identity/Credential" {
  import Credential = __esri.Credential;
  export = Credential;
}

declare module "esri/identity/IdentityManager" {
  const IdentityManager: __esri.IdentityManager;
  export = IdentityManager;
}

declare module "esri/identity/OAuthInfo" {
  import OAuthInfo = __esri.OAuthInfo;
  export = OAuthInfo;
}

declare module "esri/identity/ServerInfo" {
  import ServerInfo = __esri.ServerInfo;
  export = ServerInfo;
}

declare module "esri/layers/BaseDynamicLayer" {
  import BaseDynamicLayer = __esri.BaseDynamicLayer;
  export = BaseDynamicLayer;
}

declare module "esri/layers/BaseElevationLayer" {
  import BaseElevationLayer = __esri.BaseElevationLayer;
  export = BaseElevationLayer;
}

declare module "esri/layers/BaseTileLayer" {
  import BaseTileLayer = __esri.BaseTileLayer;
  export = BaseTileLayer;
}

declare module "esri/layers/BingMapsLayer" {
  import BingMapsLayer = __esri.BingMapsLayer;
  export = BingMapsLayer;
}

declare module "esri/layers/BuildingSceneLayer" {
  import BuildingSceneLayer = __esri.BuildingSceneLayer;
  export = BuildingSceneLayer;
}

declare module "esri/layers/buildingSublayers/BuildingComponentSublayer" {
  import BuildingComponentSublayer = __esri.BuildingComponentSublayer;
  export = BuildingComponentSublayer;
}

declare module "esri/layers/buildingSublayers/BuildingGroupSublayer" {
  import BuildingGroupSublayer = __esri.BuildingGroupSublayer;
  export = BuildingGroupSublayer;
}

declare module "esri/layers/buildingSublayers/BuildingSublayer" {
  import BuildingSublayer = __esri.BuildingSublayer;
  export = BuildingSublayer;
}

declare module "esri/layers/CatalogLayer" {
  import CatalogLayer = __esri.CatalogLayer;
  export = CatalogLayer;
}

declare module "esri/layers/catalog/CatalogDynamicGroupLayer" {
  import CatalogDynamicGroupLayer = __esri.CatalogDynamicGroupLayer;
  export = CatalogDynamicGroupLayer;
}

declare module "esri/layers/catalog/CatalogFootprintLayer" {
  import CatalogFootprintLayer = __esri.CatalogFootprintLayer;
  export = CatalogFootprintLayer;
}

declare module "esri/layers/CSVLayer" {
  import CSVLayer = __esri.CSVLayer;
  export = CSVLayer;
}

declare module "esri/layers/DimensionLayer" {
  import DimensionLayer = __esri.DimensionLayer;
  export = DimensionLayer;
}

declare module "esri/layers/ElevationLayer" {
  import ElevationLayer = __esri.ElevationLayer;
  export = ElevationLayer;
}

declare module "esri/layers/FeatureLayer" {
  import FeatureLayer = __esri.FeatureLayer;
  export = FeatureLayer;
}

declare module "esri/layers/GeoJSONLayer" {
  import GeoJSONLayer = __esri.GeoJSONLayer;
  export = GeoJSONLayer;
}

declare module "esri/layers/GeoRSSLayer" {
  import GeoRSSLayer = __esri.GeoRSSLayer;
  export = GeoRSSLayer;
}

declare module "esri/layers/GraphicsLayer" {
  import GraphicsLayer = __esri.GraphicsLayer;
  export = GraphicsLayer;
}

declare module "esri/layers/GroupLayer" {
  import GroupLayer = __esri.GroupLayer;
  export = GroupLayer;
}

declare module "esri/layers/ImageryLayer" {
  import ImageryLayer = __esri.ImageryLayer;
  export = ImageryLayer;
}

declare module "esri/layers/ImageryTileLayer" {
  import ImageryTileLayer = __esri.ImageryTileLayer;
  export = ImageryTileLayer;
}

declare module "esri/layers/IntegratedMeshLayer" {
  import IntegratedMeshLayer = __esri.IntegratedMeshLayer;
  export = IntegratedMeshLayer;
}

declare module "esri/layers/IntegratedMesh3DTilesLayer" {
  import IntegratedMesh3DTilesLayer = __esri.IntegratedMesh3DTilesLayer;
  export = IntegratedMesh3DTilesLayer;
}

declare module "esri/layers/KMLLayer" {
  import KMLLayer = __esri.KMLLayer;
  export = KMLLayer;
}

declare module "esri/layers/KnowledgeGraphLayer" {
  import KnowledgeGraphLayer = __esri.KnowledgeGraphLayer;
  export = KnowledgeGraphLayer;
}

declare module "esri/layers/knowledgeGraph/KnowledgeGraphSublayer" {
  import KnowledgeGraphSublayer = __esri.KnowledgeGraphSublayer;
  export = KnowledgeGraphSublayer;
}

declare module "esri/layers/Layer" {
  import Layer = __esri.Layer;
  export = Layer;
}

declare module "esri/layers/LineOfSightLayer" {
  import LineOfSightLayer = __esri.LineOfSightLayer;
  export = LineOfSightLayer;
}

declare module "esri/layers/LinkChartLayer" {
  import LinkChartLayer = __esri.LinkChartLayer;
  export = LinkChartLayer;
}

declare module "esri/layers/MapImageLayer" {
  import MapImageLayer = __esri.MapImageLayer;
  export = MapImageLayer;
}

declare module "esri/layers/MapNotesLayer" {
  import MapNotesLayer = __esri.MapNotesLayer;
  export = MapNotesLayer;
}

declare module "esri/layers/MediaLayer" {
  import MediaLayer = __esri.MediaLayer;
  export = MediaLayer;
}

declare module "esri/layers/OGCFeatureLayer" {
  import OGCFeatureLayer = __esri.OGCFeatureLayer;
  export = OGCFeatureLayer;
}

declare module "esri/layers/OpenStreetMapLayer" {
  import OpenStreetMapLayer = __esri.OpenStreetMapLayer;
  export = OpenStreetMapLayer;
}

declare module "esri/layers/OrientedImageryLayer" {
  import OrientedImageryLayer = __esri.OrientedImageryLayer;
  export = OrientedImageryLayer;
}

declare module "esri/layers/pointCloudFilters/PointCloudFilter" {
  import PointCloudFilter = __esri.PointCloudFilter;
  export = PointCloudFilter;
}

declare module "esri/layers/pointCloudFilters/PointCloudReturnFilter" {
  import PointCloudReturnFilter = __esri.PointCloudReturnFilter;
  export = PointCloudReturnFilter;
}

declare module "esri/layers/pointCloudFilters/PointCloudValueFilter" {
  import PointCloudValueFilter = __esri.PointCloudValueFilter;
  export = PointCloudValueFilter;
}

declare module "esri/layers/pointCloudFilters/PointCloudBitfieldFilter" {
  import PointCloudBitfieldFilter = __esri.PointCloudBitfieldFilter;
  export = PointCloudBitfieldFilter;
}

declare module "esri/layers/PointCloudLayer" {
  import PointCloudLayer = __esri.PointCloudLayer;
  export = PointCloudLayer;
}

declare module "esri/layers/RouteLayer" {
  import RouteLayer = __esri.RouteLayer;
  export = RouteLayer;
}

declare module "esri/layers/SceneLayer" {
  import SceneLayer = __esri.SceneLayer;
  export = SceneLayer;
}

declare module "esri/layers/StreamLayer" {
  import StreamLayer = __esri.StreamLayer;
  export = StreamLayer;
}

declare module "esri/layers/SubtypeGroupLayer" {
  import SubtypeGroupLayer = __esri.SubtypeGroupLayer;
  export = SubtypeGroupLayer;
}

declare module "esri/layers/support/AggregateField" {
  import AggregateField = __esri.AggregateField;
  export = AggregateField;
}

declare module "esri/layers/support/BuildingFilter" {
  import BuildingFilter = __esri.BuildingFilter;
  export = BuildingFilter;
}

declare module "esri/layers/support/BuildingSummaryStatistics" {
  import BuildingSummaryStatistics = __esri.BuildingSummaryStatistics;
  export = BuildingSummaryStatistics;
}

declare module "esri/layers/support/CodedValueDomain" {
  import CodedValueDomain = __esri.CodedValueDomain;
  export = CodedValueDomain;
}

declare module "esri/layers/support/ControlPointsGeoreference" {
  import ControlPointsGeoreference = __esri.ControlPointsGeoreference;
  export = ControlPointsGeoreference;
}

declare module "esri/layers/support/CornersGeoreference" {
  import CornersGeoreference = __esri.CornersGeoreference;
  export = CornersGeoreference;
}

declare module "esri/layers/support/DimensionalDefinition" {
  import DimensionalDefinition = __esri.DimensionalDefinition;
  export = DimensionalDefinition;
}

declare module "esri/layers/support/Domain" {
  import Domain = __esri.Domain;
  export = Domain;
}

declare module "esri/layers/support/ElevationSampler" {
  import ElevationSampler = __esri.ElevationSampler;
  export = ElevationSampler;
}

declare module "esri/layers/support/ExpressionInfo" {
  import supportExpressionInfo = __esri.supportExpressionInfo;
  export = supportExpressionInfo;
}

declare module "esri/layers/support/ExtentAndRotationGeoreference" {
  import ExtentAndRotationGeoreference = __esri.ExtentAndRotationGeoreference;
  export = ExtentAndRotationGeoreference;
}

declare module "esri/layers/support/FacilityLayerInfo" {
  import FacilityLayerInfo = __esri.FacilityLayerInfo;
  export = FacilityLayerInfo;
}

declare module "esri/layers/support/FeatureEffect" {
  import FeatureEffect = __esri.FeatureEffect;
  export = FeatureEffect;
}

declare module "esri/layers/support/FeatureFilter" {
  import FeatureFilter = __esri.FeatureFilter;
  export = FeatureFilter;
}

declare module "esri/layers/support/FeatureReductionBinning" {
  import FeatureReductionBinning = __esri.FeatureReductionBinning;
  export = FeatureReductionBinning;
}

declare module "esri/layers/support/FeatureReductionCluster" {
  import FeatureReductionCluster = __esri.FeatureReductionCluster;
  export = FeatureReductionCluster;
}

declare module "esri/layers/support/FeatureReductionSelection" {
  import FeatureReductionSelection = __esri.FeatureReductionSelection;
  export = FeatureReductionSelection;
}

declare module "esri/layers/support/FeatureTemplate" {
  import FeatureTemplate = __esri.FeatureTemplate;
  export = FeatureTemplate;
}

declare module "esri/layers/support/FeatureType" {
  import FeatureType = __esri.FeatureType;
  export = FeatureType;
}

declare module "esri/layers/support/Field" {
  import Field = __esri.Field;
  export = Field;
}

declare module "esri/layers/support/FieldsIndex" {
  import FieldsIndex = __esri.FieldsIndex;
  export = FieldsIndex;
}

declare module "esri/layers/support/GeometryFieldsInfo" {
  import GeometryFieldsInfo = __esri.GeometryFieldsInfo;
  export = GeometryFieldsInfo;
}

declare module "esri/layers/support/ImageElement" {
  import ImageElement = __esri.ImageElement;
  export = ImageElement;
}

declare module "esri/layers/support/InheritedDomain" {
  import InheritedDomain = __esri.InheritedDomain;
  export = InheritedDomain;
}

declare module "esri/layers/support/KMLSublayer" {
  import KMLSublayer = __esri.KMLSublayer;
  export = KMLSublayer;
}

declare module "esri/layers/support/LabelClass" {
  import LabelClass = __esri.LabelClass;
  export = LabelClass;
}

declare module "esri/layers/support/LayerFloorInfo" {
  import LayerFloorInfo = __esri.LayerFloorInfo;
  export = LayerFloorInfo;
}

declare module "esri/layers/support/LevelLayerInfo" {
  import LevelLayerInfo = __esri.LevelLayerInfo;
  export = LevelLayerInfo;
}

declare module "esri/layers/support/LocalMediaElementSource" {
  import LocalMediaElementSource = __esri.LocalMediaElementSource;
  export = LocalMediaElementSource;
}

declare module "esri/layers/support/LOD" {
  import LOD = __esri.LOD;
  export = LOD;
}

declare module "esri/layers/support/MapImage" {
  import MapImage = __esri.MapImage;
  export = MapImage;
}

declare module "esri/layers/support/MosaicRule" {
  import MosaicRule = __esri.MosaicRule;
  export = MosaicRule;
}

declare module "esri/layers/support/MultidimensionalSubset" {
  import MultidimensionalSubset = __esri.MultidimensionalSubset;
  export = MultidimensionalSubset;
}

declare module "esri/layers/support/PixelBlock" {
  import PixelBlock = __esri.PixelBlock;
  export = PixelBlock;
}

declare module "esri/layers/support/PublishingInfo" {
  import PublishingInfo = __esri.PublishingInfo;
  export = PublishingInfo;
}

declare module "esri/layers/support/RangeDomain" {
  import RangeDomain = __esri.RangeDomain;
  export = RangeDomain;
}

declare module "esri/layers/support/RasterBandInfo" {
  import RasterBandInfo = __esri.RasterBandInfo;
  export = RasterBandInfo;
}

declare module "esri/layers/support/RasterFunction" {
  import RasterFunction = __esri.RasterFunction;
  export = RasterFunction;
}

declare module "esri/layers/support/RasterInfo" {
  import RasterInfo = __esri.RasterInfo;
  export = RasterInfo;
}

declare module "esri/layers/support/RasterSensorInfo" {
  import RasterSensorInfo = __esri.RasterSensorInfo;
  export = RasterSensorInfo;
}

declare module "esri/layers/support/Relationship" {
  import Relationship = __esri.Relationship;
  export = Relationship;
}

declare module "esri/layers/support/RouteStopSymbols" {
  import RouteStopSymbols = __esri.RouteStopSymbols;
  export = RouteStopSymbols;
}

declare module "esri/layers/support/RouteSymbols" {
  import RouteSymbols = __esri.RouteSymbols;
  export = RouteSymbols;
}

declare module "esri/layers/support/SceneFilter" {
  import SceneFilter = __esri.SceneFilter;
  export = SceneFilter;
}

declare module "esri/layers/support/SceneModification" {
  import SceneModification = __esri.SceneModification;
  export = SceneModification;
}

declare module "esri/layers/support/SceneModifications" {
  import SceneModifications = __esri.SceneModifications;
  export = SceneModifications;
}

declare module "esri/layers/support/SiteLayerInfo" {
  import SiteLayerInfo = __esri.SiteLayerInfo;
  export = SiteLayerInfo;
}

declare module "esri/layers/support/Sublayer" {
  import Sublayer = __esri.Sublayer;
  export = Sublayer;
}

declare module "esri/layers/support/Subtype" {
  import Subtype = __esri.Subtype;
  export = Subtype;
}

declare module "esri/layers/support/SubtypeSublayer" {
  import SubtypeSublayer = __esri.SubtypeSublayer;
  export = SubtypeSublayer;
}

declare module "esri/layers/support/TelemetryData" {
  import TelemetryData = __esri.TelemetryData;
  export = TelemetryData;
}

declare module "esri/layers/support/TelemetryDisplay" {
  import TelemetryDisplay = __esri.TelemetryDisplay;
  export = TelemetryDisplay;
}

declare module "esri/layers/support/TileInfo" {
  import TileInfo = __esri.TileInfo;
  export = TileInfo;
}

declare module "esri/layers/support/TileMatrixSet" {
  import TileMatrixSet = __esri.TileMatrixSet;
  export = TileMatrixSet;
}

declare module "esri/layers/support/TimeInfo" {
  import TimeInfo = __esri.TimeInfo;
  export = TimeInfo;
}

declare module "esri/layers/support/VideoElement" {
  import VideoElement = __esri.VideoElement;
  export = VideoElement;
}

declare module "esri/layers/support/VideoTimeExtent" {
  import VideoTimeExtent = __esri.VideoTimeExtent;
  export = VideoTimeExtent;
}

declare module "esri/layers/ViewshedLayer" {
  import ViewshedLayer = __esri.ViewshedLayer;
  export = ViewshedLayer;
}

declare module "esri/layers/voxel/VoxelDynamicSection" {
  import VoxelDynamicSection = __esri.VoxelDynamicSection;
  export = VoxelDynamicSection;
}

declare module "esri/layers/voxel/VoxelSlice" {
  import VoxelSlice = __esri.VoxelSlice;
  export = VoxelSlice;
}

declare module "esri/layers/voxel/VoxelTransferFunctionStyle" {
  import VoxelTransferFunctionStyle = __esri.VoxelTransferFunctionStyle;
  export = VoxelTransferFunctionStyle;
}

declare module "esri/layers/voxel/VoxelVariableStyle" {
  import VoxelVariableStyle = __esri.VoxelVariableStyle;
  export = VoxelVariableStyle;
}

declare module "esri/layers/voxel/VoxelVolumeStyle" {
  import VoxelVolumeStyle = __esri.VoxelVolumeStyle;
  export = VoxelVolumeStyle;
}

declare module "esri/layers/support/WMSSublayer" {
  import WMSSublayer = __esri.WMSSublayer;
  export = WMSSublayer;
}

declare module "esri/layers/support/WMTSStyle" {
  import WMTSStyle = __esri.WMTSStyle;
  export = WMTSStyle;
}

declare module "esri/layers/support/WMTSSublayer" {
  import WMTSSublayer = __esri.WMTSSublayer;
  export = WMTSSublayer;
}

declare module "esri/layers/TileLayer" {
  import TileLayer = __esri.TileLayer;
  export = TileLayer;
}

declare module "esri/layers/UnknownLayer" {
  import UnknownLayer = __esri.UnknownLayer;
  export = UnknownLayer;
}

declare module "esri/layers/UnsupportedLayer" {
  import UnsupportedLayer = __esri.UnsupportedLayer;
  export = UnsupportedLayer;
}

declare module "esri/layers/VectorTileLayer" {
  import VectorTileLayer = __esri.VectorTileLayer;
  export = VectorTileLayer;
}

declare module "esri/layers/VideoLayer" {
  import VideoLayer = __esri.VideoLayer;
  export = VideoLayer;
}

declare module "esri/layers/VoxelLayer" {
  import VoxelLayer = __esri.VoxelLayer;
  export = VoxelLayer;
}

declare module "esri/layers/WCSLayer" {
  import WCSLayer = __esri.WCSLayer;
  export = WCSLayer;
}

declare module "esri/layers/WebTileLayer" {
  import WebTileLayer = __esri.WebTileLayer;
  export = WebTileLayer;
}

declare module "esri/layers/WFSLayer" {
  import WFSLayer = __esri.WFSLayer;
  export = WFSLayer;
}

declare module "esri/layers/WMSLayer" {
  import WMSLayer = __esri.WMSLayer;
  export = WMSLayer;
}

declare module "esri/layers/WMTSLayer" {
  import WMTSLayer = __esri.WMTSLayer;
  export = WMTSLayer;
}

declare module "esri/Map" {
  import Map = __esri.Map;
  export = Map;
}

declare module "esri/networks/Network" {
  import Network = __esri.Network;
  export = Network;
}

declare module "esri/networks/UtilityNetwork" {
  import UtilityNetwork = __esri.UtilityNetwork;
  export = UtilityNetwork;
}

declare module "esri/networks/support/NamedTraceConfiguration" {
  import NamedTraceConfiguration = __esri.NamedTraceConfiguration;
  export = NamedTraceConfiguration;
}

declare module "esri/networks/support/NetworkSystemLayers" {
  import NetworkSystemLayers = __esri.NetworkSystemLayers;
  export = NetworkSystemLayers;
}

declare module "esri/networks/support/Terminal" {
  import Terminal = __esri.Terminal;
  export = Terminal;
}

declare module "esri/networks/support/TerminalConfiguration" {
  import TerminalConfiguration = __esri.TerminalConfiguration;
  export = TerminalConfiguration;
}

declare module "esri/networks/support/TopologyValidationJobInfo" {
  import TopologyValidationJobInfo = __esri.TopologyValidationJobInfo;
  export = TopologyValidationJobInfo;
}

declare module "esri/networks/support/TraceConfiguration" {
  import TraceConfiguration = __esri.TraceConfiguration;
  export = TraceConfiguration;
}

declare module "esri/networks/support/TraceJobInfo" {
  import TraceJobInfo = __esri.TraceJobInfo;
  export = TraceJobInfo;
}

declare module "esri/networks/support/UNTraceConfiguration" {
  import UNTraceConfiguration = __esri.UNTraceConfiguration;
  export = UNTraceConfiguration;
}

declare module "esri/popup/content/AttachmentsContent" {
  import AttachmentsContent = __esri.AttachmentsContent;
  export = AttachmentsContent;
}

declare module "esri/popup/content/BarChartMediaInfo" {
  import BarChartMediaInfo = __esri.BarChartMediaInfo;
  export = BarChartMediaInfo;
}

declare module "esri/popup/content/ColumnChartMediaInfo" {
  import ColumnChartMediaInfo = __esri.ColumnChartMediaInfo;
  export = ColumnChartMediaInfo;
}

declare module "esri/popup/content/Content" {
  import Content = __esri.Content;
  export = Content;
}

declare module "esri/popup/content/CustomContent" {
  import CustomContent = __esri.CustomContent;
  export = CustomContent;
}

declare module "esri/popup/content/ExpressionContent" {
  import ExpressionContent = __esri.ExpressionContent;
  export = ExpressionContent;
}

declare module "esri/popup/content/FieldsContent" {
  import FieldsContent = __esri.FieldsContent;
  export = FieldsContent;
}

declare module "esri/popup/content/ImageMediaInfo" {
  import ImageMediaInfo = __esri.ImageMediaInfo;
  export = ImageMediaInfo;
}

declare module "esri/popup/content/LineChartMediaInfo" {
  import LineChartMediaInfo = __esri.LineChartMediaInfo;
  export = LineChartMediaInfo;
}

declare module "esri/popup/content/MediaContent" {
  import MediaContent = __esri.MediaContent;
  export = MediaContent;
}

declare module "esri/popup/content/PieChartMediaInfo" {
  import PieChartMediaInfo = __esri.PieChartMediaInfo;
  export = PieChartMediaInfo;
}

declare module "esri/popup/content/RelationshipContent" {
  import RelationshipContent = __esri.RelationshipContent;
  export = RelationshipContent;
}

declare module "esri/popup/content/support/ChartMediaInfoValue" {
  import ChartMediaInfoValue = __esri.ChartMediaInfoValue;
  export = ChartMediaInfoValue;
}

declare module "esri/popup/content/support/ChartMediaInfoValueSeries" {
  import ChartMediaInfoValueSeries = __esri.ChartMediaInfoValueSeries;
  export = ChartMediaInfoValueSeries;
}

declare module "esri/popup/content/support/ImageMediaInfoValue" {
  import ImageMediaInfoValue = __esri.ImageMediaInfoValue;
  export = ImageMediaInfoValue;
}

declare module "esri/popup/content/TextContent" {
  import TextContent = __esri.TextContent;
  export = TextContent;
}

declare module "esri/popup/content/UtilityNetworkAssociationsContent" {
  import UtilityNetworkAssociationsContent = __esri.UtilityNetworkAssociationsContent;
  export = UtilityNetworkAssociationsContent;
}

declare module "esri/popup/ElementExpressionInfo" {
  import ElementExpressionInfo = __esri.ElementExpressionInfo;
  export = ElementExpressionInfo;
}

declare module "esri/popup/ExpressionInfo" {
  import popupExpressionInfo = __esri.popupExpressionInfo;
  export = popupExpressionInfo;
}

declare module "esri/popup/FieldInfo" {
  import FieldInfo = __esri.FieldInfo;
  export = FieldInfo;
}

declare module "esri/popup/LayerOptions" {
  import LayerOptions = __esri.LayerOptions;
  export = LayerOptions;
}

declare module "esri/popup/RelatedRecordsInfo" {
  import RelatedRecordsInfo = __esri.RelatedRecordsInfo;
  export = RelatedRecordsInfo;
}

declare module "esri/popup/support/FieldInfoFormat" {
  import FieldInfoFormat = __esri.FieldInfoFormat;
  export = FieldInfoFormat;
}

declare module "esri/popup/support/RelatedRecordsInfoFieldOrder" {
  import RelatedRecordsInfoFieldOrder = __esri.RelatedRecordsInfoFieldOrder;
  export = RelatedRecordsInfoFieldOrder;
}

declare module "esri/PopupTemplate" {
  import PopupTemplate = __esri.PopupTemplate;
  export = PopupTemplate;
}

declare module "esri/portal/Portal" {
  import Portal = __esri.Portal;
  export = Portal;
}

declare module "esri/portal/PortalFolder" {
  import PortalFolder = __esri.PortalFolder;
  export = PortalFolder;
}

declare module "esri/portal/PortalGroup" {
  import PortalGroup = __esri.PortalGroup;
  export = PortalGroup;
}

declare module "esri/portal/PortalItem" {
  import PortalItem = __esri.PortalItem;
  export = PortalItem;
}

declare module "esri/portal/PortalItemResource" {
  import PortalItemResource = __esri.PortalItemResource;
  export = PortalItemResource;
}

declare module "esri/portal/PortalQueryParams" {
  import PortalQueryParams = __esri.PortalQueryParams;
  export = PortalQueryParams;
}

declare module "esri/portal/PortalQueryResult" {
  import PortalQueryResult = __esri.PortalQueryResult;
  export = PortalQueryResult;
}

declare module "esri/portal/PortalRating" {
  import PortalRating = __esri.PortalRating;
  export = PortalRating;
}

declare module "esri/portal/PortalUser" {
  import PortalUser = __esri.PortalUser;
  export = PortalUser;
}

declare module "esri/renderers/ClassBreaksRenderer" {
  import ClassBreaksRenderer = __esri.ClassBreaksRenderer;
  export = ClassBreaksRenderer;
}

declare module "esri/renderers/DictionaryRenderer" {
  import DictionaryRenderer = __esri.DictionaryRenderer;
  export = DictionaryRenderer;
}

declare module "esri/renderers/DotDensityRenderer" {
  import DotDensityRenderer = __esri.DotDensityRenderer;
  export = DotDensityRenderer;
}

declare module "esri/renderers/FlowRenderer" {
  import FlowRenderer = __esri.FlowRenderer;
  export = FlowRenderer;
}

declare module "esri/renderers/HeatmapRenderer" {
  import HeatmapRenderer = __esri.HeatmapRenderer;
  export = HeatmapRenderer;
}

declare module "esri/renderers/PieChartRenderer" {
  import PieChartRenderer = __esri.PieChartRenderer;
  export = PieChartRenderer;
}

declare module "esri/renderers/PointCloudClassBreaksRenderer" {
  import PointCloudClassBreaksRenderer = __esri.PointCloudClassBreaksRenderer;
  export = PointCloudClassBreaksRenderer;
}

declare module "esri/renderers/PointCloudRenderer" {
  import PointCloudRenderer = __esri.PointCloudRenderer;
  export = PointCloudRenderer;
}

declare module "esri/renderers/PointCloudRGBRenderer" {
  import PointCloudRGBRenderer = __esri.PointCloudRGBRenderer;
  export = PointCloudRGBRenderer;
}

declare module "esri/renderers/PointCloudStretchRenderer" {
  import PointCloudStretchRenderer = __esri.PointCloudStretchRenderer;
  export = PointCloudStretchRenderer;
}

declare module "esri/renderers/PointCloudUniqueValueRenderer" {
  import PointCloudUniqueValueRenderer = __esri.PointCloudUniqueValueRenderer;
  export = PointCloudUniqueValueRenderer;
}

declare module "esri/renderers/RasterColormapRenderer" {
  import RasterColormapRenderer = __esri.RasterColormapRenderer;
  export = RasterColormapRenderer;
}

declare module "esri/renderers/support/RasterPresetRenderer" {
  import RasterPresetRenderer = __esri.RasterPresetRenderer;
  export = RasterPresetRenderer;
}

declare module "esri/renderers/RasterShadedReliefRenderer" {
  import RasterShadedReliefRenderer = __esri.RasterShadedReliefRenderer;
  export = RasterShadedReliefRenderer;
}

declare module "esri/renderers/RasterStretchRenderer" {
  import RasterStretchRenderer = __esri.RasterStretchRenderer;
  export = RasterStretchRenderer;
}

declare module "esri/renderers/Renderer" {
  import Renderer = __esri.Renderer;
  export = Renderer;
}

declare module "esri/renderers/SimpleRenderer" {
  import SimpleRenderer = __esri.SimpleRenderer;
  export = SimpleRenderer;
}

declare module "esri/renderers/support/AttributeColorInfo" {
  import AttributeColorInfo = __esri.AttributeColorInfo;
  export = AttributeColorInfo;
}

declare module "esri/renderers/support/AuthoringInfo" {
  import AuthoringInfo = __esri.AuthoringInfo;
  export = AuthoringInfo;
}

declare module "esri/renderers/support/AuthoringInfoVisualVariable" {
  import AuthoringInfoVisualVariable = __esri.AuthoringInfoVisualVariable;
  export = AuthoringInfoVisualVariable;
}

declare module "esri/renderers/support/ClassBreakInfo" {
  import ClassBreakInfo = __esri.ClassBreakInfo;
  export = ClassBreakInfo;
}

declare module "esri/renderers/support/ColormapInfo" {
  import ColormapInfo = __esri.ColormapInfo;
  export = ColormapInfo;
}

declare module "esri/renderers/support/HeatmapColorStop" {
  import HeatmapColorStop = __esri.HeatmapColorStop;
  export = HeatmapColorStop;
}

declare module "esri/renderers/support/UniqueValue" {
  import UniqueValue = __esri.UniqueValue;
  export = UniqueValue;
}

declare module "esri/renderers/support/UniqueValueClass" {
  import UniqueValueClass = __esri.UniqueValueClass;
  export = UniqueValueClass;
}

declare module "esri/renderers/support/UniqueValueGroup" {
  import UniqueValueGroup = __esri.UniqueValueGroup;
  export = UniqueValueGroup;
}

declare module "esri/renderers/support/UniqueValueInfo" {
  import UniqueValueInfo = __esri.UniqueValueInfo;
  export = UniqueValueInfo;
}

declare module "esri/renderers/UniqueValueRenderer" {
  import UniqueValueRenderer = __esri.UniqueValueRenderer;
  export = UniqueValueRenderer;
}

declare module "esri/renderers/VectorFieldRenderer" {
  import VectorFieldRenderer = __esri.VectorFieldRenderer;
  export = VectorFieldRenderer;
}

declare module "esri/renderers/visualVariables/ColorVariable" {
  import ColorVariable = __esri.ColorVariable;
  export = ColorVariable;
}

declare module "esri/renderers/visualVariables/OpacityVariable" {
  import OpacityVariable = __esri.OpacityVariable;
  export = OpacityVariable;
}

declare module "esri/renderers/visualVariables/RotationVariable" {
  import RotationVariable = __esri.RotationVariable;
  export = RotationVariable;
}

declare module "esri/renderers/visualVariables/SizeVariable" {
  import SizeVariable = __esri.SizeVariable;
  export = SizeVariable;
}

declare module "esri/renderers/visualVariables/support/ColorSizeStop" {
  import ColorSizeStop = __esri.ColorSizeStop;
  export = ColorSizeStop;
}

declare module "esri/renderers/visualVariables/support/ColorStop" {
  import ColorStop = __esri.ColorStop;
  export = ColorStop;
}

declare module "esri/renderers/visualVariables/support/OpacityStop" {
  import OpacityStop = __esri.OpacityStop;
  export = OpacityStop;
}

declare module "esri/renderers/visualVariables/support/SizeStop" {
  import SizeStop = __esri.SizeStop;
  export = SizeStop;
}

declare module "esri/renderers/visualVariables/VisualVariable" {
  import VisualVariable = __esri.VisualVariable;
  export = VisualVariable;
}

declare module "esri/rest/featureService/FeatureService" {
  import FeatureService = __esri.FeatureService;
  export = FeatureService;
}

declare module "esri/rest/knowledgeGraph/DataModel" {
  import DataModel = __esri.DataModel;
  export = DataModel;
}

declare module "esri/rest/knowledgeGraph/Entity" {
  import Entity = __esri.Entity;
  export = Entity;
}

declare module "esri/rest/knowledgeGraph/EntityType" {
  import EntityType = __esri.EntityType;
  export = EntityType;
}

declare module "esri/rest/knowledgeGraph/FieldIndex" {
  import FieldIndex = __esri.FieldIndex;
  export = FieldIndex;
}

declare module "esri/rest/knowledgeGraph/GraphApplyEdits" {
  import GraphApplyEdits = __esri.GraphApplyEdits;
  export = GraphApplyEdits;
}

declare module "esri/rest/knowledgeGraph/GraphApplyEditsResult" {
  import GraphApplyEditsResult = __esri.GraphApplyEditsResult;
  export = GraphApplyEditsResult;
}

declare module "esri/rest/knowledgeGraph/GraphNamedObject" {
  import GraphNamedObject = __esri.GraphNamedObject;
  export = GraphNamedObject;
}

declare module "esri/rest/knowledgeGraph/GraphObject" {
  import GraphObject = __esri.GraphObject;
  export = GraphObject;
}

declare module "esri/rest/knowledgeGraph/GraphObjectType" {
  import GraphObjectType = __esri.GraphObjectType;
  export = GraphObjectType;
}

declare module "esri/rest/knowledgeGraph/GraphProperty" {
  import GraphProperty = __esri.GraphProperty;
  export = GraphProperty;
}

declare module "esri/rest/knowledgeGraph/GraphQuery" {
  import GraphQuery = __esri.GraphQuery;
  export = GraphQuery;
}

declare module "esri/rest/knowledgeGraph/GraphQueryResult" {
  import GraphQueryResult = __esri.GraphQueryResult;
  export = GraphQueryResult;
}

declare module "esri/rest/knowledgeGraph/GraphQueryStreaming" {
  import GraphQueryStreaming = __esri.GraphQueryStreaming;
  export = GraphQueryStreaming;
}

declare module "esri/rest/knowledgeGraph/GraphQueryStreamingResult" {
  import GraphQueryStreamingResult = __esri.GraphQueryStreamingResult;
  export = GraphQueryStreamingResult;
}

declare module "esri/rest/knowledgeGraph/GraphSearch" {
  import GraphSearch = __esri.GraphSearch;
  export = GraphSearch;
}

declare module "esri/rest/knowledgeGraph/GraphSearchStreaming" {
  import GraphSearchStreaming = __esri.GraphSearchStreaming;
  export = GraphSearchStreaming;
}

declare module "esri/rest/knowledgeGraph/InputQuantizationParameters" {
  import InputQuantizationParameters = __esri.InputQuantizationParameters;
  export = InputQuantizationParameters;
}

declare module "esri/rest/knowledgeGraph/KnowledgeGraph" {
  import KnowledgeGraph = __esri.KnowledgeGraph;
  export = KnowledgeGraph;
}

declare module "esri/rest/knowledgeGraph/OutputQuantizationParameters" {
  import OutputQuantizationParameters = __esri.OutputQuantizationParameters;
  export = OutputQuantizationParameters;
}

declare module "esri/rest/knowledgeGraph/Path" {
  import Path = __esri.Path;
  export = Path;
}

declare module "esri/rest/knowledgeGraph/Relationship" {
  import knowledgeGraphRelationship = __esri.knowledgeGraphRelationship;
  export = knowledgeGraphRelationship;
}

declare module "esri/rest/knowledgeGraph/RelationshipType" {
  import RelationshipType = __esri.RelationshipType;
  export = RelationshipType;
}

declare module "esri/rest/knowledgeGraph/SearchIndex" {
  import SearchIndex = __esri.SearchIndex;
  export = SearchIndex;
}

declare module "esri/rest/knowledgeGraph/ServiceDefinition" {
  import ServiceDefinition = __esri.ServiceDefinition;
  export = ServiceDefinition;
}

declare module "esri/rest/networks/support/AggregatedGeometry" {
  import AggregatedGeometry = __esri.AggregatedGeometry;
  export = AggregatedGeometry;
}

declare module "esri/rest/networks/support/Association" {
  import Association = __esri.Association;
  export = Association;
}

declare module "esri/rest/networks/support/AssociationGeometriesResult" {
  import AssociationGeometriesResult = __esri.AssociationGeometriesResult;
  export = AssociationGeometriesResult;
}

declare module "esri/rest/networks/support/FunctionResult" {
  import FunctionResult = __esri.FunctionResult;
  export = FunctionResult;
}

declare module "esri/rest/networks/support/NetworkElement" {
  import NetworkElement = __esri.NetworkElement;
  export = NetworkElement;
}

declare module "esri/rest/networks/support/QueryAssociationsParameters" {
  import QueryAssociationsParameters = __esri.QueryAssociationsParameters;
  export = QueryAssociationsParameters;
}

declare module "esri/rest/networks/support/QueryAssociationsResult" {
  import QueryAssociationsResult = __esri.QueryAssociationsResult;
  export = QueryAssociationsResult;
}

declare module "esri/rest/networks/support/SynthesizeAssociationGeometriesParameters" {
  import SynthesizeAssociationGeometriesParameters = __esri.SynthesizeAssociationGeometriesParameters;
  export = SynthesizeAssociationGeometriesParameters;
}

declare module "esri/rest/networks/support/TraceLocation" {
  import TraceLocation = __esri.TraceLocation;
  export = TraceLocation;
}

declare module "esri/rest/networks/support/TraceParameters" {
  import TraceParameters = __esri.TraceParameters;
  export = TraceParameters;
}

declare module "esri/rest/networks/support/TraceResult" {
  import TraceResult = __esri.TraceResult;
  export = TraceResult;
}

declare module "esri/rest/networks/support/ValidateNetworkTopologyParameters" {
  import ValidateNetworkTopologyParameters = __esri.ValidateNetworkTopologyParameters;
  export = ValidateNetworkTopologyParameters;
}

declare module "esri/rest/query/support/AttachmentInfo" {
  import AttachmentInfo = __esri.AttachmentInfo;
  export = AttachmentInfo;
}

declare module "esri/rest/support/AddressCandidate" {
  import AddressCandidate = __esri.AddressCandidate;
  export = AddressCandidate;
}

declare module "esri/rest/support/AlgorithmicColorRamp" {
  import AlgorithmicColorRamp = __esri.AlgorithmicColorRamp;
  export = AlgorithmicColorRamp;
}

declare module "esri/rest/support/ArealUnit" {
  import ArealUnit = __esri.ArealUnit;
  export = ArealUnit;
}

declare module "esri/rest/support/AreasAndLengthsParameters" {
  import AreasAndLengthsParameters = __esri.AreasAndLengthsParameters;
  export = AreasAndLengthsParameters;
}

declare module "esri/rest/support/AttachmentQuery" {
  import AttachmentQuery = __esri.AttachmentQuery;
  export = AttachmentQuery;
}

declare module "esri/rest/support/BufferParameters" {
  import BufferParameters = __esri.BufferParameters;
  export = BufferParameters;
}

declare module "esri/rest/support/CameraInfo" {
  import CameraInfo = __esri.CameraInfo;
  export = CameraInfo;
}

declare module "esri/rest/support/ClosestFacilityParameters" {
  import ClosestFacilityParameters = __esri.ClosestFacilityParameters;
  export = ClosestFacilityParameters;
}

declare module "esri/rest/support/ClosestFacilitySolveResult" {
  import ClosestFacilitySolveResult = __esri.ClosestFacilitySolveResult;
  export = ClosestFacilitySolveResult;
}

declare module "esri/rest/support/ColorRamp" {
  import ColorRamp = __esri.ColorRamp;
  export = ColorRamp;
}

declare module "esri/rest/support/DataFile" {
  import DataFile = __esri.DataFile;
  export = DataFile;
}

declare module "esri/rest/support/DataLayer" {
  import DataLayer = __esri.DataLayer;
  export = DataLayer;
}

declare module "esri/rest/support/DensifyParameters" {
  import DensifyParameters = __esri.DensifyParameters;
  export = DensifyParameters;
}

declare module "esri/rest/support/DirectionLine" {
  import DirectionLine = __esri.DirectionLine;
  export = DirectionLine;
}

declare module "esri/rest/support/DirectionPoint" {
  import DirectionPoint = __esri.DirectionPoint;
  export = DirectionPoint;
}

declare module "esri/rest/support/DirectionsEvent" {
  import DirectionsEvent = __esri.DirectionsEvent;
  export = DirectionsEvent;
}

declare module "esri/rest/support/DirectionsFeature" {
  import DirectionsFeature = __esri.DirectionsFeature;
  export = DirectionsFeature;
}

declare module "esri/rest/support/DirectionsFeatureSet" {
  import DirectionsFeatureSet = __esri.DirectionsFeatureSet;
  export = DirectionsFeatureSet;
}

declare module "esri/rest/support/DistanceParameters" {
  import DistanceParameters = __esri.DistanceParameters;
  export = DistanceParameters;
}

declare module "esri/rest/support/FeatureSet" {
  import FeatureSet = __esri.FeatureSet;
  export = FeatureSet;
}

declare module "esri/rest/support/FetchPlaceParameters" {
  import FetchPlaceParameters = __esri.FetchPlaceParameters;
  export = FetchPlaceParameters;
}

declare module "esri/rest/support/FindImagesParameters" {
  import FindImagesParameters = __esri.FindImagesParameters;
  export = FindImagesParameters;
}

declare module "esri/rest/support/FindImagesResult" {
  import FindImagesResult = __esri.FindImagesResult;
  export = FindImagesResult;
}

declare module "esri/rest/support/FindParameters" {
  import FindParameters = __esri.FindParameters;
  export = FindParameters;
}

declare module "esri/rest/support/FindResult" {
  import FindResult = __esri.FindResult;
  export = FindResult;
}

declare module "esri/rest/support/GeneralizeParameters" {
  import GeneralizeParameters = __esri.GeneralizeParameters;
  export = GeneralizeParameters;
}

declare module "esri/rest/support/GPMessage" {
  import GPMessage = __esri.GPMessage;
  export = GPMessage;
}

declare module "esri/rest/support/IdentifyParameters" {
  import IdentifyParameters = __esri.IdentifyParameters;
  export = IdentifyParameters;
}

declare module "esri/rest/support/IdentifyResult" {
  import IdentifyResult = __esri.IdentifyResult;
  export = IdentifyResult;
}

declare module "esri/rest/support/ImageAngleParameters" {
  import ImageAngleParameters = __esri.ImageAngleParameters;
  export = ImageAngleParameters;
}

declare module "esri/rest/support/ImageAngleResult" {
  import ImageAngleResult = __esri.ImageAngleResult;
  export = ImageAngleResult;
}

declare module "esri/rest/support/ImageAreaParameters" {
  import ImageAreaParameters = __esri.ImageAreaParameters;
  export = ImageAreaParameters;
}

declare module "esri/rest/support/ImageAreaResult" {
  import ImageAreaResult = __esri.ImageAreaResult;
  export = ImageAreaResult;
}

declare module "esri/rest/support/ImageBoundaryParameters" {
  import ImageBoundaryParameters = __esri.ImageBoundaryParameters;
  export = ImageBoundaryParameters;
}

declare module "esri/rest/support/ImageBoundaryResult" {
  import ImageBoundaryResult = __esri.ImageBoundaryResult;
  export = ImageBoundaryResult;
}

declare module "esri/rest/support/ImageDistanceParameters" {
  import ImageDistanceParameters = __esri.ImageDistanceParameters;
  export = ImageDistanceParameters;
}

declare module "esri/rest/support/ImageDistanceResult" {
  import ImageDistanceResult = __esri.ImageDistanceResult;
  export = ImageDistanceResult;
}

declare module "esri/rest/support/ImageGPSInfo" {
  import ImageGPSInfo = __esri.ImageGPSInfo;
  export = ImageGPSInfo;
}

declare module "esri/rest/support/ImageGPSInfoParameters" {
  import ImageGPSInfoParameters = __esri.ImageGPSInfoParameters;
  export = ImageGPSInfoParameters;
}

declare module "esri/rest/support/ImageGPSInfoResult" {
  import ImageGPSInfoResult = __esri.ImageGPSInfoResult;
  export = ImageGPSInfoResult;
}

declare module "esri/rest/support/ImageHeightParameters" {
  import ImageHeightParameters = __esri.ImageHeightParameters;
  export = ImageHeightParameters;
}

declare module "esri/rest/support/ImageHeightResult" {
  import ImageHeightResult = __esri.ImageHeightResult;
  export = ImageHeightResult;
}

declare module "esri/rest/support/ImageHistogramParameters" {
  import ImageHistogramParameters = __esri.ImageHistogramParameters;
  export = ImageHistogramParameters;
}

declare module "esri/rest/support/ImageIdentifyParameters" {
  import ImageIdentifyParameters = __esri.ImageIdentifyParameters;
  export = ImageIdentifyParameters;
}

declare module "esri/rest/support/ImageIdentifyResult" {
  import ImageIdentifyResult = __esri.ImageIdentifyResult;
  export = ImageIdentifyResult;
}

declare module "esri/rest/support/ImageInspectionInfo" {
  import ImageInspectionInfo = __esri.ImageInspectionInfo;
  export = ImageInspectionInfo;
}

declare module "esri/rest/support/ImageParameters" {
  import ImageParameters = __esri.ImageParameters;
  export = ImageParameters;
}

declare module "esri/rest/support/ImagePixelLocationParameters" {
  import ImagePixelLocationParameters = __esri.ImagePixelLocationParameters;
  export = ImagePixelLocationParameters;
}

declare module "esri/rest/support/ImagePixelLocationResult" {
  import ImagePixelLocationResult = __esri.ImagePixelLocationResult;
  export = ImagePixelLocationResult;
}

declare module "esri/rest/support/ImagePointParameters" {
  import ImagePointParameters = __esri.ImagePointParameters;
  export = ImagePointParameters;
}

declare module "esri/rest/support/ImagePointResult" {
  import ImagePointResult = __esri.ImagePointResult;
  export = ImagePointResult;
}

declare module "esri/rest/support/ImageSample" {
  import ImageSample = __esri.ImageSample;
  export = ImageSample;
}

declare module "esri/rest/support/ImageSampleParameters" {
  import ImageSampleParameters = __esri.ImageSampleParameters;
  export = ImageSampleParameters;
}

declare module "esri/rest/support/ImageSampleResult" {
  import ImageSampleResult = __esri.ImageSampleResult;
  export = ImageSampleResult;
}

declare module "esri/rest/support/ImageToMapMultirayParameters" {
  import ImageToMapMultirayParameters = __esri.ImageToMapMultirayParameters;
  export = ImageToMapMultirayParameters;
}

declare module "esri/rest/support/ImageToMapParameters" {
  import ImageToMapParameters = __esri.ImageToMapParameters;
  export = ImageToMapParameters;
}

declare module "esri/rest/support/ImageUrlParameters" {
  import ImageUrlParameters = __esri.ImageUrlParameters;
  export = ImageUrlParameters;
}

declare module "esri/rest/support/ImageUrlResult" {
  import ImageUrlResult = __esri.ImageUrlResult;
  export = ImageUrlResult;
}

declare module "esri/rest/support/JobInfo" {
  import JobInfo = __esri.JobInfo;
  export = JobInfo;
}

declare module "esri/rest/support/LegendLayer" {
  import LegendLayer = __esri.LegendLayer;
  export = LegendLayer;
}

declare module "esri/rest/support/LengthsParameters" {
  import LengthsParameters = __esri.LengthsParameters;
  export = LengthsParameters;
}

declare module "esri/rest/support/LinearUnit" {
  import LinearUnit = __esri.LinearUnit;
  export = LinearUnit;
}

declare module "esri/rest/support/MapToImageParameters" {
  import MapToImageParameters = __esri.MapToImageParameters;
  export = MapToImageParameters;
}

declare module "esri/rest/support/MeasureAreaFromImageResult" {
  import MeasureAreaFromImageResult = __esri.MeasureAreaFromImageResult;
  export = MeasureAreaFromImageResult;
}

declare module "esri/rest/support/MeasureFromImageParameters" {
  import MeasureFromImageParameters = __esri.MeasureFromImageParameters;
  export = MeasureFromImageParameters;
}

declare module "esri/rest/support/MeasureLengthFromImageResult" {
  import MeasureLengthFromImageResult = __esri.MeasureLengthFromImageResult;
  export = MeasureLengthFromImageResult;
}

declare module "esri/rest/support/MultipartColorRamp" {
  import MultipartColorRamp = __esri.MultipartColorRamp;
  export = MultipartColorRamp;
}

declare module "esri/rest/support/NAMessage" {
  import NAMessage = __esri.NAMessage;
  export = NAMessage;
}

declare module "esri/rest/support/NetworkFeatureSet" {
  import NetworkFeatureSet = __esri.NetworkFeatureSet;
  export = NetworkFeatureSet;
}

declare module "esri/rest/support/NetworkUrl" {
  import NetworkUrl = __esri.NetworkUrl;
  export = NetworkUrl;
}

declare module "esri/rest/support/OffsetParameters" {
  import OffsetParameters = __esri.OffsetParameters;
  export = OffsetParameters;
}

declare module "esri/rest/support/ParameterValue" {
  import ParameterValue = __esri.ParameterValue;
  export = ParameterValue;
}

declare module "esri/rest/support/PlaceResult" {
  import PlaceResult = __esri.PlaceResult;
  export = PlaceResult;
}

declare module "esri/rest/support/PlacesParameters" {
  import PlacesParameters = __esri.PlacesParameters;
  export = PlacesParameters;
}

declare module "esri/rest/support/PlacesQueryParameters" {
  import PlacesQueryParameters = __esri.PlacesQueryParameters;
  export = PlacesQueryParameters;
}

declare module "esri/rest/support/PlacesQueryResult" {
  import PlacesQueryResult = __esri.PlacesQueryResult;
  export = PlacesQueryResult;
}

declare module "esri/rest/support/PointBarrier" {
  import PointBarrier = __esri.PointBarrier;
  export = PointBarrier;
}

declare module "esri/rest/support/PolygonBarrier" {
  import PolygonBarrier = __esri.PolygonBarrier;
  export = PolygonBarrier;
}

declare module "esri/rest/support/PolylineBarrier" {
  import PolylineBarrier = __esri.PolylineBarrier;
  export = PolylineBarrier;
}

declare module "esri/rest/support/PrintParameters" {
  import PrintParameters = __esri.PrintParameters;
  export = PrintParameters;
}

declare module "esri/rest/support/PrintTemplate" {
  import PrintTemplate = __esri.PrintTemplate;
  export = PrintTemplate;
}

declare module "esri/rest/support/ProjectParameters" {
  import ProjectParameters = __esri.ProjectParameters;
  export = ProjectParameters;
}

declare module "esri/rest/support/Query" {
  import Query = __esri.Query;
  export = Query;
}

declare module "esri/rest/support/RasterData" {
  import RasterData = __esri.RasterData;
  export = RasterData;
}

declare module "esri/rest/support/RelationParameters" {
  import RelationParameters = __esri.RelationParameters;
  export = RelationParameters;
}

declare module "esri/rest/support/RelationshipQuery" {
  import RelationshipQuery = __esri.RelationshipQuery;
  export = RelationshipQuery;
}

declare module "esri/rest/support/RouteInfo" {
  import RouteInfo = __esri.RouteInfo;
  export = RouteInfo;
}

declare module "esri/rest/support/RouteParameters" {
  import RouteParameters = __esri.RouteParameters;
  export = RouteParameters;
}

declare module "esri/rest/support/RouteResult" {
  import RouteResult = __esri.RouteResult;
  export = RouteResult;
}

declare module "esri/rest/support/RouteSolveResult" {
  import RouteSolveResult = __esri.RouteSolveResult;
  export = RouteSolveResult;
}

declare module "esri/rest/support/ServiceAreaParameters" {
  import ServiceAreaParameters = __esri.ServiceAreaParameters;
  export = ServiceAreaParameters;
}

declare module "esri/rest/support/ServiceAreaSolveResult" {
  import ServiceAreaSolveResult = __esri.ServiceAreaSolveResult;
  export = ServiceAreaSolveResult;
}

declare module "esri/rest/support/StatisticDefinition" {
  import StatisticDefinition = __esri.StatisticDefinition;
  export = StatisticDefinition;
}

declare module "esri/rest/support/Stop" {
  import Stop = __esri.Stop;
  export = Stop;
}

declare module "esri/rest/support/TopFeaturesQuery" {
  import TopFeaturesQuery = __esri.TopFeaturesQuery;
  export = TopFeaturesQuery;
}

declare module "esri/rest/support/TopFilter" {
  import TopFilter = __esri.TopFilter;
  export = TopFilter;
}

declare module "esri/rest/support/TravelMode" {
  import TravelMode = __esri.TravelMode;
  export = TravelMode;
}

declare module "esri/rest/support/TrimExtendParameters" {
  import TrimExtendParameters = __esri.TrimExtendParameters;
  export = TrimExtendParameters;
}

declare module "esri/support/actions/ActionBase" {
  import ActionBase = __esri.ActionBase;
  export = ActionBase;
}

declare module "esri/support/actions/ActionButton" {
  import ActionButton = __esri.ActionButton;
  export = ActionButton;
}

declare module "esri/support/actions/ActionToggle" {
  import ActionToggle = __esri.ActionToggle;
  export = ActionToggle;
}

declare module "esri/support/BasemapStyle" {
  import BasemapStyle = __esri.BasemapStyle;
  export = BasemapStyle;
}

declare module "esri/support/MapFloorInfo" {
  import MapFloorInfo = __esri.MapFloorInfo;
  export = MapFloorInfo;
}

declare module "esri/symbols/callouts/Callout3D" {
  import Callout3D = __esri.Callout3D;
  export = Callout3D;
}

declare module "esri/symbols/callouts/LineCallout3D" {
  import LineCallout3D = __esri.LineCallout3D;
  export = LineCallout3D;
}

declare module "esri/symbols/CIMSymbol" {
  import CIMSymbol = __esri.CIMSymbol;
  export = CIMSymbol;
}

declare module "esri/symbols/edges/Edges3D" {
  import Edges3D = __esri.Edges3D;
  export = Edges3D;
}

declare module "esri/symbols/edges/SketchEdges3D" {
  import SketchEdges3D = __esri.SketchEdges3D;
  export = SketchEdges3D;
}

declare module "esri/symbols/edges/SolidEdges3D" {
  import SolidEdges3D = __esri.SolidEdges3D;
  export = SolidEdges3D;
}

declare module "esri/symbols/ExtrudeSymbol3DLayer" {
  import ExtrudeSymbol3DLayer = __esri.ExtrudeSymbol3DLayer;
  export = ExtrudeSymbol3DLayer;
}

declare module "esri/symbols/FillSymbol" {
  import FillSymbol = __esri.FillSymbol;
  export = FillSymbol;
}

declare module "esri/symbols/FillSymbol3DLayer" {
  import FillSymbol3DLayer = __esri.FillSymbol3DLayer;
  export = FillSymbol3DLayer;
}

declare module "esri/symbols/Font" {
  import Font = __esri.Font;
  export = Font;
}

declare module "esri/symbols/IconSymbol3DLayer" {
  import IconSymbol3DLayer = __esri.IconSymbol3DLayer;
  export = IconSymbol3DLayer;
}

declare module "esri/symbols/LabelSymbol3D" {
  import LabelSymbol3D = __esri.LabelSymbol3D;
  export = LabelSymbol3D;
}

declare module "esri/symbols/LineStyleMarker3D" {
  import LineStyleMarker3D = __esri.LineStyleMarker3D;
  export = LineStyleMarker3D;
}

declare module "esri/symbols/LineSymbol" {
  import LineSymbol = __esri.LineSymbol;
  export = LineSymbol;
}

declare module "esri/symbols/LineSymbol3D" {
  import LineSymbol3D = __esri.LineSymbol3D;
  export = LineSymbol3D;
}

declare module "esri/symbols/LineSymbol3DLayer" {
  import LineSymbol3DLayer = __esri.LineSymbol3DLayer;
  export = LineSymbol3DLayer;
}

declare module "esri/symbols/LineSymbolMarker" {
  import LineSymbolMarker = __esri.LineSymbolMarker;
  export = LineSymbolMarker;
}

declare module "esri/symbols/MarkerSymbol" {
  import MarkerSymbol = __esri.MarkerSymbol;
  export = MarkerSymbol;
}

declare module "esri/symbols/MeshSymbol3D" {
  import MeshSymbol3D = __esri.MeshSymbol3D;
  export = MeshSymbol3D;
}

declare module "esri/symbols/ObjectSymbol3DLayer" {
  import ObjectSymbol3DLayer = __esri.ObjectSymbol3DLayer;
  export = ObjectSymbol3DLayer;
}

declare module "esri/symbols/PathSymbol3DLayer" {
  import PathSymbol3DLayer = __esri.PathSymbol3DLayer;
  export = PathSymbol3DLayer;
}

declare module "esri/symbols/patterns/LineStylePattern3D" {
  import LineStylePattern3D = __esri.LineStylePattern3D;
  export = LineStylePattern3D;
}

declare module "esri/symbols/patterns/StylePattern3D" {
  import StylePattern3D = __esri.StylePattern3D;
  export = StylePattern3D;
}

declare module "esri/symbols/PictureFillSymbol" {
  import PictureFillSymbol = __esri.PictureFillSymbol;
  export = PictureFillSymbol;
}

declare module "esri/symbols/PictureMarkerSymbol" {
  import PictureMarkerSymbol = __esri.PictureMarkerSymbol;
  export = PictureMarkerSymbol;
}

declare module "esri/symbols/PointSymbol3D" {
  import PointSymbol3D = __esri.PointSymbol3D;
  export = PointSymbol3D;
}

declare module "esri/symbols/PolygonSymbol3D" {
  import PolygonSymbol3D = __esri.PolygonSymbol3D;
  export = PolygonSymbol3D;
}

declare module "esri/symbols/SimpleFillSymbol" {
  import SimpleFillSymbol = __esri.SimpleFillSymbol;
  export = SimpleFillSymbol;
}

declare module "esri/symbols/SimpleLineSymbol" {
  import SimpleLineSymbol = __esri.SimpleLineSymbol;
  export = SimpleLineSymbol;
}

declare module "esri/symbols/SimpleMarkerSymbol" {
  import SimpleMarkerSymbol = __esri.SimpleMarkerSymbol;
  export = SimpleMarkerSymbol;
}

declare module "esri/symbols/support/StyleOrigin" {
  import StyleOrigin = __esri.StyleOrigin;
  export = StyleOrigin;
}

declare module "esri/symbols/support/Symbol3DVerticalOffset" {
  import Symbol3DVerticalOffset = __esri.Symbol3DVerticalOffset;
  export = Symbol3DVerticalOffset;
}

declare module "esri/symbols/Symbol" {
  import Symbol = __esri.Symbol;
  export = Symbol;
}

declare module "esri/symbols/Symbol3D" {
  import Symbol3D = __esri.Symbol3D;
  export = Symbol3D;
}

declare module "esri/symbols/Symbol3DLayer" {
  import Symbol3DLayer = __esri.Symbol3DLayer;
  export = Symbol3DLayer;
}

declare module "esri/symbols/TextSymbol" {
  import TextSymbol = __esri.TextSymbol;
  export = TextSymbol;
}

declare module "esri/symbols/TextSymbol3DLayer" {
  import TextSymbol3DLayer = __esri.TextSymbol3DLayer;
  export = TextSymbol3DLayer;
}

declare module "esri/symbols/WaterSymbol3DLayer" {
  import WaterSymbol3DLayer = __esri.WaterSymbol3DLayer;
  export = WaterSymbol3DLayer;
}

declare module "esri/symbols/WebStyleSymbol" {
  import WebStyleSymbol = __esri.WebStyleSymbol;
  export = WebStyleSymbol;
}

declare module "esri/tables/AttributeTableTemplate" {
  import AttributeTableTemplate = __esri.AttributeTableTemplate;
  export = AttributeTableTemplate;
}

declare module "esri/tables/elements/AttributeTableAttachmentElement" {
  import AttributeTableAttachmentElement = __esri.AttributeTableAttachmentElement;
  export = AttributeTableAttachmentElement;
}

declare module "esri/tables/elements/AttributeTableElement" {
  import AttributeTableElement = __esri.AttributeTableElement;
  export = AttributeTableElement;
}

declare module "esri/tables/elements/AttributeTableGroupElement" {
  import AttributeTableGroupElement = __esri.AttributeTableGroupElement;
  export = AttributeTableGroupElement;
}

declare module "esri/tables/elements/AttributeTableFieldElement" {
  import AttributeTableFieldElement = __esri.AttributeTableFieldElement;
  export = AttributeTableFieldElement;
}

declare module "esri/tables/elements/AttributeTableRelationshipElement" {
  import AttributeTableRelationshipElement = __esri.AttributeTableRelationshipElement;
  export = AttributeTableRelationshipElement;
}

declare module "esri/time/TimeExtent" {
  import TimeExtent = __esri.TimeExtent;
  export = TimeExtent;
}

declare module "esri/time/TimeInterval" {
  import TimeInterval = __esri.TimeInterval;
  export = TimeInterval;
}

declare module "esri/TimeExtent" {
  import esriTimeExtent = __esri.esriTimeExtent;
  export = esriTimeExtent;
}

declare module "esri/TimeInterval" {
  import esriTimeInterval = __esri.esriTimeInterval;
  export = esriTimeInterval;
}

declare module "esri/versionManagement/VersioningState" {
  import VersioningState = __esri.VersioningState;
  export = VersioningState;
}

declare module "esri/versionManagement/VersionManagementService" {
  import VersionManagementService = __esri.VersionManagementService;
  export = VersionManagementService;
}

declare module "esri/Viewpoint" {
  import Viewpoint = __esri.Viewpoint;
  export = Viewpoint;
}

declare module "esri/views/2d/layers/BaseLayerView2D" {
  import BaseLayerView2D = __esri.BaseLayerView2D;
  export = BaseLayerView2D;
}

declare module "esri/views/2d/layers/BaseLayerViewGL2D" {
  import BaseLayerViewGL2D = __esri.BaseLayerViewGL2D;
  export = BaseLayerViewGL2D;
}

declare module "esri/views/2d/ViewState" {
  import ViewState = __esri.ViewState;
  export = ViewState;
}

declare module "esri/views/3d/analysis/AreaMeasurementAnalysisView3D" {
  import AreaMeasurementAnalysisView3D = __esri.AreaMeasurementAnalysisView3D;
  export = AreaMeasurementAnalysisView3D;
}

declare module "esri/views/3d/analysis/DirectLineMeasurementAnalysisView3D" {
  import DirectLineMeasurementAnalysisView3D = __esri.DirectLineMeasurementAnalysisView3D;
  export = DirectLineMeasurementAnalysisView3D;
}

declare module "esri/views/3d/analysis/LineOfSightAnalysisResult" {
  import LineOfSightAnalysisResult = __esri.LineOfSightAnalysisResult;
  export = LineOfSightAnalysisResult;
}

declare module "esri/views/3d/analysis/LineOfSightAnalysisView3D" {
  import LineOfSightAnalysisView3D = __esri.LineOfSightAnalysisView3D;
  export = LineOfSightAnalysisView3D;
}

declare module "esri/views/3d/analysis/SliceAnalysisView3D" {
  import SliceAnalysisView3D = __esri.SliceAnalysisView3D;
  export = SliceAnalysisView3D;
}

declare module "esri/views/3d/analysis/ViewshedAnalysisView3D" {
  import ViewshedAnalysisView3D = __esri.ViewshedAnalysisView3D;
  export = ViewshedAnalysisView3D;
}

declare module "esri/views/3d/environment/CloudyWeather" {
  import CloudyWeather = __esri.CloudyWeather;
  export = CloudyWeather;
}

declare module "esri/views/3d/environment/FoggyWeather" {
  import FoggyWeather = __esri.FoggyWeather;
  export = FoggyWeather;
}

declare module "esri/views/3d/environment/RainyWeather" {
  import RainyWeather = __esri.RainyWeather;
  export = RainyWeather;
}

declare module "esri/views/3d/environment/SnowyWeather" {
  import SnowyWeather = __esri.SnowyWeather;
  export = SnowyWeather;
}

declare module "esri/views/3d/environment/SunLighting" {
  import SunLighting = __esri.SunLighting;
  export = SunLighting;
}

declare module "esri/views/3d/environment/SunnyWeather" {
  import SunnyWeather = __esri.SunnyWeather;
  export = SunnyWeather;
}

declare module "esri/views/3d/environment/VirtualLighting" {
  import VirtualLighting = __esri.VirtualLighting;
  export = VirtualLighting;
}

declare module "esri/views/3d/webgl/RenderNode" {
  import RenderNode = __esri.RenderNode;
  export = RenderNode;
}

declare module "esri/views/analysis/DimensionAnalysisView" {
  import DimensionAnalysisView = __esri.DimensionAnalysisView;
  export = DimensionAnalysisView;
}

declare module "esri/views/BasemapView" {
  import BasemapView = __esri.BasemapView;
  export = BasemapView;
}

declare module "esri/views/draw/Draw" {
  import Draw = __esri.Draw;
  export = Draw;
}

declare module "esri/views/draw/DrawAction" {
  import DrawAction = __esri.DrawAction;
  export = DrawAction;
}

declare module "esri/views/draw/MultipointDrawAction" {
  import MultipointDrawAction = __esri.MultipointDrawAction;
  export = MultipointDrawAction;
}

declare module "esri/views/draw/PointDrawAction" {
  import PointDrawAction = __esri.PointDrawAction;
  export = PointDrawAction;
}

declare module "esri/views/draw/PolygonDrawAction" {
  import PolygonDrawAction = __esri.PolygonDrawAction;
  export = PolygonDrawAction;
}

declare module "esri/views/draw/PolylineDrawAction" {
  import PolylineDrawAction = __esri.PolylineDrawAction;
  export = PolylineDrawAction;
}

declare module "esri/views/draw/SegmentDrawAction" {
  import SegmentDrawAction = __esri.SegmentDrawAction;
  export = SegmentDrawAction;
}

declare module "esri/views/GroundView" {
  import GroundView = __esri.GroundView;
  export = GroundView;
}

declare module "esri/views/input/gamepad/GamepadInputDevice" {
  import GamepadInputDevice = __esri.GamepadInputDevice;
  export = GamepadInputDevice;
}

declare module "esri/views/input/gamepad/GamepadSettings" {
  import GamepadSettings = __esri.GamepadSettings;
  export = GamepadSettings;
}

declare module "esri/views/input/Input" {
  import Input = __esri.Input;
  export = Input;
}

declare module "esri/views/interactive/sketch/SketchLabelOptions" {
  import SketchLabelOptions = __esri.SketchLabelOptions;
  export = SketchLabelOptions;
}

declare module "esri/views/interactive/sketch/SketchTooltipOptions" {
  import SketchTooltipOptions = __esri.SketchTooltipOptions;
  export = SketchTooltipOptions;
}

declare module "esri/views/interactive/sketch/SketchValueOptions" {
  import SketchValueOptions = __esri.SketchValueOptions;
  export = SketchValueOptions;
}

declare module "esri/views/interactive/snapping/FeatureSnappingLayerSource" {
  import FeatureSnappingLayerSource = __esri.FeatureSnappingLayerSource;
  export = FeatureSnappingLayerSource;
}

declare module "esri/views/interactive/snapping/SnappingOptions" {
  import SnappingOptions = __esri.SnappingOptions;
  export = SnappingOptions;
}

declare module "esri/views/interactive/Tooltip" {
  import Tooltip = __esri.Tooltip;
  export = Tooltip;
}

declare module "esri/views/layers/BuildingComponentSublayerView" {
  import BuildingComponentSublayerView = __esri.BuildingComponentSublayerView;
  export = BuildingComponentSublayerView;
}

declare module "esri/views/layers/BuildingSceneLayerView" {
  import BuildingSceneLayerView = __esri.BuildingSceneLayerView;
  export = BuildingSceneLayerView;
}

declare module "esri/views/layers/CatalogLayerView" {
  import CatalogLayerView = __esri.CatalogLayerView;
  export = CatalogLayerView;
}

declare module "esri/views/layers/CatalogDynamicGroupLayerView" {
  import CatalogDynamicGroupLayerView = __esri.CatalogDynamicGroupLayerView;
  export = CatalogDynamicGroupLayerView;
}

declare module "esri/views/layers/CatalogFootprintLayerView" {
  import CatalogFootprintLayerView = __esri.CatalogFootprintLayerView;
  export = CatalogFootprintLayerView;
}

declare module "esri/views/layers/CSVLayerView" {
  type CSVLayerView = __esri.CSVLayerView;
  export = CSVLayerView;
}

declare module "esri/views/layers/DimensionLayerView" {
  import DimensionLayerView = __esri.DimensionLayerView;
  export = DimensionLayerView;
}

declare module "esri/views/layers/FeatureLayerView" {
  import FeatureLayerView = __esri.FeatureLayerView;
  export = FeatureLayerView;
}

declare module "esri/views/layers/GeoJSONLayerView" {
  type GeoJSONLayerView = __esri.GeoJSONLayerView;
  export = GeoJSONLayerView;
}

declare module "esri/views/layers/GeoRSSLayerView" {
  type GeoRSSLayerView = __esri.GeoRSSLayerView;
  export = GeoRSSLayerView;
}

declare module "esri/views/layers/GraphicsLayerView" {
  type GraphicsLayerView = __esri.GraphicsLayerView;
  export = GraphicsLayerView;
}

declare module "esri/views/layers/GroupLayerView" {
  import GroupLayerView = __esri.GroupLayerView;
  export = GroupLayerView;
}

declare module "esri/views/layers/ImageryLayerView" {
  import ImageryLayerView = __esri.ImageryLayerView;
  export = ImageryLayerView;
}

declare module "esri/views/layers/KMLLayerView" {
  type KMLLayerView = __esri.KMLLayerView;
  export = KMLLayerView;
}

declare module "esri/views/layers/LayerView" {
  import LayerView = __esri.LayerView;
  export = LayerView;
}

declare module "esri/views/layers/MediaLayerView" {
  import MediaLayerView = __esri.MediaLayerView;
  export = MediaLayerView;
}

declare module "esri/views/layers/OGCFeatureLayerView" {
  import OGCFeatureLayerView = __esri.OGCFeatureLayerView;
  export = OGCFeatureLayerView;
}

declare module "esri/views/layers/PointCloudLayerView" {
  type PointCloudLayerView = __esri.PointCloudLayerView;
  export = PointCloudLayerView;
}

declare module "esri/views/layers/SceneLayerView" {
  import SceneLayerView = __esri.SceneLayerView;
  export = SceneLayerView;
}

declare module "esri/views/layers/StreamLayerView" {
  import StreamLayerView = __esri.StreamLayerView;
  export = StreamLayerView;
}

declare module "esri/views/layers/VectorTileLayerView" {
  type VectorTileLayerView = __esri.VectorTileLayerView;
  export = VectorTileLayerView;
}

declare module "esri/views/layers/ViewshedLayerView" {
  import ViewshedLayerView = __esri.ViewshedLayerView;
  export = ViewshedLayerView;
}

declare module "esri/views/layers/WFSLayerView" {
  type WFSLayerView = __esri.WFSLayerView;
  export = WFSLayerView;
}

declare module "esri/views/LinkChartView" {
  import LinkChartView = __esri.LinkChartView;
  export = LinkChartView;
}

declare module "esri/views/Magnifier" {
  import Magnifier = __esri.Magnifier;
  export = Magnifier;
}

declare module "esri/views/MapView" {
  import MapView = __esri.MapView;
  export = MapView;
}

declare module "esri/views/navigation/gamepad/GamepadSettings" {
  import gamepadGamepadSettings = __esri.gamepadGamepadSettings;
  export = gamepadGamepadSettings;
}

declare module "esri/views/navigation/Navigation" {
  import Navigation = __esri.Navigation;
  export = Navigation;
}

declare module "esri/views/SceneView" {
  import SceneView = __esri.SceneView;
  export = SceneView;
}

declare module "esri/views/ui/DefaultUI" {
  import DefaultUI = __esri.DefaultUI;
  export = DefaultUI;
}

declare module "esri/views/ui/UI" {
  import UI = __esri.UI;
  export = UI;
}

declare module "esri/views/Theme" {
  import Theme = __esri.Theme;
  export = Theme;
}

declare module "esri/views/View" {
  import View = __esri.View;
  export = View;
}

declare module "esri/views/View2D" {
  import View2D = __esri.View2D;
  export = View2D;
}

declare module "esri/views/ViewAnimation" {
  import ViewAnimation = __esri.ViewAnimation;
  export = ViewAnimation;
}

declare module "esri/webdoc/applicationProperties/Search" {
  import Search = __esri.Search;
  export = Search;
}

declare module "esri/webdoc/applicationProperties/SearchLayer" {
  import SearchLayer = __esri.SearchLayer;
  export = SearchLayer;
}

declare module "esri/webdoc/applicationProperties/SearchLayerField" {
  import SearchLayerField = __esri.SearchLayerField;
  export = SearchLayerField;
}

declare module "esri/webdoc/applicationProperties/SearchTable" {
  import SearchTable = __esri.SearchTable;
  export = SearchTable;
}

declare module "esri/webdoc/applicationProperties/SearchTableField" {
  import SearchTableField = __esri.SearchTableField;
  export = SearchTableField;
}

declare module "esri/webdoc/applicationProperties/Viewing" {
  import Viewing = __esri.Viewing;
  export = Viewing;
}

declare module "esri/webdoc/GeotriggersInfo" {
  import GeotriggersInfo = __esri.GeotriggersInfo;
  export = GeotriggersInfo;
}

declare module "esri/webdoc/geotriggersInfo/DeviceLocationFeed" {
  import DeviceLocationFeed = __esri.DeviceLocationFeed;
  export = DeviceLocationFeed;
}

declare module "esri/webdoc/geotriggersInfo/ExpressionInfo" {
  import geotriggersInfoExpressionInfo = __esri.geotriggersInfoExpressionInfo;
  export = geotriggersInfoExpressionInfo;
}

declare module "esri/webdoc/geotriggersInfo/FeatureFenceParameters" {
  import FeatureFenceParameters = __esri.FeatureFenceParameters;
  export = FeatureFenceParameters;
}

declare module "esri/webdoc/geotriggersInfo/FeatureFilter" {
  import geotriggersInfoFeatureFilter = __esri.geotriggersInfoFeatureFilter;
  export = geotriggersInfoFeatureFilter;
}

declare module "esri/webdoc/geotriggersInfo/FeatureLayerSource" {
  import FeatureLayerSource = __esri.FeatureLayerSource;
  export = FeatureLayerSource;
}

declare module "esri/webdoc/geotriggersInfo/FenceGeotrigger" {
  import FenceGeotrigger = __esri.FenceGeotrigger;
  export = FenceGeotrigger;
}

declare module "esri/webdoc/geotriggersInfo/Geotrigger" {
  import Geotrigger = __esri.Geotrigger;
  export = Geotrigger;
}

declare module "esri/webdoc/geotriggersInfo/GeotriggerNotificationOptions" {
  import GeotriggerNotificationOptions = __esri.GeotriggerNotificationOptions;
  export = GeotriggerNotificationOptions;
}

declare module "esri/webdoc/IPSInfo" {
  import IPSInfo = __esri.IPSInfo;
  export = IPSInfo;
}

declare module "esri/webdoc/ips/PositioningService" {
  import PositioningService = __esri.PositioningService;
  export = PositioningService;
}

declare module "esri/webdoc/widgets/TimeSlider" {
  import TimeSlider = __esri.TimeSlider;
  export = TimeSlider;
}

declare module "esri/WebDocument2D" {
  import WebDocument2D = __esri.WebDocument2D;
  export = WebDocument2D;
}

declare module "esri/WebLinkChart" {
  import WebLinkChart = __esri.WebLinkChart;
  export = WebLinkChart;
}

declare module "esri/WebMap" {
  import WebMap = __esri.WebMap;
  export = WebMap;
}

declare module "esri/webmap/ApplicationProperties" {
  import ApplicationProperties = __esri.ApplicationProperties;
  export = ApplicationProperties;
}

declare module "esri/webmap/background/ColorBackground" {
  import ColorBackground = __esri.ColorBackground;
  export = ColorBackground;
}

declare module "esri/webmap/Bookmark" {
  import Bookmark = __esri.Bookmark;
  export = Bookmark;
}

declare module "esri/webmap/InitialViewProperties" {
  import InitialViewProperties = __esri.InitialViewProperties;
  export = InitialViewProperties;
}

declare module "esri/WebScene" {
  import WebScene = __esri.WebScene;
  export = WebScene;
}

declare module "esri/webscene/ApplicationProperties" {
  import websceneApplicationProperties = __esri.websceneApplicationProperties;
  export = websceneApplicationProperties;
}

declare module "esri/webscene/background/Background" {
  import Background = __esri.Background;
  export = Background;
}

declare module "esri/webscene/background/ColorBackground" {
  import backgroundColorBackground = __esri.backgroundColorBackground;
  export = backgroundColorBackground;
}

declare module "esri/webscene/Environment" {
  import Environment = __esri.Environment;
  export = Environment;
}

declare module "esri/webscene/InitialViewProperties" {
  import websceneInitialViewProperties = __esri.websceneInitialViewProperties;
  export = websceneInitialViewProperties;
}

declare module "esri/webscene/Presentation" {
  import Presentation = __esri.Presentation;
  export = Presentation;
}

declare module "esri/webscene/Slide" {
  import Slide = __esri.Slide;
  export = Slide;
}

declare module "esri/webscene/SunLighting" {
  import websceneSunLighting = __esri.websceneSunLighting;
  export = websceneSunLighting;
}

declare module "esri/webscene/VirtualLighting" {
  import websceneVirtualLighting = __esri.websceneVirtualLighting;
  export = websceneVirtualLighting;
}

declare module "esri/widgets/AreaMeasurement2D" {
  import AreaMeasurement2D = __esri.AreaMeasurement2D;
  export = AreaMeasurement2D;
}

declare module "esri/widgets/AreaMeasurement2D/AreaMeasurement2DViewModel" {
  import AreaMeasurement2DViewModel = __esri.AreaMeasurement2DViewModel;
  export = AreaMeasurement2DViewModel;
}

declare module "esri/widgets/AreaMeasurement3D" {
  import AreaMeasurement3D = __esri.AreaMeasurement3D;
  export = AreaMeasurement3D;
}

declare module "esri/widgets/AreaMeasurement3D/AreaMeasurement3DViewModel" {
  import AreaMeasurement3DViewModel = __esri.AreaMeasurement3DViewModel;
  export = AreaMeasurement3DViewModel;
}

declare module "esri/widgets/Attachments" {
  import Attachments = __esri.Attachments;
  export = Attachments;
}

declare module "esri/widgets/Attachments/AttachmentsViewModel" {
  import AttachmentsViewModel = __esri.AttachmentsViewModel;
  export = AttachmentsViewModel;
}

declare module "esri/widgets/Attribution" {
  import Attribution = __esri.Attribution;
  export = Attribution;
}

declare module "esri/widgets/Attribution/AttributionViewModel" {
  import AttributionViewModel = __esri.AttributionViewModel;
  export = AttributionViewModel;
}

declare module "esri/widgets/BasemapGallery" {
  import BasemapGallery = __esri.BasemapGallery;
  export = BasemapGallery;
}

declare module "esri/widgets/BasemapGallery/BasemapGalleryViewModel" {
  import BasemapGalleryViewModel = __esri.BasemapGalleryViewModel;
  export = BasemapGalleryViewModel;
}

declare module "esri/widgets/BasemapGallery/support/BasemapGalleryItem" {
  import BasemapGalleryItem = __esri.BasemapGalleryItem;
  export = BasemapGalleryItem;
}

declare module "esri/widgets/BasemapGallery/support/LocalBasemapsSource" {
  import LocalBasemapsSource = __esri.LocalBasemapsSource;
  export = LocalBasemapsSource;
}

declare module "esri/widgets/BasemapGallery/support/PortalBasemapsSource" {
  import PortalBasemapsSource = __esri.PortalBasemapsSource;
  export = PortalBasemapsSource;
}

declare module "esri/widgets/BasemapLayerList" {
  import BasemapLayerList = __esri.BasemapLayerList;
  export = BasemapLayerList;
}

declare module "esri/widgets/BasemapLayerList/BasemapLayerListViewModel" {
  import BasemapLayerListViewModel = __esri.BasemapLayerListViewModel;
  export = BasemapLayerListViewModel;
}

declare module "esri/widgets/BasemapToggle" {
  import BasemapToggle = __esri.BasemapToggle;
  export = BasemapToggle;
}

declare module "esri/widgets/BasemapToggle/BasemapToggleViewModel" {
  import BasemapToggleViewModel = __esri.BasemapToggleViewModel;
  export = BasemapToggleViewModel;
}

declare module "esri/widgets/BatchAttributeForm" {
  import BatchAttributeForm = __esri.BatchAttributeForm;
  export = BatchAttributeForm;
}

declare module "esri/widgets/Bookmarks" {
  import Bookmarks = __esri.Bookmarks;
  export = Bookmarks;
}

declare module "esri/widgets/Bookmarks/BookmarksViewModel" {
  import BookmarksViewModel = __esri.BookmarksViewModel;
  export = BookmarksViewModel;
}

declare module "esri/widgets/BuildingExplorer" {
  import BuildingExplorer = __esri.BuildingExplorer;
  export = BuildingExplorer;
}

declare module "esri/widgets/BuildingExplorer/BuildingExplorerViewModel" {
  import BuildingExplorerViewModel = __esri.BuildingExplorerViewModel;
  export = BuildingExplorerViewModel;
}

declare module "esri/widgets/BuildingExplorer/BuildingLevel" {
  import BuildingLevel = __esri.BuildingLevel;
  export = BuildingLevel;
}

declare module "esri/widgets/BuildingExplorer/BuildingPhase" {
  import BuildingPhase = __esri.BuildingPhase;
  export = BuildingPhase;
}

declare module "esri/widgets/CatalogLayerList" {
  import CatalogLayerList = __esri.CatalogLayerList;
  export = CatalogLayerList;
}

declare module "esri/widgets/CatalogLayerList/CatalogLayerListViewModel" {
  import CatalogLayerListViewModel = __esri.CatalogLayerListViewModel;
  export = CatalogLayerListViewModel;
}

declare module "esri/widgets/Compass" {
  import Compass = __esri.Compass;
  export = Compass;
}

declare module "esri/widgets/Compass/CompassViewModel" {
  import CompassViewModel = __esri.CompassViewModel;
  export = CompassViewModel;
}

declare module "esri/widgets/CoordinateConversion" {
  import CoordinateConversion = __esri.CoordinateConversion;
  export = CoordinateConversion;
}

declare module "esri/widgets/CoordinateConversion/CoordinateConversionViewModel" {
  import CoordinateConversionViewModel = __esri.CoordinateConversionViewModel;
  export = CoordinateConversionViewModel;
}

declare module "esri/widgets/CoordinateConversion/support/Conversion" {
  import Conversion = __esri.Conversion;
  export = Conversion;
}

declare module "esri/widgets/CoordinateConversion/support/Format" {
  import Format = __esri.Format;
  export = Format;
}

declare module "esri/widgets/Daylight" {
  import Daylight = __esri.Daylight;
  export = Daylight;
}

declare module "esri/widgets/Daylight/DaylightViewModel" {
  import DaylightViewModel = __esri.DaylightViewModel;
  export = DaylightViewModel;
}

declare module "esri/widgets/DirectionalPad" {
  import DirectionalPad = __esri.DirectionalPad;
  export = DirectionalPad;
}

declare module "esri/widgets/DirectionalPad/DirectionalPadViewModel" {
  import DirectionalPadViewModel = __esri.DirectionalPadViewModel;
  export = DirectionalPadViewModel;
}

declare module "esri/widgets/Directions" {
  import Directions = __esri.Directions;
  export = Directions;
}

declare module "esri/widgets/Directions/DirectionsViewModel" {
  import DirectionsViewModel = __esri.DirectionsViewModel;
  export = DirectionsViewModel;
}

declare module "esri/widgets/DirectLineMeasurement3D" {
  import DirectLineMeasurement3D = __esri.DirectLineMeasurement3D;
  export = DirectLineMeasurement3D;
}

declare module "esri/widgets/DirectLineMeasurement3D/DirectLineMeasurement3DViewModel" {
  import DirectLineMeasurement3DViewModel = __esri.DirectLineMeasurement3DViewModel;
  export = DirectLineMeasurement3DViewModel;
}

declare module "esri/widgets/DistanceMeasurement2D" {
  import DistanceMeasurement2D = __esri.DistanceMeasurement2D;
  export = DistanceMeasurement2D;
}

declare module "esri/widgets/DistanceMeasurement2D/DistanceMeasurement2DViewModel" {
  import DistanceMeasurement2DViewModel = __esri.DistanceMeasurement2DViewModel;
  export = DistanceMeasurement2DViewModel;
}

declare module "esri/widgets/Editor" {
  import Editor = __esri.Editor;
  export = Editor;
}

declare module "esri/widgets/Editor/CreateFeaturesWorkflow" {
  import CreateFeaturesWorkflow = __esri.CreateFeaturesWorkflow;
  export = CreateFeaturesWorkflow;
}

declare module "esri/widgets/Editor/CreateFeaturesWorkflowData" {
  import CreateFeaturesWorkflowData = __esri.CreateFeaturesWorkflowData;
  export = CreateFeaturesWorkflowData;
}

declare module "esri/widgets/Editor/EditorViewModel" {
  import EditorViewModel = __esri.EditorViewModel;
  export = EditorViewModel;
}

declare module "esri/widgets/Editor/Edits" {
  import Edits = __esri.Edits;
  export = Edits;
}

declare module "esri/widgets/Editor/UpdateWorkflow" {
  import UpdateWorkflow = __esri.UpdateWorkflow;
  export = UpdateWorkflow;
}

declare module "esri/widgets/Editor/UpdateWorkflowData" {
  import UpdateWorkflowData = __esri.UpdateWorkflowData;
  export = UpdateWorkflowData;
}

declare module "esri/widgets/Editor/Workflow" {
  import Workflow = __esri.Workflow;
  export = Workflow;
}

declare module "esri/widgets/Editor/support/EditorItem" {
  import EditorItem = __esri.EditorItem;
  export = EditorItem;
}

declare module "esri/widgets/ElevationProfile" {
  import ElevationProfile = __esri.ElevationProfile;
  export = ElevationProfile;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileLine" {
  import ElevationProfileLine = __esri.ElevationProfileLine;
  export = ElevationProfileLine;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileLineGround" {
  import ElevationProfileLineGround = __esri.ElevationProfileLineGround;
  export = ElevationProfileLineGround;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileLineInput" {
  import ElevationProfileLineInput = __esri.ElevationProfileLineInput;
  export = ElevationProfileLineInput;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileLineQuery" {
  import ElevationProfileLineQuery = __esri.ElevationProfileLineQuery;
  export = ElevationProfileLineQuery;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileLineView" {
  import ElevationProfileLineView = __esri.ElevationProfileLineView;
  export = ElevationProfileLineView;
}

declare module "esri/widgets/ElevationProfile/ElevationProfileViewModel" {
  import ElevationProfileViewModel = __esri.ElevationProfileViewModel;
  export = ElevationProfileViewModel;
}

declare module "esri/widgets/Expand" {
  import Expand = __esri.Expand;
  export = Expand;
}

declare module "esri/widgets/Expand/ExpandViewModel" {
  import ExpandViewModel = __esri.ExpandViewModel;
  export = ExpandViewModel;
}

declare module "esri/widgets/Feature" {
  import Feature = __esri.Feature;
  export = Feature;
}

declare module "esri/widgets/Feature/FeatureViewModel" {
  import FeatureViewModel = __esri.FeatureViewModel;
  export = FeatureViewModel;
}

declare module "esri/widgets/FeatureForm" {
  import FeatureForm = __esri.FeatureForm;
  export = FeatureForm;
}

declare module "esri/widgets/FeatureForm/FeatureFormViewModel" {
  import FeatureFormViewModel = __esri.FeatureFormViewModel;
  export = FeatureFormViewModel;
}

declare module "esri/widgets/FeatureForm/FieldInput" {
  import FieldInput = __esri.FieldInput;
  export = FieldInput;
}

declare module "esri/widgets/FeatureForm/GroupInput" {
  import GroupInput = __esri.GroupInput;
  export = GroupInput;
}

declare module "esri/widgets/FeatureForm/RelationshipInput" {
  import RelationshipInput = __esri.RelationshipInput;
  export = RelationshipInput;
}

declare module "esri/widgets/Features" {
  import Features = __esri.Features;
  export = Features;
}

declare module "esri/widgets/Features/FeaturesViewModel" {
  import FeaturesViewModel = __esri.FeaturesViewModel;
  export = FeaturesViewModel;
}

declare module "esri/widgets/FeatureTable" {
  import FeatureTable = __esri.FeatureTable;
  export = FeatureTable;
}

declare module "esri/widgets/FeatureTable/AttachmentsColumn" {
  import AttachmentsColumn = __esri.AttachmentsColumn;
  export = AttachmentsColumn;
}

declare module "esri/widgets/FeatureTable/ActionColumn" {
  import ActionColumn = __esri.ActionColumn;
  export = ActionColumn;
}

declare module "esri/widgets/FeatureTable/FeatureTableViewModel" {
  import FeatureTableViewModel = __esri.FeatureTableViewModel;
  export = FeatureTableViewModel;
}

declare module "esri/widgets/FeatureTable/FieldColumn" {
  import FieldColumn = __esri.FieldColumn;
  export = FieldColumn;
}

declare module "esri/widgets/FeatureTable/RelationshipColumn" {
  import RelationshipColumn = __esri.RelationshipColumn;
  export = RelationshipColumn;
}

declare module "esri/widgets/FeatureTable/Grid/Column" {
  import Column = __esri.Column;
  export = Column;
}

declare module "esri/widgets/FeatureTable/Grid/GroupColumn" {
  import GroupColumn = __esri.GroupColumn;
  export = GroupColumn;
}

declare module "esri/widgets/FeatureTable/Grid/support/ButtonMenu" {
  import ButtonMenu = __esri.ButtonMenu;
  export = ButtonMenu;
}

declare module "esri/widgets/FeatureTable/Grid/support/ButtonMenuItem" {
  import ButtonMenuItem = __esri.ButtonMenuItem;
  export = ButtonMenuItem;
}

declare module "esri/widgets/FeatureTable/Grid/support/ButtonMenuViewModel" {
  import ButtonMenuViewModel = __esri.ButtonMenuViewModel;
  export = ButtonMenuViewModel;
}

declare module "esri/widgets/FeatureTable/support/ColumnTemplate" {
  import ColumnTemplate = __esri.ColumnTemplate;
  export = ColumnTemplate;
}

declare module "esri/widgets/FeatureTable/support/ColumnTemplateBase" {
  import ColumnTemplateBase = __esri.ColumnTemplateBase;
  export = ColumnTemplateBase;
}

declare module "esri/widgets/FeatureTable/support/FieldColumnTemplate" {
  import FieldColumnTemplate = __esri.FieldColumnTemplate;
  export = FieldColumnTemplate;
}

declare module "esri/widgets/FeatureTable/support/GroupColumnTemplate" {
  import GroupColumnTemplate = __esri.GroupColumnTemplate;
  export = GroupColumnTemplate;
}

declare module "esri/widgets/FeatureTable/support/TableTemplate" {
  import TableTemplate = __esri.TableTemplate;
  export = TableTemplate;
}

declare module "esri/widgets/FeatureTemplates" {
  import FeatureTemplates = __esri.FeatureTemplates;
  export = FeatureTemplates;
}

declare module "esri/widgets/FeatureTemplates/FeatureTemplatesViewModel" {
  import FeatureTemplatesViewModel = __esri.FeatureTemplatesViewModel;
  export = FeatureTemplatesViewModel;
}

declare module "esri/widgets/FeatureTemplates/TemplateItem" {
  import TemplateItem = __esri.TemplateItem;
  export = TemplateItem;
}

declare module "esri/widgets/FeatureTemplates/TemplateItemGroup" {
  import TemplateItemGroup = __esri.TemplateItemGroup;
  export = TemplateItemGroup;
}

declare module "esri/widgets/FloorFilter" {
  import FloorFilter = __esri.FloorFilter;
  export = FloorFilter;
}

declare module "esri/widgets/FloorFilter/FloorFilterViewModel" {
  import FloorFilterViewModel = __esri.FloorFilterViewModel;
  export = FloorFilterViewModel;
}

declare module "esri/widgets/Fullscreen" {
  import Fullscreen = __esri.Fullscreen;
  export = Fullscreen;
}

declare module "esri/widgets/Fullscreen/FullscreenViewModel" {
  import FullscreenViewModel = __esri.FullscreenViewModel;
  export = FullscreenViewModel;
}

declare module "esri/widgets/Histogram" {
  import Histogram = __esri.Histogram;
  export = Histogram;
}

declare module "esri/widgets/Histogram/HistogramViewModel" {
  import HistogramViewModel = __esri.HistogramViewModel;
  export = HistogramViewModel;
}

declare module "esri/widgets/HistogramRangeSlider" {
  import HistogramRangeSlider = __esri.HistogramRangeSlider;
  export = HistogramRangeSlider;
}

declare module "esri/widgets/HistogramRangeSlider/HistogramRangeSliderViewModel" {
  import HistogramRangeSliderViewModel = __esri.HistogramRangeSliderViewModel;
  export = HistogramRangeSliderViewModel;
}

declare module "esri/widgets/Home" {
  import Home = __esri.Home;
  export = Home;
}

declare module "esri/widgets/Home/HomeViewModel" {
  import HomeViewModel = __esri.HomeViewModel;
  export = HomeViewModel;
}

declare module "esri/widgets/LayerList" {
  import LayerList = __esri.LayerList;
  export = LayerList;
}

declare module "esri/widgets/LayerList/LayerListViewModel" {
  import LayerListViewModel = __esri.LayerListViewModel;
  export = LayerListViewModel;
}

declare module "esri/widgets/LayerList/ListItem" {
  import ListItem = __esri.ListItem;
  export = ListItem;
}

declare module "esri/widgets/LayerList/ListItemPanel" {
  import ListItemPanel = __esri.ListItemPanel;
  export = ListItemPanel;
}

declare module "esri/widgets/Legend" {
  import Legend = __esri.Legend;
  export = Legend;
}

declare module "esri/widgets/Legend/LegendViewModel" {
  import LegendViewModel = __esri.LegendViewModel;
  export = LegendViewModel;
}

declare module "esri/widgets/Legend/support/ActiveLayerInfo" {
  import ActiveLayerInfo = __esri.ActiveLayerInfo;
  export = ActiveLayerInfo;
}

declare module "esri/widgets/LineOfSight" {
  import LineOfSight = __esri.LineOfSight;
  export = LineOfSight;
}

declare module "esri/widgets/LineOfSight/LineOfSightTarget" {
  import LineOfSightTarget = __esri.LineOfSightTarget;
  export = LineOfSightTarget;
}

declare module "esri/widgets/LineOfSight/LineOfSightViewModel" {
  import LineOfSightViewModel = __esri.LineOfSightViewModel;
  export = LineOfSightViewModel;
}

declare module "esri/widgets/Locate" {
  import Locate = __esri.Locate;
  export = Locate;
}

declare module "esri/widgets/Locate/LocateViewModel" {
  import LocateViewModel = __esri.LocateViewModel;
  export = LocateViewModel;
}

declare module "esri/widgets/Measurement" {
  import Measurement = __esri.Measurement;
  export = Measurement;
}

declare module "esri/widgets/Measurement/MeasurementViewModel" {
  import MeasurementViewModel = __esri.MeasurementViewModel;
  export = MeasurementViewModel;
}

declare module "esri/widgets/NavigationToggle" {
  import NavigationToggle = __esri.NavigationToggle;
  export = NavigationToggle;
}

declare module "esri/widgets/NavigationToggle/NavigationToggleViewModel" {
  import NavigationToggleViewModel = __esri.NavigationToggleViewModel;
  export = NavigationToggleViewModel;
}

declare module "esri/widgets/OrientedImageryViewer" {
  import OrientedImageryViewer = __esri.OrientedImageryViewer;
  export = OrientedImageryViewer;
}

declare module "esri/widgets/OrientedImageryViewer/OrientedImageryViewerViewModel" {
  import OrientedImageryViewerViewModel = __esri.OrientedImageryViewerViewModel;
  export = OrientedImageryViewerViewModel;
}

declare module "esri/widgets/Popup" {
  import Popup = __esri.Popup;
  export = Popup;
}

declare module "esri/widgets/Popup/PopupViewModel" {
  import PopupViewModel = __esri.PopupViewModel;
  export = PopupViewModel;
}

declare module "esri/widgets/Print" {
  import Print = __esri.Print;
  export = Print;
}

declare module "esri/widgets/Print/CustomTemplate" {
  import CustomTemplate = __esri.CustomTemplate;
  export = CustomTemplate;
}

declare module "esri/widgets/Print/PrintViewModel" {
  import PrintViewModel = __esri.PrintViewModel;
  export = PrintViewModel;
}

declare module "esri/widgets/Print/TemplateOptions" {
  import TemplateOptions = __esri.TemplateOptions;
  export = TemplateOptions;
}

declare module "esri/widgets/ScaleBar" {
  import ScaleBar = __esri.ScaleBar;
  export = ScaleBar;
}

declare module "esri/widgets/ScaleBar/ScaleBarViewModel" {
  import ScaleBarViewModel = __esri.ScaleBarViewModel;
  export = ScaleBarViewModel;
}

declare module "esri/widgets/ScaleRangeSlider" {
  import ScaleRangeSlider = __esri.ScaleRangeSlider;
  export = ScaleRangeSlider;
}

declare module "esri/widgets/ScaleRangeSlider/ScaleRanges" {
  import ScaleRanges = __esri.ScaleRanges;
  export = ScaleRanges;
}

declare module "esri/widgets/ScaleRangeSlider/ScaleRangeSliderViewModel" {
  import ScaleRangeSliderViewModel = __esri.ScaleRangeSliderViewModel;
  export = ScaleRangeSliderViewModel;
}

declare module "esri/widgets/Search" {
  import widgetsSearch = __esri.widgetsSearch;
  export = widgetsSearch;
}

declare module "esri/widgets/Search/LayerSearchSource" {
  import LayerSearchSource = __esri.LayerSearchSource;
  export = LayerSearchSource;
}

declare module "esri/widgets/Search/LocatorSearchSource" {
  import LocatorSearchSource = __esri.LocatorSearchSource;
  export = LocatorSearchSource;
}

declare module "esri/widgets/Search/SearchResultRenderer" {
  import SearchResultRenderer = __esri.SearchResultRenderer;
  export = SearchResultRenderer;
}

declare module "esri/widgets/Search/SearchSource" {
  import SearchSource = __esri.SearchSource;
  export = SearchSource;
}

declare module "esri/widgets/Search/SearchViewModel" {
  import SearchViewModel = __esri.SearchViewModel;
  export = SearchViewModel;
}

declare module "esri/widgets/ShadowCast" {
  import ShadowCast = __esri.ShadowCast;
  export = ShadowCast;
}

declare module "esri/widgets/ShadowCast/ShadowCastViewModel" {
  import ShadowCastViewModel = __esri.ShadowCastViewModel;
  export = ShadowCastViewModel;
}

declare module "esri/widgets/Sketch" {
  import Sketch = __esri.Sketch;
  export = Sketch;
}

declare module "esri/widgets/Sketch/SketchViewModel" {
  import SketchViewModel = __esri.SketchViewModel;
  export = SketchViewModel;
}

declare module "esri/widgets/Slice" {
  import Slice = __esri.Slice;
  export = Slice;
}

declare module "esri/widgets/Slice/SliceViewModel" {
  import SliceViewModel = __esri.SliceViewModel;
  export = SliceViewModel;
}

declare module "esri/widgets/Slider" {
  import Slider = __esri.Slider;
  export = Slider;
}

declare module "esri/widgets/Slider/SliderViewModel" {
  import SliderViewModel = __esri.SliderViewModel;
  export = SliderViewModel;
}

declare module "esri/widgets/smartMapping/BinaryColorSizeSlider" {
  import BinaryColorSizeSlider = __esri.BinaryColorSizeSlider;
  export = BinaryColorSizeSlider;
}

declare module "esri/widgets/smartMapping/BinaryColorSizeSlider/BinaryColorSizeSliderViewModel" {
  import BinaryColorSizeSliderViewModel = __esri.BinaryColorSizeSliderViewModel;
  export = BinaryColorSizeSliderViewModel;
}

declare module "esri/widgets/smartMapping/ClassedColorSlider" {
  import ClassedColorSlider = __esri.ClassedColorSlider;
  export = ClassedColorSlider;
}

declare module "esri/widgets/smartMapping/ClassedColorSlider/ClassedColorSliderViewModel" {
  import ClassedColorSliderViewModel = __esri.ClassedColorSliderViewModel;
  export = ClassedColorSliderViewModel;
}

declare module "esri/widgets/smartMapping/ClassedSizeSlider" {
  import ClassedSizeSlider = __esri.ClassedSizeSlider;
  export = ClassedSizeSlider;
}

declare module "esri/widgets/smartMapping/ClassedSizeSlider/ClassedSizeSliderViewModel" {
  import ClassedSizeSliderViewModel = __esri.ClassedSizeSliderViewModel;
  export = ClassedSizeSliderViewModel;
}

declare module "esri/widgets/smartMapping/ColorSizeSlider" {
  import ColorSizeSlider = __esri.ColorSizeSlider;
  export = ColorSizeSlider;
}

declare module "esri/widgets/smartMapping/ColorSizeSlider/ColorSizeSliderViewModel" {
  import ColorSizeSliderViewModel = __esri.ColorSizeSliderViewModel;
  export = ColorSizeSliderViewModel;
}

declare module "esri/widgets/smartMapping/ColorSlider" {
  import ColorSlider = __esri.ColorSlider;
  export = ColorSlider;
}

declare module "esri/widgets/smartMapping/ColorSlider/ColorSliderViewModel" {
  import ColorSliderViewModel = __esri.ColorSliderViewModel;
  export = ColorSliderViewModel;
}

declare module "esri/widgets/smartMapping/HeatmapSlider" {
  import HeatmapSlider = __esri.HeatmapSlider;
  export = HeatmapSlider;
}

declare module "esri/widgets/smartMapping/HeatmapSlider/HeatmapSliderViewModel" {
  import HeatmapSliderViewModel = __esri.HeatmapSliderViewModel;
  export = HeatmapSliderViewModel;
}

declare module "esri/widgets/smartMapping/OpacitySlider" {
  import OpacitySlider = __esri.OpacitySlider;
  export = OpacitySlider;
}

declare module "esri/widgets/smartMapping/OpacitySlider/OpacitySliderViewModel" {
  import OpacitySliderViewModel = __esri.OpacitySliderViewModel;
  export = OpacitySliderViewModel;
}

declare module "esri/widgets/smartMapping/SizeSlider" {
  import SizeSlider = __esri.SizeSlider;
  export = SizeSlider;
}

declare module "esri/widgets/smartMapping/SizeSlider/SizeSliderViewModel" {
  import SizeSliderViewModel = __esri.SizeSliderViewModel;
  export = SizeSliderViewModel;
}

declare module "esri/widgets/smartMapping/SmartMappingPrimaryHandleSliderViewModel" {
  import SmartMappingPrimaryHandleSliderViewModel = __esri.SmartMappingPrimaryHandleSliderViewModel;
  export = SmartMappingPrimaryHandleSliderViewModel;
}

declare module "esri/widgets/smartMapping/SmartMappingSliderBase" {
  import SmartMappingSliderBase = __esri.SmartMappingSliderBase;
  export = SmartMappingSliderBase;
}

declare module "esri/widgets/smartMapping/SmartMappingSliderViewModel" {
  import SmartMappingSliderViewModel = __esri.SmartMappingSliderViewModel;
  export = SmartMappingSliderViewModel;
}

declare module "esri/widgets/support/SnappingControls" {
  import SnappingControls = __esri.SnappingControls;
  export = SnappingControls;
}

declare module "esri/widgets/support/GridControls" {
  import GridControls = __esri.GridControls;
  export = GridControls;
}

declare module "esri/widgets/support/SnappingControls/SnappingControlsViewModel" {
  import SnappingControlsViewModel = __esri.SnappingControlsViewModel;
  export = SnappingControlsViewModel;
}

declare module "esri/widgets/support/GridControls/GridControlsViewModel" {
  import GridControlsViewModel = __esri.GridControlsViewModel;
  export = GridControlsViewModel;
}

declare module "esri/widgets/Swipe" {
  import Swipe = __esri.Swipe;
  export = Swipe;
}

declare module "esri/widgets/Swipe/SwipeViewModel" {
  import SwipeViewModel = __esri.SwipeViewModel;
  export = SwipeViewModel;
}

declare module "esri/widgets/TableList" {
  import TableList = __esri.TableList;
  export = TableList;
}

declare module "esri/widgets/TableList/ListItem" {
  import TableListListItem = __esri.TableListListItem;
  export = TableListListItem;
}

declare module "esri/widgets/TableList/ListItemPanel" {
  import TableListListItemPanel = __esri.TableListListItemPanel;
  export = TableListListItemPanel;
}

declare module "esri/widgets/TableList/TableListViewModel" {
  import TableListViewModel = __esri.TableListViewModel;
  export = TableListViewModel;
}

declare module "esri/widgets/TimeSlider" {
  import widgetsTimeSlider = __esri.widgetsTimeSlider;
  export = widgetsTimeSlider;
}

declare module "esri/widgets/TimeSlider/TimeSliderViewModel" {
  import TimeSliderViewModel = __esri.TimeSliderViewModel;
  export = TimeSliderViewModel;
}

declare module "esri/widgets/TimeZoneLabel" {
  import TimeZoneLabel = __esri.TimeZoneLabel;
  export = TimeZoneLabel;
}

declare module "esri/widgets/Track" {
  import Track = __esri.Track;
  export = Track;
}

declare module "esri/widgets/Track/TrackViewModel" {
  import TrackViewModel = __esri.TrackViewModel;
  export = TrackViewModel;
}

declare module "esri/widgets/UtilityNetworkAssociations" {
  import UtilityNetworkAssociations = __esri.UtilityNetworkAssociations;
  export = UtilityNetworkAssociations;
}

declare module "esri/widgets/UtilityNetworkAssociations/UtilityNetworkAssociationsViewModel" {
  import UtilityNetworkAssociationsViewModel = __esri.UtilityNetworkAssociationsViewModel;
  export = UtilityNetworkAssociationsViewModel;
}

declare module "esri/widgets/UtilityNetworkTrace" {
  import UtilityNetworkTrace = __esri.UtilityNetworkTrace;
  export = UtilityNetworkTrace;
}

declare module "esri/widgets/UtilityNetworkTrace/UtilityNetworkTraceViewModel" {
  import UtilityNetworkTraceViewModel = __esri.UtilityNetworkTraceViewModel;
  export = UtilityNetworkTraceViewModel;
}

declare module "esri/widgets/UtilityNetworkValidateTopology" {
  import UtilityNetworkValidateTopology = __esri.UtilityNetworkValidateTopology;
  export = UtilityNetworkValidateTopology;
}

declare module "esri/widgets/UtilityNetworkValidateTopology/UtilityNetworkValidateTopologyViewModel" {
  import UtilityNetworkValidateTopologyViewModel = __esri.UtilityNetworkValidateTopologyViewModel;
  export = UtilityNetworkValidateTopologyViewModel;
}

declare module "esri/widgets/ValuePicker" {
  import ValuePicker = __esri.ValuePicker;
  export = ValuePicker;
}

declare module "esri/widgets/ValuePicker/ValuePickerCollection" {
  import ValuePickerCollection = __esri.ValuePickerCollection;
  export = ValuePickerCollection;
}

declare module "esri/widgets/ValuePicker/ValuePickerCombobox" {
  import ValuePickerCombobox = __esri.ValuePickerCombobox;
  export = ValuePickerCombobox;
}

declare module "esri/widgets/ValuePicker/ValuePickerLabel" {
  import ValuePickerLabel = __esri.ValuePickerLabel;
  export = ValuePickerLabel;
}

declare module "esri/widgets/ValuePicker/ValuePickerSlider" {
  import ValuePickerSlider = __esri.ValuePickerSlider;
  export = ValuePickerSlider;
}

declare module "esri/widgets/VersionManagement/VersionManagementViewModel" {
  import VersionManagementViewModel = __esri.VersionManagementViewModel;
  export = VersionManagementViewModel;
}

declare module "esri/widgets/VideoPlayer" {
  import VideoPlayer = __esri.VideoPlayer;
  export = VideoPlayer;
}

declare module "esri/widgets/VideoPlayer/VideoPlayerViewModel" {
  import VideoPlayerViewModel = __esri.VideoPlayerViewModel;
  export = VideoPlayerViewModel;
}

declare module "esri/widgets/Weather" {
  import Weather = __esri.Weather;
  export = Weather;
}

declare module "esri/widgets/Weather/WeatherViewModel" {
  import WeatherViewModel = __esri.WeatherViewModel;
  export = WeatherViewModel;
}

declare module "esri/widgets/Widget" {
  import Widget = __esri.Widget;
  export = Widget;
}

declare module "esri/widgets/Zoom" {
  import Zoom = __esri.Zoom;
  export = Zoom;
}

declare module "esri/widgets/Zoom/ZoomViewModel" {
  import ZoomViewModel = __esri.ZoomViewModel;
  export = ZoomViewModel;
}

declare module "esri/core/Clonable" {
  import Clonable = __esri.Clonable;
  export = Clonable;
}

declare module "esri/core/Evented" {
  import Evented = __esri.Evented;
  export = Evented;
}

declare module "esri/core/Identifiable" {
  import Identifiable = __esri.Identifiable;
  export = Identifiable;
}

declare module "esri/core/JSONSupport" {
  import JSONSupport = __esri.JSONSupport;
  export = JSONSupport;
}

declare module "esri/core/Loadable" {
  import Loadable = __esri.Loadable;
  export = Loadable;
}

declare module "esri/core/Promise" {
  import corePromise = __esri.corePromise;
  export = corePromise;
}

declare module "esri/form/elements/inputs/TextInput" {
  import TextInput = __esri.TextInput;
  export = TextInput;
}

declare module "esri/layers/mixins/APIKeyMixin" {
  import APIKeyMixin = __esri.APIKeyMixin;
  export = APIKeyMixin;
}

declare module "esri/layers/mixins/ArcGISCachedService" {
  import ArcGISCachedService = __esri.ArcGISCachedService;
  export = ArcGISCachedService;
}

declare module "esri/layers/mixins/ArcGISImageService" {
  import ArcGISImageService = __esri.ArcGISImageService;
  export = ArcGISImageService;
}

declare module "esri/layers/mixins/ArcGISMapService" {
  import ArcGISMapService = __esri.ArcGISMapService;
  export = ArcGISMapService;
}

declare module "esri/layers/mixins/BlendLayer" {
  import BlendLayer = __esri.BlendLayer;
  export = BlendLayer;
}

declare module "esri/layers/mixins/CustomParametersMixin" {
  import CustomParametersMixin = __esri.CustomParametersMixin;
  export = CustomParametersMixin;
}

declare module "esri/layers/mixins/FeatureEffectLayer" {
  import FeatureEffectLayer = __esri.FeatureEffectLayer;
  export = FeatureEffectLayer;
}

declare module "esri/layers/mixins/FeatureLayerBase" {
  import FeatureLayerBase = __esri.FeatureLayerBase;
  export = FeatureLayerBase;
}

declare module "esri/layers/mixins/FeatureReductionLayer" {
  import FeatureReductionLayer = __esri.FeatureReductionLayer;
  export = FeatureReductionLayer;
}

declare module "esri/layers/mixins/ImageryTileMixin" {
  import ImageryTileMixin = __esri.ImageryTileMixin;
  export = ImageryTileMixin;
}

declare module "esri/layers/mixins/OperationalLayer" {
  import OperationalLayer = __esri.OperationalLayer;
  export = OperationalLayer;
}

declare module "esri/layers/mixins/OrderedLayer" {
  import OrderedLayer = __esri.OrderedLayer;
  export = OrderedLayer;
}

declare module "esri/layers/mixins/PortalLayer" {
  import PortalLayer = __esri.PortalLayer;
  export = PortalLayer;
}

declare module "esri/layers/mixins/PublishableLayer" {
  import PublishableLayer = __esri.PublishableLayer;
  export = PublishableLayer;
}

declare module "esri/layers/mixins/RasterPresetRendererMixin" {
  import RasterPresetRendererMixin = __esri.RasterPresetRendererMixin;
  export = RasterPresetRendererMixin;
}

declare module "esri/layers/mixins/RefreshableLayer" {
  import RefreshableLayer = __esri.RefreshableLayer;
  export = RefreshableLayer;
}

declare module "esri/layers/mixins/ScaleRangeLayer" {
  import ScaleRangeLayer = __esri.ScaleRangeLayer;
  export = ScaleRangeLayer;
}

declare module "esri/layers/mixins/SceneService" {
  import SceneService = __esri.SceneService;
  export = SceneService;
}

declare module "esri/layers/mixins/SublayersOwner" {
  import SublayersOwner = __esri.SublayersOwner;
  export = SublayersOwner;
}

declare module "esri/layers/mixins/TemporalLayer" {
  import TemporalLayer = __esri.TemporalLayer;
  export = TemporalLayer;
}

declare module "esri/layers/mixins/TemporalSceneLayer" {
  import TemporalSceneLayer = __esri.TemporalSceneLayer;
  export = TemporalSceneLayer;
}

declare module "esri/layers/support/MediaElementBase" {
  import MediaElementBase = __esri.MediaElementBase;
  export = MediaElementBase;
}

declare module "esri/popup/content/mixins/ChartMediaInfo" {
  import ChartMediaInfo = __esri.ChartMediaInfo;
  export = ChartMediaInfo;
}

declare module "esri/popup/content/mixins/MediaInfo" {
  import MediaInfo = __esri.MediaInfo;
  export = MediaInfo;
}

declare module "esri/renderers/mixins/VisualVariablesMixin" {
  import VisualVariablesMixin = __esri.VisualVariablesMixin;
  export = VisualVariablesMixin;
}

declare module "esri/rest/support/BaseImageMeasureParameters" {
  import BaseImageMeasureParameters = __esri.BaseImageMeasureParameters;
  export = BaseImageMeasureParameters;
}

declare module "esri/rest/support/BaseImageMeasureResult" {
  import BaseImageMeasureResult = __esri.BaseImageMeasureResult;
  export = BaseImageMeasureResult;
}

declare module "esri/rest/support/QueryMixin" {
  type QueryMixin = __esri.QueryMixin;
  export = QueryMixin;
}

declare module "esri/support/LayersMixin" {
  import LayersMixin = __esri.LayersMixin;
  export = LayersMixin;
}

declare module "esri/support/TablesMixin" {
  import TablesMixin = __esri.TablesMixin;
  export = TablesMixin;
}

declare module "esri/views/BreakpointsOwner" {
  import BreakpointsOwner = __esri.BreakpointsOwner;
  export = BreakpointsOwner;
}

declare module "esri/views/DOMContainer" {
  import DOMContainer = __esri.DOMContainer;
  export = DOMContainer;
}

declare module "esri/views/PopupView" {
  import PopupView = __esri.PopupView;
  export = PopupView;
}

declare module "esri/views/layers/FeatureLayerViewMixin" {
  type FeatureLayerViewMixin = __esri.FeatureLayerViewMixin;
  export = FeatureLayerViewMixin;
}

declare module "esri/views/layers/HighlightLayerViewMixin" {
  type HighlightLayerViewMixin = __esri.HighlightLayerViewMixin;
  export = HighlightLayerViewMixin;
}

declare module "esri/widgets/FeatureForm/EditableInput" {
  import EditableInput = __esri.EditableInput;
  export = EditableInput;
}

declare module "esri/widgets/FeatureForm/InputBase" {
  import InputBase = __esri.InputBase;
  export = InputBase;
}

declare module "esri/widgets/support/GeolocationPositioning" {
  import GeolocationPositioning = __esri.GeolocationPositioning;
  export = GeolocationPositioning;
}

declare module "esri/widgets/support/GoTo" {
  import GoTo = __esri.GoTo;
  export = GoTo;
}

declare module "esri/arcade" {
  import arcade = __esri.arcade;
  export = arcade;
}

declare module "esri/config" {
  import config = __esri.config;
  export = config;
}

declare module "esri/core/accessorSupport/decorators" {
  import decorators = __esri.decorators;
  export = decorators;
}

declare module "esri/core/lang" {
  import lang = __esri.lang;
  export = lang;
}

declare module "esri/core/promiseUtils" {
  import promiseUtils = __esri.promiseUtils;
  export = promiseUtils;
}

declare module "esri/core/quantity" {
  type quantity = __esri.quantity;
  export = quantity;
}

declare module "esri/core/reactiveUtils" {
  import reactiveUtils = __esri.reactiveUtils;
  export = reactiveUtils;
}

declare module "esri/core/scheduling" {
  import scheduling = __esri.scheduling;
  export = scheduling;
}

declare module "esri/core/sql" {
  import sql = __esri.sql;
  export = sql;
}

declare module "esri/core/sql/WhereClause" {
  import WhereClause = __esri.WhereClause;
  export = WhereClause;
}

declare module "esri/core/units" {
  import units = __esri.units;
  export = units;
}

declare module "esri/core/urlUtils" {
  import urlUtils = __esri.urlUtils;
  export = urlUtils;
}

declare module "esri/core/workers" {
  import workers = __esri.workers;
  export = workers;
}

declare module "esri/form/elements/inputs/attachments/support/inputs" {
  import inputs = __esri.inputs;
  export = inputs;
}

declare module "esri/form/elements/support/inputs" {
  import supportInputs = __esri.supportInputs;
  export = supportInputs;
}

declare module "esri/form/support/elements" {
  import elements = __esri.elements;
  export = elements;
}

declare module "esri/geometry" {
  import geometry = __esri.geometry;
  export = geometry;
}

declare module "esri/geometry/coordinateFormatter" {
  import coordinateFormatter = __esri.coordinateFormatter;
  export = coordinateFormatter;
}

declare module "esri/geometry/geometryEngine" {
  import geometryEngine = __esri.geometryEngine;
  export = geometryEngine;
}

declare module "esri/geometry/geometryEngineAsync" {
  import geometryEngineAsync = __esri.geometryEngineAsync;
  export = geometryEngineAsync;
}

declare module "esri/geometry/projection" {
  import projection = __esri.projection;
  export = projection;
}

declare module "esri/geometry/operators/affineTransformOperator" {
  import affineTransformOperator = __esri.affineTransformOperator;
  export = affineTransformOperator;
}

declare module "esri/geometry/operators/alphaShapeOperator" {
  import alphaShapeOperator = __esri.alphaShapeOperator;
  export = alphaShapeOperator;
}

declare module "esri/geometry/operators/areaOperator" {
  import areaOperator = __esri.areaOperator;
  export = areaOperator;
}

declare module "esri/geometry/operators/autoCompleteOperator" {
  import autoCompleteOperator = __esri.autoCompleteOperator;
  export = autoCompleteOperator;
}

declare module "esri/geometry/operators/boundaryOperator" {
  import boundaryOperator = __esri.boundaryOperator;
  export = boundaryOperator;
}

declare module "esri/geometry/operators/bufferOperator" {
  import bufferOperator = __esri.bufferOperator;
  export = bufferOperator;
}

declare module "esri/geometry/operators/centroidOperator" {
  import centroidOperator = __esri.centroidOperator;
  export = centroidOperator;
}

declare module "esri/geometry/operators/clipOperator" {
  import clipOperator = __esri.clipOperator;
  export = clipOperator;
}

declare module "esri/geometry/operators/containsOperator" {
  import containsOperator = __esri.containsOperator;
  export = containsOperator;
}

declare module "esri/geometry/operators/convexHullOperator" {
  import convexHullOperator = __esri.convexHullOperator;
  export = convexHullOperator;
}

declare module "esri/geometry/operators/crossesOperator" {
  import crossesOperator = __esri.crossesOperator;
  export = crossesOperator;
}

declare module "esri/geometry/operators/cutOperator" {
  import cutOperator = __esri.cutOperator;
  export = cutOperator;
}

declare module "esri/geometry/operators/densifyOperator" {
  import densifyOperator = __esri.densifyOperator;
  export = densifyOperator;
}

declare module "esri/geometry/operators/differenceOperator" {
  import differenceOperator = __esri.differenceOperator;
  export = differenceOperator;
}

declare module "esri/geometry/operators/disjointOperator" {
  import disjointOperator = __esri.disjointOperator;
  export = disjointOperator;
}

declare module "esri/geometry/operators/distanceOperator" {
  import distanceOperator = __esri.distanceOperator;
  export = distanceOperator;
}

declare module "esri/geometry/operators/equalsOperator" {
  import equalsOperator = __esri.equalsOperator;
  export = equalsOperator;
}

declare module "esri/geometry/operators/extendOperator" {
  import extendOperator = __esri.extendOperator;
  export = extendOperator;
}

declare module "esri/geometry/operators/generalizeOperator" {
  import generalizeOperator = __esri.generalizeOperator;
  export = generalizeOperator;
}

declare module "esri/geometry/operators/geodesicBufferOperator" {
  import geodesicBufferOperator = __esri.geodesicBufferOperator;
  export = geodesicBufferOperator;
}

declare module "esri/geometry/operators/geodesicProximityOperator" {
  import geodesicProximityOperator = __esri.geodesicProximityOperator;
  export = geodesicProximityOperator;
}

declare module "esri/geometry/operators/geodeticAreaOperator" {
  import geodeticAreaOperator = __esri.geodeticAreaOperator;
  export = geodeticAreaOperator;
}

declare module "esri/geometry/operators/geodeticDensifyOperator" {
  import geodeticDensifyOperator = __esri.geodeticDensifyOperator;
  export = geodeticDensifyOperator;
}

declare module "esri/geometry/operators/geodeticDistanceOperator" {
  import geodeticDistanceOperator = __esri.geodeticDistanceOperator;
  export = geodeticDistanceOperator;
}

declare module "esri/geometry/operators/geodeticLengthOperator" {
  import geodeticLengthOperator = __esri.geodeticLengthOperator;
  export = geodeticLengthOperator;
}

declare module "esri/geometry/operators/graphicBufferOperator" {
  import graphicBufferOperator = __esri.graphicBufferOperator;
  export = graphicBufferOperator;
}

declare module "esri/geometry/operators/integrateOperator" {
  import integrateOperator = __esri.integrateOperator;
  export = integrateOperator;
}

declare module "esri/geometry/operators/intersectionOperator" {
  import intersectionOperator = __esri.intersectionOperator;
  export = intersectionOperator;
}

declare module "esri/geometry/operators/intersectsOperator" {
  import intersectsOperator = __esri.intersectsOperator;
  export = intersectsOperator;
}

declare module "esri/geometry/operators/isNearOperator" {
  import isNearOperator = __esri.isNearOperator;
  export = isNearOperator;
}

declare module "esri/geometry/operators/labelPointOperator" {
  import labelPointOperator = __esri.labelPointOperator;
  export = labelPointOperator;
}

declare module "esri/geometry/operators/lengthOperator" {
  import lengthOperator = __esri.lengthOperator;
  export = lengthOperator;
}

declare module "esri/geometry/operators/linesToPolygonsOperator" {
  import linesToPolygonsOperator = __esri.linesToPolygonsOperator;
  export = linesToPolygonsOperator;
}

declare module "esri/geometry/operators/locateBetweenOperator" {
  import locateBetweenOperator = __esri.locateBetweenOperator;
  export = locateBetweenOperator;
}

declare module "esri/geometry/operators/minimumBoundingCircleOperator" {
  import minimumBoundingCircleOperator = __esri.minimumBoundingCircleOperator;
  export = minimumBoundingCircleOperator;
}

declare module "esri/geometry/operators/multiPartToSinglePartOperator" {
  import multiPartToSinglePartOperator = __esri.multiPartToSinglePartOperator;
  export = multiPartToSinglePartOperator;
}

declare module "esri/geometry/operators/offsetOperator" {
  import offsetOperator = __esri.offsetOperator;
  export = offsetOperator;
}

declare module "esri/geometry/operators/overlapsOperator" {
  import overlapsOperator = __esri.overlapsOperator;
  export = overlapsOperator;
}

declare module "esri/geometry/operators/polygonOverlayOperator" {
  import polygonOverlayOperator = __esri.polygonOverlayOperator;
  export = polygonOverlayOperator;
}

declare module "esri/geometry/operators/polygonSlicerOperator" {
  import polygonSlicerOperator = __esri.polygonSlicerOperator;
  export = polygonSlicerOperator;
}

declare module "esri/geometry/operators/proximityOperator" {
  import proximityOperator = __esri.proximityOperator;
  export = proximityOperator;
}

declare module "esri/geometry/operators/relateOperator" {
  import relateOperator = __esri.relateOperator;
  export = relateOperator;
}

declare module "esri/geometry/operators/reshapeOperator" {
  import reshapeOperator = __esri.reshapeOperator;
  export = reshapeOperator;
}

declare module "esri/geometry/operators/simplifyOperator" {
  import simplifyOperator = __esri.simplifyOperator;
  export = simplifyOperator;
}

declare module "esri/geometry/operators/symmetricDifferenceOperator" {
  import symmetricDifferenceOperator = __esri.symmetricDifferenceOperator;
  export = symmetricDifferenceOperator;
}

declare module "esri/geometry/operators/touchesOperator" {
  import touchesOperator = __esri.touchesOperator;
  export = touchesOperator;
}

declare module "esri/geometry/operators/unionOperator" {
  import unionOperator = __esri.unionOperator;
  export = unionOperator;
}

declare module "esri/geometry/operators/withinOperator" {
  import withinOperator = __esri.withinOperator;
  export = withinOperator;
}

declare module "esri/geometry/support/geodesicUtils" {
  import geodesicUtils = __esri.geodesicUtils;
  export = geodesicUtils;
}

declare module "esri/geometry/support/jsonUtils" {
  import jsonUtils = __esri.jsonUtils;
  export = jsonUtils;
}

declare module "esri/geometry/support/meshUtils" {
  import meshUtils = __esri.meshUtils;
  export = meshUtils;
}

declare module "esri/geometry/support/normalizeUtils" {
  import normalizeUtils = __esri.normalizeUtils;
  export = normalizeUtils;
}

declare module "esri/geometry/support/webMercatorUtils" {
  import webMercatorUtils = __esri.webMercatorUtils;
  export = webMercatorUtils;
}

declare module "esri/intl" {
  import intl = __esri.intl;
  export = intl;
}

declare module "esri/kernel" {
  import kernel = __esri.kernel;
  export = kernel;
}

declare module "esri/layers/catalog/catalogUtils" {
  import catalogUtils = __esri.catalogUtils;
  export = catalogUtils;
}

declare module "esri/layers/ogc/wfsUtils" {
  import wfsUtils = __esri.wfsUtils;
  export = wfsUtils;
}

declare module "esri/layers/ogc/wcsUtils" {
  import wcsUtils = __esri.wcsUtils;
  export = wcsUtils;
}

declare module "esri/layers/orientedImagery/transformations/worldToImage" {
  import worldToImage = __esri.worldToImage;
  export = worldToImage;
}

declare module "esri/layers/orientedImagery/transformations/imageToWorld" {
  import imageToWorld = __esri.imageToWorld;
  export = imageToWorld;
}

declare module "esri/layers/support/arcadeUtils" {
  import arcadeUtils = __esri.arcadeUtils;
  export = arcadeUtils;
}

declare module "esri/layers/support/fieldUtils" {
  import fieldUtils = __esri.fieldUtils;
  export = fieldUtils;
}

declare module "esri/layers/support/rasterFunctionConstants" {
  import rasterFunctionConstants = __esri.rasterFunctionConstants;
  export = rasterFunctionConstants;
}

declare module "esri/layers/support/rasterFunctionUtils" {
  import rasterFunctionUtils = __esri.rasterFunctionUtils;
  export = rasterFunctionUtils;
}

declare module "esri/layers/support/StreamConnection" {
  import StreamConnection = __esri.StreamConnection;
  export = StreamConnection;
}

declare module "esri/layers/voxel/VoxelVariable" {
  import VoxelVariable = __esri.VoxelVariable;
  export = VoxelVariable;
}

declare module "esri/layers/voxel/VoxelVolume" {
  import VoxelVolume = __esri.VoxelVolume;
  export = VoxelVolume;
}

declare module "esri/pointCloudRenderers" {
  import pointCloudRenderers = __esri.pointCloudRenderers;
  export = pointCloudRenderers;
}

declare module "esri/popup/content" {
  import content = __esri.content;
  export = content;
}

declare module "esri/popup/support/UtilityNetworkAssociationType" {
  import UtilityNetworkAssociationType = __esri.UtilityNetworkAssociationType;
  export = UtilityNetworkAssociationType;
}

declare module "esri/rasterRenderers" {
  import rasterRenderers = __esri.rasterRenderers;
  export = rasterRenderers;
}

declare module "esri/renderers" {
  import renderers = __esri.renderers;
  export = renderers;
}

declare module "esri/renderers/support/jsonUtils" {
  import supportJsonUtils = __esri.supportJsonUtils;
  export = supportJsonUtils;
}

declare module "esri/renderers/support/utils" {
  import utils = __esri.utils;
  export = utils;
}

declare module "esri/request" {
  import request = __esri.request;
  export = request;
}

declare module "esri/rest/closestFacility" {
  import closestFacility = __esri.closestFacility;
  export = closestFacility;
}

declare module "esri/rest/featureService/utils" {
  import featureServiceUtils = __esri.featureServiceUtils;
  export = featureServiceUtils;
}

declare module "esri/rest/find" {
  import find = __esri.find;
  export = find;
}

declare module "esri/rest/geometryService" {
  import geometryService = __esri.geometryService;
  export = geometryService;
}

declare module "esri/rest/geoprocessor" {
  import geoprocessor = __esri.geoprocessor;
  export = geoprocessor;
}

declare module "esri/rest/geoprocessor/GPOptions" {
  import GPOptions = __esri.GPOptions;
  export = GPOptions;
}

declare module "esri/rest/identify" {
  import identify = __esri.identify;
  export = identify;
}

declare module "esri/rest/imageService" {
  import imageService = __esri.imageService;
  export = imageService;
}

declare module "esri/rest/knowledgeGraphService" {
  import knowledgeGraphService = __esri.knowledgeGraphService;
  export = knowledgeGraphService;
}

declare module "esri/rest/locator" {
  import locator = __esri.locator;
  export = locator;
}

declare module "esri/rest/networks/queryAssociations" {
  import queryAssociations = __esri.queryAssociations;
  export = queryAssociations;
}

declare module "esri/rest/networks/support/ValidateNetworkTopologyResult" {
  import ValidateNetworkTopologyResult = __esri.ValidateNetworkTopologyResult;
  export = ValidateNetworkTopologyResult;
}

declare module "esri/rest/networks/synthesizeAssociationGeometries" {
  import synthesizeAssociationGeometries = __esri.synthesizeAssociationGeometries;
  export = synthesizeAssociationGeometries;
}

declare module "esri/rest/networks/trace" {
  import trace = __esri.trace;
  export = trace;
}

declare module "esri/rest/networkService" {
  import networkService = __esri.networkService;
  export = networkService;
}

declare module "esri/rest/places" {
  import places = __esri.places;
  export = places;
}

declare module "esri/rest/print" {
  import print = __esri.print;
  export = print;
}

declare module "esri/rest/query" {
  import query = __esri.query;
  export = query;
}

declare module "esri/rest/route" {
  import route = __esri.route;
  export = route;
}

declare module "esri/rest/serviceArea" {
  import serviceArea = __esri.serviceArea;
  export = serviceArea;
}

declare module "esri/rest/symbolService" {
  import symbolService = __esri.symbolService;
  export = symbolService;
}

declare module "esri/smartMapping/heuristics/binLevel" {
  import binLevel = __esri.binLevel;
  export = binLevel;
}

declare module "esri/smartMapping/heuristics/scaleRange" {
  import scaleRange = __esri.scaleRange;
  export = scaleRange;
}

declare module "esri/smartMapping/heuristics/sizeRange" {
  import sizeRange = __esri.sizeRange;
  export = sizeRange;
}

declare module "esri/smartMapping/labels/bins" {
  import bins = __esri.bins;
  export = bins;
}

declare module "esri/smartMapping/labels/clusters" {
  import clusters = __esri.clusters;
  export = clusters;
}

declare module "esri/smartMapping/popup/clusters" {
  import popupClusters = __esri.popupClusters;
  export = popupClusters;
}

declare module "esri/smartMapping/popup/templates" {
  import templates = __esri.templates;
  export = templates;
}

declare module "esri/smartMapping/raster/renderers/classBreaks" {
  import classBreaks = __esri.classBreaks;
  export = classBreaks;
}

declare module "esri/smartMapping/raster/renderers/colormap" {
  import colormap = __esri.colormap;
  export = colormap;
}

declare module "esri/smartMapping/raster/renderers/flow" {
  import flow = __esri.flow;
  export = flow;
}

declare module "esri/smartMapping/raster/renderers/rgb" {
  import rgb = __esri.rgb;
  export = rgb;
}

declare module "esri/smartMapping/raster/renderers/shadedRelief" {
  import shadedRelief = __esri.shadedRelief;
  export = shadedRelief;
}

declare module "esri/smartMapping/raster/renderers/stretch" {
  import stretch = __esri.stretch;
  export = stretch;
}

declare module "esri/smartMapping/raster/renderers/uniqueValue" {
  import uniqueValue = __esri.uniqueValue;
  export = uniqueValue;
}

declare module "esri/smartMapping/raster/renderers/vectorField" {
  import vectorField = __esri.vectorField;
  export = vectorField;
}

declare module "esri/smartMapping/raster/support/colorRamps" {
  import colorRamps = __esri.colorRamps;
  export = colorRamps;
}

declare module "esri/smartMapping/raster/support/utils" {
  import supportUtils = __esri.supportUtils;
  export = supportUtils;
}

declare module "esri/smartMapping/renderers/color" {
  import color = __esri.color;
  export = color;
}

declare module "esri/smartMapping/renderers/dotDensity" {
  import dotDensity = __esri.dotDensity;
  export = dotDensity;
}

declare module "esri/smartMapping/renderers/heatmap" {
  import heatmap = __esri.heatmap;
  export = heatmap;
}

declare module "esri/smartMapping/renderers/location" {
  import location = __esri.location;
  export = location;
}

declare module "esri/smartMapping/renderers/opacity" {
  import opacity = __esri.opacity;
  export = opacity;
}

declare module "esri/smartMapping/renderers/pieChart" {
  import pieChart = __esri.pieChart;
  export = pieChart;
}

declare module "esri/smartMapping/renderers/predominance" {
  import predominance = __esri.predominance;
  export = predominance;
}

declare module "esri/smartMapping/renderers/relationship" {
  import relationship = __esri.relationship;
  export = relationship;
}

declare module "esri/smartMapping/renderers/size" {
  import size = __esri.size;
  export = size;
}

declare module "esri/smartMapping/renderers/type" {
  import type = __esri.type;
  export = type;
}

declare module "esri/smartMapping/renderers/univariateColorSize" {
  import univariateColorSize = __esri.univariateColorSize;
  export = univariateColorSize;
}

declare module "esri/smartMapping/renderers/support/rendererUtils" {
  import rendererUtils = __esri.rendererUtils;
  export = rendererUtils;
}

declare module "esri/smartMapping/statistics/classBreaks" {
  import statisticsClassBreaks = __esri.statisticsClassBreaks;
  export = statisticsClassBreaks;
}

declare module "esri/smartMapping/statistics/heatmapStatistics" {
  import heatmapStatistics = __esri.heatmapStatistics;
  export = heatmapStatistics;
}

declare module "esri/smartMapping/statistics/histogram" {
  import histogram = __esri.histogram;
  export = histogram;
}

declare module "esri/smartMapping/statistics/predominantCategories" {
  import predominantCategories = __esri.predominantCategories;
  export = predominantCategories;
}

declare module "esri/smartMapping/statistics/summaryStatistics" {
  import summaryStatistics = __esri.summaryStatistics;
  export = summaryStatistics;
}

declare module "esri/smartMapping/statistics/summaryStatisticsForAge" {
  import summaryStatisticsForAge = __esri.summaryStatisticsForAge;
  export = summaryStatisticsForAge;
}

declare module "esri/smartMapping/statistics/support/ageUtils" {
  import ageUtils = __esri.ageUtils;
  export = ageUtils;
}

declare module "esri/smartMapping/statistics/support/predominanceUtils" {
  import predominanceUtils = __esri.predominanceUtils;
  export = predominanceUtils;
}

declare module "esri/smartMapping/statistics/uniqueValues" {
  import uniqueValues = __esri.uniqueValues;
  export = uniqueValues;
}

declare module "esri/smartMapping/symbology/color" {
  import symbologyColor = __esri.symbologyColor;
  export = symbologyColor;
}

declare module "esri/smartMapping/symbology/dotDensity" {
  import symbologyDotDensity = __esri.symbologyDotDensity;
  export = symbologyDotDensity;
}

declare module "esri/smartMapping/symbology/flow" {
  import symbologyFlow = __esri.symbologyFlow;
  export = symbologyFlow;
}

declare module "esri/smartMapping/symbology/heatmap" {
  import symbologyHeatmap = __esri.symbologyHeatmap;
  export = symbologyHeatmap;
}

declare module "esri/smartMapping/symbology/location" {
  import symbologyLocation = __esri.symbologyLocation;
  export = symbologyLocation;
}

declare module "esri/smartMapping/symbology/pieChart" {
  import symbologyPieChart = __esri.symbologyPieChart;
  export = symbologyPieChart;
}

declare module "esri/smartMapping/symbology/predominance" {
  import symbologyPredominance = __esri.symbologyPredominance;
  export = symbologyPredominance;
}

declare module "esri/smartMapping/symbology/relationship" {
  import symbologyRelationship = __esri.symbologyRelationship;
  export = symbologyRelationship;
}

declare module "esri/smartMapping/symbology/size" {
  import symbologySize = __esri.symbologySize;
  export = symbologySize;
}

declare module "esri/smartMapping/symbology/support/colorRamps" {
  import supportColorRamps = __esri.supportColorRamps;
  export = supportColorRamps;
}

declare module "esri/smartMapping/symbology/type" {
  import symbologyType = __esri.symbologyType;
  export = symbologyType;
}

declare module "esri/support/popupUtils" {
  import popupUtils = __esri.popupUtils;
  export = popupUtils;
}

declare module "esri/support/timeUtils" {
  import timeUtils = __esri.timeUtils;
  export = timeUtils;
}

declare module "esri/symbols" {
  import symbols = __esri.symbols;
  export = symbols;
}

declare module "esri/symbols/support/cimConversionUtils" {
  import cimConversionUtils = __esri.cimConversionUtils;
  export = cimConversionUtils;
}

declare module "esri/symbols/support/cimSymbolUtils" {
  import cimSymbolUtils = __esri.cimSymbolUtils;
  export = cimSymbolUtils;
}

declare module "esri/symbols/support/jsonUtils" {
  import symbolsSupportJsonUtils = __esri.symbolsSupportJsonUtils;
  export = symbolsSupportJsonUtils;
}

declare module "esri/symbols/support/symbolUtils" {
  import symbolUtils = __esri.symbolUtils;
  export = symbolUtils;
}

declare module "esri/tables/support/elements" {
  import supportElements = __esri.supportElements;
  export = supportElements;
}

declare module "esri/versionManagement/utils" {
  import versionManagementUtils = __esri.versionManagementUtils;
  export = versionManagementUtils;
}

declare module "esri/versionManagement/versionAdapters/utils" {
  import versionAdaptersUtils = __esri.versionAdaptersUtils;
  export = versionAdaptersUtils;
}

declare module "esri/views/3d/externalRenderers" {
  import externalRenderers = __esri.externalRenderers;
  export = externalRenderers;
}

declare module "esri/views/3d/webgl/ManagedFBO" {
  import ManagedFBO = __esri.ManagedFBO;
  export = ManagedFBO;
}

declare module "esri/views/3d/webgl/RenderCamera" {
  import RenderCamera = __esri.RenderCamera;
  export = RenderCamera;
}

declare module "esri/views/3d/webgl" {
  import webgl = __esri.webgl;
  export = webgl;
}

declare module "esri/views/3d/support/LayerPerformanceInfo" {
  import LayerPerformanceInfo = __esri.LayerPerformanceInfo;
  export = LayerPerformanceInfo;
}

declare module "esri/views/3d/support/SceneViewPerformanceInfo" {
  import SceneViewPerformanceInfo = __esri.SceneViewPerformanceInfo;
  export = SceneViewPerformanceInfo;
}

declare module "esri/views/analysis/LengthDimensionResult" {
  import LengthDimensionResult = __esri.LengthDimensionResult;
  export = LengthDimensionResult;
}

declare module "esri/views/support/colorUtils" {
  import colorUtils = __esri.colorUtils;
  export = colorUtils;
}

declare module "esri/widgets/BatchAttributeForm/BatchAttributeFormViewModel" {
  import BatchAttributeFormViewModel = __esri.BatchAttributeFormViewModel;
  export = BatchAttributeFormViewModel;
}

declare module "esri/widgets/smartMapping/support/utils" {
  import smartMappingSupportUtils = __esri.smartMappingSupportUtils;
  export = smartMappingSupportUtils;
}

declare module "esri/widgets/support/widget" {
  import widget = __esri.widget;
  export = widget;
}

declare module "esri/applications/Components/basemapUtils" {
  import basemapUtils = __esri.basemapUtils;
  export = basemapUtils;
}

declare module "esri/applications/Components/drawUtils" {
  import drawUtils = __esri.drawUtils;
  export = drawUtils;
}

declare module "esri/applications/Components/getDefaultUnits" {
  import getDefaultUnits = __esri.getDefaultUnits;
  export = getDefaultUnits;
}

declare module "esri/applications/Components/gfxUtils" {
  import gfxUtils = __esri.gfxUtils;
  export = gfxUtils;
}

declare module "esri/applications/Components/layerOriginUtils" {
  import layerOriginUtils = __esri.layerOriginUtils;
  export = layerOriginUtils;
}

declare module "esri/applications/Components/previewSymbol2D" {
  import previewSymbol2D = __esri.previewSymbol2D;
  export = previewSymbol2D;
}

declare module "esri/applications/Components/styleUtils" {
  import styleUtils = __esri.styleUtils;
  export = styleUtils;
}

declare module "esri/applications/Components/svgUtils" {
  import svgUtils = __esri.svgUtils;
  export = svgUtils;
}

declare module "esri/applications/Components/webStyleSymbolUtils" {
  import webStyleSymbolUtils = __esri.webStyleSymbolUtils;
  export = webStyleSymbolUtils;
}

declare module "esri/applications/Excalibur/videoViewUtils" {
  import videoViewUtils = __esri.videoViewUtils;
  export = videoViewUtils;
}

declare module "esri/applications/ExperienceBuilder/sketchUtils" {
  import sketchUtils = __esri.sketchUtils;
  export = sketchUtils;
}

declare module "esri/applications/MapViewer/mediaUtils" {
  import mediaUtils = __esri.mediaUtils;
  export = mediaUtils;
}

declare module "esri/applications/MapViewer/templateUtils" {
  import templateUtils = __esri.templateUtils;
  export = templateUtils;
}

declare module "esri/applications/SceneViewer/colorUtils" {
  import SceneViewerColorUtils = __esri.SceneViewerColorUtils;
  export = SceneViewerColorUtils;
}

declare module "esri/applications/SceneViewer/devEnvironmentUtils" {
  import devEnvironmentUtils = __esri.devEnvironmentUtils;
  export = devEnvironmentUtils;
}

declare module "esri/applications/SceneViewer/layerUtils" {
  import layerUtils = __esri.layerUtils;
  export = layerUtils;
}

declare module "esri/applications/SceneViewer/sceneViewerUtils" {
  import sceneViewerUtils = __esri.sceneViewerUtils;
  export = sceneViewerUtils;
}

declare module "esri/applications/SceneViewer/symbolUtils" {
  import SceneViewerSymbolUtils = __esri.SceneViewerSymbolUtils;
  export = SceneViewerSymbolUtils;
}

declare module "esri/applications/Urban/meshUtils" {
  import UrbanMeshUtils = __esri.UrbanMeshUtils;
  export = UrbanMeshUtils;
}

declare module "esri/applications/WebEditor/sketchUtils" {
  import WebEditorSketchUtils = __esri.WebEditorSketchUtils;
  export = WebEditorSketchUtils;
}
